{% extends "!page.html" %}
{% block extrahead %}
  {{ super() }}
  <link rel="stylesheet" href="{{ pathto('_static/theme-overrides.css', 1) }}" type="text/css" />
{% endblock %}
{% block menu %}
  {{ super() }}
{% endblock %}
