/* Generated by Cython 0.29.24 */

/* BEGIN: Cython Metadata
{
    "distutils": {
        "name": "drf_turbo.fields",
        "sources": [
            "drf_turbo/fields.pyx"
        ]
    },
    "module_name": "drf_turbo.fields"
}
END: Cython Metadata */

#ifndef PY_SSIZE_T_CLEAN
#define PY_SSIZE_T_CLEAN
#endif /* PY_SSIZE_T_CLEAN */
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03030000)
    #error Cython requires Python 2.6+ or Python 3.3+.
#else
#define CYTHON_ABI "0_29_24"
#define CYTHON_HEX_VERSION 0x001D18F0
#define CYTHON_FUTURE_DIVISION 0
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef HAVE_LONG_LONG
  #if PY_VERSION_HEX >= 0x02070000
    #define HAVE_LONG_LONG
  #endif
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
#elif defined(PYSTON_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 0
  #elif !defined(CYTHON_USE_PYTYPE_LOOKUP)
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLONG_INTERNALS)
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #ifndef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL 1
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT (PY_VERSION_HEX >= 0x03050000)
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE (PY_VERSION_HEX >= 0x030400a1)
  #endif
  #ifndef CYTHON_USE_DICT_VERSIONS
    #define CYTHON_USE_DICT_VERSIONS (PY_VERSION_HEX >= 0x030600B1)
  #endif
  #ifndef CYTHON_USE_EXC_INFO_STACK
    #define CYTHON_USE_EXC_INFO_STACK (PY_VERSION_HEX >= 0x030700A3)
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #include "longintrepr.h"
  #undef SHIFT
  #undef BASE
  #undef MASK
  #ifdef SIZEOF_VOID_P
    enum { __pyx_check_sizeof_voidp = 1 / (int)(SIZEOF_VOID_P == sizeof(void*)) };
  #endif
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_MAYBE_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_MAYBE_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
           typedef unsigned char     uint8_t;
           typedef unsigned int      uint32_t;
        #else
           typedef unsigned __int8   uint8_t;
           typedef unsigned __int32  uint32_t;
        #endif
    #endif
#else
   #include <stdint.h>
#endif
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus) && __cplusplus >= 201103L
    #if __has_cpp_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH [[fallthrough]]
    #elif __has_cpp_attribute(clang::fallthrough)
      #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
    #elif __has_cpp_attribute(gnu::fallthrough)
      #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__ ) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif

#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
  #define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
#if PY_VERSION_HEX >= 0x030800A4 && PY_VERSION_HEX < 0x030800B2
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, 0, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#else
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#endif
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#ifndef METH_STACKLESS
  #define METH_STACKLESS 0
#endif
#if PY_VERSION_HEX <= 0x030700A3 || !defined(METH_FASTCALL)
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject *const *args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject *const *args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #define __Pyx_PyCFunctionFast _PyCFunctionFast
  #define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
#endif
#if CYTHON_FAST_PYCCALL
#define __Pyx_PyFastCFunction_Check(func)\
    ((PyCFunction_Check(func) && (METH_FASTCALL == (PyCFunction_GET_FLAGS(func) & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)))))
#else
#define __Pyx_PyFastCFunction_Check(func) 0
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030400A1
  #define PyMem_RawMalloc(n)           PyMem_Malloc(n)
  #define PyMem_RawRealloc(p, n)       PyMem_Realloc(p, n)
  #define PyMem_RawFree(p)             PyMem_Free(p)
#endif
#if CYTHON_COMPILING_IN_PYSTON
  #define __Pyx_PyCode_HasFreeVars(co)  PyCode_HasFreeVars(co)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#if !CYTHON_FAST_THREAD_STATE || PY_VERSION_HEX < 0x02070000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x03060000
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#elif PY_VERSION_HEX >= 0x03000000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_Current
#endif
#if PY_VERSION_HEX < 0x030700A2 && !defined(PyThread_tss_create) && !defined(Py_tss_NEEDS_INIT)
#include "pythread.h"
#define Py_tss_NEEDS_INIT 0
typedef int Py_tss_t;
static CYTHON_INLINE int PyThread_tss_create(Py_tss_t *key) {
  *key = PyThread_create_key();
  return 0;
}
static CYTHON_INLINE Py_tss_t * PyThread_tss_alloc(void) {
  Py_tss_t *key = (Py_tss_t *)PyObject_Malloc(sizeof(Py_tss_t));
  *key = Py_tss_NEEDS_INIT;
  return key;
}
static CYTHON_INLINE void PyThread_tss_free(Py_tss_t *key) {
  PyObject_Free(key);
}
static CYTHON_INLINE int PyThread_tss_is_created(Py_tss_t *key) {
  return *key != Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE void PyThread_tss_delete(Py_tss_t *key) {
  PyThread_delete_key(*key);
  *key = Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE int PyThread_tss_set(Py_tss_t *key, void *value) {
  return PyThread_set_key_value(*key, value);
}
static CYTHON_INLINE void * PyThread_tss_get(Py_tss_t *key) {
  return PyThread_get_key_value(*key);
}
#endif
#if CYTHON_COMPILING_IN_CPYTHON || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#if PY_MAJOR_VERSION >= 3 || CYTHON_FUTURE_DIVISION
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1 && CYTHON_USE_UNICODE_INTERNALS
#define __Pyx_PyDict_GetItemStr(dict, name)  _PyDict_GetItem_KnownHash(dict, name, ((PyASCIIObject *) name)->hash)
#else
#define __Pyx_PyDict_GetItemStr(dict, name)  PyDict_GetItem(dict, name)
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #if defined(PyUnicode_IS_READY)
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #else
  #define __Pyx_PyUnicode_READY(op)       (0)
  #endif
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, ch)
  #if defined(PyUnicode_IS_READY) && defined(PyUnicode_GET_SIZE)
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03090000
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : ((PyCompactUnicodeObject *)(u))->wstr_length))
  #else
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
  #endif
  #else
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_LENGTH(u))
  #endif
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535 : 1114111)
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyByteArray_Check)
  #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Format)
  #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None || (PyString_Check(b) && !PyString_CheckExact(b)))) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None || (PyUnicode_Check(b) && !PyUnicode_CheckExact(b)))) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#ifndef PyObject_Unicode
  #define PyObject_Unicode             PyObject_Str
#endif
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#if PY_VERSION_HEX >= 0x030900A4
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_SET_REFCNT(obj, refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SET_SIZE(obj, size)
#else
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_REFCNT(obj) = (refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SIZE(obj) = (size)
#endif
#if CYTHON_ASSUME_SAFE_MACROS
  #define __Pyx_PySequence_SIZE(seq)  Py_SIZE(seq)
#else
  #define __Pyx_PySequence_SIZE(seq)  PySequence_Size(seq)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   PyInt_AsLong
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   PyInt_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? ((void)(klass), PyMethod_New(func, self)) : __Pyx_NewRef(func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef __Pyx_PyAsyncMethodsStruct
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
#endif

#if defined(WIN32) || defined(MS_WINDOWS)
  #define _USE_MATH_DEFINES
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif

#define __PYX_MARK_ERR_POS(f_index, lineno) \
    { __pyx_filename = __pyx_f[f_index]; (void)__pyx_filename; __pyx_lineno = lineno; (void)__pyx_lineno; __pyx_clineno = __LINE__; (void)__pyx_clineno; }
#define __PYX_ERR(f_index, lineno, Ln_error) \
    { __PYX_MARK_ERR_POS(f_index, lineno) goto Ln_error; }

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__drf_turbo__fields
#define __PYX_HAVE_API__drf_turbo__fields
/* Early includes */
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_UTF8 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT (PY_MAJOR_VERSION >= 3 && __PYX_DEFAULT_STRING_ENCODING_IS_UTF8)
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
static CYTHON_INLINE int __Pyx_is_valid_index(Py_ssize_t i, Py_ssize_t limit) {
    return (size_t) i < (size_t) limit;
}
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyObject_AsWritableString(s)    ((char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u) {
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b);
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#define __Pyx_PyNumber_Float(x) (PyFloat_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Float(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c) + 1);
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }

static PyObject *__pyx_m = NULL;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_cython_runtime = NULL;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static PyObject *__pyx_empty_unicode;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;


static const char *__pyx_f[] = {
  "drf_turbo/fields.pyx",
  "stringsource",
  "drf_turbo/fields.pxd",
};

/*--- Type declarations ---*/
struct __pyx_obj_9drf_turbo_6fields_SkipField;
struct __pyx_obj_9drf_turbo_6fields_Field;
struct __pyx_obj_9drf_turbo_6fields_StrField;
struct __pyx_obj_9drf_turbo_6fields_EmailField;
struct __pyx_obj_9drf_turbo_6fields_URLField;
struct __pyx_obj_9drf_turbo_6fields_RegexField;
struct __pyx_obj_9drf_turbo_6fields_IPField;
struct __pyx_obj_9drf_turbo_6fields_PasswordField;
struct __pyx_obj_9drf_turbo_6fields_UUIDField;
struct __pyx_obj_9drf_turbo_6fields_SlugField;
struct __pyx_obj_9drf_turbo_6fields_IntField;
struct __pyx_obj_9drf_turbo_6fields_FloatField;
struct __pyx_obj_9drf_turbo_6fields_DecimalField;
struct __pyx_obj_9drf_turbo_6fields_BoolField;
struct __pyx_obj_9drf_turbo_6fields_ChoiceField;
struct __pyx_obj_9drf_turbo_6fields_MultipleChoiceField;
struct __pyx_obj_9drf_turbo_6fields_DateTimeField;
struct __pyx_obj_9drf_turbo_6fields_DateField;
struct __pyx_obj_9drf_turbo_6fields_TimeField;
struct __pyx_obj_9drf_turbo_6fields_FileField;
struct __pyx_obj_9drf_turbo_6fields_ArrayField;
struct __pyx_obj_9drf_turbo_6fields_DictField;
struct __pyx_obj_9drf_turbo_6fields_JSONField;
struct __pyx_obj_9drf_turbo_6fields_RelatedField;
struct __pyx_obj_9drf_turbo_6fields_ManyRelatedField;
struct __pyx_obj_9drf_turbo_6fields_ConstantField;
struct __pyx_obj_9drf_turbo_6fields_RecursiveField;
struct __pyx_obj_9drf_turbo_6fields_MethodField;
struct __pyx_opt_args_9drf_turbo_6fields_5Field_get_attribute;

/* "drf_turbo/fields.pxd":31
 *     cpdef run_validation(self,object data,dict context)
 *     cpdef get_initial(self)
 *     cpdef get_attribute(self, instance,attr=*)             # <<<<<<<<<<<<<<
 *     cpdef get_default_value(self)
 *     cpdef tuple validate_empty_values(self, data)
 */
struct __pyx_opt_args_9drf_turbo_6fields_5Field_get_attribute {
  int __pyx_n;
  PyObject *attr;
};

/* "drf_turbo/fields.pxd":3
 * cdef object NO_DEFAULT
 * 
 * cdef class SkipField(Exception):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */
struct __pyx_obj_9drf_turbo_6fields_SkipField {
  PyBaseExceptionObject __pyx_base;
};


/* "drf_turbo/fields.pxd":6
 *     pass
 * 
 * cdef class Field :             # <<<<<<<<<<<<<<
 *     cdef public :
 *         str attr
 */
struct __pyx_obj_9drf_turbo_6fields_Field {
  PyObject_HEAD
  struct __pyx_vtabstruct_9drf_turbo_6fields_Field *__pyx_vtab;
  PyObject *attr;
  int call;
  int required;
  int write_only;
  int read_only;
  int allow_null;
  PyObject *label;
  PyObject *default_value;
  PyObject *initial;
  PyObject *help_text;
  PyObject *style;
  PyObject *field_name;
  PyObject *root;
  PyObject *validators;
  PyObject *error_messages;
  PyObject *attrs;
};


/* "drf_turbo/fields.pxd":37
 * 
 * 
 * cdef class StrField(Field):             # <<<<<<<<<<<<<<
 *     cdef public:
 *         allow_blank
 */
struct __pyx_obj_9drf_turbo_6fields_StrField {
  struct __pyx_obj_9drf_turbo_6fields_Field __pyx_base;
  PyObject *allow_blank;
  PyObject *trim_whitespace;
  PyObject *max_length;
  PyObject *min_length;
};


/* "drf_turbo/fields.pxd":44
 *         min_length
 * 
 * cdef class EmailField(StrField):             # <<<<<<<<<<<<<<
 *     cdef public:
 *         bint to_lower
 */
struct __pyx_obj_9drf_turbo_6fields_EmailField {
  struct __pyx_obj_9drf_turbo_6fields_StrField __pyx_base;
  int to_lower;
};


/* "drf_turbo/fields.pxd":48
 *         bint to_lower
 * 
 * cdef class URLField(StrField):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */
struct __pyx_obj_9drf_turbo_6fields_URLField {
  struct __pyx_obj_9drf_turbo_6fields_StrField __pyx_base;
};


/* "drf_turbo/fields.pxd":51
 *     pass
 * 
 * cdef class RegexField(StrField):             # <<<<<<<<<<<<<<
 *     cdef public:
 *         regex
 */
struct __pyx_obj_9drf_turbo_6fields_RegexField {
  struct __pyx_obj_9drf_turbo_6fields_StrField __pyx_base;
  PyObject *regex;
};


/* "drf_turbo/fields.pxd":55
 *         regex
 * 
 * cdef class IPField(StrField):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */
struct __pyx_obj_9drf_turbo_6fields_IPField {
  struct __pyx_obj_9drf_turbo_6fields_StrField __pyx_base;
};


/* "drf_turbo/fields.pxd":58
 *     pass
 * 
 * cdef class PasswordField(StrField):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */
struct __pyx_obj_9drf_turbo_6fields_PasswordField {
  struct __pyx_obj_9drf_turbo_6fields_StrField __pyx_base;
};


/* "drf_turbo/fields.pxd":61
 *     pass
 * 
 * cdef class UUIDField(StrField):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */
struct __pyx_obj_9drf_turbo_6fields_UUIDField {
  struct __pyx_obj_9drf_turbo_6fields_StrField __pyx_base;
};


/* "drf_turbo/fields.pxd":64
 *     pass
 * 
 * cdef class SlugField(Field):             # <<<<<<<<<<<<<<
 *     cdef public:
 *         allow_unicode
 */
struct __pyx_obj_9drf_turbo_6fields_SlugField {
  struct __pyx_obj_9drf_turbo_6fields_Field __pyx_base;
  PyObject *allow_unicode;
};


/* "drf_turbo/fields.pxd":68
 *         allow_unicode
 * 
 * cdef class IntField(Field):             # <<<<<<<<<<<<<<
 *     cdef public :
 *         max_value
 */
struct __pyx_obj_9drf_turbo_6fields_IntField {
  struct __pyx_obj_9drf_turbo_6fields_Field __pyx_base;
  PyObject *max_value;
  PyObject *min_value;
};


/* "drf_turbo/fields.pxd":73
 *         min_value
 * 
 * cdef class FloatField(Field):             # <<<<<<<<<<<<<<
 *     cdef public :
 *         max_value
 */
struct __pyx_obj_9drf_turbo_6fields_FloatField {
  struct __pyx_obj_9drf_turbo_6fields_Field __pyx_base;
  PyObject *max_value;
  PyObject *min_value;
};


/* "drf_turbo/fields.pxd":78
 *         min_value
 * 
 * cdef class DecimalField(Field):             # <<<<<<<<<<<<<<
 *     cdef public:
 *         max_digits
 */
struct __pyx_obj_9drf_turbo_6fields_DecimalField {
  struct __pyx_obj_9drf_turbo_6fields_Field __pyx_base;
  PyObject *max_digits;
  PyObject *decimal_places;
  PyObject *max_value;
  PyObject *min_value;
  PyObject *rounding;
  PyObject *coerce_to_string;
  PyObject *max_whole_digits;
};


/* "drf_turbo/fields.pxd":91
 *     cdef quantize(self,value)
 * 
 * cdef class BoolField(Field):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */
struct __pyx_obj_9drf_turbo_6fields_BoolField {
  struct __pyx_obj_9drf_turbo_6fields_Field __pyx_base;
};


/* "drf_turbo/fields.pxd":94
 *     pass
 * 
 * cdef class ChoiceField(Field):             # <<<<<<<<<<<<<<
 *     cdef public :
 *         choices
 */
struct __pyx_obj_9drf_turbo_6fields_ChoiceField {
  struct __pyx_obj_9drf_turbo_6fields_Field __pyx_base;
  PyObject *choices;
  PyObject *choice_strings_to_values;
  PyObject *choice_strings_to_display;
  PyObject *allow_blank;
};


/* "drf_turbo/fields.pxd":101
 *         allow_blank
 * 
 * cdef class MultipleChoiceField(ChoiceField):             # <<<<<<<<<<<<<<
 *         cdef public:
 *             allow_empty
 */
struct __pyx_obj_9drf_turbo_6fields_MultipleChoiceField {
  struct __pyx_obj_9drf_turbo_6fields_ChoiceField __pyx_base;
  PyObject *allow_empty;
};


/* "drf_turbo/fields.pxd":105
 *             allow_empty
 * 
 * cdef class DateTimeField(Field):             # <<<<<<<<<<<<<<
 *     cdef public:
 *         format
 */
struct __pyx_obj_9drf_turbo_6fields_DateTimeField {
  struct __pyx_obj_9drf_turbo_6fields_Field __pyx_base;
  PyObject *format;
  PyObject *input_formats;
  PyObject *default_timezone;
  PyObject *timezone;
};


/* "drf_turbo/fields.pxd":115
 *     cpdef enforce_timezone(self, value)
 * 
 * cdef class DateField(Field):             # <<<<<<<<<<<<<<
 *     cdef public:
 *         format
 */
struct __pyx_obj_9drf_turbo_6fields_DateField {
  struct __pyx_obj_9drf_turbo_6fields_Field __pyx_base;
  PyObject *format;
  PyObject *input_formats;
};


/* "drf_turbo/fields.pxd":120
 *         input_formats
 * 
 * cdef class TimeField(Field):             # <<<<<<<<<<<<<<
 *     cdef public:
 *         format
 */
struct __pyx_obj_9drf_turbo_6fields_TimeField {
  struct __pyx_obj_9drf_turbo_6fields_Field __pyx_base;
  PyObject *format;
  PyObject *input_formats;
};


/* "drf_turbo/fields.pxd":125
 *         input_formats
 * 
 * cdef class FileField(Field):             # <<<<<<<<<<<<<<
 *     cdef public :
 *         max_length
 */
struct __pyx_obj_9drf_turbo_6fields_FileField {
  struct __pyx_obj_9drf_turbo_6fields_Field __pyx_base;
  PyObject *max_length;
  PyObject *allow_empty_file;
};


/* "drf_turbo/fields.pxd":130
 *         allow_empty_file
 * 
 * cdef class ArrayField(Field):             # <<<<<<<<<<<<<<
 *     cdef public :
 *         child
 */
struct __pyx_obj_9drf_turbo_6fields_ArrayField {
  struct __pyx_obj_9drf_turbo_6fields_Field __pyx_base;
  PyObject *child;
  PyObject *allow_empty;
  PyObject *min_items;
  PyObject *max_items;
  PyObject *exact_items;
};


/* "drf_turbo/fields.pxd":140
 *     cpdef run_child_validation(self,data,dict context)
 * 
 * cdef class DictField(Field):             # <<<<<<<<<<<<<<
 *     cdef public :
 *         child
 */
struct __pyx_obj_9drf_turbo_6fields_DictField {
  struct __pyx_obj_9drf_turbo_6fields_Field __pyx_base;
  PyObject *child;
  PyObject *allow_empty;
};


/* "drf_turbo/fields.pxd":148
 * 
 * 
 * cdef class JSONField(Field):             # <<<<<<<<<<<<<<
 *     cdef public :
 *         binary
 */
struct __pyx_obj_9drf_turbo_6fields_JSONField {
  struct __pyx_obj_9drf_turbo_6fields_Field __pyx_base;
  PyObject *binary;
  PyObject *encoder;
  PyObject *decoder;
};


/* "drf_turbo/fields.pxd":155
 * 
 * 
 * cdef class RelatedField(Field):             # <<<<<<<<<<<<<<
 *     cdef public:
 *         queryset
 */
struct __pyx_obj_9drf_turbo_6fields_RelatedField {
  struct __pyx_obj_9drf_turbo_6fields_Field __pyx_base;
  PyObject *queryset;
};


/* "drf_turbo/fields.pxd":159
 *         queryset
 * 
 * cdef class ManyRelatedField(Field):             # <<<<<<<<<<<<<<
 *     cdef public :
 *         child_relation
 */
struct __pyx_obj_9drf_turbo_6fields_ManyRelatedField {
  struct __pyx_obj_9drf_turbo_6fields_Field __pyx_base;
  PyObject *child_relation;
  PyObject *allow_empty;
};


/* "drf_turbo/fields.pxd":164
 *         allow_empty
 * 
 * cdef class ConstantField(Field):             # <<<<<<<<<<<<<<
 *     cdef public :
 *         constant
 */
struct __pyx_obj_9drf_turbo_6fields_ConstantField {
  struct __pyx_obj_9drf_turbo_6fields_Field __pyx_base;
  PyObject *constant;
};


/* "drf_turbo/fields.pxd":168
 *         constant
 * 
 * cdef class RecursiveField(Field):             # <<<<<<<<<<<<<<
 *     cdef public :
 *         many
 */
struct __pyx_obj_9drf_turbo_6fields_RecursiveField {
  struct __pyx_obj_9drf_turbo_6fields_Field __pyx_base;
  PyObject *many;
  PyObject *context;
  PyObject *only;
  PyObject *exclude;
};


/* "drf_turbo/fields.pxd":176
 *     cpdef serialize(self,value,dict context)
 * 
 * cdef class MethodField(Field):             # <<<<<<<<<<<<<<
 *     cdef public :
 *         method_name
 */
struct __pyx_obj_9drf_turbo_6fields_MethodField {
  struct __pyx_obj_9drf_turbo_6fields_Field __pyx_base;
  PyObject *method_name;
};



/* "drf_turbo/fields.pyx":42
 * 
 * 
 * cdef class Field :             # <<<<<<<<<<<<<<
 *     """
 *     Basic field from which other fields should extend. It applies no
 */

struct __pyx_vtabstruct_9drf_turbo_6fields_Field {
  PyObject *(*serialize)(struct __pyx_obj_9drf_turbo_6fields_Field *, PyObject *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*deserialize)(struct __pyx_obj_9drf_turbo_6fields_Field *, PyObject *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*method_getter)(struct __pyx_obj_9drf_turbo_6fields_Field *, PyObject *, PyObject *, int __pyx_skip_dispatch);
  void (*bind)(struct __pyx_obj_9drf_turbo_6fields_Field *, PyObject *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*run_validation)(struct __pyx_obj_9drf_turbo_6fields_Field *, PyObject *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*get_initial)(struct __pyx_obj_9drf_turbo_6fields_Field *, int __pyx_skip_dispatch);
  PyObject *(*get_attribute)(struct __pyx_obj_9drf_turbo_6fields_Field *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_9drf_turbo_6fields_5Field_get_attribute *__pyx_optional_args);
  PyObject *(*get_default_value)(struct __pyx_obj_9drf_turbo_6fields_Field *, int __pyx_skip_dispatch);
  PyObject *(*validate_empty_values)(struct __pyx_obj_9drf_turbo_6fields_Field *, PyObject *, int __pyx_skip_dispatch);
  long (*validate_or_raise)(struct __pyx_obj_9drf_turbo_6fields_Field *, PyObject *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_9drf_turbo_6fields_Field *__pyx_vtabptr_9drf_turbo_6fields_Field;


/* "drf_turbo/fields.pyx":291
 * 
 * 
 * cdef class StrField(Field):             # <<<<<<<<<<<<<<
 *     """"
 *     A field that validates input as an string.
 */

struct __pyx_vtabstruct_9drf_turbo_6fields_StrField {
  struct __pyx_vtabstruct_9drf_turbo_6fields_Field __pyx_base;
};
static struct __pyx_vtabstruct_9drf_turbo_6fields_StrField *__pyx_vtabptr_9drf_turbo_6fields_StrField;


/* "drf_turbo/fields.pyx":337
 * 
 * @cython.final
 * cdef class EmailField(StrField):             # <<<<<<<<<<<<<<
 *     """
 *     A field that validates input as an E-Mail address.
 */

struct __pyx_vtabstruct_9drf_turbo_6fields_EmailField {
  struct __pyx_vtabstruct_9drf_turbo_6fields_StrField __pyx_base;
};
static struct __pyx_vtabstruct_9drf_turbo_6fields_EmailField *__pyx_vtabptr_9drf_turbo_6fields_EmailField;


/* "drf_turbo/fields.pyx":367
 * 
 * @cython.final
 * cdef class URLField(StrField):             # <<<<<<<<<<<<<<
 *     """
 *     A field that validates input as an URL.
 */

struct __pyx_vtabstruct_9drf_turbo_6fields_URLField {
  struct __pyx_vtabstruct_9drf_turbo_6fields_StrField __pyx_base;
};
static struct __pyx_vtabstruct_9drf_turbo_6fields_URLField *__pyx_vtabptr_9drf_turbo_6fields_URLField;


/* "drf_turbo/fields.pyx":388
 * 
 * @cython.final
 * cdef class RegexField(StrField):             # <<<<<<<<<<<<<<
 *     """
 *     A field that validates input against a given regular expression.
 */

struct __pyx_vtabstruct_9drf_turbo_6fields_RegexField {
  struct __pyx_vtabstruct_9drf_turbo_6fields_StrField __pyx_base;
};
static struct __pyx_vtabstruct_9drf_turbo_6fields_RegexField *__pyx_vtabptr_9drf_turbo_6fields_RegexField;


/* "drf_turbo/fields.pyx":413
 * 
 * @cython.final
 * cdef class IPField(StrField):             # <<<<<<<<<<<<<<
 *     """
 *     A field that validates that input is an IP address.
 */

struct __pyx_vtabstruct_9drf_turbo_6fields_IPField {
  struct __pyx_vtabstruct_9drf_turbo_6fields_StrField __pyx_base;
};
static struct __pyx_vtabstruct_9drf_turbo_6fields_IPField *__pyx_vtabptr_9drf_turbo_6fields_IPField;


/* "drf_turbo/fields.pyx":431
 * 
 * @cython.final
 * cdef class PasswordField(StrField):             # <<<<<<<<<<<<<<
 *     """
 *     A field that validates input as a password.
 */

struct __pyx_vtabstruct_9drf_turbo_6fields_PasswordField {
  struct __pyx_vtabstruct_9drf_turbo_6fields_StrField __pyx_base;
};
static struct __pyx_vtabstruct_9drf_turbo_6fields_PasswordField *__pyx_vtabptr_9drf_turbo_6fields_PasswordField;


/* "drf_turbo/fields.pyx":446
 * 
 * @cython.final
 * cdef class UUIDField(StrField):             # <<<<<<<<<<<<<<
 *     """
 *     A field that validates input as an UUID.
 */

struct __pyx_vtabstruct_9drf_turbo_6fields_UUIDField {
  struct __pyx_vtabstruct_9drf_turbo_6fields_StrField __pyx_base;
};
static struct __pyx_vtabstruct_9drf_turbo_6fields_UUIDField *__pyx_vtabptr_9drf_turbo_6fields_UUIDField;


/* "drf_turbo/fields.pyx":476
 * 
 * @cython.final
 * cdef class SlugField(Field):             # <<<<<<<<<<<<<<
 *     """
 *     Slug field type.
 */

struct __pyx_vtabstruct_9drf_turbo_6fields_SlugField {
  struct __pyx_vtabstruct_9drf_turbo_6fields_Field __pyx_base;
};
static struct __pyx_vtabstruct_9drf_turbo_6fields_SlugField *__pyx_vtabptr_9drf_turbo_6fields_SlugField;


/* "drf_turbo/fields.pyx":506
 * 
 * @cython.final
 * cdef class IntField(Field):             # <<<<<<<<<<<<<<
 *     """
 *     A field that validates input as an integer.
 */

struct __pyx_vtabstruct_9drf_turbo_6fields_IntField {
  struct __pyx_vtabstruct_9drf_turbo_6fields_Field __pyx_base;
};
static struct __pyx_vtabstruct_9drf_turbo_6fields_IntField *__pyx_vtabptr_9drf_turbo_6fields_IntField;


/* "drf_turbo/fields.pyx":541
 * 
 * @cython.final
 * cdef class FloatField(Field):             # <<<<<<<<<<<<<<
 *     """
 *     A field that validates input as a float.
 */

struct __pyx_vtabstruct_9drf_turbo_6fields_FloatField {
  struct __pyx_vtabstruct_9drf_turbo_6fields_Field __pyx_base;
};
static struct __pyx_vtabstruct_9drf_turbo_6fields_FloatField *__pyx_vtabptr_9drf_turbo_6fields_FloatField;


/* "drf_turbo/fields.pyx":579
 * 
 * @cython.final
 * cdef class DecimalField(Field):             # <<<<<<<<<<<<<<
 *     """
 *     A field that validates input as a Python Decimal.
 */

struct __pyx_vtabstruct_9drf_turbo_6fields_DecimalField {
  struct __pyx_vtabstruct_9drf_turbo_6fields_Field __pyx_base;
  PyObject *(*validate_precision)(struct __pyx_obj_9drf_turbo_6fields_DecimalField *, PyObject *);
  PyObject *(*quantize)(struct __pyx_obj_9drf_turbo_6fields_DecimalField *, PyObject *);
};
static struct __pyx_vtabstruct_9drf_turbo_6fields_DecimalField *__pyx_vtabptr_9drf_turbo_6fields_DecimalField;
static PyObject *__pyx_f_9drf_turbo_6fields_12DecimalField_validate_precision(struct __pyx_obj_9drf_turbo_6fields_DecimalField *, PyObject *);
static PyObject *__pyx_f_9drf_turbo_6fields_12DecimalField_quantize(struct __pyx_obj_9drf_turbo_6fields_DecimalField *, PyObject *);


/* "drf_turbo/fields.pyx":730
 * 
 * @cython.final
 * cdef class BoolField(Field):             # <<<<<<<<<<<<<<
 *     """
 *     Boolean field type.
 */

struct __pyx_vtabstruct_9drf_turbo_6fields_BoolField {
  struct __pyx_vtabstruct_9drf_turbo_6fields_Field __pyx_base;
};
static struct __pyx_vtabstruct_9drf_turbo_6fields_BoolField *__pyx_vtabptr_9drf_turbo_6fields_BoolField;


/* "drf_turbo/fields.pyx":781
 *         return data
 * 
 * cdef class ChoiceField(Field):             # <<<<<<<<<<<<<<
 *     """
 *     Choice field type.
 */

struct __pyx_vtabstruct_9drf_turbo_6fields_ChoiceField {
  struct __pyx_vtabstruct_9drf_turbo_6fields_Field __pyx_base;
};
static struct __pyx_vtabstruct_9drf_turbo_6fields_ChoiceField *__pyx_vtabptr_9drf_turbo_6fields_ChoiceField;


/* "drf_turbo/fields.pyx":837
 * 
 * @cython.final
 * cdef class MultipleChoiceField(ChoiceField):             # <<<<<<<<<<<<<<
 *     """
 *     Multiple choice field type.
 */

struct __pyx_vtabstruct_9drf_turbo_6fields_MultipleChoiceField {
  struct __pyx_vtabstruct_9drf_turbo_6fields_ChoiceField __pyx_base;
};
static struct __pyx_vtabstruct_9drf_turbo_6fields_MultipleChoiceField *__pyx_vtabptr_9drf_turbo_6fields_MultipleChoiceField;


/* "drf_turbo/fields.pyx":880
 * 
 * @cython.final
 * cdef class DateTimeField(Field):             # <<<<<<<<<<<<<<
 *     """
 *     A field that (de)serializes to a :class:`datetime.datetime` object.
 */

struct __pyx_vtabstruct_9drf_turbo_6fields_DateTimeField {
  struct __pyx_vtabstruct_9drf_turbo_6fields_Field __pyx_base;
  PyObject *(*get_default_timezone)(struct __pyx_obj_9drf_turbo_6fields_DateTimeField *, int __pyx_skip_dispatch);
  PyObject *(*enforce_timezone)(struct __pyx_obj_9drf_turbo_6fields_DateTimeField *, PyObject *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_9drf_turbo_6fields_DateTimeField *__pyx_vtabptr_9drf_turbo_6fields_DateTimeField;
static PyObject *__pyx_f_9drf_turbo_6fields_13DateTimeField_get_default_timezone(struct __pyx_obj_9drf_turbo_6fields_DateTimeField *, int __pyx_skip_dispatch);
static PyObject *__pyx_f_9drf_turbo_6fields_13DateTimeField_enforce_timezone(struct __pyx_obj_9drf_turbo_6fields_DateTimeField *, PyObject *, int __pyx_skip_dispatch);


/* "drf_turbo/fields.pyx":982
 * 
 * @cython.final
 * cdef class DateField(Field):             # <<<<<<<<<<<<<<
 *     """
 *     A field that (de)serializes to a :class:`datetime.date` object.
 */

struct __pyx_vtabstruct_9drf_turbo_6fields_DateField {
  struct __pyx_vtabstruct_9drf_turbo_6fields_Field __pyx_base;
};
static struct __pyx_vtabstruct_9drf_turbo_6fields_DateField *__pyx_vtabptr_9drf_turbo_6fields_DateField;


/* "drf_turbo/fields.pyx":1058
 * 
 * @cython.final
 * cdef class TimeField(Field):             # <<<<<<<<<<<<<<
 *     """
 *     A field that (de)serializes to a :class:`datetime.time` object.
 */

struct __pyx_vtabstruct_9drf_turbo_6fields_TimeField {
  struct __pyx_vtabstruct_9drf_turbo_6fields_Field __pyx_base;
};
static struct __pyx_vtabstruct_9drf_turbo_6fields_TimeField *__pyx_vtabptr_9drf_turbo_6fields_TimeField;


/* "drf_turbo/fields.pyx":1132
 * 
 * @cython.final
 * cdef class FileField(Field):             # <<<<<<<<<<<<<<
 *     """
 *     A file field.
 */

struct __pyx_vtabstruct_9drf_turbo_6fields_FileField {
  struct __pyx_vtabstruct_9drf_turbo_6fields_Field __pyx_base;
};
static struct __pyx_vtabstruct_9drf_turbo_6fields_FileField *__pyx_vtabptr_9drf_turbo_6fields_FileField;


/* "drf_turbo/fields.pyx":1180
 * 
 * @cython.final
 * cdef class ArrayField(Field):             # <<<<<<<<<<<<<<
 *     """
 *     An Array Field.
 */

struct __pyx_vtabstruct_9drf_turbo_6fields_ArrayField {
  struct __pyx_vtabstruct_9drf_turbo_6fields_Field __pyx_base;
  PyObject *(*run_child_validation)(struct __pyx_obj_9drf_turbo_6fields_ArrayField *, PyObject *, PyObject *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_9drf_turbo_6fields_ArrayField *__pyx_vtabptr_9drf_turbo_6fields_ArrayField;
static PyObject *__pyx_f_9drf_turbo_6fields_10ArrayField_run_child_validation(struct __pyx_obj_9drf_turbo_6fields_ArrayField *, PyObject *, PyObject *, int __pyx_skip_dispatch);


/* "drf_turbo/fields.pyx":1259
 * 
 * @cython.final
 * cdef class DictField(Field):             # <<<<<<<<<<<<<<
 *     """A Dict Field.
 * 
 */

struct __pyx_vtabstruct_9drf_turbo_6fields_DictField {
  struct __pyx_vtabstruct_9drf_turbo_6fields_Field __pyx_base;
  PyObject *(*run_child_validation)(struct __pyx_obj_9drf_turbo_6fields_DictField *, PyObject *, PyObject *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_9drf_turbo_6fields_DictField *__pyx_vtabptr_9drf_turbo_6fields_DictField;
static PyObject *__pyx_f_9drf_turbo_6fields_9DictField_run_child_validation(struct __pyx_obj_9drf_turbo_6fields_DictField *, PyObject *, PyObject *, int __pyx_skip_dispatch);


/* "drf_turbo/fields.pyx":1312
 * 
 * @cython.final
 * cdef class JSONField(Field):             # <<<<<<<<<<<<<<
 *     """A JSON Field.
 * 
 */

struct __pyx_vtabstruct_9drf_turbo_6fields_JSONField {
  struct __pyx_vtabstruct_9drf_turbo_6fields_Field __pyx_base;
};
static struct __pyx_vtabstruct_9drf_turbo_6fields_JSONField *__pyx_vtabptr_9drf_turbo_6fields_JSONField;


/* "drf_turbo/fields.pyx":1350
 * 
 * @cython.final
 * cdef class RelatedField(Field):             # <<<<<<<<<<<<<<
 *     """A Related Field.
 * 
 */

struct __pyx_vtabstruct_9drf_turbo_6fields_RelatedField {
  struct __pyx_vtabstruct_9drf_turbo_6fields_Field __pyx_base;
};
static struct __pyx_vtabstruct_9drf_turbo_6fields_RelatedField *__pyx_vtabptr_9drf_turbo_6fields_RelatedField;


/* "drf_turbo/fields.pyx":1377
 * 
 * @cython.final
 * cdef class ManyRelatedField(Field):             # <<<<<<<<<<<<<<
 *     """
 *     A field used to represent a to-many relationship.
 */

struct __pyx_vtabstruct_9drf_turbo_6fields_ManyRelatedField {
  struct __pyx_vtabstruct_9drf_turbo_6fields_Field __pyx_base;
};
static struct __pyx_vtabstruct_9drf_turbo_6fields_ManyRelatedField *__pyx_vtabptr_9drf_turbo_6fields_ManyRelatedField;


/* "drf_turbo/fields.pyx":1413
 * 
 * @cython.final
 * cdef class ConstantField(Field):             # <<<<<<<<<<<<<<
 *     """
 *     A field that always outputs a fixed value.
 */

struct __pyx_vtabstruct_9drf_turbo_6fields_ConstantField {
  struct __pyx_vtabstruct_9drf_turbo_6fields_Field __pyx_base;
};
static struct __pyx_vtabstruct_9drf_turbo_6fields_ConstantField *__pyx_vtabptr_9drf_turbo_6fields_ConstantField;


/* "drf_turbo/fields.pyx":1438
 * 
 * @cython.final
 * cdef class RecursiveField(Field):             # <<<<<<<<<<<<<<
 *     """
 *     A field that recursively validates its data.
 */

struct __pyx_vtabstruct_9drf_turbo_6fields_RecursiveField {
  struct __pyx_vtabstruct_9drf_turbo_6fields_Field __pyx_base;
};
static struct __pyx_vtabstruct_9drf_turbo_6fields_RecursiveField *__pyx_vtabptr_9drf_turbo_6fields_RecursiveField;


/* "drf_turbo/fields.pyx":1472
 * 
 * @cython.final
 * cdef class MethodField(Field):             # <<<<<<<<<<<<<<
 *     """
 *     A field that calls a method on the serializer instead of simply returning a value.
 */

struct __pyx_vtabstruct_9drf_turbo_6fields_MethodField {
  struct __pyx_vtabstruct_9drf_turbo_6fields_Field __pyx_base;
};
static struct __pyx_vtabstruct_9drf_turbo_6fields_MethodField *__pyx_vtabptr_9drf_turbo_6fields_MethodField;

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* PyErrExceptionMatches.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_ExceptionMatches(err) __Pyx_PyErr_ExceptionMatchesInState(__pyx_tstate, err)
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err);
#else
#define __Pyx_PyErr_ExceptionMatches(err)  PyErr_ExceptionMatches(err)
#endif

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#define __Pyx_PyErr_Occurred()  __pyx_tstate->curexc_type
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  PyErr_Occurred()
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* GetAttr.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *, PyObject *);

/* GetAttr3.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *, PyObject *, PyObject *);

/* PyDictVersioning.proto */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
#define __PYX_DICT_VERSION_INIT  ((PY_UINT64_T) -1)
#define __PYX_GET_DICT_VERSION(dict)  (((PyDictObject*)(dict))->ma_version_tag)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)\
    (version_var) = __PYX_GET_DICT_VERSION(dict);\
    (cache_var) = (value);
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP) {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    if (likely(__PYX_GET_DICT_VERSION(DICT) == __pyx_dict_version)) {\
        (VAR) = __pyx_dict_cached_value;\
    } else {\
        (VAR) = __pyx_dict_cached_value = (LOOKUP);\
        __pyx_dict_version = __PYX_GET_DICT_VERSION(DICT);\
    }\
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj);
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj);
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version);
#else
#define __PYX_GET_DICT_VERSION(dict)  (0)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP)  (VAR) = (LOOKUP);
#endif

/* GetModuleGlobalName.proto */
#if CYTHON_USE_DICT_VERSIONS
#define __Pyx_GetModuleGlobalName(var, name)  {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    (var) = (likely(__pyx_dict_version == __PYX_GET_DICT_VERSION(__pyx_d))) ?\
        (likely(__pyx_dict_cached_value) ? __Pyx_NewRef(__pyx_dict_cached_value) : __Pyx_GetBuiltinName(name)) :\
        __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
}
#define __Pyx_GetModuleGlobalNameUncached(var, name)  {\
    PY_UINT64_T __pyx_dict_version;\
    PyObject *__pyx_dict_cached_value;\
    (var) = __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
}
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value);
#else
#define __Pyx_GetModuleGlobalName(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name);
#endif

/* RaiseDoubleKeywords.proto */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywords.proto */
static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[],\
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,\
    const char* function_name);

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* ArgTypeTest.proto */
#define __Pyx_ArgTypeTest(obj, type, none_allowed, name, exact)\
    ((likely((Py_TYPE(obj) == type) | (none_allowed && (obj == Py_None)))) ? 1 :\
        __Pyx__ArgTypeTest(obj, type, name, exact))
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact);

/* PySequenceContains.proto */
static CYTHON_INLINE int __Pyx_PySequence_ContainsTF(PyObject* item, PyObject* seq, int eq) {
    int result = PySequence_Contains(seq, item);
    return unlikely(result < 0) ? result : (result == (eq == Py_EQ));
}

/* CallableCheck.proto */
#if CYTHON_USE_TYPE_SLOTS && PY_MAJOR_VERSION >= 3
#define __Pyx_PyCallable_Check(obj)   (Py_TYPE(obj)->tp_call != NULL)
#else
#define __Pyx_PyCallable_Check(obj)   PyCallable_Check(obj)
#endif

/* PyCFunctionFastCall.proto */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject *__Pyx_PyCFunction_FastCall(PyObject *func, PyObject **args, Py_ssize_t nargs);
#else
#define __Pyx_PyCFunction_FastCall(func, args, nargs)  (assert(0), NULL)
#endif

/* PyFunctionFastCall.proto */
#if CYTHON_FAST_PYCALL
#define __Pyx_PyFunction_FastCall(func, args, nargs)\
    __Pyx_PyFunction_FastCallDict((func), (args), (nargs), NULL)
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs);
#else
#define __Pyx_PyFunction_FastCallDict(func, args, nargs, kwargs) _PyFunction_FastCallDict(func, args, nargs, kwargs)
#endif
#define __Pyx_BUILD_ASSERT_EXPR(cond)\
    (sizeof(char [1 - 2*!(cond)]) - 1)
#ifndef Py_MEMBER_SIZE
#define Py_MEMBER_SIZE(type, member) sizeof(((type *)0)->member)
#endif
  static size_t __pyx_pyframe_localsplus_offset = 0;
  #include "frameobject.h"
  #define __Pxy_PyFrame_Initialize_Offsets()\
    ((void)__Pyx_BUILD_ASSERT_EXPR(sizeof(PyFrameObject) == offsetof(PyFrameObject, f_localsplus) + Py_MEMBER_SIZE(PyFrameObject, f_localsplus)),\
     (void)(__pyx_pyframe_localsplus_offset = ((size_t)PyFrame_Type.tp_basicsize) - Py_MEMBER_SIZE(PyFrameObject, f_localsplus)))
  #define __Pyx_PyFrame_GetLocalsplus(frame)\
    (assert(__pyx_pyframe_localsplus_offset), (PyObject **)(((char *)(frame)) + __pyx_pyframe_localsplus_offset))
#endif

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* PyObjectCall2Args.proto */
static CYTHON_UNUSED PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2);

/* PyObjectCallMethO.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectCallOneArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* RaiseException.proto */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* UnpackUnboundCMethod.proto */
typedef struct {
    PyObject *type;
    PyObject **method_name;
    PyCFunction func;
    PyObject *method;
    int flag;
} __Pyx_CachedCFunction;

/* CallUnboundCMethod1.proto */
static PyObject* __Pyx__CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg);
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg);
#else
#define __Pyx_CallUnboundCMethod1(cfunc, self, arg)  __Pyx__CallUnboundCMethod1(cfunc, self, arg)
#endif

/* DictGetItem.proto */
#if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key);
#define __Pyx_PyObject_Dict_GetItem(obj, name)\
    (likely(PyDict_CheckExact(obj)) ?\
     __Pyx_PyDict_GetItem(obj, name) : PyObject_GetItem(obj, name))
#else
#define __Pyx_PyDict_GetItem(d, key) PyObject_GetItem(d, key)
#define __Pyx_PyObject_Dict_GetItem(obj, name)  PyObject_GetItem(obj, name)
#endif

/* GetTopmostException.proto */
#if CYTHON_USE_EXC_INFO_STACK
static _PyErr_StackItem * __Pyx_PyErr_GetTopmostException(PyThreadState *tstate);
#endif

/* SaveResetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSave(type, value, tb)  __Pyx__ExceptionSave(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#define __Pyx_ExceptionReset(type, value, tb)  __Pyx__ExceptionReset(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
#else
#define __Pyx_ExceptionSave(type, value, tb)   PyErr_GetExcInfo(type, value, tb)
#define __Pyx_ExceptionReset(type, value, tb)  PyErr_SetExcInfo(type, value, tb)
#endif

/* GetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_GetException(type, value, tb)  __Pyx__GetException(__pyx_tstate, type, value, tb)
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* PyObjectCallNoArg.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);
#else
#define __Pyx_PyObject_CallNoArg(func) __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL)
#endif

/* WriteUnraisableException.proto */
static void __Pyx_WriteUnraisable(const char *name, int clineno,
                                  int lineno, const char *filename,
                                  int full_traceback, int nogil);

/* RaiseTooManyValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected);

/* RaiseNeedMoreValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index);

/* RaiseNoneIterError.proto */
static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void);

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
#define __Pyx_PyErr_GivenExceptionMatches2(err, type1, type2) (PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2))
#endif
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)

/* ListExtend.proto */
static CYTHON_INLINE int __Pyx_PyList_Extend(PyObject* L, PyObject* v) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject* none = _PyList_Extend((PyListObject*)L, v);
    if (unlikely(!none))
        return -1;
    Py_DECREF(none);
    return 0;
#else
    return PyList_SetSlice(L, PY_SSIZE_T_MAX, PY_SSIZE_T_MAX, v);
#endif
}

/* KeywordStringCheck.proto */
static int __Pyx_CheckKeywordStrings(PyObject *kwdict, const char* function_name, int kw_allowed);

/* py_dict_pop.proto */
static CYTHON_INLINE PyObject *__Pyx_PyDict_Pop(PyObject *d, PyObject *key, PyObject *default_value);

/* CallUnboundCMethod2.proto */
static PyObject* __Pyx__CallUnboundCMethod2(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg1, PyObject* arg2);
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030600B1
static CYTHON_INLINE PyObject *__Pyx_CallUnboundCMethod2(__Pyx_CachedCFunction *cfunc, PyObject *self, PyObject *arg1, PyObject *arg2);
#else
#define __Pyx_CallUnboundCMethod2(cfunc, self, arg1, arg2)  __Pyx__CallUnboundCMethod2(cfunc, self, arg1, arg2)
#endif

/* IncludeStringH.proto */
#include <string.h>

/* BytesEquals.proto */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals);

/* UnicodeEquals.proto */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals);

/* StrEquals.proto */
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyString_Equals __Pyx_PyUnicode_Equals
#else
#define __Pyx_PyString_Equals __Pyx_PyBytes_Equals
#endif

/* IterFinish.proto */
static CYTHON_INLINE int __Pyx_IterFinish(void);

/* UnpackItemEndCheck.proto */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected);

/* ListCompAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_ListComp_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len)) {
        Py_INCREF(x);
        PyList_SET_ITEM(list, len, x);
        __Pyx_SET_SIZE(list, len + 1);
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_ListComp_Append(L,x) PyList_Append(L,x)
#endif

/* py_abs.proto */
#if CYTHON_USE_PYLONG_INTERNALS
static PyObject *__Pyx_PyLong_AbsNeg(PyObject *num);
#define __Pyx_PyNumber_Absolute(x)\
    ((likely(PyLong_CheckExact(x))) ?\
         (likely(Py_SIZE(x) >= 0) ? (Py_INCREF(x), (x)) : __Pyx_PyLong_AbsNeg(x)) :\
         PyNumber_Absolute(x))
#else
#define __Pyx_PyNumber_Absolute(x)  PyNumber_Absolute(x)
#endif

/* PyObjectSetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
#define __Pyx_PyObject_DelAttrStr(o,n) __Pyx_PyObject_SetAttrStr(o, n, NULL)
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value);
#else
#define __Pyx_PyObject_DelAttrStr(o,n)   PyObject_DelAttr(o,n)
#define __Pyx_PyObject_SetAttrStr(o,n,v) PyObject_SetAttr(o,n,v)
#endif

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck);

/* ObjectGetItem.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject* key);
#else
#define __Pyx_PyObject_GetItem(obj, key)  PyObject_GetItem(obj, key)
#endif

/* HasAttr.proto */
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *, PyObject *);

/* SliceObject.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetSlice(
        PyObject* obj, Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** py_start, PyObject** py_stop, PyObject** py_slice,
        int has_cstart, int has_cstop, int wraparound);

/* dict_getitem_default.proto */
static PyObject* __Pyx_PyDict_GetItemDefault(PyObject* d, PyObject* key, PyObject* default_value);

/* ListAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_PyList_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len) & likely(len > (L->allocated >> 1))) {
        Py_INCREF(x);
        PyList_SET_ITEM(list, len, x);
        __Pyx_SET_SIZE(list, len + 1);
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_PyList_Append(L,x) PyList_Append(L,x)
#endif

/* py_dict_items.proto */
static CYTHON_INLINE PyObject* __Pyx_PyDict_Items(PyObject* d);

/* CallUnboundCMethod0.proto */
static PyObject* __Pyx__CallUnboundCMethod0(__Pyx_CachedCFunction* cfunc, PyObject* self);
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_CallUnboundCMethod0(cfunc, self)\
    (likely((cfunc)->func) ?\
        (likely((cfunc)->flag == METH_NOARGS) ?  (*((cfunc)->func))(self, NULL) :\
         (PY_VERSION_HEX >= 0x030600B1 && likely((cfunc)->flag == METH_FASTCALL) ?\
            (PY_VERSION_HEX >= 0x030700A0 ?\
                (*(__Pyx_PyCFunctionFast)(void*)(PyCFunction)(cfunc)->func)(self, &__pyx_empty_tuple, 0) :\
                (*(__Pyx_PyCFunctionFastWithKeywords)(void*)(PyCFunction)(cfunc)->func)(self, &__pyx_empty_tuple, 0, NULL)) :\
          (PY_VERSION_HEX >= 0x030700A0 && (cfunc)->flag == (METH_FASTCALL | METH_KEYWORDS) ?\
            (*(__Pyx_PyCFunctionFastWithKeywords)(void*)(PyCFunction)(cfunc)->func)(self, &__pyx_empty_tuple, 0, NULL) :\
            (likely((cfunc)->flag == (METH_VARARGS | METH_KEYWORDS)) ?  ((*(PyCFunctionWithKeywords)(void*)(PyCFunction)(cfunc)->func)(self, __pyx_empty_tuple, NULL)) :\
               ((cfunc)->flag == METH_VARARGS ?  (*((cfunc)->func))(self, __pyx_empty_tuple) :\
               __Pyx__CallUnboundCMethod0(cfunc, self)))))) :\
        __Pyx__CallUnboundCMethod0(cfunc, self))
#else
#define __Pyx_CallUnboundCMethod0(cfunc, self)  __Pyx__CallUnboundCMethod0(cfunc, self)
#endif

/* PyDictContains.proto */
static CYTHON_INLINE int __Pyx_PyDict_ContainsTF(PyObject* item, PyObject* dict, int eq) {
    int result = PyDict_Contains(dict, item);
    return unlikely(result < 0) ? result : (result == (eq == Py_EQ));
}

/* Import.proto */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

/* ImportFrom.proto */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name);

/* PyObject_GenericGetAttrNoDict.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttrNoDict PyObject_GenericGetAttr
#endif

/* PyObject_GenericGetAttr.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttr PyObject_GenericGetAttr
#endif

/* PyObjectGetAttrStrNoError.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name);

/* SetupReduce.proto */
static int __Pyx_setup_reduce(PyObject* type_obj);

/* SetVTable.proto */
static int __Pyx_SetVtable(PyObject *dict, void *vtable);

/* CLineInTraceback.proto */
#ifdef CYTHON_CLINE_IN_TRACEBACK
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#else
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#endif

/* CodeObjectCache.proto */
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* GCCDiagnostics.proto */
#if defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6))
#define __Pyx_HAS_GCC_DIAGNOSTIC
#endif

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value);

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* CStringEquals.proto */
static CYTHON_INLINE int __Pyx_StrEq(const char *, const char *);

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(void);

/* VoidPtrExport.proto */
static int __Pyx_ExportVoidPtr(PyObject *name, void *p, const char *sig);

/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);

static PyObject *__pyx_f_9drf_turbo_6fields_5Field_serialize(CYTHON_UNUSED struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, PyObject *__pyx_v_value, CYTHON_UNUSED PyObject *__pyx_v_context, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_5Field_deserialize(CYTHON_UNUSED struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, PyObject *__pyx_v_data, CYTHON_UNUSED PyObject *__pyx_v_context, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_5Field_method_getter(CYTHON_UNUSED struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_field_name, CYTHON_UNUSED PyObject *__pyx_v_root, int __pyx_skip_dispatch); /* proto*/
static void __pyx_f_9drf_turbo_6fields_5Field_bind(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, PyObject *__pyx_v_field_name, PyObject *__pyx_v_root, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_5Field_get_default_value(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_5Field_get_initial(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_5Field_get_attribute(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, PyObject *__pyx_v_instance, int __pyx_skip_dispatch, struct __pyx_opt_args_9drf_turbo_6fields_5Field_get_attribute *__pyx_optional_args); /* proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_5Field_validate_empty_values(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, PyObject *__pyx_v_data, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_5Field_run_validation(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context, int __pyx_skip_dispatch); /* proto*/
static long __pyx_f_9drf_turbo_6fields_5Field_validate_or_raise(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, PyObject *__pyx_v_value, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_8StrField_serialize(CYTHON_UNUSED struct __pyx_obj_9drf_turbo_6fields_StrField *__pyx_v_self, PyObject *__pyx_v_value, CYTHON_UNUSED PyObject *__pyx_v_context, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_8StrField_deserialize(struct __pyx_obj_9drf_turbo_6fields_StrField *__pyx_v_self, PyObject *__pyx_v_data, CYTHON_UNUSED PyObject *__pyx_v_context, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_10EmailField_serialize(struct __pyx_obj_9drf_turbo_6fields_EmailField *__pyx_v_self, PyObject *__pyx_v_value, CYTHON_UNUSED PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_10EmailField_deserialize(struct __pyx_obj_9drf_turbo_6fields_EmailField *__pyx_v_self, PyObject *__pyx_v_data, CYTHON_UNUSED PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_8URLField_deserialize(struct __pyx_obj_9drf_turbo_6fields_URLField *__pyx_v_self, PyObject *__pyx_v_data, CYTHON_UNUSED PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_10RegexField_deserialize(struct __pyx_obj_9drf_turbo_6fields_RegexField *__pyx_v_self, PyObject *__pyx_v_data, CYTHON_UNUSED PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_7IPField_deserialize(struct __pyx_obj_9drf_turbo_6fields_IPField *__pyx_v_self, PyObject *__pyx_v_data, CYTHON_UNUSED PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_9UUIDField_deserialize(struct __pyx_obj_9drf_turbo_6fields_UUIDField *__pyx_v_self, PyObject *__pyx_v_data, CYTHON_UNUSED PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_9SlugField_deserialize(struct __pyx_obj_9drf_turbo_6fields_SlugField *__pyx_v_self, PyObject *__pyx_v_data, CYTHON_UNUSED PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_8IntField_serialize(CYTHON_UNUSED struct __pyx_obj_9drf_turbo_6fields_IntField *__pyx_v_self, PyObject *__pyx_v_value, CYTHON_UNUSED PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_8IntField_deserialize(struct __pyx_obj_9drf_turbo_6fields_IntField *__pyx_v_self, PyObject *__pyx_v_data, CYTHON_UNUSED PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_10FloatField_serialize(CYTHON_UNUSED struct __pyx_obj_9drf_turbo_6fields_FloatField *__pyx_v_self, PyObject *__pyx_v_value, CYTHON_UNUSED PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_10FloatField_deserialize(struct __pyx_obj_9drf_turbo_6fields_FloatField *__pyx_v_self, PyObject *__pyx_v_data, CYTHON_UNUSED PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_12DecimalField_serialize(struct __pyx_obj_9drf_turbo_6fields_DecimalField *__pyx_v_self, PyObject *__pyx_v_value, CYTHON_UNUSED PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_12DecimalField_deserialize(struct __pyx_obj_9drf_turbo_6fields_DecimalField *__pyx_v_self, PyObject *__pyx_v_data, CYTHON_UNUSED PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_12DecimalField_validate_precision(struct __pyx_obj_9drf_turbo_6fields_DecimalField *__pyx_v_self, PyObject *__pyx_v_value); /* proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_12DecimalField_quantize(struct __pyx_obj_9drf_turbo_6fields_DecimalField *__pyx_v_self, PyObject *__pyx_v_value); /* proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_9BoolField_serialize(struct __pyx_obj_9drf_turbo_6fields_BoolField *__pyx_v_self, PyObject *__pyx_v_value, CYTHON_UNUSED PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_9BoolField_deserialize(struct __pyx_obj_9drf_turbo_6fields_BoolField *__pyx_v_self, PyObject *__pyx_v_data, CYTHON_UNUSED PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_11ChoiceField_serialize(struct __pyx_obj_9drf_turbo_6fields_ChoiceField *__pyx_v_self, PyObject *__pyx_v_value, CYTHON_UNUSED PyObject *__pyx_v_context, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_11ChoiceField_deserialize(struct __pyx_obj_9drf_turbo_6fields_ChoiceField *__pyx_v_self, PyObject *__pyx_v_data, CYTHON_UNUSED PyObject *__pyx_v_context, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_19MultipleChoiceField_serialize(struct __pyx_obj_9drf_turbo_6fields_MultipleChoiceField *__pyx_v_self, PyObject *__pyx_v_value, CYTHON_UNUSED PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_19MultipleChoiceField_deserialize(struct __pyx_obj_9drf_turbo_6fields_MultipleChoiceField *__pyx_v_self, PyObject *__pyx_v_data, CYTHON_UNUSED PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_13DateTimeField_get_default_timezone(CYTHON_UNUSED struct __pyx_obj_9drf_turbo_6fields_DateTimeField *__pyx_v_self, CYTHON_UNUSED int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_13DateTimeField_enforce_timezone(struct __pyx_obj_9drf_turbo_6fields_DateTimeField *__pyx_v_self, PyObject *__pyx_v_value, CYTHON_UNUSED int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_13DateTimeField_serialize(struct __pyx_obj_9drf_turbo_6fields_DateTimeField *__pyx_v_self, PyObject *__pyx_v_value, CYTHON_UNUSED PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_13DateTimeField_deserialize(struct __pyx_obj_9drf_turbo_6fields_DateTimeField *__pyx_v_self, PyObject *__pyx_v_data, CYTHON_UNUSED PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_9DateField_serialize(struct __pyx_obj_9drf_turbo_6fields_DateField *__pyx_v_self, PyObject *__pyx_v_value, CYTHON_UNUSED PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_9DateField_deserialize(struct __pyx_obj_9drf_turbo_6fields_DateField *__pyx_v_self, PyObject *__pyx_v_data, CYTHON_UNUSED PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_9TimeField_serialize(struct __pyx_obj_9drf_turbo_6fields_TimeField *__pyx_v_self, PyObject *__pyx_v_value, CYTHON_UNUSED PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_9TimeField_deserialize(struct __pyx_obj_9drf_turbo_6fields_TimeField *__pyx_v_self, PyObject *__pyx_v_data, CYTHON_UNUSED PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_9FileField_serialize(CYTHON_UNUSED struct __pyx_obj_9drf_turbo_6fields_FileField *__pyx_v_self, PyObject *__pyx_v_value, PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_9FileField_deserialize(struct __pyx_obj_9drf_turbo_6fields_FileField *__pyx_v_self, PyObject *__pyx_v_value, CYTHON_UNUSED PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_10ArrayField_serialize(struct __pyx_obj_9drf_turbo_6fields_ArrayField *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_10ArrayField_deserialize(struct __pyx_obj_9drf_turbo_6fields_ArrayField *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_10ArrayField_run_child_validation(struct __pyx_obj_9drf_turbo_6fields_ArrayField *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_9DictField_serialize(struct __pyx_obj_9drf_turbo_6fields_DictField *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_9DictField_deserialize(struct __pyx_obj_9drf_turbo_6fields_DictField *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_9DictField_run_child_validation(struct __pyx_obj_9drf_turbo_6fields_DictField *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_9JSONField_serialize(struct __pyx_obj_9drf_turbo_6fields_JSONField *__pyx_v_self, PyObject *__pyx_v_value, CYTHON_UNUSED PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_9JSONField_deserialize(struct __pyx_obj_9drf_turbo_6fields_JSONField *__pyx_v_self, PyObject *__pyx_v_data, CYTHON_UNUSED PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_12RelatedField_deserialize(struct __pyx_obj_9drf_turbo_6fields_RelatedField *__pyx_v_self, PyObject *__pyx_v_data, CYTHON_UNUSED PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_16ManyRelatedField_serialize(CYTHON_UNUSED struct __pyx_obj_9drf_turbo_6fields_ManyRelatedField *__pyx_v_self, PyObject *__pyx_v_value, CYTHON_UNUSED PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_16ManyRelatedField_deserialize(struct __pyx_obj_9drf_turbo_6fields_ManyRelatedField *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_13ConstantField_deserialize(struct __pyx_obj_9drf_turbo_6fields_ConstantField *__pyx_v_self, PyObject *__pyx_v_data, CYTHON_UNUSED PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_14RecursiveField_serialize(struct __pyx_obj_9drf_turbo_6fields_RecursiveField *__pyx_v_self, PyObject *__pyx_v_value, CYTHON_UNUSED PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_11MethodField_method_getter(struct __pyx_obj_9drf_turbo_6fields_MethodField *__pyx_v_self, PyObject *__pyx_v_field_name, PyObject *__pyx_v_root, CYTHON_UNUSED int __pyx_skip_dispatch); /* proto*/

/* Module declarations from 'cython' */

/* Module declarations from 'drf_turbo.fields' */
static PyTypeObject *__pyx_ptype_9drf_turbo_6fields_SkipField = 0;
static PyTypeObject *__pyx_ptype_9drf_turbo_6fields_Field = 0;
static PyTypeObject *__pyx_ptype_9drf_turbo_6fields_StrField = 0;
static PyTypeObject *__pyx_ptype_9drf_turbo_6fields_EmailField = 0;
static PyTypeObject *__pyx_ptype_9drf_turbo_6fields_URLField = 0;
static PyTypeObject *__pyx_ptype_9drf_turbo_6fields_RegexField = 0;
static PyTypeObject *__pyx_ptype_9drf_turbo_6fields_IPField = 0;
static PyTypeObject *__pyx_ptype_9drf_turbo_6fields_PasswordField = 0;
static PyTypeObject *__pyx_ptype_9drf_turbo_6fields_UUIDField = 0;
static PyTypeObject *__pyx_ptype_9drf_turbo_6fields_SlugField = 0;
static PyTypeObject *__pyx_ptype_9drf_turbo_6fields_IntField = 0;
static PyTypeObject *__pyx_ptype_9drf_turbo_6fields_FloatField = 0;
static PyTypeObject *__pyx_ptype_9drf_turbo_6fields_DecimalField = 0;
static PyTypeObject *__pyx_ptype_9drf_turbo_6fields_BoolField = 0;
static PyTypeObject *__pyx_ptype_9drf_turbo_6fields_ChoiceField = 0;
static PyTypeObject *__pyx_ptype_9drf_turbo_6fields_MultipleChoiceField = 0;
static PyTypeObject *__pyx_ptype_9drf_turbo_6fields_DateTimeField = 0;
static PyTypeObject *__pyx_ptype_9drf_turbo_6fields_DateField = 0;
static PyTypeObject *__pyx_ptype_9drf_turbo_6fields_TimeField = 0;
static PyTypeObject *__pyx_ptype_9drf_turbo_6fields_FileField = 0;
static PyTypeObject *__pyx_ptype_9drf_turbo_6fields_ArrayField = 0;
static PyTypeObject *__pyx_ptype_9drf_turbo_6fields_DictField = 0;
static PyTypeObject *__pyx_ptype_9drf_turbo_6fields_JSONField = 0;
static PyTypeObject *__pyx_ptype_9drf_turbo_6fields_RelatedField = 0;
static PyTypeObject *__pyx_ptype_9drf_turbo_6fields_ManyRelatedField = 0;
static PyTypeObject *__pyx_ptype_9drf_turbo_6fields_ConstantField = 0;
static PyTypeObject *__pyx_ptype_9drf_turbo_6fields_RecursiveField = 0;
static PyTypeObject *__pyx_ptype_9drf_turbo_6fields_MethodField = 0;
static PyObject *__pyx_v_9drf_turbo_6fields_NO_DEFAULT = 0;
static PyObject *__pyx_f_9drf_turbo_6fields___pyx_unpickle_SkipField__set_state(struct __pyx_obj_9drf_turbo_6fields_SkipField *, PyObject *); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields___pyx_unpickle_Field__set_state(struct __pyx_obj_9drf_turbo_6fields_Field *, PyObject *); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields___pyx_unpickle_StrField__set_state(struct __pyx_obj_9drf_turbo_6fields_StrField *, PyObject *); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields___pyx_unpickle_EmailField__set_state(struct __pyx_obj_9drf_turbo_6fields_EmailField *, PyObject *); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields___pyx_unpickle_URLField__set_state(struct __pyx_obj_9drf_turbo_6fields_URLField *, PyObject *); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields___pyx_unpickle_RegexField__set_state(struct __pyx_obj_9drf_turbo_6fields_RegexField *, PyObject *); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields___pyx_unpickle_IPField__set_state(struct __pyx_obj_9drf_turbo_6fields_IPField *, PyObject *); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields___pyx_unpickle_PasswordField__set_state(struct __pyx_obj_9drf_turbo_6fields_PasswordField *, PyObject *); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields___pyx_unpickle_UUIDField__set_state(struct __pyx_obj_9drf_turbo_6fields_UUIDField *, PyObject *); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields___pyx_unpickle_SlugField__set_state(struct __pyx_obj_9drf_turbo_6fields_SlugField *, PyObject *); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields___pyx_unpickle_IntField__set_state(struct __pyx_obj_9drf_turbo_6fields_IntField *, PyObject *); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields___pyx_unpickle_FloatField__set_state(struct __pyx_obj_9drf_turbo_6fields_FloatField *, PyObject *); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields___pyx_unpickle_DecimalField__set_state(struct __pyx_obj_9drf_turbo_6fields_DecimalField *, PyObject *); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields___pyx_unpickle_BoolField__set_state(struct __pyx_obj_9drf_turbo_6fields_BoolField *, PyObject *); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields___pyx_unpickle_ChoiceField__set_state(struct __pyx_obj_9drf_turbo_6fields_ChoiceField *, PyObject *); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields___pyx_unpickle_MultipleChoiceField__set_state(struct __pyx_obj_9drf_turbo_6fields_MultipleChoiceField *, PyObject *); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields___pyx_unpickle_DateTimeField__set_state(struct __pyx_obj_9drf_turbo_6fields_DateTimeField *, PyObject *); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields___pyx_unpickle_DateField__set_state(struct __pyx_obj_9drf_turbo_6fields_DateField *, PyObject *); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields___pyx_unpickle_TimeField__set_state(struct __pyx_obj_9drf_turbo_6fields_TimeField *, PyObject *); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields___pyx_unpickle_FileField__set_state(struct __pyx_obj_9drf_turbo_6fields_FileField *, PyObject *); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields___pyx_unpickle_ArrayField__set_state(struct __pyx_obj_9drf_turbo_6fields_ArrayField *, PyObject *); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields___pyx_unpickle_DictField__set_state(struct __pyx_obj_9drf_turbo_6fields_DictField *, PyObject *); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields___pyx_unpickle_JSONField__set_state(struct __pyx_obj_9drf_turbo_6fields_JSONField *, PyObject *); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields___pyx_unpickle_RelatedField__set_state(struct __pyx_obj_9drf_turbo_6fields_RelatedField *, PyObject *); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields___pyx_unpickle_ManyRelatedField__set_state(struct __pyx_obj_9drf_turbo_6fields_ManyRelatedField *, PyObject *); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields___pyx_unpickle_ConstantField__set_state(struct __pyx_obj_9drf_turbo_6fields_ConstantField *, PyObject *); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields___pyx_unpickle_RecursiveField__set_state(struct __pyx_obj_9drf_turbo_6fields_RecursiveField *, PyObject *); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields___pyx_unpickle_MethodField__set_state(struct __pyx_obj_9drf_turbo_6fields_MethodField *, PyObject *); /*proto*/
#define __Pyx_MODULE_NAME "drf_turbo.fields"
extern int __pyx_module_is_main_drf_turbo__fields;
int __pyx_module_is_main_drf_turbo__fields = 0;

/* Implementation of 'drf_turbo.fields' */
static PyObject *__pyx_builtin_object;
static PyObject *__pyx_builtin_ValueError;
static PyObject *__pyx_builtin_reversed;
static PyObject *__pyx_builtin_KeyError;
static PyObject *__pyx_builtin_AssertionError;
static PyObject *__pyx_builtin_AttributeError;
static PyObject *__pyx_builtin_super;
static PyObject *__pyx_builtin_TypeError;
static PyObject *__pyx_builtin_vars;
static PyObject *__pyx_builtin_all;
static PyObject *__pyx_builtin_OverflowError;
static const char __pyx_k_0[] = "0";
static const char __pyx_k_1[] = ".1";
static const char __pyx_k_F[] = "F";
static const char __pyx_k_T[] = "T";
static const char __pyx_k_Z[] = "Z";
static const char __pyx_k_f[] = "{:f}";
static const char __pyx_k_t[] = "t";
static const char __pyx_k__3[] = ".";
static const char __pyx_k__5[] = "_";
static const char __pyx_k__6[] = " ";
static const char __pyx_k__9[] = "";
static const char __pyx_k_on[] = "on";
static const char __pyx_k_pk[] = "pk";
static const char __pyx_k_re[] = "re";
static const char __pyx_k_0_s[] = "\\.0*\\s*$";
static const char __pyx_k_1_2[] = "1";
static const char __pyx_k_Inf[] = "Inf";
static const char __pyx_k__12[] = "\000";
static const char __pyx_k__31[] = "*";
static const char __pyx_k_all[] = "all";
static const char __pyx_k_cls[] = "cls";
static const char __pyx_k_f_2[] = "f";
static const char __pyx_k_get[] = "get";
static const char __pyx_k_hex[] = "hex";
static const char __pyx_k_int[] = "int";
static const char __pyx_k_key[] = "key";
static const char __pyx_k_mro[] = "__mro__";
static const char __pyx_k_new[] = "__new__";
static const char __pyx_k_off[] = "off";
static const char __pyx_k_pop[] = "pop";
static const char __pyx_k_sub[] = "sub";
static const char __pyx_k_url[] = "url";
static const char __pyx_k_utc[] = "utc";
static const char __pyx_k_w_Z[] = "^[-\\w]+\\Z";
static const char __pyx_k_NONE[] = "NONE";
static const char __pyx_k_NULL[] = "NULL";
static const char __pyx_k_None[] = "None";
static const char __pyx_k_Null[] = "Null";
static const char __pyx_k_TRUE[] = "TRUE";
static const char __pyx_k_True[] = "True";
static const char __pyx_k_UUID[] = "UUID";
static const char __pyx_k_attr[] = "attr";
static const char __pyx_k_bind[] = "bind";
static const char __pyx_k_call[] = "call";
static const char __pyx_k_copy[] = "copy";
static const char __pyx_k_data[] = "data";
static const char __pyx_k_date[] = "date";
static const char __pyx_k_dict[] = "__dict__";
static const char __pyx_k_init[] = "__init__";
static const char __pyx_k_iter[] = "__iter__";
static const char __pyx_k_json[] = "json";
static const char __pyx_k_keys[] = "keys";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_many[] = "many";
static const char __pyx_k_name[] = "__name__";
static const char __pyx_k_none[] = "none";
static const char __pyx_k_null[] = "null";
static const char __pyx_k_only[] = "only";
static const char __pyx_k_prec[] = "prec";
static const char __pyx_k_root[] = "root";
static const char __pyx_k_size[] = "size";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_time[] = "time";
static const char __pyx_k_true[] = "true";
static const char __pyx_k_uuid[] = "uuid";
static const char __pyx_k_vars[] = "vars";
static const char __pyx_k_00_00[] = "+00:00";
static const char __pyx_k_FALSE[] = "FALSE";
static const char __pyx_k_False[] = "False";
static const char __pyx_k_Field[] = "Field";
static const char __pyx_k_Inf_2[] = "-Inf";
static const char __pyx_k_ROUND[] = "ROUND_";
static const char __pyx_k_blank[] = "blank";
static const char __pyx_k_bytes[] = "bytes";
static const char __pyx_k_child[] = "child";
static const char __pyx_k_class[] = "__class__";
static const char __pyx_k_dumps[] = "dumps";
static const char __pyx_k_empty[] = "empty";
static const char __pyx_k_false[] = "false";
static const char __pyx_k_flags[] = "flags";
static const char __pyx_k_input[] = "input";
static const char __pyx_k_items[] = "items";
static const char __pyx_k_label[] = "label";
static const char __pyx_k_loads[] = "loads";
static const char __pyx_k_lower[] = "lower";
static const char __pyx_k_match[] = "match";
static const char __pyx_k_regex[] = "regex";
static const char __pyx_k_split[] = "split";
static const char __pyx_k_strip[] = "strip";
static const char __pyx_k_style[] = "style";
static const char __pyx_k_super[] = "super";
static const char __pyx_k_value[] = "value";
static const char __pyx_k_USE_TZ[] = "USE_TZ";
static const char __pyx_k_binary[] = "binary";
static const char __pyx_k_decode[] = "decode";
static const char __pyx_k_detail[] = "detail";
static const char __pyx_k_encode[] = "encode";
static const char __pyx_k_format[] = "format";
static const char __pyx_k_get__0[] = "get_{0}";
static const char __pyx_k_import[] = "__import__";
static const char __pyx_k_is_nan[] = "is_nan";
static const char __pyx_k_length[] = "length";
static const char __pyx_k_name_2[] = "name";
static const char __pyx_k_object[] = "object";
static const char __pyx_k_pickle[] = "pickle";
static const char __pyx_k_reduce[] = "__reduce__";
static const char __pyx_k_search[] = "search";
static const char __pyx_k_update[] = "update";
static const char __pyx_k_Decimal[] = "Decimal";
static const char __pyx_k_IPField[] = "IPField";
static const char __pyx_k_UNICODE[] = "UNICODE";
static const char __pyx_k_choices[] = "choices";
static const char __pyx_k_compile[] = "compile";
static const char __pyx_k_context[] = "context";
static const char __pyx_k_decimal[] = "decimal";
static const char __pyx_k_decoder[] = "decoder";
static const char __pyx_k_display[] = "display";
static const char __pyx_k_encoder[] = "encoder";
static const char __pyx_k_exclude[] = "exclude";
static const char __pyx_k_initial[] = "initial";
static const char __pyx_k_invalid[] = "invalid";
static const char __pyx_k_no_name[] = "no_name";
static const char __pyx_k_partial[] = "partial";
static const char __pyx_k_replace[] = "replace";
static const char __pyx_k_request[] = "request";
static const char __pyx_k_ISO_8601[] = "ISO_8601";
static const char __pyx_k_IntField[] = "IntField";
static const char __pyx_k_KeyError[] = "KeyError";
static const char __pyx_k_StrField[] = "StrField";
static const char __pyx_k_URLField[] = "URLField";
static const char __pyx_k_as_tuple[] = "as_tuple";
static const char __pyx_k_constant[] = "constant";
static const char __pyx_k_datetime[] = "datetime";
static const char __pyx_k_deepcopy[] = "deepcopy";
static const char __pyx_k_endswith[] = "endswith";
static const char __pyx_k_exc_type[] = "exc_type";
static const char __pyx_k_getstate[] = "__getstate__";
static const char __pyx_k_instance[] = "instance";
static const char __pyx_k_is_aware[] = "is_aware";
static const char __pyx_k_overflow[] = "overflow";
static const char __pyx_k_pk_value[] = "pk_value";
static const char __pyx_k_pyx_capi[] = "__pyx_capi__";
static const char __pyx_k_pyx_type[] = "__pyx_type";
static const char __pyx_k_quantize[] = "quantize";
static const char __pyx_k_queryset[] = "queryset";
static const char __pyx_k_required[] = "required";
static const char __pyx_k_reversed[] = "reversed";
static const char __pyx_k_rounding[] = "rounding";
static const char __pyx_k_setstate[] = "__setstate__";
static const char __pyx_k_settings[] = "settings";
static const char __pyx_k_strftime[] = "strftime";
static const char __pyx_k_strptime[] = "strptime";
static const char __pyx_k_timezone[] = "timezone";
static const char __pyx_k_to_lower[] = "to_lower";
static const char __pyx_k_BoolField[] = "BoolField";
static const char __pyx_k_DateField[] = "DateField";
static const char __pyx_k_DictField[] = "DictField";
static const char __pyx_k_FileField[] = "FileField";
static const char __pyx_k_JSONField[] = "JSONField";
static const char __pyx_k_SkipField[] = "SkipField";
static const char __pyx_k_SlugField[] = "SlugField";
static const char __pyx_k_TimeField[] = "TimeField";
static const char __pyx_k_TypeError[] = "TypeError";
static const char __pyx_k_URL_REGEX[] = "URL_REGEX";
static const char __pyx_k_UUIDField[] = "UUIDField";
static const char __pyx_k_a_zA_Z0_9[] = "^[-a-zA-Z0-9_]+$";
static const char __pyx_k_data_type[] = "data_type";
static const char __pyx_k_force_str[] = "force_str";
static const char __pyx_k_help_text[] = "help_text";
static const char __pyx_k_initial_2[] = "_initial";
static const char __pyx_k_ipaddress[] = "ipaddress";
static const char __pyx_k_isoformat[] = "isoformat";
static const char __pyx_k_max_items[] = "max_items";
static const char __pyx_k_max_value[] = "max_value";
static const char __pyx_k_min_items[] = "min_items";
static const char __pyx_k_min_value[] = "min_value";
static const char __pyx_k_pyx_state[] = "__pyx_state";
static const char __pyx_k_read_only[] = "read_only";
static const char __pyx_k_reduce_ex[] = "__reduce_ex__";
static const char __pyx_k_serialize[] = "serialize";
static const char __pyx_k_ArrayField[] = "ArrayField";
static const char __pyx_k_EmailField[] = "EmailField";
static const char __pyx_k_FloatField[] = "FloatField";
static const char __pyx_k_IGNORECASE[] = "IGNORECASE";
static const char __pyx_k_NO_DEFAULT[] = "NO_DEFAULT";
static const char __pyx_k_RegexField[] = "RegexField";
static const char __pyx_k_ValueError[] = "ValueError";
static const char __pyx_k_allow_null[] = "allow_null";
static const char __pyx_k_astimezone[] = "astimezone";
static const char __pyx_k_capitalize[] = "capitalize";
static const char __pyx_k_field_name[] = "field_name";
static const char __pyx_k_getcontext[] = "getcontext";
static const char __pyx_k_input_type[] = "input_type";
static const char __pyx_k_ip_address[] = "ip_address";
static const char __pyx_k_make_aware[] = "make_aware";
static const char __pyx_k_make_naive[] = "make_naive";
static const char __pyx_k_max_digits[] = "max_digits";
static const char __pyx_k_max_length[] = "max_length";
static const char __pyx_k_min_length[] = "min_length";
static const char __pyx_k_not_a_dict[] = "not_a_dict";
static const char __pyx_k_not_a_list[] = "not_a_list";
static const char __pyx_k_parse_date[] = "parse_date";
static const char __pyx_k_parse_time[] = "parse_time";
static const char __pyx_k_pyx_result[] = "__pyx_result";
static const char __pyx_k_pyx_vtable[] = "__pyx_vtable__";
static const char __pyx_k_re_decimal[] = "re_decimal";
static const char __pyx_k_startswith[] = "startswith";
static const char __pyx_k_validators[] = "validators";
static const char __pyx_k_write_only[] = "write_only";
static const char __pyx_k_ChoiceField[] = "ChoiceField";
static const char __pyx_k_DATE_FORMAT[] = "DATE_FORMAT";
static const char __pyx_k_EMAIL_REGEX[] = "EMAIL_REGEX";
static const char __pyx_k_MethodField[] = "MethodField";
static const char __pyx_k_PickleError[] = "PickleError";
static const char __pyx_k_allow_blank[] = "allow_blank";
static const char __pyx_k_allow_empty[] = "allow_empty";
static const char __pyx_k_deserialize[] = "deserialize";
static const char __pyx_k_django_conf[] = "django.conf";
static const char __pyx_k_exact_items[] = "exact_items";
static const char __pyx_k_get_initial[] = "get_initial";
static const char __pyx_k_method_name[] = "method_name";
static const char __pyx_k_DecimalField[] = "DecimalField";
static const char __pyx_k_Must_be_None[] = "Must be None.";
static const char __pyx_k_RelatedField[] = "RelatedField";
static const char __pyx_k_api_settings[] = "api_settings";
static const char __pyx_k_django_utils[] = "django.utils";
static const char __pyx_k_pyx_checksum[] = "__pyx_checksum";
static const char __pyx_k_stringsource[] = "stringsource";
static const char __pyx_k_ConstantField[] = "ConstantField";
static const char __pyx_k_DateTimeField[] = "DateTimeField";
static const char __pyx_k_OverflowError[] = "OverflowError";
static const char __pyx_k_PasswordField[] = "PasswordField";
static const char __pyx_k_allow_unicode[] = "allow_unicode";
static const char __pyx_k_coerce_values[] = "coerce_values";
static const char __pyx_k_default_value[] = "default_value";
static const char __pyx_k_get_attribute[] = "get_attribute";
static const char __pyx_k_input_formats[] = "input_formats";
static const char __pyx_k_is_collection[] = "is_collection";
static const char __pyx_k_method_getter[] = "method_getter";
static const char __pyx_k_raise_if_fail[] = "raise_if_fail";
static const char __pyx_k_reduce_cython[] = "__reduce_cython__";
static const char __pyx_k_AssertionError[] = "AssertionError";
static const char __pyx_k_AttributeError[] = "AttributeError";
static const char __pyx_k_RecursiveField[] = "RecursiveField";
static const char __pyx_k_child_relation[] = "child_relation";
static const char __pyx_k_decimal_places[] = "decimal_places";
static const char __pyx_k_does_not_exist[] = "does_not_exist";
static const char __pyx_k_error_messages[] = "error_messages";
static const char __pyx_k_incorrect_type[] = "incorrect_type";
static const char __pyx_k_invalid_choice[] = "invalid_choice";
static const char __pyx_k_parse_datetime[] = "parse_datetime";
static const char __pyx_k_rest_framework[] = "rest_framework";
static const char __pyx_k_run_validation[] = "run_validation";
static const char __pyx_k_DATETIME_FORMAT[] = "DATETIME_FORMAT";
static const char __pyx_k_ValidationError[] = "ValidationError";
static const char __pyx_k_datetime_parser[] = "datetime_parser";
static const char __pyx_k_drf_turbo_utils[] = "drf_turbo.utils";
static const char __pyx_k_invalid_unicode[] = "invalid_unicode";
static const char __pyx_k_is_method_field[] = "is_method_field";
static const char __pyx_k_pytz_exceptions[] = "pytz.exceptions";
static const char __pyx_k_pyx_PickleError[] = "__pyx_PickleError";
static const char __pyx_k_setstate_cython[] = "__setstate_cython__";
static const char __pyx_k_trim_whitespace[] = "trim_whitespace";
static const char __pyx_k_DecimalException[] = "DecimalException";
static const char __pyx_k_InvalidTimeError[] = "InvalidTimeError";
static const char __pyx_k_ManyRelatedField[] = "ManyRelatedField";
static const char __pyx_k_May_not_be_blank[] = "May not be blank.";
static const char __pyx_k_Must_be_constant[] = "Must be \"{constant}\".";
static const char __pyx_k_Not_a_valid_JSON[] = "Not a valid JSON.";
static const char __pyx_k_Not_a_valid_UUID[] = "Not a valid UUID.";
static const char __pyx_k_Not_a_valid_date[] = "Not a valid date.";
static const char __pyx_k_Not_a_valid_slug[] = "Not a valid slug.";
static const char __pyx_k_Not_a_valid_time[] = "Not a valid time.";
static const char __pyx_k_allow_empty_file[] = "allow_empty_file";
static const char __pyx_k_coerce_to_string[] = "coerce_to_string";
static const char __pyx_k_default_timezone[] = "default_timezone";
static const char __pyx_k_drf_turbo_fields[] = "drf_turbo.fields";
static const char __pyx_k_get_error_detail[] = "get_error_detail";
static const char __pyx_k_max_whole_digits[] = "max_whole_digits";
static const char __pyx_k_Enter_a_valid_URL[] = "Enter a valid URL.";
static const char __pyx_k_MAX_STRING_LENGTH[] = "MAX_STRING_LENGTH";
static const char __pyx_k_get_default_value[] = "get_default_value";
static const char __pyx_k_max_string_length[] = "max_string_length";
static const char __pyx_k_validate_or_raise[] = "validate_or_raise";
static const char __pyx_k_DATE_INPUT_FORMATS[] = "DATE_INPUT_FORMATS";
static const char __pyx_k_Not_a_valid_string[] = "Not a valid string.";
static const char __pyx_k_ObjectDoesNotExist[] = "ObjectDoesNotExist";
static const char __pyx_k_TIME_INPUT_FORMATS[] = "TIME_INPUT_FORMATS";
static const char __pyx_k_build_absolute_uri[] = "build_absolute_uri";
static const char __pyx_k_cline_in_traceback[] = "cline_in_traceback";
static const char __pyx_k_coerce_null_values[] = "coerce_null_values";
static const char __pyx_k_max_decimal_places[] = "max_decimal_places";
static const char __pyx_k_pyx_unpickle_Field[] = "__pyx_unpickle_Field";
static const char __pyx_k_MultipleChoiceField[] = "MultipleChoiceField";
static const char __pyx_k_Not_a_valid_boolean[] = "Not a valid boolean.";
static const char __pyx_k_OnlyAndExcludeError[] = "OnlyAndExcludeError";
static const char __pyx_k_Not_a_valid_datetime[] = "Not a valid datetime.";
static const char __pyx_k_drf_turbo_exceptions[] = "drf_turbo.exceptions";
static const char __pyx_k_get_current_timezone[] = "get_current_timezone";
static const char __pyx_k_pyx_unpickle_IPField[] = "__pyx_unpickle_IPField";
static const char __pyx_k_DjangoValidationError[] = "DjangoValidationError";
static const char __pyx_k_No_file_was_submitted[] = "No file was submitted.";
static const char __pyx_k_pyx_unpickle_IntField[] = "__pyx_unpickle_IntField";
static const char __pyx_k_pyx_unpickle_StrField[] = "__pyx_unpickle_StrField";
static const char __pyx_k_pyx_unpickle_URLField[] = "__pyx_unpickle_URLField";
static const char __pyx_k_validate_empty_values[] = "validate_empty_values";
static const char __pyx_k_DATETIME_INPUT_FORMATS[] = "DATETIME_INPUT_FORMATS";
static const char __pyx_k_String_value_too_large[] = "String value too large.";
static const char __pyx_k_This_field_is_required[] = "This field is required.";
static const char __pyx_k_default_error_messages[] = "default_error_messages";
static const char __pyx_k_django_core_exceptions[] = "django.core.exceptions";
static const char __pyx_k_django_utils_dateparse[] = "django.utils.dateparse";
static const char __pyx_k_pyx_unpickle_BoolField[] = "__pyx_unpickle_BoolField";
static const char __pyx_k_pyx_unpickle_DateField[] = "__pyx_unpickle_DateField";
static const char __pyx_k_pyx_unpickle_DictField[] = "__pyx_unpickle_DictField";
static const char __pyx_k_pyx_unpickle_FileField[] = "__pyx_unpickle_FileField";
static const char __pyx_k_pyx_unpickle_JSONField[] = "__pyx_unpickle_JSONField";
static const char __pyx_k_pyx_unpickle_SkipField[] = "__pyx_unpickle_SkipField";
static const char __pyx_k_pyx_unpickle_SlugField[] = "__pyx_unpickle_SlugField";
static const char __pyx_k_pyx_unpickle_TimeField[] = "__pyx_unpickle_TimeField";
static const char __pyx_k_pyx_unpickle_UUIDField[] = "__pyx_unpickle_UUIDField";
static const char __pyx_k_pyx_unpickle_ArrayField[] = "__pyx_unpickle_ArrayField";
static const char __pyx_k_pyx_unpickle_EmailField[] = "__pyx_unpickle_EmailField";
static const char __pyx_k_pyx_unpickle_FloatField[] = "__pyx_unpickle_FloatField";
static const char __pyx_k_pyx_unpickle_RegexField[] = "__pyx_unpickle_RegexField";
static const char __pyx_k_rest_framework_settings[] = "rest_framework.settings";
static const char __pyx_k_COERCE_DECIMAL_TO_STRING[] = "COERCE_DECIMAL_TO_STRING";
static const char __pyx_k_StringNotCollectionError[] = "StringNotCollectionError";
static const char __pyx_k_pyx_unpickle_ChoiceField[] = "__pyx_unpickle_ChoiceField";
static const char __pyx_k_pyx_unpickle_MethodField[] = "__pyx_unpickle_MethodField";
static const char __pyx_k_Nnot_a_valid_unicode_slug[] = "Nnot a valid unicode slug.";
static const char __pyx_k_pyx_unpickle_DecimalField[] = "__pyx_unpickle_DecimalField";
static const char __pyx_k_pyx_unpickle_RelatedField[] = "__pyx_unpickle_RelatedField";
static const char __pyx_k_A_valid_number_is_required[] = "A valid number is required.";
static const char __pyx_k_This_dict_may_not_be_empty[] = "This dict may not be empty.";
static const char __pyx_k_This_field_may_not_be_null[] = "This field may not be null.";
static const char __pyx_k_This_list_may_not_be_empty[] = "This list may not be empty.";
static const char __pyx_k_is_iterable_and_not_string[] = "is_iterable_and_not_string";
static const char __pyx_k_pyx_unpickle_ConstantField[] = "__pyx_unpickle_ConstantField";
static const char __pyx_k_pyx_unpickle_DateTimeField[] = "__pyx_unpickle_DateTimeField";
static const char __pyx_k_pyx_unpickle_PasswordField[] = "__pyx_unpickle_PasswordField";
static const char __pyx_k_A_valid_integer_is_required[] = "A valid integer is required.";
static const char __pyx_k_Datetime_value_out_of_range[] = "Datetime value out of range.";
static const char __pyx_k_Enter_a_valid_email_address[] = "Enter a valid email address.";
static const char __pyx_k_Must_have_exact_items_items[] = "Must have {exact_items} items.";
static const char __pyx_k_The_submitted_file_is_empty[] = "The submitted file is empty.";
static const char __pyx_k_input_is_not_a_valid_choice[] = "\"{input}\" is not a valid choice.";
static const char __pyx_k_pyx_unpickle_RecursiveField[] = "__pyx_unpickle_RecursiveField";
static const char __pyx_k_0_9A_Z___0_9A_Z_001_010_013_0[] = "(^[-!#$%&'*+/=?^_`{}|~0-9A-Z]+(\\.[-!#$%&'*+/=?^_`{}|~0-9A-Z]+)*|^\"([\\001-\\010\\013\\014\\016-\\037!#-\\[\\]-\\177]|\\\\[\\001-\\011\\013\\014\\016-\\177])*)@(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\\.)+[A-Z]{2,63}(?<!-)\\.?$";
static const char __pyx_k_pyx_unpickle_ManyRelatedField[] = "__pyx_unpickle_ManyRelatedField";
static const char __pyx_k_pyx_unpickle_MultipleChoiceFie[] = "__pyx_unpickle_MultipleChoiceField";
static const char __pyx_k_Got_exc_type_when_attempting_to[] = "Got {exc_type} when attempting to get a value for field";
static const char __pyx_k_Invalid_pk_pk_value_object_does[] = "Invalid pk \"{pk_value}\" - object does not exist.";
static const char __pyx_k_Invalid_rounding_option_s_Valid[] = "Invalid rounding option %s. Valid values for rounding are: %s";
static const char __pyx_k_No_filename_could_be_determined[] = "No filename could be determined.";
static const char __pyx_k_Null_characters_are_not_allowed[] = "Null characters are not allowed.";
static const char __pyx_k_This_selection_may_not_be_empty[] = "This selection may not be empty.";
static const char __pyx_k_exclude_should_be_a_list_of_str[] = "\"exclude\" should be a list of strings";
static const char __pyx_k_http_ftp_s_A_Z0_9_A_Z0_9_0_61_A[] = "^(?:http|ftp)s?://(?:(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\\.)+(?:[A-Z]{2,6}\\.?|[A-Z0-9-]{2,}\\.?)|localhost|\\d{1,3}\\.\\d{1,3}\\.\\d{1,3}\\.\\d{1,3})(?::\\d+)?(?:/?|[/?]\\S+)$";
static const char __pyx_k_only_should_be_a_list_of_string[] = "\"only\" should be a list of strings";
static const char __pyx_k_Ensure_that_there_are_no_more_th[] = "Ensure that there are no more than {max_digits} digits in total.";
static const char __pyx_k_Ensure_this_filename_has_at_most[] = "Ensure this filename has at most {max_length} characters (it has {length}).";
static const char __pyx_k_Enter_a_valid_IPv4_or_IPv6_addre[] = "Enter a valid IPv4 or IPv6 address.";
static const char __pyx_k_Expected_a_date_but_got_a_dateti[] = "Expected a `date`, but got a `datetime`. Refusing to coerce, as this may mean losing timezone information. Use a custom read-only field and deal with timezone issues explicitly.";
static const char __pyx_k_Expected_a_datetime_but_got_a_da[] = "Expected a datetime but got a date.";
static const char __pyx_k_Expected_a_dict_of_items_but_got[] = "Expected a dict of items but got type \"{input_type}\".";
static const char __pyx_k_Expected_a_list_of_items_but_got[] = "Expected a list of items but got type \"{input_type}\".";
static const char __pyx_k_Expected_a_time_but_got_a_dateti[] = "Expected a `time`, but got a `datetime`. Refusing to coerce, as this may mean losing timezone information. Use a custom read-only field and deal with timezone issues explicitly.";
static const char __pyx_k_Incompatible_checksums_s_vs_0x03[] = "Incompatible checksums (%s vs 0x033cda3 = (allow_empty, allow_null, attr, attrs, call, child, default_value, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))";
static const char __pyx_k_Incompatible_checksums_s_vs_0x0c[] = "Incompatible checksums (%s vs 0x0ca6ac6 = (allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, queryset, read_only, required, root, style, validators, write_only))";
static const char __pyx_k_Incompatible_checksums_s_vs_0x0f[] = "Incompatible checksums (%s vs 0x0fbedf5 = (allow_blank, allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_length, min_length, read_only, required, root, style, trim_whitespace, validators, write_only))";
static const char __pyx_k_Incompatible_checksums_s_vs_0x36[] = "Incompatible checksums (%s vs 0x369fe29 = (allow_null, attr, attrs, call, constant, default_value, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))";
static const char __pyx_k_Incompatible_checksums_s_vs_0x38[] = "Incompatible checksums (%s vs 0x380e494 = (allow_blank, allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_length, min_length, read_only, regex, required, root, style, trim_whitespace, validators, write_only))";
static const char __pyx_k_Incompatible_checksums_s_vs_0x4f[] = "Incompatible checksums (%s vs 0x4f70d27 = (allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))";
static const char __pyx_k_Incompatible_checksums_s_vs_0x57[] = "Incompatible checksums (%s vs 0x5775012 = (allow_empty, allow_null, attr, attrs, call, child, default_value, error_messages, exact_items, field_name, help_text, initial, label, max_items, min_items, read_only, required, root, style, validators, write_only))";
static const char __pyx_k_Incompatible_checksums_s_vs_0x5e[] = "Incompatible checksums (%s vs 0x5e76b7e = (allow_null, attr, attrs, call, default_timezone, default_value, error_messages, field_name, format, help_text, initial, input_formats, label, read_only, required, root, style, timezone, validators, write_only))";
static const char __pyx_k_Incompatible_checksums_s_vs_0x66[] = "Incompatible checksums (%s vs 0x66626e5 = (allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_value, min_value, read_only, required, root, style, validators, write_only))";
static const char __pyx_k_Incompatible_checksums_s_vs_0x69[] = "Incompatible checksums (%s vs 0x69fcb1b = (allow_blank, allow_empty, allow_null, attr, attrs, call, choice_strings_to_display, choice_strings_to_values, choices, default_value, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))";
static const char __pyx_k_Incompatible_checksums_s_vs_0x72[] = "Incompatible checksums (%s vs 0x7207e31 = (allow_null, allow_unicode, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))";
static const char __pyx_k_Incompatible_checksums_s_vs_0x91[] = "Incompatible checksums (%s vs 0x91e774b = (allow_null, attr, attrs, binary, call, decoder, default_value, encoder, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))";
static const char __pyx_k_Incompatible_checksums_s_vs_0xc7[] = "Incompatible checksums (%s vs 0xc705305 = (allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, method_name, read_only, required, root, style, validators, write_only))";
static const char __pyx_k_Incompatible_checksums_s_vs_0xd4[] = "Incompatible checksums (%s vs 0xd41d8cd = ())";
static const char __pyx_k_Incompatible_checksums_s_vs_0xda[] = "Incompatible checksums (%s vs 0xdaed7b0 = (allow_null, attr, attrs, call, coerce_to_string, decimal_places, default_value, error_messages, field_name, help_text, initial, label, max_digits, max_value, max_whole_digits, min_value, read_only, required, root, rounding, style, validators, write_only))";
static const char __pyx_k_Incompatible_checksums_s_vs_0xe1[] = "Incompatible checksums (%s vs 0xe1e0ed9 = (allow_blank, allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_length, min_length, read_only, required, root, style, to_lower, trim_whitespace, validators, write_only))";
static const char __pyx_k_Incompatible_checksums_s_vs_0xe8[] = "Incompatible checksums (%s vs 0xe82609b = (allow_empty_file, allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_length, read_only, required, root, style, validators, write_only))";
static const char __pyx_k_Incompatible_checksums_s_vs_0xf1[] = "Incompatible checksums (%s vs 0xf172868 = (allow_blank, allow_null, attr, attrs, call, choice_strings_to_display, choice_strings_to_values, choices, default_value, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))";
static const char __pyx_k_Incompatible_checksums_s_vs_0xf3[] = "Incompatible checksums (%s vs 0xf36a8d7 = (allow_null, attr, attrs, call, default_value, error_messages, field_name, format, help_text, initial, input_formats, label, read_only, required, root, style, validators, write_only))";
static const char __pyx_k_Incompatible_checksums_s_vs_0xf4[] = "Incompatible checksums (%s vs 0xf4706f5 = (allow_empty, allow_null, attr, attrs, call, child_relation, default_value, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))";
static const char __pyx_k_Incompatible_checksums_s_vs_0xf7[] = "Incompatible checksums (%s vs 0xf725730 = (allow_null, attr, attrs, call, context, default_value, error_messages, exclude, field_name, help_text, initial, label, many, only, read_only, required, root, style, validators, write_only))";
static const char __pyx_k_Incorrect_type_Expected_pk_value[] = "Incorrect type. Expected pk value, received {data_type}.";
static const char __pyx_k_Invalid_datetime_for_the_timezon[] = "Invalid datetime for the timezone \"{timezone}\".";
static const char __pyx_k_May_not_set_both_read_only_and_w[] = "May not set both `read_only` and `write_only`";
static const char __pyx_k_May_not_set_both_required_and_de[] = "May not set both `required` and `default_value`";
static const char __pyx_k_Must_be_greater_than_or_equal_to[] = "Must be greater than or equal to {min_value}.";
static const char __pyx_k_Must_be_less_than_or_equal_to_ma[] = "Must be less than or equal to {max_value}.";
static const char __pyx_k_Must_have_at_least_min_items_ite[] = "Must have at least {min_items} items.";
static const char __pyx_k_Must_have_at_least_min_length_ch[] = "Must have at least {min_length} characters.";
static const char __pyx_k_Must_have_no_more_than_max_items[] = "Must have no more than {max_items} items.";
static const char __pyx_k_Must_have_no_more_than_max_lengt[] = "Must have no more than {max_length} characters.";
static const char __pyx_k_The_submitted_data_was_not_a_fil[] = "The submitted data was not a file. Check the encoding type on the form.";
static const char __pyx_k_The_validators_parameter_must_be[] = "The 'validators' parameter must be a callable or a collection of callables.";
static const char __pyx_k_This_value_does_not_match_the_re[] = "This value does not match the required pattern.";
static const char __pyx_k_You_should_use_either_only_or_ex[] = "You should use either \"only\" or \"exclude\"";
static const char __pyx_k_Ensure_that_there_are_no_more_th_2[] = "Ensure that there are no more than {max_decimal_places} decimal places.";
static const char __pyx_k_Ensure_that_there_are_no_more_th_3[] = "Ensure that there are no more than {max_whole_digits} digits before the decimal point.";
static const char __pyx_k_Expected_a_date_but_got_a_dateti_2[] = "Expected a date but got a datetime.";
static PyObject *__pyx_kp_s_0;
static PyObject *__pyx_kp_s_00_00;
static PyObject *__pyx_kp_s_0_9A_Z___0_9A_Z_001_010_013_0;
static PyObject *__pyx_kp_s_0_s;
static PyObject *__pyx_kp_s_1;
static PyObject *__pyx_kp_s_1_2;
static PyObject *__pyx_kp_s_A_valid_integer_is_required;
static PyObject *__pyx_kp_s_A_valid_number_is_required;
static PyObject *__pyx_n_s_ArrayField;
static PyObject *__pyx_n_s_AssertionError;
static PyObject *__pyx_n_s_AttributeError;
static PyObject *__pyx_n_s_BoolField;
static PyObject *__pyx_n_s_COERCE_DECIMAL_TO_STRING;
static PyObject *__pyx_n_s_ChoiceField;
static PyObject *__pyx_n_s_ConstantField;
static PyObject *__pyx_n_s_DATETIME_FORMAT;
static PyObject *__pyx_n_s_DATETIME_INPUT_FORMATS;
static PyObject *__pyx_n_s_DATE_FORMAT;
static PyObject *__pyx_n_s_DATE_INPUT_FORMATS;
static PyObject *__pyx_n_s_DateField;
static PyObject *__pyx_n_s_DateTimeField;
static PyObject *__pyx_kp_s_Datetime_value_out_of_range;
static PyObject *__pyx_n_s_Decimal;
static PyObject *__pyx_n_s_DecimalException;
static PyObject *__pyx_n_s_DecimalField;
static PyObject *__pyx_n_s_DictField;
static PyObject *__pyx_n_s_DjangoValidationError;
static PyObject *__pyx_n_s_EMAIL_REGEX;
static PyObject *__pyx_n_s_EmailField;
static PyObject *__pyx_kp_s_Ensure_that_there_are_no_more_th;
static PyObject *__pyx_kp_s_Ensure_that_there_are_no_more_th_2;
static PyObject *__pyx_kp_s_Ensure_that_there_are_no_more_th_3;
static PyObject *__pyx_kp_s_Ensure_this_filename_has_at_most;
static PyObject *__pyx_kp_s_Enter_a_valid_IPv4_or_IPv6_addre;
static PyObject *__pyx_kp_s_Enter_a_valid_URL;
static PyObject *__pyx_kp_s_Enter_a_valid_email_address;
static PyObject *__pyx_kp_s_Expected_a_date_but_got_a_dateti;
static PyObject *__pyx_kp_s_Expected_a_date_but_got_a_dateti_2;
static PyObject *__pyx_kp_s_Expected_a_datetime_but_got_a_da;
static PyObject *__pyx_kp_s_Expected_a_dict_of_items_but_got;
static PyObject *__pyx_kp_s_Expected_a_list_of_items_but_got;
static PyObject *__pyx_kp_s_Expected_a_time_but_got_a_dateti;
static PyObject *__pyx_n_s_F;
static PyObject *__pyx_n_s_FALSE;
static PyObject *__pyx_n_s_False;
static PyObject *__pyx_n_s_Field;
static PyObject *__pyx_n_s_FileField;
static PyObject *__pyx_n_s_FloatField;
static PyObject *__pyx_kp_s_Got_exc_type_when_attempting_to;
static PyObject *__pyx_n_s_IGNORECASE;
static PyObject *__pyx_n_s_IPField;
static PyObject *__pyx_n_s_ISO_8601;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0x03;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0x0c;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0x0f;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0x36;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0x38;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0x4f;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0x57;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0x5e;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0x66;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0x69;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0x72;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0x91;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0xc7;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0xd4;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0xda;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0xe1;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0xe8;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0xf1;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0xf3;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0xf4;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0xf7;
static PyObject *__pyx_kp_s_Incorrect_type_Expected_pk_value;
static PyObject *__pyx_n_s_Inf;
static PyObject *__pyx_kp_s_Inf_2;
static PyObject *__pyx_n_s_IntField;
static PyObject *__pyx_n_s_InvalidTimeError;
static PyObject *__pyx_kp_s_Invalid_datetime_for_the_timezon;
static PyObject *__pyx_kp_s_Invalid_pk_pk_value_object_does;
static PyObject *__pyx_kp_s_Invalid_rounding_option_s_Valid;
static PyObject *__pyx_n_s_JSONField;
static PyObject *__pyx_n_s_KeyError;
static PyObject *__pyx_n_s_MAX_STRING_LENGTH;
static PyObject *__pyx_n_s_ManyRelatedField;
static PyObject *__pyx_kp_s_May_not_be_blank;
static PyObject *__pyx_kp_s_May_not_set_both_read_only_and_w;
static PyObject *__pyx_kp_s_May_not_set_both_required_and_de;
static PyObject *__pyx_n_s_MethodField;
static PyObject *__pyx_n_s_MultipleChoiceField;
static PyObject *__pyx_kp_s_Must_be_None;
static PyObject *__pyx_kp_s_Must_be_constant;
static PyObject *__pyx_kp_s_Must_be_greater_than_or_equal_to;
static PyObject *__pyx_kp_s_Must_be_less_than_or_equal_to_ma;
static PyObject *__pyx_kp_s_Must_have_at_least_min_items_ite;
static PyObject *__pyx_kp_s_Must_have_at_least_min_length_ch;
static PyObject *__pyx_kp_s_Must_have_exact_items_items;
static PyObject *__pyx_kp_s_Must_have_no_more_than_max_items;
static PyObject *__pyx_kp_s_Must_have_no_more_than_max_lengt;
static PyObject *__pyx_n_s_NONE;
static PyObject *__pyx_n_s_NO_DEFAULT;
static PyObject *__pyx_n_s_NULL;
static PyObject *__pyx_kp_s_Nnot_a_valid_unicode_slug;
static PyObject *__pyx_kp_s_No_file_was_submitted;
static PyObject *__pyx_kp_s_No_filename_could_be_determined;
static PyObject *__pyx_n_s_None;
static PyObject *__pyx_kp_s_Not_a_valid_JSON;
static PyObject *__pyx_kp_s_Not_a_valid_UUID;
static PyObject *__pyx_kp_s_Not_a_valid_boolean;
static PyObject *__pyx_kp_s_Not_a_valid_date;
static PyObject *__pyx_kp_s_Not_a_valid_datetime;
static PyObject *__pyx_kp_s_Not_a_valid_slug;
static PyObject *__pyx_kp_s_Not_a_valid_string;
static PyObject *__pyx_kp_s_Not_a_valid_time;
static PyObject *__pyx_n_s_Null;
static PyObject *__pyx_kp_s_Null_characters_are_not_allowed;
static PyObject *__pyx_n_s_ObjectDoesNotExist;
static PyObject *__pyx_n_s_OnlyAndExcludeError;
static PyObject *__pyx_n_s_OverflowError;
static PyObject *__pyx_n_s_PasswordField;
static PyObject *__pyx_n_s_PickleError;
static PyObject *__pyx_n_s_ROUND;
static PyObject *__pyx_n_s_RecursiveField;
static PyObject *__pyx_n_s_RegexField;
static PyObject *__pyx_n_s_RelatedField;
static PyObject *__pyx_n_s_SkipField;
static PyObject *__pyx_n_s_SlugField;
static PyObject *__pyx_n_s_StrField;
static PyObject *__pyx_n_s_StringNotCollectionError;
static PyObject *__pyx_kp_s_String_value_too_large;
static PyObject *__pyx_n_s_T;
static PyObject *__pyx_n_s_TIME_INPUT_FORMATS;
static PyObject *__pyx_n_s_TRUE;
static PyObject *__pyx_kp_s_The_submitted_data_was_not_a_fil;
static PyObject *__pyx_kp_s_The_submitted_file_is_empty;
static PyObject *__pyx_kp_s_The_validators_parameter_must_be;
static PyObject *__pyx_kp_s_This_dict_may_not_be_empty;
static PyObject *__pyx_kp_s_This_field_is_required;
static PyObject *__pyx_kp_s_This_field_may_not_be_null;
static PyObject *__pyx_kp_s_This_list_may_not_be_empty;
static PyObject *__pyx_kp_s_This_selection_may_not_be_empty;
static PyObject *__pyx_kp_s_This_value_does_not_match_the_re;
static PyObject *__pyx_n_s_TimeField;
static PyObject *__pyx_n_s_True;
static PyObject *__pyx_n_s_TypeError;
static PyObject *__pyx_n_s_UNICODE;
static PyObject *__pyx_n_s_URLField;
static PyObject *__pyx_n_s_URL_REGEX;
static PyObject *__pyx_n_s_USE_TZ;
static PyObject *__pyx_n_s_UUID;
static PyObject *__pyx_n_s_UUIDField;
static PyObject *__pyx_n_s_ValidationError;
static PyObject *__pyx_n_s_ValueError;
static PyObject *__pyx_kp_s_You_should_use_either_only_or_ex;
static PyObject *__pyx_n_s_Z;
static PyObject *__pyx_kp_s__12;
static PyObject *__pyx_kp_s__3;
static PyObject *__pyx_n_s__31;
static PyObject *__pyx_n_s__5;
static PyObject *__pyx_kp_s__6;
static PyObject *__pyx_kp_s__9;
static PyObject *__pyx_kp_s_a_zA_Z0_9;
static PyObject *__pyx_n_s_all;
static PyObject *__pyx_n_s_allow_blank;
static PyObject *__pyx_n_s_allow_empty;
static PyObject *__pyx_n_s_allow_empty_file;
static PyObject *__pyx_n_s_allow_null;
static PyObject *__pyx_n_s_allow_unicode;
static PyObject *__pyx_n_s_api_settings;
static PyObject *__pyx_n_s_as_tuple;
static PyObject *__pyx_n_s_astimezone;
static PyObject *__pyx_n_s_attr;
static PyObject *__pyx_n_s_binary;
static PyObject *__pyx_n_s_bind;
static PyObject *__pyx_n_s_blank;
static PyObject *__pyx_n_s_build_absolute_uri;
static PyObject *__pyx_n_s_bytes;
static PyObject *__pyx_n_s_call;
static PyObject *__pyx_n_s_capitalize;
static PyObject *__pyx_n_s_child;
static PyObject *__pyx_n_s_child_relation;
static PyObject *__pyx_n_s_choices;
static PyObject *__pyx_n_s_class;
static PyObject *__pyx_n_s_cline_in_traceback;
static PyObject *__pyx_n_s_cls;
static PyObject *__pyx_n_s_coerce_null_values;
static PyObject *__pyx_n_s_coerce_to_string;
static PyObject *__pyx_n_s_coerce_values;
static PyObject *__pyx_n_s_compile;
static PyObject *__pyx_n_s_constant;
static PyObject *__pyx_n_s_context;
static PyObject *__pyx_n_s_copy;
static PyObject *__pyx_n_s_data;
static PyObject *__pyx_n_s_data_type;
static PyObject *__pyx_n_s_date;
static PyObject *__pyx_n_s_datetime;
static PyObject *__pyx_n_s_datetime_parser;
static PyObject *__pyx_n_s_decimal;
static PyObject *__pyx_n_s_decimal_places;
static PyObject *__pyx_n_s_decode;
static PyObject *__pyx_n_s_decoder;
static PyObject *__pyx_n_s_deepcopy;
static PyObject *__pyx_n_s_default_error_messages;
static PyObject *__pyx_n_s_default_timezone;
static PyObject *__pyx_n_s_default_value;
static PyObject *__pyx_n_s_deserialize;
static PyObject *__pyx_n_s_detail;
static PyObject *__pyx_n_s_dict;
static PyObject *__pyx_n_s_display;
static PyObject *__pyx_n_s_django_conf;
static PyObject *__pyx_n_s_django_core_exceptions;
static PyObject *__pyx_n_s_django_utils;
static PyObject *__pyx_n_s_django_utils_dateparse;
static PyObject *__pyx_n_s_does_not_exist;
static PyObject *__pyx_n_s_drf_turbo_exceptions;
static PyObject *__pyx_n_s_drf_turbo_fields;
static PyObject *__pyx_n_s_drf_turbo_utils;
static PyObject *__pyx_n_s_dumps;
static PyObject *__pyx_n_s_empty;
static PyObject *__pyx_n_s_encode;
static PyObject *__pyx_n_s_encoder;
static PyObject *__pyx_n_s_endswith;
static PyObject *__pyx_n_s_error_messages;
static PyObject *__pyx_n_s_exact_items;
static PyObject *__pyx_n_s_exc_type;
static PyObject *__pyx_n_s_exclude;
static PyObject *__pyx_kp_s_exclude_should_be_a_list_of_str;
static PyObject *__pyx_kp_s_f;
static PyObject *__pyx_n_s_f_2;
static PyObject *__pyx_n_s_false;
static PyObject *__pyx_n_s_field_name;
static PyObject *__pyx_n_s_flags;
static PyObject *__pyx_n_s_force_str;
static PyObject *__pyx_n_s_format;
static PyObject *__pyx_n_s_get;
static PyObject *__pyx_kp_s_get__0;
static PyObject *__pyx_n_s_get_attribute;
static PyObject *__pyx_n_s_get_current_timezone;
static PyObject *__pyx_n_s_get_default_value;
static PyObject *__pyx_n_s_get_error_detail;
static PyObject *__pyx_n_s_get_initial;
static PyObject *__pyx_n_s_getcontext;
static PyObject *__pyx_n_s_getstate;
static PyObject *__pyx_n_s_help_text;
static PyObject *__pyx_n_s_hex;
static PyObject *__pyx_kp_s_http_ftp_s_A_Z0_9_A_Z0_9_0_61_A;
static PyObject *__pyx_n_s_import;
static PyObject *__pyx_n_s_incorrect_type;
static PyObject *__pyx_n_s_init;
static PyObject *__pyx_n_s_initial;
static PyObject *__pyx_n_s_initial_2;
static PyObject *__pyx_n_s_input;
static PyObject *__pyx_n_s_input_formats;
static PyObject *__pyx_kp_s_input_is_not_a_valid_choice;
static PyObject *__pyx_n_s_input_type;
static PyObject *__pyx_n_s_instance;
static PyObject *__pyx_n_s_int;
static PyObject *__pyx_n_s_invalid;
static PyObject *__pyx_n_s_invalid_choice;
static PyObject *__pyx_n_s_invalid_unicode;
static PyObject *__pyx_n_s_ip_address;
static PyObject *__pyx_n_s_ipaddress;
static PyObject *__pyx_n_s_is_aware;
static PyObject *__pyx_n_s_is_collection;
static PyObject *__pyx_n_s_is_iterable_and_not_string;
static PyObject *__pyx_n_s_is_method_field;
static PyObject *__pyx_n_s_is_nan;
static PyObject *__pyx_n_s_isoformat;
static PyObject *__pyx_n_s_items;
static PyObject *__pyx_n_s_iter;
static PyObject *__pyx_n_s_json;
static PyObject *__pyx_n_s_key;
static PyObject *__pyx_n_s_keys;
static PyObject *__pyx_n_s_label;
static PyObject *__pyx_n_s_length;
static PyObject *__pyx_n_s_loads;
static PyObject *__pyx_n_s_lower;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_n_s_make_aware;
static PyObject *__pyx_n_s_make_naive;
static PyObject *__pyx_n_s_many;
static PyObject *__pyx_n_s_match;
static PyObject *__pyx_n_s_max_decimal_places;
static PyObject *__pyx_n_s_max_digits;
static PyObject *__pyx_n_s_max_items;
static PyObject *__pyx_n_s_max_length;
static PyObject *__pyx_n_s_max_string_length;
static PyObject *__pyx_n_s_max_value;
static PyObject *__pyx_n_s_max_whole_digits;
static PyObject *__pyx_n_s_method_getter;
static PyObject *__pyx_n_s_method_name;
static PyObject *__pyx_n_s_min_items;
static PyObject *__pyx_n_s_min_length;
static PyObject *__pyx_n_s_min_value;
static PyObject *__pyx_n_s_mro;
static PyObject *__pyx_n_s_name;
static PyObject *__pyx_n_s_name_2;
static PyObject *__pyx_n_s_new;
static PyObject *__pyx_n_s_no_name;
static PyObject *__pyx_n_s_none;
static PyObject *__pyx_n_s_not_a_dict;
static PyObject *__pyx_n_s_not_a_list;
static PyObject *__pyx_n_s_null;
static PyObject *__pyx_n_s_object;
static PyObject *__pyx_n_s_off;
static PyObject *__pyx_n_s_on;
static PyObject *__pyx_n_s_only;
static PyObject *__pyx_kp_s_only_should_be_a_list_of_string;
static PyObject *__pyx_n_s_overflow;
static PyObject *__pyx_n_s_parse_date;
static PyObject *__pyx_n_s_parse_datetime;
static PyObject *__pyx_n_s_parse_time;
static PyObject *__pyx_n_s_partial;
static PyObject *__pyx_n_s_pickle;
static PyObject *__pyx_n_s_pk;
static PyObject *__pyx_n_s_pk_value;
static PyObject *__pyx_n_s_pop;
static PyObject *__pyx_n_s_prec;
static PyObject *__pyx_n_s_pytz_exceptions;
static PyObject *__pyx_n_s_pyx_PickleError;
static PyObject *__pyx_n_s_pyx_capi;
static PyObject *__pyx_n_s_pyx_checksum;
static PyObject *__pyx_n_s_pyx_result;
static PyObject *__pyx_n_s_pyx_state;
static PyObject *__pyx_n_s_pyx_type;
static PyObject *__pyx_n_s_pyx_unpickle_ArrayField;
static PyObject *__pyx_n_s_pyx_unpickle_BoolField;
static PyObject *__pyx_n_s_pyx_unpickle_ChoiceField;
static PyObject *__pyx_n_s_pyx_unpickle_ConstantField;
static PyObject *__pyx_n_s_pyx_unpickle_DateField;
static PyObject *__pyx_n_s_pyx_unpickle_DateTimeField;
static PyObject *__pyx_n_s_pyx_unpickle_DecimalField;
static PyObject *__pyx_n_s_pyx_unpickle_DictField;
static PyObject *__pyx_n_s_pyx_unpickle_EmailField;
static PyObject *__pyx_n_s_pyx_unpickle_Field;
static PyObject *__pyx_n_s_pyx_unpickle_FileField;
static PyObject *__pyx_n_s_pyx_unpickle_FloatField;
static PyObject *__pyx_n_s_pyx_unpickle_IPField;
static PyObject *__pyx_n_s_pyx_unpickle_IntField;
static PyObject *__pyx_n_s_pyx_unpickle_JSONField;
static PyObject *__pyx_n_s_pyx_unpickle_ManyRelatedField;
static PyObject *__pyx_n_s_pyx_unpickle_MethodField;
static PyObject *__pyx_n_s_pyx_unpickle_MultipleChoiceFie;
static PyObject *__pyx_n_s_pyx_unpickle_PasswordField;
static PyObject *__pyx_n_s_pyx_unpickle_RecursiveField;
static PyObject *__pyx_n_s_pyx_unpickle_RegexField;
static PyObject *__pyx_n_s_pyx_unpickle_RelatedField;
static PyObject *__pyx_n_s_pyx_unpickle_SkipField;
static PyObject *__pyx_n_s_pyx_unpickle_SlugField;
static PyObject *__pyx_n_s_pyx_unpickle_StrField;
static PyObject *__pyx_n_s_pyx_unpickle_TimeField;
static PyObject *__pyx_n_s_pyx_unpickle_URLField;
static PyObject *__pyx_n_s_pyx_unpickle_UUIDField;
static PyObject *__pyx_n_s_pyx_vtable;
static PyObject *__pyx_n_s_quantize;
static PyObject *__pyx_n_s_queryset;
static PyObject *__pyx_n_s_raise_if_fail;
static PyObject *__pyx_n_s_re;
static PyObject *__pyx_n_s_re_decimal;
static PyObject *__pyx_n_s_read_only;
static PyObject *__pyx_n_s_reduce;
static PyObject *__pyx_n_s_reduce_cython;
static PyObject *__pyx_n_s_reduce_ex;
static PyObject *__pyx_n_s_regex;
static PyObject *__pyx_n_s_replace;
static PyObject *__pyx_n_s_request;
static PyObject *__pyx_n_s_required;
static PyObject *__pyx_n_s_rest_framework;
static PyObject *__pyx_n_s_rest_framework_settings;
static PyObject *__pyx_n_s_reversed;
static PyObject *__pyx_n_s_root;
static PyObject *__pyx_n_s_rounding;
static PyObject *__pyx_n_s_run_validation;
static PyObject *__pyx_n_s_search;
static PyObject *__pyx_n_s_serialize;
static PyObject *__pyx_n_s_setstate;
static PyObject *__pyx_n_s_setstate_cython;
static PyObject *__pyx_n_s_settings;
static PyObject *__pyx_n_s_size;
static PyObject *__pyx_n_s_split;
static PyObject *__pyx_n_s_startswith;
static PyObject *__pyx_n_s_strftime;
static PyObject *__pyx_kp_s_stringsource;
static PyObject *__pyx_n_s_strip;
static PyObject *__pyx_n_s_strptime;
static PyObject *__pyx_n_s_style;
static PyObject *__pyx_n_s_sub;
static PyObject *__pyx_n_s_super;
static PyObject *__pyx_n_s_t;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_n_s_time;
static PyObject *__pyx_n_s_timezone;
static PyObject *__pyx_n_s_to_lower;
static PyObject *__pyx_n_s_trim_whitespace;
static PyObject *__pyx_n_s_true;
static PyObject *__pyx_n_s_update;
static PyObject *__pyx_n_s_url;
static PyObject *__pyx_n_s_utc;
static PyObject *__pyx_n_s_uuid;
static PyObject *__pyx_n_s_validate_empty_values;
static PyObject *__pyx_n_s_validate_or_raise;
static PyObject *__pyx_n_s_validators;
static PyObject *__pyx_n_s_value;
static PyObject *__pyx_n_s_vars;
static PyObject *__pyx_kp_s_w_Z;
static PyObject *__pyx_n_s_write_only;
static PyObject *__pyx_pf_9drf_turbo_6fields_9SkipField___reduce_cython__(struct __pyx_obj_9drf_turbo_6fields_SkipField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_9SkipField_2__setstate_cython__(struct __pyx_obj_9drf_turbo_6fields_SkipField *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_9drf_turbo_6fields_5Field___init__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, PyObject *__pyx_v_attr, int __pyx_v_call, int __pyx_v_required, int __pyx_v_write_only, int __pyx_v_read_only, int __pyx_v_allow_null, PyObject *__pyx_v_label, PyObject *__pyx_v_help_text, PyObject *__pyx_v_style, PyObject *__pyx_v_validators, PyObject *__pyx_v_default_value, PyObject *__pyx_v_initial, PyObject *__pyx_v_field_name, PyObject *__pyx_v_root, PyObject *__pyx_v_error_messages); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_5Field_2raise_if_fail(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_kwargs); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_5Field_4serialize(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, PyObject *__pyx_v_value, PyObject *__pyx_v_context); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_5Field_6deserialize(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_5Field_8method_getter(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, PyObject *__pyx_v_field_name, PyObject *__pyx_v_root); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_5Field_10bind(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, PyObject *__pyx_v_field_name, PyObject *__pyx_v_root); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_5Field_12get_default_value(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_5Field_14get_initial(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_5Field_16get_attribute(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, PyObject *__pyx_v_instance, PyObject *__pyx_v_attr); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_5Field_18validate_empty_values(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, PyObject *__pyx_v_data); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_5Field_20run_validation(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_5Field_22validate_or_raise(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_5Field_4attr___get__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self); /* proto */
static int __pyx_pf_9drf_turbo_6fields_5Field_4attr_2__set__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9drf_turbo_6fields_5Field_4attr_4__del__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_5Field_4call___get__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self); /* proto */
static int __pyx_pf_9drf_turbo_6fields_5Field_4call_2__set__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_5Field_8required___get__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self); /* proto */
static int __pyx_pf_9drf_turbo_6fields_5Field_8required_2__set__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_5Field_10write_only___get__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self); /* proto */
static int __pyx_pf_9drf_turbo_6fields_5Field_10write_only_2__set__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_5Field_9read_only___get__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self); /* proto */
static int __pyx_pf_9drf_turbo_6fields_5Field_9read_only_2__set__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_5Field_10allow_null___get__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self); /* proto */
static int __pyx_pf_9drf_turbo_6fields_5Field_10allow_null_2__set__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_5Field_5label___get__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self); /* proto */
static int __pyx_pf_9drf_turbo_6fields_5Field_5label_2__set__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9drf_turbo_6fields_5Field_5label_4__del__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_5Field_13default_value___get__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self); /* proto */
static int __pyx_pf_9drf_turbo_6fields_5Field_13default_value_2__set__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9drf_turbo_6fields_5Field_13default_value_4__del__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_5Field_7initial___get__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self); /* proto */
static int __pyx_pf_9drf_turbo_6fields_5Field_7initial_2__set__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9drf_turbo_6fields_5Field_7initial_4__del__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_5Field_9help_text___get__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self); /* proto */
static int __pyx_pf_9drf_turbo_6fields_5Field_9help_text_2__set__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9drf_turbo_6fields_5Field_9help_text_4__del__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_5Field_5style___get__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self); /* proto */
static int __pyx_pf_9drf_turbo_6fields_5Field_5style_2__set__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9drf_turbo_6fields_5Field_5style_4__del__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_5Field_10field_name___get__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self); /* proto */
static int __pyx_pf_9drf_turbo_6fields_5Field_10field_name_2__set__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9drf_turbo_6fields_5Field_10field_name_4__del__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_5Field_4root___get__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self); /* proto */
static int __pyx_pf_9drf_turbo_6fields_5Field_4root_2__set__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9drf_turbo_6fields_5Field_4root_4__del__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_5Field_10validators___get__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self); /* proto */
static int __pyx_pf_9drf_turbo_6fields_5Field_10validators_2__set__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9drf_turbo_6fields_5Field_10validators_4__del__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_5Field_14error_messages___get__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self); /* proto */
static int __pyx_pf_9drf_turbo_6fields_5Field_14error_messages_2__set__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9drf_turbo_6fields_5Field_14error_messages_4__del__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_5Field_5attrs___get__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self); /* proto */
static int __pyx_pf_9drf_turbo_6fields_5Field_5attrs_2__set__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9drf_turbo_6fields_5Field_5attrs_4__del__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_5Field_24__reduce_cython__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_5Field_26__setstate_cython__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_9drf_turbo_6fields_8StrField___init__(struct __pyx_obj_9drf_turbo_6fields_StrField *__pyx_v_self, PyObject *__pyx_v_kwargs); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_8StrField_2serialize(struct __pyx_obj_9drf_turbo_6fields_StrField *__pyx_v_self, PyObject *__pyx_v_value, PyObject *__pyx_v_context); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_8StrField_4deserialize(struct __pyx_obj_9drf_turbo_6fields_StrField *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_8StrField_11allow_blank___get__(struct __pyx_obj_9drf_turbo_6fields_StrField *__pyx_v_self); /* proto */
static int __pyx_pf_9drf_turbo_6fields_8StrField_11allow_blank_2__set__(struct __pyx_obj_9drf_turbo_6fields_StrField *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9drf_turbo_6fields_8StrField_11allow_blank_4__del__(struct __pyx_obj_9drf_turbo_6fields_StrField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_8StrField_15trim_whitespace___get__(struct __pyx_obj_9drf_turbo_6fields_StrField *__pyx_v_self); /* proto */
static int __pyx_pf_9drf_turbo_6fields_8StrField_15trim_whitespace_2__set__(struct __pyx_obj_9drf_turbo_6fields_StrField *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9drf_turbo_6fields_8StrField_15trim_whitespace_4__del__(struct __pyx_obj_9drf_turbo_6fields_StrField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_8StrField_10max_length___get__(struct __pyx_obj_9drf_turbo_6fields_StrField *__pyx_v_self); /* proto */
static int __pyx_pf_9drf_turbo_6fields_8StrField_10max_length_2__set__(struct __pyx_obj_9drf_turbo_6fields_StrField *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9drf_turbo_6fields_8StrField_10max_length_4__del__(struct __pyx_obj_9drf_turbo_6fields_StrField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_8StrField_10min_length___get__(struct __pyx_obj_9drf_turbo_6fields_StrField *__pyx_v_self); /* proto */
static int __pyx_pf_9drf_turbo_6fields_8StrField_10min_length_2__set__(struct __pyx_obj_9drf_turbo_6fields_StrField *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9drf_turbo_6fields_8StrField_10min_length_4__del__(struct __pyx_obj_9drf_turbo_6fields_StrField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_8StrField_6__reduce_cython__(struct __pyx_obj_9drf_turbo_6fields_StrField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_8StrField_8__setstate_cython__(struct __pyx_obj_9drf_turbo_6fields_StrField *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_9drf_turbo_6fields_10EmailField___init__(struct __pyx_obj_9drf_turbo_6fields_EmailField *__pyx_v_self, PyObject *__pyx_v_kwargs); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_10EmailField_2serialize(struct __pyx_obj_9drf_turbo_6fields_EmailField *__pyx_v_self, PyObject *__pyx_v_value, PyObject *__pyx_v_context); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_10EmailField_4deserialize(struct __pyx_obj_9drf_turbo_6fields_EmailField *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_10EmailField_8to_lower___get__(struct __pyx_obj_9drf_turbo_6fields_EmailField *__pyx_v_self); /* proto */
static int __pyx_pf_9drf_turbo_6fields_10EmailField_8to_lower_2__set__(struct __pyx_obj_9drf_turbo_6fields_EmailField *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_10EmailField_6__reduce_cython__(struct __pyx_obj_9drf_turbo_6fields_EmailField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_10EmailField_8__setstate_cython__(struct __pyx_obj_9drf_turbo_6fields_EmailField *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_8URLField_deserialize(struct __pyx_obj_9drf_turbo_6fields_URLField *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_8URLField_2__reduce_cython__(struct __pyx_obj_9drf_turbo_6fields_URLField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_8URLField_4__setstate_cython__(struct __pyx_obj_9drf_turbo_6fields_URLField *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_9drf_turbo_6fields_10RegexField___init__(struct __pyx_obj_9drf_turbo_6fields_RegexField *__pyx_v_self, PyObject *__pyx_v_regex, PyObject *__pyx_v_kwargs); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_10RegexField_2deserialize(struct __pyx_obj_9drf_turbo_6fields_RegexField *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_10RegexField_5regex___get__(struct __pyx_obj_9drf_turbo_6fields_RegexField *__pyx_v_self); /* proto */
static int __pyx_pf_9drf_turbo_6fields_10RegexField_5regex_2__set__(struct __pyx_obj_9drf_turbo_6fields_RegexField *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9drf_turbo_6fields_10RegexField_5regex_4__del__(struct __pyx_obj_9drf_turbo_6fields_RegexField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_10RegexField_4__reduce_cython__(struct __pyx_obj_9drf_turbo_6fields_RegexField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_10RegexField_6__setstate_cython__(struct __pyx_obj_9drf_turbo_6fields_RegexField *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_7IPField_deserialize(struct __pyx_obj_9drf_turbo_6fields_IPField *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_7IPField_2__reduce_cython__(struct __pyx_obj_9drf_turbo_6fields_IPField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_7IPField_4__setstate_cython__(struct __pyx_obj_9drf_turbo_6fields_IPField *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_9drf_turbo_6fields_13PasswordField___init__(struct __pyx_obj_9drf_turbo_6fields_PasswordField *__pyx_v_self, PyObject *__pyx_v_kwargs); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_13PasswordField_2__reduce_cython__(struct __pyx_obj_9drf_turbo_6fields_PasswordField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_13PasswordField_4__setstate_cython__(struct __pyx_obj_9drf_turbo_6fields_PasswordField *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_9UUIDField_deserialize(struct __pyx_obj_9drf_turbo_6fields_UUIDField *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_9UUIDField_2__reduce_cython__(struct __pyx_obj_9drf_turbo_6fields_UUIDField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_9UUIDField_4__setstate_cython__(struct __pyx_obj_9drf_turbo_6fields_UUIDField *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_9drf_turbo_6fields_9SlugField___init__(struct __pyx_obj_9drf_turbo_6fields_SlugField *__pyx_v_self, PyObject *__pyx_v_allow_unicode, PyObject *__pyx_v_kwargs); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_9SlugField_2deserialize(struct __pyx_obj_9drf_turbo_6fields_SlugField *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_9SlugField_13allow_unicode___get__(struct __pyx_obj_9drf_turbo_6fields_SlugField *__pyx_v_self); /* proto */
static int __pyx_pf_9drf_turbo_6fields_9SlugField_13allow_unicode_2__set__(struct __pyx_obj_9drf_turbo_6fields_SlugField *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9drf_turbo_6fields_9SlugField_13allow_unicode_4__del__(struct __pyx_obj_9drf_turbo_6fields_SlugField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_9SlugField_4__reduce_cython__(struct __pyx_obj_9drf_turbo_6fields_SlugField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_9SlugField_6__setstate_cython__(struct __pyx_obj_9drf_turbo_6fields_SlugField *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_9drf_turbo_6fields_8IntField___init__(struct __pyx_obj_9drf_turbo_6fields_IntField *__pyx_v_self, PyObject *__pyx_v_kwargs); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_8IntField_2serialize(struct __pyx_obj_9drf_turbo_6fields_IntField *__pyx_v_self, PyObject *__pyx_v_value, PyObject *__pyx_v_context); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_8IntField_4deserialize(struct __pyx_obj_9drf_turbo_6fields_IntField *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_8IntField_9max_value___get__(struct __pyx_obj_9drf_turbo_6fields_IntField *__pyx_v_self); /* proto */
static int __pyx_pf_9drf_turbo_6fields_8IntField_9max_value_2__set__(struct __pyx_obj_9drf_turbo_6fields_IntField *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9drf_turbo_6fields_8IntField_9max_value_4__del__(struct __pyx_obj_9drf_turbo_6fields_IntField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_8IntField_9min_value___get__(struct __pyx_obj_9drf_turbo_6fields_IntField *__pyx_v_self); /* proto */
static int __pyx_pf_9drf_turbo_6fields_8IntField_9min_value_2__set__(struct __pyx_obj_9drf_turbo_6fields_IntField *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9drf_turbo_6fields_8IntField_9min_value_4__del__(struct __pyx_obj_9drf_turbo_6fields_IntField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_8IntField_6__reduce_cython__(struct __pyx_obj_9drf_turbo_6fields_IntField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_8IntField_8__setstate_cython__(struct __pyx_obj_9drf_turbo_6fields_IntField *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_9drf_turbo_6fields_10FloatField___init__(struct __pyx_obj_9drf_turbo_6fields_FloatField *__pyx_v_self, PyObject *__pyx_v_kwargs); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_10FloatField_2serialize(struct __pyx_obj_9drf_turbo_6fields_FloatField *__pyx_v_self, PyObject *__pyx_v_value, PyObject *__pyx_v_context); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_10FloatField_4deserialize(struct __pyx_obj_9drf_turbo_6fields_FloatField *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_10FloatField_9max_value___get__(struct __pyx_obj_9drf_turbo_6fields_FloatField *__pyx_v_self); /* proto */
static int __pyx_pf_9drf_turbo_6fields_10FloatField_9max_value_2__set__(struct __pyx_obj_9drf_turbo_6fields_FloatField *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9drf_turbo_6fields_10FloatField_9max_value_4__del__(struct __pyx_obj_9drf_turbo_6fields_FloatField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_10FloatField_9min_value___get__(struct __pyx_obj_9drf_turbo_6fields_FloatField *__pyx_v_self); /* proto */
static int __pyx_pf_9drf_turbo_6fields_10FloatField_9min_value_2__set__(struct __pyx_obj_9drf_turbo_6fields_FloatField *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9drf_turbo_6fields_10FloatField_9min_value_4__del__(struct __pyx_obj_9drf_turbo_6fields_FloatField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_10FloatField_6__reduce_cython__(struct __pyx_obj_9drf_turbo_6fields_FloatField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_10FloatField_8__setstate_cython__(struct __pyx_obj_9drf_turbo_6fields_FloatField *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_9drf_turbo_6fields_12DecimalField___init__(struct __pyx_obj_9drf_turbo_6fields_DecimalField *__pyx_v_self, PyObject *__pyx_v_max_digits, PyObject *__pyx_v_decimal_places, PyObject *__pyx_v_max_value, PyObject *__pyx_v_min_value, PyObject *__pyx_v_coerce_to_string, PyObject *__pyx_v_rounding, PyObject *__pyx_v_kwargs); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_12DecimalField_2serialize(struct __pyx_obj_9drf_turbo_6fields_DecimalField *__pyx_v_self, PyObject *__pyx_v_value, PyObject *__pyx_v_context); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_12DecimalField_4deserialize(struct __pyx_obj_9drf_turbo_6fields_DecimalField *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_12DecimalField_10max_digits___get__(struct __pyx_obj_9drf_turbo_6fields_DecimalField *__pyx_v_self); /* proto */
static int __pyx_pf_9drf_turbo_6fields_12DecimalField_10max_digits_2__set__(struct __pyx_obj_9drf_turbo_6fields_DecimalField *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9drf_turbo_6fields_12DecimalField_10max_digits_4__del__(struct __pyx_obj_9drf_turbo_6fields_DecimalField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_12DecimalField_14decimal_places___get__(struct __pyx_obj_9drf_turbo_6fields_DecimalField *__pyx_v_self); /* proto */
static int __pyx_pf_9drf_turbo_6fields_12DecimalField_14decimal_places_2__set__(struct __pyx_obj_9drf_turbo_6fields_DecimalField *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9drf_turbo_6fields_12DecimalField_14decimal_places_4__del__(struct __pyx_obj_9drf_turbo_6fields_DecimalField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_12DecimalField_9max_value___get__(struct __pyx_obj_9drf_turbo_6fields_DecimalField *__pyx_v_self); /* proto */
static int __pyx_pf_9drf_turbo_6fields_12DecimalField_9max_value_2__set__(struct __pyx_obj_9drf_turbo_6fields_DecimalField *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9drf_turbo_6fields_12DecimalField_9max_value_4__del__(struct __pyx_obj_9drf_turbo_6fields_DecimalField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_12DecimalField_9min_value___get__(struct __pyx_obj_9drf_turbo_6fields_DecimalField *__pyx_v_self); /* proto */
static int __pyx_pf_9drf_turbo_6fields_12DecimalField_9min_value_2__set__(struct __pyx_obj_9drf_turbo_6fields_DecimalField *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9drf_turbo_6fields_12DecimalField_9min_value_4__del__(struct __pyx_obj_9drf_turbo_6fields_DecimalField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_12DecimalField_8rounding___get__(struct __pyx_obj_9drf_turbo_6fields_DecimalField *__pyx_v_self); /* proto */
static int __pyx_pf_9drf_turbo_6fields_12DecimalField_8rounding_2__set__(struct __pyx_obj_9drf_turbo_6fields_DecimalField *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9drf_turbo_6fields_12DecimalField_8rounding_4__del__(struct __pyx_obj_9drf_turbo_6fields_DecimalField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_12DecimalField_16coerce_to_string___get__(struct __pyx_obj_9drf_turbo_6fields_DecimalField *__pyx_v_self); /* proto */
static int __pyx_pf_9drf_turbo_6fields_12DecimalField_16coerce_to_string_2__set__(struct __pyx_obj_9drf_turbo_6fields_DecimalField *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9drf_turbo_6fields_12DecimalField_16coerce_to_string_4__del__(struct __pyx_obj_9drf_turbo_6fields_DecimalField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_12DecimalField_16max_whole_digits___get__(struct __pyx_obj_9drf_turbo_6fields_DecimalField *__pyx_v_self); /* proto */
static int __pyx_pf_9drf_turbo_6fields_12DecimalField_16max_whole_digits_2__set__(struct __pyx_obj_9drf_turbo_6fields_DecimalField *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9drf_turbo_6fields_12DecimalField_16max_whole_digits_4__del__(struct __pyx_obj_9drf_turbo_6fields_DecimalField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_12DecimalField_6__reduce_cython__(struct __pyx_obj_9drf_turbo_6fields_DecimalField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_12DecimalField_8__setstate_cython__(struct __pyx_obj_9drf_turbo_6fields_DecimalField *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_9BoolField_serialize(struct __pyx_obj_9drf_turbo_6fields_BoolField *__pyx_v_self, PyObject *__pyx_v_value, PyObject *__pyx_v_context); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_9BoolField_2deserialize(struct __pyx_obj_9drf_turbo_6fields_BoolField *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_9BoolField_4__reduce_cython__(struct __pyx_obj_9drf_turbo_6fields_BoolField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_9BoolField_6__setstate_cython__(struct __pyx_obj_9drf_turbo_6fields_BoolField *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_9drf_turbo_6fields_11ChoiceField___init__(struct __pyx_obj_9drf_turbo_6fields_ChoiceField *__pyx_v_self, PyObject *__pyx_v_choices, PyObject *__pyx_v_kwargs); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_11ChoiceField_2serialize(struct __pyx_obj_9drf_turbo_6fields_ChoiceField *__pyx_v_self, PyObject *__pyx_v_value, PyObject *__pyx_v_context); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_11ChoiceField_4deserialize(struct __pyx_obj_9drf_turbo_6fields_ChoiceField *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_11ChoiceField_7choices___get__(struct __pyx_obj_9drf_turbo_6fields_ChoiceField *__pyx_v_self); /* proto */
static int __pyx_pf_9drf_turbo_6fields_11ChoiceField_7choices_2__set__(struct __pyx_obj_9drf_turbo_6fields_ChoiceField *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9drf_turbo_6fields_11ChoiceField_7choices_4__del__(struct __pyx_obj_9drf_turbo_6fields_ChoiceField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_11ChoiceField_24choice_strings_to_values___get__(struct __pyx_obj_9drf_turbo_6fields_ChoiceField *__pyx_v_self); /* proto */
static int __pyx_pf_9drf_turbo_6fields_11ChoiceField_24choice_strings_to_values_2__set__(struct __pyx_obj_9drf_turbo_6fields_ChoiceField *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9drf_turbo_6fields_11ChoiceField_24choice_strings_to_values_4__del__(struct __pyx_obj_9drf_turbo_6fields_ChoiceField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_11ChoiceField_25choice_strings_to_display___get__(struct __pyx_obj_9drf_turbo_6fields_ChoiceField *__pyx_v_self); /* proto */
static int __pyx_pf_9drf_turbo_6fields_11ChoiceField_25choice_strings_to_display_2__set__(struct __pyx_obj_9drf_turbo_6fields_ChoiceField *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9drf_turbo_6fields_11ChoiceField_25choice_strings_to_display_4__del__(struct __pyx_obj_9drf_turbo_6fields_ChoiceField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_11ChoiceField_11allow_blank___get__(struct __pyx_obj_9drf_turbo_6fields_ChoiceField *__pyx_v_self); /* proto */
static int __pyx_pf_9drf_turbo_6fields_11ChoiceField_11allow_blank_2__set__(struct __pyx_obj_9drf_turbo_6fields_ChoiceField *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9drf_turbo_6fields_11ChoiceField_11allow_blank_4__del__(struct __pyx_obj_9drf_turbo_6fields_ChoiceField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_11ChoiceField_6__reduce_cython__(struct __pyx_obj_9drf_turbo_6fields_ChoiceField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_11ChoiceField_8__setstate_cython__(struct __pyx_obj_9drf_turbo_6fields_ChoiceField *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_9drf_turbo_6fields_19MultipleChoiceField___init__(struct __pyx_obj_9drf_turbo_6fields_MultipleChoiceField *__pyx_v_self, PyObject *__pyx_v_kwargs); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_19MultipleChoiceField_2serialize(struct __pyx_obj_9drf_turbo_6fields_MultipleChoiceField *__pyx_v_self, PyObject *__pyx_v_value, PyObject *__pyx_v_context); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_19MultipleChoiceField_4deserialize(struct __pyx_obj_9drf_turbo_6fields_MultipleChoiceField *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_19MultipleChoiceField_11allow_empty___get__(struct __pyx_obj_9drf_turbo_6fields_MultipleChoiceField *__pyx_v_self); /* proto */
static int __pyx_pf_9drf_turbo_6fields_19MultipleChoiceField_11allow_empty_2__set__(struct __pyx_obj_9drf_turbo_6fields_MultipleChoiceField *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9drf_turbo_6fields_19MultipleChoiceField_11allow_empty_4__del__(struct __pyx_obj_9drf_turbo_6fields_MultipleChoiceField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_19MultipleChoiceField_6__reduce_cython__(struct __pyx_obj_9drf_turbo_6fields_MultipleChoiceField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_19MultipleChoiceField_8__setstate_cython__(struct __pyx_obj_9drf_turbo_6fields_MultipleChoiceField *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_9drf_turbo_6fields_13DateTimeField___init__(struct __pyx_obj_9drf_turbo_6fields_DateTimeField *__pyx_v_self, PyObject *__pyx_v_format, PyObject *__pyx_v_input_formats, PyObject *__pyx_v_default_timezone, PyObject *__pyx_v_kwargs); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_13DateTimeField_2get_default_timezone(struct __pyx_obj_9drf_turbo_6fields_DateTimeField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_13DateTimeField_4enforce_timezone(struct __pyx_obj_9drf_turbo_6fields_DateTimeField *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_13DateTimeField_6serialize(struct __pyx_obj_9drf_turbo_6fields_DateTimeField *__pyx_v_self, PyObject *__pyx_v_value, PyObject *__pyx_v_context); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_13DateTimeField_8deserialize(struct __pyx_obj_9drf_turbo_6fields_DateTimeField *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_13DateTimeField_6format___get__(struct __pyx_obj_9drf_turbo_6fields_DateTimeField *__pyx_v_self); /* proto */
static int __pyx_pf_9drf_turbo_6fields_13DateTimeField_6format_2__set__(struct __pyx_obj_9drf_turbo_6fields_DateTimeField *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9drf_turbo_6fields_13DateTimeField_6format_4__del__(struct __pyx_obj_9drf_turbo_6fields_DateTimeField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_13DateTimeField_13input_formats___get__(struct __pyx_obj_9drf_turbo_6fields_DateTimeField *__pyx_v_self); /* proto */
static int __pyx_pf_9drf_turbo_6fields_13DateTimeField_13input_formats_2__set__(struct __pyx_obj_9drf_turbo_6fields_DateTimeField *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9drf_turbo_6fields_13DateTimeField_13input_formats_4__del__(struct __pyx_obj_9drf_turbo_6fields_DateTimeField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_13DateTimeField_16default_timezone___get__(struct __pyx_obj_9drf_turbo_6fields_DateTimeField *__pyx_v_self); /* proto */
static int __pyx_pf_9drf_turbo_6fields_13DateTimeField_16default_timezone_2__set__(struct __pyx_obj_9drf_turbo_6fields_DateTimeField *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9drf_turbo_6fields_13DateTimeField_16default_timezone_4__del__(struct __pyx_obj_9drf_turbo_6fields_DateTimeField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_13DateTimeField_8timezone___get__(struct __pyx_obj_9drf_turbo_6fields_DateTimeField *__pyx_v_self); /* proto */
static int __pyx_pf_9drf_turbo_6fields_13DateTimeField_8timezone_2__set__(struct __pyx_obj_9drf_turbo_6fields_DateTimeField *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9drf_turbo_6fields_13DateTimeField_8timezone_4__del__(struct __pyx_obj_9drf_turbo_6fields_DateTimeField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_13DateTimeField_10__reduce_cython__(struct __pyx_obj_9drf_turbo_6fields_DateTimeField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_13DateTimeField_12__setstate_cython__(struct __pyx_obj_9drf_turbo_6fields_DateTimeField *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_9drf_turbo_6fields_9DateField___init__(struct __pyx_obj_9drf_turbo_6fields_DateField *__pyx_v_self, PyObject *__pyx_v_format, PyObject *__pyx_v_input_formats, PyObject *__pyx_v_kwargs); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_9DateField_2serialize(struct __pyx_obj_9drf_turbo_6fields_DateField *__pyx_v_self, PyObject *__pyx_v_value, PyObject *__pyx_v_context); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_9DateField_4deserialize(struct __pyx_obj_9drf_turbo_6fields_DateField *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_9DateField_6format___get__(struct __pyx_obj_9drf_turbo_6fields_DateField *__pyx_v_self); /* proto */
static int __pyx_pf_9drf_turbo_6fields_9DateField_6format_2__set__(struct __pyx_obj_9drf_turbo_6fields_DateField *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9drf_turbo_6fields_9DateField_6format_4__del__(struct __pyx_obj_9drf_turbo_6fields_DateField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_9DateField_13input_formats___get__(struct __pyx_obj_9drf_turbo_6fields_DateField *__pyx_v_self); /* proto */
static int __pyx_pf_9drf_turbo_6fields_9DateField_13input_formats_2__set__(struct __pyx_obj_9drf_turbo_6fields_DateField *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9drf_turbo_6fields_9DateField_13input_formats_4__del__(struct __pyx_obj_9drf_turbo_6fields_DateField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_9DateField_6__reduce_cython__(struct __pyx_obj_9drf_turbo_6fields_DateField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_9DateField_8__setstate_cython__(struct __pyx_obj_9drf_turbo_6fields_DateField *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_9drf_turbo_6fields_9TimeField___init__(struct __pyx_obj_9drf_turbo_6fields_TimeField *__pyx_v_self, PyObject *__pyx_v_format, PyObject *__pyx_v_input_formats, PyObject *__pyx_v_kwargs); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_9TimeField_2serialize(struct __pyx_obj_9drf_turbo_6fields_TimeField *__pyx_v_self, PyObject *__pyx_v_value, PyObject *__pyx_v_context); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_9TimeField_4deserialize(struct __pyx_obj_9drf_turbo_6fields_TimeField *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_9TimeField_6format___get__(struct __pyx_obj_9drf_turbo_6fields_TimeField *__pyx_v_self); /* proto */
static int __pyx_pf_9drf_turbo_6fields_9TimeField_6format_2__set__(struct __pyx_obj_9drf_turbo_6fields_TimeField *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9drf_turbo_6fields_9TimeField_6format_4__del__(struct __pyx_obj_9drf_turbo_6fields_TimeField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_9TimeField_13input_formats___get__(struct __pyx_obj_9drf_turbo_6fields_TimeField *__pyx_v_self); /* proto */
static int __pyx_pf_9drf_turbo_6fields_9TimeField_13input_formats_2__set__(struct __pyx_obj_9drf_turbo_6fields_TimeField *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9drf_turbo_6fields_9TimeField_13input_formats_4__del__(struct __pyx_obj_9drf_turbo_6fields_TimeField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_9TimeField_6__reduce_cython__(struct __pyx_obj_9drf_turbo_6fields_TimeField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_9TimeField_8__setstate_cython__(struct __pyx_obj_9drf_turbo_6fields_TimeField *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_9drf_turbo_6fields_9FileField___init__(struct __pyx_obj_9drf_turbo_6fields_FileField *__pyx_v_self, PyObject *__pyx_v_kwargs); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_9FileField_2serialize(struct __pyx_obj_9drf_turbo_6fields_FileField *__pyx_v_self, PyObject *__pyx_v_value, PyObject *__pyx_v_context); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_9FileField_4deserialize(struct __pyx_obj_9drf_turbo_6fields_FileField *__pyx_v_self, PyObject *__pyx_v_value, PyObject *__pyx_v_context); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_9FileField_10max_length___get__(struct __pyx_obj_9drf_turbo_6fields_FileField *__pyx_v_self); /* proto */
static int __pyx_pf_9drf_turbo_6fields_9FileField_10max_length_2__set__(struct __pyx_obj_9drf_turbo_6fields_FileField *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9drf_turbo_6fields_9FileField_10max_length_4__del__(struct __pyx_obj_9drf_turbo_6fields_FileField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_9FileField_16allow_empty_file___get__(struct __pyx_obj_9drf_turbo_6fields_FileField *__pyx_v_self); /* proto */
static int __pyx_pf_9drf_turbo_6fields_9FileField_16allow_empty_file_2__set__(struct __pyx_obj_9drf_turbo_6fields_FileField *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9drf_turbo_6fields_9FileField_16allow_empty_file_4__del__(struct __pyx_obj_9drf_turbo_6fields_FileField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_9FileField_6__reduce_cython__(struct __pyx_obj_9drf_turbo_6fields_FileField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_9FileField_8__setstate_cython__(struct __pyx_obj_9drf_turbo_6fields_FileField *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_9drf_turbo_6fields_10ArrayField___init__(struct __pyx_obj_9drf_turbo_6fields_ArrayField *__pyx_v_self, PyObject *__pyx_v_child, PyObject *__pyx_v_kwargs); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_10ArrayField_2serialize(struct __pyx_obj_9drf_turbo_6fields_ArrayField *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_10ArrayField_4deserialize(struct __pyx_obj_9drf_turbo_6fields_ArrayField *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_10ArrayField_6run_child_validation(struct __pyx_obj_9drf_turbo_6fields_ArrayField *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_10ArrayField_5child___get__(struct __pyx_obj_9drf_turbo_6fields_ArrayField *__pyx_v_self); /* proto */
static int __pyx_pf_9drf_turbo_6fields_10ArrayField_5child_2__set__(struct __pyx_obj_9drf_turbo_6fields_ArrayField *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9drf_turbo_6fields_10ArrayField_5child_4__del__(struct __pyx_obj_9drf_turbo_6fields_ArrayField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_10ArrayField_11allow_empty___get__(struct __pyx_obj_9drf_turbo_6fields_ArrayField *__pyx_v_self); /* proto */
static int __pyx_pf_9drf_turbo_6fields_10ArrayField_11allow_empty_2__set__(struct __pyx_obj_9drf_turbo_6fields_ArrayField *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9drf_turbo_6fields_10ArrayField_11allow_empty_4__del__(struct __pyx_obj_9drf_turbo_6fields_ArrayField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_10ArrayField_9min_items___get__(struct __pyx_obj_9drf_turbo_6fields_ArrayField *__pyx_v_self); /* proto */
static int __pyx_pf_9drf_turbo_6fields_10ArrayField_9min_items_2__set__(struct __pyx_obj_9drf_turbo_6fields_ArrayField *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9drf_turbo_6fields_10ArrayField_9min_items_4__del__(struct __pyx_obj_9drf_turbo_6fields_ArrayField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_10ArrayField_9max_items___get__(struct __pyx_obj_9drf_turbo_6fields_ArrayField *__pyx_v_self); /* proto */
static int __pyx_pf_9drf_turbo_6fields_10ArrayField_9max_items_2__set__(struct __pyx_obj_9drf_turbo_6fields_ArrayField *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9drf_turbo_6fields_10ArrayField_9max_items_4__del__(struct __pyx_obj_9drf_turbo_6fields_ArrayField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_10ArrayField_11exact_items___get__(struct __pyx_obj_9drf_turbo_6fields_ArrayField *__pyx_v_self); /* proto */
static int __pyx_pf_9drf_turbo_6fields_10ArrayField_11exact_items_2__set__(struct __pyx_obj_9drf_turbo_6fields_ArrayField *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9drf_turbo_6fields_10ArrayField_11exact_items_4__del__(struct __pyx_obj_9drf_turbo_6fields_ArrayField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_10ArrayField_8__reduce_cython__(struct __pyx_obj_9drf_turbo_6fields_ArrayField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_10ArrayField_10__setstate_cython__(struct __pyx_obj_9drf_turbo_6fields_ArrayField *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_9drf_turbo_6fields_9DictField___init__(struct __pyx_obj_9drf_turbo_6fields_DictField *__pyx_v_self, PyObject *__pyx_v_child, PyObject *__pyx_v_kwargs); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_9DictField_2serialize(struct __pyx_obj_9drf_turbo_6fields_DictField *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_9DictField_4deserialize(struct __pyx_obj_9drf_turbo_6fields_DictField *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_9DictField_6run_child_validation(struct __pyx_obj_9drf_turbo_6fields_DictField *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_9DictField_5child___get__(struct __pyx_obj_9drf_turbo_6fields_DictField *__pyx_v_self); /* proto */
static int __pyx_pf_9drf_turbo_6fields_9DictField_5child_2__set__(struct __pyx_obj_9drf_turbo_6fields_DictField *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9drf_turbo_6fields_9DictField_5child_4__del__(struct __pyx_obj_9drf_turbo_6fields_DictField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_9DictField_11allow_empty___get__(struct __pyx_obj_9drf_turbo_6fields_DictField *__pyx_v_self); /* proto */
static int __pyx_pf_9drf_turbo_6fields_9DictField_11allow_empty_2__set__(struct __pyx_obj_9drf_turbo_6fields_DictField *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9drf_turbo_6fields_9DictField_11allow_empty_4__del__(struct __pyx_obj_9drf_turbo_6fields_DictField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_9DictField_8__reduce_cython__(struct __pyx_obj_9drf_turbo_6fields_DictField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_9DictField_10__setstate_cython__(struct __pyx_obj_9drf_turbo_6fields_DictField *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_9drf_turbo_6fields_9JSONField___init__(struct __pyx_obj_9drf_turbo_6fields_JSONField *__pyx_v_self, PyObject *__pyx_v_kwargs); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_9JSONField_2serialize(struct __pyx_obj_9drf_turbo_6fields_JSONField *__pyx_v_self, PyObject *__pyx_v_value, PyObject *__pyx_v_context); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_9JSONField_4deserialize(struct __pyx_obj_9drf_turbo_6fields_JSONField *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_9JSONField_6binary___get__(struct __pyx_obj_9drf_turbo_6fields_JSONField *__pyx_v_self); /* proto */
static int __pyx_pf_9drf_turbo_6fields_9JSONField_6binary_2__set__(struct __pyx_obj_9drf_turbo_6fields_JSONField *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9drf_turbo_6fields_9JSONField_6binary_4__del__(struct __pyx_obj_9drf_turbo_6fields_JSONField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_9JSONField_7encoder___get__(struct __pyx_obj_9drf_turbo_6fields_JSONField *__pyx_v_self); /* proto */
static int __pyx_pf_9drf_turbo_6fields_9JSONField_7encoder_2__set__(struct __pyx_obj_9drf_turbo_6fields_JSONField *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9drf_turbo_6fields_9JSONField_7encoder_4__del__(struct __pyx_obj_9drf_turbo_6fields_JSONField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_9JSONField_7decoder___get__(struct __pyx_obj_9drf_turbo_6fields_JSONField *__pyx_v_self); /* proto */
static int __pyx_pf_9drf_turbo_6fields_9JSONField_7decoder_2__set__(struct __pyx_obj_9drf_turbo_6fields_JSONField *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9drf_turbo_6fields_9JSONField_7decoder_4__del__(struct __pyx_obj_9drf_turbo_6fields_JSONField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_9JSONField_6__reduce_cython__(struct __pyx_obj_9drf_turbo_6fields_JSONField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_9JSONField_8__setstate_cython__(struct __pyx_obj_9drf_turbo_6fields_JSONField *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_9drf_turbo_6fields_12RelatedField___init__(struct __pyx_obj_9drf_turbo_6fields_RelatedField *__pyx_v_self, PyObject *__pyx_v_kwargs); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_12RelatedField_2deserialize(struct __pyx_obj_9drf_turbo_6fields_RelatedField *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_12RelatedField_8queryset___get__(struct __pyx_obj_9drf_turbo_6fields_RelatedField *__pyx_v_self); /* proto */
static int __pyx_pf_9drf_turbo_6fields_12RelatedField_8queryset_2__set__(struct __pyx_obj_9drf_turbo_6fields_RelatedField *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9drf_turbo_6fields_12RelatedField_8queryset_4__del__(struct __pyx_obj_9drf_turbo_6fields_RelatedField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_12RelatedField_4__reduce_cython__(struct __pyx_obj_9drf_turbo_6fields_RelatedField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_12RelatedField_6__setstate_cython__(struct __pyx_obj_9drf_turbo_6fields_RelatedField *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_9drf_turbo_6fields_16ManyRelatedField___init__(struct __pyx_obj_9drf_turbo_6fields_ManyRelatedField *__pyx_v_self, PyObject *__pyx_v_kwargs); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_16ManyRelatedField_2serialize(struct __pyx_obj_9drf_turbo_6fields_ManyRelatedField *__pyx_v_self, PyObject *__pyx_v_value, PyObject *__pyx_v_context); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_16ManyRelatedField_4deserialize(struct __pyx_obj_9drf_turbo_6fields_ManyRelatedField *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_16ManyRelatedField_14child_relation___get__(struct __pyx_obj_9drf_turbo_6fields_ManyRelatedField *__pyx_v_self); /* proto */
static int __pyx_pf_9drf_turbo_6fields_16ManyRelatedField_14child_relation_2__set__(struct __pyx_obj_9drf_turbo_6fields_ManyRelatedField *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9drf_turbo_6fields_16ManyRelatedField_14child_relation_4__del__(struct __pyx_obj_9drf_turbo_6fields_ManyRelatedField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_16ManyRelatedField_11allow_empty___get__(struct __pyx_obj_9drf_turbo_6fields_ManyRelatedField *__pyx_v_self); /* proto */
static int __pyx_pf_9drf_turbo_6fields_16ManyRelatedField_11allow_empty_2__set__(struct __pyx_obj_9drf_turbo_6fields_ManyRelatedField *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9drf_turbo_6fields_16ManyRelatedField_11allow_empty_4__del__(struct __pyx_obj_9drf_turbo_6fields_ManyRelatedField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_16ManyRelatedField_6__reduce_cython__(struct __pyx_obj_9drf_turbo_6fields_ManyRelatedField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_16ManyRelatedField_8__setstate_cython__(struct __pyx_obj_9drf_turbo_6fields_ManyRelatedField *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_9drf_turbo_6fields_13ConstantField___init__(struct __pyx_obj_9drf_turbo_6fields_ConstantField *__pyx_v_self, PyObject *__pyx_v_constant, PyObject *__pyx_v_kwargs); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_13ConstantField_2deserialize(struct __pyx_obj_9drf_turbo_6fields_ConstantField *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_13ConstantField_8constant___get__(struct __pyx_obj_9drf_turbo_6fields_ConstantField *__pyx_v_self); /* proto */
static int __pyx_pf_9drf_turbo_6fields_13ConstantField_8constant_2__set__(struct __pyx_obj_9drf_turbo_6fields_ConstantField *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9drf_turbo_6fields_13ConstantField_8constant_4__del__(struct __pyx_obj_9drf_turbo_6fields_ConstantField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_13ConstantField_4__reduce_cython__(struct __pyx_obj_9drf_turbo_6fields_ConstantField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_13ConstantField_6__setstate_cython__(struct __pyx_obj_9drf_turbo_6fields_ConstantField *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_9drf_turbo_6fields_14RecursiveField___init__(struct __pyx_obj_9drf_turbo_6fields_RecursiveField *__pyx_v_self, PyObject *__pyx_v_kwargs); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_14RecursiveField_2serialize(struct __pyx_obj_9drf_turbo_6fields_RecursiveField *__pyx_v_self, PyObject *__pyx_v_value, PyObject *__pyx_v_context); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_14RecursiveField_4many___get__(struct __pyx_obj_9drf_turbo_6fields_RecursiveField *__pyx_v_self); /* proto */
static int __pyx_pf_9drf_turbo_6fields_14RecursiveField_4many_2__set__(struct __pyx_obj_9drf_turbo_6fields_RecursiveField *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9drf_turbo_6fields_14RecursiveField_4many_4__del__(struct __pyx_obj_9drf_turbo_6fields_RecursiveField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_14RecursiveField_7context___get__(struct __pyx_obj_9drf_turbo_6fields_RecursiveField *__pyx_v_self); /* proto */
static int __pyx_pf_9drf_turbo_6fields_14RecursiveField_7context_2__set__(struct __pyx_obj_9drf_turbo_6fields_RecursiveField *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9drf_turbo_6fields_14RecursiveField_7context_4__del__(struct __pyx_obj_9drf_turbo_6fields_RecursiveField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_14RecursiveField_4only___get__(struct __pyx_obj_9drf_turbo_6fields_RecursiveField *__pyx_v_self); /* proto */
static int __pyx_pf_9drf_turbo_6fields_14RecursiveField_4only_2__set__(struct __pyx_obj_9drf_turbo_6fields_RecursiveField *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9drf_turbo_6fields_14RecursiveField_4only_4__del__(struct __pyx_obj_9drf_turbo_6fields_RecursiveField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_14RecursiveField_7exclude___get__(struct __pyx_obj_9drf_turbo_6fields_RecursiveField *__pyx_v_self); /* proto */
static int __pyx_pf_9drf_turbo_6fields_14RecursiveField_7exclude_2__set__(struct __pyx_obj_9drf_turbo_6fields_RecursiveField *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9drf_turbo_6fields_14RecursiveField_7exclude_4__del__(struct __pyx_obj_9drf_turbo_6fields_RecursiveField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_14RecursiveField_4__reduce_cython__(struct __pyx_obj_9drf_turbo_6fields_RecursiveField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_14RecursiveField_6__setstate_cython__(struct __pyx_obj_9drf_turbo_6fields_RecursiveField *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_9drf_turbo_6fields_11MethodField___init__(struct __pyx_obj_9drf_turbo_6fields_MethodField *__pyx_v_self, PyObject *__pyx_v_method_name, PyObject *__pyx_v_kwargs); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_11MethodField_2method_getter(struct __pyx_obj_9drf_turbo_6fields_MethodField *__pyx_v_self, PyObject *__pyx_v_field_name, PyObject *__pyx_v_root); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_11MethodField_11method_name___get__(struct __pyx_obj_9drf_turbo_6fields_MethodField *__pyx_v_self); /* proto */
static int __pyx_pf_9drf_turbo_6fields_11MethodField_11method_name_2__set__(struct __pyx_obj_9drf_turbo_6fields_MethodField *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9drf_turbo_6fields_11MethodField_11method_name_4__del__(struct __pyx_obj_9drf_turbo_6fields_MethodField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_11MethodField_4__reduce_cython__(struct __pyx_obj_9drf_turbo_6fields_MethodField *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_11MethodField_6__setstate_cython__(struct __pyx_obj_9drf_turbo_6fields_MethodField *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields___pyx_unpickle_SkipField(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_2__pyx_unpickle_Field(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_4__pyx_unpickle_StrField(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_6__pyx_unpickle_EmailField(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_8__pyx_unpickle_URLField(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_10__pyx_unpickle_RegexField(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_12__pyx_unpickle_IPField(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_14__pyx_unpickle_PasswordField(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_16__pyx_unpickle_UUIDField(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_18__pyx_unpickle_SlugField(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_20__pyx_unpickle_IntField(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_22__pyx_unpickle_FloatField(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_24__pyx_unpickle_DecimalField(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_26__pyx_unpickle_BoolField(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_28__pyx_unpickle_ChoiceField(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_30__pyx_unpickle_MultipleChoiceField(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_32__pyx_unpickle_DateTimeField(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_34__pyx_unpickle_DateField(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_36__pyx_unpickle_TimeField(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_38__pyx_unpickle_FileField(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_40__pyx_unpickle_ArrayField(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_42__pyx_unpickle_DictField(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_44__pyx_unpickle_JSONField(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_46__pyx_unpickle_RelatedField(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_48__pyx_unpickle_ManyRelatedField(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_50__pyx_unpickle_ConstantField(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_52__pyx_unpickle_RecursiveField(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_9drf_turbo_6fields_54__pyx_unpickle_MethodField(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_tp_new_9drf_turbo_6fields_SkipField(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_9drf_turbo_6fields_Field(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_9drf_turbo_6fields_StrField(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_9drf_turbo_6fields_EmailField(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_9drf_turbo_6fields_URLField(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_9drf_turbo_6fields_RegexField(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_9drf_turbo_6fields_IPField(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_9drf_turbo_6fields_PasswordField(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_9drf_turbo_6fields_UUIDField(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_9drf_turbo_6fields_SlugField(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_9drf_turbo_6fields_IntField(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_9drf_turbo_6fields_FloatField(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_9drf_turbo_6fields_DecimalField(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_9drf_turbo_6fields_BoolField(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_9drf_turbo_6fields_ChoiceField(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_9drf_turbo_6fields_MultipleChoiceField(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_9drf_turbo_6fields_DateTimeField(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_9drf_turbo_6fields_DateField(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_9drf_turbo_6fields_TimeField(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_9drf_turbo_6fields_FileField(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_9drf_turbo_6fields_ArrayField(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_9drf_turbo_6fields_DictField(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_9drf_turbo_6fields_JSONField(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_9drf_turbo_6fields_RelatedField(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_9drf_turbo_6fields_ManyRelatedField(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_9drf_turbo_6fields_ConstantField(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_9drf_turbo_6fields_RecursiveField(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_9drf_turbo_6fields_MethodField(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static __Pyx_CachedCFunction __pyx_umethod_PyDict_Type_get = {0, &__pyx_n_s_get, 0, 0, 0};
static __Pyx_CachedCFunction __pyx_umethod_PyDict_Type_items = {0, &__pyx_n_s_items, 0, 0, 0};
static __Pyx_CachedCFunction __pyx_umethod_PyDict_Type_pop = {0, &__pyx_n_s_pop, 0, 0, 0};
static __Pyx_CachedCFunction __pyx_umethod_PyDict_Type_update = {0, &__pyx_n_s_update, 0, 0, 0};
static PyObject *__pyx_int_0;
static PyObject *__pyx_int_1;
static PyObject *__pyx_int_4;
static PyObject *__pyx_int_1000;
static PyObject *__pyx_int_3394979;
static PyObject *__pyx_int_13265606;
static PyObject *__pyx_int_16510453;
static PyObject *__pyx_int_57277993;
static PyObject *__pyx_int_58778772;
static PyObject *__pyx_int_83299623;
static PyObject *__pyx_int_91705362;
static PyObject *__pyx_int_99052414;
static PyObject *__pyx_int_107357925;
static PyObject *__pyx_int_111135515;
static PyObject *__pyx_int_119569969;
static PyObject *__pyx_int_152991563;
static PyObject *__pyx_int_208687877;
static PyObject *__pyx_int_222419149;
static PyObject *__pyx_int_229562288;
static PyObject *__pyx_int_236850905;
static PyObject *__pyx_int_243425435;
static PyObject *__pyx_int_253175912;
static PyObject *__pyx_int_255240407;
static PyObject *__pyx_int_256313077;
static PyObject *__pyx_int_259151664;
static PyObject *__pyx_int_neg_6;
static PyObject *__pyx_k_;
static PyObject *__pyx_k__2;
static PyObject *__pyx_k__20;
static PyObject *__pyx_k__23;
static PyObject *__pyx_tuple__4;
static PyObject *__pyx_tuple__7;
static PyObject *__pyx_tuple__8;
static PyObject *__pyx_slice__22;
static PyObject *__pyx_tuple__10;
static PyObject *__pyx_tuple__11;
static PyObject *__pyx_tuple__13;
static PyObject *__pyx_tuple__14;
static PyObject *__pyx_tuple__15;
static PyObject *__pyx_tuple__16;
static PyObject *__pyx_tuple__17;
static PyObject *__pyx_tuple__18;
static PyObject *__pyx_tuple__19;
static PyObject *__pyx_tuple__21;
static PyObject *__pyx_tuple__24;
static PyObject *__pyx_tuple__25;
static PyObject *__pyx_tuple__26;
static PyObject *__pyx_tuple__27;
static PyObject *__pyx_tuple__28;
static PyObject *__pyx_tuple__29;
static PyObject *__pyx_tuple__30;
static PyObject *__pyx_tuple__32;
static PyObject *__pyx_tuple__34;
static PyObject *__pyx_tuple__36;
static PyObject *__pyx_tuple__38;
static PyObject *__pyx_tuple__40;
static PyObject *__pyx_tuple__42;
static PyObject *__pyx_tuple__44;
static PyObject *__pyx_tuple__46;
static PyObject *__pyx_tuple__48;
static PyObject *__pyx_tuple__50;
static PyObject *__pyx_tuple__52;
static PyObject *__pyx_tuple__54;
static PyObject *__pyx_tuple__56;
static PyObject *__pyx_tuple__58;
static PyObject *__pyx_tuple__60;
static PyObject *__pyx_tuple__62;
static PyObject *__pyx_tuple__64;
static PyObject *__pyx_tuple__66;
static PyObject *__pyx_tuple__68;
static PyObject *__pyx_tuple__70;
static PyObject *__pyx_tuple__72;
static PyObject *__pyx_tuple__74;
static PyObject *__pyx_tuple__76;
static PyObject *__pyx_tuple__78;
static PyObject *__pyx_tuple__80;
static PyObject *__pyx_tuple__82;
static PyObject *__pyx_tuple__84;
static PyObject *__pyx_tuple__86;
static PyObject *__pyx_codeobj__33;
static PyObject *__pyx_codeobj__35;
static PyObject *__pyx_codeobj__37;
static PyObject *__pyx_codeobj__39;
static PyObject *__pyx_codeobj__41;
static PyObject *__pyx_codeobj__43;
static PyObject *__pyx_codeobj__45;
static PyObject *__pyx_codeobj__47;
static PyObject *__pyx_codeobj__49;
static PyObject *__pyx_codeobj__51;
static PyObject *__pyx_codeobj__53;
static PyObject *__pyx_codeobj__55;
static PyObject *__pyx_codeobj__57;
static PyObject *__pyx_codeobj__59;
static PyObject *__pyx_codeobj__61;
static PyObject *__pyx_codeobj__63;
static PyObject *__pyx_codeobj__65;
static PyObject *__pyx_codeobj__67;
static PyObject *__pyx_codeobj__69;
static PyObject *__pyx_codeobj__71;
static PyObject *__pyx_codeobj__73;
static PyObject *__pyx_codeobj__75;
static PyObject *__pyx_codeobj__77;
static PyObject *__pyx_codeobj__79;
static PyObject *__pyx_codeobj__81;
static PyObject *__pyx_codeobj__83;
static PyObject *__pyx_codeobj__85;
static PyObject *__pyx_codeobj__87;
/* Late includes */

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_9SkipField_1__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_9SkipField_1__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9SkipField___reduce_cython__(((struct __pyx_obj_9drf_turbo_6fields_SkipField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_9SkipField___reduce_cython__(struct __pyx_obj_9drf_turbo_6fields_SkipField *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = ()             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __Pyx_INCREF(__pyx_empty_tuple);
  __pyx_v_state = __pyx_empty_tuple;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = ()
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_1 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v__dict = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "(tree fragment)":7
 *     state = ()
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_2 = (__pyx_v__dict != Py_None);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v__dict);
    __pyx_t_4 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_4));
    __pyx_t_4 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = False
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = ()
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = False             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_SkipField, (type(self), 0xd41d8cd, None), state
 */
  /*else*/ {
    __pyx_v_use_setstate = 0;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = False
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_SkipField, (type(self), 0xd41d8cd, None), state
 *     else:
 */
  __pyx_t_3 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_3) {

    /* "(tree fragment)":13
 *         use_setstate = False
 *     if use_setstate:
 *         return __pyx_unpickle_SkipField, (type(self), 0xd41d8cd, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_SkipField, (type(self), 0xd41d8cd, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_pyx_unpickle_SkipField); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_222419149);
    __Pyx_GIVEREF(__pyx_int_222419149);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_int_222419149);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_1, 2, Py_None);
    __pyx_t_5 = PyTuple_New(3); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_1);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_v_state);
    __pyx_t_4 = 0;
    __pyx_t_1 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = False
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_SkipField, (type(self), 0xd41d8cd, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_SkipField, (type(self), 0xd41d8cd, None), state
 *     else:
 *         return __pyx_unpickle_SkipField, (type(self), 0xd41d8cd, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_SkipField__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_pyx_unpickle_SkipField); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_222419149);
    __Pyx_GIVEREF(__pyx_int_222419149);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_int_222419149);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_state);
    __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_1);
    __pyx_t_5 = 0;
    __pyx_t_1 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("drf_turbo.fields.SkipField.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_SkipField, (type(self), 0xd41d8cd, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_SkipField__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_9SkipField_3__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_9SkipField_3__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9SkipField_2__setstate_cython__(((struct __pyx_obj_9drf_turbo_6fields_SkipField *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_9SkipField_2__setstate_cython__(struct __pyx_obj_9drf_turbo_6fields_SkipField *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_SkipField, (type(self), 0xd41d8cd, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_SkipField__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields___pyx_unpickle_SkipField__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_SkipField, (type(self), 0xd41d8cd, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_SkipField__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.SkipField.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":74
 *     _initial = None
 * 
 *     def __init__(             # <<<<<<<<<<<<<<
 *         self,
 *         basestring attr = None,
 */

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_5Field_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_5Field_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_attr = 0;
  int __pyx_v_call;
  int __pyx_v_required;
  int __pyx_v_write_only;
  int __pyx_v_read_only;
  int __pyx_v_allow_null;
  PyObject *__pyx_v_label = 0;
  PyObject *__pyx_v_help_text = 0;
  PyObject *__pyx_v_style = 0;
  PyObject *__pyx_v_validators = 0;
  PyObject *__pyx_v_default_value = 0;
  PyObject *__pyx_v_initial = 0;
  PyObject *__pyx_v_field_name = 0;
  PyObject *__pyx_v_root = 0;
  PyObject *__pyx_v_error_messages = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_attr,&__pyx_n_s_call,&__pyx_n_s_required,&__pyx_n_s_write_only,&__pyx_n_s_read_only,&__pyx_n_s_allow_null,&__pyx_n_s_label,&__pyx_n_s_help_text,&__pyx_n_s_style,&__pyx_n_s_validators,&__pyx_n_s_default_value,&__pyx_n_s_initial,&__pyx_n_s_field_name,&__pyx_n_s_root,&__pyx_n_s_error_messages,0};
    PyObject* values[15] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};

    /* "drf_turbo/fields.pyx":76
 *     def __init__(
 *         self,
 *         basestring attr = None,             # <<<<<<<<<<<<<<
 *         bint call= False,
 *         bint required= True,
 */
    values[0] = ((PyObject*)Py_None);

    /* "drf_turbo/fields.pyx":82
 *         bint read_only = False,
 *         bint allow_null = False,
 *         basestring label = None,             # <<<<<<<<<<<<<<
 *         basestring help_text = None,
 *         dict style = None ,
 */
    values[6] = ((PyObject*)Py_None);

    /* "drf_turbo/fields.pyx":83
 *         bint allow_null = False,
 *         basestring label = None,
 *         basestring help_text = None,             # <<<<<<<<<<<<<<
 *         dict style = None ,
 *         object validators = None,
 */
    values[7] = ((PyObject*)Py_None);

    /* "drf_turbo/fields.pyx":84
 *         basestring label = None,
 *         basestring help_text = None,
 *         dict style = None ,             # <<<<<<<<<<<<<<
 *         object validators = None,
 *         object default_value  = NO_DEFAULT,
 */
    values[8] = ((PyObject*)Py_None);

    /* "drf_turbo/fields.pyx":85
 *         basestring help_text = None,
 *         dict style = None ,
 *         object validators = None,             # <<<<<<<<<<<<<<
 *         object default_value  = NO_DEFAULT,
 *         object initial = NO_DEFAULT,
 */
    values[9] = ((PyObject *)Py_None);
    values[10] = __pyx_k_;
    values[11] = __pyx_k__2;

    /* "drf_turbo/fields.pyx":88
 *         object default_value  = NO_DEFAULT,
 *         object initial = NO_DEFAULT,
 *         basestring field_name = None,             # <<<<<<<<<<<<<<
 *         object root = None ,
 *         dict error_messages = None,
 */
    values[12] = ((PyObject*)Py_None);

    /* "drf_turbo/fields.pyx":89
 *         object initial = NO_DEFAULT,
 *         basestring field_name = None,
 *         object root = None ,             # <<<<<<<<<<<<<<
 *         dict error_messages = None,
 * 
 */
    values[13] = ((PyObject *)Py_None);

    /* "drf_turbo/fields.pyx":90
 *         basestring field_name = None,
 *         object root = None ,
 *         dict error_messages = None,             # <<<<<<<<<<<<<<
 * 
 *     ):
 */
    values[14] = ((PyObject*)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case 15: values[14] = PyTuple_GET_ITEM(__pyx_args, 14);
        CYTHON_FALLTHROUGH;
        case 14: values[13] = PyTuple_GET_ITEM(__pyx_args, 13);
        CYTHON_FALLTHROUGH;
        case 13: values[12] = PyTuple_GET_ITEM(__pyx_args, 12);
        CYTHON_FALLTHROUGH;
        case 12: values[11] = PyTuple_GET_ITEM(__pyx_args, 11);
        CYTHON_FALLTHROUGH;
        case 11: values[10] = PyTuple_GET_ITEM(__pyx_args, 10);
        CYTHON_FALLTHROUGH;
        case 10: values[9] = PyTuple_GET_ITEM(__pyx_args, 9);
        CYTHON_FALLTHROUGH;
        case  9: values[8] = PyTuple_GET_ITEM(__pyx_args, 8);
        CYTHON_FALLTHROUGH;
        case  8: values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
        CYTHON_FALLTHROUGH;
        case  7: values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
        CYTHON_FALLTHROUGH;
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_attr);
          if (value) { values[0] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_call);
          if (value) { values[1] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_required);
          if (value) { values[2] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_write_only);
          if (value) { values[3] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_read_only);
          if (value) { values[4] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  5:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_allow_null);
          if (value) { values[5] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  6:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_label);
          if (value) { values[6] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  7:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_help_text);
          if (value) { values[7] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  8:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_style);
          if (value) { values[8] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  9:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_validators);
          if (value) { values[9] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case 10:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_default_value);
          if (value) { values[10] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case 11:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_initial);
          if (value) { values[11] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case 12:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_field_name);
          if (value) { values[12] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case 13:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_root);
          if (value) { values[13] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case 14:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_error_messages);
          if (value) { values[14] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 74, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case 15: values[14] = PyTuple_GET_ITEM(__pyx_args, 14);
        CYTHON_FALLTHROUGH;
        case 14: values[13] = PyTuple_GET_ITEM(__pyx_args, 13);
        CYTHON_FALLTHROUGH;
        case 13: values[12] = PyTuple_GET_ITEM(__pyx_args, 12);
        CYTHON_FALLTHROUGH;
        case 12: values[11] = PyTuple_GET_ITEM(__pyx_args, 11);
        CYTHON_FALLTHROUGH;
        case 11: values[10] = PyTuple_GET_ITEM(__pyx_args, 10);
        CYTHON_FALLTHROUGH;
        case 10: values[9] = PyTuple_GET_ITEM(__pyx_args, 9);
        CYTHON_FALLTHROUGH;
        case  9: values[8] = PyTuple_GET_ITEM(__pyx_args, 8);
        CYTHON_FALLTHROUGH;
        case  8: values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
        CYTHON_FALLTHROUGH;
        case  7: values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
        CYTHON_FALLTHROUGH;
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_attr = ((PyObject*)values[0]);
    if (values[1]) {
      __pyx_v_call = __Pyx_PyObject_IsTrue(values[1]); if (unlikely((__pyx_v_call == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 77, __pyx_L3_error)
    } else {

      /* "drf_turbo/fields.pyx":77
 *         self,
 *         basestring attr = None,
 *         bint call= False,             # <<<<<<<<<<<<<<
 *         bint required= True,
 *         bint write_only = False,
 */
      __pyx_v_call = ((int)0);
    }
    if (values[2]) {
      __pyx_v_required = __Pyx_PyObject_IsTrue(values[2]); if (unlikely((__pyx_v_required == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 78, __pyx_L3_error)
    } else {

      /* "drf_turbo/fields.pyx":78
 *         basestring attr = None,
 *         bint call= False,
 *         bint required= True,             # <<<<<<<<<<<<<<
 *         bint write_only = False,
 *         bint read_only = False,
 */
      __pyx_v_required = ((int)1);
    }
    if (values[3]) {
      __pyx_v_write_only = __Pyx_PyObject_IsTrue(values[3]); if (unlikely((__pyx_v_write_only == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 79, __pyx_L3_error)
    } else {

      /* "drf_turbo/fields.pyx":79
 *         bint call= False,
 *         bint required= True,
 *         bint write_only = False,             # <<<<<<<<<<<<<<
 *         bint read_only = False,
 *         bint allow_null = False,
 */
      __pyx_v_write_only = ((int)0);
    }
    if (values[4]) {
      __pyx_v_read_only = __Pyx_PyObject_IsTrue(values[4]); if (unlikely((__pyx_v_read_only == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 80, __pyx_L3_error)
    } else {

      /* "drf_turbo/fields.pyx":80
 *         bint required= True,
 *         bint write_only = False,
 *         bint read_only = False,             # <<<<<<<<<<<<<<
 *         bint allow_null = False,
 *         basestring label = None,
 */
      __pyx_v_read_only = ((int)0);
    }
    if (values[5]) {
      __pyx_v_allow_null = __Pyx_PyObject_IsTrue(values[5]); if (unlikely((__pyx_v_allow_null == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 81, __pyx_L3_error)
    } else {

      /* "drf_turbo/fields.pyx":81
 *         bint write_only = False,
 *         bint read_only = False,
 *         bint allow_null = False,             # <<<<<<<<<<<<<<
 *         basestring label = None,
 *         basestring help_text = None,
 */
      __pyx_v_allow_null = ((int)0);
    }
    __pyx_v_label = ((PyObject*)values[6]);
    __pyx_v_help_text = ((PyObject*)values[7]);
    __pyx_v_style = ((PyObject*)values[8]);
    __pyx_v_validators = values[9];
    __pyx_v_default_value = values[10];
    __pyx_v_initial = values[11];
    __pyx_v_field_name = ((PyObject*)values[12]);
    __pyx_v_root = values[13];
    __pyx_v_error_messages = ((PyObject*)values[14]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 0, 15, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 74, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.Field.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_attr), (&PyBaseString_Type), 1, "attr", 1))) __PYX_ERR(0, 76, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_label), (&PyBaseString_Type), 1, "label", 1))) __PYX_ERR(0, 82, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_help_text), (&PyBaseString_Type), 1, "help_text", 1))) __PYX_ERR(0, 83, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_style), (&PyDict_Type), 1, "style", 1))) __PYX_ERR(0, 84, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_field_name), (&PyBaseString_Type), 1, "field_name", 1))) __PYX_ERR(0, 88, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_error_messages), (&PyDict_Type), 1, "error_messages", 1))) __PYX_ERR(0, 90, __pyx_L1_error)
  __pyx_r = __pyx_pf_9drf_turbo_6fields_5Field___init__(((struct __pyx_obj_9drf_turbo_6fields_Field *)__pyx_v_self), __pyx_v_attr, __pyx_v_call, __pyx_v_required, __pyx_v_write_only, __pyx_v_read_only, __pyx_v_allow_null, __pyx_v_label, __pyx_v_help_text, __pyx_v_style, __pyx_v_validators, __pyx_v_default_value, __pyx_v_initial, __pyx_v_field_name, __pyx_v_root, __pyx_v_error_messages);

  /* "drf_turbo/fields.pyx":74
 *     _initial = None
 * 
 *     def __init__(             # <<<<<<<<<<<<<<
 *         self,
 *         basestring attr = None,
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_5Field___init__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, PyObject *__pyx_v_attr, int __pyx_v_call, int __pyx_v_required, int __pyx_v_write_only, int __pyx_v_read_only, int __pyx_v_allow_null, PyObject *__pyx_v_label, PyObject *__pyx_v_help_text, PyObject *__pyx_v_style, PyObject *__pyx_v_validators, PyObject *__pyx_v_default_value, PyObject *__pyx_v_initial, PyObject *__pyx_v_field_name, PyObject *__pyx_v_root, PyObject *__pyx_v_error_messages) {
  PyObject *__pyx_v_messages = NULL;
  PyObject *__pyx_v_cls = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  Py_ssize_t __pyx_t_7;
  PyObject *(*__pyx_t_8)(PyObject *);
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "drf_turbo/fields.pyx":93
 * 
 *     ):
 *         required = False if default_value is not NO_DEFAULT else required             # <<<<<<<<<<<<<<
 *         assert not (read_only and write_only), 'May not set both `read_only` and `write_only`'
 *         assert not (required and default_value is not NO_DEFAULT), 'May not set both `required` and `default_value`'
 */
  __pyx_t_2 = (__pyx_v_default_value != __pyx_v_9drf_turbo_6fields_NO_DEFAULT);
  if ((__pyx_t_2 != 0)) {
    __pyx_t_1 = 0;
  } else {
    __pyx_t_1 = __pyx_v_required;
  }
  __pyx_v_required = __pyx_t_1;

  /* "drf_turbo/fields.pyx":94
 *     ):
 *         required = False if default_value is not NO_DEFAULT else required
 *         assert not (read_only and write_only), 'May not set both `read_only` and `write_only`'             # <<<<<<<<<<<<<<
 *         assert not (required and default_value is not NO_DEFAULT), 'May not set both `required` and `default_value`'
 * 
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    __pyx_t_2 = (__pyx_v_read_only != 0);
    if (__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L3_bool_binop_done;
    }
    __pyx_t_2 = (__pyx_v_write_only != 0);
    __pyx_t_1 = __pyx_t_2;
    __pyx_L3_bool_binop_done:;
    if (unlikely(!((!__pyx_t_1) != 0))) {
      PyErr_SetObject(PyExc_AssertionError, __pyx_kp_s_May_not_set_both_read_only_and_w);
      __PYX_ERR(0, 94, __pyx_L1_error)
    }
  }
  #endif

  /* "drf_turbo/fields.pyx":95
 *         required = False if default_value is not NO_DEFAULT else required
 *         assert not (read_only and write_only), 'May not set both `read_only` and `write_only`'
 *         assert not (required and default_value is not NO_DEFAULT), 'May not set both `required` and `default_value`'             # <<<<<<<<<<<<<<
 * 
 *         self.attr = attr
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    __pyx_t_2 = (__pyx_v_required != 0);
    if (__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L5_bool_binop_done;
    }
    __pyx_t_2 = (__pyx_v_default_value != __pyx_v_9drf_turbo_6fields_NO_DEFAULT);
    __pyx_t_3 = (__pyx_t_2 != 0);
    __pyx_t_1 = __pyx_t_3;
    __pyx_L5_bool_binop_done:;
    if (unlikely(!((!__pyx_t_1) != 0))) {
      PyErr_SetObject(PyExc_AssertionError, __pyx_kp_s_May_not_set_both_required_and_de);
      __PYX_ERR(0, 95, __pyx_L1_error)
    }
  }
  #endif

  /* "drf_turbo/fields.pyx":97
 *         assert not (required and default_value is not NO_DEFAULT), 'May not set both `required` and `default_value`'
 * 
 *         self.attr = attr             # <<<<<<<<<<<<<<
 *         self.call = call
 *         self.required = required
 */
  if (!(likely(PyString_CheckExact(__pyx_v_attr))||((__pyx_v_attr) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_v_attr)->tp_name), 0))) __PYX_ERR(0, 97, __pyx_L1_error)
  __pyx_t_4 = __pyx_v_attr;
  __Pyx_INCREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_4);
  __Pyx_GOTREF(__pyx_v_self->attr);
  __Pyx_DECREF(__pyx_v_self->attr);
  __pyx_v_self->attr = ((PyObject*)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "drf_turbo/fields.pyx":98
 * 
 *         self.attr = attr
 *         self.call = call             # <<<<<<<<<<<<<<
 *         self.required = required
 *         self.write_only = write_only
 */
  __pyx_v_self->call = __pyx_v_call;

  /* "drf_turbo/fields.pyx":99
 *         self.attr = attr
 *         self.call = call
 *         self.required = required             # <<<<<<<<<<<<<<
 *         self.write_only = write_only
 *         self.read_only = (read_only or call or
 */
  __pyx_v_self->required = __pyx_v_required;

  /* "drf_turbo/fields.pyx":100
 *         self.call = call
 *         self.required = required
 *         self.write_only = write_only             # <<<<<<<<<<<<<<
 *         self.read_only = (read_only or call or
 *                           (attr is not None and '.' in attr)) # type: ignore
 */
  __pyx_v_self->write_only = __pyx_v_write_only;

  /* "drf_turbo/fields.pyx":101
 *         self.required = required
 *         self.write_only = write_only
 *         self.read_only = (read_only or call or             # <<<<<<<<<<<<<<
 *                           (attr is not None and '.' in attr)) # type: ignore
 *         self.allow_null = allow_null
 */
  __pyx_t_3 = (__pyx_v_read_only != 0);
  if (!__pyx_t_3) {
  } else {
    __pyx_t_1 = __pyx_t_3;
    goto __pyx_L7_bool_binop_done;
  }
  __pyx_t_3 = (__pyx_v_call != 0);
  if (!__pyx_t_3) {
  } else {
    __pyx_t_1 = __pyx_t_3;
    goto __pyx_L7_bool_binop_done;
  }

  /* "drf_turbo/fields.pyx":102
 *         self.write_only = write_only
 *         self.read_only = (read_only or call or
 *                           (attr is not None and '.' in attr)) # type: ignore             # <<<<<<<<<<<<<<
 *         self.allow_null = allow_null
 *         self.label = label
 */
  __pyx_t_3 = (__pyx_v_attr != ((PyObject*)Py_None));
  __pyx_t_2 = (__pyx_t_3 != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L7_bool_binop_done;
  }
  __pyx_t_2 = (__Pyx_PySequence_ContainsTF(__pyx_kp_s__3, __pyx_v_attr, Py_EQ)); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 102, __pyx_L1_error)
  __pyx_t_3 = (__pyx_t_2 != 0);
  __pyx_t_1 = __pyx_t_3;
  __pyx_L7_bool_binop_done:;

  /* "drf_turbo/fields.pyx":101
 *         self.required = required
 *         self.write_only = write_only
 *         self.read_only = (read_only or call or             # <<<<<<<<<<<<<<
 *                           (attr is not None and '.' in attr)) # type: ignore
 *         self.allow_null = allow_null
 */
  __pyx_v_self->read_only = __pyx_t_1;

  /* "drf_turbo/fields.pyx":103
 *         self.read_only = (read_only or call or
 *                           (attr is not None and '.' in attr)) # type: ignore
 *         self.allow_null = allow_null             # <<<<<<<<<<<<<<
 *         self.label = label
 *         self.default_value = default_value
 */
  __pyx_v_self->allow_null = __pyx_v_allow_null;

  /* "drf_turbo/fields.pyx":104
 *                           (attr is not None and '.' in attr)) # type: ignore
 *         self.allow_null = allow_null
 *         self.label = label             # <<<<<<<<<<<<<<
 *         self.default_value = default_value
 *         self.initial = self._initial if (initial is NO_DEFAULT) else initial
 */
  if (!(likely(PyString_CheckExact(__pyx_v_label))||((__pyx_v_label) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_v_label)->tp_name), 0))) __PYX_ERR(0, 104, __pyx_L1_error)
  __pyx_t_4 = __pyx_v_label;
  __Pyx_INCREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_4);
  __Pyx_GOTREF(__pyx_v_self->label);
  __Pyx_DECREF(__pyx_v_self->label);
  __pyx_v_self->label = ((PyObject*)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "drf_turbo/fields.pyx":105
 *         self.allow_null = allow_null
 *         self.label = label
 *         self.default_value = default_value             # <<<<<<<<<<<<<<
 *         self.initial = self._initial if (initial is NO_DEFAULT) else initial
 *         self.help_text = help_text
 */
  __Pyx_INCREF(__pyx_v_default_value);
  __Pyx_GIVEREF(__pyx_v_default_value);
  __Pyx_GOTREF(__pyx_v_self->default_value);
  __Pyx_DECREF(__pyx_v_self->default_value);
  __pyx_v_self->default_value = __pyx_v_default_value;

  /* "drf_turbo/fields.pyx":106
 *         self.label = label
 *         self.default_value = default_value
 *         self.initial = self._initial if (initial is NO_DEFAULT) else initial             # <<<<<<<<<<<<<<
 *         self.help_text = help_text
 *         self.style = {} if style is None else style
 */
  __pyx_t_1 = (__pyx_v_initial == __pyx_v_9drf_turbo_6fields_NO_DEFAULT);
  if ((__pyx_t_1 != 0)) {
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_initial_2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 106, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_4 = __pyx_t_5;
    __pyx_t_5 = 0;
  } else {
    __Pyx_INCREF(__pyx_v_initial);
    __pyx_t_4 = __pyx_v_initial;
  }
  __Pyx_GIVEREF(__pyx_t_4);
  __Pyx_GOTREF(__pyx_v_self->initial);
  __Pyx_DECREF(__pyx_v_self->initial);
  __pyx_v_self->initial = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "drf_turbo/fields.pyx":107
 *         self.default_value = default_value
 *         self.initial = self._initial if (initial is NO_DEFAULT) else initial
 *         self.help_text = help_text             # <<<<<<<<<<<<<<
 *         self.style = {} if style is None else style
 *         self.field_name = field_name
 */
  if (!(likely(PyString_CheckExact(__pyx_v_help_text))||((__pyx_v_help_text) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_v_help_text)->tp_name), 0))) __PYX_ERR(0, 107, __pyx_L1_error)
  __pyx_t_4 = __pyx_v_help_text;
  __Pyx_INCREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_4);
  __Pyx_GOTREF(__pyx_v_self->help_text);
  __Pyx_DECREF(__pyx_v_self->help_text);
  __pyx_v_self->help_text = ((PyObject*)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "drf_turbo/fields.pyx":108
 *         self.initial = self._initial if (initial is NO_DEFAULT) else initial
 *         self.help_text = help_text
 *         self.style = {} if style is None else style             # <<<<<<<<<<<<<<
 *         self.field_name = field_name
 *         self.root = root
 */
  __pyx_t_1 = (__pyx_v_style == ((PyObject*)Py_None));
  if ((__pyx_t_1 != 0)) {
    __pyx_t_5 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 108, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_4 = __pyx_t_5;
    __pyx_t_5 = 0;
  } else {
    __Pyx_INCREF(__pyx_v_style);
    __pyx_t_4 = __pyx_v_style;
  }
  __Pyx_GIVEREF(__pyx_t_4);
  __Pyx_GOTREF(__pyx_v_self->style);
  __Pyx_DECREF(__pyx_v_self->style);
  __pyx_v_self->style = ((PyObject*)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "drf_turbo/fields.pyx":109
 *         self.help_text = help_text
 *         self.style = {} if style is None else style
 *         self.field_name = field_name             # <<<<<<<<<<<<<<
 *         self.root = root
 *         if validators is None:
 */
  if (!(likely(PyString_CheckExact(__pyx_v_field_name))||((__pyx_v_field_name) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_v_field_name)->tp_name), 0))) __PYX_ERR(0, 109, __pyx_L1_error)
  __pyx_t_4 = __pyx_v_field_name;
  __Pyx_INCREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_4);
  __Pyx_GOTREF(__pyx_v_self->field_name);
  __Pyx_DECREF(__pyx_v_self->field_name);
  __pyx_v_self->field_name = ((PyObject*)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "drf_turbo/fields.pyx":110
 *         self.style = {} if style is None else style
 *         self.field_name = field_name
 *         self.root = root             # <<<<<<<<<<<<<<
 *         if validators is None:
 *             self.validators = []
 */
  __Pyx_INCREF(__pyx_v_root);
  __Pyx_GIVEREF(__pyx_v_root);
  __Pyx_GOTREF(__pyx_v_self->root);
  __Pyx_DECREF(__pyx_v_self->root);
  __pyx_v_self->root = __pyx_v_root;

  /* "drf_turbo/fields.pyx":111
 *         self.field_name = field_name
 *         self.root = root
 *         if validators is None:             # <<<<<<<<<<<<<<
 *             self.validators = []
 *         elif callable(validators):
 */
  __pyx_t_1 = (__pyx_v_validators == Py_None);
  __pyx_t_3 = (__pyx_t_1 != 0);
  if (__pyx_t_3) {

    /* "drf_turbo/fields.pyx":112
 *         self.root = root
 *         if validators is None:
 *             self.validators = []             # <<<<<<<<<<<<<<
 *         elif callable(validators):
 *             self.validators = [validators]
 */
    __pyx_t_4 = PyList_New(0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 112, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_4);
    __Pyx_GOTREF(__pyx_v_self->validators);
    __Pyx_DECREF(__pyx_v_self->validators);
    __pyx_v_self->validators = __pyx_t_4;
    __pyx_t_4 = 0;

    /* "drf_turbo/fields.pyx":111
 *         self.field_name = field_name
 *         self.root = root
 *         if validators is None:             # <<<<<<<<<<<<<<
 *             self.validators = []
 *         elif callable(validators):
 */
    goto __pyx_L11;
  }

  /* "drf_turbo/fields.pyx":113
 *         if validators is None:
 *             self.validators = []
 *         elif callable(validators):             # <<<<<<<<<<<<<<
 *             self.validators = [validators]
 *         elif is_iterable_and_not_string(validators) :
 */
  __pyx_t_3 = __Pyx_PyCallable_Check(__pyx_v_validators); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(0, 113, __pyx_L1_error)
  __pyx_t_1 = (__pyx_t_3 != 0);
  if (__pyx_t_1) {

    /* "drf_turbo/fields.pyx":114
 *             self.validators = []
 *         elif callable(validators):
 *             self.validators = [validators]             # <<<<<<<<<<<<<<
 *         elif is_iterable_and_not_string(validators) :
 *             self.validators = list(validators)
 */
    __pyx_t_4 = PyList_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 114, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(__pyx_v_validators);
    __Pyx_GIVEREF(__pyx_v_validators);
    PyList_SET_ITEM(__pyx_t_4, 0, __pyx_v_validators);
    __Pyx_GIVEREF(__pyx_t_4);
    __Pyx_GOTREF(__pyx_v_self->validators);
    __Pyx_DECREF(__pyx_v_self->validators);
    __pyx_v_self->validators = __pyx_t_4;
    __pyx_t_4 = 0;

    /* "drf_turbo/fields.pyx":113
 *         if validators is None:
 *             self.validators = []
 *         elif callable(validators):             # <<<<<<<<<<<<<<
 *             self.validators = [validators]
 *         elif is_iterable_and_not_string(validators) :
 */
    goto __pyx_L11;
  }

  /* "drf_turbo/fields.pyx":115
 *         elif callable(validators):
 *             self.validators = [validators]
 *         elif is_iterable_and_not_string(validators) :             # <<<<<<<<<<<<<<
 *             self.validators = list(validators)
 *         else:
 */
  __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_is_iterable_and_not_string); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 115, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
    }
  }
  __pyx_t_4 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_6, __pyx_v_validators) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_v_validators);
  __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 115, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 115, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (likely(__pyx_t_1)) {

    /* "drf_turbo/fields.pyx":116
 *             self.validators = [validators]
 *         elif is_iterable_and_not_string(validators) :
 *             self.validators = list(validators)             # <<<<<<<<<<<<<<
 *         else:
 *             raise ValueError(
 */
    __pyx_t_4 = PySequence_List(__pyx_v_validators); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 116, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_4);
    __Pyx_GOTREF(__pyx_v_self->validators);
    __Pyx_DECREF(__pyx_v_self->validators);
    __pyx_v_self->validators = __pyx_t_4;
    __pyx_t_4 = 0;

    /* "drf_turbo/fields.pyx":115
 *         elif callable(validators):
 *             self.validators = [validators]
 *         elif is_iterable_and_not_string(validators) :             # <<<<<<<<<<<<<<
 *             self.validators = list(validators)
 *         else:
 */
    goto __pyx_L11;
  }

  /* "drf_turbo/fields.pyx":118
 *             self.validators = list(validators)
 *         else:
 *             raise ValueError(             # <<<<<<<<<<<<<<
 *                 "The 'validators' parameter must be a callable "
 *                 "or a collection of callables."
 */
  /*else*/ {
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__4, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 118, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(0, 118, __pyx_L1_error)
  }
  __pyx_L11:;

  /* "drf_turbo/fields.pyx":123
 *             )
 * 
 *         messages = {}             # <<<<<<<<<<<<<<
 *         for cls in reversed(self.__class__.__mro__):
 *             messages.update(getattr(cls, 'default_error_messages', {}))
 */
  __pyx_t_4 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 123, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_v_messages = ((PyObject*)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "drf_turbo/fields.pyx":124
 * 
 *         messages = {}
 *         for cls in reversed(self.__class__.__mro__):             # <<<<<<<<<<<<<<
 *             messages.update(getattr(cls, 'default_error_messages', {}))
 *         messages.update(error_messages or {})
 */
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_class); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 124, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_mro); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 124, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_builtin_reversed, __pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 124, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (likely(PyList_CheckExact(__pyx_t_4)) || PyTuple_CheckExact(__pyx_t_4)) {
    __pyx_t_5 = __pyx_t_4; __Pyx_INCREF(__pyx_t_5); __pyx_t_7 = 0;
    __pyx_t_8 = NULL;
  } else {
    __pyx_t_7 = -1; __pyx_t_5 = PyObject_GetIter(__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 124, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_8 = Py_TYPE(__pyx_t_5)->tp_iternext; if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 124, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  for (;;) {
    if (likely(!__pyx_t_8)) {
      if (likely(PyList_CheckExact(__pyx_t_5))) {
        if (__pyx_t_7 >= PyList_GET_SIZE(__pyx_t_5)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_4 = PyList_GET_ITEM(__pyx_t_5, __pyx_t_7); __Pyx_INCREF(__pyx_t_4); __pyx_t_7++; if (unlikely(0 < 0)) __PYX_ERR(0, 124, __pyx_L1_error)
        #else
        __pyx_t_4 = PySequence_ITEM(__pyx_t_5, __pyx_t_7); __pyx_t_7++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 124, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        #endif
      } else {
        if (__pyx_t_7 >= PyTuple_GET_SIZE(__pyx_t_5)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_4 = PyTuple_GET_ITEM(__pyx_t_5, __pyx_t_7); __Pyx_INCREF(__pyx_t_4); __pyx_t_7++; if (unlikely(0 < 0)) __PYX_ERR(0, 124, __pyx_L1_error)
        #else
        __pyx_t_4 = PySequence_ITEM(__pyx_t_5, __pyx_t_7); __pyx_t_7++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 124, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        #endif
      }
    } else {
      __pyx_t_4 = __pyx_t_8(__pyx_t_5);
      if (unlikely(!__pyx_t_4)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 124, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_4);
    }
    __Pyx_XDECREF_SET(__pyx_v_cls, __pyx_t_4);
    __pyx_t_4 = 0;

    /* "drf_turbo/fields.pyx":125
 *         messages = {}
 *         for cls in reversed(self.__class__.__mro__):
 *             messages.update(getattr(cls, 'default_error_messages', {}))             # <<<<<<<<<<<<<<
 *         messages.update(error_messages or {})
 *         self.error_messages = messages
 */
    __pyx_t_4 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 125, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = __Pyx_GetAttr3(__pyx_v_cls, __pyx_n_s_default_error_messages, __pyx_t_4); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 125, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_CallUnboundCMethod1(&__pyx_umethod_PyDict_Type_update, __pyx_v_messages, __pyx_t_6); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 125, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "drf_turbo/fields.pyx":124
 * 
 *         messages = {}
 *         for cls in reversed(self.__class__.__mro__):             # <<<<<<<<<<<<<<
 *             messages.update(getattr(cls, 'default_error_messages', {}))
 *         messages.update(error_messages or {})
 */
  }
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "drf_turbo/fields.pyx":126
 *         for cls in reversed(self.__class__.__mro__):
 *             messages.update(getattr(cls, 'default_error_messages', {}))
 *         messages.update(error_messages or {})             # <<<<<<<<<<<<<<
 *         self.error_messages = messages
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_error_messages); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 126, __pyx_L1_error)
  if (!__pyx_t_1) {
  } else {
    __Pyx_INCREF(__pyx_v_error_messages);
    __pyx_t_5 = __pyx_v_error_messages;
    goto __pyx_L14_bool_binop_done;
  }
  __pyx_t_4 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 126, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_INCREF(__pyx_t_4);
  __pyx_t_5 = __pyx_t_4;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_L14_bool_binop_done:;
  __pyx_t_4 = __Pyx_CallUnboundCMethod1(&__pyx_umethod_PyDict_Type_update, __pyx_v_messages, __pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 126, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "drf_turbo/fields.pyx":127
 *             messages.update(getattr(cls, 'default_error_messages', {}))
 *         messages.update(error_messages or {})
 *         self.error_messages = messages             # <<<<<<<<<<<<<<
 * 
 *     def raise_if_fail(self, key: str, **kwargs) :
 */
  __Pyx_INCREF(__pyx_v_messages);
  __Pyx_GIVEREF(__pyx_v_messages);
  __Pyx_GOTREF(__pyx_v_self->error_messages);
  __Pyx_DECREF(__pyx_v_self->error_messages);
  __pyx_v_self->error_messages = __pyx_v_messages;

  /* "drf_turbo/fields.pyx":74
 *     _initial = None
 * 
 *     def __init__(             # <<<<<<<<<<<<<<
 *         self,
 *         basestring attr = None,
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("drf_turbo.fields.Field.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_messages);
  __Pyx_XDECREF(__pyx_v_cls);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":129
 *         self.error_messages = messages
 * 
 *     def raise_if_fail(self, key: str, **kwargs) :             # <<<<<<<<<<<<<<
 *         """Helper method to make a `ValidationError` with an error message
 *         from ``self.error_messages``.
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_5Field_3raise_if_fail(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_9drf_turbo_6fields_5Field_2raise_if_fail[] = "Helper method to make a `ValidationError` with an error message\n        from ``self.error_messages``.\n        ";
static PyObject *__pyx_pw_9drf_turbo_6fields_5Field_3raise_if_fail(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_key = 0;
  PyObject *__pyx_v_kwargs = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("raise_if_fail (wrapper)", 0);
  __pyx_v_kwargs = PyDict_New(); if (unlikely(!__pyx_v_kwargs)) return NULL;
  __Pyx_GOTREF(__pyx_v_kwargs);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_key,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_key)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, __pyx_v_kwargs, values, pos_args, "raise_if_fail") < 0)) __PYX_ERR(0, 129, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_key = ((PyObject*)values[0]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("raise_if_fail", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 129, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_DECREF(__pyx_v_kwargs); __pyx_v_kwargs = 0;
  __Pyx_AddTraceback("drf_turbo.fields.Field.raise_if_fail", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_key), (&PyString_Type), 1, "key", 1))) __PYX_ERR(0, 129, __pyx_L1_error)
  __pyx_r = __pyx_pf_9drf_turbo_6fields_5Field_2raise_if_fail(((struct __pyx_obj_9drf_turbo_6fields_Field *)__pyx_v_self), __pyx_v_key, __pyx_v_kwargs);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_5Field_2raise_if_fail(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_kwargs) {
  PyObject *__pyx_v_msg = NULL;
  PyObject *__pyx_v_error = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  int __pyx_t_10;
  int __pyx_t_11;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("raise_if_fail", 0);

  /* "drf_turbo/fields.pyx":133
 *         from ``self.error_messages``.
 *         """
 *         try:             # <<<<<<<<<<<<<<
 *             msg = self.error_messages[key]
 *         except KeyError as error:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "drf_turbo/fields.pyx":134
 *         """
 *         try:
 *             msg = self.error_messages[key]             # <<<<<<<<<<<<<<
 *         except KeyError as error:
 *             raise AssertionError(error)
 */
      if (unlikely(__pyx_v_self->error_messages == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
        __PYX_ERR(0, 134, __pyx_L3_error)
      }
      __pyx_t_4 = __Pyx_PyDict_GetItem(__pyx_v_self->error_messages, __pyx_v_key); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 134, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_v_msg = __pyx_t_4;
      __pyx_t_4 = 0;

      /* "drf_turbo/fields.pyx":133
 *         from ``self.error_messages``.
 *         """
 *         try:             # <<<<<<<<<<<<<<
 *             msg = self.error_messages[key]
 *         except KeyError as error:
 */
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "drf_turbo/fields.pyx":135
 *         try:
 *             msg = self.error_messages[key]
 *         except KeyError as error:             # <<<<<<<<<<<<<<
 *             raise AssertionError(error)
 *         if isinstance(msg, (str, bytes)):
 */
    __pyx_t_5 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_KeyError);
    if (__pyx_t_5) {
      __Pyx_AddTraceback("drf_turbo.fields.Field.raise_if_fail", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_4, &__pyx_t_6, &__pyx_t_7) < 0) __PYX_ERR(0, 135, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_INCREF(__pyx_t_6);
      __pyx_v_error = __pyx_t_6;

      /* "drf_turbo/fields.pyx":136
 *             msg = self.error_messages[key]
 *         except KeyError as error:
 *             raise AssertionError(error)             # <<<<<<<<<<<<<<
 *         if isinstance(msg, (str, bytes)):
 *             msg = msg.format(**kwargs)
 */
      __pyx_t_8 = __Pyx_PyObject_CallOneArg(__pyx_builtin_AssertionError, __pyx_v_error); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 136, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_Raise(__pyx_t_8, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __PYX_ERR(0, 136, __pyx_L5_except_error)
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;

    /* "drf_turbo/fields.pyx":133
 *         from ``self.error_messages``.
 *         """
 *         try:             # <<<<<<<<<<<<<<
 *             msg = self.error_messages[key]
 *         except KeyError as error:
 */
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L8_try_end:;
  }

  /* "drf_turbo/fields.pyx":137
 *         except KeyError as error:
 *             raise AssertionError(error)
 *         if isinstance(msg, (str, bytes)):             # <<<<<<<<<<<<<<
 *             msg = msg.format(**kwargs)
 *         return ValidationError(msg)
 */
  __pyx_t_10 = PyString_Check(__pyx_v_msg); 
  __pyx_t_11 = (__pyx_t_10 != 0);
  if (!__pyx_t_11) {
  } else {
    __pyx_t_9 = __pyx_t_11;
    goto __pyx_L12_bool_binop_done;
  }
  __pyx_t_11 = PyBytes_Check(__pyx_v_msg); 
  __pyx_t_10 = (__pyx_t_11 != 0);
  __pyx_t_9 = __pyx_t_10;
  __pyx_L12_bool_binop_done:;
  __pyx_t_10 = (__pyx_t_9 != 0);
  if (__pyx_t_10) {

    /* "drf_turbo/fields.pyx":138
 *             raise AssertionError(error)
 *         if isinstance(msg, (str, bytes)):
 *             msg = msg.format(**kwargs)             # <<<<<<<<<<<<<<
 *         return ValidationError(msg)
 * 
 */
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_msg, __pyx_n_s_format); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 138, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_6 = PyDict_Copy(__pyx_v_kwargs); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 138, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_empty_tuple, __pyx_t_6); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 138, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF_SET(__pyx_v_msg, __pyx_t_4);
    __pyx_t_4 = 0;

    /* "drf_turbo/fields.pyx":137
 *         except KeyError as error:
 *             raise AssertionError(error)
 *         if isinstance(msg, (str, bytes)):             # <<<<<<<<<<<<<<
 *             msg = msg.format(**kwargs)
 *         return ValidationError(msg)
 */
  }

  /* "drf_turbo/fields.pyx":139
 *         if isinstance(msg, (str, bytes)):
 *             msg = msg.format(**kwargs)
 *         return ValidationError(msg)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_n_s_ValidationError); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 139, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_6))) {
    __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_6);
    if (likely(__pyx_t_7)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_7);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_6, function);
    }
  }
  __pyx_t_4 = (__pyx_t_7) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_7, __pyx_v_msg) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_v_msg);
  __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 139, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "drf_turbo/fields.pyx":129
 *         self.error_messages = messages
 * 
 *     def raise_if_fail(self, key: str, **kwargs) :             # <<<<<<<<<<<<<<
 *         """Helper method to make a `ValidationError` with an error message
 *         from ``self.error_messages``.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("drf_turbo.fields.Field.raise_if_fail", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_msg);
  __Pyx_XDECREF(__pyx_v_error);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":142
 * 
 * 
 *     cpdef serialize(self,value, dict context):             # <<<<<<<<<<<<<<
 *         """
 *         Transform the *outgoing* native value into primitive data
 */

static PyObject *__pyx_pw_9drf_turbo_6fields_5Field_5serialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_5Field_serialize(CYTHON_UNUSED struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, PyObject *__pyx_v_value, CYTHON_UNUSED PyObject *__pyx_v_context, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("serialize", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_serialize); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 142, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_9drf_turbo_6fields_5Field_5serialize)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        __pyx_t_5 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
            __pyx_t_5 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_value, __pyx_v_context};
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 142, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_value, __pyx_v_context};
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 142, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        {
          __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 142, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          if (__pyx_t_4) {
            __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
          }
          __Pyx_INCREF(__pyx_v_value);
          __Pyx_GIVEREF(__pyx_v_value);
          PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_v_value);
          __Pyx_INCREF(__pyx_v_context);
          __Pyx_GIVEREF(__pyx_v_context);
          PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_v_context);
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_6, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 142, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        }
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "drf_turbo/fields.pyx":149
 *         :param context: The context for the request.
 *         """
 *         return value             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_value);
  __pyx_r = __pyx_v_value;
  goto __pyx_L0;

  /* "drf_turbo/fields.pyx":142
 * 
 * 
 *     cpdef serialize(self,value, dict context):             # <<<<<<<<<<<<<<
 *         """
 *         Transform the *outgoing* native value into primitive data
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("drf_turbo.fields.Field.serialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_5Field_5serialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_9drf_turbo_6fields_5Field_4serialize[] = "\n        Transform the *outgoing* native value into primitive data\n\n        :param value: The outgoing value.\n        :param context: The context for the request.\n        ";
static PyObject *__pyx_pw_9drf_turbo_6fields_5Field_5serialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_value = 0;
  PyObject *__pyx_v_context = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("serialize (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_value,&__pyx_n_s_context,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_value)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_context)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("serialize", 1, 2, 2, 1); __PYX_ERR(0, 142, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "serialize") < 0)) __PYX_ERR(0, 142, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_value = values[0];
    __pyx_v_context = ((PyObject*)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("serialize", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 142, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.Field.serialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_context), (&PyDict_Type), 1, "context", 1))) __PYX_ERR(0, 142, __pyx_L1_error)
  __pyx_r = __pyx_pf_9drf_turbo_6fields_5Field_4serialize(((struct __pyx_obj_9drf_turbo_6fields_Field *)__pyx_v_self), __pyx_v_value, __pyx_v_context);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_5Field_4serialize(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, PyObject *__pyx_v_value, PyObject *__pyx_v_context) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("serialize", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields_5Field_serialize(__pyx_v_self, __pyx_v_value, __pyx_v_context, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 142, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.Field.serialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":152
 * 
 * 
 *     cpdef deserialize(self,data, dict context):             # <<<<<<<<<<<<<<
 *         """
 *         Transform the *incoming* primitive data into a native value.
 */

static PyObject *__pyx_pw_9drf_turbo_6fields_5Field_7deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_5Field_deserialize(CYTHON_UNUSED struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, PyObject *__pyx_v_data, CYTHON_UNUSED PyObject *__pyx_v_context, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("deserialize", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_deserialize); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 152, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_9drf_turbo_6fields_5Field_7deserialize)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        __pyx_t_5 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
            __pyx_t_5 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_data, __pyx_v_context};
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 152, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_data, __pyx_v_context};
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 152, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        {
          __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 152, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          if (__pyx_t_4) {
            __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
          }
          __Pyx_INCREF(__pyx_v_data);
          __Pyx_GIVEREF(__pyx_v_data);
          PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_v_data);
          __Pyx_INCREF(__pyx_v_context);
          __Pyx_GIVEREF(__pyx_v_context);
          PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_v_context);
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_6, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 152, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        }
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "drf_turbo/fields.pyx":159
 *         :param context: The context for the request.
 *         """
 *         return data             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_data);
  __pyx_r = __pyx_v_data;
  goto __pyx_L0;

  /* "drf_turbo/fields.pyx":152
 * 
 * 
 *     cpdef deserialize(self,data, dict context):             # <<<<<<<<<<<<<<
 *         """
 *         Transform the *incoming* primitive data into a native value.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("drf_turbo.fields.Field.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_5Field_7deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_9drf_turbo_6fields_5Field_6deserialize[] = "\n        Transform the *incoming* primitive data into a native value.\n        \n        :param data: The incoming data.\n        :param context: The context for the request.\n        ";
static PyObject *__pyx_pw_9drf_turbo_6fields_5Field_7deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_data = 0;
  PyObject *__pyx_v_context = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("deserialize (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_data,&__pyx_n_s_context,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_data)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_context)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("deserialize", 1, 2, 2, 1); __PYX_ERR(0, 152, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "deserialize") < 0)) __PYX_ERR(0, 152, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_data = values[0];
    __pyx_v_context = ((PyObject*)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("deserialize", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 152, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.Field.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_context), (&PyDict_Type), 1, "context", 1))) __PYX_ERR(0, 152, __pyx_L1_error)
  __pyx_r = __pyx_pf_9drf_turbo_6fields_5Field_6deserialize(((struct __pyx_obj_9drf_turbo_6fields_Field *)__pyx_v_self), __pyx_v_data, __pyx_v_context);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_5Field_6deserialize(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("deserialize", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields_5Field_deserialize(__pyx_v_self, __pyx_v_data, __pyx_v_context, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 152, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.Field.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":162
 * 
 * 
 *     cpdef method_getter(self,field_name, root) :             # <<<<<<<<<<<<<<
 *         """
 *         Returns a function that fetches an attribute from an object.
 */

static PyObject *__pyx_pw_9drf_turbo_6fields_5Field_9method_getter(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_5Field_method_getter(CYTHON_UNUSED struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_field_name, CYTHON_UNUSED PyObject *__pyx_v_root, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("method_getter", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_method_getter); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 162, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_9drf_turbo_6fields_5Field_9method_getter)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        __pyx_t_5 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
            __pyx_t_5 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_field_name, __pyx_v_root};
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 162, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_field_name, __pyx_v_root};
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 162, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        {
          __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 162, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          if (__pyx_t_4) {
            __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
          }
          __Pyx_INCREF(__pyx_v_field_name);
          __Pyx_GIVEREF(__pyx_v_field_name);
          PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_v_field_name);
          __Pyx_INCREF(__pyx_v_root);
          __Pyx_GIVEREF(__pyx_v_root);
          PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_v_root);
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_6, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 162, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        }
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "drf_turbo/fields.pyx":169
 *         :root: The root of the field.
 *         """
 *         return None             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;

  /* "drf_turbo/fields.pyx":162
 * 
 * 
 *     cpdef method_getter(self,field_name, root) :             # <<<<<<<<<<<<<<
 *         """
 *         Returns a function that fetches an attribute from an object.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("drf_turbo.fields.Field.method_getter", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_5Field_9method_getter(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_9drf_turbo_6fields_5Field_8method_getter[] = "\n        Returns a function that fetches an attribute from an object.\n\n        :field_name: The name of the attribute to get.\n        :root: The root of the field.\n        ";
static PyObject *__pyx_pw_9drf_turbo_6fields_5Field_9method_getter(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_field_name = 0;
  PyObject *__pyx_v_root = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("method_getter (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_field_name,&__pyx_n_s_root,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_field_name)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_root)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("method_getter", 1, 2, 2, 1); __PYX_ERR(0, 162, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "method_getter") < 0)) __PYX_ERR(0, 162, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_field_name = values[0];
    __pyx_v_root = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("method_getter", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 162, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.Field.method_getter", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9drf_turbo_6fields_5Field_8method_getter(((struct __pyx_obj_9drf_turbo_6fields_Field *)__pyx_v_self), __pyx_v_field_name, __pyx_v_root);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_5Field_8method_getter(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, PyObject *__pyx_v_field_name, PyObject *__pyx_v_root) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("method_getter", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields_5Field_method_getter(__pyx_v_self, __pyx_v_field_name, __pyx_v_root, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 162, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.Field.method_getter", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":172
 * 
 * 
 *     cpdef void bind(self,basestring field_name, object root):             # <<<<<<<<<<<<<<
 *         """
 *         Update the field name and root for the field instance.
 */

static PyObject *__pyx_pw_9drf_turbo_6fields_5Field_11bind(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static void __pyx_f_9drf_turbo_6fields_5Field_bind(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, PyObject *__pyx_v_field_name, PyObject *__pyx_v_root, int __pyx_skip_dispatch) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("bind", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_bind); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 172, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_9drf_turbo_6fields_5Field_11bind)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        __pyx_t_5 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
            __pyx_t_5 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_field_name, __pyx_v_root};
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 172, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_field_name, __pyx_v_root};
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 172, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        {
          __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 172, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          if (__pyx_t_4) {
            __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
          }
          __Pyx_INCREF(__pyx_v_field_name);
          __Pyx_GIVEREF(__pyx_v_field_name);
          PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_v_field_name);
          __Pyx_INCREF(__pyx_v_root);
          __Pyx_GIVEREF(__pyx_v_root);
          PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_v_root);
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_6, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 172, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        }
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "drf_turbo/fields.pyx":179
 *         :root: The root of the field.
 *         """
 *         self.field_name = field_name             # <<<<<<<<<<<<<<
 *         self.root = root
 *         if self.label is None:
 */
  if (!(likely(PyString_CheckExact(__pyx_v_field_name))||((__pyx_v_field_name) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_v_field_name)->tp_name), 0))) __PYX_ERR(0, 179, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_field_name;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->field_name);
  __Pyx_DECREF(__pyx_v_self->field_name);
  __pyx_v_self->field_name = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "drf_turbo/fields.pyx":180
 *         """
 *         self.field_name = field_name
 *         self.root = root             # <<<<<<<<<<<<<<
 *         if self.label is None:
 *             self.label = field_name.replace('_', ' ').capitalize()
 */
  __Pyx_INCREF(__pyx_v_root);
  __Pyx_GIVEREF(__pyx_v_root);
  __Pyx_GOTREF(__pyx_v_self->root);
  __Pyx_DECREF(__pyx_v_self->root);
  __pyx_v_self->root = __pyx_v_root;

  /* "drf_turbo/fields.pyx":181
 *         self.field_name = field_name
 *         self.root = root
 *         if self.label is None:             # <<<<<<<<<<<<<<
 *             self.label = field_name.replace('_', ' ').capitalize()
 * 
 */
  __pyx_t_7 = (__pyx_v_self->label == ((PyObject*)Py_None));
  __pyx_t_8 = (__pyx_t_7 != 0);
  if (__pyx_t_8) {

    /* "drf_turbo/fields.pyx":182
 *         self.root = root
 *         if self.label is None:
 *             self.label = field_name.replace('_', ' ').capitalize()             # <<<<<<<<<<<<<<
 * 
 *         if self.attr is None:
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_field_name, __pyx_n_s_replace); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 182, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__7, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 182, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_capitalize); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 182, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 182, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 182, __pyx_L1_error)
    __Pyx_GIVEREF(__pyx_t_1);
    __Pyx_GOTREF(__pyx_v_self->label);
    __Pyx_DECREF(__pyx_v_self->label);
    __pyx_v_self->label = ((PyObject*)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "drf_turbo/fields.pyx":181
 *         self.field_name = field_name
 *         self.root = root
 *         if self.label is None:             # <<<<<<<<<<<<<<
 *             self.label = field_name.replace('_', ' ').capitalize()
 * 
 */
  }

  /* "drf_turbo/fields.pyx":184
 *             self.label = field_name.replace('_', ' ').capitalize()
 * 
 *         if self.attr is None:             # <<<<<<<<<<<<<<
 *             self.attr = field_name
 * 
 */
  __pyx_t_8 = (__pyx_v_self->attr == ((PyObject*)Py_None));
  __pyx_t_7 = (__pyx_t_8 != 0);
  if (__pyx_t_7) {

    /* "drf_turbo/fields.pyx":185
 * 
 *         if self.attr is None:
 *             self.attr = field_name             # <<<<<<<<<<<<<<
 * 
 *         self.attrs = self.attr.split('.') if self.attr else []
 */
    if (!(likely(PyString_CheckExact(__pyx_v_field_name))||((__pyx_v_field_name) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_v_field_name)->tp_name), 0))) __PYX_ERR(0, 185, __pyx_L1_error)
    __pyx_t_1 = __pyx_v_field_name;
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_1);
    __Pyx_GOTREF(__pyx_v_self->attr);
    __Pyx_DECREF(__pyx_v_self->attr);
    __pyx_v_self->attr = ((PyObject*)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "drf_turbo/fields.pyx":184
 *             self.label = field_name.replace('_', ' ').capitalize()
 * 
 *         if self.attr is None:             # <<<<<<<<<<<<<<
 *             self.attr = field_name
 * 
 */
  }

  /* "drf_turbo/fields.pyx":187
 *             self.attr = field_name
 * 
 *         self.attrs = self.attr.split('.') if self.attr else []             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_v_self->attr); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 187, __pyx_L1_error)
  if (__pyx_t_7) {
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->attr, __pyx_n_s_split); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 187, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_6 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_2 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_6, __pyx_kp_s__3) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_kp_s__3);
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 187, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (!(likely(PyList_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_2)->tp_name), 0))) __PYX_ERR(0, 187, __pyx_L1_error)
    __pyx_t_1 = __pyx_t_2;
    __pyx_t_2 = 0;
  } else {
    __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 187, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = __pyx_t_2;
    __pyx_t_2 = 0;
  }
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->attrs);
  __Pyx_DECREF(__pyx_v_self->attrs);
  __pyx_v_self->attrs = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "drf_turbo/fields.pyx":172
 * 
 * 
 *     cpdef void bind(self,basestring field_name, object root):             # <<<<<<<<<<<<<<
 *         """
 *         Update the field name and root for the field instance.
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_WriteUnraisable("drf_turbo.fields.Field.bind", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_5Field_11bind(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_9drf_turbo_6fields_5Field_10bind[] = "\n        Update the field name and root for the field instance.\n\n        :field_name: The name of the field.\n        :root: The root of the field.\n        ";
static PyObject *__pyx_pw_9drf_turbo_6fields_5Field_11bind(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_field_name = 0;
  PyObject *__pyx_v_root = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("bind (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_field_name,&__pyx_n_s_root,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_field_name)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_root)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("bind", 1, 2, 2, 1); __PYX_ERR(0, 172, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "bind") < 0)) __PYX_ERR(0, 172, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_field_name = ((PyObject*)values[0]);
    __pyx_v_root = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("bind", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 172, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.Field.bind", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_field_name), (&PyBaseString_Type), 1, "field_name", 1))) __PYX_ERR(0, 172, __pyx_L1_error)
  __pyx_r = __pyx_pf_9drf_turbo_6fields_5Field_10bind(((struct __pyx_obj_9drf_turbo_6fields_Field *)__pyx_v_self), __pyx_v_field_name, __pyx_v_root);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_5Field_10bind(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, PyObject *__pyx_v_field_name, PyObject *__pyx_v_root) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("bind", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_void_to_None(__pyx_f_9drf_turbo_6fields_5Field_bind(__pyx_v_self, __pyx_v_field_name, __pyx_v_root, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 172, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.Field.bind", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":190
 * 
 * 
 *     cpdef get_default_value(self):             # <<<<<<<<<<<<<<
 *         """
 *         Return the default value for this field.
 */

static PyObject *__pyx_pw_9drf_turbo_6fields_5Field_13get_default_value(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_5Field_get_default_value(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_default_value", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_get_default_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 190, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_9drf_turbo_6fields_5Field_13get_default_value)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 190, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "drf_turbo/fields.pyx":194
 *         Return the default value for this field.
 *         """
 *         if self.default_value is NO_DEFAULT or getattr(self.root, 'partial', False):             # <<<<<<<<<<<<<<
 *             raise SkipField()
 *         if callable(self.default_value):
 */
  __pyx_t_6 = (__pyx_v_self->default_value == __pyx_v_9drf_turbo_6fields_NO_DEFAULT);
  __pyx_t_7 = (__pyx_t_6 != 0);
  if (!__pyx_t_7) {
  } else {
    __pyx_t_5 = __pyx_t_7;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_1 = __pyx_v_self->root;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_GetAttr3(__pyx_t_1, __pyx_n_s_partial, Py_False); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 194, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 194, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_5 = __pyx_t_7;
  __pyx_L4_bool_binop_done:;
  if (unlikely(__pyx_t_5)) {

    /* "drf_turbo/fields.pyx":195
 *         """
 *         if self.default_value is NO_DEFAULT or getattr(self.root, 'partial', False):
 *             raise SkipField()             # <<<<<<<<<<<<<<
 *         if callable(self.default_value):
 *             return self.default_value()
 */
    __pyx_t_2 = __Pyx_PyObject_CallNoArg(((PyObject *)__pyx_ptype_9drf_turbo_6fields_SkipField)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 195, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 195, __pyx_L1_error)

    /* "drf_turbo/fields.pyx":194
 *         Return the default value for this field.
 *         """
 *         if self.default_value is NO_DEFAULT or getattr(self.root, 'partial', False):             # <<<<<<<<<<<<<<
 *             raise SkipField()
 *         if callable(self.default_value):
 */
  }

  /* "drf_turbo/fields.pyx":196
 *         if self.default_value is NO_DEFAULT or getattr(self.root, 'partial', False):
 *             raise SkipField()
 *         if callable(self.default_value):             # <<<<<<<<<<<<<<
 *             return self.default_value()
 *         return self.default_value
 */
  __pyx_t_2 = __pyx_v_self->default_value;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_5 = __Pyx_PyCallable_Check(__pyx_t_2); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 196, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = (__pyx_t_5 != 0);
  if (__pyx_t_7) {

    /* "drf_turbo/fields.pyx":197
 *             raise SkipField()
 *         if callable(self.default_value):
 *             return self.default_value()             # <<<<<<<<<<<<<<
 *         return self.default_value
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_self->default_value);
    __pyx_t_1 = __pyx_v_self->default_value; __pyx_t_3 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
      }
    }
    __pyx_t_2 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_1);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 197, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "drf_turbo/fields.pyx":196
 *         if self.default_value is NO_DEFAULT or getattr(self.root, 'partial', False):
 *             raise SkipField()
 *         if callable(self.default_value):             # <<<<<<<<<<<<<<
 *             return self.default_value()
 *         return self.default_value
 */
  }

  /* "drf_turbo/fields.pyx":198
 *         if callable(self.default_value):
 *             return self.default_value()
 *         return self.default_value             # <<<<<<<<<<<<<<
 * 
 *     cpdef get_initial(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->default_value);
  __pyx_r = __pyx_v_self->default_value;
  goto __pyx_L0;

  /* "drf_turbo/fields.pyx":190
 * 
 * 
 *     cpdef get_default_value(self):             # <<<<<<<<<<<<<<
 *         """
 *         Return the default value for this field.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("drf_turbo.fields.Field.get_default_value", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_5Field_13get_default_value(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_9drf_turbo_6fields_5Field_12get_default_value[] = "\n        Return the default value for this field.\n        ";
static PyObject *__pyx_pw_9drf_turbo_6fields_5Field_13get_default_value(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_default_value (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_5Field_12get_default_value(((struct __pyx_obj_9drf_turbo_6fields_Field *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_5Field_12get_default_value(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_default_value", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields_5Field_get_default_value(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 190, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.Field.get_default_value", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":200
 *         return self.default_value
 * 
 *     cpdef get_initial(self):             # <<<<<<<<<<<<<<
 *         """
 *         Return the initial value for this field.
 */

static PyObject *__pyx_pw_9drf_turbo_6fields_5Field_15get_initial(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_5Field_get_initial(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_initial", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_get_initial); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 200, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_9drf_turbo_6fields_5Field_15get_initial)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 200, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "drf_turbo/fields.pyx":204
 *         Return the initial value for this field.
 *         """
 *         if callable(self.initial):             # <<<<<<<<<<<<<<
 *             return self.initial()
 *         return self.initial
 */
  __pyx_t_1 = __pyx_v_self->initial;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_5 = __Pyx_PyCallable_Check(__pyx_t_1); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 204, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_6 = (__pyx_t_5 != 0);
  if (__pyx_t_6) {

    /* "drf_turbo/fields.pyx":205
 *         """
 *         if callable(self.initial):
 *             return self.initial()             # <<<<<<<<<<<<<<
 *         return self.initial
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_self->initial);
    __pyx_t_2 = __pyx_v_self->initial; __pyx_t_3 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 205, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "drf_turbo/fields.pyx":204
 *         Return the initial value for this field.
 *         """
 *         if callable(self.initial):             # <<<<<<<<<<<<<<
 *             return self.initial()
 *         return self.initial
 */
  }

  /* "drf_turbo/fields.pyx":206
 *         if callable(self.initial):
 *             return self.initial()
 *         return self.initial             # <<<<<<<<<<<<<<
 * 
 *     cpdef get_attribute(self, instance , attr=None):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->initial);
  __pyx_r = __pyx_v_self->initial;
  goto __pyx_L0;

  /* "drf_turbo/fields.pyx":200
 *         return self.default_value
 * 
 *     cpdef get_initial(self):             # <<<<<<<<<<<<<<
 *         """
 *         Return the initial value for this field.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("drf_turbo.fields.Field.get_initial", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_5Field_15get_initial(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_9drf_turbo_6fields_5Field_14get_initial[] = "\n        Return the initial value for this field.\n        ";
static PyObject *__pyx_pw_9drf_turbo_6fields_5Field_15get_initial(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_initial (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_5Field_14get_initial(((struct __pyx_obj_9drf_turbo_6fields_Field *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_5Field_14get_initial(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_initial", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields_5Field_get_initial(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 200, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.Field.get_initial", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":208
 *         return self.initial
 * 
 *     cpdef get_attribute(self, instance , attr=None):             # <<<<<<<<<<<<<<
 *         """
 *         Return the value of the field from the provided instance.
 */

static PyObject *__pyx_pw_9drf_turbo_6fields_5Field_17get_attribute(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_5Field_get_attribute(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, PyObject *__pyx_v_instance, int __pyx_skip_dispatch, struct __pyx_opt_args_9drf_turbo_6fields_5Field_get_attribute *__pyx_optional_args) {
  PyObject *__pyx_v_attr = ((PyObject *)Py_None);
  PyObject *__pyx_v_exc = NULL;
  PyObject *__pyx_v_msg = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_t_10;
  int __pyx_t_11;
  PyObject *__pyx_t_12 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_attribute", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_attr = __pyx_optional_args->attr;
    }
  }
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_get_attribute); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 208, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_9drf_turbo_6fields_5Field_17get_attribute)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        __pyx_t_5 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
            __pyx_t_5 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_instance, __pyx_v_attr};
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 208, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_instance, __pyx_v_attr};
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 208, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        {
          __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 208, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          if (__pyx_t_4) {
            __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
          }
          __Pyx_INCREF(__pyx_v_instance);
          __Pyx_GIVEREF(__pyx_v_instance);
          PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_v_instance);
          __Pyx_INCREF(__pyx_v_attr);
          __Pyx_GIVEREF(__pyx_v_attr);
          PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_v_attr);
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_6, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 208, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        }
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "drf_turbo/fields.pyx":212
 *         Return the value of the field from the provided instance.
 *         """
 *         try:             # <<<<<<<<<<<<<<
 *             if attr is None:
 *                 return get_attribute(instance, self.attrs)
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_7, &__pyx_t_8, &__pyx_t_9);
    __Pyx_XGOTREF(__pyx_t_7);
    __Pyx_XGOTREF(__pyx_t_8);
    __Pyx_XGOTREF(__pyx_t_9);
    /*try:*/ {

      /* "drf_turbo/fields.pyx":213
 *         """
 *         try:
 *             if attr is None:             # <<<<<<<<<<<<<<
 *                 return get_attribute(instance, self.attrs)
 *             return get_attribute(instance, attr)
 */
      __pyx_t_10 = (__pyx_v_attr == Py_None);
      __pyx_t_11 = (__pyx_t_10 != 0);
      if (__pyx_t_11) {

        /* "drf_turbo/fields.pyx":214
 *         try:
 *             if attr is None:
 *                 return get_attribute(instance, self.attrs)             # <<<<<<<<<<<<<<
 *             return get_attribute(instance, attr)
 *         except (KeyError, AttributeError) as exc:
 */
        __Pyx_XDECREF(__pyx_r);
        __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_get_attribute); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 214, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_2);
        __pyx_t_3 = NULL;
        __pyx_t_5 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
          __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
          if (likely(__pyx_t_3)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
            __Pyx_INCREF(__pyx_t_3);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_2, function);
            __pyx_t_5 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_2)) {
          PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_instance, __pyx_v_self->attrs};
          __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 214, __pyx_L3_error)
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_GOTREF(__pyx_t_1);
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
          PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_instance, __pyx_v_self->attrs};
          __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 214, __pyx_L3_error)
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_GOTREF(__pyx_t_1);
        } else
        #endif
        {
          __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 214, __pyx_L3_error)
          __Pyx_GOTREF(__pyx_t_6);
          if (__pyx_t_3) {
            __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_3); __pyx_t_3 = NULL;
          }
          __Pyx_INCREF(__pyx_v_instance);
          __Pyx_GIVEREF(__pyx_v_instance);
          PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_v_instance);
          __Pyx_INCREF(__pyx_v_self->attrs);
          __Pyx_GIVEREF(__pyx_v_self->attrs);
          PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_v_self->attrs);
          __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 214, __pyx_L3_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        }
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_1;
        __pyx_t_1 = 0;
        goto __pyx_L7_try_return;

        /* "drf_turbo/fields.pyx":213
 *         """
 *         try:
 *             if attr is None:             # <<<<<<<<<<<<<<
 *                 return get_attribute(instance, self.attrs)
 *             return get_attribute(instance, attr)
 */
      }

      /* "drf_turbo/fields.pyx":215
 *             if attr is None:
 *                 return get_attribute(instance, self.attrs)
 *             return get_attribute(instance, attr)             # <<<<<<<<<<<<<<
 *         except (KeyError, AttributeError) as exc:
 *             if self.default_value is not NO_DEFAULT:
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_get_attribute); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 215, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_6 = NULL;
      __pyx_t_5 = 0;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
        __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
        if (likely(__pyx_t_6)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
          __Pyx_INCREF(__pyx_t_6);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_2, function);
          __pyx_t_5 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_2)) {
        PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_v_instance, __pyx_v_attr};
        __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 215, __pyx_L3_error)
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_GOTREF(__pyx_t_1);
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
        PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_v_instance, __pyx_v_attr};
        __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 215, __pyx_L3_error)
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_GOTREF(__pyx_t_1);
      } else
      #endif
      {
        __pyx_t_3 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 215, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_3);
        if (__pyx_t_6) {
          __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_6); __pyx_t_6 = NULL;
        }
        __Pyx_INCREF(__pyx_v_instance);
        __Pyx_GIVEREF(__pyx_v_instance);
        PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_5, __pyx_v_instance);
        __Pyx_INCREF(__pyx_v_attr);
        __Pyx_GIVEREF(__pyx_v_attr);
        PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_5, __pyx_v_attr);
        __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 215, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      }
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_r = __pyx_t_1;
      __pyx_t_1 = 0;
      goto __pyx_L7_try_return;

      /* "drf_turbo/fields.pyx":212
 *         Return the value of the field from the provided instance.
 *         """
 *         try:             # <<<<<<<<<<<<<<
 *             if attr is None:
 *                 return get_attribute(instance, self.attrs)
 */
    }
    __pyx_L3_error:;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;

    /* "drf_turbo/fields.pyx":216
 *                 return get_attribute(instance, self.attrs)
 *             return get_attribute(instance, attr)
 *         except (KeyError, AttributeError) as exc:             # <<<<<<<<<<<<<<
 *             if self.default_value is not NO_DEFAULT:
 *                 return self.get_default_value()
 */
    __pyx_t_5 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_KeyError) || __Pyx_PyErr_ExceptionMatches(__pyx_builtin_AttributeError);
    if (__pyx_t_5) {
      __Pyx_AddTraceback("drf_turbo.fields.Field.get_attribute", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3) < 0) __PYX_ERR(0, 216, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __pyx_v_exc = __pyx_t_2;

      /* "drf_turbo/fields.pyx":217
 *             return get_attribute(instance, attr)
 *         except (KeyError, AttributeError) as exc:
 *             if self.default_value is not NO_DEFAULT:             # <<<<<<<<<<<<<<
 *                 return self.get_default_value()
 *             if self.allow_null:
 */
      __pyx_t_11 = (__pyx_v_self->default_value != __pyx_v_9drf_turbo_6fields_NO_DEFAULT);
      __pyx_t_10 = (__pyx_t_11 != 0);
      if (__pyx_t_10) {

        /* "drf_turbo/fields.pyx":218
 *         except (KeyError, AttributeError) as exc:
 *             if self.default_value is not NO_DEFAULT:
 *                 return self.get_default_value()             # <<<<<<<<<<<<<<
 *             if self.allow_null:
 *                 return None
 */
        __Pyx_XDECREF(__pyx_r);
        __pyx_t_6 = ((struct __pyx_vtabstruct_9drf_turbo_6fields_Field *)__pyx_v_self->__pyx_vtab)->get_default_value(__pyx_v_self, 0); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 218, __pyx_L5_except_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_r = __pyx_t_6;
        __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        goto __pyx_L6_except_return;

        /* "drf_turbo/fields.pyx":217
 *             return get_attribute(instance, attr)
 *         except (KeyError, AttributeError) as exc:
 *             if self.default_value is not NO_DEFAULT:             # <<<<<<<<<<<<<<
 *                 return self.get_default_value()
 *             if self.allow_null:
 */
      }

      /* "drf_turbo/fields.pyx":219
 *             if self.default_value is not NO_DEFAULT:
 *                 return self.get_default_value()
 *             if self.allow_null:             # <<<<<<<<<<<<<<
 *                 return None
 *             if not self.required:
 */
      __pyx_t_10 = (__pyx_v_self->allow_null != 0);
      if (__pyx_t_10) {

        /* "drf_turbo/fields.pyx":220
 *                 return self.get_default_value()
 *             if self.allow_null:
 *                 return None             # <<<<<<<<<<<<<<
 *             if not self.required:
 *                 raise SkipField()
 */
        __Pyx_XDECREF(__pyx_r);
        __pyx_r = Py_None; __Pyx_INCREF(Py_None);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        goto __pyx_L6_except_return;

        /* "drf_turbo/fields.pyx":219
 *             if self.default_value is not NO_DEFAULT:
 *                 return self.get_default_value()
 *             if self.allow_null:             # <<<<<<<<<<<<<<
 *                 return None
 *             if not self.required:
 */
      }

      /* "drf_turbo/fields.pyx":221
 *             if self.allow_null:
 *                 return None
 *             if not self.required:             # <<<<<<<<<<<<<<
 *                 raise SkipField()
 *             msg = (
 */
      __pyx_t_10 = ((!(__pyx_v_self->required != 0)) != 0);
      if (unlikely(__pyx_t_10)) {

        /* "drf_turbo/fields.pyx":222
 *                 return None
 *             if not self.required:
 *                 raise SkipField()             # <<<<<<<<<<<<<<
 *             msg = (
 *                 'Got {exc_type} when attempting to get a value for field'.format(
 */
        __pyx_t_6 = __Pyx_PyObject_CallNoArg(((PyObject *)__pyx_ptype_9drf_turbo_6fields_SkipField)); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 222, __pyx_L5_except_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_Raise(__pyx_t_6, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __PYX_ERR(0, 222, __pyx_L5_except_error)

        /* "drf_turbo/fields.pyx":221
 *             if self.allow_null:
 *                 return None
 *             if not self.required:             # <<<<<<<<<<<<<<
 *                 raise SkipField()
 *             msg = (
 */
      }

      /* "drf_turbo/fields.pyx":224
 *                 raise SkipField()
 *             msg = (
 *                 'Got {exc_type} when attempting to get a value for field'.format(             # <<<<<<<<<<<<<<
 *                     exc_type=type(exc).__name__,
 *                 )
 */
      __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_Got_exc_type_when_attempting_to, __pyx_n_s_format); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 224, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_6);

      /* "drf_turbo/fields.pyx":225
 *             msg = (
 *                 'Got {exc_type} when attempting to get a value for field'.format(
 *                     exc_type=type(exc).__name__,             # <<<<<<<<<<<<<<
 *                 )
 *             )
 */
      __pyx_t_4 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 225, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_12 = __Pyx_PyObject_GetAttrStr(((PyObject *)Py_TYPE(__pyx_v_exc)), __pyx_n_s_name); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 225, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_12);
      if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_exc_type, __pyx_t_12) < 0) __PYX_ERR(0, 225, __pyx_L5_except_error)
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

      /* "drf_turbo/fields.pyx":224
 *                 raise SkipField()
 *             msg = (
 *                 'Got {exc_type} when attempting to get a value for field'.format(             # <<<<<<<<<<<<<<
 *                     exc_type=type(exc).__name__,
 *                 )
 */
      __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_empty_tuple, __pyx_t_4); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 224, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_12);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_v_msg = __pyx_t_12;
      __pyx_t_12 = 0;

      /* "drf_turbo/fields.pyx":228
 *                 )
 *             )
 *             raise type(exc)(msg)             # <<<<<<<<<<<<<<
 * 
 *     cpdef tuple validate_empty_values(self, data):
 */
      __Pyx_INCREF(((PyObject *)Py_TYPE(__pyx_v_exc)));
      __pyx_t_4 = ((PyObject *)Py_TYPE(__pyx_v_exc)); __pyx_t_6 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_4);
        if (likely(__pyx_t_6)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_6);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_4, function);
        }
      }
      __pyx_t_12 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_6, __pyx_v_msg) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_v_msg);
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 228, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_12);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_Raise(__pyx_t_12, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      __PYX_ERR(0, 228, __pyx_L5_except_error)
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;

    /* "drf_turbo/fields.pyx":212
 *         Return the value of the field from the provided instance.
 *         """
 *         try:             # <<<<<<<<<<<<<<
 *             if attr is None:
 *                 return get_attribute(instance, self.attrs)
 */
    __Pyx_XGIVEREF(__pyx_t_7);
    __Pyx_XGIVEREF(__pyx_t_8);
    __Pyx_XGIVEREF(__pyx_t_9);
    __Pyx_ExceptionReset(__pyx_t_7, __pyx_t_8, __pyx_t_9);
    goto __pyx_L1_error;
    __pyx_L7_try_return:;
    __Pyx_XGIVEREF(__pyx_t_7);
    __Pyx_XGIVEREF(__pyx_t_8);
    __Pyx_XGIVEREF(__pyx_t_9);
    __Pyx_ExceptionReset(__pyx_t_7, __pyx_t_8, __pyx_t_9);
    goto __pyx_L0;
    __pyx_L6_except_return:;
    __Pyx_XGIVEREF(__pyx_t_7);
    __Pyx_XGIVEREF(__pyx_t_8);
    __Pyx_XGIVEREF(__pyx_t_9);
    __Pyx_ExceptionReset(__pyx_t_7, __pyx_t_8, __pyx_t_9);
    goto __pyx_L0;
  }

  /* "drf_turbo/fields.pyx":208
 *         return self.initial
 * 
 *     cpdef get_attribute(self, instance , attr=None):             # <<<<<<<<<<<<<<
 *         """
 *         Return the value of the field from the provided instance.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_AddTraceback("drf_turbo.fields.Field.get_attribute", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_exc);
  __Pyx_XDECREF(__pyx_v_msg);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_5Field_17get_attribute(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_9drf_turbo_6fields_5Field_16get_attribute[] = "\n        Return the value of the field from the provided instance.\n        ";
static PyObject *__pyx_pw_9drf_turbo_6fields_5Field_17get_attribute(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_instance = 0;
  PyObject *__pyx_v_attr = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_attribute (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_instance,&__pyx_n_s_attr,0};
    PyObject* values[2] = {0,0};
    values[1] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_instance)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_attr);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "get_attribute") < 0)) __PYX_ERR(0, 208, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_instance = values[0];
    __pyx_v_attr = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("get_attribute", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 208, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.Field.get_attribute", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9drf_turbo_6fields_5Field_16get_attribute(((struct __pyx_obj_9drf_turbo_6fields_Field *)__pyx_v_self), __pyx_v_instance, __pyx_v_attr);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_5Field_16get_attribute(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, PyObject *__pyx_v_instance, PyObject *__pyx_v_attr) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_9drf_turbo_6fields_5Field_get_attribute __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_attribute", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 1;
  __pyx_t_2.attr = __pyx_v_attr;
  __pyx_t_1 = __pyx_vtabptr_9drf_turbo_6fields_Field->get_attribute(__pyx_v_self, __pyx_v_instance, 1, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 208, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.Field.get_attribute", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":230
 *             raise type(exc)(msg)
 * 
 *     cpdef tuple validate_empty_values(self, data):             # <<<<<<<<<<<<<<
 *         """
 *         Validate empty values, and either:
 */

static PyObject *__pyx_pw_9drf_turbo_6fields_5Field_19validate_empty_values(PyObject *__pyx_v_self, PyObject *__pyx_v_data); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_5Field_validate_empty_values(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, PyObject *__pyx_v_data, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("validate_empty_values", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_validate_empty_values); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 230, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_9drf_turbo_6fields_5Field_19validate_empty_values)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_data) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_data);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 230, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (!(likely(PyTuple_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_t_2)->tp_name), 0))) __PYX_ERR(0, 230, __pyx_L1_error)
        __pyx_r = ((PyObject*)__pyx_t_2);
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "drf_turbo/fields.pyx":239
 *           have validation applied as normal.
 *         """
 *         if self.read_only:             # <<<<<<<<<<<<<<
 *             return (True, self.get_default_value())
 * 
 */
  __pyx_t_5 = (__pyx_v_self->read_only != 0);
  if (__pyx_t_5) {

    /* "drf_turbo/fields.pyx":240
 *         """
 *         if self.read_only:
 *             return (True, self.get_default_value())             # <<<<<<<<<<<<<<
 * 
 *         if data is NO_DEFAULT:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = ((struct __pyx_vtabstruct_9drf_turbo_6fields_Field *)__pyx_v_self->__pyx_vtab)->get_default_value(__pyx_v_self, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 240, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 240, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(Py_True);
    __Pyx_GIVEREF(Py_True);
    PyTuple_SET_ITEM(__pyx_t_2, 0, Py_True);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_1);
    __pyx_t_1 = 0;
    __pyx_r = ((PyObject*)__pyx_t_2);
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "drf_turbo/fields.pyx":239
 *           have validation applied as normal.
 *         """
 *         if self.read_only:             # <<<<<<<<<<<<<<
 *             return (True, self.get_default_value())
 * 
 */
  }

  /* "drf_turbo/fields.pyx":242
 *             return (True, self.get_default_value())
 * 
 *         if data is NO_DEFAULT:             # <<<<<<<<<<<<<<
 *             if getattr(self.root, 'partial', False):
 *                 raise SkipField()
 */
  __pyx_t_5 = (__pyx_v_data == __pyx_v_9drf_turbo_6fields_NO_DEFAULT);
  __pyx_t_6 = (__pyx_t_5 != 0);
  if (__pyx_t_6) {

    /* "drf_turbo/fields.pyx":243
 * 
 *         if data is NO_DEFAULT:
 *             if getattr(self.root, 'partial', False):             # <<<<<<<<<<<<<<
 *                 raise SkipField()
 *             if self.required:
 */
    __pyx_t_2 = __pyx_v_self->root;
    __Pyx_INCREF(__pyx_t_2);
    __pyx_t_1 = __Pyx_GetAttr3(__pyx_t_2, __pyx_n_s_partial, Py_False); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 243, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 243, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (unlikely(__pyx_t_6)) {

      /* "drf_turbo/fields.pyx":244
 *         if data is NO_DEFAULT:
 *             if getattr(self.root, 'partial', False):
 *                 raise SkipField()             # <<<<<<<<<<<<<<
 *             if self.required:
 *                 raise self.raise_if_fail('required')
 */
      __pyx_t_1 = __Pyx_PyObject_CallNoArg(((PyObject *)__pyx_ptype_9drf_turbo_6fields_SkipField)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 244, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_Raise(__pyx_t_1, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __PYX_ERR(0, 244, __pyx_L1_error)

      /* "drf_turbo/fields.pyx":243
 * 
 *         if data is NO_DEFAULT:
 *             if getattr(self.root, 'partial', False):             # <<<<<<<<<<<<<<
 *                 raise SkipField()
 *             if self.required:
 */
    }

    /* "drf_turbo/fields.pyx":245
 *             if getattr(self.root, 'partial', False):
 *                 raise SkipField()
 *             if self.required:             # <<<<<<<<<<<<<<
 *                 raise self.raise_if_fail('required')
 *             return (True, self.get_default_value())
 */
    __pyx_t_6 = (__pyx_v_self->required != 0);
    if (unlikely(__pyx_t_6)) {

      /* "drf_turbo/fields.pyx":246
 *                 raise SkipField()
 *             if self.required:
 *                 raise self.raise_if_fail('required')             # <<<<<<<<<<<<<<
 *             return (True, self.get_default_value())
 * 
 */
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_raise_if_fail); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 246, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_3 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
        __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
        if (likely(__pyx_t_3)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_2, function);
        }
      }
      __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_n_s_required) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_n_s_required);
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 246, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_Raise(__pyx_t_1, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __PYX_ERR(0, 246, __pyx_L1_error)

      /* "drf_turbo/fields.pyx":245
 *             if getattr(self.root, 'partial', False):
 *                 raise SkipField()
 *             if self.required:             # <<<<<<<<<<<<<<
 *                 raise self.raise_if_fail('required')
 *             return (True, self.get_default_value())
 */
    }

    /* "drf_turbo/fields.pyx":247
 *             if self.required:
 *                 raise self.raise_if_fail('required')
 *             return (True, self.get_default_value())             # <<<<<<<<<<<<<<
 * 
 *         if data is None:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = ((struct __pyx_vtabstruct_9drf_turbo_6fields_Field *)__pyx_v_self->__pyx_vtab)->get_default_value(__pyx_v_self, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 247, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 247, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(Py_True);
    __Pyx_GIVEREF(Py_True);
    PyTuple_SET_ITEM(__pyx_t_2, 0, Py_True);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_1);
    __pyx_t_1 = 0;
    __pyx_r = ((PyObject*)__pyx_t_2);
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "drf_turbo/fields.pyx":242
 *             return (True, self.get_default_value())
 * 
 *         if data is NO_DEFAULT:             # <<<<<<<<<<<<<<
 *             if getattr(self.root, 'partial', False):
 *                 raise SkipField()
 */
  }

  /* "drf_turbo/fields.pyx":249
 *             return (True, self.get_default_value())
 * 
 *         if data is None:             # <<<<<<<<<<<<<<
 *             if not self.allow_null:
 *                 raise self.raise_if_fail('null')
 */
  __pyx_t_6 = (__pyx_v_data == Py_None);
  __pyx_t_5 = (__pyx_t_6 != 0);
  if (__pyx_t_5) {

    /* "drf_turbo/fields.pyx":250
 * 
 *         if data is None:
 *             if not self.allow_null:             # <<<<<<<<<<<<<<
 *                 raise self.raise_if_fail('null')
 *             return (True, None)
 */
    __pyx_t_5 = ((!(__pyx_v_self->allow_null != 0)) != 0);
    if (unlikely(__pyx_t_5)) {

      /* "drf_turbo/fields.pyx":251
 *         if data is None:
 *             if not self.allow_null:
 *                 raise self.raise_if_fail('null')             # <<<<<<<<<<<<<<
 *             return (True, None)
 * 
 */
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_raise_if_fail); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 251, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_3 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
        __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_1);
        if (likely(__pyx_t_3)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_1, function);
        }
      }
      __pyx_t_2 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_3, __pyx_n_s_null) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_n_s_null);
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 251, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_Raise(__pyx_t_2, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __PYX_ERR(0, 251, __pyx_L1_error)

      /* "drf_turbo/fields.pyx":250
 * 
 *         if data is None:
 *             if not self.allow_null:             # <<<<<<<<<<<<<<
 *                 raise self.raise_if_fail('null')
 *             return (True, None)
 */
    }

    /* "drf_turbo/fields.pyx":252
 *             if not self.allow_null:
 *                 raise self.raise_if_fail('null')
 *             return (True, None)             # <<<<<<<<<<<<<<
 * 
 *         return (False, data)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_tuple__8);
    __pyx_r = __pyx_tuple__8;
    goto __pyx_L0;

    /* "drf_turbo/fields.pyx":249
 *             return (True, self.get_default_value())
 * 
 *         if data is None:             # <<<<<<<<<<<<<<
 *             if not self.allow_null:
 *                 raise self.raise_if_fail('null')
 */
  }

  /* "drf_turbo/fields.pyx":254
 *             return (True, None)
 * 
 *         return (False, data)             # <<<<<<<<<<<<<<
 * 
 *     cpdef run_validation(self,object data,dict context) :
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 254, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(Py_False);
  __Pyx_GIVEREF(Py_False);
  PyTuple_SET_ITEM(__pyx_t_2, 0, Py_False);
  __Pyx_INCREF(__pyx_v_data);
  __Pyx_GIVEREF(__pyx_v_data);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_v_data);
  __pyx_r = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "drf_turbo/fields.pyx":230
 *             raise type(exc)(msg)
 * 
 *     cpdef tuple validate_empty_values(self, data):             # <<<<<<<<<<<<<<
 *         """
 *         Validate empty values, and either:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("drf_turbo.fields.Field.validate_empty_values", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_5Field_19validate_empty_values(PyObject *__pyx_v_self, PyObject *__pyx_v_data); /*proto*/
static char __pyx_doc_9drf_turbo_6fields_5Field_18validate_empty_values[] = "\n        Validate empty values, and either:\n        * Raise `ValidationError`, indicating invalid data.\n        * Return (True, data), indicating an empty value that should be\n          returned without any further validation being applied.\n        * Return (False, data), indicating a non-empty value, that should\n          have validation applied as normal.\n        ";
static PyObject *__pyx_pw_9drf_turbo_6fields_5Field_19validate_empty_values(PyObject *__pyx_v_self, PyObject *__pyx_v_data) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("validate_empty_values (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_5Field_18validate_empty_values(((struct __pyx_obj_9drf_turbo_6fields_Field *)__pyx_v_self), ((PyObject *)__pyx_v_data));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_5Field_18validate_empty_values(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, PyObject *__pyx_v_data) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("validate_empty_values", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields_5Field_validate_empty_values(__pyx_v_self, __pyx_v_data, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 230, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.Field.validate_empty_values", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":256
 *         return (False, data)
 * 
 *     cpdef run_validation(self,object data,dict context) :             # <<<<<<<<<<<<<<
 *         """
 *         Validate an input data.
 */

static PyObject *__pyx_pw_9drf_turbo_6fields_5Field_21run_validation(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_5Field_run_validation(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context, int __pyx_skip_dispatch) {
  int __pyx_v_is_empty_value;
  PyObject *__pyx_v_value = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  long __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("run_validation", 0);
  __Pyx_INCREF(__pyx_v_data);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_run_validation); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 256, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_9drf_turbo_6fields_5Field_21run_validation)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        __pyx_t_5 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
            __pyx_t_5 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_data, __pyx_v_context};
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 256, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_data, __pyx_v_context};
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 256, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        {
          __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 256, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          if (__pyx_t_4) {
            __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
          }
          __Pyx_INCREF(__pyx_v_data);
          __Pyx_GIVEREF(__pyx_v_data);
          PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_v_data);
          __Pyx_INCREF(__pyx_v_context);
          __Pyx_GIVEREF(__pyx_v_context);
          PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_v_context);
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_6, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 256, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        }
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "drf_turbo/fields.pyx":261
 *         """
 *         cdef bint is_empty_value
 *         (is_empty_value, data) = self.validate_empty_values(data)             # <<<<<<<<<<<<<<
 *         if is_empty_value:
 *             return data
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_9drf_turbo_6fields_Field *)__pyx_v_self->__pyx_vtab)->validate_empty_values(__pyx_v_self, __pyx_v_data, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 261, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (likely(__pyx_t_1 != Py_None)) {
    PyObject* sequence = __pyx_t_1;
    Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
    if (unlikely(size != 2)) {
      if (size > 2) __Pyx_RaiseTooManyValuesError(2);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(0, 261, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_2 = PyTuple_GET_ITEM(sequence, 0); 
    __pyx_t_3 = PyTuple_GET_ITEM(sequence, 1); 
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_3);
    #else
    __pyx_t_2 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 261, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 261, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    #endif
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  } else {
    __Pyx_RaiseNoneNotIterableError(); __PYX_ERR(0, 261, __pyx_L1_error)
  }
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_7 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 261, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_is_empty_value = __pyx_t_7;
  __Pyx_DECREF_SET(__pyx_v_data, __pyx_t_3);
  __pyx_t_3 = 0;

  /* "drf_turbo/fields.pyx":262
 *         cdef bint is_empty_value
 *         (is_empty_value, data) = self.validate_empty_values(data)
 *         if is_empty_value:             # <<<<<<<<<<<<<<
 *             return data
 *         value = self.deserialize(data,context)
 */
  __pyx_t_7 = (__pyx_v_is_empty_value != 0);
  if (__pyx_t_7) {

    /* "drf_turbo/fields.pyx":263
 *         (is_empty_value, data) = self.validate_empty_values(data)
 *         if is_empty_value:
 *             return data             # <<<<<<<<<<<<<<
 *         value = self.deserialize(data,context)
 *         self.validate_or_raise(value)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_data);
    __pyx_r = __pyx_v_data;
    goto __pyx_L0;

    /* "drf_turbo/fields.pyx":262
 *         cdef bint is_empty_value
 *         (is_empty_value, data) = self.validate_empty_values(data)
 *         if is_empty_value:             # <<<<<<<<<<<<<<
 *             return data
 *         value = self.deserialize(data,context)
 */
  }

  /* "drf_turbo/fields.pyx":264
 *         if is_empty_value:
 *             return data
 *         value = self.deserialize(data,context)             # <<<<<<<<<<<<<<
 *         self.validate_or_raise(value)
 *         return value
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_9drf_turbo_6fields_Field *)__pyx_v_self->__pyx_vtab)->deserialize(__pyx_v_self, __pyx_v_data, __pyx_v_context, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 264, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_value = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "drf_turbo/fields.pyx":265
 *             return data
 *         value = self.deserialize(data,context)
 *         self.validate_or_raise(value)             # <<<<<<<<<<<<<<
 *         return value
 * 
 */
  __pyx_t_8 = ((struct __pyx_vtabstruct_9drf_turbo_6fields_Field *)__pyx_v_self->__pyx_vtab)->validate_or_raise(__pyx_v_self, __pyx_v_value, 0); if (unlikely(__pyx_t_8 == ((long)-1L))) __PYX_ERR(0, 265, __pyx_L1_error)

  /* "drf_turbo/fields.pyx":266
 *         value = self.deserialize(data,context)
 *         self.validate_or_raise(value)
 *         return value             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_value);
  __pyx_r = __pyx_v_value;
  goto __pyx_L0;

  /* "drf_turbo/fields.pyx":256
 *         return (False, data)
 * 
 *     cpdef run_validation(self,object data,dict context) :             # <<<<<<<<<<<<<<
 *         """
 *         Validate an input data.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("drf_turbo.fields.Field.run_validation", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_value);
  __Pyx_XDECREF(__pyx_v_data);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_5Field_21run_validation(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_9drf_turbo_6fields_5Field_20run_validation[] = "\n        Validate an input data.\n        ";
static PyObject *__pyx_pw_9drf_turbo_6fields_5Field_21run_validation(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_data = 0;
  PyObject *__pyx_v_context = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("run_validation (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_data,&__pyx_n_s_context,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_data)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_context)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("run_validation", 1, 2, 2, 1); __PYX_ERR(0, 256, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "run_validation") < 0)) __PYX_ERR(0, 256, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_data = values[0];
    __pyx_v_context = ((PyObject*)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("run_validation", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 256, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.Field.run_validation", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_context), (&PyDict_Type), 1, "context", 1))) __PYX_ERR(0, 256, __pyx_L1_error)
  __pyx_r = __pyx_pf_9drf_turbo_6fields_5Field_20run_validation(((struct __pyx_obj_9drf_turbo_6fields_Field *)__pyx_v_self), __pyx_v_data, __pyx_v_context);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_5Field_20run_validation(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("run_validation", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields_5Field_run_validation(__pyx_v_self, __pyx_v_data, __pyx_v_context, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 256, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.Field.run_validation", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":269
 * 
 * 
 *     cpdef long validate_or_raise(self,value) except -1 :             # <<<<<<<<<<<<<<
 *         """
 *         Validate the value and raise a `ValidationError` if validation fails.
 */

static PyObject *__pyx_pw_9drf_turbo_6fields_5Field_23validate_or_raise(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static long __pyx_f_9drf_turbo_6fields_5Field_validate_or_raise(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, PyObject *__pyx_v_value, int __pyx_skip_dispatch) {
  PyObject *__pyx_v_errors = 0;
  PyObject *__pyx_v_validator = NULL;
  PyObject *__pyx_v_exc = NULL;
  long __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  long __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  PyObject *(*__pyx_t_7)(PyObject *);
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  int __pyx_t_12;
  int __pyx_t_13;
  int __pyx_t_14;
  int __pyx_t_15;
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("validate_or_raise", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_validate_or_raise); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 269, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_9drf_turbo_6fields_5Field_23validate_or_raise)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_value) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_value);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 269, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __Pyx_PyInt_As_long(__pyx_t_2); if (unlikely((__pyx_t_5 == (long)-1) && PyErr_Occurred())) __PYX_ERR(0, 269, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "drf_turbo/fields.pyx":274
 *         """
 * 
 *         cdef list errors = []             # <<<<<<<<<<<<<<
 *         for validator in self.validators :
 *             try :
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 274, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_errors = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "drf_turbo/fields.pyx":275
 * 
 *         cdef list errors = []
 *         for validator in self.validators :             # <<<<<<<<<<<<<<
 *             try :
 *                 validator(value)
 */
  if (likely(PyList_CheckExact(__pyx_v_self->validators)) || PyTuple_CheckExact(__pyx_v_self->validators)) {
    __pyx_t_1 = __pyx_v_self->validators; __Pyx_INCREF(__pyx_t_1); __pyx_t_6 = 0;
    __pyx_t_7 = NULL;
  } else {
    __pyx_t_6 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_v_self->validators); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 275, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_7 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 275, __pyx_L1_error)
  }
  for (;;) {
    if (likely(!__pyx_t_7)) {
      if (likely(PyList_CheckExact(__pyx_t_1))) {
        if (__pyx_t_6 >= PyList_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_2 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_6); __Pyx_INCREF(__pyx_t_2); __pyx_t_6++; if (unlikely(0 < 0)) __PYX_ERR(0, 275, __pyx_L1_error)
        #else
        __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 275, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        #endif
      } else {
        if (__pyx_t_6 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_2 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_6); __Pyx_INCREF(__pyx_t_2); __pyx_t_6++; if (unlikely(0 < 0)) __PYX_ERR(0, 275, __pyx_L1_error)
        #else
        __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 275, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        #endif
      }
    } else {
      __pyx_t_2 = __pyx_t_7(__pyx_t_1);
      if (unlikely(!__pyx_t_2)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 275, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_XDECREF_SET(__pyx_v_validator, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "drf_turbo/fields.pyx":276
 *         cdef list errors = []
 *         for validator in self.validators :
 *             try :             # <<<<<<<<<<<<<<
 *                 validator(value)
 *             except ValidationError as exc:
 */
    {
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __Pyx_ExceptionSave(&__pyx_t_8, &__pyx_t_9, &__pyx_t_10);
      __Pyx_XGOTREF(__pyx_t_8);
      __Pyx_XGOTREF(__pyx_t_9);
      __Pyx_XGOTREF(__pyx_t_10);
      /*try:*/ {

        /* "drf_turbo/fields.pyx":277
 *         for validator in self.validators :
 *             try :
 *                 validator(value)             # <<<<<<<<<<<<<<
 *             except ValidationError as exc:
 *                 if isinstance(exc.detail, dict):
 */
        __Pyx_INCREF(__pyx_v_validator);
        __pyx_t_3 = __pyx_v_validator; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_value) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_value);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 277, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "drf_turbo/fields.pyx":276
 *         cdef list errors = []
 *         for validator in self.validators :
 *             try :             # <<<<<<<<<<<<<<
 *                 validator(value)
 *             except ValidationError as exc:
 */
      }
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
      goto __pyx_L12_try_end;
      __pyx_L5_error:;
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "drf_turbo/fields.pyx":278
 *             try :
 *                 validator(value)
 *             except ValidationError as exc:             # <<<<<<<<<<<<<<
 *                 if isinstance(exc.detail, dict):
 *                     raise
 */
      __Pyx_ErrFetch(&__pyx_t_2, &__pyx_t_3, &__pyx_t_4);
      __Pyx_GetModuleGlobalName(__pyx_t_11, __pyx_n_s_ValidationError); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 278, __pyx_L7_except_error)
      __Pyx_GOTREF(__pyx_t_11);
      __pyx_t_12 = __Pyx_PyErr_GivenExceptionMatches(__pyx_t_2, __pyx_t_11);
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      __Pyx_ErrRestore(__pyx_t_2, __pyx_t_3, __pyx_t_4);
      __pyx_t_2 = 0; __pyx_t_3 = 0; __pyx_t_4 = 0;
      if (__pyx_t_12) {
        __Pyx_AddTraceback("drf_turbo.fields.Field.validate_or_raise", __pyx_clineno, __pyx_lineno, __pyx_filename);
        if (__Pyx_GetException(&__pyx_t_4, &__pyx_t_3, &__pyx_t_2) < 0) __PYX_ERR(0, 278, __pyx_L7_except_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_XDECREF_SET(__pyx_v_exc, __pyx_t_3);

        /* "drf_turbo/fields.pyx":279
 *                 validator(value)
 *             except ValidationError as exc:
 *                 if isinstance(exc.detail, dict):             # <<<<<<<<<<<<<<
 *                     raise
 *                 errors.extend(exc.detail)
 */
        __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_v_exc, __pyx_n_s_detail); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 279, __pyx_L7_except_error)
        __Pyx_GOTREF(__pyx_t_11);
        __pyx_t_13 = PyDict_Check(__pyx_t_11); 
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        __pyx_t_14 = (__pyx_t_13 != 0);
        if (unlikely(__pyx_t_14)) {

          /* "drf_turbo/fields.pyx":280
 *             except ValidationError as exc:
 *                 if isinstance(exc.detail, dict):
 *                     raise             # <<<<<<<<<<<<<<
 *                 errors.extend(exc.detail)
 *             except DjangoValidationError as exc:
 */
          __Pyx_GIVEREF(__pyx_t_4);
          __Pyx_GIVEREF(__pyx_t_3);
          __Pyx_XGIVEREF(__pyx_t_2);
          __Pyx_ErrRestoreWithState(__pyx_t_4, __pyx_t_3, __pyx_t_2);
          __pyx_t_4 = 0; __pyx_t_3 = 0; __pyx_t_2 = 0; 
          __PYX_ERR(0, 280, __pyx_L7_except_error)

          /* "drf_turbo/fields.pyx":279
 *                 validator(value)
 *             except ValidationError as exc:
 *                 if isinstance(exc.detail, dict):             # <<<<<<<<<<<<<<
 *                     raise
 *                 errors.extend(exc.detail)
 */
        }

        /* "drf_turbo/fields.pyx":281
 *                 if isinstance(exc.detail, dict):
 *                     raise
 *                 errors.extend(exc.detail)             # <<<<<<<<<<<<<<
 *             except DjangoValidationError as exc:
 *                 errors.extend(get_error_detail(exc))
 */
        __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_v_exc, __pyx_n_s_detail); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 281, __pyx_L7_except_error)
        __Pyx_GOTREF(__pyx_t_11);
        __pyx_t_15 = __Pyx_PyList_Extend(__pyx_v_errors, __pyx_t_11); if (unlikely(__pyx_t_15 == ((int)-1))) __PYX_ERR(0, 281, __pyx_L7_except_error)
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        goto __pyx_L6_exception_handled;
      }

      /* "drf_turbo/fields.pyx":282
 *                     raise
 *                 errors.extend(exc.detail)
 *             except DjangoValidationError as exc:             # <<<<<<<<<<<<<<
 *                 errors.extend(get_error_detail(exc))
 *         if errors:
 */
      __Pyx_ErrFetch(&__pyx_t_2, &__pyx_t_3, &__pyx_t_4);
      __Pyx_GetModuleGlobalName(__pyx_t_11, __pyx_n_s_DjangoValidationError); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 282, __pyx_L7_except_error)
      __Pyx_GOTREF(__pyx_t_11);
      __pyx_t_12 = __Pyx_PyErr_GivenExceptionMatches(__pyx_t_2, __pyx_t_11);
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      __Pyx_ErrRestore(__pyx_t_2, __pyx_t_3, __pyx_t_4);
      __pyx_t_2 = 0; __pyx_t_3 = 0; __pyx_t_4 = 0;
      if (__pyx_t_12) {
        __Pyx_AddTraceback("drf_turbo.fields.Field.validate_or_raise", __pyx_clineno, __pyx_lineno, __pyx_filename);
        if (__Pyx_GetException(&__pyx_t_4, &__pyx_t_3, &__pyx_t_2) < 0) __PYX_ERR(0, 282, __pyx_L7_except_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_XDECREF_SET(__pyx_v_exc, __pyx_t_3);

        /* "drf_turbo/fields.pyx":283
 *                 errors.extend(exc.detail)
 *             except DjangoValidationError as exc:
 *                 errors.extend(get_error_detail(exc))             # <<<<<<<<<<<<<<
 *         if errors:
 *             raise ValidationError(errors)
 */
        __Pyx_GetModuleGlobalName(__pyx_t_16, __pyx_n_s_get_error_detail); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 283, __pyx_L7_except_error)
        __Pyx_GOTREF(__pyx_t_16);
        __pyx_t_17 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_16))) {
          __pyx_t_17 = PyMethod_GET_SELF(__pyx_t_16);
          if (likely(__pyx_t_17)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_16);
            __Pyx_INCREF(__pyx_t_17);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_16, function);
          }
        }
        __pyx_t_11 = (__pyx_t_17) ? __Pyx_PyObject_Call2Args(__pyx_t_16, __pyx_t_17, __pyx_v_exc) : __Pyx_PyObject_CallOneArg(__pyx_t_16, __pyx_v_exc);
        __Pyx_XDECREF(__pyx_t_17); __pyx_t_17 = 0;
        if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 283, __pyx_L7_except_error)
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
        __pyx_t_15 = __Pyx_PyList_Extend(__pyx_v_errors, __pyx_t_11); if (unlikely(__pyx_t_15 == ((int)-1))) __PYX_ERR(0, 283, __pyx_L7_except_error)
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        goto __pyx_L6_exception_handled;
      }
      goto __pyx_L7_except_error;
      __pyx_L7_except_error:;

      /* "drf_turbo/fields.pyx":276
 *         cdef list errors = []
 *         for validator in self.validators :
 *             try :             # <<<<<<<<<<<<<<
 *                 validator(value)
 *             except ValidationError as exc:
 */
      __Pyx_XGIVEREF(__pyx_t_8);
      __Pyx_XGIVEREF(__pyx_t_9);
      __Pyx_XGIVEREF(__pyx_t_10);
      __Pyx_ExceptionReset(__pyx_t_8, __pyx_t_9, __pyx_t_10);
      goto __pyx_L1_error;
      __pyx_L6_exception_handled:;
      __Pyx_XGIVEREF(__pyx_t_8);
      __Pyx_XGIVEREF(__pyx_t_9);
      __Pyx_XGIVEREF(__pyx_t_10);
      __Pyx_ExceptionReset(__pyx_t_8, __pyx_t_9, __pyx_t_10);
      __pyx_L12_try_end:;
    }

    /* "drf_turbo/fields.pyx":275
 * 
 *         cdef list errors = []
 *         for validator in self.validators :             # <<<<<<<<<<<<<<
 *             try :
 *                 validator(value)
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "drf_turbo/fields.pyx":284
 *             except DjangoValidationError as exc:
 *                 errors.extend(get_error_detail(exc))
 *         if errors:             # <<<<<<<<<<<<<<
 *             raise ValidationError(errors)
 * 
 */
  __pyx_t_14 = (PyList_GET_SIZE(__pyx_v_errors) != 0);
  if (unlikely(__pyx_t_14)) {

    /* "drf_turbo/fields.pyx":285
 *                 errors.extend(get_error_detail(exc))
 *         if errors:
 *             raise ValidationError(errors)             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_ValidationError); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 285, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_v_errors) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_errors);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 285, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 285, __pyx_L1_error)

    /* "drf_turbo/fields.pyx":284
 *             except DjangoValidationError as exc:
 *                 errors.extend(get_error_detail(exc))
 *         if errors:             # <<<<<<<<<<<<<<
 *             raise ValidationError(errors)
 * 
 */
  }

  /* "drf_turbo/fields.pyx":269
 * 
 * 
 *     cpdef long validate_or_raise(self,value) except -1 :             # <<<<<<<<<<<<<<
 *         """
 *         Validate the value and raise a `ValidationError` if validation fails.
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_XDECREF(__pyx_t_17);
  __Pyx_AddTraceback("drf_turbo.fields.Field.validate_or_raise", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1L;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_errors);
  __Pyx_XDECREF(__pyx_v_validator);
  __Pyx_XDECREF(__pyx_v_exc);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_5Field_23validate_or_raise(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static char __pyx_doc_9drf_turbo_6fields_5Field_22validate_or_raise[] = "\n        Validate the value and raise a `ValidationError` if validation fails.\n        ";
static PyObject *__pyx_pw_9drf_turbo_6fields_5Field_23validate_or_raise(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("validate_or_raise (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_5Field_22validate_or_raise(((struct __pyx_obj_9drf_turbo_6fields_Field *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_5Field_22validate_or_raise(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, PyObject *__pyx_v_value) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  long __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("validate_or_raise", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields_5Field_validate_or_raise(__pyx_v_self, __pyx_v_value, 1); if (unlikely(__pyx_t_1 == ((long)-1L))) __PYX_ERR(0, 269, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 269, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("drf_turbo.fields.Field.validate_or_raise", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pxd":8
 * cdef class Field :
 *     cdef public :
 *         str attr             # <<<<<<<<<<<<<<
 *         bint call
 *         bint required
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_5Field_4attr_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_5Field_4attr_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_5Field_4attr___get__(((struct __pyx_obj_9drf_turbo_6fields_Field *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_5Field_4attr___get__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->attr);
  __pyx_r = __pyx_v_self->attr;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_5Field_4attr_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_5Field_4attr_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_5Field_4attr_2__set__(((struct __pyx_obj_9drf_turbo_6fields_Field *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_5Field_4attr_2__set__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(PyString_CheckExact(__pyx_v_value))||((__pyx_v_value) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_v_value)->tp_name), 0))) __PYX_ERR(2, 8, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->attr);
  __Pyx_DECREF(__pyx_v_self->attr);
  __pyx_v_self->attr = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.Field.attr.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_5Field_4attr_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_5Field_4attr_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_5Field_4attr_4__del__(((struct __pyx_obj_9drf_turbo_6fields_Field *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_5Field_4attr_4__del__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->attr);
  __Pyx_DECREF(__pyx_v_self->attr);
  __pyx_v_self->attr = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pxd":9
 *     cdef public :
 *         str attr
 *         bint call             # <<<<<<<<<<<<<<
 *         bint required
 *         bint write_only
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_5Field_4call_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_5Field_4call_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_5Field_4call___get__(((struct __pyx_obj_9drf_turbo_6fields_Field *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_5Field_4call___get__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->call); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.Field.call.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_5Field_4call_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_5Field_4call_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_5Field_4call_2__set__(((struct __pyx_obj_9drf_turbo_6fields_Field *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_5Field_4call_2__set__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 9, __pyx_L1_error)
  __pyx_v_self->call = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("drf_turbo.fields.Field.call.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pxd":10
 *         str attr
 *         bint call
 *         bint required             # <<<<<<<<<<<<<<
 *         bint write_only
 *         bint read_only
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_5Field_8required_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_5Field_8required_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_5Field_8required___get__(((struct __pyx_obj_9drf_turbo_6fields_Field *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_5Field_8required___get__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->required); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.Field.required.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_5Field_8required_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_5Field_8required_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_5Field_8required_2__set__(((struct __pyx_obj_9drf_turbo_6fields_Field *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_5Field_8required_2__set__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 10, __pyx_L1_error)
  __pyx_v_self->required = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("drf_turbo.fields.Field.required.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pxd":11
 *         bint call
 *         bint required
 *         bint write_only             # <<<<<<<<<<<<<<
 *         bint read_only
 *         bint allow_null
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_5Field_10write_only_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_5Field_10write_only_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_5Field_10write_only___get__(((struct __pyx_obj_9drf_turbo_6fields_Field *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_5Field_10write_only___get__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->write_only); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 11, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.Field.write_only.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_5Field_10write_only_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_5Field_10write_only_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_5Field_10write_only_2__set__(((struct __pyx_obj_9drf_turbo_6fields_Field *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_5Field_10write_only_2__set__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 11, __pyx_L1_error)
  __pyx_v_self->write_only = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("drf_turbo.fields.Field.write_only.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pxd":12
 *         bint required
 *         bint write_only
 *         bint read_only             # <<<<<<<<<<<<<<
 *         bint allow_null
 *         str label
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_5Field_9read_only_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_5Field_9read_only_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_5Field_9read_only___get__(((struct __pyx_obj_9drf_turbo_6fields_Field *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_5Field_9read_only___get__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->read_only); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.Field.read_only.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_5Field_9read_only_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_5Field_9read_only_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_5Field_9read_only_2__set__(((struct __pyx_obj_9drf_turbo_6fields_Field *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_5Field_9read_only_2__set__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __pyx_v_self->read_only = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("drf_turbo.fields.Field.read_only.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pxd":13
 *         bint write_only
 *         bint read_only
 *         bint allow_null             # <<<<<<<<<<<<<<
 *         str label
 *         object default_value
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_5Field_10allow_null_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_5Field_10allow_null_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_5Field_10allow_null___get__(((struct __pyx_obj_9drf_turbo_6fields_Field *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_5Field_10allow_null___get__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->allow_null); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 13, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.Field.allow_null.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_5Field_10allow_null_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_5Field_10allow_null_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_5Field_10allow_null_2__set__(((struct __pyx_obj_9drf_turbo_6fields_Field *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_5Field_10allow_null_2__set__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_v_self->allow_null = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("drf_turbo.fields.Field.allow_null.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pxd":14
 *         bint read_only
 *         bint allow_null
 *         str label             # <<<<<<<<<<<<<<
 *         object default_value
 *         object initial
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_5Field_5label_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_5Field_5label_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_5Field_5label___get__(((struct __pyx_obj_9drf_turbo_6fields_Field *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_5Field_5label___get__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->label);
  __pyx_r = __pyx_v_self->label;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_5Field_5label_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_5Field_5label_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_5Field_5label_2__set__(((struct __pyx_obj_9drf_turbo_6fields_Field *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_5Field_5label_2__set__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(PyString_CheckExact(__pyx_v_value))||((__pyx_v_value) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_v_value)->tp_name), 0))) __PYX_ERR(2, 14, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->label);
  __Pyx_DECREF(__pyx_v_self->label);
  __pyx_v_self->label = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.Field.label.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_5Field_5label_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_5Field_5label_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_5Field_5label_4__del__(((struct __pyx_obj_9drf_turbo_6fields_Field *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_5Field_5label_4__del__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->label);
  __Pyx_DECREF(__pyx_v_self->label);
  __pyx_v_self->label = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pxd":15
 *         bint allow_null
 *         str label
 *         object default_value             # <<<<<<<<<<<<<<
 *         object initial
 *         str help_text
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_5Field_13default_value_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_5Field_13default_value_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_5Field_13default_value___get__(((struct __pyx_obj_9drf_turbo_6fields_Field *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_5Field_13default_value___get__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->default_value);
  __pyx_r = __pyx_v_self->default_value;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_5Field_13default_value_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_5Field_13default_value_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_5Field_13default_value_2__set__(((struct __pyx_obj_9drf_turbo_6fields_Field *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_5Field_13default_value_2__set__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->default_value);
  __Pyx_DECREF(__pyx_v_self->default_value);
  __pyx_v_self->default_value = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_5Field_13default_value_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_5Field_13default_value_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_5Field_13default_value_4__del__(((struct __pyx_obj_9drf_turbo_6fields_Field *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_5Field_13default_value_4__del__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->default_value);
  __Pyx_DECREF(__pyx_v_self->default_value);
  __pyx_v_self->default_value = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pxd":16
 *         str label
 *         object default_value
 *         object initial             # <<<<<<<<<<<<<<
 *         str help_text
 *         dict style
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_5Field_7initial_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_5Field_7initial_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_5Field_7initial___get__(((struct __pyx_obj_9drf_turbo_6fields_Field *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_5Field_7initial___get__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->initial);
  __pyx_r = __pyx_v_self->initial;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_5Field_7initial_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_5Field_7initial_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_5Field_7initial_2__set__(((struct __pyx_obj_9drf_turbo_6fields_Field *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_5Field_7initial_2__set__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->initial);
  __Pyx_DECREF(__pyx_v_self->initial);
  __pyx_v_self->initial = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_5Field_7initial_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_5Field_7initial_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_5Field_7initial_4__del__(((struct __pyx_obj_9drf_turbo_6fields_Field *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_5Field_7initial_4__del__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->initial);
  __Pyx_DECREF(__pyx_v_self->initial);
  __pyx_v_self->initial = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pxd":17
 *         object default_value
 *         object initial
 *         str help_text             # <<<<<<<<<<<<<<
 *         dict style
 *         str field_name
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_5Field_9help_text_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_5Field_9help_text_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_5Field_9help_text___get__(((struct __pyx_obj_9drf_turbo_6fields_Field *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_5Field_9help_text___get__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->help_text);
  __pyx_r = __pyx_v_self->help_text;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_5Field_9help_text_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_5Field_9help_text_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_5Field_9help_text_2__set__(((struct __pyx_obj_9drf_turbo_6fields_Field *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_5Field_9help_text_2__set__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(PyString_CheckExact(__pyx_v_value))||((__pyx_v_value) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_v_value)->tp_name), 0))) __PYX_ERR(2, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->help_text);
  __Pyx_DECREF(__pyx_v_self->help_text);
  __pyx_v_self->help_text = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.Field.help_text.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_5Field_9help_text_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_5Field_9help_text_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_5Field_9help_text_4__del__(((struct __pyx_obj_9drf_turbo_6fields_Field *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_5Field_9help_text_4__del__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->help_text);
  __Pyx_DECREF(__pyx_v_self->help_text);
  __pyx_v_self->help_text = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pxd":18
 *         object initial
 *         str help_text
 *         dict style             # <<<<<<<<<<<<<<
 *         str field_name
 *         object root
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_5Field_5style_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_5Field_5style_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_5Field_5style___get__(((struct __pyx_obj_9drf_turbo_6fields_Field *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_5Field_5style___get__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->style);
  __pyx_r = __pyx_v_self->style;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_5Field_5style_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_5Field_5style_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_5Field_5style_2__set__(((struct __pyx_obj_9drf_turbo_6fields_Field *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_5Field_5style_2__set__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(PyDict_CheckExact(__pyx_v_value))||((__pyx_v_value) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_v_value)->tp_name), 0))) __PYX_ERR(2, 18, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->style);
  __Pyx_DECREF(__pyx_v_self->style);
  __pyx_v_self->style = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.Field.style.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_5Field_5style_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_5Field_5style_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_5Field_5style_4__del__(((struct __pyx_obj_9drf_turbo_6fields_Field *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_5Field_5style_4__del__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->style);
  __Pyx_DECREF(__pyx_v_self->style);
  __pyx_v_self->style = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pxd":19
 *         str help_text
 *         dict style
 *         str field_name             # <<<<<<<<<<<<<<
 *         object root
 *         object validators
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_5Field_10field_name_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_5Field_10field_name_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_5Field_10field_name___get__(((struct __pyx_obj_9drf_turbo_6fields_Field *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_5Field_10field_name___get__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->field_name);
  __pyx_r = __pyx_v_self->field_name;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_5Field_10field_name_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_5Field_10field_name_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_5Field_10field_name_2__set__(((struct __pyx_obj_9drf_turbo_6fields_Field *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_5Field_10field_name_2__set__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(PyString_CheckExact(__pyx_v_value))||((__pyx_v_value) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_v_value)->tp_name), 0))) __PYX_ERR(2, 19, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->field_name);
  __Pyx_DECREF(__pyx_v_self->field_name);
  __pyx_v_self->field_name = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.Field.field_name.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_5Field_10field_name_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_5Field_10field_name_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_5Field_10field_name_4__del__(((struct __pyx_obj_9drf_turbo_6fields_Field *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_5Field_10field_name_4__del__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->field_name);
  __Pyx_DECREF(__pyx_v_self->field_name);
  __pyx_v_self->field_name = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pxd":20
 *         dict style
 *         str field_name
 *         object root             # <<<<<<<<<<<<<<
 *         object validators
 *         dict error_messages
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_5Field_4root_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_5Field_4root_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_5Field_4root___get__(((struct __pyx_obj_9drf_turbo_6fields_Field *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_5Field_4root___get__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->root);
  __pyx_r = __pyx_v_self->root;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_5Field_4root_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_5Field_4root_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_5Field_4root_2__set__(((struct __pyx_obj_9drf_turbo_6fields_Field *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_5Field_4root_2__set__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->root);
  __Pyx_DECREF(__pyx_v_self->root);
  __pyx_v_self->root = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_5Field_4root_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_5Field_4root_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_5Field_4root_4__del__(((struct __pyx_obj_9drf_turbo_6fields_Field *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_5Field_4root_4__del__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->root);
  __Pyx_DECREF(__pyx_v_self->root);
  __pyx_v_self->root = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pxd":21
 *         str field_name
 *         object root
 *         object validators             # <<<<<<<<<<<<<<
 *         dict error_messages
 *         list attrs
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_5Field_10validators_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_5Field_10validators_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_5Field_10validators___get__(((struct __pyx_obj_9drf_turbo_6fields_Field *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_5Field_10validators___get__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->validators);
  __pyx_r = __pyx_v_self->validators;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_5Field_10validators_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_5Field_10validators_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_5Field_10validators_2__set__(((struct __pyx_obj_9drf_turbo_6fields_Field *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_5Field_10validators_2__set__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->validators);
  __Pyx_DECREF(__pyx_v_self->validators);
  __pyx_v_self->validators = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_5Field_10validators_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_5Field_10validators_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_5Field_10validators_4__del__(((struct __pyx_obj_9drf_turbo_6fields_Field *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_5Field_10validators_4__del__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->validators);
  __Pyx_DECREF(__pyx_v_self->validators);
  __pyx_v_self->validators = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pxd":22
 *         object root
 *         object validators
 *         dict error_messages             # <<<<<<<<<<<<<<
 *         list attrs
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_5Field_14error_messages_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_5Field_14error_messages_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_5Field_14error_messages___get__(((struct __pyx_obj_9drf_turbo_6fields_Field *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_5Field_14error_messages___get__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->error_messages);
  __pyx_r = __pyx_v_self->error_messages;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_5Field_14error_messages_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_5Field_14error_messages_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_5Field_14error_messages_2__set__(((struct __pyx_obj_9drf_turbo_6fields_Field *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_5Field_14error_messages_2__set__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(PyDict_CheckExact(__pyx_v_value))||((__pyx_v_value) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_v_value)->tp_name), 0))) __PYX_ERR(2, 22, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->error_messages);
  __Pyx_DECREF(__pyx_v_self->error_messages);
  __pyx_v_self->error_messages = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.Field.error_messages.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_5Field_14error_messages_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_5Field_14error_messages_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_5Field_14error_messages_4__del__(((struct __pyx_obj_9drf_turbo_6fields_Field *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_5Field_14error_messages_4__del__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->error_messages);
  __Pyx_DECREF(__pyx_v_self->error_messages);
  __pyx_v_self->error_messages = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pxd":23
 *         object validators
 *         dict error_messages
 *         list attrs             # <<<<<<<<<<<<<<
 * 
 *     cpdef serialize(self, value , dict context)
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_5Field_5attrs_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_5Field_5attrs_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_5Field_5attrs___get__(((struct __pyx_obj_9drf_turbo_6fields_Field *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_5Field_5attrs___get__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->attrs);
  __pyx_r = __pyx_v_self->attrs;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_5Field_5attrs_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_5Field_5attrs_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_5Field_5attrs_2__set__(((struct __pyx_obj_9drf_turbo_6fields_Field *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_5Field_5attrs_2__set__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(PyList_CheckExact(__pyx_v_value))||((__pyx_v_value) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_v_value)->tp_name), 0))) __PYX_ERR(2, 23, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->attrs);
  __Pyx_DECREF(__pyx_v_self->attrs);
  __pyx_v_self->attrs = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.Field.attrs.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_5Field_5attrs_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_5Field_5attrs_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_5Field_5attrs_4__del__(((struct __pyx_obj_9drf_turbo_6fields_Field *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_5Field_5attrs_4__del__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->attrs);
  __Pyx_DECREF(__pyx_v_self->attrs);
  __pyx_v_self->attrs = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_5Field_25__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_5Field_25__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_5Field_24__reduce_cython__(((struct __pyx_obj_9drf_turbo_6fields_Field *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_5Field_24__reduce_cython__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self.allow_null, self.attr, self.attrs, self.call, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->allow_null); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_self->call); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyBool_FromLong(__pyx_v_self->read_only); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyBool_FromLong(__pyx_v_self->required); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyBool_FromLong(__pyx_v_self->write_only); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = PyTuple_New(16); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->attr);
  __Pyx_GIVEREF(__pyx_v_self->attr);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_v_self->attr);
  __Pyx_INCREF(__pyx_v_self->attrs);
  __Pyx_GIVEREF(__pyx_v_self->attrs);
  PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_self->attrs);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_6, 3, __pyx_t_2);
  __Pyx_INCREF(__pyx_v_self->default_value);
  __Pyx_GIVEREF(__pyx_v_self->default_value);
  PyTuple_SET_ITEM(__pyx_t_6, 4, __pyx_v_self->default_value);
  __Pyx_INCREF(__pyx_v_self->error_messages);
  __Pyx_GIVEREF(__pyx_v_self->error_messages);
  PyTuple_SET_ITEM(__pyx_t_6, 5, __pyx_v_self->error_messages);
  __Pyx_INCREF(__pyx_v_self->field_name);
  __Pyx_GIVEREF(__pyx_v_self->field_name);
  PyTuple_SET_ITEM(__pyx_t_6, 6, __pyx_v_self->field_name);
  __Pyx_INCREF(__pyx_v_self->help_text);
  __Pyx_GIVEREF(__pyx_v_self->help_text);
  PyTuple_SET_ITEM(__pyx_t_6, 7, __pyx_v_self->help_text);
  __Pyx_INCREF(__pyx_v_self->initial);
  __Pyx_GIVEREF(__pyx_v_self->initial);
  PyTuple_SET_ITEM(__pyx_t_6, 8, __pyx_v_self->initial);
  __Pyx_INCREF(__pyx_v_self->label);
  __Pyx_GIVEREF(__pyx_v_self->label);
  PyTuple_SET_ITEM(__pyx_t_6, 9, __pyx_v_self->label);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 10, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_6, 11, __pyx_t_4);
  __Pyx_INCREF(__pyx_v_self->root);
  __Pyx_GIVEREF(__pyx_v_self->root);
  PyTuple_SET_ITEM(__pyx_t_6, 12, __pyx_v_self->root);
  __Pyx_INCREF(__pyx_v_self->style);
  __Pyx_GIVEREF(__pyx_v_self->style);
  PyTuple_SET_ITEM(__pyx_t_6, 13, __pyx_v_self->style);
  __Pyx_INCREF(__pyx_v_self->validators);
  __Pyx_GIVEREF(__pyx_v_self->validators);
  PyTuple_SET_ITEM(__pyx_t_6, 14, __pyx_v_self->validators);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_6, 15, __pyx_t_5);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_6);
  __pyx_t_6 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self.allow_null, self.attr, self.attrs, self.call, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_6 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_v__dict = __pyx_t_6;
  __pyx_t_6 = 0;

  /* "(tree fragment)":7
 *     state = (self.allow_null, self.attr, self.attrs, self.call, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_7 = (__pyx_v__dict != Py_None);
  __pyx_t_8 = (__pyx_t_7 != 0);
  if (__pyx_t_8) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_6 = PyTuple_New(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v__dict);
    __pyx_t_5 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_6); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_5));
    __pyx_t_5 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.root is not None or self.style is not None or self.validators is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self.allow_null, self.attr, self.attrs, self.call, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.root is not None or self.style is not None or self.validators is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_Field, (type(self), 0x4f70d27, None), state
 */
  /*else*/ {
    __pyx_t_7 = (__pyx_v_self->attr != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->attrs != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->default_value != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->error_messages != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->field_name != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->help_text != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->initial != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->label != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->root != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->style != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->validators != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    __pyx_t_8 = __pyx_t_9;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_8;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.root is not None or self.style is not None or self.validators is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_Field, (type(self), 0x4f70d27, None), state
 *     else:
 */
  __pyx_t_8 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_8) {

    /* "(tree fragment)":13
 *         use_setstate = self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.root is not None or self.style is not None or self.validators is not None
 *     if use_setstate:
 *         return __pyx_unpickle_Field, (type(self), 0x4f70d27, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_Field, (type(self), 0x4f70d27, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_pyx_unpickle_Field); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_6, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_83299623);
    __Pyx_GIVEREF(__pyx_int_83299623);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_int_83299623);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_6, 2, Py_None);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_6);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_state);
    __pyx_t_5 = 0;
    __pyx_t_6 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.root is not None or self.style is not None or self.validators is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_Field, (type(self), 0x4f70d27, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_Field, (type(self), 0x4f70d27, None), state
 *     else:
 *         return __pyx_unpickle_Field, (type(self), 0x4f70d27, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_Field__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_pyx_unpickle_Field); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_6, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_83299623);
    __Pyx_GIVEREF(__pyx_int_83299623);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_int_83299623);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_state);
    __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_6);
    __pyx_t_4 = 0;
    __pyx_t_6 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("drf_turbo.fields.Field.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_Field, (type(self), 0x4f70d27, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_Field__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_5Field_27__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_5Field_27__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_5Field_26__setstate_cython__(((struct __pyx_obj_9drf_turbo_6fields_Field *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_5Field_26__setstate_cython__(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_Field, (type(self), 0x4f70d27, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_Field__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields___pyx_unpickle_Field__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_Field, (type(self), 0x4f70d27, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_Field__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.Field.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":307
 *     }
 *     _initial = ''
 *     def __init__(self,**kwargs) :             # <<<<<<<<<<<<<<
 *         self.allow_blank = kwargs.pop('allow_blank', False)
 *         self.trim_whitespace = kwargs.pop('trim_whitespace', True)
 */

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_8StrField_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_8StrField_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_kwargs = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  if (unlikely(PyTuple_GET_SIZE(__pyx_args) > 0)) {
    __Pyx_RaiseArgtupleInvalid("__init__", 1, 0, 0, PyTuple_GET_SIZE(__pyx_args)); return -1;}
  if (__pyx_kwds && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "__init__", 1))) return -1;
  __pyx_v_kwargs = (__pyx_kwds) ? PyDict_Copy(__pyx_kwds) : PyDict_New(); if (unlikely(!__pyx_v_kwargs)) return -1;
  __Pyx_GOTREF(__pyx_v_kwargs);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_8StrField___init__(((struct __pyx_obj_9drf_turbo_6fields_StrField *)__pyx_v_self), __pyx_v_kwargs);

  /* function exit code */
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_8StrField___init__(struct __pyx_obj_9drf_turbo_6fields_StrField *__pyx_v_self, PyObject *__pyx_v_kwargs) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "drf_turbo/fields.pyx":308
 *     _initial = ''
 *     def __init__(self,**kwargs) :
 *         self.allow_blank = kwargs.pop('allow_blank', False)             # <<<<<<<<<<<<<<
 *         self.trim_whitespace = kwargs.pop('trim_whitespace', True)
 *         self.max_length = kwargs.pop('max_length', None)
 */
  __pyx_t_1 = __Pyx_PyDict_Pop(__pyx_v_kwargs, __pyx_n_s_allow_blank, Py_False); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 308, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->allow_blank);
  __Pyx_DECREF(__pyx_v_self->allow_blank);
  __pyx_v_self->allow_blank = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "drf_turbo/fields.pyx":309
 *     def __init__(self,**kwargs) :
 *         self.allow_blank = kwargs.pop('allow_blank', False)
 *         self.trim_whitespace = kwargs.pop('trim_whitespace', True)             # <<<<<<<<<<<<<<
 *         self.max_length = kwargs.pop('max_length', None)
 *         self.min_length = kwargs.pop('min_length', None)
 */
  __pyx_t_1 = __Pyx_PyDict_Pop(__pyx_v_kwargs, __pyx_n_s_trim_whitespace, Py_True); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 309, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->trim_whitespace);
  __Pyx_DECREF(__pyx_v_self->trim_whitespace);
  __pyx_v_self->trim_whitespace = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "drf_turbo/fields.pyx":310
 *         self.allow_blank = kwargs.pop('allow_blank', False)
 *         self.trim_whitespace = kwargs.pop('trim_whitespace', True)
 *         self.max_length = kwargs.pop('max_length', None)             # <<<<<<<<<<<<<<
 *         self.min_length = kwargs.pop('min_length', None)
 *         super().__init__(**kwargs)
 */
  __pyx_t_1 = __Pyx_PyDict_Pop(__pyx_v_kwargs, __pyx_n_s_max_length, Py_None); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 310, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->max_length);
  __Pyx_DECREF(__pyx_v_self->max_length);
  __pyx_v_self->max_length = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "drf_turbo/fields.pyx":311
 *         self.trim_whitespace = kwargs.pop('trim_whitespace', True)
 *         self.max_length = kwargs.pop('max_length', None)
 *         self.min_length = kwargs.pop('min_length', None)             # <<<<<<<<<<<<<<
 *         super().__init__(**kwargs)
 * 
 */
  __pyx_t_1 = __Pyx_PyDict_Pop(__pyx_v_kwargs, __pyx_n_s_min_length, Py_None); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 311, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->min_length);
  __Pyx_DECREF(__pyx_v_self->min_length);
  __pyx_v_self->min_length = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "drf_turbo/fields.pyx":312
 *         self.max_length = kwargs.pop('max_length', None)
 *         self.min_length = kwargs.pop('min_length', None)
 *         super().__init__(**kwargs)             # <<<<<<<<<<<<<<
 * 
 *     cpdef serialize(self,value,dict context) :
 */
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 312, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_9drf_turbo_6fields_StrField));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_9drf_turbo_6fields_StrField));
  PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)__pyx_ptype_9drf_turbo_6fields_StrField));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_1, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 312, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_init); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 312, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyDict_Copy(__pyx_v_kwargs); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 312, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 312, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "drf_turbo/fields.pyx":307
 *     }
 *     _initial = ''
 *     def __init__(self,**kwargs) :             # <<<<<<<<<<<<<<
 *         self.allow_blank = kwargs.pop('allow_blank', False)
 *         self.trim_whitespace = kwargs.pop('trim_whitespace', True)
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("drf_turbo.fields.StrField.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":314
 *         super().__init__(**kwargs)
 * 
 *     cpdef serialize(self,value,dict context) :             # <<<<<<<<<<<<<<
 *         return str(value)
 * 
 */

static PyObject *__pyx_pw_9drf_turbo_6fields_8StrField_3serialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_8StrField_serialize(CYTHON_UNUSED struct __pyx_obj_9drf_turbo_6fields_StrField *__pyx_v_self, PyObject *__pyx_v_value, CYTHON_UNUSED PyObject *__pyx_v_context, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("serialize", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_serialize); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 314, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_9drf_turbo_6fields_8StrField_3serialize)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        __pyx_t_5 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
            __pyx_t_5 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_value, __pyx_v_context};
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 314, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_value, __pyx_v_context};
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 314, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        {
          __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 314, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          if (__pyx_t_4) {
            __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
          }
          __Pyx_INCREF(__pyx_v_value);
          __Pyx_GIVEREF(__pyx_v_value);
          PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_v_value);
          __Pyx_INCREF(__pyx_v_context);
          __Pyx_GIVEREF(__pyx_v_context);
          PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_v_context);
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_6, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 314, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        }
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "drf_turbo/fields.pyx":315
 * 
 *     cpdef serialize(self,value,dict context) :
 *         return str(value)             # <<<<<<<<<<<<<<
 * 
 *     cpdef deserialize(self,data,dict context) :
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), __pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 315, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "drf_turbo/fields.pyx":314
 *         super().__init__(**kwargs)
 * 
 *     cpdef serialize(self,value,dict context) :             # <<<<<<<<<<<<<<
 *         return str(value)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("drf_turbo.fields.StrField.serialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_8StrField_3serialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_8StrField_3serialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_value = 0;
  PyObject *__pyx_v_context = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("serialize (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_value,&__pyx_n_s_context,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_value)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_context)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("serialize", 1, 2, 2, 1); __PYX_ERR(0, 314, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "serialize") < 0)) __PYX_ERR(0, 314, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_value = values[0];
    __pyx_v_context = ((PyObject*)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("serialize", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 314, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.StrField.serialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_context), (&PyDict_Type), 1, "context", 1))) __PYX_ERR(0, 314, __pyx_L1_error)
  __pyx_r = __pyx_pf_9drf_turbo_6fields_8StrField_2serialize(((struct __pyx_obj_9drf_turbo_6fields_StrField *)__pyx_v_self), __pyx_v_value, __pyx_v_context);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_8StrField_2serialize(struct __pyx_obj_9drf_turbo_6fields_StrField *__pyx_v_self, PyObject *__pyx_v_value, PyObject *__pyx_v_context) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("serialize", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields_8StrField_serialize(__pyx_v_self, __pyx_v_value, __pyx_v_context, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 314, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.StrField.serialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":317
 *         return str(value)
 * 
 *     cpdef deserialize(self,data,dict context) :             # <<<<<<<<<<<<<<
 *         if data == '' or (self.trim_whitespace and str(data).strip() == ''):
 *             if not self.allow_blank:
 */

static PyObject *__pyx_pw_9drf_turbo_6fields_8StrField_5deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_8StrField_deserialize(struct __pyx_obj_9drf_turbo_6fields_StrField *__pyx_v_self, PyObject *__pyx_v_data, CYTHON_UNUSED PyObject *__pyx_v_context, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  int __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("deserialize", 0);
  __Pyx_INCREF(__pyx_v_data);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_deserialize); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 317, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_9drf_turbo_6fields_8StrField_5deserialize)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        __pyx_t_5 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
            __pyx_t_5 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_data, __pyx_v_context};
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 317, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_data, __pyx_v_context};
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 317, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        {
          __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 317, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          if (__pyx_t_4) {
            __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
          }
          __Pyx_INCREF(__pyx_v_data);
          __Pyx_GIVEREF(__pyx_v_data);
          PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_v_data);
          __Pyx_INCREF(__pyx_v_context);
          __Pyx_GIVEREF(__pyx_v_context);
          PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_v_context);
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_6, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 317, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        }
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "drf_turbo/fields.pyx":318
 * 
 *     cpdef deserialize(self,data,dict context) :
 *         if data == '' or (self.trim_whitespace and str(data).strip() == ''):             # <<<<<<<<<<<<<<
 *             if not self.allow_blank:
 *                 raise self.raise_if_fail('blank')
 */
  __pyx_t_8 = (__Pyx_PyString_Equals(__pyx_v_data, __pyx_kp_s__9, Py_EQ)); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 318, __pyx_L1_error)
  if (!__pyx_t_8) {
  } else {
    __pyx_t_7 = __pyx_t_8;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_v_self->trim_whitespace); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 318, __pyx_L1_error)
  if (__pyx_t_8) {
  } else {
    __pyx_t_7 = __pyx_t_8;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), __pyx_v_data); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 318, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_strip); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 318, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 318, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_8 = (__Pyx_PyString_Equals(__pyx_t_1, __pyx_kp_s__9, Py_EQ)); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 318, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_7 = __pyx_t_8;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_7) {

    /* "drf_turbo/fields.pyx":319
 *     cpdef deserialize(self,data,dict context) :
 *         if data == '' or (self.trim_whitespace and str(data).strip() == ''):
 *             if not self.allow_blank:             # <<<<<<<<<<<<<<
 *                 raise self.raise_if_fail('blank')
 *         if isinstance(data, bool) or not isinstance(data, (str, int, float,)):
 */
    __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_v_self->allow_blank); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 319, __pyx_L1_error)
    __pyx_t_8 = ((!__pyx_t_7) != 0);
    if (unlikely(__pyx_t_8)) {

      /* "drf_turbo/fields.pyx":320
 *         if data == '' or (self.trim_whitespace and str(data).strip() == ''):
 *             if not self.allow_blank:
 *                 raise self.raise_if_fail('blank')             # <<<<<<<<<<<<<<
 *         if isinstance(data, bool) or not isinstance(data, (str, int, float,)):
 *            raise self.raise_if_fail('invalid')
 */
      __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_raise_if_fail); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 320, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_2 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_2)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_2);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_2, __pyx_n_s_blank) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_n_s_blank);
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 320, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_Raise(__pyx_t_1, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __PYX_ERR(0, 320, __pyx_L1_error)

      /* "drf_turbo/fields.pyx":319
 *     cpdef deserialize(self,data,dict context) :
 *         if data == '' or (self.trim_whitespace and str(data).strip() == ''):
 *             if not self.allow_blank:             # <<<<<<<<<<<<<<
 *                 raise self.raise_if_fail('blank')
 *         if isinstance(data, bool) or not isinstance(data, (str, int, float,)):
 */
    }

    /* "drf_turbo/fields.pyx":318
 * 
 *     cpdef deserialize(self,data,dict context) :
 *         if data == '' or (self.trim_whitespace and str(data).strip() == ''):             # <<<<<<<<<<<<<<
 *             if not self.allow_blank:
 *                 raise self.raise_if_fail('blank')
 */
  }

  /* "drf_turbo/fields.pyx":321
 *             if not self.allow_blank:
 *                 raise self.raise_if_fail('blank')
 *         if isinstance(data, bool) or not isinstance(data, (str, int, float,)):             # <<<<<<<<<<<<<<
 *            raise self.raise_if_fail('invalid')
 * 
 */
  __pyx_t_1 = ((PyObject*)&PyBool_Type);
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_7 = PyObject_IsInstance(__pyx_v_data, __pyx_t_1); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 321, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_9 = (__pyx_t_7 != 0);
  if (!__pyx_t_9) {
  } else {
    __pyx_t_8 = __pyx_t_9;
    goto __pyx_L9_bool_binop_done;
  }
  __pyx_t_7 = PyString_Check(__pyx_v_data); 
  __pyx_t_10 = (__pyx_t_7 != 0);
  if (!__pyx_t_10) {
  } else {
    __pyx_t_9 = __pyx_t_10;
    goto __pyx_L11_bool_binop_done;
  }
  __pyx_t_10 = PyInt_Check(__pyx_v_data); 
  __pyx_t_7 = (__pyx_t_10 != 0);
  if (!__pyx_t_7) {
  } else {
    __pyx_t_9 = __pyx_t_7;
    goto __pyx_L11_bool_binop_done;
  }
  __pyx_t_7 = PyFloat_Check(__pyx_v_data); 
  __pyx_t_10 = (__pyx_t_7 != 0);
  __pyx_t_9 = __pyx_t_10;
  __pyx_L11_bool_binop_done:;
  __pyx_t_10 = ((!(__pyx_t_9 != 0)) != 0);
  __pyx_t_8 = __pyx_t_10;
  __pyx_L9_bool_binop_done:;
  if (unlikely(__pyx_t_8)) {

    /* "drf_turbo/fields.pyx":322
 *                 raise self.raise_if_fail('blank')
 *         if isinstance(data, bool) or not isinstance(data, (str, int, float,)):
 *            raise self.raise_if_fail('invalid')             # <<<<<<<<<<<<<<
 * 
 *         if self.min_length is not None:
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_raise_if_fail); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 322, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_2)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_2, __pyx_n_s_invalid) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_n_s_invalid);
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 322, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 322, __pyx_L1_error)

    /* "drf_turbo/fields.pyx":321
 *             if not self.allow_blank:
 *                 raise self.raise_if_fail('blank')
 *         if isinstance(data, bool) or not isinstance(data, (str, int, float,)):             # <<<<<<<<<<<<<<
 *            raise self.raise_if_fail('invalid')
 * 
 */
  }

  /* "drf_turbo/fields.pyx":324
 *            raise self.raise_if_fail('invalid')
 * 
 *         if self.min_length is not None:             # <<<<<<<<<<<<<<
 *             if len(data) < self.min_length:
 *                 raise self.raise_if_fail("min_length",min_length=self.min_length)
 */
  __pyx_t_8 = (__pyx_v_self->min_length != Py_None);
  __pyx_t_10 = (__pyx_t_8 != 0);
  if (__pyx_t_10) {

    /* "drf_turbo/fields.pyx":325
 * 
 *         if self.min_length is not None:
 *             if len(data) < self.min_length:             # <<<<<<<<<<<<<<
 *                 raise self.raise_if_fail("min_length",min_length=self.min_length)
 *         if self.max_length is not None:
 */
    __pyx_t_11 = PyObject_Length(__pyx_v_data); if (unlikely(__pyx_t_11 == ((Py_ssize_t)-1))) __PYX_ERR(0, 325, __pyx_L1_error)
    __pyx_t_1 = PyInt_FromSsize_t(__pyx_t_11); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 325, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = PyObject_RichCompare(__pyx_t_1, __pyx_v_self->min_length, Py_LT); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 325, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_10 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 325, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(__pyx_t_10)) {

      /* "drf_turbo/fields.pyx":326
 *         if self.min_length is not None:
 *             if len(data) < self.min_length:
 *                 raise self.raise_if_fail("min_length",min_length=self.min_length)             # <<<<<<<<<<<<<<
 *         if self.max_length is not None:
 *             if len(data) > self.max_length:
 */
      __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_raise_if_fail); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 326, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 326, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_min_length, __pyx_v_self->min_length) < 0) __PYX_ERR(0, 326, __pyx_L1_error)
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_tuple__10, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 326, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_Raise(__pyx_t_2, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __PYX_ERR(0, 326, __pyx_L1_error)

      /* "drf_turbo/fields.pyx":325
 * 
 *         if self.min_length is not None:
 *             if len(data) < self.min_length:             # <<<<<<<<<<<<<<
 *                 raise self.raise_if_fail("min_length",min_length=self.min_length)
 *         if self.max_length is not None:
 */
    }

    /* "drf_turbo/fields.pyx":324
 *            raise self.raise_if_fail('invalid')
 * 
 *         if self.min_length is not None:             # <<<<<<<<<<<<<<
 *             if len(data) < self.min_length:
 *                 raise self.raise_if_fail("min_length",min_length=self.min_length)
 */
  }

  /* "drf_turbo/fields.pyx":327
 *             if len(data) < self.min_length:
 *                 raise self.raise_if_fail("min_length",min_length=self.min_length)
 *         if self.max_length is not None:             # <<<<<<<<<<<<<<
 *             if len(data) > self.max_length:
 *                 raise self.raise_if_fail("max_length",max_length=self.max_length)
 */
  __pyx_t_10 = (__pyx_v_self->max_length != Py_None);
  __pyx_t_8 = (__pyx_t_10 != 0);
  if (__pyx_t_8) {

    /* "drf_turbo/fields.pyx":328
 *                 raise self.raise_if_fail("min_length",min_length=self.min_length)
 *         if self.max_length is not None:
 *             if len(data) > self.max_length:             # <<<<<<<<<<<<<<
 *                 raise self.raise_if_fail("max_length",max_length=self.max_length)
 *         if '\x00' in str(data):
 */
    __pyx_t_11 = PyObject_Length(__pyx_v_data); if (unlikely(__pyx_t_11 == ((Py_ssize_t)-1))) __PYX_ERR(0, 328, __pyx_L1_error)
    __pyx_t_2 = PyInt_FromSsize_t(__pyx_t_11); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 328, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = PyObject_RichCompare(__pyx_t_2, __pyx_v_self->max_length, Py_GT); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 328, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 328, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (unlikely(__pyx_t_8)) {

      /* "drf_turbo/fields.pyx":329
 *         if self.max_length is not None:
 *             if len(data) > self.max_length:
 *                 raise self.raise_if_fail("max_length",max_length=self.max_length)             # <<<<<<<<<<<<<<
 *         if '\x00' in str(data):
 *             raise self.raise_if_fail('null')
 */
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_raise_if_fail); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 329, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_2 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 329, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_max_length, __pyx_v_self->max_length) < 0) __PYX_ERR(0, 329, __pyx_L1_error)
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__11, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 329, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __PYX_ERR(0, 329, __pyx_L1_error)

      /* "drf_turbo/fields.pyx":328
 *                 raise self.raise_if_fail("min_length",min_length=self.min_length)
 *         if self.max_length is not None:
 *             if len(data) > self.max_length:             # <<<<<<<<<<<<<<
 *                 raise self.raise_if_fail("max_length",max_length=self.max_length)
 *         if '\x00' in str(data):
 */
    }

    /* "drf_turbo/fields.pyx":327
 *             if len(data) < self.min_length:
 *                 raise self.raise_if_fail("min_length",min_length=self.min_length)
 *         if self.max_length is not None:             # <<<<<<<<<<<<<<
 *             if len(data) > self.max_length:
 *                 raise self.raise_if_fail("max_length",max_length=self.max_length)
 */
  }

  /* "drf_turbo/fields.pyx":330
 *             if len(data) > self.max_length:
 *                 raise self.raise_if_fail("max_length",max_length=self.max_length)
 *         if '\x00' in str(data):             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail('null')
 * 
 */
  __pyx_t_3 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), __pyx_v_data); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 330, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_8 = (__Pyx_PySequence_ContainsTF(__pyx_kp_s__12, __pyx_t_3, Py_EQ)); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 330, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_10 = (__pyx_t_8 != 0);
  if (unlikely(__pyx_t_10)) {

    /* "drf_turbo/fields.pyx":331
 *                 raise self.raise_if_fail("max_length",max_length=self.max_length)
 *         if '\x00' in str(data):
 *             raise self.raise_if_fail('null')             # <<<<<<<<<<<<<<
 * 
 *         data = str(data)
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_raise_if_fail); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 331, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_1)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_1) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_1, __pyx_n_s_null) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_n_s_null);
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 331, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 331, __pyx_L1_error)

    /* "drf_turbo/fields.pyx":330
 *             if len(data) > self.max_length:
 *                 raise self.raise_if_fail("max_length",max_length=self.max_length)
 *         if '\x00' in str(data):             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail('null')
 * 
 */
  }

  /* "drf_turbo/fields.pyx":333
 *             raise self.raise_if_fail('null')
 * 
 *         data = str(data)             # <<<<<<<<<<<<<<
 *         return data.strip() if self.trim_whitespace else data
 * 
 */
  __pyx_t_3 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), __pyx_v_data); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 333, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF_SET(__pyx_v_data, __pyx_t_3);
  __pyx_t_3 = 0;

  /* "drf_turbo/fields.pyx":334
 * 
 *         data = str(data)
 *         return data.strip() if self.trim_whitespace else data             # <<<<<<<<<<<<<<
 * 
 * @cython.final
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_10 = __Pyx_PyObject_IsTrue(__pyx_v_self->trim_whitespace); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 334, __pyx_L1_error)
  if (__pyx_t_10) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_data, __pyx_n_s_strip); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 334, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_6 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
      }
    }
    __pyx_t_2 = (__pyx_t_6) ? __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_6) : __Pyx_PyObject_CallNoArg(__pyx_t_1);
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 334, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_3 = __pyx_t_2;
    __pyx_t_2 = 0;
  } else {
    __Pyx_INCREF(__pyx_v_data);
    __pyx_t_3 = __pyx_v_data;
  }
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "drf_turbo/fields.pyx":317
 *         return str(value)
 * 
 *     cpdef deserialize(self,data,dict context) :             # <<<<<<<<<<<<<<
 *         if data == '' or (self.trim_whitespace and str(data).strip() == ''):
 *             if not self.allow_blank:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("drf_turbo.fields.StrField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_data);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_8StrField_5deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_8StrField_5deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_data = 0;
  PyObject *__pyx_v_context = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("deserialize (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_data,&__pyx_n_s_context,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_data)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_context)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("deserialize", 1, 2, 2, 1); __PYX_ERR(0, 317, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "deserialize") < 0)) __PYX_ERR(0, 317, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_data = values[0];
    __pyx_v_context = ((PyObject*)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("deserialize", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 317, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.StrField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_context), (&PyDict_Type), 1, "context", 1))) __PYX_ERR(0, 317, __pyx_L1_error)
  __pyx_r = __pyx_pf_9drf_turbo_6fields_8StrField_4deserialize(((struct __pyx_obj_9drf_turbo_6fields_StrField *)__pyx_v_self), __pyx_v_data, __pyx_v_context);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_8StrField_4deserialize(struct __pyx_obj_9drf_turbo_6fields_StrField *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("deserialize", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields_8StrField_deserialize(__pyx_v_self, __pyx_v_data, __pyx_v_context, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 317, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.StrField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pxd":39
 * cdef class StrField(Field):
 *     cdef public:
 *         allow_blank             # <<<<<<<<<<<<<<
 *         trim_whitespace
 *         max_length
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_8StrField_11allow_blank_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_8StrField_11allow_blank_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_8StrField_11allow_blank___get__(((struct __pyx_obj_9drf_turbo_6fields_StrField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_8StrField_11allow_blank___get__(struct __pyx_obj_9drf_turbo_6fields_StrField *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->allow_blank);
  __pyx_r = __pyx_v_self->allow_blank;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_8StrField_11allow_blank_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_8StrField_11allow_blank_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_8StrField_11allow_blank_2__set__(((struct __pyx_obj_9drf_turbo_6fields_StrField *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_8StrField_11allow_blank_2__set__(struct __pyx_obj_9drf_turbo_6fields_StrField *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->allow_blank);
  __Pyx_DECREF(__pyx_v_self->allow_blank);
  __pyx_v_self->allow_blank = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_8StrField_11allow_blank_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_8StrField_11allow_blank_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_8StrField_11allow_blank_4__del__(((struct __pyx_obj_9drf_turbo_6fields_StrField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_8StrField_11allow_blank_4__del__(struct __pyx_obj_9drf_turbo_6fields_StrField *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->allow_blank);
  __Pyx_DECREF(__pyx_v_self->allow_blank);
  __pyx_v_self->allow_blank = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pxd":40
 *     cdef public:
 *         allow_blank
 *         trim_whitespace             # <<<<<<<<<<<<<<
 *         max_length
 *         min_length
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_8StrField_15trim_whitespace_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_8StrField_15trim_whitespace_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_8StrField_15trim_whitespace___get__(((struct __pyx_obj_9drf_turbo_6fields_StrField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_8StrField_15trim_whitespace___get__(struct __pyx_obj_9drf_turbo_6fields_StrField *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->trim_whitespace);
  __pyx_r = __pyx_v_self->trim_whitespace;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_8StrField_15trim_whitespace_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_8StrField_15trim_whitespace_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_8StrField_15trim_whitespace_2__set__(((struct __pyx_obj_9drf_turbo_6fields_StrField *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_8StrField_15trim_whitespace_2__set__(struct __pyx_obj_9drf_turbo_6fields_StrField *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->trim_whitespace);
  __Pyx_DECREF(__pyx_v_self->trim_whitespace);
  __pyx_v_self->trim_whitespace = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_8StrField_15trim_whitespace_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_8StrField_15trim_whitespace_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_8StrField_15trim_whitespace_4__del__(((struct __pyx_obj_9drf_turbo_6fields_StrField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_8StrField_15trim_whitespace_4__del__(struct __pyx_obj_9drf_turbo_6fields_StrField *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->trim_whitespace);
  __Pyx_DECREF(__pyx_v_self->trim_whitespace);
  __pyx_v_self->trim_whitespace = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pxd":41
 *         allow_blank
 *         trim_whitespace
 *         max_length             # <<<<<<<<<<<<<<
 *         min_length
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_8StrField_10max_length_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_8StrField_10max_length_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_8StrField_10max_length___get__(((struct __pyx_obj_9drf_turbo_6fields_StrField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_8StrField_10max_length___get__(struct __pyx_obj_9drf_turbo_6fields_StrField *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->max_length);
  __pyx_r = __pyx_v_self->max_length;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_8StrField_10max_length_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_8StrField_10max_length_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_8StrField_10max_length_2__set__(((struct __pyx_obj_9drf_turbo_6fields_StrField *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_8StrField_10max_length_2__set__(struct __pyx_obj_9drf_turbo_6fields_StrField *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->max_length);
  __Pyx_DECREF(__pyx_v_self->max_length);
  __pyx_v_self->max_length = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_8StrField_10max_length_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_8StrField_10max_length_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_8StrField_10max_length_4__del__(((struct __pyx_obj_9drf_turbo_6fields_StrField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_8StrField_10max_length_4__del__(struct __pyx_obj_9drf_turbo_6fields_StrField *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->max_length);
  __Pyx_DECREF(__pyx_v_self->max_length);
  __pyx_v_self->max_length = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pxd":42
 *         trim_whitespace
 *         max_length
 *         min_length             # <<<<<<<<<<<<<<
 * 
 * cdef class EmailField(StrField):
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_8StrField_10min_length_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_8StrField_10min_length_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_8StrField_10min_length___get__(((struct __pyx_obj_9drf_turbo_6fields_StrField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_8StrField_10min_length___get__(struct __pyx_obj_9drf_turbo_6fields_StrField *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->min_length);
  __pyx_r = __pyx_v_self->min_length;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_8StrField_10min_length_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_8StrField_10min_length_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_8StrField_10min_length_2__set__(((struct __pyx_obj_9drf_turbo_6fields_StrField *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_8StrField_10min_length_2__set__(struct __pyx_obj_9drf_turbo_6fields_StrField *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->min_length);
  __Pyx_DECREF(__pyx_v_self->min_length);
  __pyx_v_self->min_length = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_8StrField_10min_length_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_8StrField_10min_length_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_8StrField_10min_length_4__del__(((struct __pyx_obj_9drf_turbo_6fields_StrField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_8StrField_10min_length_4__del__(struct __pyx_obj_9drf_turbo_6fields_StrField *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->min_length);
  __Pyx_DECREF(__pyx_v_self->min_length);
  __pyx_v_self->min_length = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_8StrField_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_8StrField_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_8StrField_6__reduce_cython__(((struct __pyx_obj_9drf_turbo_6fields_StrField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_8StrField_6__reduce_cython__(struct __pyx_obj_9drf_turbo_6fields_StrField *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self.allow_blank, self.allow_null, self.attr, self.attrs, self.call, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.max_length, self.min_length, self.read_only, self.required, self.root, self.style, self.trim_whitespace, self.validators, self.write_only)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.allow_null); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.call); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.read_only); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.required); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.write_only); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = PyTuple_New(20); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_INCREF(__pyx_v_self->allow_blank);
  __Pyx_GIVEREF(__pyx_v_self->allow_blank);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v_self->allow_blank);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.attr);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.attr);
  PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_self->__pyx_base.attr);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.attrs);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.attrs);
  PyTuple_SET_ITEM(__pyx_t_6, 3, __pyx_v_self->__pyx_base.attrs);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_6, 4, __pyx_t_2);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.default_value);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.default_value);
  PyTuple_SET_ITEM(__pyx_t_6, 5, __pyx_v_self->__pyx_base.default_value);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.error_messages);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.error_messages);
  PyTuple_SET_ITEM(__pyx_t_6, 6, __pyx_v_self->__pyx_base.error_messages);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.field_name);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.field_name);
  PyTuple_SET_ITEM(__pyx_t_6, 7, __pyx_v_self->__pyx_base.field_name);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.help_text);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.help_text);
  PyTuple_SET_ITEM(__pyx_t_6, 8, __pyx_v_self->__pyx_base.help_text);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.initial);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.initial);
  PyTuple_SET_ITEM(__pyx_t_6, 9, __pyx_v_self->__pyx_base.initial);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.label);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.label);
  PyTuple_SET_ITEM(__pyx_t_6, 10, __pyx_v_self->__pyx_base.label);
  __Pyx_INCREF(__pyx_v_self->max_length);
  __Pyx_GIVEREF(__pyx_v_self->max_length);
  PyTuple_SET_ITEM(__pyx_t_6, 11, __pyx_v_self->max_length);
  __Pyx_INCREF(__pyx_v_self->min_length);
  __Pyx_GIVEREF(__pyx_v_self->min_length);
  PyTuple_SET_ITEM(__pyx_t_6, 12, __pyx_v_self->min_length);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 13, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_6, 14, __pyx_t_4);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.root);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.root);
  PyTuple_SET_ITEM(__pyx_t_6, 15, __pyx_v_self->__pyx_base.root);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.style);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.style);
  PyTuple_SET_ITEM(__pyx_t_6, 16, __pyx_v_self->__pyx_base.style);
  __Pyx_INCREF(__pyx_v_self->trim_whitespace);
  __Pyx_GIVEREF(__pyx_v_self->trim_whitespace);
  PyTuple_SET_ITEM(__pyx_t_6, 17, __pyx_v_self->trim_whitespace);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.validators);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.validators);
  PyTuple_SET_ITEM(__pyx_t_6, 18, __pyx_v_self->__pyx_base.validators);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_6, 19, __pyx_t_5);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_6);
  __pyx_t_6 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self.allow_blank, self.allow_null, self.attr, self.attrs, self.call, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.max_length, self.min_length, self.read_only, self.required, self.root, self.style, self.trim_whitespace, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_6 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_v__dict = __pyx_t_6;
  __pyx_t_6 = 0;

  /* "(tree fragment)":7
 *     state = (self.allow_blank, self.allow_null, self.attr, self.attrs, self.call, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.max_length, self.min_length, self.read_only, self.required, self.root, self.style, self.trim_whitespace, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_7 = (__pyx_v__dict != Py_None);
  __pyx_t_8 = (__pyx_t_7 != 0);
  if (__pyx_t_8) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_6 = PyTuple_New(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v__dict);
    __pyx_t_5 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_6); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_5));
    __pyx_t_5 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.allow_blank is not None or self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.max_length is not None or self.min_length is not None or self.root is not None or self.style is not None or self.trim_whitespace is not None or self.validators is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self.allow_blank, self.allow_null, self.attr, self.attrs, self.call, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.max_length, self.min_length, self.read_only, self.required, self.root, self.style, self.trim_whitespace, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self.allow_blank is not None or self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.max_length is not None or self.min_length is not None or self.root is not None or self.style is not None or self.trim_whitespace is not None or self.validators is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_StrField, (type(self), 0x0fbedf5, None), state
 */
  /*else*/ {
    __pyx_t_7 = (__pyx_v_self->allow_blank != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.attr != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.attrs != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.default_value != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.error_messages != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.field_name != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.help_text != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.initial != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.label != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->max_length != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->min_length != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.root != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.style != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->trim_whitespace != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.validators != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    __pyx_t_8 = __pyx_t_9;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_8;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.allow_blank is not None or self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.max_length is not None or self.min_length is not None or self.root is not None or self.style is not None or self.trim_whitespace is not None or self.validators is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_StrField, (type(self), 0x0fbedf5, None), state
 *     else:
 */
  __pyx_t_8 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_8) {

    /* "(tree fragment)":13
 *         use_setstate = self.allow_blank is not None or self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.max_length is not None or self.min_length is not None or self.root is not None or self.style is not None or self.trim_whitespace is not None or self.validators is not None
 *     if use_setstate:
 *         return __pyx_unpickle_StrField, (type(self), 0x0fbedf5, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_StrField, (type(self), 0x0fbedf5, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_pyx_unpickle_StrField); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_6, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_16510453);
    __Pyx_GIVEREF(__pyx_int_16510453);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_int_16510453);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_6, 2, Py_None);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_6);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_state);
    __pyx_t_5 = 0;
    __pyx_t_6 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.allow_blank is not None or self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.max_length is not None or self.min_length is not None or self.root is not None or self.style is not None or self.trim_whitespace is not None or self.validators is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_StrField, (type(self), 0x0fbedf5, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_StrField, (type(self), 0x0fbedf5, None), state
 *     else:
 *         return __pyx_unpickle_StrField, (type(self), 0x0fbedf5, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_StrField__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_pyx_unpickle_StrField); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_6, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_16510453);
    __Pyx_GIVEREF(__pyx_int_16510453);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_int_16510453);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_state);
    __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_6);
    __pyx_t_4 = 0;
    __pyx_t_6 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("drf_turbo.fields.StrField.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_StrField, (type(self), 0x0fbedf5, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_StrField__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_8StrField_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_8StrField_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_8StrField_8__setstate_cython__(((struct __pyx_obj_9drf_turbo_6fields_StrField *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_8StrField_8__setstate_cython__(struct __pyx_obj_9drf_turbo_6fields_StrField *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_StrField, (type(self), 0x0fbedf5, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_StrField__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields___pyx_unpickle_StrField__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_StrField, (type(self), 0x0fbedf5, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_StrField__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.StrField.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":348
 *         'invalid': 'Enter a valid email address.'
 *     }
 *     def __init__(self, **kwargs):             # <<<<<<<<<<<<<<
 *         self.to_lower = kwargs.pop('to_lower', False)
 *         super().__init__(**kwargs)
 */

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_10EmailField_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_10EmailField_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_kwargs = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  if (unlikely(PyTuple_GET_SIZE(__pyx_args) > 0)) {
    __Pyx_RaiseArgtupleInvalid("__init__", 1, 0, 0, PyTuple_GET_SIZE(__pyx_args)); return -1;}
  if (__pyx_kwds && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "__init__", 1))) return -1;
  __pyx_v_kwargs = (__pyx_kwds) ? PyDict_Copy(__pyx_kwds) : PyDict_New(); if (unlikely(!__pyx_v_kwargs)) return -1;
  __Pyx_GOTREF(__pyx_v_kwargs);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_10EmailField___init__(((struct __pyx_obj_9drf_turbo_6fields_EmailField *)__pyx_v_self), __pyx_v_kwargs);

  /* function exit code */
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_10EmailField___init__(struct __pyx_obj_9drf_turbo_6fields_EmailField *__pyx_v_self, PyObject *__pyx_v_kwargs) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "drf_turbo/fields.pyx":349
 *     }
 *     def __init__(self, **kwargs):
 *         self.to_lower = kwargs.pop('to_lower', False)             # <<<<<<<<<<<<<<
 *         super().__init__(**kwargs)
 * 
 */
  __pyx_t_1 = __Pyx_PyDict_Pop(__pyx_v_kwargs, __pyx_n_s_to_lower, Py_False); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 349, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 349, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_self->to_lower = __pyx_t_2;

  /* "drf_turbo/fields.pyx":350
 *     def __init__(self, **kwargs):
 *         self.to_lower = kwargs.pop('to_lower', False)
 *         super().__init__(**kwargs)             # <<<<<<<<<<<<<<
 * 
 *     cpdef inline serialize(self,value,dict context):
 */
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 350, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_9drf_turbo_6fields_EmailField));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_9drf_turbo_6fields_EmailField));
  PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)__pyx_ptype_9drf_turbo_6fields_EmailField));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_1, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_1, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 350, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_init); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 350, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyDict_Copy(__pyx_v_kwargs); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 350, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 350, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "drf_turbo/fields.pyx":348
 *         'invalid': 'Enter a valid email address.'
 *     }
 *     def __init__(self, **kwargs):             # <<<<<<<<<<<<<<
 *         self.to_lower = kwargs.pop('to_lower', False)
 *         super().__init__(**kwargs)
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("drf_turbo.fields.EmailField.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":352
 *         super().__init__(**kwargs)
 * 
 *     cpdef inline serialize(self,value,dict context):             # <<<<<<<<<<<<<<
 *         if self.to_lower :
 *             return value.lower()
 */

static PyObject *__pyx_pw_9drf_turbo_6fields_10EmailField_3serialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_10EmailField_serialize(struct __pyx_obj_9drf_turbo_6fields_EmailField *__pyx_v_self, PyObject *__pyx_v_value, CYTHON_UNUSED PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("serialize", 0);

  /* "drf_turbo/fields.pyx":353
 * 
 *     cpdef inline serialize(self,value,dict context):
 *         if self.to_lower :             # <<<<<<<<<<<<<<
 *             return value.lower()
 *         return value
 */
  __pyx_t_1 = (__pyx_v_self->to_lower != 0);
  if (__pyx_t_1) {

    /* "drf_turbo/fields.pyx":354
 *     cpdef inline serialize(self,value,dict context):
 *         if self.to_lower :
 *             return value.lower()             # <<<<<<<<<<<<<<
 *         return value
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_value, __pyx_n_s_lower); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 354, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 354, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "drf_turbo/fields.pyx":353
 * 
 *     cpdef inline serialize(self,value,dict context):
 *         if self.to_lower :             # <<<<<<<<<<<<<<
 *             return value.lower()
 *         return value
 */
  }

  /* "drf_turbo/fields.pyx":355
 *         if self.to_lower :
 *             return value.lower()
 *         return value             # <<<<<<<<<<<<<<
 * 
 *     cpdef inline deserialize(self,data,dict context):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_value);
  __pyx_r = __pyx_v_value;
  goto __pyx_L0;

  /* "drf_turbo/fields.pyx":352
 *         super().__init__(**kwargs)
 * 
 *     cpdef inline serialize(self,value,dict context):             # <<<<<<<<<<<<<<
 *         if self.to_lower :
 *             return value.lower()
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("drf_turbo.fields.EmailField.serialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_10EmailField_3serialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_10EmailField_3serialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_value = 0;
  PyObject *__pyx_v_context = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("serialize (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_value,&__pyx_n_s_context,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_value)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_context)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("serialize", 1, 2, 2, 1); __PYX_ERR(0, 352, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "serialize") < 0)) __PYX_ERR(0, 352, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_value = values[0];
    __pyx_v_context = ((PyObject*)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("serialize", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 352, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.EmailField.serialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_context), (&PyDict_Type), 1, "context", 1))) __PYX_ERR(0, 352, __pyx_L1_error)
  __pyx_r = __pyx_pf_9drf_turbo_6fields_10EmailField_2serialize(((struct __pyx_obj_9drf_turbo_6fields_EmailField *)__pyx_v_self), __pyx_v_value, __pyx_v_context);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_10EmailField_2serialize(struct __pyx_obj_9drf_turbo_6fields_EmailField *__pyx_v_self, PyObject *__pyx_v_value, PyObject *__pyx_v_context) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("serialize", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields_10EmailField_serialize(__pyx_v_self, __pyx_v_value, __pyx_v_context, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 352, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.EmailField.serialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":357
 *         return value
 * 
 *     cpdef inline deserialize(self,data,dict context):             # <<<<<<<<<<<<<<
 *         match = EMAIL_REGEX.match(data)
 *         if not match:
 */

static PyObject *__pyx_pw_9drf_turbo_6fields_10EmailField_5deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_10EmailField_deserialize(struct __pyx_obj_9drf_turbo_6fields_EmailField *__pyx_v_self, PyObject *__pyx_v_data, CYTHON_UNUSED PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_v_match = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("deserialize", 0);
  __Pyx_INCREF(__pyx_v_data);

  /* "drf_turbo/fields.pyx":358
 * 
 *     cpdef inline deserialize(self,data,dict context):
 *         match = EMAIL_REGEX.match(data)             # <<<<<<<<<<<<<<
 *         if not match:
 *             raise self.raise_if_fail("invalid")
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_EMAIL_REGEX); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 358, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_match); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 358, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_2, __pyx_v_data) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_data);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 358, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_match = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "drf_turbo/fields.pyx":359
 *     cpdef inline deserialize(self,data,dict context):
 *         match = EMAIL_REGEX.match(data)
 *         if not match:             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail("invalid")
 *         data = str(data)
 */
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_v_match); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 359, __pyx_L1_error)
  __pyx_t_5 = ((!__pyx_t_4) != 0);
  if (unlikely(__pyx_t_5)) {

    /* "drf_turbo/fields.pyx":360
 *         match = EMAIL_REGEX.match(data)
 *         if not match:
 *             raise self.raise_if_fail("invalid")             # <<<<<<<<<<<<<<
 *         data = str(data)
 *         if self.to_lower :
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_raise_if_fail); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 360, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_2)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_2, __pyx_n_s_invalid) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_n_s_invalid);
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 360, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 360, __pyx_L1_error)

    /* "drf_turbo/fields.pyx":359
 *     cpdef inline deserialize(self,data,dict context):
 *         match = EMAIL_REGEX.match(data)
 *         if not match:             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail("invalid")
 *         data = str(data)
 */
  }

  /* "drf_turbo/fields.pyx":361
 *         if not match:
 *             raise self.raise_if_fail("invalid")
 *         data = str(data)             # <<<<<<<<<<<<<<
 *         if self.to_lower :
 *             return data.lower()
 */
  __pyx_t_1 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), __pyx_v_data); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 361, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF_SET(__pyx_v_data, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "drf_turbo/fields.pyx":362
 *             raise self.raise_if_fail("invalid")
 *         data = str(data)
 *         if self.to_lower :             # <<<<<<<<<<<<<<
 *             return data.lower()
 *         return data
 */
  __pyx_t_5 = (__pyx_v_self->to_lower != 0);
  if (__pyx_t_5) {

    /* "drf_turbo/fields.pyx":363
 *         data = str(data)
 *         if self.to_lower :
 *             return data.lower()             # <<<<<<<<<<<<<<
 *         return data
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_data, __pyx_n_s_lower); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 363, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_2)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 363, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "drf_turbo/fields.pyx":362
 *             raise self.raise_if_fail("invalid")
 *         data = str(data)
 *         if self.to_lower :             # <<<<<<<<<<<<<<
 *             return data.lower()
 *         return data
 */
  }

  /* "drf_turbo/fields.pyx":364
 *         if self.to_lower :
 *             return data.lower()
 *         return data             # <<<<<<<<<<<<<<
 * 
 * @cython.final
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_data);
  __pyx_r = __pyx_v_data;
  goto __pyx_L0;

  /* "drf_turbo/fields.pyx":357
 *         return value
 * 
 *     cpdef inline deserialize(self,data,dict context):             # <<<<<<<<<<<<<<
 *         match = EMAIL_REGEX.match(data)
 *         if not match:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("drf_turbo.fields.EmailField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_match);
  __Pyx_XDECREF(__pyx_v_data);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_10EmailField_5deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_10EmailField_5deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_data = 0;
  PyObject *__pyx_v_context = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("deserialize (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_data,&__pyx_n_s_context,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_data)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_context)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("deserialize", 1, 2, 2, 1); __PYX_ERR(0, 357, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "deserialize") < 0)) __PYX_ERR(0, 357, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_data = values[0];
    __pyx_v_context = ((PyObject*)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("deserialize", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 357, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.EmailField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_context), (&PyDict_Type), 1, "context", 1))) __PYX_ERR(0, 357, __pyx_L1_error)
  __pyx_r = __pyx_pf_9drf_turbo_6fields_10EmailField_4deserialize(((struct __pyx_obj_9drf_turbo_6fields_EmailField *)__pyx_v_self), __pyx_v_data, __pyx_v_context);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_10EmailField_4deserialize(struct __pyx_obj_9drf_turbo_6fields_EmailField *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("deserialize", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields_10EmailField_deserialize(__pyx_v_self, __pyx_v_data, __pyx_v_context, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 357, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.EmailField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pxd":46
 * cdef class EmailField(StrField):
 *     cdef public:
 *         bint to_lower             # <<<<<<<<<<<<<<
 * 
 * cdef class URLField(StrField):
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_10EmailField_8to_lower_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_10EmailField_8to_lower_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_10EmailField_8to_lower___get__(((struct __pyx_obj_9drf_turbo_6fields_EmailField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_10EmailField_8to_lower___get__(struct __pyx_obj_9drf_turbo_6fields_EmailField *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->to_lower); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 46, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.EmailField.to_lower.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_10EmailField_8to_lower_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_10EmailField_8to_lower_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_10EmailField_8to_lower_2__set__(((struct __pyx_obj_9drf_turbo_6fields_EmailField *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_10EmailField_8to_lower_2__set__(struct __pyx_obj_9drf_turbo_6fields_EmailField *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 46, __pyx_L1_error)
  __pyx_v_self->to_lower = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("drf_turbo.fields.EmailField.to_lower.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_10EmailField_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_10EmailField_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_10EmailField_6__reduce_cython__(((struct __pyx_obj_9drf_turbo_6fields_EmailField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_10EmailField_6__reduce_cython__(struct __pyx_obj_9drf_turbo_6fields_EmailField *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  int __pyx_t_9;
  int __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self.allow_blank, self.allow_null, self.attr, self.attrs, self.call, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.max_length, self.min_length, self.read_only, self.required, self.root, self.style, self.to_lower, self.trim_whitespace, self.validators, self.write_only)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base.allow_null); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base.call); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base.read_only); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base.required); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyBool_FromLong(__pyx_v_self->to_lower); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base.write_only); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = PyTuple_New(21); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.allow_blank);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.allow_blank);
  PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_v_self->__pyx_base.allow_blank);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_7, 1, __pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.attr);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.attr);
  PyTuple_SET_ITEM(__pyx_t_7, 2, __pyx_v_self->__pyx_base.__pyx_base.attr);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.attrs);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.attrs);
  PyTuple_SET_ITEM(__pyx_t_7, 3, __pyx_v_self->__pyx_base.__pyx_base.attrs);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_7, 4, __pyx_t_2);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.default_value);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.default_value);
  PyTuple_SET_ITEM(__pyx_t_7, 5, __pyx_v_self->__pyx_base.__pyx_base.default_value);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.error_messages);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.error_messages);
  PyTuple_SET_ITEM(__pyx_t_7, 6, __pyx_v_self->__pyx_base.__pyx_base.error_messages);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.field_name);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.field_name);
  PyTuple_SET_ITEM(__pyx_t_7, 7, __pyx_v_self->__pyx_base.__pyx_base.field_name);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.help_text);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.help_text);
  PyTuple_SET_ITEM(__pyx_t_7, 8, __pyx_v_self->__pyx_base.__pyx_base.help_text);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.initial);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.initial);
  PyTuple_SET_ITEM(__pyx_t_7, 9, __pyx_v_self->__pyx_base.__pyx_base.initial);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.label);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.label);
  PyTuple_SET_ITEM(__pyx_t_7, 10, __pyx_v_self->__pyx_base.__pyx_base.label);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.max_length);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.max_length);
  PyTuple_SET_ITEM(__pyx_t_7, 11, __pyx_v_self->__pyx_base.max_length);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.min_length);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.min_length);
  PyTuple_SET_ITEM(__pyx_t_7, 12, __pyx_v_self->__pyx_base.min_length);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_7, 13, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_7, 14, __pyx_t_4);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.root);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.root);
  PyTuple_SET_ITEM(__pyx_t_7, 15, __pyx_v_self->__pyx_base.__pyx_base.root);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.style);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.style);
  PyTuple_SET_ITEM(__pyx_t_7, 16, __pyx_v_self->__pyx_base.__pyx_base.style);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_7, 17, __pyx_t_5);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.trim_whitespace);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.trim_whitespace);
  PyTuple_SET_ITEM(__pyx_t_7, 18, __pyx_v_self->__pyx_base.trim_whitespace);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.validators);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.validators);
  PyTuple_SET_ITEM(__pyx_t_7, 19, __pyx_v_self->__pyx_base.__pyx_base.validators);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_7, 20, __pyx_t_6);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_t_6 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_7);
  __pyx_t_7 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self.allow_blank, self.allow_null, self.attr, self.attrs, self.call, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.max_length, self.min_length, self.read_only, self.required, self.root, self.style, self.to_lower, self.trim_whitespace, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_7 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_v__dict = __pyx_t_7;
  __pyx_t_7 = 0;

  /* "(tree fragment)":7
 *     state = (self.allow_blank, self.allow_null, self.attr, self.attrs, self.call, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.max_length, self.min_length, self.read_only, self.required, self.root, self.style, self.to_lower, self.trim_whitespace, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_8 = (__pyx_v__dict != Py_None);
  __pyx_t_9 = (__pyx_t_8 != 0);
  if (__pyx_t_9) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_7 = PyTuple_New(1); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_v__dict);
    __pyx_t_6 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_7); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_6));
    __pyx_t_6 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.allow_blank is not None or self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.max_length is not None or self.min_length is not None or self.root is not None or self.style is not None or self.trim_whitespace is not None or self.validators is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self.allow_blank, self.allow_null, self.attr, self.attrs, self.call, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.max_length, self.min_length, self.read_only, self.required, self.root, self.style, self.to_lower, self.trim_whitespace, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self.allow_blank is not None or self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.max_length is not None or self.min_length is not None or self.root is not None or self.style is not None or self.trim_whitespace is not None or self.validators is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_EmailField, (type(self), 0xe1e0ed9, None), state
 */
  /*else*/ {
    __pyx_t_8 = (__pyx_v_self->__pyx_base.allow_blank != Py_None);
    __pyx_t_10 = (__pyx_t_8 != 0);
    if (!__pyx_t_10) {
    } else {
      __pyx_t_9 = __pyx_t_10;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_10 = (__pyx_v_self->__pyx_base.__pyx_base.attr != ((PyObject*)Py_None));
    __pyx_t_8 = (__pyx_t_10 != 0);
    if (!__pyx_t_8) {
    } else {
      __pyx_t_9 = __pyx_t_8;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_8 = (__pyx_v_self->__pyx_base.__pyx_base.attrs != ((PyObject*)Py_None));
    __pyx_t_10 = (__pyx_t_8 != 0);
    if (!__pyx_t_10) {
    } else {
      __pyx_t_9 = __pyx_t_10;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_10 = (__pyx_v_self->__pyx_base.__pyx_base.default_value != Py_None);
    __pyx_t_8 = (__pyx_t_10 != 0);
    if (!__pyx_t_8) {
    } else {
      __pyx_t_9 = __pyx_t_8;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_8 = (__pyx_v_self->__pyx_base.__pyx_base.error_messages != ((PyObject*)Py_None));
    __pyx_t_10 = (__pyx_t_8 != 0);
    if (!__pyx_t_10) {
    } else {
      __pyx_t_9 = __pyx_t_10;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_10 = (__pyx_v_self->__pyx_base.__pyx_base.field_name != ((PyObject*)Py_None));
    __pyx_t_8 = (__pyx_t_10 != 0);
    if (!__pyx_t_8) {
    } else {
      __pyx_t_9 = __pyx_t_8;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_8 = (__pyx_v_self->__pyx_base.__pyx_base.help_text != ((PyObject*)Py_None));
    __pyx_t_10 = (__pyx_t_8 != 0);
    if (!__pyx_t_10) {
    } else {
      __pyx_t_9 = __pyx_t_10;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_10 = (__pyx_v_self->__pyx_base.__pyx_base.initial != Py_None);
    __pyx_t_8 = (__pyx_t_10 != 0);
    if (!__pyx_t_8) {
    } else {
      __pyx_t_9 = __pyx_t_8;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_8 = (__pyx_v_self->__pyx_base.__pyx_base.label != ((PyObject*)Py_None));
    __pyx_t_10 = (__pyx_t_8 != 0);
    if (!__pyx_t_10) {
    } else {
      __pyx_t_9 = __pyx_t_10;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_10 = (__pyx_v_self->__pyx_base.max_length != Py_None);
    __pyx_t_8 = (__pyx_t_10 != 0);
    if (!__pyx_t_8) {
    } else {
      __pyx_t_9 = __pyx_t_8;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_8 = (__pyx_v_self->__pyx_base.min_length != Py_None);
    __pyx_t_10 = (__pyx_t_8 != 0);
    if (!__pyx_t_10) {
    } else {
      __pyx_t_9 = __pyx_t_10;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_10 = (__pyx_v_self->__pyx_base.__pyx_base.root != Py_None);
    __pyx_t_8 = (__pyx_t_10 != 0);
    if (!__pyx_t_8) {
    } else {
      __pyx_t_9 = __pyx_t_8;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_8 = (__pyx_v_self->__pyx_base.__pyx_base.style != ((PyObject*)Py_None));
    __pyx_t_10 = (__pyx_t_8 != 0);
    if (!__pyx_t_10) {
    } else {
      __pyx_t_9 = __pyx_t_10;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_10 = (__pyx_v_self->__pyx_base.trim_whitespace != Py_None);
    __pyx_t_8 = (__pyx_t_10 != 0);
    if (!__pyx_t_8) {
    } else {
      __pyx_t_9 = __pyx_t_8;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_8 = (__pyx_v_self->__pyx_base.__pyx_base.validators != Py_None);
    __pyx_t_10 = (__pyx_t_8 != 0);
    __pyx_t_9 = __pyx_t_10;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_9;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.allow_blank is not None or self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.max_length is not None or self.min_length is not None or self.root is not None or self.style is not None or self.trim_whitespace is not None or self.validators is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_EmailField, (type(self), 0xe1e0ed9, None), state
 *     else:
 */
  __pyx_t_9 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_9) {

    /* "(tree fragment)":13
 *         use_setstate = self.allow_blank is not None or self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.max_length is not None or self.min_length is not None or self.root is not None or self.style is not None or self.trim_whitespace is not None or self.validators is not None
 *     if use_setstate:
 *         return __pyx_unpickle_EmailField, (type(self), 0xe1e0ed9, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_EmailField, (type(self), 0xe1e0ed9, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_n_s_pyx_unpickle_EmailField); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = PyTuple_New(3); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_7, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_236850905);
    __Pyx_GIVEREF(__pyx_int_236850905);
    PyTuple_SET_ITEM(__pyx_t_7, 1, __pyx_int_236850905);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_7, 2, Py_None);
    __pyx_t_5 = PyTuple_New(3); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_7);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_7);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_v_state);
    __pyx_t_6 = 0;
    __pyx_t_7 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.allow_blank is not None or self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.max_length is not None or self.min_length is not None or self.root is not None or self.style is not None or self.trim_whitespace is not None or self.validators is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_EmailField, (type(self), 0xe1e0ed9, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_EmailField, (type(self), 0xe1e0ed9, None), state
 *     else:
 *         return __pyx_unpickle_EmailField, (type(self), 0xe1e0ed9, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_EmailField__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_pyx_unpickle_EmailField); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_7 = PyTuple_New(3); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_7, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_236850905);
    __Pyx_GIVEREF(__pyx_int_236850905);
    PyTuple_SET_ITEM(__pyx_t_7, 1, __pyx_int_236850905);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_7, 2, __pyx_v_state);
    __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_7);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_7);
    __pyx_t_5 = 0;
    __pyx_t_7 = 0;
    __pyx_r = __pyx_t_6;
    __pyx_t_6 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("drf_turbo.fields.EmailField.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_EmailField, (type(self), 0xe1e0ed9, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_EmailField__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_10EmailField_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_10EmailField_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_10EmailField_8__setstate_cython__(((struct __pyx_obj_9drf_turbo_6fields_EmailField *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_10EmailField_8__setstate_cython__(struct __pyx_obj_9drf_turbo_6fields_EmailField *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_EmailField, (type(self), 0xe1e0ed9, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_EmailField__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields___pyx_unpickle_EmailField__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_EmailField, (type(self), 0xe1e0ed9, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_EmailField__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.EmailField.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":378
 *     }
 * 
 *     cpdef inline deserialize(self,data,dict context):             # <<<<<<<<<<<<<<
 *         match = URL_REGEX.match(data)
 *         if not match:
 */

static PyObject *__pyx_pw_9drf_turbo_6fields_8URLField_1deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_8URLField_deserialize(struct __pyx_obj_9drf_turbo_6fields_URLField *__pyx_v_self, PyObject *__pyx_v_data, CYTHON_UNUSED PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_v_match = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("deserialize", 0);
  __Pyx_INCREF(__pyx_v_data);

  /* "drf_turbo/fields.pyx":379
 * 
 *     cpdef inline deserialize(self,data,dict context):
 *         match = URL_REGEX.match(data)             # <<<<<<<<<<<<<<
 *         if not match:
 *             raise self.raise_if_fail("invalid")
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_URL_REGEX); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 379, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_match); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 379, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_2, __pyx_v_data) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_data);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 379, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_match = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "drf_turbo/fields.pyx":380
 *     cpdef inline deserialize(self,data,dict context):
 *         match = URL_REGEX.match(data)
 *         if not match:             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail("invalid")
 *         data = str(data)
 */
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_v_match); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 380, __pyx_L1_error)
  __pyx_t_5 = ((!__pyx_t_4) != 0);
  if (unlikely(__pyx_t_5)) {

    /* "drf_turbo/fields.pyx":381
 *         match = URL_REGEX.match(data)
 *         if not match:
 *             raise self.raise_if_fail("invalid")             # <<<<<<<<<<<<<<
 *         data = str(data)
 *         return data
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_raise_if_fail); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 381, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_2)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_2, __pyx_n_s_invalid) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_n_s_invalid);
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 381, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 381, __pyx_L1_error)

    /* "drf_turbo/fields.pyx":380
 *     cpdef inline deserialize(self,data,dict context):
 *         match = URL_REGEX.match(data)
 *         if not match:             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail("invalid")
 *         data = str(data)
 */
  }

  /* "drf_turbo/fields.pyx":382
 *         if not match:
 *             raise self.raise_if_fail("invalid")
 *         data = str(data)             # <<<<<<<<<<<<<<
 *         return data
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), __pyx_v_data); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 382, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF_SET(__pyx_v_data, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "drf_turbo/fields.pyx":383
 *             raise self.raise_if_fail("invalid")
 *         data = str(data)
 *         return data             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_data);
  __pyx_r = __pyx_v_data;
  goto __pyx_L0;

  /* "drf_turbo/fields.pyx":378
 *     }
 * 
 *     cpdef inline deserialize(self,data,dict context):             # <<<<<<<<<<<<<<
 *         match = URL_REGEX.match(data)
 *         if not match:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("drf_turbo.fields.URLField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_match);
  __Pyx_XDECREF(__pyx_v_data);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_8URLField_1deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_8URLField_1deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_data = 0;
  PyObject *__pyx_v_context = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("deserialize (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_data,&__pyx_n_s_context,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_data)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_context)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("deserialize", 1, 2, 2, 1); __PYX_ERR(0, 378, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "deserialize") < 0)) __PYX_ERR(0, 378, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_data = values[0];
    __pyx_v_context = ((PyObject*)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("deserialize", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 378, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.URLField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_context), (&PyDict_Type), 1, "context", 1))) __PYX_ERR(0, 378, __pyx_L1_error)
  __pyx_r = __pyx_pf_9drf_turbo_6fields_8URLField_deserialize(((struct __pyx_obj_9drf_turbo_6fields_URLField *)__pyx_v_self), __pyx_v_data, __pyx_v_context);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_8URLField_deserialize(struct __pyx_obj_9drf_turbo_6fields_URLField *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("deserialize", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields_8URLField_deserialize(__pyx_v_self, __pyx_v_data, __pyx_v_context, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 378, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.URLField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_8URLField_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_8URLField_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_8URLField_2__reduce_cython__(((struct __pyx_obj_9drf_turbo_6fields_URLField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_8URLField_2__reduce_cython__(struct __pyx_obj_9drf_turbo_6fields_URLField *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self.allow_blank, self.allow_null, self.attr, self.attrs, self.call, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.max_length, self.min_length, self.read_only, self.required, self.root, self.style, self.trim_whitespace, self.validators, self.write_only)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base.allow_null); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base.call); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base.read_only); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base.required); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base.write_only); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = PyTuple_New(20); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.allow_blank);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.allow_blank);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v_self->__pyx_base.allow_blank);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.attr);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.attr);
  PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_self->__pyx_base.__pyx_base.attr);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.attrs);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.attrs);
  PyTuple_SET_ITEM(__pyx_t_6, 3, __pyx_v_self->__pyx_base.__pyx_base.attrs);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_6, 4, __pyx_t_2);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.default_value);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.default_value);
  PyTuple_SET_ITEM(__pyx_t_6, 5, __pyx_v_self->__pyx_base.__pyx_base.default_value);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.error_messages);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.error_messages);
  PyTuple_SET_ITEM(__pyx_t_6, 6, __pyx_v_self->__pyx_base.__pyx_base.error_messages);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.field_name);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.field_name);
  PyTuple_SET_ITEM(__pyx_t_6, 7, __pyx_v_self->__pyx_base.__pyx_base.field_name);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.help_text);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.help_text);
  PyTuple_SET_ITEM(__pyx_t_6, 8, __pyx_v_self->__pyx_base.__pyx_base.help_text);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.initial);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.initial);
  PyTuple_SET_ITEM(__pyx_t_6, 9, __pyx_v_self->__pyx_base.__pyx_base.initial);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.label);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.label);
  PyTuple_SET_ITEM(__pyx_t_6, 10, __pyx_v_self->__pyx_base.__pyx_base.label);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.max_length);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.max_length);
  PyTuple_SET_ITEM(__pyx_t_6, 11, __pyx_v_self->__pyx_base.max_length);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.min_length);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.min_length);
  PyTuple_SET_ITEM(__pyx_t_6, 12, __pyx_v_self->__pyx_base.min_length);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 13, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_6, 14, __pyx_t_4);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.root);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.root);
  PyTuple_SET_ITEM(__pyx_t_6, 15, __pyx_v_self->__pyx_base.__pyx_base.root);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.style);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.style);
  PyTuple_SET_ITEM(__pyx_t_6, 16, __pyx_v_self->__pyx_base.__pyx_base.style);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.trim_whitespace);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.trim_whitespace);
  PyTuple_SET_ITEM(__pyx_t_6, 17, __pyx_v_self->__pyx_base.trim_whitespace);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.validators);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.validators);
  PyTuple_SET_ITEM(__pyx_t_6, 18, __pyx_v_self->__pyx_base.__pyx_base.validators);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_6, 19, __pyx_t_5);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_6);
  __pyx_t_6 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self.allow_blank, self.allow_null, self.attr, self.attrs, self.call, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.max_length, self.min_length, self.read_only, self.required, self.root, self.style, self.trim_whitespace, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_6 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_v__dict = __pyx_t_6;
  __pyx_t_6 = 0;

  /* "(tree fragment)":7
 *     state = (self.allow_blank, self.allow_null, self.attr, self.attrs, self.call, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.max_length, self.min_length, self.read_only, self.required, self.root, self.style, self.trim_whitespace, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_7 = (__pyx_v__dict != Py_None);
  __pyx_t_8 = (__pyx_t_7 != 0);
  if (__pyx_t_8) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_6 = PyTuple_New(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v__dict);
    __pyx_t_5 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_6); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_5));
    __pyx_t_5 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.allow_blank is not None or self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.max_length is not None or self.min_length is not None or self.root is not None or self.style is not None or self.trim_whitespace is not None or self.validators is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self.allow_blank, self.allow_null, self.attr, self.attrs, self.call, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.max_length, self.min_length, self.read_only, self.required, self.root, self.style, self.trim_whitespace, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self.allow_blank is not None or self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.max_length is not None or self.min_length is not None or self.root is not None or self.style is not None or self.trim_whitespace is not None or self.validators is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_URLField, (type(self), 0x0fbedf5, None), state
 */
  /*else*/ {
    __pyx_t_7 = (__pyx_v_self->__pyx_base.allow_blank != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.__pyx_base.attr != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.__pyx_base.attrs != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.__pyx_base.default_value != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.__pyx_base.error_messages != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.__pyx_base.field_name != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.__pyx_base.help_text != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.__pyx_base.initial != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.__pyx_base.label != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.max_length != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.min_length != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.__pyx_base.root != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.__pyx_base.style != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.trim_whitespace != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.__pyx_base.validators != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    __pyx_t_8 = __pyx_t_9;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_8;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.allow_blank is not None or self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.max_length is not None or self.min_length is not None or self.root is not None or self.style is not None or self.trim_whitespace is not None or self.validators is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_URLField, (type(self), 0x0fbedf5, None), state
 *     else:
 */
  __pyx_t_8 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_8) {

    /* "(tree fragment)":13
 *         use_setstate = self.allow_blank is not None or self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.max_length is not None or self.min_length is not None or self.root is not None or self.style is not None or self.trim_whitespace is not None or self.validators is not None
 *     if use_setstate:
 *         return __pyx_unpickle_URLField, (type(self), 0x0fbedf5, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_URLField, (type(self), 0x0fbedf5, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_pyx_unpickle_URLField); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_6, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_16510453);
    __Pyx_GIVEREF(__pyx_int_16510453);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_int_16510453);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_6, 2, Py_None);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_6);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_state);
    __pyx_t_5 = 0;
    __pyx_t_6 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.allow_blank is not None or self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.max_length is not None or self.min_length is not None or self.root is not None or self.style is not None or self.trim_whitespace is not None or self.validators is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_URLField, (type(self), 0x0fbedf5, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_URLField, (type(self), 0x0fbedf5, None), state
 *     else:
 *         return __pyx_unpickle_URLField, (type(self), 0x0fbedf5, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_URLField__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_pyx_unpickle_URLField); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_6, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_16510453);
    __Pyx_GIVEREF(__pyx_int_16510453);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_int_16510453);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_state);
    __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_6);
    __pyx_t_4 = 0;
    __pyx_t_6 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("drf_turbo.fields.URLField.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_URLField, (type(self), 0x0fbedf5, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_URLField__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_8URLField_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_8URLField_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_8URLField_4__setstate_cython__(((struct __pyx_obj_9drf_turbo_6fields_URLField *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_8URLField_4__setstate_cython__(struct __pyx_obj_9drf_turbo_6fields_URLField *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_URLField, (type(self), 0x0fbedf5, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_URLField__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields___pyx_unpickle_URLField__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_URLField, (type(self), 0x0fbedf5, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_URLField__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.URLField.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":400
 *     }
 * 
 *     def __init__(self, regex, **kwargs):             # <<<<<<<<<<<<<<
 *         self.regex = regex
 *         super().__init__(**kwargs)
 */

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_10RegexField_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_10RegexField_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_regex = 0;
  PyObject *__pyx_v_kwargs = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  __pyx_v_kwargs = PyDict_New(); if (unlikely(!__pyx_v_kwargs)) return -1;
  __Pyx_GOTREF(__pyx_v_kwargs);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_regex,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_regex)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, __pyx_v_kwargs, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 400, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_regex = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 400, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_DECREF(__pyx_v_kwargs); __pyx_v_kwargs = 0;
  __Pyx_AddTraceback("drf_turbo.fields.RegexField.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9drf_turbo_6fields_10RegexField___init__(((struct __pyx_obj_9drf_turbo_6fields_RegexField *)__pyx_v_self), __pyx_v_regex, __pyx_v_kwargs);

  /* function exit code */
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_10RegexField___init__(struct __pyx_obj_9drf_turbo_6fields_RegexField *__pyx_v_self, PyObject *__pyx_v_regex, PyObject *__pyx_v_kwargs) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "drf_turbo/fields.pyx":401
 * 
 *     def __init__(self, regex, **kwargs):
 *         self.regex = regex             # <<<<<<<<<<<<<<
 *         super().__init__(**kwargs)
 * 
 */
  __Pyx_INCREF(__pyx_v_regex);
  __Pyx_GIVEREF(__pyx_v_regex);
  __Pyx_GOTREF(__pyx_v_self->regex);
  __Pyx_DECREF(__pyx_v_self->regex);
  __pyx_v_self->regex = __pyx_v_regex;

  /* "drf_turbo/fields.pyx":402
 *     def __init__(self, regex, **kwargs):
 *         self.regex = regex
 *         super().__init__(**kwargs)             # <<<<<<<<<<<<<<
 * 
 *     cpdef inline deserialize(self,data,dict context):
 */
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 402, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_9drf_turbo_6fields_RegexField));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_9drf_turbo_6fields_RegexField));
  PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)__pyx_ptype_9drf_turbo_6fields_RegexField));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_1, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 402, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_init); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 402, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyDict_Copy(__pyx_v_kwargs); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 402, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 402, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "drf_turbo/fields.pyx":400
 *     }
 * 
 *     def __init__(self, regex, **kwargs):             # <<<<<<<<<<<<<<
 *         self.regex = regex
 *         super().__init__(**kwargs)
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("drf_turbo.fields.RegexField.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":404
 *         super().__init__(**kwargs)
 * 
 *     cpdef inline deserialize(self,data,dict context):             # <<<<<<<<<<<<<<
 *         compiled = re.compile(self.regex,flags=0)
 *         match = compiled.search(str(data))
 */

static PyObject *__pyx_pw_9drf_turbo_6fields_10RegexField_3deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_10RegexField_deserialize(struct __pyx_obj_9drf_turbo_6fields_RegexField *__pyx_v_self, PyObject *__pyx_v_data, CYTHON_UNUSED PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_v_compiled = NULL;
  PyObject *__pyx_v_match = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("deserialize", 0);

  /* "drf_turbo/fields.pyx":405
 * 
 *     cpdef inline deserialize(self,data,dict context):
 *         compiled = re.compile(self.regex,flags=0)             # <<<<<<<<<<<<<<
 *         match = compiled.search(str(data))
 *         if not match:
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_re); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 405, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_compile); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 405, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 405, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->regex);
  __Pyx_GIVEREF(__pyx_v_self->regex);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_self->regex);
  __pyx_t_3 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 405, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_flags, __pyx_int_0) < 0) __PYX_ERR(0, 405, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_1, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 405, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_compiled = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "drf_turbo/fields.pyx":406
 *     cpdef inline deserialize(self,data,dict context):
 *         compiled = re.compile(self.regex,flags=0)
 *         match = compiled.search(str(data))             # <<<<<<<<<<<<<<
 *         if not match:
 *             raise self.raise_if_fail("invalid")
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_compiled, __pyx_n_s_search); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 406, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), __pyx_v_data); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 406, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_4 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_2, __pyx_t_1) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 406, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_match = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "drf_turbo/fields.pyx":407
 *         compiled = re.compile(self.regex,flags=0)
 *         match = compiled.search(str(data))
 *         if not match:             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail("invalid")
 *         return str(data)
 */
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_v_match); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 407, __pyx_L1_error)
  __pyx_t_6 = ((!__pyx_t_5) != 0);
  if (unlikely(__pyx_t_6)) {

    /* "drf_turbo/fields.pyx":408
 *         match = compiled.search(str(data))
 *         if not match:
 *             raise self.raise_if_fail("invalid")             # <<<<<<<<<<<<<<
 *         return str(data)
 * 
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_raise_if_fail); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 408, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_1 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_1)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_4 = (__pyx_t_1) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_1, __pyx_n_s_invalid) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_n_s_invalid);
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 408, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(0, 408, __pyx_L1_error)

    /* "drf_turbo/fields.pyx":407
 *         compiled = re.compile(self.regex,flags=0)
 *         match = compiled.search(str(data))
 *         if not match:             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail("invalid")
 *         return str(data)
 */
  }

  /* "drf_turbo/fields.pyx":409
 *         if not match:
 *             raise self.raise_if_fail("invalid")
 *         return str(data)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_4 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), __pyx_v_data); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 409, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "drf_turbo/fields.pyx":404
 *         super().__init__(**kwargs)
 * 
 *     cpdef inline deserialize(self,data,dict context):             # <<<<<<<<<<<<<<
 *         compiled = re.compile(self.regex,flags=0)
 *         match = compiled.search(str(data))
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("drf_turbo.fields.RegexField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_compiled);
  __Pyx_XDECREF(__pyx_v_match);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_10RegexField_3deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_10RegexField_3deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_data = 0;
  PyObject *__pyx_v_context = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("deserialize (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_data,&__pyx_n_s_context,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_data)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_context)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("deserialize", 1, 2, 2, 1); __PYX_ERR(0, 404, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "deserialize") < 0)) __PYX_ERR(0, 404, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_data = values[0];
    __pyx_v_context = ((PyObject*)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("deserialize", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 404, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.RegexField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_context), (&PyDict_Type), 1, "context", 1))) __PYX_ERR(0, 404, __pyx_L1_error)
  __pyx_r = __pyx_pf_9drf_turbo_6fields_10RegexField_2deserialize(((struct __pyx_obj_9drf_turbo_6fields_RegexField *)__pyx_v_self), __pyx_v_data, __pyx_v_context);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_10RegexField_2deserialize(struct __pyx_obj_9drf_turbo_6fields_RegexField *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("deserialize", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields_10RegexField_deserialize(__pyx_v_self, __pyx_v_data, __pyx_v_context, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 404, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.RegexField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pxd":53
 * cdef class RegexField(StrField):
 *     cdef public:
 *         regex             # <<<<<<<<<<<<<<
 * 
 * cdef class IPField(StrField):
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_10RegexField_5regex_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_10RegexField_5regex_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_10RegexField_5regex___get__(((struct __pyx_obj_9drf_turbo_6fields_RegexField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_10RegexField_5regex___get__(struct __pyx_obj_9drf_turbo_6fields_RegexField *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->regex);
  __pyx_r = __pyx_v_self->regex;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_10RegexField_5regex_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_10RegexField_5regex_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_10RegexField_5regex_2__set__(((struct __pyx_obj_9drf_turbo_6fields_RegexField *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_10RegexField_5regex_2__set__(struct __pyx_obj_9drf_turbo_6fields_RegexField *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->regex);
  __Pyx_DECREF(__pyx_v_self->regex);
  __pyx_v_self->regex = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_10RegexField_5regex_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_10RegexField_5regex_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_10RegexField_5regex_4__del__(((struct __pyx_obj_9drf_turbo_6fields_RegexField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_10RegexField_5regex_4__del__(struct __pyx_obj_9drf_turbo_6fields_RegexField *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->regex);
  __Pyx_DECREF(__pyx_v_self->regex);
  __pyx_v_self->regex = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_10RegexField_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_10RegexField_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_10RegexField_4__reduce_cython__(((struct __pyx_obj_9drf_turbo_6fields_RegexField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_10RegexField_4__reduce_cython__(struct __pyx_obj_9drf_turbo_6fields_RegexField *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self.allow_blank, self.allow_null, self.attr, self.attrs, self.call, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.max_length, self.min_length, self.read_only, self.regex, self.required, self.root, self.style, self.trim_whitespace, self.validators, self.write_only)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base.allow_null); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base.call); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base.read_only); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base.required); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base.write_only); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = PyTuple_New(21); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.allow_blank);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.allow_blank);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v_self->__pyx_base.allow_blank);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.attr);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.attr);
  PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_self->__pyx_base.__pyx_base.attr);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.attrs);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.attrs);
  PyTuple_SET_ITEM(__pyx_t_6, 3, __pyx_v_self->__pyx_base.__pyx_base.attrs);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_6, 4, __pyx_t_2);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.default_value);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.default_value);
  PyTuple_SET_ITEM(__pyx_t_6, 5, __pyx_v_self->__pyx_base.__pyx_base.default_value);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.error_messages);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.error_messages);
  PyTuple_SET_ITEM(__pyx_t_6, 6, __pyx_v_self->__pyx_base.__pyx_base.error_messages);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.field_name);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.field_name);
  PyTuple_SET_ITEM(__pyx_t_6, 7, __pyx_v_self->__pyx_base.__pyx_base.field_name);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.help_text);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.help_text);
  PyTuple_SET_ITEM(__pyx_t_6, 8, __pyx_v_self->__pyx_base.__pyx_base.help_text);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.initial);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.initial);
  PyTuple_SET_ITEM(__pyx_t_6, 9, __pyx_v_self->__pyx_base.__pyx_base.initial);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.label);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.label);
  PyTuple_SET_ITEM(__pyx_t_6, 10, __pyx_v_self->__pyx_base.__pyx_base.label);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.max_length);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.max_length);
  PyTuple_SET_ITEM(__pyx_t_6, 11, __pyx_v_self->__pyx_base.max_length);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.min_length);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.min_length);
  PyTuple_SET_ITEM(__pyx_t_6, 12, __pyx_v_self->__pyx_base.min_length);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 13, __pyx_t_3);
  __Pyx_INCREF(__pyx_v_self->regex);
  __Pyx_GIVEREF(__pyx_v_self->regex);
  PyTuple_SET_ITEM(__pyx_t_6, 14, __pyx_v_self->regex);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_6, 15, __pyx_t_4);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.root);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.root);
  PyTuple_SET_ITEM(__pyx_t_6, 16, __pyx_v_self->__pyx_base.__pyx_base.root);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.style);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.style);
  PyTuple_SET_ITEM(__pyx_t_6, 17, __pyx_v_self->__pyx_base.__pyx_base.style);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.trim_whitespace);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.trim_whitespace);
  PyTuple_SET_ITEM(__pyx_t_6, 18, __pyx_v_self->__pyx_base.trim_whitespace);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.validators);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.validators);
  PyTuple_SET_ITEM(__pyx_t_6, 19, __pyx_v_self->__pyx_base.__pyx_base.validators);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_6, 20, __pyx_t_5);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_6);
  __pyx_t_6 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self.allow_blank, self.allow_null, self.attr, self.attrs, self.call, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.max_length, self.min_length, self.read_only, self.regex, self.required, self.root, self.style, self.trim_whitespace, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_6 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_v__dict = __pyx_t_6;
  __pyx_t_6 = 0;

  /* "(tree fragment)":7
 *     state = (self.allow_blank, self.allow_null, self.attr, self.attrs, self.call, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.max_length, self.min_length, self.read_only, self.regex, self.required, self.root, self.style, self.trim_whitespace, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_7 = (__pyx_v__dict != Py_None);
  __pyx_t_8 = (__pyx_t_7 != 0);
  if (__pyx_t_8) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_6 = PyTuple_New(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v__dict);
    __pyx_t_5 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_6); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_5));
    __pyx_t_5 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.allow_blank is not None or self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.max_length is not None or self.min_length is not None or self.regex is not None or self.root is not None or self.style is not None or self.trim_whitespace is not None or self.validators is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self.allow_blank, self.allow_null, self.attr, self.attrs, self.call, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.max_length, self.min_length, self.read_only, self.regex, self.required, self.root, self.style, self.trim_whitespace, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self.allow_blank is not None or self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.max_length is not None or self.min_length is not None or self.regex is not None or self.root is not None or self.style is not None or self.trim_whitespace is not None or self.validators is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_RegexField, (type(self), 0x380e494, None), state
 */
  /*else*/ {
    __pyx_t_7 = (__pyx_v_self->__pyx_base.allow_blank != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.__pyx_base.attr != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.__pyx_base.attrs != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.__pyx_base.default_value != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.__pyx_base.error_messages != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.__pyx_base.field_name != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.__pyx_base.help_text != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.__pyx_base.initial != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.__pyx_base.label != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.max_length != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.min_length != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->regex != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.__pyx_base.root != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.__pyx_base.style != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.trim_whitespace != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.__pyx_base.validators != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    __pyx_t_8 = __pyx_t_7;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_8;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.allow_blank is not None or self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.max_length is not None or self.min_length is not None or self.regex is not None or self.root is not None or self.style is not None or self.trim_whitespace is not None or self.validators is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_RegexField, (type(self), 0x380e494, None), state
 *     else:
 */
  __pyx_t_8 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_8) {

    /* "(tree fragment)":13
 *         use_setstate = self.allow_blank is not None or self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.max_length is not None or self.min_length is not None or self.regex is not None or self.root is not None or self.style is not None or self.trim_whitespace is not None or self.validators is not None
 *     if use_setstate:
 *         return __pyx_unpickle_RegexField, (type(self), 0x380e494, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_RegexField, (type(self), 0x380e494, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_pyx_unpickle_RegexField); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_6, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_58778772);
    __Pyx_GIVEREF(__pyx_int_58778772);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_int_58778772);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_6, 2, Py_None);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_6);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_state);
    __pyx_t_5 = 0;
    __pyx_t_6 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.allow_blank is not None or self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.max_length is not None or self.min_length is not None or self.regex is not None or self.root is not None or self.style is not None or self.trim_whitespace is not None or self.validators is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_RegexField, (type(self), 0x380e494, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_RegexField, (type(self), 0x380e494, None), state
 *     else:
 *         return __pyx_unpickle_RegexField, (type(self), 0x380e494, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_RegexField__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_pyx_unpickle_RegexField); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_6, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_58778772);
    __Pyx_GIVEREF(__pyx_int_58778772);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_int_58778772);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_state);
    __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_6);
    __pyx_t_4 = 0;
    __pyx_t_6 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("drf_turbo.fields.RegexField.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_RegexField, (type(self), 0x380e494, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_RegexField__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_10RegexField_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_10RegexField_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_10RegexField_6__setstate_cython__(((struct __pyx_obj_9drf_turbo_6fields_RegexField *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_10RegexField_6__setstate_cython__(struct __pyx_obj_9drf_turbo_6fields_RegexField *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_RegexField, (type(self), 0x380e494, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_RegexField__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields___pyx_unpickle_RegexField__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_RegexField, (type(self), 0x380e494, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_RegexField__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.RegexField.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":424
 *     }
 * 
 *     cpdef inline deserialize(self,data,dict context):             # <<<<<<<<<<<<<<
 *         try:
 *            return ipaddress.ip_address(data)
 */

static PyObject *__pyx_pw_9drf_turbo_6fields_7IPField_1deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_7IPField_deserialize(struct __pyx_obj_9drf_turbo_6fields_IPField *__pyx_v_self, PyObject *__pyx_v_data, CYTHON_UNUSED PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("deserialize", 0);

  /* "drf_turbo/fields.pyx":425
 * 
 *     cpdef inline deserialize(self,data,dict context):
 *         try:             # <<<<<<<<<<<<<<
 *            return ipaddress.ip_address(data)
 *         except (ValueError, TypeError) :
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "drf_turbo/fields.pyx":426
 *     cpdef inline deserialize(self,data,dict context):
 *         try:
 *            return ipaddress.ip_address(data)             # <<<<<<<<<<<<<<
 *         except (ValueError, TypeError) :
 *             raise self.raise_if_fail('invalid')
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_ipaddress); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 426, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_ip_address); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 426, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_5 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_6))) {
        __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_6);
        if (likely(__pyx_t_5)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
          __Pyx_INCREF(__pyx_t_5);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_6, function);
        }
      }
      __pyx_t_4 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_5, __pyx_v_data) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_v_data);
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 426, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_r = __pyx_t_4;
      __pyx_t_4 = 0;
      goto __pyx_L7_try_return;

      /* "drf_turbo/fields.pyx":425
 * 
 *     cpdef inline deserialize(self,data,dict context):
 *         try:             # <<<<<<<<<<<<<<
 *            return ipaddress.ip_address(data)
 *         except (ValueError, TypeError) :
 */
    }
    __pyx_L3_error:;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;

    /* "drf_turbo/fields.pyx":427
 *         try:
 *            return ipaddress.ip_address(data)
 *         except (ValueError, TypeError) :             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail('invalid')
 * 
 */
    __pyx_t_7 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_ValueError) || __Pyx_PyErr_ExceptionMatches(__pyx_builtin_TypeError);
    if (__pyx_t_7) {
      __Pyx_AddTraceback("drf_turbo.fields.IPField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_4, &__pyx_t_6, &__pyx_t_5) < 0) __PYX_ERR(0, 427, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GOTREF(__pyx_t_5);

      /* "drf_turbo/fields.pyx":428
 *            return ipaddress.ip_address(data)
 *         except (ValueError, TypeError) :
 *             raise self.raise_if_fail('invalid')             # <<<<<<<<<<<<<<
 * 
 * @cython.final
 */
      __pyx_t_9 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_raise_if_fail); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 428, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_10 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_9))) {
        __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_9);
        if (likely(__pyx_t_10)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
          __Pyx_INCREF(__pyx_t_10);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_9, function);
        }
      }
      __pyx_t_8 = (__pyx_t_10) ? __Pyx_PyObject_Call2Args(__pyx_t_9, __pyx_t_10, __pyx_n_s_invalid) : __Pyx_PyObject_CallOneArg(__pyx_t_9, __pyx_n_s_invalid);
      __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
      if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 428, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_Raise(__pyx_t_8, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __PYX_ERR(0, 428, __pyx_L5_except_error)
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;

    /* "drf_turbo/fields.pyx":425
 * 
 *     cpdef inline deserialize(self,data,dict context):
 *         try:             # <<<<<<<<<<<<<<
 *            return ipaddress.ip_address(data)
 *         except (ValueError, TypeError) :
 */
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L7_try_return:;
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L0;
  }

  /* "drf_turbo/fields.pyx":424
 *     }
 * 
 *     cpdef inline deserialize(self,data,dict context):             # <<<<<<<<<<<<<<
 *         try:
 *            return ipaddress.ip_address(data)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("drf_turbo.fields.IPField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_7IPField_1deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_7IPField_1deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_data = 0;
  PyObject *__pyx_v_context = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("deserialize (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_data,&__pyx_n_s_context,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_data)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_context)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("deserialize", 1, 2, 2, 1); __PYX_ERR(0, 424, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "deserialize") < 0)) __PYX_ERR(0, 424, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_data = values[0];
    __pyx_v_context = ((PyObject*)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("deserialize", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 424, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.IPField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_context), (&PyDict_Type), 1, "context", 1))) __PYX_ERR(0, 424, __pyx_L1_error)
  __pyx_r = __pyx_pf_9drf_turbo_6fields_7IPField_deserialize(((struct __pyx_obj_9drf_turbo_6fields_IPField *)__pyx_v_self), __pyx_v_data, __pyx_v_context);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_7IPField_deserialize(struct __pyx_obj_9drf_turbo_6fields_IPField *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("deserialize", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields_7IPField_deserialize(__pyx_v_self, __pyx_v_data, __pyx_v_context, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 424, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.IPField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_7IPField_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_7IPField_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_7IPField_2__reduce_cython__(((struct __pyx_obj_9drf_turbo_6fields_IPField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_7IPField_2__reduce_cython__(struct __pyx_obj_9drf_turbo_6fields_IPField *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self.allow_blank, self.allow_null, self.attr, self.attrs, self.call, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.max_length, self.min_length, self.read_only, self.required, self.root, self.style, self.trim_whitespace, self.validators, self.write_only)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base.allow_null); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base.call); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base.read_only); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base.required); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base.write_only); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = PyTuple_New(20); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.allow_blank);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.allow_blank);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v_self->__pyx_base.allow_blank);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.attr);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.attr);
  PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_self->__pyx_base.__pyx_base.attr);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.attrs);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.attrs);
  PyTuple_SET_ITEM(__pyx_t_6, 3, __pyx_v_self->__pyx_base.__pyx_base.attrs);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_6, 4, __pyx_t_2);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.default_value);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.default_value);
  PyTuple_SET_ITEM(__pyx_t_6, 5, __pyx_v_self->__pyx_base.__pyx_base.default_value);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.error_messages);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.error_messages);
  PyTuple_SET_ITEM(__pyx_t_6, 6, __pyx_v_self->__pyx_base.__pyx_base.error_messages);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.field_name);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.field_name);
  PyTuple_SET_ITEM(__pyx_t_6, 7, __pyx_v_self->__pyx_base.__pyx_base.field_name);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.help_text);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.help_text);
  PyTuple_SET_ITEM(__pyx_t_6, 8, __pyx_v_self->__pyx_base.__pyx_base.help_text);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.initial);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.initial);
  PyTuple_SET_ITEM(__pyx_t_6, 9, __pyx_v_self->__pyx_base.__pyx_base.initial);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.label);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.label);
  PyTuple_SET_ITEM(__pyx_t_6, 10, __pyx_v_self->__pyx_base.__pyx_base.label);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.max_length);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.max_length);
  PyTuple_SET_ITEM(__pyx_t_6, 11, __pyx_v_self->__pyx_base.max_length);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.min_length);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.min_length);
  PyTuple_SET_ITEM(__pyx_t_6, 12, __pyx_v_self->__pyx_base.min_length);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 13, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_6, 14, __pyx_t_4);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.root);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.root);
  PyTuple_SET_ITEM(__pyx_t_6, 15, __pyx_v_self->__pyx_base.__pyx_base.root);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.style);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.style);
  PyTuple_SET_ITEM(__pyx_t_6, 16, __pyx_v_self->__pyx_base.__pyx_base.style);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.trim_whitespace);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.trim_whitespace);
  PyTuple_SET_ITEM(__pyx_t_6, 17, __pyx_v_self->__pyx_base.trim_whitespace);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.validators);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.validators);
  PyTuple_SET_ITEM(__pyx_t_6, 18, __pyx_v_self->__pyx_base.__pyx_base.validators);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_6, 19, __pyx_t_5);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_6);
  __pyx_t_6 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self.allow_blank, self.allow_null, self.attr, self.attrs, self.call, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.max_length, self.min_length, self.read_only, self.required, self.root, self.style, self.trim_whitespace, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_6 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_v__dict = __pyx_t_6;
  __pyx_t_6 = 0;

  /* "(tree fragment)":7
 *     state = (self.allow_blank, self.allow_null, self.attr, self.attrs, self.call, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.max_length, self.min_length, self.read_only, self.required, self.root, self.style, self.trim_whitespace, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_7 = (__pyx_v__dict != Py_None);
  __pyx_t_8 = (__pyx_t_7 != 0);
  if (__pyx_t_8) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_6 = PyTuple_New(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v__dict);
    __pyx_t_5 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_6); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_5));
    __pyx_t_5 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.allow_blank is not None or self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.max_length is not None or self.min_length is not None or self.root is not None or self.style is not None or self.trim_whitespace is not None or self.validators is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self.allow_blank, self.allow_null, self.attr, self.attrs, self.call, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.max_length, self.min_length, self.read_only, self.required, self.root, self.style, self.trim_whitespace, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self.allow_blank is not None or self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.max_length is not None or self.min_length is not None or self.root is not None or self.style is not None or self.trim_whitespace is not None or self.validators is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_IPField, (type(self), 0x0fbedf5, None), state
 */
  /*else*/ {
    __pyx_t_7 = (__pyx_v_self->__pyx_base.allow_blank != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.__pyx_base.attr != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.__pyx_base.attrs != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.__pyx_base.default_value != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.__pyx_base.error_messages != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.__pyx_base.field_name != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.__pyx_base.help_text != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.__pyx_base.initial != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.__pyx_base.label != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.max_length != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.min_length != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.__pyx_base.root != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.__pyx_base.style != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.trim_whitespace != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.__pyx_base.validators != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    __pyx_t_8 = __pyx_t_9;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_8;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.allow_blank is not None or self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.max_length is not None or self.min_length is not None or self.root is not None or self.style is not None or self.trim_whitespace is not None or self.validators is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_IPField, (type(self), 0x0fbedf5, None), state
 *     else:
 */
  __pyx_t_8 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_8) {

    /* "(tree fragment)":13
 *         use_setstate = self.allow_blank is not None or self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.max_length is not None or self.min_length is not None or self.root is not None or self.style is not None or self.trim_whitespace is not None or self.validators is not None
 *     if use_setstate:
 *         return __pyx_unpickle_IPField, (type(self), 0x0fbedf5, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_IPField, (type(self), 0x0fbedf5, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_pyx_unpickle_IPField); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_6, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_16510453);
    __Pyx_GIVEREF(__pyx_int_16510453);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_int_16510453);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_6, 2, Py_None);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_6);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_state);
    __pyx_t_5 = 0;
    __pyx_t_6 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.allow_blank is not None or self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.max_length is not None or self.min_length is not None or self.root is not None or self.style is not None or self.trim_whitespace is not None or self.validators is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_IPField, (type(self), 0x0fbedf5, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_IPField, (type(self), 0x0fbedf5, None), state
 *     else:
 *         return __pyx_unpickle_IPField, (type(self), 0x0fbedf5, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_IPField__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_pyx_unpickle_IPField); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_6, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_16510453);
    __Pyx_GIVEREF(__pyx_int_16510453);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_int_16510453);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_state);
    __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_6);
    __pyx_t_4 = 0;
    __pyx_t_6 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("drf_turbo.fields.IPField.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_IPField, (type(self), 0x0fbedf5, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_IPField__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_7IPField_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_7IPField_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_7IPField_4__setstate_cython__(((struct __pyx_obj_9drf_turbo_6fields_IPField *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_7IPField_4__setstate_cython__(struct __pyx_obj_9drf_turbo_6fields_IPField *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_IPField, (type(self), 0x0fbedf5, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_IPField__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields___pyx_unpickle_IPField__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_IPField, (type(self), 0x0fbedf5, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_IPField__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.IPField.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":438
 *     """
 * 
 *     def __init__(self,**kwargs):             # <<<<<<<<<<<<<<
 *         kwargs['write_only'] = True
 *         kwargs['min_length'] = 4
 */

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_13PasswordField_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_13PasswordField_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_kwargs = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  if (unlikely(PyTuple_GET_SIZE(__pyx_args) > 0)) {
    __Pyx_RaiseArgtupleInvalid("__init__", 1, 0, 0, PyTuple_GET_SIZE(__pyx_args)); return -1;}
  if (__pyx_kwds && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "__init__", 1))) return -1;
  __pyx_v_kwargs = (__pyx_kwds) ? PyDict_Copy(__pyx_kwds) : PyDict_New(); if (unlikely(!__pyx_v_kwargs)) return -1;
  __Pyx_GOTREF(__pyx_v_kwargs);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_13PasswordField___init__(((struct __pyx_obj_9drf_turbo_6fields_PasswordField *)__pyx_v_self), __pyx_v_kwargs);

  /* function exit code */
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_13PasswordField___init__(struct __pyx_obj_9drf_turbo_6fields_PasswordField *__pyx_v_self, PyObject *__pyx_v_kwargs) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "drf_turbo/fields.pyx":439
 * 
 *     def __init__(self,**kwargs):
 *         kwargs['write_only'] = True             # <<<<<<<<<<<<<<
 *         kwargs['min_length'] = 4
 *         kwargs['required'] = True
 */
  if (unlikely(PyDict_SetItem(__pyx_v_kwargs, __pyx_n_s_write_only, Py_True) < 0)) __PYX_ERR(0, 439, __pyx_L1_error)

  /* "drf_turbo/fields.pyx":440
 *     def __init__(self,**kwargs):
 *         kwargs['write_only'] = True
 *         kwargs['min_length'] = 4             # <<<<<<<<<<<<<<
 *         kwargs['required'] = True
 *         super().__init__(**kwargs)
 */
  if (unlikely(PyDict_SetItem(__pyx_v_kwargs, __pyx_n_s_min_length, __pyx_int_4) < 0)) __PYX_ERR(0, 440, __pyx_L1_error)

  /* "drf_turbo/fields.pyx":441
 *         kwargs['write_only'] = True
 *         kwargs['min_length'] = 4
 *         kwargs['required'] = True             # <<<<<<<<<<<<<<
 *         super().__init__(**kwargs)
 * 
 */
  if (unlikely(PyDict_SetItem(__pyx_v_kwargs, __pyx_n_s_required, Py_True) < 0)) __PYX_ERR(0, 441, __pyx_L1_error)

  /* "drf_turbo/fields.pyx":442
 *         kwargs['min_length'] = 4
 *         kwargs['required'] = True
 *         super().__init__(**kwargs)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 442, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_9drf_turbo_6fields_PasswordField));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_9drf_turbo_6fields_PasswordField));
  PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)__pyx_ptype_9drf_turbo_6fields_PasswordField));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_1, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 442, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_init); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 442, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyDict_Copy(__pyx_v_kwargs); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 442, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 442, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "drf_turbo/fields.pyx":438
 *     """
 * 
 *     def __init__(self,**kwargs):             # <<<<<<<<<<<<<<
 *         kwargs['write_only'] = True
 *         kwargs['min_length'] = 4
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("drf_turbo.fields.PasswordField.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_13PasswordField_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_13PasswordField_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_13PasswordField_2__reduce_cython__(((struct __pyx_obj_9drf_turbo_6fields_PasswordField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_13PasswordField_2__reduce_cython__(struct __pyx_obj_9drf_turbo_6fields_PasswordField *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self.allow_blank, self.allow_null, self.attr, self.attrs, self.call, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.max_length, self.min_length, self.read_only, self.required, self.root, self.style, self.trim_whitespace, self.validators, self.write_only)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base.allow_null); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base.call); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base.read_only); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base.required); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base.write_only); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = PyTuple_New(20); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.allow_blank);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.allow_blank);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v_self->__pyx_base.allow_blank);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.attr);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.attr);
  PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_self->__pyx_base.__pyx_base.attr);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.attrs);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.attrs);
  PyTuple_SET_ITEM(__pyx_t_6, 3, __pyx_v_self->__pyx_base.__pyx_base.attrs);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_6, 4, __pyx_t_2);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.default_value);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.default_value);
  PyTuple_SET_ITEM(__pyx_t_6, 5, __pyx_v_self->__pyx_base.__pyx_base.default_value);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.error_messages);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.error_messages);
  PyTuple_SET_ITEM(__pyx_t_6, 6, __pyx_v_self->__pyx_base.__pyx_base.error_messages);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.field_name);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.field_name);
  PyTuple_SET_ITEM(__pyx_t_6, 7, __pyx_v_self->__pyx_base.__pyx_base.field_name);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.help_text);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.help_text);
  PyTuple_SET_ITEM(__pyx_t_6, 8, __pyx_v_self->__pyx_base.__pyx_base.help_text);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.initial);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.initial);
  PyTuple_SET_ITEM(__pyx_t_6, 9, __pyx_v_self->__pyx_base.__pyx_base.initial);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.label);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.label);
  PyTuple_SET_ITEM(__pyx_t_6, 10, __pyx_v_self->__pyx_base.__pyx_base.label);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.max_length);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.max_length);
  PyTuple_SET_ITEM(__pyx_t_6, 11, __pyx_v_self->__pyx_base.max_length);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.min_length);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.min_length);
  PyTuple_SET_ITEM(__pyx_t_6, 12, __pyx_v_self->__pyx_base.min_length);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 13, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_6, 14, __pyx_t_4);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.root);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.root);
  PyTuple_SET_ITEM(__pyx_t_6, 15, __pyx_v_self->__pyx_base.__pyx_base.root);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.style);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.style);
  PyTuple_SET_ITEM(__pyx_t_6, 16, __pyx_v_self->__pyx_base.__pyx_base.style);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.trim_whitespace);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.trim_whitespace);
  PyTuple_SET_ITEM(__pyx_t_6, 17, __pyx_v_self->__pyx_base.trim_whitespace);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.validators);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.validators);
  PyTuple_SET_ITEM(__pyx_t_6, 18, __pyx_v_self->__pyx_base.__pyx_base.validators);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_6, 19, __pyx_t_5);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_6);
  __pyx_t_6 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self.allow_blank, self.allow_null, self.attr, self.attrs, self.call, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.max_length, self.min_length, self.read_only, self.required, self.root, self.style, self.trim_whitespace, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_6 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_v__dict = __pyx_t_6;
  __pyx_t_6 = 0;

  /* "(tree fragment)":7
 *     state = (self.allow_blank, self.allow_null, self.attr, self.attrs, self.call, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.max_length, self.min_length, self.read_only, self.required, self.root, self.style, self.trim_whitespace, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_7 = (__pyx_v__dict != Py_None);
  __pyx_t_8 = (__pyx_t_7 != 0);
  if (__pyx_t_8) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_6 = PyTuple_New(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v__dict);
    __pyx_t_5 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_6); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_5));
    __pyx_t_5 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.allow_blank is not None or self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.max_length is not None or self.min_length is not None or self.root is not None or self.style is not None or self.trim_whitespace is not None or self.validators is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self.allow_blank, self.allow_null, self.attr, self.attrs, self.call, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.max_length, self.min_length, self.read_only, self.required, self.root, self.style, self.trim_whitespace, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self.allow_blank is not None or self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.max_length is not None or self.min_length is not None or self.root is not None or self.style is not None or self.trim_whitespace is not None or self.validators is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_PasswordField, (type(self), 0x0fbedf5, None), state
 */
  /*else*/ {
    __pyx_t_7 = (__pyx_v_self->__pyx_base.allow_blank != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.__pyx_base.attr != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.__pyx_base.attrs != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.__pyx_base.default_value != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.__pyx_base.error_messages != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.__pyx_base.field_name != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.__pyx_base.help_text != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.__pyx_base.initial != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.__pyx_base.label != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.max_length != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.min_length != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.__pyx_base.root != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.__pyx_base.style != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.trim_whitespace != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.__pyx_base.validators != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    __pyx_t_8 = __pyx_t_9;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_8;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.allow_blank is not None or self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.max_length is not None or self.min_length is not None or self.root is not None or self.style is not None or self.trim_whitespace is not None or self.validators is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_PasswordField, (type(self), 0x0fbedf5, None), state
 *     else:
 */
  __pyx_t_8 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_8) {

    /* "(tree fragment)":13
 *         use_setstate = self.allow_blank is not None or self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.max_length is not None or self.min_length is not None or self.root is not None or self.style is not None or self.trim_whitespace is not None or self.validators is not None
 *     if use_setstate:
 *         return __pyx_unpickle_PasswordField, (type(self), 0x0fbedf5, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_PasswordField, (type(self), 0x0fbedf5, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_pyx_unpickle_PasswordField); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_6, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_16510453);
    __Pyx_GIVEREF(__pyx_int_16510453);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_int_16510453);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_6, 2, Py_None);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_6);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_state);
    __pyx_t_5 = 0;
    __pyx_t_6 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.allow_blank is not None or self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.max_length is not None or self.min_length is not None or self.root is not None or self.style is not None or self.trim_whitespace is not None or self.validators is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_PasswordField, (type(self), 0x0fbedf5, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_PasswordField, (type(self), 0x0fbedf5, None), state
 *     else:
 *         return __pyx_unpickle_PasswordField, (type(self), 0x0fbedf5, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_PasswordField__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_pyx_unpickle_PasswordField); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_6, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_16510453);
    __Pyx_GIVEREF(__pyx_int_16510453);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_int_16510453);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_state);
    __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_6);
    __pyx_t_4 = 0;
    __pyx_t_6 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("drf_turbo.fields.PasswordField.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_PasswordField, (type(self), 0x0fbedf5, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_PasswordField__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_13PasswordField_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_13PasswordField_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_13PasswordField_4__setstate_cython__(((struct __pyx_obj_9drf_turbo_6fields_PasswordField *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_13PasswordField_4__setstate_cython__(struct __pyx_obj_9drf_turbo_6fields_PasswordField *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_PasswordField, (type(self), 0x0fbedf5, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_PasswordField__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields___pyx_unpickle_PasswordField__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_PasswordField, (type(self), 0x0fbedf5, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_PasswordField__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.PasswordField.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":458
 *     }
 * 
 *     cpdef inline deserialize(self,data,dict context):             # <<<<<<<<<<<<<<
 *         if data is None:
 *             return None
 */

static PyObject *__pyx_pw_9drf_turbo_6fields_9UUIDField_1deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_9UUIDField_deserialize(struct __pyx_obj_9drf_turbo_6fields_UUIDField *__pyx_v_self, PyObject *__pyx_v_data, CYTHON_UNUSED PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("deserialize", 0);

  /* "drf_turbo/fields.pyx":459
 * 
 *     cpdef inline deserialize(self,data,dict context):
 *         if data is None:             # <<<<<<<<<<<<<<
 *             return None
 *         if isinstance(data, uuid.UUID):
 */
  __pyx_t_1 = (__pyx_v_data == Py_None);
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "drf_turbo/fields.pyx":460
 *     cpdef inline deserialize(self,data,dict context):
 *         if data is None:
 *             return None             # <<<<<<<<<<<<<<
 *         if isinstance(data, uuid.UUID):
 *             return data
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "drf_turbo/fields.pyx":459
 * 
 *     cpdef inline deserialize(self,data,dict context):
 *         if data is None:             # <<<<<<<<<<<<<<
 *             return None
 *         if isinstance(data, uuid.UUID):
 */
  }

  /* "drf_turbo/fields.pyx":461
 *         if data is None:
 *             return None
 *         if isinstance(data, uuid.UUID):             # <<<<<<<<<<<<<<
 *             return data
 *         try:
 */
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_uuid); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 461, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_UUID); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 461, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_2 = PyObject_IsInstance(__pyx_v_data, __pyx_t_4); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 461, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_1 = (__pyx_t_2 != 0);
  if (__pyx_t_1) {

    /* "drf_turbo/fields.pyx":462
 *             return None
 *         if isinstance(data, uuid.UUID):
 *             return data             # <<<<<<<<<<<<<<
 *         try:
 *             if isinstance(data, bytes) and len(data) == 16:
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_data);
    __pyx_r = __pyx_v_data;
    goto __pyx_L0;

    /* "drf_turbo/fields.pyx":461
 *         if data is None:
 *             return None
 *         if isinstance(data, uuid.UUID):             # <<<<<<<<<<<<<<
 *             return data
 *         try:
 */
  }

  /* "drf_turbo/fields.pyx":463
 *         if isinstance(data, uuid.UUID):
 *             return data
 *         try:             # <<<<<<<<<<<<<<
 *             if isinstance(data, bytes) and len(data) == 16:
 *                 return uuid.UUID(bytes=data)
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7);
    __Pyx_XGOTREF(__pyx_t_5);
    __Pyx_XGOTREF(__pyx_t_6);
    __Pyx_XGOTREF(__pyx_t_7);
    /*try:*/ {

      /* "drf_turbo/fields.pyx":464
 *             return data
 *         try:
 *             if isinstance(data, bytes) and len(data) == 16:             # <<<<<<<<<<<<<<
 *                 return uuid.UUID(bytes=data)
 *             elif isinstance(data,int):
 */
      __pyx_t_2 = PyBytes_Check(__pyx_v_data); 
      __pyx_t_8 = (__pyx_t_2 != 0);
      if (__pyx_t_8) {
      } else {
        __pyx_t_1 = __pyx_t_8;
        goto __pyx_L12_bool_binop_done;
      }
      __pyx_t_9 = PyObject_Length(__pyx_v_data); if (unlikely(__pyx_t_9 == ((Py_ssize_t)-1))) __PYX_ERR(0, 464, __pyx_L5_error)
      __pyx_t_8 = ((__pyx_t_9 == 16) != 0);
      __pyx_t_1 = __pyx_t_8;
      __pyx_L12_bool_binop_done:;
      if (__pyx_t_1) {

        /* "drf_turbo/fields.pyx":465
 *         try:
 *             if isinstance(data, bytes) and len(data) == 16:
 *                 return uuid.UUID(bytes=data)             # <<<<<<<<<<<<<<
 *             elif isinstance(data,int):
 *                 return uuid.UUID(int=data)
 */
        __Pyx_XDECREF(__pyx_r);
        __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_uuid); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 465, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_UUID); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 465, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __pyx_t_4 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 465, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_4);
        if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_bytes, __pyx_v_data) < 0) __PYX_ERR(0, 465, __pyx_L5_error)
        __pyx_t_10 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_empty_tuple, __pyx_t_4); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 465, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_10);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __pyx_r = __pyx_t_10;
        __pyx_t_10 = 0;
        goto __pyx_L9_try_return;

        /* "drf_turbo/fields.pyx":464
 *             return data
 *         try:
 *             if isinstance(data, bytes) and len(data) == 16:             # <<<<<<<<<<<<<<
 *                 return uuid.UUID(bytes=data)
 *             elif isinstance(data,int):
 */
      }

      /* "drf_turbo/fields.pyx":466
 *             if isinstance(data, bytes) and len(data) == 16:
 *                 return uuid.UUID(bytes=data)
 *             elif isinstance(data,int):             # <<<<<<<<<<<<<<
 *                 return uuid.UUID(int=data)
 *             elif isinstance(data,str):
 */
      __pyx_t_1 = PyInt_Check(__pyx_v_data); 
      __pyx_t_8 = (__pyx_t_1 != 0);
      if (__pyx_t_8) {

        /* "drf_turbo/fields.pyx":467
 *                 return uuid.UUID(bytes=data)
 *             elif isinstance(data,int):
 *                 return uuid.UUID(int=data)             # <<<<<<<<<<<<<<
 *             elif isinstance(data,str):
 *                 return uuid.UUID(hex=data)
 */
        __Pyx_XDECREF(__pyx_r);
        __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_n_s_uuid); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 467, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_10);
        __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_n_s_UUID); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 467, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        __pyx_t_10 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 467, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_10);
        if (PyDict_SetItem(__pyx_t_10, __pyx_n_s_int, __pyx_v_data) < 0) __PYX_ERR(0, 467, __pyx_L5_error)
        __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_empty_tuple, __pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 467, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        __pyx_r = __pyx_t_3;
        __pyx_t_3 = 0;
        goto __pyx_L9_try_return;

        /* "drf_turbo/fields.pyx":466
 *             if isinstance(data, bytes) and len(data) == 16:
 *                 return uuid.UUID(bytes=data)
 *             elif isinstance(data,int):             # <<<<<<<<<<<<<<
 *                 return uuid.UUID(int=data)
 *             elif isinstance(data,str):
 */
      }

      /* "drf_turbo/fields.pyx":468
 *             elif isinstance(data,int):
 *                 return uuid.UUID(int=data)
 *             elif isinstance(data,str):             # <<<<<<<<<<<<<<
 *                 return uuid.UUID(hex=data)
 *             else:
 */
      __pyx_t_8 = PyString_Check(__pyx_v_data); 
      __pyx_t_1 = (__pyx_t_8 != 0);
      if (__pyx_t_1) {

        /* "drf_turbo/fields.pyx":469
 *                 return uuid.UUID(int=data)
 *             elif isinstance(data,str):
 *                 return uuid.UUID(hex=data)             # <<<<<<<<<<<<<<
 *             else:
 *                 return uuid.UUID(data)
 */
        __Pyx_XDECREF(__pyx_r);
        __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_uuid); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 469, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_UUID); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 469, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_10);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_3 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 469, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_3);
        if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_hex, __pyx_v_data) < 0) __PYX_ERR(0, 469, __pyx_L5_error)
        __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_10, __pyx_empty_tuple, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 469, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_4;
        __pyx_t_4 = 0;
        goto __pyx_L9_try_return;

        /* "drf_turbo/fields.pyx":468
 *             elif isinstance(data,int):
 *                 return uuid.UUID(int=data)
 *             elif isinstance(data,str):             # <<<<<<<<<<<<<<
 *                 return uuid.UUID(hex=data)
 *             else:
 */
      }

      /* "drf_turbo/fields.pyx":471
 *                 return uuid.UUID(hex=data)
 *             else:
 *                 return uuid.UUID(data)             # <<<<<<<<<<<<<<
 *         except :
 *             raise self.raise_if_fail('invalid')
 */
      /*else*/ {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_uuid); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 471, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_UUID); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 471, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_10);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_3 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_10))) {
          __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_10);
          if (likely(__pyx_t_3)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
            __Pyx_INCREF(__pyx_t_3);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_10, function);
          }
        }
        __pyx_t_4 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_10, __pyx_t_3, __pyx_v_data) : __Pyx_PyObject_CallOneArg(__pyx_t_10, __pyx_v_data);
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 471, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        __pyx_r = __pyx_t_4;
        __pyx_t_4 = 0;
        goto __pyx_L9_try_return;
      }

      /* "drf_turbo/fields.pyx":463
 *         if isinstance(data, uuid.UUID):
 *             return data
 *         try:             # <<<<<<<<<<<<<<
 *             if isinstance(data, bytes) and len(data) == 16:
 *                 return uuid.UUID(bytes=data)
 */
    }
    __pyx_L5_error:;
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "drf_turbo/fields.pyx":472
 *             else:
 *                 return uuid.UUID(data)
 *         except :             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail('invalid')
 * 
 */
    /*except:*/ {
      __Pyx_AddTraceback("drf_turbo.fields.UUIDField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_4, &__pyx_t_10, &__pyx_t_3) < 0) __PYX_ERR(0, 472, __pyx_L7_except_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_GOTREF(__pyx_t_3);

      /* "drf_turbo/fields.pyx":473
 *                 return uuid.UUID(data)
 *         except :
 *             raise self.raise_if_fail('invalid')             # <<<<<<<<<<<<<<
 * 
 * @cython.final
 */
      __pyx_t_12 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_raise_if_fail); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 473, __pyx_L7_except_error)
      __Pyx_GOTREF(__pyx_t_12);
      __pyx_t_13 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_12))) {
        __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_12);
        if (likely(__pyx_t_13)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_12);
          __Pyx_INCREF(__pyx_t_13);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_12, function);
        }
      }
      __pyx_t_11 = (__pyx_t_13) ? __Pyx_PyObject_Call2Args(__pyx_t_12, __pyx_t_13, __pyx_n_s_invalid) : __Pyx_PyObject_CallOneArg(__pyx_t_12, __pyx_n_s_invalid);
      __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
      if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 473, __pyx_L7_except_error)
      __Pyx_GOTREF(__pyx_t_11);
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      __Pyx_Raise(__pyx_t_11, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      __PYX_ERR(0, 473, __pyx_L7_except_error)
    }
    __pyx_L7_except_error:;

    /* "drf_turbo/fields.pyx":463
 *         if isinstance(data, uuid.UUID):
 *             return data
 *         try:             # <<<<<<<<<<<<<<
 *             if isinstance(data, bytes) and len(data) == 16:
 *                 return uuid.UUID(bytes=data)
 */
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_XGIVEREF(__pyx_t_7);
    __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_6, __pyx_t_7);
    goto __pyx_L1_error;
    __pyx_L9_try_return:;
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_XGIVEREF(__pyx_t_7);
    __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_6, __pyx_t_7);
    goto __pyx_L0;
  }

  /* "drf_turbo/fields.pyx":458
 *     }
 * 
 *     cpdef inline deserialize(self,data,dict context):             # <<<<<<<<<<<<<<
 *         if data is None:
 *             return None
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_AddTraceback("drf_turbo.fields.UUIDField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_9UUIDField_1deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_9UUIDField_1deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_data = 0;
  PyObject *__pyx_v_context = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("deserialize (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_data,&__pyx_n_s_context,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_data)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_context)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("deserialize", 1, 2, 2, 1); __PYX_ERR(0, 458, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "deserialize") < 0)) __PYX_ERR(0, 458, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_data = values[0];
    __pyx_v_context = ((PyObject*)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("deserialize", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 458, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.UUIDField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_context), (&PyDict_Type), 1, "context", 1))) __PYX_ERR(0, 458, __pyx_L1_error)
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9UUIDField_deserialize(((struct __pyx_obj_9drf_turbo_6fields_UUIDField *)__pyx_v_self), __pyx_v_data, __pyx_v_context);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_9UUIDField_deserialize(struct __pyx_obj_9drf_turbo_6fields_UUIDField *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("deserialize", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields_9UUIDField_deserialize(__pyx_v_self, __pyx_v_data, __pyx_v_context, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 458, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.UUIDField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_9UUIDField_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_9UUIDField_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9UUIDField_2__reduce_cython__(((struct __pyx_obj_9drf_turbo_6fields_UUIDField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_9UUIDField_2__reduce_cython__(struct __pyx_obj_9drf_turbo_6fields_UUIDField *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self.allow_blank, self.allow_null, self.attr, self.attrs, self.call, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.max_length, self.min_length, self.read_only, self.required, self.root, self.style, self.trim_whitespace, self.validators, self.write_only)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base.allow_null); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base.call); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base.read_only); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base.required); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base.write_only); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = PyTuple_New(20); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.allow_blank);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.allow_blank);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v_self->__pyx_base.allow_blank);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.attr);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.attr);
  PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_self->__pyx_base.__pyx_base.attr);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.attrs);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.attrs);
  PyTuple_SET_ITEM(__pyx_t_6, 3, __pyx_v_self->__pyx_base.__pyx_base.attrs);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_6, 4, __pyx_t_2);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.default_value);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.default_value);
  PyTuple_SET_ITEM(__pyx_t_6, 5, __pyx_v_self->__pyx_base.__pyx_base.default_value);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.error_messages);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.error_messages);
  PyTuple_SET_ITEM(__pyx_t_6, 6, __pyx_v_self->__pyx_base.__pyx_base.error_messages);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.field_name);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.field_name);
  PyTuple_SET_ITEM(__pyx_t_6, 7, __pyx_v_self->__pyx_base.__pyx_base.field_name);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.help_text);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.help_text);
  PyTuple_SET_ITEM(__pyx_t_6, 8, __pyx_v_self->__pyx_base.__pyx_base.help_text);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.initial);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.initial);
  PyTuple_SET_ITEM(__pyx_t_6, 9, __pyx_v_self->__pyx_base.__pyx_base.initial);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.label);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.label);
  PyTuple_SET_ITEM(__pyx_t_6, 10, __pyx_v_self->__pyx_base.__pyx_base.label);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.max_length);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.max_length);
  PyTuple_SET_ITEM(__pyx_t_6, 11, __pyx_v_self->__pyx_base.max_length);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.min_length);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.min_length);
  PyTuple_SET_ITEM(__pyx_t_6, 12, __pyx_v_self->__pyx_base.min_length);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 13, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_6, 14, __pyx_t_4);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.root);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.root);
  PyTuple_SET_ITEM(__pyx_t_6, 15, __pyx_v_self->__pyx_base.__pyx_base.root);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.style);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.style);
  PyTuple_SET_ITEM(__pyx_t_6, 16, __pyx_v_self->__pyx_base.__pyx_base.style);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.trim_whitespace);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.trim_whitespace);
  PyTuple_SET_ITEM(__pyx_t_6, 17, __pyx_v_self->__pyx_base.trim_whitespace);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.validators);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.validators);
  PyTuple_SET_ITEM(__pyx_t_6, 18, __pyx_v_self->__pyx_base.__pyx_base.validators);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_6, 19, __pyx_t_5);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_6);
  __pyx_t_6 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self.allow_blank, self.allow_null, self.attr, self.attrs, self.call, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.max_length, self.min_length, self.read_only, self.required, self.root, self.style, self.trim_whitespace, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_6 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_v__dict = __pyx_t_6;
  __pyx_t_6 = 0;

  /* "(tree fragment)":7
 *     state = (self.allow_blank, self.allow_null, self.attr, self.attrs, self.call, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.max_length, self.min_length, self.read_only, self.required, self.root, self.style, self.trim_whitespace, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_7 = (__pyx_v__dict != Py_None);
  __pyx_t_8 = (__pyx_t_7 != 0);
  if (__pyx_t_8) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_6 = PyTuple_New(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v__dict);
    __pyx_t_5 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_6); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_5));
    __pyx_t_5 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.allow_blank is not None or self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.max_length is not None or self.min_length is not None or self.root is not None or self.style is not None or self.trim_whitespace is not None or self.validators is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self.allow_blank, self.allow_null, self.attr, self.attrs, self.call, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.max_length, self.min_length, self.read_only, self.required, self.root, self.style, self.trim_whitespace, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self.allow_blank is not None or self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.max_length is not None or self.min_length is not None or self.root is not None or self.style is not None or self.trim_whitespace is not None or self.validators is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_UUIDField, (type(self), 0x0fbedf5, None), state
 */
  /*else*/ {
    __pyx_t_7 = (__pyx_v_self->__pyx_base.allow_blank != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.__pyx_base.attr != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.__pyx_base.attrs != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.__pyx_base.default_value != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.__pyx_base.error_messages != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.__pyx_base.field_name != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.__pyx_base.help_text != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.__pyx_base.initial != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.__pyx_base.label != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.max_length != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.min_length != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.__pyx_base.root != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.__pyx_base.style != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.trim_whitespace != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.__pyx_base.validators != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    __pyx_t_8 = __pyx_t_9;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_8;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.allow_blank is not None or self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.max_length is not None or self.min_length is not None or self.root is not None or self.style is not None or self.trim_whitespace is not None or self.validators is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_UUIDField, (type(self), 0x0fbedf5, None), state
 *     else:
 */
  __pyx_t_8 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_8) {

    /* "(tree fragment)":13
 *         use_setstate = self.allow_blank is not None or self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.max_length is not None or self.min_length is not None or self.root is not None or self.style is not None or self.trim_whitespace is not None or self.validators is not None
 *     if use_setstate:
 *         return __pyx_unpickle_UUIDField, (type(self), 0x0fbedf5, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_UUIDField, (type(self), 0x0fbedf5, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_pyx_unpickle_UUIDField); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_6, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_16510453);
    __Pyx_GIVEREF(__pyx_int_16510453);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_int_16510453);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_6, 2, Py_None);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_6);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_state);
    __pyx_t_5 = 0;
    __pyx_t_6 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.allow_blank is not None or self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.max_length is not None or self.min_length is not None or self.root is not None or self.style is not None or self.trim_whitespace is not None or self.validators is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_UUIDField, (type(self), 0x0fbedf5, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_UUIDField, (type(self), 0x0fbedf5, None), state
 *     else:
 *         return __pyx_unpickle_UUIDField, (type(self), 0x0fbedf5, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_UUIDField__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_pyx_unpickle_UUIDField); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_6, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_16510453);
    __Pyx_GIVEREF(__pyx_int_16510453);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_int_16510453);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_state);
    __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_6);
    __pyx_t_4 = 0;
    __pyx_t_6 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("drf_turbo.fields.UUIDField.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_UUIDField, (type(self), 0x0fbedf5, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_UUIDField__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_9UUIDField_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_9UUIDField_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9UUIDField_4__setstate_cython__(((struct __pyx_obj_9drf_turbo_6fields_UUIDField *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_9UUIDField_4__setstate_cython__(struct __pyx_obj_9drf_turbo_6fields_UUIDField *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_UUIDField, (type(self), 0x0fbedf5, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_UUIDField__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields___pyx_unpickle_UUIDField__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_UUIDField, (type(self), 0x0fbedf5, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_UUIDField__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.UUIDField.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":488
 *     }
 * 
 *     def __init__(self, allow_unicode=False, **kwargs):             # <<<<<<<<<<<<<<
 *         self.allow_unicode = allow_unicode
 *         super().__init__(**kwargs)
 */

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_9SlugField_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_9SlugField_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_allow_unicode = 0;
  PyObject *__pyx_v_kwargs = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  __pyx_v_kwargs = PyDict_New(); if (unlikely(!__pyx_v_kwargs)) return -1;
  __Pyx_GOTREF(__pyx_v_kwargs);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_allow_unicode,0};
    PyObject* values[1] = {0};
    values[0] = ((PyObject *)Py_False);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_allow_unicode);
          if (value) { values[0] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, __pyx_v_kwargs, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 488, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_allow_unicode = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 0, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 488, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_DECREF(__pyx_v_kwargs); __pyx_v_kwargs = 0;
  __Pyx_AddTraceback("drf_turbo.fields.SlugField.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9SlugField___init__(((struct __pyx_obj_9drf_turbo_6fields_SlugField *)__pyx_v_self), __pyx_v_allow_unicode, __pyx_v_kwargs);

  /* function exit code */
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_9SlugField___init__(struct __pyx_obj_9drf_turbo_6fields_SlugField *__pyx_v_self, PyObject *__pyx_v_allow_unicode, PyObject *__pyx_v_kwargs) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "drf_turbo/fields.pyx":489
 * 
 *     def __init__(self, allow_unicode=False, **kwargs):
 *         self.allow_unicode = allow_unicode             # <<<<<<<<<<<<<<
 *         super().__init__(**kwargs)
 * 
 */
  __Pyx_INCREF(__pyx_v_allow_unicode);
  __Pyx_GIVEREF(__pyx_v_allow_unicode);
  __Pyx_GOTREF(__pyx_v_self->allow_unicode);
  __Pyx_DECREF(__pyx_v_self->allow_unicode);
  __pyx_v_self->allow_unicode = __pyx_v_allow_unicode;

  /* "drf_turbo/fields.pyx":490
 *     def __init__(self, allow_unicode=False, **kwargs):
 *         self.allow_unicode = allow_unicode
 *         super().__init__(**kwargs)             # <<<<<<<<<<<<<<
 * 
 *     cpdef inline deserialize(self,data,dict context):
 */
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 490, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_9drf_turbo_6fields_SlugField));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_9drf_turbo_6fields_SlugField));
  PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)__pyx_ptype_9drf_turbo_6fields_SlugField));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_1, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 490, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_init); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 490, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyDict_Copy(__pyx_v_kwargs); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 490, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 490, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "drf_turbo/fields.pyx":488
 *     }
 * 
 *     def __init__(self, allow_unicode=False, **kwargs):             # <<<<<<<<<<<<<<
 *         self.allow_unicode = allow_unicode
 *         super().__init__(**kwargs)
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("drf_turbo.fields.SlugField.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":492
 *         super().__init__(**kwargs)
 * 
 *     cpdef inline deserialize(self,data,dict context):             # <<<<<<<<<<<<<<
 *         if self.allow_unicode :
 *             regex = re.compile(r'^[-\w]+\Z', re.UNICODE)
 */

static PyObject *__pyx_pw_9drf_turbo_6fields_9SlugField_3deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_9SlugField_deserialize(struct __pyx_obj_9drf_turbo_6fields_SlugField *__pyx_v_self, PyObject *__pyx_v_data, CYTHON_UNUSED PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_v_regex = NULL;
  PyObject *__pyx_v_compiled = NULL;
  PyObject *__pyx_v_match = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  int __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("deserialize", 0);

  /* "drf_turbo/fields.pyx":493
 * 
 *     cpdef inline deserialize(self,data,dict context):
 *         if self.allow_unicode :             # <<<<<<<<<<<<<<
 *             regex = re.compile(r'^[-\w]+\Z', re.UNICODE)
 *         else:
 */
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_self->allow_unicode); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 493, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "drf_turbo/fields.pyx":494
 *     cpdef inline deserialize(self,data,dict context):
 *         if self.allow_unicode :
 *             regex = re.compile(r'^[-\w]+\Z', re.UNICODE)             # <<<<<<<<<<<<<<
 *         else:
 *             regex = re.compile(r'^[-a-zA-Z0-9_]+$')
 */
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_re); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 494, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_compile); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 494, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_re); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 494, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_UNICODE); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 494, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = NULL;
    __pyx_t_6 = 0;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
        __pyx_t_6 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_kp_s_w_Z, __pyx_t_5};
      __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 494, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_kp_s_w_Z, __pyx_t_5};
      __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 494, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    } else
    #endif
    {
      __pyx_t_7 = PyTuple_New(2+__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 494, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      if (__pyx_t_3) {
        __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_3); __pyx_t_3 = NULL;
      }
      __Pyx_INCREF(__pyx_kp_s_w_Z);
      __Pyx_GIVEREF(__pyx_kp_s_w_Z);
      PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_6, __pyx_kp_s_w_Z);
      __Pyx_GIVEREF(__pyx_t_5);
      PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_6, __pyx_t_5);
      __pyx_t_5 = 0;
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_7, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 494, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_v_regex = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "drf_turbo/fields.pyx":493
 * 
 *     cpdef inline deserialize(self,data,dict context):
 *         if self.allow_unicode :             # <<<<<<<<<<<<<<
 *             regex = re.compile(r'^[-\w]+\Z', re.UNICODE)
 *         else:
 */
    goto __pyx_L3;
  }

  /* "drf_turbo/fields.pyx":496
 *             regex = re.compile(r'^[-\w]+\Z', re.UNICODE)
 *         else:
 *             regex = re.compile(r'^[-a-zA-Z0-9_]+$')             # <<<<<<<<<<<<<<
 *         compiled = re.compile(regex,flags=0)
 *         match = compiled.search(str(data))
 */
  /*else*/ {
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_re); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 496, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_compile); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 496, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
      }
    }
    __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_4, __pyx_kp_s_a_zA_Z0_9) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_kp_s_a_zA_Z0_9);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 496, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_v_regex = __pyx_t_2;
    __pyx_t_2 = 0;
  }
  __pyx_L3:;

  /* "drf_turbo/fields.pyx":497
 *         else:
 *             regex = re.compile(r'^[-a-zA-Z0-9_]+$')
 *         compiled = re.compile(regex,flags=0)             # <<<<<<<<<<<<<<
 *         match = compiled.search(str(data))
 *         if not match and not self.allow_unicode:
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_re); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 497, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_compile); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 497, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 497, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_v_regex);
  __Pyx_GIVEREF(__pyx_v_regex);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_regex);
  __pyx_t_4 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 497, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_flags, __pyx_int_0) < 0) __PYX_ERR(0, 497, __pyx_L1_error)
  __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_2, __pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 497, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_compiled = __pyx_t_5;
  __pyx_t_5 = 0;

  /* "drf_turbo/fields.pyx":498
 *             regex = re.compile(r'^[-a-zA-Z0-9_]+$')
 *         compiled = re.compile(regex,flags=0)
 *         match = compiled.search(str(data))             # <<<<<<<<<<<<<<
 *         if not match and not self.allow_unicode:
 *             raise self.raise_if_fail("invalid")
 */
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_compiled, __pyx_n_s_search); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 498, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), __pyx_v_data); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 498, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_7 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_7)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_7);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  __pyx_t_5 = (__pyx_t_7) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_7, __pyx_t_2) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_2);
  __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 498, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_match = __pyx_t_5;
  __pyx_t_5 = 0;

  /* "drf_turbo/fields.pyx":499
 *         compiled = re.compile(regex,flags=0)
 *         match = compiled.search(str(data))
 *         if not match and not self.allow_unicode:             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail("invalid")
 *         elif not match and self.allow_unicode:
 */
  __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_v_match); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 499, __pyx_L1_error)
  __pyx_t_9 = ((!__pyx_t_8) != 0);
  if (__pyx_t_9) {
  } else {
    __pyx_t_1 = __pyx_t_9;
    goto __pyx_L5_bool_binop_done;
  }
  __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_v_self->allow_unicode); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 499, __pyx_L1_error)
  __pyx_t_8 = ((!__pyx_t_9) != 0);
  __pyx_t_1 = __pyx_t_8;
  __pyx_L5_bool_binop_done:;
  if (unlikely(__pyx_t_1)) {

    /* "drf_turbo/fields.pyx":500
 *         match = compiled.search(str(data))
 *         if not match and not self.allow_unicode:
 *             raise self.raise_if_fail("invalid")             # <<<<<<<<<<<<<<
 *         elif not match and self.allow_unicode:
 *             raise self.raise_if_fail("invalid_unicode")
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_raise_if_fail); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 500, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_2 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_2)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    __pyx_t_5 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_2, __pyx_n_s_invalid) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_n_s_invalid);
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 500, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_Raise(__pyx_t_5, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __PYX_ERR(0, 500, __pyx_L1_error)

    /* "drf_turbo/fields.pyx":499
 *         compiled = re.compile(regex,flags=0)
 *         match = compiled.search(str(data))
 *         if not match and not self.allow_unicode:             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail("invalid")
 *         elif not match and self.allow_unicode:
 */
  }

  /* "drf_turbo/fields.pyx":501
 *         if not match and not self.allow_unicode:
 *             raise self.raise_if_fail("invalid")
 *         elif not match and self.allow_unicode:             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail("invalid_unicode")
 *         return str(data)
 */
  __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_v_match); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 501, __pyx_L1_error)
  __pyx_t_9 = ((!__pyx_t_8) != 0);
  if (__pyx_t_9) {
  } else {
    __pyx_t_1 = __pyx_t_9;
    goto __pyx_L7_bool_binop_done;
  }
  __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_v_self->allow_unicode); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 501, __pyx_L1_error)
  __pyx_t_1 = __pyx_t_9;
  __pyx_L7_bool_binop_done:;
  if (unlikely(__pyx_t_1)) {

    /* "drf_turbo/fields.pyx":502
 *             raise self.raise_if_fail("invalid")
 *         elif not match and self.allow_unicode:
 *             raise self.raise_if_fail("invalid_unicode")             # <<<<<<<<<<<<<<
 *         return str(data)
 * 
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_raise_if_fail); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 502, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_2 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_2)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    __pyx_t_5 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_2, __pyx_n_s_invalid_unicode) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_n_s_invalid_unicode);
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 502, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_Raise(__pyx_t_5, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __PYX_ERR(0, 502, __pyx_L1_error)

    /* "drf_turbo/fields.pyx":501
 *         if not match and not self.allow_unicode:
 *             raise self.raise_if_fail("invalid")
 *         elif not match and self.allow_unicode:             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail("invalid_unicode")
 *         return str(data)
 */
  }

  /* "drf_turbo/fields.pyx":503
 *         elif not match and self.allow_unicode:
 *             raise self.raise_if_fail("invalid_unicode")
 *         return str(data)             # <<<<<<<<<<<<<<
 * 
 * @cython.final
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_5 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), __pyx_v_data); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 503, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_r = __pyx_t_5;
  __pyx_t_5 = 0;
  goto __pyx_L0;

  /* "drf_turbo/fields.pyx":492
 *         super().__init__(**kwargs)
 * 
 *     cpdef inline deserialize(self,data,dict context):             # <<<<<<<<<<<<<<
 *         if self.allow_unicode :
 *             regex = re.compile(r'^[-\w]+\Z', re.UNICODE)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("drf_turbo.fields.SlugField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_regex);
  __Pyx_XDECREF(__pyx_v_compiled);
  __Pyx_XDECREF(__pyx_v_match);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_9SlugField_3deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_9SlugField_3deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_data = 0;
  PyObject *__pyx_v_context = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("deserialize (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_data,&__pyx_n_s_context,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_data)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_context)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("deserialize", 1, 2, 2, 1); __PYX_ERR(0, 492, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "deserialize") < 0)) __PYX_ERR(0, 492, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_data = values[0];
    __pyx_v_context = ((PyObject*)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("deserialize", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 492, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.SlugField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_context), (&PyDict_Type), 1, "context", 1))) __PYX_ERR(0, 492, __pyx_L1_error)
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9SlugField_2deserialize(((struct __pyx_obj_9drf_turbo_6fields_SlugField *)__pyx_v_self), __pyx_v_data, __pyx_v_context);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_9SlugField_2deserialize(struct __pyx_obj_9drf_turbo_6fields_SlugField *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("deserialize", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields_9SlugField_deserialize(__pyx_v_self, __pyx_v_data, __pyx_v_context, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 492, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.SlugField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pxd":66
 * cdef class SlugField(Field):
 *     cdef public:
 *         allow_unicode             # <<<<<<<<<<<<<<
 * 
 * cdef class IntField(Field):
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_9SlugField_13allow_unicode_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_9SlugField_13allow_unicode_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9SlugField_13allow_unicode___get__(((struct __pyx_obj_9drf_turbo_6fields_SlugField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_9SlugField_13allow_unicode___get__(struct __pyx_obj_9drf_turbo_6fields_SlugField *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->allow_unicode);
  __pyx_r = __pyx_v_self->allow_unicode;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_9SlugField_13allow_unicode_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_9SlugField_13allow_unicode_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9SlugField_13allow_unicode_2__set__(((struct __pyx_obj_9drf_turbo_6fields_SlugField *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_9SlugField_13allow_unicode_2__set__(struct __pyx_obj_9drf_turbo_6fields_SlugField *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->allow_unicode);
  __Pyx_DECREF(__pyx_v_self->allow_unicode);
  __pyx_v_self->allow_unicode = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_9SlugField_13allow_unicode_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_9SlugField_13allow_unicode_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9SlugField_13allow_unicode_4__del__(((struct __pyx_obj_9drf_turbo_6fields_SlugField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_9SlugField_13allow_unicode_4__del__(struct __pyx_obj_9drf_turbo_6fields_SlugField *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->allow_unicode);
  __Pyx_DECREF(__pyx_v_self->allow_unicode);
  __pyx_v_self->allow_unicode = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_9SlugField_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_9SlugField_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9SlugField_4__reduce_cython__(((struct __pyx_obj_9drf_turbo_6fields_SlugField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_9SlugField_4__reduce_cython__(struct __pyx_obj_9drf_turbo_6fields_SlugField *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self.allow_null, self.allow_unicode, self.attr, self.attrs, self.call, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.allow_null); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.call); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.read_only); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.required); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.write_only); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = PyTuple_New(17); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->allow_unicode);
  __Pyx_GIVEREF(__pyx_v_self->allow_unicode);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_v_self->allow_unicode);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.attr);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.attr);
  PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_self->__pyx_base.attr);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.attrs);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.attrs);
  PyTuple_SET_ITEM(__pyx_t_6, 3, __pyx_v_self->__pyx_base.attrs);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_6, 4, __pyx_t_2);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.default_value);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.default_value);
  PyTuple_SET_ITEM(__pyx_t_6, 5, __pyx_v_self->__pyx_base.default_value);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.error_messages);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.error_messages);
  PyTuple_SET_ITEM(__pyx_t_6, 6, __pyx_v_self->__pyx_base.error_messages);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.field_name);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.field_name);
  PyTuple_SET_ITEM(__pyx_t_6, 7, __pyx_v_self->__pyx_base.field_name);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.help_text);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.help_text);
  PyTuple_SET_ITEM(__pyx_t_6, 8, __pyx_v_self->__pyx_base.help_text);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.initial);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.initial);
  PyTuple_SET_ITEM(__pyx_t_6, 9, __pyx_v_self->__pyx_base.initial);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.label);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.label);
  PyTuple_SET_ITEM(__pyx_t_6, 10, __pyx_v_self->__pyx_base.label);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 11, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_6, 12, __pyx_t_4);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.root);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.root);
  PyTuple_SET_ITEM(__pyx_t_6, 13, __pyx_v_self->__pyx_base.root);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.style);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.style);
  PyTuple_SET_ITEM(__pyx_t_6, 14, __pyx_v_self->__pyx_base.style);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.validators);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.validators);
  PyTuple_SET_ITEM(__pyx_t_6, 15, __pyx_v_self->__pyx_base.validators);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_6, 16, __pyx_t_5);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_6);
  __pyx_t_6 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self.allow_null, self.allow_unicode, self.attr, self.attrs, self.call, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_6 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_v__dict = __pyx_t_6;
  __pyx_t_6 = 0;

  /* "(tree fragment)":7
 *     state = (self.allow_null, self.allow_unicode, self.attr, self.attrs, self.call, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_7 = (__pyx_v__dict != Py_None);
  __pyx_t_8 = (__pyx_t_7 != 0);
  if (__pyx_t_8) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_6 = PyTuple_New(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v__dict);
    __pyx_t_5 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_6); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_5));
    __pyx_t_5 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.allow_unicode is not None or self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.root is not None or self.style is not None or self.validators is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self.allow_null, self.allow_unicode, self.attr, self.attrs, self.call, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self.allow_unicode is not None or self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.root is not None or self.style is not None or self.validators is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_SlugField, (type(self), 0x7207e31, None), state
 */
  /*else*/ {
    __pyx_t_7 = (__pyx_v_self->allow_unicode != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.attr != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.attrs != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.default_value != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.error_messages != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.field_name != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.help_text != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.initial != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.label != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.root != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.style != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.validators != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    __pyx_t_8 = __pyx_t_7;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_8;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.allow_unicode is not None or self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.root is not None or self.style is not None or self.validators is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_SlugField, (type(self), 0x7207e31, None), state
 *     else:
 */
  __pyx_t_8 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_8) {

    /* "(tree fragment)":13
 *         use_setstate = self.allow_unicode is not None or self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.root is not None or self.style is not None or self.validators is not None
 *     if use_setstate:
 *         return __pyx_unpickle_SlugField, (type(self), 0x7207e31, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_SlugField, (type(self), 0x7207e31, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_pyx_unpickle_SlugField); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_6, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_119569969);
    __Pyx_GIVEREF(__pyx_int_119569969);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_int_119569969);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_6, 2, Py_None);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_6);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_state);
    __pyx_t_5 = 0;
    __pyx_t_6 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.allow_unicode is not None or self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.root is not None or self.style is not None or self.validators is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_SlugField, (type(self), 0x7207e31, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_SlugField, (type(self), 0x7207e31, None), state
 *     else:
 *         return __pyx_unpickle_SlugField, (type(self), 0x7207e31, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_SlugField__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_pyx_unpickle_SlugField); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_6, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_119569969);
    __Pyx_GIVEREF(__pyx_int_119569969);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_int_119569969);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_state);
    __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_6);
    __pyx_t_4 = 0;
    __pyx_t_6 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("drf_turbo.fields.SlugField.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_SlugField, (type(self), 0x7207e31, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_SlugField__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_9SlugField_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_9SlugField_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9SlugField_6__setstate_cython__(((struct __pyx_obj_9drf_turbo_6fields_SlugField *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_9SlugField_6__setstate_cython__(struct __pyx_obj_9drf_turbo_6fields_SlugField *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_SlugField, (type(self), 0x7207e31, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_SlugField__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields___pyx_unpickle_SlugField__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_SlugField, (type(self), 0x7207e31, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_SlugField__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.SlugField.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":520
 *     }
 *     re_decimal = re.compile(r'\.0*\s*$')  # allow e.g. '1.0' as an int, but not '1.2'
 *     def __init__(self,**kwargs):             # <<<<<<<<<<<<<<
 *         self.max_value = kwargs.pop('max_value', None)
 *         self.min_value = kwargs.pop('min_value', None)
 */

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_8IntField_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_8IntField_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_kwargs = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  if (unlikely(PyTuple_GET_SIZE(__pyx_args) > 0)) {
    __Pyx_RaiseArgtupleInvalid("__init__", 1, 0, 0, PyTuple_GET_SIZE(__pyx_args)); return -1;}
  if (__pyx_kwds && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "__init__", 1))) return -1;
  __pyx_v_kwargs = (__pyx_kwds) ? PyDict_Copy(__pyx_kwds) : PyDict_New(); if (unlikely(!__pyx_v_kwargs)) return -1;
  __Pyx_GOTREF(__pyx_v_kwargs);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_8IntField___init__(((struct __pyx_obj_9drf_turbo_6fields_IntField *)__pyx_v_self), __pyx_v_kwargs);

  /* function exit code */
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_8IntField___init__(struct __pyx_obj_9drf_turbo_6fields_IntField *__pyx_v_self, PyObject *__pyx_v_kwargs) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "drf_turbo/fields.pyx":521
 *     re_decimal = re.compile(r'\.0*\s*$')  # allow e.g. '1.0' as an int, but not '1.2'
 *     def __init__(self,**kwargs):
 *         self.max_value = kwargs.pop('max_value', None)             # <<<<<<<<<<<<<<
 *         self.min_value = kwargs.pop('min_value', None)
 *         super().__init__(**kwargs)
 */
  __pyx_t_1 = __Pyx_PyDict_Pop(__pyx_v_kwargs, __pyx_n_s_max_value, Py_None); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 521, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->max_value);
  __Pyx_DECREF(__pyx_v_self->max_value);
  __pyx_v_self->max_value = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "drf_turbo/fields.pyx":522
 *     def __init__(self,**kwargs):
 *         self.max_value = kwargs.pop('max_value', None)
 *         self.min_value = kwargs.pop('min_value', None)             # <<<<<<<<<<<<<<
 *         super().__init__(**kwargs)
 * 
 */
  __pyx_t_1 = __Pyx_PyDict_Pop(__pyx_v_kwargs, __pyx_n_s_min_value, Py_None); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 522, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->min_value);
  __Pyx_DECREF(__pyx_v_self->min_value);
  __pyx_v_self->min_value = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "drf_turbo/fields.pyx":523
 *         self.max_value = kwargs.pop('max_value', None)
 *         self.min_value = kwargs.pop('min_value', None)
 *         super().__init__(**kwargs)             # <<<<<<<<<<<<<<
 * 
 *     cpdef inline serialize(self,value,dict context):
 */
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 523, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_9drf_turbo_6fields_IntField));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_9drf_turbo_6fields_IntField));
  PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)__pyx_ptype_9drf_turbo_6fields_IntField));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_1, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 523, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_init); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 523, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyDict_Copy(__pyx_v_kwargs); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 523, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 523, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "drf_turbo/fields.pyx":520
 *     }
 *     re_decimal = re.compile(r'\.0*\s*$')  # allow e.g. '1.0' as an int, but not '1.2'
 *     def __init__(self,**kwargs):             # <<<<<<<<<<<<<<
 *         self.max_value = kwargs.pop('max_value', None)
 *         self.min_value = kwargs.pop('min_value', None)
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("drf_turbo.fields.IntField.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":525
 *         super().__init__(**kwargs)
 * 
 *     cpdef inline serialize(self,value,dict context):             # <<<<<<<<<<<<<<
 *         return int(value)
 * 
 */

static PyObject *__pyx_pw_9drf_turbo_6fields_8IntField_3serialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_8IntField_serialize(CYTHON_UNUSED struct __pyx_obj_9drf_turbo_6fields_IntField *__pyx_v_self, PyObject *__pyx_v_value, CYTHON_UNUSED PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("serialize", 0);

  /* "drf_turbo/fields.pyx":526
 * 
 *     cpdef inline serialize(self,value,dict context):
 *         return int(value)             # <<<<<<<<<<<<<<
 * 
 *     cpdef inline deserialize(self,data,dict context):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyNumber_Int(__pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 526, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "drf_turbo/fields.pyx":525
 *         super().__init__(**kwargs)
 * 
 *     cpdef inline serialize(self,value,dict context):             # <<<<<<<<<<<<<<
 *         return int(value)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.IntField.serialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_8IntField_3serialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_8IntField_3serialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_value = 0;
  PyObject *__pyx_v_context = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("serialize (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_value,&__pyx_n_s_context,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_value)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_context)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("serialize", 1, 2, 2, 1); __PYX_ERR(0, 525, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "serialize") < 0)) __PYX_ERR(0, 525, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_value = values[0];
    __pyx_v_context = ((PyObject*)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("serialize", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 525, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.IntField.serialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_context), (&PyDict_Type), 1, "context", 1))) __PYX_ERR(0, 525, __pyx_L1_error)
  __pyx_r = __pyx_pf_9drf_turbo_6fields_8IntField_2serialize(((struct __pyx_obj_9drf_turbo_6fields_IntField *)__pyx_v_self), __pyx_v_value, __pyx_v_context);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_8IntField_2serialize(struct __pyx_obj_9drf_turbo_6fields_IntField *__pyx_v_self, PyObject *__pyx_v_value, PyObject *__pyx_v_context) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("serialize", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields_8IntField_serialize(__pyx_v_self, __pyx_v_value, __pyx_v_context, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 525, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.IntField.serialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":528
 *         return int(value)
 * 
 *     cpdef inline deserialize(self,data,dict context):             # <<<<<<<<<<<<<<
 *         try:
 *             data = int(self.re_decimal.sub('', str(data)))
 */

static PyObject *__pyx_pw_9drf_turbo_6fields_8IntField_5deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_8IntField_deserialize(struct __pyx_obj_9drf_turbo_6fields_IntField *__pyx_v_self, PyObject *__pyx_v_data, CYTHON_UNUSED PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_t_11;
  int __pyx_t_12;
  int __pyx_t_13;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("deserialize", 0);
  __Pyx_INCREF(__pyx_v_data);

  /* "drf_turbo/fields.pyx":529
 * 
 *     cpdef inline deserialize(self,data,dict context):
 *         try:             # <<<<<<<<<<<<<<
 *             data = int(self.re_decimal.sub('', str(data)))
 *         except (ValueError, TypeError):
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "drf_turbo/fields.pyx":530
 *     cpdef inline deserialize(self,data,dict context):
 *         try:
 *             data = int(self.re_decimal.sub('', str(data)))             # <<<<<<<<<<<<<<
 *         except (ValueError, TypeError):
 *             raise self.raise_if_fail('invalid')
 */
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_re_decimal); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 530, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_sub); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 530, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_5 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), __pyx_v_data); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 530, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_7 = NULL;
      __pyx_t_8 = 0;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
        __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_6);
        if (likely(__pyx_t_7)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
          __Pyx_INCREF(__pyx_t_7);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_6, function);
          __pyx_t_8 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_6)) {
        PyObject *__pyx_temp[3] = {__pyx_t_7, __pyx_kp_s__9, __pyx_t_5};
        __pyx_t_4 = __Pyx_PyFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_8, 2+__pyx_t_8); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 530, __pyx_L3_error)
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_6)) {
        PyObject *__pyx_temp[3] = {__pyx_t_7, __pyx_kp_s__9, __pyx_t_5};
        __pyx_t_4 = __Pyx_PyCFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_8, 2+__pyx_t_8); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 530, __pyx_L3_error)
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      } else
      #endif
      {
        __pyx_t_9 = PyTuple_New(2+__pyx_t_8); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 530, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_9);
        if (__pyx_t_7) {
          __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_7); __pyx_t_7 = NULL;
        }
        __Pyx_INCREF(__pyx_kp_s__9);
        __Pyx_GIVEREF(__pyx_kp_s__9);
        PyTuple_SET_ITEM(__pyx_t_9, 0+__pyx_t_8, __pyx_kp_s__9);
        __Pyx_GIVEREF(__pyx_t_5);
        PyTuple_SET_ITEM(__pyx_t_9, 1+__pyx_t_8, __pyx_t_5);
        __pyx_t_5 = 0;
        __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_9, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 530, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      }
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_6 = __Pyx_PyNumber_Int(__pyx_t_4); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 530, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF_SET(__pyx_v_data, __pyx_t_6);
      __pyx_t_6 = 0;

      /* "drf_turbo/fields.pyx":529
 * 
 *     cpdef inline deserialize(self,data,dict context):
 *         try:             # <<<<<<<<<<<<<<
 *             data = int(self.re_decimal.sub('', str(data)))
 *         except (ValueError, TypeError):
 */
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;

    /* "drf_turbo/fields.pyx":531
 *         try:
 *             data = int(self.re_decimal.sub('', str(data)))
 *         except (ValueError, TypeError):             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail('invalid')
 *         if self.min_value is not None and data < self.min_value:
 */
    __pyx_t_8 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_ValueError) || __Pyx_PyErr_ExceptionMatches(__pyx_builtin_TypeError);
    if (__pyx_t_8) {
      __Pyx_AddTraceback("drf_turbo.fields.IntField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_6, &__pyx_t_4, &__pyx_t_9) < 0) __PYX_ERR(0, 531, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GOTREF(__pyx_t_9);

      /* "drf_turbo/fields.pyx":532
 *             data = int(self.re_decimal.sub('', str(data)))
 *         except (ValueError, TypeError):
 *             raise self.raise_if_fail('invalid')             # <<<<<<<<<<<<<<
 *         if self.min_value is not None and data < self.min_value:
 *             raise self.raise_if_fail("min_value",min_value=self.min_value)
 */
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_raise_if_fail); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 532, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_10 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
        __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_7);
        if (likely(__pyx_t_10)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
          __Pyx_INCREF(__pyx_t_10);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_7, function);
        }
      }
      __pyx_t_5 = (__pyx_t_10) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_10, __pyx_n_s_invalid) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_n_s_invalid);
      __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
      if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 532, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_Raise(__pyx_t_5, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __PYX_ERR(0, 532, __pyx_L5_except_error)
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;

    /* "drf_turbo/fields.pyx":529
 * 
 *     cpdef inline deserialize(self,data,dict context):
 *         try:             # <<<<<<<<<<<<<<
 *             data = int(self.re_decimal.sub('', str(data)))
 *         except (ValueError, TypeError):
 */
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L8_try_end:;
  }

  /* "drf_turbo/fields.pyx":533
 *         except (ValueError, TypeError):
 *             raise self.raise_if_fail('invalid')
 *         if self.min_value is not None and data < self.min_value:             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail("min_value",min_value=self.min_value)
 *         if self.max_value is not None and data > self.max_value:
 */
  __pyx_t_12 = (__pyx_v_self->min_value != Py_None);
  __pyx_t_13 = (__pyx_t_12 != 0);
  if (__pyx_t_13) {
  } else {
    __pyx_t_11 = __pyx_t_13;
    goto __pyx_L12_bool_binop_done;
  }
  __pyx_t_9 = PyObject_RichCompare(__pyx_v_data, __pyx_v_self->min_value, Py_LT); __Pyx_XGOTREF(__pyx_t_9); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 533, __pyx_L1_error)
  __pyx_t_13 = __Pyx_PyObject_IsTrue(__pyx_t_9); if (unlikely(__pyx_t_13 < 0)) __PYX_ERR(0, 533, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_t_11 = __pyx_t_13;
  __pyx_L12_bool_binop_done:;
  if (unlikely(__pyx_t_11)) {

    /* "drf_turbo/fields.pyx":534
 *             raise self.raise_if_fail('invalid')
 *         if self.min_value is not None and data < self.min_value:
 *             raise self.raise_if_fail("min_value",min_value=self.min_value)             # <<<<<<<<<<<<<<
 *         if self.max_value is not None and data > self.max_value:
 *             raise self.raise_if_fail("max_value",max_value=self.max_value)
 */
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_raise_if_fail); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 534, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_4 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 534, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_min_value, __pyx_v_self->min_value) < 0) __PYX_ERR(0, 534, __pyx_L1_error)
    __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_tuple__13, __pyx_t_4); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 534, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_Raise(__pyx_t_6, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __PYX_ERR(0, 534, __pyx_L1_error)

    /* "drf_turbo/fields.pyx":533
 *         except (ValueError, TypeError):
 *             raise self.raise_if_fail('invalid')
 *         if self.min_value is not None and data < self.min_value:             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail("min_value",min_value=self.min_value)
 *         if self.max_value is not None and data > self.max_value:
 */
  }

  /* "drf_turbo/fields.pyx":535
 *         if self.min_value is not None and data < self.min_value:
 *             raise self.raise_if_fail("min_value",min_value=self.min_value)
 *         if self.max_value is not None and data > self.max_value:             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail("max_value",max_value=self.max_value)
 * 
 */
  __pyx_t_13 = (__pyx_v_self->max_value != Py_None);
  __pyx_t_12 = (__pyx_t_13 != 0);
  if (__pyx_t_12) {
  } else {
    __pyx_t_11 = __pyx_t_12;
    goto __pyx_L15_bool_binop_done;
  }
  __pyx_t_6 = PyObject_RichCompare(__pyx_v_data, __pyx_v_self->max_value, Py_GT); __Pyx_XGOTREF(__pyx_t_6); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 535, __pyx_L1_error)
  __pyx_t_12 = __Pyx_PyObject_IsTrue(__pyx_t_6); if (unlikely(__pyx_t_12 < 0)) __PYX_ERR(0, 535, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_11 = __pyx_t_12;
  __pyx_L15_bool_binop_done:;
  if (unlikely(__pyx_t_11)) {

    /* "drf_turbo/fields.pyx":536
 *             raise self.raise_if_fail("min_value",min_value=self.min_value)
 *         if self.max_value is not None and data > self.max_value:
 *             raise self.raise_if_fail("max_value",max_value=self.max_value)             # <<<<<<<<<<<<<<
 * 
 *         return data
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_raise_if_fail); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 536, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_4 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 536, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_max_value, __pyx_v_self->max_value) < 0) __PYX_ERR(0, 536, __pyx_L1_error)
    __pyx_t_9 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_tuple__14, __pyx_t_4); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 536, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_Raise(__pyx_t_9, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __PYX_ERR(0, 536, __pyx_L1_error)

    /* "drf_turbo/fields.pyx":535
 *         if self.min_value is not None and data < self.min_value:
 *             raise self.raise_if_fail("min_value",min_value=self.min_value)
 *         if self.max_value is not None and data > self.max_value:             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail("max_value",max_value=self.max_value)
 * 
 */
  }

  /* "drf_turbo/fields.pyx":538
 *             raise self.raise_if_fail("max_value",max_value=self.max_value)
 * 
 *         return data             # <<<<<<<<<<<<<<
 * 
 * @cython.final
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_data);
  __pyx_r = __pyx_v_data;
  goto __pyx_L0;

  /* "drf_turbo/fields.pyx":528
 *         return int(value)
 * 
 *     cpdef inline deserialize(self,data,dict context):             # <<<<<<<<<<<<<<
 *         try:
 *             data = int(self.re_decimal.sub('', str(data)))
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("drf_turbo.fields.IntField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_data);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_8IntField_5deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_8IntField_5deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_data = 0;
  PyObject *__pyx_v_context = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("deserialize (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_data,&__pyx_n_s_context,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_data)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_context)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("deserialize", 1, 2, 2, 1); __PYX_ERR(0, 528, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "deserialize") < 0)) __PYX_ERR(0, 528, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_data = values[0];
    __pyx_v_context = ((PyObject*)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("deserialize", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 528, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.IntField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_context), (&PyDict_Type), 1, "context", 1))) __PYX_ERR(0, 528, __pyx_L1_error)
  __pyx_r = __pyx_pf_9drf_turbo_6fields_8IntField_4deserialize(((struct __pyx_obj_9drf_turbo_6fields_IntField *)__pyx_v_self), __pyx_v_data, __pyx_v_context);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_8IntField_4deserialize(struct __pyx_obj_9drf_turbo_6fields_IntField *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("deserialize", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields_8IntField_deserialize(__pyx_v_self, __pyx_v_data, __pyx_v_context, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 528, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.IntField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pxd":70
 * cdef class IntField(Field):
 *     cdef public :
 *         max_value             # <<<<<<<<<<<<<<
 *         min_value
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_8IntField_9max_value_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_8IntField_9max_value_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_8IntField_9max_value___get__(((struct __pyx_obj_9drf_turbo_6fields_IntField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_8IntField_9max_value___get__(struct __pyx_obj_9drf_turbo_6fields_IntField *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->max_value);
  __pyx_r = __pyx_v_self->max_value;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_8IntField_9max_value_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_8IntField_9max_value_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_8IntField_9max_value_2__set__(((struct __pyx_obj_9drf_turbo_6fields_IntField *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_8IntField_9max_value_2__set__(struct __pyx_obj_9drf_turbo_6fields_IntField *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->max_value);
  __Pyx_DECREF(__pyx_v_self->max_value);
  __pyx_v_self->max_value = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_8IntField_9max_value_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_8IntField_9max_value_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_8IntField_9max_value_4__del__(((struct __pyx_obj_9drf_turbo_6fields_IntField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_8IntField_9max_value_4__del__(struct __pyx_obj_9drf_turbo_6fields_IntField *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->max_value);
  __Pyx_DECREF(__pyx_v_self->max_value);
  __pyx_v_self->max_value = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pxd":71
 *     cdef public :
 *         max_value
 *         min_value             # <<<<<<<<<<<<<<
 * 
 * cdef class FloatField(Field):
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_8IntField_9min_value_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_8IntField_9min_value_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_8IntField_9min_value___get__(((struct __pyx_obj_9drf_turbo_6fields_IntField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_8IntField_9min_value___get__(struct __pyx_obj_9drf_turbo_6fields_IntField *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->min_value);
  __pyx_r = __pyx_v_self->min_value;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_8IntField_9min_value_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_8IntField_9min_value_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_8IntField_9min_value_2__set__(((struct __pyx_obj_9drf_turbo_6fields_IntField *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_8IntField_9min_value_2__set__(struct __pyx_obj_9drf_turbo_6fields_IntField *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->min_value);
  __Pyx_DECREF(__pyx_v_self->min_value);
  __pyx_v_self->min_value = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_8IntField_9min_value_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_8IntField_9min_value_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_8IntField_9min_value_4__del__(((struct __pyx_obj_9drf_turbo_6fields_IntField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_8IntField_9min_value_4__del__(struct __pyx_obj_9drf_turbo_6fields_IntField *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->min_value);
  __Pyx_DECREF(__pyx_v_self->min_value);
  __pyx_v_self->min_value = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_8IntField_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_8IntField_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_8IntField_6__reduce_cython__(((struct __pyx_obj_9drf_turbo_6fields_IntField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_8IntField_6__reduce_cython__(struct __pyx_obj_9drf_turbo_6fields_IntField *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self.allow_null, self.attr, self.attrs, self.call, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.max_value, self.min_value, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.allow_null); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.call); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.read_only); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.required); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.write_only); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = PyTuple_New(18); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.attr);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.attr);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_v_self->__pyx_base.attr);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.attrs);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.attrs);
  PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_self->__pyx_base.attrs);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_6, 3, __pyx_t_2);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.default_value);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.default_value);
  PyTuple_SET_ITEM(__pyx_t_6, 4, __pyx_v_self->__pyx_base.default_value);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.error_messages);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.error_messages);
  PyTuple_SET_ITEM(__pyx_t_6, 5, __pyx_v_self->__pyx_base.error_messages);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.field_name);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.field_name);
  PyTuple_SET_ITEM(__pyx_t_6, 6, __pyx_v_self->__pyx_base.field_name);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.help_text);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.help_text);
  PyTuple_SET_ITEM(__pyx_t_6, 7, __pyx_v_self->__pyx_base.help_text);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.initial);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.initial);
  PyTuple_SET_ITEM(__pyx_t_6, 8, __pyx_v_self->__pyx_base.initial);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.label);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.label);
  PyTuple_SET_ITEM(__pyx_t_6, 9, __pyx_v_self->__pyx_base.label);
  __Pyx_INCREF(__pyx_v_self->max_value);
  __Pyx_GIVEREF(__pyx_v_self->max_value);
  PyTuple_SET_ITEM(__pyx_t_6, 10, __pyx_v_self->max_value);
  __Pyx_INCREF(__pyx_v_self->min_value);
  __Pyx_GIVEREF(__pyx_v_self->min_value);
  PyTuple_SET_ITEM(__pyx_t_6, 11, __pyx_v_self->min_value);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 12, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_6, 13, __pyx_t_4);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.root);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.root);
  PyTuple_SET_ITEM(__pyx_t_6, 14, __pyx_v_self->__pyx_base.root);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.style);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.style);
  PyTuple_SET_ITEM(__pyx_t_6, 15, __pyx_v_self->__pyx_base.style);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.validators);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.validators);
  PyTuple_SET_ITEM(__pyx_t_6, 16, __pyx_v_self->__pyx_base.validators);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_6, 17, __pyx_t_5);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_6);
  __pyx_t_6 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self.allow_null, self.attr, self.attrs, self.call, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.max_value, self.min_value, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_6 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_v__dict = __pyx_t_6;
  __pyx_t_6 = 0;

  /* "(tree fragment)":7
 *     state = (self.allow_null, self.attr, self.attrs, self.call, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.max_value, self.min_value, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_7 = (__pyx_v__dict != Py_None);
  __pyx_t_8 = (__pyx_t_7 != 0);
  if (__pyx_t_8) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_6 = PyTuple_New(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v__dict);
    __pyx_t_5 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_6); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_5));
    __pyx_t_5 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.max_value is not None or self.min_value is not None or self.root is not None or self.style is not None or self.validators is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self.allow_null, self.attr, self.attrs, self.call, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.max_value, self.min_value, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.max_value is not None or self.min_value is not None or self.root is not None or self.style is not None or self.validators is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_IntField, (type(self), 0x66626e5, None), state
 */
  /*else*/ {
    __pyx_t_7 = (__pyx_v_self->__pyx_base.attr != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.attrs != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.default_value != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.error_messages != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.field_name != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.help_text != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.initial != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.label != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->max_value != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->min_value != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.root != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.style != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.validators != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    __pyx_t_8 = __pyx_t_9;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_8;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.max_value is not None or self.min_value is not None or self.root is not None or self.style is not None or self.validators is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_IntField, (type(self), 0x66626e5, None), state
 *     else:
 */
  __pyx_t_8 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_8) {

    /* "(tree fragment)":13
 *         use_setstate = self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.max_value is not None or self.min_value is not None or self.root is not None or self.style is not None or self.validators is not None
 *     if use_setstate:
 *         return __pyx_unpickle_IntField, (type(self), 0x66626e5, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_IntField, (type(self), 0x66626e5, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_pyx_unpickle_IntField); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_6, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_107357925);
    __Pyx_GIVEREF(__pyx_int_107357925);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_int_107357925);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_6, 2, Py_None);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_6);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_state);
    __pyx_t_5 = 0;
    __pyx_t_6 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.max_value is not None or self.min_value is not None or self.root is not None or self.style is not None or self.validators is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_IntField, (type(self), 0x66626e5, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_IntField, (type(self), 0x66626e5, None), state
 *     else:
 *         return __pyx_unpickle_IntField, (type(self), 0x66626e5, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_IntField__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_pyx_unpickle_IntField); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_6, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_107357925);
    __Pyx_GIVEREF(__pyx_int_107357925);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_int_107357925);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_state);
    __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_6);
    __pyx_t_4 = 0;
    __pyx_t_6 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("drf_turbo.fields.IntField.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_IntField, (type(self), 0x66626e5, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_IntField__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_8IntField_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_8IntField_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_8IntField_8__setstate_cython__(((struct __pyx_obj_9drf_turbo_6fields_IntField *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_8IntField_8__setstate_cython__(struct __pyx_obj_9drf_turbo_6fields_IntField *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_IntField, (type(self), 0x66626e5, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_IntField__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields___pyx_unpickle_IntField__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_IntField, (type(self), 0x66626e5, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_IntField__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.IntField.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":557
 *     }
 * 
 *     def __init__(self,**kwargs):             # <<<<<<<<<<<<<<
 *         self.max_value = kwargs.pop('max_value', None)
 *         self.min_value = kwargs.pop('min_value', None)
 */

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_10FloatField_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_10FloatField_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_kwargs = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  if (unlikely(PyTuple_GET_SIZE(__pyx_args) > 0)) {
    __Pyx_RaiseArgtupleInvalid("__init__", 1, 0, 0, PyTuple_GET_SIZE(__pyx_args)); return -1;}
  if (__pyx_kwds && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "__init__", 1))) return -1;
  __pyx_v_kwargs = (__pyx_kwds) ? PyDict_Copy(__pyx_kwds) : PyDict_New(); if (unlikely(!__pyx_v_kwargs)) return -1;
  __Pyx_GOTREF(__pyx_v_kwargs);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_10FloatField___init__(((struct __pyx_obj_9drf_turbo_6fields_FloatField *)__pyx_v_self), __pyx_v_kwargs);

  /* function exit code */
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_10FloatField___init__(struct __pyx_obj_9drf_turbo_6fields_FloatField *__pyx_v_self, PyObject *__pyx_v_kwargs) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "drf_turbo/fields.pyx":558
 * 
 *     def __init__(self,**kwargs):
 *         self.max_value = kwargs.pop('max_value', None)             # <<<<<<<<<<<<<<
 *         self.min_value = kwargs.pop('min_value', None)
 *         super().__init__(**kwargs)
 */
  __pyx_t_1 = __Pyx_PyDict_Pop(__pyx_v_kwargs, __pyx_n_s_max_value, Py_None); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 558, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->max_value);
  __Pyx_DECREF(__pyx_v_self->max_value);
  __pyx_v_self->max_value = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "drf_turbo/fields.pyx":559
 *     def __init__(self,**kwargs):
 *         self.max_value = kwargs.pop('max_value', None)
 *         self.min_value = kwargs.pop('min_value', None)             # <<<<<<<<<<<<<<
 *         super().__init__(**kwargs)
 * 
 */
  __pyx_t_1 = __Pyx_PyDict_Pop(__pyx_v_kwargs, __pyx_n_s_min_value, Py_None); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 559, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->min_value);
  __Pyx_DECREF(__pyx_v_self->min_value);
  __pyx_v_self->min_value = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "drf_turbo/fields.pyx":560
 *         self.max_value = kwargs.pop('max_value', None)
 *         self.min_value = kwargs.pop('min_value', None)
 *         super().__init__(**kwargs)             # <<<<<<<<<<<<<<
 * 
 *     cpdef inline serialize(self,value,dict context):
 */
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 560, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_9drf_turbo_6fields_FloatField));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_9drf_turbo_6fields_FloatField));
  PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)__pyx_ptype_9drf_turbo_6fields_FloatField));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_1, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 560, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_init); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 560, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyDict_Copy(__pyx_v_kwargs); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 560, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 560, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "drf_turbo/fields.pyx":557
 *     }
 * 
 *     def __init__(self,**kwargs):             # <<<<<<<<<<<<<<
 *         self.max_value = kwargs.pop('max_value', None)
 *         self.min_value = kwargs.pop('min_value', None)
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("drf_turbo.fields.FloatField.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":562
 *         super().__init__(**kwargs)
 * 
 *     cpdef inline serialize(self,value,dict context):             # <<<<<<<<<<<<<<
 *         return float(value)
 * 
 */

static PyObject *__pyx_pw_9drf_turbo_6fields_10FloatField_3serialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_10FloatField_serialize(CYTHON_UNUSED struct __pyx_obj_9drf_turbo_6fields_FloatField *__pyx_v_self, PyObject *__pyx_v_value, CYTHON_UNUSED PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("serialize", 0);

  /* "drf_turbo/fields.pyx":563
 * 
 *     cpdef inline serialize(self,value,dict context):
 *         return float(value)             # <<<<<<<<<<<<<<
 * 
 *     cpdef inline deserialize(self,data,dict context):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyNumber_Float(__pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 563, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "drf_turbo/fields.pyx":562
 *         super().__init__(**kwargs)
 * 
 *     cpdef inline serialize(self,value,dict context):             # <<<<<<<<<<<<<<
 *         return float(value)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.FloatField.serialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_10FloatField_3serialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_10FloatField_3serialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_value = 0;
  PyObject *__pyx_v_context = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("serialize (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_value,&__pyx_n_s_context,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_value)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_context)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("serialize", 1, 2, 2, 1); __PYX_ERR(0, 562, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "serialize") < 0)) __PYX_ERR(0, 562, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_value = values[0];
    __pyx_v_context = ((PyObject*)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("serialize", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 562, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.FloatField.serialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_context), (&PyDict_Type), 1, "context", 1))) __PYX_ERR(0, 562, __pyx_L1_error)
  __pyx_r = __pyx_pf_9drf_turbo_6fields_10FloatField_2serialize(((struct __pyx_obj_9drf_turbo_6fields_FloatField *)__pyx_v_self), __pyx_v_value, __pyx_v_context);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_10FloatField_2serialize(struct __pyx_obj_9drf_turbo_6fields_FloatField *__pyx_v_self, PyObject *__pyx_v_value, PyObject *__pyx_v_context) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("serialize", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields_10FloatField_serialize(__pyx_v_self, __pyx_v_value, __pyx_v_context, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 562, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.FloatField.serialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":565
 *         return float(value)
 * 
 *     cpdef inline deserialize(self,data,dict context):             # <<<<<<<<<<<<<<
 *         try:
 *             data =  float(data)
 */

static PyObject *__pyx_pw_9drf_turbo_6fields_10FloatField_5deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_10FloatField_deserialize(struct __pyx_obj_9drf_turbo_6fields_FloatField *__pyx_v_self, PyObject *__pyx_v_data, CYTHON_UNUSED PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_t_11;
  int __pyx_t_12;
  int __pyx_t_13;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("deserialize", 0);
  __Pyx_INCREF(__pyx_v_data);

  /* "drf_turbo/fields.pyx":566
 * 
 *     cpdef inline deserialize(self,data,dict context):
 *         try:             # <<<<<<<<<<<<<<
 *             data =  float(data)
 *         except (TypeError, ValueError):
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "drf_turbo/fields.pyx":567
 *     cpdef inline deserialize(self,data,dict context):
 *         try:
 *             data =  float(data)             # <<<<<<<<<<<<<<
 *         except (TypeError, ValueError):
 *            raise self.raise_if_fail('invalid')
 */
      __pyx_t_4 = __Pyx_PyNumber_Float(__pyx_v_data); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 567, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF_SET(__pyx_v_data, __pyx_t_4);
      __pyx_t_4 = 0;

      /* "drf_turbo/fields.pyx":566
 * 
 *     cpdef inline deserialize(self,data,dict context):
 *         try:             # <<<<<<<<<<<<<<
 *             data =  float(data)
 *         except (TypeError, ValueError):
 */
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "drf_turbo/fields.pyx":568
 *         try:
 *             data =  float(data)
 *         except (TypeError, ValueError):             # <<<<<<<<<<<<<<
 *            raise self.raise_if_fail('invalid')
 * 
 */
    __pyx_t_5 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_TypeError) || __Pyx_PyErr_ExceptionMatches(__pyx_builtin_ValueError);
    if (__pyx_t_5) {
      __Pyx_AddTraceback("drf_turbo.fields.FloatField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_4, &__pyx_t_6, &__pyx_t_7) < 0) __PYX_ERR(0, 568, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GOTREF(__pyx_t_7);

      /* "drf_turbo/fields.pyx":569
 *             data =  float(data)
 *         except (TypeError, ValueError):
 *            raise self.raise_if_fail('invalid')             # <<<<<<<<<<<<<<
 * 
 *         if self.min_value is not None and data < self.min_value:
 */
      __pyx_t_9 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_raise_if_fail); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 569, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_10 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_9))) {
        __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_9);
        if (likely(__pyx_t_10)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
          __Pyx_INCREF(__pyx_t_10);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_9, function);
        }
      }
      __pyx_t_8 = (__pyx_t_10) ? __Pyx_PyObject_Call2Args(__pyx_t_9, __pyx_t_10, __pyx_n_s_invalid) : __Pyx_PyObject_CallOneArg(__pyx_t_9, __pyx_n_s_invalid);
      __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
      if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 569, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_Raise(__pyx_t_8, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __PYX_ERR(0, 569, __pyx_L5_except_error)
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;

    /* "drf_turbo/fields.pyx":566
 * 
 *     cpdef inline deserialize(self,data,dict context):
 *         try:             # <<<<<<<<<<<<<<
 *             data =  float(data)
 *         except (TypeError, ValueError):
 */
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L8_try_end:;
  }

  /* "drf_turbo/fields.pyx":571
 *            raise self.raise_if_fail('invalid')
 * 
 *         if self.min_value is not None and data < self.min_value:             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail("min_value",min_value=self.min_value)
 *         if self.max_value is not None and data > self.max_value:
 */
  __pyx_t_12 = (__pyx_v_self->min_value != Py_None);
  __pyx_t_13 = (__pyx_t_12 != 0);
  if (__pyx_t_13) {
  } else {
    __pyx_t_11 = __pyx_t_13;
    goto __pyx_L12_bool_binop_done;
  }
  __pyx_t_7 = PyObject_RichCompare(__pyx_v_data, __pyx_v_self->min_value, Py_LT); __Pyx_XGOTREF(__pyx_t_7); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 571, __pyx_L1_error)
  __pyx_t_13 = __Pyx_PyObject_IsTrue(__pyx_t_7); if (unlikely(__pyx_t_13 < 0)) __PYX_ERR(0, 571, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_11 = __pyx_t_13;
  __pyx_L12_bool_binop_done:;
  if (unlikely(__pyx_t_11)) {

    /* "drf_turbo/fields.pyx":572
 * 
 *         if self.min_value is not None and data < self.min_value:
 *             raise self.raise_if_fail("min_value",min_value=self.min_value)             # <<<<<<<<<<<<<<
 *         if self.max_value is not None and data > self.max_value:
 *             raise self.raise_if_fail("max_value",max_value=self.max_value)
 */
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_raise_if_fail); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 572, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_6 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 572, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    if (PyDict_SetItem(__pyx_t_6, __pyx_n_s_min_value, __pyx_v_self->min_value) < 0) __PYX_ERR(0, 572, __pyx_L1_error)
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_tuple__13, __pyx_t_6); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 572, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(0, 572, __pyx_L1_error)

    /* "drf_turbo/fields.pyx":571
 *            raise self.raise_if_fail('invalid')
 * 
 *         if self.min_value is not None and data < self.min_value:             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail("min_value",min_value=self.min_value)
 *         if self.max_value is not None and data > self.max_value:
 */
  }

  /* "drf_turbo/fields.pyx":573
 *         if self.min_value is not None and data < self.min_value:
 *             raise self.raise_if_fail("min_value",min_value=self.min_value)
 *         if self.max_value is not None and data > self.max_value:             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail("max_value",max_value=self.max_value)
 * 
 */
  __pyx_t_13 = (__pyx_v_self->max_value != Py_None);
  __pyx_t_12 = (__pyx_t_13 != 0);
  if (__pyx_t_12) {
  } else {
    __pyx_t_11 = __pyx_t_12;
    goto __pyx_L15_bool_binop_done;
  }
  __pyx_t_4 = PyObject_RichCompare(__pyx_v_data, __pyx_v_self->max_value, Py_GT); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 573, __pyx_L1_error)
  __pyx_t_12 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_12 < 0)) __PYX_ERR(0, 573, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_11 = __pyx_t_12;
  __pyx_L15_bool_binop_done:;
  if (unlikely(__pyx_t_11)) {

    /* "drf_turbo/fields.pyx":574
 *             raise self.raise_if_fail("min_value",min_value=self.min_value)
 *         if self.max_value is not None and data > self.max_value:
 *             raise self.raise_if_fail("max_value",max_value=self.max_value)             # <<<<<<<<<<<<<<
 * 
 *         return data
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_raise_if_fail); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 574, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 574, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    if (PyDict_SetItem(__pyx_t_6, __pyx_n_s_max_value, __pyx_v_self->max_value) < 0) __PYX_ERR(0, 574, __pyx_L1_error)
    __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_tuple__14, __pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 574, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_Raise(__pyx_t_7, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __PYX_ERR(0, 574, __pyx_L1_error)

    /* "drf_turbo/fields.pyx":573
 *         if self.min_value is not None and data < self.min_value:
 *             raise self.raise_if_fail("min_value",min_value=self.min_value)
 *         if self.max_value is not None and data > self.max_value:             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail("max_value",max_value=self.max_value)
 * 
 */
  }

  /* "drf_turbo/fields.pyx":576
 *             raise self.raise_if_fail("max_value",max_value=self.max_value)
 * 
 *         return data             # <<<<<<<<<<<<<<
 * 
 * @cython.final
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_data);
  __pyx_r = __pyx_v_data;
  goto __pyx_L0;

  /* "drf_turbo/fields.pyx":565
 *         return float(value)
 * 
 *     cpdef inline deserialize(self,data,dict context):             # <<<<<<<<<<<<<<
 *         try:
 *             data =  float(data)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("drf_turbo.fields.FloatField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_data);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_10FloatField_5deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_10FloatField_5deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_data = 0;
  PyObject *__pyx_v_context = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("deserialize (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_data,&__pyx_n_s_context,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_data)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_context)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("deserialize", 1, 2, 2, 1); __PYX_ERR(0, 565, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "deserialize") < 0)) __PYX_ERR(0, 565, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_data = values[0];
    __pyx_v_context = ((PyObject*)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("deserialize", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 565, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.FloatField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_context), (&PyDict_Type), 1, "context", 1))) __PYX_ERR(0, 565, __pyx_L1_error)
  __pyx_r = __pyx_pf_9drf_turbo_6fields_10FloatField_4deserialize(((struct __pyx_obj_9drf_turbo_6fields_FloatField *)__pyx_v_self), __pyx_v_data, __pyx_v_context);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_10FloatField_4deserialize(struct __pyx_obj_9drf_turbo_6fields_FloatField *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("deserialize", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields_10FloatField_deserialize(__pyx_v_self, __pyx_v_data, __pyx_v_context, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 565, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.FloatField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pxd":75
 * cdef class FloatField(Field):
 *     cdef public :
 *         max_value             # <<<<<<<<<<<<<<
 *         min_value
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_10FloatField_9max_value_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_10FloatField_9max_value_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_10FloatField_9max_value___get__(((struct __pyx_obj_9drf_turbo_6fields_FloatField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_10FloatField_9max_value___get__(struct __pyx_obj_9drf_turbo_6fields_FloatField *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->max_value);
  __pyx_r = __pyx_v_self->max_value;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_10FloatField_9max_value_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_10FloatField_9max_value_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_10FloatField_9max_value_2__set__(((struct __pyx_obj_9drf_turbo_6fields_FloatField *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_10FloatField_9max_value_2__set__(struct __pyx_obj_9drf_turbo_6fields_FloatField *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->max_value);
  __Pyx_DECREF(__pyx_v_self->max_value);
  __pyx_v_self->max_value = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_10FloatField_9max_value_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_10FloatField_9max_value_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_10FloatField_9max_value_4__del__(((struct __pyx_obj_9drf_turbo_6fields_FloatField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_10FloatField_9max_value_4__del__(struct __pyx_obj_9drf_turbo_6fields_FloatField *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->max_value);
  __Pyx_DECREF(__pyx_v_self->max_value);
  __pyx_v_self->max_value = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pxd":76
 *     cdef public :
 *         max_value
 *         min_value             # <<<<<<<<<<<<<<
 * 
 * cdef class DecimalField(Field):
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_10FloatField_9min_value_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_10FloatField_9min_value_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_10FloatField_9min_value___get__(((struct __pyx_obj_9drf_turbo_6fields_FloatField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_10FloatField_9min_value___get__(struct __pyx_obj_9drf_turbo_6fields_FloatField *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->min_value);
  __pyx_r = __pyx_v_self->min_value;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_10FloatField_9min_value_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_10FloatField_9min_value_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_10FloatField_9min_value_2__set__(((struct __pyx_obj_9drf_turbo_6fields_FloatField *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_10FloatField_9min_value_2__set__(struct __pyx_obj_9drf_turbo_6fields_FloatField *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->min_value);
  __Pyx_DECREF(__pyx_v_self->min_value);
  __pyx_v_self->min_value = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_10FloatField_9min_value_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_10FloatField_9min_value_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_10FloatField_9min_value_4__del__(((struct __pyx_obj_9drf_turbo_6fields_FloatField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_10FloatField_9min_value_4__del__(struct __pyx_obj_9drf_turbo_6fields_FloatField *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->min_value);
  __Pyx_DECREF(__pyx_v_self->min_value);
  __pyx_v_self->min_value = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_10FloatField_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_10FloatField_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_10FloatField_6__reduce_cython__(((struct __pyx_obj_9drf_turbo_6fields_FloatField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_10FloatField_6__reduce_cython__(struct __pyx_obj_9drf_turbo_6fields_FloatField *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self.allow_null, self.attr, self.attrs, self.call, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.max_value, self.min_value, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.allow_null); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.call); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.read_only); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.required); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.write_only); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = PyTuple_New(18); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.attr);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.attr);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_v_self->__pyx_base.attr);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.attrs);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.attrs);
  PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_self->__pyx_base.attrs);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_6, 3, __pyx_t_2);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.default_value);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.default_value);
  PyTuple_SET_ITEM(__pyx_t_6, 4, __pyx_v_self->__pyx_base.default_value);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.error_messages);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.error_messages);
  PyTuple_SET_ITEM(__pyx_t_6, 5, __pyx_v_self->__pyx_base.error_messages);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.field_name);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.field_name);
  PyTuple_SET_ITEM(__pyx_t_6, 6, __pyx_v_self->__pyx_base.field_name);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.help_text);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.help_text);
  PyTuple_SET_ITEM(__pyx_t_6, 7, __pyx_v_self->__pyx_base.help_text);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.initial);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.initial);
  PyTuple_SET_ITEM(__pyx_t_6, 8, __pyx_v_self->__pyx_base.initial);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.label);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.label);
  PyTuple_SET_ITEM(__pyx_t_6, 9, __pyx_v_self->__pyx_base.label);
  __Pyx_INCREF(__pyx_v_self->max_value);
  __Pyx_GIVEREF(__pyx_v_self->max_value);
  PyTuple_SET_ITEM(__pyx_t_6, 10, __pyx_v_self->max_value);
  __Pyx_INCREF(__pyx_v_self->min_value);
  __Pyx_GIVEREF(__pyx_v_self->min_value);
  PyTuple_SET_ITEM(__pyx_t_6, 11, __pyx_v_self->min_value);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 12, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_6, 13, __pyx_t_4);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.root);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.root);
  PyTuple_SET_ITEM(__pyx_t_6, 14, __pyx_v_self->__pyx_base.root);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.style);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.style);
  PyTuple_SET_ITEM(__pyx_t_6, 15, __pyx_v_self->__pyx_base.style);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.validators);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.validators);
  PyTuple_SET_ITEM(__pyx_t_6, 16, __pyx_v_self->__pyx_base.validators);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_6, 17, __pyx_t_5);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_6);
  __pyx_t_6 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self.allow_null, self.attr, self.attrs, self.call, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.max_value, self.min_value, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_6 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_v__dict = __pyx_t_6;
  __pyx_t_6 = 0;

  /* "(tree fragment)":7
 *     state = (self.allow_null, self.attr, self.attrs, self.call, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.max_value, self.min_value, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_7 = (__pyx_v__dict != Py_None);
  __pyx_t_8 = (__pyx_t_7 != 0);
  if (__pyx_t_8) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_6 = PyTuple_New(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v__dict);
    __pyx_t_5 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_6); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_5));
    __pyx_t_5 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.max_value is not None or self.min_value is not None or self.root is not None or self.style is not None or self.validators is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self.allow_null, self.attr, self.attrs, self.call, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.max_value, self.min_value, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.max_value is not None or self.min_value is not None or self.root is not None or self.style is not None or self.validators is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_FloatField, (type(self), 0x66626e5, None), state
 */
  /*else*/ {
    __pyx_t_7 = (__pyx_v_self->__pyx_base.attr != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.attrs != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.default_value != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.error_messages != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.field_name != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.help_text != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.initial != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.label != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->max_value != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->min_value != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.root != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.style != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.validators != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    __pyx_t_8 = __pyx_t_9;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_8;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.max_value is not None or self.min_value is not None or self.root is not None or self.style is not None or self.validators is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_FloatField, (type(self), 0x66626e5, None), state
 *     else:
 */
  __pyx_t_8 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_8) {

    /* "(tree fragment)":13
 *         use_setstate = self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.max_value is not None or self.min_value is not None or self.root is not None or self.style is not None or self.validators is not None
 *     if use_setstate:
 *         return __pyx_unpickle_FloatField, (type(self), 0x66626e5, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_FloatField, (type(self), 0x66626e5, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_pyx_unpickle_FloatField); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_6, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_107357925);
    __Pyx_GIVEREF(__pyx_int_107357925);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_int_107357925);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_6, 2, Py_None);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_6);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_state);
    __pyx_t_5 = 0;
    __pyx_t_6 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.max_value is not None or self.min_value is not None or self.root is not None or self.style is not None or self.validators is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_FloatField, (type(self), 0x66626e5, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_FloatField, (type(self), 0x66626e5, None), state
 *     else:
 *         return __pyx_unpickle_FloatField, (type(self), 0x66626e5, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_FloatField__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_pyx_unpickle_FloatField); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_6, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_107357925);
    __Pyx_GIVEREF(__pyx_int_107357925);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_int_107357925);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_state);
    __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_6);
    __pyx_t_4 = 0;
    __pyx_t_6 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("drf_turbo.fields.FloatField.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_FloatField, (type(self), 0x66626e5, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_FloatField__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_10FloatField_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_10FloatField_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_10FloatField_8__setstate_cython__(((struct __pyx_obj_9drf_turbo_6fields_FloatField *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_10FloatField_8__setstate_cython__(struct __pyx_obj_9drf_turbo_6fields_FloatField *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_FloatField, (type(self), 0x66626e5, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_FloatField__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields___pyx_unpickle_FloatField__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_FloatField, (type(self), 0x66626e5, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_FloatField__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.FloatField.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":603
 *     MAX_STRING_LENGTH = 1000  # Guard against malicious string inputs.
 * 
 *     def __init__(self, max_digits, decimal_places, max_value=None, min_value=None,coerce_to_string=None,rounding=None,**kwargs):             # <<<<<<<<<<<<<<
 *         self.max_digits = max_digits
 *         self.decimal_places = decimal_places
 */

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_12DecimalField_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_12DecimalField_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_max_digits = 0;
  PyObject *__pyx_v_decimal_places = 0;
  PyObject *__pyx_v_max_value = 0;
  PyObject *__pyx_v_min_value = 0;
  PyObject *__pyx_v_coerce_to_string = 0;
  PyObject *__pyx_v_rounding = 0;
  PyObject *__pyx_v_kwargs = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  __pyx_v_kwargs = PyDict_New(); if (unlikely(!__pyx_v_kwargs)) return -1;
  __Pyx_GOTREF(__pyx_v_kwargs);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_max_digits,&__pyx_n_s_decimal_places,&__pyx_n_s_max_value,&__pyx_n_s_min_value,&__pyx_n_s_coerce_to_string,&__pyx_n_s_rounding,0};
    PyObject* values[6] = {0,0,0,0,0,0};
    values[2] = ((PyObject *)Py_None);
    values[3] = ((PyObject *)Py_None);
    values[4] = ((PyObject *)Py_None);
    values[5] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_max_digits)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_decimal_places)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 0, 2, 6, 1); __PYX_ERR(0, 603, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_max_value);
          if (value) { values[2] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_min_value);
          if (value) { values[3] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_coerce_to_string);
          if (value) { values[4] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  5:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_rounding);
          if (value) { values[5] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, __pyx_v_kwargs, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 603, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_max_digits = values[0];
    __pyx_v_decimal_places = values[1];
    __pyx_v_max_value = values[2];
    __pyx_v_min_value = values[3];
    __pyx_v_coerce_to_string = values[4];
    __pyx_v_rounding = values[5];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 2, 6, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 603, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_DECREF(__pyx_v_kwargs); __pyx_v_kwargs = 0;
  __Pyx_AddTraceback("drf_turbo.fields.DecimalField.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9drf_turbo_6fields_12DecimalField___init__(((struct __pyx_obj_9drf_turbo_6fields_DecimalField *)__pyx_v_self), __pyx_v_max_digits, __pyx_v_decimal_places, __pyx_v_max_value, __pyx_v_min_value, __pyx_v_coerce_to_string, __pyx_v_rounding, __pyx_v_kwargs);

  /* function exit code */
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_12DecimalField___init__(struct __pyx_obj_9drf_turbo_6fields_DecimalField *__pyx_v_self, PyObject *__pyx_v_max_digits, PyObject *__pyx_v_decimal_places, PyObject *__pyx_v_max_value, PyObject *__pyx_v_min_value, PyObject *__pyx_v_coerce_to_string, PyObject *__pyx_v_rounding, PyObject *__pyx_v_kwargs) {
  PyObject *__pyx_v_valid_roundings = NULL;
  PyObject *__pyx_v_k = NULL;
  PyObject *__pyx_v_v = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  Py_ssize_t __pyx_t_8;
  PyObject *(*__pyx_t_9)(PyObject *);
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *(*__pyx_t_12)(PyObject *);
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "drf_turbo/fields.pyx":604
 * 
 *     def __init__(self, max_digits, decimal_places, max_value=None, min_value=None,coerce_to_string=None,rounding=None,**kwargs):
 *         self.max_digits = max_digits             # <<<<<<<<<<<<<<
 *         self.decimal_places = decimal_places
 *         self.max_value = max_value
 */
  __Pyx_INCREF(__pyx_v_max_digits);
  __Pyx_GIVEREF(__pyx_v_max_digits);
  __Pyx_GOTREF(__pyx_v_self->max_digits);
  __Pyx_DECREF(__pyx_v_self->max_digits);
  __pyx_v_self->max_digits = __pyx_v_max_digits;

  /* "drf_turbo/fields.pyx":605
 *     def __init__(self, max_digits, decimal_places, max_value=None, min_value=None,coerce_to_string=None,rounding=None,**kwargs):
 *         self.max_digits = max_digits
 *         self.decimal_places = decimal_places             # <<<<<<<<<<<<<<
 *         self.max_value = max_value
 *         self.min_value = min_value
 */
  __Pyx_INCREF(__pyx_v_decimal_places);
  __Pyx_GIVEREF(__pyx_v_decimal_places);
  __Pyx_GOTREF(__pyx_v_self->decimal_places);
  __Pyx_DECREF(__pyx_v_self->decimal_places);
  __pyx_v_self->decimal_places = __pyx_v_decimal_places;

  /* "drf_turbo/fields.pyx":606
 *         self.max_digits = max_digits
 *         self.decimal_places = decimal_places
 *         self.max_value = max_value             # <<<<<<<<<<<<<<
 *         self.min_value = min_value
 *         if self.max_digits is not None and self.decimal_places is not None:
 */
  __Pyx_INCREF(__pyx_v_max_value);
  __Pyx_GIVEREF(__pyx_v_max_value);
  __Pyx_GOTREF(__pyx_v_self->max_value);
  __Pyx_DECREF(__pyx_v_self->max_value);
  __pyx_v_self->max_value = __pyx_v_max_value;

  /* "drf_turbo/fields.pyx":607
 *         self.decimal_places = decimal_places
 *         self.max_value = max_value
 *         self.min_value = min_value             # <<<<<<<<<<<<<<
 *         if self.max_digits is not None and self.decimal_places is not None:
 *             self.max_whole_digits = self.max_digits - self.decimal_places
 */
  __Pyx_INCREF(__pyx_v_min_value);
  __Pyx_GIVEREF(__pyx_v_min_value);
  __Pyx_GOTREF(__pyx_v_self->min_value);
  __Pyx_DECREF(__pyx_v_self->min_value);
  __pyx_v_self->min_value = __pyx_v_min_value;

  /* "drf_turbo/fields.pyx":608
 *         self.max_value = max_value
 *         self.min_value = min_value
 *         if self.max_digits is not None and self.decimal_places is not None:             # <<<<<<<<<<<<<<
 *             self.max_whole_digits = self.max_digits - self.decimal_places
 *         else:
 */
  __pyx_t_2 = (__pyx_v_self->max_digits != Py_None);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {
  } else {
    __pyx_t_1 = __pyx_t_3;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_3 = (__pyx_v_self->decimal_places != Py_None);
  __pyx_t_2 = (__pyx_t_3 != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "drf_turbo/fields.pyx":609
 *         self.min_value = min_value
 *         if self.max_digits is not None and self.decimal_places is not None:
 *             self.max_whole_digits = self.max_digits - self.decimal_places             # <<<<<<<<<<<<<<
 *         else:
 *             self.max_whole_digits = None
 */
    __pyx_t_4 = PyNumber_Subtract(__pyx_v_self->max_digits, __pyx_v_self->decimal_places); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 609, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_4);
    __Pyx_GOTREF(__pyx_v_self->max_whole_digits);
    __Pyx_DECREF(__pyx_v_self->max_whole_digits);
    __pyx_v_self->max_whole_digits = __pyx_t_4;
    __pyx_t_4 = 0;

    /* "drf_turbo/fields.pyx":608
 *         self.max_value = max_value
 *         self.min_value = min_value
 *         if self.max_digits is not None and self.decimal_places is not None:             # <<<<<<<<<<<<<<
 *             self.max_whole_digits = self.max_digits - self.decimal_places
 *         else:
 */
    goto __pyx_L3;
  }

  /* "drf_turbo/fields.pyx":611
 *             self.max_whole_digits = self.max_digits - self.decimal_places
 *         else:
 *             self.max_whole_digits = None             # <<<<<<<<<<<<<<
 * 
 *         if coerce_to_string is None:
 */
  /*else*/ {
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    __Pyx_GOTREF(__pyx_v_self->max_whole_digits);
    __Pyx_DECREF(__pyx_v_self->max_whole_digits);
    __pyx_v_self->max_whole_digits = Py_None;
  }
  __pyx_L3:;

  /* "drf_turbo/fields.pyx":613
 *             self.max_whole_digits = None
 * 
 *         if coerce_to_string is None:             # <<<<<<<<<<<<<<
 *             self.coerce_to_string = api_settings.COERCE_DECIMAL_TO_STRING
 *         else:
 */
  __pyx_t_1 = (__pyx_v_coerce_to_string == Py_None);
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "drf_turbo/fields.pyx":614
 * 
 *         if coerce_to_string is None:
 *             self.coerce_to_string = api_settings.COERCE_DECIMAL_TO_STRING             # <<<<<<<<<<<<<<
 *         else:
 *             self.coerce_to_string = coerce_to_string
 */
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_api_settings); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 614, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_COERCE_DECIMAL_TO_STRING); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 614, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GIVEREF(__pyx_t_5);
    __Pyx_GOTREF(__pyx_v_self->coerce_to_string);
    __Pyx_DECREF(__pyx_v_self->coerce_to_string);
    __pyx_v_self->coerce_to_string = __pyx_t_5;
    __pyx_t_5 = 0;

    /* "drf_turbo/fields.pyx":613
 *             self.max_whole_digits = None
 * 
 *         if coerce_to_string is None:             # <<<<<<<<<<<<<<
 *             self.coerce_to_string = api_settings.COERCE_DECIMAL_TO_STRING
 *         else:
 */
    goto __pyx_L6;
  }

  /* "drf_turbo/fields.pyx":616
 *             self.coerce_to_string = api_settings.COERCE_DECIMAL_TO_STRING
 *         else:
 *             self.coerce_to_string = coerce_to_string             # <<<<<<<<<<<<<<
 * 
 *         super().__init__(**kwargs)
 */
  /*else*/ {
    __Pyx_INCREF(__pyx_v_coerce_to_string);
    __Pyx_GIVEREF(__pyx_v_coerce_to_string);
    __Pyx_GOTREF(__pyx_v_self->coerce_to_string);
    __Pyx_DECREF(__pyx_v_self->coerce_to_string);
    __pyx_v_self->coerce_to_string = __pyx_v_coerce_to_string;
  }
  __pyx_L6:;

  /* "drf_turbo/fields.pyx":618
 *             self.coerce_to_string = coerce_to_string
 * 
 *         super().__init__(**kwargs)             # <<<<<<<<<<<<<<
 * 
 *         if rounding is not None:
 */
  __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 618, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_9drf_turbo_6fields_DecimalField));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_9drf_turbo_6fields_DecimalField));
  PyTuple_SET_ITEM(__pyx_t_5, 0, ((PyObject *)__pyx_ptype_9drf_turbo_6fields_DecimalField));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_5, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_5, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 618, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_init); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 618, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyDict_Copy(__pyx_v_kwargs); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 618, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_empty_tuple, __pyx_t_4); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 618, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

  /* "drf_turbo/fields.pyx":620
 *         super().__init__(**kwargs)
 * 
 *         if rounding is not None:             # <<<<<<<<<<<<<<
 *             valid_roundings = [v for k, v in vars(decimal).items() if k.startswith('ROUND_')]
 *             assert rounding in valid_roundings, (
 */
  __pyx_t_2 = (__pyx_v_rounding != Py_None);
  __pyx_t_1 = (__pyx_t_2 != 0);
  if (__pyx_t_1) {

    /* "drf_turbo/fields.pyx":621
 * 
 *         if rounding is not None:
 *             valid_roundings = [v for k, v in vars(decimal).items() if k.startswith('ROUND_')]             # <<<<<<<<<<<<<<
 *             assert rounding in valid_roundings, (
 *                 'Invalid rounding option %s. Valid values for rounding are: %s' % (rounding, valid_roundings))
 */
    __pyx_t_6 = PyList_New(0); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 621, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_decimal); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 621, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_7 = __Pyx_PyObject_CallOneArg(__pyx_builtin_vars, __pyx_t_5); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 621, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_items); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 621, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_7 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_7)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_7);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
      }
    }
    __pyx_t_4 = (__pyx_t_7) ? __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_7) : __Pyx_PyObject_CallNoArg(__pyx_t_5);
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 621, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (likely(PyList_CheckExact(__pyx_t_4)) || PyTuple_CheckExact(__pyx_t_4)) {
      __pyx_t_5 = __pyx_t_4; __Pyx_INCREF(__pyx_t_5); __pyx_t_8 = 0;
      __pyx_t_9 = NULL;
    } else {
      __pyx_t_8 = -1; __pyx_t_5 = PyObject_GetIter(__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 621, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_9 = Py_TYPE(__pyx_t_5)->tp_iternext; if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 621, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    for (;;) {
      if (likely(!__pyx_t_9)) {
        if (likely(PyList_CheckExact(__pyx_t_5))) {
          if (__pyx_t_8 >= PyList_GET_SIZE(__pyx_t_5)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_4 = PyList_GET_ITEM(__pyx_t_5, __pyx_t_8); __Pyx_INCREF(__pyx_t_4); __pyx_t_8++; if (unlikely(0 < 0)) __PYX_ERR(0, 621, __pyx_L1_error)
          #else
          __pyx_t_4 = PySequence_ITEM(__pyx_t_5, __pyx_t_8); __pyx_t_8++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 621, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          #endif
        } else {
          if (__pyx_t_8 >= PyTuple_GET_SIZE(__pyx_t_5)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_4 = PyTuple_GET_ITEM(__pyx_t_5, __pyx_t_8); __Pyx_INCREF(__pyx_t_4); __pyx_t_8++; if (unlikely(0 < 0)) __PYX_ERR(0, 621, __pyx_L1_error)
          #else
          __pyx_t_4 = PySequence_ITEM(__pyx_t_5, __pyx_t_8); __pyx_t_8++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 621, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          #endif
        }
      } else {
        __pyx_t_4 = __pyx_t_9(__pyx_t_5);
        if (unlikely(!__pyx_t_4)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 621, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_4);
      }
      if ((likely(PyTuple_CheckExact(__pyx_t_4))) || (PyList_CheckExact(__pyx_t_4))) {
        PyObject* sequence = __pyx_t_4;
        Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
        if (unlikely(size != 2)) {
          if (size > 2) __Pyx_RaiseTooManyValuesError(2);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          __PYX_ERR(0, 621, __pyx_L1_error)
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_7 = PyTuple_GET_ITEM(sequence, 0); 
          __pyx_t_10 = PyTuple_GET_ITEM(sequence, 1); 
        } else {
          __pyx_t_7 = PyList_GET_ITEM(sequence, 0); 
          __pyx_t_10 = PyList_GET_ITEM(sequence, 1); 
        }
        __Pyx_INCREF(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_10);
        #else
        __pyx_t_7 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 621, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_10 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 621, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        #endif
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_11 = PyObject_GetIter(__pyx_t_4); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 621, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __pyx_t_12 = Py_TYPE(__pyx_t_11)->tp_iternext;
        index = 0; __pyx_t_7 = __pyx_t_12(__pyx_t_11); if (unlikely(!__pyx_t_7)) goto __pyx_L10_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_7);
        index = 1; __pyx_t_10 = __pyx_t_12(__pyx_t_11); if (unlikely(!__pyx_t_10)) goto __pyx_L10_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_10);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_12(__pyx_t_11), 2) < 0) __PYX_ERR(0, 621, __pyx_L1_error)
        __pyx_t_12 = NULL;
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        goto __pyx_L11_unpacking_done;
        __pyx_L10_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        __pyx_t_12 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        __PYX_ERR(0, 621, __pyx_L1_error)
        __pyx_L11_unpacking_done:;
      }
      __Pyx_XDECREF_SET(__pyx_v_k, __pyx_t_7);
      __pyx_t_7 = 0;
      __Pyx_XDECREF_SET(__pyx_v_v, __pyx_t_10);
      __pyx_t_10 = 0;
      __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_v_k, __pyx_n_s_startswith); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 621, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __pyx_t_7 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_10))) {
        __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_10);
        if (likely(__pyx_t_7)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
          __Pyx_INCREF(__pyx_t_7);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_10, function);
        }
      }
      __pyx_t_4 = (__pyx_t_7) ? __Pyx_PyObject_Call2Args(__pyx_t_10, __pyx_t_7, __pyx_n_s_ROUND) : __Pyx_PyObject_CallOneArg(__pyx_t_10, __pyx_n_s_ROUND);
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 621, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 621, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_1) {
        if (unlikely(__Pyx_ListComp_Append(__pyx_t_6, (PyObject*)__pyx_v_v))) __PYX_ERR(0, 621, __pyx_L1_error)
      }
    }
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_v_valid_roundings = ((PyObject*)__pyx_t_6);
    __pyx_t_6 = 0;

    /* "drf_turbo/fields.pyx":622
 *         if rounding is not None:
 *             valid_roundings = [v for k, v in vars(decimal).items() if k.startswith('ROUND_')]
 *             assert rounding in valid_roundings, (             # <<<<<<<<<<<<<<
 *                 'Invalid rounding option %s. Valid values for rounding are: %s' % (rounding, valid_roundings))
 *         self.rounding = rounding
 */
    #ifndef CYTHON_WITHOUT_ASSERTIONS
    if (unlikely(!Py_OptimizeFlag)) {
      __pyx_t_1 = (__Pyx_PySequence_ContainsTF(__pyx_v_rounding, __pyx_v_valid_roundings, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 622, __pyx_L1_error)
      if (unlikely(!(__pyx_t_1 != 0))) {

        /* "drf_turbo/fields.pyx":623
 *             valid_roundings = [v for k, v in vars(decimal).items() if k.startswith('ROUND_')]
 *             assert rounding in valid_roundings, (
 *                 'Invalid rounding option %s. Valid values for rounding are: %s' % (rounding, valid_roundings))             # <<<<<<<<<<<<<<
 *         self.rounding = rounding
 * 
 */
        __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 623, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_INCREF(__pyx_v_rounding);
        __Pyx_GIVEREF(__pyx_v_rounding);
        PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v_rounding);
        __Pyx_INCREF(__pyx_v_valid_roundings);
        __Pyx_GIVEREF(__pyx_v_valid_roundings);
        PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_v_valid_roundings);
        __pyx_t_5 = __Pyx_PyString_Format(__pyx_kp_s_Invalid_rounding_option_s_Valid, __pyx_t_6); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 623, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        PyErr_SetObject(PyExc_AssertionError, __pyx_t_5);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __PYX_ERR(0, 622, __pyx_L1_error)
      }
    }
    #endif

    /* "drf_turbo/fields.pyx":620
 *         super().__init__(**kwargs)
 * 
 *         if rounding is not None:             # <<<<<<<<<<<<<<
 *             valid_roundings = [v for k, v in vars(decimal).items() if k.startswith('ROUND_')]
 *             assert rounding in valid_roundings, (
 */
  }

  /* "drf_turbo/fields.pyx":624
 *             assert rounding in valid_roundings, (
 *                 'Invalid rounding option %s. Valid values for rounding are: %s' % (rounding, valid_roundings))
 *         self.rounding = rounding             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_INCREF(__pyx_v_rounding);
  __Pyx_GIVEREF(__pyx_v_rounding);
  __Pyx_GOTREF(__pyx_v_self->rounding);
  __Pyx_DECREF(__pyx_v_self->rounding);
  __pyx_v_self->rounding = __pyx_v_rounding;

  /* "drf_turbo/fields.pyx":603
 *     MAX_STRING_LENGTH = 1000  # Guard against malicious string inputs.
 * 
 *     def __init__(self, max_digits, decimal_places, max_value=None, min_value=None,coerce_to_string=None,rounding=None,**kwargs):             # <<<<<<<<<<<<<<
 *         self.max_digits = max_digits
 *         self.decimal_places = decimal_places
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("drf_turbo.fields.DecimalField.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_valid_roundings);
  __Pyx_XDECREF(__pyx_v_k);
  __Pyx_XDECREF(__pyx_v_v);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":627
 * 
 * 
 *     cpdef inline serialize(self,value,dict context):             # <<<<<<<<<<<<<<
 *         if value is None:
 *             if self.coerce_to_string:
 */

static PyObject *__pyx_pw_9drf_turbo_6fields_12DecimalField_3serialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_12DecimalField_serialize(struct __pyx_obj_9drf_turbo_6fields_DecimalField *__pyx_v_self, PyObject *__pyx_v_value, CYTHON_UNUSED PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_v_quantized = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("serialize", 0);
  __Pyx_INCREF(__pyx_v_value);

  /* "drf_turbo/fields.pyx":628
 * 
 *     cpdef inline serialize(self,value,dict context):
 *         if value is None:             # <<<<<<<<<<<<<<
 *             if self.coerce_to_string:
 *                 return ''
 */
  __pyx_t_1 = (__pyx_v_value == Py_None);
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "drf_turbo/fields.pyx":629
 *     cpdef inline serialize(self,value,dict context):
 *         if value is None:
 *             if self.coerce_to_string:             # <<<<<<<<<<<<<<
 *                 return ''
 *             else:
 */
    __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_self->coerce_to_string); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 629, __pyx_L1_error)
    if (__pyx_t_2) {

      /* "drf_turbo/fields.pyx":630
 *         if value is None:
 *             if self.coerce_to_string:
 *                 return ''             # <<<<<<<<<<<<<<
 *             else:
 *                 return None
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_s__9);
      __pyx_r = __pyx_kp_s__9;
      goto __pyx_L0;

      /* "drf_turbo/fields.pyx":629
 *     cpdef inline serialize(self,value,dict context):
 *         if value is None:
 *             if self.coerce_to_string:             # <<<<<<<<<<<<<<
 *                 return ''
 *             else:
 */
    }

    /* "drf_turbo/fields.pyx":632
 *                 return ''
 *             else:
 *                 return None             # <<<<<<<<<<<<<<
 * 
 *         if not isinstance(value, decimal.Decimal):
 */
    /*else*/ {
      __Pyx_XDECREF(__pyx_r);
      __pyx_r = Py_None; __Pyx_INCREF(Py_None);
      goto __pyx_L0;
    }

    /* "drf_turbo/fields.pyx":628
 * 
 *     cpdef inline serialize(self,value,dict context):
 *         if value is None:             # <<<<<<<<<<<<<<
 *             if self.coerce_to_string:
 *                 return ''
 */
  }

  /* "drf_turbo/fields.pyx":634
 *                 return None
 * 
 *         if not isinstance(value, decimal.Decimal):             # <<<<<<<<<<<<<<
 *             value = decimal.Decimal(str(value).strip())
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_decimal); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 634, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_Decimal); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 634, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_2 = PyObject_IsInstance(__pyx_v_value, __pyx_t_4); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 634, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_1 = ((!(__pyx_t_2 != 0)) != 0);
  if (__pyx_t_1) {

    /* "drf_turbo/fields.pyx":635
 * 
 *         if not isinstance(value, decimal.Decimal):
 *             value = decimal.Decimal(str(value).strip())             # <<<<<<<<<<<<<<
 * 
 *         quantized = self.quantize(value)
 */
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_decimal); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 635, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_Decimal); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 635, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_6 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), __pyx_v_value); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 635, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_strip); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 635, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_6 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
      }
    }
    __pyx_t_3 = (__pyx_t_6) ? __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_6) : __Pyx_PyObject_CallNoArg(__pyx_t_7);
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 635, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_7 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_7)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_7);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
      }
    }
    __pyx_t_4 = (__pyx_t_7) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_7, __pyx_t_3) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_3);
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 635, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF_SET(__pyx_v_value, __pyx_t_4);
    __pyx_t_4 = 0;

    /* "drf_turbo/fields.pyx":634
 *                 return None
 * 
 *         if not isinstance(value, decimal.Decimal):             # <<<<<<<<<<<<<<
 *             value = decimal.Decimal(str(value).strip())
 * 
 */
  }

  /* "drf_turbo/fields.pyx":637
 *             value = decimal.Decimal(str(value).strip())
 * 
 *         quantized = self.quantize(value)             # <<<<<<<<<<<<<<
 *         if not self.coerce_to_string:
 *             return quantized
 */
  __pyx_t_4 = __pyx_f_9drf_turbo_6fields_12DecimalField_quantize(__pyx_v_self, __pyx_v_value); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 637, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_v_quantized = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "drf_turbo/fields.pyx":638
 * 
 *         quantized = self.quantize(value)
 *         if not self.coerce_to_string:             # <<<<<<<<<<<<<<
 *             return quantized
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_self->coerce_to_string); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 638, __pyx_L1_error)
  __pyx_t_2 = ((!__pyx_t_1) != 0);
  if (__pyx_t_2) {

    /* "drf_turbo/fields.pyx":639
 *         quantized = self.quantize(value)
 *         if not self.coerce_to_string:
 *             return quantized             # <<<<<<<<<<<<<<
 * 
 *         return '{:f}'.format(quantized)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_quantized);
    __pyx_r = __pyx_v_quantized;
    goto __pyx_L0;

    /* "drf_turbo/fields.pyx":638
 * 
 *         quantized = self.quantize(value)
 *         if not self.coerce_to_string:             # <<<<<<<<<<<<<<
 *             return quantized
 * 
 */
  }

  /* "drf_turbo/fields.pyx":641
 *             return quantized
 * 
 *         return '{:f}'.format(quantized)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_f, __pyx_n_s_format); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 641, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
    }
  }
  __pyx_t_4 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_3, __pyx_v_quantized) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_v_quantized);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 641, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "drf_turbo/fields.pyx":627
 * 
 * 
 *     cpdef inline serialize(self,value,dict context):             # <<<<<<<<<<<<<<
 *         if value is None:
 *             if self.coerce_to_string:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("drf_turbo.fields.DecimalField.serialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_quantized);
  __Pyx_XDECREF(__pyx_v_value);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_12DecimalField_3serialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_12DecimalField_3serialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_value = 0;
  PyObject *__pyx_v_context = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("serialize (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_value,&__pyx_n_s_context,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_value)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_context)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("serialize", 1, 2, 2, 1); __PYX_ERR(0, 627, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "serialize") < 0)) __PYX_ERR(0, 627, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_value = values[0];
    __pyx_v_context = ((PyObject*)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("serialize", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 627, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.DecimalField.serialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_context), (&PyDict_Type), 1, "context", 1))) __PYX_ERR(0, 627, __pyx_L1_error)
  __pyx_r = __pyx_pf_9drf_turbo_6fields_12DecimalField_2serialize(((struct __pyx_obj_9drf_turbo_6fields_DecimalField *)__pyx_v_self), __pyx_v_value, __pyx_v_context);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_12DecimalField_2serialize(struct __pyx_obj_9drf_turbo_6fields_DecimalField *__pyx_v_self, PyObject *__pyx_v_value, PyObject *__pyx_v_context) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("serialize", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields_12DecimalField_serialize(__pyx_v_self, __pyx_v_value, __pyx_v_context, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 627, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.DecimalField.serialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":645
 * 
 * 
 *     cpdef inline deserialize(self,data,dict context):             # <<<<<<<<<<<<<<
 *         """
 *         Validate that the input is a decimal number and return a Decimal
 */

static PyObject *__pyx_pw_9drf_turbo_6fields_12DecimalField_5deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_12DecimalField_deserialize(struct __pyx_obj_9drf_turbo_6fields_DecimalField *__pyx_v_self, PyObject *__pyx_v_data, CYTHON_UNUSED PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_v_value = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  int __pyx_t_12;
  PyObject *__pyx_t_13 = NULL;
  int __pyx_t_14;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("deserialize", 0);
  __Pyx_INCREF(__pyx_v_data);

  /* "drf_turbo/fields.pyx":651
 *         """
 * 
 *         data = force_str(data).strip()             # <<<<<<<<<<<<<<
 * 
 *         if data == '' and self.allow_null:
 */
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_force_str); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 651, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_data) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_data);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 651, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_strip); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 651, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 651, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF_SET(__pyx_v_data, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "drf_turbo/fields.pyx":653
 *         data = force_str(data).strip()
 * 
 *         if data == '' and self.allow_null:             # <<<<<<<<<<<<<<
 *             return None
 * 
 */
  __pyx_t_6 = (__Pyx_PyString_Equals(__pyx_v_data, __pyx_kp_s__9, Py_EQ)); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 653, __pyx_L1_error)
  if (__pyx_t_6) {
  } else {
    __pyx_t_5 = __pyx_t_6;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_6 = (__pyx_v_self->__pyx_base.allow_null != 0);
  __pyx_t_5 = __pyx_t_6;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_5) {

    /* "drf_turbo/fields.pyx":654
 * 
 *         if data == '' and self.allow_null:
 *             return None             # <<<<<<<<<<<<<<
 * 
 *         if len(data) > self.MAX_STRING_LENGTH:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "drf_turbo/fields.pyx":653
 *         data = force_str(data).strip()
 * 
 *         if data == '' and self.allow_null:             # <<<<<<<<<<<<<<
 *             return None
 * 
 */
  }

  /* "drf_turbo/fields.pyx":656
 *             return None
 * 
 *         if len(data) > self.MAX_STRING_LENGTH:             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail('max_string_length')
 * 
 */
  __pyx_t_7 = PyObject_Length(__pyx_v_data); if (unlikely(__pyx_t_7 == ((Py_ssize_t)-1))) __PYX_ERR(0, 656, __pyx_L1_error)
  __pyx_t_1 = PyInt_FromSsize_t(__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 656, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_MAX_STRING_LENGTH); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 656, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = PyObject_RichCompare(__pyx_t_1, __pyx_t_3, Py_GT); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 656, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 656, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(__pyx_t_5)) {

    /* "drf_turbo/fields.pyx":657
 * 
 *         if len(data) > self.MAX_STRING_LENGTH:
 *             raise self.raise_if_fail('max_string_length')             # <<<<<<<<<<<<<<
 * 
 *         try:
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_raise_if_fail); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 657, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_1 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_1)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_2 = (__pyx_t_1) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_1, __pyx_n_s_max_string_length) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_n_s_max_string_length);
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 657, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 657, __pyx_L1_error)

    /* "drf_turbo/fields.pyx":656
 *             return None
 * 
 *         if len(data) > self.MAX_STRING_LENGTH:             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail('max_string_length')
 * 
 */
  }

  /* "drf_turbo/fields.pyx":659
 *             raise self.raise_if_fail('max_string_length')
 * 
 *         try:             # <<<<<<<<<<<<<<
 *             value = decimal.Decimal(data)
 *         except decimal.DecimalException:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_8, &__pyx_t_9, &__pyx_t_10);
    __Pyx_XGOTREF(__pyx_t_8);
    __Pyx_XGOTREF(__pyx_t_9);
    __Pyx_XGOTREF(__pyx_t_10);
    /*try:*/ {

      /* "drf_turbo/fields.pyx":660
 * 
 *         try:
 *             value = decimal.Decimal(data)             # <<<<<<<<<<<<<<
 *         except decimal.DecimalException:
 *             raise self.raise_if_fail('invalid')
 */
      __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_decimal); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 660, __pyx_L7_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_Decimal); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 660, __pyx_L7_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_3 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
        __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_1);
        if (likely(__pyx_t_3)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_1, function);
        }
      }
      __pyx_t_2 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_3, __pyx_v_data) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_v_data);
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 660, __pyx_L7_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_v_value = __pyx_t_2;
      __pyx_t_2 = 0;

      /* "drf_turbo/fields.pyx":659
 *             raise self.raise_if_fail('max_string_length')
 * 
 *         try:             # <<<<<<<<<<<<<<
 *             value = decimal.Decimal(data)
 *         except decimal.DecimalException:
 */
    }
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    goto __pyx_L12_try_end;
    __pyx_L7_error:;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "drf_turbo/fields.pyx":661
 *         try:
 *             value = decimal.Decimal(data)
 *         except decimal.DecimalException:             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail('invalid')
 * 
 */
    __Pyx_ErrFetch(&__pyx_t_2, &__pyx_t_1, &__pyx_t_3);
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_decimal); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 661, __pyx_L9_except_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_DecimalException); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 661, __pyx_L9_except_error)
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_12 = __Pyx_PyErr_GivenExceptionMatches(__pyx_t_2, __pyx_t_11);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_ErrRestore(__pyx_t_2, __pyx_t_1, __pyx_t_3);
    __pyx_t_2 = 0; __pyx_t_1 = 0; __pyx_t_3 = 0;
    if (__pyx_t_12) {
      __Pyx_AddTraceback("drf_turbo.fields.DecimalField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_3, &__pyx_t_1, &__pyx_t_2) < 0) __PYX_ERR(0, 661, __pyx_L9_except_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_GOTREF(__pyx_t_2);

      /* "drf_turbo/fields.pyx":662
 *             value = decimal.Decimal(data)
 *         except decimal.DecimalException:
 *             raise self.raise_if_fail('invalid')             # <<<<<<<<<<<<<<
 * 
 *         if value.is_nan():
 */
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_raise_if_fail); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 662, __pyx_L9_except_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_13 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_4);
        if (likely(__pyx_t_13)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_13);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_4, function);
        }
      }
      __pyx_t_11 = (__pyx_t_13) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_13, __pyx_n_s_invalid) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_n_s_invalid);
      __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
      if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 662, __pyx_L9_except_error)
      __Pyx_GOTREF(__pyx_t_11);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_Raise(__pyx_t_11, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      __PYX_ERR(0, 662, __pyx_L9_except_error)
    }
    goto __pyx_L9_except_error;
    __pyx_L9_except_error:;

    /* "drf_turbo/fields.pyx":659
 *             raise self.raise_if_fail('max_string_length')
 * 
 *         try:             # <<<<<<<<<<<<<<
 *             value = decimal.Decimal(data)
 *         except decimal.DecimalException:
 */
    __Pyx_XGIVEREF(__pyx_t_8);
    __Pyx_XGIVEREF(__pyx_t_9);
    __Pyx_XGIVEREF(__pyx_t_10);
    __Pyx_ExceptionReset(__pyx_t_8, __pyx_t_9, __pyx_t_10);
    goto __pyx_L1_error;
    __pyx_L12_try_end:;
  }

  /* "drf_turbo/fields.pyx":664
 *             raise self.raise_if_fail('invalid')
 * 
 *         if value.is_nan():             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail('invalid')
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_value, __pyx_n_s_is_nan); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 664, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
    }
  }
  __pyx_t_2 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 664, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 664, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(__pyx_t_5)) {

    /* "drf_turbo/fields.pyx":665
 * 
 *         if value.is_nan():
 *             raise self.raise_if_fail('invalid')             # <<<<<<<<<<<<<<
 * 
 *         if value in (decimal.Decimal('Inf'), decimal.Decimal('-Inf')):
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_raise_if_fail); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 665, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
      }
    }
    __pyx_t_2 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_3, __pyx_n_s_invalid) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_n_s_invalid);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 665, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 665, __pyx_L1_error)

    /* "drf_turbo/fields.pyx":664
 *             raise self.raise_if_fail('invalid')
 * 
 *         if value.is_nan():             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail('invalid')
 * 
 */
  }

  /* "drf_turbo/fields.pyx":667
 *             raise self.raise_if_fail('invalid')
 * 
 *         if value in (decimal.Decimal('Inf'), decimal.Decimal('-Inf')):             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail('invalid')
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_decimal); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 667, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_Decimal); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 667, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_1)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_2 = (__pyx_t_1) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_1, __pyx_n_s_Inf) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_n_s_Inf);
  __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 667, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_decimal); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 667, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_Decimal); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 667, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_11))) {
    __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_11);
    if (likely(__pyx_t_1)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_11);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_11, function);
    }
  }
  __pyx_t_3 = (__pyx_t_1) ? __Pyx_PyObject_Call2Args(__pyx_t_11, __pyx_t_1, __pyx_kp_s_Inf_2) : __Pyx_PyObject_CallOneArg(__pyx_t_11, __pyx_kp_s_Inf_2);
  __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 667, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __Pyx_INCREF(__pyx_v_value);
  __pyx_t_11 = __pyx_v_value;
  __pyx_t_1 = PyObject_RichCompare(__pyx_t_11, __pyx_t_2, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 667, __pyx_L1_error)
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 667, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (!__pyx_t_6) {
  } else {
    __pyx_t_5 = __pyx_t_6;
    goto __pyx_L17_bool_binop_done;
  }
  __pyx_t_1 = PyObject_RichCompare(__pyx_t_11, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 667, __pyx_L1_error)
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 667, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_5 = __pyx_t_6;
  __pyx_L17_bool_binop_done:;
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_6 = (__pyx_t_5 != 0);
  if (unlikely(__pyx_t_6)) {

    /* "drf_turbo/fields.pyx":668
 * 
 *         if value in (decimal.Decimal('Inf'), decimal.Decimal('-Inf')):
 *             raise self.raise_if_fail('invalid')             # <<<<<<<<<<<<<<
 * 
 *         if self.min_value is not None and value < self.min_value:
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_raise_if_fail); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 668, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_11 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_11)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_11);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_2 = (__pyx_t_11) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_11, __pyx_n_s_invalid) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_n_s_invalid);
    __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 668, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 668, __pyx_L1_error)

    /* "drf_turbo/fields.pyx":667
 *             raise self.raise_if_fail('invalid')
 * 
 *         if value in (decimal.Decimal('Inf'), decimal.Decimal('-Inf')):             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail('invalid')
 * 
 */
  }

  /* "drf_turbo/fields.pyx":670
 *             raise self.raise_if_fail('invalid')
 * 
 *         if self.min_value is not None and value < self.min_value:             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail("min_value",min_value=self.min_value)
 * 
 */
  __pyx_t_5 = (__pyx_v_self->min_value != Py_None);
  __pyx_t_14 = (__pyx_t_5 != 0);
  if (__pyx_t_14) {
  } else {
    __pyx_t_6 = __pyx_t_14;
    goto __pyx_L20_bool_binop_done;
  }
  __pyx_t_2 = PyObject_RichCompare(__pyx_v_value, __pyx_v_self->min_value, Py_LT); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 670, __pyx_L1_error)
  __pyx_t_14 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_14 < 0)) __PYX_ERR(0, 670, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_6 = __pyx_t_14;
  __pyx_L20_bool_binop_done:;
  if (unlikely(__pyx_t_6)) {

    /* "drf_turbo/fields.pyx":671
 * 
 *         if self.min_value is not None and value < self.min_value:
 *             raise self.raise_if_fail("min_value",min_value=self.min_value)             # <<<<<<<<<<<<<<
 * 
 *         if self.max_value is not None and value > self.max_value:
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_raise_if_fail); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 671, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 671, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_min_value, __pyx_v_self->min_value) < 0) __PYX_ERR(0, 671, __pyx_L1_error)
    __pyx_t_11 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__13, __pyx_t_3); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 671, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_Raise(__pyx_t_11, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __PYX_ERR(0, 671, __pyx_L1_error)

    /* "drf_turbo/fields.pyx":670
 *             raise self.raise_if_fail('invalid')
 * 
 *         if self.min_value is not None and value < self.min_value:             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail("min_value",min_value=self.min_value)
 * 
 */
  }

  /* "drf_turbo/fields.pyx":673
 *             raise self.raise_if_fail("min_value",min_value=self.min_value)
 * 
 *         if self.max_value is not None and value > self.max_value:             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail("max_value",max_value=self.max_value)
 * 
 */
  __pyx_t_14 = (__pyx_v_self->max_value != Py_None);
  __pyx_t_5 = (__pyx_t_14 != 0);
  if (__pyx_t_5) {
  } else {
    __pyx_t_6 = __pyx_t_5;
    goto __pyx_L23_bool_binop_done;
  }
  __pyx_t_11 = PyObject_RichCompare(__pyx_v_value, __pyx_v_self->max_value, Py_GT); __Pyx_XGOTREF(__pyx_t_11); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 673, __pyx_L1_error)
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_11); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 673, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __pyx_t_6 = __pyx_t_5;
  __pyx_L23_bool_binop_done:;
  if (unlikely(__pyx_t_6)) {

    /* "drf_turbo/fields.pyx":674
 * 
 *         if self.max_value is not None and value > self.max_value:
 *             raise self.raise_if_fail("max_value",max_value=self.max_value)             # <<<<<<<<<<<<<<
 * 
 *         return self.quantize(self.validate_precision(value))
 */
    __pyx_t_11 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_raise_if_fail); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 674, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __pyx_t_3 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 674, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_max_value, __pyx_v_self->max_value) < 0) __PYX_ERR(0, 674, __pyx_L1_error)
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_11, __pyx_tuple__14, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 674, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 674, __pyx_L1_error)

    /* "drf_turbo/fields.pyx":673
 *             raise self.raise_if_fail("min_value",min_value=self.min_value)
 * 
 *         if self.max_value is not None and value > self.max_value:             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail("max_value",max_value=self.max_value)
 * 
 */
  }

  /* "drf_turbo/fields.pyx":676
 *             raise self.raise_if_fail("max_value",max_value=self.max_value)
 * 
 *         return self.quantize(self.validate_precision(value))             # <<<<<<<<<<<<<<
 * 
 *     cdef inline validate_precision(self, value):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __pyx_f_9drf_turbo_6fields_12DecimalField_validate_precision(__pyx_v_self, __pyx_v_value); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 676, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_f_9drf_turbo_6fields_12DecimalField_quantize(__pyx_v_self, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 676, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "drf_turbo/fields.pyx":645
 * 
 * 
 *     cpdef inline deserialize(self,data,dict context):             # <<<<<<<<<<<<<<
 *         """
 *         Validate that the input is a decimal number and return a Decimal
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_AddTraceback("drf_turbo.fields.DecimalField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_value);
  __Pyx_XDECREF(__pyx_v_data);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_12DecimalField_5deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_9drf_turbo_6fields_12DecimalField_4deserialize[] = "\n        Validate that the input is a decimal number and return a Decimal\n        instance.\n        ";
static PyObject *__pyx_pw_9drf_turbo_6fields_12DecimalField_5deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_data = 0;
  PyObject *__pyx_v_context = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("deserialize (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_data,&__pyx_n_s_context,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_data)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_context)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("deserialize", 1, 2, 2, 1); __PYX_ERR(0, 645, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "deserialize") < 0)) __PYX_ERR(0, 645, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_data = values[0];
    __pyx_v_context = ((PyObject*)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("deserialize", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 645, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.DecimalField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_context), (&PyDict_Type), 1, "context", 1))) __PYX_ERR(0, 645, __pyx_L1_error)
  __pyx_r = __pyx_pf_9drf_turbo_6fields_12DecimalField_4deserialize(((struct __pyx_obj_9drf_turbo_6fields_DecimalField *)__pyx_v_self), __pyx_v_data, __pyx_v_context);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_12DecimalField_4deserialize(struct __pyx_obj_9drf_turbo_6fields_DecimalField *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("deserialize", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields_12DecimalField_deserialize(__pyx_v_self, __pyx_v_data, __pyx_v_context, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 645, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.DecimalField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":678
 *         return self.quantize(self.validate_precision(value))
 * 
 *     cdef inline validate_precision(self, value):             # <<<<<<<<<<<<<<
 *         """
 *         Ensure that there are no more than max_digits in the number, and no
 */

static PyObject *__pyx_f_9drf_turbo_6fields_12DecimalField_validate_precision(struct __pyx_obj_9drf_turbo_6fields_DecimalField *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *__pyx_v_sign = NULL;
  PyObject *__pyx_v_digittuple = NULL;
  PyObject *__pyx_v_exponent = NULL;
  PyObject *__pyx_v_total_digits = NULL;
  PyObject *__pyx_v_whole_digits = NULL;
  PyObject *__pyx_v_decimal_places = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *(*__pyx_t_6)(PyObject *);
  int __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  int __pyx_t_9;
  int __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("validate_precision", 0);

  /* "drf_turbo/fields.pyx":685
 *         values or to enhance it in any way you need to.
 *         """
 *         sign, digittuple, exponent = value.as_tuple()             # <<<<<<<<<<<<<<
 * 
 *         if exponent >= 0:
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_value, __pyx_n_s_as_tuple); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 685, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 685, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
    PyObject* sequence = __pyx_t_1;
    Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
    if (unlikely(size != 3)) {
      if (size > 3) __Pyx_RaiseTooManyValuesError(3);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(0, 685, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    if (likely(PyTuple_CheckExact(sequence))) {
      __pyx_t_2 = PyTuple_GET_ITEM(sequence, 0); 
      __pyx_t_3 = PyTuple_GET_ITEM(sequence, 1); 
      __pyx_t_4 = PyTuple_GET_ITEM(sequence, 2); 
    } else {
      __pyx_t_2 = PyList_GET_ITEM(sequence, 0); 
      __pyx_t_3 = PyList_GET_ITEM(sequence, 1); 
      __pyx_t_4 = PyList_GET_ITEM(sequence, 2); 
    }
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_t_4);
    #else
    __pyx_t_2 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 685, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 685, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PySequence_ITEM(sequence, 2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 685, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    #endif
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  } else {
    Py_ssize_t index = -1;
    __pyx_t_5 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 685, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_6 = Py_TYPE(__pyx_t_5)->tp_iternext;
    index = 0; __pyx_t_2 = __pyx_t_6(__pyx_t_5); if (unlikely(!__pyx_t_2)) goto __pyx_L3_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_2);
    index = 1; __pyx_t_3 = __pyx_t_6(__pyx_t_5); if (unlikely(!__pyx_t_3)) goto __pyx_L3_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_3);
    index = 2; __pyx_t_4 = __pyx_t_6(__pyx_t_5); if (unlikely(!__pyx_t_4)) goto __pyx_L3_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_4);
    if (__Pyx_IternextUnpackEndCheck(__pyx_t_6(__pyx_t_5), 3) < 0) __PYX_ERR(0, 685, __pyx_L1_error)
    __pyx_t_6 = NULL;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    goto __pyx_L4_unpacking_done;
    __pyx_L3_unpacking_failed:;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_6 = NULL;
    if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
    __PYX_ERR(0, 685, __pyx_L1_error)
    __pyx_L4_unpacking_done:;
  }
  __pyx_v_sign = __pyx_t_2;
  __pyx_t_2 = 0;
  __pyx_v_digittuple = __pyx_t_3;
  __pyx_t_3 = 0;
  __pyx_v_exponent = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "drf_turbo/fields.pyx":687
 *         sign, digittuple, exponent = value.as_tuple()
 * 
 *         if exponent >= 0:             # <<<<<<<<<<<<<<
 *             # 1234500.0
 *             total_digits = len(digittuple) + exponent
 */
  __pyx_t_1 = PyObject_RichCompare(__pyx_v_exponent, __pyx_int_0, Py_GE); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 687, __pyx_L1_error)
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 687, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_7) {

    /* "drf_turbo/fields.pyx":689
 *         if exponent >= 0:
 *             # 1234500.0
 *             total_digits = len(digittuple) + exponent             # <<<<<<<<<<<<<<
 *             whole_digits = total_digits
 *             decimal_places = 0
 */
    __pyx_t_8 = PyObject_Length(__pyx_v_digittuple); if (unlikely(__pyx_t_8 == ((Py_ssize_t)-1))) __PYX_ERR(0, 689, __pyx_L1_error)
    __pyx_t_1 = PyInt_FromSsize_t(__pyx_t_8); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 689, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = PyNumber_Add(__pyx_t_1, __pyx_v_exponent); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 689, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_total_digits = __pyx_t_4;
    __pyx_t_4 = 0;

    /* "drf_turbo/fields.pyx":690
 *             # 1234500.0
 *             total_digits = len(digittuple) + exponent
 *             whole_digits = total_digits             # <<<<<<<<<<<<<<
 *             decimal_places = 0
 *         elif len(digittuple) > abs(exponent):
 */
    __Pyx_INCREF(__pyx_v_total_digits);
    __pyx_v_whole_digits = __pyx_v_total_digits;

    /* "drf_turbo/fields.pyx":691
 *             total_digits = len(digittuple) + exponent
 *             whole_digits = total_digits
 *             decimal_places = 0             # <<<<<<<<<<<<<<
 *         elif len(digittuple) > abs(exponent):
 *             # 123.45
 */
    __Pyx_INCREF(__pyx_int_0);
    __pyx_v_decimal_places = __pyx_int_0;

    /* "drf_turbo/fields.pyx":687
 *         sign, digittuple, exponent = value.as_tuple()
 * 
 *         if exponent >= 0:             # <<<<<<<<<<<<<<
 *             # 1234500.0
 *             total_digits = len(digittuple) + exponent
 */
    goto __pyx_L5;
  }

  /* "drf_turbo/fields.pyx":692
 *             whole_digits = total_digits
 *             decimal_places = 0
 *         elif len(digittuple) > abs(exponent):             # <<<<<<<<<<<<<<
 *             # 123.45
 *             total_digits = len(digittuple)
 */
  __pyx_t_8 = PyObject_Length(__pyx_v_digittuple); if (unlikely(__pyx_t_8 == ((Py_ssize_t)-1))) __PYX_ERR(0, 692, __pyx_L1_error)
  __pyx_t_4 = PyInt_FromSsize_t(__pyx_t_8); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 692, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_1 = __Pyx_PyNumber_Absolute(__pyx_v_exponent); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 692, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = PyObject_RichCompare(__pyx_t_4, __pyx_t_1, Py_GT); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 692, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 692, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__pyx_t_7) {

    /* "drf_turbo/fields.pyx":694
 *         elif len(digittuple) > abs(exponent):
 *             # 123.45
 *             total_digits = len(digittuple)             # <<<<<<<<<<<<<<
 *             whole_digits = total_digits - abs(exponent)
 *             decimal_places = abs(exponent)
 */
    __pyx_t_8 = PyObject_Length(__pyx_v_digittuple); if (unlikely(__pyx_t_8 == ((Py_ssize_t)-1))) __PYX_ERR(0, 694, __pyx_L1_error)
    __pyx_t_3 = PyInt_FromSsize_t(__pyx_t_8); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 694, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_v_total_digits = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "drf_turbo/fields.pyx":695
 *             # 123.45
 *             total_digits = len(digittuple)
 *             whole_digits = total_digits - abs(exponent)             # <<<<<<<<<<<<<<
 *             decimal_places = abs(exponent)
 *         else:
 */
    __pyx_t_3 = __Pyx_PyNumber_Absolute(__pyx_v_exponent); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 695, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_1 = PyNumber_Subtract(__pyx_v_total_digits, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 695, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_v_whole_digits = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "drf_turbo/fields.pyx":696
 *             total_digits = len(digittuple)
 *             whole_digits = total_digits - abs(exponent)
 *             decimal_places = abs(exponent)             # <<<<<<<<<<<<<<
 *         else:
 *             # 0.001234
 */
    __pyx_t_1 = __Pyx_PyNumber_Absolute(__pyx_v_exponent); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 696, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_v_decimal_places = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "drf_turbo/fields.pyx":692
 *             whole_digits = total_digits
 *             decimal_places = 0
 *         elif len(digittuple) > abs(exponent):             # <<<<<<<<<<<<<<
 *             # 123.45
 *             total_digits = len(digittuple)
 */
    goto __pyx_L5;
  }

  /* "drf_turbo/fields.pyx":699
 *         else:
 *             # 0.001234
 *             total_digits = abs(exponent)             # <<<<<<<<<<<<<<
 *             whole_digits = 0
 *             decimal_places = total_digits
 */
  /*else*/ {
    __pyx_t_1 = __Pyx_PyNumber_Absolute(__pyx_v_exponent); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 699, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_v_total_digits = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "drf_turbo/fields.pyx":700
 *             # 0.001234
 *             total_digits = abs(exponent)
 *             whole_digits = 0             # <<<<<<<<<<<<<<
 *             decimal_places = total_digits
 * 
 */
    __Pyx_INCREF(__pyx_int_0);
    __pyx_v_whole_digits = __pyx_int_0;

    /* "drf_turbo/fields.pyx":701
 *             total_digits = abs(exponent)
 *             whole_digits = 0
 *             decimal_places = total_digits             # <<<<<<<<<<<<<<
 * 
 *         if self.max_digits is not None and total_digits > self.max_digits:
 */
    __Pyx_INCREF(__pyx_v_total_digits);
    __pyx_v_decimal_places = __pyx_v_total_digits;
  }
  __pyx_L5:;

  /* "drf_turbo/fields.pyx":703
 *             decimal_places = total_digits
 * 
 *         if self.max_digits is not None and total_digits > self.max_digits:             # <<<<<<<<<<<<<<
 *            raise self.raise_if_fail('max_digits', max_digits=self.max_digits)
 *         if self.decimal_places is not None and decimal_places > self.decimal_places:
 */
  __pyx_t_9 = (__pyx_v_self->max_digits != Py_None);
  __pyx_t_10 = (__pyx_t_9 != 0);
  if (__pyx_t_10) {
  } else {
    __pyx_t_7 = __pyx_t_10;
    goto __pyx_L7_bool_binop_done;
  }
  __pyx_t_1 = PyObject_RichCompare(__pyx_v_total_digits, __pyx_v_self->max_digits, Py_GT); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 703, __pyx_L1_error)
  __pyx_t_10 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 703, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_7 = __pyx_t_10;
  __pyx_L7_bool_binop_done:;
  if (unlikely(__pyx_t_7)) {

    /* "drf_turbo/fields.pyx":704
 * 
 *         if self.max_digits is not None and total_digits > self.max_digits:
 *            raise self.raise_if_fail('max_digits', max_digits=self.max_digits)             # <<<<<<<<<<<<<<
 *         if self.decimal_places is not None and decimal_places > self.decimal_places:
 *             raise self.raise_if_fail('max_decimal_places', max_decimal_places=self.decimal_places)
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_raise_if_fail); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 704, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 704, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_max_digits, __pyx_v_self->max_digits) < 0) __PYX_ERR(0, 704, __pyx_L1_error)
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__15, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 704, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(0, 704, __pyx_L1_error)

    /* "drf_turbo/fields.pyx":703
 *             decimal_places = total_digits
 * 
 *         if self.max_digits is not None and total_digits > self.max_digits:             # <<<<<<<<<<<<<<
 *            raise self.raise_if_fail('max_digits', max_digits=self.max_digits)
 *         if self.decimal_places is not None and decimal_places > self.decimal_places:
 */
  }

  /* "drf_turbo/fields.pyx":705
 *         if self.max_digits is not None and total_digits > self.max_digits:
 *            raise self.raise_if_fail('max_digits', max_digits=self.max_digits)
 *         if self.decimal_places is not None and decimal_places > self.decimal_places:             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail('max_decimal_places', max_decimal_places=self.decimal_places)
 *         if self.max_whole_digits is not None and whole_digits > self.max_whole_digits:
 */
  __pyx_t_10 = (__pyx_v_self->decimal_places != Py_None);
  __pyx_t_9 = (__pyx_t_10 != 0);
  if (__pyx_t_9) {
  } else {
    __pyx_t_7 = __pyx_t_9;
    goto __pyx_L10_bool_binop_done;
  }
  __pyx_t_4 = PyObject_RichCompare(__pyx_v_decimal_places, __pyx_v_self->decimal_places, Py_GT); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 705, __pyx_L1_error)
  __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 705, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_7 = __pyx_t_9;
  __pyx_L10_bool_binop_done:;
  if (unlikely(__pyx_t_7)) {

    /* "drf_turbo/fields.pyx":706
 *            raise self.raise_if_fail('max_digits', max_digits=self.max_digits)
 *         if self.decimal_places is not None and decimal_places > self.decimal_places:
 *             raise self.raise_if_fail('max_decimal_places', max_decimal_places=self.decimal_places)             # <<<<<<<<<<<<<<
 *         if self.max_whole_digits is not None and whole_digits > self.max_whole_digits:
 *             raise self.raise_if_fail('max_whole_digits', max_whole_digits=self.max_whole_digits)
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_raise_if_fail); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 706, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_3 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 706, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_max_decimal_places, __pyx_v_self->decimal_places) < 0) __PYX_ERR(0, 706, __pyx_L1_error)
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_tuple__16, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 706, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 706, __pyx_L1_error)

    /* "drf_turbo/fields.pyx":705
 *         if self.max_digits is not None and total_digits > self.max_digits:
 *            raise self.raise_if_fail('max_digits', max_digits=self.max_digits)
 *         if self.decimal_places is not None and decimal_places > self.decimal_places:             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail('max_decimal_places', max_decimal_places=self.decimal_places)
 *         if self.max_whole_digits is not None and whole_digits > self.max_whole_digits:
 */
  }

  /* "drf_turbo/fields.pyx":707
 *         if self.decimal_places is not None and decimal_places > self.decimal_places:
 *             raise self.raise_if_fail('max_decimal_places', max_decimal_places=self.decimal_places)
 *         if self.max_whole_digits is not None and whole_digits > self.max_whole_digits:             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail('max_whole_digits', max_whole_digits=self.max_whole_digits)
 * 
 */
  __pyx_t_9 = (__pyx_v_self->max_whole_digits != Py_None);
  __pyx_t_10 = (__pyx_t_9 != 0);
  if (__pyx_t_10) {
  } else {
    __pyx_t_7 = __pyx_t_10;
    goto __pyx_L13_bool_binop_done;
  }
  __pyx_t_1 = PyObject_RichCompare(__pyx_v_whole_digits, __pyx_v_self->max_whole_digits, Py_GT); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 707, __pyx_L1_error)
  __pyx_t_10 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 707, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_7 = __pyx_t_10;
  __pyx_L13_bool_binop_done:;
  if (unlikely(__pyx_t_7)) {

    /* "drf_turbo/fields.pyx":708
 *             raise self.raise_if_fail('max_decimal_places', max_decimal_places=self.decimal_places)
 *         if self.max_whole_digits is not None and whole_digits > self.max_whole_digits:
 *             raise self.raise_if_fail('max_whole_digits', max_whole_digits=self.max_whole_digits)             # <<<<<<<<<<<<<<
 * 
 *         return value
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_raise_if_fail); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 708, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 708, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_max_whole_digits, __pyx_v_self->max_whole_digits) < 0) __PYX_ERR(0, 708, __pyx_L1_error)
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__17, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 708, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(0, 708, __pyx_L1_error)

    /* "drf_turbo/fields.pyx":707
 *         if self.decimal_places is not None and decimal_places > self.decimal_places:
 *             raise self.raise_if_fail('max_decimal_places', max_decimal_places=self.decimal_places)
 *         if self.max_whole_digits is not None and whole_digits > self.max_whole_digits:             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail('max_whole_digits', max_whole_digits=self.max_whole_digits)
 * 
 */
  }

  /* "drf_turbo/fields.pyx":710
 *             raise self.raise_if_fail('max_whole_digits', max_whole_digits=self.max_whole_digits)
 * 
 *         return value             # <<<<<<<<<<<<<<
 * 
 *     cdef inline quantize(self, value):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_value);
  __pyx_r = __pyx_v_value;
  goto __pyx_L0;

  /* "drf_turbo/fields.pyx":678
 *         return self.quantize(self.validate_precision(value))
 * 
 *     cdef inline validate_precision(self, value):             # <<<<<<<<<<<<<<
 *         """
 *         Ensure that there are no more than max_digits in the number, and no
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("drf_turbo.fields.DecimalField.validate_precision", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_sign);
  __Pyx_XDECREF(__pyx_v_digittuple);
  __Pyx_XDECREF(__pyx_v_exponent);
  __Pyx_XDECREF(__pyx_v_total_digits);
  __Pyx_XDECREF(__pyx_v_whole_digits);
  __Pyx_XDECREF(__pyx_v_decimal_places);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":712
 *         return value
 * 
 *     cdef inline quantize(self, value):             # <<<<<<<<<<<<<<
 *         """
 *         Quantize the decimal value to the configured precision.
 */

static PyObject *__pyx_f_9drf_turbo_6fields_12DecimalField_quantize(struct __pyx_obj_9drf_turbo_6fields_DecimalField *__pyx_v_self, PyObject *__pyx_v_value) {
  PyObject *__pyx_v_context = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("quantize", 0);

  /* "drf_turbo/fields.pyx":716
 *         Quantize the decimal value to the configured precision.
 *         """
 *         if self.decimal_places is None:             # <<<<<<<<<<<<<<
 *             return value
 * 
 */
  __pyx_t_1 = (__pyx_v_self->decimal_places == Py_None);
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "drf_turbo/fields.pyx":717
 *         """
 *         if self.decimal_places is None:
 *             return value             # <<<<<<<<<<<<<<
 * 
 *         context = decimal.getcontext().copy()
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_value);
    __pyx_r = __pyx_v_value;
    goto __pyx_L0;

    /* "drf_turbo/fields.pyx":716
 *         Quantize the decimal value to the configured precision.
 *         """
 *         if self.decimal_places is None:             # <<<<<<<<<<<<<<
 *             return value
 * 
 */
  }

  /* "drf_turbo/fields.pyx":719
 *             return value
 * 
 *         context = decimal.getcontext().copy()             # <<<<<<<<<<<<<<
 *         if self.max_digits is not None:
 *             context.prec = self.max_digits
 */
  __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_decimal); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 719, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_getcontext); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 719, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_6))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_6);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_6, function);
    }
  }
  __pyx_t_4 = (__pyx_t_5) ? __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_5) : __Pyx_PyObject_CallNoArg(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 719, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_copy); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 719, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_6);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_6, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 719, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_v_context = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "drf_turbo/fields.pyx":720
 * 
 *         context = decimal.getcontext().copy()
 *         if self.max_digits is not None:             # <<<<<<<<<<<<<<
 *             context.prec = self.max_digits
 *         return value.quantize(
 */
  __pyx_t_2 = (__pyx_v_self->max_digits != Py_None);
  __pyx_t_1 = (__pyx_t_2 != 0);
  if (__pyx_t_1) {

    /* "drf_turbo/fields.pyx":721
 *         context = decimal.getcontext().copy()
 *         if self.max_digits is not None:
 *             context.prec = self.max_digits             # <<<<<<<<<<<<<<
 *         return value.quantize(
 *             decimal.Decimal('.1') ** self.decimal_places,
 */
    __pyx_t_3 = __pyx_v_self->max_digits;
    __Pyx_INCREF(__pyx_t_3);
    if (__Pyx_PyObject_SetAttrStr(__pyx_v_context, __pyx_n_s_prec, __pyx_t_3) < 0) __PYX_ERR(0, 721, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "drf_turbo/fields.pyx":720
 * 
 *         context = decimal.getcontext().copy()
 *         if self.max_digits is not None:             # <<<<<<<<<<<<<<
 *             context.prec = self.max_digits
 *         return value.quantize(
 */
  }

  /* "drf_turbo/fields.pyx":722
 *         if self.max_digits is not None:
 *             context.prec = self.max_digits
 *         return value.quantize(             # <<<<<<<<<<<<<<
 *             decimal.Decimal('.1') ** self.decimal_places,
 *             rounding=self.rounding,
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_value, __pyx_n_s_quantize); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 722, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "drf_turbo/fields.pyx":723
 *             context.prec = self.max_digits
 *         return value.quantize(
 *             decimal.Decimal('.1') ** self.decimal_places,             # <<<<<<<<<<<<<<
 *             rounding=self.rounding,
 *             context=context
 */
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_decimal); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 723, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_Decimal); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 723, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
    }
  }
  __pyx_t_6 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_4, __pyx_kp_s_1) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_kp_s_1);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 723, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = PyNumber_Power(__pyx_t_6, __pyx_v_self->decimal_places, Py_None); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 723, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

  /* "drf_turbo/fields.pyx":722
 *         if self.max_digits is not None:
 *             context.prec = self.max_digits
 *         return value.quantize(             # <<<<<<<<<<<<<<
 *             decimal.Decimal('.1') ** self.decimal_places,
 *             rounding=self.rounding,
 */
  __pyx_t_6 = PyTuple_New(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 722, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5);
  __pyx_t_5 = 0;

  /* "drf_turbo/fields.pyx":724
 *         return value.quantize(
 *             decimal.Decimal('.1') ** self.decimal_places,
 *             rounding=self.rounding,             # <<<<<<<<<<<<<<
 *             context=context
 *         )
 */
  __pyx_t_5 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 724, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  if (PyDict_SetItem(__pyx_t_5, __pyx_n_s_rounding, __pyx_v_self->rounding) < 0) __PYX_ERR(0, 724, __pyx_L1_error)

  /* "drf_turbo/fields.pyx":725
 *             decimal.Decimal('.1') ** self.decimal_places,
 *             rounding=self.rounding,
 *             context=context             # <<<<<<<<<<<<<<
 *         )
 * 
 */
  if (PyDict_SetItem(__pyx_t_5, __pyx_n_s_context, __pyx_v_context) < 0) __PYX_ERR(0, 724, __pyx_L1_error)

  /* "drf_turbo/fields.pyx":722
 *         if self.max_digits is not None:
 *             context.prec = self.max_digits
 *         return value.quantize(             # <<<<<<<<<<<<<<
 *             decimal.Decimal('.1') ** self.decimal_places,
 *             rounding=self.rounding,
 */
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_6, __pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 722, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "drf_turbo/fields.pyx":712
 *         return value
 * 
 *     cdef inline quantize(self, value):             # <<<<<<<<<<<<<<
 *         """
 *         Quantize the decimal value to the configured precision.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("drf_turbo.fields.DecimalField.quantize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_context);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pxd":80
 * cdef class DecimalField(Field):
 *     cdef public:
 *         max_digits             # <<<<<<<<<<<<<<
 *         decimal_places
 *         max_value
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_12DecimalField_10max_digits_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_12DecimalField_10max_digits_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_12DecimalField_10max_digits___get__(((struct __pyx_obj_9drf_turbo_6fields_DecimalField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_12DecimalField_10max_digits___get__(struct __pyx_obj_9drf_turbo_6fields_DecimalField *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->max_digits);
  __pyx_r = __pyx_v_self->max_digits;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_12DecimalField_10max_digits_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_12DecimalField_10max_digits_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_12DecimalField_10max_digits_2__set__(((struct __pyx_obj_9drf_turbo_6fields_DecimalField *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_12DecimalField_10max_digits_2__set__(struct __pyx_obj_9drf_turbo_6fields_DecimalField *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->max_digits);
  __Pyx_DECREF(__pyx_v_self->max_digits);
  __pyx_v_self->max_digits = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_12DecimalField_10max_digits_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_12DecimalField_10max_digits_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_12DecimalField_10max_digits_4__del__(((struct __pyx_obj_9drf_turbo_6fields_DecimalField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_12DecimalField_10max_digits_4__del__(struct __pyx_obj_9drf_turbo_6fields_DecimalField *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->max_digits);
  __Pyx_DECREF(__pyx_v_self->max_digits);
  __pyx_v_self->max_digits = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pxd":81
 *     cdef public:
 *         max_digits
 *         decimal_places             # <<<<<<<<<<<<<<
 *         max_value
 *         min_value
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_12DecimalField_14decimal_places_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_12DecimalField_14decimal_places_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_12DecimalField_14decimal_places___get__(((struct __pyx_obj_9drf_turbo_6fields_DecimalField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_12DecimalField_14decimal_places___get__(struct __pyx_obj_9drf_turbo_6fields_DecimalField *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->decimal_places);
  __pyx_r = __pyx_v_self->decimal_places;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_12DecimalField_14decimal_places_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_12DecimalField_14decimal_places_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_12DecimalField_14decimal_places_2__set__(((struct __pyx_obj_9drf_turbo_6fields_DecimalField *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_12DecimalField_14decimal_places_2__set__(struct __pyx_obj_9drf_turbo_6fields_DecimalField *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->decimal_places);
  __Pyx_DECREF(__pyx_v_self->decimal_places);
  __pyx_v_self->decimal_places = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_12DecimalField_14decimal_places_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_12DecimalField_14decimal_places_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_12DecimalField_14decimal_places_4__del__(((struct __pyx_obj_9drf_turbo_6fields_DecimalField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_12DecimalField_14decimal_places_4__del__(struct __pyx_obj_9drf_turbo_6fields_DecimalField *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->decimal_places);
  __Pyx_DECREF(__pyx_v_self->decimal_places);
  __pyx_v_self->decimal_places = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pxd":82
 *         max_digits
 *         decimal_places
 *         max_value             # <<<<<<<<<<<<<<
 *         min_value
 *         rounding
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_12DecimalField_9max_value_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_12DecimalField_9max_value_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_12DecimalField_9max_value___get__(((struct __pyx_obj_9drf_turbo_6fields_DecimalField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_12DecimalField_9max_value___get__(struct __pyx_obj_9drf_turbo_6fields_DecimalField *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->max_value);
  __pyx_r = __pyx_v_self->max_value;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_12DecimalField_9max_value_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_12DecimalField_9max_value_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_12DecimalField_9max_value_2__set__(((struct __pyx_obj_9drf_turbo_6fields_DecimalField *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_12DecimalField_9max_value_2__set__(struct __pyx_obj_9drf_turbo_6fields_DecimalField *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->max_value);
  __Pyx_DECREF(__pyx_v_self->max_value);
  __pyx_v_self->max_value = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_12DecimalField_9max_value_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_12DecimalField_9max_value_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_12DecimalField_9max_value_4__del__(((struct __pyx_obj_9drf_turbo_6fields_DecimalField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_12DecimalField_9max_value_4__del__(struct __pyx_obj_9drf_turbo_6fields_DecimalField *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->max_value);
  __Pyx_DECREF(__pyx_v_self->max_value);
  __pyx_v_self->max_value = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pxd":83
 *         decimal_places
 *         max_value
 *         min_value             # <<<<<<<<<<<<<<
 *         rounding
 *         coerce_to_string
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_12DecimalField_9min_value_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_12DecimalField_9min_value_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_12DecimalField_9min_value___get__(((struct __pyx_obj_9drf_turbo_6fields_DecimalField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_12DecimalField_9min_value___get__(struct __pyx_obj_9drf_turbo_6fields_DecimalField *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->min_value);
  __pyx_r = __pyx_v_self->min_value;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_12DecimalField_9min_value_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_12DecimalField_9min_value_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_12DecimalField_9min_value_2__set__(((struct __pyx_obj_9drf_turbo_6fields_DecimalField *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_12DecimalField_9min_value_2__set__(struct __pyx_obj_9drf_turbo_6fields_DecimalField *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->min_value);
  __Pyx_DECREF(__pyx_v_self->min_value);
  __pyx_v_self->min_value = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_12DecimalField_9min_value_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_12DecimalField_9min_value_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_12DecimalField_9min_value_4__del__(((struct __pyx_obj_9drf_turbo_6fields_DecimalField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_12DecimalField_9min_value_4__del__(struct __pyx_obj_9drf_turbo_6fields_DecimalField *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->min_value);
  __Pyx_DECREF(__pyx_v_self->min_value);
  __pyx_v_self->min_value = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pxd":84
 *         max_value
 *         min_value
 *         rounding             # <<<<<<<<<<<<<<
 *         coerce_to_string
 *         max_whole_digits
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_12DecimalField_8rounding_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_12DecimalField_8rounding_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_12DecimalField_8rounding___get__(((struct __pyx_obj_9drf_turbo_6fields_DecimalField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_12DecimalField_8rounding___get__(struct __pyx_obj_9drf_turbo_6fields_DecimalField *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->rounding);
  __pyx_r = __pyx_v_self->rounding;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_12DecimalField_8rounding_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_12DecimalField_8rounding_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_12DecimalField_8rounding_2__set__(((struct __pyx_obj_9drf_turbo_6fields_DecimalField *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_12DecimalField_8rounding_2__set__(struct __pyx_obj_9drf_turbo_6fields_DecimalField *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->rounding);
  __Pyx_DECREF(__pyx_v_self->rounding);
  __pyx_v_self->rounding = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_12DecimalField_8rounding_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_12DecimalField_8rounding_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_12DecimalField_8rounding_4__del__(((struct __pyx_obj_9drf_turbo_6fields_DecimalField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_12DecimalField_8rounding_4__del__(struct __pyx_obj_9drf_turbo_6fields_DecimalField *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->rounding);
  __Pyx_DECREF(__pyx_v_self->rounding);
  __pyx_v_self->rounding = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pxd":85
 *         min_value
 *         rounding
 *         coerce_to_string             # <<<<<<<<<<<<<<
 *         max_whole_digits
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_12DecimalField_16coerce_to_string_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_12DecimalField_16coerce_to_string_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_12DecimalField_16coerce_to_string___get__(((struct __pyx_obj_9drf_turbo_6fields_DecimalField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_12DecimalField_16coerce_to_string___get__(struct __pyx_obj_9drf_turbo_6fields_DecimalField *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->coerce_to_string);
  __pyx_r = __pyx_v_self->coerce_to_string;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_12DecimalField_16coerce_to_string_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_12DecimalField_16coerce_to_string_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_12DecimalField_16coerce_to_string_2__set__(((struct __pyx_obj_9drf_turbo_6fields_DecimalField *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_12DecimalField_16coerce_to_string_2__set__(struct __pyx_obj_9drf_turbo_6fields_DecimalField *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->coerce_to_string);
  __Pyx_DECREF(__pyx_v_self->coerce_to_string);
  __pyx_v_self->coerce_to_string = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_12DecimalField_16coerce_to_string_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_12DecimalField_16coerce_to_string_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_12DecimalField_16coerce_to_string_4__del__(((struct __pyx_obj_9drf_turbo_6fields_DecimalField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_12DecimalField_16coerce_to_string_4__del__(struct __pyx_obj_9drf_turbo_6fields_DecimalField *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->coerce_to_string);
  __Pyx_DECREF(__pyx_v_self->coerce_to_string);
  __pyx_v_self->coerce_to_string = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pxd":86
 *         rounding
 *         coerce_to_string
 *         max_whole_digits             # <<<<<<<<<<<<<<
 * 
 *     cdef validate_precision(self,value)
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_12DecimalField_16max_whole_digits_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_12DecimalField_16max_whole_digits_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_12DecimalField_16max_whole_digits___get__(((struct __pyx_obj_9drf_turbo_6fields_DecimalField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_12DecimalField_16max_whole_digits___get__(struct __pyx_obj_9drf_turbo_6fields_DecimalField *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->max_whole_digits);
  __pyx_r = __pyx_v_self->max_whole_digits;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_12DecimalField_16max_whole_digits_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_12DecimalField_16max_whole_digits_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_12DecimalField_16max_whole_digits_2__set__(((struct __pyx_obj_9drf_turbo_6fields_DecimalField *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_12DecimalField_16max_whole_digits_2__set__(struct __pyx_obj_9drf_turbo_6fields_DecimalField *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->max_whole_digits);
  __Pyx_DECREF(__pyx_v_self->max_whole_digits);
  __pyx_v_self->max_whole_digits = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_12DecimalField_16max_whole_digits_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_12DecimalField_16max_whole_digits_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_12DecimalField_16max_whole_digits_4__del__(((struct __pyx_obj_9drf_turbo_6fields_DecimalField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_12DecimalField_16max_whole_digits_4__del__(struct __pyx_obj_9drf_turbo_6fields_DecimalField *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->max_whole_digits);
  __Pyx_DECREF(__pyx_v_self->max_whole_digits);
  __pyx_v_self->max_whole_digits = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_12DecimalField_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_12DecimalField_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_12DecimalField_6__reduce_cython__(((struct __pyx_obj_9drf_turbo_6fields_DecimalField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_12DecimalField_6__reduce_cython__(struct __pyx_obj_9drf_turbo_6fields_DecimalField *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self.allow_null, self.attr, self.attrs, self.call, self.coerce_to_string, self.decimal_places, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.max_digits, self.max_value, self.max_whole_digits, self.min_value, self.read_only, self.required, self.root, self.rounding, self.style, self.validators, self.write_only)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.allow_null); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.call); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.read_only); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.required); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.write_only); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = PyTuple_New(23); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.attr);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.attr);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_v_self->__pyx_base.attr);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.attrs);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.attrs);
  PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_self->__pyx_base.attrs);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_6, 3, __pyx_t_2);
  __Pyx_INCREF(__pyx_v_self->coerce_to_string);
  __Pyx_GIVEREF(__pyx_v_self->coerce_to_string);
  PyTuple_SET_ITEM(__pyx_t_6, 4, __pyx_v_self->coerce_to_string);
  __Pyx_INCREF(__pyx_v_self->decimal_places);
  __Pyx_GIVEREF(__pyx_v_self->decimal_places);
  PyTuple_SET_ITEM(__pyx_t_6, 5, __pyx_v_self->decimal_places);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.default_value);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.default_value);
  PyTuple_SET_ITEM(__pyx_t_6, 6, __pyx_v_self->__pyx_base.default_value);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.error_messages);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.error_messages);
  PyTuple_SET_ITEM(__pyx_t_6, 7, __pyx_v_self->__pyx_base.error_messages);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.field_name);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.field_name);
  PyTuple_SET_ITEM(__pyx_t_6, 8, __pyx_v_self->__pyx_base.field_name);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.help_text);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.help_text);
  PyTuple_SET_ITEM(__pyx_t_6, 9, __pyx_v_self->__pyx_base.help_text);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.initial);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.initial);
  PyTuple_SET_ITEM(__pyx_t_6, 10, __pyx_v_self->__pyx_base.initial);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.label);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.label);
  PyTuple_SET_ITEM(__pyx_t_6, 11, __pyx_v_self->__pyx_base.label);
  __Pyx_INCREF(__pyx_v_self->max_digits);
  __Pyx_GIVEREF(__pyx_v_self->max_digits);
  PyTuple_SET_ITEM(__pyx_t_6, 12, __pyx_v_self->max_digits);
  __Pyx_INCREF(__pyx_v_self->max_value);
  __Pyx_GIVEREF(__pyx_v_self->max_value);
  PyTuple_SET_ITEM(__pyx_t_6, 13, __pyx_v_self->max_value);
  __Pyx_INCREF(__pyx_v_self->max_whole_digits);
  __Pyx_GIVEREF(__pyx_v_self->max_whole_digits);
  PyTuple_SET_ITEM(__pyx_t_6, 14, __pyx_v_self->max_whole_digits);
  __Pyx_INCREF(__pyx_v_self->min_value);
  __Pyx_GIVEREF(__pyx_v_self->min_value);
  PyTuple_SET_ITEM(__pyx_t_6, 15, __pyx_v_self->min_value);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 16, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_6, 17, __pyx_t_4);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.root);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.root);
  PyTuple_SET_ITEM(__pyx_t_6, 18, __pyx_v_self->__pyx_base.root);
  __Pyx_INCREF(__pyx_v_self->rounding);
  __Pyx_GIVEREF(__pyx_v_self->rounding);
  PyTuple_SET_ITEM(__pyx_t_6, 19, __pyx_v_self->rounding);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.style);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.style);
  PyTuple_SET_ITEM(__pyx_t_6, 20, __pyx_v_self->__pyx_base.style);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.validators);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.validators);
  PyTuple_SET_ITEM(__pyx_t_6, 21, __pyx_v_self->__pyx_base.validators);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_6, 22, __pyx_t_5);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_6);
  __pyx_t_6 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self.allow_null, self.attr, self.attrs, self.call, self.coerce_to_string, self.decimal_places, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.max_digits, self.max_value, self.max_whole_digits, self.min_value, self.read_only, self.required, self.root, self.rounding, self.style, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_6 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_v__dict = __pyx_t_6;
  __pyx_t_6 = 0;

  /* "(tree fragment)":7
 *     state = (self.allow_null, self.attr, self.attrs, self.call, self.coerce_to_string, self.decimal_places, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.max_digits, self.max_value, self.max_whole_digits, self.min_value, self.read_only, self.required, self.root, self.rounding, self.style, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_7 = (__pyx_v__dict != Py_None);
  __pyx_t_8 = (__pyx_t_7 != 0);
  if (__pyx_t_8) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_6 = PyTuple_New(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v__dict);
    __pyx_t_5 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_6); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_5));
    __pyx_t_5 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.attr is not None or self.attrs is not None or self.coerce_to_string is not None or self.decimal_places is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.max_digits is not None or self.max_value is not None or self.max_whole_digits is not None or self.min_value is not None or self.root is not None or self.rounding is not None or self.style is not None or self.validators is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self.allow_null, self.attr, self.attrs, self.call, self.coerce_to_string, self.decimal_places, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.max_digits, self.max_value, self.max_whole_digits, self.min_value, self.read_only, self.required, self.root, self.rounding, self.style, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self.attr is not None or self.attrs is not None or self.coerce_to_string is not None or self.decimal_places is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.max_digits is not None or self.max_value is not None or self.max_whole_digits is not None or self.min_value is not None or self.root is not None or self.rounding is not None or self.style is not None or self.validators is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_DecimalField, (type(self), 0xdaed7b0, None), state
 */
  /*else*/ {
    __pyx_t_7 = (__pyx_v_self->__pyx_base.attr != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.attrs != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->coerce_to_string != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->decimal_places != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.default_value != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.error_messages != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.field_name != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.help_text != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.initial != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.label != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->max_digits != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->max_value != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->max_whole_digits != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->min_value != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.root != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->rounding != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.style != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.validators != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    __pyx_t_8 = __pyx_t_7;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_8;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.attr is not None or self.attrs is not None or self.coerce_to_string is not None or self.decimal_places is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.max_digits is not None or self.max_value is not None or self.max_whole_digits is not None or self.min_value is not None or self.root is not None or self.rounding is not None or self.style is not None or self.validators is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_DecimalField, (type(self), 0xdaed7b0, None), state
 *     else:
 */
  __pyx_t_8 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_8) {

    /* "(tree fragment)":13
 *         use_setstate = self.attr is not None or self.attrs is not None or self.coerce_to_string is not None or self.decimal_places is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.max_digits is not None or self.max_value is not None or self.max_whole_digits is not None or self.min_value is not None or self.root is not None or self.rounding is not None or self.style is not None or self.validators is not None
 *     if use_setstate:
 *         return __pyx_unpickle_DecimalField, (type(self), 0xdaed7b0, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_DecimalField, (type(self), 0xdaed7b0, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_pyx_unpickle_DecimalField); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_6, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_229562288);
    __Pyx_GIVEREF(__pyx_int_229562288);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_int_229562288);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_6, 2, Py_None);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_6);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_state);
    __pyx_t_5 = 0;
    __pyx_t_6 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.attr is not None or self.attrs is not None or self.coerce_to_string is not None or self.decimal_places is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.max_digits is not None or self.max_value is not None or self.max_whole_digits is not None or self.min_value is not None or self.root is not None or self.rounding is not None or self.style is not None or self.validators is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_DecimalField, (type(self), 0xdaed7b0, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_DecimalField, (type(self), 0xdaed7b0, None), state
 *     else:
 *         return __pyx_unpickle_DecimalField, (type(self), 0xdaed7b0, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_DecimalField__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_pyx_unpickle_DecimalField); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_6, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_229562288);
    __Pyx_GIVEREF(__pyx_int_229562288);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_int_229562288);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_state);
    __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_6);
    __pyx_t_4 = 0;
    __pyx_t_6 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("drf_turbo.fields.DecimalField.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_DecimalField, (type(self), 0xdaed7b0, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_DecimalField__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_12DecimalField_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_12DecimalField_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_12DecimalField_8__setstate_cython__(((struct __pyx_obj_9drf_turbo_6fields_DecimalField *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_12DecimalField_8__setstate_cython__(struct __pyx_obj_9drf_turbo_6fields_DecimalField *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_DecimalField, (type(self), 0xdaed7b0, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_DecimalField__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields___pyx_unpickle_DecimalField__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_DecimalField, (type(self), 0xdaed7b0, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_DecimalField__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.DecimalField.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":763
 *     coerce_null_values = {"", "null","Null","NULL","none","None","NONE"}
 * 
 *     cpdef inline serialize(self,value,dict context):             # <<<<<<<<<<<<<<
 *         try:
 *             if self.allow_null and value in self.coerce_null_values:
 */

static PyObject *__pyx_pw_9drf_turbo_6fields_9BoolField_1serialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_9BoolField_serialize(struct __pyx_obj_9drf_turbo_6fields_BoolField *__pyx_v_self, PyObject *__pyx_v_value, CYTHON_UNUSED PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("serialize", 0);
  __Pyx_INCREF(__pyx_v_value);

  /* "drf_turbo/fields.pyx":764
 * 
 *     cpdef inline serialize(self,value,dict context):
 *         try:             # <<<<<<<<<<<<<<
 *             if self.allow_null and value in self.coerce_null_values:
 *                 return None
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "drf_turbo/fields.pyx":765
 *     cpdef inline serialize(self,value,dict context):
 *         try:
 *             if self.allow_null and value in self.coerce_null_values:             # <<<<<<<<<<<<<<
 *                 return None
 *             value = self.coerce_values[value]
 */
      __pyx_t_5 = (__pyx_v_self->__pyx_base.allow_null != 0);
      if (__pyx_t_5) {
      } else {
        __pyx_t_4 = __pyx_t_5;
        goto __pyx_L10_bool_binop_done;
      }
      __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_coerce_null_values); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 765, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_5 = (__Pyx_PySequence_ContainsTF(__pyx_v_value, __pyx_t_6, Py_EQ)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 765, __pyx_L3_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_7 = (__pyx_t_5 != 0);
      __pyx_t_4 = __pyx_t_7;
      __pyx_L10_bool_binop_done:;
      if (__pyx_t_4) {

        /* "drf_turbo/fields.pyx":766
 *         try:
 *             if self.allow_null and value in self.coerce_null_values:
 *                 return None             # <<<<<<<<<<<<<<
 *             value = self.coerce_values[value]
 *         except (KeyError,TypeError):
 */
        __Pyx_XDECREF(__pyx_r);
        __pyx_r = Py_None; __Pyx_INCREF(Py_None);
        goto __pyx_L7_try_return;

        /* "drf_turbo/fields.pyx":765
 *     cpdef inline serialize(self,value,dict context):
 *         try:
 *             if self.allow_null and value in self.coerce_null_values:             # <<<<<<<<<<<<<<
 *                 return None
 *             value = self.coerce_values[value]
 */
      }

      /* "drf_turbo/fields.pyx":767
 *             if self.allow_null and value in self.coerce_null_values:
 *                 return None
 *             value = self.coerce_values[value]             # <<<<<<<<<<<<<<
 *         except (KeyError,TypeError):
 *             pass
 */
      __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_coerce_values); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 767, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_8 = __Pyx_PyObject_GetItem(__pyx_t_6, __pyx_v_value); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 767, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF_SET(__pyx_v_value, __pyx_t_8);
      __pyx_t_8 = 0;

      /* "drf_turbo/fields.pyx":764
 * 
 *     cpdef inline serialize(self,value,dict context):
 *         try:             # <<<<<<<<<<<<<<
 *             if self.allow_null and value in self.coerce_null_values:
 *                 return None
 */
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;

    /* "drf_turbo/fields.pyx":768
 *                 return None
 *             value = self.coerce_values[value]
 *         except (KeyError,TypeError):             # <<<<<<<<<<<<<<
 *             pass
 *         return bool(value)
 */
    __pyx_t_9 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_KeyError) || __Pyx_PyErr_ExceptionMatches(__pyx_builtin_TypeError);
    if (__pyx_t_9) {
      __Pyx_ErrRestore(0,0,0);
      goto __pyx_L4_exception_handled;
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;

    /* "drf_turbo/fields.pyx":764
 * 
 *     cpdef inline serialize(self,value,dict context):
 *         try:             # <<<<<<<<<<<<<<
 *             if self.allow_null and value in self.coerce_null_values:
 *                 return None
 */
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L7_try_return:;
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L0;
    __pyx_L4_exception_handled:;
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    __pyx_L8_try_end:;
  }

  /* "drf_turbo/fields.pyx":770
 *         except (KeyError,TypeError):
 *             pass
 *         return bool(value)             # <<<<<<<<<<<<<<
 * 
 *     cpdef inline deserialize(self,data,dict context):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_v_value); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 770, __pyx_L1_error)
  __pyx_t_8 = __Pyx_PyBool_FromLong((!(!__pyx_t_4))); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 770, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_r = __pyx_t_8;
  __pyx_t_8 = 0;
  goto __pyx_L0;

  /* "drf_turbo/fields.pyx":763
 *     coerce_null_values = {"", "null","Null","NULL","none","None","NONE"}
 * 
 *     cpdef inline serialize(self,value,dict context):             # <<<<<<<<<<<<<<
 *         try:
 *             if self.allow_null and value in self.coerce_null_values:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("drf_turbo.fields.BoolField.serialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_value);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_9BoolField_1serialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_9BoolField_1serialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_value = 0;
  PyObject *__pyx_v_context = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("serialize (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_value,&__pyx_n_s_context,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_value)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_context)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("serialize", 1, 2, 2, 1); __PYX_ERR(0, 763, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "serialize") < 0)) __PYX_ERR(0, 763, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_value = values[0];
    __pyx_v_context = ((PyObject*)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("serialize", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 763, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.BoolField.serialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_context), (&PyDict_Type), 1, "context", 1))) __PYX_ERR(0, 763, __pyx_L1_error)
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9BoolField_serialize(((struct __pyx_obj_9drf_turbo_6fields_BoolField *)__pyx_v_self), __pyx_v_value, __pyx_v_context);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_9BoolField_serialize(struct __pyx_obj_9drf_turbo_6fields_BoolField *__pyx_v_self, PyObject *__pyx_v_value, PyObject *__pyx_v_context) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("serialize", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields_9BoolField_serialize(__pyx_v_self, __pyx_v_value, __pyx_v_context, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 763, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.BoolField.serialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":772
 *         return bool(value)
 * 
 *     cpdef inline deserialize(self,data,dict context):             # <<<<<<<<<<<<<<
 *         try:
 *             if self.allow_null and data in self.coerce_null_values:
 */

static PyObject *__pyx_pw_9drf_turbo_6fields_9BoolField_3deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_9BoolField_deserialize(struct __pyx_obj_9drf_turbo_6fields_BoolField *__pyx_v_self, PyObject *__pyx_v_data, CYTHON_UNUSED PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("deserialize", 0);
  __Pyx_INCREF(__pyx_v_data);

  /* "drf_turbo/fields.pyx":773
 * 
 *     cpdef inline deserialize(self,data,dict context):
 *         try:             # <<<<<<<<<<<<<<
 *             if self.allow_null and data in self.coerce_null_values:
 *                 return None
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "drf_turbo/fields.pyx":774
 *     cpdef inline deserialize(self,data,dict context):
 *         try:
 *             if self.allow_null and data in self.coerce_null_values:             # <<<<<<<<<<<<<<
 *                 return None
 *             data = self.coerce_values[data]
 */
      __pyx_t_5 = (__pyx_v_self->__pyx_base.allow_null != 0);
      if (__pyx_t_5) {
      } else {
        __pyx_t_4 = __pyx_t_5;
        goto __pyx_L10_bool_binop_done;
      }
      __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_coerce_null_values); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 774, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_5 = (__Pyx_PySequence_ContainsTF(__pyx_v_data, __pyx_t_6, Py_EQ)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 774, __pyx_L3_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_7 = (__pyx_t_5 != 0);
      __pyx_t_4 = __pyx_t_7;
      __pyx_L10_bool_binop_done:;
      if (__pyx_t_4) {

        /* "drf_turbo/fields.pyx":775
 *         try:
 *             if self.allow_null and data in self.coerce_null_values:
 *                 return None             # <<<<<<<<<<<<<<
 *             data = self.coerce_values[data]
 *         except (KeyError,TypeError):
 */
        __Pyx_XDECREF(__pyx_r);
        __pyx_r = Py_None; __Pyx_INCREF(Py_None);
        goto __pyx_L7_try_return;

        /* "drf_turbo/fields.pyx":774
 *     cpdef inline deserialize(self,data,dict context):
 *         try:
 *             if self.allow_null and data in self.coerce_null_values:             # <<<<<<<<<<<<<<
 *                 return None
 *             data = self.coerce_values[data]
 */
      }

      /* "drf_turbo/fields.pyx":776
 *             if self.allow_null and data in self.coerce_null_values:
 *                 return None
 *             data = self.coerce_values[data]             # <<<<<<<<<<<<<<
 *         except (KeyError,TypeError):
 *             raise self.raise_if_fail('invalid')
 */
      __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_coerce_values); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 776, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_8 = __Pyx_PyObject_GetItem(__pyx_t_6, __pyx_v_data); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 776, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF_SET(__pyx_v_data, __pyx_t_8);
      __pyx_t_8 = 0;

      /* "drf_turbo/fields.pyx":773
 * 
 *     cpdef inline deserialize(self,data,dict context):
 *         try:             # <<<<<<<<<<<<<<
 *             if self.allow_null and data in self.coerce_null_values:
 *                 return None
 */
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;

    /* "drf_turbo/fields.pyx":777
 *                 return None
 *             data = self.coerce_values[data]
 *         except (KeyError,TypeError):             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail('invalid')
 *         return data
 */
    __pyx_t_9 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_KeyError) || __Pyx_PyErr_ExceptionMatches(__pyx_builtin_TypeError);
    if (__pyx_t_9) {
      __Pyx_AddTraceback("drf_turbo.fields.BoolField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_8, &__pyx_t_6, &__pyx_t_10) < 0) __PYX_ERR(0, 777, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GOTREF(__pyx_t_10);

      /* "drf_turbo/fields.pyx":778
 *             data = self.coerce_values[data]
 *         except (KeyError,TypeError):
 *             raise self.raise_if_fail('invalid')             # <<<<<<<<<<<<<<
 *         return data
 * 
 */
      __pyx_t_12 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_raise_if_fail); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 778, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_12);
      __pyx_t_13 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_12))) {
        __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_12);
        if (likely(__pyx_t_13)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_12);
          __Pyx_INCREF(__pyx_t_13);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_12, function);
        }
      }
      __pyx_t_11 = (__pyx_t_13) ? __Pyx_PyObject_Call2Args(__pyx_t_12, __pyx_t_13, __pyx_n_s_invalid) : __Pyx_PyObject_CallOneArg(__pyx_t_12, __pyx_n_s_invalid);
      __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
      if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 778, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_11);
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      __Pyx_Raise(__pyx_t_11, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      __PYX_ERR(0, 778, __pyx_L5_except_error)
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;

    /* "drf_turbo/fields.pyx":773
 * 
 *     cpdef inline deserialize(self,data,dict context):
 *         try:             # <<<<<<<<<<<<<<
 *             if self.allow_null and data in self.coerce_null_values:
 *                 return None
 */
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L7_try_return:;
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L0;
    __pyx_L8_try_end:;
  }

  /* "drf_turbo/fields.pyx":779
 *         except (KeyError,TypeError):
 *             raise self.raise_if_fail('invalid')
 *         return data             # <<<<<<<<<<<<<<
 * 
 * cdef class ChoiceField(Field):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_data);
  __pyx_r = __pyx_v_data;
  goto __pyx_L0;

  /* "drf_turbo/fields.pyx":772
 *         return bool(value)
 * 
 *     cpdef inline deserialize(self,data,dict context):             # <<<<<<<<<<<<<<
 *         try:
 *             if self.allow_null and data in self.coerce_null_values:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_AddTraceback("drf_turbo.fields.BoolField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_data);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_9BoolField_3deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_9BoolField_3deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_data = 0;
  PyObject *__pyx_v_context = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("deserialize (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_data,&__pyx_n_s_context,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_data)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_context)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("deserialize", 1, 2, 2, 1); __PYX_ERR(0, 772, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "deserialize") < 0)) __PYX_ERR(0, 772, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_data = values[0];
    __pyx_v_context = ((PyObject*)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("deserialize", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 772, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.BoolField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_context), (&PyDict_Type), 1, "context", 1))) __PYX_ERR(0, 772, __pyx_L1_error)
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9BoolField_2deserialize(((struct __pyx_obj_9drf_turbo_6fields_BoolField *)__pyx_v_self), __pyx_v_data, __pyx_v_context);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_9BoolField_2deserialize(struct __pyx_obj_9drf_turbo_6fields_BoolField *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("deserialize", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields_9BoolField_deserialize(__pyx_v_self, __pyx_v_data, __pyx_v_context, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 772, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.BoolField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_9BoolField_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_9BoolField_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9BoolField_4__reduce_cython__(((struct __pyx_obj_9drf_turbo_6fields_BoolField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_9BoolField_4__reduce_cython__(struct __pyx_obj_9drf_turbo_6fields_BoolField *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self.allow_null, self.attr, self.attrs, self.call, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.allow_null); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.call); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.read_only); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.required); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.write_only); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = PyTuple_New(16); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.attr);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.attr);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_v_self->__pyx_base.attr);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.attrs);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.attrs);
  PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_self->__pyx_base.attrs);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_6, 3, __pyx_t_2);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.default_value);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.default_value);
  PyTuple_SET_ITEM(__pyx_t_6, 4, __pyx_v_self->__pyx_base.default_value);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.error_messages);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.error_messages);
  PyTuple_SET_ITEM(__pyx_t_6, 5, __pyx_v_self->__pyx_base.error_messages);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.field_name);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.field_name);
  PyTuple_SET_ITEM(__pyx_t_6, 6, __pyx_v_self->__pyx_base.field_name);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.help_text);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.help_text);
  PyTuple_SET_ITEM(__pyx_t_6, 7, __pyx_v_self->__pyx_base.help_text);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.initial);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.initial);
  PyTuple_SET_ITEM(__pyx_t_6, 8, __pyx_v_self->__pyx_base.initial);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.label);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.label);
  PyTuple_SET_ITEM(__pyx_t_6, 9, __pyx_v_self->__pyx_base.label);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 10, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_6, 11, __pyx_t_4);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.root);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.root);
  PyTuple_SET_ITEM(__pyx_t_6, 12, __pyx_v_self->__pyx_base.root);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.style);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.style);
  PyTuple_SET_ITEM(__pyx_t_6, 13, __pyx_v_self->__pyx_base.style);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.validators);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.validators);
  PyTuple_SET_ITEM(__pyx_t_6, 14, __pyx_v_self->__pyx_base.validators);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_6, 15, __pyx_t_5);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_6);
  __pyx_t_6 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self.allow_null, self.attr, self.attrs, self.call, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_6 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_v__dict = __pyx_t_6;
  __pyx_t_6 = 0;

  /* "(tree fragment)":7
 *     state = (self.allow_null, self.attr, self.attrs, self.call, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_7 = (__pyx_v__dict != Py_None);
  __pyx_t_8 = (__pyx_t_7 != 0);
  if (__pyx_t_8) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_6 = PyTuple_New(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v__dict);
    __pyx_t_5 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_6); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_5));
    __pyx_t_5 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.root is not None or self.style is not None or self.validators is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self.allow_null, self.attr, self.attrs, self.call, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.root is not None or self.style is not None or self.validators is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_BoolField, (type(self), 0x4f70d27, None), state
 */
  /*else*/ {
    __pyx_t_7 = (__pyx_v_self->__pyx_base.attr != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.attrs != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.default_value != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.error_messages != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.field_name != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.help_text != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.initial != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.label != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.root != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.style != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.validators != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    __pyx_t_8 = __pyx_t_9;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_8;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.root is not None or self.style is not None or self.validators is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_BoolField, (type(self), 0x4f70d27, None), state
 *     else:
 */
  __pyx_t_8 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_8) {

    /* "(tree fragment)":13
 *         use_setstate = self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.root is not None or self.style is not None or self.validators is not None
 *     if use_setstate:
 *         return __pyx_unpickle_BoolField, (type(self), 0x4f70d27, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_BoolField, (type(self), 0x4f70d27, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_pyx_unpickle_BoolField); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_6, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_83299623);
    __Pyx_GIVEREF(__pyx_int_83299623);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_int_83299623);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_6, 2, Py_None);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_6);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_state);
    __pyx_t_5 = 0;
    __pyx_t_6 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.root is not None or self.style is not None or self.validators is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_BoolField, (type(self), 0x4f70d27, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_BoolField, (type(self), 0x4f70d27, None), state
 *     else:
 *         return __pyx_unpickle_BoolField, (type(self), 0x4f70d27, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_BoolField__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_pyx_unpickle_BoolField); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_6, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_83299623);
    __Pyx_GIVEREF(__pyx_int_83299623);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_int_83299623);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_state);
    __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_6);
    __pyx_t_4 = 0;
    __pyx_t_6 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("drf_turbo.fields.BoolField.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_BoolField, (type(self), 0x4f70d27, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_BoolField__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_9BoolField_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_9BoolField_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9BoolField_6__setstate_cython__(((struct __pyx_obj_9drf_turbo_6fields_BoolField *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_9BoolField_6__setstate_cython__(struct __pyx_obj_9drf_turbo_6fields_BoolField *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_BoolField, (type(self), 0x4f70d27, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_BoolField__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields___pyx_unpickle_BoolField__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_BoolField, (type(self), 0x4f70d27, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_BoolField__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.BoolField.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":792
 *     }
 * 
 *     def __init__(self, choices, **kwargs):             # <<<<<<<<<<<<<<
 * 
 *         pairs = [
 */

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_11ChoiceField_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_11ChoiceField_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_choices = 0;
  PyObject *__pyx_v_kwargs = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  __pyx_v_kwargs = PyDict_New(); if (unlikely(!__pyx_v_kwargs)) return -1;
  __Pyx_GOTREF(__pyx_v_kwargs);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_choices,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_choices)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, __pyx_v_kwargs, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 792, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_choices = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 792, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_DECREF(__pyx_v_kwargs); __pyx_v_kwargs = 0;
  __Pyx_AddTraceback("drf_turbo.fields.ChoiceField.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9drf_turbo_6fields_11ChoiceField___init__(((struct __pyx_obj_9drf_turbo_6fields_ChoiceField *)__pyx_v_self), __pyx_v_choices, __pyx_v_kwargs);

  /* function exit code */
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_11ChoiceField___init__(struct __pyx_obj_9drf_turbo_6fields_ChoiceField *__pyx_v_self, PyObject *__pyx_v_choices, PyObject *__pyx_v_kwargs) {
  PyObject *__pyx_v_pairs = NULL;
  PyObject *__pyx_v_item = NULL;
  PyObject *__pyx_v_key = NULL;
  PyObject *__pyx_v_display_value = NULL;
  PyObject *__pyx_v_value = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  PyObject *(*__pyx_t_4)(PyObject *);
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  Py_ssize_t __pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *(*__pyx_t_13)(PyObject *);
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "drf_turbo/fields.pyx":794
 *     def __init__(self, choices, **kwargs):
 * 
 *         pairs = [             # <<<<<<<<<<<<<<
 *             isinstance(item, (list, tuple)) and len(item) == 2
 *             for item in choices
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 794, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);

  /* "drf_turbo/fields.pyx":796
 *         pairs = [
 *             isinstance(item, (list, tuple)) and len(item) == 2
 *             for item in choices             # <<<<<<<<<<<<<<
 *         ]
 *         if all(pairs):
 */
  if (likely(PyList_CheckExact(__pyx_v_choices)) || PyTuple_CheckExact(__pyx_v_choices)) {
    __pyx_t_2 = __pyx_v_choices; __Pyx_INCREF(__pyx_t_2); __pyx_t_3 = 0;
    __pyx_t_4 = NULL;
  } else {
    __pyx_t_3 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_v_choices); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 796, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 796, __pyx_L1_error)
  }
  for (;;) {
    if (likely(!__pyx_t_4)) {
      if (likely(PyList_CheckExact(__pyx_t_2))) {
        if (__pyx_t_3 >= PyList_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_5 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_3); __Pyx_INCREF(__pyx_t_5); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 796, __pyx_L1_error)
        #else
        __pyx_t_5 = PySequence_ITEM(__pyx_t_2, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 796, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        #endif
      } else {
        if (__pyx_t_3 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_5 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_3); __Pyx_INCREF(__pyx_t_5); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 796, __pyx_L1_error)
        #else
        __pyx_t_5 = PySequence_ITEM(__pyx_t_2, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 796, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        #endif
      }
    } else {
      __pyx_t_5 = __pyx_t_4(__pyx_t_2);
      if (unlikely(!__pyx_t_5)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 796, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_5);
    }
    __Pyx_XDECREF_SET(__pyx_v_item, __pyx_t_5);
    __pyx_t_5 = 0;

    /* "drf_turbo/fields.pyx":795
 * 
 *         pairs = [
 *             isinstance(item, (list, tuple)) and len(item) == 2             # <<<<<<<<<<<<<<
 *             for item in choices
 *         ]
 */
    __pyx_t_7 = PyList_Check(__pyx_v_item); 
    __pyx_t_8 = (__pyx_t_7 != 0);
    if (!__pyx_t_8) {
    } else {
      __pyx_t_6 = __pyx_t_8;
      goto __pyx_L7_bool_binop_done;
    }
    __pyx_t_8 = PyTuple_Check(__pyx_v_item); 
    __pyx_t_7 = (__pyx_t_8 != 0);
    __pyx_t_6 = __pyx_t_7;
    __pyx_L7_bool_binop_done:;
    if (__pyx_t_6) {
    } else {
      __pyx_t_9 = __Pyx_PyBool_FromLong(__pyx_t_6); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 795, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_5 = __pyx_t_9;
      __pyx_t_9 = 0;
      goto __pyx_L5_bool_binop_done;
    }
    __pyx_t_10 = PyObject_Length(__pyx_v_item); if (unlikely(__pyx_t_10 == ((Py_ssize_t)-1))) __PYX_ERR(0, 795, __pyx_L1_error)
    __pyx_t_6 = (__pyx_t_10 == 2);
    __pyx_t_9 = __Pyx_PyBool_FromLong(__pyx_t_6); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 795, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_5 = __pyx_t_9;
    __pyx_t_9 = 0;
    __pyx_L5_bool_binop_done:;
    if (unlikely(__Pyx_ListComp_Append(__pyx_t_1, (PyObject*)__pyx_t_5))) __PYX_ERR(0, 794, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "drf_turbo/fields.pyx":796
 *         pairs = [
 *             isinstance(item, (list, tuple)) and len(item) == 2
 *             for item in choices             # <<<<<<<<<<<<<<
 *         ]
 *         if all(pairs):
 */
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_pairs = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "drf_turbo/fields.pyx":798
 *             for item in choices
 *         ]
 *         if all(pairs):             # <<<<<<<<<<<<<<
 *             self.choices = dict([(key, display_value) for key, display_value in choices])
 *         else:
 */
  __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_builtin_all, __pyx_v_pairs); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 798, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 798, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_6) {

    /* "drf_turbo/fields.pyx":799
 *         ]
 *         if all(pairs):
 *             self.choices = dict([(key, display_value) for key, display_value in choices])             # <<<<<<<<<<<<<<
 *         else:
 *             self.choices = dict([(item, item) for item in choices])
 */
    __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 799, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (likely(PyList_CheckExact(__pyx_v_choices)) || PyTuple_CheckExact(__pyx_v_choices)) {
      __pyx_t_2 = __pyx_v_choices; __Pyx_INCREF(__pyx_t_2); __pyx_t_3 = 0;
      __pyx_t_4 = NULL;
    } else {
      __pyx_t_3 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_v_choices); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 799, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_4 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 799, __pyx_L1_error)
    }
    for (;;) {
      if (likely(!__pyx_t_4)) {
        if (likely(PyList_CheckExact(__pyx_t_2))) {
          if (__pyx_t_3 >= PyList_GET_SIZE(__pyx_t_2)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_5 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_3); __Pyx_INCREF(__pyx_t_5); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 799, __pyx_L1_error)
          #else
          __pyx_t_5 = PySequence_ITEM(__pyx_t_2, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 799, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          #endif
        } else {
          if (__pyx_t_3 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_5 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_3); __Pyx_INCREF(__pyx_t_5); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 799, __pyx_L1_error)
          #else
          __pyx_t_5 = PySequence_ITEM(__pyx_t_2, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 799, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          #endif
        }
      } else {
        __pyx_t_5 = __pyx_t_4(__pyx_t_2);
        if (unlikely(!__pyx_t_5)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 799, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_5);
      }
      if ((likely(PyTuple_CheckExact(__pyx_t_5))) || (PyList_CheckExact(__pyx_t_5))) {
        PyObject* sequence = __pyx_t_5;
        Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
        if (unlikely(size != 2)) {
          if (size > 2) __Pyx_RaiseTooManyValuesError(2);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          __PYX_ERR(0, 799, __pyx_L1_error)
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_9 = PyTuple_GET_ITEM(sequence, 0); 
          __pyx_t_11 = PyTuple_GET_ITEM(sequence, 1); 
        } else {
          __pyx_t_9 = PyList_GET_ITEM(sequence, 0); 
          __pyx_t_11 = PyList_GET_ITEM(sequence, 1); 
        }
        __Pyx_INCREF(__pyx_t_9);
        __Pyx_INCREF(__pyx_t_11);
        #else
        __pyx_t_9 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 799, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_9);
        __pyx_t_11 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 799, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        #endif
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_12 = PyObject_GetIter(__pyx_t_5); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 799, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_12);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __pyx_t_13 = Py_TYPE(__pyx_t_12)->tp_iternext;
        index = 0; __pyx_t_9 = __pyx_t_13(__pyx_t_12); if (unlikely(!__pyx_t_9)) goto __pyx_L12_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_9);
        index = 1; __pyx_t_11 = __pyx_t_13(__pyx_t_12); if (unlikely(!__pyx_t_11)) goto __pyx_L12_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_11);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_13(__pyx_t_12), 2) < 0) __PYX_ERR(0, 799, __pyx_L1_error)
        __pyx_t_13 = NULL;
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        goto __pyx_L13_unpacking_done;
        __pyx_L12_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        __pyx_t_13 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        __PYX_ERR(0, 799, __pyx_L1_error)
        __pyx_L13_unpacking_done:;
      }
      __Pyx_XDECREF_SET(__pyx_v_key, __pyx_t_9);
      __pyx_t_9 = 0;
      __Pyx_XDECREF_SET(__pyx_v_display_value, __pyx_t_11);
      __pyx_t_11 = 0;
      __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 799, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_INCREF(__pyx_v_key);
      __Pyx_GIVEREF(__pyx_v_key);
      PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_v_key);
      __Pyx_INCREF(__pyx_v_display_value);
      __Pyx_GIVEREF(__pyx_v_display_value);
      PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_v_display_value);
      if (unlikely(__Pyx_ListComp_Append(__pyx_t_1, (PyObject*)__pyx_t_5))) __PYX_ERR(0, 799, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyDict_Type)), __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 799, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_GIVEREF(__pyx_t_2);
    __Pyx_GOTREF(__pyx_v_self->choices);
    __Pyx_DECREF(__pyx_v_self->choices);
    __pyx_v_self->choices = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "drf_turbo/fields.pyx":798
 *             for item in choices
 *         ]
 *         if all(pairs):             # <<<<<<<<<<<<<<
 *             self.choices = dict([(key, display_value) for key, display_value in choices])
 *         else:
 */
    goto __pyx_L9;
  }

  /* "drf_turbo/fields.pyx":801
 *             self.choices = dict([(key, display_value) for key, display_value in choices])
 *         else:
 *             self.choices = dict([(item, item) for item in choices])             # <<<<<<<<<<<<<<
 * 
 *         self.choice_strings_to_values = dict([
 */
  /*else*/ {
    __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 801, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    if (likely(PyList_CheckExact(__pyx_v_choices)) || PyTuple_CheckExact(__pyx_v_choices)) {
      __pyx_t_1 = __pyx_v_choices; __Pyx_INCREF(__pyx_t_1); __pyx_t_3 = 0;
      __pyx_t_4 = NULL;
    } else {
      __pyx_t_3 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_v_choices); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 801, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_4 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 801, __pyx_L1_error)
    }
    for (;;) {
      if (likely(!__pyx_t_4)) {
        if (likely(PyList_CheckExact(__pyx_t_1))) {
          if (__pyx_t_3 >= PyList_GET_SIZE(__pyx_t_1)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_5 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_3); __Pyx_INCREF(__pyx_t_5); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 801, __pyx_L1_error)
          #else
          __pyx_t_5 = PySequence_ITEM(__pyx_t_1, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 801, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          #endif
        } else {
          if (__pyx_t_3 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_5 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_3); __Pyx_INCREF(__pyx_t_5); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 801, __pyx_L1_error)
          #else
          __pyx_t_5 = PySequence_ITEM(__pyx_t_1, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 801, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          #endif
        }
      } else {
        __pyx_t_5 = __pyx_t_4(__pyx_t_1);
        if (unlikely(!__pyx_t_5)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 801, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_5);
      }
      __Pyx_XDECREF_SET(__pyx_v_item, __pyx_t_5);
      __pyx_t_5 = 0;
      __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 801, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_INCREF(__pyx_v_item);
      __Pyx_GIVEREF(__pyx_v_item);
      PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_v_item);
      __Pyx_INCREF(__pyx_v_item);
      __Pyx_GIVEREF(__pyx_v_item);
      PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_v_item);
      if (unlikely(__Pyx_ListComp_Append(__pyx_t_2, (PyObject*)__pyx_t_5))) __PYX_ERR(0, 801, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyDict_Type)), __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 801, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GIVEREF(__pyx_t_1);
    __Pyx_GOTREF(__pyx_v_self->choices);
    __Pyx_DECREF(__pyx_v_self->choices);
    __pyx_v_self->choices = __pyx_t_1;
    __pyx_t_1 = 0;
  }
  __pyx_L9:;

  /* "drf_turbo/fields.pyx":803
 *             self.choices = dict([(item, item) for item in choices])
 * 
 *         self.choice_strings_to_values = dict([             # <<<<<<<<<<<<<<
 *             (str(key), key) for key in self.choices.keys()
 *         ])
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 803, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);

  /* "drf_turbo/fields.pyx":804
 * 
 *         self.choice_strings_to_values = dict([
 *             (str(key), key) for key in self.choices.keys()             # <<<<<<<<<<<<<<
 *         ])
 * 
 */
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->choices, __pyx_n_s_keys); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 804, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_11 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_11)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_11);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
    }
  }
  __pyx_t_2 = (__pyx_t_11) ? __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_11) : __Pyx_PyObject_CallNoArg(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 804, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (likely(PyList_CheckExact(__pyx_t_2)) || PyTuple_CheckExact(__pyx_t_2)) {
    __pyx_t_5 = __pyx_t_2; __Pyx_INCREF(__pyx_t_5); __pyx_t_3 = 0;
    __pyx_t_4 = NULL;
  } else {
    __pyx_t_3 = -1; __pyx_t_5 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 804, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_4 = Py_TYPE(__pyx_t_5)->tp_iternext; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 804, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  for (;;) {
    if (likely(!__pyx_t_4)) {
      if (likely(PyList_CheckExact(__pyx_t_5))) {
        if (__pyx_t_3 >= PyList_GET_SIZE(__pyx_t_5)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_2 = PyList_GET_ITEM(__pyx_t_5, __pyx_t_3); __Pyx_INCREF(__pyx_t_2); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 804, __pyx_L1_error)
        #else
        __pyx_t_2 = PySequence_ITEM(__pyx_t_5, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 804, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        #endif
      } else {
        if (__pyx_t_3 >= PyTuple_GET_SIZE(__pyx_t_5)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_2 = PyTuple_GET_ITEM(__pyx_t_5, __pyx_t_3); __Pyx_INCREF(__pyx_t_2); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 804, __pyx_L1_error)
        #else
        __pyx_t_2 = PySequence_ITEM(__pyx_t_5, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 804, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        #endif
      }
    } else {
      __pyx_t_2 = __pyx_t_4(__pyx_t_5);
      if (unlikely(!__pyx_t_2)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 804, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_XDECREF_SET(__pyx_v_key, __pyx_t_2);
    __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), __pyx_v_key); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 804, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_11 = PyTuple_New(2); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 804, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_2);
    __Pyx_INCREF(__pyx_v_key);
    __Pyx_GIVEREF(__pyx_v_key);
    PyTuple_SET_ITEM(__pyx_t_11, 1, __pyx_v_key);
    __pyx_t_2 = 0;
    if (unlikely(__Pyx_ListComp_Append(__pyx_t_1, (PyObject*)__pyx_t_11))) __PYX_ERR(0, 803, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  }
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "drf_turbo/fields.pyx":803
 *             self.choices = dict([(item, item) for item in choices])
 * 
 *         self.choice_strings_to_values = dict([             # <<<<<<<<<<<<<<
 *             (str(key), key) for key in self.choices.keys()
 *         ])
 */
  __pyx_t_5 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyDict_Type)), __pyx_t_1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 803, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_GIVEREF(__pyx_t_5);
  __Pyx_GOTREF(__pyx_v_self->choice_strings_to_values);
  __Pyx_DECREF(__pyx_v_self->choice_strings_to_values);
  __pyx_v_self->choice_strings_to_values = __pyx_t_5;
  __pyx_t_5 = 0;

  /* "drf_turbo/fields.pyx":807
 *         ])
 * 
 *         self.choice_strings_to_display = dict([             # <<<<<<<<<<<<<<
 *             (str(key), value) for key,value in self.choices.items()
 *         ])
 */
  __pyx_t_5 = PyList_New(0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 807, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);

  /* "drf_turbo/fields.pyx":808
 * 
 *         self.choice_strings_to_display = dict([
 *             (str(key), value) for key,value in self.choices.items()             # <<<<<<<<<<<<<<
 *         ])
 * 
 */
  __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->choices, __pyx_n_s_items); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 808, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_11))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_11);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_11);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_11, function);
    }
  }
  __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_11, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 808, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
    __pyx_t_11 = __pyx_t_1; __Pyx_INCREF(__pyx_t_11); __pyx_t_3 = 0;
    __pyx_t_4 = NULL;
  } else {
    __pyx_t_3 = -1; __pyx_t_11 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 808, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __pyx_t_4 = Py_TYPE(__pyx_t_11)->tp_iternext; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 808, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  for (;;) {
    if (likely(!__pyx_t_4)) {
      if (likely(PyList_CheckExact(__pyx_t_11))) {
        if (__pyx_t_3 >= PyList_GET_SIZE(__pyx_t_11)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyList_GET_ITEM(__pyx_t_11, __pyx_t_3); __Pyx_INCREF(__pyx_t_1); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 808, __pyx_L1_error)
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_11, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 808, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      } else {
        if (__pyx_t_3 >= PyTuple_GET_SIZE(__pyx_t_11)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_11, __pyx_t_3); __Pyx_INCREF(__pyx_t_1); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 808, __pyx_L1_error)
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_11, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 808, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      }
    } else {
      __pyx_t_1 = __pyx_t_4(__pyx_t_11);
      if (unlikely(!__pyx_t_1)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 808, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_1);
    }
    if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
      PyObject* sequence = __pyx_t_1;
      Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
      if (unlikely(size != 2)) {
        if (size > 2) __Pyx_RaiseTooManyValuesError(2);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 808, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      if (likely(PyTuple_CheckExact(sequence))) {
        __pyx_t_2 = PyTuple_GET_ITEM(sequence, 0); 
        __pyx_t_9 = PyTuple_GET_ITEM(sequence, 1); 
      } else {
        __pyx_t_2 = PyList_GET_ITEM(sequence, 0); 
        __pyx_t_9 = PyList_GET_ITEM(sequence, 1); 
      }
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_9);
      #else
      __pyx_t_2 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 808, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_9 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 808, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    } else {
      Py_ssize_t index = -1;
      __pyx_t_12 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 808, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_12);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_13 = Py_TYPE(__pyx_t_12)->tp_iternext;
      index = 0; __pyx_t_2 = __pyx_t_13(__pyx_t_12); if (unlikely(!__pyx_t_2)) goto __pyx_L20_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_2);
      index = 1; __pyx_t_9 = __pyx_t_13(__pyx_t_12); if (unlikely(!__pyx_t_9)) goto __pyx_L20_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_9);
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_13(__pyx_t_12), 2) < 0) __PYX_ERR(0, 808, __pyx_L1_error)
      __pyx_t_13 = NULL;
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      goto __pyx_L21_unpacking_done;
      __pyx_L20_unpacking_failed:;
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      __pyx_t_13 = NULL;
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
      __PYX_ERR(0, 808, __pyx_L1_error)
      __pyx_L21_unpacking_done:;
    }
    __Pyx_XDECREF_SET(__pyx_v_key, __pyx_t_2);
    __pyx_t_2 = 0;
    __Pyx_XDECREF_SET(__pyx_v_value, __pyx_t_9);
    __pyx_t_9 = 0;
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), __pyx_v_key); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 808, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_9 = PyTuple_New(2); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 808, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_1);
    __Pyx_INCREF(__pyx_v_value);
    __Pyx_GIVEREF(__pyx_v_value);
    PyTuple_SET_ITEM(__pyx_t_9, 1, __pyx_v_value);
    __pyx_t_1 = 0;
    if (unlikely(__Pyx_ListComp_Append(__pyx_t_5, (PyObject*)__pyx_t_9))) __PYX_ERR(0, 807, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  }
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

  /* "drf_turbo/fields.pyx":807
 *         ])
 * 
 *         self.choice_strings_to_display = dict([             # <<<<<<<<<<<<<<
 *             (str(key), value) for key,value in self.choices.items()
 *         ])
 */
  __pyx_t_11 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyDict_Type)), __pyx_t_5); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 807, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_GIVEREF(__pyx_t_11);
  __Pyx_GOTREF(__pyx_v_self->choice_strings_to_display);
  __Pyx_DECREF(__pyx_v_self->choice_strings_to_display);
  __pyx_v_self->choice_strings_to_display = __pyx_t_11;
  __pyx_t_11 = 0;

  /* "drf_turbo/fields.pyx":812
 * 
 * 
 *         self.allow_blank = kwargs.pop('allow_blank', False)             # <<<<<<<<<<<<<<
 *         super().__init__(**kwargs)
 * 
 */
  __pyx_t_11 = __Pyx_PyDict_Pop(__pyx_v_kwargs, __pyx_n_s_allow_blank, Py_False); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 812, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_GIVEREF(__pyx_t_11);
  __Pyx_GOTREF(__pyx_v_self->allow_blank);
  __Pyx_DECREF(__pyx_v_self->allow_blank);
  __pyx_v_self->allow_blank = __pyx_t_11;
  __pyx_t_11 = 0;

  /* "drf_turbo/fields.pyx":813
 * 
 *         self.allow_blank = kwargs.pop('allow_blank', False)
 *         super().__init__(**kwargs)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_11 = PyTuple_New(2); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 813, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_9drf_turbo_6fields_ChoiceField));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_9drf_turbo_6fields_ChoiceField));
  PyTuple_SET_ITEM(__pyx_t_11, 0, ((PyObject *)__pyx_ptype_9drf_turbo_6fields_ChoiceField));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_11, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_5 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_11, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 813, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_init); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 813, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = PyDict_Copy(__pyx_v_kwargs); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 813, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_9 = __Pyx_PyObject_Call(__pyx_t_11, __pyx_empty_tuple, __pyx_t_5); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 813, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

  /* "drf_turbo/fields.pyx":792
 *     }
 * 
 *     def __init__(self, choices, **kwargs):             # <<<<<<<<<<<<<<
 * 
 *         pairs = [
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_AddTraceback("drf_turbo.fields.ChoiceField.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_pairs);
  __Pyx_XDECREF(__pyx_v_item);
  __Pyx_XDECREF(__pyx_v_key);
  __Pyx_XDECREF(__pyx_v_display_value);
  __Pyx_XDECREF(__pyx_v_value);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":817
 * 
 * 
 *     cpdef serialize(self,value,dict context):             # <<<<<<<<<<<<<<
 *         if value in ('', None):
 *             return value
 */

static PyObject *__pyx_pw_9drf_turbo_6fields_11ChoiceField_3serialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_11ChoiceField_serialize(struct __pyx_obj_9drf_turbo_6fields_ChoiceField *__pyx_v_self, PyObject *__pyx_v_value, CYTHON_UNUSED PyObject *__pyx_v_context, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("serialize", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_serialize); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 817, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_9drf_turbo_6fields_11ChoiceField_3serialize)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        __pyx_t_5 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
            __pyx_t_5 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_value, __pyx_v_context};
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 817, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_value, __pyx_v_context};
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 817, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        {
          __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 817, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          if (__pyx_t_4) {
            __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
          }
          __Pyx_INCREF(__pyx_v_value);
          __Pyx_GIVEREF(__pyx_v_value);
          PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_v_value);
          __Pyx_INCREF(__pyx_v_context);
          __Pyx_GIVEREF(__pyx_v_context);
          PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_v_context);
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_6, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 817, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        }
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "drf_turbo/fields.pyx":818
 * 
 *     cpdef serialize(self,value,dict context):
 *         if value in ('', None):             # <<<<<<<<<<<<<<
 *             return value
 * 
 */
  __Pyx_INCREF(__pyx_v_value);
  __pyx_t_1 = __pyx_v_value;
  __pyx_t_8 = (__Pyx_PyString_Equals(__pyx_t_1, __pyx_kp_s__9, Py_EQ)); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 818, __pyx_L1_error)
  if (!__pyx_t_8) {
  } else {
    __pyx_t_7 = __pyx_t_8;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = PyObject_RichCompare(__pyx_t_1, Py_None, Py_EQ); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 818, __pyx_L1_error)
  __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 818, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __pyx_t_8;
  __pyx_L4_bool_binop_done:;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = (__pyx_t_7 != 0);
  if (__pyx_t_8) {

    /* "drf_turbo/fields.pyx":819
 *     cpdef serialize(self,value,dict context):
 *         if value in ('', None):
 *             return value             # <<<<<<<<<<<<<<
 * 
 *         return {
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_value);
    __pyx_r = __pyx_v_value;
    goto __pyx_L0;

    /* "drf_turbo/fields.pyx":818
 * 
 *     cpdef serialize(self,value,dict context):
 *         if value in ('', None):             # <<<<<<<<<<<<<<
 *             return value
 * 
 */
  }

  /* "drf_turbo/fields.pyx":821
 *             return value
 * 
 *         return {             # <<<<<<<<<<<<<<
 *             'value': self.choice_strings_to_values.get(str(value), value),
 *             'display': self.choice_strings_to_display.get(str(value), value),
 */
  __Pyx_XDECREF(__pyx_r);

  /* "drf_turbo/fields.pyx":822
 * 
 *         return {
 *             'value': self.choice_strings_to_values.get(str(value), value),             # <<<<<<<<<<<<<<
 *             'display': self.choice_strings_to_display.get(str(value), value),
 *         }
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 822, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->choice_strings_to_values, __pyx_n_s_get); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 822, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_6 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), __pyx_v_value); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 822, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
      __pyx_t_5 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_3)) {
    PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_t_6, __pyx_v_value};
    __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 822, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
    PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_t_6, __pyx_v_value};
    __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 822, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  } else
  #endif
  {
    __pyx_t_9 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 822, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    if (__pyx_t_4) {
      __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_4); __pyx_t_4 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_9, 0+__pyx_t_5, __pyx_t_6);
    __Pyx_INCREF(__pyx_v_value);
    __Pyx_GIVEREF(__pyx_v_value);
    PyTuple_SET_ITEM(__pyx_t_9, 1+__pyx_t_5, __pyx_v_value);
    __pyx_t_6 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_9, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 822, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_value, __pyx_t_2) < 0) __PYX_ERR(0, 822, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "drf_turbo/fields.pyx":823
 *         return {
 *             'value': self.choice_strings_to_values.get(str(value), value),
 *             'display': self.choice_strings_to_display.get(str(value), value),             # <<<<<<<<<<<<<<
 *         }
 * 
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->choice_strings_to_display, __pyx_n_s_get); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 823, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_9 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), __pyx_v_value); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 823, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_6 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
      __pyx_t_5 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_3)) {
    PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_t_9, __pyx_v_value};
    __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 823, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
    PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_t_9, __pyx_v_value};
    __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 823, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  } else
  #endif
  {
    __pyx_t_4 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 823, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    if (__pyx_t_6) {
      __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_6); __pyx_t_6 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_9);
    PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_5, __pyx_t_9);
    __Pyx_INCREF(__pyx_v_value);
    __Pyx_GIVEREF(__pyx_v_value);
    PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_5, __pyx_v_value);
    __pyx_t_9 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_4, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 823, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_display, __pyx_t_2) < 0) __PYX_ERR(0, 822, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "drf_turbo/fields.pyx":817
 * 
 * 
 *     cpdef serialize(self,value,dict context):             # <<<<<<<<<<<<<<
 *         if value in ('', None):
 *             return value
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("drf_turbo.fields.ChoiceField.serialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_11ChoiceField_3serialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_11ChoiceField_3serialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_value = 0;
  PyObject *__pyx_v_context = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("serialize (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_value,&__pyx_n_s_context,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_value)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_context)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("serialize", 1, 2, 2, 1); __PYX_ERR(0, 817, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "serialize") < 0)) __PYX_ERR(0, 817, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_value = values[0];
    __pyx_v_context = ((PyObject*)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("serialize", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 817, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.ChoiceField.serialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_context), (&PyDict_Type), 1, "context", 1))) __PYX_ERR(0, 817, __pyx_L1_error)
  __pyx_r = __pyx_pf_9drf_turbo_6fields_11ChoiceField_2serialize(((struct __pyx_obj_9drf_turbo_6fields_ChoiceField *)__pyx_v_self), __pyx_v_value, __pyx_v_context);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_11ChoiceField_2serialize(struct __pyx_obj_9drf_turbo_6fields_ChoiceField *__pyx_v_self, PyObject *__pyx_v_value, PyObject *__pyx_v_context) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("serialize", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields_11ChoiceField_serialize(__pyx_v_self, __pyx_v_value, __pyx_v_context, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 817, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.ChoiceField.serialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":827
 * 
 * 
 *     cpdef deserialize(self,data,dict context) :             # <<<<<<<<<<<<<<
 *         if data == '' and self.allow_blank:
 *             return ''
 */

static PyObject *__pyx_pw_9drf_turbo_6fields_11ChoiceField_5deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_11ChoiceField_deserialize(struct __pyx_obj_9drf_turbo_6fields_ChoiceField *__pyx_v_self, PyObject *__pyx_v_data, CYTHON_UNUSED PyObject *__pyx_v_context, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("deserialize", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_deserialize); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 827, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_9drf_turbo_6fields_11ChoiceField_5deserialize)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        __pyx_t_5 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
            __pyx_t_5 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_data, __pyx_v_context};
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 827, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_data, __pyx_v_context};
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 827, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        {
          __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 827, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          if (__pyx_t_4) {
            __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
          }
          __Pyx_INCREF(__pyx_v_data);
          __Pyx_GIVEREF(__pyx_v_data);
          PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_v_data);
          __Pyx_INCREF(__pyx_v_context);
          __Pyx_GIVEREF(__pyx_v_context);
          PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_v_context);
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_6, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 827, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        }
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "drf_turbo/fields.pyx":828
 * 
 *     cpdef deserialize(self,data,dict context) :
 *         if data == '' and self.allow_blank:             # <<<<<<<<<<<<<<
 *             return ''
 *         try:
 */
  __pyx_t_8 = (__Pyx_PyString_Equals(__pyx_v_data, __pyx_kp_s__9, Py_EQ)); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 828, __pyx_L1_error)
  if (__pyx_t_8) {
  } else {
    __pyx_t_7 = __pyx_t_8;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_v_self->allow_blank); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 828, __pyx_L1_error)
  __pyx_t_7 = __pyx_t_8;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_7) {

    /* "drf_turbo/fields.pyx":829
 *     cpdef deserialize(self,data,dict context) :
 *         if data == '' and self.allow_blank:
 *             return ''             # <<<<<<<<<<<<<<
 *         try:
 *             return self.choice_strings_to_values[str(data)]
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_s__9);
    __pyx_r = __pyx_kp_s__9;
    goto __pyx_L0;

    /* "drf_turbo/fields.pyx":828
 * 
 *     cpdef deserialize(self,data,dict context) :
 *         if data == '' and self.allow_blank:             # <<<<<<<<<<<<<<
 *             return ''
 *         try:
 */
  }

  /* "drf_turbo/fields.pyx":830
 *         if data == '' and self.allow_blank:
 *             return ''
 *         try:             # <<<<<<<<<<<<<<
 *             return self.choice_strings_to_values[str(data)]
 *         except:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_9, &__pyx_t_10, &__pyx_t_11);
    __Pyx_XGOTREF(__pyx_t_9);
    __Pyx_XGOTREF(__pyx_t_10);
    __Pyx_XGOTREF(__pyx_t_11);
    /*try:*/ {

      /* "drf_turbo/fields.pyx":831
 *             return ''
 *         try:
 *             return self.choice_strings_to_values[str(data)]             # <<<<<<<<<<<<<<
 *         except:
 *             raise self.raise_if_fail('invalid_choice', input=data)
 */
      __Pyx_XDECREF(__pyx_r);
      __pyx_t_1 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), __pyx_v_data); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 831, __pyx_L6_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_2 = __Pyx_PyObject_GetItem(__pyx_v_self->choice_strings_to_values, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 831, __pyx_L6_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_r = __pyx_t_2;
      __pyx_t_2 = 0;
      goto __pyx_L10_try_return;

      /* "drf_turbo/fields.pyx":830
 *         if data == '' and self.allow_blank:
 *             return ''
 *         try:             # <<<<<<<<<<<<<<
 *             return self.choice_strings_to_values[str(data)]
 *         except:
 */
    }
    __pyx_L6_error:;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;

    /* "drf_turbo/fields.pyx":832
 *         try:
 *             return self.choice_strings_to_values[str(data)]
 *         except:             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail('invalid_choice', input=data)
 * 
 */
    /*except:*/ {
      __Pyx_AddTraceback("drf_turbo.fields.ChoiceField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_2, &__pyx_t_1, &__pyx_t_3) < 0) __PYX_ERR(0, 832, __pyx_L8_except_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_GOTREF(__pyx_t_3);

      /* "drf_turbo/fields.pyx":833
 *             return self.choice_strings_to_values[str(data)]
 *         except:
 *             raise self.raise_if_fail('invalid_choice', input=data)             # <<<<<<<<<<<<<<
 * 
 * 
 */
      __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_raise_if_fail); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 833, __pyx_L8_except_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_4 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 833, __pyx_L8_except_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_input, __pyx_v_data) < 0) __PYX_ERR(0, 833, __pyx_L8_except_error)
      __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_tuple__18, __pyx_t_4); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 833, __pyx_L8_except_error)
      __Pyx_GOTREF(__pyx_t_12);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_Raise(__pyx_t_12, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      __PYX_ERR(0, 833, __pyx_L8_except_error)
    }
    __pyx_L8_except_error:;

    /* "drf_turbo/fields.pyx":830
 *         if data == '' and self.allow_blank:
 *             return ''
 *         try:             # <<<<<<<<<<<<<<
 *             return self.choice_strings_to_values[str(data)]
 *         except:
 */
    __Pyx_XGIVEREF(__pyx_t_9);
    __Pyx_XGIVEREF(__pyx_t_10);
    __Pyx_XGIVEREF(__pyx_t_11);
    __Pyx_ExceptionReset(__pyx_t_9, __pyx_t_10, __pyx_t_11);
    goto __pyx_L1_error;
    __pyx_L10_try_return:;
    __Pyx_XGIVEREF(__pyx_t_9);
    __Pyx_XGIVEREF(__pyx_t_10);
    __Pyx_XGIVEREF(__pyx_t_11);
    __Pyx_ExceptionReset(__pyx_t_9, __pyx_t_10, __pyx_t_11);
    goto __pyx_L0;
  }

  /* "drf_turbo/fields.pyx":827
 * 
 * 
 *     cpdef deserialize(self,data,dict context) :             # <<<<<<<<<<<<<<
 *         if data == '' and self.allow_blank:
 *             return ''
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_AddTraceback("drf_turbo.fields.ChoiceField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_11ChoiceField_5deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_11ChoiceField_5deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_data = 0;
  PyObject *__pyx_v_context = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("deserialize (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_data,&__pyx_n_s_context,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_data)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_context)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("deserialize", 1, 2, 2, 1); __PYX_ERR(0, 827, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "deserialize") < 0)) __PYX_ERR(0, 827, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_data = values[0];
    __pyx_v_context = ((PyObject*)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("deserialize", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 827, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.ChoiceField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_context), (&PyDict_Type), 1, "context", 1))) __PYX_ERR(0, 827, __pyx_L1_error)
  __pyx_r = __pyx_pf_9drf_turbo_6fields_11ChoiceField_4deserialize(((struct __pyx_obj_9drf_turbo_6fields_ChoiceField *)__pyx_v_self), __pyx_v_data, __pyx_v_context);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_11ChoiceField_4deserialize(struct __pyx_obj_9drf_turbo_6fields_ChoiceField *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("deserialize", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields_11ChoiceField_deserialize(__pyx_v_self, __pyx_v_data, __pyx_v_context, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 827, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.ChoiceField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pxd":96
 * cdef class ChoiceField(Field):
 *     cdef public :
 *         choices             # <<<<<<<<<<<<<<
 *         choice_strings_to_values
 *         choice_strings_to_display
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_11ChoiceField_7choices_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_11ChoiceField_7choices_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_11ChoiceField_7choices___get__(((struct __pyx_obj_9drf_turbo_6fields_ChoiceField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_11ChoiceField_7choices___get__(struct __pyx_obj_9drf_turbo_6fields_ChoiceField *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->choices);
  __pyx_r = __pyx_v_self->choices;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_11ChoiceField_7choices_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_11ChoiceField_7choices_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_11ChoiceField_7choices_2__set__(((struct __pyx_obj_9drf_turbo_6fields_ChoiceField *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_11ChoiceField_7choices_2__set__(struct __pyx_obj_9drf_turbo_6fields_ChoiceField *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->choices);
  __Pyx_DECREF(__pyx_v_self->choices);
  __pyx_v_self->choices = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_11ChoiceField_7choices_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_11ChoiceField_7choices_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_11ChoiceField_7choices_4__del__(((struct __pyx_obj_9drf_turbo_6fields_ChoiceField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_11ChoiceField_7choices_4__del__(struct __pyx_obj_9drf_turbo_6fields_ChoiceField *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->choices);
  __Pyx_DECREF(__pyx_v_self->choices);
  __pyx_v_self->choices = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pxd":97
 *     cdef public :
 *         choices
 *         choice_strings_to_values             # <<<<<<<<<<<<<<
 *         choice_strings_to_display
 *         allow_blank
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_11ChoiceField_24choice_strings_to_values_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_11ChoiceField_24choice_strings_to_values_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_11ChoiceField_24choice_strings_to_values___get__(((struct __pyx_obj_9drf_turbo_6fields_ChoiceField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_11ChoiceField_24choice_strings_to_values___get__(struct __pyx_obj_9drf_turbo_6fields_ChoiceField *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->choice_strings_to_values);
  __pyx_r = __pyx_v_self->choice_strings_to_values;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_11ChoiceField_24choice_strings_to_values_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_11ChoiceField_24choice_strings_to_values_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_11ChoiceField_24choice_strings_to_values_2__set__(((struct __pyx_obj_9drf_turbo_6fields_ChoiceField *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_11ChoiceField_24choice_strings_to_values_2__set__(struct __pyx_obj_9drf_turbo_6fields_ChoiceField *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->choice_strings_to_values);
  __Pyx_DECREF(__pyx_v_self->choice_strings_to_values);
  __pyx_v_self->choice_strings_to_values = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_11ChoiceField_24choice_strings_to_values_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_11ChoiceField_24choice_strings_to_values_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_11ChoiceField_24choice_strings_to_values_4__del__(((struct __pyx_obj_9drf_turbo_6fields_ChoiceField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_11ChoiceField_24choice_strings_to_values_4__del__(struct __pyx_obj_9drf_turbo_6fields_ChoiceField *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->choice_strings_to_values);
  __Pyx_DECREF(__pyx_v_self->choice_strings_to_values);
  __pyx_v_self->choice_strings_to_values = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pxd":98
 *         choices
 *         choice_strings_to_values
 *         choice_strings_to_display             # <<<<<<<<<<<<<<
 *         allow_blank
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_11ChoiceField_25choice_strings_to_display_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_11ChoiceField_25choice_strings_to_display_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_11ChoiceField_25choice_strings_to_display___get__(((struct __pyx_obj_9drf_turbo_6fields_ChoiceField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_11ChoiceField_25choice_strings_to_display___get__(struct __pyx_obj_9drf_turbo_6fields_ChoiceField *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->choice_strings_to_display);
  __pyx_r = __pyx_v_self->choice_strings_to_display;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_11ChoiceField_25choice_strings_to_display_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_11ChoiceField_25choice_strings_to_display_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_11ChoiceField_25choice_strings_to_display_2__set__(((struct __pyx_obj_9drf_turbo_6fields_ChoiceField *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_11ChoiceField_25choice_strings_to_display_2__set__(struct __pyx_obj_9drf_turbo_6fields_ChoiceField *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->choice_strings_to_display);
  __Pyx_DECREF(__pyx_v_self->choice_strings_to_display);
  __pyx_v_self->choice_strings_to_display = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_11ChoiceField_25choice_strings_to_display_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_11ChoiceField_25choice_strings_to_display_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_11ChoiceField_25choice_strings_to_display_4__del__(((struct __pyx_obj_9drf_turbo_6fields_ChoiceField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_11ChoiceField_25choice_strings_to_display_4__del__(struct __pyx_obj_9drf_turbo_6fields_ChoiceField *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->choice_strings_to_display);
  __Pyx_DECREF(__pyx_v_self->choice_strings_to_display);
  __pyx_v_self->choice_strings_to_display = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pxd":99
 *         choice_strings_to_values
 *         choice_strings_to_display
 *         allow_blank             # <<<<<<<<<<<<<<
 * 
 * cdef class MultipleChoiceField(ChoiceField):
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_11ChoiceField_11allow_blank_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_11ChoiceField_11allow_blank_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_11ChoiceField_11allow_blank___get__(((struct __pyx_obj_9drf_turbo_6fields_ChoiceField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_11ChoiceField_11allow_blank___get__(struct __pyx_obj_9drf_turbo_6fields_ChoiceField *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->allow_blank);
  __pyx_r = __pyx_v_self->allow_blank;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_11ChoiceField_11allow_blank_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_11ChoiceField_11allow_blank_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_11ChoiceField_11allow_blank_2__set__(((struct __pyx_obj_9drf_turbo_6fields_ChoiceField *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_11ChoiceField_11allow_blank_2__set__(struct __pyx_obj_9drf_turbo_6fields_ChoiceField *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->allow_blank);
  __Pyx_DECREF(__pyx_v_self->allow_blank);
  __pyx_v_self->allow_blank = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_11ChoiceField_11allow_blank_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_11ChoiceField_11allow_blank_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_11ChoiceField_11allow_blank_4__del__(((struct __pyx_obj_9drf_turbo_6fields_ChoiceField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_11ChoiceField_11allow_blank_4__del__(struct __pyx_obj_9drf_turbo_6fields_ChoiceField *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->allow_blank);
  __Pyx_DECREF(__pyx_v_self->allow_blank);
  __pyx_v_self->allow_blank = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_11ChoiceField_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_11ChoiceField_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_11ChoiceField_6__reduce_cython__(((struct __pyx_obj_9drf_turbo_6fields_ChoiceField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_11ChoiceField_6__reduce_cython__(struct __pyx_obj_9drf_turbo_6fields_ChoiceField *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self.allow_blank, self.allow_null, self.attr, self.attrs, self.call, self.choice_strings_to_display, self.choice_strings_to_values, self.choices, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.allow_null); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.call); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.read_only); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.required); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.write_only); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = PyTuple_New(20); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_INCREF(__pyx_v_self->allow_blank);
  __Pyx_GIVEREF(__pyx_v_self->allow_blank);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v_self->allow_blank);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.attr);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.attr);
  PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_self->__pyx_base.attr);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.attrs);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.attrs);
  PyTuple_SET_ITEM(__pyx_t_6, 3, __pyx_v_self->__pyx_base.attrs);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_6, 4, __pyx_t_2);
  __Pyx_INCREF(__pyx_v_self->choice_strings_to_display);
  __Pyx_GIVEREF(__pyx_v_self->choice_strings_to_display);
  PyTuple_SET_ITEM(__pyx_t_6, 5, __pyx_v_self->choice_strings_to_display);
  __Pyx_INCREF(__pyx_v_self->choice_strings_to_values);
  __Pyx_GIVEREF(__pyx_v_self->choice_strings_to_values);
  PyTuple_SET_ITEM(__pyx_t_6, 6, __pyx_v_self->choice_strings_to_values);
  __Pyx_INCREF(__pyx_v_self->choices);
  __Pyx_GIVEREF(__pyx_v_self->choices);
  PyTuple_SET_ITEM(__pyx_t_6, 7, __pyx_v_self->choices);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.default_value);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.default_value);
  PyTuple_SET_ITEM(__pyx_t_6, 8, __pyx_v_self->__pyx_base.default_value);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.error_messages);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.error_messages);
  PyTuple_SET_ITEM(__pyx_t_6, 9, __pyx_v_self->__pyx_base.error_messages);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.field_name);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.field_name);
  PyTuple_SET_ITEM(__pyx_t_6, 10, __pyx_v_self->__pyx_base.field_name);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.help_text);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.help_text);
  PyTuple_SET_ITEM(__pyx_t_6, 11, __pyx_v_self->__pyx_base.help_text);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.initial);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.initial);
  PyTuple_SET_ITEM(__pyx_t_6, 12, __pyx_v_self->__pyx_base.initial);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.label);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.label);
  PyTuple_SET_ITEM(__pyx_t_6, 13, __pyx_v_self->__pyx_base.label);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 14, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_6, 15, __pyx_t_4);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.root);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.root);
  PyTuple_SET_ITEM(__pyx_t_6, 16, __pyx_v_self->__pyx_base.root);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.style);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.style);
  PyTuple_SET_ITEM(__pyx_t_6, 17, __pyx_v_self->__pyx_base.style);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.validators);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.validators);
  PyTuple_SET_ITEM(__pyx_t_6, 18, __pyx_v_self->__pyx_base.validators);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_6, 19, __pyx_t_5);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_6);
  __pyx_t_6 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self.allow_blank, self.allow_null, self.attr, self.attrs, self.call, self.choice_strings_to_display, self.choice_strings_to_values, self.choices, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_6 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_v__dict = __pyx_t_6;
  __pyx_t_6 = 0;

  /* "(tree fragment)":7
 *     state = (self.allow_blank, self.allow_null, self.attr, self.attrs, self.call, self.choice_strings_to_display, self.choice_strings_to_values, self.choices, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_7 = (__pyx_v__dict != Py_None);
  __pyx_t_8 = (__pyx_t_7 != 0);
  if (__pyx_t_8) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_6 = PyTuple_New(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v__dict);
    __pyx_t_5 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_6); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_5));
    __pyx_t_5 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.allow_blank is not None or self.attr is not None or self.attrs is not None or self.choice_strings_to_display is not None or self.choice_strings_to_values is not None or self.choices is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.root is not None or self.style is not None or self.validators is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self.allow_blank, self.allow_null, self.attr, self.attrs, self.call, self.choice_strings_to_display, self.choice_strings_to_values, self.choices, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self.allow_blank is not None or self.attr is not None or self.attrs is not None or self.choice_strings_to_display is not None or self.choice_strings_to_values is not None or self.choices is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.root is not None or self.style is not None or self.validators is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_ChoiceField, (type(self), 0xf172868, None), state
 */
  /*else*/ {
    __pyx_t_7 = (__pyx_v_self->allow_blank != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.attr != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.attrs != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->choice_strings_to_display != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->choice_strings_to_values != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->choices != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.default_value != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.error_messages != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.field_name != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.help_text != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.initial != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.label != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.root != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.style != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.validators != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    __pyx_t_8 = __pyx_t_9;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_8;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.allow_blank is not None or self.attr is not None or self.attrs is not None or self.choice_strings_to_display is not None or self.choice_strings_to_values is not None or self.choices is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.root is not None or self.style is not None or self.validators is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_ChoiceField, (type(self), 0xf172868, None), state
 *     else:
 */
  __pyx_t_8 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_8) {

    /* "(tree fragment)":13
 *         use_setstate = self.allow_blank is not None or self.attr is not None or self.attrs is not None or self.choice_strings_to_display is not None or self.choice_strings_to_values is not None or self.choices is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.root is not None or self.style is not None or self.validators is not None
 *     if use_setstate:
 *         return __pyx_unpickle_ChoiceField, (type(self), 0xf172868, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_ChoiceField, (type(self), 0xf172868, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_pyx_unpickle_ChoiceField); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_6, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_253175912);
    __Pyx_GIVEREF(__pyx_int_253175912);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_int_253175912);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_6, 2, Py_None);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_6);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_state);
    __pyx_t_5 = 0;
    __pyx_t_6 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.allow_blank is not None or self.attr is not None or self.attrs is not None or self.choice_strings_to_display is not None or self.choice_strings_to_values is not None or self.choices is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.root is not None or self.style is not None or self.validators is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_ChoiceField, (type(self), 0xf172868, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_ChoiceField, (type(self), 0xf172868, None), state
 *     else:
 *         return __pyx_unpickle_ChoiceField, (type(self), 0xf172868, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_ChoiceField__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_pyx_unpickle_ChoiceField); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_6, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_253175912);
    __Pyx_GIVEREF(__pyx_int_253175912);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_int_253175912);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_state);
    __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_6);
    __pyx_t_4 = 0;
    __pyx_t_6 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("drf_turbo.fields.ChoiceField.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_ChoiceField, (type(self), 0xf172868, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_ChoiceField__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_11ChoiceField_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_11ChoiceField_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_11ChoiceField_8__setstate_cython__(((struct __pyx_obj_9drf_turbo_6fields_ChoiceField *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_11ChoiceField_8__setstate_cython__(struct __pyx_obj_9drf_turbo_6fields_ChoiceField *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_ChoiceField, (type(self), 0xf172868, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_ChoiceField__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields___pyx_unpickle_ChoiceField__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_ChoiceField, (type(self), 0xf172868, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_ChoiceField__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.ChoiceField.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":850
 *     }
 * 
 *     def __init__(self, **kwargs):             # <<<<<<<<<<<<<<
 *         self.allow_empty = kwargs.pop('allow_empty', True)
 *         super().__init__(**kwargs)
 */

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_19MultipleChoiceField_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_19MultipleChoiceField_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_kwargs = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  if (unlikely(PyTuple_GET_SIZE(__pyx_args) > 0)) {
    __Pyx_RaiseArgtupleInvalid("__init__", 1, 0, 0, PyTuple_GET_SIZE(__pyx_args)); return -1;}
  if (__pyx_kwds && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "__init__", 1))) return -1;
  __pyx_v_kwargs = (__pyx_kwds) ? PyDict_Copy(__pyx_kwds) : PyDict_New(); if (unlikely(!__pyx_v_kwargs)) return -1;
  __Pyx_GOTREF(__pyx_v_kwargs);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_19MultipleChoiceField___init__(((struct __pyx_obj_9drf_turbo_6fields_MultipleChoiceField *)__pyx_v_self), __pyx_v_kwargs);

  /* function exit code */
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_19MultipleChoiceField___init__(struct __pyx_obj_9drf_turbo_6fields_MultipleChoiceField *__pyx_v_self, PyObject *__pyx_v_kwargs) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "drf_turbo/fields.pyx":851
 * 
 *     def __init__(self, **kwargs):
 *         self.allow_empty = kwargs.pop('allow_empty', True)             # <<<<<<<<<<<<<<
 *         super().__init__(**kwargs)
 * 
 */
  __pyx_t_1 = __Pyx_PyDict_Pop(__pyx_v_kwargs, __pyx_n_s_allow_empty, Py_True); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 851, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->allow_empty);
  __Pyx_DECREF(__pyx_v_self->allow_empty);
  __pyx_v_self->allow_empty = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "drf_turbo/fields.pyx":852
 *     def __init__(self, **kwargs):
 *         self.allow_empty = kwargs.pop('allow_empty', True)
 *         super().__init__(**kwargs)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 852, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_9drf_turbo_6fields_MultipleChoiceField));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_9drf_turbo_6fields_MultipleChoiceField));
  PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)__pyx_ptype_9drf_turbo_6fields_MultipleChoiceField));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_1, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 852, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_init); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 852, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyDict_Copy(__pyx_v_kwargs); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 852, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 852, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "drf_turbo/fields.pyx":850
 *     }
 * 
 *     def __init__(self, **kwargs):             # <<<<<<<<<<<<<<
 *         self.allow_empty = kwargs.pop('allow_empty', True)
 *         super().__init__(**kwargs)
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("drf_turbo.fields.MultipleChoiceField.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":855
 * 
 * 
 *     cpdef inline serialize(self,value,dict context):             # <<<<<<<<<<<<<<
 *         return {
 *             self.choice_strings_to_values.get(str(item), item) for item in value
 */

static PyObject *__pyx_pw_9drf_turbo_6fields_19MultipleChoiceField_3serialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_19MultipleChoiceField_serialize(struct __pyx_obj_9drf_turbo_6fields_MultipleChoiceField *__pyx_v_self, PyObject *__pyx_v_value, CYTHON_UNUSED PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_7genexpr__pyx_v_item = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  PyObject *(*__pyx_t_4)(PyObject *);
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("serialize", 0);

  /* "drf_turbo/fields.pyx":856
 * 
 *     cpdef inline serialize(self,value,dict context):
 *         return {             # <<<<<<<<<<<<<<
 *             self.choice_strings_to_values.get(str(item), item) for item in value
 *         }
 */
  __Pyx_XDECREF(__pyx_r);
  { /* enter inner scope */
    __pyx_t_1 = PySet_New(NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 856, __pyx_L5_error)
    __Pyx_GOTREF(__pyx_t_1);

    /* "drf_turbo/fields.pyx":857
 *     cpdef inline serialize(self,value,dict context):
 *         return {
 *             self.choice_strings_to_values.get(str(item), item) for item in value             # <<<<<<<<<<<<<<
 *         }
 * 
 */
    if (likely(PyList_CheckExact(__pyx_v_value)) || PyTuple_CheckExact(__pyx_v_value)) {
      __pyx_t_2 = __pyx_v_value; __Pyx_INCREF(__pyx_t_2); __pyx_t_3 = 0;
      __pyx_t_4 = NULL;
    } else {
      __pyx_t_3 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_v_value); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 857, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_4 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 857, __pyx_L5_error)
    }
    for (;;) {
      if (likely(!__pyx_t_4)) {
        if (likely(PyList_CheckExact(__pyx_t_2))) {
          if (__pyx_t_3 >= PyList_GET_SIZE(__pyx_t_2)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_5 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_3); __Pyx_INCREF(__pyx_t_5); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 857, __pyx_L5_error)
          #else
          __pyx_t_5 = PySequence_ITEM(__pyx_t_2, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 857, __pyx_L5_error)
          __Pyx_GOTREF(__pyx_t_5);
          #endif
        } else {
          if (__pyx_t_3 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_5 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_3); __Pyx_INCREF(__pyx_t_5); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 857, __pyx_L5_error)
          #else
          __pyx_t_5 = PySequence_ITEM(__pyx_t_2, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 857, __pyx_L5_error)
          __Pyx_GOTREF(__pyx_t_5);
          #endif
        }
      } else {
        __pyx_t_5 = __pyx_t_4(__pyx_t_2);
        if (unlikely(!__pyx_t_5)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 857, __pyx_L5_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_5);
      }
      __Pyx_XDECREF_SET(__pyx_7genexpr__pyx_v_item, __pyx_t_5);
      __pyx_t_5 = 0;
      __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->__pyx_base.choice_strings_to_values, __pyx_n_s_get); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 857, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_7 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), __pyx_7genexpr__pyx_v_item); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 857, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_8 = NULL;
      __pyx_t_9 = 0;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
        __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_6);
        if (likely(__pyx_t_8)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
          __Pyx_INCREF(__pyx_t_8);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_6, function);
          __pyx_t_9 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_6)) {
        PyObject *__pyx_temp[3] = {__pyx_t_8, __pyx_t_7, __pyx_7genexpr__pyx_v_item};
        __pyx_t_5 = __Pyx_PyFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_9, 2+__pyx_t_9); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 857, __pyx_L5_error)
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_6)) {
        PyObject *__pyx_temp[3] = {__pyx_t_8, __pyx_t_7, __pyx_7genexpr__pyx_v_item};
        __pyx_t_5 = __Pyx_PyCFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_9, 2+__pyx_t_9); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 857, __pyx_L5_error)
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      } else
      #endif
      {
        __pyx_t_10 = PyTuple_New(2+__pyx_t_9); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 857, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_10);
        if (__pyx_t_8) {
          __Pyx_GIVEREF(__pyx_t_8); PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_8); __pyx_t_8 = NULL;
        }
        __Pyx_GIVEREF(__pyx_t_7);
        PyTuple_SET_ITEM(__pyx_t_10, 0+__pyx_t_9, __pyx_t_7);
        __Pyx_INCREF(__pyx_7genexpr__pyx_v_item);
        __Pyx_GIVEREF(__pyx_7genexpr__pyx_v_item);
        PyTuple_SET_ITEM(__pyx_t_10, 1+__pyx_t_9, __pyx_7genexpr__pyx_v_item);
        __pyx_t_7 = 0;
        __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_10, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 857, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      }
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(PySet_Add(__pyx_t_1, (PyObject*)__pyx_t_5))) __PYX_ERR(0, 857, __pyx_L5_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_7genexpr__pyx_v_item); __pyx_7genexpr__pyx_v_item = 0;
    goto __pyx_L8_exit_scope;
    __pyx_L5_error:;
    __Pyx_XDECREF(__pyx_7genexpr__pyx_v_item); __pyx_7genexpr__pyx_v_item = 0;
    goto __pyx_L1_error;
    __pyx_L8_exit_scope:;
  } /* exit inner scope */
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "drf_turbo/fields.pyx":855
 * 
 * 
 *     cpdef inline serialize(self,value,dict context):             # <<<<<<<<<<<<<<
 *         return {
 *             self.choice_strings_to_values.get(str(item), item) for item in value
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("drf_turbo.fields.MultipleChoiceField.serialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_7genexpr__pyx_v_item);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_19MultipleChoiceField_3serialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_19MultipleChoiceField_3serialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_value = 0;
  PyObject *__pyx_v_context = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("serialize (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_value,&__pyx_n_s_context,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_value)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_context)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("serialize", 1, 2, 2, 1); __PYX_ERR(0, 855, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "serialize") < 0)) __PYX_ERR(0, 855, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_value = values[0];
    __pyx_v_context = ((PyObject*)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("serialize", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 855, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.MultipleChoiceField.serialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_context), (&PyDict_Type), 1, "context", 1))) __PYX_ERR(0, 855, __pyx_L1_error)
  __pyx_r = __pyx_pf_9drf_turbo_6fields_19MultipleChoiceField_2serialize(((struct __pyx_obj_9drf_turbo_6fields_MultipleChoiceField *)__pyx_v_self), __pyx_v_value, __pyx_v_context);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_19MultipleChoiceField_2serialize(struct __pyx_obj_9drf_turbo_6fields_MultipleChoiceField *__pyx_v_self, PyObject *__pyx_v_value, PyObject *__pyx_v_context) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("serialize", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields_19MultipleChoiceField_serialize(__pyx_v_self, __pyx_v_value, __pyx_v_context, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 855, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.MultipleChoiceField.serialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":860
 *         }
 * 
 *     cpdef inline deserialize(self,data,dict context) :             # <<<<<<<<<<<<<<
 *         if isinstance(data, str) or not hasattr(data, '__iter__'):
 *             raise self.raise_if_fail('not_a_list', input_type=type(data).__name__)
 */

static PyObject *__pyx_pw_9drf_turbo_6fields_19MultipleChoiceField_5deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_19MultipleChoiceField_deserialize(struct __pyx_obj_9drf_turbo_6fields_MultipleChoiceField *__pyx_v_self, PyObject *__pyx_v_data, CYTHON_UNUSED PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_v_new_data = NULL;
  PyObject *__pyx_v_item = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  Py_ssize_t __pyx_t_7;
  PyObject *(*__pyx_t_8)(PyObject *);
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  int __pyx_t_12;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("deserialize", 0);

  /* "drf_turbo/fields.pyx":861
 * 
 *     cpdef inline deserialize(self,data,dict context) :
 *         if isinstance(data, str) or not hasattr(data, '__iter__'):             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail('not_a_list', input_type=type(data).__name__)
 *         if not self.allow_empty and len(data) == 0:
 */
  __pyx_t_2 = PyString_Check(__pyx_v_data); 
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (!__pyx_t_3) {
  } else {
    __pyx_t_1 = __pyx_t_3;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_3 = __Pyx_HasAttr(__pyx_v_data, __pyx_n_s_iter); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(0, 861, __pyx_L1_error)
  __pyx_t_2 = ((!(__pyx_t_3 != 0)) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (unlikely(__pyx_t_1)) {

    /* "drf_turbo/fields.pyx":862
 *     cpdef inline deserialize(self,data,dict context) :
 *         if isinstance(data, str) or not hasattr(data, '__iter__'):
 *             raise self.raise_if_fail('not_a_list', input_type=type(data).__name__)             # <<<<<<<<<<<<<<
 *         if not self.allow_empty and len(data) == 0:
 *             raise self.raise_if_fail('empty')
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_raise_if_fail); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 862, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 862, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)Py_TYPE(__pyx_v_data)), __pyx_n_s_name); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 862, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    if (PyDict_SetItem(__pyx_t_5, __pyx_n_s_input_type, __pyx_t_6) < 0) __PYX_ERR(0, 862, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_tuple__19, __pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 862, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_Raise(__pyx_t_6, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __PYX_ERR(0, 862, __pyx_L1_error)

    /* "drf_turbo/fields.pyx":861
 * 
 *     cpdef inline deserialize(self,data,dict context) :
 *         if isinstance(data, str) or not hasattr(data, '__iter__'):             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail('not_a_list', input_type=type(data).__name__)
 *         if not self.allow_empty and len(data) == 0:
 */
  }

  /* "drf_turbo/fields.pyx":863
 *         if isinstance(data, str) or not hasattr(data, '__iter__'):
 *             raise self.raise_if_fail('not_a_list', input_type=type(data).__name__)
 *         if not self.allow_empty and len(data) == 0:             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail('empty')
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_self->allow_empty); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 863, __pyx_L1_error)
  __pyx_t_3 = ((!__pyx_t_2) != 0);
  if (__pyx_t_3) {
  } else {
    __pyx_t_1 = __pyx_t_3;
    goto __pyx_L7_bool_binop_done;
  }
  __pyx_t_7 = PyObject_Length(__pyx_v_data); if (unlikely(__pyx_t_7 == ((Py_ssize_t)-1))) __PYX_ERR(0, 863, __pyx_L1_error)
  __pyx_t_3 = ((__pyx_t_7 == 0) != 0);
  __pyx_t_1 = __pyx_t_3;
  __pyx_L7_bool_binop_done:;
  if (unlikely(__pyx_t_1)) {

    /* "drf_turbo/fields.pyx":864
 *             raise self.raise_if_fail('not_a_list', input_type=type(data).__name__)
 *         if not self.allow_empty and len(data) == 0:
 *             raise self.raise_if_fail('empty')             # <<<<<<<<<<<<<<
 * 
 *         new_data = set()
 */
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_raise_if_fail); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 864, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
      }
    }
    __pyx_t_6 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_4, __pyx_n_s_empty) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_n_s_empty);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 864, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_Raise(__pyx_t_6, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __PYX_ERR(0, 864, __pyx_L1_error)

    /* "drf_turbo/fields.pyx":863
 *         if isinstance(data, str) or not hasattr(data, '__iter__'):
 *             raise self.raise_if_fail('not_a_list', input_type=type(data).__name__)
 *         if not self.allow_empty and len(data) == 0:             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail('empty')
 * 
 */
  }

  /* "drf_turbo/fields.pyx":866
 *             raise self.raise_if_fail('empty')
 * 
 *         new_data = set()             # <<<<<<<<<<<<<<
 *         for item in data :
 *             if item == '' and self.allow_blank:
 */
  __pyx_t_6 = PySet_New(0); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 866, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_v_new_data = ((PyObject*)__pyx_t_6);
  __pyx_t_6 = 0;

  /* "drf_turbo/fields.pyx":867
 * 
 *         new_data = set()
 *         for item in data :             # <<<<<<<<<<<<<<
 *             if item == '' and self.allow_blank:
 *                 return ''
 */
  if (likely(PyList_CheckExact(__pyx_v_data)) || PyTuple_CheckExact(__pyx_v_data)) {
    __pyx_t_6 = __pyx_v_data; __Pyx_INCREF(__pyx_t_6); __pyx_t_7 = 0;
    __pyx_t_8 = NULL;
  } else {
    __pyx_t_7 = -1; __pyx_t_6 = PyObject_GetIter(__pyx_v_data); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 867, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_8 = Py_TYPE(__pyx_t_6)->tp_iternext; if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 867, __pyx_L1_error)
  }
  for (;;) {
    if (likely(!__pyx_t_8)) {
      if (likely(PyList_CheckExact(__pyx_t_6))) {
        if (__pyx_t_7 >= PyList_GET_SIZE(__pyx_t_6)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_5 = PyList_GET_ITEM(__pyx_t_6, __pyx_t_7); __Pyx_INCREF(__pyx_t_5); __pyx_t_7++; if (unlikely(0 < 0)) __PYX_ERR(0, 867, __pyx_L1_error)
        #else
        __pyx_t_5 = PySequence_ITEM(__pyx_t_6, __pyx_t_7); __pyx_t_7++; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 867, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        #endif
      } else {
        if (__pyx_t_7 >= PyTuple_GET_SIZE(__pyx_t_6)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_5 = PyTuple_GET_ITEM(__pyx_t_6, __pyx_t_7); __Pyx_INCREF(__pyx_t_5); __pyx_t_7++; if (unlikely(0 < 0)) __PYX_ERR(0, 867, __pyx_L1_error)
        #else
        __pyx_t_5 = PySequence_ITEM(__pyx_t_6, __pyx_t_7); __pyx_t_7++; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 867, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        #endif
      }
    } else {
      __pyx_t_5 = __pyx_t_8(__pyx_t_6);
      if (unlikely(!__pyx_t_5)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 867, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_5);
    }
    __Pyx_XDECREF_SET(__pyx_v_item, __pyx_t_5);
    __pyx_t_5 = 0;

    /* "drf_turbo/fields.pyx":868
 *         new_data = set()
 *         for item in data :
 *             if item == '' and self.allow_blank:             # <<<<<<<<<<<<<<
 *                 return ''
 *             try:
 */
    __pyx_t_3 = (__Pyx_PyString_Equals(__pyx_v_item, __pyx_kp_s__9, Py_EQ)); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 868, __pyx_L1_error)
    if (__pyx_t_3) {
    } else {
      __pyx_t_1 = __pyx_t_3;
      goto __pyx_L12_bool_binop_done;
    }
    __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_v_self->__pyx_base.allow_blank); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 868, __pyx_L1_error)
    __pyx_t_1 = __pyx_t_3;
    __pyx_L12_bool_binop_done:;
    if (__pyx_t_1) {

      /* "drf_turbo/fields.pyx":869
 *         for item in data :
 *             if item == '' and self.allow_blank:
 *                 return ''             # <<<<<<<<<<<<<<
 *             try:
 *                 new_data.add(self.choice_strings_to_values[str(item)])
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_s__9);
      __pyx_r = __pyx_kp_s__9;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      goto __pyx_L0;

      /* "drf_turbo/fields.pyx":868
 *         new_data = set()
 *         for item in data :
 *             if item == '' and self.allow_blank:             # <<<<<<<<<<<<<<
 *                 return ''
 *             try:
 */
    }

    /* "drf_turbo/fields.pyx":870
 *             if item == '' and self.allow_blank:
 *                 return ''
 *             try:             # <<<<<<<<<<<<<<
 *                 new_data.add(self.choice_strings_to_values[str(item)])
 *             except:
 */
    {
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __Pyx_ExceptionSave(&__pyx_t_9, &__pyx_t_10, &__pyx_t_11);
      __Pyx_XGOTREF(__pyx_t_9);
      __Pyx_XGOTREF(__pyx_t_10);
      __Pyx_XGOTREF(__pyx_t_11);
      /*try:*/ {

        /* "drf_turbo/fields.pyx":871
 *                 return ''
 *             try:
 *                 new_data.add(self.choice_strings_to_values[str(item)])             # <<<<<<<<<<<<<<
 *             except:
 *                 raise self.raise_if_fail('invalid_choice', input=item)
 */
        __pyx_t_5 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), __pyx_v_item); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 871, __pyx_L14_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_4 = __Pyx_PyObject_GetItem(__pyx_v_self->__pyx_base.choice_strings_to_values, __pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 871, __pyx_L14_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __pyx_t_12 = PySet_Add(__pyx_v_new_data, __pyx_t_4); if (unlikely(__pyx_t_12 == ((int)-1))) __PYX_ERR(0, 871, __pyx_L14_error)
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

        /* "drf_turbo/fields.pyx":870
 *             if item == '' and self.allow_blank:
 *                 return ''
 *             try:             # <<<<<<<<<<<<<<
 *                 new_data.add(self.choice_strings_to_values[str(item)])
 *             except:
 */
      }
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
      goto __pyx_L21_try_end;
      __pyx_L14_error:;
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;

      /* "drf_turbo/fields.pyx":872
 *             try:
 *                 new_data.add(self.choice_strings_to_values[str(item)])
 *             except:             # <<<<<<<<<<<<<<
 *                 raise self.raise_if_fail('invalid_choice', input=item)
 *         return new_data
 */
      /*except:*/ {
        __Pyx_AddTraceback("drf_turbo.fields.MultipleChoiceField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
        if (__Pyx_GetException(&__pyx_t_4, &__pyx_t_5, &__pyx_t_13) < 0) __PYX_ERR(0, 872, __pyx_L16_except_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_GOTREF(__pyx_t_13);

        /* "drf_turbo/fields.pyx":873
 *                 new_data.add(self.choice_strings_to_values[str(item)])
 *             except:
 *                 raise self.raise_if_fail('invalid_choice', input=item)             # <<<<<<<<<<<<<<
 *         return new_data
 * 
 */
        __pyx_t_14 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_raise_if_fail); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 873, __pyx_L16_except_error)
        __Pyx_GOTREF(__pyx_t_14);
        __pyx_t_15 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 873, __pyx_L16_except_error)
        __Pyx_GOTREF(__pyx_t_15);
        if (PyDict_SetItem(__pyx_t_15, __pyx_n_s_input, __pyx_v_item) < 0) __PYX_ERR(0, 873, __pyx_L16_except_error)
        __pyx_t_16 = __Pyx_PyObject_Call(__pyx_t_14, __pyx_tuple__18, __pyx_t_15); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 873, __pyx_L16_except_error)
        __Pyx_GOTREF(__pyx_t_16);
        __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
        __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        __Pyx_Raise(__pyx_t_16, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
        __PYX_ERR(0, 873, __pyx_L16_except_error)
      }
      __pyx_L16_except_error:;

      /* "drf_turbo/fields.pyx":870
 *             if item == '' and self.allow_blank:
 *                 return ''
 *             try:             # <<<<<<<<<<<<<<
 *                 new_data.add(self.choice_strings_to_values[str(item)])
 *             except:
 */
      __Pyx_XGIVEREF(__pyx_t_9);
      __Pyx_XGIVEREF(__pyx_t_10);
      __Pyx_XGIVEREF(__pyx_t_11);
      __Pyx_ExceptionReset(__pyx_t_9, __pyx_t_10, __pyx_t_11);
      goto __pyx_L1_error;
      __pyx_L21_try_end:;
    }

    /* "drf_turbo/fields.pyx":867
 * 
 *         new_data = set()
 *         for item in data :             # <<<<<<<<<<<<<<
 *             if item == '' and self.allow_blank:
 *                 return ''
 */
  }
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

  /* "drf_turbo/fields.pyx":874
 *             except:
 *                 raise self.raise_if_fail('invalid_choice', input=item)
 *         return new_data             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_new_data);
  __pyx_r = __pyx_v_new_data;
  goto __pyx_L0;

  /* "drf_turbo/fields.pyx":860
 *         }
 * 
 *     cpdef inline deserialize(self,data,dict context) :             # <<<<<<<<<<<<<<
 *         if isinstance(data, str) or not hasattr(data, '__iter__'):
 *             raise self.raise_if_fail('not_a_list', input_type=type(data).__name__)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_XDECREF(__pyx_t_15);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_AddTraceback("drf_turbo.fields.MultipleChoiceField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_new_data);
  __Pyx_XDECREF(__pyx_v_item);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_19MultipleChoiceField_5deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_19MultipleChoiceField_5deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_data = 0;
  PyObject *__pyx_v_context = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("deserialize (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_data,&__pyx_n_s_context,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_data)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_context)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("deserialize", 1, 2, 2, 1); __PYX_ERR(0, 860, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "deserialize") < 0)) __PYX_ERR(0, 860, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_data = values[0];
    __pyx_v_context = ((PyObject*)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("deserialize", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 860, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.MultipleChoiceField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_context), (&PyDict_Type), 1, "context", 1))) __PYX_ERR(0, 860, __pyx_L1_error)
  __pyx_r = __pyx_pf_9drf_turbo_6fields_19MultipleChoiceField_4deserialize(((struct __pyx_obj_9drf_turbo_6fields_MultipleChoiceField *)__pyx_v_self), __pyx_v_data, __pyx_v_context);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_19MultipleChoiceField_4deserialize(struct __pyx_obj_9drf_turbo_6fields_MultipleChoiceField *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("deserialize", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields_19MultipleChoiceField_deserialize(__pyx_v_self, __pyx_v_data, __pyx_v_context, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 860, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.MultipleChoiceField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pxd":103
 * cdef class MultipleChoiceField(ChoiceField):
 *         cdef public:
 *             allow_empty             # <<<<<<<<<<<<<<
 * 
 * cdef class DateTimeField(Field):
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_19MultipleChoiceField_11allow_empty_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_19MultipleChoiceField_11allow_empty_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_19MultipleChoiceField_11allow_empty___get__(((struct __pyx_obj_9drf_turbo_6fields_MultipleChoiceField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_19MultipleChoiceField_11allow_empty___get__(struct __pyx_obj_9drf_turbo_6fields_MultipleChoiceField *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->allow_empty);
  __pyx_r = __pyx_v_self->allow_empty;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_19MultipleChoiceField_11allow_empty_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_19MultipleChoiceField_11allow_empty_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_19MultipleChoiceField_11allow_empty_2__set__(((struct __pyx_obj_9drf_turbo_6fields_MultipleChoiceField *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_19MultipleChoiceField_11allow_empty_2__set__(struct __pyx_obj_9drf_turbo_6fields_MultipleChoiceField *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->allow_empty);
  __Pyx_DECREF(__pyx_v_self->allow_empty);
  __pyx_v_self->allow_empty = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_19MultipleChoiceField_11allow_empty_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_19MultipleChoiceField_11allow_empty_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_19MultipleChoiceField_11allow_empty_4__del__(((struct __pyx_obj_9drf_turbo_6fields_MultipleChoiceField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_19MultipleChoiceField_11allow_empty_4__del__(struct __pyx_obj_9drf_turbo_6fields_MultipleChoiceField *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->allow_empty);
  __Pyx_DECREF(__pyx_v_self->allow_empty);
  __pyx_v_self->allow_empty = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_19MultipleChoiceField_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_19MultipleChoiceField_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_19MultipleChoiceField_6__reduce_cython__(((struct __pyx_obj_9drf_turbo_6fields_MultipleChoiceField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_19MultipleChoiceField_6__reduce_cython__(struct __pyx_obj_9drf_turbo_6fields_MultipleChoiceField *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self.allow_blank, self.allow_empty, self.allow_null, self.attr, self.attrs, self.call, self.choice_strings_to_display, self.choice_strings_to_values, self.choices, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base.allow_null); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base.call); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base.read_only); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base.required); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base.write_only); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = PyTuple_New(21); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.allow_blank);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.allow_blank);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v_self->__pyx_base.allow_blank);
  __Pyx_INCREF(__pyx_v_self->allow_empty);
  __Pyx_GIVEREF(__pyx_v_self->allow_empty);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_v_self->allow_empty);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.attr);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.attr);
  PyTuple_SET_ITEM(__pyx_t_6, 3, __pyx_v_self->__pyx_base.__pyx_base.attr);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.attrs);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.attrs);
  PyTuple_SET_ITEM(__pyx_t_6, 4, __pyx_v_self->__pyx_base.__pyx_base.attrs);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_6, 5, __pyx_t_2);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.choice_strings_to_display);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.choice_strings_to_display);
  PyTuple_SET_ITEM(__pyx_t_6, 6, __pyx_v_self->__pyx_base.choice_strings_to_display);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.choice_strings_to_values);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.choice_strings_to_values);
  PyTuple_SET_ITEM(__pyx_t_6, 7, __pyx_v_self->__pyx_base.choice_strings_to_values);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.choices);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.choices);
  PyTuple_SET_ITEM(__pyx_t_6, 8, __pyx_v_self->__pyx_base.choices);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.default_value);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.default_value);
  PyTuple_SET_ITEM(__pyx_t_6, 9, __pyx_v_self->__pyx_base.__pyx_base.default_value);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.error_messages);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.error_messages);
  PyTuple_SET_ITEM(__pyx_t_6, 10, __pyx_v_self->__pyx_base.__pyx_base.error_messages);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.field_name);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.field_name);
  PyTuple_SET_ITEM(__pyx_t_6, 11, __pyx_v_self->__pyx_base.__pyx_base.field_name);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.help_text);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.help_text);
  PyTuple_SET_ITEM(__pyx_t_6, 12, __pyx_v_self->__pyx_base.__pyx_base.help_text);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.initial);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.initial);
  PyTuple_SET_ITEM(__pyx_t_6, 13, __pyx_v_self->__pyx_base.__pyx_base.initial);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.label);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.label);
  PyTuple_SET_ITEM(__pyx_t_6, 14, __pyx_v_self->__pyx_base.__pyx_base.label);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 15, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_6, 16, __pyx_t_4);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.root);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.root);
  PyTuple_SET_ITEM(__pyx_t_6, 17, __pyx_v_self->__pyx_base.__pyx_base.root);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.style);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.style);
  PyTuple_SET_ITEM(__pyx_t_6, 18, __pyx_v_self->__pyx_base.__pyx_base.style);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.validators);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.validators);
  PyTuple_SET_ITEM(__pyx_t_6, 19, __pyx_v_self->__pyx_base.__pyx_base.validators);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_6, 20, __pyx_t_5);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_6);
  __pyx_t_6 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self.allow_blank, self.allow_empty, self.allow_null, self.attr, self.attrs, self.call, self.choice_strings_to_display, self.choice_strings_to_values, self.choices, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_6 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_v__dict = __pyx_t_6;
  __pyx_t_6 = 0;

  /* "(tree fragment)":7
 *     state = (self.allow_blank, self.allow_empty, self.allow_null, self.attr, self.attrs, self.call, self.choice_strings_to_display, self.choice_strings_to_values, self.choices, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_7 = (__pyx_v__dict != Py_None);
  __pyx_t_8 = (__pyx_t_7 != 0);
  if (__pyx_t_8) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_6 = PyTuple_New(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v__dict);
    __pyx_t_5 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_6); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_5));
    __pyx_t_5 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.allow_blank is not None or self.allow_empty is not None or self.attr is not None or self.attrs is not None or self.choice_strings_to_display is not None or self.choice_strings_to_values is not None or self.choices is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.root is not None or self.style is not None or self.validators is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self.allow_blank, self.allow_empty, self.allow_null, self.attr, self.attrs, self.call, self.choice_strings_to_display, self.choice_strings_to_values, self.choices, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self.allow_blank is not None or self.allow_empty is not None or self.attr is not None or self.attrs is not None or self.choice_strings_to_display is not None or self.choice_strings_to_values is not None or self.choices is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.root is not None or self.style is not None or self.validators is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_MultipleChoiceField, (type(self), 0x69fcb1b, None), state
 */
  /*else*/ {
    __pyx_t_7 = (__pyx_v_self->__pyx_base.allow_blank != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->allow_empty != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.__pyx_base.attr != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.__pyx_base.attrs != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.choice_strings_to_display != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.choice_strings_to_values != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.choices != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.__pyx_base.default_value != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.__pyx_base.error_messages != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.__pyx_base.field_name != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.__pyx_base.help_text != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.__pyx_base.initial != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.__pyx_base.label != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.__pyx_base.root != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.__pyx_base.style != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.__pyx_base.validators != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    __pyx_t_8 = __pyx_t_7;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_8;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.allow_blank is not None or self.allow_empty is not None or self.attr is not None or self.attrs is not None or self.choice_strings_to_display is not None or self.choice_strings_to_values is not None or self.choices is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.root is not None or self.style is not None or self.validators is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MultipleChoiceField, (type(self), 0x69fcb1b, None), state
 *     else:
 */
  __pyx_t_8 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_8) {

    /* "(tree fragment)":13
 *         use_setstate = self.allow_blank is not None or self.allow_empty is not None or self.attr is not None or self.attrs is not None or self.choice_strings_to_display is not None or self.choice_strings_to_values is not None or self.choices is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.root is not None or self.style is not None or self.validators is not None
 *     if use_setstate:
 *         return __pyx_unpickle_MultipleChoiceField, (type(self), 0x69fcb1b, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_MultipleChoiceField, (type(self), 0x69fcb1b, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_pyx_unpickle_MultipleChoiceFie); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_6, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_111135515);
    __Pyx_GIVEREF(__pyx_int_111135515);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_int_111135515);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_6, 2, Py_None);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_6);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_state);
    __pyx_t_5 = 0;
    __pyx_t_6 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.allow_blank is not None or self.allow_empty is not None or self.attr is not None or self.attrs is not None or self.choice_strings_to_display is not None or self.choice_strings_to_values is not None or self.choices is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.root is not None or self.style is not None or self.validators is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MultipleChoiceField, (type(self), 0x69fcb1b, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_MultipleChoiceField, (type(self), 0x69fcb1b, None), state
 *     else:
 *         return __pyx_unpickle_MultipleChoiceField, (type(self), 0x69fcb1b, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MultipleChoiceField__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_pyx_unpickle_MultipleChoiceFie); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_6, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_111135515);
    __Pyx_GIVEREF(__pyx_int_111135515);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_int_111135515);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_state);
    __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_6);
    __pyx_t_4 = 0;
    __pyx_t_6 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("drf_turbo.fields.MultipleChoiceField.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MultipleChoiceField, (type(self), 0x69fcb1b, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MultipleChoiceField__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_19MultipleChoiceField_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_19MultipleChoiceField_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_19MultipleChoiceField_8__setstate_cython__(((struct __pyx_obj_9drf_turbo_6fields_MultipleChoiceField *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_19MultipleChoiceField_8__setstate_cython__(struct __pyx_obj_9drf_turbo_6fields_MultipleChoiceField *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_MultipleChoiceField, (type(self), 0x69fcb1b, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MultipleChoiceField__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields___pyx_unpickle_MultipleChoiceField__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MultipleChoiceField, (type(self), 0x69fcb1b, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MultipleChoiceField__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.MultipleChoiceField.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":897
 *     datetime_parser = datetime.datetime.strptime
 * 
 *     def __init__(self, format=NO_DEFAULT,input_formats=None,default_timezone=None,**kwargs):             # <<<<<<<<<<<<<<
 *         if format is NO_DEFAULT:
 *             self.format = api_settings.DATETIME_FORMAT
 */

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_13DateTimeField_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_13DateTimeField_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_format = 0;
  PyObject *__pyx_v_input_formats = 0;
  PyObject *__pyx_v_default_timezone = 0;
  PyObject *__pyx_v_kwargs = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  __pyx_v_kwargs = PyDict_New(); if (unlikely(!__pyx_v_kwargs)) return -1;
  __Pyx_GOTREF(__pyx_v_kwargs);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_format,&__pyx_n_s_input_formats,&__pyx_n_s_default_timezone,0};
    PyObject* values[3] = {0,0,0};
    values[0] = __pyx_k__20;
    values[1] = ((PyObject *)Py_None);
    values[2] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_format);
          if (value) { values[0] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_input_formats);
          if (value) { values[1] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_default_timezone);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, __pyx_v_kwargs, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 897, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_format = values[0];
    __pyx_v_input_formats = values[1];
    __pyx_v_default_timezone = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 0, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 897, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_DECREF(__pyx_v_kwargs); __pyx_v_kwargs = 0;
  __Pyx_AddTraceback("drf_turbo.fields.DateTimeField.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9drf_turbo_6fields_13DateTimeField___init__(((struct __pyx_obj_9drf_turbo_6fields_DateTimeField *)__pyx_v_self), __pyx_v_format, __pyx_v_input_formats, __pyx_v_default_timezone, __pyx_v_kwargs);

  /* function exit code */
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_13DateTimeField___init__(struct __pyx_obj_9drf_turbo_6fields_DateTimeField *__pyx_v_self, PyObject *__pyx_v_format, PyObject *__pyx_v_input_formats, PyObject *__pyx_v_default_timezone, PyObject *__pyx_v_kwargs) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "drf_turbo/fields.pyx":898
 * 
 *     def __init__(self, format=NO_DEFAULT,input_formats=None,default_timezone=None,**kwargs):
 *         if format is NO_DEFAULT:             # <<<<<<<<<<<<<<
 *             self.format = api_settings.DATETIME_FORMAT
 *         else:
 */
  __pyx_t_1 = (__pyx_v_format == __pyx_v_9drf_turbo_6fields_NO_DEFAULT);
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "drf_turbo/fields.pyx":899
 *     def __init__(self, format=NO_DEFAULT,input_formats=None,default_timezone=None,**kwargs):
 *         if format is NO_DEFAULT:
 *             self.format = api_settings.DATETIME_FORMAT             # <<<<<<<<<<<<<<
 *         else:
 *             self.format = format
 */
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_api_settings); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 899, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_DATETIME_FORMAT); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 899, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GIVEREF(__pyx_t_4);
    __Pyx_GOTREF(__pyx_v_self->format);
    __Pyx_DECREF(__pyx_v_self->format);
    __pyx_v_self->format = __pyx_t_4;
    __pyx_t_4 = 0;

    /* "drf_turbo/fields.pyx":898
 * 
 *     def __init__(self, format=NO_DEFAULT,input_formats=None,default_timezone=None,**kwargs):
 *         if format is NO_DEFAULT:             # <<<<<<<<<<<<<<
 *             self.format = api_settings.DATETIME_FORMAT
 *         else:
 */
    goto __pyx_L3;
  }

  /* "drf_turbo/fields.pyx":901
 *             self.format = api_settings.DATETIME_FORMAT
 *         else:
 *             self.format = format             # <<<<<<<<<<<<<<
 *         if input_formats is None:
 *             self.input_formats = api_settings.DATETIME_INPUT_FORMATS
 */
  /*else*/ {
    __Pyx_INCREF(__pyx_v_format);
    __Pyx_GIVEREF(__pyx_v_format);
    __Pyx_GOTREF(__pyx_v_self->format);
    __Pyx_DECREF(__pyx_v_self->format);
    __pyx_v_self->format = __pyx_v_format;
  }
  __pyx_L3:;

  /* "drf_turbo/fields.pyx":902
 *         else:
 *             self.format = format
 *         if input_formats is None:             # <<<<<<<<<<<<<<
 *             self.input_formats = api_settings.DATETIME_INPUT_FORMATS
 *         else:
 */
  __pyx_t_2 = (__pyx_v_input_formats == Py_None);
  __pyx_t_1 = (__pyx_t_2 != 0);
  if (__pyx_t_1) {

    /* "drf_turbo/fields.pyx":903
 *             self.format = format
 *         if input_formats is None:
 *             self.input_formats = api_settings.DATETIME_INPUT_FORMATS             # <<<<<<<<<<<<<<
 *         else:
 *             self.input_formats = input_formats
 */
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_api_settings); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 903, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_DATETIME_INPUT_FORMATS); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 903, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GIVEREF(__pyx_t_3);
    __Pyx_GOTREF(__pyx_v_self->input_formats);
    __Pyx_DECREF(__pyx_v_self->input_formats);
    __pyx_v_self->input_formats = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "drf_turbo/fields.pyx":902
 *         else:
 *             self.format = format
 *         if input_formats is None:             # <<<<<<<<<<<<<<
 *             self.input_formats = api_settings.DATETIME_INPUT_FORMATS
 *         else:
 */
    goto __pyx_L4;
  }

  /* "drf_turbo/fields.pyx":905
 *             self.input_formats = api_settings.DATETIME_INPUT_FORMATS
 *         else:
 *             self.input_formats = input_formats             # <<<<<<<<<<<<<<
 * 
 *         if default_timezone is None:
 */
  /*else*/ {
    __Pyx_INCREF(__pyx_v_input_formats);
    __Pyx_GIVEREF(__pyx_v_input_formats);
    __Pyx_GOTREF(__pyx_v_self->input_formats);
    __Pyx_DECREF(__pyx_v_self->input_formats);
    __pyx_v_self->input_formats = __pyx_v_input_formats;
  }
  __pyx_L4:;

  /* "drf_turbo/fields.pyx":907
 *             self.input_formats = input_formats
 * 
 *         if default_timezone is None:             # <<<<<<<<<<<<<<
 *             self.timezone = self.get_default_timezone()
 *         else:
 */
  __pyx_t_1 = (__pyx_v_default_timezone == Py_None);
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "drf_turbo/fields.pyx":908
 * 
 *         if default_timezone is None:
 *             self.timezone = self.get_default_timezone()             # <<<<<<<<<<<<<<
 *         else:
 *             self.timezone = default_timezone
 */
    __pyx_t_3 = __pyx_f_9drf_turbo_6fields_13DateTimeField_get_default_timezone(__pyx_v_self, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 908, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_3);
    __Pyx_GOTREF(__pyx_v_self->timezone);
    __Pyx_DECREF(__pyx_v_self->timezone);
    __pyx_v_self->timezone = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "drf_turbo/fields.pyx":907
 *             self.input_formats = input_formats
 * 
 *         if default_timezone is None:             # <<<<<<<<<<<<<<
 *             self.timezone = self.get_default_timezone()
 *         else:
 */
    goto __pyx_L5;
  }

  /* "drf_turbo/fields.pyx":910
 *             self.timezone = self.get_default_timezone()
 *         else:
 *             self.timezone = default_timezone             # <<<<<<<<<<<<<<
 * 
 *         super().__init__(**kwargs)
 */
  /*else*/ {
    __Pyx_INCREF(__pyx_v_default_timezone);
    __Pyx_GIVEREF(__pyx_v_default_timezone);
    __Pyx_GOTREF(__pyx_v_self->timezone);
    __Pyx_DECREF(__pyx_v_self->timezone);
    __pyx_v_self->timezone = __pyx_v_default_timezone;
  }
  __pyx_L5:;

  /* "drf_turbo/fields.pyx":912
 *             self.timezone = default_timezone
 * 
 *         super().__init__(**kwargs)             # <<<<<<<<<<<<<<
 * 
 *     cpdef inline get_default_timezone(self):
 */
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 912, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_9drf_turbo_6fields_DateTimeField));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_9drf_turbo_6fields_DateTimeField));
  PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)__pyx_ptype_9drf_turbo_6fields_DateTimeField));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_3, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_3, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 912, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_init); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 912, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyDict_Copy(__pyx_v_kwargs); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 912, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_empty_tuple, __pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 912, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "drf_turbo/fields.pyx":897
 *     datetime_parser = datetime.datetime.strptime
 * 
 *     def __init__(self, format=NO_DEFAULT,input_formats=None,default_timezone=None,**kwargs):             # <<<<<<<<<<<<<<
 *         if format is NO_DEFAULT:
 *             self.format = api_settings.DATETIME_FORMAT
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("drf_turbo.fields.DateTimeField.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":914
 *         super().__init__(**kwargs)
 * 
 *     cpdef inline get_default_timezone(self):             # <<<<<<<<<<<<<<
 *         return timezone.get_current_timezone() if settings.USE_TZ else None
 * 
 */

static PyObject *__pyx_pw_9drf_turbo_6fields_13DateTimeField_3get_default_timezone(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_13DateTimeField_get_default_timezone(CYTHON_UNUSED struct __pyx_obj_9drf_turbo_6fields_DateTimeField *__pyx_v_self, CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_default_timezone", 0);

  /* "drf_turbo/fields.pyx":915
 * 
 *     cpdef inline get_default_timezone(self):
 *         return timezone.get_current_timezone() if settings.USE_TZ else None             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_settings); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 915, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_USE_TZ); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 915, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 915, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__pyx_t_4) {
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_timezone); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 915, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_get_current_timezone); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 915, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_2)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
      }
    }
    __pyx_t_3 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_5);
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 915, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_1 = __pyx_t_3;
    __pyx_t_3 = 0;
  } else {
    __Pyx_INCREF(Py_None);
    __pyx_t_1 = Py_None;
  }
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "drf_turbo/fields.pyx":914
 *         super().__init__(**kwargs)
 * 
 *     cpdef inline get_default_timezone(self):             # <<<<<<<<<<<<<<
 *         return timezone.get_current_timezone() if settings.USE_TZ else None
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("drf_turbo.fields.DateTimeField.get_default_timezone", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_13DateTimeField_3get_default_timezone(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_13DateTimeField_3get_default_timezone(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_default_timezone (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_13DateTimeField_2get_default_timezone(((struct __pyx_obj_9drf_turbo_6fields_DateTimeField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_13DateTimeField_2get_default_timezone(struct __pyx_obj_9drf_turbo_6fields_DateTimeField *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_default_timezone", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields_13DateTimeField_get_default_timezone(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 914, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.DateTimeField.get_default_timezone", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":918
 * 
 * 
 *     cpdef inline enforce_timezone(self, value):             # <<<<<<<<<<<<<<
 *         """
 *         When `self.default_timezone` is `None`, always return naive datetimes.
 */

static PyObject *__pyx_pw_9drf_turbo_6fields_13DateTimeField_5enforce_timezone(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_13DateTimeField_enforce_timezone(struct __pyx_obj_9drf_turbo_6fields_DateTimeField *__pyx_v_self, PyObject *__pyx_v_value, CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  int __pyx_t_13;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("enforce_timezone", 0);

  /* "drf_turbo/fields.pyx":923
 *         When `self.default_timezone` is not `None`, always return aware datetimes.
 *         """
 *         if self.timezone is not None:             # <<<<<<<<<<<<<<
 *             if timezone.is_aware(value):
 *                 try:
 */
  __pyx_t_1 = (__pyx_v_self->timezone != Py_None);
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "drf_turbo/fields.pyx":924
 *         """
 *         if self.timezone is not None:
 *             if timezone.is_aware(value):             # <<<<<<<<<<<<<<
 *                 try:
 *                     return value.astimezone(self.timezone)
 */
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_timezone); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 924, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_is_aware); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 924, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
      }
    }
    __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_4, __pyx_v_value) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_v_value);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 924, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 924, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_2) {

      /* "drf_turbo/fields.pyx":925
 *         if self.timezone is not None:
 *             if timezone.is_aware(value):
 *                 try:             # <<<<<<<<<<<<<<
 *                     return value.astimezone(self.timezone)
 *                 except OverflowError:
 */
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_6, &__pyx_t_7, &__pyx_t_8);
        __Pyx_XGOTREF(__pyx_t_6);
        __Pyx_XGOTREF(__pyx_t_7);
        __Pyx_XGOTREF(__pyx_t_8);
        /*try:*/ {

          /* "drf_turbo/fields.pyx":926
 *             if timezone.is_aware(value):
 *                 try:
 *                     return value.astimezone(self.timezone)             # <<<<<<<<<<<<<<
 *                 except OverflowError:
 *                    raise self.raise_if_fail('overflow')
 */
          __Pyx_XDECREF(__pyx_r);
          __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_value, __pyx_n_s_astimezone); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 926, __pyx_L5_error)
          __Pyx_GOTREF(__pyx_t_5);
          __pyx_t_4 = NULL;
          if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
            __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_5);
            if (likely(__pyx_t_4)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
              __Pyx_INCREF(__pyx_t_4);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_5, function);
            }
          }
          __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_4, __pyx_v_self->timezone) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_v_self->timezone);
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 926, __pyx_L5_error)
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __pyx_r = __pyx_t_3;
          __pyx_t_3 = 0;
          goto __pyx_L9_try_return;

          /* "drf_turbo/fields.pyx":925
 *         if self.timezone is not None:
 *             if timezone.is_aware(value):
 *                 try:             # <<<<<<<<<<<<<<
 *                     return value.astimezone(self.timezone)
 *                 except OverflowError:
 */
        }
        __pyx_L5_error:;
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;

        /* "drf_turbo/fields.pyx":927
 *                 try:
 *                     return value.astimezone(self.timezone)
 *                 except OverflowError:             # <<<<<<<<<<<<<<
 *                    raise self.raise_if_fail('overflow')
 *             try:
 */
        __pyx_t_9 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_OverflowError);
        if (__pyx_t_9) {
          __Pyx_AddTraceback("drf_turbo.fields.DateTimeField.enforce_timezone", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_3, &__pyx_t_5, &__pyx_t_4) < 0) __PYX_ERR(0, 927, __pyx_L7_except_error)
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_GOTREF(__pyx_t_4);

          /* "drf_turbo/fields.pyx":928
 *                     return value.astimezone(self.timezone)
 *                 except OverflowError:
 *                    raise self.raise_if_fail('overflow')             # <<<<<<<<<<<<<<
 *             try:
 *                 return timezone.make_aware(value, self.timezone)
 */
          __pyx_t_11 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_raise_if_fail); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 928, __pyx_L7_except_error)
          __Pyx_GOTREF(__pyx_t_11);
          __pyx_t_12 = NULL;
          if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_11))) {
            __pyx_t_12 = PyMethod_GET_SELF(__pyx_t_11);
            if (likely(__pyx_t_12)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_11);
              __Pyx_INCREF(__pyx_t_12);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_11, function);
            }
          }
          __pyx_t_10 = (__pyx_t_12) ? __Pyx_PyObject_Call2Args(__pyx_t_11, __pyx_t_12, __pyx_n_s_overflow) : __Pyx_PyObject_CallOneArg(__pyx_t_11, __pyx_n_s_overflow);
          __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
          if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 928, __pyx_L7_except_error)
          __Pyx_GOTREF(__pyx_t_10);
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_Raise(__pyx_t_10, 0, 0, 0);
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          __PYX_ERR(0, 928, __pyx_L7_except_error)
        }
        goto __pyx_L7_except_error;
        __pyx_L7_except_error:;

        /* "drf_turbo/fields.pyx":925
 *         if self.timezone is not None:
 *             if timezone.is_aware(value):
 *                 try:             # <<<<<<<<<<<<<<
 *                     return value.astimezone(self.timezone)
 *                 except OverflowError:
 */
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_7, __pyx_t_8);
        goto __pyx_L1_error;
        __pyx_L9_try_return:;
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_7, __pyx_t_8);
        goto __pyx_L0;
      }

      /* "drf_turbo/fields.pyx":924
 *         """
 *         if self.timezone is not None:
 *             if timezone.is_aware(value):             # <<<<<<<<<<<<<<
 *                 try:
 *                     return value.astimezone(self.timezone)
 */
    }

    /* "drf_turbo/fields.pyx":929
 *                 except OverflowError:
 *                    raise self.raise_if_fail('overflow')
 *             try:             # <<<<<<<<<<<<<<
 *                 return timezone.make_aware(value, self.timezone)
 *             except InvalidTimeError:
 */
    {
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __Pyx_ExceptionSave(&__pyx_t_8, &__pyx_t_7, &__pyx_t_6);
      __Pyx_XGOTREF(__pyx_t_8);
      __Pyx_XGOTREF(__pyx_t_7);
      __Pyx_XGOTREF(__pyx_t_6);
      /*try:*/ {

        /* "drf_turbo/fields.pyx":930
 *                    raise self.raise_if_fail('overflow')
 *             try:
 *                 return timezone.make_aware(value, self.timezone)             # <<<<<<<<<<<<<<
 *             except InvalidTimeError:
 *                 raise self.raise_if_fail('make_aware', timezone=self.timezone)
 */
        __Pyx_XDECREF(__pyx_r);
        __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_timezone); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 930, __pyx_L13_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_make_aware); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 930, __pyx_L13_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __pyx_t_5 = NULL;
        __pyx_t_9 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_5)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_5);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
            __pyx_t_9 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_v_value, __pyx_v_self->timezone};
          __pyx_t_4 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_9, 2+__pyx_t_9); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 930, __pyx_L13_error)
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_GOTREF(__pyx_t_4);
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_v_value, __pyx_v_self->timezone};
          __pyx_t_4 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_9, 2+__pyx_t_9); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 930, __pyx_L13_error)
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_GOTREF(__pyx_t_4);
        } else
        #endif
        {
          __pyx_t_10 = PyTuple_New(2+__pyx_t_9); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 930, __pyx_L13_error)
          __Pyx_GOTREF(__pyx_t_10);
          if (__pyx_t_5) {
            __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_5); __pyx_t_5 = NULL;
          }
          __Pyx_INCREF(__pyx_v_value);
          __Pyx_GIVEREF(__pyx_v_value);
          PyTuple_SET_ITEM(__pyx_t_10, 0+__pyx_t_9, __pyx_v_value);
          __Pyx_INCREF(__pyx_v_self->timezone);
          __Pyx_GIVEREF(__pyx_v_self->timezone);
          PyTuple_SET_ITEM(__pyx_t_10, 1+__pyx_t_9, __pyx_v_self->timezone);
          __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_10, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 930, __pyx_L13_error)
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        }
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_4;
        __pyx_t_4 = 0;
        goto __pyx_L17_try_return;

        /* "drf_turbo/fields.pyx":929
 *                 except OverflowError:
 *                    raise self.raise_if_fail('overflow')
 *             try:             # <<<<<<<<<<<<<<
 *                 return timezone.make_aware(value, self.timezone)
 *             except InvalidTimeError:
 */
      }
      __pyx_L13_error:;
      __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
      __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;

      /* "drf_turbo/fields.pyx":931
 *             try:
 *                 return timezone.make_aware(value, self.timezone)
 *             except InvalidTimeError:             # <<<<<<<<<<<<<<
 *                 raise self.raise_if_fail('make_aware', timezone=self.timezone)
 *         elif (self.timezone is None) and timezone.is_aware(value):
 */
      __Pyx_ErrFetch(&__pyx_t_4, &__pyx_t_3, &__pyx_t_10);
      __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_InvalidTimeError); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 931, __pyx_L15_except_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_9 = __Pyx_PyErr_GivenExceptionMatches(__pyx_t_4, __pyx_t_5);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_ErrRestore(__pyx_t_4, __pyx_t_3, __pyx_t_10);
      __pyx_t_4 = 0; __pyx_t_3 = 0; __pyx_t_10 = 0;
      if (__pyx_t_9) {
        __Pyx_AddTraceback("drf_turbo.fields.DateTimeField.enforce_timezone", __pyx_clineno, __pyx_lineno, __pyx_filename);
        if (__Pyx_GetException(&__pyx_t_10, &__pyx_t_3, &__pyx_t_4) < 0) __PYX_ERR(0, 931, __pyx_L15_except_error)
        __Pyx_GOTREF(__pyx_t_10);
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_GOTREF(__pyx_t_4);

        /* "drf_turbo/fields.pyx":932
 *                 return timezone.make_aware(value, self.timezone)
 *             except InvalidTimeError:
 *                 raise self.raise_if_fail('make_aware', timezone=self.timezone)             # <<<<<<<<<<<<<<
 *         elif (self.timezone is None) and timezone.is_aware(value):
 *             return timezone.make_naive(value, timezone.utc)
 */
        __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_raise_if_fail); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 932, __pyx_L15_except_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_11 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 932, __pyx_L15_except_error)
        __Pyx_GOTREF(__pyx_t_11);
        if (PyDict_SetItem(__pyx_t_11, __pyx_n_s_timezone, __pyx_v_self->timezone) < 0) __PYX_ERR(0, 932, __pyx_L15_except_error)
        __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_tuple__21, __pyx_t_11); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 932, __pyx_L15_except_error)
        __Pyx_GOTREF(__pyx_t_12);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        __Pyx_Raise(__pyx_t_12, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        __PYX_ERR(0, 932, __pyx_L15_except_error)
      }
      goto __pyx_L15_except_error;
      __pyx_L15_except_error:;

      /* "drf_turbo/fields.pyx":929
 *                 except OverflowError:
 *                    raise self.raise_if_fail('overflow')
 *             try:             # <<<<<<<<<<<<<<
 *                 return timezone.make_aware(value, self.timezone)
 *             except InvalidTimeError:
 */
      __Pyx_XGIVEREF(__pyx_t_8);
      __Pyx_XGIVEREF(__pyx_t_7);
      __Pyx_XGIVEREF(__pyx_t_6);
      __Pyx_ExceptionReset(__pyx_t_8, __pyx_t_7, __pyx_t_6);
      goto __pyx_L1_error;
      __pyx_L17_try_return:;
      __Pyx_XGIVEREF(__pyx_t_8);
      __Pyx_XGIVEREF(__pyx_t_7);
      __Pyx_XGIVEREF(__pyx_t_6);
      __Pyx_ExceptionReset(__pyx_t_8, __pyx_t_7, __pyx_t_6);
      goto __pyx_L0;
    }

    /* "drf_turbo/fields.pyx":923
 *         When `self.default_timezone` is not `None`, always return aware datetimes.
 *         """
 *         if self.timezone is not None:             # <<<<<<<<<<<<<<
 *             if timezone.is_aware(value):
 *                 try:
 */
  }

  /* "drf_turbo/fields.pyx":933
 *             except InvalidTimeError:
 *                 raise self.raise_if_fail('make_aware', timezone=self.timezone)
 *         elif (self.timezone is None) and timezone.is_aware(value):             # <<<<<<<<<<<<<<
 *             return timezone.make_naive(value, timezone.utc)
 *         return value
 */
  __pyx_t_1 = (__pyx_v_self->timezone == Py_None);
  __pyx_t_13 = (__pyx_t_1 != 0);
  if (__pyx_t_13) {
  } else {
    __pyx_t_2 = __pyx_t_13;
    goto __pyx_L21_bool_binop_done;
  }
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_timezone); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 933, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_is_aware); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 933, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_10))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_10);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_10, function);
    }
  }
  __pyx_t_4 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_10, __pyx_t_3, __pyx_v_value) : __Pyx_PyObject_CallOneArg(__pyx_t_10, __pyx_v_value);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 933, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_13 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_13 < 0)) __PYX_ERR(0, 933, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_2 = __pyx_t_13;
  __pyx_L21_bool_binop_done:;
  if (__pyx_t_2) {

    /* "drf_turbo/fields.pyx":934
 *                 raise self.raise_if_fail('make_aware', timezone=self.timezone)
 *         elif (self.timezone is None) and timezone.is_aware(value):
 *             return timezone.make_naive(value, timezone.utc)             # <<<<<<<<<<<<<<
 *         return value
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_n_s_timezone); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 934, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_n_s_make_naive); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 934, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_n_s_timezone); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 934, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_n_s_utc); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 934, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_12);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __pyx_t_10 = NULL;
    __pyx_t_9 = 0;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_10)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
        __pyx_t_9 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[3] = {__pyx_t_10, __pyx_v_value, __pyx_t_12};
      __pyx_t_4 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_9, 2+__pyx_t_9); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 934, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[3] = {__pyx_t_10, __pyx_v_value, __pyx_t_12};
      __pyx_t_4 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_9, 2+__pyx_t_9); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 934, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    } else
    #endif
    {
      __pyx_t_11 = PyTuple_New(2+__pyx_t_9); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 934, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      if (__pyx_t_10) {
        __Pyx_GIVEREF(__pyx_t_10); PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_10); __pyx_t_10 = NULL;
      }
      __Pyx_INCREF(__pyx_v_value);
      __Pyx_GIVEREF(__pyx_v_value);
      PyTuple_SET_ITEM(__pyx_t_11, 0+__pyx_t_9, __pyx_v_value);
      __Pyx_GIVEREF(__pyx_t_12);
      PyTuple_SET_ITEM(__pyx_t_11, 1+__pyx_t_9, __pyx_t_12);
      __pyx_t_12 = 0;
      __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_11, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 934, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;

    /* "drf_turbo/fields.pyx":933
 *             except InvalidTimeError:
 *                 raise self.raise_if_fail('make_aware', timezone=self.timezone)
 *         elif (self.timezone is None) and timezone.is_aware(value):             # <<<<<<<<<<<<<<
 *             return timezone.make_naive(value, timezone.utc)
 *         return value
 */
  }

  /* "drf_turbo/fields.pyx":935
 *         elif (self.timezone is None) and timezone.is_aware(value):
 *             return timezone.make_naive(value, timezone.utc)
 *         return value             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_value);
  __pyx_r = __pyx_v_value;
  goto __pyx_L0;

  /* "drf_turbo/fields.pyx":918
 * 
 * 
 *     cpdef inline enforce_timezone(self, value):             # <<<<<<<<<<<<<<
 *         """
 *         When `self.default_timezone` is `None`, always return naive datetimes.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_AddTraceback("drf_turbo.fields.DateTimeField.enforce_timezone", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_13DateTimeField_5enforce_timezone(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static char __pyx_doc_9drf_turbo_6fields_13DateTimeField_4enforce_timezone[] = "\n        When `self.default_timezone` is `None`, always return naive datetimes.\n        When `self.default_timezone` is not `None`, always return aware datetimes.\n        ";
static PyObject *__pyx_pw_9drf_turbo_6fields_13DateTimeField_5enforce_timezone(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("enforce_timezone (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_13DateTimeField_4enforce_timezone(((struct __pyx_obj_9drf_turbo_6fields_DateTimeField *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_13DateTimeField_4enforce_timezone(struct __pyx_obj_9drf_turbo_6fields_DateTimeField *__pyx_v_self, PyObject *__pyx_v_value) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("enforce_timezone", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields_13DateTimeField_enforce_timezone(__pyx_v_self, __pyx_v_value, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 918, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.DateTimeField.enforce_timezone", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":938
 * 
 * 
 *     cpdef inline serialize(self,value,dict context):             # <<<<<<<<<<<<<<
 *         if not value:
 *             return None
 */

static PyObject *__pyx_pw_9drf_turbo_6fields_13DateTimeField_7serialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_13DateTimeField_serialize(struct __pyx_obj_9drf_turbo_6fields_DateTimeField *__pyx_v_self, PyObject *__pyx_v_value, CYTHON_UNUSED PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("serialize", 0);
  __Pyx_INCREF(__pyx_v_value);

  /* "drf_turbo/fields.pyx":939
 * 
 *     cpdef inline serialize(self,value,dict context):
 *         if not value:             # <<<<<<<<<<<<<<
 *             return None
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_value); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 939, __pyx_L1_error)
  __pyx_t_2 = ((!__pyx_t_1) != 0);
  if (__pyx_t_2) {

    /* "drf_turbo/fields.pyx":940
 *     cpdef inline serialize(self,value,dict context):
 *         if not value:
 *             return None             # <<<<<<<<<<<<<<
 * 
 *         if self.format is None or isinstance(value, str):
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "drf_turbo/fields.pyx":939
 * 
 *     cpdef inline serialize(self,value,dict context):
 *         if not value:             # <<<<<<<<<<<<<<
 *             return None
 * 
 */
  }

  /* "drf_turbo/fields.pyx":942
 *             return None
 * 
 *         if self.format is None or isinstance(value, str):             # <<<<<<<<<<<<<<
 *             return value
 * 
 */
  __pyx_t_1 = (__pyx_v_self->format == Py_None);
  __pyx_t_3 = (__pyx_t_1 != 0);
  if (!__pyx_t_3) {
  } else {
    __pyx_t_2 = __pyx_t_3;
    goto __pyx_L5_bool_binop_done;
  }
  __pyx_t_3 = PyString_Check(__pyx_v_value); 
  __pyx_t_1 = (__pyx_t_3 != 0);
  __pyx_t_2 = __pyx_t_1;
  __pyx_L5_bool_binop_done:;
  if (__pyx_t_2) {

    /* "drf_turbo/fields.pyx":943
 * 
 *         if self.format is None or isinstance(value, str):
 *             return value             # <<<<<<<<<<<<<<
 * 
 *         value = self.enforce_timezone(value)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_value);
    __pyx_r = __pyx_v_value;
    goto __pyx_L0;

    /* "drf_turbo/fields.pyx":942
 *             return None
 * 
 *         if self.format is None or isinstance(value, str):             # <<<<<<<<<<<<<<
 *             return value
 * 
 */
  }

  /* "drf_turbo/fields.pyx":945
 *             return value
 * 
 *         value = self.enforce_timezone(value)             # <<<<<<<<<<<<<<
 * 
 *         if self.format.lower() == ISO_8601:
 */
  __pyx_t_4 = __pyx_f_9drf_turbo_6fields_13DateTimeField_enforce_timezone(__pyx_v_self, __pyx_v_value, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 945, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF_SET(__pyx_v_value, __pyx_t_4);
  __pyx_t_4 = 0;

  /* "drf_turbo/fields.pyx":947
 *         value = self.enforce_timezone(value)
 * 
 *         if self.format.lower() == ISO_8601:             # <<<<<<<<<<<<<<
 *             value = value.isoformat()
 *             if value.endswith('+00:00'):
 */
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->format, __pyx_n_s_lower); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 947, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
    }
  }
  __pyx_t_4 = (__pyx_t_6) ? __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_6) : __Pyx_PyObject_CallNoArg(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 947, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_ISO_8601); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 947, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = PyObject_RichCompare(__pyx_t_4, __pyx_t_5, Py_EQ); __Pyx_XGOTREF(__pyx_t_6); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 947, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_6); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 947, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (__pyx_t_2) {

    /* "drf_turbo/fields.pyx":948
 * 
 *         if self.format.lower() == ISO_8601:
 *             value = value.isoformat()             # <<<<<<<<<<<<<<
 *             if value.endswith('+00:00'):
 *                 value = value[:-6] + 'Z'
 */
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_value, __pyx_n_s_isoformat); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 948, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
      }
    }
    __pyx_t_6 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_5);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 948, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF_SET(__pyx_v_value, __pyx_t_6);
    __pyx_t_6 = 0;

    /* "drf_turbo/fields.pyx":949
 *         if self.format.lower() == ISO_8601:
 *             value = value.isoformat()
 *             if value.endswith('+00:00'):             # <<<<<<<<<<<<<<
 *                 value = value[:-6] + 'Z'
 *             return value
 */
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_value, __pyx_n_s_endswith); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 949, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
      }
    }
    __pyx_t_6 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_4, __pyx_kp_s_00_00) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_kp_s_00_00);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 949, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_6); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 949, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (__pyx_t_2) {

      /* "drf_turbo/fields.pyx":950
 *             value = value.isoformat()
 *             if value.endswith('+00:00'):
 *                 value = value[:-6] + 'Z'             # <<<<<<<<<<<<<<
 *             return value
 *         return value.strftime(self.format)
 */
      __pyx_t_6 = __Pyx_PyObject_GetSlice(__pyx_v_value, 0, -6L, NULL, NULL, &__pyx_slice__22, 0, 1, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 950, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_5 = PyNumber_Add(__pyx_t_6, __pyx_n_s_Z); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 950, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF_SET(__pyx_v_value, __pyx_t_5);
      __pyx_t_5 = 0;

      /* "drf_turbo/fields.pyx":949
 *         if self.format.lower() == ISO_8601:
 *             value = value.isoformat()
 *             if value.endswith('+00:00'):             # <<<<<<<<<<<<<<
 *                 value = value[:-6] + 'Z'
 *             return value
 */
    }

    /* "drf_turbo/fields.pyx":951
 *             if value.endswith('+00:00'):
 *                 value = value[:-6] + 'Z'
 *             return value             # <<<<<<<<<<<<<<
 *         return value.strftime(self.format)
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_value);
    __pyx_r = __pyx_v_value;
    goto __pyx_L0;

    /* "drf_turbo/fields.pyx":947
 *         value = self.enforce_timezone(value)
 * 
 *         if self.format.lower() == ISO_8601:             # <<<<<<<<<<<<<<
 *             value = value.isoformat()
 *             if value.endswith('+00:00'):
 */
  }

  /* "drf_turbo/fields.pyx":952
 *                 value = value[:-6] + 'Z'
 *             return value
 *         return value.strftime(self.format)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_value, __pyx_n_s_strftime); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 952, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_6);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_6, function);
    }
  }
  __pyx_t_5 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_4, __pyx_v_self->format) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_v_self->format);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 952, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_r = __pyx_t_5;
  __pyx_t_5 = 0;
  goto __pyx_L0;

  /* "drf_turbo/fields.pyx":938
 * 
 * 
 *     cpdef inline serialize(self,value,dict context):             # <<<<<<<<<<<<<<
 *         if not value:
 *             return None
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("drf_turbo.fields.DateTimeField.serialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_value);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_13DateTimeField_7serialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_13DateTimeField_7serialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_value = 0;
  PyObject *__pyx_v_context = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("serialize (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_value,&__pyx_n_s_context,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_value)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_context)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("serialize", 1, 2, 2, 1); __PYX_ERR(0, 938, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "serialize") < 0)) __PYX_ERR(0, 938, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_value = values[0];
    __pyx_v_context = ((PyObject*)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("serialize", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 938, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.DateTimeField.serialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_context), (&PyDict_Type), 1, "context", 1))) __PYX_ERR(0, 938, __pyx_L1_error)
  __pyx_r = __pyx_pf_9drf_turbo_6fields_13DateTimeField_6serialize(((struct __pyx_obj_9drf_turbo_6fields_DateTimeField *)__pyx_v_self), __pyx_v_value, __pyx_v_context);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_13DateTimeField_6serialize(struct __pyx_obj_9drf_turbo_6fields_DateTimeField *__pyx_v_self, PyObject *__pyx_v_value, PyObject *__pyx_v_context) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("serialize", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields_13DateTimeField_serialize(__pyx_v_self, __pyx_v_value, __pyx_v_context, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 938, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.DateTimeField.serialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":955
 * 
 * 
 *     cpdef inline deserialize(self,data,dict context) :             # <<<<<<<<<<<<<<
 * 
 * 
 */

static PyObject *__pyx_pw_9drf_turbo_6fields_13DateTimeField_9deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_13DateTimeField_deserialize(struct __pyx_obj_9drf_turbo_6fields_DateTimeField *__pyx_v_self, PyObject *__pyx_v_data, CYTHON_UNUSED PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_v_input_format = NULL;
  PyObject *__pyx_v_parsed = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  Py_ssize_t __pyx_t_7;
  PyObject *(*__pyx_t_8)(PyObject *);
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  int __pyx_t_13;
  PyObject *__pyx_t_14 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("deserialize", 0);

  /* "drf_turbo/fields.pyx":958
 * 
 * 
 *         if isinstance(data, datetime.date) and not isinstance(data, datetime.datetime):             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail('date')
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_datetime); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 958, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_date); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 958, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = PyObject_IsInstance(__pyx_v_data, __pyx_t_3); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(0, 958, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_5 = (__pyx_t_4 != 0);
  if (__pyx_t_5) {
  } else {
    __pyx_t_1 = __pyx_t_5;
    goto __pyx_L4_bool_binop_done;
  }
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_datetime); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 958, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_datetime); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 958, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_5 = PyObject_IsInstance(__pyx_v_data, __pyx_t_2); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 958, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = ((!(__pyx_t_5 != 0)) != 0);
  __pyx_t_1 = __pyx_t_4;
  __pyx_L4_bool_binop_done:;
  if (unlikely(__pyx_t_1)) {

    /* "drf_turbo/fields.pyx":959
 * 
 *         if isinstance(data, datetime.date) and not isinstance(data, datetime.datetime):
 *             raise self.raise_if_fail('date')             # <<<<<<<<<<<<<<
 * 
 *         if isinstance(data, datetime.datetime):
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_raise_if_fail); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 959, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_6 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_2 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_6, __pyx_n_s_date) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_n_s_date);
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 959, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 959, __pyx_L1_error)

    /* "drf_turbo/fields.pyx":958
 * 
 * 
 *         if isinstance(data, datetime.date) and not isinstance(data, datetime.datetime):             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail('date')
 * 
 */
  }

  /* "drf_turbo/fields.pyx":961
 *             raise self.raise_if_fail('date')
 * 
 *         if isinstance(data, datetime.datetime):             # <<<<<<<<<<<<<<
 *             return self.enforce_timezone(data)
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_datetime); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 961, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_datetime); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 961, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_1 = PyObject_IsInstance(__pyx_v_data, __pyx_t_3); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(0, 961, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_4 = (__pyx_t_1 != 0);
  if (__pyx_t_4) {

    /* "drf_turbo/fields.pyx":962
 * 
 *         if isinstance(data, datetime.datetime):
 *             return self.enforce_timezone(data)             # <<<<<<<<<<<<<<
 * 
 *         for input_format in self.input_formats:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __pyx_f_9drf_turbo_6fields_13DateTimeField_enforce_timezone(__pyx_v_self, __pyx_v_data, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 962, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "drf_turbo/fields.pyx":961
 *             raise self.raise_if_fail('date')
 * 
 *         if isinstance(data, datetime.datetime):             # <<<<<<<<<<<<<<
 *             return self.enforce_timezone(data)
 * 
 */
  }

  /* "drf_turbo/fields.pyx":964
 *             return self.enforce_timezone(data)
 * 
 *         for input_format in self.input_formats:             # <<<<<<<<<<<<<<
 *             if input_format.lower() == ISO_8601:
 *                 try:
 */
  if (likely(PyList_CheckExact(__pyx_v_self->input_formats)) || PyTuple_CheckExact(__pyx_v_self->input_formats)) {
    __pyx_t_3 = __pyx_v_self->input_formats; __Pyx_INCREF(__pyx_t_3); __pyx_t_7 = 0;
    __pyx_t_8 = NULL;
  } else {
    __pyx_t_7 = -1; __pyx_t_3 = PyObject_GetIter(__pyx_v_self->input_formats); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 964, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_8 = Py_TYPE(__pyx_t_3)->tp_iternext; if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 964, __pyx_L1_error)
  }
  for (;;) {
    if (likely(!__pyx_t_8)) {
      if (likely(PyList_CheckExact(__pyx_t_3))) {
        if (__pyx_t_7 >= PyList_GET_SIZE(__pyx_t_3)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_2 = PyList_GET_ITEM(__pyx_t_3, __pyx_t_7); __Pyx_INCREF(__pyx_t_2); __pyx_t_7++; if (unlikely(0 < 0)) __PYX_ERR(0, 964, __pyx_L1_error)
        #else
        __pyx_t_2 = PySequence_ITEM(__pyx_t_3, __pyx_t_7); __pyx_t_7++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 964, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        #endif
      } else {
        if (__pyx_t_7 >= PyTuple_GET_SIZE(__pyx_t_3)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_2 = PyTuple_GET_ITEM(__pyx_t_3, __pyx_t_7); __Pyx_INCREF(__pyx_t_2); __pyx_t_7++; if (unlikely(0 < 0)) __PYX_ERR(0, 964, __pyx_L1_error)
        #else
        __pyx_t_2 = PySequence_ITEM(__pyx_t_3, __pyx_t_7); __pyx_t_7++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 964, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        #endif
      }
    } else {
      __pyx_t_2 = __pyx_t_8(__pyx_t_3);
      if (unlikely(!__pyx_t_2)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 964, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_XDECREF_SET(__pyx_v_input_format, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "drf_turbo/fields.pyx":965
 * 
 *         for input_format in self.input_formats:
 *             if input_format.lower() == ISO_8601:             # <<<<<<<<<<<<<<
 *                 try:
 *                     parsed = parse_datetime(data)
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_input_format, __pyx_n_s_lower); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 965, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_9 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
      __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_6);
      if (likely(__pyx_t_9)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
        __Pyx_INCREF(__pyx_t_9);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_6, function);
      }
    }
    __pyx_t_2 = (__pyx_t_9) ? __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_9) : __Pyx_PyObject_CallNoArg(__pyx_t_6);
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 965, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_n_s_ISO_8601); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 965, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_9 = PyObject_RichCompare(__pyx_t_2, __pyx_t_6, Py_EQ); __Pyx_XGOTREF(__pyx_t_9); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 965, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_9); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 965, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (__pyx_t_4) {

      /* "drf_turbo/fields.pyx":966
 *         for input_format in self.input_formats:
 *             if input_format.lower() == ISO_8601:
 *                 try:             # <<<<<<<<<<<<<<
 *                     parsed = parse_datetime(data)
 *                     if parsed is not None:
 */
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_10, &__pyx_t_11, &__pyx_t_12);
        __Pyx_XGOTREF(__pyx_t_10);
        __Pyx_XGOTREF(__pyx_t_11);
        __Pyx_XGOTREF(__pyx_t_12);
        /*try:*/ {

          /* "drf_turbo/fields.pyx":967
 *             if input_format.lower() == ISO_8601:
 *                 try:
 *                     parsed = parse_datetime(data)             # <<<<<<<<<<<<<<
 *                     if parsed is not None:
 *                         return self.enforce_timezone(parsed)
 */
          __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_n_s_parse_datetime); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 967, __pyx_L10_error)
          __Pyx_GOTREF(__pyx_t_6);
          __pyx_t_2 = NULL;
          if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_6))) {
            __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_6);
            if (likely(__pyx_t_2)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
              __Pyx_INCREF(__pyx_t_2);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_6, function);
            }
          }
          __pyx_t_9 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_2, __pyx_v_data) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_v_data);
          __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
          if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 967, __pyx_L10_error)
          __Pyx_GOTREF(__pyx_t_9);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_XDECREF_SET(__pyx_v_parsed, __pyx_t_9);
          __pyx_t_9 = 0;

          /* "drf_turbo/fields.pyx":968
 *                 try:
 *                     parsed = parse_datetime(data)
 *                     if parsed is not None:             # <<<<<<<<<<<<<<
 *                         return self.enforce_timezone(parsed)
 *                 except (ValueError, TypeError):
 */
          __pyx_t_4 = (__pyx_v_parsed != Py_None);
          __pyx_t_1 = (__pyx_t_4 != 0);
          if (__pyx_t_1) {

            /* "drf_turbo/fields.pyx":969
 *                     parsed = parse_datetime(data)
 *                     if parsed is not None:
 *                         return self.enforce_timezone(parsed)             # <<<<<<<<<<<<<<
 *                 except (ValueError, TypeError):
 *                     pass
 */
            __Pyx_XDECREF(__pyx_r);
            __pyx_t_9 = __pyx_f_9drf_turbo_6fields_13DateTimeField_enforce_timezone(__pyx_v_self, __pyx_v_parsed, 0); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 969, __pyx_L10_error)
            __Pyx_GOTREF(__pyx_t_9);
            __pyx_r = __pyx_t_9;
            __pyx_t_9 = 0;
            __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
            goto __pyx_L14_try_return;

            /* "drf_turbo/fields.pyx":968
 *                 try:
 *                     parsed = parse_datetime(data)
 *                     if parsed is not None:             # <<<<<<<<<<<<<<
 *                         return self.enforce_timezone(parsed)
 *                 except (ValueError, TypeError):
 */
          }

          /* "drf_turbo/fields.pyx":966
 *         for input_format in self.input_formats:
 *             if input_format.lower() == ISO_8601:
 *                 try:             # <<<<<<<<<<<<<<
 *                     parsed = parse_datetime(data)
 *                     if parsed is not None:
 */
        }
        __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
        __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
        __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
        goto __pyx_L17_try_end;
        __pyx_L10_error:;
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;

        /* "drf_turbo/fields.pyx":970
 *                     if parsed is not None:
 *                         return self.enforce_timezone(parsed)
 *                 except (ValueError, TypeError):             # <<<<<<<<<<<<<<
 *                     pass
 *             else:
 */
        __pyx_t_13 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_ValueError) || __Pyx_PyErr_ExceptionMatches(__pyx_builtin_TypeError);
        if (__pyx_t_13) {
          __Pyx_ErrRestore(0,0,0);
          goto __pyx_L11_exception_handled;
        }
        goto __pyx_L12_except_error;
        __pyx_L12_except_error:;

        /* "drf_turbo/fields.pyx":966
 *         for input_format in self.input_formats:
 *             if input_format.lower() == ISO_8601:
 *                 try:             # <<<<<<<<<<<<<<
 *                     parsed = parse_datetime(data)
 *                     if parsed is not None:
 */
        __Pyx_XGIVEREF(__pyx_t_10);
        __Pyx_XGIVEREF(__pyx_t_11);
        __Pyx_XGIVEREF(__pyx_t_12);
        __Pyx_ExceptionReset(__pyx_t_10, __pyx_t_11, __pyx_t_12);
        goto __pyx_L1_error;
        __pyx_L14_try_return:;
        __Pyx_XGIVEREF(__pyx_t_10);
        __Pyx_XGIVEREF(__pyx_t_11);
        __Pyx_XGIVEREF(__pyx_t_12);
        __Pyx_ExceptionReset(__pyx_t_10, __pyx_t_11, __pyx_t_12);
        goto __pyx_L0;
        __pyx_L11_exception_handled:;
        __Pyx_XGIVEREF(__pyx_t_10);
        __Pyx_XGIVEREF(__pyx_t_11);
        __Pyx_XGIVEREF(__pyx_t_12);
        __Pyx_ExceptionReset(__pyx_t_10, __pyx_t_11, __pyx_t_12);
        __pyx_L17_try_end:;
      }

      /* "drf_turbo/fields.pyx":965
 * 
 *         for input_format in self.input_formats:
 *             if input_format.lower() == ISO_8601:             # <<<<<<<<<<<<<<
 *                 try:
 *                     parsed = parse_datetime(data)
 */
      goto __pyx_L9;
    }

    /* "drf_turbo/fields.pyx":973
 *                     pass
 *             else:
 *                 try:             # <<<<<<<<<<<<<<
 *                     parsed = self.datetime_parser(data, input_format)
 *                     return self.enforce_timezone(parsed)
 */
    /*else*/ {
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_12, &__pyx_t_11, &__pyx_t_10);
        __Pyx_XGOTREF(__pyx_t_12);
        __Pyx_XGOTREF(__pyx_t_11);
        __Pyx_XGOTREF(__pyx_t_10);
        /*try:*/ {

          /* "drf_turbo/fields.pyx":974
 *             else:
 *                 try:
 *                     parsed = self.datetime_parser(data, input_format)             # <<<<<<<<<<<<<<
 *                     return self.enforce_timezone(parsed)
 *                 except (ValueError, TypeError):
 */
          __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_datetime_parser); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 974, __pyx_L19_error)
          __Pyx_GOTREF(__pyx_t_6);
          __pyx_t_2 = NULL;
          __pyx_t_13 = 0;
          if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
            __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_6);
            if (likely(__pyx_t_2)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
              __Pyx_INCREF(__pyx_t_2);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_6, function);
              __pyx_t_13 = 1;
            }
          }
          #if CYTHON_FAST_PYCALL
          if (PyFunction_Check(__pyx_t_6)) {
            PyObject *__pyx_temp[3] = {__pyx_t_2, __pyx_v_data, __pyx_v_input_format};
            __pyx_t_9 = __Pyx_PyFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_13, 2+__pyx_t_13); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 974, __pyx_L19_error)
            __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
            __Pyx_GOTREF(__pyx_t_9);
          } else
          #endif
          #if CYTHON_FAST_PYCCALL
          if (__Pyx_PyFastCFunction_Check(__pyx_t_6)) {
            PyObject *__pyx_temp[3] = {__pyx_t_2, __pyx_v_data, __pyx_v_input_format};
            __pyx_t_9 = __Pyx_PyCFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_13, 2+__pyx_t_13); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 974, __pyx_L19_error)
            __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
            __Pyx_GOTREF(__pyx_t_9);
          } else
          #endif
          {
            __pyx_t_14 = PyTuple_New(2+__pyx_t_13); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 974, __pyx_L19_error)
            __Pyx_GOTREF(__pyx_t_14);
            if (__pyx_t_2) {
              __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_14, 0, __pyx_t_2); __pyx_t_2 = NULL;
            }
            __Pyx_INCREF(__pyx_v_data);
            __Pyx_GIVEREF(__pyx_v_data);
            PyTuple_SET_ITEM(__pyx_t_14, 0+__pyx_t_13, __pyx_v_data);
            __Pyx_INCREF(__pyx_v_input_format);
            __Pyx_GIVEREF(__pyx_v_input_format);
            PyTuple_SET_ITEM(__pyx_t_14, 1+__pyx_t_13, __pyx_v_input_format);
            __pyx_t_9 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_14, NULL); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 974, __pyx_L19_error)
            __Pyx_GOTREF(__pyx_t_9);
            __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
          }
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_XDECREF_SET(__pyx_v_parsed, __pyx_t_9);
          __pyx_t_9 = 0;

          /* "drf_turbo/fields.pyx":975
 *                 try:
 *                     parsed = self.datetime_parser(data, input_format)
 *                     return self.enforce_timezone(parsed)             # <<<<<<<<<<<<<<
 *                 except (ValueError, TypeError):
 *                     pass
 */
          __Pyx_XDECREF(__pyx_r);
          __pyx_t_9 = __pyx_f_9drf_turbo_6fields_13DateTimeField_enforce_timezone(__pyx_v_self, __pyx_v_parsed, 0); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 975, __pyx_L19_error)
          __Pyx_GOTREF(__pyx_t_9);
          __pyx_r = __pyx_t_9;
          __pyx_t_9 = 0;
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          goto __pyx_L23_try_return;

          /* "drf_turbo/fields.pyx":973
 *                     pass
 *             else:
 *                 try:             # <<<<<<<<<<<<<<
 *                     parsed = self.datetime_parser(data, input_format)
 *                     return self.enforce_timezone(parsed)
 */
        }
        __pyx_L19_error:;
        __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;

        /* "drf_turbo/fields.pyx":976
 *                     parsed = self.datetime_parser(data, input_format)
 *                     return self.enforce_timezone(parsed)
 *                 except (ValueError, TypeError):             # <<<<<<<<<<<<<<
 *                     pass
 * 
 */
        __pyx_t_13 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_ValueError) || __Pyx_PyErr_ExceptionMatches(__pyx_builtin_TypeError);
        if (__pyx_t_13) {
          __Pyx_ErrRestore(0,0,0);
          goto __pyx_L20_exception_handled;
        }
        goto __pyx_L21_except_error;
        __pyx_L21_except_error:;

        /* "drf_turbo/fields.pyx":973
 *                     pass
 *             else:
 *                 try:             # <<<<<<<<<<<<<<
 *                     parsed = self.datetime_parser(data, input_format)
 *                     return self.enforce_timezone(parsed)
 */
        __Pyx_XGIVEREF(__pyx_t_12);
        __Pyx_XGIVEREF(__pyx_t_11);
        __Pyx_XGIVEREF(__pyx_t_10);
        __Pyx_ExceptionReset(__pyx_t_12, __pyx_t_11, __pyx_t_10);
        goto __pyx_L1_error;
        __pyx_L23_try_return:;
        __Pyx_XGIVEREF(__pyx_t_12);
        __Pyx_XGIVEREF(__pyx_t_11);
        __Pyx_XGIVEREF(__pyx_t_10);
        __Pyx_ExceptionReset(__pyx_t_12, __pyx_t_11, __pyx_t_10);
        goto __pyx_L0;
        __pyx_L20_exception_handled:;
        __Pyx_XGIVEREF(__pyx_t_12);
        __Pyx_XGIVEREF(__pyx_t_11);
        __Pyx_XGIVEREF(__pyx_t_10);
        __Pyx_ExceptionReset(__pyx_t_12, __pyx_t_11, __pyx_t_10);
      }
    }
    __pyx_L9:;

    /* "drf_turbo/fields.pyx":964
 *             return self.enforce_timezone(data)
 * 
 *         for input_format in self.input_formats:             # <<<<<<<<<<<<<<
 *             if input_format.lower() == ISO_8601:
 *                 try:
 */
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "drf_turbo/fields.pyx":979
 *                     pass
 * 
 *         raise self.raise_if_fail('invalid')             # <<<<<<<<<<<<<<
 * 
 * @cython.final
 */
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_raise_if_fail); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 979, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_9))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_9);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_9, function);
    }
  }
  __pyx_t_3 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_9, __pyx_t_6, __pyx_n_s_invalid) : __Pyx_PyObject_CallOneArg(__pyx_t_9, __pyx_n_s_invalid);
  __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 979, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __Pyx_Raise(__pyx_t_3, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __PYX_ERR(0, 979, __pyx_L1_error)

  /* "drf_turbo/fields.pyx":955
 * 
 * 
 *     cpdef inline deserialize(self,data,dict context) :             # <<<<<<<<<<<<<<
 * 
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_AddTraceback("drf_turbo.fields.DateTimeField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_input_format);
  __Pyx_XDECREF(__pyx_v_parsed);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_13DateTimeField_9deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_13DateTimeField_9deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_data = 0;
  PyObject *__pyx_v_context = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("deserialize (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_data,&__pyx_n_s_context,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_data)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_context)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("deserialize", 1, 2, 2, 1); __PYX_ERR(0, 955, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "deserialize") < 0)) __PYX_ERR(0, 955, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_data = values[0];
    __pyx_v_context = ((PyObject*)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("deserialize", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 955, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.DateTimeField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_context), (&PyDict_Type), 1, "context", 1))) __PYX_ERR(0, 955, __pyx_L1_error)
  __pyx_r = __pyx_pf_9drf_turbo_6fields_13DateTimeField_8deserialize(((struct __pyx_obj_9drf_turbo_6fields_DateTimeField *)__pyx_v_self), __pyx_v_data, __pyx_v_context);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_13DateTimeField_8deserialize(struct __pyx_obj_9drf_turbo_6fields_DateTimeField *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("deserialize", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields_13DateTimeField_deserialize(__pyx_v_self, __pyx_v_data, __pyx_v_context, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 955, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.DateTimeField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pxd":107
 * cdef class DateTimeField(Field):
 *     cdef public:
 *         format             # <<<<<<<<<<<<<<
 *         input_formats
 *         default_timezone
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_13DateTimeField_6format_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_13DateTimeField_6format_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_13DateTimeField_6format___get__(((struct __pyx_obj_9drf_turbo_6fields_DateTimeField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_13DateTimeField_6format___get__(struct __pyx_obj_9drf_turbo_6fields_DateTimeField *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->format);
  __pyx_r = __pyx_v_self->format;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_13DateTimeField_6format_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_13DateTimeField_6format_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_13DateTimeField_6format_2__set__(((struct __pyx_obj_9drf_turbo_6fields_DateTimeField *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_13DateTimeField_6format_2__set__(struct __pyx_obj_9drf_turbo_6fields_DateTimeField *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->format);
  __Pyx_DECREF(__pyx_v_self->format);
  __pyx_v_self->format = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_13DateTimeField_6format_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_13DateTimeField_6format_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_13DateTimeField_6format_4__del__(((struct __pyx_obj_9drf_turbo_6fields_DateTimeField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_13DateTimeField_6format_4__del__(struct __pyx_obj_9drf_turbo_6fields_DateTimeField *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->format);
  __Pyx_DECREF(__pyx_v_self->format);
  __pyx_v_self->format = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pxd":108
 *     cdef public:
 *         format
 *         input_formats             # <<<<<<<<<<<<<<
 *         default_timezone
 *         timezone
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_13DateTimeField_13input_formats_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_13DateTimeField_13input_formats_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_13DateTimeField_13input_formats___get__(((struct __pyx_obj_9drf_turbo_6fields_DateTimeField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_13DateTimeField_13input_formats___get__(struct __pyx_obj_9drf_turbo_6fields_DateTimeField *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->input_formats);
  __pyx_r = __pyx_v_self->input_formats;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_13DateTimeField_13input_formats_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_13DateTimeField_13input_formats_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_13DateTimeField_13input_formats_2__set__(((struct __pyx_obj_9drf_turbo_6fields_DateTimeField *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_13DateTimeField_13input_formats_2__set__(struct __pyx_obj_9drf_turbo_6fields_DateTimeField *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->input_formats);
  __Pyx_DECREF(__pyx_v_self->input_formats);
  __pyx_v_self->input_formats = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_13DateTimeField_13input_formats_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_13DateTimeField_13input_formats_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_13DateTimeField_13input_formats_4__del__(((struct __pyx_obj_9drf_turbo_6fields_DateTimeField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_13DateTimeField_13input_formats_4__del__(struct __pyx_obj_9drf_turbo_6fields_DateTimeField *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->input_formats);
  __Pyx_DECREF(__pyx_v_self->input_formats);
  __pyx_v_self->input_formats = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pxd":109
 *         format
 *         input_formats
 *         default_timezone             # <<<<<<<<<<<<<<
 *         timezone
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_13DateTimeField_16default_timezone_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_13DateTimeField_16default_timezone_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_13DateTimeField_16default_timezone___get__(((struct __pyx_obj_9drf_turbo_6fields_DateTimeField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_13DateTimeField_16default_timezone___get__(struct __pyx_obj_9drf_turbo_6fields_DateTimeField *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->default_timezone);
  __pyx_r = __pyx_v_self->default_timezone;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_13DateTimeField_16default_timezone_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_13DateTimeField_16default_timezone_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_13DateTimeField_16default_timezone_2__set__(((struct __pyx_obj_9drf_turbo_6fields_DateTimeField *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_13DateTimeField_16default_timezone_2__set__(struct __pyx_obj_9drf_turbo_6fields_DateTimeField *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->default_timezone);
  __Pyx_DECREF(__pyx_v_self->default_timezone);
  __pyx_v_self->default_timezone = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_13DateTimeField_16default_timezone_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_13DateTimeField_16default_timezone_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_13DateTimeField_16default_timezone_4__del__(((struct __pyx_obj_9drf_turbo_6fields_DateTimeField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_13DateTimeField_16default_timezone_4__del__(struct __pyx_obj_9drf_turbo_6fields_DateTimeField *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->default_timezone);
  __Pyx_DECREF(__pyx_v_self->default_timezone);
  __pyx_v_self->default_timezone = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pxd":110
 *         input_formats
 *         default_timezone
 *         timezone             # <<<<<<<<<<<<<<
 * 
 *     cpdef get_default_timezone(self)
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_13DateTimeField_8timezone_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_13DateTimeField_8timezone_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_13DateTimeField_8timezone___get__(((struct __pyx_obj_9drf_turbo_6fields_DateTimeField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_13DateTimeField_8timezone___get__(struct __pyx_obj_9drf_turbo_6fields_DateTimeField *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->timezone);
  __pyx_r = __pyx_v_self->timezone;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_13DateTimeField_8timezone_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_13DateTimeField_8timezone_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_13DateTimeField_8timezone_2__set__(((struct __pyx_obj_9drf_turbo_6fields_DateTimeField *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_13DateTimeField_8timezone_2__set__(struct __pyx_obj_9drf_turbo_6fields_DateTimeField *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->timezone);
  __Pyx_DECREF(__pyx_v_self->timezone);
  __pyx_v_self->timezone = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_13DateTimeField_8timezone_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_13DateTimeField_8timezone_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_13DateTimeField_8timezone_4__del__(((struct __pyx_obj_9drf_turbo_6fields_DateTimeField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_13DateTimeField_8timezone_4__del__(struct __pyx_obj_9drf_turbo_6fields_DateTimeField *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->timezone);
  __Pyx_DECREF(__pyx_v_self->timezone);
  __pyx_v_self->timezone = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_13DateTimeField_11__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_13DateTimeField_11__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_13DateTimeField_10__reduce_cython__(((struct __pyx_obj_9drf_turbo_6fields_DateTimeField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_13DateTimeField_10__reduce_cython__(struct __pyx_obj_9drf_turbo_6fields_DateTimeField *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self.allow_null, self.attr, self.attrs, self.call, self.default_timezone, self.default_value, self.error_messages, self.field_name, self.format, self.help_text, self.initial, self.input_formats, self.label, self.read_only, self.required, self.root, self.style, self.timezone, self.validators, self.write_only)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.allow_null); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.call); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.read_only); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.required); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.write_only); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = PyTuple_New(20); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.attr);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.attr);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_v_self->__pyx_base.attr);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.attrs);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.attrs);
  PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_self->__pyx_base.attrs);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_6, 3, __pyx_t_2);
  __Pyx_INCREF(__pyx_v_self->default_timezone);
  __Pyx_GIVEREF(__pyx_v_self->default_timezone);
  PyTuple_SET_ITEM(__pyx_t_6, 4, __pyx_v_self->default_timezone);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.default_value);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.default_value);
  PyTuple_SET_ITEM(__pyx_t_6, 5, __pyx_v_self->__pyx_base.default_value);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.error_messages);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.error_messages);
  PyTuple_SET_ITEM(__pyx_t_6, 6, __pyx_v_self->__pyx_base.error_messages);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.field_name);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.field_name);
  PyTuple_SET_ITEM(__pyx_t_6, 7, __pyx_v_self->__pyx_base.field_name);
  __Pyx_INCREF(__pyx_v_self->format);
  __Pyx_GIVEREF(__pyx_v_self->format);
  PyTuple_SET_ITEM(__pyx_t_6, 8, __pyx_v_self->format);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.help_text);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.help_text);
  PyTuple_SET_ITEM(__pyx_t_6, 9, __pyx_v_self->__pyx_base.help_text);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.initial);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.initial);
  PyTuple_SET_ITEM(__pyx_t_6, 10, __pyx_v_self->__pyx_base.initial);
  __Pyx_INCREF(__pyx_v_self->input_formats);
  __Pyx_GIVEREF(__pyx_v_self->input_formats);
  PyTuple_SET_ITEM(__pyx_t_6, 11, __pyx_v_self->input_formats);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.label);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.label);
  PyTuple_SET_ITEM(__pyx_t_6, 12, __pyx_v_self->__pyx_base.label);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 13, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_6, 14, __pyx_t_4);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.root);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.root);
  PyTuple_SET_ITEM(__pyx_t_6, 15, __pyx_v_self->__pyx_base.root);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.style);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.style);
  PyTuple_SET_ITEM(__pyx_t_6, 16, __pyx_v_self->__pyx_base.style);
  __Pyx_INCREF(__pyx_v_self->timezone);
  __Pyx_GIVEREF(__pyx_v_self->timezone);
  PyTuple_SET_ITEM(__pyx_t_6, 17, __pyx_v_self->timezone);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.validators);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.validators);
  PyTuple_SET_ITEM(__pyx_t_6, 18, __pyx_v_self->__pyx_base.validators);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_6, 19, __pyx_t_5);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_6);
  __pyx_t_6 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self.allow_null, self.attr, self.attrs, self.call, self.default_timezone, self.default_value, self.error_messages, self.field_name, self.format, self.help_text, self.initial, self.input_formats, self.label, self.read_only, self.required, self.root, self.style, self.timezone, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_6 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_v__dict = __pyx_t_6;
  __pyx_t_6 = 0;

  /* "(tree fragment)":7
 *     state = (self.allow_null, self.attr, self.attrs, self.call, self.default_timezone, self.default_value, self.error_messages, self.field_name, self.format, self.help_text, self.initial, self.input_formats, self.label, self.read_only, self.required, self.root, self.style, self.timezone, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_7 = (__pyx_v__dict != Py_None);
  __pyx_t_8 = (__pyx_t_7 != 0);
  if (__pyx_t_8) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_6 = PyTuple_New(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v__dict);
    __pyx_t_5 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_6); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_5));
    __pyx_t_5 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.attr is not None or self.attrs is not None or self.default_timezone is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.format is not None or self.help_text is not None or self.initial is not None or self.input_formats is not None or self.label is not None or self.root is not None or self.style is not None or self.timezone is not None or self.validators is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self.allow_null, self.attr, self.attrs, self.call, self.default_timezone, self.default_value, self.error_messages, self.field_name, self.format, self.help_text, self.initial, self.input_formats, self.label, self.read_only, self.required, self.root, self.style, self.timezone, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self.attr is not None or self.attrs is not None or self.default_timezone is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.format is not None or self.help_text is not None or self.initial is not None or self.input_formats is not None or self.label is not None or self.root is not None or self.style is not None or self.timezone is not None or self.validators is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_DateTimeField, (type(self), 0x5e76b7e, None), state
 */
  /*else*/ {
    __pyx_t_7 = (__pyx_v_self->__pyx_base.attr != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.attrs != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->default_timezone != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.default_value != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.error_messages != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.field_name != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->format != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.help_text != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.initial != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->input_formats != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.label != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.root != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.style != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->timezone != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.validators != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    __pyx_t_8 = __pyx_t_9;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_8;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.attr is not None or self.attrs is not None or self.default_timezone is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.format is not None or self.help_text is not None or self.initial is not None or self.input_formats is not None or self.label is not None or self.root is not None or self.style is not None or self.timezone is not None or self.validators is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_DateTimeField, (type(self), 0x5e76b7e, None), state
 *     else:
 */
  __pyx_t_8 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_8) {

    /* "(tree fragment)":13
 *         use_setstate = self.attr is not None or self.attrs is not None or self.default_timezone is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.format is not None or self.help_text is not None or self.initial is not None or self.input_formats is not None or self.label is not None or self.root is not None or self.style is not None or self.timezone is not None or self.validators is not None
 *     if use_setstate:
 *         return __pyx_unpickle_DateTimeField, (type(self), 0x5e76b7e, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_DateTimeField, (type(self), 0x5e76b7e, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_pyx_unpickle_DateTimeField); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_6, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_99052414);
    __Pyx_GIVEREF(__pyx_int_99052414);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_int_99052414);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_6, 2, Py_None);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_6);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_state);
    __pyx_t_5 = 0;
    __pyx_t_6 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.attr is not None or self.attrs is not None or self.default_timezone is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.format is not None or self.help_text is not None or self.initial is not None or self.input_formats is not None or self.label is not None or self.root is not None or self.style is not None or self.timezone is not None or self.validators is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_DateTimeField, (type(self), 0x5e76b7e, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_DateTimeField, (type(self), 0x5e76b7e, None), state
 *     else:
 *         return __pyx_unpickle_DateTimeField, (type(self), 0x5e76b7e, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_DateTimeField__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_pyx_unpickle_DateTimeField); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_6, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_99052414);
    __Pyx_GIVEREF(__pyx_int_99052414);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_int_99052414);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_state);
    __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_6);
    __pyx_t_4 = 0;
    __pyx_t_6 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("drf_turbo.fields.DateTimeField.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_DateTimeField, (type(self), 0x5e76b7e, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_DateTimeField__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_13DateTimeField_13__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_13DateTimeField_13__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_13DateTimeField_12__setstate_cython__(((struct __pyx_obj_9drf_turbo_6fields_DateTimeField *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_13DateTimeField_12__setstate_cython__(struct __pyx_obj_9drf_turbo_6fields_DateTimeField *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_DateTimeField, (type(self), 0x5e76b7e, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_DateTimeField__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields___pyx_unpickle_DateTimeField__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_DateTimeField, (type(self), 0x5e76b7e, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_DateTimeField__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.DateTimeField.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":998
 *     }
 * 
 *     def __init__(self, format=None,input_formats=None,**kwargs):             # <<<<<<<<<<<<<<
 *         if format is None:
 *             self.format = api_settings.DATE_FORMAT
 */

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_9DateField_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_9DateField_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_format = 0;
  PyObject *__pyx_v_input_formats = 0;
  PyObject *__pyx_v_kwargs = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  __pyx_v_kwargs = PyDict_New(); if (unlikely(!__pyx_v_kwargs)) return -1;
  __Pyx_GOTREF(__pyx_v_kwargs);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_format,&__pyx_n_s_input_formats,0};
    PyObject* values[2] = {0,0};
    values[0] = ((PyObject *)Py_None);
    values[1] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_format);
          if (value) { values[0] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_input_formats);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, __pyx_v_kwargs, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 998, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_format = values[0];
    __pyx_v_input_formats = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 0, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 998, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_DECREF(__pyx_v_kwargs); __pyx_v_kwargs = 0;
  __Pyx_AddTraceback("drf_turbo.fields.DateField.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9DateField___init__(((struct __pyx_obj_9drf_turbo_6fields_DateField *)__pyx_v_self), __pyx_v_format, __pyx_v_input_formats, __pyx_v_kwargs);

  /* function exit code */
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_9DateField___init__(struct __pyx_obj_9drf_turbo_6fields_DateField *__pyx_v_self, PyObject *__pyx_v_format, PyObject *__pyx_v_input_formats, PyObject *__pyx_v_kwargs) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "drf_turbo/fields.pyx":999
 * 
 *     def __init__(self, format=None,input_formats=None,**kwargs):
 *         if format is None:             # <<<<<<<<<<<<<<
 *             self.format = api_settings.DATE_FORMAT
 *         else:
 */
  __pyx_t_1 = (__pyx_v_format == Py_None);
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "drf_turbo/fields.pyx":1000
 *     def __init__(self, format=None,input_formats=None,**kwargs):
 *         if format is None:
 *             self.format = api_settings.DATE_FORMAT             # <<<<<<<<<<<<<<
 *         else:
 *             self.format = format
 */
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_api_settings); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1000, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_DATE_FORMAT); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1000, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GIVEREF(__pyx_t_4);
    __Pyx_GOTREF(__pyx_v_self->format);
    __Pyx_DECREF(__pyx_v_self->format);
    __pyx_v_self->format = __pyx_t_4;
    __pyx_t_4 = 0;

    /* "drf_turbo/fields.pyx":999
 * 
 *     def __init__(self, format=None,input_formats=None,**kwargs):
 *         if format is None:             # <<<<<<<<<<<<<<
 *             self.format = api_settings.DATE_FORMAT
 *         else:
 */
    goto __pyx_L3;
  }

  /* "drf_turbo/fields.pyx":1002
 *             self.format = api_settings.DATE_FORMAT
 *         else:
 *             self.format = format             # <<<<<<<<<<<<<<
 * 
 *         if input_formats is None:
 */
  /*else*/ {
    __Pyx_INCREF(__pyx_v_format);
    __Pyx_GIVEREF(__pyx_v_format);
    __Pyx_GOTREF(__pyx_v_self->format);
    __Pyx_DECREF(__pyx_v_self->format);
    __pyx_v_self->format = __pyx_v_format;
  }
  __pyx_L3:;

  /* "drf_turbo/fields.pyx":1004
 *             self.format = format
 * 
 *         if input_formats is None:             # <<<<<<<<<<<<<<
 *             self.input_formats = api_settings.DATE_INPUT_FORMATS
 *         else:
 */
  __pyx_t_2 = (__pyx_v_input_formats == Py_None);
  __pyx_t_1 = (__pyx_t_2 != 0);
  if (__pyx_t_1) {

    /* "drf_turbo/fields.pyx":1005
 * 
 *         if input_formats is None:
 *             self.input_formats = api_settings.DATE_INPUT_FORMATS             # <<<<<<<<<<<<<<
 *         else:
 *             self.input_formats = input_formats
 */
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_api_settings); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1005, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_DATE_INPUT_FORMATS); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1005, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GIVEREF(__pyx_t_3);
    __Pyx_GOTREF(__pyx_v_self->input_formats);
    __Pyx_DECREF(__pyx_v_self->input_formats);
    __pyx_v_self->input_formats = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "drf_turbo/fields.pyx":1004
 *             self.format = format
 * 
 *         if input_formats is None:             # <<<<<<<<<<<<<<
 *             self.input_formats = api_settings.DATE_INPUT_FORMATS
 *         else:
 */
    goto __pyx_L4;
  }

  /* "drf_turbo/fields.pyx":1007
 *             self.input_formats = api_settings.DATE_INPUT_FORMATS
 *         else:
 *             self.input_formats = input_formats             # <<<<<<<<<<<<<<
 * 
 *         super().__init__(**kwargs)
 */
  /*else*/ {
    __Pyx_INCREF(__pyx_v_input_formats);
    __Pyx_GIVEREF(__pyx_v_input_formats);
    __Pyx_GOTREF(__pyx_v_self->input_formats);
    __Pyx_DECREF(__pyx_v_self->input_formats);
    __pyx_v_self->input_formats = __pyx_v_input_formats;
  }
  __pyx_L4:;

  /* "drf_turbo/fields.pyx":1009
 *             self.input_formats = input_formats
 * 
 *         super().__init__(**kwargs)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1009, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_9drf_turbo_6fields_DateField));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_9drf_turbo_6fields_DateField));
  PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)__pyx_ptype_9drf_turbo_6fields_DateField));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_3, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_3, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1009, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_init); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1009, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyDict_Copy(__pyx_v_kwargs); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1009, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_empty_tuple, __pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1009, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "drf_turbo/fields.pyx":998
 *     }
 * 
 *     def __init__(self, format=None,input_formats=None,**kwargs):             # <<<<<<<<<<<<<<
 *         if format is None:
 *             self.format = api_settings.DATE_FORMAT
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("drf_turbo.fields.DateField.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":1012
 * 
 * 
 *     cpdef inline serialize(self, value,dict context):             # <<<<<<<<<<<<<<
 *         if not value:
 *             return None
 */

static PyObject *__pyx_pw_9drf_turbo_6fields_9DateField_3serialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_9DateField_serialize(struct __pyx_obj_9drf_turbo_6fields_DateField *__pyx_v_self, PyObject *__pyx_v_value, CYTHON_UNUSED PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("serialize", 0);

  /* "drf_turbo/fields.pyx":1013
 * 
 *     cpdef inline serialize(self, value,dict context):
 *         if not value:             # <<<<<<<<<<<<<<
 *             return None
 *         if isinstance(value, str):
 */
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_value); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 1013, __pyx_L1_error)
  __pyx_t_2 = ((!__pyx_t_1) != 0);
  if (__pyx_t_2) {

    /* "drf_turbo/fields.pyx":1014
 *     cpdef inline serialize(self, value,dict context):
 *         if not value:
 *             return None             # <<<<<<<<<<<<<<
 *         if isinstance(value, str):
 *             return value
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "drf_turbo/fields.pyx":1013
 * 
 *     cpdef inline serialize(self, value,dict context):
 *         if not value:             # <<<<<<<<<<<<<<
 *             return None
 *         if isinstance(value, str):
 */
  }

  /* "drf_turbo/fields.pyx":1015
 *         if not value:
 *             return None
 *         if isinstance(value, str):             # <<<<<<<<<<<<<<
 *             return value
 *         assert not isinstance(value, datetime.datetime), (
 */
  __pyx_t_2 = PyString_Check(__pyx_v_value); 
  __pyx_t_1 = (__pyx_t_2 != 0);
  if (__pyx_t_1) {

    /* "drf_turbo/fields.pyx":1016
 *             return None
 *         if isinstance(value, str):
 *             return value             # <<<<<<<<<<<<<<
 *         assert not isinstance(value, datetime.datetime), (
 *             'Expected a `date`, but got a `datetime`. Refusing to coerce, '
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_value);
    __pyx_r = __pyx_v_value;
    goto __pyx_L0;

    /* "drf_turbo/fields.pyx":1015
 *         if not value:
 *             return None
 *         if isinstance(value, str):             # <<<<<<<<<<<<<<
 *             return value
 *         assert not isinstance(value, datetime.datetime), (
 */
  }

  /* "drf_turbo/fields.pyx":1017
 *         if isinstance(value, str):
 *             return value
 *         assert not isinstance(value, datetime.datetime), (             # <<<<<<<<<<<<<<
 *             'Expected a `date`, but got a `datetime`. Refusing to coerce, '
 *             'as this may mean losing timezone information. Use a custom '
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_datetime); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1017, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_datetime); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1017, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_1 = PyObject_IsInstance(__pyx_v_value, __pyx_t_4); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(0, 1017, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!((!(__pyx_t_1 != 0)) != 0))) {
      PyErr_SetObject(PyExc_AssertionError, __pyx_kp_s_Expected_a_date_but_got_a_dateti);
      __PYX_ERR(0, 1017, __pyx_L1_error)
    }
  }
  #endif

  /* "drf_turbo/fields.pyx":1023
 *         )
 * 
 *         if self.format.lower() == ISO_8601:             # <<<<<<<<<<<<<<
 *             return value.isoformat()
 * 
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->format, __pyx_n_s_lower); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1023, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_4 = (__pyx_t_5) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_5) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1023, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_ISO_8601); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1023, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = PyObject_RichCompare(__pyx_t_4, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_5); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1023, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_5); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 1023, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (__pyx_t_1) {

    /* "drf_turbo/fields.pyx":1024
 * 
 *         if self.format.lower() == ISO_8601:
 *             return value.isoformat()             # <<<<<<<<<<<<<<
 * 
 *         return value.strftime(self.format)
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_value, __pyx_n_s_isoformat); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1024, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_5 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1024, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;

    /* "drf_turbo/fields.pyx":1023
 *         )
 * 
 *         if self.format.lower() == ISO_8601:             # <<<<<<<<<<<<<<
 *             return value.isoformat()
 * 
 */
  }

  /* "drf_turbo/fields.pyx":1026
 *             return value.isoformat()
 * 
 *         return value.strftime(self.format)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_value, __pyx_n_s_strftime); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1026, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_5 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_self->format) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_self->format);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1026, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_r = __pyx_t_5;
  __pyx_t_5 = 0;
  goto __pyx_L0;

  /* "drf_turbo/fields.pyx":1012
 * 
 * 
 *     cpdef inline serialize(self, value,dict context):             # <<<<<<<<<<<<<<
 *         if not value:
 *             return None
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("drf_turbo.fields.DateField.serialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_9DateField_3serialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_9DateField_3serialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_value = 0;
  PyObject *__pyx_v_context = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("serialize (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_value,&__pyx_n_s_context,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_value)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_context)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("serialize", 1, 2, 2, 1); __PYX_ERR(0, 1012, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "serialize") < 0)) __PYX_ERR(0, 1012, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_value = values[0];
    __pyx_v_context = ((PyObject*)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("serialize", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1012, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.DateField.serialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_context), (&PyDict_Type), 1, "context", 1))) __PYX_ERR(0, 1012, __pyx_L1_error)
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9DateField_2serialize(((struct __pyx_obj_9drf_turbo_6fields_DateField *)__pyx_v_self), __pyx_v_value, __pyx_v_context);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_9DateField_2serialize(struct __pyx_obj_9drf_turbo_6fields_DateField *__pyx_v_self, PyObject *__pyx_v_value, PyObject *__pyx_v_context) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("serialize", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields_9DateField_serialize(__pyx_v_self, __pyx_v_value, __pyx_v_context, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1012, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.DateField.serialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":1029
 * 
 * 
 *     cpdef inline deserialize(self, data,dict context) :             # <<<<<<<<<<<<<<
 * 
 *         if isinstance(data, datetime.datetime):
 */

static PyObject *__pyx_pw_9drf_turbo_6fields_9DateField_5deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_9DateField_deserialize(struct __pyx_obj_9drf_turbo_6fields_DateField *__pyx_v_self, PyObject *__pyx_v_data, CYTHON_UNUSED PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_v_input_format = NULL;
  PyObject *__pyx_v_parsed = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  Py_ssize_t __pyx_t_6;
  PyObject *(*__pyx_t_7)(PyObject *);
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  int __pyx_t_12;
  PyObject *__pyx_t_13 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("deserialize", 0);

  /* "drf_turbo/fields.pyx":1031
 *     cpdef inline deserialize(self, data,dict context) :
 * 
 *         if isinstance(data, datetime.datetime):             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail('datetime')
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_datetime); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1031, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_datetime); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1031, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = PyObject_IsInstance(__pyx_v_data, __pyx_t_2); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(0, 1031, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = (__pyx_t_3 != 0);
  if (unlikely(__pyx_t_4)) {

    /* "drf_turbo/fields.pyx":1032
 * 
 *         if isinstance(data, datetime.datetime):
 *             raise self.raise_if_fail('datetime')             # <<<<<<<<<<<<<<
 * 
 *         if isinstance(data, datetime.date):
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_raise_if_fail); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1032, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
      }
    }
    __pyx_t_2 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_5, __pyx_n_s_datetime) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_n_s_datetime);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1032, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1032, __pyx_L1_error)

    /* "drf_turbo/fields.pyx":1031
 *     cpdef inline deserialize(self, data,dict context) :
 * 
 *         if isinstance(data, datetime.datetime):             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail('datetime')
 * 
 */
  }

  /* "drf_turbo/fields.pyx":1034
 *             raise self.raise_if_fail('datetime')
 * 
 *         if isinstance(data, datetime.date):             # <<<<<<<<<<<<<<
 *             return data
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_datetime); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1034, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_date); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1034, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = PyObject_IsInstance(__pyx_v_data, __pyx_t_1); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(0, 1034, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = (__pyx_t_4 != 0);
  if (__pyx_t_3) {

    /* "drf_turbo/fields.pyx":1035
 * 
 *         if isinstance(data, datetime.date):
 *             return data             # <<<<<<<<<<<<<<
 * 
 *         for input_format in self.input_formats:
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_data);
    __pyx_r = __pyx_v_data;
    goto __pyx_L0;

    /* "drf_turbo/fields.pyx":1034
 *             raise self.raise_if_fail('datetime')
 * 
 *         if isinstance(data, datetime.date):             # <<<<<<<<<<<<<<
 *             return data
 * 
 */
  }

  /* "drf_turbo/fields.pyx":1037
 *             return data
 * 
 *         for input_format in self.input_formats:             # <<<<<<<<<<<<<<
 *             if input_format.lower() == ISO_8601:
 *                 try:
 */
  if (likely(PyList_CheckExact(__pyx_v_self->input_formats)) || PyTuple_CheckExact(__pyx_v_self->input_formats)) {
    __pyx_t_1 = __pyx_v_self->input_formats; __Pyx_INCREF(__pyx_t_1); __pyx_t_6 = 0;
    __pyx_t_7 = NULL;
  } else {
    __pyx_t_6 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_v_self->input_formats); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1037, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_7 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1037, __pyx_L1_error)
  }
  for (;;) {
    if (likely(!__pyx_t_7)) {
      if (likely(PyList_CheckExact(__pyx_t_1))) {
        if (__pyx_t_6 >= PyList_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_2 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_6); __Pyx_INCREF(__pyx_t_2); __pyx_t_6++; if (unlikely(0 < 0)) __PYX_ERR(0, 1037, __pyx_L1_error)
        #else
        __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1037, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        #endif
      } else {
        if (__pyx_t_6 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_2 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_6); __Pyx_INCREF(__pyx_t_2); __pyx_t_6++; if (unlikely(0 < 0)) __PYX_ERR(0, 1037, __pyx_L1_error)
        #else
        __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1037, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        #endif
      }
    } else {
      __pyx_t_2 = __pyx_t_7(__pyx_t_1);
      if (unlikely(!__pyx_t_2)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 1037, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_XDECREF_SET(__pyx_v_input_format, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "drf_turbo/fields.pyx":1038
 * 
 *         for input_format in self.input_formats:
 *             if input_format.lower() == ISO_8601:             # <<<<<<<<<<<<<<
 *                 try:
 *                     parsed = parse_date(data)
 */
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_input_format, __pyx_n_s_lower); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1038, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_8 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
      }
    }
    __pyx_t_2 = (__pyx_t_8) ? __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_8) : __Pyx_PyObject_CallNoArg(__pyx_t_5);
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1038, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_ISO_8601); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1038, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_8 = PyObject_RichCompare(__pyx_t_2, __pyx_t_5, Py_EQ); __Pyx_XGOTREF(__pyx_t_8); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1038, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_8); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 1038, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (__pyx_t_3) {

      /* "drf_turbo/fields.pyx":1039
 *         for input_format in self.input_formats:
 *             if input_format.lower() == ISO_8601:
 *                 try:             # <<<<<<<<<<<<<<
 *                     parsed = parse_date(data)
 *                 except (ValueError, TypeError):
 */
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_9, &__pyx_t_10, &__pyx_t_11);
        __Pyx_XGOTREF(__pyx_t_9);
        __Pyx_XGOTREF(__pyx_t_10);
        __Pyx_XGOTREF(__pyx_t_11);
        /*try:*/ {

          /* "drf_turbo/fields.pyx":1040
 *             if input_format.lower() == ISO_8601:
 *                 try:
 *                     parsed = parse_date(data)             # <<<<<<<<<<<<<<
 *                 except (ValueError, TypeError):
 *                     pass
 */
          __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_parse_date); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1040, __pyx_L8_error)
          __Pyx_GOTREF(__pyx_t_5);
          __pyx_t_2 = NULL;
          if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_5))) {
            __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_5);
            if (likely(__pyx_t_2)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
              __Pyx_INCREF(__pyx_t_2);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_5, function);
            }
          }
          __pyx_t_8 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_2, __pyx_v_data) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_v_data);
          __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
          if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1040, __pyx_L8_error)
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_XDECREF_SET(__pyx_v_parsed, __pyx_t_8);
          __pyx_t_8 = 0;

          /* "drf_turbo/fields.pyx":1039
 *         for input_format in self.input_formats:
 *             if input_format.lower() == ISO_8601:
 *                 try:             # <<<<<<<<<<<<<<
 *                     parsed = parse_date(data)
 *                 except (ValueError, TypeError):
 */
        }

        /* "drf_turbo/fields.pyx":1044
 *                     pass
 *                 else:
 *                     if parsed is not None:             # <<<<<<<<<<<<<<
 *                         return parsed
 *             else:
 */
        /*else:*/ {
          __pyx_t_3 = (__pyx_v_parsed != Py_None);
          __pyx_t_4 = (__pyx_t_3 != 0);
          if (__pyx_t_4) {

            /* "drf_turbo/fields.pyx":1045
 *                 else:
 *                     if parsed is not None:
 *                         return parsed             # <<<<<<<<<<<<<<
 *             else:
 *                 try:
 */
            __Pyx_XDECREF(__pyx_r);
            __Pyx_INCREF(__pyx_v_parsed);
            __pyx_r = __pyx_v_parsed;
            __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
            goto __pyx_L11_except_return;

            /* "drf_turbo/fields.pyx":1044
 *                     pass
 *                 else:
 *                     if parsed is not None:             # <<<<<<<<<<<<<<
 *                         return parsed
 *             else:
 */
          }
        }
        __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
        __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
        __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
        goto __pyx_L15_try_end;
        __pyx_L8_error:;
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;

        /* "drf_turbo/fields.pyx":1041
 *                 try:
 *                     parsed = parse_date(data)
 *                 except (ValueError, TypeError):             # <<<<<<<<<<<<<<
 *                     pass
 *                 else:
 */
        __pyx_t_12 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_ValueError) || __Pyx_PyErr_ExceptionMatches(__pyx_builtin_TypeError);
        if (__pyx_t_12) {
          __Pyx_ErrRestore(0,0,0);
          goto __pyx_L9_exception_handled;
        }
        goto __pyx_L10_except_error;
        __pyx_L10_except_error:;

        /* "drf_turbo/fields.pyx":1039
 *         for input_format in self.input_formats:
 *             if input_format.lower() == ISO_8601:
 *                 try:             # <<<<<<<<<<<<<<
 *                     parsed = parse_date(data)
 *                 except (ValueError, TypeError):
 */
        __Pyx_XGIVEREF(__pyx_t_9);
        __Pyx_XGIVEREF(__pyx_t_10);
        __Pyx_XGIVEREF(__pyx_t_11);
        __Pyx_ExceptionReset(__pyx_t_9, __pyx_t_10, __pyx_t_11);
        goto __pyx_L1_error;
        __pyx_L11_except_return:;
        __Pyx_XGIVEREF(__pyx_t_9);
        __Pyx_XGIVEREF(__pyx_t_10);
        __Pyx_XGIVEREF(__pyx_t_11);
        __Pyx_ExceptionReset(__pyx_t_9, __pyx_t_10, __pyx_t_11);
        goto __pyx_L0;
        __pyx_L9_exception_handled:;
        __Pyx_XGIVEREF(__pyx_t_9);
        __Pyx_XGIVEREF(__pyx_t_10);
        __Pyx_XGIVEREF(__pyx_t_11);
        __Pyx_ExceptionReset(__pyx_t_9, __pyx_t_10, __pyx_t_11);
        __pyx_L15_try_end:;
      }

      /* "drf_turbo/fields.pyx":1038
 * 
 *         for input_format in self.input_formats:
 *             if input_format.lower() == ISO_8601:             # <<<<<<<<<<<<<<
 *                 try:
 *                     parsed = parse_date(data)
 */
      goto __pyx_L7;
    }

    /* "drf_turbo/fields.pyx":1047
 *                         return parsed
 *             else:
 *                 try:             # <<<<<<<<<<<<<<
 *                     parsed = self.datetime_parser(data, input_format)
 *                 except (ValueError, TypeError):
 */
    /*else*/ {
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_11, &__pyx_t_10, &__pyx_t_9);
        __Pyx_XGOTREF(__pyx_t_11);
        __Pyx_XGOTREF(__pyx_t_10);
        __Pyx_XGOTREF(__pyx_t_9);
        /*try:*/ {

          /* "drf_turbo/fields.pyx":1048
 *             else:
 *                 try:
 *                     parsed = self.datetime_parser(data, input_format)             # <<<<<<<<<<<<<<
 *                 except (ValueError, TypeError):
 *                     pass
 */
          __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_datetime_parser); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1048, __pyx_L17_error)
          __Pyx_GOTREF(__pyx_t_5);
          __pyx_t_2 = NULL;
          __pyx_t_12 = 0;
          if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
            __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_5);
            if (likely(__pyx_t_2)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
              __Pyx_INCREF(__pyx_t_2);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_5, function);
              __pyx_t_12 = 1;
            }
          }
          #if CYTHON_FAST_PYCALL
          if (PyFunction_Check(__pyx_t_5)) {
            PyObject *__pyx_temp[3] = {__pyx_t_2, __pyx_v_data, __pyx_v_input_format};
            __pyx_t_8 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_12, 2+__pyx_t_12); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1048, __pyx_L17_error)
            __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
            __Pyx_GOTREF(__pyx_t_8);
          } else
          #endif
          #if CYTHON_FAST_PYCCALL
          if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
            PyObject *__pyx_temp[3] = {__pyx_t_2, __pyx_v_data, __pyx_v_input_format};
            __pyx_t_8 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_12, 2+__pyx_t_12); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1048, __pyx_L17_error)
            __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
            __Pyx_GOTREF(__pyx_t_8);
          } else
          #endif
          {
            __pyx_t_13 = PyTuple_New(2+__pyx_t_12); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1048, __pyx_L17_error)
            __Pyx_GOTREF(__pyx_t_13);
            if (__pyx_t_2) {
              __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_13, 0, __pyx_t_2); __pyx_t_2 = NULL;
            }
            __Pyx_INCREF(__pyx_v_data);
            __Pyx_GIVEREF(__pyx_v_data);
            PyTuple_SET_ITEM(__pyx_t_13, 0+__pyx_t_12, __pyx_v_data);
            __Pyx_INCREF(__pyx_v_input_format);
            __Pyx_GIVEREF(__pyx_v_input_format);
            PyTuple_SET_ITEM(__pyx_t_13, 1+__pyx_t_12, __pyx_v_input_format);
            __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_13, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1048, __pyx_L17_error)
            __Pyx_GOTREF(__pyx_t_8);
            __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
          }
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_XDECREF_SET(__pyx_v_parsed, __pyx_t_8);
          __pyx_t_8 = 0;

          /* "drf_turbo/fields.pyx":1047
 *                         return parsed
 *             else:
 *                 try:             # <<<<<<<<<<<<<<
 *                     parsed = self.datetime_parser(data, input_format)
 *                 except (ValueError, TypeError):
 */
        }

        /* "drf_turbo/fields.pyx":1052
 *                     pass
 *                 else:
 *                     return parsed.date()             # <<<<<<<<<<<<<<
 * 
 *         raise self.raise_if_fail('invalid')
 */
        /*else:*/ {
          __Pyx_XDECREF(__pyx_r);
          __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_parsed, __pyx_n_s_date); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1052, __pyx_L19_except_error)
          __Pyx_GOTREF(__pyx_t_5);
          __pyx_t_13 = NULL;
          if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
            __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_5);
            if (likely(__pyx_t_13)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
              __Pyx_INCREF(__pyx_t_13);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_5, function);
            }
          }
          __pyx_t_8 = (__pyx_t_13) ? __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_13) : __Pyx_PyObject_CallNoArg(__pyx_t_5);
          __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
          if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1052, __pyx_L19_except_error)
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __pyx_r = __pyx_t_8;
          __pyx_t_8 = 0;
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          goto __pyx_L20_except_return;
        }
        __pyx_L17_error:;
        __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;

        /* "drf_turbo/fields.pyx":1049
 *                 try:
 *                     parsed = self.datetime_parser(data, input_format)
 *                 except (ValueError, TypeError):             # <<<<<<<<<<<<<<
 *                     pass
 *                 else:
 */
        __pyx_t_12 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_ValueError) || __Pyx_PyErr_ExceptionMatches(__pyx_builtin_TypeError);
        if (__pyx_t_12) {
          __Pyx_ErrRestore(0,0,0);
          goto __pyx_L18_exception_handled;
        }
        goto __pyx_L19_except_error;
        __pyx_L19_except_error:;

        /* "drf_turbo/fields.pyx":1047
 *                         return parsed
 *             else:
 *                 try:             # <<<<<<<<<<<<<<
 *                     parsed = self.datetime_parser(data, input_format)
 *                 except (ValueError, TypeError):
 */
        __Pyx_XGIVEREF(__pyx_t_11);
        __Pyx_XGIVEREF(__pyx_t_10);
        __Pyx_XGIVEREF(__pyx_t_9);
        __Pyx_ExceptionReset(__pyx_t_11, __pyx_t_10, __pyx_t_9);
        goto __pyx_L1_error;
        __pyx_L20_except_return:;
        __Pyx_XGIVEREF(__pyx_t_11);
        __Pyx_XGIVEREF(__pyx_t_10);
        __Pyx_XGIVEREF(__pyx_t_9);
        __Pyx_ExceptionReset(__pyx_t_11, __pyx_t_10, __pyx_t_9);
        goto __pyx_L0;
        __pyx_L18_exception_handled:;
        __Pyx_XGIVEREF(__pyx_t_11);
        __Pyx_XGIVEREF(__pyx_t_10);
        __Pyx_XGIVEREF(__pyx_t_9);
        __Pyx_ExceptionReset(__pyx_t_11, __pyx_t_10, __pyx_t_9);
      }
    }
    __pyx_L7:;

    /* "drf_turbo/fields.pyx":1037
 *             return data
 * 
 *         for input_format in self.input_formats:             # <<<<<<<<<<<<<<
 *             if input_format.lower() == ISO_8601:
 *                 try:
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "drf_turbo/fields.pyx":1054
 *                     return parsed.date()
 * 
 *         raise self.raise_if_fail('invalid')             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_raise_if_fail); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1054, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_5 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_8))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_8);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_8, function);
    }
  }
  __pyx_t_1 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_8, __pyx_t_5, __pyx_n_s_invalid) : __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_n_s_invalid);
  __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1054, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(0, 1054, __pyx_L1_error)

  /* "drf_turbo/fields.pyx":1029
 * 
 * 
 *     cpdef inline deserialize(self, data,dict context) :             # <<<<<<<<<<<<<<
 * 
 *         if isinstance(data, datetime.datetime):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_AddTraceback("drf_turbo.fields.DateField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_input_format);
  __Pyx_XDECREF(__pyx_v_parsed);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_9DateField_5deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_9DateField_5deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_data = 0;
  PyObject *__pyx_v_context = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("deserialize (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_data,&__pyx_n_s_context,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_data)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_context)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("deserialize", 1, 2, 2, 1); __PYX_ERR(0, 1029, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "deserialize") < 0)) __PYX_ERR(0, 1029, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_data = values[0];
    __pyx_v_context = ((PyObject*)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("deserialize", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1029, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.DateField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_context), (&PyDict_Type), 1, "context", 1))) __PYX_ERR(0, 1029, __pyx_L1_error)
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9DateField_4deserialize(((struct __pyx_obj_9drf_turbo_6fields_DateField *)__pyx_v_self), __pyx_v_data, __pyx_v_context);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_9DateField_4deserialize(struct __pyx_obj_9drf_turbo_6fields_DateField *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("deserialize", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields_9DateField_deserialize(__pyx_v_self, __pyx_v_data, __pyx_v_context, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1029, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.DateField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pxd":117
 * cdef class DateField(Field):
 *     cdef public:
 *         format             # <<<<<<<<<<<<<<
 *         input_formats
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_9DateField_6format_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_9DateField_6format_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9DateField_6format___get__(((struct __pyx_obj_9drf_turbo_6fields_DateField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_9DateField_6format___get__(struct __pyx_obj_9drf_turbo_6fields_DateField *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->format);
  __pyx_r = __pyx_v_self->format;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_9DateField_6format_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_9DateField_6format_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9DateField_6format_2__set__(((struct __pyx_obj_9drf_turbo_6fields_DateField *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_9DateField_6format_2__set__(struct __pyx_obj_9drf_turbo_6fields_DateField *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->format);
  __Pyx_DECREF(__pyx_v_self->format);
  __pyx_v_self->format = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_9DateField_6format_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_9DateField_6format_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9DateField_6format_4__del__(((struct __pyx_obj_9drf_turbo_6fields_DateField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_9DateField_6format_4__del__(struct __pyx_obj_9drf_turbo_6fields_DateField *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->format);
  __Pyx_DECREF(__pyx_v_self->format);
  __pyx_v_self->format = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pxd":118
 *     cdef public:
 *         format
 *         input_formats             # <<<<<<<<<<<<<<
 * 
 * cdef class TimeField(Field):
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_9DateField_13input_formats_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_9DateField_13input_formats_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9DateField_13input_formats___get__(((struct __pyx_obj_9drf_turbo_6fields_DateField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_9DateField_13input_formats___get__(struct __pyx_obj_9drf_turbo_6fields_DateField *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->input_formats);
  __pyx_r = __pyx_v_self->input_formats;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_9DateField_13input_formats_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_9DateField_13input_formats_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9DateField_13input_formats_2__set__(((struct __pyx_obj_9drf_turbo_6fields_DateField *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_9DateField_13input_formats_2__set__(struct __pyx_obj_9drf_turbo_6fields_DateField *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->input_formats);
  __Pyx_DECREF(__pyx_v_self->input_formats);
  __pyx_v_self->input_formats = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_9DateField_13input_formats_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_9DateField_13input_formats_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9DateField_13input_formats_4__del__(((struct __pyx_obj_9drf_turbo_6fields_DateField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_9DateField_13input_formats_4__del__(struct __pyx_obj_9drf_turbo_6fields_DateField *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->input_formats);
  __Pyx_DECREF(__pyx_v_self->input_formats);
  __pyx_v_self->input_formats = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_9DateField_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_9DateField_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9DateField_6__reduce_cython__(((struct __pyx_obj_9drf_turbo_6fields_DateField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_9DateField_6__reduce_cython__(struct __pyx_obj_9drf_turbo_6fields_DateField *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self.allow_null, self.attr, self.attrs, self.call, self.default_value, self.error_messages, self.field_name, self.format, self.help_text, self.initial, self.input_formats, self.label, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.allow_null); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.call); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.read_only); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.required); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.write_only); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = PyTuple_New(18); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.attr);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.attr);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_v_self->__pyx_base.attr);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.attrs);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.attrs);
  PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_self->__pyx_base.attrs);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_6, 3, __pyx_t_2);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.default_value);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.default_value);
  PyTuple_SET_ITEM(__pyx_t_6, 4, __pyx_v_self->__pyx_base.default_value);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.error_messages);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.error_messages);
  PyTuple_SET_ITEM(__pyx_t_6, 5, __pyx_v_self->__pyx_base.error_messages);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.field_name);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.field_name);
  PyTuple_SET_ITEM(__pyx_t_6, 6, __pyx_v_self->__pyx_base.field_name);
  __Pyx_INCREF(__pyx_v_self->format);
  __Pyx_GIVEREF(__pyx_v_self->format);
  PyTuple_SET_ITEM(__pyx_t_6, 7, __pyx_v_self->format);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.help_text);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.help_text);
  PyTuple_SET_ITEM(__pyx_t_6, 8, __pyx_v_self->__pyx_base.help_text);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.initial);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.initial);
  PyTuple_SET_ITEM(__pyx_t_6, 9, __pyx_v_self->__pyx_base.initial);
  __Pyx_INCREF(__pyx_v_self->input_formats);
  __Pyx_GIVEREF(__pyx_v_self->input_formats);
  PyTuple_SET_ITEM(__pyx_t_6, 10, __pyx_v_self->input_formats);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.label);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.label);
  PyTuple_SET_ITEM(__pyx_t_6, 11, __pyx_v_self->__pyx_base.label);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 12, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_6, 13, __pyx_t_4);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.root);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.root);
  PyTuple_SET_ITEM(__pyx_t_6, 14, __pyx_v_self->__pyx_base.root);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.style);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.style);
  PyTuple_SET_ITEM(__pyx_t_6, 15, __pyx_v_self->__pyx_base.style);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.validators);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.validators);
  PyTuple_SET_ITEM(__pyx_t_6, 16, __pyx_v_self->__pyx_base.validators);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_6, 17, __pyx_t_5);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_6);
  __pyx_t_6 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self.allow_null, self.attr, self.attrs, self.call, self.default_value, self.error_messages, self.field_name, self.format, self.help_text, self.initial, self.input_formats, self.label, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_6 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_v__dict = __pyx_t_6;
  __pyx_t_6 = 0;

  /* "(tree fragment)":7
 *     state = (self.allow_null, self.attr, self.attrs, self.call, self.default_value, self.error_messages, self.field_name, self.format, self.help_text, self.initial, self.input_formats, self.label, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_7 = (__pyx_v__dict != Py_None);
  __pyx_t_8 = (__pyx_t_7 != 0);
  if (__pyx_t_8) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_6 = PyTuple_New(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v__dict);
    __pyx_t_5 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_6); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_5));
    __pyx_t_5 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.format is not None or self.help_text is not None or self.initial is not None or self.input_formats is not None or self.label is not None or self.root is not None or self.style is not None or self.validators is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self.allow_null, self.attr, self.attrs, self.call, self.default_value, self.error_messages, self.field_name, self.format, self.help_text, self.initial, self.input_formats, self.label, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.format is not None or self.help_text is not None or self.initial is not None or self.input_formats is not None or self.label is not None or self.root is not None or self.style is not None or self.validators is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_DateField, (type(self), 0xf36a8d7, None), state
 */
  /*else*/ {
    __pyx_t_7 = (__pyx_v_self->__pyx_base.attr != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.attrs != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.default_value != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.error_messages != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.field_name != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->format != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.help_text != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.initial != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->input_formats != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.label != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.root != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.style != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.validators != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    __pyx_t_8 = __pyx_t_9;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_8;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.format is not None or self.help_text is not None or self.initial is not None or self.input_formats is not None or self.label is not None or self.root is not None or self.style is not None or self.validators is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_DateField, (type(self), 0xf36a8d7, None), state
 *     else:
 */
  __pyx_t_8 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_8) {

    /* "(tree fragment)":13
 *         use_setstate = self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.format is not None or self.help_text is not None or self.initial is not None or self.input_formats is not None or self.label is not None or self.root is not None or self.style is not None or self.validators is not None
 *     if use_setstate:
 *         return __pyx_unpickle_DateField, (type(self), 0xf36a8d7, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_DateField, (type(self), 0xf36a8d7, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_pyx_unpickle_DateField); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_6, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_255240407);
    __Pyx_GIVEREF(__pyx_int_255240407);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_int_255240407);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_6, 2, Py_None);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_6);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_state);
    __pyx_t_5 = 0;
    __pyx_t_6 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.format is not None or self.help_text is not None or self.initial is not None or self.input_formats is not None or self.label is not None or self.root is not None or self.style is not None or self.validators is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_DateField, (type(self), 0xf36a8d7, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_DateField, (type(self), 0xf36a8d7, None), state
 *     else:
 *         return __pyx_unpickle_DateField, (type(self), 0xf36a8d7, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_DateField__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_pyx_unpickle_DateField); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_6, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_255240407);
    __Pyx_GIVEREF(__pyx_int_255240407);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_int_255240407);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_state);
    __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_6);
    __pyx_t_4 = 0;
    __pyx_t_6 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("drf_turbo.fields.DateField.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_DateField, (type(self), 0xf36a8d7, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_DateField__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_9DateField_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_9DateField_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9DateField_8__setstate_cython__(((struct __pyx_obj_9drf_turbo_6fields_DateField *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_9DateField_8__setstate_cython__(struct __pyx_obj_9drf_turbo_6fields_DateField *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_DateField, (type(self), 0xf36a8d7, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_DateField__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields___pyx_unpickle_DateField__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_DateField, (type(self), 0xf36a8d7, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_DateField__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.DateField.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":1072
 *     datetime_parser = datetime.datetime.strptime
 * 
 *     def __init__(self, format=NO_DEFAULT,input_formats=None, **kwargs):             # <<<<<<<<<<<<<<
 *         if format is NO_DEFAULT:
 *             self.format = api_settings.DATETIME_FORMAT
 */

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_9TimeField_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_9TimeField_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_format = 0;
  PyObject *__pyx_v_input_formats = 0;
  PyObject *__pyx_v_kwargs = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  __pyx_v_kwargs = PyDict_New(); if (unlikely(!__pyx_v_kwargs)) return -1;
  __Pyx_GOTREF(__pyx_v_kwargs);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_format,&__pyx_n_s_input_formats,0};
    PyObject* values[2] = {0,0};
    values[0] = __pyx_k__23;
    values[1] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_format);
          if (value) { values[0] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_input_formats);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, __pyx_v_kwargs, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 1072, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_format = values[0];
    __pyx_v_input_formats = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 0, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1072, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_DECREF(__pyx_v_kwargs); __pyx_v_kwargs = 0;
  __Pyx_AddTraceback("drf_turbo.fields.TimeField.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9TimeField___init__(((struct __pyx_obj_9drf_turbo_6fields_TimeField *)__pyx_v_self), __pyx_v_format, __pyx_v_input_formats, __pyx_v_kwargs);

  /* function exit code */
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_9TimeField___init__(struct __pyx_obj_9drf_turbo_6fields_TimeField *__pyx_v_self, PyObject *__pyx_v_format, PyObject *__pyx_v_input_formats, PyObject *__pyx_v_kwargs) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "drf_turbo/fields.pyx":1073
 * 
 *     def __init__(self, format=NO_DEFAULT,input_formats=None, **kwargs):
 *         if format is NO_DEFAULT:             # <<<<<<<<<<<<<<
 *             self.format = api_settings.DATETIME_FORMAT
 *         else:
 */
  __pyx_t_1 = (__pyx_v_format == __pyx_v_9drf_turbo_6fields_NO_DEFAULT);
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "drf_turbo/fields.pyx":1074
 *     def __init__(self, format=NO_DEFAULT,input_formats=None, **kwargs):
 *         if format is NO_DEFAULT:
 *             self.format = api_settings.DATETIME_FORMAT             # <<<<<<<<<<<<<<
 *         else:
 *             self.format = format
 */
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_api_settings); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1074, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_DATETIME_FORMAT); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1074, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GIVEREF(__pyx_t_4);
    __Pyx_GOTREF(__pyx_v_self->format);
    __Pyx_DECREF(__pyx_v_self->format);
    __pyx_v_self->format = __pyx_t_4;
    __pyx_t_4 = 0;

    /* "drf_turbo/fields.pyx":1073
 * 
 *     def __init__(self, format=NO_DEFAULT,input_formats=None, **kwargs):
 *         if format is NO_DEFAULT:             # <<<<<<<<<<<<<<
 *             self.format = api_settings.DATETIME_FORMAT
 *         else:
 */
    goto __pyx_L3;
  }

  /* "drf_turbo/fields.pyx":1076
 *             self.format = api_settings.DATETIME_FORMAT
 *         else:
 *             self.format = format             # <<<<<<<<<<<<<<
 *         if input_formats is None:
 *             self.input_formats = api_settings.TIME_INPUT_FORMATS
 */
  /*else*/ {
    __Pyx_INCREF(__pyx_v_format);
    __Pyx_GIVEREF(__pyx_v_format);
    __Pyx_GOTREF(__pyx_v_self->format);
    __Pyx_DECREF(__pyx_v_self->format);
    __pyx_v_self->format = __pyx_v_format;
  }
  __pyx_L3:;

  /* "drf_turbo/fields.pyx":1077
 *         else:
 *             self.format = format
 *         if input_formats is None:             # <<<<<<<<<<<<<<
 *             self.input_formats = api_settings.TIME_INPUT_FORMATS
 *         else:
 */
  __pyx_t_2 = (__pyx_v_input_formats == Py_None);
  __pyx_t_1 = (__pyx_t_2 != 0);
  if (__pyx_t_1) {

    /* "drf_turbo/fields.pyx":1078
 *             self.format = format
 *         if input_formats is None:
 *             self.input_formats = api_settings.TIME_INPUT_FORMATS             # <<<<<<<<<<<<<<
 *         else:
 *             self.input_formats = input_formats
 */
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_api_settings); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1078, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_TIME_INPUT_FORMATS); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1078, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GIVEREF(__pyx_t_3);
    __Pyx_GOTREF(__pyx_v_self->input_formats);
    __Pyx_DECREF(__pyx_v_self->input_formats);
    __pyx_v_self->input_formats = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "drf_turbo/fields.pyx":1077
 *         else:
 *             self.format = format
 *         if input_formats is None:             # <<<<<<<<<<<<<<
 *             self.input_formats = api_settings.TIME_INPUT_FORMATS
 *         else:
 */
    goto __pyx_L4;
  }

  /* "drf_turbo/fields.pyx":1080
 *             self.input_formats = api_settings.TIME_INPUT_FORMATS
 *         else:
 *             self.input_formats = input_formats             # <<<<<<<<<<<<<<
 * 
 *         super().__init__(**kwargs)
 */
  /*else*/ {
    __Pyx_INCREF(__pyx_v_input_formats);
    __Pyx_GIVEREF(__pyx_v_input_formats);
    __Pyx_GOTREF(__pyx_v_self->input_formats);
    __Pyx_DECREF(__pyx_v_self->input_formats);
    __pyx_v_self->input_formats = __pyx_v_input_formats;
  }
  __pyx_L4:;

  /* "drf_turbo/fields.pyx":1082
 *             self.input_formats = input_formats
 * 
 *         super().__init__(**kwargs)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1082, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_9drf_turbo_6fields_TimeField));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_9drf_turbo_6fields_TimeField));
  PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)__pyx_ptype_9drf_turbo_6fields_TimeField));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_3, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_3, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1082, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_init); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1082, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyDict_Copy(__pyx_v_kwargs); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1082, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_empty_tuple, __pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1082, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "drf_turbo/fields.pyx":1072
 *     datetime_parser = datetime.datetime.strptime
 * 
 *     def __init__(self, format=NO_DEFAULT,input_formats=None, **kwargs):             # <<<<<<<<<<<<<<
 *         if format is NO_DEFAULT:
 *             self.format = api_settings.DATETIME_FORMAT
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("drf_turbo.fields.TimeField.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":1085
 * 
 * 
 *     cpdef inline serialize(self, value,dict context):             # <<<<<<<<<<<<<<
 *         if value in (None, ''):
 *             return None
 */

static PyObject *__pyx_pw_9drf_turbo_6fields_9TimeField_3serialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_9TimeField_serialize(struct __pyx_obj_9drf_turbo_6fields_TimeField *__pyx_v_self, PyObject *__pyx_v_value, CYTHON_UNUSED PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("serialize", 0);

  /* "drf_turbo/fields.pyx":1086
 * 
 *     cpdef inline serialize(self, value,dict context):
 *         if value in (None, ''):             # <<<<<<<<<<<<<<
 *             return None
 * 
 */
  __Pyx_INCREF(__pyx_v_value);
  __pyx_t_1 = __pyx_v_value;
  __pyx_t_3 = PyObject_RichCompare(__pyx_t_1, Py_None, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1086, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 1086, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (!__pyx_t_4) {
  } else {
    __pyx_t_2 = __pyx_t_4;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_4 = (__Pyx_PyString_Equals(__pyx_t_1, __pyx_kp_s__9, Py_EQ)); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 1086, __pyx_L1_error)
  __pyx_t_2 = __pyx_t_4;
  __pyx_L4_bool_binop_done:;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_4 = (__pyx_t_2 != 0);
  if (__pyx_t_4) {

    /* "drf_turbo/fields.pyx":1087
 *     cpdef inline serialize(self, value,dict context):
 *         if value in (None, ''):
 *             return None             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "drf_turbo/fields.pyx":1086
 * 
 *     cpdef inline serialize(self, value,dict context):
 *         if value in (None, ''):             # <<<<<<<<<<<<<<
 *             return None
 * 
 */
  }

  /* "drf_turbo/fields.pyx":1090
 * 
 * 
 *         if self.format is None or isinstance(value, str):             # <<<<<<<<<<<<<<
 *             return value
 * 
 */
  __pyx_t_2 = (__pyx_v_self->format == Py_None);
  __pyx_t_5 = (__pyx_t_2 != 0);
  if (!__pyx_t_5) {
  } else {
    __pyx_t_4 = __pyx_t_5;
    goto __pyx_L7_bool_binop_done;
  }
  __pyx_t_5 = PyString_Check(__pyx_v_value); 
  __pyx_t_2 = (__pyx_t_5 != 0);
  __pyx_t_4 = __pyx_t_2;
  __pyx_L7_bool_binop_done:;
  if (__pyx_t_4) {

    /* "drf_turbo/fields.pyx":1091
 * 
 *         if self.format is None or isinstance(value, str):
 *             return value             # <<<<<<<<<<<<<<
 * 
 *         assert not isinstance(value, datetime.datetime), (
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_value);
    __pyx_r = __pyx_v_value;
    goto __pyx_L0;

    /* "drf_turbo/fields.pyx":1090
 * 
 * 
 *         if self.format is None or isinstance(value, str):             # <<<<<<<<<<<<<<
 *             return value
 * 
 */
  }

  /* "drf_turbo/fields.pyx":1093
 *             return value
 * 
 *         assert not isinstance(value, datetime.datetime), (             # <<<<<<<<<<<<<<
 *             'Expected a `time`, but got a `datetime`. Refusing to coerce, '
 *             'as this may mean losing timezone information. Use a custom '
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_datetime); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1093, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_datetime); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1093, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_4 = PyObject_IsInstance(__pyx_v_value, __pyx_t_3); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(0, 1093, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!((!(__pyx_t_4 != 0)) != 0))) {
      PyErr_SetObject(PyExc_AssertionError, __pyx_kp_s_Expected_a_time_but_got_a_dateti);
      __PYX_ERR(0, 1093, __pyx_L1_error)
    }
  }
  #endif

  /* "drf_turbo/fields.pyx":1099
 *         )
 * 
 *         if self.format.lower() == ISO_8601:             # <<<<<<<<<<<<<<
 *             return value.isoformat()
 *         return value.strftime(self.format)
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->format, __pyx_n_s_lower); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1099, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
    }
  }
  __pyx_t_3 = (__pyx_t_6) ? __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_6) : __Pyx_PyObject_CallNoArg(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1099, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_ISO_8601); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1099, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_6 = PyObject_RichCompare(__pyx_t_3, __pyx_t_1, Py_EQ); __Pyx_XGOTREF(__pyx_t_6); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1099, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_6); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 1099, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (__pyx_t_4) {

    /* "drf_turbo/fields.pyx":1100
 * 
 *         if self.format.lower() == ISO_8601:
 *             return value.isoformat()             # <<<<<<<<<<<<<<
 *         return value.strftime(self.format)
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_value, __pyx_n_s_isoformat); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1100, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
      }
    }
    __pyx_t_6 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_1);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1100, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_r = __pyx_t_6;
    __pyx_t_6 = 0;
    goto __pyx_L0;

    /* "drf_turbo/fields.pyx":1099
 *         )
 * 
 *         if self.format.lower() == ISO_8601:             # <<<<<<<<<<<<<<
 *             return value.isoformat()
 *         return value.strftime(self.format)
 */
  }

  /* "drf_turbo/fields.pyx":1101
 *         if self.format.lower() == ISO_8601:
 *             return value.isoformat()
 *         return value.strftime(self.format)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_value, __pyx_n_s_strftime); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1101, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
    }
  }
  __pyx_t_6 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_3, __pyx_v_self->format) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_v_self->format);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1101, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_6;
  __pyx_t_6 = 0;
  goto __pyx_L0;

  /* "drf_turbo/fields.pyx":1085
 * 
 * 
 *     cpdef inline serialize(self, value,dict context):             # <<<<<<<<<<<<<<
 *         if value in (None, ''):
 *             return None
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("drf_turbo.fields.TimeField.serialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_9TimeField_3serialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_9TimeField_3serialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_value = 0;
  PyObject *__pyx_v_context = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("serialize (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_value,&__pyx_n_s_context,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_value)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_context)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("serialize", 1, 2, 2, 1); __PYX_ERR(0, 1085, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "serialize") < 0)) __PYX_ERR(0, 1085, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_value = values[0];
    __pyx_v_context = ((PyObject*)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("serialize", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1085, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.TimeField.serialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_context), (&PyDict_Type), 1, "context", 1))) __PYX_ERR(0, 1085, __pyx_L1_error)
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9TimeField_2serialize(((struct __pyx_obj_9drf_turbo_6fields_TimeField *)__pyx_v_self), __pyx_v_value, __pyx_v_context);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_9TimeField_2serialize(struct __pyx_obj_9drf_turbo_6fields_TimeField *__pyx_v_self, PyObject *__pyx_v_value, PyObject *__pyx_v_context) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("serialize", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields_9TimeField_serialize(__pyx_v_self, __pyx_v_value, __pyx_v_context, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1085, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.TimeField.serialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":1105
 * 
 * 
 *     cpdef inline deserialize(self, data, dict context):             # <<<<<<<<<<<<<<
 * 
 *         if isinstance(data, datetime.time):
 */

static PyObject *__pyx_pw_9drf_turbo_6fields_9TimeField_5deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_9TimeField_deserialize(struct __pyx_obj_9drf_turbo_6fields_TimeField *__pyx_v_self, PyObject *__pyx_v_data, CYTHON_UNUSED PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_v_input_format = NULL;
  PyObject *__pyx_v_parsed = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  PyObject *(*__pyx_t_6)(PyObject *);
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  int __pyx_t_12;
  PyObject *__pyx_t_13 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("deserialize", 0);

  /* "drf_turbo/fields.pyx":1107
 *     cpdef inline deserialize(self, data, dict context):
 * 
 *         if isinstance(data, datetime.time):             # <<<<<<<<<<<<<<
 *             return data
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_datetime); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1107, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_time); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1107, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = PyObject_IsInstance(__pyx_v_data, __pyx_t_2); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(0, 1107, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = (__pyx_t_3 != 0);
  if (__pyx_t_4) {

    /* "drf_turbo/fields.pyx":1108
 * 
 *         if isinstance(data, datetime.time):
 *             return data             # <<<<<<<<<<<<<<
 * 
 *         for input_format in self.input_formats:
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_data);
    __pyx_r = __pyx_v_data;
    goto __pyx_L0;

    /* "drf_turbo/fields.pyx":1107
 *     cpdef inline deserialize(self, data, dict context):
 * 
 *         if isinstance(data, datetime.time):             # <<<<<<<<<<<<<<
 *             return data
 * 
 */
  }

  /* "drf_turbo/fields.pyx":1110
 *             return data
 * 
 *         for input_format in self.input_formats:             # <<<<<<<<<<<<<<
 *             if input_format.lower() == ISO_8601:
 *                 try:
 */
  if (likely(PyList_CheckExact(__pyx_v_self->input_formats)) || PyTuple_CheckExact(__pyx_v_self->input_formats)) {
    __pyx_t_2 = __pyx_v_self->input_formats; __Pyx_INCREF(__pyx_t_2); __pyx_t_5 = 0;
    __pyx_t_6 = NULL;
  } else {
    __pyx_t_5 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_v_self->input_formats); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1110, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_6 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1110, __pyx_L1_error)
  }
  for (;;) {
    if (likely(!__pyx_t_6)) {
      if (likely(PyList_CheckExact(__pyx_t_2))) {
        if (__pyx_t_5 >= PyList_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_5); __Pyx_INCREF(__pyx_t_1); __pyx_t_5++; if (unlikely(0 < 0)) __PYX_ERR(0, 1110, __pyx_L1_error)
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_2, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1110, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      } else {
        if (__pyx_t_5 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_5); __Pyx_INCREF(__pyx_t_1); __pyx_t_5++; if (unlikely(0 < 0)) __PYX_ERR(0, 1110, __pyx_L1_error)
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_2, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1110, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      }
    } else {
      __pyx_t_1 = __pyx_t_6(__pyx_t_2);
      if (unlikely(!__pyx_t_1)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 1110, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_XDECREF_SET(__pyx_v_input_format, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "drf_turbo/fields.pyx":1111
 * 
 *         for input_format in self.input_formats:
 *             if input_format.lower() == ISO_8601:             # <<<<<<<<<<<<<<
 *                 try:
 *                     parsed = parse_time(data)
 */
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_input_format, __pyx_n_s_lower); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1111, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
      }
    }
    __pyx_t_1 = (__pyx_t_8) ? __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_8) : __Pyx_PyObject_CallNoArg(__pyx_t_7);
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1111, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_n_s_ISO_8601); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1111, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = PyObject_RichCompare(__pyx_t_1, __pyx_t_7, Py_EQ); __Pyx_XGOTREF(__pyx_t_8); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1111, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_8); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 1111, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (__pyx_t_4) {

      /* "drf_turbo/fields.pyx":1112
 *         for input_format in self.input_formats:
 *             if input_format.lower() == ISO_8601:
 *                 try:             # <<<<<<<<<<<<<<
 *                     parsed = parse_time(data)
 *                 except (ValueError, TypeError):
 */
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_9, &__pyx_t_10, &__pyx_t_11);
        __Pyx_XGOTREF(__pyx_t_9);
        __Pyx_XGOTREF(__pyx_t_10);
        __Pyx_XGOTREF(__pyx_t_11);
        /*try:*/ {

          /* "drf_turbo/fields.pyx":1113
 *             if input_format.lower() == ISO_8601:
 *                 try:
 *                     parsed = parse_time(data)             # <<<<<<<<<<<<<<
 *                 except (ValueError, TypeError):
 *                     pass
 */
          __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_n_s_parse_time); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1113, __pyx_L7_error)
          __Pyx_GOTREF(__pyx_t_7);
          __pyx_t_1 = NULL;
          if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_7))) {
            __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_7);
            if (likely(__pyx_t_1)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
              __Pyx_INCREF(__pyx_t_1);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_7, function);
            }
          }
          __pyx_t_8 = (__pyx_t_1) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_1, __pyx_v_data) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_v_data);
          __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
          if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1113, __pyx_L7_error)
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_XDECREF_SET(__pyx_v_parsed, __pyx_t_8);
          __pyx_t_8 = 0;

          /* "drf_turbo/fields.pyx":1112
 *         for input_format in self.input_formats:
 *             if input_format.lower() == ISO_8601:
 *                 try:             # <<<<<<<<<<<<<<
 *                     parsed = parse_time(data)
 *                 except (ValueError, TypeError):
 */
        }

        /* "drf_turbo/fields.pyx":1117
 *                     pass
 *                 else:
 *                     if parsed is not None:             # <<<<<<<<<<<<<<
 *                         return parsed
 *             else:
 */
        /*else:*/ {
          __pyx_t_4 = (__pyx_v_parsed != Py_None);
          __pyx_t_3 = (__pyx_t_4 != 0);
          if (__pyx_t_3) {

            /* "drf_turbo/fields.pyx":1118
 *                 else:
 *                     if parsed is not None:
 *                         return parsed             # <<<<<<<<<<<<<<
 *             else:
 *                 try:
 */
            __Pyx_XDECREF(__pyx_r);
            __Pyx_INCREF(__pyx_v_parsed);
            __pyx_r = __pyx_v_parsed;
            __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
            goto __pyx_L10_except_return;

            /* "drf_turbo/fields.pyx":1117
 *                     pass
 *                 else:
 *                     if parsed is not None:             # <<<<<<<<<<<<<<
 *                         return parsed
 *             else:
 */
          }
        }
        __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
        __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
        __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
        goto __pyx_L14_try_end;
        __pyx_L7_error:;
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;

        /* "drf_turbo/fields.pyx":1114
 *                 try:
 *                     parsed = parse_time(data)
 *                 except (ValueError, TypeError):             # <<<<<<<<<<<<<<
 *                     pass
 *                 else:
 */
        __pyx_t_12 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_ValueError) || __Pyx_PyErr_ExceptionMatches(__pyx_builtin_TypeError);
        if (__pyx_t_12) {
          __Pyx_ErrRestore(0,0,0);
          goto __pyx_L8_exception_handled;
        }
        goto __pyx_L9_except_error;
        __pyx_L9_except_error:;

        /* "drf_turbo/fields.pyx":1112
 *         for input_format in self.input_formats:
 *             if input_format.lower() == ISO_8601:
 *                 try:             # <<<<<<<<<<<<<<
 *                     parsed = parse_time(data)
 *                 except (ValueError, TypeError):
 */
        __Pyx_XGIVEREF(__pyx_t_9);
        __Pyx_XGIVEREF(__pyx_t_10);
        __Pyx_XGIVEREF(__pyx_t_11);
        __Pyx_ExceptionReset(__pyx_t_9, __pyx_t_10, __pyx_t_11);
        goto __pyx_L1_error;
        __pyx_L10_except_return:;
        __Pyx_XGIVEREF(__pyx_t_9);
        __Pyx_XGIVEREF(__pyx_t_10);
        __Pyx_XGIVEREF(__pyx_t_11);
        __Pyx_ExceptionReset(__pyx_t_9, __pyx_t_10, __pyx_t_11);
        goto __pyx_L0;
        __pyx_L8_exception_handled:;
        __Pyx_XGIVEREF(__pyx_t_9);
        __Pyx_XGIVEREF(__pyx_t_10);
        __Pyx_XGIVEREF(__pyx_t_11);
        __Pyx_ExceptionReset(__pyx_t_9, __pyx_t_10, __pyx_t_11);
        __pyx_L14_try_end:;
      }

      /* "drf_turbo/fields.pyx":1111
 * 
 *         for input_format in self.input_formats:
 *             if input_format.lower() == ISO_8601:             # <<<<<<<<<<<<<<
 *                 try:
 *                     parsed = parse_time(data)
 */
      goto __pyx_L6;
    }

    /* "drf_turbo/fields.pyx":1120
 *                         return parsed
 *             else:
 *                 try:             # <<<<<<<<<<<<<<
 *                     parsed = self.datetime_parser(data, input_format)
 *                 except (ValueError, TypeError):
 */
    /*else*/ {
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_11, &__pyx_t_10, &__pyx_t_9);
        __Pyx_XGOTREF(__pyx_t_11);
        __Pyx_XGOTREF(__pyx_t_10);
        __Pyx_XGOTREF(__pyx_t_9);
        /*try:*/ {

          /* "drf_turbo/fields.pyx":1121
 *             else:
 *                 try:
 *                     parsed = self.datetime_parser(data, input_format)             # <<<<<<<<<<<<<<
 *                 except (ValueError, TypeError):
 *                     pass
 */
          __pyx_t_7 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_datetime_parser); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1121, __pyx_L16_error)
          __Pyx_GOTREF(__pyx_t_7);
          __pyx_t_1 = NULL;
          __pyx_t_12 = 0;
          if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
            __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_7);
            if (likely(__pyx_t_1)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
              __Pyx_INCREF(__pyx_t_1);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_7, function);
              __pyx_t_12 = 1;
            }
          }
          #if CYTHON_FAST_PYCALL
          if (PyFunction_Check(__pyx_t_7)) {
            PyObject *__pyx_temp[3] = {__pyx_t_1, __pyx_v_data, __pyx_v_input_format};
            __pyx_t_8 = __Pyx_PyFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_12, 2+__pyx_t_12); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1121, __pyx_L16_error)
            __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
            __Pyx_GOTREF(__pyx_t_8);
          } else
          #endif
          #if CYTHON_FAST_PYCCALL
          if (__Pyx_PyFastCFunction_Check(__pyx_t_7)) {
            PyObject *__pyx_temp[3] = {__pyx_t_1, __pyx_v_data, __pyx_v_input_format};
            __pyx_t_8 = __Pyx_PyCFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_12, 2+__pyx_t_12); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1121, __pyx_L16_error)
            __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
            __Pyx_GOTREF(__pyx_t_8);
          } else
          #endif
          {
            __pyx_t_13 = PyTuple_New(2+__pyx_t_12); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1121, __pyx_L16_error)
            __Pyx_GOTREF(__pyx_t_13);
            if (__pyx_t_1) {
              __Pyx_GIVEREF(__pyx_t_1); PyTuple_SET_ITEM(__pyx_t_13, 0, __pyx_t_1); __pyx_t_1 = NULL;
            }
            __Pyx_INCREF(__pyx_v_data);
            __Pyx_GIVEREF(__pyx_v_data);
            PyTuple_SET_ITEM(__pyx_t_13, 0+__pyx_t_12, __pyx_v_data);
            __Pyx_INCREF(__pyx_v_input_format);
            __Pyx_GIVEREF(__pyx_v_input_format);
            PyTuple_SET_ITEM(__pyx_t_13, 1+__pyx_t_12, __pyx_v_input_format);
            __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_13, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1121, __pyx_L16_error)
            __Pyx_GOTREF(__pyx_t_8);
            __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
          }
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_XDECREF_SET(__pyx_v_parsed, __pyx_t_8);
          __pyx_t_8 = 0;

          /* "drf_turbo/fields.pyx":1120
 *                         return parsed
 *             else:
 *                 try:             # <<<<<<<<<<<<<<
 *                     parsed = self.datetime_parser(data, input_format)
 *                 except (ValueError, TypeError):
 */
        }

        /* "drf_turbo/fields.pyx":1125
 *                     pass
 *                 else:
 *                     return parsed.time()             # <<<<<<<<<<<<<<
 * 
 *         raise self.raise_if_fail('invalid')
 */
        /*else:*/ {
          __Pyx_XDECREF(__pyx_r);
          __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_parsed, __pyx_n_s_time); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1125, __pyx_L18_except_error)
          __Pyx_GOTREF(__pyx_t_7);
          __pyx_t_13 = NULL;
          if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
            __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_7);
            if (likely(__pyx_t_13)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
              __Pyx_INCREF(__pyx_t_13);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_7, function);
            }
          }
          __pyx_t_8 = (__pyx_t_13) ? __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_13) : __Pyx_PyObject_CallNoArg(__pyx_t_7);
          __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
          if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1125, __pyx_L18_except_error)
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
          __pyx_r = __pyx_t_8;
          __pyx_t_8 = 0;
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          goto __pyx_L19_except_return;
        }
        __pyx_L16_error:;
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;

        /* "drf_turbo/fields.pyx":1122
 *                 try:
 *                     parsed = self.datetime_parser(data, input_format)
 *                 except (ValueError, TypeError):             # <<<<<<<<<<<<<<
 *                     pass
 *                 else:
 */
        __pyx_t_12 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_ValueError) || __Pyx_PyErr_ExceptionMatches(__pyx_builtin_TypeError);
        if (__pyx_t_12) {
          __Pyx_ErrRestore(0,0,0);
          goto __pyx_L17_exception_handled;
        }
        goto __pyx_L18_except_error;
        __pyx_L18_except_error:;

        /* "drf_turbo/fields.pyx":1120
 *                         return parsed
 *             else:
 *                 try:             # <<<<<<<<<<<<<<
 *                     parsed = self.datetime_parser(data, input_format)
 *                 except (ValueError, TypeError):
 */
        __Pyx_XGIVEREF(__pyx_t_11);
        __Pyx_XGIVEREF(__pyx_t_10);
        __Pyx_XGIVEREF(__pyx_t_9);
        __Pyx_ExceptionReset(__pyx_t_11, __pyx_t_10, __pyx_t_9);
        goto __pyx_L1_error;
        __pyx_L19_except_return:;
        __Pyx_XGIVEREF(__pyx_t_11);
        __Pyx_XGIVEREF(__pyx_t_10);
        __Pyx_XGIVEREF(__pyx_t_9);
        __Pyx_ExceptionReset(__pyx_t_11, __pyx_t_10, __pyx_t_9);
        goto __pyx_L0;
        __pyx_L17_exception_handled:;
        __Pyx_XGIVEREF(__pyx_t_11);
        __Pyx_XGIVEREF(__pyx_t_10);
        __Pyx_XGIVEREF(__pyx_t_9);
        __Pyx_ExceptionReset(__pyx_t_11, __pyx_t_10, __pyx_t_9);
      }
    }
    __pyx_L6:;

    /* "drf_turbo/fields.pyx":1110
 *             return data
 * 
 *         for input_format in self.input_formats:             # <<<<<<<<<<<<<<
 *             if input_format.lower() == ISO_8601:
 *                 try:
 */
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "drf_turbo/fields.pyx":1127
 *                     return parsed.time()
 * 
 *         raise self.raise_if_fail('invalid')             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_raise_if_fail); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1127, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_7 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_8))) {
    __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_8);
    if (likely(__pyx_t_7)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
      __Pyx_INCREF(__pyx_t_7);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_8, function);
    }
  }
  __pyx_t_2 = (__pyx_t_7) ? __Pyx_PyObject_Call2Args(__pyx_t_8, __pyx_t_7, __pyx_n_s_invalid) : __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_n_s_invalid);
  __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1127, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __Pyx_Raise(__pyx_t_2, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __PYX_ERR(0, 1127, __pyx_L1_error)

  /* "drf_turbo/fields.pyx":1105
 * 
 * 
 *     cpdef inline deserialize(self, data, dict context):             # <<<<<<<<<<<<<<
 * 
 *         if isinstance(data, datetime.time):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_AddTraceback("drf_turbo.fields.TimeField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_input_format);
  __Pyx_XDECREF(__pyx_v_parsed);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_9TimeField_5deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_9TimeField_5deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_data = 0;
  PyObject *__pyx_v_context = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("deserialize (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_data,&__pyx_n_s_context,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_data)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_context)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("deserialize", 1, 2, 2, 1); __PYX_ERR(0, 1105, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "deserialize") < 0)) __PYX_ERR(0, 1105, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_data = values[0];
    __pyx_v_context = ((PyObject*)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("deserialize", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1105, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.TimeField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_context), (&PyDict_Type), 1, "context", 1))) __PYX_ERR(0, 1105, __pyx_L1_error)
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9TimeField_4deserialize(((struct __pyx_obj_9drf_turbo_6fields_TimeField *)__pyx_v_self), __pyx_v_data, __pyx_v_context);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_9TimeField_4deserialize(struct __pyx_obj_9drf_turbo_6fields_TimeField *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("deserialize", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields_9TimeField_deserialize(__pyx_v_self, __pyx_v_data, __pyx_v_context, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1105, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.TimeField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pxd":122
 * cdef class TimeField(Field):
 *     cdef public:
 *         format             # <<<<<<<<<<<<<<
 *         input_formats
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_9TimeField_6format_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_9TimeField_6format_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9TimeField_6format___get__(((struct __pyx_obj_9drf_turbo_6fields_TimeField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_9TimeField_6format___get__(struct __pyx_obj_9drf_turbo_6fields_TimeField *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->format);
  __pyx_r = __pyx_v_self->format;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_9TimeField_6format_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_9TimeField_6format_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9TimeField_6format_2__set__(((struct __pyx_obj_9drf_turbo_6fields_TimeField *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_9TimeField_6format_2__set__(struct __pyx_obj_9drf_turbo_6fields_TimeField *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->format);
  __Pyx_DECREF(__pyx_v_self->format);
  __pyx_v_self->format = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_9TimeField_6format_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_9TimeField_6format_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9TimeField_6format_4__del__(((struct __pyx_obj_9drf_turbo_6fields_TimeField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_9TimeField_6format_4__del__(struct __pyx_obj_9drf_turbo_6fields_TimeField *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->format);
  __Pyx_DECREF(__pyx_v_self->format);
  __pyx_v_self->format = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pxd":123
 *     cdef public:
 *         format
 *         input_formats             # <<<<<<<<<<<<<<
 * 
 * cdef class FileField(Field):
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_9TimeField_13input_formats_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_9TimeField_13input_formats_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9TimeField_13input_formats___get__(((struct __pyx_obj_9drf_turbo_6fields_TimeField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_9TimeField_13input_formats___get__(struct __pyx_obj_9drf_turbo_6fields_TimeField *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->input_formats);
  __pyx_r = __pyx_v_self->input_formats;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_9TimeField_13input_formats_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_9TimeField_13input_formats_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9TimeField_13input_formats_2__set__(((struct __pyx_obj_9drf_turbo_6fields_TimeField *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_9TimeField_13input_formats_2__set__(struct __pyx_obj_9drf_turbo_6fields_TimeField *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->input_formats);
  __Pyx_DECREF(__pyx_v_self->input_formats);
  __pyx_v_self->input_formats = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_9TimeField_13input_formats_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_9TimeField_13input_formats_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9TimeField_13input_formats_4__del__(((struct __pyx_obj_9drf_turbo_6fields_TimeField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_9TimeField_13input_formats_4__del__(struct __pyx_obj_9drf_turbo_6fields_TimeField *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->input_formats);
  __Pyx_DECREF(__pyx_v_self->input_formats);
  __pyx_v_self->input_formats = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_9TimeField_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_9TimeField_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9TimeField_6__reduce_cython__(((struct __pyx_obj_9drf_turbo_6fields_TimeField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_9TimeField_6__reduce_cython__(struct __pyx_obj_9drf_turbo_6fields_TimeField *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self.allow_null, self.attr, self.attrs, self.call, self.default_value, self.error_messages, self.field_name, self.format, self.help_text, self.initial, self.input_formats, self.label, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.allow_null); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.call); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.read_only); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.required); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.write_only); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = PyTuple_New(18); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.attr);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.attr);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_v_self->__pyx_base.attr);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.attrs);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.attrs);
  PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_self->__pyx_base.attrs);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_6, 3, __pyx_t_2);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.default_value);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.default_value);
  PyTuple_SET_ITEM(__pyx_t_6, 4, __pyx_v_self->__pyx_base.default_value);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.error_messages);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.error_messages);
  PyTuple_SET_ITEM(__pyx_t_6, 5, __pyx_v_self->__pyx_base.error_messages);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.field_name);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.field_name);
  PyTuple_SET_ITEM(__pyx_t_6, 6, __pyx_v_self->__pyx_base.field_name);
  __Pyx_INCREF(__pyx_v_self->format);
  __Pyx_GIVEREF(__pyx_v_self->format);
  PyTuple_SET_ITEM(__pyx_t_6, 7, __pyx_v_self->format);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.help_text);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.help_text);
  PyTuple_SET_ITEM(__pyx_t_6, 8, __pyx_v_self->__pyx_base.help_text);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.initial);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.initial);
  PyTuple_SET_ITEM(__pyx_t_6, 9, __pyx_v_self->__pyx_base.initial);
  __Pyx_INCREF(__pyx_v_self->input_formats);
  __Pyx_GIVEREF(__pyx_v_self->input_formats);
  PyTuple_SET_ITEM(__pyx_t_6, 10, __pyx_v_self->input_formats);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.label);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.label);
  PyTuple_SET_ITEM(__pyx_t_6, 11, __pyx_v_self->__pyx_base.label);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 12, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_6, 13, __pyx_t_4);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.root);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.root);
  PyTuple_SET_ITEM(__pyx_t_6, 14, __pyx_v_self->__pyx_base.root);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.style);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.style);
  PyTuple_SET_ITEM(__pyx_t_6, 15, __pyx_v_self->__pyx_base.style);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.validators);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.validators);
  PyTuple_SET_ITEM(__pyx_t_6, 16, __pyx_v_self->__pyx_base.validators);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_6, 17, __pyx_t_5);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_6);
  __pyx_t_6 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self.allow_null, self.attr, self.attrs, self.call, self.default_value, self.error_messages, self.field_name, self.format, self.help_text, self.initial, self.input_formats, self.label, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_6 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_v__dict = __pyx_t_6;
  __pyx_t_6 = 0;

  /* "(tree fragment)":7
 *     state = (self.allow_null, self.attr, self.attrs, self.call, self.default_value, self.error_messages, self.field_name, self.format, self.help_text, self.initial, self.input_formats, self.label, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_7 = (__pyx_v__dict != Py_None);
  __pyx_t_8 = (__pyx_t_7 != 0);
  if (__pyx_t_8) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_6 = PyTuple_New(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v__dict);
    __pyx_t_5 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_6); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_5));
    __pyx_t_5 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.format is not None or self.help_text is not None or self.initial is not None or self.input_formats is not None or self.label is not None or self.root is not None or self.style is not None or self.validators is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self.allow_null, self.attr, self.attrs, self.call, self.default_value, self.error_messages, self.field_name, self.format, self.help_text, self.initial, self.input_formats, self.label, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.format is not None or self.help_text is not None or self.initial is not None or self.input_formats is not None or self.label is not None or self.root is not None or self.style is not None or self.validators is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_TimeField, (type(self), 0xf36a8d7, None), state
 */
  /*else*/ {
    __pyx_t_7 = (__pyx_v_self->__pyx_base.attr != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.attrs != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.default_value != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.error_messages != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.field_name != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->format != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.help_text != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.initial != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->input_formats != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.label != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.root != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.style != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.validators != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    __pyx_t_8 = __pyx_t_9;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_8;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.format is not None or self.help_text is not None or self.initial is not None or self.input_formats is not None or self.label is not None or self.root is not None or self.style is not None or self.validators is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_TimeField, (type(self), 0xf36a8d7, None), state
 *     else:
 */
  __pyx_t_8 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_8) {

    /* "(tree fragment)":13
 *         use_setstate = self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.format is not None or self.help_text is not None or self.initial is not None or self.input_formats is not None or self.label is not None or self.root is not None or self.style is not None or self.validators is not None
 *     if use_setstate:
 *         return __pyx_unpickle_TimeField, (type(self), 0xf36a8d7, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_TimeField, (type(self), 0xf36a8d7, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_pyx_unpickle_TimeField); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_6, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_255240407);
    __Pyx_GIVEREF(__pyx_int_255240407);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_int_255240407);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_6, 2, Py_None);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_6);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_state);
    __pyx_t_5 = 0;
    __pyx_t_6 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.format is not None or self.help_text is not None or self.initial is not None or self.input_formats is not None or self.label is not None or self.root is not None or self.style is not None or self.validators is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_TimeField, (type(self), 0xf36a8d7, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_TimeField, (type(self), 0xf36a8d7, None), state
 *     else:
 *         return __pyx_unpickle_TimeField, (type(self), 0xf36a8d7, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_TimeField__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_pyx_unpickle_TimeField); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_6, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_255240407);
    __Pyx_GIVEREF(__pyx_int_255240407);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_int_255240407);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_state);
    __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_6);
    __pyx_t_4 = 0;
    __pyx_t_6 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("drf_turbo.fields.TimeField.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_TimeField, (type(self), 0xf36a8d7, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_TimeField__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_9TimeField_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_9TimeField_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9TimeField_8__setstate_cython__(((struct __pyx_obj_9drf_turbo_6fields_TimeField *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_9TimeField_8__setstate_cython__(struct __pyx_obj_9drf_turbo_6fields_TimeField *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_TimeField, (type(self), 0xf36a8d7, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_TimeField__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields___pyx_unpickle_TimeField__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_TimeField, (type(self), 0xf36a8d7, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_TimeField__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.TimeField.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":1148
 *     }
 * 
 *     def __init__(self,**kwargs):             # <<<<<<<<<<<<<<
 *         self.max_length = kwargs.pop('max_length', None)
 *         self.allow_empty_file = kwargs.pop('allow_empty_file', False)
 */

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_9FileField_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_9FileField_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_kwargs = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  if (unlikely(PyTuple_GET_SIZE(__pyx_args) > 0)) {
    __Pyx_RaiseArgtupleInvalid("__init__", 1, 0, 0, PyTuple_GET_SIZE(__pyx_args)); return -1;}
  if (__pyx_kwds && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "__init__", 1))) return -1;
  __pyx_v_kwargs = (__pyx_kwds) ? PyDict_Copy(__pyx_kwds) : PyDict_New(); if (unlikely(!__pyx_v_kwargs)) return -1;
  __Pyx_GOTREF(__pyx_v_kwargs);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9FileField___init__(((struct __pyx_obj_9drf_turbo_6fields_FileField *)__pyx_v_self), __pyx_v_kwargs);

  /* function exit code */
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_9FileField___init__(struct __pyx_obj_9drf_turbo_6fields_FileField *__pyx_v_self, PyObject *__pyx_v_kwargs) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "drf_turbo/fields.pyx":1149
 * 
 *     def __init__(self,**kwargs):
 *         self.max_length = kwargs.pop('max_length', None)             # <<<<<<<<<<<<<<
 *         self.allow_empty_file = kwargs.pop('allow_empty_file', False)
 *         super().__init__(**kwargs)
 */
  __pyx_t_1 = __Pyx_PyDict_Pop(__pyx_v_kwargs, __pyx_n_s_max_length, Py_None); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1149, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->max_length);
  __Pyx_DECREF(__pyx_v_self->max_length);
  __pyx_v_self->max_length = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "drf_turbo/fields.pyx":1150
 *     def __init__(self,**kwargs):
 *         self.max_length = kwargs.pop('max_length', None)
 *         self.allow_empty_file = kwargs.pop('allow_empty_file', False)             # <<<<<<<<<<<<<<
 *         super().__init__(**kwargs)
 * 
 */
  __pyx_t_1 = __Pyx_PyDict_Pop(__pyx_v_kwargs, __pyx_n_s_allow_empty_file, Py_False); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1150, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->allow_empty_file);
  __Pyx_DECREF(__pyx_v_self->allow_empty_file);
  __pyx_v_self->allow_empty_file = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "drf_turbo/fields.pyx":1151
 *         self.max_length = kwargs.pop('max_length', None)
 *         self.allow_empty_file = kwargs.pop('allow_empty_file', False)
 *         super().__init__(**kwargs)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1151, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_9drf_turbo_6fields_FileField));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_9drf_turbo_6fields_FileField));
  PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)__pyx_ptype_9drf_turbo_6fields_FileField));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_1, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1151, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_init); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1151, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyDict_Copy(__pyx_v_kwargs); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1151, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1151, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "drf_turbo/fields.pyx":1148
 *     }
 * 
 *     def __init__(self,**kwargs):             # <<<<<<<<<<<<<<
 *         self.max_length = kwargs.pop('max_length', None)
 *         self.allow_empty_file = kwargs.pop('allow_empty_file', False)
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("drf_turbo.fields.FileField.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":1154
 * 
 * 
 *     cpdef inline serialize(self,value,dict context):             # <<<<<<<<<<<<<<
 *         if not value :
 *             return None
 */

static PyObject *__pyx_pw_9drf_turbo_6fields_9FileField_3serialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_9FileField_serialize(CYTHON_UNUSED struct __pyx_obj_9drf_turbo_6fields_FileField *__pyx_v_self, PyObject *__pyx_v_value, PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_v_request = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("serialize", 0);

  /* "drf_turbo/fields.pyx":1155
 * 
 *     cpdef inline serialize(self,value,dict context):
 *         if not value :             # <<<<<<<<<<<<<<
 *             return None
 *         try:
 */
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_value); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 1155, __pyx_L1_error)
  __pyx_t_2 = ((!__pyx_t_1) != 0);
  if (__pyx_t_2) {

    /* "drf_turbo/fields.pyx":1156
 *     cpdef inline serialize(self,value,dict context):
 *         if not value :
 *             return None             # <<<<<<<<<<<<<<
 *         try:
 *             request = context.get('request', None)
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "drf_turbo/fields.pyx":1155
 * 
 *     cpdef inline serialize(self,value,dict context):
 *         if not value :             # <<<<<<<<<<<<<<
 *             return None
 *         try:
 */
  }

  /* "drf_turbo/fields.pyx":1157
 *         if not value :
 *             return None
 *         try:             # <<<<<<<<<<<<<<
 *             request = context.get('request', None)
 *             return request.build_absolute_uri(value.url)
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_3, &__pyx_t_4, &__pyx_t_5);
    __Pyx_XGOTREF(__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_4);
    __Pyx_XGOTREF(__pyx_t_5);
    /*try:*/ {

      /* "drf_turbo/fields.pyx":1158
 *             return None
 *         try:
 *             request = context.get('request', None)             # <<<<<<<<<<<<<<
 *             return request.build_absolute_uri(value.url)
 *         except:
 */
      if (unlikely(__pyx_v_context == Py_None)) {
        PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "get");
        __PYX_ERR(0, 1158, __pyx_L4_error)
      }
      __pyx_t_6 = __Pyx_PyDict_GetItemDefault(__pyx_v_context, __pyx_n_s_request, Py_None); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1158, __pyx_L4_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_v_request = __pyx_t_6;
      __pyx_t_6 = 0;

      /* "drf_turbo/fields.pyx":1159
 *         try:
 *             request = context.get('request', None)
 *             return request.build_absolute_uri(value.url)             # <<<<<<<<<<<<<<
 *         except:
 *             return value.url
 */
      __Pyx_XDECREF(__pyx_r);
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_request, __pyx_n_s_build_absolute_uri); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1159, __pyx_L4_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_value, __pyx_n_s_url); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1159, __pyx_L4_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_9 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
        __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_7);
        if (likely(__pyx_t_9)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
          __Pyx_INCREF(__pyx_t_9);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_7, function);
        }
      }
      __pyx_t_6 = (__pyx_t_9) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_9, __pyx_t_8) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_8);
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1159, __pyx_L4_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_r = __pyx_t_6;
      __pyx_t_6 = 0;
      goto __pyx_L8_try_return;

      /* "drf_turbo/fields.pyx":1157
 *         if not value :
 *             return None
 *         try:             # <<<<<<<<<<<<<<
 *             request = context.get('request', None)
 *             return request.build_absolute_uri(value.url)
 */
    }
    __pyx_L4_error:;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;

    /* "drf_turbo/fields.pyx":1160
 *             request = context.get('request', None)
 *             return request.build_absolute_uri(value.url)
 *         except:             # <<<<<<<<<<<<<<
 *             return value.url
 * 
 */
    /*except:*/ {
      __Pyx_AddTraceback("drf_turbo.fields.FileField.serialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_6, &__pyx_t_7, &__pyx_t_8) < 0) __PYX_ERR(0, 1160, __pyx_L6_except_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_GOTREF(__pyx_t_8);

      /* "drf_turbo/fields.pyx":1161
 *             return request.build_absolute_uri(value.url)
 *         except:
 *             return value.url             # <<<<<<<<<<<<<<
 * 
 * 
 */
      __Pyx_XDECREF(__pyx_r);
      __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_value, __pyx_n_s_url); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1161, __pyx_L6_except_error)
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_r = __pyx_t_9;
      __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      goto __pyx_L7_except_return;
    }
    __pyx_L6_except_error:;

    /* "drf_turbo/fields.pyx":1157
 *         if not value :
 *             return None
 *         try:             # <<<<<<<<<<<<<<
 *             request = context.get('request', None)
 *             return request.build_absolute_uri(value.url)
 */
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_4, __pyx_t_5);
    goto __pyx_L1_error;
    __pyx_L8_try_return:;
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_4, __pyx_t_5);
    goto __pyx_L0;
    __pyx_L7_except_return:;
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_4, __pyx_t_5);
    goto __pyx_L0;
  }

  /* "drf_turbo/fields.pyx":1154
 * 
 * 
 *     cpdef inline serialize(self,value,dict context):             # <<<<<<<<<<<<<<
 *         if not value :
 *             return None
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("drf_turbo.fields.FileField.serialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_request);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_9FileField_3serialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_9FileField_3serialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_value = 0;
  PyObject *__pyx_v_context = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("serialize (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_value,&__pyx_n_s_context,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_value)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_context)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("serialize", 1, 2, 2, 1); __PYX_ERR(0, 1154, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "serialize") < 0)) __PYX_ERR(0, 1154, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_value = values[0];
    __pyx_v_context = ((PyObject*)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("serialize", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1154, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.FileField.serialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_context), (&PyDict_Type), 1, "context", 1))) __PYX_ERR(0, 1154, __pyx_L1_error)
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9FileField_2serialize(((struct __pyx_obj_9drf_turbo_6fields_FileField *)__pyx_v_self), __pyx_v_value, __pyx_v_context);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_9FileField_2serialize(struct __pyx_obj_9drf_turbo_6fields_FileField *__pyx_v_self, PyObject *__pyx_v_value, PyObject *__pyx_v_context) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("serialize", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields_9FileField_serialize(__pyx_v_self, __pyx_v_value, __pyx_v_context, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1154, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.FileField.serialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":1164
 * 
 * 
 *     cpdef inline deserialize(self,value,dict context):             # <<<<<<<<<<<<<<
 *         try:
 *             file_name = value.name
 */

static PyObject *__pyx_pw_9drf_turbo_6fields_9FileField_5deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_9FileField_deserialize(struct __pyx_obj_9drf_turbo_6fields_FileField *__pyx_v_self, PyObject *__pyx_v_value, CYTHON_UNUSED PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_v_file_name = NULL;
  PyObject *__pyx_v_file_size = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_t_11;
  int __pyx_t_12;
  int __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("deserialize", 0);

  /* "drf_turbo/fields.pyx":1165
 * 
 *     cpdef inline deserialize(self,value,dict context):
 *         try:             # <<<<<<<<<<<<<<
 *             file_name = value.name
 *             file_size = value.size
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "drf_turbo/fields.pyx":1166
 *     cpdef inline deserialize(self,value,dict context):
 *         try:
 *             file_name = value.name             # <<<<<<<<<<<<<<
 *             file_size = value.size
 *         except AttributeError:
 */
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_value, __pyx_n_s_name_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1166, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_v_file_name = __pyx_t_4;
      __pyx_t_4 = 0;

      /* "drf_turbo/fields.pyx":1167
 *         try:
 *             file_name = value.name
 *             file_size = value.size             # <<<<<<<<<<<<<<
 *         except AttributeError:
 *             raise self.raise_if_fail('invalid')
 */
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_value, __pyx_n_s_size); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1167, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_v_file_size = __pyx_t_4;
      __pyx_t_4 = 0;

      /* "drf_turbo/fields.pyx":1165
 * 
 *     cpdef inline deserialize(self,value,dict context):
 *         try:             # <<<<<<<<<<<<<<
 *             file_name = value.name
 *             file_size = value.size
 */
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "drf_turbo/fields.pyx":1168
 *             file_name = value.name
 *             file_size = value.size
 *         except AttributeError:             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail('invalid')
 *         if not file_name:
 */
    __pyx_t_5 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_AttributeError);
    if (__pyx_t_5) {
      __Pyx_AddTraceback("drf_turbo.fields.FileField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_4, &__pyx_t_6, &__pyx_t_7) < 0) __PYX_ERR(0, 1168, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GOTREF(__pyx_t_7);

      /* "drf_turbo/fields.pyx":1169
 *             file_size = value.size
 *         except AttributeError:
 *             raise self.raise_if_fail('invalid')             # <<<<<<<<<<<<<<
 *         if not file_name:
 *             raise self.raise_if_fail('no_name')
 */
      __pyx_t_9 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_raise_if_fail); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1169, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_10 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_9))) {
        __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_9);
        if (likely(__pyx_t_10)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
          __Pyx_INCREF(__pyx_t_10);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_9, function);
        }
      }
      __pyx_t_8 = (__pyx_t_10) ? __Pyx_PyObject_Call2Args(__pyx_t_9, __pyx_t_10, __pyx_n_s_invalid) : __Pyx_PyObject_CallOneArg(__pyx_t_9, __pyx_n_s_invalid);
      __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
      if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1169, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_Raise(__pyx_t_8, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __PYX_ERR(0, 1169, __pyx_L5_except_error)
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;

    /* "drf_turbo/fields.pyx":1165
 * 
 *     cpdef inline deserialize(self,value,dict context):
 *         try:             # <<<<<<<<<<<<<<
 *             file_name = value.name
 *             file_size = value.size
 */
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L8_try_end:;
  }

  /* "drf_turbo/fields.pyx":1170
 *         except AttributeError:
 *             raise self.raise_if_fail('invalid')
 *         if not file_name:             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail('no_name')
 *         if not self.allow_empty_file and not file_size:
 */
  __pyx_t_11 = __Pyx_PyObject_IsTrue(__pyx_v_file_name); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 1170, __pyx_L1_error)
  __pyx_t_12 = ((!__pyx_t_11) != 0);
  if (unlikely(__pyx_t_12)) {

    /* "drf_turbo/fields.pyx":1171
 *             raise self.raise_if_fail('invalid')
 *         if not file_name:
 *             raise self.raise_if_fail('no_name')             # <<<<<<<<<<<<<<
 *         if not self.allow_empty_file and not file_size:
 *             raise self.raise_if_fail('empty')
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_raise_if_fail); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1171, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_6);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_6, function);
      }
    }
    __pyx_t_7 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_4, __pyx_n_s_no_name) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_n_s_no_name);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1171, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_Raise(__pyx_t_7, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __PYX_ERR(0, 1171, __pyx_L1_error)

    /* "drf_turbo/fields.pyx":1170
 *         except AttributeError:
 *             raise self.raise_if_fail('invalid')
 *         if not file_name:             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail('no_name')
 *         if not self.allow_empty_file and not file_size:
 */
  }

  /* "drf_turbo/fields.pyx":1172
 *         if not file_name:
 *             raise self.raise_if_fail('no_name')
 *         if not self.allow_empty_file and not file_size:             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail('empty')
 *         if self.max_length and len(file_name) > self.max_length:
 */
  __pyx_t_11 = __Pyx_PyObject_IsTrue(__pyx_v_self->allow_empty_file); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 1172, __pyx_L1_error)
  __pyx_t_13 = ((!__pyx_t_11) != 0);
  if (__pyx_t_13) {
  } else {
    __pyx_t_12 = __pyx_t_13;
    goto __pyx_L13_bool_binop_done;
  }
  __pyx_t_13 = __Pyx_PyObject_IsTrue(__pyx_v_file_size); if (unlikely(__pyx_t_13 < 0)) __PYX_ERR(0, 1172, __pyx_L1_error)
  __pyx_t_11 = ((!__pyx_t_13) != 0);
  __pyx_t_12 = __pyx_t_11;
  __pyx_L13_bool_binop_done:;
  if (unlikely(__pyx_t_12)) {

    /* "drf_turbo/fields.pyx":1173
 *             raise self.raise_if_fail('no_name')
 *         if not self.allow_empty_file and not file_size:
 *             raise self.raise_if_fail('empty')             # <<<<<<<<<<<<<<
 *         if self.max_length and len(file_name) > self.max_length:
 *             raise self.raise_if_fail('max_length',max_length=self.max_length, length=len(file_name))
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_raise_if_fail); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1173, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_6);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_6, function);
      }
    }
    __pyx_t_7 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_4, __pyx_n_s_empty) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_n_s_empty);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1173, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_Raise(__pyx_t_7, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __PYX_ERR(0, 1173, __pyx_L1_error)

    /* "drf_turbo/fields.pyx":1172
 *         if not file_name:
 *             raise self.raise_if_fail('no_name')
 *         if not self.allow_empty_file and not file_size:             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail('empty')
 *         if self.max_length and len(file_name) > self.max_length:
 */
  }

  /* "drf_turbo/fields.pyx":1174
 *         if not self.allow_empty_file and not file_size:
 *             raise self.raise_if_fail('empty')
 *         if self.max_length and len(file_name) > self.max_length:             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail('max_length',max_length=self.max_length, length=len(file_name))
 * 
 */
  __pyx_t_11 = __Pyx_PyObject_IsTrue(__pyx_v_self->max_length); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 1174, __pyx_L1_error)
  if (__pyx_t_11) {
  } else {
    __pyx_t_12 = __pyx_t_11;
    goto __pyx_L16_bool_binop_done;
  }
  __pyx_t_14 = PyObject_Length(__pyx_v_file_name); if (unlikely(__pyx_t_14 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1174, __pyx_L1_error)
  __pyx_t_7 = PyInt_FromSsize_t(__pyx_t_14); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1174, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_6 = PyObject_RichCompare(__pyx_t_7, __pyx_v_self->max_length, Py_GT); __Pyx_XGOTREF(__pyx_t_6); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1174, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_11 = __Pyx_PyObject_IsTrue(__pyx_t_6); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 1174, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_12 = __pyx_t_11;
  __pyx_L16_bool_binop_done:;
  if (unlikely(__pyx_t_12)) {

    /* "drf_turbo/fields.pyx":1175
 *             raise self.raise_if_fail('empty')
 *         if self.max_length and len(file_name) > self.max_length:
 *             raise self.raise_if_fail('max_length',max_length=self.max_length, length=len(file_name))             # <<<<<<<<<<<<<<
 * 
 *         return value
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_raise_if_fail); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1175, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1175, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (PyDict_SetItem(__pyx_t_7, __pyx_n_s_max_length, __pyx_v_self->max_length) < 0) __PYX_ERR(0, 1175, __pyx_L1_error)
    __pyx_t_14 = PyObject_Length(__pyx_v_file_name); if (unlikely(__pyx_t_14 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1175, __pyx_L1_error)
    __pyx_t_4 = PyInt_FromSsize_t(__pyx_t_14); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1175, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    if (PyDict_SetItem(__pyx_t_7, __pyx_n_s_length, __pyx_t_4) < 0) __PYX_ERR(0, 1175, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_tuple__11, __pyx_t_7); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1175, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(0, 1175, __pyx_L1_error)

    /* "drf_turbo/fields.pyx":1174
 *         if not self.allow_empty_file and not file_size:
 *             raise self.raise_if_fail('empty')
 *         if self.max_length and len(file_name) > self.max_length:             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail('max_length',max_length=self.max_length, length=len(file_name))
 * 
 */
  }

  /* "drf_turbo/fields.pyx":1177
 *             raise self.raise_if_fail('max_length',max_length=self.max_length, length=len(file_name))
 * 
 *         return value             # <<<<<<<<<<<<<<
 * 
 * @cython.final
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_value);
  __pyx_r = __pyx_v_value;
  goto __pyx_L0;

  /* "drf_turbo/fields.pyx":1164
 * 
 * 
 *     cpdef inline deserialize(self,value,dict context):             # <<<<<<<<<<<<<<
 *         try:
 *             file_name = value.name
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("drf_turbo.fields.FileField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_file_name);
  __Pyx_XDECREF(__pyx_v_file_size);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_9FileField_5deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_9FileField_5deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_value = 0;
  PyObject *__pyx_v_context = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("deserialize (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_value,&__pyx_n_s_context,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_value)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_context)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("deserialize", 1, 2, 2, 1); __PYX_ERR(0, 1164, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "deserialize") < 0)) __PYX_ERR(0, 1164, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_value = values[0];
    __pyx_v_context = ((PyObject*)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("deserialize", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1164, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.FileField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_context), (&PyDict_Type), 1, "context", 1))) __PYX_ERR(0, 1164, __pyx_L1_error)
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9FileField_4deserialize(((struct __pyx_obj_9drf_turbo_6fields_FileField *)__pyx_v_self), __pyx_v_value, __pyx_v_context);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_9FileField_4deserialize(struct __pyx_obj_9drf_turbo_6fields_FileField *__pyx_v_self, PyObject *__pyx_v_value, PyObject *__pyx_v_context) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("deserialize", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields_9FileField_deserialize(__pyx_v_self, __pyx_v_value, __pyx_v_context, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1164, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.FileField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pxd":127
 * cdef class FileField(Field):
 *     cdef public :
 *         max_length             # <<<<<<<<<<<<<<
 *         allow_empty_file
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_9FileField_10max_length_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_9FileField_10max_length_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9FileField_10max_length___get__(((struct __pyx_obj_9drf_turbo_6fields_FileField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_9FileField_10max_length___get__(struct __pyx_obj_9drf_turbo_6fields_FileField *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->max_length);
  __pyx_r = __pyx_v_self->max_length;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_9FileField_10max_length_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_9FileField_10max_length_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9FileField_10max_length_2__set__(((struct __pyx_obj_9drf_turbo_6fields_FileField *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_9FileField_10max_length_2__set__(struct __pyx_obj_9drf_turbo_6fields_FileField *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->max_length);
  __Pyx_DECREF(__pyx_v_self->max_length);
  __pyx_v_self->max_length = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_9FileField_10max_length_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_9FileField_10max_length_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9FileField_10max_length_4__del__(((struct __pyx_obj_9drf_turbo_6fields_FileField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_9FileField_10max_length_4__del__(struct __pyx_obj_9drf_turbo_6fields_FileField *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->max_length);
  __Pyx_DECREF(__pyx_v_self->max_length);
  __pyx_v_self->max_length = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pxd":128
 *     cdef public :
 *         max_length
 *         allow_empty_file             # <<<<<<<<<<<<<<
 * 
 * cdef class ArrayField(Field):
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_9FileField_16allow_empty_file_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_9FileField_16allow_empty_file_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9FileField_16allow_empty_file___get__(((struct __pyx_obj_9drf_turbo_6fields_FileField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_9FileField_16allow_empty_file___get__(struct __pyx_obj_9drf_turbo_6fields_FileField *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->allow_empty_file);
  __pyx_r = __pyx_v_self->allow_empty_file;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_9FileField_16allow_empty_file_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_9FileField_16allow_empty_file_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9FileField_16allow_empty_file_2__set__(((struct __pyx_obj_9drf_turbo_6fields_FileField *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_9FileField_16allow_empty_file_2__set__(struct __pyx_obj_9drf_turbo_6fields_FileField *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->allow_empty_file);
  __Pyx_DECREF(__pyx_v_self->allow_empty_file);
  __pyx_v_self->allow_empty_file = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_9FileField_16allow_empty_file_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_9FileField_16allow_empty_file_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9FileField_16allow_empty_file_4__del__(((struct __pyx_obj_9drf_turbo_6fields_FileField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_9FileField_16allow_empty_file_4__del__(struct __pyx_obj_9drf_turbo_6fields_FileField *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->allow_empty_file);
  __Pyx_DECREF(__pyx_v_self->allow_empty_file);
  __pyx_v_self->allow_empty_file = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_9FileField_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_9FileField_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9FileField_6__reduce_cython__(((struct __pyx_obj_9drf_turbo_6fields_FileField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_9FileField_6__reduce_cython__(struct __pyx_obj_9drf_turbo_6fields_FileField *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self.allow_empty_file, self.allow_null, self.attr, self.attrs, self.call, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.max_length, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.allow_null); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.call); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.read_only); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.required); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.write_only); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = PyTuple_New(18); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_INCREF(__pyx_v_self->allow_empty_file);
  __Pyx_GIVEREF(__pyx_v_self->allow_empty_file);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v_self->allow_empty_file);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.attr);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.attr);
  PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_self->__pyx_base.attr);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.attrs);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.attrs);
  PyTuple_SET_ITEM(__pyx_t_6, 3, __pyx_v_self->__pyx_base.attrs);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_6, 4, __pyx_t_2);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.default_value);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.default_value);
  PyTuple_SET_ITEM(__pyx_t_6, 5, __pyx_v_self->__pyx_base.default_value);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.error_messages);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.error_messages);
  PyTuple_SET_ITEM(__pyx_t_6, 6, __pyx_v_self->__pyx_base.error_messages);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.field_name);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.field_name);
  PyTuple_SET_ITEM(__pyx_t_6, 7, __pyx_v_self->__pyx_base.field_name);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.help_text);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.help_text);
  PyTuple_SET_ITEM(__pyx_t_6, 8, __pyx_v_self->__pyx_base.help_text);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.initial);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.initial);
  PyTuple_SET_ITEM(__pyx_t_6, 9, __pyx_v_self->__pyx_base.initial);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.label);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.label);
  PyTuple_SET_ITEM(__pyx_t_6, 10, __pyx_v_self->__pyx_base.label);
  __Pyx_INCREF(__pyx_v_self->max_length);
  __Pyx_GIVEREF(__pyx_v_self->max_length);
  PyTuple_SET_ITEM(__pyx_t_6, 11, __pyx_v_self->max_length);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 12, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_6, 13, __pyx_t_4);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.root);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.root);
  PyTuple_SET_ITEM(__pyx_t_6, 14, __pyx_v_self->__pyx_base.root);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.style);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.style);
  PyTuple_SET_ITEM(__pyx_t_6, 15, __pyx_v_self->__pyx_base.style);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.validators);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.validators);
  PyTuple_SET_ITEM(__pyx_t_6, 16, __pyx_v_self->__pyx_base.validators);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_6, 17, __pyx_t_5);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_6);
  __pyx_t_6 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self.allow_empty_file, self.allow_null, self.attr, self.attrs, self.call, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.max_length, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_6 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_v__dict = __pyx_t_6;
  __pyx_t_6 = 0;

  /* "(tree fragment)":7
 *     state = (self.allow_empty_file, self.allow_null, self.attr, self.attrs, self.call, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.max_length, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_7 = (__pyx_v__dict != Py_None);
  __pyx_t_8 = (__pyx_t_7 != 0);
  if (__pyx_t_8) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_6 = PyTuple_New(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v__dict);
    __pyx_t_5 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_6); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_5));
    __pyx_t_5 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.allow_empty_file is not None or self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.max_length is not None or self.root is not None or self.style is not None or self.validators is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self.allow_empty_file, self.allow_null, self.attr, self.attrs, self.call, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.max_length, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self.allow_empty_file is not None or self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.max_length is not None or self.root is not None or self.style is not None or self.validators is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_FileField, (type(self), 0xe82609b, None), state
 */
  /*else*/ {
    __pyx_t_7 = (__pyx_v_self->allow_empty_file != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.attr != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.attrs != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.default_value != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.error_messages != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.field_name != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.help_text != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.initial != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.label != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->max_length != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.root != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.style != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.validators != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    __pyx_t_8 = __pyx_t_9;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_8;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.allow_empty_file is not None or self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.max_length is not None or self.root is not None or self.style is not None or self.validators is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_FileField, (type(self), 0xe82609b, None), state
 *     else:
 */
  __pyx_t_8 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_8) {

    /* "(tree fragment)":13
 *         use_setstate = self.allow_empty_file is not None or self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.max_length is not None or self.root is not None or self.style is not None or self.validators is not None
 *     if use_setstate:
 *         return __pyx_unpickle_FileField, (type(self), 0xe82609b, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_FileField, (type(self), 0xe82609b, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_pyx_unpickle_FileField); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_6, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_243425435);
    __Pyx_GIVEREF(__pyx_int_243425435);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_int_243425435);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_6, 2, Py_None);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_6);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_state);
    __pyx_t_5 = 0;
    __pyx_t_6 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.allow_empty_file is not None or self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.max_length is not None or self.root is not None or self.style is not None or self.validators is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_FileField, (type(self), 0xe82609b, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_FileField, (type(self), 0xe82609b, None), state
 *     else:
 *         return __pyx_unpickle_FileField, (type(self), 0xe82609b, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_FileField__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_pyx_unpickle_FileField); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_6, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_243425435);
    __Pyx_GIVEREF(__pyx_int_243425435);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_int_243425435);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_state);
    __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_6);
    __pyx_t_4 = 0;
    __pyx_t_6 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("drf_turbo.fields.FileField.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_FileField, (type(self), 0xe82609b, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_FileField__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_9FileField_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_9FileField_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9FileField_8__setstate_cython__(((struct __pyx_obj_9drf_turbo_6fields_FileField *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_9FileField_8__setstate_cython__(struct __pyx_obj_9drf_turbo_6fields_FileField *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_FileField, (type(self), 0xe82609b, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_FileField__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields___pyx_unpickle_FileField__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_FileField, (type(self), 0xe82609b, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_FileField__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.FileField.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":1202
 *     _initial = []
 * 
 *     def __init__(self,child=None,**kwargs):             # <<<<<<<<<<<<<<
 *         self.child = kwargs.pop('child', copy.deepcopy(child))
 *         self.allow_empty = kwargs.pop('allow_empty', True)
 */

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_10ArrayField_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_10ArrayField_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_child = 0;
  PyObject *__pyx_v_kwargs = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  __pyx_v_kwargs = PyDict_New(); if (unlikely(!__pyx_v_kwargs)) return -1;
  __Pyx_GOTREF(__pyx_v_kwargs);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_child,0};
    PyObject* values[1] = {0};
    values[0] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_child);
          if (value) { values[0] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, __pyx_v_kwargs, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 1202, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_child = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 0, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1202, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_DECREF(__pyx_v_kwargs); __pyx_v_kwargs = 0;
  __Pyx_AddTraceback("drf_turbo.fields.ArrayField.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9drf_turbo_6fields_10ArrayField___init__(((struct __pyx_obj_9drf_turbo_6fields_ArrayField *)__pyx_v_self), __pyx_v_child, __pyx_v_kwargs);

  /* function exit code */
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_10ArrayField___init__(struct __pyx_obj_9drf_turbo_6fields_ArrayField *__pyx_v_self, PyObject *__pyx_v_child, PyObject *__pyx_v_kwargs) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "drf_turbo/fields.pyx":1203
 * 
 *     def __init__(self,child=None,**kwargs):
 *         self.child = kwargs.pop('child', copy.deepcopy(child))             # <<<<<<<<<<<<<<
 *         self.allow_empty = kwargs.pop('allow_empty', True)
 *         self.max_items = kwargs.pop('max_items', None)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_copy); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1203, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_deepcopy); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1203, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_2, __pyx_v_child) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_child);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1203, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyDict_Pop(__pyx_v_kwargs, __pyx_n_s_child, __pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1203, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_GIVEREF(__pyx_t_3);
  __Pyx_GOTREF(__pyx_v_self->child);
  __Pyx_DECREF(__pyx_v_self->child);
  __pyx_v_self->child = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "drf_turbo/fields.pyx":1204
 *     def __init__(self,child=None,**kwargs):
 *         self.child = kwargs.pop('child', copy.deepcopy(child))
 *         self.allow_empty = kwargs.pop('allow_empty', True)             # <<<<<<<<<<<<<<
 *         self.max_items = kwargs.pop('max_items', None)
 *         self.min_items = kwargs.pop('min_items', None)
 */
  __pyx_t_3 = __Pyx_PyDict_Pop(__pyx_v_kwargs, __pyx_n_s_allow_empty, Py_True); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1204, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_3);
  __Pyx_GOTREF(__pyx_v_self->allow_empty);
  __Pyx_DECREF(__pyx_v_self->allow_empty);
  __pyx_v_self->allow_empty = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "drf_turbo/fields.pyx":1205
 *         self.child = kwargs.pop('child', copy.deepcopy(child))
 *         self.allow_empty = kwargs.pop('allow_empty', True)
 *         self.max_items = kwargs.pop('max_items', None)             # <<<<<<<<<<<<<<
 *         self.min_items = kwargs.pop('min_items', None)
 *         self.exact_items =  kwargs.pop('exact_items', None)
 */
  __pyx_t_3 = __Pyx_PyDict_Pop(__pyx_v_kwargs, __pyx_n_s_max_items, Py_None); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1205, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_3);
  __Pyx_GOTREF(__pyx_v_self->max_items);
  __Pyx_DECREF(__pyx_v_self->max_items);
  __pyx_v_self->max_items = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "drf_turbo/fields.pyx":1206
 *         self.allow_empty = kwargs.pop('allow_empty', True)
 *         self.max_items = kwargs.pop('max_items', None)
 *         self.min_items = kwargs.pop('min_items', None)             # <<<<<<<<<<<<<<
 *         self.exact_items =  kwargs.pop('exact_items', None)
 *         super().__init__(**kwargs)
 */
  __pyx_t_3 = __Pyx_PyDict_Pop(__pyx_v_kwargs, __pyx_n_s_min_items, Py_None); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1206, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_3);
  __Pyx_GOTREF(__pyx_v_self->min_items);
  __Pyx_DECREF(__pyx_v_self->min_items);
  __pyx_v_self->min_items = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "drf_turbo/fields.pyx":1207
 *         self.max_items = kwargs.pop('max_items', None)
 *         self.min_items = kwargs.pop('min_items', None)
 *         self.exact_items =  kwargs.pop('exact_items', None)             # <<<<<<<<<<<<<<
 *         super().__init__(**kwargs)
 *         if self.child is not None:
 */
  __pyx_t_3 = __Pyx_PyDict_Pop(__pyx_v_kwargs, __pyx_n_s_exact_items, Py_None); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1207, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_3);
  __Pyx_GOTREF(__pyx_v_self->exact_items);
  __Pyx_DECREF(__pyx_v_self->exact_items);
  __pyx_v_self->exact_items = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "drf_turbo/fields.pyx":1208
 *         self.min_items = kwargs.pop('min_items', None)
 *         self.exact_items =  kwargs.pop('exact_items', None)
 *         super().__init__(**kwargs)             # <<<<<<<<<<<<<<
 *         if self.child is not None:
 *             self.child.bind(field_name='', root=self)
 */
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1208, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_9drf_turbo_6fields_ArrayField));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_9drf_turbo_6fields_ArrayField));
  PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)__pyx_ptype_9drf_turbo_6fields_ArrayField));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_3, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1208, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_init); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1208, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyDict_Copy(__pyx_v_kwargs); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1208, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1208, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "drf_turbo/fields.pyx":1209
 *         self.exact_items =  kwargs.pop('exact_items', None)
 *         super().__init__(**kwargs)
 *         if self.child is not None:             # <<<<<<<<<<<<<<
 *             self.child.bind(field_name='', root=self)
 *         if self.exact_items is not None:
 */
  __pyx_t_4 = (__pyx_v_self->child != Py_None);
  __pyx_t_5 = (__pyx_t_4 != 0);
  if (__pyx_t_5) {

    /* "drf_turbo/fields.pyx":1210
 *         super().__init__(**kwargs)
 *         if self.child is not None:
 *             self.child.bind(field_name='', root=self)             # <<<<<<<<<<<<<<
 *         if self.exact_items is not None:
 *             self.min_items = self.exact_items
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->child, __pyx_n_s_bind); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1210, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1210, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_field_name, __pyx_kp_s__9) < 0) __PYX_ERR(0, 1210, __pyx_L1_error)
    if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_root, ((PyObject *)__pyx_v_self)) < 0) __PYX_ERR(0, 1210, __pyx_L1_error)
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1210, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "drf_turbo/fields.pyx":1209
 *         self.exact_items =  kwargs.pop('exact_items', None)
 *         super().__init__(**kwargs)
 *         if self.child is not None:             # <<<<<<<<<<<<<<
 *             self.child.bind(field_name='', root=self)
 *         if self.exact_items is not None:
 */
  }

  /* "drf_turbo/fields.pyx":1211
 *         if self.child is not None:
 *             self.child.bind(field_name='', root=self)
 *         if self.exact_items is not None:             # <<<<<<<<<<<<<<
 *             self.min_items = self.exact_items
 *             self.max_items = self.exact_items
 */
  __pyx_t_5 = (__pyx_v_self->exact_items != Py_None);
  __pyx_t_4 = (__pyx_t_5 != 0);
  if (__pyx_t_4) {

    /* "drf_turbo/fields.pyx":1212
 *             self.child.bind(field_name='', root=self)
 *         if self.exact_items is not None:
 *             self.min_items = self.exact_items             # <<<<<<<<<<<<<<
 *             self.max_items = self.exact_items
 * 
 */
    __pyx_t_3 = __pyx_v_self->exact_items;
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_3);
    __Pyx_GOTREF(__pyx_v_self->min_items);
    __Pyx_DECREF(__pyx_v_self->min_items);
    __pyx_v_self->min_items = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "drf_turbo/fields.pyx":1213
 *         if self.exact_items is not None:
 *             self.min_items = self.exact_items
 *             self.max_items = self.exact_items             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_t_3 = __pyx_v_self->exact_items;
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_3);
    __Pyx_GOTREF(__pyx_v_self->max_items);
    __Pyx_DECREF(__pyx_v_self->max_items);
    __pyx_v_self->max_items = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "drf_turbo/fields.pyx":1211
 *         if self.child is not None:
 *             self.child.bind(field_name='', root=self)
 *         if self.exact_items is not None:             # <<<<<<<<<<<<<<
 *             self.min_items = self.exact_items
 *             self.max_items = self.exact_items
 */
  }

  /* "drf_turbo/fields.pyx":1202
 *     _initial = []
 * 
 *     def __init__(self,child=None,**kwargs):             # <<<<<<<<<<<<<<
 *         self.child = kwargs.pop('child', copy.deepcopy(child))
 *         self.allow_empty = kwargs.pop('allow_empty', True)
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("drf_turbo.fields.ArrayField.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":1216
 * 
 * 
 *     cpdef inline serialize(self,data ,dict context):             # <<<<<<<<<<<<<<
 *         """
 *         List of object instances -> List of dicts of primitive datatypes.
 */

static PyObject *__pyx_pw_9drf_turbo_6fields_10ArrayField_3serialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_10ArrayField_serialize(struct __pyx_obj_9drf_turbo_6fields_ArrayField *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_v_item = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  PyObject *(*__pyx_t_4)(PyObject *);
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("serialize", 0);

  /* "drf_turbo/fields.pyx":1220
 *         List of object instances -> List of dicts of primitive datatypes.
 *         """
 *         return [ self.child.serialize(item,context) if item is not None else None for item in data]             # <<<<<<<<<<<<<<
 * 
 *     cpdef inline deserialize(self,data,dict context):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1220, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (likely(PyList_CheckExact(__pyx_v_data)) || PyTuple_CheckExact(__pyx_v_data)) {
    __pyx_t_2 = __pyx_v_data; __Pyx_INCREF(__pyx_t_2); __pyx_t_3 = 0;
    __pyx_t_4 = NULL;
  } else {
    __pyx_t_3 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_v_data); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1220, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1220, __pyx_L1_error)
  }
  for (;;) {
    if (likely(!__pyx_t_4)) {
      if (likely(PyList_CheckExact(__pyx_t_2))) {
        if (__pyx_t_3 >= PyList_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_5 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_3); __Pyx_INCREF(__pyx_t_5); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 1220, __pyx_L1_error)
        #else
        __pyx_t_5 = PySequence_ITEM(__pyx_t_2, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1220, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        #endif
      } else {
        if (__pyx_t_3 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_5 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_3); __Pyx_INCREF(__pyx_t_5); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 1220, __pyx_L1_error)
        #else
        __pyx_t_5 = PySequence_ITEM(__pyx_t_2, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1220, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        #endif
      }
    } else {
      __pyx_t_5 = __pyx_t_4(__pyx_t_2);
      if (unlikely(!__pyx_t_5)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 1220, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_5);
    }
    __Pyx_XDECREF_SET(__pyx_v_item, __pyx_t_5);
    __pyx_t_5 = 0;
    __pyx_t_6 = (__pyx_v_item != Py_None);
    if ((__pyx_t_6 != 0)) {
      __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->child, __pyx_n_s_serialize); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1220, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_9 = NULL;
      __pyx_t_10 = 0;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_8))) {
        __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_8);
        if (likely(__pyx_t_9)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
          __Pyx_INCREF(__pyx_t_9);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_8, function);
          __pyx_t_10 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_8)) {
        PyObject *__pyx_temp[3] = {__pyx_t_9, __pyx_v_item, __pyx_v_context};
        __pyx_t_7 = __Pyx_PyFunction_FastCall(__pyx_t_8, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1220, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
        __Pyx_GOTREF(__pyx_t_7);
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_8)) {
        PyObject *__pyx_temp[3] = {__pyx_t_9, __pyx_v_item, __pyx_v_context};
        __pyx_t_7 = __Pyx_PyCFunction_FastCall(__pyx_t_8, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1220, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
        __Pyx_GOTREF(__pyx_t_7);
      } else
      #endif
      {
        __pyx_t_11 = PyTuple_New(2+__pyx_t_10); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1220, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        if (__pyx_t_9) {
          __Pyx_GIVEREF(__pyx_t_9); PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_9); __pyx_t_9 = NULL;
        }
        __Pyx_INCREF(__pyx_v_item);
        __Pyx_GIVEREF(__pyx_v_item);
        PyTuple_SET_ITEM(__pyx_t_11, 0+__pyx_t_10, __pyx_v_item);
        __Pyx_INCREF(__pyx_v_context);
        __Pyx_GIVEREF(__pyx_v_context);
        PyTuple_SET_ITEM(__pyx_t_11, 1+__pyx_t_10, __pyx_v_context);
        __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_t_11, NULL); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1220, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      }
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __pyx_t_5 = __pyx_t_7;
      __pyx_t_7 = 0;
    } else {
      __Pyx_INCREF(Py_None);
      __pyx_t_5 = Py_None;
    }
    if (unlikely(__Pyx_ListComp_Append(__pyx_t_1, (PyObject*)__pyx_t_5))) __PYX_ERR(0, 1220, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "drf_turbo/fields.pyx":1216
 * 
 * 
 *     cpdef inline serialize(self,data ,dict context):             # <<<<<<<<<<<<<<
 *         """
 *         List of object instances -> List of dicts of primitive datatypes.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("drf_turbo.fields.ArrayField.serialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_item);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_10ArrayField_3serialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_9drf_turbo_6fields_10ArrayField_2serialize[] = "\n        List of object instances -> List of dicts of primitive datatypes.\n        ";
static PyObject *__pyx_pw_9drf_turbo_6fields_10ArrayField_3serialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_data = 0;
  PyObject *__pyx_v_context = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("serialize (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_data,&__pyx_n_s_context,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_data)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_context)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("serialize", 1, 2, 2, 1); __PYX_ERR(0, 1216, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "serialize") < 0)) __PYX_ERR(0, 1216, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_data = values[0];
    __pyx_v_context = ((PyObject*)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("serialize", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1216, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.ArrayField.serialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_context), (&PyDict_Type), 1, "context", 1))) __PYX_ERR(0, 1216, __pyx_L1_error)
  __pyx_r = __pyx_pf_9drf_turbo_6fields_10ArrayField_2serialize(((struct __pyx_obj_9drf_turbo_6fields_ArrayField *)__pyx_v_self), __pyx_v_data, __pyx_v_context);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_10ArrayField_2serialize(struct __pyx_obj_9drf_turbo_6fields_ArrayField *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("serialize", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields_10ArrayField_serialize(__pyx_v_self, __pyx_v_data, __pyx_v_context, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1216, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.ArrayField.serialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":1222
 *         return [ self.child.serialize(item,context) if item is not None else None for item in data]
 * 
 *     cpdef inline deserialize(self,data,dict context):             # <<<<<<<<<<<<<<
 *         """
 *         List of dicts of native values <- List of dicts of primitive datatypes.
 */

static PyObject *__pyx_pw_9drf_turbo_6fields_10ArrayField_5deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_10ArrayField_deserialize(struct __pyx_obj_9drf_turbo_6fields_ArrayField *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  Py_ssize_t __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("deserialize", 0);

  /* "drf_turbo/fields.pyx":1226
 *         List of dicts of native values <- List of dicts of primitive datatypes.
 *         """
 *         if isinstance(data, (str, dict)) or not hasattr(data, '__iter__'):             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail('not_a_list', input_type=type(data).__name__)
 *         if not self.allow_empty and len(data) == 0:
 */
  __pyx_t_3 = PyString_Check(__pyx_v_data); 
  __pyx_t_4 = (__pyx_t_3 != 0);
  if (!__pyx_t_4) {
  } else {
    __pyx_t_2 = __pyx_t_4;
    goto __pyx_L6_bool_binop_done;
  }
  __pyx_t_4 = PyDict_Check(__pyx_v_data); 
  __pyx_t_3 = (__pyx_t_4 != 0);
  __pyx_t_2 = __pyx_t_3;
  __pyx_L6_bool_binop_done:;
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (!__pyx_t_3) {
  } else {
    __pyx_t_1 = __pyx_t_3;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_3 = __Pyx_HasAttr(__pyx_v_data, __pyx_n_s_iter); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(0, 1226, __pyx_L1_error)
  __pyx_t_2 = ((!(__pyx_t_3 != 0)) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (unlikely(__pyx_t_1)) {

    /* "drf_turbo/fields.pyx":1227
 *         """
 *         if isinstance(data, (str, dict)) or not hasattr(data, '__iter__'):
 *             raise self.raise_if_fail('not_a_list', input_type=type(data).__name__)             # <<<<<<<<<<<<<<
 *         if not self.allow_empty and len(data) == 0:
 *             raise self.raise_if_fail('empty')
 */
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_raise_if_fail); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1227, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1227, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(((PyObject *)Py_TYPE(__pyx_v_data)), __pyx_n_s_name); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1227, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (PyDict_SetItem(__pyx_t_6, __pyx_n_s_input_type, __pyx_t_7) < 0) __PYX_ERR(0, 1227, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_tuple__19, __pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1227, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_Raise(__pyx_t_7, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __PYX_ERR(0, 1227, __pyx_L1_error)

    /* "drf_turbo/fields.pyx":1226
 *         List of dicts of native values <- List of dicts of primitive datatypes.
 *         """
 *         if isinstance(data, (str, dict)) or not hasattr(data, '__iter__'):             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail('not_a_list', input_type=type(data).__name__)
 *         if not self.allow_empty and len(data) == 0:
 */
  }

  /* "drf_turbo/fields.pyx":1228
 *         if isinstance(data, (str, dict)) or not hasattr(data, '__iter__'):
 *             raise self.raise_if_fail('not_a_list', input_type=type(data).__name__)
 *         if not self.allow_empty and len(data) == 0:             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail('empty')
 *         if (
 */
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_self->allow_empty); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 1228, __pyx_L1_error)
  __pyx_t_3 = ((!__pyx_t_2) != 0);
  if (__pyx_t_3) {
  } else {
    __pyx_t_1 = __pyx_t_3;
    goto __pyx_L9_bool_binop_done;
  }
  __pyx_t_8 = PyObject_Length(__pyx_v_data); if (unlikely(__pyx_t_8 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1228, __pyx_L1_error)
  __pyx_t_3 = ((__pyx_t_8 == 0) != 0);
  __pyx_t_1 = __pyx_t_3;
  __pyx_L9_bool_binop_done:;
  if (unlikely(__pyx_t_1)) {

    /* "drf_turbo/fields.pyx":1229
 *             raise self.raise_if_fail('not_a_list', input_type=type(data).__name__)
 *         if not self.allow_empty and len(data) == 0:
 *             raise self.raise_if_fail('empty')             # <<<<<<<<<<<<<<
 *         if (
 *             self.min_items is not None
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_raise_if_fail); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1229, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_6);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_6, function);
      }
    }
    __pyx_t_7 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_5, __pyx_n_s_empty) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_n_s_empty);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1229, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_Raise(__pyx_t_7, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __PYX_ERR(0, 1229, __pyx_L1_error)

    /* "drf_turbo/fields.pyx":1228
 *         if isinstance(data, (str, dict)) or not hasattr(data, '__iter__'):
 *             raise self.raise_if_fail('not_a_list', input_type=type(data).__name__)
 *         if not self.allow_empty and len(data) == 0:             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail('empty')
 *         if (
 */
  }

  /* "drf_turbo/fields.pyx":1231
 *             raise self.raise_if_fail('empty')
 *         if (
 *             self.min_items is not None             # <<<<<<<<<<<<<<
 *             and self.min_items == self.max_items
 *             and len(data) != self.min_items
 */
  __pyx_t_3 = (__pyx_v_self->min_items != Py_None);
  __pyx_t_2 = (__pyx_t_3 != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L12_bool_binop_done;
  }

  /* "drf_turbo/fields.pyx":1232
 *         if (
 *             self.min_items is not None
 *             and self.min_items == self.max_items             # <<<<<<<<<<<<<<
 *             and len(data) != self.min_items
 *         ):
 */
  __pyx_t_7 = PyObject_RichCompare(__pyx_v_self->min_items, __pyx_v_self->max_items, Py_EQ); __Pyx_XGOTREF(__pyx_t_7); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1232, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_7); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 1232, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L12_bool_binop_done;
  }

  /* "drf_turbo/fields.pyx":1233
 *             self.min_items is not None
 *             and self.min_items == self.max_items
 *             and len(data) != self.min_items             # <<<<<<<<<<<<<<
 *         ):
 *             raise self.raise_if_fail("exact_items",exact_items=self.exact_items)
 */
  __pyx_t_8 = PyObject_Length(__pyx_v_data); if (unlikely(__pyx_t_8 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1233, __pyx_L1_error)
  __pyx_t_7 = PyInt_FromSsize_t(__pyx_t_8); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1233, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_6 = PyObject_RichCompare(__pyx_t_7, __pyx_v_self->min_items, Py_NE); __Pyx_XGOTREF(__pyx_t_6); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1233, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_6); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 1233, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_1 = __pyx_t_2;
  __pyx_L12_bool_binop_done:;

  /* "drf_turbo/fields.pyx":1230
 *         if not self.allow_empty and len(data) == 0:
 *             raise self.raise_if_fail('empty')
 *         if (             # <<<<<<<<<<<<<<
 *             self.min_items is not None
 *             and self.min_items == self.max_items
 */
  if (unlikely(__pyx_t_1)) {

    /* "drf_turbo/fields.pyx":1235
 *             and len(data) != self.min_items
 *         ):
 *             raise self.raise_if_fail("exact_items",exact_items=self.exact_items)             # <<<<<<<<<<<<<<
 *         if self.min_items is not None and len(data) < self.min_items:
 *             raise self.raise_if_fail("min_items",min_items=self.min_items)
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_raise_if_fail); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1235, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1235, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (PyDict_SetItem(__pyx_t_7, __pyx_n_s_exact_items, __pyx_v_self->exact_items) < 0) __PYX_ERR(0, 1235, __pyx_L1_error)
    __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_tuple__24, __pyx_t_7); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1235, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_Raise(__pyx_t_5, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __PYX_ERR(0, 1235, __pyx_L1_error)

    /* "drf_turbo/fields.pyx":1230
 *         if not self.allow_empty and len(data) == 0:
 *             raise self.raise_if_fail('empty')
 *         if (             # <<<<<<<<<<<<<<
 *             self.min_items is not None
 *             and self.min_items == self.max_items
 */
  }

  /* "drf_turbo/fields.pyx":1236
 *         ):
 *             raise self.raise_if_fail("exact_items",exact_items=self.exact_items)
 *         if self.min_items is not None and len(data) < self.min_items:             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail("min_items",min_items=self.min_items)
 *         elif self.max_items is not None and len(data) > self.max_items:
 */
  __pyx_t_2 = (__pyx_v_self->min_items != Py_None);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {
  } else {
    __pyx_t_1 = __pyx_t_3;
    goto __pyx_L16_bool_binop_done;
  }
  __pyx_t_8 = PyObject_Length(__pyx_v_data); if (unlikely(__pyx_t_8 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1236, __pyx_L1_error)
  __pyx_t_5 = PyInt_FromSsize_t(__pyx_t_8); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1236, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_7 = PyObject_RichCompare(__pyx_t_5, __pyx_v_self->min_items, Py_LT); __Pyx_XGOTREF(__pyx_t_7); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1236, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_7); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 1236, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_1 = __pyx_t_3;
  __pyx_L16_bool_binop_done:;
  if (unlikely(__pyx_t_1)) {

    /* "drf_turbo/fields.pyx":1237
 *             raise self.raise_if_fail("exact_items",exact_items=self.exact_items)
 *         if self.min_items is not None and len(data) < self.min_items:
 *             raise self.raise_if_fail("min_items",min_items=self.min_items)             # <<<<<<<<<<<<<<
 *         elif self.max_items is not None and len(data) > self.max_items:
 *             raise self.raise_if_fail("max_items",max_items=self.max_items)
 */
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_raise_if_fail); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1237, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_5 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1237, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (PyDict_SetItem(__pyx_t_5, __pyx_n_s_min_items, __pyx_v_self->min_items) < 0) __PYX_ERR(0, 1237, __pyx_L1_error)
    __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_tuple__25, __pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1237, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_Raise(__pyx_t_6, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __PYX_ERR(0, 1237, __pyx_L1_error)

    /* "drf_turbo/fields.pyx":1236
 *         ):
 *             raise self.raise_if_fail("exact_items",exact_items=self.exact_items)
 *         if self.min_items is not None and len(data) < self.min_items:             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail("min_items",min_items=self.min_items)
 *         elif self.max_items is not None and len(data) > self.max_items:
 */
  }

  /* "drf_turbo/fields.pyx":1238
 *         if self.min_items is not None and len(data) < self.min_items:
 *             raise self.raise_if_fail("min_items",min_items=self.min_items)
 *         elif self.max_items is not None and len(data) > self.max_items:             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail("max_items",max_items=self.max_items)
 * 
 */
  __pyx_t_3 = (__pyx_v_self->max_items != Py_None);
  __pyx_t_2 = (__pyx_t_3 != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L18_bool_binop_done;
  }
  __pyx_t_8 = PyObject_Length(__pyx_v_data); if (unlikely(__pyx_t_8 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1238, __pyx_L1_error)
  __pyx_t_6 = PyInt_FromSsize_t(__pyx_t_8); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1238, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_5 = PyObject_RichCompare(__pyx_t_6, __pyx_v_self->max_items, Py_GT); __Pyx_XGOTREF(__pyx_t_5); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1238, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_5); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 1238, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_1 = __pyx_t_2;
  __pyx_L18_bool_binop_done:;
  if (unlikely(__pyx_t_1)) {

    /* "drf_turbo/fields.pyx":1239
 *             raise self.raise_if_fail("min_items",min_items=self.min_items)
 *         elif self.max_items is not None and len(data) > self.max_items:
 *             raise self.raise_if_fail("max_items",max_items=self.max_items)             # <<<<<<<<<<<<<<
 * 
 *         return self.run_child_validation(data,context)
 */
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_raise_if_fail); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1239, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1239, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    if (PyDict_SetItem(__pyx_t_6, __pyx_n_s_max_items, __pyx_v_self->max_items) < 0) __PYX_ERR(0, 1239, __pyx_L1_error)
    __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_tuple__26, __pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1239, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_Raise(__pyx_t_7, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __PYX_ERR(0, 1239, __pyx_L1_error)

    /* "drf_turbo/fields.pyx":1238
 *         if self.min_items is not None and len(data) < self.min_items:
 *             raise self.raise_if_fail("min_items",min_items=self.min_items)
 *         elif self.max_items is not None and len(data) > self.max_items:             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail("max_items",max_items=self.max_items)
 * 
 */
  }

  /* "drf_turbo/fields.pyx":1241
 *             raise self.raise_if_fail("max_items",max_items=self.max_items)
 * 
 *         return self.run_child_validation(data,context)             # <<<<<<<<<<<<<<
 * 
 *     cpdef inline run_child_validation(self,data,dict context):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_7 = __pyx_f_9drf_turbo_6fields_10ArrayField_run_child_validation(__pyx_v_self, __pyx_v_data, __pyx_v_context, 0); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1241, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_r = __pyx_t_7;
  __pyx_t_7 = 0;
  goto __pyx_L0;

  /* "drf_turbo/fields.pyx":1222
 *         return [ self.child.serialize(item,context) if item is not None else None for item in data]
 * 
 *     cpdef inline deserialize(self,data,dict context):             # <<<<<<<<<<<<<<
 *         """
 *         List of dicts of native values <- List of dicts of primitive datatypes.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("drf_turbo.fields.ArrayField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_10ArrayField_5deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_9drf_turbo_6fields_10ArrayField_4deserialize[] = "\n        List of dicts of native values <- List of dicts of primitive datatypes.\n        ";
static PyObject *__pyx_pw_9drf_turbo_6fields_10ArrayField_5deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_data = 0;
  PyObject *__pyx_v_context = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("deserialize (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_data,&__pyx_n_s_context,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_data)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_context)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("deserialize", 1, 2, 2, 1); __PYX_ERR(0, 1222, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "deserialize") < 0)) __PYX_ERR(0, 1222, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_data = values[0];
    __pyx_v_context = ((PyObject*)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("deserialize", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1222, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.ArrayField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_context), (&PyDict_Type), 1, "context", 1))) __PYX_ERR(0, 1222, __pyx_L1_error)
  __pyx_r = __pyx_pf_9drf_turbo_6fields_10ArrayField_4deserialize(((struct __pyx_obj_9drf_turbo_6fields_ArrayField *)__pyx_v_self), __pyx_v_data, __pyx_v_context);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_10ArrayField_4deserialize(struct __pyx_obj_9drf_turbo_6fields_ArrayField *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("deserialize", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields_10ArrayField_deserialize(__pyx_v_self, __pyx_v_data, __pyx_v_context, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1222, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.ArrayField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":1243
 *         return self.run_child_validation(data,context)
 * 
 *     cpdef inline run_child_validation(self,data,dict context):             # <<<<<<<<<<<<<<
 *         cdef list result = []
 *         cdef dict errors = {}
 */

static PyObject *__pyx_pw_9drf_turbo_6fields_10ArrayField_7run_child_validation(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_10ArrayField_run_child_validation(struct __pyx_obj_9drf_turbo_6fields_ArrayField *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_v_result = 0;
  PyObject *__pyx_v_errors = 0;
  int __pyx_v_idx;
  PyObject *__pyx_v_item = NULL;
  PyObject *__pyx_v_e = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  PyObject *(*__pyx_t_3)(PyObject *);
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  int __pyx_t_12;
  PyObject *__pyx_t_13 = NULL;
  int __pyx_t_14;
  int __pyx_t_15;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("run_child_validation", 0);

  /* "drf_turbo/fields.pyx":1244
 * 
 *     cpdef inline run_child_validation(self,data,dict context):
 *         cdef list result = []             # <<<<<<<<<<<<<<
 *         cdef dict errors = {}
 *         cdef int idx = 0
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1244, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_result = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "drf_turbo/fields.pyx":1245
 *     cpdef inline run_child_validation(self,data,dict context):
 *         cdef list result = []
 *         cdef dict errors = {}             # <<<<<<<<<<<<<<
 *         cdef int idx = 0
 *         for item in data :
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1245, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_errors = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "drf_turbo/fields.pyx":1246
 *         cdef list result = []
 *         cdef dict errors = {}
 *         cdef int idx = 0             # <<<<<<<<<<<<<<
 *         for item in data :
 *             try:
 */
  __pyx_v_idx = 0;

  /* "drf_turbo/fields.pyx":1247
 *         cdef dict errors = {}
 *         cdef int idx = 0
 *         for item in data :             # <<<<<<<<<<<<<<
 *             try:
 *                 result.append(self.child.run_validation(item,context))
 */
  if (likely(PyList_CheckExact(__pyx_v_data)) || PyTuple_CheckExact(__pyx_v_data)) {
    __pyx_t_1 = __pyx_v_data; __Pyx_INCREF(__pyx_t_1); __pyx_t_2 = 0;
    __pyx_t_3 = NULL;
  } else {
    __pyx_t_2 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_v_data); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1247, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1247, __pyx_L1_error)
  }
  for (;;) {
    if (likely(!__pyx_t_3)) {
      if (likely(PyList_CheckExact(__pyx_t_1))) {
        if (__pyx_t_2 >= PyList_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_4 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_4); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(0, 1247, __pyx_L1_error)
        #else
        __pyx_t_4 = PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1247, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        #endif
      } else {
        if (__pyx_t_2 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_4 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_4); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(0, 1247, __pyx_L1_error)
        #else
        __pyx_t_4 = PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1247, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        #endif
      }
    } else {
      __pyx_t_4 = __pyx_t_3(__pyx_t_1);
      if (unlikely(!__pyx_t_4)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 1247, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_4);
    }
    __Pyx_XDECREF_SET(__pyx_v_item, __pyx_t_4);
    __pyx_t_4 = 0;

    /* "drf_turbo/fields.pyx":1248
 *         cdef int idx = 0
 *         for item in data :
 *             try:             # <<<<<<<<<<<<<<
 *                 result.append(self.child.run_validation(item,context))
 *             except ValidationError as e:
 */
    {
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __Pyx_ExceptionSave(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7);
      __Pyx_XGOTREF(__pyx_t_5);
      __Pyx_XGOTREF(__pyx_t_6);
      __Pyx_XGOTREF(__pyx_t_7);
      /*try:*/ {

        /* "drf_turbo/fields.pyx":1249
 *         for item in data :
 *             try:
 *                 result.append(self.child.run_validation(item,context))             # <<<<<<<<<<<<<<
 *             except ValidationError as e:
 *                 errors[idx] = e.detail
 */
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->child, __pyx_n_s_run_validation); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1249, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_9 = NULL;
        __pyx_t_10 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_8))) {
          __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_8);
          if (likely(__pyx_t_9)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
            __Pyx_INCREF(__pyx_t_9);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_8, function);
            __pyx_t_10 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_8)) {
          PyObject *__pyx_temp[3] = {__pyx_t_9, __pyx_v_item, __pyx_v_context};
          __pyx_t_4 = __Pyx_PyFunction_FastCall(__pyx_t_8, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1249, __pyx_L5_error)
          __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
          __Pyx_GOTREF(__pyx_t_4);
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_8)) {
          PyObject *__pyx_temp[3] = {__pyx_t_9, __pyx_v_item, __pyx_v_context};
          __pyx_t_4 = __Pyx_PyCFunction_FastCall(__pyx_t_8, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1249, __pyx_L5_error)
          __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
          __Pyx_GOTREF(__pyx_t_4);
        } else
        #endif
        {
          __pyx_t_11 = PyTuple_New(2+__pyx_t_10); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1249, __pyx_L5_error)
          __Pyx_GOTREF(__pyx_t_11);
          if (__pyx_t_9) {
            __Pyx_GIVEREF(__pyx_t_9); PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_9); __pyx_t_9 = NULL;
          }
          __Pyx_INCREF(__pyx_v_item);
          __Pyx_GIVEREF(__pyx_v_item);
          PyTuple_SET_ITEM(__pyx_t_11, 0+__pyx_t_10, __pyx_v_item);
          __Pyx_INCREF(__pyx_v_context);
          __Pyx_GIVEREF(__pyx_v_context);
          PyTuple_SET_ITEM(__pyx_t_11, 1+__pyx_t_10, __pyx_v_context);
          __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_t_11, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1249, __pyx_L5_error)
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        }
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_t_12 = __Pyx_PyList_Append(__pyx_v_result, __pyx_t_4); if (unlikely(__pyx_t_12 == ((int)-1))) __PYX_ERR(0, 1249, __pyx_L5_error)
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

        /* "drf_turbo/fields.pyx":1248
 *         cdef int idx = 0
 *         for item in data :
 *             try:             # <<<<<<<<<<<<<<
 *                 result.append(self.child.run_validation(item,context))
 *             except ValidationError as e:
 */
      }
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      goto __pyx_L12_try_end;
      __pyx_L5_error:;
      __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;

      /* "drf_turbo/fields.pyx":1250
 *             try:
 *                 result.append(self.child.run_validation(item,context))
 *             except ValidationError as e:             # <<<<<<<<<<<<<<
 *                 errors[idx] = e.detail
 *             idx +=1
 */
      __Pyx_ErrFetch(&__pyx_t_4, &__pyx_t_8, &__pyx_t_11);
      __Pyx_GetModuleGlobalName(__pyx_t_9, __pyx_n_s_ValidationError); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1250, __pyx_L7_except_error)
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_10 = __Pyx_PyErr_GivenExceptionMatches(__pyx_t_4, __pyx_t_9);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_ErrRestore(__pyx_t_4, __pyx_t_8, __pyx_t_11);
      __pyx_t_4 = 0; __pyx_t_8 = 0; __pyx_t_11 = 0;
      if (__pyx_t_10) {
        __Pyx_AddTraceback("drf_turbo.fields.ArrayField.run_child_validation", __pyx_clineno, __pyx_lineno, __pyx_filename);
        if (__Pyx_GetException(&__pyx_t_11, &__pyx_t_8, &__pyx_t_4) < 0) __PYX_ERR(0, 1250, __pyx_L7_except_error)
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_XDECREF_SET(__pyx_v_e, __pyx_t_8);

        /* "drf_turbo/fields.pyx":1251
 *                 result.append(self.child.run_validation(item,context))
 *             except ValidationError as e:
 *                 errors[idx] = e.detail             # <<<<<<<<<<<<<<
 *             idx +=1
 * 
 */
        __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_e, __pyx_n_s_detail); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1251, __pyx_L7_except_error)
        __Pyx_GOTREF(__pyx_t_9);
        __pyx_t_13 = __Pyx_PyInt_From_int(__pyx_v_idx); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1251, __pyx_L7_except_error)
        __Pyx_GOTREF(__pyx_t_13);
        if (unlikely(PyDict_SetItem(__pyx_v_errors, __pyx_t_13, __pyx_t_9) < 0)) __PYX_ERR(0, 1251, __pyx_L7_except_error)
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        goto __pyx_L6_exception_handled;
      }
      goto __pyx_L7_except_error;
      __pyx_L7_except_error:;

      /* "drf_turbo/fields.pyx":1248
 *         cdef int idx = 0
 *         for item in data :
 *             try:             # <<<<<<<<<<<<<<
 *                 result.append(self.child.run_validation(item,context))
 *             except ValidationError as e:
 */
      __Pyx_XGIVEREF(__pyx_t_5);
      __Pyx_XGIVEREF(__pyx_t_6);
      __Pyx_XGIVEREF(__pyx_t_7);
      __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_6, __pyx_t_7);
      goto __pyx_L1_error;
      __pyx_L6_exception_handled:;
      __Pyx_XGIVEREF(__pyx_t_5);
      __Pyx_XGIVEREF(__pyx_t_6);
      __Pyx_XGIVEREF(__pyx_t_7);
      __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_6, __pyx_t_7);
      __pyx_L12_try_end:;
    }

    /* "drf_turbo/fields.pyx":1252
 *             except ValidationError as e:
 *                 errors[idx] = e.detail
 *             idx +=1             # <<<<<<<<<<<<<<
 * 
 *         if not errors:
 */
    __pyx_v_idx = (__pyx_v_idx + 1);

    /* "drf_turbo/fields.pyx":1247
 *         cdef dict errors = {}
 *         cdef int idx = 0
 *         for item in data :             # <<<<<<<<<<<<<<
 *             try:
 *                 result.append(self.child.run_validation(item,context))
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "drf_turbo/fields.pyx":1254
 *             idx +=1
 * 
 *         if not errors:             # <<<<<<<<<<<<<<
 *             return result
 *         raise ValidationError(errors)
 */
  __pyx_t_14 = __Pyx_PyObject_IsTrue(__pyx_v_errors); if (unlikely(__pyx_t_14 < 0)) __PYX_ERR(0, 1254, __pyx_L1_error)
  __pyx_t_15 = ((!__pyx_t_14) != 0);
  if (__pyx_t_15) {

    /* "drf_turbo/fields.pyx":1255
 * 
 *         if not errors:
 *             return result             # <<<<<<<<<<<<<<
 *         raise ValidationError(errors)
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_result);
    __pyx_r = __pyx_v_result;
    goto __pyx_L0;

    /* "drf_turbo/fields.pyx":1254
 *             idx +=1
 * 
 *         if not errors:             # <<<<<<<<<<<<<<
 *             return result
 *         raise ValidationError(errors)
 */
  }

  /* "drf_turbo/fields.pyx":1256
 *         if not errors:
 *             return result
 *         raise ValidationError(errors)             # <<<<<<<<<<<<<<
 * 
 * @cython.final
 */
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_ValidationError); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1256, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_8 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_8)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_8);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  __pyx_t_1 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_8, __pyx_v_errors) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_v_errors);
  __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1256, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(0, 1256, __pyx_L1_error)

  /* "drf_turbo/fields.pyx":1243
 *         return self.run_child_validation(data,context)
 * 
 *     cpdef inline run_child_validation(self,data,dict context):             # <<<<<<<<<<<<<<
 *         cdef list result = []
 *         cdef dict errors = {}
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_AddTraceback("drf_turbo.fields.ArrayField.run_child_validation", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_result);
  __Pyx_XDECREF(__pyx_v_errors);
  __Pyx_XDECREF(__pyx_v_item);
  __Pyx_XDECREF(__pyx_v_e);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_10ArrayField_7run_child_validation(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_10ArrayField_7run_child_validation(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_data = 0;
  PyObject *__pyx_v_context = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("run_child_validation (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_data,&__pyx_n_s_context,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_data)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_context)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("run_child_validation", 1, 2, 2, 1); __PYX_ERR(0, 1243, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "run_child_validation") < 0)) __PYX_ERR(0, 1243, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_data = values[0];
    __pyx_v_context = ((PyObject*)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("run_child_validation", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1243, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.ArrayField.run_child_validation", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_context), (&PyDict_Type), 1, "context", 1))) __PYX_ERR(0, 1243, __pyx_L1_error)
  __pyx_r = __pyx_pf_9drf_turbo_6fields_10ArrayField_6run_child_validation(((struct __pyx_obj_9drf_turbo_6fields_ArrayField *)__pyx_v_self), __pyx_v_data, __pyx_v_context);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_10ArrayField_6run_child_validation(struct __pyx_obj_9drf_turbo_6fields_ArrayField *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("run_child_validation", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields_10ArrayField_run_child_validation(__pyx_v_self, __pyx_v_data, __pyx_v_context, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1243, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.ArrayField.run_child_validation", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pxd":132
 * cdef class ArrayField(Field):
 *     cdef public :
 *         child             # <<<<<<<<<<<<<<
 *         allow_empty
 *         min_items
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_10ArrayField_5child_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_10ArrayField_5child_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_10ArrayField_5child___get__(((struct __pyx_obj_9drf_turbo_6fields_ArrayField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_10ArrayField_5child___get__(struct __pyx_obj_9drf_turbo_6fields_ArrayField *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->child);
  __pyx_r = __pyx_v_self->child;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_10ArrayField_5child_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_10ArrayField_5child_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_10ArrayField_5child_2__set__(((struct __pyx_obj_9drf_turbo_6fields_ArrayField *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_10ArrayField_5child_2__set__(struct __pyx_obj_9drf_turbo_6fields_ArrayField *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->child);
  __Pyx_DECREF(__pyx_v_self->child);
  __pyx_v_self->child = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_10ArrayField_5child_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_10ArrayField_5child_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_10ArrayField_5child_4__del__(((struct __pyx_obj_9drf_turbo_6fields_ArrayField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_10ArrayField_5child_4__del__(struct __pyx_obj_9drf_turbo_6fields_ArrayField *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->child);
  __Pyx_DECREF(__pyx_v_self->child);
  __pyx_v_self->child = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pxd":133
 *     cdef public :
 *         child
 *         allow_empty             # <<<<<<<<<<<<<<
 *         min_items
 *         max_items
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_10ArrayField_11allow_empty_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_10ArrayField_11allow_empty_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_10ArrayField_11allow_empty___get__(((struct __pyx_obj_9drf_turbo_6fields_ArrayField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_10ArrayField_11allow_empty___get__(struct __pyx_obj_9drf_turbo_6fields_ArrayField *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->allow_empty);
  __pyx_r = __pyx_v_self->allow_empty;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_10ArrayField_11allow_empty_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_10ArrayField_11allow_empty_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_10ArrayField_11allow_empty_2__set__(((struct __pyx_obj_9drf_turbo_6fields_ArrayField *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_10ArrayField_11allow_empty_2__set__(struct __pyx_obj_9drf_turbo_6fields_ArrayField *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->allow_empty);
  __Pyx_DECREF(__pyx_v_self->allow_empty);
  __pyx_v_self->allow_empty = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_10ArrayField_11allow_empty_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_10ArrayField_11allow_empty_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_10ArrayField_11allow_empty_4__del__(((struct __pyx_obj_9drf_turbo_6fields_ArrayField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_10ArrayField_11allow_empty_4__del__(struct __pyx_obj_9drf_turbo_6fields_ArrayField *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->allow_empty);
  __Pyx_DECREF(__pyx_v_self->allow_empty);
  __pyx_v_self->allow_empty = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pxd":134
 *         child
 *         allow_empty
 *         min_items             # <<<<<<<<<<<<<<
 *         max_items
 *         exact_items
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_10ArrayField_9min_items_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_10ArrayField_9min_items_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_10ArrayField_9min_items___get__(((struct __pyx_obj_9drf_turbo_6fields_ArrayField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_10ArrayField_9min_items___get__(struct __pyx_obj_9drf_turbo_6fields_ArrayField *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->min_items);
  __pyx_r = __pyx_v_self->min_items;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_10ArrayField_9min_items_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_10ArrayField_9min_items_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_10ArrayField_9min_items_2__set__(((struct __pyx_obj_9drf_turbo_6fields_ArrayField *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_10ArrayField_9min_items_2__set__(struct __pyx_obj_9drf_turbo_6fields_ArrayField *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->min_items);
  __Pyx_DECREF(__pyx_v_self->min_items);
  __pyx_v_self->min_items = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_10ArrayField_9min_items_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_10ArrayField_9min_items_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_10ArrayField_9min_items_4__del__(((struct __pyx_obj_9drf_turbo_6fields_ArrayField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_10ArrayField_9min_items_4__del__(struct __pyx_obj_9drf_turbo_6fields_ArrayField *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->min_items);
  __Pyx_DECREF(__pyx_v_self->min_items);
  __pyx_v_self->min_items = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pxd":135
 *         allow_empty
 *         min_items
 *         max_items             # <<<<<<<<<<<<<<
 *         exact_items
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_10ArrayField_9max_items_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_10ArrayField_9max_items_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_10ArrayField_9max_items___get__(((struct __pyx_obj_9drf_turbo_6fields_ArrayField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_10ArrayField_9max_items___get__(struct __pyx_obj_9drf_turbo_6fields_ArrayField *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->max_items);
  __pyx_r = __pyx_v_self->max_items;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_10ArrayField_9max_items_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_10ArrayField_9max_items_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_10ArrayField_9max_items_2__set__(((struct __pyx_obj_9drf_turbo_6fields_ArrayField *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_10ArrayField_9max_items_2__set__(struct __pyx_obj_9drf_turbo_6fields_ArrayField *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->max_items);
  __Pyx_DECREF(__pyx_v_self->max_items);
  __pyx_v_self->max_items = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_10ArrayField_9max_items_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_10ArrayField_9max_items_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_10ArrayField_9max_items_4__del__(((struct __pyx_obj_9drf_turbo_6fields_ArrayField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_10ArrayField_9max_items_4__del__(struct __pyx_obj_9drf_turbo_6fields_ArrayField *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->max_items);
  __Pyx_DECREF(__pyx_v_self->max_items);
  __pyx_v_self->max_items = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pxd":136
 *         min_items
 *         max_items
 *         exact_items             # <<<<<<<<<<<<<<
 * 
 *     cpdef run_child_validation(self,data,dict context)
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_10ArrayField_11exact_items_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_10ArrayField_11exact_items_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_10ArrayField_11exact_items___get__(((struct __pyx_obj_9drf_turbo_6fields_ArrayField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_10ArrayField_11exact_items___get__(struct __pyx_obj_9drf_turbo_6fields_ArrayField *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->exact_items);
  __pyx_r = __pyx_v_self->exact_items;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_10ArrayField_11exact_items_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_10ArrayField_11exact_items_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_10ArrayField_11exact_items_2__set__(((struct __pyx_obj_9drf_turbo_6fields_ArrayField *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_10ArrayField_11exact_items_2__set__(struct __pyx_obj_9drf_turbo_6fields_ArrayField *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->exact_items);
  __Pyx_DECREF(__pyx_v_self->exact_items);
  __pyx_v_self->exact_items = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_10ArrayField_11exact_items_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_10ArrayField_11exact_items_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_10ArrayField_11exact_items_4__del__(((struct __pyx_obj_9drf_turbo_6fields_ArrayField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_10ArrayField_11exact_items_4__del__(struct __pyx_obj_9drf_turbo_6fields_ArrayField *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->exact_items);
  __Pyx_DECREF(__pyx_v_self->exact_items);
  __pyx_v_self->exact_items = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_10ArrayField_9__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_10ArrayField_9__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_10ArrayField_8__reduce_cython__(((struct __pyx_obj_9drf_turbo_6fields_ArrayField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_10ArrayField_8__reduce_cython__(struct __pyx_obj_9drf_turbo_6fields_ArrayField *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self.allow_empty, self.allow_null, self.attr, self.attrs, self.call, self.child, self.default_value, self.error_messages, self.exact_items, self.field_name, self.help_text, self.initial, self.label, self.max_items, self.min_items, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.allow_null); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.call); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.read_only); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.required); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.write_only); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = PyTuple_New(21); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_INCREF(__pyx_v_self->allow_empty);
  __Pyx_GIVEREF(__pyx_v_self->allow_empty);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v_self->allow_empty);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.attr);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.attr);
  PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_self->__pyx_base.attr);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.attrs);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.attrs);
  PyTuple_SET_ITEM(__pyx_t_6, 3, __pyx_v_self->__pyx_base.attrs);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_6, 4, __pyx_t_2);
  __Pyx_INCREF(__pyx_v_self->child);
  __Pyx_GIVEREF(__pyx_v_self->child);
  PyTuple_SET_ITEM(__pyx_t_6, 5, __pyx_v_self->child);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.default_value);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.default_value);
  PyTuple_SET_ITEM(__pyx_t_6, 6, __pyx_v_self->__pyx_base.default_value);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.error_messages);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.error_messages);
  PyTuple_SET_ITEM(__pyx_t_6, 7, __pyx_v_self->__pyx_base.error_messages);
  __Pyx_INCREF(__pyx_v_self->exact_items);
  __Pyx_GIVEREF(__pyx_v_self->exact_items);
  PyTuple_SET_ITEM(__pyx_t_6, 8, __pyx_v_self->exact_items);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.field_name);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.field_name);
  PyTuple_SET_ITEM(__pyx_t_6, 9, __pyx_v_self->__pyx_base.field_name);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.help_text);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.help_text);
  PyTuple_SET_ITEM(__pyx_t_6, 10, __pyx_v_self->__pyx_base.help_text);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.initial);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.initial);
  PyTuple_SET_ITEM(__pyx_t_6, 11, __pyx_v_self->__pyx_base.initial);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.label);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.label);
  PyTuple_SET_ITEM(__pyx_t_6, 12, __pyx_v_self->__pyx_base.label);
  __Pyx_INCREF(__pyx_v_self->max_items);
  __Pyx_GIVEREF(__pyx_v_self->max_items);
  PyTuple_SET_ITEM(__pyx_t_6, 13, __pyx_v_self->max_items);
  __Pyx_INCREF(__pyx_v_self->min_items);
  __Pyx_GIVEREF(__pyx_v_self->min_items);
  PyTuple_SET_ITEM(__pyx_t_6, 14, __pyx_v_self->min_items);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 15, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_6, 16, __pyx_t_4);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.root);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.root);
  PyTuple_SET_ITEM(__pyx_t_6, 17, __pyx_v_self->__pyx_base.root);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.style);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.style);
  PyTuple_SET_ITEM(__pyx_t_6, 18, __pyx_v_self->__pyx_base.style);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.validators);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.validators);
  PyTuple_SET_ITEM(__pyx_t_6, 19, __pyx_v_self->__pyx_base.validators);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_6, 20, __pyx_t_5);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_6);
  __pyx_t_6 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self.allow_empty, self.allow_null, self.attr, self.attrs, self.call, self.child, self.default_value, self.error_messages, self.exact_items, self.field_name, self.help_text, self.initial, self.label, self.max_items, self.min_items, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_6 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_v__dict = __pyx_t_6;
  __pyx_t_6 = 0;

  /* "(tree fragment)":7
 *     state = (self.allow_empty, self.allow_null, self.attr, self.attrs, self.call, self.child, self.default_value, self.error_messages, self.exact_items, self.field_name, self.help_text, self.initial, self.label, self.max_items, self.min_items, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_7 = (__pyx_v__dict != Py_None);
  __pyx_t_8 = (__pyx_t_7 != 0);
  if (__pyx_t_8) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_6 = PyTuple_New(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v__dict);
    __pyx_t_5 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_6); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_5));
    __pyx_t_5 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.allow_empty is not None or self.attr is not None or self.attrs is not None or self.child is not None or self.default_value is not None or self.error_messages is not None or self.exact_items is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.max_items is not None or self.min_items is not None or self.root is not None or self.style is not None or self.validators is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self.allow_empty, self.allow_null, self.attr, self.attrs, self.call, self.child, self.default_value, self.error_messages, self.exact_items, self.field_name, self.help_text, self.initial, self.label, self.max_items, self.min_items, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self.allow_empty is not None or self.attr is not None or self.attrs is not None or self.child is not None or self.default_value is not None or self.error_messages is not None or self.exact_items is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.max_items is not None or self.min_items is not None or self.root is not None or self.style is not None or self.validators is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_ArrayField, (type(self), 0x5775012, None), state
 */
  /*else*/ {
    __pyx_t_7 = (__pyx_v_self->allow_empty != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.attr != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.attrs != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->child != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.default_value != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.error_messages != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->exact_items != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.field_name != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.help_text != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.initial != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.label != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->max_items != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->min_items != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.root != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.style != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.validators != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    __pyx_t_8 = __pyx_t_7;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_8;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.allow_empty is not None or self.attr is not None or self.attrs is not None or self.child is not None or self.default_value is not None or self.error_messages is not None or self.exact_items is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.max_items is not None or self.min_items is not None or self.root is not None or self.style is not None or self.validators is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_ArrayField, (type(self), 0x5775012, None), state
 *     else:
 */
  __pyx_t_8 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_8) {

    /* "(tree fragment)":13
 *         use_setstate = self.allow_empty is not None or self.attr is not None or self.attrs is not None or self.child is not None or self.default_value is not None or self.error_messages is not None or self.exact_items is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.max_items is not None or self.min_items is not None or self.root is not None or self.style is not None or self.validators is not None
 *     if use_setstate:
 *         return __pyx_unpickle_ArrayField, (type(self), 0x5775012, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_ArrayField, (type(self), 0x5775012, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_pyx_unpickle_ArrayField); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_6, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_91705362);
    __Pyx_GIVEREF(__pyx_int_91705362);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_int_91705362);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_6, 2, Py_None);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_6);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_state);
    __pyx_t_5 = 0;
    __pyx_t_6 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.allow_empty is not None or self.attr is not None or self.attrs is not None or self.child is not None or self.default_value is not None or self.error_messages is not None or self.exact_items is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.max_items is not None or self.min_items is not None or self.root is not None or self.style is not None or self.validators is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_ArrayField, (type(self), 0x5775012, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_ArrayField, (type(self), 0x5775012, None), state
 *     else:
 *         return __pyx_unpickle_ArrayField, (type(self), 0x5775012, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_ArrayField__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_pyx_unpickle_ArrayField); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_6, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_91705362);
    __Pyx_GIVEREF(__pyx_int_91705362);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_int_91705362);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_state);
    __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_6);
    __pyx_t_4 = 0;
    __pyx_t_6 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("drf_turbo.fields.ArrayField.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_ArrayField, (type(self), 0x5775012, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_ArrayField__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_10ArrayField_11__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_10ArrayField_11__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_10ArrayField_10__setstate_cython__(((struct __pyx_obj_9drf_turbo_6fields_ArrayField *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_10ArrayField_10__setstate_cython__(struct __pyx_obj_9drf_turbo_6fields_ArrayField *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_ArrayField, (type(self), 0x5775012, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_ArrayField__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields___pyx_unpickle_ArrayField__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_ArrayField, (type(self), 0x5775012, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_ArrayField__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.ArrayField.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":1272
 *     _initial = {}
 * 
 *     def __init__(self,child=None,**kwargs):             # <<<<<<<<<<<<<<
 *         self.child = kwargs.pop('child', copy.deepcopy(child))
 *         self.allow_empty = kwargs.pop('allow_empty', True)
 */

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_9DictField_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_9DictField_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_child = 0;
  PyObject *__pyx_v_kwargs = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  __pyx_v_kwargs = PyDict_New(); if (unlikely(!__pyx_v_kwargs)) return -1;
  __Pyx_GOTREF(__pyx_v_kwargs);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_child,0};
    PyObject* values[1] = {0};
    values[0] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_child);
          if (value) { values[0] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, __pyx_v_kwargs, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 1272, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_child = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 0, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1272, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_DECREF(__pyx_v_kwargs); __pyx_v_kwargs = 0;
  __Pyx_AddTraceback("drf_turbo.fields.DictField.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9DictField___init__(((struct __pyx_obj_9drf_turbo_6fields_DictField *)__pyx_v_self), __pyx_v_child, __pyx_v_kwargs);

  /* function exit code */
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_9DictField___init__(struct __pyx_obj_9drf_turbo_6fields_DictField *__pyx_v_self, PyObject *__pyx_v_child, PyObject *__pyx_v_kwargs) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "drf_turbo/fields.pyx":1273
 * 
 *     def __init__(self,child=None,**kwargs):
 *         self.child = kwargs.pop('child', copy.deepcopy(child))             # <<<<<<<<<<<<<<
 *         self.allow_empty = kwargs.pop('allow_empty', True)
 *         super().__init__(**kwargs)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_copy); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1273, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_deepcopy); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1273, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_2, __pyx_v_child) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_child);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1273, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyDict_Pop(__pyx_v_kwargs, __pyx_n_s_child, __pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1273, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_GIVEREF(__pyx_t_3);
  __Pyx_GOTREF(__pyx_v_self->child);
  __Pyx_DECREF(__pyx_v_self->child);
  __pyx_v_self->child = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "drf_turbo/fields.pyx":1274
 *     def __init__(self,child=None,**kwargs):
 *         self.child = kwargs.pop('child', copy.deepcopy(child))
 *         self.allow_empty = kwargs.pop('allow_empty', True)             # <<<<<<<<<<<<<<
 *         super().__init__(**kwargs)
 *         if self.child is not None:
 */
  __pyx_t_3 = __Pyx_PyDict_Pop(__pyx_v_kwargs, __pyx_n_s_allow_empty, Py_True); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1274, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_3);
  __Pyx_GOTREF(__pyx_v_self->allow_empty);
  __Pyx_DECREF(__pyx_v_self->allow_empty);
  __pyx_v_self->allow_empty = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "drf_turbo/fields.pyx":1275
 *         self.child = kwargs.pop('child', copy.deepcopy(child))
 *         self.allow_empty = kwargs.pop('allow_empty', True)
 *         super().__init__(**kwargs)             # <<<<<<<<<<<<<<
 *         if self.child is not None:
 *             self.child.bind(field_name='', root=self)
 */
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1275, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_9drf_turbo_6fields_DictField));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_9drf_turbo_6fields_DictField));
  PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)__pyx_ptype_9drf_turbo_6fields_DictField));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_3, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1275, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_init); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1275, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyDict_Copy(__pyx_v_kwargs); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1275, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1275, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "drf_turbo/fields.pyx":1276
 *         self.allow_empty = kwargs.pop('allow_empty', True)
 *         super().__init__(**kwargs)
 *         if self.child is not None:             # <<<<<<<<<<<<<<
 *             self.child.bind(field_name='', root=self)
 * 
 */
  __pyx_t_4 = (__pyx_v_self->child != Py_None);
  __pyx_t_5 = (__pyx_t_4 != 0);
  if (__pyx_t_5) {

    /* "drf_turbo/fields.pyx":1277
 *         super().__init__(**kwargs)
 *         if self.child is not None:
 *             self.child.bind(field_name='', root=self)             # <<<<<<<<<<<<<<
 * 
 *     cpdef inline serialize(self, data,dict context):
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->child, __pyx_n_s_bind); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1277, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1277, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_field_name, __pyx_kp_s__9) < 0) __PYX_ERR(0, 1277, __pyx_L1_error)
    if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_root, ((PyObject *)__pyx_v_self)) < 0) __PYX_ERR(0, 1277, __pyx_L1_error)
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1277, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "drf_turbo/fields.pyx":1276
 *         self.allow_empty = kwargs.pop('allow_empty', True)
 *         super().__init__(**kwargs)
 *         if self.child is not None:             # <<<<<<<<<<<<<<
 *             self.child.bind(field_name='', root=self)
 * 
 */
  }

  /* "drf_turbo/fields.pyx":1272
 *     _initial = {}
 * 
 *     def __init__(self,child=None,**kwargs):             # <<<<<<<<<<<<<<
 *         self.child = kwargs.pop('child', copy.deepcopy(child))
 *         self.allow_empty = kwargs.pop('allow_empty', True)
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("drf_turbo.fields.DictField.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":1279
 *             self.child.bind(field_name='', root=self)
 * 
 *     cpdef inline serialize(self, data,dict context):             # <<<<<<<<<<<<<<
 *         """
 *         List of object instances -> List of dicts of primitive datatypes.
 */

static PyObject *__pyx_pw_9drf_turbo_6fields_9DictField_3serialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_9DictField_serialize(struct __pyx_obj_9drf_turbo_6fields_DictField *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_8genexpr1__pyx_v_key = NULL;
  PyObject *__pyx_8genexpr1__pyx_v_value = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *(*__pyx_t_6)(PyObject *);
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *(*__pyx_t_9)(PyObject *);
  int __pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  int __pyx_t_12;
  PyObject *__pyx_t_13 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("serialize", 0);

  /* "drf_turbo/fields.pyx":1283
 *         List of object instances -> List of dicts of primitive datatypes.
 *         """
 *         return {             # <<<<<<<<<<<<<<
 *             str(key) : self.child.serialize(value,context) if value is not None else None for key,value in data.items()
 *         }
 */
  __Pyx_XDECREF(__pyx_r);
  { /* enter inner scope */
    __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1283, __pyx_L5_error)
    __Pyx_GOTREF(__pyx_t_1);

    /* "drf_turbo/fields.pyx":1284
 *         """
 *         return {
 *             str(key) : self.child.serialize(value,context) if value is not None else None for key,value in data.items()             # <<<<<<<<<<<<<<
 *         }
 * 
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_data, __pyx_n_s_items); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1284, __pyx_L5_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1284, __pyx_L5_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (likely(PyList_CheckExact(__pyx_t_2)) || PyTuple_CheckExact(__pyx_t_2)) {
      __pyx_t_3 = __pyx_t_2; __Pyx_INCREF(__pyx_t_3); __pyx_t_5 = 0;
      __pyx_t_6 = NULL;
    } else {
      __pyx_t_5 = -1; __pyx_t_3 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1284, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_6 = Py_TYPE(__pyx_t_3)->tp_iternext; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1284, __pyx_L5_error)
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    for (;;) {
      if (likely(!__pyx_t_6)) {
        if (likely(PyList_CheckExact(__pyx_t_3))) {
          if (__pyx_t_5 >= PyList_GET_SIZE(__pyx_t_3)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_2 = PyList_GET_ITEM(__pyx_t_3, __pyx_t_5); __Pyx_INCREF(__pyx_t_2); __pyx_t_5++; if (unlikely(0 < 0)) __PYX_ERR(0, 1284, __pyx_L5_error)
          #else
          __pyx_t_2 = PySequence_ITEM(__pyx_t_3, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1284, __pyx_L5_error)
          __Pyx_GOTREF(__pyx_t_2);
          #endif
        } else {
          if (__pyx_t_5 >= PyTuple_GET_SIZE(__pyx_t_3)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_2 = PyTuple_GET_ITEM(__pyx_t_3, __pyx_t_5); __Pyx_INCREF(__pyx_t_2); __pyx_t_5++; if (unlikely(0 < 0)) __PYX_ERR(0, 1284, __pyx_L5_error)
          #else
          __pyx_t_2 = PySequence_ITEM(__pyx_t_3, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1284, __pyx_L5_error)
          __Pyx_GOTREF(__pyx_t_2);
          #endif
        }
      } else {
        __pyx_t_2 = __pyx_t_6(__pyx_t_3);
        if (unlikely(!__pyx_t_2)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 1284, __pyx_L5_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_2);
      }
      if ((likely(PyTuple_CheckExact(__pyx_t_2))) || (PyList_CheckExact(__pyx_t_2))) {
        PyObject* sequence = __pyx_t_2;
        Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
        if (unlikely(size != 2)) {
          if (size > 2) __Pyx_RaiseTooManyValuesError(2);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          __PYX_ERR(0, 1284, __pyx_L5_error)
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_4 = PyTuple_GET_ITEM(sequence, 0); 
          __pyx_t_7 = PyTuple_GET_ITEM(sequence, 1); 
        } else {
          __pyx_t_4 = PyList_GET_ITEM(sequence, 0); 
          __pyx_t_7 = PyList_GET_ITEM(sequence, 1); 
        }
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_7);
        #else
        __pyx_t_4 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1284, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_7 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1284, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_7);
        #endif
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_8 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1284, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_t_9 = Py_TYPE(__pyx_t_8)->tp_iternext;
        index = 0; __pyx_t_4 = __pyx_t_9(__pyx_t_8); if (unlikely(!__pyx_t_4)) goto __pyx_L8_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_4);
        index = 1; __pyx_t_7 = __pyx_t_9(__pyx_t_8); if (unlikely(!__pyx_t_7)) goto __pyx_L8_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_7);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_9(__pyx_t_8), 2) < 0) __PYX_ERR(0, 1284, __pyx_L5_error)
        __pyx_t_9 = NULL;
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        goto __pyx_L9_unpacking_done;
        __pyx_L8_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_t_9 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        __PYX_ERR(0, 1284, __pyx_L5_error)
        __pyx_L9_unpacking_done:;
      }
      __Pyx_XDECREF_SET(__pyx_8genexpr1__pyx_v_key, __pyx_t_4);
      __pyx_t_4 = 0;
      __Pyx_XDECREF_SET(__pyx_8genexpr1__pyx_v_value, __pyx_t_7);
      __pyx_t_7 = 0;
      __pyx_t_2 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), __pyx_8genexpr1__pyx_v_key); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1284, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_10 = (__pyx_8genexpr1__pyx_v_value != Py_None);
      if ((__pyx_t_10 != 0)) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->child, __pyx_n_s_serialize); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1284, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_11 = NULL;
        __pyx_t_12 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_8))) {
          __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_8);
          if (likely(__pyx_t_11)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
            __Pyx_INCREF(__pyx_t_11);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_8, function);
            __pyx_t_12 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_8)) {
          PyObject *__pyx_temp[3] = {__pyx_t_11, __pyx_8genexpr1__pyx_v_value, __pyx_v_context};
          __pyx_t_4 = __Pyx_PyFunction_FastCall(__pyx_t_8, __pyx_temp+1-__pyx_t_12, 2+__pyx_t_12); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1284, __pyx_L5_error)
          __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_GOTREF(__pyx_t_4);
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_8)) {
          PyObject *__pyx_temp[3] = {__pyx_t_11, __pyx_8genexpr1__pyx_v_value, __pyx_v_context};
          __pyx_t_4 = __Pyx_PyCFunction_FastCall(__pyx_t_8, __pyx_temp+1-__pyx_t_12, 2+__pyx_t_12); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1284, __pyx_L5_error)
          __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_GOTREF(__pyx_t_4);
        } else
        #endif
        {
          __pyx_t_13 = PyTuple_New(2+__pyx_t_12); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1284, __pyx_L5_error)
          __Pyx_GOTREF(__pyx_t_13);
          if (__pyx_t_11) {
            __Pyx_GIVEREF(__pyx_t_11); PyTuple_SET_ITEM(__pyx_t_13, 0, __pyx_t_11); __pyx_t_11 = NULL;
          }
          __Pyx_INCREF(__pyx_8genexpr1__pyx_v_value);
          __Pyx_GIVEREF(__pyx_8genexpr1__pyx_v_value);
          PyTuple_SET_ITEM(__pyx_t_13, 0+__pyx_t_12, __pyx_8genexpr1__pyx_v_value);
          __Pyx_INCREF(__pyx_v_context);
          __Pyx_GIVEREF(__pyx_v_context);
          PyTuple_SET_ITEM(__pyx_t_13, 1+__pyx_t_12, __pyx_v_context);
          __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_t_13, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1284, __pyx_L5_error)
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        }
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_t_7 = __pyx_t_4;
        __pyx_t_4 = 0;
      } else {
        __Pyx_INCREF(Py_None);
        __pyx_t_7 = Py_None;
      }
      if (unlikely(PyDict_SetItem(__pyx_t_1, (PyObject*)__pyx_t_2, (PyObject*)__pyx_t_7))) __PYX_ERR(0, 1284, __pyx_L5_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_XDECREF(__pyx_8genexpr1__pyx_v_key); __pyx_8genexpr1__pyx_v_key = 0;
    __Pyx_XDECREF(__pyx_8genexpr1__pyx_v_value); __pyx_8genexpr1__pyx_v_value = 0;
    goto __pyx_L10_exit_scope;
    __pyx_L5_error:;
    __Pyx_XDECREF(__pyx_8genexpr1__pyx_v_key); __pyx_8genexpr1__pyx_v_key = 0;
    __Pyx_XDECREF(__pyx_8genexpr1__pyx_v_value); __pyx_8genexpr1__pyx_v_value = 0;
    goto __pyx_L1_error;
    __pyx_L10_exit_scope:;
  } /* exit inner scope */
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "drf_turbo/fields.pyx":1279
 *             self.child.bind(field_name='', root=self)
 * 
 *     cpdef inline serialize(self, data,dict context):             # <<<<<<<<<<<<<<
 *         """
 *         List of object instances -> List of dicts of primitive datatypes.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_AddTraceback("drf_turbo.fields.DictField.serialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_8genexpr1__pyx_v_key);
  __Pyx_XDECREF(__pyx_8genexpr1__pyx_v_value);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_9DictField_3serialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_9drf_turbo_6fields_9DictField_2serialize[] = "\n        List of object instances -> List of dicts of primitive datatypes.\n        ";
static PyObject *__pyx_pw_9drf_turbo_6fields_9DictField_3serialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_data = 0;
  PyObject *__pyx_v_context = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("serialize (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_data,&__pyx_n_s_context,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_data)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_context)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("serialize", 1, 2, 2, 1); __PYX_ERR(0, 1279, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "serialize") < 0)) __PYX_ERR(0, 1279, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_data = values[0];
    __pyx_v_context = ((PyObject*)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("serialize", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1279, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.DictField.serialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_context), (&PyDict_Type), 1, "context", 1))) __PYX_ERR(0, 1279, __pyx_L1_error)
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9DictField_2serialize(((struct __pyx_obj_9drf_turbo_6fields_DictField *)__pyx_v_self), __pyx_v_data, __pyx_v_context);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_9DictField_2serialize(struct __pyx_obj_9drf_turbo_6fields_DictField *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("serialize", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields_9DictField_serialize(__pyx_v_self, __pyx_v_data, __pyx_v_context, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1279, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.DictField.serialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":1287
 *         }
 * 
 *     cpdef inline deserialize(self,data,dict context):             # <<<<<<<<<<<<<<
 *         """
 *         List of dicts of native values <- List of dicts of primitive datatypes.
 */

static PyObject *__pyx_pw_9drf_turbo_6fields_9DictField_5deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_9DictField_deserialize(struct __pyx_obj_9drf_turbo_6fields_DictField *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("deserialize", 0);

  /* "drf_turbo/fields.pyx":1291
 *         List of dicts of native values <- List of dicts of primitive datatypes.
 *         """
 *         if not isinstance(data,dict):             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail('not_a_dict', input_type=type(data).__name__)
 *         if not self.allow_empty and len(data) == 0:
 */
  __pyx_t_1 = PyDict_Check(__pyx_v_data); 
  __pyx_t_2 = ((!(__pyx_t_1 != 0)) != 0);
  if (unlikely(__pyx_t_2)) {

    /* "drf_turbo/fields.pyx":1292
 *         """
 *         if not isinstance(data,dict):
 *             raise self.raise_if_fail('not_a_dict', input_type=type(data).__name__)             # <<<<<<<<<<<<<<
 *         if not self.allow_empty and len(data) == 0:
 *             raise self.raise_if_fail('empty')
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_raise_if_fail); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1292, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1292, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)Py_TYPE(__pyx_v_data)), __pyx_n_s_name); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1292, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_input_type, __pyx_t_5) < 0) __PYX_ERR(0, 1292, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_tuple__27, __pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1292, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_Raise(__pyx_t_5, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __PYX_ERR(0, 1292, __pyx_L1_error)

    /* "drf_turbo/fields.pyx":1291
 *         List of dicts of native values <- List of dicts of primitive datatypes.
 *         """
 *         if not isinstance(data,dict):             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail('not_a_dict', input_type=type(data).__name__)
 *         if not self.allow_empty and len(data) == 0:
 */
  }

  /* "drf_turbo/fields.pyx":1293
 *         if not isinstance(data,dict):
 *             raise self.raise_if_fail('not_a_dict', input_type=type(data).__name__)
 *         if not self.allow_empty and len(data) == 0:             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail('empty')
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_self->allow_empty); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 1293, __pyx_L1_error)
  __pyx_t_6 = ((!__pyx_t_1) != 0);
  if (__pyx_t_6) {
  } else {
    __pyx_t_2 = __pyx_t_6;
    goto __pyx_L5_bool_binop_done;
  }
  __pyx_t_7 = PyObject_Length(__pyx_v_data); if (unlikely(__pyx_t_7 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1293, __pyx_L1_error)
  __pyx_t_6 = ((__pyx_t_7 == 0) != 0);
  __pyx_t_2 = __pyx_t_6;
  __pyx_L5_bool_binop_done:;
  if (unlikely(__pyx_t_2)) {

    /* "drf_turbo/fields.pyx":1294
 *             raise self.raise_if_fail('not_a_dict', input_type=type(data).__name__)
 *         if not self.allow_empty and len(data) == 0:
 *             raise self.raise_if_fail('empty')             # <<<<<<<<<<<<<<
 * 
 *         return self.run_child_validation(data,context)
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_raise_if_fail); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1294, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_3 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    __pyx_t_5 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_3, __pyx_n_s_empty) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_n_s_empty);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1294, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_Raise(__pyx_t_5, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __PYX_ERR(0, 1294, __pyx_L1_error)

    /* "drf_turbo/fields.pyx":1293
 *         if not isinstance(data,dict):
 *             raise self.raise_if_fail('not_a_dict', input_type=type(data).__name__)
 *         if not self.allow_empty and len(data) == 0:             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail('empty')
 * 
 */
  }

  /* "drf_turbo/fields.pyx":1296
 *             raise self.raise_if_fail('empty')
 * 
 *         return self.run_child_validation(data,context)             # <<<<<<<<<<<<<<
 * 
 *     cpdef inline run_child_validation(self,dict data,dict context):
 */
  __Pyx_XDECREF(__pyx_r);
  if (!(likely(PyDict_CheckExact(__pyx_v_data))||((__pyx_v_data) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_v_data)->tp_name), 0))) __PYX_ERR(0, 1296, __pyx_L1_error)
  __pyx_t_5 = __pyx_f_9drf_turbo_6fields_9DictField_run_child_validation(__pyx_v_self, ((PyObject*)__pyx_v_data), __pyx_v_context, 0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1296, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_r = __pyx_t_5;
  __pyx_t_5 = 0;
  goto __pyx_L0;

  /* "drf_turbo/fields.pyx":1287
 *         }
 * 
 *     cpdef inline deserialize(self,data,dict context):             # <<<<<<<<<<<<<<
 *         """
 *         List of dicts of native values <- List of dicts of primitive datatypes.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("drf_turbo.fields.DictField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_9DictField_5deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_9drf_turbo_6fields_9DictField_4deserialize[] = "\n        List of dicts of native values <- List of dicts of primitive datatypes.\n        ";
static PyObject *__pyx_pw_9drf_turbo_6fields_9DictField_5deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_data = 0;
  PyObject *__pyx_v_context = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("deserialize (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_data,&__pyx_n_s_context,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_data)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_context)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("deserialize", 1, 2, 2, 1); __PYX_ERR(0, 1287, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "deserialize") < 0)) __PYX_ERR(0, 1287, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_data = values[0];
    __pyx_v_context = ((PyObject*)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("deserialize", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1287, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.DictField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_context), (&PyDict_Type), 1, "context", 1))) __PYX_ERR(0, 1287, __pyx_L1_error)
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9DictField_4deserialize(((struct __pyx_obj_9drf_turbo_6fields_DictField *)__pyx_v_self), __pyx_v_data, __pyx_v_context);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_9DictField_4deserialize(struct __pyx_obj_9drf_turbo_6fields_DictField *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("deserialize", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields_9DictField_deserialize(__pyx_v_self, __pyx_v_data, __pyx_v_context, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1287, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.DictField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":1298
 *         return self.run_child_validation(data,context)
 * 
 *     cpdef inline run_child_validation(self,dict data,dict context):             # <<<<<<<<<<<<<<
 *         cdef dict result = {}
 *         cdef dict errors = {}
 */

static PyObject *__pyx_pw_9drf_turbo_6fields_9DictField_7run_child_validation(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_9DictField_run_child_validation(struct __pyx_obj_9drf_turbo_6fields_DictField *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_v_result = 0;
  PyObject *__pyx_v_errors = 0;
  PyObject *__pyx_v_key = NULL;
  PyObject *__pyx_v_value = NULL;
  PyObject *__pyx_v_e = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  PyObject *(*__pyx_t_4)(PyObject *);
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *(*__pyx_t_8)(PyObject *);
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  int __pyx_t_12;
  int __pyx_t_13;
  int __pyx_t_14;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("run_child_validation", 0);

  /* "drf_turbo/fields.pyx":1299
 * 
 *     cpdef inline run_child_validation(self,dict data,dict context):
 *         cdef dict result = {}             # <<<<<<<<<<<<<<
 *         cdef dict errors = {}
 *         for key,value in data.items():
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1299, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_result = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "drf_turbo/fields.pyx":1300
 *     cpdef inline run_child_validation(self,dict data,dict context):
 *         cdef dict result = {}
 *         cdef dict errors = {}             # <<<<<<<<<<<<<<
 *         for key,value in data.items():
 *             try:
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1300, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_errors = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "drf_turbo/fields.pyx":1301
 *         cdef dict result = {}
 *         cdef dict errors = {}
 *         for key,value in data.items():             # <<<<<<<<<<<<<<
 *             try:
 *                 result[str(key)] = self.child.run_validation(value,context)
 */
  if (unlikely(__pyx_v_data == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "items");
    __PYX_ERR(0, 1301, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_Items(__pyx_v_data); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1301, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
    __pyx_t_2 = __pyx_t_1; __Pyx_INCREF(__pyx_t_2); __pyx_t_3 = 0;
    __pyx_t_4 = NULL;
  } else {
    __pyx_t_3 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1301, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1301, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  for (;;) {
    if (likely(!__pyx_t_4)) {
      if (likely(PyList_CheckExact(__pyx_t_2))) {
        if (__pyx_t_3 >= PyList_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_3); __Pyx_INCREF(__pyx_t_1); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 1301, __pyx_L1_error)
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_2, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1301, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      } else {
        if (__pyx_t_3 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_3); __Pyx_INCREF(__pyx_t_1); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 1301, __pyx_L1_error)
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_2, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1301, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      }
    } else {
      __pyx_t_1 = __pyx_t_4(__pyx_t_2);
      if (unlikely(!__pyx_t_1)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 1301, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_1);
    }
    if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
      PyObject* sequence = __pyx_t_1;
      Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
      if (unlikely(size != 2)) {
        if (size > 2) __Pyx_RaiseTooManyValuesError(2);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 1301, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      if (likely(PyTuple_CheckExact(sequence))) {
        __pyx_t_5 = PyTuple_GET_ITEM(sequence, 0); 
        __pyx_t_6 = PyTuple_GET_ITEM(sequence, 1); 
      } else {
        __pyx_t_5 = PyList_GET_ITEM(sequence, 0); 
        __pyx_t_6 = PyList_GET_ITEM(sequence, 1); 
      }
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_6);
      #else
      __pyx_t_5 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1301, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_6 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1301, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    } else {
      Py_ssize_t index = -1;
      __pyx_t_7 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1301, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_8 = Py_TYPE(__pyx_t_7)->tp_iternext;
      index = 0; __pyx_t_5 = __pyx_t_8(__pyx_t_7); if (unlikely(!__pyx_t_5)) goto __pyx_L5_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_5);
      index = 1; __pyx_t_6 = __pyx_t_8(__pyx_t_7); if (unlikely(!__pyx_t_6)) goto __pyx_L5_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_6);
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_8(__pyx_t_7), 2) < 0) __PYX_ERR(0, 1301, __pyx_L1_error)
      __pyx_t_8 = NULL;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      goto __pyx_L6_unpacking_done;
      __pyx_L5_unpacking_failed:;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_8 = NULL;
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
      __PYX_ERR(0, 1301, __pyx_L1_error)
      __pyx_L6_unpacking_done:;
    }
    __Pyx_XDECREF_SET(__pyx_v_key, __pyx_t_5);
    __pyx_t_5 = 0;
    __Pyx_XDECREF_SET(__pyx_v_value, __pyx_t_6);
    __pyx_t_6 = 0;

    /* "drf_turbo/fields.pyx":1302
 *         cdef dict errors = {}
 *         for key,value in data.items():
 *             try:             # <<<<<<<<<<<<<<
 *                 result[str(key)] = self.child.run_validation(value,context)
 *             except ValidationError as e:
 */
    {
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __Pyx_ExceptionSave(&__pyx_t_9, &__pyx_t_10, &__pyx_t_11);
      __Pyx_XGOTREF(__pyx_t_9);
      __Pyx_XGOTREF(__pyx_t_10);
      __Pyx_XGOTREF(__pyx_t_11);
      /*try:*/ {

        /* "drf_turbo/fields.pyx":1303
 *         for key,value in data.items():
 *             try:
 *                 result[str(key)] = self.child.run_validation(value,context)             # <<<<<<<<<<<<<<
 *             except ValidationError as e:
 *                 errors[key] = e.detail
 */
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->child, __pyx_n_s_run_validation); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1303, __pyx_L7_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_5 = NULL;
        __pyx_t_12 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
          __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_6);
          if (likely(__pyx_t_5)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
            __Pyx_INCREF(__pyx_t_5);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_6, function);
            __pyx_t_12 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_6)) {
          PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_v_value, __pyx_v_context};
          __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_12, 2+__pyx_t_12); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1303, __pyx_L7_error)
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_GOTREF(__pyx_t_1);
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_6)) {
          PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_v_value, __pyx_v_context};
          __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_12, 2+__pyx_t_12); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1303, __pyx_L7_error)
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_GOTREF(__pyx_t_1);
        } else
        #endif
        {
          __pyx_t_7 = PyTuple_New(2+__pyx_t_12); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1303, __pyx_L7_error)
          __Pyx_GOTREF(__pyx_t_7);
          if (__pyx_t_5) {
            __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_5); __pyx_t_5 = NULL;
          }
          __Pyx_INCREF(__pyx_v_value);
          __Pyx_GIVEREF(__pyx_v_value);
          PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_12, __pyx_v_value);
          __Pyx_INCREF(__pyx_v_context);
          __Pyx_GIVEREF(__pyx_v_context);
          PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_12, __pyx_v_context);
          __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_7, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1303, __pyx_L7_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        }
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __pyx_t_6 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), __pyx_v_key); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1303, __pyx_L7_error)
        __Pyx_GOTREF(__pyx_t_6);
        if (unlikely(PyDict_SetItem(__pyx_v_result, __pyx_t_6, __pyx_t_1) < 0)) __PYX_ERR(0, 1303, __pyx_L7_error)
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

        /* "drf_turbo/fields.pyx":1302
 *         cdef dict errors = {}
 *         for key,value in data.items():
 *             try:             # <<<<<<<<<<<<<<
 *                 result[str(key)] = self.child.run_validation(value,context)
 *             except ValidationError as e:
 */
      }
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
      goto __pyx_L14_try_end;
      __pyx_L7_error:;
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;

      /* "drf_turbo/fields.pyx":1304
 *             try:
 *                 result[str(key)] = self.child.run_validation(value,context)
 *             except ValidationError as e:             # <<<<<<<<<<<<<<
 *                 errors[key] = e.detail
 * 
 */
      __Pyx_ErrFetch(&__pyx_t_1, &__pyx_t_6, &__pyx_t_7);
      __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_ValidationError); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1304, __pyx_L9_except_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_12 = __Pyx_PyErr_GivenExceptionMatches(__pyx_t_1, __pyx_t_5);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_ErrRestore(__pyx_t_1, __pyx_t_6, __pyx_t_7);
      __pyx_t_1 = 0; __pyx_t_6 = 0; __pyx_t_7 = 0;
      if (__pyx_t_12) {
        __Pyx_AddTraceback("drf_turbo.fields.DictField.run_child_validation", __pyx_clineno, __pyx_lineno, __pyx_filename);
        if (__Pyx_GetException(&__pyx_t_7, &__pyx_t_6, &__pyx_t_1) < 0) __PYX_ERR(0, 1304, __pyx_L9_except_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_XDECREF_SET(__pyx_v_e, __pyx_t_6);

        /* "drf_turbo/fields.pyx":1305
 *                 result[str(key)] = self.child.run_validation(value,context)
 *             except ValidationError as e:
 *                 errors[key] = e.detail             # <<<<<<<<<<<<<<
 * 
 *         if not errors:
 */
        __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_e, __pyx_n_s_detail); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1305, __pyx_L9_except_error)
        __Pyx_GOTREF(__pyx_t_5);
        if (unlikely(PyDict_SetItem(__pyx_v_errors, __pyx_v_key, __pyx_t_5) < 0)) __PYX_ERR(0, 1305, __pyx_L9_except_error)
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L8_exception_handled;
      }
      goto __pyx_L9_except_error;
      __pyx_L9_except_error:;

      /* "drf_turbo/fields.pyx":1302
 *         cdef dict errors = {}
 *         for key,value in data.items():
 *             try:             # <<<<<<<<<<<<<<
 *                 result[str(key)] = self.child.run_validation(value,context)
 *             except ValidationError as e:
 */
      __Pyx_XGIVEREF(__pyx_t_9);
      __Pyx_XGIVEREF(__pyx_t_10);
      __Pyx_XGIVEREF(__pyx_t_11);
      __Pyx_ExceptionReset(__pyx_t_9, __pyx_t_10, __pyx_t_11);
      goto __pyx_L1_error;
      __pyx_L8_exception_handled:;
      __Pyx_XGIVEREF(__pyx_t_9);
      __Pyx_XGIVEREF(__pyx_t_10);
      __Pyx_XGIVEREF(__pyx_t_11);
      __Pyx_ExceptionReset(__pyx_t_9, __pyx_t_10, __pyx_t_11);
      __pyx_L14_try_end:;
    }

    /* "drf_turbo/fields.pyx":1301
 *         cdef dict result = {}
 *         cdef dict errors = {}
 *         for key,value in data.items():             # <<<<<<<<<<<<<<
 *             try:
 *                 result[str(key)] = self.child.run_validation(value,context)
 */
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "drf_turbo/fields.pyx":1307
 *                 errors[key] = e.detail
 * 
 *         if not errors:             # <<<<<<<<<<<<<<
 *             return result
 *         raise ValidationError(errors)
 */
  __pyx_t_13 = __Pyx_PyObject_IsTrue(__pyx_v_errors); if (unlikely(__pyx_t_13 < 0)) __PYX_ERR(0, 1307, __pyx_L1_error)
  __pyx_t_14 = ((!__pyx_t_13) != 0);
  if (__pyx_t_14) {

    /* "drf_turbo/fields.pyx":1308
 * 
 *         if not errors:
 *             return result             # <<<<<<<<<<<<<<
 *         raise ValidationError(errors)
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_result);
    __pyx_r = __pyx_v_result;
    goto __pyx_L0;

    /* "drf_turbo/fields.pyx":1307
 *                 errors[key] = e.detail
 * 
 *         if not errors:             # <<<<<<<<<<<<<<
 *             return result
 *         raise ValidationError(errors)
 */
  }

  /* "drf_turbo/fields.pyx":1309
 *         if not errors:
 *             return result
 *         raise ValidationError(errors)             # <<<<<<<<<<<<<<
 * 
 * @cython.final
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_ValidationError); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1309, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
    }
  }
  __pyx_t_2 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_6, __pyx_v_errors) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_v_errors);
  __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1309, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_Raise(__pyx_t_2, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __PYX_ERR(0, 1309, __pyx_L1_error)

  /* "drf_turbo/fields.pyx":1298
 *         return self.run_child_validation(data,context)
 * 
 *     cpdef inline run_child_validation(self,dict data,dict context):             # <<<<<<<<<<<<<<
 *         cdef dict result = {}
 *         cdef dict errors = {}
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("drf_turbo.fields.DictField.run_child_validation", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_result);
  __Pyx_XDECREF(__pyx_v_errors);
  __Pyx_XDECREF(__pyx_v_key);
  __Pyx_XDECREF(__pyx_v_value);
  __Pyx_XDECREF(__pyx_v_e);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_9DictField_7run_child_validation(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_9DictField_7run_child_validation(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_data = 0;
  PyObject *__pyx_v_context = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("run_child_validation (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_data,&__pyx_n_s_context,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_data)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_context)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("run_child_validation", 1, 2, 2, 1); __PYX_ERR(0, 1298, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "run_child_validation") < 0)) __PYX_ERR(0, 1298, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_data = ((PyObject*)values[0]);
    __pyx_v_context = ((PyObject*)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("run_child_validation", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1298, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.DictField.run_child_validation", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_data), (&PyDict_Type), 1, "data", 1))) __PYX_ERR(0, 1298, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_context), (&PyDict_Type), 1, "context", 1))) __PYX_ERR(0, 1298, __pyx_L1_error)
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9DictField_6run_child_validation(((struct __pyx_obj_9drf_turbo_6fields_DictField *)__pyx_v_self), __pyx_v_data, __pyx_v_context);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_9DictField_6run_child_validation(struct __pyx_obj_9drf_turbo_6fields_DictField *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("run_child_validation", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields_9DictField_run_child_validation(__pyx_v_self, __pyx_v_data, __pyx_v_context, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1298, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.DictField.run_child_validation", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pxd":142
 * cdef class DictField(Field):
 *     cdef public :
 *         child             # <<<<<<<<<<<<<<
 *         allow_empty
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_9DictField_5child_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_9DictField_5child_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9DictField_5child___get__(((struct __pyx_obj_9drf_turbo_6fields_DictField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_9DictField_5child___get__(struct __pyx_obj_9drf_turbo_6fields_DictField *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->child);
  __pyx_r = __pyx_v_self->child;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_9DictField_5child_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_9DictField_5child_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9DictField_5child_2__set__(((struct __pyx_obj_9drf_turbo_6fields_DictField *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_9DictField_5child_2__set__(struct __pyx_obj_9drf_turbo_6fields_DictField *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->child);
  __Pyx_DECREF(__pyx_v_self->child);
  __pyx_v_self->child = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_9DictField_5child_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_9DictField_5child_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9DictField_5child_4__del__(((struct __pyx_obj_9drf_turbo_6fields_DictField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_9DictField_5child_4__del__(struct __pyx_obj_9drf_turbo_6fields_DictField *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->child);
  __Pyx_DECREF(__pyx_v_self->child);
  __pyx_v_self->child = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pxd":143
 *     cdef public :
 *         child
 *         allow_empty             # <<<<<<<<<<<<<<
 * 
 *     cpdef run_child_validation(self,dict data,dict context)
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_9DictField_11allow_empty_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_9DictField_11allow_empty_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9DictField_11allow_empty___get__(((struct __pyx_obj_9drf_turbo_6fields_DictField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_9DictField_11allow_empty___get__(struct __pyx_obj_9drf_turbo_6fields_DictField *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->allow_empty);
  __pyx_r = __pyx_v_self->allow_empty;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_9DictField_11allow_empty_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_9DictField_11allow_empty_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9DictField_11allow_empty_2__set__(((struct __pyx_obj_9drf_turbo_6fields_DictField *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_9DictField_11allow_empty_2__set__(struct __pyx_obj_9drf_turbo_6fields_DictField *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->allow_empty);
  __Pyx_DECREF(__pyx_v_self->allow_empty);
  __pyx_v_self->allow_empty = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_9DictField_11allow_empty_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_9DictField_11allow_empty_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9DictField_11allow_empty_4__del__(((struct __pyx_obj_9drf_turbo_6fields_DictField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_9DictField_11allow_empty_4__del__(struct __pyx_obj_9drf_turbo_6fields_DictField *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->allow_empty);
  __Pyx_DECREF(__pyx_v_self->allow_empty);
  __pyx_v_self->allow_empty = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_9DictField_9__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_9DictField_9__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9DictField_8__reduce_cython__(((struct __pyx_obj_9drf_turbo_6fields_DictField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_9DictField_8__reduce_cython__(struct __pyx_obj_9drf_turbo_6fields_DictField *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self.allow_empty, self.allow_null, self.attr, self.attrs, self.call, self.child, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.allow_null); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.call); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.read_only); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.required); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.write_only); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = PyTuple_New(18); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_INCREF(__pyx_v_self->allow_empty);
  __Pyx_GIVEREF(__pyx_v_self->allow_empty);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v_self->allow_empty);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.attr);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.attr);
  PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_self->__pyx_base.attr);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.attrs);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.attrs);
  PyTuple_SET_ITEM(__pyx_t_6, 3, __pyx_v_self->__pyx_base.attrs);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_6, 4, __pyx_t_2);
  __Pyx_INCREF(__pyx_v_self->child);
  __Pyx_GIVEREF(__pyx_v_self->child);
  PyTuple_SET_ITEM(__pyx_t_6, 5, __pyx_v_self->child);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.default_value);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.default_value);
  PyTuple_SET_ITEM(__pyx_t_6, 6, __pyx_v_self->__pyx_base.default_value);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.error_messages);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.error_messages);
  PyTuple_SET_ITEM(__pyx_t_6, 7, __pyx_v_self->__pyx_base.error_messages);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.field_name);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.field_name);
  PyTuple_SET_ITEM(__pyx_t_6, 8, __pyx_v_self->__pyx_base.field_name);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.help_text);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.help_text);
  PyTuple_SET_ITEM(__pyx_t_6, 9, __pyx_v_self->__pyx_base.help_text);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.initial);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.initial);
  PyTuple_SET_ITEM(__pyx_t_6, 10, __pyx_v_self->__pyx_base.initial);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.label);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.label);
  PyTuple_SET_ITEM(__pyx_t_6, 11, __pyx_v_self->__pyx_base.label);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 12, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_6, 13, __pyx_t_4);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.root);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.root);
  PyTuple_SET_ITEM(__pyx_t_6, 14, __pyx_v_self->__pyx_base.root);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.style);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.style);
  PyTuple_SET_ITEM(__pyx_t_6, 15, __pyx_v_self->__pyx_base.style);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.validators);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.validators);
  PyTuple_SET_ITEM(__pyx_t_6, 16, __pyx_v_self->__pyx_base.validators);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_6, 17, __pyx_t_5);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_6);
  __pyx_t_6 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self.allow_empty, self.allow_null, self.attr, self.attrs, self.call, self.child, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_6 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_v__dict = __pyx_t_6;
  __pyx_t_6 = 0;

  /* "(tree fragment)":7
 *     state = (self.allow_empty, self.allow_null, self.attr, self.attrs, self.call, self.child, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_7 = (__pyx_v__dict != Py_None);
  __pyx_t_8 = (__pyx_t_7 != 0);
  if (__pyx_t_8) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_6 = PyTuple_New(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v__dict);
    __pyx_t_5 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_6); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_5));
    __pyx_t_5 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.allow_empty is not None or self.attr is not None or self.attrs is not None or self.child is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.root is not None or self.style is not None or self.validators is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self.allow_empty, self.allow_null, self.attr, self.attrs, self.call, self.child, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self.allow_empty is not None or self.attr is not None or self.attrs is not None or self.child is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.root is not None or self.style is not None or self.validators is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_DictField, (type(self), 0x033cda3, None), state
 */
  /*else*/ {
    __pyx_t_7 = (__pyx_v_self->allow_empty != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.attr != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.attrs != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->child != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.default_value != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.error_messages != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.field_name != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.help_text != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.initial != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.label != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.root != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.style != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.validators != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    __pyx_t_8 = __pyx_t_9;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_8;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.allow_empty is not None or self.attr is not None or self.attrs is not None or self.child is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.root is not None or self.style is not None or self.validators is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_DictField, (type(self), 0x033cda3, None), state
 *     else:
 */
  __pyx_t_8 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_8) {

    /* "(tree fragment)":13
 *         use_setstate = self.allow_empty is not None or self.attr is not None or self.attrs is not None or self.child is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.root is not None or self.style is not None or self.validators is not None
 *     if use_setstate:
 *         return __pyx_unpickle_DictField, (type(self), 0x033cda3, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_DictField, (type(self), 0x033cda3, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_pyx_unpickle_DictField); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_6, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_3394979);
    __Pyx_GIVEREF(__pyx_int_3394979);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_int_3394979);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_6, 2, Py_None);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_6);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_state);
    __pyx_t_5 = 0;
    __pyx_t_6 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.allow_empty is not None or self.attr is not None or self.attrs is not None or self.child is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.root is not None or self.style is not None or self.validators is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_DictField, (type(self), 0x033cda3, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_DictField, (type(self), 0x033cda3, None), state
 *     else:
 *         return __pyx_unpickle_DictField, (type(self), 0x033cda3, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_DictField__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_pyx_unpickle_DictField); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_6, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_3394979);
    __Pyx_GIVEREF(__pyx_int_3394979);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_int_3394979);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_state);
    __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_6);
    __pyx_t_4 = 0;
    __pyx_t_6 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("drf_turbo.fields.DictField.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_DictField, (type(self), 0x033cda3, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_DictField__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_9DictField_11__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_9DictField_11__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9DictField_10__setstate_cython__(((struct __pyx_obj_9drf_turbo_6fields_DictField *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_9DictField_10__setstate_cython__(struct __pyx_obj_9drf_turbo_6fields_DictField *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_DictField, (type(self), 0x033cda3, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_DictField__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields___pyx_unpickle_DictField__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_DictField, (type(self), 0x033cda3, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_DictField__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.DictField.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":1324
 *     }
 * 
 *     def __init__(self, **kwargs):             # <<<<<<<<<<<<<<
 *         self.binary = kwargs.pop('binary', False)
 *         self.encoder = kwargs.pop('encoder', None)
 */

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_9JSONField_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_9JSONField_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_kwargs = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  if (unlikely(PyTuple_GET_SIZE(__pyx_args) > 0)) {
    __Pyx_RaiseArgtupleInvalid("__init__", 1, 0, 0, PyTuple_GET_SIZE(__pyx_args)); return -1;}
  if (__pyx_kwds && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "__init__", 1))) return -1;
  __pyx_v_kwargs = (__pyx_kwds) ? PyDict_Copy(__pyx_kwds) : PyDict_New(); if (unlikely(!__pyx_v_kwargs)) return -1;
  __Pyx_GOTREF(__pyx_v_kwargs);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9JSONField___init__(((struct __pyx_obj_9drf_turbo_6fields_JSONField *)__pyx_v_self), __pyx_v_kwargs);

  /* function exit code */
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_9JSONField___init__(struct __pyx_obj_9drf_turbo_6fields_JSONField *__pyx_v_self, PyObject *__pyx_v_kwargs) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "drf_turbo/fields.pyx":1325
 * 
 *     def __init__(self, **kwargs):
 *         self.binary = kwargs.pop('binary', False)             # <<<<<<<<<<<<<<
 *         self.encoder = kwargs.pop('encoder', None)
 *         self.decoder = kwargs.pop('decoder', None)
 */
  __pyx_t_1 = __Pyx_PyDict_Pop(__pyx_v_kwargs, __pyx_n_s_binary, Py_False); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1325, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->binary);
  __Pyx_DECREF(__pyx_v_self->binary);
  __pyx_v_self->binary = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "drf_turbo/fields.pyx":1326
 *     def __init__(self, **kwargs):
 *         self.binary = kwargs.pop('binary', False)
 *         self.encoder = kwargs.pop('encoder', None)             # <<<<<<<<<<<<<<
 *         self.decoder = kwargs.pop('decoder', None)
 *         super().__init__(**kwargs)
 */
  __pyx_t_1 = __Pyx_PyDict_Pop(__pyx_v_kwargs, __pyx_n_s_encoder, Py_None); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1326, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->encoder);
  __Pyx_DECREF(__pyx_v_self->encoder);
  __pyx_v_self->encoder = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "drf_turbo/fields.pyx":1327
 *         self.binary = kwargs.pop('binary', False)
 *         self.encoder = kwargs.pop('encoder', None)
 *         self.decoder = kwargs.pop('decoder', None)             # <<<<<<<<<<<<<<
 *         super().__init__(**kwargs)
 * 
 */
  __pyx_t_1 = __Pyx_PyDict_Pop(__pyx_v_kwargs, __pyx_n_s_decoder, Py_None); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1327, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->decoder);
  __Pyx_DECREF(__pyx_v_self->decoder);
  __pyx_v_self->decoder = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "drf_turbo/fields.pyx":1328
 *         self.encoder = kwargs.pop('encoder', None)
 *         self.decoder = kwargs.pop('decoder', None)
 *         super().__init__(**kwargs)             # <<<<<<<<<<<<<<
 * 
 *     cpdef inline serialize(self,value,dict context):
 */
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1328, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_9drf_turbo_6fields_JSONField));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_9drf_turbo_6fields_JSONField));
  PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)__pyx_ptype_9drf_turbo_6fields_JSONField));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_1, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1328, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_init); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1328, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyDict_Copy(__pyx_v_kwargs); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1328, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1328, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "drf_turbo/fields.pyx":1324
 *     }
 * 
 *     def __init__(self, **kwargs):             # <<<<<<<<<<<<<<
 *         self.binary = kwargs.pop('binary', False)
 *         self.encoder = kwargs.pop('encoder', None)
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("drf_turbo.fields.JSONField.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":1330
 *         super().__init__(**kwargs)
 * 
 *     cpdef inline serialize(self,value,dict context):             # <<<<<<<<<<<<<<
 *         if self.binary:
 *             value = json.dumps(value,cls=self.encoder)
 */

static PyObject *__pyx_pw_9drf_turbo_6fields_9JSONField_3serialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_9JSONField_serialize(struct __pyx_obj_9drf_turbo_6fields_JSONField *__pyx_v_self, PyObject *__pyx_v_value, CYTHON_UNUSED PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("serialize", 0);
  __Pyx_INCREF(__pyx_v_value);

  /* "drf_turbo/fields.pyx":1331
 * 
 *     cpdef inline serialize(self,value,dict context):
 *         if self.binary:             # <<<<<<<<<<<<<<
 *             value = json.dumps(value,cls=self.encoder)
 *             value = value.encode()
 */
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_self->binary); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 1331, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "drf_turbo/fields.pyx":1332
 *     cpdef inline serialize(self,value,dict context):
 *         if self.binary:
 *             value = json.dumps(value,cls=self.encoder)             # <<<<<<<<<<<<<<
 *             value = value.encode()
 *         return value
 */
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_json); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1332, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_dumps); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1332, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1332, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_v_value);
    __Pyx_GIVEREF(__pyx_v_value);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_value);
    __pyx_t_4 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1332, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_cls, __pyx_v_self->encoder) < 0) __PYX_ERR(0, 1332, __pyx_L1_error)
    __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_2, __pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1332, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF_SET(__pyx_v_value, __pyx_t_5);
    __pyx_t_5 = 0;

    /* "drf_turbo/fields.pyx":1333
 *         if self.binary:
 *             value = json.dumps(value,cls=self.encoder)
 *             value = value.encode()             # <<<<<<<<<<<<<<
 *         return value
 * 
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_value, __pyx_n_s_encode); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1333, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_2 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_2)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    __pyx_t_5 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_4);
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1333, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF_SET(__pyx_v_value, __pyx_t_5);
    __pyx_t_5 = 0;

    /* "drf_turbo/fields.pyx":1331
 * 
 *     cpdef inline serialize(self,value,dict context):
 *         if self.binary:             # <<<<<<<<<<<<<<
 *             value = json.dumps(value,cls=self.encoder)
 *             value = value.encode()
 */
  }

  /* "drf_turbo/fields.pyx":1334
 *             value = json.dumps(value,cls=self.encoder)
 *             value = value.encode()
 *         return value             # <<<<<<<<<<<<<<
 * 
 *     cpdef inline deserialize(self,data,dict context):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_value);
  __pyx_r = __pyx_v_value;
  goto __pyx_L0;

  /* "drf_turbo/fields.pyx":1330
 *         super().__init__(**kwargs)
 * 
 *     cpdef inline serialize(self,value,dict context):             # <<<<<<<<<<<<<<
 *         if self.binary:
 *             value = json.dumps(value,cls=self.encoder)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("drf_turbo.fields.JSONField.serialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_value);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_9JSONField_3serialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_9JSONField_3serialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_value = 0;
  PyObject *__pyx_v_context = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("serialize (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_value,&__pyx_n_s_context,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_value)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_context)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("serialize", 1, 2, 2, 1); __PYX_ERR(0, 1330, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "serialize") < 0)) __PYX_ERR(0, 1330, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_value = values[0];
    __pyx_v_context = ((PyObject*)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("serialize", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1330, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.JSONField.serialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_context), (&PyDict_Type), 1, "context", 1))) __PYX_ERR(0, 1330, __pyx_L1_error)
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9JSONField_2serialize(((struct __pyx_obj_9drf_turbo_6fields_JSONField *)__pyx_v_self), __pyx_v_value, __pyx_v_context);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_9JSONField_2serialize(struct __pyx_obj_9drf_turbo_6fields_JSONField *__pyx_v_self, PyObject *__pyx_v_value, PyObject *__pyx_v_context) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("serialize", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields_9JSONField_serialize(__pyx_v_self, __pyx_v_value, __pyx_v_context, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1330, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.JSONField.serialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":1336
 *         return value
 * 
 *     cpdef inline deserialize(self,data,dict context):             # <<<<<<<<<<<<<<
 *         try:
 *             if self.binary:
 */

static PyObject *__pyx_pw_9drf_turbo_6fields_9JSONField_5deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_9JSONField_deserialize(struct __pyx_obj_9drf_turbo_6fields_JSONField *__pyx_v_self, PyObject *__pyx_v_data, CYTHON_UNUSED PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch) {
  CYTHON_UNUSED PyObject *__pyx_v_e = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("deserialize", 0);
  __Pyx_INCREF(__pyx_v_data);

  /* "drf_turbo/fields.pyx":1337
 * 
 *     cpdef inline deserialize(self,data,dict context):
 *         try:             # <<<<<<<<<<<<<<
 *             if self.binary:
 *                 if isinstance(data, bytes):
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "drf_turbo/fields.pyx":1338
 *     cpdef inline deserialize(self,data,dict context):
 *         try:
 *             if self.binary:             # <<<<<<<<<<<<<<
 *                 if isinstance(data, bytes):
 *                     data = data.decode()
 */
      __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_v_self->binary); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 1338, __pyx_L3_error)
      if (__pyx_t_4) {

        /* "drf_turbo/fields.pyx":1339
 *         try:
 *             if self.binary:
 *                 if isinstance(data, bytes):             # <<<<<<<<<<<<<<
 *                     data = data.decode()
 *                 return json.loads(data,cls=self.decoder)
 */
        __pyx_t_4 = PyBytes_Check(__pyx_v_data); 
        __pyx_t_5 = (__pyx_t_4 != 0);
        if (__pyx_t_5) {

          /* "drf_turbo/fields.pyx":1340
 *             if self.binary:
 *                 if isinstance(data, bytes):
 *                     data = data.decode()             # <<<<<<<<<<<<<<
 *                 return json.loads(data,cls=self.decoder)
 *             else:
 */
          __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_data, __pyx_n_s_decode); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1340, __pyx_L3_error)
          __Pyx_GOTREF(__pyx_t_7);
          __pyx_t_8 = NULL;
          if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
            __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);
            if (likely(__pyx_t_8)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
              __Pyx_INCREF(__pyx_t_8);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_7, function);
            }
          }
          __pyx_t_6 = (__pyx_t_8) ? __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_8) : __Pyx_PyObject_CallNoArg(__pyx_t_7);
          __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
          if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1340, __pyx_L3_error)
          __Pyx_GOTREF(__pyx_t_6);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_DECREF_SET(__pyx_v_data, __pyx_t_6);
          __pyx_t_6 = 0;

          /* "drf_turbo/fields.pyx":1339
 *         try:
 *             if self.binary:
 *                 if isinstance(data, bytes):             # <<<<<<<<<<<<<<
 *                     data = data.decode()
 *                 return json.loads(data,cls=self.decoder)
 */
        }

        /* "drf_turbo/fields.pyx":1341
 *                 if isinstance(data, bytes):
 *                     data = data.decode()
 *                 return json.loads(data,cls=self.decoder)             # <<<<<<<<<<<<<<
 *             else:
 *                 json.dumps(data,cls=self.encoder)
 */
        __Pyx_XDECREF(__pyx_r);
        __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_n_s_json); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1341, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_loads); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1341, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __pyx_t_6 = PyTuple_New(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1341, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_INCREF(__pyx_v_data);
        __Pyx_GIVEREF(__pyx_v_data);
        PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v_data);
        __pyx_t_8 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1341, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_8);
        if (PyDict_SetItem(__pyx_t_8, __pyx_n_s_cls, __pyx_v_self->decoder) < 0) __PYX_ERR(0, 1341, __pyx_L3_error)
        __pyx_t_9 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_6, __pyx_t_8); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1341, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_9);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_r = __pyx_t_9;
        __pyx_t_9 = 0;
        goto __pyx_L7_try_return;

        /* "drf_turbo/fields.pyx":1338
 *     cpdef inline deserialize(self,data,dict context):
 *         try:
 *             if self.binary:             # <<<<<<<<<<<<<<
 *                 if isinstance(data, bytes):
 *                     data = data.decode()
 */
      }

      /* "drf_turbo/fields.pyx":1343
 *                 return json.loads(data,cls=self.decoder)
 *             else:
 *                 json.dumps(data,cls=self.encoder)             # <<<<<<<<<<<<<<
 *         except (TypeError, ValueError) as e:
 *             raise self.raise_if_fail('invalid')
 */
      /*else*/ {
        __Pyx_GetModuleGlobalName(__pyx_t_9, __pyx_n_s_json); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1343, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_9);
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_9, __pyx_n_s_dumps); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1343, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        __pyx_t_9 = PyTuple_New(1); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1343, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_9);
        __Pyx_INCREF(__pyx_v_data);
        __Pyx_GIVEREF(__pyx_v_data);
        PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_v_data);
        __pyx_t_6 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1343, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_6);
        if (PyDict_SetItem(__pyx_t_6, __pyx_n_s_cls, __pyx_v_self->encoder) < 0) __PYX_ERR(0, 1343, __pyx_L3_error)
        __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_t_9, __pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1343, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      }

      /* "drf_turbo/fields.pyx":1337
 * 
 *     cpdef inline deserialize(self,data,dict context):
 *         try:             # <<<<<<<<<<<<<<
 *             if self.binary:
 *                 if isinstance(data, bytes):
 */
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;

    /* "drf_turbo/fields.pyx":1344
 *             else:
 *                 json.dumps(data,cls=self.encoder)
 *         except (TypeError, ValueError) as e:             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail('invalid')
 *         return data
 */
    __pyx_t_10 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_TypeError) || __Pyx_PyErr_ExceptionMatches(__pyx_builtin_ValueError);
    if (__pyx_t_10) {
      __Pyx_AddTraceback("drf_turbo.fields.JSONField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_7, &__pyx_t_6, &__pyx_t_9) < 0) __PYX_ERR(0, 1344, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_INCREF(__pyx_t_6);
      __pyx_v_e = __pyx_t_6;

      /* "drf_turbo/fields.pyx":1345
 *                 json.dumps(data,cls=self.encoder)
 *         except (TypeError, ValueError) as e:
 *             raise self.raise_if_fail('invalid')             # <<<<<<<<<<<<<<
 *         return data
 * 
 */
      __pyx_t_11 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_raise_if_fail); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1345, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_11);
      __pyx_t_12 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_11))) {
        __pyx_t_12 = PyMethod_GET_SELF(__pyx_t_11);
        if (likely(__pyx_t_12)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_11);
          __Pyx_INCREF(__pyx_t_12);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_11, function);
        }
      }
      __pyx_t_8 = (__pyx_t_12) ? __Pyx_PyObject_Call2Args(__pyx_t_11, __pyx_t_12, __pyx_n_s_invalid) : __Pyx_PyObject_CallOneArg(__pyx_t_11, __pyx_n_s_invalid);
      __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
      if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1345, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      __Pyx_Raise(__pyx_t_8, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __PYX_ERR(0, 1345, __pyx_L5_except_error)
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;

    /* "drf_turbo/fields.pyx":1337
 * 
 *     cpdef inline deserialize(self,data,dict context):
 *         try:             # <<<<<<<<<<<<<<
 *             if self.binary:
 *                 if isinstance(data, bytes):
 */
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L7_try_return:;
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L0;
    __pyx_L8_try_end:;
  }

  /* "drf_turbo/fields.pyx":1346
 *         except (TypeError, ValueError) as e:
 *             raise self.raise_if_fail('invalid')
 *         return data             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_data);
  __pyx_r = __pyx_v_data;
  goto __pyx_L0;

  /* "drf_turbo/fields.pyx":1336
 *         return value
 * 
 *     cpdef inline deserialize(self,data,dict context):             # <<<<<<<<<<<<<<
 *         try:
 *             if self.binary:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_AddTraceback("drf_turbo.fields.JSONField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_e);
  __Pyx_XDECREF(__pyx_v_data);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_9JSONField_5deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_9JSONField_5deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_data = 0;
  PyObject *__pyx_v_context = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("deserialize (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_data,&__pyx_n_s_context,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_data)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_context)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("deserialize", 1, 2, 2, 1); __PYX_ERR(0, 1336, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "deserialize") < 0)) __PYX_ERR(0, 1336, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_data = values[0];
    __pyx_v_context = ((PyObject*)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("deserialize", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1336, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.JSONField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_context), (&PyDict_Type), 1, "context", 1))) __PYX_ERR(0, 1336, __pyx_L1_error)
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9JSONField_4deserialize(((struct __pyx_obj_9drf_turbo_6fields_JSONField *)__pyx_v_self), __pyx_v_data, __pyx_v_context);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_9JSONField_4deserialize(struct __pyx_obj_9drf_turbo_6fields_JSONField *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("deserialize", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields_9JSONField_deserialize(__pyx_v_self, __pyx_v_data, __pyx_v_context, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1336, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.JSONField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pxd":150
 * cdef class JSONField(Field):
 *     cdef public :
 *         binary             # <<<<<<<<<<<<<<
 *         encoder
 *         decoder
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_9JSONField_6binary_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_9JSONField_6binary_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9JSONField_6binary___get__(((struct __pyx_obj_9drf_turbo_6fields_JSONField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_9JSONField_6binary___get__(struct __pyx_obj_9drf_turbo_6fields_JSONField *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->binary);
  __pyx_r = __pyx_v_self->binary;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_9JSONField_6binary_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_9JSONField_6binary_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9JSONField_6binary_2__set__(((struct __pyx_obj_9drf_turbo_6fields_JSONField *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_9JSONField_6binary_2__set__(struct __pyx_obj_9drf_turbo_6fields_JSONField *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->binary);
  __Pyx_DECREF(__pyx_v_self->binary);
  __pyx_v_self->binary = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_9JSONField_6binary_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_9JSONField_6binary_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9JSONField_6binary_4__del__(((struct __pyx_obj_9drf_turbo_6fields_JSONField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_9JSONField_6binary_4__del__(struct __pyx_obj_9drf_turbo_6fields_JSONField *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->binary);
  __Pyx_DECREF(__pyx_v_self->binary);
  __pyx_v_self->binary = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pxd":151
 *     cdef public :
 *         binary
 *         encoder             # <<<<<<<<<<<<<<
 *         decoder
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_9JSONField_7encoder_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_9JSONField_7encoder_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9JSONField_7encoder___get__(((struct __pyx_obj_9drf_turbo_6fields_JSONField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_9JSONField_7encoder___get__(struct __pyx_obj_9drf_turbo_6fields_JSONField *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->encoder);
  __pyx_r = __pyx_v_self->encoder;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_9JSONField_7encoder_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_9JSONField_7encoder_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9JSONField_7encoder_2__set__(((struct __pyx_obj_9drf_turbo_6fields_JSONField *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_9JSONField_7encoder_2__set__(struct __pyx_obj_9drf_turbo_6fields_JSONField *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->encoder);
  __Pyx_DECREF(__pyx_v_self->encoder);
  __pyx_v_self->encoder = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_9JSONField_7encoder_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_9JSONField_7encoder_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9JSONField_7encoder_4__del__(((struct __pyx_obj_9drf_turbo_6fields_JSONField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_9JSONField_7encoder_4__del__(struct __pyx_obj_9drf_turbo_6fields_JSONField *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->encoder);
  __Pyx_DECREF(__pyx_v_self->encoder);
  __pyx_v_self->encoder = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pxd":152
 *         binary
 *         encoder
 *         decoder             # <<<<<<<<<<<<<<
 * 
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_9JSONField_7decoder_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_9JSONField_7decoder_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9JSONField_7decoder___get__(((struct __pyx_obj_9drf_turbo_6fields_JSONField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_9JSONField_7decoder___get__(struct __pyx_obj_9drf_turbo_6fields_JSONField *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->decoder);
  __pyx_r = __pyx_v_self->decoder;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_9JSONField_7decoder_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_9JSONField_7decoder_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9JSONField_7decoder_2__set__(((struct __pyx_obj_9drf_turbo_6fields_JSONField *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_9JSONField_7decoder_2__set__(struct __pyx_obj_9drf_turbo_6fields_JSONField *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->decoder);
  __Pyx_DECREF(__pyx_v_self->decoder);
  __pyx_v_self->decoder = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_9JSONField_7decoder_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_9JSONField_7decoder_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9JSONField_7decoder_4__del__(((struct __pyx_obj_9drf_turbo_6fields_JSONField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_9JSONField_7decoder_4__del__(struct __pyx_obj_9drf_turbo_6fields_JSONField *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->decoder);
  __Pyx_DECREF(__pyx_v_self->decoder);
  __pyx_v_self->decoder = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_9JSONField_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_9JSONField_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9JSONField_6__reduce_cython__(((struct __pyx_obj_9drf_turbo_6fields_JSONField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_9JSONField_6__reduce_cython__(struct __pyx_obj_9drf_turbo_6fields_JSONField *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self.allow_null, self.attr, self.attrs, self.binary, self.call, self.decoder, self.default_value, self.encoder, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.allow_null); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.call); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.read_only); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.required); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.write_only); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = PyTuple_New(19); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.attr);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.attr);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_v_self->__pyx_base.attr);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.attrs);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.attrs);
  PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_self->__pyx_base.attrs);
  __Pyx_INCREF(__pyx_v_self->binary);
  __Pyx_GIVEREF(__pyx_v_self->binary);
  PyTuple_SET_ITEM(__pyx_t_6, 3, __pyx_v_self->binary);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_6, 4, __pyx_t_2);
  __Pyx_INCREF(__pyx_v_self->decoder);
  __Pyx_GIVEREF(__pyx_v_self->decoder);
  PyTuple_SET_ITEM(__pyx_t_6, 5, __pyx_v_self->decoder);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.default_value);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.default_value);
  PyTuple_SET_ITEM(__pyx_t_6, 6, __pyx_v_self->__pyx_base.default_value);
  __Pyx_INCREF(__pyx_v_self->encoder);
  __Pyx_GIVEREF(__pyx_v_self->encoder);
  PyTuple_SET_ITEM(__pyx_t_6, 7, __pyx_v_self->encoder);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.error_messages);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.error_messages);
  PyTuple_SET_ITEM(__pyx_t_6, 8, __pyx_v_self->__pyx_base.error_messages);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.field_name);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.field_name);
  PyTuple_SET_ITEM(__pyx_t_6, 9, __pyx_v_self->__pyx_base.field_name);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.help_text);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.help_text);
  PyTuple_SET_ITEM(__pyx_t_6, 10, __pyx_v_self->__pyx_base.help_text);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.initial);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.initial);
  PyTuple_SET_ITEM(__pyx_t_6, 11, __pyx_v_self->__pyx_base.initial);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.label);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.label);
  PyTuple_SET_ITEM(__pyx_t_6, 12, __pyx_v_self->__pyx_base.label);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 13, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_6, 14, __pyx_t_4);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.root);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.root);
  PyTuple_SET_ITEM(__pyx_t_6, 15, __pyx_v_self->__pyx_base.root);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.style);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.style);
  PyTuple_SET_ITEM(__pyx_t_6, 16, __pyx_v_self->__pyx_base.style);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.validators);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.validators);
  PyTuple_SET_ITEM(__pyx_t_6, 17, __pyx_v_self->__pyx_base.validators);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_6, 18, __pyx_t_5);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_6);
  __pyx_t_6 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self.allow_null, self.attr, self.attrs, self.binary, self.call, self.decoder, self.default_value, self.encoder, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_6 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_v__dict = __pyx_t_6;
  __pyx_t_6 = 0;

  /* "(tree fragment)":7
 *     state = (self.allow_null, self.attr, self.attrs, self.binary, self.call, self.decoder, self.default_value, self.encoder, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_7 = (__pyx_v__dict != Py_None);
  __pyx_t_8 = (__pyx_t_7 != 0);
  if (__pyx_t_8) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_6 = PyTuple_New(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v__dict);
    __pyx_t_5 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_6); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_5));
    __pyx_t_5 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.attr is not None or self.attrs is not None or self.binary is not None or self.decoder is not None or self.default_value is not None or self.encoder is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.root is not None or self.style is not None or self.validators is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self.allow_null, self.attr, self.attrs, self.binary, self.call, self.decoder, self.default_value, self.encoder, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self.attr is not None or self.attrs is not None or self.binary is not None or self.decoder is not None or self.default_value is not None or self.encoder is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.root is not None or self.style is not None or self.validators is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_JSONField, (type(self), 0x91e774b, None), state
 */
  /*else*/ {
    __pyx_t_7 = (__pyx_v_self->__pyx_base.attr != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.attrs != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->binary != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->decoder != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.default_value != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->encoder != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.error_messages != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.field_name != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.help_text != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.initial != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.label != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.root != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.style != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.validators != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    __pyx_t_8 = __pyx_t_7;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_8;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.attr is not None or self.attrs is not None or self.binary is not None or self.decoder is not None or self.default_value is not None or self.encoder is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.root is not None or self.style is not None or self.validators is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_JSONField, (type(self), 0x91e774b, None), state
 *     else:
 */
  __pyx_t_8 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_8) {

    /* "(tree fragment)":13
 *         use_setstate = self.attr is not None or self.attrs is not None or self.binary is not None or self.decoder is not None or self.default_value is not None or self.encoder is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.root is not None or self.style is not None or self.validators is not None
 *     if use_setstate:
 *         return __pyx_unpickle_JSONField, (type(self), 0x91e774b, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_JSONField, (type(self), 0x91e774b, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_pyx_unpickle_JSONField); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_6, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_152991563);
    __Pyx_GIVEREF(__pyx_int_152991563);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_int_152991563);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_6, 2, Py_None);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_6);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_state);
    __pyx_t_5 = 0;
    __pyx_t_6 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.attr is not None or self.attrs is not None or self.binary is not None or self.decoder is not None or self.default_value is not None or self.encoder is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.root is not None or self.style is not None or self.validators is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_JSONField, (type(self), 0x91e774b, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_JSONField, (type(self), 0x91e774b, None), state
 *     else:
 *         return __pyx_unpickle_JSONField, (type(self), 0x91e774b, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_JSONField__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_pyx_unpickle_JSONField); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_6, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_152991563);
    __Pyx_GIVEREF(__pyx_int_152991563);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_int_152991563);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_state);
    __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_6);
    __pyx_t_4 = 0;
    __pyx_t_6 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("drf_turbo.fields.JSONField.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_JSONField, (type(self), 0x91e774b, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_JSONField__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_9JSONField_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_9JSONField_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_9JSONField_8__setstate_cython__(((struct __pyx_obj_9drf_turbo_6fields_JSONField *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_9JSONField_8__setstate_cython__(struct __pyx_obj_9drf_turbo_6fields_JSONField *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_JSONField, (type(self), 0x91e774b, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_JSONField__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields___pyx_unpickle_JSONField__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_JSONField, (type(self), 0x91e774b, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_JSONField__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.JSONField.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":1360
 * 
 *     }
 *     def __init__(self,**kwargs):             # <<<<<<<<<<<<<<
 *         self.queryset = kwargs.pop('queryset', None)
 *         super().__init__(**kwargs)
 */

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_12RelatedField_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_12RelatedField_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_kwargs = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  if (unlikely(PyTuple_GET_SIZE(__pyx_args) > 0)) {
    __Pyx_RaiseArgtupleInvalid("__init__", 1, 0, 0, PyTuple_GET_SIZE(__pyx_args)); return -1;}
  if (__pyx_kwds && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "__init__", 1))) return -1;
  __pyx_v_kwargs = (__pyx_kwds) ? PyDict_Copy(__pyx_kwds) : PyDict_New(); if (unlikely(!__pyx_v_kwargs)) return -1;
  __Pyx_GOTREF(__pyx_v_kwargs);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_12RelatedField___init__(((struct __pyx_obj_9drf_turbo_6fields_RelatedField *)__pyx_v_self), __pyx_v_kwargs);

  /* function exit code */
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_12RelatedField___init__(struct __pyx_obj_9drf_turbo_6fields_RelatedField *__pyx_v_self, PyObject *__pyx_v_kwargs) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "drf_turbo/fields.pyx":1361
 *     }
 *     def __init__(self,**kwargs):
 *         self.queryset = kwargs.pop('queryset', None)             # <<<<<<<<<<<<<<
 *         super().__init__(**kwargs)
 * 
 */
  __pyx_t_1 = __Pyx_PyDict_Pop(__pyx_v_kwargs, __pyx_n_s_queryset, Py_None); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1361, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->queryset);
  __Pyx_DECREF(__pyx_v_self->queryset);
  __pyx_v_self->queryset = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "drf_turbo/fields.pyx":1362
 *     def __init__(self,**kwargs):
 *         self.queryset = kwargs.pop('queryset', None)
 *         super().__init__(**kwargs)             # <<<<<<<<<<<<<<
 * 
 *     cpdef inline deserialize(self,data,dict context):
 */
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1362, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_9drf_turbo_6fields_RelatedField));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_9drf_turbo_6fields_RelatedField));
  PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)__pyx_ptype_9drf_turbo_6fields_RelatedField));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_1, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1362, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_init); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1362, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyDict_Copy(__pyx_v_kwargs); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1362, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1362, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "drf_turbo/fields.pyx":1360
 * 
 *     }
 *     def __init__(self,**kwargs):             # <<<<<<<<<<<<<<
 *         self.queryset = kwargs.pop('queryset', None)
 *         super().__init__(**kwargs)
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("drf_turbo.fields.RelatedField.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":1364
 *         super().__init__(**kwargs)
 * 
 *     cpdef inline deserialize(self,data,dict context):             # <<<<<<<<<<<<<<
 *         try:
 *             if isinstance(data, bool):
 */

static PyObject *__pyx_pw_9drf_turbo_6fields_12RelatedField_3deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_12RelatedField_deserialize(struct __pyx_obj_9drf_turbo_6fields_RelatedField *__pyx_v_self, PyObject *__pyx_v_data, CYTHON_UNUSED PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("deserialize", 0);
  __Pyx_INCREF(__pyx_v_data);

  /* "drf_turbo/fields.pyx":1365
 * 
 *     cpdef inline deserialize(self,data,dict context):
 *         try:             # <<<<<<<<<<<<<<
 *             if isinstance(data, bool):
 *                 raise TypeError
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "drf_turbo/fields.pyx":1366
 *     cpdef inline deserialize(self,data,dict context):
 *         try:
 *             if isinstance(data, bool):             # <<<<<<<<<<<<<<
 *                 raise TypeError
 *             data = self.queryset.get(pk=data)
 */
      __pyx_t_4 = ((PyObject*)&PyBool_Type);
      __Pyx_INCREF(__pyx_t_4);
      __pyx_t_5 = PyObject_IsInstance(__pyx_v_data, __pyx_t_4); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 1366, __pyx_L3_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = (__pyx_t_5 != 0);
      if (unlikely(__pyx_t_6)) {

        /* "drf_turbo/fields.pyx":1367
 *         try:
 *             if isinstance(data, bool):
 *                 raise TypeError             # <<<<<<<<<<<<<<
 *             data = self.queryset.get(pk=data)
 *         except ObjectDoesNotExist:
 */
        __Pyx_Raise(__pyx_builtin_TypeError, 0, 0, 0);
        __PYX_ERR(0, 1367, __pyx_L3_error)

        /* "drf_turbo/fields.pyx":1366
 *     cpdef inline deserialize(self,data,dict context):
 *         try:
 *             if isinstance(data, bool):             # <<<<<<<<<<<<<<
 *                 raise TypeError
 *             data = self.queryset.get(pk=data)
 */
      }

      /* "drf_turbo/fields.pyx":1368
 *             if isinstance(data, bool):
 *                 raise TypeError
 *             data = self.queryset.get(pk=data)             # <<<<<<<<<<<<<<
 *         except ObjectDoesNotExist:
 *             raise self.raise_if_fail('does_not_exist', pk_value=data)
 */
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->queryset, __pyx_n_s_get); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1368, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_7 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1368, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_7);
      if (PyDict_SetItem(__pyx_t_7, __pyx_n_s_pk, __pyx_v_data) < 0) __PYX_ERR(0, 1368, __pyx_L3_error)
      __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_empty_tuple, __pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1368, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF_SET(__pyx_v_data, __pyx_t_8);
      __pyx_t_8 = 0;

      /* "drf_turbo/fields.pyx":1365
 * 
 *     cpdef inline deserialize(self,data,dict context):
 *         try:             # <<<<<<<<<<<<<<
 *             if isinstance(data, bool):
 *                 raise TypeError
 */
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;

    /* "drf_turbo/fields.pyx":1369
 *                 raise TypeError
 *             data = self.queryset.get(pk=data)
 *         except ObjectDoesNotExist:             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail('does_not_exist', pk_value=data)
 *         except (TypeError, ValueError):
 */
    __Pyx_ErrFetch(&__pyx_t_8, &__pyx_t_7, &__pyx_t_4);
    __Pyx_GetModuleGlobalName(__pyx_t_9, __pyx_n_s_ObjectDoesNotExist); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1369, __pyx_L5_except_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_10 = __Pyx_PyErr_GivenExceptionMatches(__pyx_t_8, __pyx_t_9);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_ErrRestore(__pyx_t_8, __pyx_t_7, __pyx_t_4);
    __pyx_t_8 = 0; __pyx_t_7 = 0; __pyx_t_4 = 0;
    if (__pyx_t_10) {
      __Pyx_AddTraceback("drf_turbo.fields.RelatedField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_4, &__pyx_t_7, &__pyx_t_8) < 0) __PYX_ERR(0, 1369, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_GOTREF(__pyx_t_8);

      /* "drf_turbo/fields.pyx":1370
 *             data = self.queryset.get(pk=data)
 *         except ObjectDoesNotExist:
 *             raise self.raise_if_fail('does_not_exist', pk_value=data)             # <<<<<<<<<<<<<<
 *         except (TypeError, ValueError):
 *             raise self.raise_if_fail('incorrect_type', data_type=type(data).__name__)
 */
      __pyx_t_9 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_raise_if_fail); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1370, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_11 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1370, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_11);
      if (PyDict_SetItem(__pyx_t_11, __pyx_n_s_pk_value, __pyx_v_data) < 0) __PYX_ERR(0, 1370, __pyx_L5_except_error)
      __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_tuple__28, __pyx_t_11); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1370, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_12);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      __Pyx_Raise(__pyx_t_12, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      __PYX_ERR(0, 1370, __pyx_L5_except_error)
    }

    /* "drf_turbo/fields.pyx":1371
 *         except ObjectDoesNotExist:
 *             raise self.raise_if_fail('does_not_exist', pk_value=data)
 *         except (TypeError, ValueError):             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail('incorrect_type', data_type=type(data).__name__)
 *         return data
 */
    __pyx_t_10 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_TypeError) || __Pyx_PyErr_ExceptionMatches(__pyx_builtin_ValueError);
    if (__pyx_t_10) {
      __Pyx_AddTraceback("drf_turbo.fields.RelatedField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_8, &__pyx_t_7, &__pyx_t_4) < 0) __PYX_ERR(0, 1371, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_GOTREF(__pyx_t_4);

      /* "drf_turbo/fields.pyx":1372
 *             raise self.raise_if_fail('does_not_exist', pk_value=data)
 *         except (TypeError, ValueError):
 *             raise self.raise_if_fail('incorrect_type', data_type=type(data).__name__)             # <<<<<<<<<<<<<<
 *         return data
 * 
 */
      __pyx_t_12 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_raise_if_fail); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1372, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_12);
      __pyx_t_11 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1372, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_11);
      __pyx_t_9 = __Pyx_PyObject_GetAttrStr(((PyObject *)Py_TYPE(__pyx_v_data)), __pyx_n_s_name); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1372, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_9);
      if (PyDict_SetItem(__pyx_t_11, __pyx_n_s_data_type, __pyx_t_9) < 0) __PYX_ERR(0, 1372, __pyx_L5_except_error)
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __pyx_t_9 = __Pyx_PyObject_Call(__pyx_t_12, __pyx_tuple__29, __pyx_t_11); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1372, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      __Pyx_Raise(__pyx_t_9, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __PYX_ERR(0, 1372, __pyx_L5_except_error)
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;

    /* "drf_turbo/fields.pyx":1365
 * 
 *     cpdef inline deserialize(self,data,dict context):
 *         try:             # <<<<<<<<<<<<<<
 *             if isinstance(data, bool):
 *                 raise TypeError
 */
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L8_try_end:;
  }

  /* "drf_turbo/fields.pyx":1373
 *         except (TypeError, ValueError):
 *             raise self.raise_if_fail('incorrect_type', data_type=type(data).__name__)
 *         return data             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_data);
  __pyx_r = __pyx_v_data;
  goto __pyx_L0;

  /* "drf_turbo/fields.pyx":1364
 *         super().__init__(**kwargs)
 * 
 *     cpdef inline deserialize(self,data,dict context):             # <<<<<<<<<<<<<<
 *         try:
 *             if isinstance(data, bool):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_AddTraceback("drf_turbo.fields.RelatedField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_data);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_12RelatedField_3deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_12RelatedField_3deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_data = 0;
  PyObject *__pyx_v_context = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("deserialize (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_data,&__pyx_n_s_context,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_data)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_context)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("deserialize", 1, 2, 2, 1); __PYX_ERR(0, 1364, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "deserialize") < 0)) __PYX_ERR(0, 1364, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_data = values[0];
    __pyx_v_context = ((PyObject*)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("deserialize", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1364, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.RelatedField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_context), (&PyDict_Type), 1, "context", 1))) __PYX_ERR(0, 1364, __pyx_L1_error)
  __pyx_r = __pyx_pf_9drf_turbo_6fields_12RelatedField_2deserialize(((struct __pyx_obj_9drf_turbo_6fields_RelatedField *)__pyx_v_self), __pyx_v_data, __pyx_v_context);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_12RelatedField_2deserialize(struct __pyx_obj_9drf_turbo_6fields_RelatedField *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("deserialize", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields_12RelatedField_deserialize(__pyx_v_self, __pyx_v_data, __pyx_v_context, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1364, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.RelatedField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pxd":157
 * cdef class RelatedField(Field):
 *     cdef public:
 *         queryset             # <<<<<<<<<<<<<<
 * 
 * cdef class ManyRelatedField(Field):
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_12RelatedField_8queryset_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_12RelatedField_8queryset_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_12RelatedField_8queryset___get__(((struct __pyx_obj_9drf_turbo_6fields_RelatedField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_12RelatedField_8queryset___get__(struct __pyx_obj_9drf_turbo_6fields_RelatedField *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->queryset);
  __pyx_r = __pyx_v_self->queryset;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_12RelatedField_8queryset_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_12RelatedField_8queryset_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_12RelatedField_8queryset_2__set__(((struct __pyx_obj_9drf_turbo_6fields_RelatedField *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_12RelatedField_8queryset_2__set__(struct __pyx_obj_9drf_turbo_6fields_RelatedField *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->queryset);
  __Pyx_DECREF(__pyx_v_self->queryset);
  __pyx_v_self->queryset = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_12RelatedField_8queryset_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_12RelatedField_8queryset_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_12RelatedField_8queryset_4__del__(((struct __pyx_obj_9drf_turbo_6fields_RelatedField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_12RelatedField_8queryset_4__del__(struct __pyx_obj_9drf_turbo_6fields_RelatedField *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->queryset);
  __Pyx_DECREF(__pyx_v_self->queryset);
  __pyx_v_self->queryset = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_12RelatedField_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_12RelatedField_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_12RelatedField_4__reduce_cython__(((struct __pyx_obj_9drf_turbo_6fields_RelatedField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_12RelatedField_4__reduce_cython__(struct __pyx_obj_9drf_turbo_6fields_RelatedField *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self.allow_null, self.attr, self.attrs, self.call, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.queryset, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.allow_null); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.call); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.read_only); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.required); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.write_only); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = PyTuple_New(17); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.attr);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.attr);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_v_self->__pyx_base.attr);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.attrs);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.attrs);
  PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_self->__pyx_base.attrs);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_6, 3, __pyx_t_2);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.default_value);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.default_value);
  PyTuple_SET_ITEM(__pyx_t_6, 4, __pyx_v_self->__pyx_base.default_value);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.error_messages);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.error_messages);
  PyTuple_SET_ITEM(__pyx_t_6, 5, __pyx_v_self->__pyx_base.error_messages);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.field_name);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.field_name);
  PyTuple_SET_ITEM(__pyx_t_6, 6, __pyx_v_self->__pyx_base.field_name);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.help_text);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.help_text);
  PyTuple_SET_ITEM(__pyx_t_6, 7, __pyx_v_self->__pyx_base.help_text);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.initial);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.initial);
  PyTuple_SET_ITEM(__pyx_t_6, 8, __pyx_v_self->__pyx_base.initial);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.label);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.label);
  PyTuple_SET_ITEM(__pyx_t_6, 9, __pyx_v_self->__pyx_base.label);
  __Pyx_INCREF(__pyx_v_self->queryset);
  __Pyx_GIVEREF(__pyx_v_self->queryset);
  PyTuple_SET_ITEM(__pyx_t_6, 10, __pyx_v_self->queryset);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 11, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_6, 12, __pyx_t_4);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.root);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.root);
  PyTuple_SET_ITEM(__pyx_t_6, 13, __pyx_v_self->__pyx_base.root);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.style);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.style);
  PyTuple_SET_ITEM(__pyx_t_6, 14, __pyx_v_self->__pyx_base.style);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.validators);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.validators);
  PyTuple_SET_ITEM(__pyx_t_6, 15, __pyx_v_self->__pyx_base.validators);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_6, 16, __pyx_t_5);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_6);
  __pyx_t_6 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self.allow_null, self.attr, self.attrs, self.call, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.queryset, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_6 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_v__dict = __pyx_t_6;
  __pyx_t_6 = 0;

  /* "(tree fragment)":7
 *     state = (self.allow_null, self.attr, self.attrs, self.call, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.queryset, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_7 = (__pyx_v__dict != Py_None);
  __pyx_t_8 = (__pyx_t_7 != 0);
  if (__pyx_t_8) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_6 = PyTuple_New(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v__dict);
    __pyx_t_5 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_6); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_5));
    __pyx_t_5 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.queryset is not None or self.root is not None or self.style is not None or self.validators is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self.allow_null, self.attr, self.attrs, self.call, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.queryset, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.queryset is not None or self.root is not None or self.style is not None or self.validators is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_RelatedField, (type(self), 0x0ca6ac6, None), state
 */
  /*else*/ {
    __pyx_t_7 = (__pyx_v_self->__pyx_base.attr != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.attrs != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.default_value != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.error_messages != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.field_name != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.help_text != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.initial != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.label != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->queryset != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.root != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.style != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.validators != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    __pyx_t_8 = __pyx_t_7;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_8;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.queryset is not None or self.root is not None or self.style is not None or self.validators is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_RelatedField, (type(self), 0x0ca6ac6, None), state
 *     else:
 */
  __pyx_t_8 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_8) {

    /* "(tree fragment)":13
 *         use_setstate = self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.queryset is not None or self.root is not None or self.style is not None or self.validators is not None
 *     if use_setstate:
 *         return __pyx_unpickle_RelatedField, (type(self), 0x0ca6ac6, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_RelatedField, (type(self), 0x0ca6ac6, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_pyx_unpickle_RelatedField); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_6, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_13265606);
    __Pyx_GIVEREF(__pyx_int_13265606);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_int_13265606);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_6, 2, Py_None);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_6);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_state);
    __pyx_t_5 = 0;
    __pyx_t_6 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.queryset is not None or self.root is not None or self.style is not None or self.validators is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_RelatedField, (type(self), 0x0ca6ac6, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_RelatedField, (type(self), 0x0ca6ac6, None), state
 *     else:
 *         return __pyx_unpickle_RelatedField, (type(self), 0x0ca6ac6, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_RelatedField__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_pyx_unpickle_RelatedField); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_6, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_13265606);
    __Pyx_GIVEREF(__pyx_int_13265606);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_int_13265606);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_state);
    __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_6);
    __pyx_t_4 = 0;
    __pyx_t_6 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("drf_turbo.fields.RelatedField.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_RelatedField, (type(self), 0x0ca6ac6, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_RelatedField__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_12RelatedField_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_12RelatedField_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_12RelatedField_6__setstate_cython__(((struct __pyx_obj_9drf_turbo_6fields_RelatedField *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_12RelatedField_6__setstate_cython__(struct __pyx_obj_9drf_turbo_6fields_RelatedField *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_RelatedField, (type(self), 0x0ca6ac6, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_RelatedField__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields___pyx_unpickle_RelatedField__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_RelatedField, (type(self), 0x0ca6ac6, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_RelatedField__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.RelatedField.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":1389
 *         'empty': 'This list may not be empty.'
 *     }
 *     def __init__(self,**kwargs):             # <<<<<<<<<<<<<<
 *         self.child_relation = kwargs.pop('child_relation', None)
 *         self.allow_empty = kwargs.pop('allow_empty', True)
 */

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_16ManyRelatedField_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_16ManyRelatedField_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_kwargs = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  if (unlikely(PyTuple_GET_SIZE(__pyx_args) > 0)) {
    __Pyx_RaiseArgtupleInvalid("__init__", 1, 0, 0, PyTuple_GET_SIZE(__pyx_args)); return -1;}
  if (__pyx_kwds && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "__init__", 1))) return -1;
  __pyx_v_kwargs = (__pyx_kwds) ? PyDict_Copy(__pyx_kwds) : PyDict_New(); if (unlikely(!__pyx_v_kwargs)) return -1;
  __Pyx_GOTREF(__pyx_v_kwargs);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_16ManyRelatedField___init__(((struct __pyx_obj_9drf_turbo_6fields_ManyRelatedField *)__pyx_v_self), __pyx_v_kwargs);

  /* function exit code */
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_16ManyRelatedField___init__(struct __pyx_obj_9drf_turbo_6fields_ManyRelatedField *__pyx_v_self, PyObject *__pyx_v_kwargs) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "drf_turbo/fields.pyx":1390
 *     }
 *     def __init__(self,**kwargs):
 *         self.child_relation = kwargs.pop('child_relation', None)             # <<<<<<<<<<<<<<
 *         self.allow_empty = kwargs.pop('allow_empty', True)
 *         super().__init__(**kwargs)
 */
  __pyx_t_1 = __Pyx_PyDict_Pop(__pyx_v_kwargs, __pyx_n_s_child_relation, Py_None); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1390, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->child_relation);
  __Pyx_DECREF(__pyx_v_self->child_relation);
  __pyx_v_self->child_relation = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "drf_turbo/fields.pyx":1391
 *     def __init__(self,**kwargs):
 *         self.child_relation = kwargs.pop('child_relation', None)
 *         self.allow_empty = kwargs.pop('allow_empty', True)             # <<<<<<<<<<<<<<
 *         super().__init__(**kwargs)
 * 
 */
  __pyx_t_1 = __Pyx_PyDict_Pop(__pyx_v_kwargs, __pyx_n_s_allow_empty, Py_True); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1391, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->allow_empty);
  __Pyx_DECREF(__pyx_v_self->allow_empty);
  __pyx_v_self->allow_empty = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "drf_turbo/fields.pyx":1392
 *         self.child_relation = kwargs.pop('child_relation', None)
 *         self.allow_empty = kwargs.pop('allow_empty', True)
 *         super().__init__(**kwargs)             # <<<<<<<<<<<<<<
 * 
 *     cpdef inline serialize(self,value,dict context):
 */
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1392, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_9drf_turbo_6fields_ManyRelatedField));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_9drf_turbo_6fields_ManyRelatedField));
  PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)__pyx_ptype_9drf_turbo_6fields_ManyRelatedField));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_1, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1392, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_init); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1392, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyDict_Copy(__pyx_v_kwargs); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1392, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1392, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "drf_turbo/fields.pyx":1389
 *         'empty': 'This list may not be empty.'
 *     }
 *     def __init__(self,**kwargs):             # <<<<<<<<<<<<<<
 *         self.child_relation = kwargs.pop('child_relation', None)
 *         self.allow_empty = kwargs.pop('allow_empty', True)
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("drf_turbo.fields.ManyRelatedField.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":1394
 *         super().__init__(**kwargs)
 * 
 *     cpdef inline serialize(self,value,dict context):             # <<<<<<<<<<<<<<
 * 
 *         value = value.all() if hasattr(value, 'all') else value
 */

static PyObject *__pyx_pw_9drf_turbo_6fields_16ManyRelatedField_3serialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_16ManyRelatedField_serialize(CYTHON_UNUSED struct __pyx_obj_9drf_turbo_6fields_ManyRelatedField *__pyx_v_self, PyObject *__pyx_v_value, CYTHON_UNUSED PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_v_item = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  Py_ssize_t __pyx_t_6;
  PyObject *(*__pyx_t_7)(PyObject *);
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("serialize", 0);
  __Pyx_INCREF(__pyx_v_value);

  /* "drf_turbo/fields.pyx":1396
 *     cpdef inline serialize(self,value,dict context):
 * 
 *         value = value.all() if hasattr(value, 'all') else value             # <<<<<<<<<<<<<<
 *         return [
 *             item.pk for item in value
 */
  __pyx_t_2 = __Pyx_HasAttr(__pyx_v_value, __pyx_n_s_all); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1396, __pyx_L1_error)
  if ((__pyx_t_2 != 0)) {
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_value, __pyx_n_s_all); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1396, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_5) : __Pyx_PyObject_CallNoArg(__pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1396, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_1 = __pyx_t_3;
    __pyx_t_3 = 0;
  } else {
    __Pyx_INCREF(__pyx_v_value);
    __pyx_t_1 = __pyx_v_value;
  }
  __Pyx_DECREF_SET(__pyx_v_value, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "drf_turbo/fields.pyx":1397
 * 
 *         value = value.all() if hasattr(value, 'all') else value
 *         return [             # <<<<<<<<<<<<<<
 *             item.pk for item in value
 *         ]
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1397, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);

  /* "drf_turbo/fields.pyx":1398
 *         value = value.all() if hasattr(value, 'all') else value
 *         return [
 *             item.pk for item in value             # <<<<<<<<<<<<<<
 *         ]
 * 
 */
  if (likely(PyList_CheckExact(__pyx_v_value)) || PyTuple_CheckExact(__pyx_v_value)) {
    __pyx_t_3 = __pyx_v_value; __Pyx_INCREF(__pyx_t_3); __pyx_t_6 = 0;
    __pyx_t_7 = NULL;
  } else {
    __pyx_t_6 = -1; __pyx_t_3 = PyObject_GetIter(__pyx_v_value); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1398, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_7 = Py_TYPE(__pyx_t_3)->tp_iternext; if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1398, __pyx_L1_error)
  }
  for (;;) {
    if (likely(!__pyx_t_7)) {
      if (likely(PyList_CheckExact(__pyx_t_3))) {
        if (__pyx_t_6 >= PyList_GET_SIZE(__pyx_t_3)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_4 = PyList_GET_ITEM(__pyx_t_3, __pyx_t_6); __Pyx_INCREF(__pyx_t_4); __pyx_t_6++; if (unlikely(0 < 0)) __PYX_ERR(0, 1398, __pyx_L1_error)
        #else
        __pyx_t_4 = PySequence_ITEM(__pyx_t_3, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1398, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        #endif
      } else {
        if (__pyx_t_6 >= PyTuple_GET_SIZE(__pyx_t_3)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_4 = PyTuple_GET_ITEM(__pyx_t_3, __pyx_t_6); __Pyx_INCREF(__pyx_t_4); __pyx_t_6++; if (unlikely(0 < 0)) __PYX_ERR(0, 1398, __pyx_L1_error)
        #else
        __pyx_t_4 = PySequence_ITEM(__pyx_t_3, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1398, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        #endif
      }
    } else {
      __pyx_t_4 = __pyx_t_7(__pyx_t_3);
      if (unlikely(!__pyx_t_4)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 1398, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_4);
    }
    __Pyx_XDECREF_SET(__pyx_v_item, __pyx_t_4);
    __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_item, __pyx_n_s_pk); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1398, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    if (unlikely(__Pyx_ListComp_Append(__pyx_t_1, (PyObject*)__pyx_t_4))) __PYX_ERR(0, 1397, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "drf_turbo/fields.pyx":1394
 *         super().__init__(**kwargs)
 * 
 *     cpdef inline serialize(self,value,dict context):             # <<<<<<<<<<<<<<
 * 
 *         value = value.all() if hasattr(value, 'all') else value
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("drf_turbo.fields.ManyRelatedField.serialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_item);
  __Pyx_XDECREF(__pyx_v_value);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_16ManyRelatedField_3serialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_16ManyRelatedField_3serialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_value = 0;
  PyObject *__pyx_v_context = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("serialize (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_value,&__pyx_n_s_context,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_value)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_context)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("serialize", 1, 2, 2, 1); __PYX_ERR(0, 1394, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "serialize") < 0)) __PYX_ERR(0, 1394, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_value = values[0];
    __pyx_v_context = ((PyObject*)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("serialize", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1394, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.ManyRelatedField.serialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_context), (&PyDict_Type), 1, "context", 1))) __PYX_ERR(0, 1394, __pyx_L1_error)
  __pyx_r = __pyx_pf_9drf_turbo_6fields_16ManyRelatedField_2serialize(((struct __pyx_obj_9drf_turbo_6fields_ManyRelatedField *)__pyx_v_self), __pyx_v_value, __pyx_v_context);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_16ManyRelatedField_2serialize(struct __pyx_obj_9drf_turbo_6fields_ManyRelatedField *__pyx_v_self, PyObject *__pyx_v_value, PyObject *__pyx_v_context) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("serialize", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields_16ManyRelatedField_serialize(__pyx_v_self, __pyx_v_value, __pyx_v_context, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1394, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.ManyRelatedField.serialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":1401
 *         ]
 * 
 *     cpdef inline deserialize(self, data,dict context):             # <<<<<<<<<<<<<<
 *         if isinstance(data, str) or not hasattr(data, '__iter__'):
 *             raise self.raise_if_fail('not_a_list', input_type=type(data).__name__)
 */

static PyObject *__pyx_pw_9drf_turbo_6fields_16ManyRelatedField_5deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_16ManyRelatedField_deserialize(struct __pyx_obj_9drf_turbo_6fields_ManyRelatedField *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_v_item = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  Py_ssize_t __pyx_t_7;
  PyObject *(*__pyx_t_8)(PyObject *);
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_t_11;
  PyObject *__pyx_t_12 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("deserialize", 0);

  /* "drf_turbo/fields.pyx":1402
 * 
 *     cpdef inline deserialize(self, data,dict context):
 *         if isinstance(data, str) or not hasattr(data, '__iter__'):             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail('not_a_list', input_type=type(data).__name__)
 *         if not self.allow_empty and len(data) == 0:
 */
  __pyx_t_2 = PyString_Check(__pyx_v_data); 
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (!__pyx_t_3) {
  } else {
    __pyx_t_1 = __pyx_t_3;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_3 = __Pyx_HasAttr(__pyx_v_data, __pyx_n_s_iter); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(0, 1402, __pyx_L1_error)
  __pyx_t_2 = ((!(__pyx_t_3 != 0)) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (unlikely(__pyx_t_1)) {

    /* "drf_turbo/fields.pyx":1403
 *     cpdef inline deserialize(self, data,dict context):
 *         if isinstance(data, str) or not hasattr(data, '__iter__'):
 *             raise self.raise_if_fail('not_a_list', input_type=type(data).__name__)             # <<<<<<<<<<<<<<
 *         if not self.allow_empty and len(data) == 0:
 *             raise self.raise_if_fail('empty')
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_raise_if_fail); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1403, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1403, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)Py_TYPE(__pyx_v_data)), __pyx_n_s_name); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1403, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    if (PyDict_SetItem(__pyx_t_5, __pyx_n_s_input_type, __pyx_t_6) < 0) __PYX_ERR(0, 1403, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_tuple__19, __pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1403, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_Raise(__pyx_t_6, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __PYX_ERR(0, 1403, __pyx_L1_error)

    /* "drf_turbo/fields.pyx":1402
 * 
 *     cpdef inline deserialize(self, data,dict context):
 *         if isinstance(data, str) or not hasattr(data, '__iter__'):             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail('not_a_list', input_type=type(data).__name__)
 *         if not self.allow_empty and len(data) == 0:
 */
  }

  /* "drf_turbo/fields.pyx":1404
 *         if isinstance(data, str) or not hasattr(data, '__iter__'):
 *             raise self.raise_if_fail('not_a_list', input_type=type(data).__name__)
 *         if not self.allow_empty and len(data) == 0:             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail('empty')
 *         return [
 */
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_self->allow_empty); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 1404, __pyx_L1_error)
  __pyx_t_3 = ((!__pyx_t_2) != 0);
  if (__pyx_t_3) {
  } else {
    __pyx_t_1 = __pyx_t_3;
    goto __pyx_L7_bool_binop_done;
  }
  __pyx_t_7 = PyObject_Length(__pyx_v_data); if (unlikely(__pyx_t_7 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1404, __pyx_L1_error)
  __pyx_t_3 = ((__pyx_t_7 == 0) != 0);
  __pyx_t_1 = __pyx_t_3;
  __pyx_L7_bool_binop_done:;
  if (unlikely(__pyx_t_1)) {

    /* "drf_turbo/fields.pyx":1405
 *             raise self.raise_if_fail('not_a_list', input_type=type(data).__name__)
 *         if not self.allow_empty and len(data) == 0:
 *             raise self.raise_if_fail('empty')             # <<<<<<<<<<<<<<
 *         return [
 *             self.child_relation.deserialize(item,context)
 */
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_raise_if_fail); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1405, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
      }
    }
    __pyx_t_6 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_4, __pyx_n_s_empty) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_n_s_empty);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1405, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_Raise(__pyx_t_6, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __PYX_ERR(0, 1405, __pyx_L1_error)

    /* "drf_turbo/fields.pyx":1404
 *         if isinstance(data, str) or not hasattr(data, '__iter__'):
 *             raise self.raise_if_fail('not_a_list', input_type=type(data).__name__)
 *         if not self.allow_empty and len(data) == 0:             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail('empty')
 *         return [
 */
  }

  /* "drf_turbo/fields.pyx":1406
 *         if not self.allow_empty and len(data) == 0:
 *             raise self.raise_if_fail('empty')
 *         return [             # <<<<<<<<<<<<<<
 *             self.child_relation.deserialize(item,context)
 *             for item in data
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_6 = PyList_New(0); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1406, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);

  /* "drf_turbo/fields.pyx":1408
 *         return [
 *             self.child_relation.deserialize(item,context)
 *             for item in data             # <<<<<<<<<<<<<<
 *         ]
 * 
 */
  if (likely(PyList_CheckExact(__pyx_v_data)) || PyTuple_CheckExact(__pyx_v_data)) {
    __pyx_t_5 = __pyx_v_data; __Pyx_INCREF(__pyx_t_5); __pyx_t_7 = 0;
    __pyx_t_8 = NULL;
  } else {
    __pyx_t_7 = -1; __pyx_t_5 = PyObject_GetIter(__pyx_v_data); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1408, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_8 = Py_TYPE(__pyx_t_5)->tp_iternext; if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1408, __pyx_L1_error)
  }
  for (;;) {
    if (likely(!__pyx_t_8)) {
      if (likely(PyList_CheckExact(__pyx_t_5))) {
        if (__pyx_t_7 >= PyList_GET_SIZE(__pyx_t_5)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_4 = PyList_GET_ITEM(__pyx_t_5, __pyx_t_7); __Pyx_INCREF(__pyx_t_4); __pyx_t_7++; if (unlikely(0 < 0)) __PYX_ERR(0, 1408, __pyx_L1_error)
        #else
        __pyx_t_4 = PySequence_ITEM(__pyx_t_5, __pyx_t_7); __pyx_t_7++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1408, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        #endif
      } else {
        if (__pyx_t_7 >= PyTuple_GET_SIZE(__pyx_t_5)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_4 = PyTuple_GET_ITEM(__pyx_t_5, __pyx_t_7); __Pyx_INCREF(__pyx_t_4); __pyx_t_7++; if (unlikely(0 < 0)) __PYX_ERR(0, 1408, __pyx_L1_error)
        #else
        __pyx_t_4 = PySequence_ITEM(__pyx_t_5, __pyx_t_7); __pyx_t_7++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1408, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        #endif
      }
    } else {
      __pyx_t_4 = __pyx_t_8(__pyx_t_5);
      if (unlikely(!__pyx_t_4)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 1408, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_4);
    }
    __Pyx_XDECREF_SET(__pyx_v_item, __pyx_t_4);
    __pyx_t_4 = 0;

    /* "drf_turbo/fields.pyx":1407
 *             raise self.raise_if_fail('empty')
 *         return [
 *             self.child_relation.deserialize(item,context)             # <<<<<<<<<<<<<<
 *             for item in data
 *         ]
 */
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->child_relation, __pyx_n_s_deserialize); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1407, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_10 = NULL;
    __pyx_t_11 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_9))) {
      __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_9);
      if (likely(__pyx_t_10)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_9, function);
        __pyx_t_11 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_9)) {
      PyObject *__pyx_temp[3] = {__pyx_t_10, __pyx_v_item, __pyx_v_context};
      __pyx_t_4 = __Pyx_PyFunction_FastCall(__pyx_t_9, __pyx_temp+1-__pyx_t_11, 2+__pyx_t_11); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1407, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_GOTREF(__pyx_t_4);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_9)) {
      PyObject *__pyx_temp[3] = {__pyx_t_10, __pyx_v_item, __pyx_v_context};
      __pyx_t_4 = __Pyx_PyCFunction_FastCall(__pyx_t_9, __pyx_temp+1-__pyx_t_11, 2+__pyx_t_11); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1407, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_GOTREF(__pyx_t_4);
    } else
    #endif
    {
      __pyx_t_12 = PyTuple_New(2+__pyx_t_11); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1407, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_12);
      if (__pyx_t_10) {
        __Pyx_GIVEREF(__pyx_t_10); PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_10); __pyx_t_10 = NULL;
      }
      __Pyx_INCREF(__pyx_v_item);
      __Pyx_GIVEREF(__pyx_v_item);
      PyTuple_SET_ITEM(__pyx_t_12, 0+__pyx_t_11, __pyx_v_item);
      __Pyx_INCREF(__pyx_v_context);
      __Pyx_GIVEREF(__pyx_v_context);
      PyTuple_SET_ITEM(__pyx_t_12, 1+__pyx_t_11, __pyx_v_context);
      __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_t_12, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1407, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    }
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (unlikely(__Pyx_ListComp_Append(__pyx_t_6, (PyObject*)__pyx_t_4))) __PYX_ERR(0, 1406, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "drf_turbo/fields.pyx":1408
 *         return [
 *             self.child_relation.deserialize(item,context)
 *             for item in data             # <<<<<<<<<<<<<<
 *         ]
 * 
 */
  }
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_r = __pyx_t_6;
  __pyx_t_6 = 0;
  goto __pyx_L0;

  /* "drf_turbo/fields.pyx":1401
 *         ]
 * 
 *     cpdef inline deserialize(self, data,dict context):             # <<<<<<<<<<<<<<
 *         if isinstance(data, str) or not hasattr(data, '__iter__'):
 *             raise self.raise_if_fail('not_a_list', input_type=type(data).__name__)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_AddTraceback("drf_turbo.fields.ManyRelatedField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_item);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_16ManyRelatedField_5deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_16ManyRelatedField_5deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_data = 0;
  PyObject *__pyx_v_context = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("deserialize (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_data,&__pyx_n_s_context,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_data)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_context)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("deserialize", 1, 2, 2, 1); __PYX_ERR(0, 1401, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "deserialize") < 0)) __PYX_ERR(0, 1401, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_data = values[0];
    __pyx_v_context = ((PyObject*)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("deserialize", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1401, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.ManyRelatedField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_context), (&PyDict_Type), 1, "context", 1))) __PYX_ERR(0, 1401, __pyx_L1_error)
  __pyx_r = __pyx_pf_9drf_turbo_6fields_16ManyRelatedField_4deserialize(((struct __pyx_obj_9drf_turbo_6fields_ManyRelatedField *)__pyx_v_self), __pyx_v_data, __pyx_v_context);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_16ManyRelatedField_4deserialize(struct __pyx_obj_9drf_turbo_6fields_ManyRelatedField *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("deserialize", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields_16ManyRelatedField_deserialize(__pyx_v_self, __pyx_v_data, __pyx_v_context, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1401, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.ManyRelatedField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pxd":161
 * cdef class ManyRelatedField(Field):
 *     cdef public :
 *         child_relation             # <<<<<<<<<<<<<<
 *         allow_empty
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_16ManyRelatedField_14child_relation_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_16ManyRelatedField_14child_relation_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_16ManyRelatedField_14child_relation___get__(((struct __pyx_obj_9drf_turbo_6fields_ManyRelatedField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_16ManyRelatedField_14child_relation___get__(struct __pyx_obj_9drf_turbo_6fields_ManyRelatedField *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->child_relation);
  __pyx_r = __pyx_v_self->child_relation;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_16ManyRelatedField_14child_relation_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_16ManyRelatedField_14child_relation_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_16ManyRelatedField_14child_relation_2__set__(((struct __pyx_obj_9drf_turbo_6fields_ManyRelatedField *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_16ManyRelatedField_14child_relation_2__set__(struct __pyx_obj_9drf_turbo_6fields_ManyRelatedField *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->child_relation);
  __Pyx_DECREF(__pyx_v_self->child_relation);
  __pyx_v_self->child_relation = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_16ManyRelatedField_14child_relation_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_16ManyRelatedField_14child_relation_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_16ManyRelatedField_14child_relation_4__del__(((struct __pyx_obj_9drf_turbo_6fields_ManyRelatedField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_16ManyRelatedField_14child_relation_4__del__(struct __pyx_obj_9drf_turbo_6fields_ManyRelatedField *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->child_relation);
  __Pyx_DECREF(__pyx_v_self->child_relation);
  __pyx_v_self->child_relation = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pxd":162
 *     cdef public :
 *         child_relation
 *         allow_empty             # <<<<<<<<<<<<<<
 * 
 * cdef class ConstantField(Field):
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_16ManyRelatedField_11allow_empty_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_16ManyRelatedField_11allow_empty_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_16ManyRelatedField_11allow_empty___get__(((struct __pyx_obj_9drf_turbo_6fields_ManyRelatedField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_16ManyRelatedField_11allow_empty___get__(struct __pyx_obj_9drf_turbo_6fields_ManyRelatedField *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->allow_empty);
  __pyx_r = __pyx_v_self->allow_empty;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_16ManyRelatedField_11allow_empty_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_16ManyRelatedField_11allow_empty_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_16ManyRelatedField_11allow_empty_2__set__(((struct __pyx_obj_9drf_turbo_6fields_ManyRelatedField *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_16ManyRelatedField_11allow_empty_2__set__(struct __pyx_obj_9drf_turbo_6fields_ManyRelatedField *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->allow_empty);
  __Pyx_DECREF(__pyx_v_self->allow_empty);
  __pyx_v_self->allow_empty = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_16ManyRelatedField_11allow_empty_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_16ManyRelatedField_11allow_empty_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_16ManyRelatedField_11allow_empty_4__del__(((struct __pyx_obj_9drf_turbo_6fields_ManyRelatedField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_16ManyRelatedField_11allow_empty_4__del__(struct __pyx_obj_9drf_turbo_6fields_ManyRelatedField *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->allow_empty);
  __Pyx_DECREF(__pyx_v_self->allow_empty);
  __pyx_v_self->allow_empty = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_16ManyRelatedField_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_16ManyRelatedField_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_16ManyRelatedField_6__reduce_cython__(((struct __pyx_obj_9drf_turbo_6fields_ManyRelatedField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_16ManyRelatedField_6__reduce_cython__(struct __pyx_obj_9drf_turbo_6fields_ManyRelatedField *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self.allow_empty, self.allow_null, self.attr, self.attrs, self.call, self.child_relation, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.allow_null); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.call); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.read_only); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.required); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.write_only); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = PyTuple_New(18); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_INCREF(__pyx_v_self->allow_empty);
  __Pyx_GIVEREF(__pyx_v_self->allow_empty);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v_self->allow_empty);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.attr);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.attr);
  PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_self->__pyx_base.attr);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.attrs);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.attrs);
  PyTuple_SET_ITEM(__pyx_t_6, 3, __pyx_v_self->__pyx_base.attrs);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_6, 4, __pyx_t_2);
  __Pyx_INCREF(__pyx_v_self->child_relation);
  __Pyx_GIVEREF(__pyx_v_self->child_relation);
  PyTuple_SET_ITEM(__pyx_t_6, 5, __pyx_v_self->child_relation);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.default_value);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.default_value);
  PyTuple_SET_ITEM(__pyx_t_6, 6, __pyx_v_self->__pyx_base.default_value);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.error_messages);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.error_messages);
  PyTuple_SET_ITEM(__pyx_t_6, 7, __pyx_v_self->__pyx_base.error_messages);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.field_name);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.field_name);
  PyTuple_SET_ITEM(__pyx_t_6, 8, __pyx_v_self->__pyx_base.field_name);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.help_text);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.help_text);
  PyTuple_SET_ITEM(__pyx_t_6, 9, __pyx_v_self->__pyx_base.help_text);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.initial);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.initial);
  PyTuple_SET_ITEM(__pyx_t_6, 10, __pyx_v_self->__pyx_base.initial);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.label);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.label);
  PyTuple_SET_ITEM(__pyx_t_6, 11, __pyx_v_self->__pyx_base.label);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 12, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_6, 13, __pyx_t_4);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.root);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.root);
  PyTuple_SET_ITEM(__pyx_t_6, 14, __pyx_v_self->__pyx_base.root);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.style);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.style);
  PyTuple_SET_ITEM(__pyx_t_6, 15, __pyx_v_self->__pyx_base.style);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.validators);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.validators);
  PyTuple_SET_ITEM(__pyx_t_6, 16, __pyx_v_self->__pyx_base.validators);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_6, 17, __pyx_t_5);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_6);
  __pyx_t_6 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self.allow_empty, self.allow_null, self.attr, self.attrs, self.call, self.child_relation, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_6 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_v__dict = __pyx_t_6;
  __pyx_t_6 = 0;

  /* "(tree fragment)":7
 *     state = (self.allow_empty, self.allow_null, self.attr, self.attrs, self.call, self.child_relation, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_7 = (__pyx_v__dict != Py_None);
  __pyx_t_8 = (__pyx_t_7 != 0);
  if (__pyx_t_8) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_6 = PyTuple_New(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v__dict);
    __pyx_t_5 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_6); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_5));
    __pyx_t_5 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.allow_empty is not None or self.attr is not None or self.attrs is not None or self.child_relation is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.root is not None or self.style is not None or self.validators is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self.allow_empty, self.allow_null, self.attr, self.attrs, self.call, self.child_relation, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self.allow_empty is not None or self.attr is not None or self.attrs is not None or self.child_relation is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.root is not None or self.style is not None or self.validators is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_ManyRelatedField, (type(self), 0xf4706f5, None), state
 */
  /*else*/ {
    __pyx_t_7 = (__pyx_v_self->allow_empty != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.attr != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.attrs != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->child_relation != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.default_value != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.error_messages != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.field_name != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.help_text != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.initial != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.label != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.root != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.style != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.validators != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    __pyx_t_8 = __pyx_t_9;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_8;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.allow_empty is not None or self.attr is not None or self.attrs is not None or self.child_relation is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.root is not None or self.style is not None or self.validators is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_ManyRelatedField, (type(self), 0xf4706f5, None), state
 *     else:
 */
  __pyx_t_8 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_8) {

    /* "(tree fragment)":13
 *         use_setstate = self.allow_empty is not None or self.attr is not None or self.attrs is not None or self.child_relation is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.root is not None or self.style is not None or self.validators is not None
 *     if use_setstate:
 *         return __pyx_unpickle_ManyRelatedField, (type(self), 0xf4706f5, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_ManyRelatedField, (type(self), 0xf4706f5, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_pyx_unpickle_ManyRelatedField); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_6, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_256313077);
    __Pyx_GIVEREF(__pyx_int_256313077);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_int_256313077);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_6, 2, Py_None);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_6);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_state);
    __pyx_t_5 = 0;
    __pyx_t_6 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.allow_empty is not None or self.attr is not None or self.attrs is not None or self.child_relation is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.root is not None or self.style is not None or self.validators is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_ManyRelatedField, (type(self), 0xf4706f5, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_ManyRelatedField, (type(self), 0xf4706f5, None), state
 *     else:
 *         return __pyx_unpickle_ManyRelatedField, (type(self), 0xf4706f5, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_ManyRelatedField__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_pyx_unpickle_ManyRelatedField); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_6, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_256313077);
    __Pyx_GIVEREF(__pyx_int_256313077);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_int_256313077);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_state);
    __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_6);
    __pyx_t_4 = 0;
    __pyx_t_6 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("drf_turbo.fields.ManyRelatedField.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_ManyRelatedField, (type(self), 0xf4706f5, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_ManyRelatedField__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_16ManyRelatedField_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_16ManyRelatedField_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_16ManyRelatedField_8__setstate_cython__(((struct __pyx_obj_9drf_turbo_6fields_ManyRelatedField *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_16ManyRelatedField_8__setstate_cython__(struct __pyx_obj_9drf_turbo_6fields_ManyRelatedField *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_ManyRelatedField, (type(self), 0xf4706f5, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_ManyRelatedField__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields___pyx_unpickle_ManyRelatedField__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_ManyRelatedField, (type(self), 0xf4706f5, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_ManyRelatedField__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.ManyRelatedField.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":1424
 *         'None': 'Must be None.'
 *     }
 *     def __init__(self,constant,**kwargs):             # <<<<<<<<<<<<<<
 *         self.constant = constant
 *         super().__init__(**kwargs)
 */

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_13ConstantField_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_13ConstantField_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_constant = 0;
  PyObject *__pyx_v_kwargs = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  __pyx_v_kwargs = PyDict_New(); if (unlikely(!__pyx_v_kwargs)) return -1;
  __Pyx_GOTREF(__pyx_v_kwargs);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_constant,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_constant)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, __pyx_v_kwargs, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 1424, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_constant = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1424, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_DECREF(__pyx_v_kwargs); __pyx_v_kwargs = 0;
  __Pyx_AddTraceback("drf_turbo.fields.ConstantField.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9drf_turbo_6fields_13ConstantField___init__(((struct __pyx_obj_9drf_turbo_6fields_ConstantField *)__pyx_v_self), __pyx_v_constant, __pyx_v_kwargs);

  /* function exit code */
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_13ConstantField___init__(struct __pyx_obj_9drf_turbo_6fields_ConstantField *__pyx_v_self, PyObject *__pyx_v_constant, PyObject *__pyx_v_kwargs) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "drf_turbo/fields.pyx":1425
 *     }
 *     def __init__(self,constant,**kwargs):
 *         self.constant = constant             # <<<<<<<<<<<<<<
 *         super().__init__(**kwargs)
 *         assert "allow_null" not in kwargs
 */
  __Pyx_INCREF(__pyx_v_constant);
  __Pyx_GIVEREF(__pyx_v_constant);
  __Pyx_GOTREF(__pyx_v_self->constant);
  __Pyx_DECREF(__pyx_v_self->constant);
  __pyx_v_self->constant = __pyx_v_constant;

  /* "drf_turbo/fields.pyx":1426
 *     def __init__(self,constant,**kwargs):
 *         self.constant = constant
 *         super().__init__(**kwargs)             # <<<<<<<<<<<<<<
 *         assert "allow_null" not in kwargs
 * 
 */
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1426, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_9drf_turbo_6fields_ConstantField));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_9drf_turbo_6fields_ConstantField));
  PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)__pyx_ptype_9drf_turbo_6fields_ConstantField));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_1, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1426, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_init); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1426, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyDict_Copy(__pyx_v_kwargs); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1426, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1426, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "drf_turbo/fields.pyx":1427
 *         self.constant = constant
 *         super().__init__(**kwargs)
 *         assert "allow_null" not in kwargs             # <<<<<<<<<<<<<<
 * 
 *     cpdef inline deserialize(self,data,dict context):
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    __pyx_t_4 = (__Pyx_PyDict_ContainsTF(__pyx_n_s_allow_null, __pyx_v_kwargs, Py_NE)); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 1427, __pyx_L1_error)
    if (unlikely(!(__pyx_t_4 != 0))) {
      PyErr_SetNone(PyExc_AssertionError);
      __PYX_ERR(0, 1427, __pyx_L1_error)
    }
  }
  #endif

  /* "drf_turbo/fields.pyx":1424
 *         'None': 'Must be None.'
 *     }
 *     def __init__(self,constant,**kwargs):             # <<<<<<<<<<<<<<
 *         self.constant = constant
 *         super().__init__(**kwargs)
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("drf_turbo.fields.ConstantField.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":1429
 *         assert "allow_null" not in kwargs
 * 
 *     cpdef inline deserialize(self,data,dict context):             # <<<<<<<<<<<<<<
 *         if data != self.constant:
 *             if self.constant is None:
 */

static PyObject *__pyx_pw_9drf_turbo_6fields_13ConstantField_3deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_13ConstantField_deserialize(struct __pyx_obj_9drf_turbo_6fields_ConstantField *__pyx_v_self, PyObject *__pyx_v_data, CYTHON_UNUSED PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("deserialize", 0);

  /* "drf_turbo/fields.pyx":1430
 * 
 *     cpdef inline deserialize(self,data,dict context):
 *         if data != self.constant:             # <<<<<<<<<<<<<<
 *             if self.constant is None:
 *                 raise self.raise_if_fail("None")
 */
  __pyx_t_1 = PyObject_RichCompare(__pyx_v_data, __pyx_v_self->constant, Py_NE); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1430, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 1430, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "drf_turbo/fields.pyx":1431
 *     cpdef inline deserialize(self,data,dict context):
 *         if data != self.constant:
 *             if self.constant is None:             # <<<<<<<<<<<<<<
 *                 raise self.raise_if_fail("None")
 *             raise self.raise_if_fail("constant",constant=self.constant)
 */
    __pyx_t_2 = (__pyx_v_self->constant == Py_None);
    __pyx_t_3 = (__pyx_t_2 != 0);
    if (unlikely(__pyx_t_3)) {

      /* "drf_turbo/fields.pyx":1432
 *         if data != self.constant:
 *             if self.constant is None:
 *                 raise self.raise_if_fail("None")             # <<<<<<<<<<<<<<
 *             raise self.raise_if_fail("constant",constant=self.constant)
 *         return data
 */
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_raise_if_fail); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1432, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_5 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
        if (likely(__pyx_t_5)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_5);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_4, function);
        }
      }
      __pyx_t_1 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_5, __pyx_n_s_None) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_n_s_None);
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1432, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_Raise(__pyx_t_1, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __PYX_ERR(0, 1432, __pyx_L1_error)

      /* "drf_turbo/fields.pyx":1431
 *     cpdef inline deserialize(self,data,dict context):
 *         if data != self.constant:
 *             if self.constant is None:             # <<<<<<<<<<<<<<
 *                 raise self.raise_if_fail("None")
 *             raise self.raise_if_fail("constant",constant=self.constant)
 */
    }

    /* "drf_turbo/fields.pyx":1433
 *             if self.constant is None:
 *                 raise self.raise_if_fail("None")
 *             raise self.raise_if_fail("constant",constant=self.constant)             # <<<<<<<<<<<<<<
 *         return data
 * 
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_raise_if_fail); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1433, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1433, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_constant, __pyx_v_self->constant) < 0) __PYX_ERR(0, 1433, __pyx_L1_error)
    __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__30, __pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1433, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_Raise(__pyx_t_5, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __PYX_ERR(0, 1433, __pyx_L1_error)

    /* "drf_turbo/fields.pyx":1430
 * 
 *     cpdef inline deserialize(self,data,dict context):
 *         if data != self.constant:             # <<<<<<<<<<<<<<
 *             if self.constant is None:
 *                 raise self.raise_if_fail("None")
 */
  }

  /* "drf_turbo/fields.pyx":1434
 *                 raise self.raise_if_fail("None")
 *             raise self.raise_if_fail("constant",constant=self.constant)
 *         return data             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_data);
  __pyx_r = __pyx_v_data;
  goto __pyx_L0;

  /* "drf_turbo/fields.pyx":1429
 *         assert "allow_null" not in kwargs
 * 
 *     cpdef inline deserialize(self,data,dict context):             # <<<<<<<<<<<<<<
 *         if data != self.constant:
 *             if self.constant is None:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("drf_turbo.fields.ConstantField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_13ConstantField_3deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_13ConstantField_3deserialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_data = 0;
  PyObject *__pyx_v_context = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("deserialize (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_data,&__pyx_n_s_context,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_data)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_context)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("deserialize", 1, 2, 2, 1); __PYX_ERR(0, 1429, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "deserialize") < 0)) __PYX_ERR(0, 1429, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_data = values[0];
    __pyx_v_context = ((PyObject*)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("deserialize", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1429, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.ConstantField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_context), (&PyDict_Type), 1, "context", 1))) __PYX_ERR(0, 1429, __pyx_L1_error)
  __pyx_r = __pyx_pf_9drf_turbo_6fields_13ConstantField_2deserialize(((struct __pyx_obj_9drf_turbo_6fields_ConstantField *)__pyx_v_self), __pyx_v_data, __pyx_v_context);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_13ConstantField_2deserialize(struct __pyx_obj_9drf_turbo_6fields_ConstantField *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_context) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("deserialize", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields_13ConstantField_deserialize(__pyx_v_self, __pyx_v_data, __pyx_v_context, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1429, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.ConstantField.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pxd":166
 * cdef class ConstantField(Field):
 *     cdef public :
 *         constant             # <<<<<<<<<<<<<<
 * 
 * cdef class RecursiveField(Field):
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_13ConstantField_8constant_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_13ConstantField_8constant_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_13ConstantField_8constant___get__(((struct __pyx_obj_9drf_turbo_6fields_ConstantField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_13ConstantField_8constant___get__(struct __pyx_obj_9drf_turbo_6fields_ConstantField *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->constant);
  __pyx_r = __pyx_v_self->constant;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_13ConstantField_8constant_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_13ConstantField_8constant_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_13ConstantField_8constant_2__set__(((struct __pyx_obj_9drf_turbo_6fields_ConstantField *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_13ConstantField_8constant_2__set__(struct __pyx_obj_9drf_turbo_6fields_ConstantField *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->constant);
  __Pyx_DECREF(__pyx_v_self->constant);
  __pyx_v_self->constant = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_13ConstantField_8constant_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_13ConstantField_8constant_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_13ConstantField_8constant_4__del__(((struct __pyx_obj_9drf_turbo_6fields_ConstantField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_13ConstantField_8constant_4__del__(struct __pyx_obj_9drf_turbo_6fields_ConstantField *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->constant);
  __Pyx_DECREF(__pyx_v_self->constant);
  __pyx_v_self->constant = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_13ConstantField_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_13ConstantField_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_13ConstantField_4__reduce_cython__(((struct __pyx_obj_9drf_turbo_6fields_ConstantField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_13ConstantField_4__reduce_cython__(struct __pyx_obj_9drf_turbo_6fields_ConstantField *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self.allow_null, self.attr, self.attrs, self.call, self.constant, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.allow_null); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.call); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.read_only); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.required); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.write_only); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = PyTuple_New(17); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.attr);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.attr);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_v_self->__pyx_base.attr);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.attrs);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.attrs);
  PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_self->__pyx_base.attrs);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_6, 3, __pyx_t_2);
  __Pyx_INCREF(__pyx_v_self->constant);
  __Pyx_GIVEREF(__pyx_v_self->constant);
  PyTuple_SET_ITEM(__pyx_t_6, 4, __pyx_v_self->constant);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.default_value);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.default_value);
  PyTuple_SET_ITEM(__pyx_t_6, 5, __pyx_v_self->__pyx_base.default_value);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.error_messages);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.error_messages);
  PyTuple_SET_ITEM(__pyx_t_6, 6, __pyx_v_self->__pyx_base.error_messages);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.field_name);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.field_name);
  PyTuple_SET_ITEM(__pyx_t_6, 7, __pyx_v_self->__pyx_base.field_name);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.help_text);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.help_text);
  PyTuple_SET_ITEM(__pyx_t_6, 8, __pyx_v_self->__pyx_base.help_text);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.initial);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.initial);
  PyTuple_SET_ITEM(__pyx_t_6, 9, __pyx_v_self->__pyx_base.initial);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.label);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.label);
  PyTuple_SET_ITEM(__pyx_t_6, 10, __pyx_v_self->__pyx_base.label);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 11, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_6, 12, __pyx_t_4);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.root);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.root);
  PyTuple_SET_ITEM(__pyx_t_6, 13, __pyx_v_self->__pyx_base.root);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.style);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.style);
  PyTuple_SET_ITEM(__pyx_t_6, 14, __pyx_v_self->__pyx_base.style);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.validators);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.validators);
  PyTuple_SET_ITEM(__pyx_t_6, 15, __pyx_v_self->__pyx_base.validators);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_6, 16, __pyx_t_5);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_6);
  __pyx_t_6 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self.allow_null, self.attr, self.attrs, self.call, self.constant, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_6 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_v__dict = __pyx_t_6;
  __pyx_t_6 = 0;

  /* "(tree fragment)":7
 *     state = (self.allow_null, self.attr, self.attrs, self.call, self.constant, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_7 = (__pyx_v__dict != Py_None);
  __pyx_t_8 = (__pyx_t_7 != 0);
  if (__pyx_t_8) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_6 = PyTuple_New(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v__dict);
    __pyx_t_5 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_6); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_5));
    __pyx_t_5 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.attr is not None or self.attrs is not None or self.constant is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.root is not None or self.style is not None or self.validators is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self.allow_null, self.attr, self.attrs, self.call, self.constant, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self.attr is not None or self.attrs is not None or self.constant is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.root is not None or self.style is not None or self.validators is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_ConstantField, (type(self), 0x369fe29, None), state
 */
  /*else*/ {
    __pyx_t_7 = (__pyx_v_self->__pyx_base.attr != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.attrs != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->constant != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.default_value != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.error_messages != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.field_name != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.help_text != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.initial != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.label != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.root != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.style != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.validators != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    __pyx_t_8 = __pyx_t_7;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_8;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.attr is not None or self.attrs is not None or self.constant is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.root is not None or self.style is not None or self.validators is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_ConstantField, (type(self), 0x369fe29, None), state
 *     else:
 */
  __pyx_t_8 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_8) {

    /* "(tree fragment)":13
 *         use_setstate = self.attr is not None or self.attrs is not None or self.constant is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.root is not None or self.style is not None or self.validators is not None
 *     if use_setstate:
 *         return __pyx_unpickle_ConstantField, (type(self), 0x369fe29, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_ConstantField, (type(self), 0x369fe29, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_pyx_unpickle_ConstantField); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_6, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_57277993);
    __Pyx_GIVEREF(__pyx_int_57277993);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_int_57277993);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_6, 2, Py_None);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_6);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_state);
    __pyx_t_5 = 0;
    __pyx_t_6 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.attr is not None or self.attrs is not None or self.constant is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.root is not None or self.style is not None or self.validators is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_ConstantField, (type(self), 0x369fe29, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_ConstantField, (type(self), 0x369fe29, None), state
 *     else:
 *         return __pyx_unpickle_ConstantField, (type(self), 0x369fe29, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_ConstantField__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_pyx_unpickle_ConstantField); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_6, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_57277993);
    __Pyx_GIVEREF(__pyx_int_57277993);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_int_57277993);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_state);
    __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_6);
    __pyx_t_4 = 0;
    __pyx_t_6 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("drf_turbo.fields.ConstantField.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_ConstantField, (type(self), 0x369fe29, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_ConstantField__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_13ConstantField_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_13ConstantField_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_13ConstantField_6__setstate_cython__(((struct __pyx_obj_9drf_turbo_6fields_ConstantField *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_13ConstantField_6__setstate_cython__(struct __pyx_obj_9drf_turbo_6fields_ConstantField *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_ConstantField, (type(self), 0x369fe29, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_ConstantField__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields___pyx_unpickle_ConstantField__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_ConstantField, (type(self), 0x369fe29, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_ConstantField__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.ConstantField.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":1448
 *     :param kwargs: The same keyword arguments that :class:`Field` receives.
 *     """
 *     def __init__(self,**kwargs):             # <<<<<<<<<<<<<<
 *         self.many = kwargs.pop('many',False)
 *         self.context = kwargs.pop('context',{})
 */

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_14RecursiveField_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_14RecursiveField_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_kwargs = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  if (unlikely(PyTuple_GET_SIZE(__pyx_args) > 0)) {
    __Pyx_RaiseArgtupleInvalid("__init__", 1, 0, 0, PyTuple_GET_SIZE(__pyx_args)); return -1;}
  if (__pyx_kwds && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "__init__", 1))) return -1;
  __pyx_v_kwargs = (__pyx_kwds) ? PyDict_Copy(__pyx_kwds) : PyDict_New(); if (unlikely(!__pyx_v_kwargs)) return -1;
  __Pyx_GOTREF(__pyx_v_kwargs);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_14RecursiveField___init__(((struct __pyx_obj_9drf_turbo_6fields_RecursiveField *)__pyx_v_self), __pyx_v_kwargs);

  /* function exit code */
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_14RecursiveField___init__(struct __pyx_obj_9drf_turbo_6fields_RecursiveField *__pyx_v_self, PyObject *__pyx_v_kwargs) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "drf_turbo/fields.pyx":1449
 *     """
 *     def __init__(self,**kwargs):
 *         self.many = kwargs.pop('many',False)             # <<<<<<<<<<<<<<
 *         self.context = kwargs.pop('context',{})
 *         self.only = kwargs.pop('only',None)
 */
  __pyx_t_1 = __Pyx_PyDict_Pop(__pyx_v_kwargs, __pyx_n_s_many, Py_False); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1449, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->many);
  __Pyx_DECREF(__pyx_v_self->many);
  __pyx_v_self->many = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "drf_turbo/fields.pyx":1450
 *     def __init__(self,**kwargs):
 *         self.many = kwargs.pop('many',False)
 *         self.context = kwargs.pop('context',{})             # <<<<<<<<<<<<<<
 *         self.only = kwargs.pop('only',None)
 *         self.exclude = kwargs.pop('exclude',None)
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1450, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyDict_Pop(__pyx_v_kwargs, __pyx_n_s_context, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1450, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->context);
  __Pyx_DECREF(__pyx_v_self->context);
  __pyx_v_self->context = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "drf_turbo/fields.pyx":1451
 *         self.many = kwargs.pop('many',False)
 *         self.context = kwargs.pop('context',{})
 *         self.only = kwargs.pop('only',None)             # <<<<<<<<<<<<<<
 *         self.exclude = kwargs.pop('exclude',None)
 *         if self.only is not None and self.exclude is not None :
 */
  __pyx_t_2 = __Pyx_PyDict_Pop(__pyx_v_kwargs, __pyx_n_s_only, Py_None); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1451, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->only);
  __Pyx_DECREF(__pyx_v_self->only);
  __pyx_v_self->only = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "drf_turbo/fields.pyx":1452
 *         self.context = kwargs.pop('context',{})
 *         self.only = kwargs.pop('only',None)
 *         self.exclude = kwargs.pop('exclude',None)             # <<<<<<<<<<<<<<
 *         if self.only is not None and self.exclude is not None :
 *             raise OnlyAndExcludeError('You should use either "only" or "exclude"')
 */
  __pyx_t_2 = __Pyx_PyDict_Pop(__pyx_v_kwargs, __pyx_n_s_exclude, Py_None); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1452, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->exclude);
  __Pyx_DECREF(__pyx_v_self->exclude);
  __pyx_v_self->exclude = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "drf_turbo/fields.pyx":1453
 *         self.only = kwargs.pop('only',None)
 *         self.exclude = kwargs.pop('exclude',None)
 *         if self.only is not None and self.exclude is not None :             # <<<<<<<<<<<<<<
 *             raise OnlyAndExcludeError('You should use either "only" or "exclude"')
 *         if self.only is not None and not is_collection(self.only):
 */
  __pyx_t_4 = (__pyx_v_self->only != Py_None);
  __pyx_t_5 = (__pyx_t_4 != 0);
  if (__pyx_t_5) {
  } else {
    __pyx_t_3 = __pyx_t_5;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_5 = (__pyx_v_self->exclude != Py_None);
  __pyx_t_4 = (__pyx_t_5 != 0);
  __pyx_t_3 = __pyx_t_4;
  __pyx_L4_bool_binop_done:;
  if (unlikely(__pyx_t_3)) {

    /* "drf_turbo/fields.pyx":1454
 *         self.exclude = kwargs.pop('exclude',None)
 *         if self.only is not None and self.exclude is not None :
 *             raise OnlyAndExcludeError('You should use either "only" or "exclude"')             # <<<<<<<<<<<<<<
 *         if self.only is not None and not is_collection(self.only):
 *             raise StringNotCollectionError('"only" should be a list of strings')
 */
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_OnlyAndExcludeError); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1454, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_6 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
      }
    }
    __pyx_t_2 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_6, __pyx_kp_s_You_should_use_either_only_or_ex) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_kp_s_You_should_use_either_only_or_ex);
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1454, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1454, __pyx_L1_error)

    /* "drf_turbo/fields.pyx":1453
 *         self.only = kwargs.pop('only',None)
 *         self.exclude = kwargs.pop('exclude',None)
 *         if self.only is not None and self.exclude is not None :             # <<<<<<<<<<<<<<
 *             raise OnlyAndExcludeError('You should use either "only" or "exclude"')
 *         if self.only is not None and not is_collection(self.only):
 */
  }

  /* "drf_turbo/fields.pyx":1455
 *         if self.only is not None and self.exclude is not None :
 *             raise OnlyAndExcludeError('You should use either "only" or "exclude"')
 *         if self.only is not None and not is_collection(self.only):             # <<<<<<<<<<<<<<
 *             raise StringNotCollectionError('"only" should be a list of strings')
 *         if self.exclude is not None and not is_collection(self.exclude):
 */
  __pyx_t_4 = (__pyx_v_self->only != Py_None);
  __pyx_t_5 = (__pyx_t_4 != 0);
  if (__pyx_t_5) {
  } else {
    __pyx_t_3 = __pyx_t_5;
    goto __pyx_L7_bool_binop_done;
  }
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_is_collection); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1455, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
    }
  }
  __pyx_t_2 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_6, __pyx_v_self->only) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_v_self->only);
  __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1455, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 1455, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = ((!__pyx_t_5) != 0);
  __pyx_t_3 = __pyx_t_4;
  __pyx_L7_bool_binop_done:;
  if (unlikely(__pyx_t_3)) {

    /* "drf_turbo/fields.pyx":1456
 *             raise OnlyAndExcludeError('You should use either "only" or "exclude"')
 *         if self.only is not None and not is_collection(self.only):
 *             raise StringNotCollectionError('"only" should be a list of strings')             # <<<<<<<<<<<<<<
 *         if self.exclude is not None and not is_collection(self.exclude):
 *             raise StringNotCollectionError('"exclude" should be a list of strings')
 */
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_StringNotCollectionError); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1456, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_6 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
      }
    }
    __pyx_t_2 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_6, __pyx_kp_s_only_should_be_a_list_of_string) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_kp_s_only_should_be_a_list_of_string);
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1456, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1456, __pyx_L1_error)

    /* "drf_turbo/fields.pyx":1455
 *         if self.only is not None and self.exclude is not None :
 *             raise OnlyAndExcludeError('You should use either "only" or "exclude"')
 *         if self.only is not None and not is_collection(self.only):             # <<<<<<<<<<<<<<
 *             raise StringNotCollectionError('"only" should be a list of strings')
 *         if self.exclude is not None and not is_collection(self.exclude):
 */
  }

  /* "drf_turbo/fields.pyx":1457
 *         if self.only is not None and not is_collection(self.only):
 *             raise StringNotCollectionError('"only" should be a list of strings')
 *         if self.exclude is not None and not is_collection(self.exclude):             # <<<<<<<<<<<<<<
 *             raise StringNotCollectionError('"exclude" should be a list of strings')
 *         super().__init__(**kwargs)
 */
  __pyx_t_4 = (__pyx_v_self->exclude != Py_None);
  __pyx_t_5 = (__pyx_t_4 != 0);
  if (__pyx_t_5) {
  } else {
    __pyx_t_3 = __pyx_t_5;
    goto __pyx_L10_bool_binop_done;
  }
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_is_collection); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1457, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
    }
  }
  __pyx_t_2 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_6, __pyx_v_self->exclude) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_v_self->exclude);
  __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1457, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 1457, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = ((!__pyx_t_5) != 0);
  __pyx_t_3 = __pyx_t_4;
  __pyx_L10_bool_binop_done:;
  if (unlikely(__pyx_t_3)) {

    /* "drf_turbo/fields.pyx":1458
 *             raise StringNotCollectionError('"only" should be a list of strings')
 *         if self.exclude is not None and not is_collection(self.exclude):
 *             raise StringNotCollectionError('"exclude" should be a list of strings')             # <<<<<<<<<<<<<<
 *         super().__init__(**kwargs)
 * 
 */
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_StringNotCollectionError); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1458, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_6 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
      }
    }
    __pyx_t_2 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_6, __pyx_kp_s_exclude_should_be_a_list_of_str) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_kp_s_exclude_should_be_a_list_of_str);
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1458, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1458, __pyx_L1_error)

    /* "drf_turbo/fields.pyx":1457
 *         if self.only is not None and not is_collection(self.only):
 *             raise StringNotCollectionError('"only" should be a list of strings')
 *         if self.exclude is not None and not is_collection(self.exclude):             # <<<<<<<<<<<<<<
 *             raise StringNotCollectionError('"exclude" should be a list of strings')
 *         super().__init__(**kwargs)
 */
  }

  /* "drf_turbo/fields.pyx":1459
 *         if self.exclude is not None and not is_collection(self.exclude):
 *             raise StringNotCollectionError('"exclude" should be a list of strings')
 *         super().__init__(**kwargs)             # <<<<<<<<<<<<<<
 * 
 *     cpdef inline serialize(self,value,dict context):
 */
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1459, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_9drf_turbo_6fields_RecursiveField));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_9drf_turbo_6fields_RecursiveField));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_ptype_9drf_turbo_6fields_RecursiveField));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_2, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1459, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1459, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyDict_Copy(__pyx_v_kwargs); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1459, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1459, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

  /* "drf_turbo/fields.pyx":1448
 *     :param kwargs: The same keyword arguments that :class:`Field` receives.
 *     """
 *     def __init__(self,**kwargs):             # <<<<<<<<<<<<<<
 *         self.many = kwargs.pop('many',False)
 *         self.context = kwargs.pop('context',{})
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("drf_turbo.fields.RecursiveField.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":1461
 *         super().__init__(**kwargs)
 * 
 *     cpdef inline serialize(self,value,dict context):             # <<<<<<<<<<<<<<
 *         if self.only :
 *             serializer = self.root.__class__(value,many=self.many,only=self.only,context=self.context)
 */

static PyObject *__pyx_pw_9drf_turbo_6fields_14RecursiveField_3serialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_14RecursiveField_serialize(struct __pyx_obj_9drf_turbo_6fields_RecursiveField *__pyx_v_self, PyObject *__pyx_v_value, CYTHON_UNUSED PyObject *__pyx_v_context, CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_v_serializer = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("serialize", 0);

  /* "drf_turbo/fields.pyx":1462
 * 
 *     cpdef inline serialize(self,value,dict context):
 *         if self.only :             # <<<<<<<<<<<<<<
 *             serializer = self.root.__class__(value,many=self.many,only=self.only,context=self.context)
 *         elif self.exclude :
 */
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_self->only); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 1462, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "drf_turbo/fields.pyx":1463
 *     cpdef inline serialize(self,value,dict context):
 *         if self.only :
 *             serializer = self.root.__class__(value,many=self.many,only=self.only,context=self.context)             # <<<<<<<<<<<<<<
 *         elif self.exclude :
 *             serializer = self.root.__class__(value,many=self.many,exclude=self.exclude,context=self.context)
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->__pyx_base.root, __pyx_n_s_class); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1463, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1463, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v_value);
    __Pyx_GIVEREF(__pyx_v_value);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_value);
    __pyx_t_4 = __Pyx_PyDict_NewPresized(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1463, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_many, __pyx_v_self->many) < 0) __PYX_ERR(0, 1463, __pyx_L1_error)
    if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_only, __pyx_v_self->only) < 0) __PYX_ERR(0, 1463, __pyx_L1_error)
    if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_context, __pyx_v_self->context) < 0) __PYX_ERR(0, 1463, __pyx_L1_error)
    __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1463, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_v_serializer = __pyx_t_5;
    __pyx_t_5 = 0;

    /* "drf_turbo/fields.pyx":1462
 * 
 *     cpdef inline serialize(self,value,dict context):
 *         if self.only :             # <<<<<<<<<<<<<<
 *             serializer = self.root.__class__(value,many=self.many,only=self.only,context=self.context)
 *         elif self.exclude :
 */
    goto __pyx_L3;
  }

  /* "drf_turbo/fields.pyx":1464
 *         if self.only :
 *             serializer = self.root.__class__(value,many=self.many,only=self.only,context=self.context)
 *         elif self.exclude :             # <<<<<<<<<<<<<<
 *             serializer = self.root.__class__(value,many=self.many,exclude=self.exclude,context=self.context)
 *         else:
 */
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_self->exclude); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 1464, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "drf_turbo/fields.pyx":1465
 *             serializer = self.root.__class__(value,many=self.many,only=self.only,context=self.context)
 *         elif self.exclude :
 *             serializer = self.root.__class__(value,many=self.many,exclude=self.exclude,context=self.context)             # <<<<<<<<<<<<<<
 *         else:
 *             serializer = self.root.__class__(value,many=self.many,context=self.context)
 */
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->__pyx_base.root, __pyx_n_s_class); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1465, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1465, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(__pyx_v_value);
    __Pyx_GIVEREF(__pyx_v_value);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v_value);
    __pyx_t_3 = __Pyx_PyDict_NewPresized(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1465, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_many, __pyx_v_self->many) < 0) __PYX_ERR(0, 1465, __pyx_L1_error)
    if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_exclude, __pyx_v_self->exclude) < 0) __PYX_ERR(0, 1465, __pyx_L1_error)
    if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_context, __pyx_v_self->context) < 0) __PYX_ERR(0, 1465, __pyx_L1_error)
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_4, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1465, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_v_serializer = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "drf_turbo/fields.pyx":1464
 *         if self.only :
 *             serializer = self.root.__class__(value,many=self.many,only=self.only,context=self.context)
 *         elif self.exclude :             # <<<<<<<<<<<<<<
 *             serializer = self.root.__class__(value,many=self.many,exclude=self.exclude,context=self.context)
 *         else:
 */
    goto __pyx_L3;
  }

  /* "drf_turbo/fields.pyx":1467
 *             serializer = self.root.__class__(value,many=self.many,exclude=self.exclude,context=self.context)
 *         else:
 *             serializer = self.root.__class__(value,many=self.many,context=self.context)             # <<<<<<<<<<<<<<
 *         return serializer.data
 * 
 */
  /*else*/ {
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->__pyx_base.root, __pyx_n_s_class); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1467, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1467, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v_value);
    __Pyx_GIVEREF(__pyx_v_value);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_value);
    __pyx_t_4 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1467, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_many, __pyx_v_self->many) < 0) __PYX_ERR(0, 1467, __pyx_L1_error)
    if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_context, __pyx_v_self->context) < 0) __PYX_ERR(0, 1467, __pyx_L1_error)
    __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1467, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_v_serializer = __pyx_t_5;
    __pyx_t_5 = 0;
  }
  __pyx_L3:;

  /* "drf_turbo/fields.pyx":1468
 *         else:
 *             serializer = self.root.__class__(value,many=self.many,context=self.context)
 *         return serializer.data             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_serializer, __pyx_n_s_data); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1468, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_r = __pyx_t_5;
  __pyx_t_5 = 0;
  goto __pyx_L0;

  /* "drf_turbo/fields.pyx":1461
 *         super().__init__(**kwargs)
 * 
 *     cpdef inline serialize(self,value,dict context):             # <<<<<<<<<<<<<<
 *         if self.only :
 *             serializer = self.root.__class__(value,many=self.many,only=self.only,context=self.context)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("drf_turbo.fields.RecursiveField.serialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_serializer);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_14RecursiveField_3serialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_14RecursiveField_3serialize(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_value = 0;
  PyObject *__pyx_v_context = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("serialize (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_value,&__pyx_n_s_context,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_value)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_context)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("serialize", 1, 2, 2, 1); __PYX_ERR(0, 1461, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "serialize") < 0)) __PYX_ERR(0, 1461, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_value = values[0];
    __pyx_v_context = ((PyObject*)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("serialize", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1461, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.RecursiveField.serialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_context), (&PyDict_Type), 1, "context", 1))) __PYX_ERR(0, 1461, __pyx_L1_error)
  __pyx_r = __pyx_pf_9drf_turbo_6fields_14RecursiveField_2serialize(((struct __pyx_obj_9drf_turbo_6fields_RecursiveField *)__pyx_v_self), __pyx_v_value, __pyx_v_context);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_14RecursiveField_2serialize(struct __pyx_obj_9drf_turbo_6fields_RecursiveField *__pyx_v_self, PyObject *__pyx_v_value, PyObject *__pyx_v_context) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("serialize", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields_14RecursiveField_serialize(__pyx_v_self, __pyx_v_value, __pyx_v_context, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1461, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.RecursiveField.serialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pxd":170
 * cdef class RecursiveField(Field):
 *     cdef public :
 *         many             # <<<<<<<<<<<<<<
 *         context
 *         only
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_14RecursiveField_4many_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_14RecursiveField_4many_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_14RecursiveField_4many___get__(((struct __pyx_obj_9drf_turbo_6fields_RecursiveField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_14RecursiveField_4many___get__(struct __pyx_obj_9drf_turbo_6fields_RecursiveField *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->many);
  __pyx_r = __pyx_v_self->many;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_14RecursiveField_4many_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_14RecursiveField_4many_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_14RecursiveField_4many_2__set__(((struct __pyx_obj_9drf_turbo_6fields_RecursiveField *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_14RecursiveField_4many_2__set__(struct __pyx_obj_9drf_turbo_6fields_RecursiveField *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->many);
  __Pyx_DECREF(__pyx_v_self->many);
  __pyx_v_self->many = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_14RecursiveField_4many_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_14RecursiveField_4many_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_14RecursiveField_4many_4__del__(((struct __pyx_obj_9drf_turbo_6fields_RecursiveField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_14RecursiveField_4many_4__del__(struct __pyx_obj_9drf_turbo_6fields_RecursiveField *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->many);
  __Pyx_DECREF(__pyx_v_self->many);
  __pyx_v_self->many = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pxd":171
 *     cdef public :
 *         many
 *         context             # <<<<<<<<<<<<<<
 *         only
 *         exclude
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_14RecursiveField_7context_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_14RecursiveField_7context_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_14RecursiveField_7context___get__(((struct __pyx_obj_9drf_turbo_6fields_RecursiveField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_14RecursiveField_7context___get__(struct __pyx_obj_9drf_turbo_6fields_RecursiveField *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->context);
  __pyx_r = __pyx_v_self->context;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_14RecursiveField_7context_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_14RecursiveField_7context_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_14RecursiveField_7context_2__set__(((struct __pyx_obj_9drf_turbo_6fields_RecursiveField *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_14RecursiveField_7context_2__set__(struct __pyx_obj_9drf_turbo_6fields_RecursiveField *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->context);
  __Pyx_DECREF(__pyx_v_self->context);
  __pyx_v_self->context = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_14RecursiveField_7context_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_14RecursiveField_7context_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_14RecursiveField_7context_4__del__(((struct __pyx_obj_9drf_turbo_6fields_RecursiveField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_14RecursiveField_7context_4__del__(struct __pyx_obj_9drf_turbo_6fields_RecursiveField *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->context);
  __Pyx_DECREF(__pyx_v_self->context);
  __pyx_v_self->context = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pxd":172
 *         many
 *         context
 *         only             # <<<<<<<<<<<<<<
 *         exclude
 *     cpdef serialize(self,value,dict context)
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_14RecursiveField_4only_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_14RecursiveField_4only_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_14RecursiveField_4only___get__(((struct __pyx_obj_9drf_turbo_6fields_RecursiveField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_14RecursiveField_4only___get__(struct __pyx_obj_9drf_turbo_6fields_RecursiveField *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->only);
  __pyx_r = __pyx_v_self->only;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_14RecursiveField_4only_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_14RecursiveField_4only_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_14RecursiveField_4only_2__set__(((struct __pyx_obj_9drf_turbo_6fields_RecursiveField *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_14RecursiveField_4only_2__set__(struct __pyx_obj_9drf_turbo_6fields_RecursiveField *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->only);
  __Pyx_DECREF(__pyx_v_self->only);
  __pyx_v_self->only = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_14RecursiveField_4only_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_14RecursiveField_4only_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_14RecursiveField_4only_4__del__(((struct __pyx_obj_9drf_turbo_6fields_RecursiveField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_14RecursiveField_4only_4__del__(struct __pyx_obj_9drf_turbo_6fields_RecursiveField *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->only);
  __Pyx_DECREF(__pyx_v_self->only);
  __pyx_v_self->only = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pxd":173
 *         context
 *         only
 *         exclude             # <<<<<<<<<<<<<<
 *     cpdef serialize(self,value,dict context)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_14RecursiveField_7exclude_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_14RecursiveField_7exclude_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_14RecursiveField_7exclude___get__(((struct __pyx_obj_9drf_turbo_6fields_RecursiveField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_14RecursiveField_7exclude___get__(struct __pyx_obj_9drf_turbo_6fields_RecursiveField *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->exclude);
  __pyx_r = __pyx_v_self->exclude;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_14RecursiveField_7exclude_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_14RecursiveField_7exclude_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_14RecursiveField_7exclude_2__set__(((struct __pyx_obj_9drf_turbo_6fields_RecursiveField *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_14RecursiveField_7exclude_2__set__(struct __pyx_obj_9drf_turbo_6fields_RecursiveField *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->exclude);
  __Pyx_DECREF(__pyx_v_self->exclude);
  __pyx_v_self->exclude = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_14RecursiveField_7exclude_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_14RecursiveField_7exclude_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_14RecursiveField_7exclude_4__del__(((struct __pyx_obj_9drf_turbo_6fields_RecursiveField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_14RecursiveField_7exclude_4__del__(struct __pyx_obj_9drf_turbo_6fields_RecursiveField *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->exclude);
  __Pyx_DECREF(__pyx_v_self->exclude);
  __pyx_v_self->exclude = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_14RecursiveField_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_14RecursiveField_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_14RecursiveField_4__reduce_cython__(((struct __pyx_obj_9drf_turbo_6fields_RecursiveField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_14RecursiveField_4__reduce_cython__(struct __pyx_obj_9drf_turbo_6fields_RecursiveField *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self.allow_null, self.attr, self.attrs, self.call, self.context, self.default_value, self.error_messages, self.exclude, self.field_name, self.help_text, self.initial, self.label, self.many, self.only, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.allow_null); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.call); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.read_only); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.required); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.write_only); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = PyTuple_New(20); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.attr);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.attr);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_v_self->__pyx_base.attr);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.attrs);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.attrs);
  PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_self->__pyx_base.attrs);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_6, 3, __pyx_t_2);
  __Pyx_INCREF(__pyx_v_self->context);
  __Pyx_GIVEREF(__pyx_v_self->context);
  PyTuple_SET_ITEM(__pyx_t_6, 4, __pyx_v_self->context);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.default_value);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.default_value);
  PyTuple_SET_ITEM(__pyx_t_6, 5, __pyx_v_self->__pyx_base.default_value);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.error_messages);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.error_messages);
  PyTuple_SET_ITEM(__pyx_t_6, 6, __pyx_v_self->__pyx_base.error_messages);
  __Pyx_INCREF(__pyx_v_self->exclude);
  __Pyx_GIVEREF(__pyx_v_self->exclude);
  PyTuple_SET_ITEM(__pyx_t_6, 7, __pyx_v_self->exclude);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.field_name);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.field_name);
  PyTuple_SET_ITEM(__pyx_t_6, 8, __pyx_v_self->__pyx_base.field_name);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.help_text);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.help_text);
  PyTuple_SET_ITEM(__pyx_t_6, 9, __pyx_v_self->__pyx_base.help_text);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.initial);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.initial);
  PyTuple_SET_ITEM(__pyx_t_6, 10, __pyx_v_self->__pyx_base.initial);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.label);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.label);
  PyTuple_SET_ITEM(__pyx_t_6, 11, __pyx_v_self->__pyx_base.label);
  __Pyx_INCREF(__pyx_v_self->many);
  __Pyx_GIVEREF(__pyx_v_self->many);
  PyTuple_SET_ITEM(__pyx_t_6, 12, __pyx_v_self->many);
  __Pyx_INCREF(__pyx_v_self->only);
  __Pyx_GIVEREF(__pyx_v_self->only);
  PyTuple_SET_ITEM(__pyx_t_6, 13, __pyx_v_self->only);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 14, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_6, 15, __pyx_t_4);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.root);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.root);
  PyTuple_SET_ITEM(__pyx_t_6, 16, __pyx_v_self->__pyx_base.root);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.style);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.style);
  PyTuple_SET_ITEM(__pyx_t_6, 17, __pyx_v_self->__pyx_base.style);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.validators);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.validators);
  PyTuple_SET_ITEM(__pyx_t_6, 18, __pyx_v_self->__pyx_base.validators);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_6, 19, __pyx_t_5);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_6);
  __pyx_t_6 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self.allow_null, self.attr, self.attrs, self.call, self.context, self.default_value, self.error_messages, self.exclude, self.field_name, self.help_text, self.initial, self.label, self.many, self.only, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_6 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_v__dict = __pyx_t_6;
  __pyx_t_6 = 0;

  /* "(tree fragment)":7
 *     state = (self.allow_null, self.attr, self.attrs, self.call, self.context, self.default_value, self.error_messages, self.exclude, self.field_name, self.help_text, self.initial, self.label, self.many, self.only, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_7 = (__pyx_v__dict != Py_None);
  __pyx_t_8 = (__pyx_t_7 != 0);
  if (__pyx_t_8) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_6 = PyTuple_New(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v__dict);
    __pyx_t_5 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_6); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_5));
    __pyx_t_5 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.attr is not None or self.attrs is not None or self.context is not None or self.default_value is not None or self.error_messages is not None or self.exclude is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.many is not None or self.only is not None or self.root is not None or self.style is not None or self.validators is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self.allow_null, self.attr, self.attrs, self.call, self.context, self.default_value, self.error_messages, self.exclude, self.field_name, self.help_text, self.initial, self.label, self.many, self.only, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self.attr is not None or self.attrs is not None or self.context is not None or self.default_value is not None or self.error_messages is not None or self.exclude is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.many is not None or self.only is not None or self.root is not None or self.style is not None or self.validators is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_RecursiveField, (type(self), 0xf725730, None), state
 */
  /*else*/ {
    __pyx_t_7 = (__pyx_v_self->__pyx_base.attr != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.attrs != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->context != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.default_value != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.error_messages != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->exclude != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.field_name != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.help_text != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.initial != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.label != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->many != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->only != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.root != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.style != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.validators != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    __pyx_t_8 = __pyx_t_9;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_8;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.attr is not None or self.attrs is not None or self.context is not None or self.default_value is not None or self.error_messages is not None or self.exclude is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.many is not None or self.only is not None or self.root is not None or self.style is not None or self.validators is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_RecursiveField, (type(self), 0xf725730, None), state
 *     else:
 */
  __pyx_t_8 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_8) {

    /* "(tree fragment)":13
 *         use_setstate = self.attr is not None or self.attrs is not None or self.context is not None or self.default_value is not None or self.error_messages is not None or self.exclude is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.many is not None or self.only is not None or self.root is not None or self.style is not None or self.validators is not None
 *     if use_setstate:
 *         return __pyx_unpickle_RecursiveField, (type(self), 0xf725730, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_RecursiveField, (type(self), 0xf725730, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_pyx_unpickle_RecursiveField); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_6, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_259151664);
    __Pyx_GIVEREF(__pyx_int_259151664);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_int_259151664);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_6, 2, Py_None);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_6);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_state);
    __pyx_t_5 = 0;
    __pyx_t_6 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.attr is not None or self.attrs is not None or self.context is not None or self.default_value is not None or self.error_messages is not None or self.exclude is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.many is not None or self.only is not None or self.root is not None or self.style is not None or self.validators is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_RecursiveField, (type(self), 0xf725730, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_RecursiveField, (type(self), 0xf725730, None), state
 *     else:
 *         return __pyx_unpickle_RecursiveField, (type(self), 0xf725730, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_RecursiveField__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_pyx_unpickle_RecursiveField); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_6, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_259151664);
    __Pyx_GIVEREF(__pyx_int_259151664);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_int_259151664);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_state);
    __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_6);
    __pyx_t_4 = 0;
    __pyx_t_6 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("drf_turbo.fields.RecursiveField.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_RecursiveField, (type(self), 0xf725730, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_RecursiveField__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_14RecursiveField_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_14RecursiveField_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_14RecursiveField_6__setstate_cython__(((struct __pyx_obj_9drf_turbo_6fields_RecursiveField *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_14RecursiveField_6__setstate_cython__(struct __pyx_obj_9drf_turbo_6fields_RecursiveField *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_RecursiveField, (type(self), 0xf725730, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_RecursiveField__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields___pyx_unpickle_RecursiveField__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_RecursiveField, (type(self), 0xf725730, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_RecursiveField__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.RecursiveField.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":1481
 *     is_method_field = True
 * 
 *     def __init__(self,method_name=None,**kwargs):             # <<<<<<<<<<<<<<
 *         kwargs['read_only']= True
 *         kwargs['required'] = False
 */

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_11MethodField_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_11MethodField_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_method_name = 0;
  PyObject *__pyx_v_kwargs = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  __pyx_v_kwargs = PyDict_New(); if (unlikely(!__pyx_v_kwargs)) return -1;
  __Pyx_GOTREF(__pyx_v_kwargs);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_method_name,0};
    PyObject* values[1] = {0};
    values[0] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_method_name);
          if (value) { values[0] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, __pyx_v_kwargs, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 1481, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_method_name = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 0, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1481, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_DECREF(__pyx_v_kwargs); __pyx_v_kwargs = 0;
  __Pyx_AddTraceback("drf_turbo.fields.MethodField.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9drf_turbo_6fields_11MethodField___init__(((struct __pyx_obj_9drf_turbo_6fields_MethodField *)__pyx_v_self), __pyx_v_method_name, __pyx_v_kwargs);

  /* function exit code */
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_11MethodField___init__(struct __pyx_obj_9drf_turbo_6fields_MethodField *__pyx_v_self, PyObject *__pyx_v_method_name, PyObject *__pyx_v_kwargs) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "drf_turbo/fields.pyx":1482
 * 
 *     def __init__(self,method_name=None,**kwargs):
 *         kwargs['read_only']= True             # <<<<<<<<<<<<<<
 *         kwargs['required'] = False
 *         self.method_name = method_name
 */
  if (unlikely(PyDict_SetItem(__pyx_v_kwargs, __pyx_n_s_read_only, Py_True) < 0)) __PYX_ERR(0, 1482, __pyx_L1_error)

  /* "drf_turbo/fields.pyx":1483
 *     def __init__(self,method_name=None,**kwargs):
 *         kwargs['read_only']= True
 *         kwargs['required'] = False             # <<<<<<<<<<<<<<
 *         self.method_name = method_name
 *         super().__init__(**kwargs)
 */
  if (unlikely(PyDict_SetItem(__pyx_v_kwargs, __pyx_n_s_required, Py_False) < 0)) __PYX_ERR(0, 1483, __pyx_L1_error)

  /* "drf_turbo/fields.pyx":1484
 *         kwargs['read_only']= True
 *         kwargs['required'] = False
 *         self.method_name = method_name             # <<<<<<<<<<<<<<
 *         super().__init__(**kwargs)
 * 
 */
  __Pyx_INCREF(__pyx_v_method_name);
  __Pyx_GIVEREF(__pyx_v_method_name);
  __Pyx_GOTREF(__pyx_v_self->method_name);
  __Pyx_DECREF(__pyx_v_self->method_name);
  __pyx_v_self->method_name = __pyx_v_method_name;

  /* "drf_turbo/fields.pyx":1485
 *         kwargs['required'] = False
 *         self.method_name = method_name
 *         super().__init__(**kwargs)             # <<<<<<<<<<<<<<
 * 
 *     cpdef inline method_getter(self,field_name, root) :
 */
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1485, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_9drf_turbo_6fields_MethodField));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_9drf_turbo_6fields_MethodField));
  PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)__pyx_ptype_9drf_turbo_6fields_MethodField));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_1, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1485, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_init); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1485, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyDict_Copy(__pyx_v_kwargs); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1485, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1485, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "drf_turbo/fields.pyx":1481
 *     is_method_field = True
 * 
 *     def __init__(self,method_name=None,**kwargs):             # <<<<<<<<<<<<<<
 *         kwargs['read_only']= True
 *         kwargs['required'] = False
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("drf_turbo.fields.MethodField.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pyx":1487
 *         super().__init__(**kwargs)
 * 
 *     cpdef inline method_getter(self,field_name, root) :             # <<<<<<<<<<<<<<
 *         if self.method_name is None:
 *             self.method_name = 'get_{0}'.format(field_name)
 */

static PyObject *__pyx_pw_9drf_turbo_6fields_11MethodField_3method_getter(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_9drf_turbo_6fields_11MethodField_method_getter(struct __pyx_obj_9drf_turbo_6fields_MethodField *__pyx_v_self, PyObject *__pyx_v_field_name, PyObject *__pyx_v_root, CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("method_getter", 0);

  /* "drf_turbo/fields.pyx":1488
 * 
 *     cpdef inline method_getter(self,field_name, root) :
 *         if self.method_name is None:             # <<<<<<<<<<<<<<
 *             self.method_name = 'get_{0}'.format(field_name)
 *         return getattr(root, self.method_name)
 */
  __pyx_t_1 = (__pyx_v_self->method_name == Py_None);
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "drf_turbo/fields.pyx":1489
 *     cpdef inline method_getter(self,field_name, root) :
 *         if self.method_name is None:
 *             self.method_name = 'get_{0}'.format(field_name)             # <<<<<<<<<<<<<<
 *         return getattr(root, self.method_name)
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_get__0, __pyx_n_s_format); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1489, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_5, __pyx_v_field_name) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_v_field_name);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1489, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GIVEREF(__pyx_t_3);
    __Pyx_GOTREF(__pyx_v_self->method_name);
    __Pyx_DECREF(__pyx_v_self->method_name);
    __pyx_v_self->method_name = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "drf_turbo/fields.pyx":1488
 * 
 *     cpdef inline method_getter(self,field_name, root) :
 *         if self.method_name is None:             # <<<<<<<<<<<<<<
 *             self.method_name = 'get_{0}'.format(field_name)
 *         return getattr(root, self.method_name)
 */
  }

  /* "drf_turbo/fields.pyx":1490
 *         if self.method_name is None:
 *             self.method_name = 'get_{0}'.format(field_name)
 *         return getattr(root, self.method_name)             # <<<<<<<<<<<<<<
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_3 = __pyx_v_self->method_name;
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_GetAttr(__pyx_v_root, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1490, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "drf_turbo/fields.pyx":1487
 *         super().__init__(**kwargs)
 * 
 *     cpdef inline method_getter(self,field_name, root) :             # <<<<<<<<<<<<<<
 *         if self.method_name is None:
 *             self.method_name = 'get_{0}'.format(field_name)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("drf_turbo.fields.MethodField.method_getter", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_11MethodField_3method_getter(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_11MethodField_3method_getter(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_field_name = 0;
  PyObject *__pyx_v_root = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("method_getter (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_field_name,&__pyx_n_s_root,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_field_name)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_root)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("method_getter", 1, 2, 2, 1); __PYX_ERR(0, 1487, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "method_getter") < 0)) __PYX_ERR(0, 1487, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_field_name = values[0];
    __pyx_v_root = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("method_getter", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1487, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.MethodField.method_getter", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9drf_turbo_6fields_11MethodField_2method_getter(((struct __pyx_obj_9drf_turbo_6fields_MethodField *)__pyx_v_self), __pyx_v_field_name, __pyx_v_root);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_11MethodField_2method_getter(struct __pyx_obj_9drf_turbo_6fields_MethodField *__pyx_v_self, PyObject *__pyx_v_field_name, PyObject *__pyx_v_root) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("method_getter", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields_11MethodField_method_getter(__pyx_v_self, __pyx_v_field_name, __pyx_v_root, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1487, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.MethodField.method_getter", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "drf_turbo/fields.pxd":178
 * cdef class MethodField(Field):
 *     cdef public :
 *         method_name             # <<<<<<<<<<<<<<
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_11MethodField_11method_name_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_11MethodField_11method_name_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_11MethodField_11method_name___get__(((struct __pyx_obj_9drf_turbo_6fields_MethodField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_11MethodField_11method_name___get__(struct __pyx_obj_9drf_turbo_6fields_MethodField *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->method_name);
  __pyx_r = __pyx_v_self->method_name;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_11MethodField_11method_name_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_11MethodField_11method_name_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_11MethodField_11method_name_2__set__(((struct __pyx_obj_9drf_turbo_6fields_MethodField *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_11MethodField_11method_name_2__set__(struct __pyx_obj_9drf_turbo_6fields_MethodField *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->method_name);
  __Pyx_DECREF(__pyx_v_self->method_name);
  __pyx_v_self->method_name = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9drf_turbo_6fields_11MethodField_11method_name_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9drf_turbo_6fields_11MethodField_11method_name_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_11MethodField_11method_name_4__del__(((struct __pyx_obj_9drf_turbo_6fields_MethodField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9drf_turbo_6fields_11MethodField_11method_name_4__del__(struct __pyx_obj_9drf_turbo_6fields_MethodField *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->method_name);
  __Pyx_DECREF(__pyx_v_self->method_name);
  __pyx_v_self->method_name = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_11MethodField_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_11MethodField_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_11MethodField_4__reduce_cython__(((struct __pyx_obj_9drf_turbo_6fields_MethodField *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_11MethodField_4__reduce_cython__(struct __pyx_obj_9drf_turbo_6fields_MethodField *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self.allow_null, self.attr, self.attrs, self.call, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.method_name, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.allow_null); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.call); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.read_only); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.required); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.write_only); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = PyTuple_New(17); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.attr);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.attr);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_v_self->__pyx_base.attr);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.attrs);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.attrs);
  PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_self->__pyx_base.attrs);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_6, 3, __pyx_t_2);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.default_value);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.default_value);
  PyTuple_SET_ITEM(__pyx_t_6, 4, __pyx_v_self->__pyx_base.default_value);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.error_messages);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.error_messages);
  PyTuple_SET_ITEM(__pyx_t_6, 5, __pyx_v_self->__pyx_base.error_messages);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.field_name);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.field_name);
  PyTuple_SET_ITEM(__pyx_t_6, 6, __pyx_v_self->__pyx_base.field_name);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.help_text);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.help_text);
  PyTuple_SET_ITEM(__pyx_t_6, 7, __pyx_v_self->__pyx_base.help_text);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.initial);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.initial);
  PyTuple_SET_ITEM(__pyx_t_6, 8, __pyx_v_self->__pyx_base.initial);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.label);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.label);
  PyTuple_SET_ITEM(__pyx_t_6, 9, __pyx_v_self->__pyx_base.label);
  __Pyx_INCREF(__pyx_v_self->method_name);
  __Pyx_GIVEREF(__pyx_v_self->method_name);
  PyTuple_SET_ITEM(__pyx_t_6, 10, __pyx_v_self->method_name);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 11, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_6, 12, __pyx_t_4);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.root);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.root);
  PyTuple_SET_ITEM(__pyx_t_6, 13, __pyx_v_self->__pyx_base.root);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.style);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.style);
  PyTuple_SET_ITEM(__pyx_t_6, 14, __pyx_v_self->__pyx_base.style);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.validators);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.validators);
  PyTuple_SET_ITEM(__pyx_t_6, 15, __pyx_v_self->__pyx_base.validators);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_6, 16, __pyx_t_5);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_6);
  __pyx_t_6 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self.allow_null, self.attr, self.attrs, self.call, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.method_name, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_6 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_v__dict = __pyx_t_6;
  __pyx_t_6 = 0;

  /* "(tree fragment)":7
 *     state = (self.allow_null, self.attr, self.attrs, self.call, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.method_name, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_7 = (__pyx_v__dict != Py_None);
  __pyx_t_8 = (__pyx_t_7 != 0);
  if (__pyx_t_8) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_6 = PyTuple_New(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v__dict);
    __pyx_t_5 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_6); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_5));
    __pyx_t_5 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.method_name is not None or self.root is not None or self.style is not None or self.validators is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self.allow_null, self.attr, self.attrs, self.call, self.default_value, self.error_messages, self.field_name, self.help_text, self.initial, self.label, self.method_name, self.read_only, self.required, self.root, self.style, self.validators, self.write_only)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.method_name is not None or self.root is not None or self.style is not None or self.validators is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_MethodField, (type(self), 0xc705305, None), state
 */
  /*else*/ {
    __pyx_t_7 = (__pyx_v_self->__pyx_base.attr != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.attrs != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.default_value != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.error_messages != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.field_name != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.help_text != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.initial != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.label != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->method_name != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.root != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->__pyx_base.style != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_self->__pyx_base.validators != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    __pyx_t_8 = __pyx_t_7;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_8;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.method_name is not None or self.root is not None or self.style is not None or self.validators is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MethodField, (type(self), 0xc705305, None), state
 *     else:
 */
  __pyx_t_8 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_8) {

    /* "(tree fragment)":13
 *         use_setstate = self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.method_name is not None or self.root is not None or self.style is not None or self.validators is not None
 *     if use_setstate:
 *         return __pyx_unpickle_MethodField, (type(self), 0xc705305, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_MethodField, (type(self), 0xc705305, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_pyx_unpickle_MethodField); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_6, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_208687877);
    __Pyx_GIVEREF(__pyx_int_208687877);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_int_208687877);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_6, 2, Py_None);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_6);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_state);
    __pyx_t_5 = 0;
    __pyx_t_6 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.attr is not None or self.attrs is not None or self.default_value is not None or self.error_messages is not None or self.field_name is not None or self.help_text is not None or self.initial is not None or self.label is not None or self.method_name is not None or self.root is not None or self.style is not None or self.validators is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MethodField, (type(self), 0xc705305, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_MethodField, (type(self), 0xc705305, None), state
 *     else:
 *         return __pyx_unpickle_MethodField, (type(self), 0xc705305, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MethodField__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_pyx_unpickle_MethodField); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_6, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_208687877);
    __Pyx_GIVEREF(__pyx_int_208687877);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_int_208687877);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_state);
    __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_6);
    __pyx_t_4 = 0;
    __pyx_t_6 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("drf_turbo.fields.MethodField.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MethodField, (type(self), 0xc705305, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MethodField__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_11MethodField_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_9drf_turbo_6fields_11MethodField_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9drf_turbo_6fields_11MethodField_6__setstate_cython__(((struct __pyx_obj_9drf_turbo_6fields_MethodField *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_11MethodField_6__setstate_cython__(struct __pyx_obj_9drf_turbo_6fields_MethodField *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_MethodField, (type(self), 0xc705305, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MethodField__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_9drf_turbo_6fields___pyx_unpickle_MethodField__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MethodField, (type(self), 0xc705305, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MethodField__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("drf_turbo.fields.MethodField.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_SkipField(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_1__pyx_unpickle_SkipField(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_9drf_turbo_6fields_1__pyx_unpickle_SkipField = {"__pyx_unpickle_SkipField", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_1__pyx_unpickle_SkipField, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_9drf_turbo_6fields_1__pyx_unpickle_SkipField(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_SkipField (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_SkipField", 1, 3, 3, 1); __PYX_ERR(1, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_SkipField", 1, 3, 3, 2); __PYX_ERR(1, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_SkipField") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_SkipField", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_SkipField", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9drf_turbo_6fields___pyx_unpickle_SkipField(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields___pyx_unpickle_SkipField(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_SkipField", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xd41d8cd:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xd41d8cd = ())" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0xd41d8cd) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xd41d8cd:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xd41d8cd = ())" % __pyx_checksum)
 *     __pyx_result = SkipField.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0xd41d8cd:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xd41d8cd = ())" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = SkipField.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0xd4, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(1, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xd41d8cd:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xd41d8cd = ())" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xd41d8cd = ())" % __pyx_checksum)
 *     __pyx_result = SkipField.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_SkipField__set_state(<SkipField> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_9drf_turbo_6fields_SkipField), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xd41d8cd = ())" % __pyx_checksum)
 *     __pyx_result = SkipField.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_SkipField__set_state(<SkipField> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = SkipField.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_SkipField__set_state(<SkipField> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_SkipField__set_state(SkipField __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_9drf_turbo_6fields___pyx_unpickle_SkipField__set_state(((struct __pyx_obj_9drf_turbo_6fields_SkipField *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xd41d8cd = ())" % __pyx_checksum)
 *     __pyx_result = SkipField.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_SkipField__set_state(<SkipField> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_SkipField__set_state(<SkipField> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_SkipField__set_state(SkipField __pyx_result, tuple __pyx_state):
 *     if len(__pyx_state) > 0 and hasattr(__pyx_result, '__dict__'):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_SkipField(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_SkipField", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_SkipField__set_state(<SkipField> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_SkipField__set_state(SkipField __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 0 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[0])
 */

static PyObject *__pyx_f_9drf_turbo_6fields___pyx_unpickle_SkipField__set_state(struct __pyx_obj_9drf_turbo_6fields_SkipField *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_SkipField__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_SkipField__set_state(SkipField __pyx_result, tuple __pyx_state):
 *     if len(__pyx_state) > 0 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[0])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_2 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_2 == ((Py_ssize_t)-1))) __PYX_ERR(1, 12, __pyx_L1_error)
  __pyx_t_3 = ((__pyx_t_2 > 0) != 0);
  if (__pyx_t_3) {
  } else {
    __pyx_t_1 = __pyx_t_3;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_3 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(1, 12, __pyx_L1_error)
  __pyx_t_4 = (__pyx_t_3 != 0);
  __pyx_t_1 = __pyx_t_4;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_SkipField__set_state(SkipField __pyx_result, tuple __pyx_state):
 *     if len(__pyx_state) > 0 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[0])             # <<<<<<<<<<<<<<
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_update); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 13, __pyx_L1_error)
    }
    __pyx_t_6 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_8 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
      }
    }
    __pyx_t_5 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_8, __pyx_t_6) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_6);
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_SkipField__set_state(SkipField __pyx_result, tuple __pyx_state):
 *     if len(__pyx_state) > 0 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[0])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_SkipField__set_state(<SkipField> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_SkipField__set_state(SkipField __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 0 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[0])
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_SkipField__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_Field(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_3__pyx_unpickle_Field(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_9drf_turbo_6fields_3__pyx_unpickle_Field = {"__pyx_unpickle_Field", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_3__pyx_unpickle_Field, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_9drf_turbo_6fields_3__pyx_unpickle_Field(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_Field (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_Field", 1, 3, 3, 1); __PYX_ERR(1, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_Field", 1, 3, 3, 2); __PYX_ERR(1, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_Field") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_Field", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_Field", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9drf_turbo_6fields_2__pyx_unpickle_Field(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_2__pyx_unpickle_Field(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_Field", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x4f70d27:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x4f70d27 = (allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x4f70d27) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x4f70d27:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x4f70d27 = (allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = Field.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0x4f70d27:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x4f70d27 = (allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = Field.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x4f, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(1, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x4f70d27:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x4f70d27 = (allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x4f70d27 = (allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = Field.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_Field__set_state(<Field> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_9drf_turbo_6fields_Field), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x4f70d27 = (allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = Field.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_Field__set_state(<Field> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = Field.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_Field__set_state(<Field> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_Field__set_state(Field __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_9drf_turbo_6fields___pyx_unpickle_Field__set_state(((struct __pyx_obj_9drf_turbo_6fields_Field *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x4f70d27 = (allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = Field.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_Field__set_state(<Field> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_Field__set_state(<Field> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_Field__set_state(Field __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.default_value = __pyx_state[4]; __pyx_result.error_messages = __pyx_state[5]; __pyx_result.field_name = __pyx_state[6]; __pyx_result.help_text = __pyx_state[7]; __pyx_result.initial = __pyx_state[8]; __pyx_result.label = __pyx_state[9]; __pyx_result.read_only = __pyx_state[10]; __pyx_result.required = __pyx_state[11]; __pyx_result.root = __pyx_state[12]; __pyx_result.style = __pyx_state[13]; __pyx_result.validators = __pyx_state[14]; __pyx_result.write_only = __pyx_state[15]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_Field(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_Field", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_Field__set_state(<Field> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_Field__set_state(Field __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.default_value = __pyx_state[4]; __pyx_result.error_messages = __pyx_state[5]; __pyx_result.field_name = __pyx_state[6]; __pyx_result.help_text = __pyx_state[7]; __pyx_result.initial = __pyx_state[8]; __pyx_result.label = __pyx_state[9]; __pyx_result.read_only = __pyx_state[10]; __pyx_result.required = __pyx_state[11]; __pyx_result.root = __pyx_state[12]; __pyx_result.style = __pyx_state[13]; __pyx_result.validators = __pyx_state[14]; __pyx_result.write_only = __pyx_state[15]
 *     if len(__pyx_state) > 16 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_9drf_turbo_6fields___pyx_unpickle_Field__set_state(struct __pyx_obj_9drf_turbo_6fields_Field *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_Field__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_Field__set_state(Field __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.default_value = __pyx_state[4]; __pyx_result.error_messages = __pyx_state[5]; __pyx_result.field_name = __pyx_state[6]; __pyx_result.help_text = __pyx_state[7]; __pyx_result.initial = __pyx_state[8]; __pyx_result.label = __pyx_state[9]; __pyx_result.read_only = __pyx_state[10]; __pyx_result.required = __pyx_state[11]; __pyx_result.root = __pyx_state[12]; __pyx_result.style = __pyx_state[13]; __pyx_result.validators = __pyx_state[14]; __pyx_result.write_only = __pyx_state[15]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 16 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[16])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->allow_null = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->attr);
  __Pyx_DECREF(__pyx_v___pyx_result->attr);
  __pyx_v___pyx_result->attr = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->attrs);
  __Pyx_DECREF(__pyx_v___pyx_result->attrs);
  __pyx_v___pyx_result->attrs = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->call = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->default_value);
  __Pyx_DECREF(__pyx_v___pyx_result->default_value);
  __pyx_v___pyx_result->default_value = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->error_messages);
  __Pyx_DECREF(__pyx_v___pyx_result->error_messages);
  __pyx_v___pyx_result->error_messages = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->field_name);
  __Pyx_DECREF(__pyx_v___pyx_result->field_name);
  __pyx_v___pyx_result->field_name = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->help_text);
  __Pyx_DECREF(__pyx_v___pyx_result->help_text);
  __pyx_v___pyx_result->help_text = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 8, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->initial);
  __Pyx_DECREF(__pyx_v___pyx_result->initial);
  __pyx_v___pyx_result->initial = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 9, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->label);
  __Pyx_DECREF(__pyx_v___pyx_result->label);
  __pyx_v___pyx_result->label = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 10, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->read_only = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 11, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->required = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 12, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->root);
  __Pyx_DECREF(__pyx_v___pyx_result->root);
  __pyx_v___pyx_result->root = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 13, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->style);
  __Pyx_DECREF(__pyx_v___pyx_result->style);
  __pyx_v___pyx_result->style = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 14, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->validators);
  __Pyx_DECREF(__pyx_v___pyx_result->validators);
  __pyx_v___pyx_result->validators = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 15, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->write_only = __pyx_t_2;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_Field__set_state(Field __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.default_value = __pyx_state[4]; __pyx_result.error_messages = __pyx_state[5]; __pyx_result.field_name = __pyx_state[6]; __pyx_result.help_text = __pyx_state[7]; __pyx_result.initial = __pyx_state[8]; __pyx_result.label = __pyx_state[9]; __pyx_result.read_only = __pyx_state[10]; __pyx_result.required = __pyx_state[11]; __pyx_result.root = __pyx_state[12]; __pyx_result.style = __pyx_state[13]; __pyx_result.validators = __pyx_state[14]; __pyx_result.write_only = __pyx_state[15]
 *     if len(__pyx_state) > 16 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[16])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 13, __pyx_L1_error)
  }
  __pyx_t_3 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_4 = ((__pyx_t_3 > 16) != 0);
  if (__pyx_t_4) {
  } else {
    __pyx_t_2 = __pyx_t_4;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_4 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_5 = (__pyx_t_4 != 0);
  __pyx_t_2 = __pyx_t_5;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":14
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.default_value = __pyx_state[4]; __pyx_result.error_messages = __pyx_state[5]; __pyx_result.field_name = __pyx_state[6]; __pyx_result.help_text = __pyx_state[7]; __pyx_result.initial = __pyx_state[8]; __pyx_result.label = __pyx_state[9]; __pyx_result.read_only = __pyx_state[10]; __pyx_result.required = __pyx_state[11]; __pyx_result.root = __pyx_state[12]; __pyx_result.style = __pyx_state[13]; __pyx_result.validators = __pyx_state[14]; __pyx_result.write_only = __pyx_state[15]
 *     if len(__pyx_state) > 16 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[16])             # <<<<<<<<<<<<<<
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_update); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 14, __pyx_L1_error)
    }
    __pyx_t_6 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 16, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_8 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
      }
    }
    __pyx_t_1 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_8, __pyx_t_6) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_6);
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_Field__set_state(Field __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.default_value = __pyx_state[4]; __pyx_result.error_messages = __pyx_state[5]; __pyx_result.field_name = __pyx_state[6]; __pyx_result.help_text = __pyx_state[7]; __pyx_result.initial = __pyx_state[8]; __pyx_result.label = __pyx_state[9]; __pyx_result.read_only = __pyx_state[10]; __pyx_result.required = __pyx_state[11]; __pyx_result.root = __pyx_state[12]; __pyx_result.style = __pyx_state[13]; __pyx_result.validators = __pyx_state[14]; __pyx_result.write_only = __pyx_state[15]
 *     if len(__pyx_state) > 16 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[16])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_Field__set_state(<Field> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_Field__set_state(Field __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.default_value = __pyx_state[4]; __pyx_result.error_messages = __pyx_state[5]; __pyx_result.field_name = __pyx_state[6]; __pyx_result.help_text = __pyx_state[7]; __pyx_result.initial = __pyx_state[8]; __pyx_result.label = __pyx_state[9]; __pyx_result.read_only = __pyx_state[10]; __pyx_result.required = __pyx_state[11]; __pyx_result.root = __pyx_state[12]; __pyx_result.style = __pyx_state[13]; __pyx_result.validators = __pyx_state[14]; __pyx_result.write_only = __pyx_state[15]
 *     if len(__pyx_state) > 16 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_Field__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_StrField(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_5__pyx_unpickle_StrField(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_9drf_turbo_6fields_5__pyx_unpickle_StrField = {"__pyx_unpickle_StrField", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_5__pyx_unpickle_StrField, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_9drf_turbo_6fields_5__pyx_unpickle_StrField(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_StrField (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_StrField", 1, 3, 3, 1); __PYX_ERR(1, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_StrField", 1, 3, 3, 2); __PYX_ERR(1, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_StrField") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_StrField", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_StrField", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9drf_turbo_6fields_4__pyx_unpickle_StrField(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_4__pyx_unpickle_StrField(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_StrField", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x0fbedf5:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x0fbedf5 = (allow_blank, allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_length, min_length, read_only, required, root, style, trim_whitespace, validators, write_only))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x0fbedf5) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x0fbedf5:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x0fbedf5 = (allow_blank, allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_length, min_length, read_only, required, root, style, trim_whitespace, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = StrField.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0x0fbedf5:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x0fbedf5 = (allow_blank, allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_length, min_length, read_only, required, root, style, trim_whitespace, validators, write_only))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = StrField.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x0f, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(1, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x0fbedf5:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x0fbedf5 = (allow_blank, allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_length, min_length, read_only, required, root, style, trim_whitespace, validators, write_only))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x0fbedf5 = (allow_blank, allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_length, min_length, read_only, required, root, style, trim_whitespace, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = StrField.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_StrField__set_state(<StrField> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_9drf_turbo_6fields_StrField), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x0fbedf5 = (allow_blank, allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_length, min_length, read_only, required, root, style, trim_whitespace, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = StrField.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_StrField__set_state(<StrField> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = StrField.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_StrField__set_state(<StrField> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_StrField__set_state(StrField __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_9drf_turbo_6fields___pyx_unpickle_StrField__set_state(((struct __pyx_obj_9drf_turbo_6fields_StrField *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x0fbedf5 = (allow_blank, allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_length, min_length, read_only, required, root, style, trim_whitespace, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = StrField.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_StrField__set_state(<StrField> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_StrField__set_state(<StrField> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_StrField__set_state(StrField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_blank = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.max_length = __pyx_state[11]; __pyx_result.min_length = __pyx_state[12]; __pyx_result.read_only = __pyx_state[13]; __pyx_result.required = __pyx_state[14]; __pyx_result.root = __pyx_state[15]; __pyx_result.style = __pyx_state[16]; __pyx_result.trim_whitespace = __pyx_state[17]; __pyx_result.validators = __pyx_state[18]; __pyx_result.write_only = __pyx_state[19]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_StrField(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_StrField", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_StrField__set_state(<StrField> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_StrField__set_state(StrField __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.allow_blank = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.max_length = __pyx_state[11]; __pyx_result.min_length = __pyx_state[12]; __pyx_result.read_only = __pyx_state[13]; __pyx_result.required = __pyx_state[14]; __pyx_result.root = __pyx_state[15]; __pyx_result.style = __pyx_state[16]; __pyx_result.trim_whitespace = __pyx_state[17]; __pyx_result.validators = __pyx_state[18]; __pyx_result.write_only = __pyx_state[19]
 *     if len(__pyx_state) > 20 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_9drf_turbo_6fields___pyx_unpickle_StrField__set_state(struct __pyx_obj_9drf_turbo_6fields_StrField *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_StrField__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_StrField__set_state(StrField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_blank = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.max_length = __pyx_state[11]; __pyx_result.min_length = __pyx_state[12]; __pyx_result.read_only = __pyx_state[13]; __pyx_result.required = __pyx_state[14]; __pyx_result.root = __pyx_state[15]; __pyx_result.style = __pyx_state[16]; __pyx_result.trim_whitespace = __pyx_state[17]; __pyx_result.validators = __pyx_state[18]; __pyx_result.write_only = __pyx_state[19]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 20 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[20])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->allow_blank);
  __Pyx_DECREF(__pyx_v___pyx_result->allow_blank);
  __pyx_v___pyx_result->allow_blank = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.allow_null = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.attr);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.attr);
  __pyx_v___pyx_result->__pyx_base.attr = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.attrs);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.attrs);
  __pyx_v___pyx_result->__pyx_base.attrs = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.call = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.default_value);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.default_value);
  __pyx_v___pyx_result->__pyx_base.default_value = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.error_messages);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.error_messages);
  __pyx_v___pyx_result->__pyx_base.error_messages = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.field_name);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.field_name);
  __pyx_v___pyx_result->__pyx_base.field_name = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 8, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.help_text);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.help_text);
  __pyx_v___pyx_result->__pyx_base.help_text = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 9, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.initial);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.initial);
  __pyx_v___pyx_result->__pyx_base.initial = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 10, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.label);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.label);
  __pyx_v___pyx_result->__pyx_base.label = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 11, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->max_length);
  __Pyx_DECREF(__pyx_v___pyx_result->max_length);
  __pyx_v___pyx_result->max_length = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 12, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->min_length);
  __Pyx_DECREF(__pyx_v___pyx_result->min_length);
  __pyx_v___pyx_result->min_length = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 13, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.read_only = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 14, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.required = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 15, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.root);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.root);
  __pyx_v___pyx_result->__pyx_base.root = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 16, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.style);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.style);
  __pyx_v___pyx_result->__pyx_base.style = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 17, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->trim_whitespace);
  __Pyx_DECREF(__pyx_v___pyx_result->trim_whitespace);
  __pyx_v___pyx_result->trim_whitespace = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 18, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.validators);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.validators);
  __pyx_v___pyx_result->__pyx_base.validators = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 19, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.write_only = __pyx_t_2;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_StrField__set_state(StrField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_blank = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.max_length = __pyx_state[11]; __pyx_result.min_length = __pyx_state[12]; __pyx_result.read_only = __pyx_state[13]; __pyx_result.required = __pyx_state[14]; __pyx_result.root = __pyx_state[15]; __pyx_result.style = __pyx_state[16]; __pyx_result.trim_whitespace = __pyx_state[17]; __pyx_result.validators = __pyx_state[18]; __pyx_result.write_only = __pyx_state[19]
 *     if len(__pyx_state) > 20 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[20])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 13, __pyx_L1_error)
  }
  __pyx_t_3 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_4 = ((__pyx_t_3 > 20) != 0);
  if (__pyx_t_4) {
  } else {
    __pyx_t_2 = __pyx_t_4;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_4 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_5 = (__pyx_t_4 != 0);
  __pyx_t_2 = __pyx_t_5;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":14
 *     __pyx_result.allow_blank = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.max_length = __pyx_state[11]; __pyx_result.min_length = __pyx_state[12]; __pyx_result.read_only = __pyx_state[13]; __pyx_result.required = __pyx_state[14]; __pyx_result.root = __pyx_state[15]; __pyx_result.style = __pyx_state[16]; __pyx_result.trim_whitespace = __pyx_state[17]; __pyx_result.validators = __pyx_state[18]; __pyx_result.write_only = __pyx_state[19]
 *     if len(__pyx_state) > 20 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[20])             # <<<<<<<<<<<<<<
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_update); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 14, __pyx_L1_error)
    }
    __pyx_t_6 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 20, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_8 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
      }
    }
    __pyx_t_1 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_8, __pyx_t_6) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_6);
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_StrField__set_state(StrField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_blank = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.max_length = __pyx_state[11]; __pyx_result.min_length = __pyx_state[12]; __pyx_result.read_only = __pyx_state[13]; __pyx_result.required = __pyx_state[14]; __pyx_result.root = __pyx_state[15]; __pyx_result.style = __pyx_state[16]; __pyx_result.trim_whitespace = __pyx_state[17]; __pyx_result.validators = __pyx_state[18]; __pyx_result.write_only = __pyx_state[19]
 *     if len(__pyx_state) > 20 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[20])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_StrField__set_state(<StrField> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_StrField__set_state(StrField __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.allow_blank = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.max_length = __pyx_state[11]; __pyx_result.min_length = __pyx_state[12]; __pyx_result.read_only = __pyx_state[13]; __pyx_result.required = __pyx_state[14]; __pyx_result.root = __pyx_state[15]; __pyx_result.style = __pyx_state[16]; __pyx_result.trim_whitespace = __pyx_state[17]; __pyx_result.validators = __pyx_state[18]; __pyx_result.write_only = __pyx_state[19]
 *     if len(__pyx_state) > 20 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_StrField__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_EmailField(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_7__pyx_unpickle_EmailField(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_9drf_turbo_6fields_7__pyx_unpickle_EmailField = {"__pyx_unpickle_EmailField", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_7__pyx_unpickle_EmailField, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_9drf_turbo_6fields_7__pyx_unpickle_EmailField(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_EmailField (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_EmailField", 1, 3, 3, 1); __PYX_ERR(1, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_EmailField", 1, 3, 3, 2); __PYX_ERR(1, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_EmailField") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_EmailField", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_EmailField", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9drf_turbo_6fields_6__pyx_unpickle_EmailField(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_6__pyx_unpickle_EmailField(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_EmailField", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xe1e0ed9:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xe1e0ed9 = (allow_blank, allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_length, min_length, read_only, required, root, style, to_lower, trim_whitespace, validators, write_only))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0xe1e0ed9) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xe1e0ed9:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xe1e0ed9 = (allow_blank, allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_length, min_length, read_only, required, root, style, to_lower, trim_whitespace, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = EmailField.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0xe1e0ed9:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xe1e0ed9 = (allow_blank, allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_length, min_length, read_only, required, root, style, to_lower, trim_whitespace, validators, write_only))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = EmailField.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0xe1, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(1, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xe1e0ed9:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xe1e0ed9 = (allow_blank, allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_length, min_length, read_only, required, root, style, to_lower, trim_whitespace, validators, write_only))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xe1e0ed9 = (allow_blank, allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_length, min_length, read_only, required, root, style, to_lower, trim_whitespace, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = EmailField.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_EmailField__set_state(<EmailField> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_9drf_turbo_6fields_EmailField), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xe1e0ed9 = (allow_blank, allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_length, min_length, read_only, required, root, style, to_lower, trim_whitespace, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = EmailField.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_EmailField__set_state(<EmailField> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = EmailField.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_EmailField__set_state(<EmailField> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_EmailField__set_state(EmailField __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_9drf_turbo_6fields___pyx_unpickle_EmailField__set_state(((struct __pyx_obj_9drf_turbo_6fields_EmailField *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xe1e0ed9 = (allow_blank, allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_length, min_length, read_only, required, root, style, to_lower, trim_whitespace, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = EmailField.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_EmailField__set_state(<EmailField> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_EmailField__set_state(<EmailField> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_EmailField__set_state(EmailField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_blank = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.max_length = __pyx_state[11]; __pyx_result.min_length = __pyx_state[12]; __pyx_result.read_only = __pyx_state[13]; __pyx_result.required = __pyx_state[14]; __pyx_result.root = __pyx_state[15]; __pyx_result.style = __pyx_state[16]; __pyx_result.to_lower = __pyx_state[17]; __pyx_result.trim_whitespace = __pyx_state[18]; __pyx_result.validators = __pyx_state[19]; __pyx_result.write_only = __pyx_state[20]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_EmailField(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_EmailField", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_EmailField__set_state(<EmailField> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_EmailField__set_state(EmailField __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.allow_blank = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.max_length = __pyx_state[11]; __pyx_result.min_length = __pyx_state[12]; __pyx_result.read_only = __pyx_state[13]; __pyx_result.required = __pyx_state[14]; __pyx_result.root = __pyx_state[15]; __pyx_result.style = __pyx_state[16]; __pyx_result.to_lower = __pyx_state[17]; __pyx_result.trim_whitespace = __pyx_state[18]; __pyx_result.validators = __pyx_state[19]; __pyx_result.write_only = __pyx_state[20]
 *     if len(__pyx_state) > 21 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_9drf_turbo_6fields___pyx_unpickle_EmailField__set_state(struct __pyx_obj_9drf_turbo_6fields_EmailField *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_EmailField__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_EmailField__set_state(EmailField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_blank = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.max_length = __pyx_state[11]; __pyx_result.min_length = __pyx_state[12]; __pyx_result.read_only = __pyx_state[13]; __pyx_result.required = __pyx_state[14]; __pyx_result.root = __pyx_state[15]; __pyx_result.style = __pyx_state[16]; __pyx_result.to_lower = __pyx_state[17]; __pyx_result.trim_whitespace = __pyx_state[18]; __pyx_result.validators = __pyx_state[19]; __pyx_result.write_only = __pyx_state[20]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 21 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[21])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.allow_blank);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.allow_blank);
  __pyx_v___pyx_result->__pyx_base.allow_blank = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.allow_null = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.attr);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.attr);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.attr = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.attrs);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.attrs);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.attrs = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.call = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.default_value);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.default_value);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.default_value = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.error_messages);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.error_messages);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.error_messages = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.field_name);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.field_name);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.field_name = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 8, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.help_text);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.help_text);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.help_text = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 9, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.initial);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.initial);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.initial = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 10, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.label);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.label);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.label = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 11, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.max_length);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.max_length);
  __pyx_v___pyx_result->__pyx_base.max_length = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 12, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.min_length);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.min_length);
  __pyx_v___pyx_result->__pyx_base.min_length = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 13, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.read_only = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 14, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.required = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 15, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.root);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.root);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.root = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 16, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.style);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.style);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.style = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 17, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->to_lower = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 18, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.trim_whitespace);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.trim_whitespace);
  __pyx_v___pyx_result->__pyx_base.trim_whitespace = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 19, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.validators);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.validators);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.validators = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 20, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.write_only = __pyx_t_2;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_EmailField__set_state(EmailField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_blank = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.max_length = __pyx_state[11]; __pyx_result.min_length = __pyx_state[12]; __pyx_result.read_only = __pyx_state[13]; __pyx_result.required = __pyx_state[14]; __pyx_result.root = __pyx_state[15]; __pyx_result.style = __pyx_state[16]; __pyx_result.to_lower = __pyx_state[17]; __pyx_result.trim_whitespace = __pyx_state[18]; __pyx_result.validators = __pyx_state[19]; __pyx_result.write_only = __pyx_state[20]
 *     if len(__pyx_state) > 21 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[21])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 13, __pyx_L1_error)
  }
  __pyx_t_3 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_4 = ((__pyx_t_3 > 21) != 0);
  if (__pyx_t_4) {
  } else {
    __pyx_t_2 = __pyx_t_4;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_4 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_5 = (__pyx_t_4 != 0);
  __pyx_t_2 = __pyx_t_5;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":14
 *     __pyx_result.allow_blank = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.max_length = __pyx_state[11]; __pyx_result.min_length = __pyx_state[12]; __pyx_result.read_only = __pyx_state[13]; __pyx_result.required = __pyx_state[14]; __pyx_result.root = __pyx_state[15]; __pyx_result.style = __pyx_state[16]; __pyx_result.to_lower = __pyx_state[17]; __pyx_result.trim_whitespace = __pyx_state[18]; __pyx_result.validators = __pyx_state[19]; __pyx_result.write_only = __pyx_state[20]
 *     if len(__pyx_state) > 21 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[21])             # <<<<<<<<<<<<<<
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_update); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 14, __pyx_L1_error)
    }
    __pyx_t_6 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 21, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_8 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
      }
    }
    __pyx_t_1 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_8, __pyx_t_6) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_6);
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_EmailField__set_state(EmailField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_blank = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.max_length = __pyx_state[11]; __pyx_result.min_length = __pyx_state[12]; __pyx_result.read_only = __pyx_state[13]; __pyx_result.required = __pyx_state[14]; __pyx_result.root = __pyx_state[15]; __pyx_result.style = __pyx_state[16]; __pyx_result.to_lower = __pyx_state[17]; __pyx_result.trim_whitespace = __pyx_state[18]; __pyx_result.validators = __pyx_state[19]; __pyx_result.write_only = __pyx_state[20]
 *     if len(__pyx_state) > 21 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[21])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_EmailField__set_state(<EmailField> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_EmailField__set_state(EmailField __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.allow_blank = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.max_length = __pyx_state[11]; __pyx_result.min_length = __pyx_state[12]; __pyx_result.read_only = __pyx_state[13]; __pyx_result.required = __pyx_state[14]; __pyx_result.root = __pyx_state[15]; __pyx_result.style = __pyx_state[16]; __pyx_result.to_lower = __pyx_state[17]; __pyx_result.trim_whitespace = __pyx_state[18]; __pyx_result.validators = __pyx_state[19]; __pyx_result.write_only = __pyx_state[20]
 *     if len(__pyx_state) > 21 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_EmailField__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_URLField(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_9__pyx_unpickle_URLField(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_9drf_turbo_6fields_9__pyx_unpickle_URLField = {"__pyx_unpickle_URLField", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_9__pyx_unpickle_URLField, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_9drf_turbo_6fields_9__pyx_unpickle_URLField(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_URLField (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_URLField", 1, 3, 3, 1); __PYX_ERR(1, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_URLField", 1, 3, 3, 2); __PYX_ERR(1, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_URLField") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_URLField", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_URLField", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9drf_turbo_6fields_8__pyx_unpickle_URLField(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_8__pyx_unpickle_URLField(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_URLField", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x0fbedf5:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x0fbedf5 = (allow_blank, allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_length, min_length, read_only, required, root, style, trim_whitespace, validators, write_only))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x0fbedf5) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x0fbedf5:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x0fbedf5 = (allow_blank, allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_length, min_length, read_only, required, root, style, trim_whitespace, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = URLField.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0x0fbedf5:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x0fbedf5 = (allow_blank, allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_length, min_length, read_only, required, root, style, trim_whitespace, validators, write_only))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = URLField.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x0f, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(1, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x0fbedf5:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x0fbedf5 = (allow_blank, allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_length, min_length, read_only, required, root, style, trim_whitespace, validators, write_only))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x0fbedf5 = (allow_blank, allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_length, min_length, read_only, required, root, style, trim_whitespace, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = URLField.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_URLField__set_state(<URLField> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_9drf_turbo_6fields_URLField), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x0fbedf5 = (allow_blank, allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_length, min_length, read_only, required, root, style, trim_whitespace, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = URLField.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_URLField__set_state(<URLField> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = URLField.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_URLField__set_state(<URLField> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_URLField__set_state(URLField __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_9drf_turbo_6fields___pyx_unpickle_URLField__set_state(((struct __pyx_obj_9drf_turbo_6fields_URLField *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x0fbedf5 = (allow_blank, allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_length, min_length, read_only, required, root, style, trim_whitespace, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = URLField.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_URLField__set_state(<URLField> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_URLField__set_state(<URLField> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_URLField__set_state(URLField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_blank = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.max_length = __pyx_state[11]; __pyx_result.min_length = __pyx_state[12]; __pyx_result.read_only = __pyx_state[13]; __pyx_result.required = __pyx_state[14]; __pyx_result.root = __pyx_state[15]; __pyx_result.style = __pyx_state[16]; __pyx_result.trim_whitespace = __pyx_state[17]; __pyx_result.validators = __pyx_state[18]; __pyx_result.write_only = __pyx_state[19]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_URLField(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_URLField", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_URLField__set_state(<URLField> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_URLField__set_state(URLField __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.allow_blank = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.max_length = __pyx_state[11]; __pyx_result.min_length = __pyx_state[12]; __pyx_result.read_only = __pyx_state[13]; __pyx_result.required = __pyx_state[14]; __pyx_result.root = __pyx_state[15]; __pyx_result.style = __pyx_state[16]; __pyx_result.trim_whitespace = __pyx_state[17]; __pyx_result.validators = __pyx_state[18]; __pyx_result.write_only = __pyx_state[19]
 *     if len(__pyx_state) > 20 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_9drf_turbo_6fields___pyx_unpickle_URLField__set_state(struct __pyx_obj_9drf_turbo_6fields_URLField *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_URLField__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_URLField__set_state(URLField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_blank = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.max_length = __pyx_state[11]; __pyx_result.min_length = __pyx_state[12]; __pyx_result.read_only = __pyx_state[13]; __pyx_result.required = __pyx_state[14]; __pyx_result.root = __pyx_state[15]; __pyx_result.style = __pyx_state[16]; __pyx_result.trim_whitespace = __pyx_state[17]; __pyx_result.validators = __pyx_state[18]; __pyx_result.write_only = __pyx_state[19]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 20 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[20])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.allow_blank);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.allow_blank);
  __pyx_v___pyx_result->__pyx_base.allow_blank = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.allow_null = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.attr);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.attr);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.attr = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.attrs);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.attrs);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.attrs = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.call = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.default_value);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.default_value);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.default_value = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.error_messages);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.error_messages);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.error_messages = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.field_name);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.field_name);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.field_name = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 8, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.help_text);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.help_text);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.help_text = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 9, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.initial);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.initial);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.initial = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 10, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.label);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.label);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.label = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 11, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.max_length);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.max_length);
  __pyx_v___pyx_result->__pyx_base.max_length = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 12, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.min_length);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.min_length);
  __pyx_v___pyx_result->__pyx_base.min_length = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 13, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.read_only = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 14, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.required = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 15, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.root);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.root);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.root = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 16, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.style);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.style);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.style = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 17, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.trim_whitespace);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.trim_whitespace);
  __pyx_v___pyx_result->__pyx_base.trim_whitespace = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 18, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.validators);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.validators);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.validators = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 19, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.write_only = __pyx_t_2;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_URLField__set_state(URLField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_blank = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.max_length = __pyx_state[11]; __pyx_result.min_length = __pyx_state[12]; __pyx_result.read_only = __pyx_state[13]; __pyx_result.required = __pyx_state[14]; __pyx_result.root = __pyx_state[15]; __pyx_result.style = __pyx_state[16]; __pyx_result.trim_whitespace = __pyx_state[17]; __pyx_result.validators = __pyx_state[18]; __pyx_result.write_only = __pyx_state[19]
 *     if len(__pyx_state) > 20 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[20])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 13, __pyx_L1_error)
  }
  __pyx_t_3 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_4 = ((__pyx_t_3 > 20) != 0);
  if (__pyx_t_4) {
  } else {
    __pyx_t_2 = __pyx_t_4;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_4 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_5 = (__pyx_t_4 != 0);
  __pyx_t_2 = __pyx_t_5;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":14
 *     __pyx_result.allow_blank = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.max_length = __pyx_state[11]; __pyx_result.min_length = __pyx_state[12]; __pyx_result.read_only = __pyx_state[13]; __pyx_result.required = __pyx_state[14]; __pyx_result.root = __pyx_state[15]; __pyx_result.style = __pyx_state[16]; __pyx_result.trim_whitespace = __pyx_state[17]; __pyx_result.validators = __pyx_state[18]; __pyx_result.write_only = __pyx_state[19]
 *     if len(__pyx_state) > 20 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[20])             # <<<<<<<<<<<<<<
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_update); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 14, __pyx_L1_error)
    }
    __pyx_t_6 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 20, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_8 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
      }
    }
    __pyx_t_1 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_8, __pyx_t_6) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_6);
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_URLField__set_state(URLField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_blank = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.max_length = __pyx_state[11]; __pyx_result.min_length = __pyx_state[12]; __pyx_result.read_only = __pyx_state[13]; __pyx_result.required = __pyx_state[14]; __pyx_result.root = __pyx_state[15]; __pyx_result.style = __pyx_state[16]; __pyx_result.trim_whitespace = __pyx_state[17]; __pyx_result.validators = __pyx_state[18]; __pyx_result.write_only = __pyx_state[19]
 *     if len(__pyx_state) > 20 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[20])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_URLField__set_state(<URLField> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_URLField__set_state(URLField __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.allow_blank = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.max_length = __pyx_state[11]; __pyx_result.min_length = __pyx_state[12]; __pyx_result.read_only = __pyx_state[13]; __pyx_result.required = __pyx_state[14]; __pyx_result.root = __pyx_state[15]; __pyx_result.style = __pyx_state[16]; __pyx_result.trim_whitespace = __pyx_state[17]; __pyx_result.validators = __pyx_state[18]; __pyx_result.write_only = __pyx_state[19]
 *     if len(__pyx_state) > 20 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_URLField__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_RegexField(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_11__pyx_unpickle_RegexField(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_9drf_turbo_6fields_11__pyx_unpickle_RegexField = {"__pyx_unpickle_RegexField", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_11__pyx_unpickle_RegexField, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_9drf_turbo_6fields_11__pyx_unpickle_RegexField(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_RegexField (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_RegexField", 1, 3, 3, 1); __PYX_ERR(1, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_RegexField", 1, 3, 3, 2); __PYX_ERR(1, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_RegexField") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_RegexField", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_RegexField", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9drf_turbo_6fields_10__pyx_unpickle_RegexField(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_10__pyx_unpickle_RegexField(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_RegexField", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x380e494:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x380e494 = (allow_blank, allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_length, min_length, read_only, regex, required, root, style, trim_whitespace, validators, write_only))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x380e494) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x380e494:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x380e494 = (allow_blank, allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_length, min_length, read_only, regex, required, root, style, trim_whitespace, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = RegexField.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0x380e494:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x380e494 = (allow_blank, allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_length, min_length, read_only, regex, required, root, style, trim_whitespace, validators, write_only))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = RegexField.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x38, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(1, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x380e494:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x380e494 = (allow_blank, allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_length, min_length, read_only, regex, required, root, style, trim_whitespace, validators, write_only))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x380e494 = (allow_blank, allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_length, min_length, read_only, regex, required, root, style, trim_whitespace, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = RegexField.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_RegexField__set_state(<RegexField> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_9drf_turbo_6fields_RegexField), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x380e494 = (allow_blank, allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_length, min_length, read_only, regex, required, root, style, trim_whitespace, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = RegexField.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_RegexField__set_state(<RegexField> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = RegexField.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_RegexField__set_state(<RegexField> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_RegexField__set_state(RegexField __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_9drf_turbo_6fields___pyx_unpickle_RegexField__set_state(((struct __pyx_obj_9drf_turbo_6fields_RegexField *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x380e494 = (allow_blank, allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_length, min_length, read_only, regex, required, root, style, trim_whitespace, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = RegexField.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_RegexField__set_state(<RegexField> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_RegexField__set_state(<RegexField> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_RegexField__set_state(RegexField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_blank = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.max_length = __pyx_state[11]; __pyx_result.min_length = __pyx_state[12]; __pyx_result.read_only = __pyx_state[13]; __pyx_result.regex = __pyx_state[14]; __pyx_result.required = __pyx_state[15]; __pyx_result.root = __pyx_state[16]; __pyx_result.style = __pyx_state[17]; __pyx_result.trim_whitespace = __pyx_state[18]; __pyx_result.validators = __pyx_state[19]; __pyx_result.write_only = __pyx_state[20]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_RegexField(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_RegexField", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_RegexField__set_state(<RegexField> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_RegexField__set_state(RegexField __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.allow_blank = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.max_length = __pyx_state[11]; __pyx_result.min_length = __pyx_state[12]; __pyx_result.read_only = __pyx_state[13]; __pyx_result.regex = __pyx_state[14]; __pyx_result.required = __pyx_state[15]; __pyx_result.root = __pyx_state[16]; __pyx_result.style = __pyx_state[17]; __pyx_result.trim_whitespace = __pyx_state[18]; __pyx_result.validators = __pyx_state[19]; __pyx_result.write_only = __pyx_state[20]
 *     if len(__pyx_state) > 21 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_9drf_turbo_6fields___pyx_unpickle_RegexField__set_state(struct __pyx_obj_9drf_turbo_6fields_RegexField *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_RegexField__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_RegexField__set_state(RegexField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_blank = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.max_length = __pyx_state[11]; __pyx_result.min_length = __pyx_state[12]; __pyx_result.read_only = __pyx_state[13]; __pyx_result.regex = __pyx_state[14]; __pyx_result.required = __pyx_state[15]; __pyx_result.root = __pyx_state[16]; __pyx_result.style = __pyx_state[17]; __pyx_result.trim_whitespace = __pyx_state[18]; __pyx_result.validators = __pyx_state[19]; __pyx_result.write_only = __pyx_state[20]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 21 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[21])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.allow_blank);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.allow_blank);
  __pyx_v___pyx_result->__pyx_base.allow_blank = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.allow_null = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.attr);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.attr);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.attr = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.attrs);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.attrs);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.attrs = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.call = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.default_value);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.default_value);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.default_value = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.error_messages);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.error_messages);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.error_messages = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.field_name);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.field_name);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.field_name = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 8, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.help_text);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.help_text);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.help_text = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 9, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.initial);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.initial);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.initial = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 10, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.label);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.label);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.label = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 11, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.max_length);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.max_length);
  __pyx_v___pyx_result->__pyx_base.max_length = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 12, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.min_length);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.min_length);
  __pyx_v___pyx_result->__pyx_base.min_length = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 13, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.read_only = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 14, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->regex);
  __Pyx_DECREF(__pyx_v___pyx_result->regex);
  __pyx_v___pyx_result->regex = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 15, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.required = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 16, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.root);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.root);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.root = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 17, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.style);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.style);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.style = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 18, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.trim_whitespace);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.trim_whitespace);
  __pyx_v___pyx_result->__pyx_base.trim_whitespace = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 19, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.validators);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.validators);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.validators = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 20, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.write_only = __pyx_t_2;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_RegexField__set_state(RegexField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_blank = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.max_length = __pyx_state[11]; __pyx_result.min_length = __pyx_state[12]; __pyx_result.read_only = __pyx_state[13]; __pyx_result.regex = __pyx_state[14]; __pyx_result.required = __pyx_state[15]; __pyx_result.root = __pyx_state[16]; __pyx_result.style = __pyx_state[17]; __pyx_result.trim_whitespace = __pyx_state[18]; __pyx_result.validators = __pyx_state[19]; __pyx_result.write_only = __pyx_state[20]
 *     if len(__pyx_state) > 21 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[21])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 13, __pyx_L1_error)
  }
  __pyx_t_3 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_4 = ((__pyx_t_3 > 21) != 0);
  if (__pyx_t_4) {
  } else {
    __pyx_t_2 = __pyx_t_4;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_4 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_5 = (__pyx_t_4 != 0);
  __pyx_t_2 = __pyx_t_5;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":14
 *     __pyx_result.allow_blank = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.max_length = __pyx_state[11]; __pyx_result.min_length = __pyx_state[12]; __pyx_result.read_only = __pyx_state[13]; __pyx_result.regex = __pyx_state[14]; __pyx_result.required = __pyx_state[15]; __pyx_result.root = __pyx_state[16]; __pyx_result.style = __pyx_state[17]; __pyx_result.trim_whitespace = __pyx_state[18]; __pyx_result.validators = __pyx_state[19]; __pyx_result.write_only = __pyx_state[20]
 *     if len(__pyx_state) > 21 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[21])             # <<<<<<<<<<<<<<
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_update); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 14, __pyx_L1_error)
    }
    __pyx_t_6 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 21, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_8 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
      }
    }
    __pyx_t_1 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_8, __pyx_t_6) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_6);
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_RegexField__set_state(RegexField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_blank = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.max_length = __pyx_state[11]; __pyx_result.min_length = __pyx_state[12]; __pyx_result.read_only = __pyx_state[13]; __pyx_result.regex = __pyx_state[14]; __pyx_result.required = __pyx_state[15]; __pyx_result.root = __pyx_state[16]; __pyx_result.style = __pyx_state[17]; __pyx_result.trim_whitespace = __pyx_state[18]; __pyx_result.validators = __pyx_state[19]; __pyx_result.write_only = __pyx_state[20]
 *     if len(__pyx_state) > 21 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[21])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_RegexField__set_state(<RegexField> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_RegexField__set_state(RegexField __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.allow_blank = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.max_length = __pyx_state[11]; __pyx_result.min_length = __pyx_state[12]; __pyx_result.read_only = __pyx_state[13]; __pyx_result.regex = __pyx_state[14]; __pyx_result.required = __pyx_state[15]; __pyx_result.root = __pyx_state[16]; __pyx_result.style = __pyx_state[17]; __pyx_result.trim_whitespace = __pyx_state[18]; __pyx_result.validators = __pyx_state[19]; __pyx_result.write_only = __pyx_state[20]
 *     if len(__pyx_state) > 21 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_RegexField__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_IPField(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_13__pyx_unpickle_IPField(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_9drf_turbo_6fields_13__pyx_unpickle_IPField = {"__pyx_unpickle_IPField", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_13__pyx_unpickle_IPField, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_9drf_turbo_6fields_13__pyx_unpickle_IPField(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_IPField (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_IPField", 1, 3, 3, 1); __PYX_ERR(1, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_IPField", 1, 3, 3, 2); __PYX_ERR(1, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_IPField") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_IPField", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_IPField", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9drf_turbo_6fields_12__pyx_unpickle_IPField(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_12__pyx_unpickle_IPField(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_IPField", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x0fbedf5:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x0fbedf5 = (allow_blank, allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_length, min_length, read_only, required, root, style, trim_whitespace, validators, write_only))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x0fbedf5) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x0fbedf5:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x0fbedf5 = (allow_blank, allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_length, min_length, read_only, required, root, style, trim_whitespace, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = IPField.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0x0fbedf5:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x0fbedf5 = (allow_blank, allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_length, min_length, read_only, required, root, style, trim_whitespace, validators, write_only))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = IPField.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x0f, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(1, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x0fbedf5:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x0fbedf5 = (allow_blank, allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_length, min_length, read_only, required, root, style, trim_whitespace, validators, write_only))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x0fbedf5 = (allow_blank, allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_length, min_length, read_only, required, root, style, trim_whitespace, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = IPField.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_IPField__set_state(<IPField> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_9drf_turbo_6fields_IPField), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x0fbedf5 = (allow_blank, allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_length, min_length, read_only, required, root, style, trim_whitespace, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = IPField.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_IPField__set_state(<IPField> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = IPField.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_IPField__set_state(<IPField> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_IPField__set_state(IPField __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_9drf_turbo_6fields___pyx_unpickle_IPField__set_state(((struct __pyx_obj_9drf_turbo_6fields_IPField *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x0fbedf5 = (allow_blank, allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_length, min_length, read_only, required, root, style, trim_whitespace, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = IPField.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_IPField__set_state(<IPField> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_IPField__set_state(<IPField> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_IPField__set_state(IPField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_blank = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.max_length = __pyx_state[11]; __pyx_result.min_length = __pyx_state[12]; __pyx_result.read_only = __pyx_state[13]; __pyx_result.required = __pyx_state[14]; __pyx_result.root = __pyx_state[15]; __pyx_result.style = __pyx_state[16]; __pyx_result.trim_whitespace = __pyx_state[17]; __pyx_result.validators = __pyx_state[18]; __pyx_result.write_only = __pyx_state[19]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_IPField(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_IPField", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_IPField__set_state(<IPField> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_IPField__set_state(IPField __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.allow_blank = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.max_length = __pyx_state[11]; __pyx_result.min_length = __pyx_state[12]; __pyx_result.read_only = __pyx_state[13]; __pyx_result.required = __pyx_state[14]; __pyx_result.root = __pyx_state[15]; __pyx_result.style = __pyx_state[16]; __pyx_result.trim_whitespace = __pyx_state[17]; __pyx_result.validators = __pyx_state[18]; __pyx_result.write_only = __pyx_state[19]
 *     if len(__pyx_state) > 20 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_9drf_turbo_6fields___pyx_unpickle_IPField__set_state(struct __pyx_obj_9drf_turbo_6fields_IPField *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_IPField__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_IPField__set_state(IPField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_blank = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.max_length = __pyx_state[11]; __pyx_result.min_length = __pyx_state[12]; __pyx_result.read_only = __pyx_state[13]; __pyx_result.required = __pyx_state[14]; __pyx_result.root = __pyx_state[15]; __pyx_result.style = __pyx_state[16]; __pyx_result.trim_whitespace = __pyx_state[17]; __pyx_result.validators = __pyx_state[18]; __pyx_result.write_only = __pyx_state[19]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 20 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[20])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.allow_blank);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.allow_blank);
  __pyx_v___pyx_result->__pyx_base.allow_blank = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.allow_null = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.attr);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.attr);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.attr = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.attrs);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.attrs);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.attrs = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.call = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.default_value);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.default_value);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.default_value = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.error_messages);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.error_messages);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.error_messages = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.field_name);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.field_name);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.field_name = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 8, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.help_text);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.help_text);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.help_text = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 9, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.initial);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.initial);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.initial = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 10, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.label);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.label);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.label = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 11, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.max_length);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.max_length);
  __pyx_v___pyx_result->__pyx_base.max_length = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 12, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.min_length);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.min_length);
  __pyx_v___pyx_result->__pyx_base.min_length = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 13, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.read_only = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 14, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.required = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 15, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.root);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.root);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.root = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 16, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.style);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.style);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.style = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 17, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.trim_whitespace);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.trim_whitespace);
  __pyx_v___pyx_result->__pyx_base.trim_whitespace = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 18, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.validators);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.validators);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.validators = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 19, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.write_only = __pyx_t_2;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_IPField__set_state(IPField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_blank = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.max_length = __pyx_state[11]; __pyx_result.min_length = __pyx_state[12]; __pyx_result.read_only = __pyx_state[13]; __pyx_result.required = __pyx_state[14]; __pyx_result.root = __pyx_state[15]; __pyx_result.style = __pyx_state[16]; __pyx_result.trim_whitespace = __pyx_state[17]; __pyx_result.validators = __pyx_state[18]; __pyx_result.write_only = __pyx_state[19]
 *     if len(__pyx_state) > 20 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[20])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 13, __pyx_L1_error)
  }
  __pyx_t_3 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_4 = ((__pyx_t_3 > 20) != 0);
  if (__pyx_t_4) {
  } else {
    __pyx_t_2 = __pyx_t_4;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_4 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_5 = (__pyx_t_4 != 0);
  __pyx_t_2 = __pyx_t_5;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":14
 *     __pyx_result.allow_blank = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.max_length = __pyx_state[11]; __pyx_result.min_length = __pyx_state[12]; __pyx_result.read_only = __pyx_state[13]; __pyx_result.required = __pyx_state[14]; __pyx_result.root = __pyx_state[15]; __pyx_result.style = __pyx_state[16]; __pyx_result.trim_whitespace = __pyx_state[17]; __pyx_result.validators = __pyx_state[18]; __pyx_result.write_only = __pyx_state[19]
 *     if len(__pyx_state) > 20 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[20])             # <<<<<<<<<<<<<<
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_update); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 14, __pyx_L1_error)
    }
    __pyx_t_6 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 20, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_8 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
      }
    }
    __pyx_t_1 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_8, __pyx_t_6) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_6);
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_IPField__set_state(IPField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_blank = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.max_length = __pyx_state[11]; __pyx_result.min_length = __pyx_state[12]; __pyx_result.read_only = __pyx_state[13]; __pyx_result.required = __pyx_state[14]; __pyx_result.root = __pyx_state[15]; __pyx_result.style = __pyx_state[16]; __pyx_result.trim_whitespace = __pyx_state[17]; __pyx_result.validators = __pyx_state[18]; __pyx_result.write_only = __pyx_state[19]
 *     if len(__pyx_state) > 20 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[20])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_IPField__set_state(<IPField> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_IPField__set_state(IPField __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.allow_blank = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.max_length = __pyx_state[11]; __pyx_result.min_length = __pyx_state[12]; __pyx_result.read_only = __pyx_state[13]; __pyx_result.required = __pyx_state[14]; __pyx_result.root = __pyx_state[15]; __pyx_result.style = __pyx_state[16]; __pyx_result.trim_whitespace = __pyx_state[17]; __pyx_result.validators = __pyx_state[18]; __pyx_result.write_only = __pyx_state[19]
 *     if len(__pyx_state) > 20 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_IPField__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_PasswordField(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_15__pyx_unpickle_PasswordField(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_9drf_turbo_6fields_15__pyx_unpickle_PasswordField = {"__pyx_unpickle_PasswordField", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_15__pyx_unpickle_PasswordField, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_9drf_turbo_6fields_15__pyx_unpickle_PasswordField(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_PasswordField (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_PasswordField", 1, 3, 3, 1); __PYX_ERR(1, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_PasswordField", 1, 3, 3, 2); __PYX_ERR(1, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_PasswordField") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_PasswordField", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_PasswordField", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9drf_turbo_6fields_14__pyx_unpickle_PasswordField(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_14__pyx_unpickle_PasswordField(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_PasswordField", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x0fbedf5:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x0fbedf5 = (allow_blank, allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_length, min_length, read_only, required, root, style, trim_whitespace, validators, write_only))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x0fbedf5) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x0fbedf5:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x0fbedf5 = (allow_blank, allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_length, min_length, read_only, required, root, style, trim_whitespace, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = PasswordField.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0x0fbedf5:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x0fbedf5 = (allow_blank, allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_length, min_length, read_only, required, root, style, trim_whitespace, validators, write_only))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = PasswordField.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x0f, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(1, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x0fbedf5:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x0fbedf5 = (allow_blank, allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_length, min_length, read_only, required, root, style, trim_whitespace, validators, write_only))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x0fbedf5 = (allow_blank, allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_length, min_length, read_only, required, root, style, trim_whitespace, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = PasswordField.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_PasswordField__set_state(<PasswordField> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_9drf_turbo_6fields_PasswordField), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x0fbedf5 = (allow_blank, allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_length, min_length, read_only, required, root, style, trim_whitespace, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = PasswordField.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_PasswordField__set_state(<PasswordField> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = PasswordField.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_PasswordField__set_state(<PasswordField> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_PasswordField__set_state(PasswordField __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_9drf_turbo_6fields___pyx_unpickle_PasswordField__set_state(((struct __pyx_obj_9drf_turbo_6fields_PasswordField *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x0fbedf5 = (allow_blank, allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_length, min_length, read_only, required, root, style, trim_whitespace, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = PasswordField.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_PasswordField__set_state(<PasswordField> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_PasswordField__set_state(<PasswordField> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_PasswordField__set_state(PasswordField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_blank = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.max_length = __pyx_state[11]; __pyx_result.min_length = __pyx_state[12]; __pyx_result.read_only = __pyx_state[13]; __pyx_result.required = __pyx_state[14]; __pyx_result.root = __pyx_state[15]; __pyx_result.style = __pyx_state[16]; __pyx_result.trim_whitespace = __pyx_state[17]; __pyx_result.validators = __pyx_state[18]; __pyx_result.write_only = __pyx_state[19]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_PasswordField(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_PasswordField", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_PasswordField__set_state(<PasswordField> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_PasswordField__set_state(PasswordField __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.allow_blank = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.max_length = __pyx_state[11]; __pyx_result.min_length = __pyx_state[12]; __pyx_result.read_only = __pyx_state[13]; __pyx_result.required = __pyx_state[14]; __pyx_result.root = __pyx_state[15]; __pyx_result.style = __pyx_state[16]; __pyx_result.trim_whitespace = __pyx_state[17]; __pyx_result.validators = __pyx_state[18]; __pyx_result.write_only = __pyx_state[19]
 *     if len(__pyx_state) > 20 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_9drf_turbo_6fields___pyx_unpickle_PasswordField__set_state(struct __pyx_obj_9drf_turbo_6fields_PasswordField *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_PasswordField__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_PasswordField__set_state(PasswordField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_blank = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.max_length = __pyx_state[11]; __pyx_result.min_length = __pyx_state[12]; __pyx_result.read_only = __pyx_state[13]; __pyx_result.required = __pyx_state[14]; __pyx_result.root = __pyx_state[15]; __pyx_result.style = __pyx_state[16]; __pyx_result.trim_whitespace = __pyx_state[17]; __pyx_result.validators = __pyx_state[18]; __pyx_result.write_only = __pyx_state[19]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 20 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[20])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.allow_blank);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.allow_blank);
  __pyx_v___pyx_result->__pyx_base.allow_blank = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.allow_null = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.attr);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.attr);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.attr = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.attrs);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.attrs);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.attrs = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.call = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.default_value);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.default_value);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.default_value = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.error_messages);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.error_messages);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.error_messages = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.field_name);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.field_name);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.field_name = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 8, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.help_text);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.help_text);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.help_text = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 9, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.initial);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.initial);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.initial = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 10, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.label);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.label);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.label = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 11, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.max_length);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.max_length);
  __pyx_v___pyx_result->__pyx_base.max_length = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 12, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.min_length);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.min_length);
  __pyx_v___pyx_result->__pyx_base.min_length = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 13, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.read_only = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 14, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.required = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 15, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.root);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.root);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.root = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 16, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.style);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.style);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.style = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 17, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.trim_whitespace);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.trim_whitespace);
  __pyx_v___pyx_result->__pyx_base.trim_whitespace = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 18, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.validators);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.validators);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.validators = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 19, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.write_only = __pyx_t_2;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_PasswordField__set_state(PasswordField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_blank = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.max_length = __pyx_state[11]; __pyx_result.min_length = __pyx_state[12]; __pyx_result.read_only = __pyx_state[13]; __pyx_result.required = __pyx_state[14]; __pyx_result.root = __pyx_state[15]; __pyx_result.style = __pyx_state[16]; __pyx_result.trim_whitespace = __pyx_state[17]; __pyx_result.validators = __pyx_state[18]; __pyx_result.write_only = __pyx_state[19]
 *     if len(__pyx_state) > 20 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[20])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 13, __pyx_L1_error)
  }
  __pyx_t_3 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_4 = ((__pyx_t_3 > 20) != 0);
  if (__pyx_t_4) {
  } else {
    __pyx_t_2 = __pyx_t_4;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_4 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_5 = (__pyx_t_4 != 0);
  __pyx_t_2 = __pyx_t_5;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":14
 *     __pyx_result.allow_blank = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.max_length = __pyx_state[11]; __pyx_result.min_length = __pyx_state[12]; __pyx_result.read_only = __pyx_state[13]; __pyx_result.required = __pyx_state[14]; __pyx_result.root = __pyx_state[15]; __pyx_result.style = __pyx_state[16]; __pyx_result.trim_whitespace = __pyx_state[17]; __pyx_result.validators = __pyx_state[18]; __pyx_result.write_only = __pyx_state[19]
 *     if len(__pyx_state) > 20 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[20])             # <<<<<<<<<<<<<<
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_update); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 14, __pyx_L1_error)
    }
    __pyx_t_6 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 20, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_8 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
      }
    }
    __pyx_t_1 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_8, __pyx_t_6) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_6);
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_PasswordField__set_state(PasswordField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_blank = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.max_length = __pyx_state[11]; __pyx_result.min_length = __pyx_state[12]; __pyx_result.read_only = __pyx_state[13]; __pyx_result.required = __pyx_state[14]; __pyx_result.root = __pyx_state[15]; __pyx_result.style = __pyx_state[16]; __pyx_result.trim_whitespace = __pyx_state[17]; __pyx_result.validators = __pyx_state[18]; __pyx_result.write_only = __pyx_state[19]
 *     if len(__pyx_state) > 20 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[20])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_PasswordField__set_state(<PasswordField> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_PasswordField__set_state(PasswordField __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.allow_blank = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.max_length = __pyx_state[11]; __pyx_result.min_length = __pyx_state[12]; __pyx_result.read_only = __pyx_state[13]; __pyx_result.required = __pyx_state[14]; __pyx_result.root = __pyx_state[15]; __pyx_result.style = __pyx_state[16]; __pyx_result.trim_whitespace = __pyx_state[17]; __pyx_result.validators = __pyx_state[18]; __pyx_result.write_only = __pyx_state[19]
 *     if len(__pyx_state) > 20 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_PasswordField__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_UUIDField(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_17__pyx_unpickle_UUIDField(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_9drf_turbo_6fields_17__pyx_unpickle_UUIDField = {"__pyx_unpickle_UUIDField", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_17__pyx_unpickle_UUIDField, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_9drf_turbo_6fields_17__pyx_unpickle_UUIDField(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_UUIDField (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_UUIDField", 1, 3, 3, 1); __PYX_ERR(1, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_UUIDField", 1, 3, 3, 2); __PYX_ERR(1, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_UUIDField") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_UUIDField", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_UUIDField", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9drf_turbo_6fields_16__pyx_unpickle_UUIDField(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_16__pyx_unpickle_UUIDField(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_UUIDField", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x0fbedf5:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x0fbedf5 = (allow_blank, allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_length, min_length, read_only, required, root, style, trim_whitespace, validators, write_only))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x0fbedf5) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x0fbedf5:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x0fbedf5 = (allow_blank, allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_length, min_length, read_only, required, root, style, trim_whitespace, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = UUIDField.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0x0fbedf5:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x0fbedf5 = (allow_blank, allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_length, min_length, read_only, required, root, style, trim_whitespace, validators, write_only))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = UUIDField.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x0f, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(1, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x0fbedf5:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x0fbedf5 = (allow_blank, allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_length, min_length, read_only, required, root, style, trim_whitespace, validators, write_only))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x0fbedf5 = (allow_blank, allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_length, min_length, read_only, required, root, style, trim_whitespace, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = UUIDField.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_UUIDField__set_state(<UUIDField> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_9drf_turbo_6fields_UUIDField), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x0fbedf5 = (allow_blank, allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_length, min_length, read_only, required, root, style, trim_whitespace, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = UUIDField.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_UUIDField__set_state(<UUIDField> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = UUIDField.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_UUIDField__set_state(<UUIDField> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_UUIDField__set_state(UUIDField __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_9drf_turbo_6fields___pyx_unpickle_UUIDField__set_state(((struct __pyx_obj_9drf_turbo_6fields_UUIDField *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x0fbedf5 = (allow_blank, allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_length, min_length, read_only, required, root, style, trim_whitespace, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = UUIDField.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_UUIDField__set_state(<UUIDField> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_UUIDField__set_state(<UUIDField> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_UUIDField__set_state(UUIDField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_blank = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.max_length = __pyx_state[11]; __pyx_result.min_length = __pyx_state[12]; __pyx_result.read_only = __pyx_state[13]; __pyx_result.required = __pyx_state[14]; __pyx_result.root = __pyx_state[15]; __pyx_result.style = __pyx_state[16]; __pyx_result.trim_whitespace = __pyx_state[17]; __pyx_result.validators = __pyx_state[18]; __pyx_result.write_only = __pyx_state[19]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_UUIDField(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_UUIDField", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_UUIDField__set_state(<UUIDField> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_UUIDField__set_state(UUIDField __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.allow_blank = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.max_length = __pyx_state[11]; __pyx_result.min_length = __pyx_state[12]; __pyx_result.read_only = __pyx_state[13]; __pyx_result.required = __pyx_state[14]; __pyx_result.root = __pyx_state[15]; __pyx_result.style = __pyx_state[16]; __pyx_result.trim_whitespace = __pyx_state[17]; __pyx_result.validators = __pyx_state[18]; __pyx_result.write_only = __pyx_state[19]
 *     if len(__pyx_state) > 20 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_9drf_turbo_6fields___pyx_unpickle_UUIDField__set_state(struct __pyx_obj_9drf_turbo_6fields_UUIDField *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_UUIDField__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_UUIDField__set_state(UUIDField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_blank = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.max_length = __pyx_state[11]; __pyx_result.min_length = __pyx_state[12]; __pyx_result.read_only = __pyx_state[13]; __pyx_result.required = __pyx_state[14]; __pyx_result.root = __pyx_state[15]; __pyx_result.style = __pyx_state[16]; __pyx_result.trim_whitespace = __pyx_state[17]; __pyx_result.validators = __pyx_state[18]; __pyx_result.write_only = __pyx_state[19]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 20 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[20])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.allow_blank);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.allow_blank);
  __pyx_v___pyx_result->__pyx_base.allow_blank = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.allow_null = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.attr);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.attr);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.attr = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.attrs);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.attrs);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.attrs = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.call = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.default_value);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.default_value);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.default_value = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.error_messages);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.error_messages);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.error_messages = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.field_name);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.field_name);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.field_name = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 8, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.help_text);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.help_text);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.help_text = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 9, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.initial);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.initial);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.initial = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 10, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.label);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.label);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.label = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 11, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.max_length);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.max_length);
  __pyx_v___pyx_result->__pyx_base.max_length = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 12, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.min_length);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.min_length);
  __pyx_v___pyx_result->__pyx_base.min_length = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 13, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.read_only = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 14, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.required = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 15, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.root);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.root);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.root = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 16, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.style);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.style);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.style = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 17, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.trim_whitespace);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.trim_whitespace);
  __pyx_v___pyx_result->__pyx_base.trim_whitespace = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 18, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.validators);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.validators);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.validators = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 19, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.write_only = __pyx_t_2;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_UUIDField__set_state(UUIDField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_blank = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.max_length = __pyx_state[11]; __pyx_result.min_length = __pyx_state[12]; __pyx_result.read_only = __pyx_state[13]; __pyx_result.required = __pyx_state[14]; __pyx_result.root = __pyx_state[15]; __pyx_result.style = __pyx_state[16]; __pyx_result.trim_whitespace = __pyx_state[17]; __pyx_result.validators = __pyx_state[18]; __pyx_result.write_only = __pyx_state[19]
 *     if len(__pyx_state) > 20 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[20])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 13, __pyx_L1_error)
  }
  __pyx_t_3 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_4 = ((__pyx_t_3 > 20) != 0);
  if (__pyx_t_4) {
  } else {
    __pyx_t_2 = __pyx_t_4;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_4 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_5 = (__pyx_t_4 != 0);
  __pyx_t_2 = __pyx_t_5;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":14
 *     __pyx_result.allow_blank = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.max_length = __pyx_state[11]; __pyx_result.min_length = __pyx_state[12]; __pyx_result.read_only = __pyx_state[13]; __pyx_result.required = __pyx_state[14]; __pyx_result.root = __pyx_state[15]; __pyx_result.style = __pyx_state[16]; __pyx_result.trim_whitespace = __pyx_state[17]; __pyx_result.validators = __pyx_state[18]; __pyx_result.write_only = __pyx_state[19]
 *     if len(__pyx_state) > 20 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[20])             # <<<<<<<<<<<<<<
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_update); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 14, __pyx_L1_error)
    }
    __pyx_t_6 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 20, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_8 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
      }
    }
    __pyx_t_1 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_8, __pyx_t_6) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_6);
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_UUIDField__set_state(UUIDField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_blank = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.max_length = __pyx_state[11]; __pyx_result.min_length = __pyx_state[12]; __pyx_result.read_only = __pyx_state[13]; __pyx_result.required = __pyx_state[14]; __pyx_result.root = __pyx_state[15]; __pyx_result.style = __pyx_state[16]; __pyx_result.trim_whitespace = __pyx_state[17]; __pyx_result.validators = __pyx_state[18]; __pyx_result.write_only = __pyx_state[19]
 *     if len(__pyx_state) > 20 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[20])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_UUIDField__set_state(<UUIDField> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_UUIDField__set_state(UUIDField __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.allow_blank = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.max_length = __pyx_state[11]; __pyx_result.min_length = __pyx_state[12]; __pyx_result.read_only = __pyx_state[13]; __pyx_result.required = __pyx_state[14]; __pyx_result.root = __pyx_state[15]; __pyx_result.style = __pyx_state[16]; __pyx_result.trim_whitespace = __pyx_state[17]; __pyx_result.validators = __pyx_state[18]; __pyx_result.write_only = __pyx_state[19]
 *     if len(__pyx_state) > 20 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_UUIDField__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_SlugField(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_19__pyx_unpickle_SlugField(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_9drf_turbo_6fields_19__pyx_unpickle_SlugField = {"__pyx_unpickle_SlugField", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_19__pyx_unpickle_SlugField, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_9drf_turbo_6fields_19__pyx_unpickle_SlugField(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_SlugField (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_SlugField", 1, 3, 3, 1); __PYX_ERR(1, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_SlugField", 1, 3, 3, 2); __PYX_ERR(1, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_SlugField") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_SlugField", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_SlugField", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9drf_turbo_6fields_18__pyx_unpickle_SlugField(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_18__pyx_unpickle_SlugField(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_SlugField", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x7207e31:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7207e31 = (allow_null, allow_unicode, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x7207e31) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x7207e31:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7207e31 = (allow_null, allow_unicode, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = SlugField.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0x7207e31:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7207e31 = (allow_null, allow_unicode, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = SlugField.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x72, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(1, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x7207e31:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7207e31 = (allow_null, allow_unicode, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7207e31 = (allow_null, allow_unicode, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = SlugField.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_SlugField__set_state(<SlugField> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_9drf_turbo_6fields_SlugField), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7207e31 = (allow_null, allow_unicode, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = SlugField.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_SlugField__set_state(<SlugField> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = SlugField.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_SlugField__set_state(<SlugField> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_SlugField__set_state(SlugField __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_9drf_turbo_6fields___pyx_unpickle_SlugField__set_state(((struct __pyx_obj_9drf_turbo_6fields_SlugField *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7207e31 = (allow_null, allow_unicode, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = SlugField.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_SlugField__set_state(<SlugField> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_SlugField__set_state(<SlugField> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_SlugField__set_state(SlugField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.allow_unicode = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.read_only = __pyx_state[11]; __pyx_result.required = __pyx_state[12]; __pyx_result.root = __pyx_state[13]; __pyx_result.style = __pyx_state[14]; __pyx_result.validators = __pyx_state[15]; __pyx_result.write_only = __pyx_state[16]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_SlugField(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_SlugField", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_SlugField__set_state(<SlugField> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_SlugField__set_state(SlugField __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.allow_unicode = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.read_only = __pyx_state[11]; __pyx_result.required = __pyx_state[12]; __pyx_result.root = __pyx_state[13]; __pyx_result.style = __pyx_state[14]; __pyx_result.validators = __pyx_state[15]; __pyx_result.write_only = __pyx_state[16]
 *     if len(__pyx_state) > 17 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_9drf_turbo_6fields___pyx_unpickle_SlugField__set_state(struct __pyx_obj_9drf_turbo_6fields_SlugField *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_SlugField__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_SlugField__set_state(SlugField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.allow_unicode = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.read_only = __pyx_state[11]; __pyx_result.required = __pyx_state[12]; __pyx_result.root = __pyx_state[13]; __pyx_result.style = __pyx_state[14]; __pyx_result.validators = __pyx_state[15]; __pyx_result.write_only = __pyx_state[16]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 17 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[17])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.allow_null = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->allow_unicode);
  __Pyx_DECREF(__pyx_v___pyx_result->allow_unicode);
  __pyx_v___pyx_result->allow_unicode = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.attr);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.attr);
  __pyx_v___pyx_result->__pyx_base.attr = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.attrs);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.attrs);
  __pyx_v___pyx_result->__pyx_base.attrs = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.call = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.default_value);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.default_value);
  __pyx_v___pyx_result->__pyx_base.default_value = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.error_messages);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.error_messages);
  __pyx_v___pyx_result->__pyx_base.error_messages = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.field_name);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.field_name);
  __pyx_v___pyx_result->__pyx_base.field_name = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 8, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.help_text);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.help_text);
  __pyx_v___pyx_result->__pyx_base.help_text = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 9, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.initial);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.initial);
  __pyx_v___pyx_result->__pyx_base.initial = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 10, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.label);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.label);
  __pyx_v___pyx_result->__pyx_base.label = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 11, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.read_only = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 12, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.required = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 13, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.root);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.root);
  __pyx_v___pyx_result->__pyx_base.root = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 14, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.style);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.style);
  __pyx_v___pyx_result->__pyx_base.style = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 15, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.validators);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.validators);
  __pyx_v___pyx_result->__pyx_base.validators = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 16, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.write_only = __pyx_t_2;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_SlugField__set_state(SlugField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.allow_unicode = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.read_only = __pyx_state[11]; __pyx_result.required = __pyx_state[12]; __pyx_result.root = __pyx_state[13]; __pyx_result.style = __pyx_state[14]; __pyx_result.validators = __pyx_state[15]; __pyx_result.write_only = __pyx_state[16]
 *     if len(__pyx_state) > 17 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[17])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 13, __pyx_L1_error)
  }
  __pyx_t_3 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_4 = ((__pyx_t_3 > 17) != 0);
  if (__pyx_t_4) {
  } else {
    __pyx_t_2 = __pyx_t_4;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_4 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_5 = (__pyx_t_4 != 0);
  __pyx_t_2 = __pyx_t_5;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":14
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.allow_unicode = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.read_only = __pyx_state[11]; __pyx_result.required = __pyx_state[12]; __pyx_result.root = __pyx_state[13]; __pyx_result.style = __pyx_state[14]; __pyx_result.validators = __pyx_state[15]; __pyx_result.write_only = __pyx_state[16]
 *     if len(__pyx_state) > 17 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[17])             # <<<<<<<<<<<<<<
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_update); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 14, __pyx_L1_error)
    }
    __pyx_t_6 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 17, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_8 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
      }
    }
    __pyx_t_1 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_8, __pyx_t_6) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_6);
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_SlugField__set_state(SlugField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.allow_unicode = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.read_only = __pyx_state[11]; __pyx_result.required = __pyx_state[12]; __pyx_result.root = __pyx_state[13]; __pyx_result.style = __pyx_state[14]; __pyx_result.validators = __pyx_state[15]; __pyx_result.write_only = __pyx_state[16]
 *     if len(__pyx_state) > 17 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[17])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_SlugField__set_state(<SlugField> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_SlugField__set_state(SlugField __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.allow_unicode = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.read_only = __pyx_state[11]; __pyx_result.required = __pyx_state[12]; __pyx_result.root = __pyx_state[13]; __pyx_result.style = __pyx_state[14]; __pyx_result.validators = __pyx_state[15]; __pyx_result.write_only = __pyx_state[16]
 *     if len(__pyx_state) > 17 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_SlugField__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_IntField(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_21__pyx_unpickle_IntField(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_9drf_turbo_6fields_21__pyx_unpickle_IntField = {"__pyx_unpickle_IntField", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_21__pyx_unpickle_IntField, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_9drf_turbo_6fields_21__pyx_unpickle_IntField(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_IntField (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_IntField", 1, 3, 3, 1); __PYX_ERR(1, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_IntField", 1, 3, 3, 2); __PYX_ERR(1, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_IntField") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_IntField", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_IntField", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9drf_turbo_6fields_20__pyx_unpickle_IntField(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_20__pyx_unpickle_IntField(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_IntField", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x66626e5:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x66626e5 = (allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_value, min_value, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x66626e5) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x66626e5:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x66626e5 = (allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_value, min_value, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = IntField.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0x66626e5:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x66626e5 = (allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_value, min_value, read_only, required, root, style, validators, write_only))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = IntField.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x66, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(1, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x66626e5:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x66626e5 = (allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_value, min_value, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x66626e5 = (allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_value, min_value, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = IntField.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_IntField__set_state(<IntField> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_9drf_turbo_6fields_IntField), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x66626e5 = (allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_value, min_value, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = IntField.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_IntField__set_state(<IntField> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = IntField.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_IntField__set_state(<IntField> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_IntField__set_state(IntField __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_9drf_turbo_6fields___pyx_unpickle_IntField__set_state(((struct __pyx_obj_9drf_turbo_6fields_IntField *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x66626e5 = (allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_value, min_value, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = IntField.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_IntField__set_state(<IntField> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_IntField__set_state(<IntField> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_IntField__set_state(IntField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.default_value = __pyx_state[4]; __pyx_result.error_messages = __pyx_state[5]; __pyx_result.field_name = __pyx_state[6]; __pyx_result.help_text = __pyx_state[7]; __pyx_result.initial = __pyx_state[8]; __pyx_result.label = __pyx_state[9]; __pyx_result.max_value = __pyx_state[10]; __pyx_result.min_value = __pyx_state[11]; __pyx_result.read_only = __pyx_state[12]; __pyx_result.required = __pyx_state[13]; __pyx_result.root = __pyx_state[14]; __pyx_result.style = __pyx_state[15]; __pyx_result.validators = __pyx_state[16]; __pyx_result.write_only = __pyx_state[17]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_IntField(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_IntField", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_IntField__set_state(<IntField> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_IntField__set_state(IntField __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.default_value = __pyx_state[4]; __pyx_result.error_messages = __pyx_state[5]; __pyx_result.field_name = __pyx_state[6]; __pyx_result.help_text = __pyx_state[7]; __pyx_result.initial = __pyx_state[8]; __pyx_result.label = __pyx_state[9]; __pyx_result.max_value = __pyx_state[10]; __pyx_result.min_value = __pyx_state[11]; __pyx_result.read_only = __pyx_state[12]; __pyx_result.required = __pyx_state[13]; __pyx_result.root = __pyx_state[14]; __pyx_result.style = __pyx_state[15]; __pyx_result.validators = __pyx_state[16]; __pyx_result.write_only = __pyx_state[17]
 *     if len(__pyx_state) > 18 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_9drf_turbo_6fields___pyx_unpickle_IntField__set_state(struct __pyx_obj_9drf_turbo_6fields_IntField *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_IntField__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_IntField__set_state(IntField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.default_value = __pyx_state[4]; __pyx_result.error_messages = __pyx_state[5]; __pyx_result.field_name = __pyx_state[6]; __pyx_result.help_text = __pyx_state[7]; __pyx_result.initial = __pyx_state[8]; __pyx_result.label = __pyx_state[9]; __pyx_result.max_value = __pyx_state[10]; __pyx_result.min_value = __pyx_state[11]; __pyx_result.read_only = __pyx_state[12]; __pyx_result.required = __pyx_state[13]; __pyx_result.root = __pyx_state[14]; __pyx_result.style = __pyx_state[15]; __pyx_result.validators = __pyx_state[16]; __pyx_result.write_only = __pyx_state[17]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 18 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[18])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.allow_null = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.attr);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.attr);
  __pyx_v___pyx_result->__pyx_base.attr = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.attrs);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.attrs);
  __pyx_v___pyx_result->__pyx_base.attrs = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.call = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.default_value);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.default_value);
  __pyx_v___pyx_result->__pyx_base.default_value = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.error_messages);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.error_messages);
  __pyx_v___pyx_result->__pyx_base.error_messages = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.field_name);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.field_name);
  __pyx_v___pyx_result->__pyx_base.field_name = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.help_text);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.help_text);
  __pyx_v___pyx_result->__pyx_base.help_text = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 8, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.initial);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.initial);
  __pyx_v___pyx_result->__pyx_base.initial = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 9, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.label);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.label);
  __pyx_v___pyx_result->__pyx_base.label = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 10, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->max_value);
  __Pyx_DECREF(__pyx_v___pyx_result->max_value);
  __pyx_v___pyx_result->max_value = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 11, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->min_value);
  __Pyx_DECREF(__pyx_v___pyx_result->min_value);
  __pyx_v___pyx_result->min_value = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 12, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.read_only = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 13, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.required = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 14, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.root);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.root);
  __pyx_v___pyx_result->__pyx_base.root = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 15, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.style);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.style);
  __pyx_v___pyx_result->__pyx_base.style = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 16, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.validators);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.validators);
  __pyx_v___pyx_result->__pyx_base.validators = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 17, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.write_only = __pyx_t_2;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_IntField__set_state(IntField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.default_value = __pyx_state[4]; __pyx_result.error_messages = __pyx_state[5]; __pyx_result.field_name = __pyx_state[6]; __pyx_result.help_text = __pyx_state[7]; __pyx_result.initial = __pyx_state[8]; __pyx_result.label = __pyx_state[9]; __pyx_result.max_value = __pyx_state[10]; __pyx_result.min_value = __pyx_state[11]; __pyx_result.read_only = __pyx_state[12]; __pyx_result.required = __pyx_state[13]; __pyx_result.root = __pyx_state[14]; __pyx_result.style = __pyx_state[15]; __pyx_result.validators = __pyx_state[16]; __pyx_result.write_only = __pyx_state[17]
 *     if len(__pyx_state) > 18 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[18])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 13, __pyx_L1_error)
  }
  __pyx_t_3 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_4 = ((__pyx_t_3 > 18) != 0);
  if (__pyx_t_4) {
  } else {
    __pyx_t_2 = __pyx_t_4;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_4 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_5 = (__pyx_t_4 != 0);
  __pyx_t_2 = __pyx_t_5;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":14
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.default_value = __pyx_state[4]; __pyx_result.error_messages = __pyx_state[5]; __pyx_result.field_name = __pyx_state[6]; __pyx_result.help_text = __pyx_state[7]; __pyx_result.initial = __pyx_state[8]; __pyx_result.label = __pyx_state[9]; __pyx_result.max_value = __pyx_state[10]; __pyx_result.min_value = __pyx_state[11]; __pyx_result.read_only = __pyx_state[12]; __pyx_result.required = __pyx_state[13]; __pyx_result.root = __pyx_state[14]; __pyx_result.style = __pyx_state[15]; __pyx_result.validators = __pyx_state[16]; __pyx_result.write_only = __pyx_state[17]
 *     if len(__pyx_state) > 18 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[18])             # <<<<<<<<<<<<<<
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_update); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 14, __pyx_L1_error)
    }
    __pyx_t_6 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 18, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_8 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
      }
    }
    __pyx_t_1 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_8, __pyx_t_6) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_6);
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_IntField__set_state(IntField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.default_value = __pyx_state[4]; __pyx_result.error_messages = __pyx_state[5]; __pyx_result.field_name = __pyx_state[6]; __pyx_result.help_text = __pyx_state[7]; __pyx_result.initial = __pyx_state[8]; __pyx_result.label = __pyx_state[9]; __pyx_result.max_value = __pyx_state[10]; __pyx_result.min_value = __pyx_state[11]; __pyx_result.read_only = __pyx_state[12]; __pyx_result.required = __pyx_state[13]; __pyx_result.root = __pyx_state[14]; __pyx_result.style = __pyx_state[15]; __pyx_result.validators = __pyx_state[16]; __pyx_result.write_only = __pyx_state[17]
 *     if len(__pyx_state) > 18 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[18])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_IntField__set_state(<IntField> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_IntField__set_state(IntField __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.default_value = __pyx_state[4]; __pyx_result.error_messages = __pyx_state[5]; __pyx_result.field_name = __pyx_state[6]; __pyx_result.help_text = __pyx_state[7]; __pyx_result.initial = __pyx_state[8]; __pyx_result.label = __pyx_state[9]; __pyx_result.max_value = __pyx_state[10]; __pyx_result.min_value = __pyx_state[11]; __pyx_result.read_only = __pyx_state[12]; __pyx_result.required = __pyx_state[13]; __pyx_result.root = __pyx_state[14]; __pyx_result.style = __pyx_state[15]; __pyx_result.validators = __pyx_state[16]; __pyx_result.write_only = __pyx_state[17]
 *     if len(__pyx_state) > 18 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_IntField__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_FloatField(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_23__pyx_unpickle_FloatField(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_9drf_turbo_6fields_23__pyx_unpickle_FloatField = {"__pyx_unpickle_FloatField", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_23__pyx_unpickle_FloatField, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_9drf_turbo_6fields_23__pyx_unpickle_FloatField(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_FloatField (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_FloatField", 1, 3, 3, 1); __PYX_ERR(1, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_FloatField", 1, 3, 3, 2); __PYX_ERR(1, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_FloatField") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_FloatField", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_FloatField", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9drf_turbo_6fields_22__pyx_unpickle_FloatField(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_22__pyx_unpickle_FloatField(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_FloatField", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x66626e5:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x66626e5 = (allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_value, min_value, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x66626e5) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x66626e5:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x66626e5 = (allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_value, min_value, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = FloatField.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0x66626e5:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x66626e5 = (allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_value, min_value, read_only, required, root, style, validators, write_only))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = FloatField.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x66, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(1, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x66626e5:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x66626e5 = (allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_value, min_value, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x66626e5 = (allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_value, min_value, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = FloatField.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_FloatField__set_state(<FloatField> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_9drf_turbo_6fields_FloatField), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x66626e5 = (allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_value, min_value, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = FloatField.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_FloatField__set_state(<FloatField> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = FloatField.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_FloatField__set_state(<FloatField> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_FloatField__set_state(FloatField __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_9drf_turbo_6fields___pyx_unpickle_FloatField__set_state(((struct __pyx_obj_9drf_turbo_6fields_FloatField *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x66626e5 = (allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_value, min_value, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = FloatField.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_FloatField__set_state(<FloatField> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_FloatField__set_state(<FloatField> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_FloatField__set_state(FloatField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.default_value = __pyx_state[4]; __pyx_result.error_messages = __pyx_state[5]; __pyx_result.field_name = __pyx_state[6]; __pyx_result.help_text = __pyx_state[7]; __pyx_result.initial = __pyx_state[8]; __pyx_result.label = __pyx_state[9]; __pyx_result.max_value = __pyx_state[10]; __pyx_result.min_value = __pyx_state[11]; __pyx_result.read_only = __pyx_state[12]; __pyx_result.required = __pyx_state[13]; __pyx_result.root = __pyx_state[14]; __pyx_result.style = __pyx_state[15]; __pyx_result.validators = __pyx_state[16]; __pyx_result.write_only = __pyx_state[17]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_FloatField(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_FloatField", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_FloatField__set_state(<FloatField> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_FloatField__set_state(FloatField __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.default_value = __pyx_state[4]; __pyx_result.error_messages = __pyx_state[5]; __pyx_result.field_name = __pyx_state[6]; __pyx_result.help_text = __pyx_state[7]; __pyx_result.initial = __pyx_state[8]; __pyx_result.label = __pyx_state[9]; __pyx_result.max_value = __pyx_state[10]; __pyx_result.min_value = __pyx_state[11]; __pyx_result.read_only = __pyx_state[12]; __pyx_result.required = __pyx_state[13]; __pyx_result.root = __pyx_state[14]; __pyx_result.style = __pyx_state[15]; __pyx_result.validators = __pyx_state[16]; __pyx_result.write_only = __pyx_state[17]
 *     if len(__pyx_state) > 18 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_9drf_turbo_6fields___pyx_unpickle_FloatField__set_state(struct __pyx_obj_9drf_turbo_6fields_FloatField *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_FloatField__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_FloatField__set_state(FloatField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.default_value = __pyx_state[4]; __pyx_result.error_messages = __pyx_state[5]; __pyx_result.field_name = __pyx_state[6]; __pyx_result.help_text = __pyx_state[7]; __pyx_result.initial = __pyx_state[8]; __pyx_result.label = __pyx_state[9]; __pyx_result.max_value = __pyx_state[10]; __pyx_result.min_value = __pyx_state[11]; __pyx_result.read_only = __pyx_state[12]; __pyx_result.required = __pyx_state[13]; __pyx_result.root = __pyx_state[14]; __pyx_result.style = __pyx_state[15]; __pyx_result.validators = __pyx_state[16]; __pyx_result.write_only = __pyx_state[17]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 18 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[18])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.allow_null = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.attr);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.attr);
  __pyx_v___pyx_result->__pyx_base.attr = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.attrs);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.attrs);
  __pyx_v___pyx_result->__pyx_base.attrs = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.call = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.default_value);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.default_value);
  __pyx_v___pyx_result->__pyx_base.default_value = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.error_messages);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.error_messages);
  __pyx_v___pyx_result->__pyx_base.error_messages = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.field_name);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.field_name);
  __pyx_v___pyx_result->__pyx_base.field_name = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.help_text);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.help_text);
  __pyx_v___pyx_result->__pyx_base.help_text = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 8, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.initial);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.initial);
  __pyx_v___pyx_result->__pyx_base.initial = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 9, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.label);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.label);
  __pyx_v___pyx_result->__pyx_base.label = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 10, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->max_value);
  __Pyx_DECREF(__pyx_v___pyx_result->max_value);
  __pyx_v___pyx_result->max_value = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 11, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->min_value);
  __Pyx_DECREF(__pyx_v___pyx_result->min_value);
  __pyx_v___pyx_result->min_value = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 12, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.read_only = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 13, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.required = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 14, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.root);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.root);
  __pyx_v___pyx_result->__pyx_base.root = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 15, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.style);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.style);
  __pyx_v___pyx_result->__pyx_base.style = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 16, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.validators);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.validators);
  __pyx_v___pyx_result->__pyx_base.validators = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 17, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.write_only = __pyx_t_2;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_FloatField__set_state(FloatField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.default_value = __pyx_state[4]; __pyx_result.error_messages = __pyx_state[5]; __pyx_result.field_name = __pyx_state[6]; __pyx_result.help_text = __pyx_state[7]; __pyx_result.initial = __pyx_state[8]; __pyx_result.label = __pyx_state[9]; __pyx_result.max_value = __pyx_state[10]; __pyx_result.min_value = __pyx_state[11]; __pyx_result.read_only = __pyx_state[12]; __pyx_result.required = __pyx_state[13]; __pyx_result.root = __pyx_state[14]; __pyx_result.style = __pyx_state[15]; __pyx_result.validators = __pyx_state[16]; __pyx_result.write_only = __pyx_state[17]
 *     if len(__pyx_state) > 18 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[18])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 13, __pyx_L1_error)
  }
  __pyx_t_3 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_4 = ((__pyx_t_3 > 18) != 0);
  if (__pyx_t_4) {
  } else {
    __pyx_t_2 = __pyx_t_4;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_4 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_5 = (__pyx_t_4 != 0);
  __pyx_t_2 = __pyx_t_5;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":14
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.default_value = __pyx_state[4]; __pyx_result.error_messages = __pyx_state[5]; __pyx_result.field_name = __pyx_state[6]; __pyx_result.help_text = __pyx_state[7]; __pyx_result.initial = __pyx_state[8]; __pyx_result.label = __pyx_state[9]; __pyx_result.max_value = __pyx_state[10]; __pyx_result.min_value = __pyx_state[11]; __pyx_result.read_only = __pyx_state[12]; __pyx_result.required = __pyx_state[13]; __pyx_result.root = __pyx_state[14]; __pyx_result.style = __pyx_state[15]; __pyx_result.validators = __pyx_state[16]; __pyx_result.write_only = __pyx_state[17]
 *     if len(__pyx_state) > 18 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[18])             # <<<<<<<<<<<<<<
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_update); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 14, __pyx_L1_error)
    }
    __pyx_t_6 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 18, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_8 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
      }
    }
    __pyx_t_1 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_8, __pyx_t_6) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_6);
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_FloatField__set_state(FloatField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.default_value = __pyx_state[4]; __pyx_result.error_messages = __pyx_state[5]; __pyx_result.field_name = __pyx_state[6]; __pyx_result.help_text = __pyx_state[7]; __pyx_result.initial = __pyx_state[8]; __pyx_result.label = __pyx_state[9]; __pyx_result.max_value = __pyx_state[10]; __pyx_result.min_value = __pyx_state[11]; __pyx_result.read_only = __pyx_state[12]; __pyx_result.required = __pyx_state[13]; __pyx_result.root = __pyx_state[14]; __pyx_result.style = __pyx_state[15]; __pyx_result.validators = __pyx_state[16]; __pyx_result.write_only = __pyx_state[17]
 *     if len(__pyx_state) > 18 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[18])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_FloatField__set_state(<FloatField> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_FloatField__set_state(FloatField __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.default_value = __pyx_state[4]; __pyx_result.error_messages = __pyx_state[5]; __pyx_result.field_name = __pyx_state[6]; __pyx_result.help_text = __pyx_state[7]; __pyx_result.initial = __pyx_state[8]; __pyx_result.label = __pyx_state[9]; __pyx_result.max_value = __pyx_state[10]; __pyx_result.min_value = __pyx_state[11]; __pyx_result.read_only = __pyx_state[12]; __pyx_result.required = __pyx_state[13]; __pyx_result.root = __pyx_state[14]; __pyx_result.style = __pyx_state[15]; __pyx_result.validators = __pyx_state[16]; __pyx_result.write_only = __pyx_state[17]
 *     if len(__pyx_state) > 18 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_FloatField__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_DecimalField(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_25__pyx_unpickle_DecimalField(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_9drf_turbo_6fields_25__pyx_unpickle_DecimalField = {"__pyx_unpickle_DecimalField", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_25__pyx_unpickle_DecimalField, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_9drf_turbo_6fields_25__pyx_unpickle_DecimalField(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_DecimalField (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_DecimalField", 1, 3, 3, 1); __PYX_ERR(1, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_DecimalField", 1, 3, 3, 2); __PYX_ERR(1, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_DecimalField") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_DecimalField", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_DecimalField", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9drf_turbo_6fields_24__pyx_unpickle_DecimalField(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_24__pyx_unpickle_DecimalField(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_DecimalField", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xdaed7b0:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xdaed7b0 = (allow_null, attr, attrs, call, coerce_to_string, decimal_places, default_value, error_messages, field_name, help_text, initial, label, max_digits, max_value, max_whole_digits, min_value, read_only, required, root, rounding, style, validators, write_only))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0xdaed7b0) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xdaed7b0:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xdaed7b0 = (allow_null, attr, attrs, call, coerce_to_string, decimal_places, default_value, error_messages, field_name, help_text, initial, label, max_digits, max_value, max_whole_digits, min_value, read_only, required, root, rounding, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = DecimalField.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0xdaed7b0:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xdaed7b0 = (allow_null, attr, attrs, call, coerce_to_string, decimal_places, default_value, error_messages, field_name, help_text, initial, label, max_digits, max_value, max_whole_digits, min_value, read_only, required, root, rounding, style, validators, write_only))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = DecimalField.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0xda, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(1, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xdaed7b0:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xdaed7b0 = (allow_null, attr, attrs, call, coerce_to_string, decimal_places, default_value, error_messages, field_name, help_text, initial, label, max_digits, max_value, max_whole_digits, min_value, read_only, required, root, rounding, style, validators, write_only))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xdaed7b0 = (allow_null, attr, attrs, call, coerce_to_string, decimal_places, default_value, error_messages, field_name, help_text, initial, label, max_digits, max_value, max_whole_digits, min_value, read_only, required, root, rounding, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = DecimalField.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_DecimalField__set_state(<DecimalField> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_9drf_turbo_6fields_DecimalField), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xdaed7b0 = (allow_null, attr, attrs, call, coerce_to_string, decimal_places, default_value, error_messages, field_name, help_text, initial, label, max_digits, max_value, max_whole_digits, min_value, read_only, required, root, rounding, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = DecimalField.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_DecimalField__set_state(<DecimalField> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = DecimalField.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_DecimalField__set_state(<DecimalField> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_DecimalField__set_state(DecimalField __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_9drf_turbo_6fields___pyx_unpickle_DecimalField__set_state(((struct __pyx_obj_9drf_turbo_6fields_DecimalField *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xdaed7b0 = (allow_null, attr, attrs, call, coerce_to_string, decimal_places, default_value, error_messages, field_name, help_text, initial, label, max_digits, max_value, max_whole_digits, min_value, read_only, required, root, rounding, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = DecimalField.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_DecimalField__set_state(<DecimalField> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_DecimalField__set_state(<DecimalField> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_DecimalField__set_state(DecimalField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.coerce_to_string = __pyx_state[4]; __pyx_result.decimal_places = __pyx_state[5]; __pyx_result.default_value = __pyx_state[6]; __pyx_result.error_messages = __pyx_state[7]; __pyx_result.field_name = __pyx_state[8]; __pyx_result.help_text = __pyx_state[9]; __pyx_result.initial = __pyx_state[10]; __pyx_result.label = __pyx_state[11]; __pyx_result.max_digits = __pyx_state[12]; __pyx_result.max_value = __pyx_state[13]; __pyx_result.max_whole_digits = __pyx_state[14]; __pyx_result.min_value = __pyx_state[15]; __pyx_result.read_only = __pyx_state[16]; __pyx_result.required = __pyx_state[17]; __pyx_result.root = __pyx_state[18]; __pyx_result.rounding = __pyx_state[19]; __pyx_result.style = __pyx_state[20]; __pyx_result.validators = __pyx_state[21]; __pyx_result.write_only = __pyx_state[22]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_DecimalField(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_DecimalField", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_DecimalField__set_state(<DecimalField> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_DecimalField__set_state(DecimalField __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.coerce_to_string = __pyx_state[4]; __pyx_result.decimal_places = __pyx_state[5]; __pyx_result.default_value = __pyx_state[6]; __pyx_result.error_messages = __pyx_state[7]; __pyx_result.field_name = __pyx_state[8]; __pyx_result.help_text = __pyx_state[9]; __pyx_result.initial = __pyx_state[10]; __pyx_result.label = __pyx_state[11]; __pyx_result.max_digits = __pyx_state[12]; __pyx_result.max_value = __pyx_state[13]; __pyx_result.max_whole_digits = __pyx_state[14]; __pyx_result.min_value = __pyx_state[15]; __pyx_result.read_only = __pyx_state[16]; __pyx_result.required = __pyx_state[17]; __pyx_result.root = __pyx_state[18]; __pyx_result.rounding = __pyx_state[19]; __pyx_result.style = __pyx_state[20]; __pyx_result.validators = __pyx_state[21]; __pyx_result.write_only = __pyx_state[22]
 *     if len(__pyx_state) > 23 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_9drf_turbo_6fields___pyx_unpickle_DecimalField__set_state(struct __pyx_obj_9drf_turbo_6fields_DecimalField *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_DecimalField__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_DecimalField__set_state(DecimalField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.coerce_to_string = __pyx_state[4]; __pyx_result.decimal_places = __pyx_state[5]; __pyx_result.default_value = __pyx_state[6]; __pyx_result.error_messages = __pyx_state[7]; __pyx_result.field_name = __pyx_state[8]; __pyx_result.help_text = __pyx_state[9]; __pyx_result.initial = __pyx_state[10]; __pyx_result.label = __pyx_state[11]; __pyx_result.max_digits = __pyx_state[12]; __pyx_result.max_value = __pyx_state[13]; __pyx_result.max_whole_digits = __pyx_state[14]; __pyx_result.min_value = __pyx_state[15]; __pyx_result.read_only = __pyx_state[16]; __pyx_result.required = __pyx_state[17]; __pyx_result.root = __pyx_state[18]; __pyx_result.rounding = __pyx_state[19]; __pyx_result.style = __pyx_state[20]; __pyx_result.validators = __pyx_state[21]; __pyx_result.write_only = __pyx_state[22]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 23 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[23])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.allow_null = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.attr);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.attr);
  __pyx_v___pyx_result->__pyx_base.attr = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.attrs);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.attrs);
  __pyx_v___pyx_result->__pyx_base.attrs = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.call = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->coerce_to_string);
  __Pyx_DECREF(__pyx_v___pyx_result->coerce_to_string);
  __pyx_v___pyx_result->coerce_to_string = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->decimal_places);
  __Pyx_DECREF(__pyx_v___pyx_result->decimal_places);
  __pyx_v___pyx_result->decimal_places = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.default_value);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.default_value);
  __pyx_v___pyx_result->__pyx_base.default_value = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.error_messages);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.error_messages);
  __pyx_v___pyx_result->__pyx_base.error_messages = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 8, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.field_name);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.field_name);
  __pyx_v___pyx_result->__pyx_base.field_name = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 9, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.help_text);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.help_text);
  __pyx_v___pyx_result->__pyx_base.help_text = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 10, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.initial);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.initial);
  __pyx_v___pyx_result->__pyx_base.initial = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 11, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.label);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.label);
  __pyx_v___pyx_result->__pyx_base.label = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 12, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->max_digits);
  __Pyx_DECREF(__pyx_v___pyx_result->max_digits);
  __pyx_v___pyx_result->max_digits = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 13, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->max_value);
  __Pyx_DECREF(__pyx_v___pyx_result->max_value);
  __pyx_v___pyx_result->max_value = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 14, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->max_whole_digits);
  __Pyx_DECREF(__pyx_v___pyx_result->max_whole_digits);
  __pyx_v___pyx_result->max_whole_digits = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 15, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->min_value);
  __Pyx_DECREF(__pyx_v___pyx_result->min_value);
  __pyx_v___pyx_result->min_value = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 16, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.read_only = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 17, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.required = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 18, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.root);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.root);
  __pyx_v___pyx_result->__pyx_base.root = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 19, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->rounding);
  __Pyx_DECREF(__pyx_v___pyx_result->rounding);
  __pyx_v___pyx_result->rounding = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 20, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.style);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.style);
  __pyx_v___pyx_result->__pyx_base.style = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 21, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.validators);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.validators);
  __pyx_v___pyx_result->__pyx_base.validators = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 22, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.write_only = __pyx_t_2;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_DecimalField__set_state(DecimalField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.coerce_to_string = __pyx_state[4]; __pyx_result.decimal_places = __pyx_state[5]; __pyx_result.default_value = __pyx_state[6]; __pyx_result.error_messages = __pyx_state[7]; __pyx_result.field_name = __pyx_state[8]; __pyx_result.help_text = __pyx_state[9]; __pyx_result.initial = __pyx_state[10]; __pyx_result.label = __pyx_state[11]; __pyx_result.max_digits = __pyx_state[12]; __pyx_result.max_value = __pyx_state[13]; __pyx_result.max_whole_digits = __pyx_state[14]; __pyx_result.min_value = __pyx_state[15]; __pyx_result.read_only = __pyx_state[16]; __pyx_result.required = __pyx_state[17]; __pyx_result.root = __pyx_state[18]; __pyx_result.rounding = __pyx_state[19]; __pyx_result.style = __pyx_state[20]; __pyx_result.validators = __pyx_state[21]; __pyx_result.write_only = __pyx_state[22]
 *     if len(__pyx_state) > 23 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[23])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 13, __pyx_L1_error)
  }
  __pyx_t_3 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_4 = ((__pyx_t_3 > 23) != 0);
  if (__pyx_t_4) {
  } else {
    __pyx_t_2 = __pyx_t_4;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_4 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_5 = (__pyx_t_4 != 0);
  __pyx_t_2 = __pyx_t_5;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":14
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.coerce_to_string = __pyx_state[4]; __pyx_result.decimal_places = __pyx_state[5]; __pyx_result.default_value = __pyx_state[6]; __pyx_result.error_messages = __pyx_state[7]; __pyx_result.field_name = __pyx_state[8]; __pyx_result.help_text = __pyx_state[9]; __pyx_result.initial = __pyx_state[10]; __pyx_result.label = __pyx_state[11]; __pyx_result.max_digits = __pyx_state[12]; __pyx_result.max_value = __pyx_state[13]; __pyx_result.max_whole_digits = __pyx_state[14]; __pyx_result.min_value = __pyx_state[15]; __pyx_result.read_only = __pyx_state[16]; __pyx_result.required = __pyx_state[17]; __pyx_result.root = __pyx_state[18]; __pyx_result.rounding = __pyx_state[19]; __pyx_result.style = __pyx_state[20]; __pyx_result.validators = __pyx_state[21]; __pyx_result.write_only = __pyx_state[22]
 *     if len(__pyx_state) > 23 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[23])             # <<<<<<<<<<<<<<
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_update); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 14, __pyx_L1_error)
    }
    __pyx_t_6 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 23, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_8 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
      }
    }
    __pyx_t_1 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_8, __pyx_t_6) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_6);
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_DecimalField__set_state(DecimalField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.coerce_to_string = __pyx_state[4]; __pyx_result.decimal_places = __pyx_state[5]; __pyx_result.default_value = __pyx_state[6]; __pyx_result.error_messages = __pyx_state[7]; __pyx_result.field_name = __pyx_state[8]; __pyx_result.help_text = __pyx_state[9]; __pyx_result.initial = __pyx_state[10]; __pyx_result.label = __pyx_state[11]; __pyx_result.max_digits = __pyx_state[12]; __pyx_result.max_value = __pyx_state[13]; __pyx_result.max_whole_digits = __pyx_state[14]; __pyx_result.min_value = __pyx_state[15]; __pyx_result.read_only = __pyx_state[16]; __pyx_result.required = __pyx_state[17]; __pyx_result.root = __pyx_state[18]; __pyx_result.rounding = __pyx_state[19]; __pyx_result.style = __pyx_state[20]; __pyx_result.validators = __pyx_state[21]; __pyx_result.write_only = __pyx_state[22]
 *     if len(__pyx_state) > 23 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[23])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_DecimalField__set_state(<DecimalField> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_DecimalField__set_state(DecimalField __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.coerce_to_string = __pyx_state[4]; __pyx_result.decimal_places = __pyx_state[5]; __pyx_result.default_value = __pyx_state[6]; __pyx_result.error_messages = __pyx_state[7]; __pyx_result.field_name = __pyx_state[8]; __pyx_result.help_text = __pyx_state[9]; __pyx_result.initial = __pyx_state[10]; __pyx_result.label = __pyx_state[11]; __pyx_result.max_digits = __pyx_state[12]; __pyx_result.max_value = __pyx_state[13]; __pyx_result.max_whole_digits = __pyx_state[14]; __pyx_result.min_value = __pyx_state[15]; __pyx_result.read_only = __pyx_state[16]; __pyx_result.required = __pyx_state[17]; __pyx_result.root = __pyx_state[18]; __pyx_result.rounding = __pyx_state[19]; __pyx_result.style = __pyx_state[20]; __pyx_result.validators = __pyx_state[21]; __pyx_result.write_only = __pyx_state[22]
 *     if len(__pyx_state) > 23 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_DecimalField__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_BoolField(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_27__pyx_unpickle_BoolField(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_9drf_turbo_6fields_27__pyx_unpickle_BoolField = {"__pyx_unpickle_BoolField", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_27__pyx_unpickle_BoolField, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_9drf_turbo_6fields_27__pyx_unpickle_BoolField(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_BoolField (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_BoolField", 1, 3, 3, 1); __PYX_ERR(1, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_BoolField", 1, 3, 3, 2); __PYX_ERR(1, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_BoolField") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_BoolField", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_BoolField", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9drf_turbo_6fields_26__pyx_unpickle_BoolField(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_26__pyx_unpickle_BoolField(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_BoolField", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x4f70d27:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x4f70d27 = (allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x4f70d27) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x4f70d27:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x4f70d27 = (allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = BoolField.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0x4f70d27:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x4f70d27 = (allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = BoolField.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x4f, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(1, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x4f70d27:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x4f70d27 = (allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x4f70d27 = (allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = BoolField.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_BoolField__set_state(<BoolField> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_9drf_turbo_6fields_BoolField), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x4f70d27 = (allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = BoolField.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_BoolField__set_state(<BoolField> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = BoolField.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_BoolField__set_state(<BoolField> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_BoolField__set_state(BoolField __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_9drf_turbo_6fields___pyx_unpickle_BoolField__set_state(((struct __pyx_obj_9drf_turbo_6fields_BoolField *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x4f70d27 = (allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = BoolField.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_BoolField__set_state(<BoolField> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_BoolField__set_state(<BoolField> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_BoolField__set_state(BoolField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.default_value = __pyx_state[4]; __pyx_result.error_messages = __pyx_state[5]; __pyx_result.field_name = __pyx_state[6]; __pyx_result.help_text = __pyx_state[7]; __pyx_result.initial = __pyx_state[8]; __pyx_result.label = __pyx_state[9]; __pyx_result.read_only = __pyx_state[10]; __pyx_result.required = __pyx_state[11]; __pyx_result.root = __pyx_state[12]; __pyx_result.style = __pyx_state[13]; __pyx_result.validators = __pyx_state[14]; __pyx_result.write_only = __pyx_state[15]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_BoolField(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_BoolField", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_BoolField__set_state(<BoolField> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_BoolField__set_state(BoolField __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.default_value = __pyx_state[4]; __pyx_result.error_messages = __pyx_state[5]; __pyx_result.field_name = __pyx_state[6]; __pyx_result.help_text = __pyx_state[7]; __pyx_result.initial = __pyx_state[8]; __pyx_result.label = __pyx_state[9]; __pyx_result.read_only = __pyx_state[10]; __pyx_result.required = __pyx_state[11]; __pyx_result.root = __pyx_state[12]; __pyx_result.style = __pyx_state[13]; __pyx_result.validators = __pyx_state[14]; __pyx_result.write_only = __pyx_state[15]
 *     if len(__pyx_state) > 16 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_9drf_turbo_6fields___pyx_unpickle_BoolField__set_state(struct __pyx_obj_9drf_turbo_6fields_BoolField *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_BoolField__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_BoolField__set_state(BoolField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.default_value = __pyx_state[4]; __pyx_result.error_messages = __pyx_state[5]; __pyx_result.field_name = __pyx_state[6]; __pyx_result.help_text = __pyx_state[7]; __pyx_result.initial = __pyx_state[8]; __pyx_result.label = __pyx_state[9]; __pyx_result.read_only = __pyx_state[10]; __pyx_result.required = __pyx_state[11]; __pyx_result.root = __pyx_state[12]; __pyx_result.style = __pyx_state[13]; __pyx_result.validators = __pyx_state[14]; __pyx_result.write_only = __pyx_state[15]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 16 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[16])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.allow_null = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.attr);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.attr);
  __pyx_v___pyx_result->__pyx_base.attr = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.attrs);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.attrs);
  __pyx_v___pyx_result->__pyx_base.attrs = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.call = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.default_value);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.default_value);
  __pyx_v___pyx_result->__pyx_base.default_value = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.error_messages);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.error_messages);
  __pyx_v___pyx_result->__pyx_base.error_messages = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.field_name);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.field_name);
  __pyx_v___pyx_result->__pyx_base.field_name = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.help_text);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.help_text);
  __pyx_v___pyx_result->__pyx_base.help_text = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 8, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.initial);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.initial);
  __pyx_v___pyx_result->__pyx_base.initial = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 9, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.label);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.label);
  __pyx_v___pyx_result->__pyx_base.label = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 10, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.read_only = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 11, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.required = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 12, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.root);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.root);
  __pyx_v___pyx_result->__pyx_base.root = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 13, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.style);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.style);
  __pyx_v___pyx_result->__pyx_base.style = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 14, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.validators);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.validators);
  __pyx_v___pyx_result->__pyx_base.validators = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 15, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.write_only = __pyx_t_2;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_BoolField__set_state(BoolField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.default_value = __pyx_state[4]; __pyx_result.error_messages = __pyx_state[5]; __pyx_result.field_name = __pyx_state[6]; __pyx_result.help_text = __pyx_state[7]; __pyx_result.initial = __pyx_state[8]; __pyx_result.label = __pyx_state[9]; __pyx_result.read_only = __pyx_state[10]; __pyx_result.required = __pyx_state[11]; __pyx_result.root = __pyx_state[12]; __pyx_result.style = __pyx_state[13]; __pyx_result.validators = __pyx_state[14]; __pyx_result.write_only = __pyx_state[15]
 *     if len(__pyx_state) > 16 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[16])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 13, __pyx_L1_error)
  }
  __pyx_t_3 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_4 = ((__pyx_t_3 > 16) != 0);
  if (__pyx_t_4) {
  } else {
    __pyx_t_2 = __pyx_t_4;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_4 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_5 = (__pyx_t_4 != 0);
  __pyx_t_2 = __pyx_t_5;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":14
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.default_value = __pyx_state[4]; __pyx_result.error_messages = __pyx_state[5]; __pyx_result.field_name = __pyx_state[6]; __pyx_result.help_text = __pyx_state[7]; __pyx_result.initial = __pyx_state[8]; __pyx_result.label = __pyx_state[9]; __pyx_result.read_only = __pyx_state[10]; __pyx_result.required = __pyx_state[11]; __pyx_result.root = __pyx_state[12]; __pyx_result.style = __pyx_state[13]; __pyx_result.validators = __pyx_state[14]; __pyx_result.write_only = __pyx_state[15]
 *     if len(__pyx_state) > 16 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[16])             # <<<<<<<<<<<<<<
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_update); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 14, __pyx_L1_error)
    }
    __pyx_t_6 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 16, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_8 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
      }
    }
    __pyx_t_1 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_8, __pyx_t_6) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_6);
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_BoolField__set_state(BoolField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.default_value = __pyx_state[4]; __pyx_result.error_messages = __pyx_state[5]; __pyx_result.field_name = __pyx_state[6]; __pyx_result.help_text = __pyx_state[7]; __pyx_result.initial = __pyx_state[8]; __pyx_result.label = __pyx_state[9]; __pyx_result.read_only = __pyx_state[10]; __pyx_result.required = __pyx_state[11]; __pyx_result.root = __pyx_state[12]; __pyx_result.style = __pyx_state[13]; __pyx_result.validators = __pyx_state[14]; __pyx_result.write_only = __pyx_state[15]
 *     if len(__pyx_state) > 16 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[16])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_BoolField__set_state(<BoolField> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_BoolField__set_state(BoolField __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.default_value = __pyx_state[4]; __pyx_result.error_messages = __pyx_state[5]; __pyx_result.field_name = __pyx_state[6]; __pyx_result.help_text = __pyx_state[7]; __pyx_result.initial = __pyx_state[8]; __pyx_result.label = __pyx_state[9]; __pyx_result.read_only = __pyx_state[10]; __pyx_result.required = __pyx_state[11]; __pyx_result.root = __pyx_state[12]; __pyx_result.style = __pyx_state[13]; __pyx_result.validators = __pyx_state[14]; __pyx_result.write_only = __pyx_state[15]
 *     if len(__pyx_state) > 16 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_BoolField__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_ChoiceField(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_29__pyx_unpickle_ChoiceField(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_9drf_turbo_6fields_29__pyx_unpickle_ChoiceField = {"__pyx_unpickle_ChoiceField", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_29__pyx_unpickle_ChoiceField, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_9drf_turbo_6fields_29__pyx_unpickle_ChoiceField(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_ChoiceField (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_ChoiceField", 1, 3, 3, 1); __PYX_ERR(1, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_ChoiceField", 1, 3, 3, 2); __PYX_ERR(1, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_ChoiceField") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_ChoiceField", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_ChoiceField", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9drf_turbo_6fields_28__pyx_unpickle_ChoiceField(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_28__pyx_unpickle_ChoiceField(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_ChoiceField", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xf172868:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xf172868 = (allow_blank, allow_null, attr, attrs, call, choice_strings_to_display, choice_strings_to_values, choices, default_value, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0xf172868) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xf172868:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xf172868 = (allow_blank, allow_null, attr, attrs, call, choice_strings_to_display, choice_strings_to_values, choices, default_value, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = ChoiceField.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0xf172868:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xf172868 = (allow_blank, allow_null, attr, attrs, call, choice_strings_to_display, choice_strings_to_values, choices, default_value, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = ChoiceField.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0xf1, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(1, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xf172868:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xf172868 = (allow_blank, allow_null, attr, attrs, call, choice_strings_to_display, choice_strings_to_values, choices, default_value, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xf172868 = (allow_blank, allow_null, attr, attrs, call, choice_strings_to_display, choice_strings_to_values, choices, default_value, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = ChoiceField.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_ChoiceField__set_state(<ChoiceField> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_9drf_turbo_6fields_ChoiceField), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xf172868 = (allow_blank, allow_null, attr, attrs, call, choice_strings_to_display, choice_strings_to_values, choices, default_value, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = ChoiceField.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_ChoiceField__set_state(<ChoiceField> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = ChoiceField.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_ChoiceField__set_state(<ChoiceField> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_ChoiceField__set_state(ChoiceField __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_9drf_turbo_6fields___pyx_unpickle_ChoiceField__set_state(((struct __pyx_obj_9drf_turbo_6fields_ChoiceField *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xf172868 = (allow_blank, allow_null, attr, attrs, call, choice_strings_to_display, choice_strings_to_values, choices, default_value, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = ChoiceField.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_ChoiceField__set_state(<ChoiceField> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_ChoiceField__set_state(<ChoiceField> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_ChoiceField__set_state(ChoiceField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_blank = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.choice_strings_to_display = __pyx_state[5]; __pyx_result.choice_strings_to_values = __pyx_state[6]; __pyx_result.choices = __pyx_state[7]; __pyx_result.default_value = __pyx_state[8]; __pyx_result.error_messages = __pyx_state[9]; __pyx_result.field_name = __pyx_state[10]; __pyx_result.help_text = __pyx_state[11]; __pyx_result.initial = __pyx_state[12]; __pyx_result.label = __pyx_state[13]; __pyx_result.read_only = __pyx_state[14]; __pyx_result.required = __pyx_state[15]; __pyx_result.root = __pyx_state[16]; __pyx_result.style = __pyx_state[17]; __pyx_result.validators = __pyx_state[18]; __pyx_result.write_only = __pyx_state[19]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_ChoiceField(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_ChoiceField", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_ChoiceField__set_state(<ChoiceField> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_ChoiceField__set_state(ChoiceField __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.allow_blank = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.choice_strings_to_display = __pyx_state[5]; __pyx_result.choice_strings_to_values = __pyx_state[6]; __pyx_result.choices = __pyx_state[7]; __pyx_result.default_value = __pyx_state[8]; __pyx_result.error_messages = __pyx_state[9]; __pyx_result.field_name = __pyx_state[10]; __pyx_result.help_text = __pyx_state[11]; __pyx_result.initial = __pyx_state[12]; __pyx_result.label = __pyx_state[13]; __pyx_result.read_only = __pyx_state[14]; __pyx_result.required = __pyx_state[15]; __pyx_result.root = __pyx_state[16]; __pyx_result.style = __pyx_state[17]; __pyx_result.validators = __pyx_state[18]; __pyx_result.write_only = __pyx_state[19]
 *     if len(__pyx_state) > 20 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_9drf_turbo_6fields___pyx_unpickle_ChoiceField__set_state(struct __pyx_obj_9drf_turbo_6fields_ChoiceField *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_ChoiceField__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_ChoiceField__set_state(ChoiceField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_blank = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.choice_strings_to_display = __pyx_state[5]; __pyx_result.choice_strings_to_values = __pyx_state[6]; __pyx_result.choices = __pyx_state[7]; __pyx_result.default_value = __pyx_state[8]; __pyx_result.error_messages = __pyx_state[9]; __pyx_result.field_name = __pyx_state[10]; __pyx_result.help_text = __pyx_state[11]; __pyx_result.initial = __pyx_state[12]; __pyx_result.label = __pyx_state[13]; __pyx_result.read_only = __pyx_state[14]; __pyx_result.required = __pyx_state[15]; __pyx_result.root = __pyx_state[16]; __pyx_result.style = __pyx_state[17]; __pyx_result.validators = __pyx_state[18]; __pyx_result.write_only = __pyx_state[19]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 20 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[20])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->allow_blank);
  __Pyx_DECREF(__pyx_v___pyx_result->allow_blank);
  __pyx_v___pyx_result->allow_blank = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.allow_null = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.attr);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.attr);
  __pyx_v___pyx_result->__pyx_base.attr = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.attrs);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.attrs);
  __pyx_v___pyx_result->__pyx_base.attrs = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.call = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->choice_strings_to_display);
  __Pyx_DECREF(__pyx_v___pyx_result->choice_strings_to_display);
  __pyx_v___pyx_result->choice_strings_to_display = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->choice_strings_to_values);
  __Pyx_DECREF(__pyx_v___pyx_result->choice_strings_to_values);
  __pyx_v___pyx_result->choice_strings_to_values = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->choices);
  __Pyx_DECREF(__pyx_v___pyx_result->choices);
  __pyx_v___pyx_result->choices = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 8, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.default_value);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.default_value);
  __pyx_v___pyx_result->__pyx_base.default_value = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 9, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.error_messages);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.error_messages);
  __pyx_v___pyx_result->__pyx_base.error_messages = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 10, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.field_name);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.field_name);
  __pyx_v___pyx_result->__pyx_base.field_name = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 11, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.help_text);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.help_text);
  __pyx_v___pyx_result->__pyx_base.help_text = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 12, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.initial);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.initial);
  __pyx_v___pyx_result->__pyx_base.initial = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 13, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.label);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.label);
  __pyx_v___pyx_result->__pyx_base.label = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 14, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.read_only = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 15, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.required = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 16, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.root);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.root);
  __pyx_v___pyx_result->__pyx_base.root = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 17, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.style);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.style);
  __pyx_v___pyx_result->__pyx_base.style = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 18, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.validators);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.validators);
  __pyx_v___pyx_result->__pyx_base.validators = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 19, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.write_only = __pyx_t_2;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_ChoiceField__set_state(ChoiceField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_blank = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.choice_strings_to_display = __pyx_state[5]; __pyx_result.choice_strings_to_values = __pyx_state[6]; __pyx_result.choices = __pyx_state[7]; __pyx_result.default_value = __pyx_state[8]; __pyx_result.error_messages = __pyx_state[9]; __pyx_result.field_name = __pyx_state[10]; __pyx_result.help_text = __pyx_state[11]; __pyx_result.initial = __pyx_state[12]; __pyx_result.label = __pyx_state[13]; __pyx_result.read_only = __pyx_state[14]; __pyx_result.required = __pyx_state[15]; __pyx_result.root = __pyx_state[16]; __pyx_result.style = __pyx_state[17]; __pyx_result.validators = __pyx_state[18]; __pyx_result.write_only = __pyx_state[19]
 *     if len(__pyx_state) > 20 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[20])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 13, __pyx_L1_error)
  }
  __pyx_t_3 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_4 = ((__pyx_t_3 > 20) != 0);
  if (__pyx_t_4) {
  } else {
    __pyx_t_2 = __pyx_t_4;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_4 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_5 = (__pyx_t_4 != 0);
  __pyx_t_2 = __pyx_t_5;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":14
 *     __pyx_result.allow_blank = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.choice_strings_to_display = __pyx_state[5]; __pyx_result.choice_strings_to_values = __pyx_state[6]; __pyx_result.choices = __pyx_state[7]; __pyx_result.default_value = __pyx_state[8]; __pyx_result.error_messages = __pyx_state[9]; __pyx_result.field_name = __pyx_state[10]; __pyx_result.help_text = __pyx_state[11]; __pyx_result.initial = __pyx_state[12]; __pyx_result.label = __pyx_state[13]; __pyx_result.read_only = __pyx_state[14]; __pyx_result.required = __pyx_state[15]; __pyx_result.root = __pyx_state[16]; __pyx_result.style = __pyx_state[17]; __pyx_result.validators = __pyx_state[18]; __pyx_result.write_only = __pyx_state[19]
 *     if len(__pyx_state) > 20 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[20])             # <<<<<<<<<<<<<<
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_update); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 14, __pyx_L1_error)
    }
    __pyx_t_6 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 20, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_8 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
      }
    }
    __pyx_t_1 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_8, __pyx_t_6) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_6);
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_ChoiceField__set_state(ChoiceField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_blank = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.choice_strings_to_display = __pyx_state[5]; __pyx_result.choice_strings_to_values = __pyx_state[6]; __pyx_result.choices = __pyx_state[7]; __pyx_result.default_value = __pyx_state[8]; __pyx_result.error_messages = __pyx_state[9]; __pyx_result.field_name = __pyx_state[10]; __pyx_result.help_text = __pyx_state[11]; __pyx_result.initial = __pyx_state[12]; __pyx_result.label = __pyx_state[13]; __pyx_result.read_only = __pyx_state[14]; __pyx_result.required = __pyx_state[15]; __pyx_result.root = __pyx_state[16]; __pyx_result.style = __pyx_state[17]; __pyx_result.validators = __pyx_state[18]; __pyx_result.write_only = __pyx_state[19]
 *     if len(__pyx_state) > 20 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[20])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_ChoiceField__set_state(<ChoiceField> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_ChoiceField__set_state(ChoiceField __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.allow_blank = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.choice_strings_to_display = __pyx_state[5]; __pyx_result.choice_strings_to_values = __pyx_state[6]; __pyx_result.choices = __pyx_state[7]; __pyx_result.default_value = __pyx_state[8]; __pyx_result.error_messages = __pyx_state[9]; __pyx_result.field_name = __pyx_state[10]; __pyx_result.help_text = __pyx_state[11]; __pyx_result.initial = __pyx_state[12]; __pyx_result.label = __pyx_state[13]; __pyx_result.read_only = __pyx_state[14]; __pyx_result.required = __pyx_state[15]; __pyx_result.root = __pyx_state[16]; __pyx_result.style = __pyx_state[17]; __pyx_result.validators = __pyx_state[18]; __pyx_result.write_only = __pyx_state[19]
 *     if len(__pyx_state) > 20 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_ChoiceField__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_MultipleChoiceField(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_31__pyx_unpickle_MultipleChoiceField(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_9drf_turbo_6fields_31__pyx_unpickle_MultipleChoiceField = {"__pyx_unpickle_MultipleChoiceField", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_31__pyx_unpickle_MultipleChoiceField, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_9drf_turbo_6fields_31__pyx_unpickle_MultipleChoiceField(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_MultipleChoiceField (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MultipleChoiceField", 1, 3, 3, 1); __PYX_ERR(1, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MultipleChoiceField", 1, 3, 3, 2); __PYX_ERR(1, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_MultipleChoiceField") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MultipleChoiceField", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_MultipleChoiceField", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9drf_turbo_6fields_30__pyx_unpickle_MultipleChoiceField(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_30__pyx_unpickle_MultipleChoiceField(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MultipleChoiceField", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x69fcb1b:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x69fcb1b = (allow_blank, allow_empty, allow_null, attr, attrs, call, choice_strings_to_display, choice_strings_to_values, choices, default_value, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x69fcb1b) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x69fcb1b:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x69fcb1b = (allow_blank, allow_empty, allow_null, attr, attrs, call, choice_strings_to_display, choice_strings_to_values, choices, default_value, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = MultipleChoiceField.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0x69fcb1b:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x69fcb1b = (allow_blank, allow_empty, allow_null, attr, attrs, call, choice_strings_to_display, choice_strings_to_values, choices, default_value, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = MultipleChoiceField.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x69, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(1, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x69fcb1b:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x69fcb1b = (allow_blank, allow_empty, allow_null, attr, attrs, call, choice_strings_to_display, choice_strings_to_values, choices, default_value, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x69fcb1b = (allow_blank, allow_empty, allow_null, attr, attrs, call, choice_strings_to_display, choice_strings_to_values, choices, default_value, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = MultipleChoiceField.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MultipleChoiceField__set_state(<MultipleChoiceField> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_9drf_turbo_6fields_MultipleChoiceField), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x69fcb1b = (allow_blank, allow_empty, allow_null, attr, attrs, call, choice_strings_to_display, choice_strings_to_values, choices, default_value, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = MultipleChoiceField.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MultipleChoiceField__set_state(<MultipleChoiceField> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = MultipleChoiceField.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MultipleChoiceField__set_state(<MultipleChoiceField> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_MultipleChoiceField__set_state(MultipleChoiceField __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_9drf_turbo_6fields___pyx_unpickle_MultipleChoiceField__set_state(((struct __pyx_obj_9drf_turbo_6fields_MultipleChoiceField *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x69fcb1b = (allow_blank, allow_empty, allow_null, attr, attrs, call, choice_strings_to_display, choice_strings_to_values, choices, default_value, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = MultipleChoiceField.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MultipleChoiceField__set_state(<MultipleChoiceField> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MultipleChoiceField__set_state(<MultipleChoiceField> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_MultipleChoiceField__set_state(MultipleChoiceField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_blank = __pyx_state[0]; __pyx_result.allow_empty = __pyx_state[1]; __pyx_result.allow_null = __pyx_state[2]; __pyx_result.attr = __pyx_state[3]; __pyx_result.attrs = __pyx_state[4]; __pyx_result.call = __pyx_state[5]; __pyx_result.choice_strings_to_display = __pyx_state[6]; __pyx_result.choice_strings_to_values = __pyx_state[7]; __pyx_result.choices = __pyx_state[8]; __pyx_result.default_value = __pyx_state[9]; __pyx_result.error_messages = __pyx_state[10]; __pyx_result.field_name = __pyx_state[11]; __pyx_result.help_text = __pyx_state[12]; __pyx_result.initial = __pyx_state[13]; __pyx_result.label = __pyx_state[14]; __pyx_result.read_only = __pyx_state[15]; __pyx_result.required = __pyx_state[16]; __pyx_result.root = __pyx_state[17]; __pyx_result.style = __pyx_state[18]; __pyx_result.validators = __pyx_state[19]; __pyx_result.write_only = __pyx_state[20]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_MultipleChoiceField(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_MultipleChoiceField", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_MultipleChoiceField__set_state(<MultipleChoiceField> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MultipleChoiceField__set_state(MultipleChoiceField __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.allow_blank = __pyx_state[0]; __pyx_result.allow_empty = __pyx_state[1]; __pyx_result.allow_null = __pyx_state[2]; __pyx_result.attr = __pyx_state[3]; __pyx_result.attrs = __pyx_state[4]; __pyx_result.call = __pyx_state[5]; __pyx_result.choice_strings_to_display = __pyx_state[6]; __pyx_result.choice_strings_to_values = __pyx_state[7]; __pyx_result.choices = __pyx_state[8]; __pyx_result.default_value = __pyx_state[9]; __pyx_result.error_messages = __pyx_state[10]; __pyx_result.field_name = __pyx_state[11]; __pyx_result.help_text = __pyx_state[12]; __pyx_result.initial = __pyx_state[13]; __pyx_result.label = __pyx_state[14]; __pyx_result.read_only = __pyx_state[15]; __pyx_result.required = __pyx_state[16]; __pyx_result.root = __pyx_state[17]; __pyx_result.style = __pyx_state[18]; __pyx_result.validators = __pyx_state[19]; __pyx_result.write_only = __pyx_state[20]
 *     if len(__pyx_state) > 21 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_9drf_turbo_6fields___pyx_unpickle_MultipleChoiceField__set_state(struct __pyx_obj_9drf_turbo_6fields_MultipleChoiceField *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MultipleChoiceField__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_MultipleChoiceField__set_state(MultipleChoiceField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_blank = __pyx_state[0]; __pyx_result.allow_empty = __pyx_state[1]; __pyx_result.allow_null = __pyx_state[2]; __pyx_result.attr = __pyx_state[3]; __pyx_result.attrs = __pyx_state[4]; __pyx_result.call = __pyx_state[5]; __pyx_result.choice_strings_to_display = __pyx_state[6]; __pyx_result.choice_strings_to_values = __pyx_state[7]; __pyx_result.choices = __pyx_state[8]; __pyx_result.default_value = __pyx_state[9]; __pyx_result.error_messages = __pyx_state[10]; __pyx_result.field_name = __pyx_state[11]; __pyx_result.help_text = __pyx_state[12]; __pyx_result.initial = __pyx_state[13]; __pyx_result.label = __pyx_state[14]; __pyx_result.read_only = __pyx_state[15]; __pyx_result.required = __pyx_state[16]; __pyx_result.root = __pyx_state[17]; __pyx_result.style = __pyx_state[18]; __pyx_result.validators = __pyx_state[19]; __pyx_result.write_only = __pyx_state[20]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 21 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[21])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.allow_blank);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.allow_blank);
  __pyx_v___pyx_result->__pyx_base.allow_blank = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->allow_empty);
  __Pyx_DECREF(__pyx_v___pyx_result->allow_empty);
  __pyx_v___pyx_result->allow_empty = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.allow_null = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.attr);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.attr);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.attr = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.attrs);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.attrs);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.attrs = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.call = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.choice_strings_to_display);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.choice_strings_to_display);
  __pyx_v___pyx_result->__pyx_base.choice_strings_to_display = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.choice_strings_to_values);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.choice_strings_to_values);
  __pyx_v___pyx_result->__pyx_base.choice_strings_to_values = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 8, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.choices);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.choices);
  __pyx_v___pyx_result->__pyx_base.choices = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 9, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.default_value);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.default_value);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.default_value = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 10, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.error_messages);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.error_messages);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.error_messages = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 11, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.field_name);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.field_name);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.field_name = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 12, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.help_text);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.help_text);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.help_text = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 13, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.initial);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.initial);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.initial = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 14, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.label);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.label);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.label = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 15, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.read_only = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 16, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.required = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 17, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.root);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.root);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.root = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 18, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.style);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.style);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.style = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 19, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.validators);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.validators);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.validators = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 20, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.write_only = __pyx_t_2;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_MultipleChoiceField__set_state(MultipleChoiceField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_blank = __pyx_state[0]; __pyx_result.allow_empty = __pyx_state[1]; __pyx_result.allow_null = __pyx_state[2]; __pyx_result.attr = __pyx_state[3]; __pyx_result.attrs = __pyx_state[4]; __pyx_result.call = __pyx_state[5]; __pyx_result.choice_strings_to_display = __pyx_state[6]; __pyx_result.choice_strings_to_values = __pyx_state[7]; __pyx_result.choices = __pyx_state[8]; __pyx_result.default_value = __pyx_state[9]; __pyx_result.error_messages = __pyx_state[10]; __pyx_result.field_name = __pyx_state[11]; __pyx_result.help_text = __pyx_state[12]; __pyx_result.initial = __pyx_state[13]; __pyx_result.label = __pyx_state[14]; __pyx_result.read_only = __pyx_state[15]; __pyx_result.required = __pyx_state[16]; __pyx_result.root = __pyx_state[17]; __pyx_result.style = __pyx_state[18]; __pyx_result.validators = __pyx_state[19]; __pyx_result.write_only = __pyx_state[20]
 *     if len(__pyx_state) > 21 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[21])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 13, __pyx_L1_error)
  }
  __pyx_t_3 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_4 = ((__pyx_t_3 > 21) != 0);
  if (__pyx_t_4) {
  } else {
    __pyx_t_2 = __pyx_t_4;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_4 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_5 = (__pyx_t_4 != 0);
  __pyx_t_2 = __pyx_t_5;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":14
 *     __pyx_result.allow_blank = __pyx_state[0]; __pyx_result.allow_empty = __pyx_state[1]; __pyx_result.allow_null = __pyx_state[2]; __pyx_result.attr = __pyx_state[3]; __pyx_result.attrs = __pyx_state[4]; __pyx_result.call = __pyx_state[5]; __pyx_result.choice_strings_to_display = __pyx_state[6]; __pyx_result.choice_strings_to_values = __pyx_state[7]; __pyx_result.choices = __pyx_state[8]; __pyx_result.default_value = __pyx_state[9]; __pyx_result.error_messages = __pyx_state[10]; __pyx_result.field_name = __pyx_state[11]; __pyx_result.help_text = __pyx_state[12]; __pyx_result.initial = __pyx_state[13]; __pyx_result.label = __pyx_state[14]; __pyx_result.read_only = __pyx_state[15]; __pyx_result.required = __pyx_state[16]; __pyx_result.root = __pyx_state[17]; __pyx_result.style = __pyx_state[18]; __pyx_result.validators = __pyx_state[19]; __pyx_result.write_only = __pyx_state[20]
 *     if len(__pyx_state) > 21 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[21])             # <<<<<<<<<<<<<<
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_update); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 14, __pyx_L1_error)
    }
    __pyx_t_6 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 21, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_8 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
      }
    }
    __pyx_t_1 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_8, __pyx_t_6) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_6);
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_MultipleChoiceField__set_state(MultipleChoiceField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_blank = __pyx_state[0]; __pyx_result.allow_empty = __pyx_state[1]; __pyx_result.allow_null = __pyx_state[2]; __pyx_result.attr = __pyx_state[3]; __pyx_result.attrs = __pyx_state[4]; __pyx_result.call = __pyx_state[5]; __pyx_result.choice_strings_to_display = __pyx_state[6]; __pyx_result.choice_strings_to_values = __pyx_state[7]; __pyx_result.choices = __pyx_state[8]; __pyx_result.default_value = __pyx_state[9]; __pyx_result.error_messages = __pyx_state[10]; __pyx_result.field_name = __pyx_state[11]; __pyx_result.help_text = __pyx_state[12]; __pyx_result.initial = __pyx_state[13]; __pyx_result.label = __pyx_state[14]; __pyx_result.read_only = __pyx_state[15]; __pyx_result.required = __pyx_state[16]; __pyx_result.root = __pyx_state[17]; __pyx_result.style = __pyx_state[18]; __pyx_result.validators = __pyx_state[19]; __pyx_result.write_only = __pyx_state[20]
 *     if len(__pyx_state) > 21 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[21])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_MultipleChoiceField__set_state(<MultipleChoiceField> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MultipleChoiceField__set_state(MultipleChoiceField __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.allow_blank = __pyx_state[0]; __pyx_result.allow_empty = __pyx_state[1]; __pyx_result.allow_null = __pyx_state[2]; __pyx_result.attr = __pyx_state[3]; __pyx_result.attrs = __pyx_state[4]; __pyx_result.call = __pyx_state[5]; __pyx_result.choice_strings_to_display = __pyx_state[6]; __pyx_result.choice_strings_to_values = __pyx_state[7]; __pyx_result.choices = __pyx_state[8]; __pyx_result.default_value = __pyx_state[9]; __pyx_result.error_messages = __pyx_state[10]; __pyx_result.field_name = __pyx_state[11]; __pyx_result.help_text = __pyx_state[12]; __pyx_result.initial = __pyx_state[13]; __pyx_result.label = __pyx_state[14]; __pyx_result.read_only = __pyx_state[15]; __pyx_result.required = __pyx_state[16]; __pyx_result.root = __pyx_state[17]; __pyx_result.style = __pyx_state[18]; __pyx_result.validators = __pyx_state[19]; __pyx_result.write_only = __pyx_state[20]
 *     if len(__pyx_state) > 21 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_MultipleChoiceField__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_DateTimeField(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_33__pyx_unpickle_DateTimeField(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_9drf_turbo_6fields_33__pyx_unpickle_DateTimeField = {"__pyx_unpickle_DateTimeField", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_33__pyx_unpickle_DateTimeField, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_9drf_turbo_6fields_33__pyx_unpickle_DateTimeField(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_DateTimeField (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_DateTimeField", 1, 3, 3, 1); __PYX_ERR(1, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_DateTimeField", 1, 3, 3, 2); __PYX_ERR(1, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_DateTimeField") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_DateTimeField", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_DateTimeField", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9drf_turbo_6fields_32__pyx_unpickle_DateTimeField(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_32__pyx_unpickle_DateTimeField(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_DateTimeField", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x5e76b7e:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x5e76b7e = (allow_null, attr, attrs, call, default_timezone, default_value, error_messages, field_name, format, help_text, initial, input_formats, label, read_only, required, root, style, timezone, validators, write_only))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x5e76b7e) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x5e76b7e:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x5e76b7e = (allow_null, attr, attrs, call, default_timezone, default_value, error_messages, field_name, format, help_text, initial, input_formats, label, read_only, required, root, style, timezone, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = DateTimeField.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0x5e76b7e:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x5e76b7e = (allow_null, attr, attrs, call, default_timezone, default_value, error_messages, field_name, format, help_text, initial, input_formats, label, read_only, required, root, style, timezone, validators, write_only))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = DateTimeField.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x5e, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(1, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x5e76b7e:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x5e76b7e = (allow_null, attr, attrs, call, default_timezone, default_value, error_messages, field_name, format, help_text, initial, input_formats, label, read_only, required, root, style, timezone, validators, write_only))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x5e76b7e = (allow_null, attr, attrs, call, default_timezone, default_value, error_messages, field_name, format, help_text, initial, input_formats, label, read_only, required, root, style, timezone, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = DateTimeField.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_DateTimeField__set_state(<DateTimeField> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_9drf_turbo_6fields_DateTimeField), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x5e76b7e = (allow_null, attr, attrs, call, default_timezone, default_value, error_messages, field_name, format, help_text, initial, input_formats, label, read_only, required, root, style, timezone, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = DateTimeField.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_DateTimeField__set_state(<DateTimeField> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = DateTimeField.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_DateTimeField__set_state(<DateTimeField> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_DateTimeField__set_state(DateTimeField __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_9drf_turbo_6fields___pyx_unpickle_DateTimeField__set_state(((struct __pyx_obj_9drf_turbo_6fields_DateTimeField *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x5e76b7e = (allow_null, attr, attrs, call, default_timezone, default_value, error_messages, field_name, format, help_text, initial, input_formats, label, read_only, required, root, style, timezone, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = DateTimeField.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_DateTimeField__set_state(<DateTimeField> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_DateTimeField__set_state(<DateTimeField> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_DateTimeField__set_state(DateTimeField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.default_timezone = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.format = __pyx_state[8]; __pyx_result.help_text = __pyx_state[9]; __pyx_result.initial = __pyx_state[10]; __pyx_result.input_formats = __pyx_state[11]; __pyx_result.label = __pyx_state[12]; __pyx_result.read_only = __pyx_state[13]; __pyx_result.required = __pyx_state[14]; __pyx_result.root = __pyx_state[15]; __pyx_result.style = __pyx_state[16]; __pyx_result.timezone = __pyx_state[17]; __pyx_result.validators = __pyx_state[18]; __pyx_result.write_only = __pyx_state[19]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_DateTimeField(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_DateTimeField", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_DateTimeField__set_state(<DateTimeField> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_DateTimeField__set_state(DateTimeField __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.default_timezone = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.format = __pyx_state[8]; __pyx_result.help_text = __pyx_state[9]; __pyx_result.initial = __pyx_state[10]; __pyx_result.input_formats = __pyx_state[11]; __pyx_result.label = __pyx_state[12]; __pyx_result.read_only = __pyx_state[13]; __pyx_result.required = __pyx_state[14]; __pyx_result.root = __pyx_state[15]; __pyx_result.style = __pyx_state[16]; __pyx_result.timezone = __pyx_state[17]; __pyx_result.validators = __pyx_state[18]; __pyx_result.write_only = __pyx_state[19]
 *     if len(__pyx_state) > 20 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_9drf_turbo_6fields___pyx_unpickle_DateTimeField__set_state(struct __pyx_obj_9drf_turbo_6fields_DateTimeField *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_DateTimeField__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_DateTimeField__set_state(DateTimeField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.default_timezone = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.format = __pyx_state[8]; __pyx_result.help_text = __pyx_state[9]; __pyx_result.initial = __pyx_state[10]; __pyx_result.input_formats = __pyx_state[11]; __pyx_result.label = __pyx_state[12]; __pyx_result.read_only = __pyx_state[13]; __pyx_result.required = __pyx_state[14]; __pyx_result.root = __pyx_state[15]; __pyx_result.style = __pyx_state[16]; __pyx_result.timezone = __pyx_state[17]; __pyx_result.validators = __pyx_state[18]; __pyx_result.write_only = __pyx_state[19]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 20 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[20])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.allow_null = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.attr);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.attr);
  __pyx_v___pyx_result->__pyx_base.attr = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.attrs);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.attrs);
  __pyx_v___pyx_result->__pyx_base.attrs = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.call = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->default_timezone);
  __Pyx_DECREF(__pyx_v___pyx_result->default_timezone);
  __pyx_v___pyx_result->default_timezone = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.default_value);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.default_value);
  __pyx_v___pyx_result->__pyx_base.default_value = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.error_messages);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.error_messages);
  __pyx_v___pyx_result->__pyx_base.error_messages = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.field_name);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.field_name);
  __pyx_v___pyx_result->__pyx_base.field_name = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 8, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->format);
  __Pyx_DECREF(__pyx_v___pyx_result->format);
  __pyx_v___pyx_result->format = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 9, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.help_text);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.help_text);
  __pyx_v___pyx_result->__pyx_base.help_text = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 10, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.initial);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.initial);
  __pyx_v___pyx_result->__pyx_base.initial = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 11, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->input_formats);
  __Pyx_DECREF(__pyx_v___pyx_result->input_formats);
  __pyx_v___pyx_result->input_formats = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 12, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.label);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.label);
  __pyx_v___pyx_result->__pyx_base.label = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 13, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.read_only = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 14, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.required = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 15, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.root);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.root);
  __pyx_v___pyx_result->__pyx_base.root = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 16, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.style);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.style);
  __pyx_v___pyx_result->__pyx_base.style = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 17, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->timezone);
  __Pyx_DECREF(__pyx_v___pyx_result->timezone);
  __pyx_v___pyx_result->timezone = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 18, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.validators);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.validators);
  __pyx_v___pyx_result->__pyx_base.validators = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 19, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.write_only = __pyx_t_2;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_DateTimeField__set_state(DateTimeField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.default_timezone = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.format = __pyx_state[8]; __pyx_result.help_text = __pyx_state[9]; __pyx_result.initial = __pyx_state[10]; __pyx_result.input_formats = __pyx_state[11]; __pyx_result.label = __pyx_state[12]; __pyx_result.read_only = __pyx_state[13]; __pyx_result.required = __pyx_state[14]; __pyx_result.root = __pyx_state[15]; __pyx_result.style = __pyx_state[16]; __pyx_result.timezone = __pyx_state[17]; __pyx_result.validators = __pyx_state[18]; __pyx_result.write_only = __pyx_state[19]
 *     if len(__pyx_state) > 20 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[20])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 13, __pyx_L1_error)
  }
  __pyx_t_3 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_4 = ((__pyx_t_3 > 20) != 0);
  if (__pyx_t_4) {
  } else {
    __pyx_t_2 = __pyx_t_4;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_4 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_5 = (__pyx_t_4 != 0);
  __pyx_t_2 = __pyx_t_5;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":14
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.default_timezone = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.format = __pyx_state[8]; __pyx_result.help_text = __pyx_state[9]; __pyx_result.initial = __pyx_state[10]; __pyx_result.input_formats = __pyx_state[11]; __pyx_result.label = __pyx_state[12]; __pyx_result.read_only = __pyx_state[13]; __pyx_result.required = __pyx_state[14]; __pyx_result.root = __pyx_state[15]; __pyx_result.style = __pyx_state[16]; __pyx_result.timezone = __pyx_state[17]; __pyx_result.validators = __pyx_state[18]; __pyx_result.write_only = __pyx_state[19]
 *     if len(__pyx_state) > 20 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[20])             # <<<<<<<<<<<<<<
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_update); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 14, __pyx_L1_error)
    }
    __pyx_t_6 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 20, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_8 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
      }
    }
    __pyx_t_1 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_8, __pyx_t_6) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_6);
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_DateTimeField__set_state(DateTimeField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.default_timezone = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.format = __pyx_state[8]; __pyx_result.help_text = __pyx_state[9]; __pyx_result.initial = __pyx_state[10]; __pyx_result.input_formats = __pyx_state[11]; __pyx_result.label = __pyx_state[12]; __pyx_result.read_only = __pyx_state[13]; __pyx_result.required = __pyx_state[14]; __pyx_result.root = __pyx_state[15]; __pyx_result.style = __pyx_state[16]; __pyx_result.timezone = __pyx_state[17]; __pyx_result.validators = __pyx_state[18]; __pyx_result.write_only = __pyx_state[19]
 *     if len(__pyx_state) > 20 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[20])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_DateTimeField__set_state(<DateTimeField> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_DateTimeField__set_state(DateTimeField __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.default_timezone = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.format = __pyx_state[8]; __pyx_result.help_text = __pyx_state[9]; __pyx_result.initial = __pyx_state[10]; __pyx_result.input_formats = __pyx_state[11]; __pyx_result.label = __pyx_state[12]; __pyx_result.read_only = __pyx_state[13]; __pyx_result.required = __pyx_state[14]; __pyx_result.root = __pyx_state[15]; __pyx_result.style = __pyx_state[16]; __pyx_result.timezone = __pyx_state[17]; __pyx_result.validators = __pyx_state[18]; __pyx_result.write_only = __pyx_state[19]
 *     if len(__pyx_state) > 20 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_DateTimeField__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_DateField(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_35__pyx_unpickle_DateField(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_9drf_turbo_6fields_35__pyx_unpickle_DateField = {"__pyx_unpickle_DateField", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_35__pyx_unpickle_DateField, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_9drf_turbo_6fields_35__pyx_unpickle_DateField(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_DateField (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_DateField", 1, 3, 3, 1); __PYX_ERR(1, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_DateField", 1, 3, 3, 2); __PYX_ERR(1, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_DateField") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_DateField", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_DateField", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9drf_turbo_6fields_34__pyx_unpickle_DateField(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_34__pyx_unpickle_DateField(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_DateField", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xf36a8d7:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xf36a8d7 = (allow_null, attr, attrs, call, default_value, error_messages, field_name, format, help_text, initial, input_formats, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0xf36a8d7) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xf36a8d7:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xf36a8d7 = (allow_null, attr, attrs, call, default_value, error_messages, field_name, format, help_text, initial, input_formats, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = DateField.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0xf36a8d7:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xf36a8d7 = (allow_null, attr, attrs, call, default_value, error_messages, field_name, format, help_text, initial, input_formats, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = DateField.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0xf3, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(1, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xf36a8d7:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xf36a8d7 = (allow_null, attr, attrs, call, default_value, error_messages, field_name, format, help_text, initial, input_formats, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xf36a8d7 = (allow_null, attr, attrs, call, default_value, error_messages, field_name, format, help_text, initial, input_formats, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = DateField.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_DateField__set_state(<DateField> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_9drf_turbo_6fields_DateField), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xf36a8d7 = (allow_null, attr, attrs, call, default_value, error_messages, field_name, format, help_text, initial, input_formats, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = DateField.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_DateField__set_state(<DateField> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = DateField.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_DateField__set_state(<DateField> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_DateField__set_state(DateField __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_9drf_turbo_6fields___pyx_unpickle_DateField__set_state(((struct __pyx_obj_9drf_turbo_6fields_DateField *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xf36a8d7 = (allow_null, attr, attrs, call, default_value, error_messages, field_name, format, help_text, initial, input_formats, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = DateField.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_DateField__set_state(<DateField> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_DateField__set_state(<DateField> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_DateField__set_state(DateField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.default_value = __pyx_state[4]; __pyx_result.error_messages = __pyx_state[5]; __pyx_result.field_name = __pyx_state[6]; __pyx_result.format = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.input_formats = __pyx_state[10]; __pyx_result.label = __pyx_state[11]; __pyx_result.read_only = __pyx_state[12]; __pyx_result.required = __pyx_state[13]; __pyx_result.root = __pyx_state[14]; __pyx_result.style = __pyx_state[15]; __pyx_result.validators = __pyx_state[16]; __pyx_result.write_only = __pyx_state[17]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_DateField(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_DateField", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_DateField__set_state(<DateField> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_DateField__set_state(DateField __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.default_value = __pyx_state[4]; __pyx_result.error_messages = __pyx_state[5]; __pyx_result.field_name = __pyx_state[6]; __pyx_result.format = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.input_formats = __pyx_state[10]; __pyx_result.label = __pyx_state[11]; __pyx_result.read_only = __pyx_state[12]; __pyx_result.required = __pyx_state[13]; __pyx_result.root = __pyx_state[14]; __pyx_result.style = __pyx_state[15]; __pyx_result.validators = __pyx_state[16]; __pyx_result.write_only = __pyx_state[17]
 *     if len(__pyx_state) > 18 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_9drf_turbo_6fields___pyx_unpickle_DateField__set_state(struct __pyx_obj_9drf_turbo_6fields_DateField *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_DateField__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_DateField__set_state(DateField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.default_value = __pyx_state[4]; __pyx_result.error_messages = __pyx_state[5]; __pyx_result.field_name = __pyx_state[6]; __pyx_result.format = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.input_formats = __pyx_state[10]; __pyx_result.label = __pyx_state[11]; __pyx_result.read_only = __pyx_state[12]; __pyx_result.required = __pyx_state[13]; __pyx_result.root = __pyx_state[14]; __pyx_result.style = __pyx_state[15]; __pyx_result.validators = __pyx_state[16]; __pyx_result.write_only = __pyx_state[17]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 18 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[18])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.allow_null = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.attr);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.attr);
  __pyx_v___pyx_result->__pyx_base.attr = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.attrs);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.attrs);
  __pyx_v___pyx_result->__pyx_base.attrs = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.call = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.default_value);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.default_value);
  __pyx_v___pyx_result->__pyx_base.default_value = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.error_messages);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.error_messages);
  __pyx_v___pyx_result->__pyx_base.error_messages = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.field_name);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.field_name);
  __pyx_v___pyx_result->__pyx_base.field_name = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->format);
  __Pyx_DECREF(__pyx_v___pyx_result->format);
  __pyx_v___pyx_result->format = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 8, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.help_text);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.help_text);
  __pyx_v___pyx_result->__pyx_base.help_text = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 9, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.initial);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.initial);
  __pyx_v___pyx_result->__pyx_base.initial = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 10, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->input_formats);
  __Pyx_DECREF(__pyx_v___pyx_result->input_formats);
  __pyx_v___pyx_result->input_formats = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 11, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.label);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.label);
  __pyx_v___pyx_result->__pyx_base.label = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 12, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.read_only = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 13, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.required = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 14, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.root);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.root);
  __pyx_v___pyx_result->__pyx_base.root = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 15, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.style);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.style);
  __pyx_v___pyx_result->__pyx_base.style = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 16, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.validators);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.validators);
  __pyx_v___pyx_result->__pyx_base.validators = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 17, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.write_only = __pyx_t_2;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_DateField__set_state(DateField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.default_value = __pyx_state[4]; __pyx_result.error_messages = __pyx_state[5]; __pyx_result.field_name = __pyx_state[6]; __pyx_result.format = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.input_formats = __pyx_state[10]; __pyx_result.label = __pyx_state[11]; __pyx_result.read_only = __pyx_state[12]; __pyx_result.required = __pyx_state[13]; __pyx_result.root = __pyx_state[14]; __pyx_result.style = __pyx_state[15]; __pyx_result.validators = __pyx_state[16]; __pyx_result.write_only = __pyx_state[17]
 *     if len(__pyx_state) > 18 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[18])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 13, __pyx_L1_error)
  }
  __pyx_t_3 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_4 = ((__pyx_t_3 > 18) != 0);
  if (__pyx_t_4) {
  } else {
    __pyx_t_2 = __pyx_t_4;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_4 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_5 = (__pyx_t_4 != 0);
  __pyx_t_2 = __pyx_t_5;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":14
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.default_value = __pyx_state[4]; __pyx_result.error_messages = __pyx_state[5]; __pyx_result.field_name = __pyx_state[6]; __pyx_result.format = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.input_formats = __pyx_state[10]; __pyx_result.label = __pyx_state[11]; __pyx_result.read_only = __pyx_state[12]; __pyx_result.required = __pyx_state[13]; __pyx_result.root = __pyx_state[14]; __pyx_result.style = __pyx_state[15]; __pyx_result.validators = __pyx_state[16]; __pyx_result.write_only = __pyx_state[17]
 *     if len(__pyx_state) > 18 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[18])             # <<<<<<<<<<<<<<
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_update); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 14, __pyx_L1_error)
    }
    __pyx_t_6 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 18, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_8 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
      }
    }
    __pyx_t_1 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_8, __pyx_t_6) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_6);
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_DateField__set_state(DateField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.default_value = __pyx_state[4]; __pyx_result.error_messages = __pyx_state[5]; __pyx_result.field_name = __pyx_state[6]; __pyx_result.format = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.input_formats = __pyx_state[10]; __pyx_result.label = __pyx_state[11]; __pyx_result.read_only = __pyx_state[12]; __pyx_result.required = __pyx_state[13]; __pyx_result.root = __pyx_state[14]; __pyx_result.style = __pyx_state[15]; __pyx_result.validators = __pyx_state[16]; __pyx_result.write_only = __pyx_state[17]
 *     if len(__pyx_state) > 18 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[18])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_DateField__set_state(<DateField> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_DateField__set_state(DateField __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.default_value = __pyx_state[4]; __pyx_result.error_messages = __pyx_state[5]; __pyx_result.field_name = __pyx_state[6]; __pyx_result.format = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.input_formats = __pyx_state[10]; __pyx_result.label = __pyx_state[11]; __pyx_result.read_only = __pyx_state[12]; __pyx_result.required = __pyx_state[13]; __pyx_result.root = __pyx_state[14]; __pyx_result.style = __pyx_state[15]; __pyx_result.validators = __pyx_state[16]; __pyx_result.write_only = __pyx_state[17]
 *     if len(__pyx_state) > 18 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_DateField__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_TimeField(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_37__pyx_unpickle_TimeField(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_9drf_turbo_6fields_37__pyx_unpickle_TimeField = {"__pyx_unpickle_TimeField", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_37__pyx_unpickle_TimeField, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_9drf_turbo_6fields_37__pyx_unpickle_TimeField(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_TimeField (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_TimeField", 1, 3, 3, 1); __PYX_ERR(1, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_TimeField", 1, 3, 3, 2); __PYX_ERR(1, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_TimeField") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_TimeField", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_TimeField", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9drf_turbo_6fields_36__pyx_unpickle_TimeField(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_36__pyx_unpickle_TimeField(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_TimeField", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xf36a8d7:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xf36a8d7 = (allow_null, attr, attrs, call, default_value, error_messages, field_name, format, help_text, initial, input_formats, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0xf36a8d7) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xf36a8d7:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xf36a8d7 = (allow_null, attr, attrs, call, default_value, error_messages, field_name, format, help_text, initial, input_formats, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = TimeField.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0xf36a8d7:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xf36a8d7 = (allow_null, attr, attrs, call, default_value, error_messages, field_name, format, help_text, initial, input_formats, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = TimeField.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0xf3, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(1, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xf36a8d7:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xf36a8d7 = (allow_null, attr, attrs, call, default_value, error_messages, field_name, format, help_text, initial, input_formats, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xf36a8d7 = (allow_null, attr, attrs, call, default_value, error_messages, field_name, format, help_text, initial, input_formats, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = TimeField.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_TimeField__set_state(<TimeField> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_9drf_turbo_6fields_TimeField), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xf36a8d7 = (allow_null, attr, attrs, call, default_value, error_messages, field_name, format, help_text, initial, input_formats, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = TimeField.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_TimeField__set_state(<TimeField> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = TimeField.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_TimeField__set_state(<TimeField> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_TimeField__set_state(TimeField __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_9drf_turbo_6fields___pyx_unpickle_TimeField__set_state(((struct __pyx_obj_9drf_turbo_6fields_TimeField *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xf36a8d7 = (allow_null, attr, attrs, call, default_value, error_messages, field_name, format, help_text, initial, input_formats, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = TimeField.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_TimeField__set_state(<TimeField> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_TimeField__set_state(<TimeField> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_TimeField__set_state(TimeField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.default_value = __pyx_state[4]; __pyx_result.error_messages = __pyx_state[5]; __pyx_result.field_name = __pyx_state[6]; __pyx_result.format = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.input_formats = __pyx_state[10]; __pyx_result.label = __pyx_state[11]; __pyx_result.read_only = __pyx_state[12]; __pyx_result.required = __pyx_state[13]; __pyx_result.root = __pyx_state[14]; __pyx_result.style = __pyx_state[15]; __pyx_result.validators = __pyx_state[16]; __pyx_result.write_only = __pyx_state[17]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_TimeField(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_TimeField", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_TimeField__set_state(<TimeField> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_TimeField__set_state(TimeField __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.default_value = __pyx_state[4]; __pyx_result.error_messages = __pyx_state[5]; __pyx_result.field_name = __pyx_state[6]; __pyx_result.format = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.input_formats = __pyx_state[10]; __pyx_result.label = __pyx_state[11]; __pyx_result.read_only = __pyx_state[12]; __pyx_result.required = __pyx_state[13]; __pyx_result.root = __pyx_state[14]; __pyx_result.style = __pyx_state[15]; __pyx_result.validators = __pyx_state[16]; __pyx_result.write_only = __pyx_state[17]
 *     if len(__pyx_state) > 18 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_9drf_turbo_6fields___pyx_unpickle_TimeField__set_state(struct __pyx_obj_9drf_turbo_6fields_TimeField *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_TimeField__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_TimeField__set_state(TimeField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.default_value = __pyx_state[4]; __pyx_result.error_messages = __pyx_state[5]; __pyx_result.field_name = __pyx_state[6]; __pyx_result.format = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.input_formats = __pyx_state[10]; __pyx_result.label = __pyx_state[11]; __pyx_result.read_only = __pyx_state[12]; __pyx_result.required = __pyx_state[13]; __pyx_result.root = __pyx_state[14]; __pyx_result.style = __pyx_state[15]; __pyx_result.validators = __pyx_state[16]; __pyx_result.write_only = __pyx_state[17]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 18 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[18])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.allow_null = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.attr);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.attr);
  __pyx_v___pyx_result->__pyx_base.attr = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.attrs);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.attrs);
  __pyx_v___pyx_result->__pyx_base.attrs = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.call = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.default_value);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.default_value);
  __pyx_v___pyx_result->__pyx_base.default_value = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.error_messages);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.error_messages);
  __pyx_v___pyx_result->__pyx_base.error_messages = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.field_name);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.field_name);
  __pyx_v___pyx_result->__pyx_base.field_name = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->format);
  __Pyx_DECREF(__pyx_v___pyx_result->format);
  __pyx_v___pyx_result->format = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 8, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.help_text);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.help_text);
  __pyx_v___pyx_result->__pyx_base.help_text = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 9, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.initial);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.initial);
  __pyx_v___pyx_result->__pyx_base.initial = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 10, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->input_formats);
  __Pyx_DECREF(__pyx_v___pyx_result->input_formats);
  __pyx_v___pyx_result->input_formats = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 11, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.label);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.label);
  __pyx_v___pyx_result->__pyx_base.label = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 12, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.read_only = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 13, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.required = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 14, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.root);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.root);
  __pyx_v___pyx_result->__pyx_base.root = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 15, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.style);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.style);
  __pyx_v___pyx_result->__pyx_base.style = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 16, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.validators);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.validators);
  __pyx_v___pyx_result->__pyx_base.validators = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 17, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.write_only = __pyx_t_2;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_TimeField__set_state(TimeField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.default_value = __pyx_state[4]; __pyx_result.error_messages = __pyx_state[5]; __pyx_result.field_name = __pyx_state[6]; __pyx_result.format = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.input_formats = __pyx_state[10]; __pyx_result.label = __pyx_state[11]; __pyx_result.read_only = __pyx_state[12]; __pyx_result.required = __pyx_state[13]; __pyx_result.root = __pyx_state[14]; __pyx_result.style = __pyx_state[15]; __pyx_result.validators = __pyx_state[16]; __pyx_result.write_only = __pyx_state[17]
 *     if len(__pyx_state) > 18 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[18])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 13, __pyx_L1_error)
  }
  __pyx_t_3 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_4 = ((__pyx_t_3 > 18) != 0);
  if (__pyx_t_4) {
  } else {
    __pyx_t_2 = __pyx_t_4;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_4 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_5 = (__pyx_t_4 != 0);
  __pyx_t_2 = __pyx_t_5;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":14
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.default_value = __pyx_state[4]; __pyx_result.error_messages = __pyx_state[5]; __pyx_result.field_name = __pyx_state[6]; __pyx_result.format = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.input_formats = __pyx_state[10]; __pyx_result.label = __pyx_state[11]; __pyx_result.read_only = __pyx_state[12]; __pyx_result.required = __pyx_state[13]; __pyx_result.root = __pyx_state[14]; __pyx_result.style = __pyx_state[15]; __pyx_result.validators = __pyx_state[16]; __pyx_result.write_only = __pyx_state[17]
 *     if len(__pyx_state) > 18 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[18])             # <<<<<<<<<<<<<<
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_update); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 14, __pyx_L1_error)
    }
    __pyx_t_6 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 18, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_8 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
      }
    }
    __pyx_t_1 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_8, __pyx_t_6) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_6);
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_TimeField__set_state(TimeField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.default_value = __pyx_state[4]; __pyx_result.error_messages = __pyx_state[5]; __pyx_result.field_name = __pyx_state[6]; __pyx_result.format = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.input_formats = __pyx_state[10]; __pyx_result.label = __pyx_state[11]; __pyx_result.read_only = __pyx_state[12]; __pyx_result.required = __pyx_state[13]; __pyx_result.root = __pyx_state[14]; __pyx_result.style = __pyx_state[15]; __pyx_result.validators = __pyx_state[16]; __pyx_result.write_only = __pyx_state[17]
 *     if len(__pyx_state) > 18 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[18])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_TimeField__set_state(<TimeField> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_TimeField__set_state(TimeField __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.default_value = __pyx_state[4]; __pyx_result.error_messages = __pyx_state[5]; __pyx_result.field_name = __pyx_state[6]; __pyx_result.format = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.input_formats = __pyx_state[10]; __pyx_result.label = __pyx_state[11]; __pyx_result.read_only = __pyx_state[12]; __pyx_result.required = __pyx_state[13]; __pyx_result.root = __pyx_state[14]; __pyx_result.style = __pyx_state[15]; __pyx_result.validators = __pyx_state[16]; __pyx_result.write_only = __pyx_state[17]
 *     if len(__pyx_state) > 18 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_TimeField__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_FileField(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_39__pyx_unpickle_FileField(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_9drf_turbo_6fields_39__pyx_unpickle_FileField = {"__pyx_unpickle_FileField", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_39__pyx_unpickle_FileField, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_9drf_turbo_6fields_39__pyx_unpickle_FileField(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_FileField (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_FileField", 1, 3, 3, 1); __PYX_ERR(1, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_FileField", 1, 3, 3, 2); __PYX_ERR(1, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_FileField") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_FileField", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_FileField", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9drf_turbo_6fields_38__pyx_unpickle_FileField(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_38__pyx_unpickle_FileField(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_FileField", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xe82609b:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xe82609b = (allow_empty_file, allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_length, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0xe82609b) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xe82609b:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xe82609b = (allow_empty_file, allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_length, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = FileField.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0xe82609b:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xe82609b = (allow_empty_file, allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_length, read_only, required, root, style, validators, write_only))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = FileField.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0xe8, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(1, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xe82609b:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xe82609b = (allow_empty_file, allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_length, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xe82609b = (allow_empty_file, allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_length, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = FileField.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_FileField__set_state(<FileField> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_9drf_turbo_6fields_FileField), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xe82609b = (allow_empty_file, allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_length, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = FileField.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_FileField__set_state(<FileField> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = FileField.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_FileField__set_state(<FileField> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_FileField__set_state(FileField __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_9drf_turbo_6fields___pyx_unpickle_FileField__set_state(((struct __pyx_obj_9drf_turbo_6fields_FileField *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xe82609b = (allow_empty_file, allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, max_length, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = FileField.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_FileField__set_state(<FileField> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_FileField__set_state(<FileField> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_FileField__set_state(FileField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_empty_file = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.max_length = __pyx_state[11]; __pyx_result.read_only = __pyx_state[12]; __pyx_result.required = __pyx_state[13]; __pyx_result.root = __pyx_state[14]; __pyx_result.style = __pyx_state[15]; __pyx_result.validators = __pyx_state[16]; __pyx_result.write_only = __pyx_state[17]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_FileField(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_FileField", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_FileField__set_state(<FileField> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_FileField__set_state(FileField __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.allow_empty_file = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.max_length = __pyx_state[11]; __pyx_result.read_only = __pyx_state[12]; __pyx_result.required = __pyx_state[13]; __pyx_result.root = __pyx_state[14]; __pyx_result.style = __pyx_state[15]; __pyx_result.validators = __pyx_state[16]; __pyx_result.write_only = __pyx_state[17]
 *     if len(__pyx_state) > 18 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_9drf_turbo_6fields___pyx_unpickle_FileField__set_state(struct __pyx_obj_9drf_turbo_6fields_FileField *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_FileField__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_FileField__set_state(FileField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_empty_file = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.max_length = __pyx_state[11]; __pyx_result.read_only = __pyx_state[12]; __pyx_result.required = __pyx_state[13]; __pyx_result.root = __pyx_state[14]; __pyx_result.style = __pyx_state[15]; __pyx_result.validators = __pyx_state[16]; __pyx_result.write_only = __pyx_state[17]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 18 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[18])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->allow_empty_file);
  __Pyx_DECREF(__pyx_v___pyx_result->allow_empty_file);
  __pyx_v___pyx_result->allow_empty_file = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.allow_null = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.attr);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.attr);
  __pyx_v___pyx_result->__pyx_base.attr = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.attrs);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.attrs);
  __pyx_v___pyx_result->__pyx_base.attrs = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.call = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.default_value);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.default_value);
  __pyx_v___pyx_result->__pyx_base.default_value = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.error_messages);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.error_messages);
  __pyx_v___pyx_result->__pyx_base.error_messages = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.field_name);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.field_name);
  __pyx_v___pyx_result->__pyx_base.field_name = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 8, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.help_text);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.help_text);
  __pyx_v___pyx_result->__pyx_base.help_text = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 9, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.initial);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.initial);
  __pyx_v___pyx_result->__pyx_base.initial = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 10, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.label);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.label);
  __pyx_v___pyx_result->__pyx_base.label = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 11, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->max_length);
  __Pyx_DECREF(__pyx_v___pyx_result->max_length);
  __pyx_v___pyx_result->max_length = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 12, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.read_only = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 13, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.required = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 14, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.root);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.root);
  __pyx_v___pyx_result->__pyx_base.root = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 15, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.style);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.style);
  __pyx_v___pyx_result->__pyx_base.style = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 16, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.validators);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.validators);
  __pyx_v___pyx_result->__pyx_base.validators = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 17, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.write_only = __pyx_t_2;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_FileField__set_state(FileField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_empty_file = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.max_length = __pyx_state[11]; __pyx_result.read_only = __pyx_state[12]; __pyx_result.required = __pyx_state[13]; __pyx_result.root = __pyx_state[14]; __pyx_result.style = __pyx_state[15]; __pyx_result.validators = __pyx_state[16]; __pyx_result.write_only = __pyx_state[17]
 *     if len(__pyx_state) > 18 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[18])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 13, __pyx_L1_error)
  }
  __pyx_t_3 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_4 = ((__pyx_t_3 > 18) != 0);
  if (__pyx_t_4) {
  } else {
    __pyx_t_2 = __pyx_t_4;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_4 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_5 = (__pyx_t_4 != 0);
  __pyx_t_2 = __pyx_t_5;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":14
 *     __pyx_result.allow_empty_file = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.max_length = __pyx_state[11]; __pyx_result.read_only = __pyx_state[12]; __pyx_result.required = __pyx_state[13]; __pyx_result.root = __pyx_state[14]; __pyx_result.style = __pyx_state[15]; __pyx_result.validators = __pyx_state[16]; __pyx_result.write_only = __pyx_state[17]
 *     if len(__pyx_state) > 18 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[18])             # <<<<<<<<<<<<<<
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_update); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 14, __pyx_L1_error)
    }
    __pyx_t_6 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 18, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_8 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
      }
    }
    __pyx_t_1 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_8, __pyx_t_6) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_6);
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_FileField__set_state(FileField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_empty_file = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.max_length = __pyx_state[11]; __pyx_result.read_only = __pyx_state[12]; __pyx_result.required = __pyx_state[13]; __pyx_result.root = __pyx_state[14]; __pyx_result.style = __pyx_state[15]; __pyx_result.validators = __pyx_state[16]; __pyx_result.write_only = __pyx_state[17]
 *     if len(__pyx_state) > 18 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[18])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_FileField__set_state(<FileField> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_FileField__set_state(FileField __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.allow_empty_file = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.max_length = __pyx_state[11]; __pyx_result.read_only = __pyx_state[12]; __pyx_result.required = __pyx_state[13]; __pyx_result.root = __pyx_state[14]; __pyx_result.style = __pyx_state[15]; __pyx_result.validators = __pyx_state[16]; __pyx_result.write_only = __pyx_state[17]
 *     if len(__pyx_state) > 18 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_FileField__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_ArrayField(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_41__pyx_unpickle_ArrayField(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_9drf_turbo_6fields_41__pyx_unpickle_ArrayField = {"__pyx_unpickle_ArrayField", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_41__pyx_unpickle_ArrayField, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_9drf_turbo_6fields_41__pyx_unpickle_ArrayField(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_ArrayField (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_ArrayField", 1, 3, 3, 1); __PYX_ERR(1, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_ArrayField", 1, 3, 3, 2); __PYX_ERR(1, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_ArrayField") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_ArrayField", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_ArrayField", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9drf_turbo_6fields_40__pyx_unpickle_ArrayField(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_40__pyx_unpickle_ArrayField(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_ArrayField", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x5775012:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x5775012 = (allow_empty, allow_null, attr, attrs, call, child, default_value, error_messages, exact_items, field_name, help_text, initial, label, max_items, min_items, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x5775012) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x5775012:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x5775012 = (allow_empty, allow_null, attr, attrs, call, child, default_value, error_messages, exact_items, field_name, help_text, initial, label, max_items, min_items, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = ArrayField.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0x5775012:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x5775012 = (allow_empty, allow_null, attr, attrs, call, child, default_value, error_messages, exact_items, field_name, help_text, initial, label, max_items, min_items, read_only, required, root, style, validators, write_only))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = ArrayField.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x57, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(1, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x5775012:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x5775012 = (allow_empty, allow_null, attr, attrs, call, child, default_value, error_messages, exact_items, field_name, help_text, initial, label, max_items, min_items, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x5775012 = (allow_empty, allow_null, attr, attrs, call, child, default_value, error_messages, exact_items, field_name, help_text, initial, label, max_items, min_items, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = ArrayField.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_ArrayField__set_state(<ArrayField> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_9drf_turbo_6fields_ArrayField), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x5775012 = (allow_empty, allow_null, attr, attrs, call, child, default_value, error_messages, exact_items, field_name, help_text, initial, label, max_items, min_items, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = ArrayField.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_ArrayField__set_state(<ArrayField> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = ArrayField.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_ArrayField__set_state(<ArrayField> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_ArrayField__set_state(ArrayField __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_9drf_turbo_6fields___pyx_unpickle_ArrayField__set_state(((struct __pyx_obj_9drf_turbo_6fields_ArrayField *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x5775012 = (allow_empty, allow_null, attr, attrs, call, child, default_value, error_messages, exact_items, field_name, help_text, initial, label, max_items, min_items, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = ArrayField.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_ArrayField__set_state(<ArrayField> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_ArrayField__set_state(<ArrayField> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_ArrayField__set_state(ArrayField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_empty = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.child = __pyx_state[5]; __pyx_result.default_value = __pyx_state[6]; __pyx_result.error_messages = __pyx_state[7]; __pyx_result.exact_items = __pyx_state[8]; __pyx_result.field_name = __pyx_state[9]; __pyx_result.help_text = __pyx_state[10]; __pyx_result.initial = __pyx_state[11]; __pyx_result.label = __pyx_state[12]; __pyx_result.max_items = __pyx_state[13]; __pyx_result.min_items = __pyx_state[14]; __pyx_result.read_only = __pyx_state[15]; __pyx_result.required = __pyx_state[16]; __pyx_result.root = __pyx_state[17]; __pyx_result.style = __pyx_state[18]; __pyx_result.validators = __pyx_state[19]; __pyx_result.write_only = __pyx_state[20]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_ArrayField(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_ArrayField", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_ArrayField__set_state(<ArrayField> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_ArrayField__set_state(ArrayField __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.allow_empty = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.child = __pyx_state[5]; __pyx_result.default_value = __pyx_state[6]; __pyx_result.error_messages = __pyx_state[7]; __pyx_result.exact_items = __pyx_state[8]; __pyx_result.field_name = __pyx_state[9]; __pyx_result.help_text = __pyx_state[10]; __pyx_result.initial = __pyx_state[11]; __pyx_result.label = __pyx_state[12]; __pyx_result.max_items = __pyx_state[13]; __pyx_result.min_items = __pyx_state[14]; __pyx_result.read_only = __pyx_state[15]; __pyx_result.required = __pyx_state[16]; __pyx_result.root = __pyx_state[17]; __pyx_result.style = __pyx_state[18]; __pyx_result.validators = __pyx_state[19]; __pyx_result.write_only = __pyx_state[20]
 *     if len(__pyx_state) > 21 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_9drf_turbo_6fields___pyx_unpickle_ArrayField__set_state(struct __pyx_obj_9drf_turbo_6fields_ArrayField *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_ArrayField__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_ArrayField__set_state(ArrayField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_empty = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.child = __pyx_state[5]; __pyx_result.default_value = __pyx_state[6]; __pyx_result.error_messages = __pyx_state[7]; __pyx_result.exact_items = __pyx_state[8]; __pyx_result.field_name = __pyx_state[9]; __pyx_result.help_text = __pyx_state[10]; __pyx_result.initial = __pyx_state[11]; __pyx_result.label = __pyx_state[12]; __pyx_result.max_items = __pyx_state[13]; __pyx_result.min_items = __pyx_state[14]; __pyx_result.read_only = __pyx_state[15]; __pyx_result.required = __pyx_state[16]; __pyx_result.root = __pyx_state[17]; __pyx_result.style = __pyx_state[18]; __pyx_result.validators = __pyx_state[19]; __pyx_result.write_only = __pyx_state[20]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 21 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[21])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->allow_empty);
  __Pyx_DECREF(__pyx_v___pyx_result->allow_empty);
  __pyx_v___pyx_result->allow_empty = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.allow_null = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.attr);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.attr);
  __pyx_v___pyx_result->__pyx_base.attr = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.attrs);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.attrs);
  __pyx_v___pyx_result->__pyx_base.attrs = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.call = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->child);
  __Pyx_DECREF(__pyx_v___pyx_result->child);
  __pyx_v___pyx_result->child = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.default_value);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.default_value);
  __pyx_v___pyx_result->__pyx_base.default_value = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.error_messages);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.error_messages);
  __pyx_v___pyx_result->__pyx_base.error_messages = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 8, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->exact_items);
  __Pyx_DECREF(__pyx_v___pyx_result->exact_items);
  __pyx_v___pyx_result->exact_items = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 9, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.field_name);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.field_name);
  __pyx_v___pyx_result->__pyx_base.field_name = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 10, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.help_text);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.help_text);
  __pyx_v___pyx_result->__pyx_base.help_text = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 11, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.initial);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.initial);
  __pyx_v___pyx_result->__pyx_base.initial = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 12, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.label);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.label);
  __pyx_v___pyx_result->__pyx_base.label = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 13, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->max_items);
  __Pyx_DECREF(__pyx_v___pyx_result->max_items);
  __pyx_v___pyx_result->max_items = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 14, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->min_items);
  __Pyx_DECREF(__pyx_v___pyx_result->min_items);
  __pyx_v___pyx_result->min_items = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 15, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.read_only = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 16, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.required = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 17, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.root);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.root);
  __pyx_v___pyx_result->__pyx_base.root = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 18, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.style);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.style);
  __pyx_v___pyx_result->__pyx_base.style = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 19, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.validators);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.validators);
  __pyx_v___pyx_result->__pyx_base.validators = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 20, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.write_only = __pyx_t_2;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_ArrayField__set_state(ArrayField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_empty = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.child = __pyx_state[5]; __pyx_result.default_value = __pyx_state[6]; __pyx_result.error_messages = __pyx_state[7]; __pyx_result.exact_items = __pyx_state[8]; __pyx_result.field_name = __pyx_state[9]; __pyx_result.help_text = __pyx_state[10]; __pyx_result.initial = __pyx_state[11]; __pyx_result.label = __pyx_state[12]; __pyx_result.max_items = __pyx_state[13]; __pyx_result.min_items = __pyx_state[14]; __pyx_result.read_only = __pyx_state[15]; __pyx_result.required = __pyx_state[16]; __pyx_result.root = __pyx_state[17]; __pyx_result.style = __pyx_state[18]; __pyx_result.validators = __pyx_state[19]; __pyx_result.write_only = __pyx_state[20]
 *     if len(__pyx_state) > 21 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[21])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 13, __pyx_L1_error)
  }
  __pyx_t_3 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_4 = ((__pyx_t_3 > 21) != 0);
  if (__pyx_t_4) {
  } else {
    __pyx_t_2 = __pyx_t_4;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_4 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_5 = (__pyx_t_4 != 0);
  __pyx_t_2 = __pyx_t_5;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":14
 *     __pyx_result.allow_empty = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.child = __pyx_state[5]; __pyx_result.default_value = __pyx_state[6]; __pyx_result.error_messages = __pyx_state[7]; __pyx_result.exact_items = __pyx_state[8]; __pyx_result.field_name = __pyx_state[9]; __pyx_result.help_text = __pyx_state[10]; __pyx_result.initial = __pyx_state[11]; __pyx_result.label = __pyx_state[12]; __pyx_result.max_items = __pyx_state[13]; __pyx_result.min_items = __pyx_state[14]; __pyx_result.read_only = __pyx_state[15]; __pyx_result.required = __pyx_state[16]; __pyx_result.root = __pyx_state[17]; __pyx_result.style = __pyx_state[18]; __pyx_result.validators = __pyx_state[19]; __pyx_result.write_only = __pyx_state[20]
 *     if len(__pyx_state) > 21 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[21])             # <<<<<<<<<<<<<<
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_update); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 14, __pyx_L1_error)
    }
    __pyx_t_6 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 21, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_8 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
      }
    }
    __pyx_t_1 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_8, __pyx_t_6) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_6);
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_ArrayField__set_state(ArrayField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_empty = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.child = __pyx_state[5]; __pyx_result.default_value = __pyx_state[6]; __pyx_result.error_messages = __pyx_state[7]; __pyx_result.exact_items = __pyx_state[8]; __pyx_result.field_name = __pyx_state[9]; __pyx_result.help_text = __pyx_state[10]; __pyx_result.initial = __pyx_state[11]; __pyx_result.label = __pyx_state[12]; __pyx_result.max_items = __pyx_state[13]; __pyx_result.min_items = __pyx_state[14]; __pyx_result.read_only = __pyx_state[15]; __pyx_result.required = __pyx_state[16]; __pyx_result.root = __pyx_state[17]; __pyx_result.style = __pyx_state[18]; __pyx_result.validators = __pyx_state[19]; __pyx_result.write_only = __pyx_state[20]
 *     if len(__pyx_state) > 21 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[21])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_ArrayField__set_state(<ArrayField> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_ArrayField__set_state(ArrayField __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.allow_empty = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.child = __pyx_state[5]; __pyx_result.default_value = __pyx_state[6]; __pyx_result.error_messages = __pyx_state[7]; __pyx_result.exact_items = __pyx_state[8]; __pyx_result.field_name = __pyx_state[9]; __pyx_result.help_text = __pyx_state[10]; __pyx_result.initial = __pyx_state[11]; __pyx_result.label = __pyx_state[12]; __pyx_result.max_items = __pyx_state[13]; __pyx_result.min_items = __pyx_state[14]; __pyx_result.read_only = __pyx_state[15]; __pyx_result.required = __pyx_state[16]; __pyx_result.root = __pyx_state[17]; __pyx_result.style = __pyx_state[18]; __pyx_result.validators = __pyx_state[19]; __pyx_result.write_only = __pyx_state[20]
 *     if len(__pyx_state) > 21 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_ArrayField__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_DictField(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_43__pyx_unpickle_DictField(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_9drf_turbo_6fields_43__pyx_unpickle_DictField = {"__pyx_unpickle_DictField", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_43__pyx_unpickle_DictField, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_9drf_turbo_6fields_43__pyx_unpickle_DictField(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_DictField (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_DictField", 1, 3, 3, 1); __PYX_ERR(1, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_DictField", 1, 3, 3, 2); __PYX_ERR(1, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_DictField") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_DictField", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_DictField", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9drf_turbo_6fields_42__pyx_unpickle_DictField(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_42__pyx_unpickle_DictField(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_DictField", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x033cda3:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x033cda3 = (allow_empty, allow_null, attr, attrs, call, child, default_value, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x033cda3) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x033cda3:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x033cda3 = (allow_empty, allow_null, attr, attrs, call, child, default_value, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = DictField.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0x033cda3:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x033cda3 = (allow_empty, allow_null, attr, attrs, call, child, default_value, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = DictField.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x03, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(1, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x033cda3:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x033cda3 = (allow_empty, allow_null, attr, attrs, call, child, default_value, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x033cda3 = (allow_empty, allow_null, attr, attrs, call, child, default_value, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = DictField.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_DictField__set_state(<DictField> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_9drf_turbo_6fields_DictField), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x033cda3 = (allow_empty, allow_null, attr, attrs, call, child, default_value, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = DictField.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_DictField__set_state(<DictField> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = DictField.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_DictField__set_state(<DictField> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_DictField__set_state(DictField __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_9drf_turbo_6fields___pyx_unpickle_DictField__set_state(((struct __pyx_obj_9drf_turbo_6fields_DictField *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x033cda3 = (allow_empty, allow_null, attr, attrs, call, child, default_value, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = DictField.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_DictField__set_state(<DictField> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_DictField__set_state(<DictField> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_DictField__set_state(DictField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_empty = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.child = __pyx_state[5]; __pyx_result.default_value = __pyx_state[6]; __pyx_result.error_messages = __pyx_state[7]; __pyx_result.field_name = __pyx_state[8]; __pyx_result.help_text = __pyx_state[9]; __pyx_result.initial = __pyx_state[10]; __pyx_result.label = __pyx_state[11]; __pyx_result.read_only = __pyx_state[12]; __pyx_result.required = __pyx_state[13]; __pyx_result.root = __pyx_state[14]; __pyx_result.style = __pyx_state[15]; __pyx_result.validators = __pyx_state[16]; __pyx_result.write_only = __pyx_state[17]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_DictField(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_DictField", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_DictField__set_state(<DictField> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_DictField__set_state(DictField __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.allow_empty = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.child = __pyx_state[5]; __pyx_result.default_value = __pyx_state[6]; __pyx_result.error_messages = __pyx_state[7]; __pyx_result.field_name = __pyx_state[8]; __pyx_result.help_text = __pyx_state[9]; __pyx_result.initial = __pyx_state[10]; __pyx_result.label = __pyx_state[11]; __pyx_result.read_only = __pyx_state[12]; __pyx_result.required = __pyx_state[13]; __pyx_result.root = __pyx_state[14]; __pyx_result.style = __pyx_state[15]; __pyx_result.validators = __pyx_state[16]; __pyx_result.write_only = __pyx_state[17]
 *     if len(__pyx_state) > 18 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_9drf_turbo_6fields___pyx_unpickle_DictField__set_state(struct __pyx_obj_9drf_turbo_6fields_DictField *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_DictField__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_DictField__set_state(DictField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_empty = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.child = __pyx_state[5]; __pyx_result.default_value = __pyx_state[6]; __pyx_result.error_messages = __pyx_state[7]; __pyx_result.field_name = __pyx_state[8]; __pyx_result.help_text = __pyx_state[9]; __pyx_result.initial = __pyx_state[10]; __pyx_result.label = __pyx_state[11]; __pyx_result.read_only = __pyx_state[12]; __pyx_result.required = __pyx_state[13]; __pyx_result.root = __pyx_state[14]; __pyx_result.style = __pyx_state[15]; __pyx_result.validators = __pyx_state[16]; __pyx_result.write_only = __pyx_state[17]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 18 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[18])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->allow_empty);
  __Pyx_DECREF(__pyx_v___pyx_result->allow_empty);
  __pyx_v___pyx_result->allow_empty = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.allow_null = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.attr);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.attr);
  __pyx_v___pyx_result->__pyx_base.attr = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.attrs);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.attrs);
  __pyx_v___pyx_result->__pyx_base.attrs = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.call = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->child);
  __Pyx_DECREF(__pyx_v___pyx_result->child);
  __pyx_v___pyx_result->child = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.default_value);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.default_value);
  __pyx_v___pyx_result->__pyx_base.default_value = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.error_messages);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.error_messages);
  __pyx_v___pyx_result->__pyx_base.error_messages = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 8, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.field_name);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.field_name);
  __pyx_v___pyx_result->__pyx_base.field_name = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 9, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.help_text);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.help_text);
  __pyx_v___pyx_result->__pyx_base.help_text = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 10, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.initial);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.initial);
  __pyx_v___pyx_result->__pyx_base.initial = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 11, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.label);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.label);
  __pyx_v___pyx_result->__pyx_base.label = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 12, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.read_only = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 13, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.required = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 14, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.root);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.root);
  __pyx_v___pyx_result->__pyx_base.root = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 15, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.style);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.style);
  __pyx_v___pyx_result->__pyx_base.style = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 16, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.validators);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.validators);
  __pyx_v___pyx_result->__pyx_base.validators = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 17, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.write_only = __pyx_t_2;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_DictField__set_state(DictField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_empty = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.child = __pyx_state[5]; __pyx_result.default_value = __pyx_state[6]; __pyx_result.error_messages = __pyx_state[7]; __pyx_result.field_name = __pyx_state[8]; __pyx_result.help_text = __pyx_state[9]; __pyx_result.initial = __pyx_state[10]; __pyx_result.label = __pyx_state[11]; __pyx_result.read_only = __pyx_state[12]; __pyx_result.required = __pyx_state[13]; __pyx_result.root = __pyx_state[14]; __pyx_result.style = __pyx_state[15]; __pyx_result.validators = __pyx_state[16]; __pyx_result.write_only = __pyx_state[17]
 *     if len(__pyx_state) > 18 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[18])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 13, __pyx_L1_error)
  }
  __pyx_t_3 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_4 = ((__pyx_t_3 > 18) != 0);
  if (__pyx_t_4) {
  } else {
    __pyx_t_2 = __pyx_t_4;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_4 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_5 = (__pyx_t_4 != 0);
  __pyx_t_2 = __pyx_t_5;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":14
 *     __pyx_result.allow_empty = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.child = __pyx_state[5]; __pyx_result.default_value = __pyx_state[6]; __pyx_result.error_messages = __pyx_state[7]; __pyx_result.field_name = __pyx_state[8]; __pyx_result.help_text = __pyx_state[9]; __pyx_result.initial = __pyx_state[10]; __pyx_result.label = __pyx_state[11]; __pyx_result.read_only = __pyx_state[12]; __pyx_result.required = __pyx_state[13]; __pyx_result.root = __pyx_state[14]; __pyx_result.style = __pyx_state[15]; __pyx_result.validators = __pyx_state[16]; __pyx_result.write_only = __pyx_state[17]
 *     if len(__pyx_state) > 18 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[18])             # <<<<<<<<<<<<<<
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_update); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 14, __pyx_L1_error)
    }
    __pyx_t_6 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 18, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_8 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
      }
    }
    __pyx_t_1 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_8, __pyx_t_6) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_6);
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_DictField__set_state(DictField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_empty = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.child = __pyx_state[5]; __pyx_result.default_value = __pyx_state[6]; __pyx_result.error_messages = __pyx_state[7]; __pyx_result.field_name = __pyx_state[8]; __pyx_result.help_text = __pyx_state[9]; __pyx_result.initial = __pyx_state[10]; __pyx_result.label = __pyx_state[11]; __pyx_result.read_only = __pyx_state[12]; __pyx_result.required = __pyx_state[13]; __pyx_result.root = __pyx_state[14]; __pyx_result.style = __pyx_state[15]; __pyx_result.validators = __pyx_state[16]; __pyx_result.write_only = __pyx_state[17]
 *     if len(__pyx_state) > 18 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[18])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_DictField__set_state(<DictField> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_DictField__set_state(DictField __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.allow_empty = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.child = __pyx_state[5]; __pyx_result.default_value = __pyx_state[6]; __pyx_result.error_messages = __pyx_state[7]; __pyx_result.field_name = __pyx_state[8]; __pyx_result.help_text = __pyx_state[9]; __pyx_result.initial = __pyx_state[10]; __pyx_result.label = __pyx_state[11]; __pyx_result.read_only = __pyx_state[12]; __pyx_result.required = __pyx_state[13]; __pyx_result.root = __pyx_state[14]; __pyx_result.style = __pyx_state[15]; __pyx_result.validators = __pyx_state[16]; __pyx_result.write_only = __pyx_state[17]
 *     if len(__pyx_state) > 18 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_DictField__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_JSONField(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_45__pyx_unpickle_JSONField(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_9drf_turbo_6fields_45__pyx_unpickle_JSONField = {"__pyx_unpickle_JSONField", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_45__pyx_unpickle_JSONField, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_9drf_turbo_6fields_45__pyx_unpickle_JSONField(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_JSONField (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_JSONField", 1, 3, 3, 1); __PYX_ERR(1, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_JSONField", 1, 3, 3, 2); __PYX_ERR(1, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_JSONField") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_JSONField", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_JSONField", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9drf_turbo_6fields_44__pyx_unpickle_JSONField(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_44__pyx_unpickle_JSONField(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_JSONField", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x91e774b:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x91e774b = (allow_null, attr, attrs, binary, call, decoder, default_value, encoder, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x91e774b) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x91e774b:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x91e774b = (allow_null, attr, attrs, binary, call, decoder, default_value, encoder, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = JSONField.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0x91e774b:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x91e774b = (allow_null, attr, attrs, binary, call, decoder, default_value, encoder, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = JSONField.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x91, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(1, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x91e774b:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x91e774b = (allow_null, attr, attrs, binary, call, decoder, default_value, encoder, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x91e774b = (allow_null, attr, attrs, binary, call, decoder, default_value, encoder, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = JSONField.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_JSONField__set_state(<JSONField> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_9drf_turbo_6fields_JSONField), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x91e774b = (allow_null, attr, attrs, binary, call, decoder, default_value, encoder, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = JSONField.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_JSONField__set_state(<JSONField> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = JSONField.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_JSONField__set_state(<JSONField> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_JSONField__set_state(JSONField __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_9drf_turbo_6fields___pyx_unpickle_JSONField__set_state(((struct __pyx_obj_9drf_turbo_6fields_JSONField *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x91e774b = (allow_null, attr, attrs, binary, call, decoder, default_value, encoder, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = JSONField.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_JSONField__set_state(<JSONField> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_JSONField__set_state(<JSONField> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_JSONField__set_state(JSONField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.binary = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.decoder = __pyx_state[5]; __pyx_result.default_value = __pyx_state[6]; __pyx_result.encoder = __pyx_state[7]; __pyx_result.error_messages = __pyx_state[8]; __pyx_result.field_name = __pyx_state[9]; __pyx_result.help_text = __pyx_state[10]; __pyx_result.initial = __pyx_state[11]; __pyx_result.label = __pyx_state[12]; __pyx_result.read_only = __pyx_state[13]; __pyx_result.required = __pyx_state[14]; __pyx_result.root = __pyx_state[15]; __pyx_result.style = __pyx_state[16]; __pyx_result.validators = __pyx_state[17]; __pyx_result.write_only = __pyx_state[18]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_JSONField(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_JSONField", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_JSONField__set_state(<JSONField> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_JSONField__set_state(JSONField __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.binary = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.decoder = __pyx_state[5]; __pyx_result.default_value = __pyx_state[6]; __pyx_result.encoder = __pyx_state[7]; __pyx_result.error_messages = __pyx_state[8]; __pyx_result.field_name = __pyx_state[9]; __pyx_result.help_text = __pyx_state[10]; __pyx_result.initial = __pyx_state[11]; __pyx_result.label = __pyx_state[12]; __pyx_result.read_only = __pyx_state[13]; __pyx_result.required = __pyx_state[14]; __pyx_result.root = __pyx_state[15]; __pyx_result.style = __pyx_state[16]; __pyx_result.validators = __pyx_state[17]; __pyx_result.write_only = __pyx_state[18]
 *     if len(__pyx_state) > 19 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_9drf_turbo_6fields___pyx_unpickle_JSONField__set_state(struct __pyx_obj_9drf_turbo_6fields_JSONField *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_JSONField__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_JSONField__set_state(JSONField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.binary = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.decoder = __pyx_state[5]; __pyx_result.default_value = __pyx_state[6]; __pyx_result.encoder = __pyx_state[7]; __pyx_result.error_messages = __pyx_state[8]; __pyx_result.field_name = __pyx_state[9]; __pyx_result.help_text = __pyx_state[10]; __pyx_result.initial = __pyx_state[11]; __pyx_result.label = __pyx_state[12]; __pyx_result.read_only = __pyx_state[13]; __pyx_result.required = __pyx_state[14]; __pyx_result.root = __pyx_state[15]; __pyx_result.style = __pyx_state[16]; __pyx_result.validators = __pyx_state[17]; __pyx_result.write_only = __pyx_state[18]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 19 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[19])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.allow_null = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.attr);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.attr);
  __pyx_v___pyx_result->__pyx_base.attr = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.attrs);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.attrs);
  __pyx_v___pyx_result->__pyx_base.attrs = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->binary);
  __Pyx_DECREF(__pyx_v___pyx_result->binary);
  __pyx_v___pyx_result->binary = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.call = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->decoder);
  __Pyx_DECREF(__pyx_v___pyx_result->decoder);
  __pyx_v___pyx_result->decoder = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.default_value);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.default_value);
  __pyx_v___pyx_result->__pyx_base.default_value = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->encoder);
  __Pyx_DECREF(__pyx_v___pyx_result->encoder);
  __pyx_v___pyx_result->encoder = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 8, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.error_messages);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.error_messages);
  __pyx_v___pyx_result->__pyx_base.error_messages = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 9, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.field_name);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.field_name);
  __pyx_v___pyx_result->__pyx_base.field_name = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 10, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.help_text);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.help_text);
  __pyx_v___pyx_result->__pyx_base.help_text = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 11, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.initial);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.initial);
  __pyx_v___pyx_result->__pyx_base.initial = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 12, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.label);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.label);
  __pyx_v___pyx_result->__pyx_base.label = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 13, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.read_only = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 14, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.required = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 15, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.root);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.root);
  __pyx_v___pyx_result->__pyx_base.root = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 16, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.style);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.style);
  __pyx_v___pyx_result->__pyx_base.style = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 17, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.validators);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.validators);
  __pyx_v___pyx_result->__pyx_base.validators = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 18, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.write_only = __pyx_t_2;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_JSONField__set_state(JSONField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.binary = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.decoder = __pyx_state[5]; __pyx_result.default_value = __pyx_state[6]; __pyx_result.encoder = __pyx_state[7]; __pyx_result.error_messages = __pyx_state[8]; __pyx_result.field_name = __pyx_state[9]; __pyx_result.help_text = __pyx_state[10]; __pyx_result.initial = __pyx_state[11]; __pyx_result.label = __pyx_state[12]; __pyx_result.read_only = __pyx_state[13]; __pyx_result.required = __pyx_state[14]; __pyx_result.root = __pyx_state[15]; __pyx_result.style = __pyx_state[16]; __pyx_result.validators = __pyx_state[17]; __pyx_result.write_only = __pyx_state[18]
 *     if len(__pyx_state) > 19 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[19])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 13, __pyx_L1_error)
  }
  __pyx_t_3 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_4 = ((__pyx_t_3 > 19) != 0);
  if (__pyx_t_4) {
  } else {
    __pyx_t_2 = __pyx_t_4;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_4 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_5 = (__pyx_t_4 != 0);
  __pyx_t_2 = __pyx_t_5;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":14
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.binary = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.decoder = __pyx_state[5]; __pyx_result.default_value = __pyx_state[6]; __pyx_result.encoder = __pyx_state[7]; __pyx_result.error_messages = __pyx_state[8]; __pyx_result.field_name = __pyx_state[9]; __pyx_result.help_text = __pyx_state[10]; __pyx_result.initial = __pyx_state[11]; __pyx_result.label = __pyx_state[12]; __pyx_result.read_only = __pyx_state[13]; __pyx_result.required = __pyx_state[14]; __pyx_result.root = __pyx_state[15]; __pyx_result.style = __pyx_state[16]; __pyx_result.validators = __pyx_state[17]; __pyx_result.write_only = __pyx_state[18]
 *     if len(__pyx_state) > 19 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[19])             # <<<<<<<<<<<<<<
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_update); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 14, __pyx_L1_error)
    }
    __pyx_t_6 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 19, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_8 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
      }
    }
    __pyx_t_1 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_8, __pyx_t_6) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_6);
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_JSONField__set_state(JSONField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.binary = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.decoder = __pyx_state[5]; __pyx_result.default_value = __pyx_state[6]; __pyx_result.encoder = __pyx_state[7]; __pyx_result.error_messages = __pyx_state[8]; __pyx_result.field_name = __pyx_state[9]; __pyx_result.help_text = __pyx_state[10]; __pyx_result.initial = __pyx_state[11]; __pyx_result.label = __pyx_state[12]; __pyx_result.read_only = __pyx_state[13]; __pyx_result.required = __pyx_state[14]; __pyx_result.root = __pyx_state[15]; __pyx_result.style = __pyx_state[16]; __pyx_result.validators = __pyx_state[17]; __pyx_result.write_only = __pyx_state[18]
 *     if len(__pyx_state) > 19 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[19])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_JSONField__set_state(<JSONField> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_JSONField__set_state(JSONField __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.binary = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.decoder = __pyx_state[5]; __pyx_result.default_value = __pyx_state[6]; __pyx_result.encoder = __pyx_state[7]; __pyx_result.error_messages = __pyx_state[8]; __pyx_result.field_name = __pyx_state[9]; __pyx_result.help_text = __pyx_state[10]; __pyx_result.initial = __pyx_state[11]; __pyx_result.label = __pyx_state[12]; __pyx_result.read_only = __pyx_state[13]; __pyx_result.required = __pyx_state[14]; __pyx_result.root = __pyx_state[15]; __pyx_result.style = __pyx_state[16]; __pyx_result.validators = __pyx_state[17]; __pyx_result.write_only = __pyx_state[18]
 *     if len(__pyx_state) > 19 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_JSONField__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_RelatedField(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_47__pyx_unpickle_RelatedField(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_9drf_turbo_6fields_47__pyx_unpickle_RelatedField = {"__pyx_unpickle_RelatedField", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_47__pyx_unpickle_RelatedField, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_9drf_turbo_6fields_47__pyx_unpickle_RelatedField(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_RelatedField (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_RelatedField", 1, 3, 3, 1); __PYX_ERR(1, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_RelatedField", 1, 3, 3, 2); __PYX_ERR(1, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_RelatedField") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_RelatedField", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_RelatedField", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9drf_turbo_6fields_46__pyx_unpickle_RelatedField(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_46__pyx_unpickle_RelatedField(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_RelatedField", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x0ca6ac6:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x0ca6ac6 = (allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, queryset, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x0ca6ac6) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x0ca6ac6:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x0ca6ac6 = (allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, queryset, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = RelatedField.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0x0ca6ac6:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x0ca6ac6 = (allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, queryset, read_only, required, root, style, validators, write_only))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = RelatedField.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x0c, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(1, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x0ca6ac6:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x0ca6ac6 = (allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, queryset, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x0ca6ac6 = (allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, queryset, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = RelatedField.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_RelatedField__set_state(<RelatedField> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_9drf_turbo_6fields_RelatedField), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x0ca6ac6 = (allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, queryset, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = RelatedField.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_RelatedField__set_state(<RelatedField> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = RelatedField.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_RelatedField__set_state(<RelatedField> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_RelatedField__set_state(RelatedField __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_9drf_turbo_6fields___pyx_unpickle_RelatedField__set_state(((struct __pyx_obj_9drf_turbo_6fields_RelatedField *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x0ca6ac6 = (allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, queryset, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = RelatedField.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_RelatedField__set_state(<RelatedField> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_RelatedField__set_state(<RelatedField> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_RelatedField__set_state(RelatedField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.default_value = __pyx_state[4]; __pyx_result.error_messages = __pyx_state[5]; __pyx_result.field_name = __pyx_state[6]; __pyx_result.help_text = __pyx_state[7]; __pyx_result.initial = __pyx_state[8]; __pyx_result.label = __pyx_state[9]; __pyx_result.queryset = __pyx_state[10]; __pyx_result.read_only = __pyx_state[11]; __pyx_result.required = __pyx_state[12]; __pyx_result.root = __pyx_state[13]; __pyx_result.style = __pyx_state[14]; __pyx_result.validators = __pyx_state[15]; __pyx_result.write_only = __pyx_state[16]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_RelatedField(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_RelatedField", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_RelatedField__set_state(<RelatedField> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_RelatedField__set_state(RelatedField __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.default_value = __pyx_state[4]; __pyx_result.error_messages = __pyx_state[5]; __pyx_result.field_name = __pyx_state[6]; __pyx_result.help_text = __pyx_state[7]; __pyx_result.initial = __pyx_state[8]; __pyx_result.label = __pyx_state[9]; __pyx_result.queryset = __pyx_state[10]; __pyx_result.read_only = __pyx_state[11]; __pyx_result.required = __pyx_state[12]; __pyx_result.root = __pyx_state[13]; __pyx_result.style = __pyx_state[14]; __pyx_result.validators = __pyx_state[15]; __pyx_result.write_only = __pyx_state[16]
 *     if len(__pyx_state) > 17 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_9drf_turbo_6fields___pyx_unpickle_RelatedField__set_state(struct __pyx_obj_9drf_turbo_6fields_RelatedField *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_RelatedField__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_RelatedField__set_state(RelatedField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.default_value = __pyx_state[4]; __pyx_result.error_messages = __pyx_state[5]; __pyx_result.field_name = __pyx_state[6]; __pyx_result.help_text = __pyx_state[7]; __pyx_result.initial = __pyx_state[8]; __pyx_result.label = __pyx_state[9]; __pyx_result.queryset = __pyx_state[10]; __pyx_result.read_only = __pyx_state[11]; __pyx_result.required = __pyx_state[12]; __pyx_result.root = __pyx_state[13]; __pyx_result.style = __pyx_state[14]; __pyx_result.validators = __pyx_state[15]; __pyx_result.write_only = __pyx_state[16]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 17 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[17])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.allow_null = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.attr);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.attr);
  __pyx_v___pyx_result->__pyx_base.attr = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.attrs);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.attrs);
  __pyx_v___pyx_result->__pyx_base.attrs = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.call = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.default_value);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.default_value);
  __pyx_v___pyx_result->__pyx_base.default_value = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.error_messages);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.error_messages);
  __pyx_v___pyx_result->__pyx_base.error_messages = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.field_name);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.field_name);
  __pyx_v___pyx_result->__pyx_base.field_name = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.help_text);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.help_text);
  __pyx_v___pyx_result->__pyx_base.help_text = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 8, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.initial);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.initial);
  __pyx_v___pyx_result->__pyx_base.initial = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 9, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.label);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.label);
  __pyx_v___pyx_result->__pyx_base.label = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 10, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->queryset);
  __Pyx_DECREF(__pyx_v___pyx_result->queryset);
  __pyx_v___pyx_result->queryset = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 11, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.read_only = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 12, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.required = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 13, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.root);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.root);
  __pyx_v___pyx_result->__pyx_base.root = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 14, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.style);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.style);
  __pyx_v___pyx_result->__pyx_base.style = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 15, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.validators);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.validators);
  __pyx_v___pyx_result->__pyx_base.validators = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 16, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.write_only = __pyx_t_2;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_RelatedField__set_state(RelatedField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.default_value = __pyx_state[4]; __pyx_result.error_messages = __pyx_state[5]; __pyx_result.field_name = __pyx_state[6]; __pyx_result.help_text = __pyx_state[7]; __pyx_result.initial = __pyx_state[8]; __pyx_result.label = __pyx_state[9]; __pyx_result.queryset = __pyx_state[10]; __pyx_result.read_only = __pyx_state[11]; __pyx_result.required = __pyx_state[12]; __pyx_result.root = __pyx_state[13]; __pyx_result.style = __pyx_state[14]; __pyx_result.validators = __pyx_state[15]; __pyx_result.write_only = __pyx_state[16]
 *     if len(__pyx_state) > 17 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[17])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 13, __pyx_L1_error)
  }
  __pyx_t_3 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_4 = ((__pyx_t_3 > 17) != 0);
  if (__pyx_t_4) {
  } else {
    __pyx_t_2 = __pyx_t_4;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_4 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_5 = (__pyx_t_4 != 0);
  __pyx_t_2 = __pyx_t_5;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":14
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.default_value = __pyx_state[4]; __pyx_result.error_messages = __pyx_state[5]; __pyx_result.field_name = __pyx_state[6]; __pyx_result.help_text = __pyx_state[7]; __pyx_result.initial = __pyx_state[8]; __pyx_result.label = __pyx_state[9]; __pyx_result.queryset = __pyx_state[10]; __pyx_result.read_only = __pyx_state[11]; __pyx_result.required = __pyx_state[12]; __pyx_result.root = __pyx_state[13]; __pyx_result.style = __pyx_state[14]; __pyx_result.validators = __pyx_state[15]; __pyx_result.write_only = __pyx_state[16]
 *     if len(__pyx_state) > 17 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[17])             # <<<<<<<<<<<<<<
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_update); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 14, __pyx_L1_error)
    }
    __pyx_t_6 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 17, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_8 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
      }
    }
    __pyx_t_1 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_8, __pyx_t_6) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_6);
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_RelatedField__set_state(RelatedField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.default_value = __pyx_state[4]; __pyx_result.error_messages = __pyx_state[5]; __pyx_result.field_name = __pyx_state[6]; __pyx_result.help_text = __pyx_state[7]; __pyx_result.initial = __pyx_state[8]; __pyx_result.label = __pyx_state[9]; __pyx_result.queryset = __pyx_state[10]; __pyx_result.read_only = __pyx_state[11]; __pyx_result.required = __pyx_state[12]; __pyx_result.root = __pyx_state[13]; __pyx_result.style = __pyx_state[14]; __pyx_result.validators = __pyx_state[15]; __pyx_result.write_only = __pyx_state[16]
 *     if len(__pyx_state) > 17 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[17])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_RelatedField__set_state(<RelatedField> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_RelatedField__set_state(RelatedField __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.default_value = __pyx_state[4]; __pyx_result.error_messages = __pyx_state[5]; __pyx_result.field_name = __pyx_state[6]; __pyx_result.help_text = __pyx_state[7]; __pyx_result.initial = __pyx_state[8]; __pyx_result.label = __pyx_state[9]; __pyx_result.queryset = __pyx_state[10]; __pyx_result.read_only = __pyx_state[11]; __pyx_result.required = __pyx_state[12]; __pyx_result.root = __pyx_state[13]; __pyx_result.style = __pyx_state[14]; __pyx_result.validators = __pyx_state[15]; __pyx_result.write_only = __pyx_state[16]
 *     if len(__pyx_state) > 17 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_RelatedField__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_ManyRelatedField(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_49__pyx_unpickle_ManyRelatedField(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_9drf_turbo_6fields_49__pyx_unpickle_ManyRelatedField = {"__pyx_unpickle_ManyRelatedField", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_49__pyx_unpickle_ManyRelatedField, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_9drf_turbo_6fields_49__pyx_unpickle_ManyRelatedField(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_ManyRelatedField (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_ManyRelatedField", 1, 3, 3, 1); __PYX_ERR(1, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_ManyRelatedField", 1, 3, 3, 2); __PYX_ERR(1, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_ManyRelatedField") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_ManyRelatedField", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_ManyRelatedField", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9drf_turbo_6fields_48__pyx_unpickle_ManyRelatedField(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_48__pyx_unpickle_ManyRelatedField(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_ManyRelatedField", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xf4706f5:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xf4706f5 = (allow_empty, allow_null, attr, attrs, call, child_relation, default_value, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0xf4706f5) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xf4706f5:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xf4706f5 = (allow_empty, allow_null, attr, attrs, call, child_relation, default_value, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = ManyRelatedField.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0xf4706f5:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xf4706f5 = (allow_empty, allow_null, attr, attrs, call, child_relation, default_value, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = ManyRelatedField.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0xf4, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(1, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xf4706f5:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xf4706f5 = (allow_empty, allow_null, attr, attrs, call, child_relation, default_value, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xf4706f5 = (allow_empty, allow_null, attr, attrs, call, child_relation, default_value, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = ManyRelatedField.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_ManyRelatedField__set_state(<ManyRelatedField> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_9drf_turbo_6fields_ManyRelatedField), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xf4706f5 = (allow_empty, allow_null, attr, attrs, call, child_relation, default_value, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = ManyRelatedField.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_ManyRelatedField__set_state(<ManyRelatedField> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = ManyRelatedField.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_ManyRelatedField__set_state(<ManyRelatedField> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_ManyRelatedField__set_state(ManyRelatedField __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_9drf_turbo_6fields___pyx_unpickle_ManyRelatedField__set_state(((struct __pyx_obj_9drf_turbo_6fields_ManyRelatedField *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xf4706f5 = (allow_empty, allow_null, attr, attrs, call, child_relation, default_value, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = ManyRelatedField.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_ManyRelatedField__set_state(<ManyRelatedField> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_ManyRelatedField__set_state(<ManyRelatedField> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_ManyRelatedField__set_state(ManyRelatedField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_empty = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.child_relation = __pyx_state[5]; __pyx_result.default_value = __pyx_state[6]; __pyx_result.error_messages = __pyx_state[7]; __pyx_result.field_name = __pyx_state[8]; __pyx_result.help_text = __pyx_state[9]; __pyx_result.initial = __pyx_state[10]; __pyx_result.label = __pyx_state[11]; __pyx_result.read_only = __pyx_state[12]; __pyx_result.required = __pyx_state[13]; __pyx_result.root = __pyx_state[14]; __pyx_result.style = __pyx_state[15]; __pyx_result.validators = __pyx_state[16]; __pyx_result.write_only = __pyx_state[17]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_ManyRelatedField(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_ManyRelatedField", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_ManyRelatedField__set_state(<ManyRelatedField> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_ManyRelatedField__set_state(ManyRelatedField __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.allow_empty = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.child_relation = __pyx_state[5]; __pyx_result.default_value = __pyx_state[6]; __pyx_result.error_messages = __pyx_state[7]; __pyx_result.field_name = __pyx_state[8]; __pyx_result.help_text = __pyx_state[9]; __pyx_result.initial = __pyx_state[10]; __pyx_result.label = __pyx_state[11]; __pyx_result.read_only = __pyx_state[12]; __pyx_result.required = __pyx_state[13]; __pyx_result.root = __pyx_state[14]; __pyx_result.style = __pyx_state[15]; __pyx_result.validators = __pyx_state[16]; __pyx_result.write_only = __pyx_state[17]
 *     if len(__pyx_state) > 18 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_9drf_turbo_6fields___pyx_unpickle_ManyRelatedField__set_state(struct __pyx_obj_9drf_turbo_6fields_ManyRelatedField *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_ManyRelatedField__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_ManyRelatedField__set_state(ManyRelatedField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_empty = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.child_relation = __pyx_state[5]; __pyx_result.default_value = __pyx_state[6]; __pyx_result.error_messages = __pyx_state[7]; __pyx_result.field_name = __pyx_state[8]; __pyx_result.help_text = __pyx_state[9]; __pyx_result.initial = __pyx_state[10]; __pyx_result.label = __pyx_state[11]; __pyx_result.read_only = __pyx_state[12]; __pyx_result.required = __pyx_state[13]; __pyx_result.root = __pyx_state[14]; __pyx_result.style = __pyx_state[15]; __pyx_result.validators = __pyx_state[16]; __pyx_result.write_only = __pyx_state[17]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 18 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[18])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->allow_empty);
  __Pyx_DECREF(__pyx_v___pyx_result->allow_empty);
  __pyx_v___pyx_result->allow_empty = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.allow_null = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.attr);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.attr);
  __pyx_v___pyx_result->__pyx_base.attr = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.attrs);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.attrs);
  __pyx_v___pyx_result->__pyx_base.attrs = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.call = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->child_relation);
  __Pyx_DECREF(__pyx_v___pyx_result->child_relation);
  __pyx_v___pyx_result->child_relation = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.default_value);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.default_value);
  __pyx_v___pyx_result->__pyx_base.default_value = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.error_messages);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.error_messages);
  __pyx_v___pyx_result->__pyx_base.error_messages = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 8, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.field_name);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.field_name);
  __pyx_v___pyx_result->__pyx_base.field_name = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 9, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.help_text);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.help_text);
  __pyx_v___pyx_result->__pyx_base.help_text = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 10, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.initial);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.initial);
  __pyx_v___pyx_result->__pyx_base.initial = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 11, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.label);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.label);
  __pyx_v___pyx_result->__pyx_base.label = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 12, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.read_only = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 13, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.required = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 14, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.root);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.root);
  __pyx_v___pyx_result->__pyx_base.root = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 15, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.style);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.style);
  __pyx_v___pyx_result->__pyx_base.style = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 16, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.validators);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.validators);
  __pyx_v___pyx_result->__pyx_base.validators = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 17, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.write_only = __pyx_t_2;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_ManyRelatedField__set_state(ManyRelatedField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_empty = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.child_relation = __pyx_state[5]; __pyx_result.default_value = __pyx_state[6]; __pyx_result.error_messages = __pyx_state[7]; __pyx_result.field_name = __pyx_state[8]; __pyx_result.help_text = __pyx_state[9]; __pyx_result.initial = __pyx_state[10]; __pyx_result.label = __pyx_state[11]; __pyx_result.read_only = __pyx_state[12]; __pyx_result.required = __pyx_state[13]; __pyx_result.root = __pyx_state[14]; __pyx_result.style = __pyx_state[15]; __pyx_result.validators = __pyx_state[16]; __pyx_result.write_only = __pyx_state[17]
 *     if len(__pyx_state) > 18 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[18])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 13, __pyx_L1_error)
  }
  __pyx_t_3 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_4 = ((__pyx_t_3 > 18) != 0);
  if (__pyx_t_4) {
  } else {
    __pyx_t_2 = __pyx_t_4;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_4 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_5 = (__pyx_t_4 != 0);
  __pyx_t_2 = __pyx_t_5;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":14
 *     __pyx_result.allow_empty = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.child_relation = __pyx_state[5]; __pyx_result.default_value = __pyx_state[6]; __pyx_result.error_messages = __pyx_state[7]; __pyx_result.field_name = __pyx_state[8]; __pyx_result.help_text = __pyx_state[9]; __pyx_result.initial = __pyx_state[10]; __pyx_result.label = __pyx_state[11]; __pyx_result.read_only = __pyx_state[12]; __pyx_result.required = __pyx_state[13]; __pyx_result.root = __pyx_state[14]; __pyx_result.style = __pyx_state[15]; __pyx_result.validators = __pyx_state[16]; __pyx_result.write_only = __pyx_state[17]
 *     if len(__pyx_state) > 18 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[18])             # <<<<<<<<<<<<<<
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_update); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 14, __pyx_L1_error)
    }
    __pyx_t_6 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 18, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_8 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
      }
    }
    __pyx_t_1 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_8, __pyx_t_6) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_6);
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_ManyRelatedField__set_state(ManyRelatedField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_empty = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.child_relation = __pyx_state[5]; __pyx_result.default_value = __pyx_state[6]; __pyx_result.error_messages = __pyx_state[7]; __pyx_result.field_name = __pyx_state[8]; __pyx_result.help_text = __pyx_state[9]; __pyx_result.initial = __pyx_state[10]; __pyx_result.label = __pyx_state[11]; __pyx_result.read_only = __pyx_state[12]; __pyx_result.required = __pyx_state[13]; __pyx_result.root = __pyx_state[14]; __pyx_result.style = __pyx_state[15]; __pyx_result.validators = __pyx_state[16]; __pyx_result.write_only = __pyx_state[17]
 *     if len(__pyx_state) > 18 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[18])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_ManyRelatedField__set_state(<ManyRelatedField> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_ManyRelatedField__set_state(ManyRelatedField __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.allow_empty = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.child_relation = __pyx_state[5]; __pyx_result.default_value = __pyx_state[6]; __pyx_result.error_messages = __pyx_state[7]; __pyx_result.field_name = __pyx_state[8]; __pyx_result.help_text = __pyx_state[9]; __pyx_result.initial = __pyx_state[10]; __pyx_result.label = __pyx_state[11]; __pyx_result.read_only = __pyx_state[12]; __pyx_result.required = __pyx_state[13]; __pyx_result.root = __pyx_state[14]; __pyx_result.style = __pyx_state[15]; __pyx_result.validators = __pyx_state[16]; __pyx_result.write_only = __pyx_state[17]
 *     if len(__pyx_state) > 18 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_ManyRelatedField__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_ConstantField(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_51__pyx_unpickle_ConstantField(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_9drf_turbo_6fields_51__pyx_unpickle_ConstantField = {"__pyx_unpickle_ConstantField", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_51__pyx_unpickle_ConstantField, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_9drf_turbo_6fields_51__pyx_unpickle_ConstantField(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_ConstantField (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_ConstantField", 1, 3, 3, 1); __PYX_ERR(1, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_ConstantField", 1, 3, 3, 2); __PYX_ERR(1, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_ConstantField") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_ConstantField", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_ConstantField", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9drf_turbo_6fields_50__pyx_unpickle_ConstantField(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_50__pyx_unpickle_ConstantField(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_ConstantField", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x369fe29:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x369fe29 = (allow_null, attr, attrs, call, constant, default_value, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x369fe29) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x369fe29:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x369fe29 = (allow_null, attr, attrs, call, constant, default_value, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = ConstantField.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0x369fe29:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x369fe29 = (allow_null, attr, attrs, call, constant, default_value, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = ConstantField.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x36, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(1, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x369fe29:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x369fe29 = (allow_null, attr, attrs, call, constant, default_value, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x369fe29 = (allow_null, attr, attrs, call, constant, default_value, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = ConstantField.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_ConstantField__set_state(<ConstantField> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_9drf_turbo_6fields_ConstantField), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x369fe29 = (allow_null, attr, attrs, call, constant, default_value, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = ConstantField.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_ConstantField__set_state(<ConstantField> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = ConstantField.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_ConstantField__set_state(<ConstantField> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_ConstantField__set_state(ConstantField __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_9drf_turbo_6fields___pyx_unpickle_ConstantField__set_state(((struct __pyx_obj_9drf_turbo_6fields_ConstantField *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x369fe29 = (allow_null, attr, attrs, call, constant, default_value, error_messages, field_name, help_text, initial, label, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = ConstantField.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_ConstantField__set_state(<ConstantField> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_ConstantField__set_state(<ConstantField> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_ConstantField__set_state(ConstantField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.constant = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.read_only = __pyx_state[11]; __pyx_result.required = __pyx_state[12]; __pyx_result.root = __pyx_state[13]; __pyx_result.style = __pyx_state[14]; __pyx_result.validators = __pyx_state[15]; __pyx_result.write_only = __pyx_state[16]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_ConstantField(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_ConstantField", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_ConstantField__set_state(<ConstantField> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_ConstantField__set_state(ConstantField __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.constant = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.read_only = __pyx_state[11]; __pyx_result.required = __pyx_state[12]; __pyx_result.root = __pyx_state[13]; __pyx_result.style = __pyx_state[14]; __pyx_result.validators = __pyx_state[15]; __pyx_result.write_only = __pyx_state[16]
 *     if len(__pyx_state) > 17 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_9drf_turbo_6fields___pyx_unpickle_ConstantField__set_state(struct __pyx_obj_9drf_turbo_6fields_ConstantField *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_ConstantField__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_ConstantField__set_state(ConstantField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.constant = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.read_only = __pyx_state[11]; __pyx_result.required = __pyx_state[12]; __pyx_result.root = __pyx_state[13]; __pyx_result.style = __pyx_state[14]; __pyx_result.validators = __pyx_state[15]; __pyx_result.write_only = __pyx_state[16]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 17 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[17])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.allow_null = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.attr);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.attr);
  __pyx_v___pyx_result->__pyx_base.attr = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.attrs);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.attrs);
  __pyx_v___pyx_result->__pyx_base.attrs = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.call = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->constant);
  __Pyx_DECREF(__pyx_v___pyx_result->constant);
  __pyx_v___pyx_result->constant = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.default_value);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.default_value);
  __pyx_v___pyx_result->__pyx_base.default_value = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.error_messages);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.error_messages);
  __pyx_v___pyx_result->__pyx_base.error_messages = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.field_name);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.field_name);
  __pyx_v___pyx_result->__pyx_base.field_name = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 8, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.help_text);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.help_text);
  __pyx_v___pyx_result->__pyx_base.help_text = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 9, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.initial);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.initial);
  __pyx_v___pyx_result->__pyx_base.initial = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 10, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.label);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.label);
  __pyx_v___pyx_result->__pyx_base.label = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 11, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.read_only = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 12, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.required = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 13, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.root);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.root);
  __pyx_v___pyx_result->__pyx_base.root = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 14, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.style);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.style);
  __pyx_v___pyx_result->__pyx_base.style = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 15, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.validators);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.validators);
  __pyx_v___pyx_result->__pyx_base.validators = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 16, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.write_only = __pyx_t_2;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_ConstantField__set_state(ConstantField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.constant = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.read_only = __pyx_state[11]; __pyx_result.required = __pyx_state[12]; __pyx_result.root = __pyx_state[13]; __pyx_result.style = __pyx_state[14]; __pyx_result.validators = __pyx_state[15]; __pyx_result.write_only = __pyx_state[16]
 *     if len(__pyx_state) > 17 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[17])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 13, __pyx_L1_error)
  }
  __pyx_t_3 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_4 = ((__pyx_t_3 > 17) != 0);
  if (__pyx_t_4) {
  } else {
    __pyx_t_2 = __pyx_t_4;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_4 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_5 = (__pyx_t_4 != 0);
  __pyx_t_2 = __pyx_t_5;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":14
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.constant = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.read_only = __pyx_state[11]; __pyx_result.required = __pyx_state[12]; __pyx_result.root = __pyx_state[13]; __pyx_result.style = __pyx_state[14]; __pyx_result.validators = __pyx_state[15]; __pyx_result.write_only = __pyx_state[16]
 *     if len(__pyx_state) > 17 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[17])             # <<<<<<<<<<<<<<
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_update); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 14, __pyx_L1_error)
    }
    __pyx_t_6 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 17, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_8 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
      }
    }
    __pyx_t_1 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_8, __pyx_t_6) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_6);
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_ConstantField__set_state(ConstantField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.constant = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.read_only = __pyx_state[11]; __pyx_result.required = __pyx_state[12]; __pyx_result.root = __pyx_state[13]; __pyx_result.style = __pyx_state[14]; __pyx_result.validators = __pyx_state[15]; __pyx_result.write_only = __pyx_state[16]
 *     if len(__pyx_state) > 17 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[17])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_ConstantField__set_state(<ConstantField> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_ConstantField__set_state(ConstantField __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.constant = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.read_only = __pyx_state[11]; __pyx_result.required = __pyx_state[12]; __pyx_result.root = __pyx_state[13]; __pyx_result.style = __pyx_state[14]; __pyx_result.validators = __pyx_state[15]; __pyx_result.write_only = __pyx_state[16]
 *     if len(__pyx_state) > 17 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_ConstantField__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_RecursiveField(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_53__pyx_unpickle_RecursiveField(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_9drf_turbo_6fields_53__pyx_unpickle_RecursiveField = {"__pyx_unpickle_RecursiveField", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_53__pyx_unpickle_RecursiveField, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_9drf_turbo_6fields_53__pyx_unpickle_RecursiveField(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_RecursiveField (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_RecursiveField", 1, 3, 3, 1); __PYX_ERR(1, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_RecursiveField", 1, 3, 3, 2); __PYX_ERR(1, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_RecursiveField") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_RecursiveField", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_RecursiveField", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9drf_turbo_6fields_52__pyx_unpickle_RecursiveField(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_52__pyx_unpickle_RecursiveField(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_RecursiveField", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xf725730:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xf725730 = (allow_null, attr, attrs, call, context, default_value, error_messages, exclude, field_name, help_text, initial, label, many, only, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0xf725730) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xf725730:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xf725730 = (allow_null, attr, attrs, call, context, default_value, error_messages, exclude, field_name, help_text, initial, label, many, only, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = RecursiveField.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0xf725730:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xf725730 = (allow_null, attr, attrs, call, context, default_value, error_messages, exclude, field_name, help_text, initial, label, many, only, read_only, required, root, style, validators, write_only))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = RecursiveField.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0xf7, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(1, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xf725730:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xf725730 = (allow_null, attr, attrs, call, context, default_value, error_messages, exclude, field_name, help_text, initial, label, many, only, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xf725730 = (allow_null, attr, attrs, call, context, default_value, error_messages, exclude, field_name, help_text, initial, label, many, only, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = RecursiveField.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_RecursiveField__set_state(<RecursiveField> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_9drf_turbo_6fields_RecursiveField), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xf725730 = (allow_null, attr, attrs, call, context, default_value, error_messages, exclude, field_name, help_text, initial, label, many, only, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = RecursiveField.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_RecursiveField__set_state(<RecursiveField> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = RecursiveField.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_RecursiveField__set_state(<RecursiveField> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_RecursiveField__set_state(RecursiveField __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_9drf_turbo_6fields___pyx_unpickle_RecursiveField__set_state(((struct __pyx_obj_9drf_turbo_6fields_RecursiveField *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xf725730 = (allow_null, attr, attrs, call, context, default_value, error_messages, exclude, field_name, help_text, initial, label, many, only, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = RecursiveField.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_RecursiveField__set_state(<RecursiveField> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_RecursiveField__set_state(<RecursiveField> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_RecursiveField__set_state(RecursiveField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.context = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.exclude = __pyx_state[7]; __pyx_result.field_name = __pyx_state[8]; __pyx_result.help_text = __pyx_state[9]; __pyx_result.initial = __pyx_state[10]; __pyx_result.label = __pyx_state[11]; __pyx_result.many = __pyx_state[12]; __pyx_result.only = __pyx_state[13]; __pyx_result.read_only = __pyx_state[14]; __pyx_result.required = __pyx_state[15]; __pyx_result.root = __pyx_state[16]; __pyx_result.style = __pyx_state[17]; __pyx_result.validators = __pyx_state[18]; __pyx_result.write_only = __pyx_state[19]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_RecursiveField(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_RecursiveField", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_RecursiveField__set_state(<RecursiveField> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_RecursiveField__set_state(RecursiveField __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.context = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.exclude = __pyx_state[7]; __pyx_result.field_name = __pyx_state[8]; __pyx_result.help_text = __pyx_state[9]; __pyx_result.initial = __pyx_state[10]; __pyx_result.label = __pyx_state[11]; __pyx_result.many = __pyx_state[12]; __pyx_result.only = __pyx_state[13]; __pyx_result.read_only = __pyx_state[14]; __pyx_result.required = __pyx_state[15]; __pyx_result.root = __pyx_state[16]; __pyx_result.style = __pyx_state[17]; __pyx_result.validators = __pyx_state[18]; __pyx_result.write_only = __pyx_state[19]
 *     if len(__pyx_state) > 20 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_9drf_turbo_6fields___pyx_unpickle_RecursiveField__set_state(struct __pyx_obj_9drf_turbo_6fields_RecursiveField *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_RecursiveField__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_RecursiveField__set_state(RecursiveField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.context = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.exclude = __pyx_state[7]; __pyx_result.field_name = __pyx_state[8]; __pyx_result.help_text = __pyx_state[9]; __pyx_result.initial = __pyx_state[10]; __pyx_result.label = __pyx_state[11]; __pyx_result.many = __pyx_state[12]; __pyx_result.only = __pyx_state[13]; __pyx_result.read_only = __pyx_state[14]; __pyx_result.required = __pyx_state[15]; __pyx_result.root = __pyx_state[16]; __pyx_result.style = __pyx_state[17]; __pyx_result.validators = __pyx_state[18]; __pyx_result.write_only = __pyx_state[19]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 20 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[20])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.allow_null = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.attr);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.attr);
  __pyx_v___pyx_result->__pyx_base.attr = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.attrs);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.attrs);
  __pyx_v___pyx_result->__pyx_base.attrs = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.call = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->context);
  __Pyx_DECREF(__pyx_v___pyx_result->context);
  __pyx_v___pyx_result->context = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.default_value);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.default_value);
  __pyx_v___pyx_result->__pyx_base.default_value = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.error_messages);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.error_messages);
  __pyx_v___pyx_result->__pyx_base.error_messages = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->exclude);
  __Pyx_DECREF(__pyx_v___pyx_result->exclude);
  __pyx_v___pyx_result->exclude = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 8, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.field_name);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.field_name);
  __pyx_v___pyx_result->__pyx_base.field_name = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 9, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.help_text);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.help_text);
  __pyx_v___pyx_result->__pyx_base.help_text = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 10, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.initial);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.initial);
  __pyx_v___pyx_result->__pyx_base.initial = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 11, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.label);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.label);
  __pyx_v___pyx_result->__pyx_base.label = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 12, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->many);
  __Pyx_DECREF(__pyx_v___pyx_result->many);
  __pyx_v___pyx_result->many = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 13, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->only);
  __Pyx_DECREF(__pyx_v___pyx_result->only);
  __pyx_v___pyx_result->only = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 14, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.read_only = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 15, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.required = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 16, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.root);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.root);
  __pyx_v___pyx_result->__pyx_base.root = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 17, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.style);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.style);
  __pyx_v___pyx_result->__pyx_base.style = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 18, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.validators);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.validators);
  __pyx_v___pyx_result->__pyx_base.validators = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 19, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.write_only = __pyx_t_2;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_RecursiveField__set_state(RecursiveField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.context = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.exclude = __pyx_state[7]; __pyx_result.field_name = __pyx_state[8]; __pyx_result.help_text = __pyx_state[9]; __pyx_result.initial = __pyx_state[10]; __pyx_result.label = __pyx_state[11]; __pyx_result.many = __pyx_state[12]; __pyx_result.only = __pyx_state[13]; __pyx_result.read_only = __pyx_state[14]; __pyx_result.required = __pyx_state[15]; __pyx_result.root = __pyx_state[16]; __pyx_result.style = __pyx_state[17]; __pyx_result.validators = __pyx_state[18]; __pyx_result.write_only = __pyx_state[19]
 *     if len(__pyx_state) > 20 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[20])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 13, __pyx_L1_error)
  }
  __pyx_t_3 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_4 = ((__pyx_t_3 > 20) != 0);
  if (__pyx_t_4) {
  } else {
    __pyx_t_2 = __pyx_t_4;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_4 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_5 = (__pyx_t_4 != 0);
  __pyx_t_2 = __pyx_t_5;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":14
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.context = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.exclude = __pyx_state[7]; __pyx_result.field_name = __pyx_state[8]; __pyx_result.help_text = __pyx_state[9]; __pyx_result.initial = __pyx_state[10]; __pyx_result.label = __pyx_state[11]; __pyx_result.many = __pyx_state[12]; __pyx_result.only = __pyx_state[13]; __pyx_result.read_only = __pyx_state[14]; __pyx_result.required = __pyx_state[15]; __pyx_result.root = __pyx_state[16]; __pyx_result.style = __pyx_state[17]; __pyx_result.validators = __pyx_state[18]; __pyx_result.write_only = __pyx_state[19]
 *     if len(__pyx_state) > 20 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[20])             # <<<<<<<<<<<<<<
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_update); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 14, __pyx_L1_error)
    }
    __pyx_t_6 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 20, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_8 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
      }
    }
    __pyx_t_1 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_8, __pyx_t_6) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_6);
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_RecursiveField__set_state(RecursiveField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.context = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.exclude = __pyx_state[7]; __pyx_result.field_name = __pyx_state[8]; __pyx_result.help_text = __pyx_state[9]; __pyx_result.initial = __pyx_state[10]; __pyx_result.label = __pyx_state[11]; __pyx_result.many = __pyx_state[12]; __pyx_result.only = __pyx_state[13]; __pyx_result.read_only = __pyx_state[14]; __pyx_result.required = __pyx_state[15]; __pyx_result.root = __pyx_state[16]; __pyx_result.style = __pyx_state[17]; __pyx_result.validators = __pyx_state[18]; __pyx_result.write_only = __pyx_state[19]
 *     if len(__pyx_state) > 20 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[20])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_RecursiveField__set_state(<RecursiveField> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_RecursiveField__set_state(RecursiveField __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.context = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.exclude = __pyx_state[7]; __pyx_result.field_name = __pyx_state[8]; __pyx_result.help_text = __pyx_state[9]; __pyx_result.initial = __pyx_state[10]; __pyx_result.label = __pyx_state[11]; __pyx_result.many = __pyx_state[12]; __pyx_result.only = __pyx_state[13]; __pyx_result.read_only = __pyx_state[14]; __pyx_result.required = __pyx_state[15]; __pyx_result.root = __pyx_state[16]; __pyx_result.style = __pyx_state[17]; __pyx_result.validators = __pyx_state[18]; __pyx_result.write_only = __pyx_state[19]
 *     if len(__pyx_state) > 20 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_RecursiveField__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_MethodField(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_9drf_turbo_6fields_55__pyx_unpickle_MethodField(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_9drf_turbo_6fields_55__pyx_unpickle_MethodField = {"__pyx_unpickle_MethodField", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_55__pyx_unpickle_MethodField, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_9drf_turbo_6fields_55__pyx_unpickle_MethodField(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_MethodField (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MethodField", 1, 3, 3, 1); __PYX_ERR(1, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MethodField", 1, 3, 3, 2); __PYX_ERR(1, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_MethodField") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MethodField", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_MethodField", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9drf_turbo_6fields_54__pyx_unpickle_MethodField(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9drf_turbo_6fields_54__pyx_unpickle_MethodField(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MethodField", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xc705305:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xc705305 = (allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, method_name, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0xc705305) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xc705305:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xc705305 = (allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, method_name, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = MethodField.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0xc705305:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xc705305 = (allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, method_name, read_only, required, root, style, validators, write_only))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = MethodField.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0xc7, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(1, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xc705305:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xc705305 = (allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, method_name, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xc705305 = (allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, method_name, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = MethodField.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MethodField__set_state(<MethodField> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_9drf_turbo_6fields_MethodField), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xc705305 = (allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, method_name, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = MethodField.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MethodField__set_state(<MethodField> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = MethodField.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MethodField__set_state(<MethodField> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_MethodField__set_state(MethodField __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_9drf_turbo_6fields___pyx_unpickle_MethodField__set_state(((struct __pyx_obj_9drf_turbo_6fields_MethodField *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xc705305 = (allow_null, attr, attrs, call, default_value, error_messages, field_name, help_text, initial, label, method_name, read_only, required, root, style, validators, write_only))" % __pyx_checksum)
 *     __pyx_result = MethodField.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MethodField__set_state(<MethodField> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MethodField__set_state(<MethodField> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_MethodField__set_state(MethodField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.default_value = __pyx_state[4]; __pyx_result.error_messages = __pyx_state[5]; __pyx_result.field_name = __pyx_state[6]; __pyx_result.help_text = __pyx_state[7]; __pyx_result.initial = __pyx_state[8]; __pyx_result.label = __pyx_state[9]; __pyx_result.method_name = __pyx_state[10]; __pyx_result.read_only = __pyx_state[11]; __pyx_result.required = __pyx_state[12]; __pyx_result.root = __pyx_state[13]; __pyx_result.style = __pyx_state[14]; __pyx_result.validators = __pyx_state[15]; __pyx_result.write_only = __pyx_state[16]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_MethodField(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_MethodField", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_MethodField__set_state(<MethodField> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MethodField__set_state(MethodField __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.default_value = __pyx_state[4]; __pyx_result.error_messages = __pyx_state[5]; __pyx_result.field_name = __pyx_state[6]; __pyx_result.help_text = __pyx_state[7]; __pyx_result.initial = __pyx_state[8]; __pyx_result.label = __pyx_state[9]; __pyx_result.method_name = __pyx_state[10]; __pyx_result.read_only = __pyx_state[11]; __pyx_result.required = __pyx_state[12]; __pyx_result.root = __pyx_state[13]; __pyx_result.style = __pyx_state[14]; __pyx_result.validators = __pyx_state[15]; __pyx_result.write_only = __pyx_state[16]
 *     if len(__pyx_state) > 17 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_9drf_turbo_6fields___pyx_unpickle_MethodField__set_state(struct __pyx_obj_9drf_turbo_6fields_MethodField *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MethodField__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_MethodField__set_state(MethodField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.default_value = __pyx_state[4]; __pyx_result.error_messages = __pyx_state[5]; __pyx_result.field_name = __pyx_state[6]; __pyx_result.help_text = __pyx_state[7]; __pyx_result.initial = __pyx_state[8]; __pyx_result.label = __pyx_state[9]; __pyx_result.method_name = __pyx_state[10]; __pyx_result.read_only = __pyx_state[11]; __pyx_result.required = __pyx_state[12]; __pyx_result.root = __pyx_state[13]; __pyx_result.style = __pyx_state[14]; __pyx_result.validators = __pyx_state[15]; __pyx_result.write_only = __pyx_state[16]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 17 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[17])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.allow_null = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.attr);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.attr);
  __pyx_v___pyx_result->__pyx_base.attr = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.attrs);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.attrs);
  __pyx_v___pyx_result->__pyx_base.attrs = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.call = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.default_value);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.default_value);
  __pyx_v___pyx_result->__pyx_base.default_value = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.error_messages);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.error_messages);
  __pyx_v___pyx_result->__pyx_base.error_messages = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.field_name);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.field_name);
  __pyx_v___pyx_result->__pyx_base.field_name = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.help_text);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.help_text);
  __pyx_v___pyx_result->__pyx_base.help_text = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 8, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.initial);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.initial);
  __pyx_v___pyx_result->__pyx_base.initial = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 9, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.label);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.label);
  __pyx_v___pyx_result->__pyx_base.label = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 10, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->method_name);
  __Pyx_DECREF(__pyx_v___pyx_result->method_name);
  __pyx_v___pyx_result->method_name = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 11, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.read_only = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 12, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.required = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 13, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.root);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.root);
  __pyx_v___pyx_result->__pyx_base.root = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 14, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.style);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.style);
  __pyx_v___pyx_result->__pyx_base.style = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 15, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.validators);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.validators);
  __pyx_v___pyx_result->__pyx_base.validators = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 16, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.write_only = __pyx_t_2;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_MethodField__set_state(MethodField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.default_value = __pyx_state[4]; __pyx_result.error_messages = __pyx_state[5]; __pyx_result.field_name = __pyx_state[6]; __pyx_result.help_text = __pyx_state[7]; __pyx_result.initial = __pyx_state[8]; __pyx_result.label = __pyx_state[9]; __pyx_result.method_name = __pyx_state[10]; __pyx_result.read_only = __pyx_state[11]; __pyx_result.required = __pyx_state[12]; __pyx_result.root = __pyx_state[13]; __pyx_result.style = __pyx_state[14]; __pyx_result.validators = __pyx_state[15]; __pyx_result.write_only = __pyx_state[16]
 *     if len(__pyx_state) > 17 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[17])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 13, __pyx_L1_error)
  }
  __pyx_t_3 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_4 = ((__pyx_t_3 > 17) != 0);
  if (__pyx_t_4) {
  } else {
    __pyx_t_2 = __pyx_t_4;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_4 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_5 = (__pyx_t_4 != 0);
  __pyx_t_2 = __pyx_t_5;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":14
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.default_value = __pyx_state[4]; __pyx_result.error_messages = __pyx_state[5]; __pyx_result.field_name = __pyx_state[6]; __pyx_result.help_text = __pyx_state[7]; __pyx_result.initial = __pyx_state[8]; __pyx_result.label = __pyx_state[9]; __pyx_result.method_name = __pyx_state[10]; __pyx_result.read_only = __pyx_state[11]; __pyx_result.required = __pyx_state[12]; __pyx_result.root = __pyx_state[13]; __pyx_result.style = __pyx_state[14]; __pyx_result.validators = __pyx_state[15]; __pyx_result.write_only = __pyx_state[16]
 *     if len(__pyx_state) > 17 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[17])             # <<<<<<<<<<<<<<
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_update); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 14, __pyx_L1_error)
    }
    __pyx_t_6 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 17, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_8 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
      }
    }
    __pyx_t_1 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_8, __pyx_t_6) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_6);
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_MethodField__set_state(MethodField __pyx_result, tuple __pyx_state):
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.default_value = __pyx_state[4]; __pyx_result.error_messages = __pyx_state[5]; __pyx_result.field_name = __pyx_state[6]; __pyx_result.help_text = __pyx_state[7]; __pyx_result.initial = __pyx_state[8]; __pyx_result.label = __pyx_state[9]; __pyx_result.method_name = __pyx_state[10]; __pyx_result.read_only = __pyx_state[11]; __pyx_result.required = __pyx_state[12]; __pyx_result.root = __pyx_state[13]; __pyx_result.style = __pyx_state[14]; __pyx_result.validators = __pyx_state[15]; __pyx_result.write_only = __pyx_state[16]
 *     if len(__pyx_state) > 17 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[17])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_MethodField__set_state(<MethodField> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MethodField__set_state(MethodField __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.default_value = __pyx_state[4]; __pyx_result.error_messages = __pyx_state[5]; __pyx_result.field_name = __pyx_state[6]; __pyx_result.help_text = __pyx_state[7]; __pyx_result.initial = __pyx_state[8]; __pyx_result.label = __pyx_state[9]; __pyx_result.method_name = __pyx_state[10]; __pyx_result.read_only = __pyx_state[11]; __pyx_result.required = __pyx_state[12]; __pyx_result.root = __pyx_state[13]; __pyx_result.style = __pyx_state[14]; __pyx_result.validators = __pyx_state[15]; __pyx_result.write_only = __pyx_state[16]
 *     if len(__pyx_state) > 17 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("drf_turbo.fields.__pyx_unpickle_MethodField__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_tp_new_9drf_turbo_6fields_SkipField(PyTypeObject *t, PyObject *a, PyObject *k) {
  PyObject *o = (&((PyTypeObject*)PyExc_Exception)[0])->tp_new(t, a, k);
  if (unlikely(!o)) return 0;
  return o;
}

static void __pyx_tp_dealloc_9drf_turbo_6fields_SkipField(PyObject *o) {
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  PyObject_GC_Track(o);
  (&((PyTypeObject*)PyExc_Exception)[0])->tp_dealloc(o);
}

static int __pyx_tp_traverse_9drf_turbo_6fields_SkipField(PyObject *o, visitproc v, void *a) {
  int e;
  if (!(&((PyTypeObject*)PyExc_Exception)[0])->tp_traverse); else { e = (&((PyTypeObject*)PyExc_Exception)[0])->tp_traverse(o,v,a); if (e) return e; }
  return 0;
}

static int __pyx_tp_clear_9drf_turbo_6fields_SkipField(PyObject *o) {
  if (!(&((PyTypeObject*)PyExc_Exception)[0])->tp_clear); else (&((PyTypeObject*)PyExc_Exception)[0])->tp_clear(o);
  return 0;
}

static PyMethodDef __pyx_methods_9drf_turbo_6fields_SkipField[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_9drf_turbo_6fields_9SkipField_1__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_9drf_turbo_6fields_9SkipField_3__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_9drf_turbo_6fields_SkipField = {
  PyVarObject_HEAD_INIT(0, 0)
  "drf_turbo.fields.SkipField", /*tp_name*/
  sizeof(struct __pyx_obj_9drf_turbo_6fields_SkipField), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_9drf_turbo_6fields_SkipField, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_9drf_turbo_6fields_SkipField, /*tp_traverse*/
  __pyx_tp_clear_9drf_turbo_6fields_SkipField, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_9drf_turbo_6fields_SkipField, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_9drf_turbo_6fields_SkipField, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_9drf_turbo_6fields_Field __pyx_vtable_9drf_turbo_6fields_Field;

static PyObject *__pyx_tp_new_9drf_turbo_6fields_Field(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_9drf_turbo_6fields_Field *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_9drf_turbo_6fields_Field *)o);
  p->__pyx_vtab = __pyx_vtabptr_9drf_turbo_6fields_Field;
  p->attr = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->label = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->default_value = Py_None; Py_INCREF(Py_None);
  p->initial = Py_None; Py_INCREF(Py_None);
  p->help_text = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->style = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->field_name = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->root = Py_None; Py_INCREF(Py_None);
  p->validators = Py_None; Py_INCREF(Py_None);
  p->error_messages = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->attrs = ((PyObject*)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_9drf_turbo_6fields_Field(PyObject *o) {
  struct __pyx_obj_9drf_turbo_6fields_Field *p = (struct __pyx_obj_9drf_turbo_6fields_Field *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->attr);
  Py_CLEAR(p->label);
  Py_CLEAR(p->default_value);
  Py_CLEAR(p->initial);
  Py_CLEAR(p->help_text);
  Py_CLEAR(p->style);
  Py_CLEAR(p->field_name);
  Py_CLEAR(p->root);
  Py_CLEAR(p->validators);
  Py_CLEAR(p->error_messages);
  Py_CLEAR(p->attrs);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_9drf_turbo_6fields_Field(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_9drf_turbo_6fields_Field *p = (struct __pyx_obj_9drf_turbo_6fields_Field *)o;
  if (p->default_value) {
    e = (*v)(p->default_value, a); if (e) return e;
  }
  if (p->initial) {
    e = (*v)(p->initial, a); if (e) return e;
  }
  if (p->style) {
    e = (*v)(p->style, a); if (e) return e;
  }
  if (p->root) {
    e = (*v)(p->root, a); if (e) return e;
  }
  if (p->validators) {
    e = (*v)(p->validators, a); if (e) return e;
  }
  if (p->error_messages) {
    e = (*v)(p->error_messages, a); if (e) return e;
  }
  if (p->attrs) {
    e = (*v)(p->attrs, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_9drf_turbo_6fields_Field(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_9drf_turbo_6fields_Field *p = (struct __pyx_obj_9drf_turbo_6fields_Field *)o;
  tmp = ((PyObject*)p->default_value);
  p->default_value = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->initial);
  p->initial = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->style);
  p->style = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->root);
  p->root = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->validators);
  p->validators = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->error_messages);
  p->error_messages = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->attrs);
  p->attrs = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_9drf_turbo_6fields_5Field_attr(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9drf_turbo_6fields_5Field_4attr_1__get__(o);
}

static int __pyx_setprop_9drf_turbo_6fields_5Field_attr(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9drf_turbo_6fields_5Field_4attr_3__set__(o, v);
  }
  else {
    return __pyx_pw_9drf_turbo_6fields_5Field_4attr_5__del__(o);
  }
}

static PyObject *__pyx_getprop_9drf_turbo_6fields_5Field_call(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9drf_turbo_6fields_5Field_4call_1__get__(o);
}

static int __pyx_setprop_9drf_turbo_6fields_5Field_call(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9drf_turbo_6fields_5Field_4call_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_9drf_turbo_6fields_5Field_required(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9drf_turbo_6fields_5Field_8required_1__get__(o);
}

static int __pyx_setprop_9drf_turbo_6fields_5Field_required(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9drf_turbo_6fields_5Field_8required_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_9drf_turbo_6fields_5Field_write_only(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9drf_turbo_6fields_5Field_10write_only_1__get__(o);
}

static int __pyx_setprop_9drf_turbo_6fields_5Field_write_only(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9drf_turbo_6fields_5Field_10write_only_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_9drf_turbo_6fields_5Field_read_only(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9drf_turbo_6fields_5Field_9read_only_1__get__(o);
}

static int __pyx_setprop_9drf_turbo_6fields_5Field_read_only(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9drf_turbo_6fields_5Field_9read_only_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_9drf_turbo_6fields_5Field_allow_null(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9drf_turbo_6fields_5Field_10allow_null_1__get__(o);
}

static int __pyx_setprop_9drf_turbo_6fields_5Field_allow_null(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9drf_turbo_6fields_5Field_10allow_null_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_9drf_turbo_6fields_5Field_label(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9drf_turbo_6fields_5Field_5label_1__get__(o);
}

static int __pyx_setprop_9drf_turbo_6fields_5Field_label(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9drf_turbo_6fields_5Field_5label_3__set__(o, v);
  }
  else {
    return __pyx_pw_9drf_turbo_6fields_5Field_5label_5__del__(o);
  }
}

static PyObject *__pyx_getprop_9drf_turbo_6fields_5Field_default_value(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9drf_turbo_6fields_5Field_13default_value_1__get__(o);
}

static int __pyx_setprop_9drf_turbo_6fields_5Field_default_value(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9drf_turbo_6fields_5Field_13default_value_3__set__(o, v);
  }
  else {
    return __pyx_pw_9drf_turbo_6fields_5Field_13default_value_5__del__(o);
  }
}

static PyObject *__pyx_getprop_9drf_turbo_6fields_5Field_initial(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9drf_turbo_6fields_5Field_7initial_1__get__(o);
}

static int __pyx_setprop_9drf_turbo_6fields_5Field_initial(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9drf_turbo_6fields_5Field_7initial_3__set__(o, v);
  }
  else {
    return __pyx_pw_9drf_turbo_6fields_5Field_7initial_5__del__(o);
  }
}

static PyObject *__pyx_getprop_9drf_turbo_6fields_5Field_help_text(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9drf_turbo_6fields_5Field_9help_text_1__get__(o);
}

static int __pyx_setprop_9drf_turbo_6fields_5Field_help_text(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9drf_turbo_6fields_5Field_9help_text_3__set__(o, v);
  }
  else {
    return __pyx_pw_9drf_turbo_6fields_5Field_9help_text_5__del__(o);
  }
}

static PyObject *__pyx_getprop_9drf_turbo_6fields_5Field_style(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9drf_turbo_6fields_5Field_5style_1__get__(o);
}

static int __pyx_setprop_9drf_turbo_6fields_5Field_style(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9drf_turbo_6fields_5Field_5style_3__set__(o, v);
  }
  else {
    return __pyx_pw_9drf_turbo_6fields_5Field_5style_5__del__(o);
  }
}

static PyObject *__pyx_getprop_9drf_turbo_6fields_5Field_field_name(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9drf_turbo_6fields_5Field_10field_name_1__get__(o);
}

static int __pyx_setprop_9drf_turbo_6fields_5Field_field_name(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9drf_turbo_6fields_5Field_10field_name_3__set__(o, v);
  }
  else {
    return __pyx_pw_9drf_turbo_6fields_5Field_10field_name_5__del__(o);
  }
}

static PyObject *__pyx_getprop_9drf_turbo_6fields_5Field_root(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9drf_turbo_6fields_5Field_4root_1__get__(o);
}

static int __pyx_setprop_9drf_turbo_6fields_5Field_root(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9drf_turbo_6fields_5Field_4root_3__set__(o, v);
  }
  else {
    return __pyx_pw_9drf_turbo_6fields_5Field_4root_5__del__(o);
  }
}

static PyObject *__pyx_getprop_9drf_turbo_6fields_5Field_validators(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9drf_turbo_6fields_5Field_10validators_1__get__(o);
}

static int __pyx_setprop_9drf_turbo_6fields_5Field_validators(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9drf_turbo_6fields_5Field_10validators_3__set__(o, v);
  }
  else {
    return __pyx_pw_9drf_turbo_6fields_5Field_10validators_5__del__(o);
  }
}

static PyObject *__pyx_getprop_9drf_turbo_6fields_5Field_error_messages(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9drf_turbo_6fields_5Field_14error_messages_1__get__(o);
}

static int __pyx_setprop_9drf_turbo_6fields_5Field_error_messages(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9drf_turbo_6fields_5Field_14error_messages_3__set__(o, v);
  }
  else {
    return __pyx_pw_9drf_turbo_6fields_5Field_14error_messages_5__del__(o);
  }
}

static PyObject *__pyx_getprop_9drf_turbo_6fields_5Field_attrs(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9drf_turbo_6fields_5Field_5attrs_1__get__(o);
}

static int __pyx_setprop_9drf_turbo_6fields_5Field_attrs(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9drf_turbo_6fields_5Field_5attrs_3__set__(o, v);
  }
  else {
    return __pyx_pw_9drf_turbo_6fields_5Field_5attrs_5__del__(o);
  }
}

static PyMethodDef __pyx_methods_9drf_turbo_6fields_Field[] = {
  {"raise_if_fail", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_5Field_3raise_if_fail, METH_VARARGS|METH_KEYWORDS, __pyx_doc_9drf_turbo_6fields_5Field_2raise_if_fail},
  {"serialize", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_5Field_5serialize, METH_VARARGS|METH_KEYWORDS, __pyx_doc_9drf_turbo_6fields_5Field_4serialize},
  {"deserialize", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_5Field_7deserialize, METH_VARARGS|METH_KEYWORDS, __pyx_doc_9drf_turbo_6fields_5Field_6deserialize},
  {"method_getter", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_5Field_9method_getter, METH_VARARGS|METH_KEYWORDS, __pyx_doc_9drf_turbo_6fields_5Field_8method_getter},
  {"bind", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_5Field_11bind, METH_VARARGS|METH_KEYWORDS, __pyx_doc_9drf_turbo_6fields_5Field_10bind},
  {"get_default_value", (PyCFunction)__pyx_pw_9drf_turbo_6fields_5Field_13get_default_value, METH_NOARGS, __pyx_doc_9drf_turbo_6fields_5Field_12get_default_value},
  {"get_initial", (PyCFunction)__pyx_pw_9drf_turbo_6fields_5Field_15get_initial, METH_NOARGS, __pyx_doc_9drf_turbo_6fields_5Field_14get_initial},
  {"get_attribute", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_5Field_17get_attribute, METH_VARARGS|METH_KEYWORDS, __pyx_doc_9drf_turbo_6fields_5Field_16get_attribute},
  {"validate_empty_values", (PyCFunction)__pyx_pw_9drf_turbo_6fields_5Field_19validate_empty_values, METH_O, __pyx_doc_9drf_turbo_6fields_5Field_18validate_empty_values},
  {"run_validation", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_5Field_21run_validation, METH_VARARGS|METH_KEYWORDS, __pyx_doc_9drf_turbo_6fields_5Field_20run_validation},
  {"validate_or_raise", (PyCFunction)__pyx_pw_9drf_turbo_6fields_5Field_23validate_or_raise, METH_O, __pyx_doc_9drf_turbo_6fields_5Field_22validate_or_raise},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_9drf_turbo_6fields_5Field_25__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_9drf_turbo_6fields_5Field_27__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_9drf_turbo_6fields_Field[] = {
  {(char *)"attr", __pyx_getprop_9drf_turbo_6fields_5Field_attr, __pyx_setprop_9drf_turbo_6fields_5Field_attr, (char *)0, 0},
  {(char *)"call", __pyx_getprop_9drf_turbo_6fields_5Field_call, __pyx_setprop_9drf_turbo_6fields_5Field_call, (char *)0, 0},
  {(char *)"required", __pyx_getprop_9drf_turbo_6fields_5Field_required, __pyx_setprop_9drf_turbo_6fields_5Field_required, (char *)0, 0},
  {(char *)"write_only", __pyx_getprop_9drf_turbo_6fields_5Field_write_only, __pyx_setprop_9drf_turbo_6fields_5Field_write_only, (char *)0, 0},
  {(char *)"read_only", __pyx_getprop_9drf_turbo_6fields_5Field_read_only, __pyx_setprop_9drf_turbo_6fields_5Field_read_only, (char *)0, 0},
  {(char *)"allow_null", __pyx_getprop_9drf_turbo_6fields_5Field_allow_null, __pyx_setprop_9drf_turbo_6fields_5Field_allow_null, (char *)0, 0},
  {(char *)"label", __pyx_getprop_9drf_turbo_6fields_5Field_label, __pyx_setprop_9drf_turbo_6fields_5Field_label, (char *)0, 0},
  {(char *)"default_value", __pyx_getprop_9drf_turbo_6fields_5Field_default_value, __pyx_setprop_9drf_turbo_6fields_5Field_default_value, (char *)0, 0},
  {(char *)"initial", __pyx_getprop_9drf_turbo_6fields_5Field_initial, __pyx_setprop_9drf_turbo_6fields_5Field_initial, (char *)0, 0},
  {(char *)"help_text", __pyx_getprop_9drf_turbo_6fields_5Field_help_text, __pyx_setprop_9drf_turbo_6fields_5Field_help_text, (char *)0, 0},
  {(char *)"style", __pyx_getprop_9drf_turbo_6fields_5Field_style, __pyx_setprop_9drf_turbo_6fields_5Field_style, (char *)0, 0},
  {(char *)"field_name", __pyx_getprop_9drf_turbo_6fields_5Field_field_name, __pyx_setprop_9drf_turbo_6fields_5Field_field_name, (char *)0, 0},
  {(char *)"root", __pyx_getprop_9drf_turbo_6fields_5Field_root, __pyx_setprop_9drf_turbo_6fields_5Field_root, (char *)0, 0},
  {(char *)"validators", __pyx_getprop_9drf_turbo_6fields_5Field_validators, __pyx_setprop_9drf_turbo_6fields_5Field_validators, (char *)0, 0},
  {(char *)"error_messages", __pyx_getprop_9drf_turbo_6fields_5Field_error_messages, __pyx_setprop_9drf_turbo_6fields_5Field_error_messages, (char *)0, 0},
  {(char *)"attrs", __pyx_getprop_9drf_turbo_6fields_5Field_attrs, __pyx_setprop_9drf_turbo_6fields_5Field_attrs, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_9drf_turbo_6fields_Field = {
  PyVarObject_HEAD_INIT(0, 0)
  "drf_turbo.fields.Field", /*tp_name*/
  sizeof(struct __pyx_obj_9drf_turbo_6fields_Field), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_9drf_turbo_6fields_Field, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "\n    Basic field from which other fields should extend. It applies no\n    formatting by default, and should only be used in cases where\n    data does not need to be formatted before being serialized or deserialized.\n    On error, the name of the field will be returned.\n\n    :param str attr:  The name of the attribute to get the value from when serializing.\n    :param bool call: Whether the value should be called after it is retrieved\n        from the object. Useful if an object has a method to be serialized. \n    :param bool required: Raise a `ValidationError` if the field value\n        is not supplied during deserialization.      \n    :param bool write_only: If `True` skip this field during serialization, otherwise\n        its value will be present in the serialized data.      \n    :param bool read_only: If `True` skip this field during deserialization, otherwise\n        its value will be present in the deserialized object.  \n    :param str label: A label to use as the name of the serialized field\n        instead of using the attribute name of the field.   \n    :param list validators: A list of validators to apply against incoming data during deserialization.     \n    :param str field_name: The name of field.  \n    :param str root: The root(parent) of field.   \n    :param default_value: Default value to be used during serialization and deserialization.\n    :param initial: The initial value for the field.\n    ", /*tp_doc*/
  __pyx_tp_traverse_9drf_turbo_6fields_Field, /*tp_traverse*/
  __pyx_tp_clear_9drf_turbo_6fields_Field, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_9drf_turbo_6fields_Field, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_9drf_turbo_6fields_Field, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_9drf_turbo_6fields_5Field_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_9drf_turbo_6fields_Field, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_9drf_turbo_6fields_StrField __pyx_vtable_9drf_turbo_6fields_StrField;

static PyObject *__pyx_tp_new_9drf_turbo_6fields_StrField(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_9drf_turbo_6fields_StrField *p;
  PyObject *o = __pyx_tp_new_9drf_turbo_6fields_Field(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_9drf_turbo_6fields_StrField *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_9drf_turbo_6fields_Field*)__pyx_vtabptr_9drf_turbo_6fields_StrField;
  p->allow_blank = Py_None; Py_INCREF(Py_None);
  p->trim_whitespace = Py_None; Py_INCREF(Py_None);
  p->max_length = Py_None; Py_INCREF(Py_None);
  p->min_length = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_9drf_turbo_6fields_StrField(PyObject *o) {
  struct __pyx_obj_9drf_turbo_6fields_StrField *p = (struct __pyx_obj_9drf_turbo_6fields_StrField *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->allow_blank);
  Py_CLEAR(p->trim_whitespace);
  Py_CLEAR(p->max_length);
  Py_CLEAR(p->min_length);
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_9drf_turbo_6fields_Field(o);
}

static int __pyx_tp_traverse_9drf_turbo_6fields_StrField(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_9drf_turbo_6fields_StrField *p = (struct __pyx_obj_9drf_turbo_6fields_StrField *)o;
  e = __pyx_tp_traverse_9drf_turbo_6fields_Field(o, v, a); if (e) return e;
  if (p->allow_blank) {
    e = (*v)(p->allow_blank, a); if (e) return e;
  }
  if (p->trim_whitespace) {
    e = (*v)(p->trim_whitespace, a); if (e) return e;
  }
  if (p->max_length) {
    e = (*v)(p->max_length, a); if (e) return e;
  }
  if (p->min_length) {
    e = (*v)(p->min_length, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_9drf_turbo_6fields_StrField(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_9drf_turbo_6fields_StrField *p = (struct __pyx_obj_9drf_turbo_6fields_StrField *)o;
  __pyx_tp_clear_9drf_turbo_6fields_Field(o);
  tmp = ((PyObject*)p->allow_blank);
  p->allow_blank = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->trim_whitespace);
  p->trim_whitespace = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->max_length);
  p->max_length = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->min_length);
  p->min_length = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_9drf_turbo_6fields_8StrField_allow_blank(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9drf_turbo_6fields_8StrField_11allow_blank_1__get__(o);
}

static int __pyx_setprop_9drf_turbo_6fields_8StrField_allow_blank(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9drf_turbo_6fields_8StrField_11allow_blank_3__set__(o, v);
  }
  else {
    return __pyx_pw_9drf_turbo_6fields_8StrField_11allow_blank_5__del__(o);
  }
}

static PyObject *__pyx_getprop_9drf_turbo_6fields_8StrField_trim_whitespace(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9drf_turbo_6fields_8StrField_15trim_whitespace_1__get__(o);
}

static int __pyx_setprop_9drf_turbo_6fields_8StrField_trim_whitespace(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9drf_turbo_6fields_8StrField_15trim_whitespace_3__set__(o, v);
  }
  else {
    return __pyx_pw_9drf_turbo_6fields_8StrField_15trim_whitespace_5__del__(o);
  }
}

static PyObject *__pyx_getprop_9drf_turbo_6fields_8StrField_max_length(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9drf_turbo_6fields_8StrField_10max_length_1__get__(o);
}

static int __pyx_setprop_9drf_turbo_6fields_8StrField_max_length(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9drf_turbo_6fields_8StrField_10max_length_3__set__(o, v);
  }
  else {
    return __pyx_pw_9drf_turbo_6fields_8StrField_10max_length_5__del__(o);
  }
}

static PyObject *__pyx_getprop_9drf_turbo_6fields_8StrField_min_length(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9drf_turbo_6fields_8StrField_10min_length_1__get__(o);
}

static int __pyx_setprop_9drf_turbo_6fields_8StrField_min_length(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9drf_turbo_6fields_8StrField_10min_length_3__set__(o, v);
  }
  else {
    return __pyx_pw_9drf_turbo_6fields_8StrField_10min_length_5__del__(o);
  }
}

static PyMethodDef __pyx_methods_9drf_turbo_6fields_StrField[] = {
  {"serialize", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_8StrField_3serialize, METH_VARARGS|METH_KEYWORDS, 0},
  {"deserialize", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_8StrField_5deserialize, METH_VARARGS|METH_KEYWORDS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_9drf_turbo_6fields_8StrField_7__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_9drf_turbo_6fields_8StrField_9__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_9drf_turbo_6fields_StrField[] = {
  {(char *)"allow_blank", __pyx_getprop_9drf_turbo_6fields_8StrField_allow_blank, __pyx_setprop_9drf_turbo_6fields_8StrField_allow_blank, (char *)0, 0},
  {(char *)"trim_whitespace", __pyx_getprop_9drf_turbo_6fields_8StrField_trim_whitespace, __pyx_setprop_9drf_turbo_6fields_8StrField_trim_whitespace, (char *)0, 0},
  {(char *)"max_length", __pyx_getprop_9drf_turbo_6fields_8StrField_max_length, __pyx_setprop_9drf_turbo_6fields_8StrField_max_length, (char *)0, 0},
  {(char *)"min_length", __pyx_getprop_9drf_turbo_6fields_8StrField_min_length, __pyx_setprop_9drf_turbo_6fields_8StrField_min_length, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_9drf_turbo_6fields_StrField = {
  PyVarObject_HEAD_INIT(0, 0)
  "drf_turbo.fields.StrField", /*tp_name*/
  sizeof(struct __pyx_obj_9drf_turbo_6fields_StrField), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_9drf_turbo_6fields_StrField, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "\"\n    A field that validates input as an string.\n\n    :param kwargs: The same keyword arguments that :class:`Field` receives.\n    ", /*tp_doc*/
  __pyx_tp_traverse_9drf_turbo_6fields_StrField, /*tp_traverse*/
  __pyx_tp_clear_9drf_turbo_6fields_StrField, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_9drf_turbo_6fields_StrField, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_9drf_turbo_6fields_StrField, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_9drf_turbo_6fields_8StrField_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_9drf_turbo_6fields_StrField, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_9drf_turbo_6fields_EmailField __pyx_vtable_9drf_turbo_6fields_EmailField;

static PyObject *__pyx_tp_new_9drf_turbo_6fields_EmailField(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_9drf_turbo_6fields_EmailField *p;
  PyObject *o = __pyx_tp_new_9drf_turbo_6fields_StrField(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_9drf_turbo_6fields_EmailField *)o);
  p->__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_9drf_turbo_6fields_Field*)__pyx_vtabptr_9drf_turbo_6fields_EmailField;
  return o;
}

static PyObject *__pyx_getprop_9drf_turbo_6fields_10EmailField_to_lower(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9drf_turbo_6fields_10EmailField_8to_lower_1__get__(o);
}

static int __pyx_setprop_9drf_turbo_6fields_10EmailField_to_lower(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9drf_turbo_6fields_10EmailField_8to_lower_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyMethodDef __pyx_methods_9drf_turbo_6fields_EmailField[] = {
  {"serialize", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_10EmailField_3serialize, METH_VARARGS|METH_KEYWORDS, 0},
  {"deserialize", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_10EmailField_5deserialize, METH_VARARGS|METH_KEYWORDS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_9drf_turbo_6fields_10EmailField_7__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_9drf_turbo_6fields_10EmailField_9__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_9drf_turbo_6fields_EmailField[] = {
  {(char *)"to_lower", __pyx_getprop_9drf_turbo_6fields_10EmailField_to_lower, __pyx_setprop_9drf_turbo_6fields_10EmailField_to_lower, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_9drf_turbo_6fields_EmailField = {
  PyVarObject_HEAD_INIT(0, 0)
  "drf_turbo.fields.EmailField", /*tp_name*/
  sizeof(struct __pyx_obj_9drf_turbo_6fields_EmailField), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_9drf_turbo_6fields_StrField, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "\n    A field that validates input as an E-Mail address.\n\n    :param to_lower: If True, convert the value to lowercase before validating.\n    :param kwargs: The same keyword arguments that :class:`Field` receives.\n    ", /*tp_doc*/
  __pyx_tp_traverse_9drf_turbo_6fields_StrField, /*tp_traverse*/
  __pyx_tp_clear_9drf_turbo_6fields_StrField, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_9drf_turbo_6fields_EmailField, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_9drf_turbo_6fields_EmailField, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_9drf_turbo_6fields_10EmailField_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_9drf_turbo_6fields_EmailField, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_9drf_turbo_6fields_URLField __pyx_vtable_9drf_turbo_6fields_URLField;

static PyObject *__pyx_tp_new_9drf_turbo_6fields_URLField(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_9drf_turbo_6fields_URLField *p;
  PyObject *o = __pyx_tp_new_9drf_turbo_6fields_StrField(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_9drf_turbo_6fields_URLField *)o);
  p->__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_9drf_turbo_6fields_Field*)__pyx_vtabptr_9drf_turbo_6fields_URLField;
  return o;
}

static PyMethodDef __pyx_methods_9drf_turbo_6fields_URLField[] = {
  {"deserialize", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_8URLField_1deserialize, METH_VARARGS|METH_KEYWORDS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_9drf_turbo_6fields_8URLField_3__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_9drf_turbo_6fields_8URLField_5__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_9drf_turbo_6fields_URLField = {
  PyVarObject_HEAD_INIT(0, 0)
  "drf_turbo.fields.URLField", /*tp_name*/
  sizeof(struct __pyx_obj_9drf_turbo_6fields_URLField), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_9drf_turbo_6fields_StrField, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "\n    A field that validates input as an URL.\n\n    :param kwargs: The same keyword arguments that :class:`Field` receives.\n    ", /*tp_doc*/
  __pyx_tp_traverse_9drf_turbo_6fields_StrField, /*tp_traverse*/
  __pyx_tp_clear_9drf_turbo_6fields_StrField, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_9drf_turbo_6fields_URLField, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  #if CYTHON_COMPILING_IN_PYPY
  __pyx_pw_9drf_turbo_6fields_8StrField_1__init__, /*tp_init*/
  #else
  0, /*tp_init*/
  #endif
  0, /*tp_alloc*/
  __pyx_tp_new_9drf_turbo_6fields_URLField, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_9drf_turbo_6fields_RegexField __pyx_vtable_9drf_turbo_6fields_RegexField;

static PyObject *__pyx_tp_new_9drf_turbo_6fields_RegexField(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_9drf_turbo_6fields_RegexField *p;
  PyObject *o = __pyx_tp_new_9drf_turbo_6fields_StrField(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_9drf_turbo_6fields_RegexField *)o);
  p->__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_9drf_turbo_6fields_Field*)__pyx_vtabptr_9drf_turbo_6fields_RegexField;
  p->regex = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_9drf_turbo_6fields_RegexField(PyObject *o) {
  struct __pyx_obj_9drf_turbo_6fields_RegexField *p = (struct __pyx_obj_9drf_turbo_6fields_RegexField *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->regex);
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_9drf_turbo_6fields_StrField(o);
}

static int __pyx_tp_traverse_9drf_turbo_6fields_RegexField(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_9drf_turbo_6fields_RegexField *p = (struct __pyx_obj_9drf_turbo_6fields_RegexField *)o;
  e = __pyx_tp_traverse_9drf_turbo_6fields_StrField(o, v, a); if (e) return e;
  if (p->regex) {
    e = (*v)(p->regex, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_9drf_turbo_6fields_RegexField(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_9drf_turbo_6fields_RegexField *p = (struct __pyx_obj_9drf_turbo_6fields_RegexField *)o;
  __pyx_tp_clear_9drf_turbo_6fields_StrField(o);
  tmp = ((PyObject*)p->regex);
  p->regex = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_9drf_turbo_6fields_10RegexField_regex(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9drf_turbo_6fields_10RegexField_5regex_1__get__(o);
}

static int __pyx_setprop_9drf_turbo_6fields_10RegexField_regex(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9drf_turbo_6fields_10RegexField_5regex_3__set__(o, v);
  }
  else {
    return __pyx_pw_9drf_turbo_6fields_10RegexField_5regex_5__del__(o);
  }
}

static PyMethodDef __pyx_methods_9drf_turbo_6fields_RegexField[] = {
  {"deserialize", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_10RegexField_3deserialize, METH_VARARGS|METH_KEYWORDS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_9drf_turbo_6fields_10RegexField_5__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_9drf_turbo_6fields_10RegexField_7__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_9drf_turbo_6fields_RegexField[] = {
  {(char *)"regex", __pyx_getprop_9drf_turbo_6fields_10RegexField_regex, __pyx_setprop_9drf_turbo_6fields_10RegexField_regex, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_9drf_turbo_6fields_RegexField = {
  PyVarObject_HEAD_INIT(0, 0)
  "drf_turbo.fields.RegexField", /*tp_name*/
  sizeof(struct __pyx_obj_9drf_turbo_6fields_RegexField), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_9drf_turbo_6fields_RegexField, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "\n    A field that validates input against a given regular expression.\n\n    :param regex: The regular expression to use for validation.\n    :param kwargs: The same keyword arguments that :class:`Field` receives.\n    ", /*tp_doc*/
  __pyx_tp_traverse_9drf_turbo_6fields_RegexField, /*tp_traverse*/
  __pyx_tp_clear_9drf_turbo_6fields_RegexField, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_9drf_turbo_6fields_RegexField, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_9drf_turbo_6fields_RegexField, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_9drf_turbo_6fields_10RegexField_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_9drf_turbo_6fields_RegexField, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_9drf_turbo_6fields_IPField __pyx_vtable_9drf_turbo_6fields_IPField;

static PyObject *__pyx_tp_new_9drf_turbo_6fields_IPField(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_9drf_turbo_6fields_IPField *p;
  PyObject *o = __pyx_tp_new_9drf_turbo_6fields_StrField(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_9drf_turbo_6fields_IPField *)o);
  p->__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_9drf_turbo_6fields_Field*)__pyx_vtabptr_9drf_turbo_6fields_IPField;
  return o;
}

static PyMethodDef __pyx_methods_9drf_turbo_6fields_IPField[] = {
  {"deserialize", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_7IPField_1deserialize, METH_VARARGS|METH_KEYWORDS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_9drf_turbo_6fields_7IPField_3__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_9drf_turbo_6fields_7IPField_5__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_9drf_turbo_6fields_IPField = {
  PyVarObject_HEAD_INIT(0, 0)
  "drf_turbo.fields.IPField", /*tp_name*/
  sizeof(struct __pyx_obj_9drf_turbo_6fields_IPField), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_9drf_turbo_6fields_StrField, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "\n    A field that validates that input is an IP address.\n\n    :param kwargs: The same keyword arguments that :class:`Field` receives.\n    ", /*tp_doc*/
  __pyx_tp_traverse_9drf_turbo_6fields_StrField, /*tp_traverse*/
  __pyx_tp_clear_9drf_turbo_6fields_StrField, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_9drf_turbo_6fields_IPField, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  #if CYTHON_COMPILING_IN_PYPY
  __pyx_pw_9drf_turbo_6fields_8StrField_1__init__, /*tp_init*/
  #else
  0, /*tp_init*/
  #endif
  0, /*tp_alloc*/
  __pyx_tp_new_9drf_turbo_6fields_IPField, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_9drf_turbo_6fields_PasswordField __pyx_vtable_9drf_turbo_6fields_PasswordField;

static PyObject *__pyx_tp_new_9drf_turbo_6fields_PasswordField(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_9drf_turbo_6fields_PasswordField *p;
  PyObject *o = __pyx_tp_new_9drf_turbo_6fields_StrField(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_9drf_turbo_6fields_PasswordField *)o);
  p->__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_9drf_turbo_6fields_Field*)__pyx_vtabptr_9drf_turbo_6fields_PasswordField;
  return o;
}

static PyMethodDef __pyx_methods_9drf_turbo_6fields_PasswordField[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_9drf_turbo_6fields_13PasswordField_3__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_9drf_turbo_6fields_13PasswordField_5__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_9drf_turbo_6fields_PasswordField = {
  PyVarObject_HEAD_INIT(0, 0)
  "drf_turbo.fields.PasswordField", /*tp_name*/
  sizeof(struct __pyx_obj_9drf_turbo_6fields_PasswordField), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_9drf_turbo_6fields_StrField, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "\n    A field that validates input as a password.\n\n    :param kwargs: The same keyword arguments that :class:`Field` receives.\n    ", /*tp_doc*/
  __pyx_tp_traverse_9drf_turbo_6fields_StrField, /*tp_traverse*/
  __pyx_tp_clear_9drf_turbo_6fields_StrField, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_9drf_turbo_6fields_PasswordField, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_9drf_turbo_6fields_13PasswordField_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_9drf_turbo_6fields_PasswordField, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_9drf_turbo_6fields_UUIDField __pyx_vtable_9drf_turbo_6fields_UUIDField;

static PyObject *__pyx_tp_new_9drf_turbo_6fields_UUIDField(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_9drf_turbo_6fields_UUIDField *p;
  PyObject *o = __pyx_tp_new_9drf_turbo_6fields_StrField(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_9drf_turbo_6fields_UUIDField *)o);
  p->__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_9drf_turbo_6fields_Field*)__pyx_vtabptr_9drf_turbo_6fields_UUIDField;
  return o;
}

static PyMethodDef __pyx_methods_9drf_turbo_6fields_UUIDField[] = {
  {"deserialize", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_9UUIDField_1deserialize, METH_VARARGS|METH_KEYWORDS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_9drf_turbo_6fields_9UUIDField_3__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_9drf_turbo_6fields_9UUIDField_5__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_9drf_turbo_6fields_UUIDField = {
  PyVarObject_HEAD_INIT(0, 0)
  "drf_turbo.fields.UUIDField", /*tp_name*/
  sizeof(struct __pyx_obj_9drf_turbo_6fields_UUIDField), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_9drf_turbo_6fields_StrField, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "\n    A field that validates input as an UUID.\n\n    :param kwargs: The same keyword arguments that :class:`Field` receives.\n    ", /*tp_doc*/
  __pyx_tp_traverse_9drf_turbo_6fields_StrField, /*tp_traverse*/
  __pyx_tp_clear_9drf_turbo_6fields_StrField, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_9drf_turbo_6fields_UUIDField, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  #if CYTHON_COMPILING_IN_PYPY
  __pyx_pw_9drf_turbo_6fields_8StrField_1__init__, /*tp_init*/
  #else
  0, /*tp_init*/
  #endif
  0, /*tp_alloc*/
  __pyx_tp_new_9drf_turbo_6fields_UUIDField, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_9drf_turbo_6fields_SlugField __pyx_vtable_9drf_turbo_6fields_SlugField;

static PyObject *__pyx_tp_new_9drf_turbo_6fields_SlugField(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_9drf_turbo_6fields_SlugField *p;
  PyObject *o = __pyx_tp_new_9drf_turbo_6fields_Field(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_9drf_turbo_6fields_SlugField *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_9drf_turbo_6fields_Field*)__pyx_vtabptr_9drf_turbo_6fields_SlugField;
  p->allow_unicode = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_9drf_turbo_6fields_SlugField(PyObject *o) {
  struct __pyx_obj_9drf_turbo_6fields_SlugField *p = (struct __pyx_obj_9drf_turbo_6fields_SlugField *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->allow_unicode);
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_9drf_turbo_6fields_Field(o);
}

static int __pyx_tp_traverse_9drf_turbo_6fields_SlugField(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_9drf_turbo_6fields_SlugField *p = (struct __pyx_obj_9drf_turbo_6fields_SlugField *)o;
  e = __pyx_tp_traverse_9drf_turbo_6fields_Field(o, v, a); if (e) return e;
  if (p->allow_unicode) {
    e = (*v)(p->allow_unicode, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_9drf_turbo_6fields_SlugField(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_9drf_turbo_6fields_SlugField *p = (struct __pyx_obj_9drf_turbo_6fields_SlugField *)o;
  __pyx_tp_clear_9drf_turbo_6fields_Field(o);
  tmp = ((PyObject*)p->allow_unicode);
  p->allow_unicode = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_9drf_turbo_6fields_9SlugField_allow_unicode(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9drf_turbo_6fields_9SlugField_13allow_unicode_1__get__(o);
}

static int __pyx_setprop_9drf_turbo_6fields_9SlugField_allow_unicode(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9drf_turbo_6fields_9SlugField_13allow_unicode_3__set__(o, v);
  }
  else {
    return __pyx_pw_9drf_turbo_6fields_9SlugField_13allow_unicode_5__del__(o);
  }
}

static PyMethodDef __pyx_methods_9drf_turbo_6fields_SlugField[] = {
  {"deserialize", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_9SlugField_3deserialize, METH_VARARGS|METH_KEYWORDS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_9drf_turbo_6fields_9SlugField_5__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_9drf_turbo_6fields_9SlugField_7__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_9drf_turbo_6fields_SlugField[] = {
  {(char *)"allow_unicode", __pyx_getprop_9drf_turbo_6fields_9SlugField_allow_unicode, __pyx_setprop_9drf_turbo_6fields_9SlugField_allow_unicode, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_9drf_turbo_6fields_SlugField = {
  PyVarObject_HEAD_INIT(0, 0)
  "drf_turbo.fields.SlugField", /*tp_name*/
  sizeof(struct __pyx_obj_9drf_turbo_6fields_SlugField), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_9drf_turbo_6fields_SlugField, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "\n    Slug field type.\n\n    :param allow_unicode: If True, allow unicode characters in the field.\n    :param kwargs: The same keyword arguments that :class:`Field` receives.\n    ", /*tp_doc*/
  __pyx_tp_traverse_9drf_turbo_6fields_SlugField, /*tp_traverse*/
  __pyx_tp_clear_9drf_turbo_6fields_SlugField, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_9drf_turbo_6fields_SlugField, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_9drf_turbo_6fields_SlugField, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_9drf_turbo_6fields_9SlugField_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_9drf_turbo_6fields_SlugField, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_9drf_turbo_6fields_IntField __pyx_vtable_9drf_turbo_6fields_IntField;

static PyObject *__pyx_tp_new_9drf_turbo_6fields_IntField(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_9drf_turbo_6fields_IntField *p;
  PyObject *o = __pyx_tp_new_9drf_turbo_6fields_Field(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_9drf_turbo_6fields_IntField *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_9drf_turbo_6fields_Field*)__pyx_vtabptr_9drf_turbo_6fields_IntField;
  p->max_value = Py_None; Py_INCREF(Py_None);
  p->min_value = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_9drf_turbo_6fields_IntField(PyObject *o) {
  struct __pyx_obj_9drf_turbo_6fields_IntField *p = (struct __pyx_obj_9drf_turbo_6fields_IntField *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->max_value);
  Py_CLEAR(p->min_value);
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_9drf_turbo_6fields_Field(o);
}

static int __pyx_tp_traverse_9drf_turbo_6fields_IntField(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_9drf_turbo_6fields_IntField *p = (struct __pyx_obj_9drf_turbo_6fields_IntField *)o;
  e = __pyx_tp_traverse_9drf_turbo_6fields_Field(o, v, a); if (e) return e;
  if (p->max_value) {
    e = (*v)(p->max_value, a); if (e) return e;
  }
  if (p->min_value) {
    e = (*v)(p->min_value, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_9drf_turbo_6fields_IntField(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_9drf_turbo_6fields_IntField *p = (struct __pyx_obj_9drf_turbo_6fields_IntField *)o;
  __pyx_tp_clear_9drf_turbo_6fields_Field(o);
  tmp = ((PyObject*)p->max_value);
  p->max_value = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->min_value);
  p->min_value = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_9drf_turbo_6fields_8IntField_max_value(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9drf_turbo_6fields_8IntField_9max_value_1__get__(o);
}

static int __pyx_setprop_9drf_turbo_6fields_8IntField_max_value(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9drf_turbo_6fields_8IntField_9max_value_3__set__(o, v);
  }
  else {
    return __pyx_pw_9drf_turbo_6fields_8IntField_9max_value_5__del__(o);
  }
}

static PyObject *__pyx_getprop_9drf_turbo_6fields_8IntField_min_value(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9drf_turbo_6fields_8IntField_9min_value_1__get__(o);
}

static int __pyx_setprop_9drf_turbo_6fields_8IntField_min_value(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9drf_turbo_6fields_8IntField_9min_value_3__set__(o, v);
  }
  else {
    return __pyx_pw_9drf_turbo_6fields_8IntField_9min_value_5__del__(o);
  }
}

static PyMethodDef __pyx_methods_9drf_turbo_6fields_IntField[] = {
  {"serialize", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_8IntField_3serialize, METH_VARARGS|METH_KEYWORDS, 0},
  {"deserialize", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_8IntField_5deserialize, METH_VARARGS|METH_KEYWORDS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_9drf_turbo_6fields_8IntField_7__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_9drf_turbo_6fields_8IntField_9__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_9drf_turbo_6fields_IntField[] = {
  {(char *)"max_value", __pyx_getprop_9drf_turbo_6fields_8IntField_max_value, __pyx_setprop_9drf_turbo_6fields_8IntField_max_value, (char *)0, 0},
  {(char *)"min_value", __pyx_getprop_9drf_turbo_6fields_8IntField_min_value, __pyx_setprop_9drf_turbo_6fields_8IntField_min_value, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_9drf_turbo_6fields_IntField = {
  PyVarObject_HEAD_INIT(0, 0)
  "drf_turbo.fields.IntField", /*tp_name*/
  sizeof(struct __pyx_obj_9drf_turbo_6fields_IntField), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_9drf_turbo_6fields_IntField, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "\n    A field that validates input as an integer.\n\n    :param min_value: The minimum value allowed.\n    :param max_value : The maximum value allowed.\n    :param kwargs: The same keyword arguments that :class:`Field` receives.\n    ", /*tp_doc*/
  __pyx_tp_traverse_9drf_turbo_6fields_IntField, /*tp_traverse*/
  __pyx_tp_clear_9drf_turbo_6fields_IntField, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_9drf_turbo_6fields_IntField, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_9drf_turbo_6fields_IntField, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_9drf_turbo_6fields_8IntField_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_9drf_turbo_6fields_IntField, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_9drf_turbo_6fields_FloatField __pyx_vtable_9drf_turbo_6fields_FloatField;

static PyObject *__pyx_tp_new_9drf_turbo_6fields_FloatField(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_9drf_turbo_6fields_FloatField *p;
  PyObject *o = __pyx_tp_new_9drf_turbo_6fields_Field(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_9drf_turbo_6fields_FloatField *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_9drf_turbo_6fields_Field*)__pyx_vtabptr_9drf_turbo_6fields_FloatField;
  p->max_value = Py_None; Py_INCREF(Py_None);
  p->min_value = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_9drf_turbo_6fields_FloatField(PyObject *o) {
  struct __pyx_obj_9drf_turbo_6fields_FloatField *p = (struct __pyx_obj_9drf_turbo_6fields_FloatField *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->max_value);
  Py_CLEAR(p->min_value);
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_9drf_turbo_6fields_Field(o);
}

static int __pyx_tp_traverse_9drf_turbo_6fields_FloatField(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_9drf_turbo_6fields_FloatField *p = (struct __pyx_obj_9drf_turbo_6fields_FloatField *)o;
  e = __pyx_tp_traverse_9drf_turbo_6fields_Field(o, v, a); if (e) return e;
  if (p->max_value) {
    e = (*v)(p->max_value, a); if (e) return e;
  }
  if (p->min_value) {
    e = (*v)(p->min_value, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_9drf_turbo_6fields_FloatField(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_9drf_turbo_6fields_FloatField *p = (struct __pyx_obj_9drf_turbo_6fields_FloatField *)o;
  __pyx_tp_clear_9drf_turbo_6fields_Field(o);
  tmp = ((PyObject*)p->max_value);
  p->max_value = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->min_value);
  p->min_value = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_9drf_turbo_6fields_10FloatField_max_value(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9drf_turbo_6fields_10FloatField_9max_value_1__get__(o);
}

static int __pyx_setprop_9drf_turbo_6fields_10FloatField_max_value(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9drf_turbo_6fields_10FloatField_9max_value_3__set__(o, v);
  }
  else {
    return __pyx_pw_9drf_turbo_6fields_10FloatField_9max_value_5__del__(o);
  }
}

static PyObject *__pyx_getprop_9drf_turbo_6fields_10FloatField_min_value(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9drf_turbo_6fields_10FloatField_9min_value_1__get__(o);
}

static int __pyx_setprop_9drf_turbo_6fields_10FloatField_min_value(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9drf_turbo_6fields_10FloatField_9min_value_3__set__(o, v);
  }
  else {
    return __pyx_pw_9drf_turbo_6fields_10FloatField_9min_value_5__del__(o);
  }
}

static PyMethodDef __pyx_methods_9drf_turbo_6fields_FloatField[] = {
  {"serialize", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_10FloatField_3serialize, METH_VARARGS|METH_KEYWORDS, 0},
  {"deserialize", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_10FloatField_5deserialize, METH_VARARGS|METH_KEYWORDS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_9drf_turbo_6fields_10FloatField_7__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_9drf_turbo_6fields_10FloatField_9__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_9drf_turbo_6fields_FloatField[] = {
  {(char *)"max_value", __pyx_getprop_9drf_turbo_6fields_10FloatField_max_value, __pyx_setprop_9drf_turbo_6fields_10FloatField_max_value, (char *)0, 0},
  {(char *)"min_value", __pyx_getprop_9drf_turbo_6fields_10FloatField_min_value, __pyx_setprop_9drf_turbo_6fields_10FloatField_min_value, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_9drf_turbo_6fields_FloatField = {
  PyVarObject_HEAD_INIT(0, 0)
  "drf_turbo.fields.FloatField", /*tp_name*/
  sizeof(struct __pyx_obj_9drf_turbo_6fields_FloatField), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_9drf_turbo_6fields_FloatField, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "\n    A field that validates input as a float.\n\n    :param min_value: The minimum value allowed.\n    :param max_value: The maximum value allowed.\n    :param kwargs: The same keyword arguments that :class:`Field` receives.\n    ", /*tp_doc*/
  __pyx_tp_traverse_9drf_turbo_6fields_FloatField, /*tp_traverse*/
  __pyx_tp_clear_9drf_turbo_6fields_FloatField, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_9drf_turbo_6fields_FloatField, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_9drf_turbo_6fields_FloatField, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_9drf_turbo_6fields_10FloatField_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_9drf_turbo_6fields_FloatField, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_9drf_turbo_6fields_DecimalField __pyx_vtable_9drf_turbo_6fields_DecimalField;

static PyObject *__pyx_tp_new_9drf_turbo_6fields_DecimalField(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_9drf_turbo_6fields_DecimalField *p;
  PyObject *o = __pyx_tp_new_9drf_turbo_6fields_Field(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_9drf_turbo_6fields_DecimalField *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_9drf_turbo_6fields_Field*)__pyx_vtabptr_9drf_turbo_6fields_DecimalField;
  p->max_digits = Py_None; Py_INCREF(Py_None);
  p->decimal_places = Py_None; Py_INCREF(Py_None);
  p->max_value = Py_None; Py_INCREF(Py_None);
  p->min_value = Py_None; Py_INCREF(Py_None);
  p->rounding = Py_None; Py_INCREF(Py_None);
  p->coerce_to_string = Py_None; Py_INCREF(Py_None);
  p->max_whole_digits = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_9drf_turbo_6fields_DecimalField(PyObject *o) {
  struct __pyx_obj_9drf_turbo_6fields_DecimalField *p = (struct __pyx_obj_9drf_turbo_6fields_DecimalField *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->max_digits);
  Py_CLEAR(p->decimal_places);
  Py_CLEAR(p->max_value);
  Py_CLEAR(p->min_value);
  Py_CLEAR(p->rounding);
  Py_CLEAR(p->coerce_to_string);
  Py_CLEAR(p->max_whole_digits);
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_9drf_turbo_6fields_Field(o);
}

static int __pyx_tp_traverse_9drf_turbo_6fields_DecimalField(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_9drf_turbo_6fields_DecimalField *p = (struct __pyx_obj_9drf_turbo_6fields_DecimalField *)o;
  e = __pyx_tp_traverse_9drf_turbo_6fields_Field(o, v, a); if (e) return e;
  if (p->max_digits) {
    e = (*v)(p->max_digits, a); if (e) return e;
  }
  if (p->decimal_places) {
    e = (*v)(p->decimal_places, a); if (e) return e;
  }
  if (p->max_value) {
    e = (*v)(p->max_value, a); if (e) return e;
  }
  if (p->min_value) {
    e = (*v)(p->min_value, a); if (e) return e;
  }
  if (p->rounding) {
    e = (*v)(p->rounding, a); if (e) return e;
  }
  if (p->coerce_to_string) {
    e = (*v)(p->coerce_to_string, a); if (e) return e;
  }
  if (p->max_whole_digits) {
    e = (*v)(p->max_whole_digits, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_9drf_turbo_6fields_DecimalField(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_9drf_turbo_6fields_DecimalField *p = (struct __pyx_obj_9drf_turbo_6fields_DecimalField *)o;
  __pyx_tp_clear_9drf_turbo_6fields_Field(o);
  tmp = ((PyObject*)p->max_digits);
  p->max_digits = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->decimal_places);
  p->decimal_places = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->max_value);
  p->max_value = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->min_value);
  p->min_value = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->rounding);
  p->rounding = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->coerce_to_string);
  p->coerce_to_string = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->max_whole_digits);
  p->max_whole_digits = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_9drf_turbo_6fields_12DecimalField_max_digits(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9drf_turbo_6fields_12DecimalField_10max_digits_1__get__(o);
}

static int __pyx_setprop_9drf_turbo_6fields_12DecimalField_max_digits(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9drf_turbo_6fields_12DecimalField_10max_digits_3__set__(o, v);
  }
  else {
    return __pyx_pw_9drf_turbo_6fields_12DecimalField_10max_digits_5__del__(o);
  }
}

static PyObject *__pyx_getprop_9drf_turbo_6fields_12DecimalField_decimal_places(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9drf_turbo_6fields_12DecimalField_14decimal_places_1__get__(o);
}

static int __pyx_setprop_9drf_turbo_6fields_12DecimalField_decimal_places(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9drf_turbo_6fields_12DecimalField_14decimal_places_3__set__(o, v);
  }
  else {
    return __pyx_pw_9drf_turbo_6fields_12DecimalField_14decimal_places_5__del__(o);
  }
}

static PyObject *__pyx_getprop_9drf_turbo_6fields_12DecimalField_max_value(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9drf_turbo_6fields_12DecimalField_9max_value_1__get__(o);
}

static int __pyx_setprop_9drf_turbo_6fields_12DecimalField_max_value(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9drf_turbo_6fields_12DecimalField_9max_value_3__set__(o, v);
  }
  else {
    return __pyx_pw_9drf_turbo_6fields_12DecimalField_9max_value_5__del__(o);
  }
}

static PyObject *__pyx_getprop_9drf_turbo_6fields_12DecimalField_min_value(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9drf_turbo_6fields_12DecimalField_9min_value_1__get__(o);
}

static int __pyx_setprop_9drf_turbo_6fields_12DecimalField_min_value(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9drf_turbo_6fields_12DecimalField_9min_value_3__set__(o, v);
  }
  else {
    return __pyx_pw_9drf_turbo_6fields_12DecimalField_9min_value_5__del__(o);
  }
}

static PyObject *__pyx_getprop_9drf_turbo_6fields_12DecimalField_rounding(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9drf_turbo_6fields_12DecimalField_8rounding_1__get__(o);
}

static int __pyx_setprop_9drf_turbo_6fields_12DecimalField_rounding(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9drf_turbo_6fields_12DecimalField_8rounding_3__set__(o, v);
  }
  else {
    return __pyx_pw_9drf_turbo_6fields_12DecimalField_8rounding_5__del__(o);
  }
}

static PyObject *__pyx_getprop_9drf_turbo_6fields_12DecimalField_coerce_to_string(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9drf_turbo_6fields_12DecimalField_16coerce_to_string_1__get__(o);
}

static int __pyx_setprop_9drf_turbo_6fields_12DecimalField_coerce_to_string(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9drf_turbo_6fields_12DecimalField_16coerce_to_string_3__set__(o, v);
  }
  else {
    return __pyx_pw_9drf_turbo_6fields_12DecimalField_16coerce_to_string_5__del__(o);
  }
}

static PyObject *__pyx_getprop_9drf_turbo_6fields_12DecimalField_max_whole_digits(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9drf_turbo_6fields_12DecimalField_16max_whole_digits_1__get__(o);
}

static int __pyx_setprop_9drf_turbo_6fields_12DecimalField_max_whole_digits(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9drf_turbo_6fields_12DecimalField_16max_whole_digits_3__set__(o, v);
  }
  else {
    return __pyx_pw_9drf_turbo_6fields_12DecimalField_16max_whole_digits_5__del__(o);
  }
}

static PyMethodDef __pyx_methods_9drf_turbo_6fields_DecimalField[] = {
  {"serialize", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_12DecimalField_3serialize, METH_VARARGS|METH_KEYWORDS, 0},
  {"deserialize", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_12DecimalField_5deserialize, METH_VARARGS|METH_KEYWORDS, __pyx_doc_9drf_turbo_6fields_12DecimalField_4deserialize},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_9drf_turbo_6fields_12DecimalField_7__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_9drf_turbo_6fields_12DecimalField_9__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_9drf_turbo_6fields_DecimalField[] = {
  {(char *)"max_digits", __pyx_getprop_9drf_turbo_6fields_12DecimalField_max_digits, __pyx_setprop_9drf_turbo_6fields_12DecimalField_max_digits, (char *)0, 0},
  {(char *)"decimal_places", __pyx_getprop_9drf_turbo_6fields_12DecimalField_decimal_places, __pyx_setprop_9drf_turbo_6fields_12DecimalField_decimal_places, (char *)0, 0},
  {(char *)"max_value", __pyx_getprop_9drf_turbo_6fields_12DecimalField_max_value, __pyx_setprop_9drf_turbo_6fields_12DecimalField_max_value, (char *)0, 0},
  {(char *)"min_value", __pyx_getprop_9drf_turbo_6fields_12DecimalField_min_value, __pyx_setprop_9drf_turbo_6fields_12DecimalField_min_value, (char *)0, 0},
  {(char *)"rounding", __pyx_getprop_9drf_turbo_6fields_12DecimalField_rounding, __pyx_setprop_9drf_turbo_6fields_12DecimalField_rounding, (char *)0, 0},
  {(char *)"coerce_to_string", __pyx_getprop_9drf_turbo_6fields_12DecimalField_coerce_to_string, __pyx_setprop_9drf_turbo_6fields_12DecimalField_coerce_to_string, (char *)0, 0},
  {(char *)"max_whole_digits", __pyx_getprop_9drf_turbo_6fields_12DecimalField_max_whole_digits, __pyx_setprop_9drf_turbo_6fields_12DecimalField_max_whole_digits, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_9drf_turbo_6fields_DecimalField = {
  PyVarObject_HEAD_INIT(0, 0)
  "drf_turbo.fields.DecimalField", /*tp_name*/
  sizeof(struct __pyx_obj_9drf_turbo_6fields_DecimalField), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_9drf_turbo_6fields_DecimalField, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "\n    A field that validates input as a Python Decimal.\n\n    :param max_digits(required): Maximum number of digits.\n    :param deciaml_places(required): Number of decimal places.\n    :param min_value: The minimum value allowed.\n    :param max_value: The maximum value allowed.\n    :param coerce_to_string: If True, values will be converted to strings during serialization.\n    :param rounding: How to round the value during serialization.\n    :param kwargs: The same keyword arguments that :class:`Field` receives.\n    ", /*tp_doc*/
  __pyx_tp_traverse_9drf_turbo_6fields_DecimalField, /*tp_traverse*/
  __pyx_tp_clear_9drf_turbo_6fields_DecimalField, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_9drf_turbo_6fields_DecimalField, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_9drf_turbo_6fields_DecimalField, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_9drf_turbo_6fields_12DecimalField_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_9drf_turbo_6fields_DecimalField, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_9drf_turbo_6fields_BoolField __pyx_vtable_9drf_turbo_6fields_BoolField;

static PyObject *__pyx_tp_new_9drf_turbo_6fields_BoolField(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_9drf_turbo_6fields_BoolField *p;
  PyObject *o = __pyx_tp_new_9drf_turbo_6fields_Field(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_9drf_turbo_6fields_BoolField *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_9drf_turbo_6fields_Field*)__pyx_vtabptr_9drf_turbo_6fields_BoolField;
  return o;
}

static PyMethodDef __pyx_methods_9drf_turbo_6fields_BoolField[] = {
  {"serialize", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_9BoolField_1serialize, METH_VARARGS|METH_KEYWORDS, 0},
  {"deserialize", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_9BoolField_3deserialize, METH_VARARGS|METH_KEYWORDS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_9drf_turbo_6fields_9BoolField_5__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_9drf_turbo_6fields_9BoolField_7__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_9drf_turbo_6fields_BoolField = {
  PyVarObject_HEAD_INIT(0, 0)
  "drf_turbo.fields.BoolField", /*tp_name*/
  sizeof(struct __pyx_obj_9drf_turbo_6fields_BoolField), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_9drf_turbo_6fields_Field, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "\n    Boolean field type.\n\n    :param kwargs: The same keyword arguments that :class:`Field` receives.\n    ", /*tp_doc*/
  __pyx_tp_traverse_9drf_turbo_6fields_Field, /*tp_traverse*/
  __pyx_tp_clear_9drf_turbo_6fields_Field, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_9drf_turbo_6fields_BoolField, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  #if CYTHON_COMPILING_IN_PYPY
  __pyx_pw_9drf_turbo_6fields_5Field_1__init__, /*tp_init*/
  #else
  0, /*tp_init*/
  #endif
  0, /*tp_alloc*/
  __pyx_tp_new_9drf_turbo_6fields_BoolField, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_9drf_turbo_6fields_ChoiceField __pyx_vtable_9drf_turbo_6fields_ChoiceField;

static PyObject *__pyx_tp_new_9drf_turbo_6fields_ChoiceField(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_9drf_turbo_6fields_ChoiceField *p;
  PyObject *o = __pyx_tp_new_9drf_turbo_6fields_Field(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_9drf_turbo_6fields_ChoiceField *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_9drf_turbo_6fields_Field*)__pyx_vtabptr_9drf_turbo_6fields_ChoiceField;
  p->choices = Py_None; Py_INCREF(Py_None);
  p->choice_strings_to_values = Py_None; Py_INCREF(Py_None);
  p->choice_strings_to_display = Py_None; Py_INCREF(Py_None);
  p->allow_blank = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_9drf_turbo_6fields_ChoiceField(PyObject *o) {
  struct __pyx_obj_9drf_turbo_6fields_ChoiceField *p = (struct __pyx_obj_9drf_turbo_6fields_ChoiceField *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->choices);
  Py_CLEAR(p->choice_strings_to_values);
  Py_CLEAR(p->choice_strings_to_display);
  Py_CLEAR(p->allow_blank);
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_9drf_turbo_6fields_Field(o);
}

static int __pyx_tp_traverse_9drf_turbo_6fields_ChoiceField(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_9drf_turbo_6fields_ChoiceField *p = (struct __pyx_obj_9drf_turbo_6fields_ChoiceField *)o;
  e = __pyx_tp_traverse_9drf_turbo_6fields_Field(o, v, a); if (e) return e;
  if (p->choices) {
    e = (*v)(p->choices, a); if (e) return e;
  }
  if (p->choice_strings_to_values) {
    e = (*v)(p->choice_strings_to_values, a); if (e) return e;
  }
  if (p->choice_strings_to_display) {
    e = (*v)(p->choice_strings_to_display, a); if (e) return e;
  }
  if (p->allow_blank) {
    e = (*v)(p->allow_blank, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_9drf_turbo_6fields_ChoiceField(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_9drf_turbo_6fields_ChoiceField *p = (struct __pyx_obj_9drf_turbo_6fields_ChoiceField *)o;
  __pyx_tp_clear_9drf_turbo_6fields_Field(o);
  tmp = ((PyObject*)p->choices);
  p->choices = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->choice_strings_to_values);
  p->choice_strings_to_values = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->choice_strings_to_display);
  p->choice_strings_to_display = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->allow_blank);
  p->allow_blank = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_9drf_turbo_6fields_11ChoiceField_choices(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9drf_turbo_6fields_11ChoiceField_7choices_1__get__(o);
}

static int __pyx_setprop_9drf_turbo_6fields_11ChoiceField_choices(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9drf_turbo_6fields_11ChoiceField_7choices_3__set__(o, v);
  }
  else {
    return __pyx_pw_9drf_turbo_6fields_11ChoiceField_7choices_5__del__(o);
  }
}

static PyObject *__pyx_getprop_9drf_turbo_6fields_11ChoiceField_choice_strings_to_values(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9drf_turbo_6fields_11ChoiceField_24choice_strings_to_values_1__get__(o);
}

static int __pyx_setprop_9drf_turbo_6fields_11ChoiceField_choice_strings_to_values(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9drf_turbo_6fields_11ChoiceField_24choice_strings_to_values_3__set__(o, v);
  }
  else {
    return __pyx_pw_9drf_turbo_6fields_11ChoiceField_24choice_strings_to_values_5__del__(o);
  }
}

static PyObject *__pyx_getprop_9drf_turbo_6fields_11ChoiceField_choice_strings_to_display(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9drf_turbo_6fields_11ChoiceField_25choice_strings_to_display_1__get__(o);
}

static int __pyx_setprop_9drf_turbo_6fields_11ChoiceField_choice_strings_to_display(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9drf_turbo_6fields_11ChoiceField_25choice_strings_to_display_3__set__(o, v);
  }
  else {
    return __pyx_pw_9drf_turbo_6fields_11ChoiceField_25choice_strings_to_display_5__del__(o);
  }
}

static PyObject *__pyx_getprop_9drf_turbo_6fields_11ChoiceField_allow_blank(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9drf_turbo_6fields_11ChoiceField_11allow_blank_1__get__(o);
}

static int __pyx_setprop_9drf_turbo_6fields_11ChoiceField_allow_blank(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9drf_turbo_6fields_11ChoiceField_11allow_blank_3__set__(o, v);
  }
  else {
    return __pyx_pw_9drf_turbo_6fields_11ChoiceField_11allow_blank_5__del__(o);
  }
}

static PyMethodDef __pyx_methods_9drf_turbo_6fields_ChoiceField[] = {
  {"serialize", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_11ChoiceField_3serialize, METH_VARARGS|METH_KEYWORDS, 0},
  {"deserialize", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_11ChoiceField_5deserialize, METH_VARARGS|METH_KEYWORDS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_9drf_turbo_6fields_11ChoiceField_7__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_9drf_turbo_6fields_11ChoiceField_9__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_9drf_turbo_6fields_ChoiceField[] = {
  {(char *)"choices", __pyx_getprop_9drf_turbo_6fields_11ChoiceField_choices, __pyx_setprop_9drf_turbo_6fields_11ChoiceField_choices, (char *)0, 0},
  {(char *)"choice_strings_to_values", __pyx_getprop_9drf_turbo_6fields_11ChoiceField_choice_strings_to_values, __pyx_setprop_9drf_turbo_6fields_11ChoiceField_choice_strings_to_values, (char *)0, 0},
  {(char *)"choice_strings_to_display", __pyx_getprop_9drf_turbo_6fields_11ChoiceField_choice_strings_to_display, __pyx_setprop_9drf_turbo_6fields_11ChoiceField_choice_strings_to_display, (char *)0, 0},
  {(char *)"allow_blank", __pyx_getprop_9drf_turbo_6fields_11ChoiceField_allow_blank, __pyx_setprop_9drf_turbo_6fields_11ChoiceField_allow_blank, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_9drf_turbo_6fields_ChoiceField = {
  PyVarObject_HEAD_INIT(0, 0)
  "drf_turbo.fields.ChoiceField", /*tp_name*/
  sizeof(struct __pyx_obj_9drf_turbo_6fields_ChoiceField), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_9drf_turbo_6fields_ChoiceField, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "\n    Choice field type.\n\n    :param choices(required): A list of valid choices.\n    :param kwargs: The same keyword arguments that :class:`Field` receives.\n    ", /*tp_doc*/
  __pyx_tp_traverse_9drf_turbo_6fields_ChoiceField, /*tp_traverse*/
  __pyx_tp_clear_9drf_turbo_6fields_ChoiceField, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_9drf_turbo_6fields_ChoiceField, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_9drf_turbo_6fields_ChoiceField, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_9drf_turbo_6fields_11ChoiceField_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_9drf_turbo_6fields_ChoiceField, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_9drf_turbo_6fields_MultipleChoiceField __pyx_vtable_9drf_turbo_6fields_MultipleChoiceField;

static PyObject *__pyx_tp_new_9drf_turbo_6fields_MultipleChoiceField(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_9drf_turbo_6fields_MultipleChoiceField *p;
  PyObject *o = __pyx_tp_new_9drf_turbo_6fields_ChoiceField(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_9drf_turbo_6fields_MultipleChoiceField *)o);
  p->__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_9drf_turbo_6fields_Field*)__pyx_vtabptr_9drf_turbo_6fields_MultipleChoiceField;
  p->allow_empty = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_9drf_turbo_6fields_MultipleChoiceField(PyObject *o) {
  struct __pyx_obj_9drf_turbo_6fields_MultipleChoiceField *p = (struct __pyx_obj_9drf_turbo_6fields_MultipleChoiceField *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->allow_empty);
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_9drf_turbo_6fields_ChoiceField(o);
}

static int __pyx_tp_traverse_9drf_turbo_6fields_MultipleChoiceField(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_9drf_turbo_6fields_MultipleChoiceField *p = (struct __pyx_obj_9drf_turbo_6fields_MultipleChoiceField *)o;
  e = __pyx_tp_traverse_9drf_turbo_6fields_ChoiceField(o, v, a); if (e) return e;
  if (p->allow_empty) {
    e = (*v)(p->allow_empty, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_9drf_turbo_6fields_MultipleChoiceField(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_9drf_turbo_6fields_MultipleChoiceField *p = (struct __pyx_obj_9drf_turbo_6fields_MultipleChoiceField *)o;
  __pyx_tp_clear_9drf_turbo_6fields_ChoiceField(o);
  tmp = ((PyObject*)p->allow_empty);
  p->allow_empty = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_9drf_turbo_6fields_19MultipleChoiceField_allow_empty(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9drf_turbo_6fields_19MultipleChoiceField_11allow_empty_1__get__(o);
}

static int __pyx_setprop_9drf_turbo_6fields_19MultipleChoiceField_allow_empty(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9drf_turbo_6fields_19MultipleChoiceField_11allow_empty_3__set__(o, v);
  }
  else {
    return __pyx_pw_9drf_turbo_6fields_19MultipleChoiceField_11allow_empty_5__del__(o);
  }
}

static PyMethodDef __pyx_methods_9drf_turbo_6fields_MultipleChoiceField[] = {
  {"serialize", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_19MultipleChoiceField_3serialize, METH_VARARGS|METH_KEYWORDS, 0},
  {"deserialize", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_19MultipleChoiceField_5deserialize, METH_VARARGS|METH_KEYWORDS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_9drf_turbo_6fields_19MultipleChoiceField_7__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_9drf_turbo_6fields_19MultipleChoiceField_9__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_9drf_turbo_6fields_MultipleChoiceField[] = {
  {(char *)"allow_empty", __pyx_getprop_9drf_turbo_6fields_19MultipleChoiceField_allow_empty, __pyx_setprop_9drf_turbo_6fields_19MultipleChoiceField_allow_empty, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_9drf_turbo_6fields_MultipleChoiceField = {
  PyVarObject_HEAD_INIT(0, 0)
  "drf_turbo.fields.MultipleChoiceField", /*tp_name*/
  sizeof(struct __pyx_obj_9drf_turbo_6fields_MultipleChoiceField), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_9drf_turbo_6fields_MultipleChoiceField, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "\n    Multiple choice field type.\n\n    :param allow_empty: If True, allow the user to leave the field blank.\n    :param kwargs: The same keyword arguments that :class:`Field` and class:`ChoiceField` receives.\n    ", /*tp_doc*/
  __pyx_tp_traverse_9drf_turbo_6fields_MultipleChoiceField, /*tp_traverse*/
  __pyx_tp_clear_9drf_turbo_6fields_MultipleChoiceField, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_9drf_turbo_6fields_MultipleChoiceField, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_9drf_turbo_6fields_MultipleChoiceField, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_9drf_turbo_6fields_19MultipleChoiceField_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_9drf_turbo_6fields_MultipleChoiceField, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_9drf_turbo_6fields_DateTimeField __pyx_vtable_9drf_turbo_6fields_DateTimeField;

static PyObject *__pyx_tp_new_9drf_turbo_6fields_DateTimeField(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_9drf_turbo_6fields_DateTimeField *p;
  PyObject *o = __pyx_tp_new_9drf_turbo_6fields_Field(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_9drf_turbo_6fields_DateTimeField *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_9drf_turbo_6fields_Field*)__pyx_vtabptr_9drf_turbo_6fields_DateTimeField;
  p->format = Py_None; Py_INCREF(Py_None);
  p->input_formats = Py_None; Py_INCREF(Py_None);
  p->default_timezone = Py_None; Py_INCREF(Py_None);
  p->timezone = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_9drf_turbo_6fields_DateTimeField(PyObject *o) {
  struct __pyx_obj_9drf_turbo_6fields_DateTimeField *p = (struct __pyx_obj_9drf_turbo_6fields_DateTimeField *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->format);
  Py_CLEAR(p->input_formats);
  Py_CLEAR(p->default_timezone);
  Py_CLEAR(p->timezone);
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_9drf_turbo_6fields_Field(o);
}

static int __pyx_tp_traverse_9drf_turbo_6fields_DateTimeField(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_9drf_turbo_6fields_DateTimeField *p = (struct __pyx_obj_9drf_turbo_6fields_DateTimeField *)o;
  e = __pyx_tp_traverse_9drf_turbo_6fields_Field(o, v, a); if (e) return e;
  if (p->format) {
    e = (*v)(p->format, a); if (e) return e;
  }
  if (p->input_formats) {
    e = (*v)(p->input_formats, a); if (e) return e;
  }
  if (p->default_timezone) {
    e = (*v)(p->default_timezone, a); if (e) return e;
  }
  if (p->timezone) {
    e = (*v)(p->timezone, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_9drf_turbo_6fields_DateTimeField(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_9drf_turbo_6fields_DateTimeField *p = (struct __pyx_obj_9drf_turbo_6fields_DateTimeField *)o;
  __pyx_tp_clear_9drf_turbo_6fields_Field(o);
  tmp = ((PyObject*)p->format);
  p->format = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->input_formats);
  p->input_formats = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->default_timezone);
  p->default_timezone = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->timezone);
  p->timezone = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_9drf_turbo_6fields_13DateTimeField_format(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9drf_turbo_6fields_13DateTimeField_6format_1__get__(o);
}

static int __pyx_setprop_9drf_turbo_6fields_13DateTimeField_format(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9drf_turbo_6fields_13DateTimeField_6format_3__set__(o, v);
  }
  else {
    return __pyx_pw_9drf_turbo_6fields_13DateTimeField_6format_5__del__(o);
  }
}

static PyObject *__pyx_getprop_9drf_turbo_6fields_13DateTimeField_input_formats(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9drf_turbo_6fields_13DateTimeField_13input_formats_1__get__(o);
}

static int __pyx_setprop_9drf_turbo_6fields_13DateTimeField_input_formats(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9drf_turbo_6fields_13DateTimeField_13input_formats_3__set__(o, v);
  }
  else {
    return __pyx_pw_9drf_turbo_6fields_13DateTimeField_13input_formats_5__del__(o);
  }
}

static PyObject *__pyx_getprop_9drf_turbo_6fields_13DateTimeField_default_timezone(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9drf_turbo_6fields_13DateTimeField_16default_timezone_1__get__(o);
}

static int __pyx_setprop_9drf_turbo_6fields_13DateTimeField_default_timezone(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9drf_turbo_6fields_13DateTimeField_16default_timezone_3__set__(o, v);
  }
  else {
    return __pyx_pw_9drf_turbo_6fields_13DateTimeField_16default_timezone_5__del__(o);
  }
}

static PyObject *__pyx_getprop_9drf_turbo_6fields_13DateTimeField_timezone(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9drf_turbo_6fields_13DateTimeField_8timezone_1__get__(o);
}

static int __pyx_setprop_9drf_turbo_6fields_13DateTimeField_timezone(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9drf_turbo_6fields_13DateTimeField_8timezone_3__set__(o, v);
  }
  else {
    return __pyx_pw_9drf_turbo_6fields_13DateTimeField_8timezone_5__del__(o);
  }
}

static PyMethodDef __pyx_methods_9drf_turbo_6fields_DateTimeField[] = {
  {"get_default_timezone", (PyCFunction)__pyx_pw_9drf_turbo_6fields_13DateTimeField_3get_default_timezone, METH_NOARGS, 0},
  {"enforce_timezone", (PyCFunction)__pyx_pw_9drf_turbo_6fields_13DateTimeField_5enforce_timezone, METH_O, __pyx_doc_9drf_turbo_6fields_13DateTimeField_4enforce_timezone},
  {"serialize", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_13DateTimeField_7serialize, METH_VARARGS|METH_KEYWORDS, 0},
  {"deserialize", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_13DateTimeField_9deserialize, METH_VARARGS|METH_KEYWORDS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_9drf_turbo_6fields_13DateTimeField_11__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_9drf_turbo_6fields_13DateTimeField_13__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_9drf_turbo_6fields_DateTimeField[] = {
  {(char *)"format", __pyx_getprop_9drf_turbo_6fields_13DateTimeField_format, __pyx_setprop_9drf_turbo_6fields_13DateTimeField_format, (char *)0, 0},
  {(char *)"input_formats", __pyx_getprop_9drf_turbo_6fields_13DateTimeField_input_formats, __pyx_setprop_9drf_turbo_6fields_13DateTimeField_input_formats, (char *)0, 0},
  {(char *)"default_timezone", __pyx_getprop_9drf_turbo_6fields_13DateTimeField_default_timezone, __pyx_setprop_9drf_turbo_6fields_13DateTimeField_default_timezone, (char *)0, 0},
  {(char *)"timezone", __pyx_getprop_9drf_turbo_6fields_13DateTimeField_timezone, __pyx_setprop_9drf_turbo_6fields_13DateTimeField_timezone, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_9drf_turbo_6fields_DateTimeField = {
  PyVarObject_HEAD_INIT(0, 0)
  "drf_turbo.fields.DateTimeField", /*tp_name*/
  sizeof(struct __pyx_obj_9drf_turbo_6fields_DateTimeField), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_9drf_turbo_6fields_DateTimeField, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "\n    A field that (de)serializes to a :class:`datetime.datetime` object. \n\n    :param format: The format to use when serializing/deserializing.\n    :param input_formats: A list of formats to check for when deserializing input.\n    :param default_timezone: The timezone to use when creating datetime instances.\n    :param kwargs: The same keyword arguments that :class:`Field` receives.\n    ", /*tp_doc*/
  __pyx_tp_traverse_9drf_turbo_6fields_DateTimeField, /*tp_traverse*/
  __pyx_tp_clear_9drf_turbo_6fields_DateTimeField, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_9drf_turbo_6fields_DateTimeField, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_9drf_turbo_6fields_DateTimeField, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_9drf_turbo_6fields_13DateTimeField_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_9drf_turbo_6fields_DateTimeField, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_9drf_turbo_6fields_DateField __pyx_vtable_9drf_turbo_6fields_DateField;

static PyObject *__pyx_tp_new_9drf_turbo_6fields_DateField(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_9drf_turbo_6fields_DateField *p;
  PyObject *o = __pyx_tp_new_9drf_turbo_6fields_Field(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_9drf_turbo_6fields_DateField *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_9drf_turbo_6fields_Field*)__pyx_vtabptr_9drf_turbo_6fields_DateField;
  p->format = Py_None; Py_INCREF(Py_None);
  p->input_formats = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_9drf_turbo_6fields_DateField(PyObject *o) {
  struct __pyx_obj_9drf_turbo_6fields_DateField *p = (struct __pyx_obj_9drf_turbo_6fields_DateField *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->format);
  Py_CLEAR(p->input_formats);
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_9drf_turbo_6fields_Field(o);
}

static int __pyx_tp_traverse_9drf_turbo_6fields_DateField(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_9drf_turbo_6fields_DateField *p = (struct __pyx_obj_9drf_turbo_6fields_DateField *)o;
  e = __pyx_tp_traverse_9drf_turbo_6fields_Field(o, v, a); if (e) return e;
  if (p->format) {
    e = (*v)(p->format, a); if (e) return e;
  }
  if (p->input_formats) {
    e = (*v)(p->input_formats, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_9drf_turbo_6fields_DateField(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_9drf_turbo_6fields_DateField *p = (struct __pyx_obj_9drf_turbo_6fields_DateField *)o;
  __pyx_tp_clear_9drf_turbo_6fields_Field(o);
  tmp = ((PyObject*)p->format);
  p->format = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->input_formats);
  p->input_formats = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_9drf_turbo_6fields_9DateField_format(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9drf_turbo_6fields_9DateField_6format_1__get__(o);
}

static int __pyx_setprop_9drf_turbo_6fields_9DateField_format(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9drf_turbo_6fields_9DateField_6format_3__set__(o, v);
  }
  else {
    return __pyx_pw_9drf_turbo_6fields_9DateField_6format_5__del__(o);
  }
}

static PyObject *__pyx_getprop_9drf_turbo_6fields_9DateField_input_formats(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9drf_turbo_6fields_9DateField_13input_formats_1__get__(o);
}

static int __pyx_setprop_9drf_turbo_6fields_9DateField_input_formats(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9drf_turbo_6fields_9DateField_13input_formats_3__set__(o, v);
  }
  else {
    return __pyx_pw_9drf_turbo_6fields_9DateField_13input_formats_5__del__(o);
  }
}

static PyMethodDef __pyx_methods_9drf_turbo_6fields_DateField[] = {
  {"serialize", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_9DateField_3serialize, METH_VARARGS|METH_KEYWORDS, 0},
  {"deserialize", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_9DateField_5deserialize, METH_VARARGS|METH_KEYWORDS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_9drf_turbo_6fields_9DateField_7__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_9drf_turbo_6fields_9DateField_9__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_9drf_turbo_6fields_DateField[] = {
  {(char *)"format", __pyx_getprop_9drf_turbo_6fields_9DateField_format, __pyx_setprop_9drf_turbo_6fields_9DateField_format, (char *)0, 0},
  {(char *)"input_formats", __pyx_getprop_9drf_turbo_6fields_9DateField_input_formats, __pyx_setprop_9drf_turbo_6fields_9DateField_input_formats, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_9drf_turbo_6fields_DateField = {
  PyVarObject_HEAD_INIT(0, 0)
  "drf_turbo.fields.DateField", /*tp_name*/
  sizeof(struct __pyx_obj_9drf_turbo_6fields_DateField), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_9drf_turbo_6fields_DateField, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "\n    A field that (de)serializes to a :class:`datetime.date` object.\n    \n    :param format: Either ``\"%Y-%m-%d\"`` or ``\"%m/%d/%Y\"``, or a custom\n        string of the format passed to ``strftime``.\n    :param input_formats: A list of strings, or a custom list of input formats\n        to be used to parse the input date string.\n    :param kwargs: The same keyword arguments that :class:`Field` receives.\n    ", /*tp_doc*/
  __pyx_tp_traverse_9drf_turbo_6fields_DateField, /*tp_traverse*/
  __pyx_tp_clear_9drf_turbo_6fields_DateField, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_9drf_turbo_6fields_DateField, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_9drf_turbo_6fields_DateField, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_9drf_turbo_6fields_9DateField_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_9drf_turbo_6fields_DateField, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_9drf_turbo_6fields_TimeField __pyx_vtable_9drf_turbo_6fields_TimeField;

static PyObject *__pyx_tp_new_9drf_turbo_6fields_TimeField(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_9drf_turbo_6fields_TimeField *p;
  PyObject *o = __pyx_tp_new_9drf_turbo_6fields_Field(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_9drf_turbo_6fields_TimeField *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_9drf_turbo_6fields_Field*)__pyx_vtabptr_9drf_turbo_6fields_TimeField;
  p->format = Py_None; Py_INCREF(Py_None);
  p->input_formats = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_9drf_turbo_6fields_TimeField(PyObject *o) {
  struct __pyx_obj_9drf_turbo_6fields_TimeField *p = (struct __pyx_obj_9drf_turbo_6fields_TimeField *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->format);
  Py_CLEAR(p->input_formats);
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_9drf_turbo_6fields_Field(o);
}

static int __pyx_tp_traverse_9drf_turbo_6fields_TimeField(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_9drf_turbo_6fields_TimeField *p = (struct __pyx_obj_9drf_turbo_6fields_TimeField *)o;
  e = __pyx_tp_traverse_9drf_turbo_6fields_Field(o, v, a); if (e) return e;
  if (p->format) {
    e = (*v)(p->format, a); if (e) return e;
  }
  if (p->input_formats) {
    e = (*v)(p->input_formats, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_9drf_turbo_6fields_TimeField(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_9drf_turbo_6fields_TimeField *p = (struct __pyx_obj_9drf_turbo_6fields_TimeField *)o;
  __pyx_tp_clear_9drf_turbo_6fields_Field(o);
  tmp = ((PyObject*)p->format);
  p->format = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->input_formats);
  p->input_formats = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_9drf_turbo_6fields_9TimeField_format(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9drf_turbo_6fields_9TimeField_6format_1__get__(o);
}

static int __pyx_setprop_9drf_turbo_6fields_9TimeField_format(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9drf_turbo_6fields_9TimeField_6format_3__set__(o, v);
  }
  else {
    return __pyx_pw_9drf_turbo_6fields_9TimeField_6format_5__del__(o);
  }
}

static PyObject *__pyx_getprop_9drf_turbo_6fields_9TimeField_input_formats(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9drf_turbo_6fields_9TimeField_13input_formats_1__get__(o);
}

static int __pyx_setprop_9drf_turbo_6fields_9TimeField_input_formats(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9drf_turbo_6fields_9TimeField_13input_formats_3__set__(o, v);
  }
  else {
    return __pyx_pw_9drf_turbo_6fields_9TimeField_13input_formats_5__del__(o);
  }
}

static PyMethodDef __pyx_methods_9drf_turbo_6fields_TimeField[] = {
  {"serialize", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_9TimeField_3serialize, METH_VARARGS|METH_KEYWORDS, 0},
  {"deserialize", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_9TimeField_5deserialize, METH_VARARGS|METH_KEYWORDS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_9drf_turbo_6fields_9TimeField_7__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_9drf_turbo_6fields_9TimeField_9__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_9drf_turbo_6fields_TimeField[] = {
  {(char *)"format", __pyx_getprop_9drf_turbo_6fields_9TimeField_format, __pyx_setprop_9drf_turbo_6fields_9TimeField_format, (char *)0, 0},
  {(char *)"input_formats", __pyx_getprop_9drf_turbo_6fields_9TimeField_input_formats, __pyx_setprop_9drf_turbo_6fields_9TimeField_input_formats, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_9drf_turbo_6fields_TimeField = {
  PyVarObject_HEAD_INIT(0, 0)
  "drf_turbo.fields.TimeField", /*tp_name*/
  sizeof(struct __pyx_obj_9drf_turbo_6fields_TimeField), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_9drf_turbo_6fields_TimeField, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "\n    A field that (de)serializes to a :class:`datetime.time` object.\n\n    :param format: Either \"time\" or \"datetime\" for serialization to display\n        time in either 24 hour or 12 hour+minute+second format.\n    :param input_formats: Optional list of formats to also be accepted.\n    :param kwargs: The same keyword arguments that :class:`Field` receives.\n    ", /*tp_doc*/
  __pyx_tp_traverse_9drf_turbo_6fields_TimeField, /*tp_traverse*/
  __pyx_tp_clear_9drf_turbo_6fields_TimeField, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_9drf_turbo_6fields_TimeField, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_9drf_turbo_6fields_TimeField, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_9drf_turbo_6fields_9TimeField_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_9drf_turbo_6fields_TimeField, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_9drf_turbo_6fields_FileField __pyx_vtable_9drf_turbo_6fields_FileField;

static PyObject *__pyx_tp_new_9drf_turbo_6fields_FileField(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_9drf_turbo_6fields_FileField *p;
  PyObject *o = __pyx_tp_new_9drf_turbo_6fields_Field(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_9drf_turbo_6fields_FileField *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_9drf_turbo_6fields_Field*)__pyx_vtabptr_9drf_turbo_6fields_FileField;
  p->max_length = Py_None; Py_INCREF(Py_None);
  p->allow_empty_file = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_9drf_turbo_6fields_FileField(PyObject *o) {
  struct __pyx_obj_9drf_turbo_6fields_FileField *p = (struct __pyx_obj_9drf_turbo_6fields_FileField *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->max_length);
  Py_CLEAR(p->allow_empty_file);
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_9drf_turbo_6fields_Field(o);
}

static int __pyx_tp_traverse_9drf_turbo_6fields_FileField(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_9drf_turbo_6fields_FileField *p = (struct __pyx_obj_9drf_turbo_6fields_FileField *)o;
  e = __pyx_tp_traverse_9drf_turbo_6fields_Field(o, v, a); if (e) return e;
  if (p->max_length) {
    e = (*v)(p->max_length, a); if (e) return e;
  }
  if (p->allow_empty_file) {
    e = (*v)(p->allow_empty_file, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_9drf_turbo_6fields_FileField(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_9drf_turbo_6fields_FileField *p = (struct __pyx_obj_9drf_turbo_6fields_FileField *)o;
  __pyx_tp_clear_9drf_turbo_6fields_Field(o);
  tmp = ((PyObject*)p->max_length);
  p->max_length = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->allow_empty_file);
  p->allow_empty_file = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_9drf_turbo_6fields_9FileField_max_length(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9drf_turbo_6fields_9FileField_10max_length_1__get__(o);
}

static int __pyx_setprop_9drf_turbo_6fields_9FileField_max_length(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9drf_turbo_6fields_9FileField_10max_length_3__set__(o, v);
  }
  else {
    return __pyx_pw_9drf_turbo_6fields_9FileField_10max_length_5__del__(o);
  }
}

static PyObject *__pyx_getprop_9drf_turbo_6fields_9FileField_allow_empty_file(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9drf_turbo_6fields_9FileField_16allow_empty_file_1__get__(o);
}

static int __pyx_setprop_9drf_turbo_6fields_9FileField_allow_empty_file(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9drf_turbo_6fields_9FileField_16allow_empty_file_3__set__(o, v);
  }
  else {
    return __pyx_pw_9drf_turbo_6fields_9FileField_16allow_empty_file_5__del__(o);
  }
}

static PyMethodDef __pyx_methods_9drf_turbo_6fields_FileField[] = {
  {"serialize", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_9FileField_3serialize, METH_VARARGS|METH_KEYWORDS, 0},
  {"deserialize", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_9FileField_5deserialize, METH_VARARGS|METH_KEYWORDS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_9drf_turbo_6fields_9FileField_7__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_9drf_turbo_6fields_9FileField_9__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_9drf_turbo_6fields_FileField[] = {
  {(char *)"max_length", __pyx_getprop_9drf_turbo_6fields_9FileField_max_length, __pyx_setprop_9drf_turbo_6fields_9FileField_max_length, (char *)0, 0},
  {(char *)"allow_empty_file", __pyx_getprop_9drf_turbo_6fields_9FileField_allow_empty_file, __pyx_setprop_9drf_turbo_6fields_9FileField_allow_empty_file, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_9drf_turbo_6fields_FileField = {
  PyVarObject_HEAD_INIT(0, 0)
  "drf_turbo.fields.FileField", /*tp_name*/
  sizeof(struct __pyx_obj_9drf_turbo_6fields_FileField), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_9drf_turbo_6fields_FileField, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "\n    A file field.    \n\n    :param max_length: The maximum file size.\n    :param alow_empty_file: Whether to allow uploading empty files.\n    :param kwargs: The same keyword arguments that :class:`Field` receives.\n    ", /*tp_doc*/
  __pyx_tp_traverse_9drf_turbo_6fields_FileField, /*tp_traverse*/
  __pyx_tp_clear_9drf_turbo_6fields_FileField, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_9drf_turbo_6fields_FileField, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_9drf_turbo_6fields_FileField, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_9drf_turbo_6fields_9FileField_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_9drf_turbo_6fields_FileField, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_9drf_turbo_6fields_ArrayField __pyx_vtable_9drf_turbo_6fields_ArrayField;

static PyObject *__pyx_tp_new_9drf_turbo_6fields_ArrayField(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_9drf_turbo_6fields_ArrayField *p;
  PyObject *o = __pyx_tp_new_9drf_turbo_6fields_Field(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_9drf_turbo_6fields_ArrayField *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_9drf_turbo_6fields_Field*)__pyx_vtabptr_9drf_turbo_6fields_ArrayField;
  p->child = Py_None; Py_INCREF(Py_None);
  p->allow_empty = Py_None; Py_INCREF(Py_None);
  p->min_items = Py_None; Py_INCREF(Py_None);
  p->max_items = Py_None; Py_INCREF(Py_None);
  p->exact_items = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_9drf_turbo_6fields_ArrayField(PyObject *o) {
  struct __pyx_obj_9drf_turbo_6fields_ArrayField *p = (struct __pyx_obj_9drf_turbo_6fields_ArrayField *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->child);
  Py_CLEAR(p->allow_empty);
  Py_CLEAR(p->min_items);
  Py_CLEAR(p->max_items);
  Py_CLEAR(p->exact_items);
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_9drf_turbo_6fields_Field(o);
}

static int __pyx_tp_traverse_9drf_turbo_6fields_ArrayField(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_9drf_turbo_6fields_ArrayField *p = (struct __pyx_obj_9drf_turbo_6fields_ArrayField *)o;
  e = __pyx_tp_traverse_9drf_turbo_6fields_Field(o, v, a); if (e) return e;
  if (p->child) {
    e = (*v)(p->child, a); if (e) return e;
  }
  if (p->allow_empty) {
    e = (*v)(p->allow_empty, a); if (e) return e;
  }
  if (p->min_items) {
    e = (*v)(p->min_items, a); if (e) return e;
  }
  if (p->max_items) {
    e = (*v)(p->max_items, a); if (e) return e;
  }
  if (p->exact_items) {
    e = (*v)(p->exact_items, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_9drf_turbo_6fields_ArrayField(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_9drf_turbo_6fields_ArrayField *p = (struct __pyx_obj_9drf_turbo_6fields_ArrayField *)o;
  __pyx_tp_clear_9drf_turbo_6fields_Field(o);
  tmp = ((PyObject*)p->child);
  p->child = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->allow_empty);
  p->allow_empty = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->min_items);
  p->min_items = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->max_items);
  p->max_items = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->exact_items);
  p->exact_items = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_9drf_turbo_6fields_10ArrayField_child(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9drf_turbo_6fields_10ArrayField_5child_1__get__(o);
}

static int __pyx_setprop_9drf_turbo_6fields_10ArrayField_child(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9drf_turbo_6fields_10ArrayField_5child_3__set__(o, v);
  }
  else {
    return __pyx_pw_9drf_turbo_6fields_10ArrayField_5child_5__del__(o);
  }
}

static PyObject *__pyx_getprop_9drf_turbo_6fields_10ArrayField_allow_empty(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9drf_turbo_6fields_10ArrayField_11allow_empty_1__get__(o);
}

static int __pyx_setprop_9drf_turbo_6fields_10ArrayField_allow_empty(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9drf_turbo_6fields_10ArrayField_11allow_empty_3__set__(o, v);
  }
  else {
    return __pyx_pw_9drf_turbo_6fields_10ArrayField_11allow_empty_5__del__(o);
  }
}

static PyObject *__pyx_getprop_9drf_turbo_6fields_10ArrayField_min_items(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9drf_turbo_6fields_10ArrayField_9min_items_1__get__(o);
}

static int __pyx_setprop_9drf_turbo_6fields_10ArrayField_min_items(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9drf_turbo_6fields_10ArrayField_9min_items_3__set__(o, v);
  }
  else {
    return __pyx_pw_9drf_turbo_6fields_10ArrayField_9min_items_5__del__(o);
  }
}

static PyObject *__pyx_getprop_9drf_turbo_6fields_10ArrayField_max_items(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9drf_turbo_6fields_10ArrayField_9max_items_1__get__(o);
}

static int __pyx_setprop_9drf_turbo_6fields_10ArrayField_max_items(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9drf_turbo_6fields_10ArrayField_9max_items_3__set__(o, v);
  }
  else {
    return __pyx_pw_9drf_turbo_6fields_10ArrayField_9max_items_5__del__(o);
  }
}

static PyObject *__pyx_getprop_9drf_turbo_6fields_10ArrayField_exact_items(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9drf_turbo_6fields_10ArrayField_11exact_items_1__get__(o);
}

static int __pyx_setprop_9drf_turbo_6fields_10ArrayField_exact_items(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9drf_turbo_6fields_10ArrayField_11exact_items_3__set__(o, v);
  }
  else {
    return __pyx_pw_9drf_turbo_6fields_10ArrayField_11exact_items_5__del__(o);
  }
}

static PyMethodDef __pyx_methods_9drf_turbo_6fields_ArrayField[] = {
  {"serialize", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_10ArrayField_3serialize, METH_VARARGS|METH_KEYWORDS, __pyx_doc_9drf_turbo_6fields_10ArrayField_2serialize},
  {"deserialize", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_10ArrayField_5deserialize, METH_VARARGS|METH_KEYWORDS, __pyx_doc_9drf_turbo_6fields_10ArrayField_4deserialize},
  {"run_child_validation", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_10ArrayField_7run_child_validation, METH_VARARGS|METH_KEYWORDS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_9drf_turbo_6fields_10ArrayField_9__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_9drf_turbo_6fields_10ArrayField_11__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_9drf_turbo_6fields_ArrayField[] = {
  {(char *)"child", __pyx_getprop_9drf_turbo_6fields_10ArrayField_child, __pyx_setprop_9drf_turbo_6fields_10ArrayField_child, (char *)0, 0},
  {(char *)"allow_empty", __pyx_getprop_9drf_turbo_6fields_10ArrayField_allow_empty, __pyx_setprop_9drf_turbo_6fields_10ArrayField_allow_empty, (char *)0, 0},
  {(char *)"min_items", __pyx_getprop_9drf_turbo_6fields_10ArrayField_min_items, __pyx_setprop_9drf_turbo_6fields_10ArrayField_min_items, (char *)0, 0},
  {(char *)"max_items", __pyx_getprop_9drf_turbo_6fields_10ArrayField_max_items, __pyx_setprop_9drf_turbo_6fields_10ArrayField_max_items, (char *)0, 0},
  {(char *)"exact_items", __pyx_getprop_9drf_turbo_6fields_10ArrayField_exact_items, __pyx_setprop_9drf_turbo_6fields_10ArrayField_exact_items, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_9drf_turbo_6fields_ArrayField = {
  PyVarObject_HEAD_INIT(0, 0)
  "drf_turbo.fields.ArrayField", /*tp_name*/
  sizeof(struct __pyx_obj_9drf_turbo_6fields_ArrayField), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_9drf_turbo_6fields_ArrayField, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "\n    An Array Field.\n\n    :param child: The field to validate the array elements.\n    :param allow_empty: Whether the array can be empty.\n    :param max_items: The maximum number of items allowed in the array.\n    :param min_items: The minimum number of items required in the array.\n    :param exac_items: The exact number of items required in the array.\n    :param kwargs: The same keyword arguments that :class:`Field` receives.\n    ", /*tp_doc*/
  __pyx_tp_traverse_9drf_turbo_6fields_ArrayField, /*tp_traverse*/
  __pyx_tp_clear_9drf_turbo_6fields_ArrayField, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_9drf_turbo_6fields_ArrayField, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_9drf_turbo_6fields_ArrayField, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_9drf_turbo_6fields_10ArrayField_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_9drf_turbo_6fields_ArrayField, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_9drf_turbo_6fields_DictField __pyx_vtable_9drf_turbo_6fields_DictField;

static PyObject *__pyx_tp_new_9drf_turbo_6fields_DictField(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_9drf_turbo_6fields_DictField *p;
  PyObject *o = __pyx_tp_new_9drf_turbo_6fields_Field(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_9drf_turbo_6fields_DictField *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_9drf_turbo_6fields_Field*)__pyx_vtabptr_9drf_turbo_6fields_DictField;
  p->child = Py_None; Py_INCREF(Py_None);
  p->allow_empty = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_9drf_turbo_6fields_DictField(PyObject *o) {
  struct __pyx_obj_9drf_turbo_6fields_DictField *p = (struct __pyx_obj_9drf_turbo_6fields_DictField *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->child);
  Py_CLEAR(p->allow_empty);
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_9drf_turbo_6fields_Field(o);
}

static int __pyx_tp_traverse_9drf_turbo_6fields_DictField(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_9drf_turbo_6fields_DictField *p = (struct __pyx_obj_9drf_turbo_6fields_DictField *)o;
  e = __pyx_tp_traverse_9drf_turbo_6fields_Field(o, v, a); if (e) return e;
  if (p->child) {
    e = (*v)(p->child, a); if (e) return e;
  }
  if (p->allow_empty) {
    e = (*v)(p->allow_empty, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_9drf_turbo_6fields_DictField(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_9drf_turbo_6fields_DictField *p = (struct __pyx_obj_9drf_turbo_6fields_DictField *)o;
  __pyx_tp_clear_9drf_turbo_6fields_Field(o);
  tmp = ((PyObject*)p->child);
  p->child = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->allow_empty);
  p->allow_empty = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_9drf_turbo_6fields_9DictField_child(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9drf_turbo_6fields_9DictField_5child_1__get__(o);
}

static int __pyx_setprop_9drf_turbo_6fields_9DictField_child(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9drf_turbo_6fields_9DictField_5child_3__set__(o, v);
  }
  else {
    return __pyx_pw_9drf_turbo_6fields_9DictField_5child_5__del__(o);
  }
}

static PyObject *__pyx_getprop_9drf_turbo_6fields_9DictField_allow_empty(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9drf_turbo_6fields_9DictField_11allow_empty_1__get__(o);
}

static int __pyx_setprop_9drf_turbo_6fields_9DictField_allow_empty(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9drf_turbo_6fields_9DictField_11allow_empty_3__set__(o, v);
  }
  else {
    return __pyx_pw_9drf_turbo_6fields_9DictField_11allow_empty_5__del__(o);
  }
}

static PyMethodDef __pyx_methods_9drf_turbo_6fields_DictField[] = {
  {"serialize", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_9DictField_3serialize, METH_VARARGS|METH_KEYWORDS, __pyx_doc_9drf_turbo_6fields_9DictField_2serialize},
  {"deserialize", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_9DictField_5deserialize, METH_VARARGS|METH_KEYWORDS, __pyx_doc_9drf_turbo_6fields_9DictField_4deserialize},
  {"run_child_validation", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_9DictField_7run_child_validation, METH_VARARGS|METH_KEYWORDS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_9drf_turbo_6fields_9DictField_9__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_9drf_turbo_6fields_9DictField_11__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_9drf_turbo_6fields_DictField[] = {
  {(char *)"child", __pyx_getprop_9drf_turbo_6fields_9DictField_child, __pyx_setprop_9drf_turbo_6fields_9DictField_child, (char *)0, 0},
  {(char *)"allow_empty", __pyx_getprop_9drf_turbo_6fields_9DictField_allow_empty, __pyx_setprop_9drf_turbo_6fields_9DictField_allow_empty, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_9drf_turbo_6fields_DictField = {
  PyVarObject_HEAD_INIT(0, 0)
  "drf_turbo.fields.DictField", /*tp_name*/
  sizeof(struct __pyx_obj_9drf_turbo_6fields_DictField), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_9drf_turbo_6fields_DictField, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "A Dict Field.\n\n    :param child: The field to validate the dict values.\n    :param allow_empty: Whether the dict can be empty.\n    :param kwargs: The same keyword arguments that :class:`Field` receives.\n    ", /*tp_doc*/
  __pyx_tp_traverse_9drf_turbo_6fields_DictField, /*tp_traverse*/
  __pyx_tp_clear_9drf_turbo_6fields_DictField, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_9drf_turbo_6fields_DictField, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_9drf_turbo_6fields_DictField, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_9drf_turbo_6fields_9DictField_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_9drf_turbo_6fields_DictField, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_9drf_turbo_6fields_JSONField __pyx_vtable_9drf_turbo_6fields_JSONField;

static PyObject *__pyx_tp_new_9drf_turbo_6fields_JSONField(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_9drf_turbo_6fields_JSONField *p;
  PyObject *o = __pyx_tp_new_9drf_turbo_6fields_Field(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_9drf_turbo_6fields_JSONField *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_9drf_turbo_6fields_Field*)__pyx_vtabptr_9drf_turbo_6fields_JSONField;
  p->binary = Py_None; Py_INCREF(Py_None);
  p->encoder = Py_None; Py_INCREF(Py_None);
  p->decoder = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_9drf_turbo_6fields_JSONField(PyObject *o) {
  struct __pyx_obj_9drf_turbo_6fields_JSONField *p = (struct __pyx_obj_9drf_turbo_6fields_JSONField *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->binary);
  Py_CLEAR(p->encoder);
  Py_CLEAR(p->decoder);
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_9drf_turbo_6fields_Field(o);
}

static int __pyx_tp_traverse_9drf_turbo_6fields_JSONField(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_9drf_turbo_6fields_JSONField *p = (struct __pyx_obj_9drf_turbo_6fields_JSONField *)o;
  e = __pyx_tp_traverse_9drf_turbo_6fields_Field(o, v, a); if (e) return e;
  if (p->binary) {
    e = (*v)(p->binary, a); if (e) return e;
  }
  if (p->encoder) {
    e = (*v)(p->encoder, a); if (e) return e;
  }
  if (p->decoder) {
    e = (*v)(p->decoder, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_9drf_turbo_6fields_JSONField(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_9drf_turbo_6fields_JSONField *p = (struct __pyx_obj_9drf_turbo_6fields_JSONField *)o;
  __pyx_tp_clear_9drf_turbo_6fields_Field(o);
  tmp = ((PyObject*)p->binary);
  p->binary = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->encoder);
  p->encoder = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->decoder);
  p->decoder = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_9drf_turbo_6fields_9JSONField_binary(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9drf_turbo_6fields_9JSONField_6binary_1__get__(o);
}

static int __pyx_setprop_9drf_turbo_6fields_9JSONField_binary(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9drf_turbo_6fields_9JSONField_6binary_3__set__(o, v);
  }
  else {
    return __pyx_pw_9drf_turbo_6fields_9JSONField_6binary_5__del__(o);
  }
}

static PyObject *__pyx_getprop_9drf_turbo_6fields_9JSONField_encoder(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9drf_turbo_6fields_9JSONField_7encoder_1__get__(o);
}

static int __pyx_setprop_9drf_turbo_6fields_9JSONField_encoder(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9drf_turbo_6fields_9JSONField_7encoder_3__set__(o, v);
  }
  else {
    return __pyx_pw_9drf_turbo_6fields_9JSONField_7encoder_5__del__(o);
  }
}

static PyObject *__pyx_getprop_9drf_turbo_6fields_9JSONField_decoder(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9drf_turbo_6fields_9JSONField_7decoder_1__get__(o);
}

static int __pyx_setprop_9drf_turbo_6fields_9JSONField_decoder(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9drf_turbo_6fields_9JSONField_7decoder_3__set__(o, v);
  }
  else {
    return __pyx_pw_9drf_turbo_6fields_9JSONField_7decoder_5__del__(o);
  }
}

static PyMethodDef __pyx_methods_9drf_turbo_6fields_JSONField[] = {
  {"serialize", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_9JSONField_3serialize, METH_VARARGS|METH_KEYWORDS, 0},
  {"deserialize", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_9JSONField_5deserialize, METH_VARARGS|METH_KEYWORDS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_9drf_turbo_6fields_9JSONField_7__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_9drf_turbo_6fields_9JSONField_9__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_9drf_turbo_6fields_JSONField[] = {
  {(char *)"binary", __pyx_getprop_9drf_turbo_6fields_9JSONField_binary, __pyx_setprop_9drf_turbo_6fields_9JSONField_binary, (char *)0, 0},
  {(char *)"encoder", __pyx_getprop_9drf_turbo_6fields_9JSONField_encoder, __pyx_setprop_9drf_turbo_6fields_9JSONField_encoder, (char *)0, 0},
  {(char *)"decoder", __pyx_getprop_9drf_turbo_6fields_9JSONField_decoder, __pyx_setprop_9drf_turbo_6fields_9JSONField_decoder, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_9drf_turbo_6fields_JSONField = {
  PyVarObject_HEAD_INIT(0, 0)
  "drf_turbo.fields.JSONField", /*tp_name*/
  sizeof(struct __pyx_obj_9drf_turbo_6fields_JSONField), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_9drf_turbo_6fields_JSONField, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "A JSON Field.\n\n    :param binary: Whether to load/dump JSON as binary data.\n    :param encoder: The JSON encoder class to use.\n    :param decoder: The JSON decoder class to use.\n    :param kwargs: The same keyword arguments that :class:`Field` receives.\n    ", /*tp_doc*/
  __pyx_tp_traverse_9drf_turbo_6fields_JSONField, /*tp_traverse*/
  __pyx_tp_clear_9drf_turbo_6fields_JSONField, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_9drf_turbo_6fields_JSONField, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_9drf_turbo_6fields_JSONField, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_9drf_turbo_6fields_9JSONField_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_9drf_turbo_6fields_JSONField, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_9drf_turbo_6fields_RelatedField __pyx_vtable_9drf_turbo_6fields_RelatedField;

static PyObject *__pyx_tp_new_9drf_turbo_6fields_RelatedField(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_9drf_turbo_6fields_RelatedField *p;
  PyObject *o = __pyx_tp_new_9drf_turbo_6fields_Field(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_9drf_turbo_6fields_RelatedField *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_9drf_turbo_6fields_Field*)__pyx_vtabptr_9drf_turbo_6fields_RelatedField;
  p->queryset = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_9drf_turbo_6fields_RelatedField(PyObject *o) {
  struct __pyx_obj_9drf_turbo_6fields_RelatedField *p = (struct __pyx_obj_9drf_turbo_6fields_RelatedField *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->queryset);
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_9drf_turbo_6fields_Field(o);
}

static int __pyx_tp_traverse_9drf_turbo_6fields_RelatedField(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_9drf_turbo_6fields_RelatedField *p = (struct __pyx_obj_9drf_turbo_6fields_RelatedField *)o;
  e = __pyx_tp_traverse_9drf_turbo_6fields_Field(o, v, a); if (e) return e;
  if (p->queryset) {
    e = (*v)(p->queryset, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_9drf_turbo_6fields_RelatedField(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_9drf_turbo_6fields_RelatedField *p = (struct __pyx_obj_9drf_turbo_6fields_RelatedField *)o;
  __pyx_tp_clear_9drf_turbo_6fields_Field(o);
  tmp = ((PyObject*)p->queryset);
  p->queryset = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_9drf_turbo_6fields_12RelatedField_queryset(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9drf_turbo_6fields_12RelatedField_8queryset_1__get__(o);
}

static int __pyx_setprop_9drf_turbo_6fields_12RelatedField_queryset(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9drf_turbo_6fields_12RelatedField_8queryset_3__set__(o, v);
  }
  else {
    return __pyx_pw_9drf_turbo_6fields_12RelatedField_8queryset_5__del__(o);
  }
}

static PyMethodDef __pyx_methods_9drf_turbo_6fields_RelatedField[] = {
  {"deserialize", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_12RelatedField_3deserialize, METH_VARARGS|METH_KEYWORDS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_9drf_turbo_6fields_12RelatedField_5__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_9drf_turbo_6fields_12RelatedField_7__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_9drf_turbo_6fields_RelatedField[] = {
  {(char *)"queryset", __pyx_getprop_9drf_turbo_6fields_12RelatedField_queryset, __pyx_setprop_9drf_turbo_6fields_12RelatedField_queryset, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_9drf_turbo_6fields_RelatedField = {
  PyVarObject_HEAD_INIT(0, 0)
  "drf_turbo.fields.RelatedField", /*tp_name*/
  sizeof(struct __pyx_obj_9drf_turbo_6fields_RelatedField), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_9drf_turbo_6fields_RelatedField, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "A Related Field.\n\n    :param queryset: The queryset to use for getting the instance from the given value.\n    ", /*tp_doc*/
  __pyx_tp_traverse_9drf_turbo_6fields_RelatedField, /*tp_traverse*/
  __pyx_tp_clear_9drf_turbo_6fields_RelatedField, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_9drf_turbo_6fields_RelatedField, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_9drf_turbo_6fields_RelatedField, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_9drf_turbo_6fields_12RelatedField_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_9drf_turbo_6fields_RelatedField, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_9drf_turbo_6fields_ManyRelatedField __pyx_vtable_9drf_turbo_6fields_ManyRelatedField;

static PyObject *__pyx_tp_new_9drf_turbo_6fields_ManyRelatedField(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_9drf_turbo_6fields_ManyRelatedField *p;
  PyObject *o = __pyx_tp_new_9drf_turbo_6fields_Field(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_9drf_turbo_6fields_ManyRelatedField *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_9drf_turbo_6fields_Field*)__pyx_vtabptr_9drf_turbo_6fields_ManyRelatedField;
  p->child_relation = Py_None; Py_INCREF(Py_None);
  p->allow_empty = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_9drf_turbo_6fields_ManyRelatedField(PyObject *o) {
  struct __pyx_obj_9drf_turbo_6fields_ManyRelatedField *p = (struct __pyx_obj_9drf_turbo_6fields_ManyRelatedField *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->child_relation);
  Py_CLEAR(p->allow_empty);
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_9drf_turbo_6fields_Field(o);
}

static int __pyx_tp_traverse_9drf_turbo_6fields_ManyRelatedField(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_9drf_turbo_6fields_ManyRelatedField *p = (struct __pyx_obj_9drf_turbo_6fields_ManyRelatedField *)o;
  e = __pyx_tp_traverse_9drf_turbo_6fields_Field(o, v, a); if (e) return e;
  if (p->child_relation) {
    e = (*v)(p->child_relation, a); if (e) return e;
  }
  if (p->allow_empty) {
    e = (*v)(p->allow_empty, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_9drf_turbo_6fields_ManyRelatedField(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_9drf_turbo_6fields_ManyRelatedField *p = (struct __pyx_obj_9drf_turbo_6fields_ManyRelatedField *)o;
  __pyx_tp_clear_9drf_turbo_6fields_Field(o);
  tmp = ((PyObject*)p->child_relation);
  p->child_relation = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->allow_empty);
  p->allow_empty = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_9drf_turbo_6fields_16ManyRelatedField_child_relation(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9drf_turbo_6fields_16ManyRelatedField_14child_relation_1__get__(o);
}

static int __pyx_setprop_9drf_turbo_6fields_16ManyRelatedField_child_relation(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9drf_turbo_6fields_16ManyRelatedField_14child_relation_3__set__(o, v);
  }
  else {
    return __pyx_pw_9drf_turbo_6fields_16ManyRelatedField_14child_relation_5__del__(o);
  }
}

static PyObject *__pyx_getprop_9drf_turbo_6fields_16ManyRelatedField_allow_empty(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9drf_turbo_6fields_16ManyRelatedField_11allow_empty_1__get__(o);
}

static int __pyx_setprop_9drf_turbo_6fields_16ManyRelatedField_allow_empty(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9drf_turbo_6fields_16ManyRelatedField_11allow_empty_3__set__(o, v);
  }
  else {
    return __pyx_pw_9drf_turbo_6fields_16ManyRelatedField_11allow_empty_5__del__(o);
  }
}

static PyMethodDef __pyx_methods_9drf_turbo_6fields_ManyRelatedField[] = {
  {"serialize", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_16ManyRelatedField_3serialize, METH_VARARGS|METH_KEYWORDS, 0},
  {"deserialize", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_16ManyRelatedField_5deserialize, METH_VARARGS|METH_KEYWORDS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_9drf_turbo_6fields_16ManyRelatedField_7__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_9drf_turbo_6fields_16ManyRelatedField_9__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_9drf_turbo_6fields_ManyRelatedField[] = {
  {(char *)"child_relation", __pyx_getprop_9drf_turbo_6fields_16ManyRelatedField_child_relation, __pyx_setprop_9drf_turbo_6fields_16ManyRelatedField_child_relation, (char *)0, 0},
  {(char *)"allow_empty", __pyx_getprop_9drf_turbo_6fields_16ManyRelatedField_allow_empty, __pyx_setprop_9drf_turbo_6fields_16ManyRelatedField_allow_empty, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_9drf_turbo_6fields_ManyRelatedField = {
  PyVarObject_HEAD_INIT(0, 0)
  "drf_turbo.fields.ManyRelatedField", /*tp_name*/
  sizeof(struct __pyx_obj_9drf_turbo_6fields_ManyRelatedField), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_9drf_turbo_6fields_ManyRelatedField, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "\n    A field used to represent a to-many relationship.\n\n    :param child_relation: The model field that is the reverse of the relation.\n    :param allow_empty: Whether the list can be empty.\n    :param kwargs: The same keyword arguments that :class:`Field` receives.\n    ", /*tp_doc*/
  __pyx_tp_traverse_9drf_turbo_6fields_ManyRelatedField, /*tp_traverse*/
  __pyx_tp_clear_9drf_turbo_6fields_ManyRelatedField, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_9drf_turbo_6fields_ManyRelatedField, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_9drf_turbo_6fields_ManyRelatedField, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_9drf_turbo_6fields_16ManyRelatedField_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_9drf_turbo_6fields_ManyRelatedField, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_9drf_turbo_6fields_ConstantField __pyx_vtable_9drf_turbo_6fields_ConstantField;

static PyObject *__pyx_tp_new_9drf_turbo_6fields_ConstantField(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_9drf_turbo_6fields_ConstantField *p;
  PyObject *o = __pyx_tp_new_9drf_turbo_6fields_Field(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_9drf_turbo_6fields_ConstantField *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_9drf_turbo_6fields_Field*)__pyx_vtabptr_9drf_turbo_6fields_ConstantField;
  p->constant = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_9drf_turbo_6fields_ConstantField(PyObject *o) {
  struct __pyx_obj_9drf_turbo_6fields_ConstantField *p = (struct __pyx_obj_9drf_turbo_6fields_ConstantField *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->constant);
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_9drf_turbo_6fields_Field(o);
}

static int __pyx_tp_traverse_9drf_turbo_6fields_ConstantField(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_9drf_turbo_6fields_ConstantField *p = (struct __pyx_obj_9drf_turbo_6fields_ConstantField *)o;
  e = __pyx_tp_traverse_9drf_turbo_6fields_Field(o, v, a); if (e) return e;
  if (p->constant) {
    e = (*v)(p->constant, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_9drf_turbo_6fields_ConstantField(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_9drf_turbo_6fields_ConstantField *p = (struct __pyx_obj_9drf_turbo_6fields_ConstantField *)o;
  __pyx_tp_clear_9drf_turbo_6fields_Field(o);
  tmp = ((PyObject*)p->constant);
  p->constant = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_9drf_turbo_6fields_13ConstantField_constant(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9drf_turbo_6fields_13ConstantField_8constant_1__get__(o);
}

static int __pyx_setprop_9drf_turbo_6fields_13ConstantField_constant(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9drf_turbo_6fields_13ConstantField_8constant_3__set__(o, v);
  }
  else {
    return __pyx_pw_9drf_turbo_6fields_13ConstantField_8constant_5__del__(o);
  }
}

static PyMethodDef __pyx_methods_9drf_turbo_6fields_ConstantField[] = {
  {"deserialize", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_13ConstantField_3deserialize, METH_VARARGS|METH_KEYWORDS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_9drf_turbo_6fields_13ConstantField_5__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_9drf_turbo_6fields_13ConstantField_7__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_9drf_turbo_6fields_ConstantField[] = {
  {(char *)"constant", __pyx_getprop_9drf_turbo_6fields_13ConstantField_constant, __pyx_setprop_9drf_turbo_6fields_13ConstantField_constant, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_9drf_turbo_6fields_ConstantField = {
  PyVarObject_HEAD_INIT(0, 0)
  "drf_turbo.fields.ConstantField", /*tp_name*/
  sizeof(struct __pyx_obj_9drf_turbo_6fields_ConstantField), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_9drf_turbo_6fields_ConstantField, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "\n    A field that always outputs a fixed value.\n\n    :param constant: The value to return.\n    :param kwargs: The same keyword arguments that :class:`Field` receives.\n    ", /*tp_doc*/
  __pyx_tp_traverse_9drf_turbo_6fields_ConstantField, /*tp_traverse*/
  __pyx_tp_clear_9drf_turbo_6fields_ConstantField, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_9drf_turbo_6fields_ConstantField, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_9drf_turbo_6fields_ConstantField, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_9drf_turbo_6fields_13ConstantField_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_9drf_turbo_6fields_ConstantField, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_9drf_turbo_6fields_RecursiveField __pyx_vtable_9drf_turbo_6fields_RecursiveField;

static PyObject *__pyx_tp_new_9drf_turbo_6fields_RecursiveField(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_9drf_turbo_6fields_RecursiveField *p;
  PyObject *o = __pyx_tp_new_9drf_turbo_6fields_Field(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_9drf_turbo_6fields_RecursiveField *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_9drf_turbo_6fields_Field*)__pyx_vtabptr_9drf_turbo_6fields_RecursiveField;
  p->many = Py_None; Py_INCREF(Py_None);
  p->context = Py_None; Py_INCREF(Py_None);
  p->only = Py_None; Py_INCREF(Py_None);
  p->exclude = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_9drf_turbo_6fields_RecursiveField(PyObject *o) {
  struct __pyx_obj_9drf_turbo_6fields_RecursiveField *p = (struct __pyx_obj_9drf_turbo_6fields_RecursiveField *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->many);
  Py_CLEAR(p->context);
  Py_CLEAR(p->only);
  Py_CLEAR(p->exclude);
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_9drf_turbo_6fields_Field(o);
}

static int __pyx_tp_traverse_9drf_turbo_6fields_RecursiveField(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_9drf_turbo_6fields_RecursiveField *p = (struct __pyx_obj_9drf_turbo_6fields_RecursiveField *)o;
  e = __pyx_tp_traverse_9drf_turbo_6fields_Field(o, v, a); if (e) return e;
  if (p->many) {
    e = (*v)(p->many, a); if (e) return e;
  }
  if (p->context) {
    e = (*v)(p->context, a); if (e) return e;
  }
  if (p->only) {
    e = (*v)(p->only, a); if (e) return e;
  }
  if (p->exclude) {
    e = (*v)(p->exclude, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_9drf_turbo_6fields_RecursiveField(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_9drf_turbo_6fields_RecursiveField *p = (struct __pyx_obj_9drf_turbo_6fields_RecursiveField *)o;
  __pyx_tp_clear_9drf_turbo_6fields_Field(o);
  tmp = ((PyObject*)p->many);
  p->many = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->context);
  p->context = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->only);
  p->only = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->exclude);
  p->exclude = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_9drf_turbo_6fields_14RecursiveField_many(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9drf_turbo_6fields_14RecursiveField_4many_1__get__(o);
}

static int __pyx_setprop_9drf_turbo_6fields_14RecursiveField_many(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9drf_turbo_6fields_14RecursiveField_4many_3__set__(o, v);
  }
  else {
    return __pyx_pw_9drf_turbo_6fields_14RecursiveField_4many_5__del__(o);
  }
}

static PyObject *__pyx_getprop_9drf_turbo_6fields_14RecursiveField_context(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9drf_turbo_6fields_14RecursiveField_7context_1__get__(o);
}

static int __pyx_setprop_9drf_turbo_6fields_14RecursiveField_context(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9drf_turbo_6fields_14RecursiveField_7context_3__set__(o, v);
  }
  else {
    return __pyx_pw_9drf_turbo_6fields_14RecursiveField_7context_5__del__(o);
  }
}

static PyObject *__pyx_getprop_9drf_turbo_6fields_14RecursiveField_only(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9drf_turbo_6fields_14RecursiveField_4only_1__get__(o);
}

static int __pyx_setprop_9drf_turbo_6fields_14RecursiveField_only(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9drf_turbo_6fields_14RecursiveField_4only_3__set__(o, v);
  }
  else {
    return __pyx_pw_9drf_turbo_6fields_14RecursiveField_4only_5__del__(o);
  }
}

static PyObject *__pyx_getprop_9drf_turbo_6fields_14RecursiveField_exclude(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9drf_turbo_6fields_14RecursiveField_7exclude_1__get__(o);
}

static int __pyx_setprop_9drf_turbo_6fields_14RecursiveField_exclude(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9drf_turbo_6fields_14RecursiveField_7exclude_3__set__(o, v);
  }
  else {
    return __pyx_pw_9drf_turbo_6fields_14RecursiveField_7exclude_5__del__(o);
  }
}

static PyMethodDef __pyx_methods_9drf_turbo_6fields_RecursiveField[] = {
  {"serialize", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_14RecursiveField_3serialize, METH_VARARGS|METH_KEYWORDS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_9drf_turbo_6fields_14RecursiveField_5__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_9drf_turbo_6fields_14RecursiveField_7__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_9drf_turbo_6fields_RecursiveField[] = {
  {(char *)"many", __pyx_getprop_9drf_turbo_6fields_14RecursiveField_many, __pyx_setprop_9drf_turbo_6fields_14RecursiveField_many, (char *)0, 0},
  {(char *)"context", __pyx_getprop_9drf_turbo_6fields_14RecursiveField_context, __pyx_setprop_9drf_turbo_6fields_14RecursiveField_context, (char *)0, 0},
  {(char *)"only", __pyx_getprop_9drf_turbo_6fields_14RecursiveField_only, __pyx_setprop_9drf_turbo_6fields_14RecursiveField_only, (char *)0, 0},
  {(char *)"exclude", __pyx_getprop_9drf_turbo_6fields_14RecursiveField_exclude, __pyx_setprop_9drf_turbo_6fields_14RecursiveField_exclude, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_9drf_turbo_6fields_RecursiveField = {
  PyVarObject_HEAD_INIT(0, 0)
  "drf_turbo.fields.RecursiveField", /*tp_name*/
  sizeof(struct __pyx_obj_9drf_turbo_6fields_RecursiveField), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_9drf_turbo_6fields_RecursiveField, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "\n    A field that recursively validates its data.\n\n    :param many: Whether the field is a collection of items.\n    :param context: The context passed to the field's :meth:`run_validation`.\n    :param only: A tuple or list of field names to include. \n    :param exclude: A tuple or list of field names to exclude.\n    :param kwargs: The same keyword arguments that :class:`Field` receives.\n    ", /*tp_doc*/
  __pyx_tp_traverse_9drf_turbo_6fields_RecursiveField, /*tp_traverse*/
  __pyx_tp_clear_9drf_turbo_6fields_RecursiveField, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_9drf_turbo_6fields_RecursiveField, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_9drf_turbo_6fields_RecursiveField, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_9drf_turbo_6fields_14RecursiveField_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_9drf_turbo_6fields_RecursiveField, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_9drf_turbo_6fields_MethodField __pyx_vtable_9drf_turbo_6fields_MethodField;

static PyObject *__pyx_tp_new_9drf_turbo_6fields_MethodField(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_9drf_turbo_6fields_MethodField *p;
  PyObject *o = __pyx_tp_new_9drf_turbo_6fields_Field(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_9drf_turbo_6fields_MethodField *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_9drf_turbo_6fields_Field*)__pyx_vtabptr_9drf_turbo_6fields_MethodField;
  p->method_name = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_9drf_turbo_6fields_MethodField(PyObject *o) {
  struct __pyx_obj_9drf_turbo_6fields_MethodField *p = (struct __pyx_obj_9drf_turbo_6fields_MethodField *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->method_name);
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_9drf_turbo_6fields_Field(o);
}

static int __pyx_tp_traverse_9drf_turbo_6fields_MethodField(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_9drf_turbo_6fields_MethodField *p = (struct __pyx_obj_9drf_turbo_6fields_MethodField *)o;
  e = __pyx_tp_traverse_9drf_turbo_6fields_Field(o, v, a); if (e) return e;
  if (p->method_name) {
    e = (*v)(p->method_name, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_9drf_turbo_6fields_MethodField(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_9drf_turbo_6fields_MethodField *p = (struct __pyx_obj_9drf_turbo_6fields_MethodField *)o;
  __pyx_tp_clear_9drf_turbo_6fields_Field(o);
  tmp = ((PyObject*)p->method_name);
  p->method_name = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_9drf_turbo_6fields_11MethodField_method_name(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9drf_turbo_6fields_11MethodField_11method_name_1__get__(o);
}

static int __pyx_setprop_9drf_turbo_6fields_11MethodField_method_name(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9drf_turbo_6fields_11MethodField_11method_name_3__set__(o, v);
  }
  else {
    return __pyx_pw_9drf_turbo_6fields_11MethodField_11method_name_5__del__(o);
  }
}

static PyMethodDef __pyx_methods_9drf_turbo_6fields_MethodField[] = {
  {"method_getter", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9drf_turbo_6fields_11MethodField_3method_getter, METH_VARARGS|METH_KEYWORDS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_9drf_turbo_6fields_11MethodField_5__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_9drf_turbo_6fields_11MethodField_7__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_9drf_turbo_6fields_MethodField[] = {
  {(char *)"method_name", __pyx_getprop_9drf_turbo_6fields_11MethodField_method_name, __pyx_setprop_9drf_turbo_6fields_11MethodField_method_name, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_9drf_turbo_6fields_MethodField = {
  PyVarObject_HEAD_INIT(0, 0)
  "drf_turbo.fields.MethodField", /*tp_name*/
  sizeof(struct __pyx_obj_9drf_turbo_6fields_MethodField), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_9drf_turbo_6fields_MethodField, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "\n    A field that calls a method on the serializer instead of simply returning a value.\n\n    :param method_name: The name of the method to call.\n    :param kwargs: The same keyword arguments that :class:`Field` receives.\n    ", /*tp_doc*/
  __pyx_tp_traverse_9drf_turbo_6fields_MethodField, /*tp_traverse*/
  __pyx_tp_clear_9drf_turbo_6fields_MethodField, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_9drf_turbo_6fields_MethodField, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_9drf_turbo_6fields_MethodField, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_9drf_turbo_6fields_11MethodField_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_9drf_turbo_6fields_MethodField, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};

static int __pyx_import_star_set(PyObject *o, PyObject* py_name, char *name) {
  static const char* internal_type_names[] = {
    "ArrayField",
    "BoolField",
    "ChoiceField",
    "ConstantField",
    "DateField",
    "DateTimeField",
    "DecimalField",
    "DictField",
    "EmailField",
    "Field",
    "FileField",
    "FloatField",
    "IPField",
    "IntField",
    "JSONField",
    "ManyRelatedField",
    "MethodField",
    "MultipleChoiceField",
    "PasswordField",
    "RecursiveField",
    "RegexField",
    "RelatedField",
    "SkipField",
    "SlugField",
    "StrField",
    "TimeField",
    "URLField",
    "UUIDField",
    "__pyx_opt_args_9drf_turbo_6fields_5Field_get_attribute",
    0
  };
  const char** type_name = internal_type_names;
  while (*type_name) {
    if (__Pyx_StrEq(name, *type_name)) {
      PyErr_Format(PyExc_TypeError, "Cannot overwrite C type %s", name);
      goto bad;
    }
    type_name++;
  }
  if (0);
  else if (__Pyx_StrEq(name, "NO_DEFAULT")) {
    Py_INCREF(o);
    Py_DECREF(__pyx_v_9drf_turbo_6fields_NO_DEFAULT);
    __pyx_v_9drf_turbo_6fields_NO_DEFAULT = o;
  }
  else {
    if (PyObject_SetAttr(__pyx_m, py_name, o) < 0) goto bad;
  }
  return 0;
  bad:
  return -1;
}

static int
__Pyx_import_all_from(PyObject *locals, PyObject *v)
{
    PyObject *all = PyObject_GetAttrString(v, "__all__");
    PyObject *dict, *name, *value;
    int skip_leading_underscores = 0;
    int pos, err;
    if (all == NULL) {
        if (!PyErr_ExceptionMatches(PyExc_AttributeError))
            return -1;
        PyErr_Clear();
        dict = PyObject_GetAttrString(v, "__dict__");
        if (dict == NULL) {
            if (!PyErr_ExceptionMatches(PyExc_AttributeError))
                return -1;
            PyErr_SetString(PyExc_ImportError,
            "from-import-* object has no __dict__ and no __all__");
            return -1;
        }
#if PY_MAJOR_VERSION < 3
        all = PyObject_CallMethod(dict, (char *)"keys", NULL);
#else
        all = PyMapping_Keys(dict);
#endif
        Py_DECREF(dict);
        if (all == NULL)
            return -1;
        skip_leading_underscores = 1;
    }
    for (pos = 0, err = 0; ; pos++) {
        name = PySequence_GetItem(all, pos);
        if (name == NULL) {
            if (!PyErr_ExceptionMatches(PyExc_IndexError))
                err = -1;
            else
                PyErr_Clear();
            break;
        }
        if (skip_leading_underscores &&
#if PY_MAJOR_VERSION < 3
            likely(PyString_Check(name)) &&
            PyString_AS_STRING(name)[0] == '_')
#else
            likely(PyUnicode_Check(name)) &&
            likely(__Pyx_PyUnicode_GET_LENGTH(name)) &&
            __Pyx_PyUnicode_READ_CHAR(name, 0) == '_')
#endif
        {
            Py_DECREF(name);
            continue;
        }
        value = PyObject_GetAttr(v, name);
        if (value == NULL)
            err = -1;
        else if (PyDict_CheckExact(locals))
            err = PyDict_SetItem(locals, name, value);
        else
            err = PyObject_SetItem(locals, name, value);
        Py_DECREF(name);
        Py_XDECREF(value);
        if (err != 0)
            break;
    }
    Py_DECREF(all);
    return err;
}
static int __pyx_import_star(PyObject* m) {
    int i;
    int ret = -1;
    char* s;
    PyObject *locals = 0;
    PyObject *list = 0;
#if PY_MAJOR_VERSION >= 3
    PyObject *utf8_name = 0;
#endif
    PyObject *name;
    PyObject *item;
    locals = PyDict_New();              if (!locals) goto bad;
    if (__Pyx_import_all_from(locals, m) < 0) goto bad;
    list = PyDict_Items(locals);        if (!list) goto bad;
    for(i=0; i<PyList_GET_SIZE(list); i++) {
        name = PyTuple_GET_ITEM(PyList_GET_ITEM(list, i), 0);
        item = PyTuple_GET_ITEM(PyList_GET_ITEM(list, i), 1);
#if PY_MAJOR_VERSION >= 3
        utf8_name = PyUnicode_AsUTF8String(name);
        if (!utf8_name) goto bad;
        s = PyBytes_AS_STRING(utf8_name);
        if (__pyx_import_star_set(item, name, s) < 0) goto bad;
        Py_DECREF(utf8_name); utf8_name = 0;
#else
        s = PyString_AsString(name);
        if (!s) goto bad;
        if (__pyx_import_star_set(item, name, s) < 0) goto bad;
#endif
    }
    ret = 0;
bad:
    Py_XDECREF(locals);
    Py_XDECREF(list);
#if PY_MAJOR_VERSION >= 3
    Py_XDECREF(utf8_name);
#endif
    return ret;
}



#if PY_MAJOR_VERSION >= 3
#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec_fields(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec_fields},
  {0, NULL}
};
#endif

static struct PyModuleDef __pyx_moduledef = {
    PyModuleDef_HEAD_INIT,
    "fields",
    0, /* m_doc */
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    0, /* m_size */
  #else
    -1, /* m_size */
  #endif
    __pyx_methods /* m_methods */,
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    __pyx_moduledef_slots, /* m_slots */
  #else
    NULL, /* m_reload */
  #endif
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif
#ifndef CYTHON_SMALL_CODE
#if defined(__clang__)
    #define CYTHON_SMALL_CODE
#elif defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3))
    #define CYTHON_SMALL_CODE __attribute__((cold))
#else
    #define CYTHON_SMALL_CODE
#endif
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_kp_s_0, __pyx_k_0, sizeof(__pyx_k_0), 0, 0, 1, 0},
  {&__pyx_kp_s_00_00, __pyx_k_00_00, sizeof(__pyx_k_00_00), 0, 0, 1, 0},
  {&__pyx_kp_s_0_9A_Z___0_9A_Z_001_010_013_0, __pyx_k_0_9A_Z___0_9A_Z_001_010_013_0, sizeof(__pyx_k_0_9A_Z___0_9A_Z_001_010_013_0), 0, 0, 1, 0},
  {&__pyx_kp_s_0_s, __pyx_k_0_s, sizeof(__pyx_k_0_s), 0, 0, 1, 0},
  {&__pyx_kp_s_1, __pyx_k_1, sizeof(__pyx_k_1), 0, 0, 1, 0},
  {&__pyx_kp_s_1_2, __pyx_k_1_2, sizeof(__pyx_k_1_2), 0, 0, 1, 0},
  {&__pyx_kp_s_A_valid_integer_is_required, __pyx_k_A_valid_integer_is_required, sizeof(__pyx_k_A_valid_integer_is_required), 0, 0, 1, 0},
  {&__pyx_kp_s_A_valid_number_is_required, __pyx_k_A_valid_number_is_required, sizeof(__pyx_k_A_valid_number_is_required), 0, 0, 1, 0},
  {&__pyx_n_s_ArrayField, __pyx_k_ArrayField, sizeof(__pyx_k_ArrayField), 0, 0, 1, 1},
  {&__pyx_n_s_AssertionError, __pyx_k_AssertionError, sizeof(__pyx_k_AssertionError), 0, 0, 1, 1},
  {&__pyx_n_s_AttributeError, __pyx_k_AttributeError, sizeof(__pyx_k_AttributeError), 0, 0, 1, 1},
  {&__pyx_n_s_BoolField, __pyx_k_BoolField, sizeof(__pyx_k_BoolField), 0, 0, 1, 1},
  {&__pyx_n_s_COERCE_DECIMAL_TO_STRING, __pyx_k_COERCE_DECIMAL_TO_STRING, sizeof(__pyx_k_COERCE_DECIMAL_TO_STRING), 0, 0, 1, 1},
  {&__pyx_n_s_ChoiceField, __pyx_k_ChoiceField, sizeof(__pyx_k_ChoiceField), 0, 0, 1, 1},
  {&__pyx_n_s_ConstantField, __pyx_k_ConstantField, sizeof(__pyx_k_ConstantField), 0, 0, 1, 1},
  {&__pyx_n_s_DATETIME_FORMAT, __pyx_k_DATETIME_FORMAT, sizeof(__pyx_k_DATETIME_FORMAT), 0, 0, 1, 1},
  {&__pyx_n_s_DATETIME_INPUT_FORMATS, __pyx_k_DATETIME_INPUT_FORMATS, sizeof(__pyx_k_DATETIME_INPUT_FORMATS), 0, 0, 1, 1},
  {&__pyx_n_s_DATE_FORMAT, __pyx_k_DATE_FORMAT, sizeof(__pyx_k_DATE_FORMAT), 0, 0, 1, 1},
  {&__pyx_n_s_DATE_INPUT_FORMATS, __pyx_k_DATE_INPUT_FORMATS, sizeof(__pyx_k_DATE_INPUT_FORMATS), 0, 0, 1, 1},
  {&__pyx_n_s_DateField, __pyx_k_DateField, sizeof(__pyx_k_DateField), 0, 0, 1, 1},
  {&__pyx_n_s_DateTimeField, __pyx_k_DateTimeField, sizeof(__pyx_k_DateTimeField), 0, 0, 1, 1},
  {&__pyx_kp_s_Datetime_value_out_of_range, __pyx_k_Datetime_value_out_of_range, sizeof(__pyx_k_Datetime_value_out_of_range), 0, 0, 1, 0},
  {&__pyx_n_s_Decimal, __pyx_k_Decimal, sizeof(__pyx_k_Decimal), 0, 0, 1, 1},
  {&__pyx_n_s_DecimalException, __pyx_k_DecimalException, sizeof(__pyx_k_DecimalException), 0, 0, 1, 1},
  {&__pyx_n_s_DecimalField, __pyx_k_DecimalField, sizeof(__pyx_k_DecimalField), 0, 0, 1, 1},
  {&__pyx_n_s_DictField, __pyx_k_DictField, sizeof(__pyx_k_DictField), 0, 0, 1, 1},
  {&__pyx_n_s_DjangoValidationError, __pyx_k_DjangoValidationError, sizeof(__pyx_k_DjangoValidationError), 0, 0, 1, 1},
  {&__pyx_n_s_EMAIL_REGEX, __pyx_k_EMAIL_REGEX, sizeof(__pyx_k_EMAIL_REGEX), 0, 0, 1, 1},
  {&__pyx_n_s_EmailField, __pyx_k_EmailField, sizeof(__pyx_k_EmailField), 0, 0, 1, 1},
  {&__pyx_kp_s_Ensure_that_there_are_no_more_th, __pyx_k_Ensure_that_there_are_no_more_th, sizeof(__pyx_k_Ensure_that_there_are_no_more_th), 0, 0, 1, 0},
  {&__pyx_kp_s_Ensure_that_there_are_no_more_th_2, __pyx_k_Ensure_that_there_are_no_more_th_2, sizeof(__pyx_k_Ensure_that_there_are_no_more_th_2), 0, 0, 1, 0},
  {&__pyx_kp_s_Ensure_that_there_are_no_more_th_3, __pyx_k_Ensure_that_there_are_no_more_th_3, sizeof(__pyx_k_Ensure_that_there_are_no_more_th_3), 0, 0, 1, 0},
  {&__pyx_kp_s_Ensure_this_filename_has_at_most, __pyx_k_Ensure_this_filename_has_at_most, sizeof(__pyx_k_Ensure_this_filename_has_at_most), 0, 0, 1, 0},
  {&__pyx_kp_s_Enter_a_valid_IPv4_or_IPv6_addre, __pyx_k_Enter_a_valid_IPv4_or_IPv6_addre, sizeof(__pyx_k_Enter_a_valid_IPv4_or_IPv6_addre), 0, 0, 1, 0},
  {&__pyx_kp_s_Enter_a_valid_URL, __pyx_k_Enter_a_valid_URL, sizeof(__pyx_k_Enter_a_valid_URL), 0, 0, 1, 0},
  {&__pyx_kp_s_Enter_a_valid_email_address, __pyx_k_Enter_a_valid_email_address, sizeof(__pyx_k_Enter_a_valid_email_address), 0, 0, 1, 0},
  {&__pyx_kp_s_Expected_a_date_but_got_a_dateti, __pyx_k_Expected_a_date_but_got_a_dateti, sizeof(__pyx_k_Expected_a_date_but_got_a_dateti), 0, 0, 1, 0},
  {&__pyx_kp_s_Expected_a_date_but_got_a_dateti_2, __pyx_k_Expected_a_date_but_got_a_dateti_2, sizeof(__pyx_k_Expected_a_date_but_got_a_dateti_2), 0, 0, 1, 0},
  {&__pyx_kp_s_Expected_a_datetime_but_got_a_da, __pyx_k_Expected_a_datetime_but_got_a_da, sizeof(__pyx_k_Expected_a_datetime_but_got_a_da), 0, 0, 1, 0},
  {&__pyx_kp_s_Expected_a_dict_of_items_but_got, __pyx_k_Expected_a_dict_of_items_but_got, sizeof(__pyx_k_Expected_a_dict_of_items_but_got), 0, 0, 1, 0},
  {&__pyx_kp_s_Expected_a_list_of_items_but_got, __pyx_k_Expected_a_list_of_items_but_got, sizeof(__pyx_k_Expected_a_list_of_items_but_got), 0, 0, 1, 0},
  {&__pyx_kp_s_Expected_a_time_but_got_a_dateti, __pyx_k_Expected_a_time_but_got_a_dateti, sizeof(__pyx_k_Expected_a_time_but_got_a_dateti), 0, 0, 1, 0},
  {&__pyx_n_s_F, __pyx_k_F, sizeof(__pyx_k_F), 0, 0, 1, 1},
  {&__pyx_n_s_FALSE, __pyx_k_FALSE, sizeof(__pyx_k_FALSE), 0, 0, 1, 1},
  {&__pyx_n_s_False, __pyx_k_False, sizeof(__pyx_k_False), 0, 0, 1, 1},
  {&__pyx_n_s_Field, __pyx_k_Field, sizeof(__pyx_k_Field), 0, 0, 1, 1},
  {&__pyx_n_s_FileField, __pyx_k_FileField, sizeof(__pyx_k_FileField), 0, 0, 1, 1},
  {&__pyx_n_s_FloatField, __pyx_k_FloatField, sizeof(__pyx_k_FloatField), 0, 0, 1, 1},
  {&__pyx_kp_s_Got_exc_type_when_attempting_to, __pyx_k_Got_exc_type_when_attempting_to, sizeof(__pyx_k_Got_exc_type_when_attempting_to), 0, 0, 1, 0},
  {&__pyx_n_s_IGNORECASE, __pyx_k_IGNORECASE, sizeof(__pyx_k_IGNORECASE), 0, 0, 1, 1},
  {&__pyx_n_s_IPField, __pyx_k_IPField, sizeof(__pyx_k_IPField), 0, 0, 1, 1},
  {&__pyx_n_s_ISO_8601, __pyx_k_ISO_8601, sizeof(__pyx_k_ISO_8601), 0, 0, 1, 1},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0x03, __pyx_k_Incompatible_checksums_s_vs_0x03, sizeof(__pyx_k_Incompatible_checksums_s_vs_0x03), 0, 0, 1, 0},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0x0c, __pyx_k_Incompatible_checksums_s_vs_0x0c, sizeof(__pyx_k_Incompatible_checksums_s_vs_0x0c), 0, 0, 1, 0},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0x0f, __pyx_k_Incompatible_checksums_s_vs_0x0f, sizeof(__pyx_k_Incompatible_checksums_s_vs_0x0f), 0, 0, 1, 0},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0x36, __pyx_k_Incompatible_checksums_s_vs_0x36, sizeof(__pyx_k_Incompatible_checksums_s_vs_0x36), 0, 0, 1, 0},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0x38, __pyx_k_Incompatible_checksums_s_vs_0x38, sizeof(__pyx_k_Incompatible_checksums_s_vs_0x38), 0, 0, 1, 0},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0x4f, __pyx_k_Incompatible_checksums_s_vs_0x4f, sizeof(__pyx_k_Incompatible_checksums_s_vs_0x4f), 0, 0, 1, 0},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0x57, __pyx_k_Incompatible_checksums_s_vs_0x57, sizeof(__pyx_k_Incompatible_checksums_s_vs_0x57), 0, 0, 1, 0},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0x5e, __pyx_k_Incompatible_checksums_s_vs_0x5e, sizeof(__pyx_k_Incompatible_checksums_s_vs_0x5e), 0, 0, 1, 0},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0x66, __pyx_k_Incompatible_checksums_s_vs_0x66, sizeof(__pyx_k_Incompatible_checksums_s_vs_0x66), 0, 0, 1, 0},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0x69, __pyx_k_Incompatible_checksums_s_vs_0x69, sizeof(__pyx_k_Incompatible_checksums_s_vs_0x69), 0, 0, 1, 0},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0x72, __pyx_k_Incompatible_checksums_s_vs_0x72, sizeof(__pyx_k_Incompatible_checksums_s_vs_0x72), 0, 0, 1, 0},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0x91, __pyx_k_Incompatible_checksums_s_vs_0x91, sizeof(__pyx_k_Incompatible_checksums_s_vs_0x91), 0, 0, 1, 0},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0xc7, __pyx_k_Incompatible_checksums_s_vs_0xc7, sizeof(__pyx_k_Incompatible_checksums_s_vs_0xc7), 0, 0, 1, 0},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0xd4, __pyx_k_Incompatible_checksums_s_vs_0xd4, sizeof(__pyx_k_Incompatible_checksums_s_vs_0xd4), 0, 0, 1, 0},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0xda, __pyx_k_Incompatible_checksums_s_vs_0xda, sizeof(__pyx_k_Incompatible_checksums_s_vs_0xda), 0, 0, 1, 0},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0xe1, __pyx_k_Incompatible_checksums_s_vs_0xe1, sizeof(__pyx_k_Incompatible_checksums_s_vs_0xe1), 0, 0, 1, 0},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0xe8, __pyx_k_Incompatible_checksums_s_vs_0xe8, sizeof(__pyx_k_Incompatible_checksums_s_vs_0xe8), 0, 0, 1, 0},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0xf1, __pyx_k_Incompatible_checksums_s_vs_0xf1, sizeof(__pyx_k_Incompatible_checksums_s_vs_0xf1), 0, 0, 1, 0},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0xf3, __pyx_k_Incompatible_checksums_s_vs_0xf3, sizeof(__pyx_k_Incompatible_checksums_s_vs_0xf3), 0, 0, 1, 0},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0xf4, __pyx_k_Incompatible_checksums_s_vs_0xf4, sizeof(__pyx_k_Incompatible_checksums_s_vs_0xf4), 0, 0, 1, 0},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0xf7, __pyx_k_Incompatible_checksums_s_vs_0xf7, sizeof(__pyx_k_Incompatible_checksums_s_vs_0xf7), 0, 0, 1, 0},
  {&__pyx_kp_s_Incorrect_type_Expected_pk_value, __pyx_k_Incorrect_type_Expected_pk_value, sizeof(__pyx_k_Incorrect_type_Expected_pk_value), 0, 0, 1, 0},
  {&__pyx_n_s_Inf, __pyx_k_Inf, sizeof(__pyx_k_Inf), 0, 0, 1, 1},
  {&__pyx_kp_s_Inf_2, __pyx_k_Inf_2, sizeof(__pyx_k_Inf_2), 0, 0, 1, 0},
  {&__pyx_n_s_IntField, __pyx_k_IntField, sizeof(__pyx_k_IntField), 0, 0, 1, 1},
  {&__pyx_n_s_InvalidTimeError, __pyx_k_InvalidTimeError, sizeof(__pyx_k_InvalidTimeError), 0, 0, 1, 1},
  {&__pyx_kp_s_Invalid_datetime_for_the_timezon, __pyx_k_Invalid_datetime_for_the_timezon, sizeof(__pyx_k_Invalid_datetime_for_the_timezon), 0, 0, 1, 0},
  {&__pyx_kp_s_Invalid_pk_pk_value_object_does, __pyx_k_Invalid_pk_pk_value_object_does, sizeof(__pyx_k_Invalid_pk_pk_value_object_does), 0, 0, 1, 0},
  {&__pyx_kp_s_Invalid_rounding_option_s_Valid, __pyx_k_Invalid_rounding_option_s_Valid, sizeof(__pyx_k_Invalid_rounding_option_s_Valid), 0, 0, 1, 0},
  {&__pyx_n_s_JSONField, __pyx_k_JSONField, sizeof(__pyx_k_JSONField), 0, 0, 1, 1},
  {&__pyx_n_s_KeyError, __pyx_k_KeyError, sizeof(__pyx_k_KeyError), 0, 0, 1, 1},
  {&__pyx_n_s_MAX_STRING_LENGTH, __pyx_k_MAX_STRING_LENGTH, sizeof(__pyx_k_MAX_STRING_LENGTH), 0, 0, 1, 1},
  {&__pyx_n_s_ManyRelatedField, __pyx_k_ManyRelatedField, sizeof(__pyx_k_ManyRelatedField), 0, 0, 1, 1},
  {&__pyx_kp_s_May_not_be_blank, __pyx_k_May_not_be_blank, sizeof(__pyx_k_May_not_be_blank), 0, 0, 1, 0},
  {&__pyx_kp_s_May_not_set_both_read_only_and_w, __pyx_k_May_not_set_both_read_only_and_w, sizeof(__pyx_k_May_not_set_both_read_only_and_w), 0, 0, 1, 0},
  {&__pyx_kp_s_May_not_set_both_required_and_de, __pyx_k_May_not_set_both_required_and_de, sizeof(__pyx_k_May_not_set_both_required_and_de), 0, 0, 1, 0},
  {&__pyx_n_s_MethodField, __pyx_k_MethodField, sizeof(__pyx_k_MethodField), 0, 0, 1, 1},
  {&__pyx_n_s_MultipleChoiceField, __pyx_k_MultipleChoiceField, sizeof(__pyx_k_MultipleChoiceField), 0, 0, 1, 1},
  {&__pyx_kp_s_Must_be_None, __pyx_k_Must_be_None, sizeof(__pyx_k_Must_be_None), 0, 0, 1, 0},
  {&__pyx_kp_s_Must_be_constant, __pyx_k_Must_be_constant, sizeof(__pyx_k_Must_be_constant), 0, 0, 1, 0},
  {&__pyx_kp_s_Must_be_greater_than_or_equal_to, __pyx_k_Must_be_greater_than_or_equal_to, sizeof(__pyx_k_Must_be_greater_than_or_equal_to), 0, 0, 1, 0},
  {&__pyx_kp_s_Must_be_less_than_or_equal_to_ma, __pyx_k_Must_be_less_than_or_equal_to_ma, sizeof(__pyx_k_Must_be_less_than_or_equal_to_ma), 0, 0, 1, 0},
  {&__pyx_kp_s_Must_have_at_least_min_items_ite, __pyx_k_Must_have_at_least_min_items_ite, sizeof(__pyx_k_Must_have_at_least_min_items_ite), 0, 0, 1, 0},
  {&__pyx_kp_s_Must_have_at_least_min_length_ch, __pyx_k_Must_have_at_least_min_length_ch, sizeof(__pyx_k_Must_have_at_least_min_length_ch), 0, 0, 1, 0},
  {&__pyx_kp_s_Must_have_exact_items_items, __pyx_k_Must_have_exact_items_items, sizeof(__pyx_k_Must_have_exact_items_items), 0, 0, 1, 0},
  {&__pyx_kp_s_Must_have_no_more_than_max_items, __pyx_k_Must_have_no_more_than_max_items, sizeof(__pyx_k_Must_have_no_more_than_max_items), 0, 0, 1, 0},
  {&__pyx_kp_s_Must_have_no_more_than_max_lengt, __pyx_k_Must_have_no_more_than_max_lengt, sizeof(__pyx_k_Must_have_no_more_than_max_lengt), 0, 0, 1, 0},
  {&__pyx_n_s_NONE, __pyx_k_NONE, sizeof(__pyx_k_NONE), 0, 0, 1, 1},
  {&__pyx_n_s_NO_DEFAULT, __pyx_k_NO_DEFAULT, sizeof(__pyx_k_NO_DEFAULT), 0, 0, 1, 1},
  {&__pyx_n_s_NULL, __pyx_k_NULL, sizeof(__pyx_k_NULL), 0, 0, 1, 1},
  {&__pyx_kp_s_Nnot_a_valid_unicode_slug, __pyx_k_Nnot_a_valid_unicode_slug, sizeof(__pyx_k_Nnot_a_valid_unicode_slug), 0, 0, 1, 0},
  {&__pyx_kp_s_No_file_was_submitted, __pyx_k_No_file_was_submitted, sizeof(__pyx_k_No_file_was_submitted), 0, 0, 1, 0},
  {&__pyx_kp_s_No_filename_could_be_determined, __pyx_k_No_filename_could_be_determined, sizeof(__pyx_k_No_filename_could_be_determined), 0, 0, 1, 0},
  {&__pyx_n_s_None, __pyx_k_None, sizeof(__pyx_k_None), 0, 0, 1, 1},
  {&__pyx_kp_s_Not_a_valid_JSON, __pyx_k_Not_a_valid_JSON, sizeof(__pyx_k_Not_a_valid_JSON), 0, 0, 1, 0},
  {&__pyx_kp_s_Not_a_valid_UUID, __pyx_k_Not_a_valid_UUID, sizeof(__pyx_k_Not_a_valid_UUID), 0, 0, 1, 0},
  {&__pyx_kp_s_Not_a_valid_boolean, __pyx_k_Not_a_valid_boolean, sizeof(__pyx_k_Not_a_valid_boolean), 0, 0, 1, 0},
  {&__pyx_kp_s_Not_a_valid_date, __pyx_k_Not_a_valid_date, sizeof(__pyx_k_Not_a_valid_date), 0, 0, 1, 0},
  {&__pyx_kp_s_Not_a_valid_datetime, __pyx_k_Not_a_valid_datetime, sizeof(__pyx_k_Not_a_valid_datetime), 0, 0, 1, 0},
  {&__pyx_kp_s_Not_a_valid_slug, __pyx_k_Not_a_valid_slug, sizeof(__pyx_k_Not_a_valid_slug), 0, 0, 1, 0},
  {&__pyx_kp_s_Not_a_valid_string, __pyx_k_Not_a_valid_string, sizeof(__pyx_k_Not_a_valid_string), 0, 0, 1, 0},
  {&__pyx_kp_s_Not_a_valid_time, __pyx_k_Not_a_valid_time, sizeof(__pyx_k_Not_a_valid_time), 0, 0, 1, 0},
  {&__pyx_n_s_Null, __pyx_k_Null, sizeof(__pyx_k_Null), 0, 0, 1, 1},
  {&__pyx_kp_s_Null_characters_are_not_allowed, __pyx_k_Null_characters_are_not_allowed, sizeof(__pyx_k_Null_characters_are_not_allowed), 0, 0, 1, 0},
  {&__pyx_n_s_ObjectDoesNotExist, __pyx_k_ObjectDoesNotExist, sizeof(__pyx_k_ObjectDoesNotExist), 0, 0, 1, 1},
  {&__pyx_n_s_OnlyAndExcludeError, __pyx_k_OnlyAndExcludeError, sizeof(__pyx_k_OnlyAndExcludeError), 0, 0, 1, 1},
  {&__pyx_n_s_OverflowError, __pyx_k_OverflowError, sizeof(__pyx_k_OverflowError), 0, 0, 1, 1},
  {&__pyx_n_s_PasswordField, __pyx_k_PasswordField, sizeof(__pyx_k_PasswordField), 0, 0, 1, 1},
  {&__pyx_n_s_PickleError, __pyx_k_PickleError, sizeof(__pyx_k_PickleError), 0, 0, 1, 1},
  {&__pyx_n_s_ROUND, __pyx_k_ROUND, sizeof(__pyx_k_ROUND), 0, 0, 1, 1},
  {&__pyx_n_s_RecursiveField, __pyx_k_RecursiveField, sizeof(__pyx_k_RecursiveField), 0, 0, 1, 1},
  {&__pyx_n_s_RegexField, __pyx_k_RegexField, sizeof(__pyx_k_RegexField), 0, 0, 1, 1},
  {&__pyx_n_s_RelatedField, __pyx_k_RelatedField, sizeof(__pyx_k_RelatedField), 0, 0, 1, 1},
  {&__pyx_n_s_SkipField, __pyx_k_SkipField, sizeof(__pyx_k_SkipField), 0, 0, 1, 1},
  {&__pyx_n_s_SlugField, __pyx_k_SlugField, sizeof(__pyx_k_SlugField), 0, 0, 1, 1},
  {&__pyx_n_s_StrField, __pyx_k_StrField, sizeof(__pyx_k_StrField), 0, 0, 1, 1},
  {&__pyx_n_s_StringNotCollectionError, __pyx_k_StringNotCollectionError, sizeof(__pyx_k_StringNotCollectionError), 0, 0, 1, 1},
  {&__pyx_kp_s_String_value_too_large, __pyx_k_String_value_too_large, sizeof(__pyx_k_String_value_too_large), 0, 0, 1, 0},
  {&__pyx_n_s_T, __pyx_k_T, sizeof(__pyx_k_T), 0, 0, 1, 1},
  {&__pyx_n_s_TIME_INPUT_FORMATS, __pyx_k_TIME_INPUT_FORMATS, sizeof(__pyx_k_TIME_INPUT_FORMATS), 0, 0, 1, 1},
  {&__pyx_n_s_TRUE, __pyx_k_TRUE, sizeof(__pyx_k_TRUE), 0, 0, 1, 1},
  {&__pyx_kp_s_The_submitted_data_was_not_a_fil, __pyx_k_The_submitted_data_was_not_a_fil, sizeof(__pyx_k_The_submitted_data_was_not_a_fil), 0, 0, 1, 0},
  {&__pyx_kp_s_The_submitted_file_is_empty, __pyx_k_The_submitted_file_is_empty, sizeof(__pyx_k_The_submitted_file_is_empty), 0, 0, 1, 0},
  {&__pyx_kp_s_The_validators_parameter_must_be, __pyx_k_The_validators_parameter_must_be, sizeof(__pyx_k_The_validators_parameter_must_be), 0, 0, 1, 0},
  {&__pyx_kp_s_This_dict_may_not_be_empty, __pyx_k_This_dict_may_not_be_empty, sizeof(__pyx_k_This_dict_may_not_be_empty), 0, 0, 1, 0},
  {&__pyx_kp_s_This_field_is_required, __pyx_k_This_field_is_required, sizeof(__pyx_k_This_field_is_required), 0, 0, 1, 0},
  {&__pyx_kp_s_This_field_may_not_be_null, __pyx_k_This_field_may_not_be_null, sizeof(__pyx_k_This_field_may_not_be_null), 0, 0, 1, 0},
  {&__pyx_kp_s_This_list_may_not_be_empty, __pyx_k_This_list_may_not_be_empty, sizeof(__pyx_k_This_list_may_not_be_empty), 0, 0, 1, 0},
  {&__pyx_kp_s_This_selection_may_not_be_empty, __pyx_k_This_selection_may_not_be_empty, sizeof(__pyx_k_This_selection_may_not_be_empty), 0, 0, 1, 0},
  {&__pyx_kp_s_This_value_does_not_match_the_re, __pyx_k_This_value_does_not_match_the_re, sizeof(__pyx_k_This_value_does_not_match_the_re), 0, 0, 1, 0},
  {&__pyx_n_s_TimeField, __pyx_k_TimeField, sizeof(__pyx_k_TimeField), 0, 0, 1, 1},
  {&__pyx_n_s_True, __pyx_k_True, sizeof(__pyx_k_True), 0, 0, 1, 1},
  {&__pyx_n_s_TypeError, __pyx_k_TypeError, sizeof(__pyx_k_TypeError), 0, 0, 1, 1},
  {&__pyx_n_s_UNICODE, __pyx_k_UNICODE, sizeof(__pyx_k_UNICODE), 0, 0, 1, 1},
  {&__pyx_n_s_URLField, __pyx_k_URLField, sizeof(__pyx_k_URLField), 0, 0, 1, 1},
  {&__pyx_n_s_URL_REGEX, __pyx_k_URL_REGEX, sizeof(__pyx_k_URL_REGEX), 0, 0, 1, 1},
  {&__pyx_n_s_USE_TZ, __pyx_k_USE_TZ, sizeof(__pyx_k_USE_TZ), 0, 0, 1, 1},
  {&__pyx_n_s_UUID, __pyx_k_UUID, sizeof(__pyx_k_UUID), 0, 0, 1, 1},
  {&__pyx_n_s_UUIDField, __pyx_k_UUIDField, sizeof(__pyx_k_UUIDField), 0, 0, 1, 1},
  {&__pyx_n_s_ValidationError, __pyx_k_ValidationError, sizeof(__pyx_k_ValidationError), 0, 0, 1, 1},
  {&__pyx_n_s_ValueError, __pyx_k_ValueError, sizeof(__pyx_k_ValueError), 0, 0, 1, 1},
  {&__pyx_kp_s_You_should_use_either_only_or_ex, __pyx_k_You_should_use_either_only_or_ex, sizeof(__pyx_k_You_should_use_either_only_or_ex), 0, 0, 1, 0},
  {&__pyx_n_s_Z, __pyx_k_Z, sizeof(__pyx_k_Z), 0, 0, 1, 1},
  {&__pyx_kp_s__12, __pyx_k__12, sizeof(__pyx_k__12), 0, 0, 1, 0},
  {&__pyx_kp_s__3, __pyx_k__3, sizeof(__pyx_k__3), 0, 0, 1, 0},
  {&__pyx_n_s__31, __pyx_k__31, sizeof(__pyx_k__31), 0, 0, 1, 1},
  {&__pyx_n_s__5, __pyx_k__5, sizeof(__pyx_k__5), 0, 0, 1, 1},
  {&__pyx_kp_s__6, __pyx_k__6, sizeof(__pyx_k__6), 0, 0, 1, 0},
  {&__pyx_kp_s__9, __pyx_k__9, sizeof(__pyx_k__9), 0, 0, 1, 0},
  {&__pyx_kp_s_a_zA_Z0_9, __pyx_k_a_zA_Z0_9, sizeof(__pyx_k_a_zA_Z0_9), 0, 0, 1, 0},
  {&__pyx_n_s_all, __pyx_k_all, sizeof(__pyx_k_all), 0, 0, 1, 1},
  {&__pyx_n_s_allow_blank, __pyx_k_allow_blank, sizeof(__pyx_k_allow_blank), 0, 0, 1, 1},
  {&__pyx_n_s_allow_empty, __pyx_k_allow_empty, sizeof(__pyx_k_allow_empty), 0, 0, 1, 1},
  {&__pyx_n_s_allow_empty_file, __pyx_k_allow_empty_file, sizeof(__pyx_k_allow_empty_file), 0, 0, 1, 1},
  {&__pyx_n_s_allow_null, __pyx_k_allow_null, sizeof(__pyx_k_allow_null), 0, 0, 1, 1},
  {&__pyx_n_s_allow_unicode, __pyx_k_allow_unicode, sizeof(__pyx_k_allow_unicode), 0, 0, 1, 1},
  {&__pyx_n_s_api_settings, __pyx_k_api_settings, sizeof(__pyx_k_api_settings), 0, 0, 1, 1},
  {&__pyx_n_s_as_tuple, __pyx_k_as_tuple, sizeof(__pyx_k_as_tuple), 0, 0, 1, 1},
  {&__pyx_n_s_astimezone, __pyx_k_astimezone, sizeof(__pyx_k_astimezone), 0, 0, 1, 1},
  {&__pyx_n_s_attr, __pyx_k_attr, sizeof(__pyx_k_attr), 0, 0, 1, 1},
  {&__pyx_n_s_binary, __pyx_k_binary, sizeof(__pyx_k_binary), 0, 0, 1, 1},
  {&__pyx_n_s_bind, __pyx_k_bind, sizeof(__pyx_k_bind), 0, 0, 1, 1},
  {&__pyx_n_s_blank, __pyx_k_blank, sizeof(__pyx_k_blank), 0, 0, 1, 1},
  {&__pyx_n_s_build_absolute_uri, __pyx_k_build_absolute_uri, sizeof(__pyx_k_build_absolute_uri), 0, 0, 1, 1},
  {&__pyx_n_s_bytes, __pyx_k_bytes, sizeof(__pyx_k_bytes), 0, 0, 1, 1},
  {&__pyx_n_s_call, __pyx_k_call, sizeof(__pyx_k_call), 0, 0, 1, 1},
  {&__pyx_n_s_capitalize, __pyx_k_capitalize, sizeof(__pyx_k_capitalize), 0, 0, 1, 1},
  {&__pyx_n_s_child, __pyx_k_child, sizeof(__pyx_k_child), 0, 0, 1, 1},
  {&__pyx_n_s_child_relation, __pyx_k_child_relation, sizeof(__pyx_k_child_relation), 0, 0, 1, 1},
  {&__pyx_n_s_choices, __pyx_k_choices, sizeof(__pyx_k_choices), 0, 0, 1, 1},
  {&__pyx_n_s_class, __pyx_k_class, sizeof(__pyx_k_class), 0, 0, 1, 1},
  {&__pyx_n_s_cline_in_traceback, __pyx_k_cline_in_traceback, sizeof(__pyx_k_cline_in_traceback), 0, 0, 1, 1},
  {&__pyx_n_s_cls, __pyx_k_cls, sizeof(__pyx_k_cls), 0, 0, 1, 1},
  {&__pyx_n_s_coerce_null_values, __pyx_k_coerce_null_values, sizeof(__pyx_k_coerce_null_values), 0, 0, 1, 1},
  {&__pyx_n_s_coerce_to_string, __pyx_k_coerce_to_string, sizeof(__pyx_k_coerce_to_string), 0, 0, 1, 1},
  {&__pyx_n_s_coerce_values, __pyx_k_coerce_values, sizeof(__pyx_k_coerce_values), 0, 0, 1, 1},
  {&__pyx_n_s_compile, __pyx_k_compile, sizeof(__pyx_k_compile), 0, 0, 1, 1},
  {&__pyx_n_s_constant, __pyx_k_constant, sizeof(__pyx_k_constant), 0, 0, 1, 1},
  {&__pyx_n_s_context, __pyx_k_context, sizeof(__pyx_k_context), 0, 0, 1, 1},
  {&__pyx_n_s_copy, __pyx_k_copy, sizeof(__pyx_k_copy), 0, 0, 1, 1},
  {&__pyx_n_s_data, __pyx_k_data, sizeof(__pyx_k_data), 0, 0, 1, 1},
  {&__pyx_n_s_data_type, __pyx_k_data_type, sizeof(__pyx_k_data_type), 0, 0, 1, 1},
  {&__pyx_n_s_date, __pyx_k_date, sizeof(__pyx_k_date), 0, 0, 1, 1},
  {&__pyx_n_s_datetime, __pyx_k_datetime, sizeof(__pyx_k_datetime), 0, 0, 1, 1},
  {&__pyx_n_s_datetime_parser, __pyx_k_datetime_parser, sizeof(__pyx_k_datetime_parser), 0, 0, 1, 1},
  {&__pyx_n_s_decimal, __pyx_k_decimal, sizeof(__pyx_k_decimal), 0, 0, 1, 1},
  {&__pyx_n_s_decimal_places, __pyx_k_decimal_places, sizeof(__pyx_k_decimal_places), 0, 0, 1, 1},
  {&__pyx_n_s_decode, __pyx_k_decode, sizeof(__pyx_k_decode), 0, 0, 1, 1},
  {&__pyx_n_s_decoder, __pyx_k_decoder, sizeof(__pyx_k_decoder), 0, 0, 1, 1},
  {&__pyx_n_s_deepcopy, __pyx_k_deepcopy, sizeof(__pyx_k_deepcopy), 0, 0, 1, 1},
  {&__pyx_n_s_default_error_messages, __pyx_k_default_error_messages, sizeof(__pyx_k_default_error_messages), 0, 0, 1, 1},
  {&__pyx_n_s_default_timezone, __pyx_k_default_timezone, sizeof(__pyx_k_default_timezone), 0, 0, 1, 1},
  {&__pyx_n_s_default_value, __pyx_k_default_value, sizeof(__pyx_k_default_value), 0, 0, 1, 1},
  {&__pyx_n_s_deserialize, __pyx_k_deserialize, sizeof(__pyx_k_deserialize), 0, 0, 1, 1},
  {&__pyx_n_s_detail, __pyx_k_detail, sizeof(__pyx_k_detail), 0, 0, 1, 1},
  {&__pyx_n_s_dict, __pyx_k_dict, sizeof(__pyx_k_dict), 0, 0, 1, 1},
  {&__pyx_n_s_display, __pyx_k_display, sizeof(__pyx_k_display), 0, 0, 1, 1},
  {&__pyx_n_s_django_conf, __pyx_k_django_conf, sizeof(__pyx_k_django_conf), 0, 0, 1, 1},
  {&__pyx_n_s_django_core_exceptions, __pyx_k_django_core_exceptions, sizeof(__pyx_k_django_core_exceptions), 0, 0, 1, 1},
  {&__pyx_n_s_django_utils, __pyx_k_django_utils, sizeof(__pyx_k_django_utils), 0, 0, 1, 1},
  {&__pyx_n_s_django_utils_dateparse, __pyx_k_django_utils_dateparse, sizeof(__pyx_k_django_utils_dateparse), 0, 0, 1, 1},
  {&__pyx_n_s_does_not_exist, __pyx_k_does_not_exist, sizeof(__pyx_k_does_not_exist), 0, 0, 1, 1},
  {&__pyx_n_s_drf_turbo_exceptions, __pyx_k_drf_turbo_exceptions, sizeof(__pyx_k_drf_turbo_exceptions), 0, 0, 1, 1},
  {&__pyx_n_s_drf_turbo_fields, __pyx_k_drf_turbo_fields, sizeof(__pyx_k_drf_turbo_fields), 0, 0, 1, 1},
  {&__pyx_n_s_drf_turbo_utils, __pyx_k_drf_turbo_utils, sizeof(__pyx_k_drf_turbo_utils), 0, 0, 1, 1},
  {&__pyx_n_s_dumps, __pyx_k_dumps, sizeof(__pyx_k_dumps), 0, 0, 1, 1},
  {&__pyx_n_s_empty, __pyx_k_empty, sizeof(__pyx_k_empty), 0, 0, 1, 1},
  {&__pyx_n_s_encode, __pyx_k_encode, sizeof(__pyx_k_encode), 0, 0, 1, 1},
  {&__pyx_n_s_encoder, __pyx_k_encoder, sizeof(__pyx_k_encoder), 0, 0, 1, 1},
  {&__pyx_n_s_endswith, __pyx_k_endswith, sizeof(__pyx_k_endswith), 0, 0, 1, 1},
  {&__pyx_n_s_error_messages, __pyx_k_error_messages, sizeof(__pyx_k_error_messages), 0, 0, 1, 1},
  {&__pyx_n_s_exact_items, __pyx_k_exact_items, sizeof(__pyx_k_exact_items), 0, 0, 1, 1},
  {&__pyx_n_s_exc_type, __pyx_k_exc_type, sizeof(__pyx_k_exc_type), 0, 0, 1, 1},
  {&__pyx_n_s_exclude, __pyx_k_exclude, sizeof(__pyx_k_exclude), 0, 0, 1, 1},
  {&__pyx_kp_s_exclude_should_be_a_list_of_str, __pyx_k_exclude_should_be_a_list_of_str, sizeof(__pyx_k_exclude_should_be_a_list_of_str), 0, 0, 1, 0},
  {&__pyx_kp_s_f, __pyx_k_f, sizeof(__pyx_k_f), 0, 0, 1, 0},
  {&__pyx_n_s_f_2, __pyx_k_f_2, sizeof(__pyx_k_f_2), 0, 0, 1, 1},
  {&__pyx_n_s_false, __pyx_k_false, sizeof(__pyx_k_false), 0, 0, 1, 1},
  {&__pyx_n_s_field_name, __pyx_k_field_name, sizeof(__pyx_k_field_name), 0, 0, 1, 1},
  {&__pyx_n_s_flags, __pyx_k_flags, sizeof(__pyx_k_flags), 0, 0, 1, 1},
  {&__pyx_n_s_force_str, __pyx_k_force_str, sizeof(__pyx_k_force_str), 0, 0, 1, 1},
  {&__pyx_n_s_format, __pyx_k_format, sizeof(__pyx_k_format), 0, 0, 1, 1},
  {&__pyx_n_s_get, __pyx_k_get, sizeof(__pyx_k_get), 0, 0, 1, 1},
  {&__pyx_kp_s_get__0, __pyx_k_get__0, sizeof(__pyx_k_get__0), 0, 0, 1, 0},
  {&__pyx_n_s_get_attribute, __pyx_k_get_attribute, sizeof(__pyx_k_get_attribute), 0, 0, 1, 1},
  {&__pyx_n_s_get_current_timezone, __pyx_k_get_current_timezone, sizeof(__pyx_k_get_current_timezone), 0, 0, 1, 1},
  {&__pyx_n_s_get_default_value, __pyx_k_get_default_value, sizeof(__pyx_k_get_default_value), 0, 0, 1, 1},
  {&__pyx_n_s_get_error_detail, __pyx_k_get_error_detail, sizeof(__pyx_k_get_error_detail), 0, 0, 1, 1},
  {&__pyx_n_s_get_initial, __pyx_k_get_initial, sizeof(__pyx_k_get_initial), 0, 0, 1, 1},
  {&__pyx_n_s_getcontext, __pyx_k_getcontext, sizeof(__pyx_k_getcontext), 0, 0, 1, 1},
  {&__pyx_n_s_getstate, __pyx_k_getstate, sizeof(__pyx_k_getstate), 0, 0, 1, 1},
  {&__pyx_n_s_help_text, __pyx_k_help_text, sizeof(__pyx_k_help_text), 0, 0, 1, 1},
  {&__pyx_n_s_hex, __pyx_k_hex, sizeof(__pyx_k_hex), 0, 0, 1, 1},
  {&__pyx_kp_s_http_ftp_s_A_Z0_9_A_Z0_9_0_61_A, __pyx_k_http_ftp_s_A_Z0_9_A_Z0_9_0_61_A, sizeof(__pyx_k_http_ftp_s_A_Z0_9_A_Z0_9_0_61_A), 0, 0, 1, 0},
  {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
  {&__pyx_n_s_incorrect_type, __pyx_k_incorrect_type, sizeof(__pyx_k_incorrect_type), 0, 0, 1, 1},
  {&__pyx_n_s_init, __pyx_k_init, sizeof(__pyx_k_init), 0, 0, 1, 1},
  {&__pyx_n_s_initial, __pyx_k_initial, sizeof(__pyx_k_initial), 0, 0, 1, 1},
  {&__pyx_n_s_initial_2, __pyx_k_initial_2, sizeof(__pyx_k_initial_2), 0, 0, 1, 1},
  {&__pyx_n_s_input, __pyx_k_input, sizeof(__pyx_k_input), 0, 0, 1, 1},
  {&__pyx_n_s_input_formats, __pyx_k_input_formats, sizeof(__pyx_k_input_formats), 0, 0, 1, 1},
  {&__pyx_kp_s_input_is_not_a_valid_choice, __pyx_k_input_is_not_a_valid_choice, sizeof(__pyx_k_input_is_not_a_valid_choice), 0, 0, 1, 0},
  {&__pyx_n_s_input_type, __pyx_k_input_type, sizeof(__pyx_k_input_type), 0, 0, 1, 1},
  {&__pyx_n_s_instance, __pyx_k_instance, sizeof(__pyx_k_instance), 0, 0, 1, 1},
  {&__pyx_n_s_int, __pyx_k_int, sizeof(__pyx_k_int), 0, 0, 1, 1},
  {&__pyx_n_s_invalid, __pyx_k_invalid, sizeof(__pyx_k_invalid), 0, 0, 1, 1},
  {&__pyx_n_s_invalid_choice, __pyx_k_invalid_choice, sizeof(__pyx_k_invalid_choice), 0, 0, 1, 1},
  {&__pyx_n_s_invalid_unicode, __pyx_k_invalid_unicode, sizeof(__pyx_k_invalid_unicode), 0, 0, 1, 1},
  {&__pyx_n_s_ip_address, __pyx_k_ip_address, sizeof(__pyx_k_ip_address), 0, 0, 1, 1},
  {&__pyx_n_s_ipaddress, __pyx_k_ipaddress, sizeof(__pyx_k_ipaddress), 0, 0, 1, 1},
  {&__pyx_n_s_is_aware, __pyx_k_is_aware, sizeof(__pyx_k_is_aware), 0, 0, 1, 1},
  {&__pyx_n_s_is_collection, __pyx_k_is_collection, sizeof(__pyx_k_is_collection), 0, 0, 1, 1},
  {&__pyx_n_s_is_iterable_and_not_string, __pyx_k_is_iterable_and_not_string, sizeof(__pyx_k_is_iterable_and_not_string), 0, 0, 1, 1},
  {&__pyx_n_s_is_method_field, __pyx_k_is_method_field, sizeof(__pyx_k_is_method_field), 0, 0, 1, 1},
  {&__pyx_n_s_is_nan, __pyx_k_is_nan, sizeof(__pyx_k_is_nan), 0, 0, 1, 1},
  {&__pyx_n_s_isoformat, __pyx_k_isoformat, sizeof(__pyx_k_isoformat), 0, 0, 1, 1},
  {&__pyx_n_s_items, __pyx_k_items, sizeof(__pyx_k_items), 0, 0, 1, 1},
  {&__pyx_n_s_iter, __pyx_k_iter, sizeof(__pyx_k_iter), 0, 0, 1, 1},
  {&__pyx_n_s_json, __pyx_k_json, sizeof(__pyx_k_json), 0, 0, 1, 1},
  {&__pyx_n_s_key, __pyx_k_key, sizeof(__pyx_k_key), 0, 0, 1, 1},
  {&__pyx_n_s_keys, __pyx_k_keys, sizeof(__pyx_k_keys), 0, 0, 1, 1},
  {&__pyx_n_s_label, __pyx_k_label, sizeof(__pyx_k_label), 0, 0, 1, 1},
  {&__pyx_n_s_length, __pyx_k_length, sizeof(__pyx_k_length), 0, 0, 1, 1},
  {&__pyx_n_s_loads, __pyx_k_loads, sizeof(__pyx_k_loads), 0, 0, 1, 1},
  {&__pyx_n_s_lower, __pyx_k_lower, sizeof(__pyx_k_lower), 0, 0, 1, 1},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_n_s_make_aware, __pyx_k_make_aware, sizeof(__pyx_k_make_aware), 0, 0, 1, 1},
  {&__pyx_n_s_make_naive, __pyx_k_make_naive, sizeof(__pyx_k_make_naive), 0, 0, 1, 1},
  {&__pyx_n_s_many, __pyx_k_many, sizeof(__pyx_k_many), 0, 0, 1, 1},
  {&__pyx_n_s_match, __pyx_k_match, sizeof(__pyx_k_match), 0, 0, 1, 1},
  {&__pyx_n_s_max_decimal_places, __pyx_k_max_decimal_places, sizeof(__pyx_k_max_decimal_places), 0, 0, 1, 1},
  {&__pyx_n_s_max_digits, __pyx_k_max_digits, sizeof(__pyx_k_max_digits), 0, 0, 1, 1},
  {&__pyx_n_s_max_items, __pyx_k_max_items, sizeof(__pyx_k_max_items), 0, 0, 1, 1},
  {&__pyx_n_s_max_length, __pyx_k_max_length, sizeof(__pyx_k_max_length), 0, 0, 1, 1},
  {&__pyx_n_s_max_string_length, __pyx_k_max_string_length, sizeof(__pyx_k_max_string_length), 0, 0, 1, 1},
  {&__pyx_n_s_max_value, __pyx_k_max_value, sizeof(__pyx_k_max_value), 0, 0, 1, 1},
  {&__pyx_n_s_max_whole_digits, __pyx_k_max_whole_digits, sizeof(__pyx_k_max_whole_digits), 0, 0, 1, 1},
  {&__pyx_n_s_method_getter, __pyx_k_method_getter, sizeof(__pyx_k_method_getter), 0, 0, 1, 1},
  {&__pyx_n_s_method_name, __pyx_k_method_name, sizeof(__pyx_k_method_name), 0, 0, 1, 1},
  {&__pyx_n_s_min_items, __pyx_k_min_items, sizeof(__pyx_k_min_items), 0, 0, 1, 1},
  {&__pyx_n_s_min_length, __pyx_k_min_length, sizeof(__pyx_k_min_length), 0, 0, 1, 1},
  {&__pyx_n_s_min_value, __pyx_k_min_value, sizeof(__pyx_k_min_value), 0, 0, 1, 1},
  {&__pyx_n_s_mro, __pyx_k_mro, sizeof(__pyx_k_mro), 0, 0, 1, 1},
  {&__pyx_n_s_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
  {&__pyx_n_s_name_2, __pyx_k_name_2, sizeof(__pyx_k_name_2), 0, 0, 1, 1},
  {&__pyx_n_s_new, __pyx_k_new, sizeof(__pyx_k_new), 0, 0, 1, 1},
  {&__pyx_n_s_no_name, __pyx_k_no_name, sizeof(__pyx_k_no_name), 0, 0, 1, 1},
  {&__pyx_n_s_none, __pyx_k_none, sizeof(__pyx_k_none), 0, 0, 1, 1},
  {&__pyx_n_s_not_a_dict, __pyx_k_not_a_dict, sizeof(__pyx_k_not_a_dict), 0, 0, 1, 1},
  {&__pyx_n_s_not_a_list, __pyx_k_not_a_list, sizeof(__pyx_k_not_a_list), 0, 0, 1, 1},
  {&__pyx_n_s_null, __pyx_k_null, sizeof(__pyx_k_null), 0, 0, 1, 1},
  {&__pyx_n_s_object, __pyx_k_object, sizeof(__pyx_k_object), 0, 0, 1, 1},
  {&__pyx_n_s_off, __pyx_k_off, sizeof(__pyx_k_off), 0, 0, 1, 1},
  {&__pyx_n_s_on, __pyx_k_on, sizeof(__pyx_k_on), 0, 0, 1, 1},
  {&__pyx_n_s_only, __pyx_k_only, sizeof(__pyx_k_only), 0, 0, 1, 1},
  {&__pyx_kp_s_only_should_be_a_list_of_string, __pyx_k_only_should_be_a_list_of_string, sizeof(__pyx_k_only_should_be_a_list_of_string), 0, 0, 1, 0},
  {&__pyx_n_s_overflow, __pyx_k_overflow, sizeof(__pyx_k_overflow), 0, 0, 1, 1},
  {&__pyx_n_s_parse_date, __pyx_k_parse_date, sizeof(__pyx_k_parse_date), 0, 0, 1, 1},
  {&__pyx_n_s_parse_datetime, __pyx_k_parse_datetime, sizeof(__pyx_k_parse_datetime), 0, 0, 1, 1},
  {&__pyx_n_s_parse_time, __pyx_k_parse_time, sizeof(__pyx_k_parse_time), 0, 0, 1, 1},
  {&__pyx_n_s_partial, __pyx_k_partial, sizeof(__pyx_k_partial), 0, 0, 1, 1},
  {&__pyx_n_s_pickle, __pyx_k_pickle, sizeof(__pyx_k_pickle), 0, 0, 1, 1},
  {&__pyx_n_s_pk, __pyx_k_pk, sizeof(__pyx_k_pk), 0, 0, 1, 1},
  {&__pyx_n_s_pk_value, __pyx_k_pk_value, sizeof(__pyx_k_pk_value), 0, 0, 1, 1},
  {&__pyx_n_s_pop, __pyx_k_pop, sizeof(__pyx_k_pop), 0, 0, 1, 1},
  {&__pyx_n_s_prec, __pyx_k_prec, sizeof(__pyx_k_prec), 0, 0, 1, 1},
  {&__pyx_n_s_pytz_exceptions, __pyx_k_pytz_exceptions, sizeof(__pyx_k_pytz_exceptions), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_PickleError, __pyx_k_pyx_PickleError, sizeof(__pyx_k_pyx_PickleError), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_capi, __pyx_k_pyx_capi, sizeof(__pyx_k_pyx_capi), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_checksum, __pyx_k_pyx_checksum, sizeof(__pyx_k_pyx_checksum), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_result, __pyx_k_pyx_result, sizeof(__pyx_k_pyx_result), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_state, __pyx_k_pyx_state, sizeof(__pyx_k_pyx_state), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_type, __pyx_k_pyx_type, sizeof(__pyx_k_pyx_type), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_ArrayField, __pyx_k_pyx_unpickle_ArrayField, sizeof(__pyx_k_pyx_unpickle_ArrayField), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_BoolField, __pyx_k_pyx_unpickle_BoolField, sizeof(__pyx_k_pyx_unpickle_BoolField), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_ChoiceField, __pyx_k_pyx_unpickle_ChoiceField, sizeof(__pyx_k_pyx_unpickle_ChoiceField), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_ConstantField, __pyx_k_pyx_unpickle_ConstantField, sizeof(__pyx_k_pyx_unpickle_ConstantField), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_DateField, __pyx_k_pyx_unpickle_DateField, sizeof(__pyx_k_pyx_unpickle_DateField), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_DateTimeField, __pyx_k_pyx_unpickle_DateTimeField, sizeof(__pyx_k_pyx_unpickle_DateTimeField), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_DecimalField, __pyx_k_pyx_unpickle_DecimalField, sizeof(__pyx_k_pyx_unpickle_DecimalField), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_DictField, __pyx_k_pyx_unpickle_DictField, sizeof(__pyx_k_pyx_unpickle_DictField), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_EmailField, __pyx_k_pyx_unpickle_EmailField, sizeof(__pyx_k_pyx_unpickle_EmailField), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_Field, __pyx_k_pyx_unpickle_Field, sizeof(__pyx_k_pyx_unpickle_Field), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_FileField, __pyx_k_pyx_unpickle_FileField, sizeof(__pyx_k_pyx_unpickle_FileField), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_FloatField, __pyx_k_pyx_unpickle_FloatField, sizeof(__pyx_k_pyx_unpickle_FloatField), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_IPField, __pyx_k_pyx_unpickle_IPField, sizeof(__pyx_k_pyx_unpickle_IPField), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_IntField, __pyx_k_pyx_unpickle_IntField, sizeof(__pyx_k_pyx_unpickle_IntField), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_JSONField, __pyx_k_pyx_unpickle_JSONField, sizeof(__pyx_k_pyx_unpickle_JSONField), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_ManyRelatedField, __pyx_k_pyx_unpickle_ManyRelatedField, sizeof(__pyx_k_pyx_unpickle_ManyRelatedField), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_MethodField, __pyx_k_pyx_unpickle_MethodField, sizeof(__pyx_k_pyx_unpickle_MethodField), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_MultipleChoiceFie, __pyx_k_pyx_unpickle_MultipleChoiceFie, sizeof(__pyx_k_pyx_unpickle_MultipleChoiceFie), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_PasswordField, __pyx_k_pyx_unpickle_PasswordField, sizeof(__pyx_k_pyx_unpickle_PasswordField), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_RecursiveField, __pyx_k_pyx_unpickle_RecursiveField, sizeof(__pyx_k_pyx_unpickle_RecursiveField), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_RegexField, __pyx_k_pyx_unpickle_RegexField, sizeof(__pyx_k_pyx_unpickle_RegexField), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_RelatedField, __pyx_k_pyx_unpickle_RelatedField, sizeof(__pyx_k_pyx_unpickle_RelatedField), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_SkipField, __pyx_k_pyx_unpickle_SkipField, sizeof(__pyx_k_pyx_unpickle_SkipField), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_SlugField, __pyx_k_pyx_unpickle_SlugField, sizeof(__pyx_k_pyx_unpickle_SlugField), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_StrField, __pyx_k_pyx_unpickle_StrField, sizeof(__pyx_k_pyx_unpickle_StrField), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_TimeField, __pyx_k_pyx_unpickle_TimeField, sizeof(__pyx_k_pyx_unpickle_TimeField), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_URLField, __pyx_k_pyx_unpickle_URLField, sizeof(__pyx_k_pyx_unpickle_URLField), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_UUIDField, __pyx_k_pyx_unpickle_UUIDField, sizeof(__pyx_k_pyx_unpickle_UUIDField), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_vtable, __pyx_k_pyx_vtable, sizeof(__pyx_k_pyx_vtable), 0, 0, 1, 1},
  {&__pyx_n_s_quantize, __pyx_k_quantize, sizeof(__pyx_k_quantize), 0, 0, 1, 1},
  {&__pyx_n_s_queryset, __pyx_k_queryset, sizeof(__pyx_k_queryset), 0, 0, 1, 1},
  {&__pyx_n_s_raise_if_fail, __pyx_k_raise_if_fail, sizeof(__pyx_k_raise_if_fail), 0, 0, 1, 1},
  {&__pyx_n_s_re, __pyx_k_re, sizeof(__pyx_k_re), 0, 0, 1, 1},
  {&__pyx_n_s_re_decimal, __pyx_k_re_decimal, sizeof(__pyx_k_re_decimal), 0, 0, 1, 1},
  {&__pyx_n_s_read_only, __pyx_k_read_only, sizeof(__pyx_k_read_only), 0, 0, 1, 1},
  {&__pyx_n_s_reduce, __pyx_k_reduce, sizeof(__pyx_k_reduce), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_cython, __pyx_k_reduce_cython, sizeof(__pyx_k_reduce_cython), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_ex, __pyx_k_reduce_ex, sizeof(__pyx_k_reduce_ex), 0, 0, 1, 1},
  {&__pyx_n_s_regex, __pyx_k_regex, sizeof(__pyx_k_regex), 0, 0, 1, 1},
  {&__pyx_n_s_replace, __pyx_k_replace, sizeof(__pyx_k_replace), 0, 0, 1, 1},
  {&__pyx_n_s_request, __pyx_k_request, sizeof(__pyx_k_request), 0, 0, 1, 1},
  {&__pyx_n_s_required, __pyx_k_required, sizeof(__pyx_k_required), 0, 0, 1, 1},
  {&__pyx_n_s_rest_framework, __pyx_k_rest_framework, sizeof(__pyx_k_rest_framework), 0, 0, 1, 1},
  {&__pyx_n_s_rest_framework_settings, __pyx_k_rest_framework_settings, sizeof(__pyx_k_rest_framework_settings), 0, 0, 1, 1},
  {&__pyx_n_s_reversed, __pyx_k_reversed, sizeof(__pyx_k_reversed), 0, 0, 1, 1},
  {&__pyx_n_s_root, __pyx_k_root, sizeof(__pyx_k_root), 0, 0, 1, 1},
  {&__pyx_n_s_rounding, __pyx_k_rounding, sizeof(__pyx_k_rounding), 0, 0, 1, 1},
  {&__pyx_n_s_run_validation, __pyx_k_run_validation, sizeof(__pyx_k_run_validation), 0, 0, 1, 1},
  {&__pyx_n_s_search, __pyx_k_search, sizeof(__pyx_k_search), 0, 0, 1, 1},
  {&__pyx_n_s_serialize, __pyx_k_serialize, sizeof(__pyx_k_serialize), 0, 0, 1, 1},
  {&__pyx_n_s_setstate, __pyx_k_setstate, sizeof(__pyx_k_setstate), 0, 0, 1, 1},
  {&__pyx_n_s_setstate_cython, __pyx_k_setstate_cython, sizeof(__pyx_k_setstate_cython), 0, 0, 1, 1},
  {&__pyx_n_s_settings, __pyx_k_settings, sizeof(__pyx_k_settings), 0, 0, 1, 1},
  {&__pyx_n_s_size, __pyx_k_size, sizeof(__pyx_k_size), 0, 0, 1, 1},
  {&__pyx_n_s_split, __pyx_k_split, sizeof(__pyx_k_split), 0, 0, 1, 1},
  {&__pyx_n_s_startswith, __pyx_k_startswith, sizeof(__pyx_k_startswith), 0, 0, 1, 1},
  {&__pyx_n_s_strftime, __pyx_k_strftime, sizeof(__pyx_k_strftime), 0, 0, 1, 1},
  {&__pyx_kp_s_stringsource, __pyx_k_stringsource, sizeof(__pyx_k_stringsource), 0, 0, 1, 0},
  {&__pyx_n_s_strip, __pyx_k_strip, sizeof(__pyx_k_strip), 0, 0, 1, 1},
  {&__pyx_n_s_strptime, __pyx_k_strptime, sizeof(__pyx_k_strptime), 0, 0, 1, 1},
  {&__pyx_n_s_style, __pyx_k_style, sizeof(__pyx_k_style), 0, 0, 1, 1},
  {&__pyx_n_s_sub, __pyx_k_sub, sizeof(__pyx_k_sub), 0, 0, 1, 1},
  {&__pyx_n_s_super, __pyx_k_super, sizeof(__pyx_k_super), 0, 0, 1, 1},
  {&__pyx_n_s_t, __pyx_k_t, sizeof(__pyx_k_t), 0, 0, 1, 1},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_n_s_time, __pyx_k_time, sizeof(__pyx_k_time), 0, 0, 1, 1},
  {&__pyx_n_s_timezone, __pyx_k_timezone, sizeof(__pyx_k_timezone), 0, 0, 1, 1},
  {&__pyx_n_s_to_lower, __pyx_k_to_lower, sizeof(__pyx_k_to_lower), 0, 0, 1, 1},
  {&__pyx_n_s_trim_whitespace, __pyx_k_trim_whitespace, sizeof(__pyx_k_trim_whitespace), 0, 0, 1, 1},
  {&__pyx_n_s_true, __pyx_k_true, sizeof(__pyx_k_true), 0, 0, 1, 1},
  {&__pyx_n_s_update, __pyx_k_update, sizeof(__pyx_k_update), 0, 0, 1, 1},
  {&__pyx_n_s_url, __pyx_k_url, sizeof(__pyx_k_url), 0, 0, 1, 1},
  {&__pyx_n_s_utc, __pyx_k_utc, sizeof(__pyx_k_utc), 0, 0, 1, 1},
  {&__pyx_n_s_uuid, __pyx_k_uuid, sizeof(__pyx_k_uuid), 0, 0, 1, 1},
  {&__pyx_n_s_validate_empty_values, __pyx_k_validate_empty_values, sizeof(__pyx_k_validate_empty_values), 0, 0, 1, 1},
  {&__pyx_n_s_validate_or_raise, __pyx_k_validate_or_raise, sizeof(__pyx_k_validate_or_raise), 0, 0, 1, 1},
  {&__pyx_n_s_validators, __pyx_k_validators, sizeof(__pyx_k_validators), 0, 0, 1, 1},
  {&__pyx_n_s_value, __pyx_k_value, sizeof(__pyx_k_value), 0, 0, 1, 1},
  {&__pyx_n_s_vars, __pyx_k_vars, sizeof(__pyx_k_vars), 0, 0, 1, 1},
  {&__pyx_kp_s_w_Z, __pyx_k_w_Z, sizeof(__pyx_k_w_Z), 0, 0, 1, 0},
  {&__pyx_n_s_write_only, __pyx_k_write_only, sizeof(__pyx_k_write_only), 0, 0, 1, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static CYTHON_SMALL_CODE int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_object = __Pyx_GetBuiltinName(__pyx_n_s_object); if (!__pyx_builtin_object) __PYX_ERR(0, 35, __pyx_L1_error)
  __pyx_builtin_ValueError = __Pyx_GetBuiltinName(__pyx_n_s_ValueError); if (!__pyx_builtin_ValueError) __PYX_ERR(0, 118, __pyx_L1_error)
  __pyx_builtin_reversed = __Pyx_GetBuiltinName(__pyx_n_s_reversed); if (!__pyx_builtin_reversed) __PYX_ERR(0, 124, __pyx_L1_error)
  __pyx_builtin_KeyError = __Pyx_GetBuiltinName(__pyx_n_s_KeyError); if (!__pyx_builtin_KeyError) __PYX_ERR(0, 135, __pyx_L1_error)
  __pyx_builtin_AssertionError = __Pyx_GetBuiltinName(__pyx_n_s_AssertionError); if (!__pyx_builtin_AssertionError) __PYX_ERR(0, 136, __pyx_L1_error)
  __pyx_builtin_AttributeError = __Pyx_GetBuiltinName(__pyx_n_s_AttributeError); if (!__pyx_builtin_AttributeError) __PYX_ERR(0, 216, __pyx_L1_error)
  __pyx_builtin_super = __Pyx_GetBuiltinName(__pyx_n_s_super); if (!__pyx_builtin_super) __PYX_ERR(0, 312, __pyx_L1_error)
  __pyx_builtin_TypeError = __Pyx_GetBuiltinName(__pyx_n_s_TypeError); if (!__pyx_builtin_TypeError) __PYX_ERR(0, 427, __pyx_L1_error)
  __pyx_builtin_vars = __Pyx_GetBuiltinName(__pyx_n_s_vars); if (!__pyx_builtin_vars) __PYX_ERR(0, 621, __pyx_L1_error)
  __pyx_builtin_all = __Pyx_GetBuiltinName(__pyx_n_s_all); if (!__pyx_builtin_all) __PYX_ERR(0, 798, __pyx_L1_error)
  __pyx_builtin_OverflowError = __Pyx_GetBuiltinName(__pyx_n_s_OverflowError); if (!__pyx_builtin_OverflowError) __PYX_ERR(0, 927, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "drf_turbo/fields.pyx":118
 *             self.validators = list(validators)
 *         else:
 *             raise ValueError(             # <<<<<<<<<<<<<<
 *                 "The 'validators' parameter must be a callable "
 *                 "or a collection of callables."
 */
  __pyx_tuple__4 = PyTuple_Pack(1, __pyx_kp_s_The_validators_parameter_must_be); if (unlikely(!__pyx_tuple__4)) __PYX_ERR(0, 118, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__4);
  __Pyx_GIVEREF(__pyx_tuple__4);

  /* "drf_turbo/fields.pyx":182
 *         self.root = root
 *         if self.label is None:
 *             self.label = field_name.replace('_', ' ').capitalize()             # <<<<<<<<<<<<<<
 * 
 *         if self.attr is None:
 */
  __pyx_tuple__7 = PyTuple_Pack(2, __pyx_n_s__5, __pyx_kp_s__6); if (unlikely(!__pyx_tuple__7)) __PYX_ERR(0, 182, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__7);
  __Pyx_GIVEREF(__pyx_tuple__7);

  /* "drf_turbo/fields.pyx":252
 *             if not self.allow_null:
 *                 raise self.raise_if_fail('null')
 *             return (True, None)             # <<<<<<<<<<<<<<
 * 
 *         return (False, data)
 */
  __pyx_tuple__8 = PyTuple_Pack(2, Py_True, Py_None); if (unlikely(!__pyx_tuple__8)) __PYX_ERR(0, 252, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__8);
  __Pyx_GIVEREF(__pyx_tuple__8);

  /* "drf_turbo/fields.pyx":326
 *         if self.min_length is not None:
 *             if len(data) < self.min_length:
 *                 raise self.raise_if_fail("min_length",min_length=self.min_length)             # <<<<<<<<<<<<<<
 *         if self.max_length is not None:
 *             if len(data) > self.max_length:
 */
  __pyx_tuple__10 = PyTuple_Pack(1, __pyx_n_s_min_length); if (unlikely(!__pyx_tuple__10)) __PYX_ERR(0, 326, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__10);
  __Pyx_GIVEREF(__pyx_tuple__10);

  /* "drf_turbo/fields.pyx":329
 *         if self.max_length is not None:
 *             if len(data) > self.max_length:
 *                 raise self.raise_if_fail("max_length",max_length=self.max_length)             # <<<<<<<<<<<<<<
 *         if '\x00' in str(data):
 *             raise self.raise_if_fail('null')
 */
  __pyx_tuple__11 = PyTuple_Pack(1, __pyx_n_s_max_length); if (unlikely(!__pyx_tuple__11)) __PYX_ERR(0, 329, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__11);
  __Pyx_GIVEREF(__pyx_tuple__11);

  /* "drf_turbo/fields.pyx":534
 *             raise self.raise_if_fail('invalid')
 *         if self.min_value is not None and data < self.min_value:
 *             raise self.raise_if_fail("min_value",min_value=self.min_value)             # <<<<<<<<<<<<<<
 *         if self.max_value is not None and data > self.max_value:
 *             raise self.raise_if_fail("max_value",max_value=self.max_value)
 */
  __pyx_tuple__13 = PyTuple_Pack(1, __pyx_n_s_min_value); if (unlikely(!__pyx_tuple__13)) __PYX_ERR(0, 534, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__13);
  __Pyx_GIVEREF(__pyx_tuple__13);

  /* "drf_turbo/fields.pyx":536
 *             raise self.raise_if_fail("min_value",min_value=self.min_value)
 *         if self.max_value is not None and data > self.max_value:
 *             raise self.raise_if_fail("max_value",max_value=self.max_value)             # <<<<<<<<<<<<<<
 * 
 *         return data
 */
  __pyx_tuple__14 = PyTuple_Pack(1, __pyx_n_s_max_value); if (unlikely(!__pyx_tuple__14)) __PYX_ERR(0, 536, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__14);
  __Pyx_GIVEREF(__pyx_tuple__14);

  /* "drf_turbo/fields.pyx":704
 * 
 *         if self.max_digits is not None and total_digits > self.max_digits:
 *            raise self.raise_if_fail('max_digits', max_digits=self.max_digits)             # <<<<<<<<<<<<<<
 *         if self.decimal_places is not None and decimal_places > self.decimal_places:
 *             raise self.raise_if_fail('max_decimal_places', max_decimal_places=self.decimal_places)
 */
  __pyx_tuple__15 = PyTuple_Pack(1, __pyx_n_s_max_digits); if (unlikely(!__pyx_tuple__15)) __PYX_ERR(0, 704, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__15);
  __Pyx_GIVEREF(__pyx_tuple__15);

  /* "drf_turbo/fields.pyx":706
 *            raise self.raise_if_fail('max_digits', max_digits=self.max_digits)
 *         if self.decimal_places is not None and decimal_places > self.decimal_places:
 *             raise self.raise_if_fail('max_decimal_places', max_decimal_places=self.decimal_places)             # <<<<<<<<<<<<<<
 *         if self.max_whole_digits is not None and whole_digits > self.max_whole_digits:
 *             raise self.raise_if_fail('max_whole_digits', max_whole_digits=self.max_whole_digits)
 */
  __pyx_tuple__16 = PyTuple_Pack(1, __pyx_n_s_max_decimal_places); if (unlikely(!__pyx_tuple__16)) __PYX_ERR(0, 706, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__16);
  __Pyx_GIVEREF(__pyx_tuple__16);

  /* "drf_turbo/fields.pyx":708
 *             raise self.raise_if_fail('max_decimal_places', max_decimal_places=self.decimal_places)
 *         if self.max_whole_digits is not None and whole_digits > self.max_whole_digits:
 *             raise self.raise_if_fail('max_whole_digits', max_whole_digits=self.max_whole_digits)             # <<<<<<<<<<<<<<
 * 
 *         return value
 */
  __pyx_tuple__17 = PyTuple_Pack(1, __pyx_n_s_max_whole_digits); if (unlikely(!__pyx_tuple__17)) __PYX_ERR(0, 708, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__17);
  __Pyx_GIVEREF(__pyx_tuple__17);

  /* "drf_turbo/fields.pyx":833
 *             return self.choice_strings_to_values[str(data)]
 *         except:
 *             raise self.raise_if_fail('invalid_choice', input=data)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_tuple__18 = PyTuple_Pack(1, __pyx_n_s_invalid_choice); if (unlikely(!__pyx_tuple__18)) __PYX_ERR(0, 833, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__18);
  __Pyx_GIVEREF(__pyx_tuple__18);

  /* "drf_turbo/fields.pyx":862
 *     cpdef inline deserialize(self,data,dict context) :
 *         if isinstance(data, str) or not hasattr(data, '__iter__'):
 *             raise self.raise_if_fail('not_a_list', input_type=type(data).__name__)             # <<<<<<<<<<<<<<
 *         if not self.allow_empty and len(data) == 0:
 *             raise self.raise_if_fail('empty')
 */
  __pyx_tuple__19 = PyTuple_Pack(1, __pyx_n_s_not_a_list); if (unlikely(!__pyx_tuple__19)) __PYX_ERR(0, 862, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__19);
  __Pyx_GIVEREF(__pyx_tuple__19);

  /* "drf_turbo/fields.pyx":932
 *                 return timezone.make_aware(value, self.timezone)
 *             except InvalidTimeError:
 *                 raise self.raise_if_fail('make_aware', timezone=self.timezone)             # <<<<<<<<<<<<<<
 *         elif (self.timezone is None) and timezone.is_aware(value):
 *             return timezone.make_naive(value, timezone.utc)
 */
  __pyx_tuple__21 = PyTuple_Pack(1, __pyx_n_s_make_aware); if (unlikely(!__pyx_tuple__21)) __PYX_ERR(0, 932, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__21);
  __Pyx_GIVEREF(__pyx_tuple__21);

  /* "drf_turbo/fields.pyx":950
 *             value = value.isoformat()
 *             if value.endswith('+00:00'):
 *                 value = value[:-6] + 'Z'             # <<<<<<<<<<<<<<
 *             return value
 *         return value.strftime(self.format)
 */
  __pyx_slice__22 = PySlice_New(Py_None, __pyx_int_neg_6, Py_None); if (unlikely(!__pyx_slice__22)) __PYX_ERR(0, 950, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_slice__22);
  __Pyx_GIVEREF(__pyx_slice__22);

  /* "drf_turbo/fields.pyx":1235
 *             and len(data) != self.min_items
 *         ):
 *             raise self.raise_if_fail("exact_items",exact_items=self.exact_items)             # <<<<<<<<<<<<<<
 *         if self.min_items is not None and len(data) < self.min_items:
 *             raise self.raise_if_fail("min_items",min_items=self.min_items)
 */
  __pyx_tuple__24 = PyTuple_Pack(1, __pyx_n_s_exact_items); if (unlikely(!__pyx_tuple__24)) __PYX_ERR(0, 1235, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__24);
  __Pyx_GIVEREF(__pyx_tuple__24);

  /* "drf_turbo/fields.pyx":1237
 *             raise self.raise_if_fail("exact_items",exact_items=self.exact_items)
 *         if self.min_items is not None and len(data) < self.min_items:
 *             raise self.raise_if_fail("min_items",min_items=self.min_items)             # <<<<<<<<<<<<<<
 *         elif self.max_items is not None and len(data) > self.max_items:
 *             raise self.raise_if_fail("max_items",max_items=self.max_items)
 */
  __pyx_tuple__25 = PyTuple_Pack(1, __pyx_n_s_min_items); if (unlikely(!__pyx_tuple__25)) __PYX_ERR(0, 1237, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__25);
  __Pyx_GIVEREF(__pyx_tuple__25);

  /* "drf_turbo/fields.pyx":1239
 *             raise self.raise_if_fail("min_items",min_items=self.min_items)
 *         elif self.max_items is not None and len(data) > self.max_items:
 *             raise self.raise_if_fail("max_items",max_items=self.max_items)             # <<<<<<<<<<<<<<
 * 
 *         return self.run_child_validation(data,context)
 */
  __pyx_tuple__26 = PyTuple_Pack(1, __pyx_n_s_max_items); if (unlikely(!__pyx_tuple__26)) __PYX_ERR(0, 1239, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__26);
  __Pyx_GIVEREF(__pyx_tuple__26);

  /* "drf_turbo/fields.pyx":1292
 *         """
 *         if not isinstance(data,dict):
 *             raise self.raise_if_fail('not_a_dict', input_type=type(data).__name__)             # <<<<<<<<<<<<<<
 *         if not self.allow_empty and len(data) == 0:
 *             raise self.raise_if_fail('empty')
 */
  __pyx_tuple__27 = PyTuple_Pack(1, __pyx_n_s_not_a_dict); if (unlikely(!__pyx_tuple__27)) __PYX_ERR(0, 1292, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__27);
  __Pyx_GIVEREF(__pyx_tuple__27);

  /* "drf_turbo/fields.pyx":1370
 *             data = self.queryset.get(pk=data)
 *         except ObjectDoesNotExist:
 *             raise self.raise_if_fail('does_not_exist', pk_value=data)             # <<<<<<<<<<<<<<
 *         except (TypeError, ValueError):
 *             raise self.raise_if_fail('incorrect_type', data_type=type(data).__name__)
 */
  __pyx_tuple__28 = PyTuple_Pack(1, __pyx_n_s_does_not_exist); if (unlikely(!__pyx_tuple__28)) __PYX_ERR(0, 1370, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__28);
  __Pyx_GIVEREF(__pyx_tuple__28);

  /* "drf_turbo/fields.pyx":1372
 *             raise self.raise_if_fail('does_not_exist', pk_value=data)
 *         except (TypeError, ValueError):
 *             raise self.raise_if_fail('incorrect_type', data_type=type(data).__name__)             # <<<<<<<<<<<<<<
 *         return data
 * 
 */
  __pyx_tuple__29 = PyTuple_Pack(1, __pyx_n_s_incorrect_type); if (unlikely(!__pyx_tuple__29)) __PYX_ERR(0, 1372, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__29);
  __Pyx_GIVEREF(__pyx_tuple__29);

  /* "drf_turbo/fields.pyx":1433
 *             if self.constant is None:
 *                 raise self.raise_if_fail("None")
 *             raise self.raise_if_fail("constant",constant=self.constant)             # <<<<<<<<<<<<<<
 *         return data
 * 
 */
  __pyx_tuple__30 = PyTuple_Pack(1, __pyx_n_s_constant); if (unlikely(!__pyx_tuple__30)) __PYX_ERR(0, 1433, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__30);
  __Pyx_GIVEREF(__pyx_tuple__30);

  /* "(tree fragment)":1
 * def __pyx_unpickle_SkipField(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_tuple__32 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__32)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__32);
  __Pyx_GIVEREF(__pyx_tuple__32);
  __pyx_codeobj__33 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__32, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_SkipField, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__33)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_tuple__34 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__34)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__34);
  __Pyx_GIVEREF(__pyx_tuple__34);
  __pyx_codeobj__35 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__34, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_Field, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__35)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_tuple__36 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__36)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__36);
  __Pyx_GIVEREF(__pyx_tuple__36);
  __pyx_codeobj__37 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__36, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_StrField, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__37)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_tuple__38 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__38)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__38);
  __Pyx_GIVEREF(__pyx_tuple__38);
  __pyx_codeobj__39 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__38, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_EmailField, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__39)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_tuple__40 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__40)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__40);
  __Pyx_GIVEREF(__pyx_tuple__40);
  __pyx_codeobj__41 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__40, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_URLField, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__41)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_tuple__42 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__42)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__42);
  __Pyx_GIVEREF(__pyx_tuple__42);
  __pyx_codeobj__43 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__42, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_RegexField, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__43)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_tuple__44 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__44)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__44);
  __Pyx_GIVEREF(__pyx_tuple__44);
  __pyx_codeobj__45 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__44, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_IPField, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__45)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_tuple__46 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__46)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__46);
  __Pyx_GIVEREF(__pyx_tuple__46);
  __pyx_codeobj__47 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__46, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_PasswordField, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__47)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_tuple__48 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__48)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__48);
  __Pyx_GIVEREF(__pyx_tuple__48);
  __pyx_codeobj__49 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__48, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_UUIDField, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__49)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_tuple__50 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__50)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__50);
  __Pyx_GIVEREF(__pyx_tuple__50);
  __pyx_codeobj__51 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__50, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_SlugField, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__51)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_tuple__52 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__52)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__52);
  __Pyx_GIVEREF(__pyx_tuple__52);
  __pyx_codeobj__53 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__52, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_IntField, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__53)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_tuple__54 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__54)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__54);
  __Pyx_GIVEREF(__pyx_tuple__54);
  __pyx_codeobj__55 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__54, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_FloatField, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__55)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_tuple__56 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__56)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__56);
  __Pyx_GIVEREF(__pyx_tuple__56);
  __pyx_codeobj__57 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__56, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_DecimalField, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__57)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_tuple__58 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__58)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__58);
  __Pyx_GIVEREF(__pyx_tuple__58);
  __pyx_codeobj__59 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__58, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_BoolField, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__59)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_tuple__60 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__60)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__60);
  __Pyx_GIVEREF(__pyx_tuple__60);
  __pyx_codeobj__61 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__60, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_ChoiceField, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__61)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_tuple__62 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__62)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__62);
  __Pyx_GIVEREF(__pyx_tuple__62);
  __pyx_codeobj__63 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__62, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_MultipleChoiceFie, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__63)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_tuple__64 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__64)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__64);
  __Pyx_GIVEREF(__pyx_tuple__64);
  __pyx_codeobj__65 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__64, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_DateTimeField, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__65)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_tuple__66 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__66)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__66);
  __Pyx_GIVEREF(__pyx_tuple__66);
  __pyx_codeobj__67 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__66, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_DateField, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__67)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_tuple__68 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__68)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__68);
  __Pyx_GIVEREF(__pyx_tuple__68);
  __pyx_codeobj__69 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__68, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_TimeField, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__69)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_tuple__70 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__70)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__70);
  __Pyx_GIVEREF(__pyx_tuple__70);
  __pyx_codeobj__71 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__70, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_FileField, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__71)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_tuple__72 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__72)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__72);
  __Pyx_GIVEREF(__pyx_tuple__72);
  __pyx_codeobj__73 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__72, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_ArrayField, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__73)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_tuple__74 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__74)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__74);
  __Pyx_GIVEREF(__pyx_tuple__74);
  __pyx_codeobj__75 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__74, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_DictField, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__75)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_tuple__76 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__76)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__76);
  __Pyx_GIVEREF(__pyx_tuple__76);
  __pyx_codeobj__77 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__76, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_JSONField, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__77)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_tuple__78 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__78)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__78);
  __Pyx_GIVEREF(__pyx_tuple__78);
  __pyx_codeobj__79 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__78, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_RelatedField, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__79)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_tuple__80 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__80)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__80);
  __Pyx_GIVEREF(__pyx_tuple__80);
  __pyx_codeobj__81 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__80, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_ManyRelatedField, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__81)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_tuple__82 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__82)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__82);
  __Pyx_GIVEREF(__pyx_tuple__82);
  __pyx_codeobj__83 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__82, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_ConstantField, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__83)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_tuple__84 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__84)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__84);
  __Pyx_GIVEREF(__pyx_tuple__84);
  __pyx_codeobj__85 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__84, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_RecursiveField, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__85)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_tuple__86 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__86)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__86);
  __Pyx_GIVEREF(__pyx_tuple__86);
  __pyx_codeobj__87 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__86, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_MethodField, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__87)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_InitGlobals(void) {
  __pyx_umethod_PyDict_Type_get.type = (PyObject*)&PyDict_Type;
  __pyx_umethod_PyDict_Type_items.type = (PyObject*)&PyDict_Type;
  __pyx_umethod_PyDict_Type_pop.type = (PyObject*)&PyDict_Type;
  __pyx_umethod_PyDict_Type_update.type = (PyObject*)&PyDict_Type;
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  __pyx_int_0 = PyInt_FromLong(0); if (unlikely(!__pyx_int_0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_1 = PyInt_FromLong(1); if (unlikely(!__pyx_int_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_4 = PyInt_FromLong(4); if (unlikely(!__pyx_int_4)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_1000 = PyInt_FromLong(1000); if (unlikely(!__pyx_int_1000)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_3394979 = PyInt_FromLong(3394979L); if (unlikely(!__pyx_int_3394979)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_13265606 = PyInt_FromLong(13265606L); if (unlikely(!__pyx_int_13265606)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_16510453 = PyInt_FromLong(16510453L); if (unlikely(!__pyx_int_16510453)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_57277993 = PyInt_FromLong(57277993L); if (unlikely(!__pyx_int_57277993)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_58778772 = PyInt_FromLong(58778772L); if (unlikely(!__pyx_int_58778772)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_83299623 = PyInt_FromLong(83299623L); if (unlikely(!__pyx_int_83299623)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_91705362 = PyInt_FromLong(91705362L); if (unlikely(!__pyx_int_91705362)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_99052414 = PyInt_FromLong(99052414L); if (unlikely(!__pyx_int_99052414)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_107357925 = PyInt_FromLong(107357925L); if (unlikely(!__pyx_int_107357925)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_111135515 = PyInt_FromLong(111135515L); if (unlikely(!__pyx_int_111135515)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_119569969 = PyInt_FromLong(119569969L); if (unlikely(!__pyx_int_119569969)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_152991563 = PyInt_FromLong(152991563L); if (unlikely(!__pyx_int_152991563)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_208687877 = PyInt_FromLong(208687877L); if (unlikely(!__pyx_int_208687877)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_222419149 = PyInt_FromLong(222419149L); if (unlikely(!__pyx_int_222419149)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_229562288 = PyInt_FromLong(229562288L); if (unlikely(!__pyx_int_229562288)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_236850905 = PyInt_FromLong(236850905L); if (unlikely(!__pyx_int_236850905)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_243425435 = PyInt_FromLong(243425435L); if (unlikely(!__pyx_int_243425435)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_253175912 = PyInt_FromLong(253175912L); if (unlikely(!__pyx_int_253175912)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_255240407 = PyInt_FromLong(255240407L); if (unlikely(!__pyx_int_255240407)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_256313077 = PyInt_FromLong(256313077L); if (unlikely(!__pyx_int_256313077)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_259151664 = PyInt_FromLong(259151664L); if (unlikely(!__pyx_int_259151664)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_neg_6 = PyInt_FromLong(-6); if (unlikely(!__pyx_int_neg_6)) __PYX_ERR(0, 1, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_modinit_global_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_import_code(void); /*proto*/

static int __Pyx_modinit_global_init_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_global_init_code", 0);
  /*--- Global init code ---*/
  __pyx_v_9drf_turbo_6fields_NO_DEFAULT = Py_None; Py_INCREF(Py_None);
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_variable_export_code(void) {
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_export_code", 0);
  /*--- Variable export code ---*/
  if (__Pyx_ExportVoidPtr(__pyx_n_s_NO_DEFAULT, (void *)&__pyx_v_9drf_turbo_6fields_NO_DEFAULT, "PyObject *") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_function_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_export_code", 0);
  /*--- Function export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_type_init_code(void) {
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_init_code", 0);
  /*--- Type init code ---*/
  __pyx_type_9drf_turbo_6fields_SkipField.tp_base = (&((PyTypeObject*)PyExc_Exception)[0]);
  if (PyType_Ready(&__pyx_type_9drf_turbo_6fields_SkipField) < 0) __PYX_ERR(0, 38, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_9drf_turbo_6fields_SkipField.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_9drf_turbo_6fields_SkipField.tp_dictoffset && __pyx_type_9drf_turbo_6fields_SkipField.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_9drf_turbo_6fields_SkipField.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_SkipField, (PyObject *)&__pyx_type_9drf_turbo_6fields_SkipField) < 0) __PYX_ERR(0, 38, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_9drf_turbo_6fields_SkipField) < 0) __PYX_ERR(0, 38, __pyx_L1_error)
  __pyx_ptype_9drf_turbo_6fields_SkipField = &__pyx_type_9drf_turbo_6fields_SkipField;
  __pyx_vtabptr_9drf_turbo_6fields_Field = &__pyx_vtable_9drf_turbo_6fields_Field;
  __pyx_vtable_9drf_turbo_6fields_Field.serialize = (PyObject *(*)(struct __pyx_obj_9drf_turbo_6fields_Field *, PyObject *, PyObject *, int __pyx_skip_dispatch))__pyx_f_9drf_turbo_6fields_5Field_serialize;
  __pyx_vtable_9drf_turbo_6fields_Field.deserialize = (PyObject *(*)(struct __pyx_obj_9drf_turbo_6fields_Field *, PyObject *, PyObject *, int __pyx_skip_dispatch))__pyx_f_9drf_turbo_6fields_5Field_deserialize;
  __pyx_vtable_9drf_turbo_6fields_Field.method_getter = (PyObject *(*)(struct __pyx_obj_9drf_turbo_6fields_Field *, PyObject *, PyObject *, int __pyx_skip_dispatch))__pyx_f_9drf_turbo_6fields_5Field_method_getter;
  __pyx_vtable_9drf_turbo_6fields_Field.bind = (void (*)(struct __pyx_obj_9drf_turbo_6fields_Field *, PyObject *, PyObject *, int __pyx_skip_dispatch))__pyx_f_9drf_turbo_6fields_5Field_bind;
  __pyx_vtable_9drf_turbo_6fields_Field.run_validation = (PyObject *(*)(struct __pyx_obj_9drf_turbo_6fields_Field *, PyObject *, PyObject *, int __pyx_skip_dispatch))__pyx_f_9drf_turbo_6fields_5Field_run_validation;
  __pyx_vtable_9drf_turbo_6fields_Field.get_initial = (PyObject *(*)(struct __pyx_obj_9drf_turbo_6fields_Field *, int __pyx_skip_dispatch))__pyx_f_9drf_turbo_6fields_5Field_get_initial;
  __pyx_vtable_9drf_turbo_6fields_Field.get_attribute = (PyObject *(*)(struct __pyx_obj_9drf_turbo_6fields_Field *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_9drf_turbo_6fields_5Field_get_attribute *__pyx_optional_args))__pyx_f_9drf_turbo_6fields_5Field_get_attribute;
  __pyx_vtable_9drf_turbo_6fields_Field.get_default_value = (PyObject *(*)(struct __pyx_obj_9drf_turbo_6fields_Field *, int __pyx_skip_dispatch))__pyx_f_9drf_turbo_6fields_5Field_get_default_value;
  __pyx_vtable_9drf_turbo_6fields_Field.validate_empty_values = (PyObject *(*)(struct __pyx_obj_9drf_turbo_6fields_Field *, PyObject *, int __pyx_skip_dispatch))__pyx_f_9drf_turbo_6fields_5Field_validate_empty_values;
  __pyx_vtable_9drf_turbo_6fields_Field.validate_or_raise = (long (*)(struct __pyx_obj_9drf_turbo_6fields_Field *, PyObject *, int __pyx_skip_dispatch))__pyx_f_9drf_turbo_6fields_5Field_validate_or_raise;
  if (PyType_Ready(&__pyx_type_9drf_turbo_6fields_Field) < 0) __PYX_ERR(0, 42, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_9drf_turbo_6fields_Field.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_9drf_turbo_6fields_Field.tp_dictoffset && __pyx_type_9drf_turbo_6fields_Field.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_9drf_turbo_6fields_Field.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_9drf_turbo_6fields_Field.tp_dict, __pyx_vtabptr_9drf_turbo_6fields_Field) < 0) __PYX_ERR(0, 42, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_Field, (PyObject *)&__pyx_type_9drf_turbo_6fields_Field) < 0) __PYX_ERR(0, 42, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_9drf_turbo_6fields_Field) < 0) __PYX_ERR(0, 42, __pyx_L1_error)
  __pyx_ptype_9drf_turbo_6fields_Field = &__pyx_type_9drf_turbo_6fields_Field;
  __pyx_vtabptr_9drf_turbo_6fields_StrField = &__pyx_vtable_9drf_turbo_6fields_StrField;
  __pyx_vtable_9drf_turbo_6fields_StrField.__pyx_base = *__pyx_vtabptr_9drf_turbo_6fields_Field;
  __pyx_vtable_9drf_turbo_6fields_StrField.__pyx_base.serialize = (PyObject *(*)(struct __pyx_obj_9drf_turbo_6fields_Field *, PyObject *, PyObject *, int __pyx_skip_dispatch))__pyx_f_9drf_turbo_6fields_8StrField_serialize;
  __pyx_vtable_9drf_turbo_6fields_StrField.__pyx_base.deserialize = (PyObject *(*)(struct __pyx_obj_9drf_turbo_6fields_Field *, PyObject *, PyObject *, int __pyx_skip_dispatch))__pyx_f_9drf_turbo_6fields_8StrField_deserialize;
  __pyx_type_9drf_turbo_6fields_StrField.tp_base = __pyx_ptype_9drf_turbo_6fields_Field;
  if (PyType_Ready(&__pyx_type_9drf_turbo_6fields_StrField) < 0) __PYX_ERR(0, 291, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_9drf_turbo_6fields_StrField.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_9drf_turbo_6fields_StrField.tp_dictoffset && __pyx_type_9drf_turbo_6fields_StrField.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_9drf_turbo_6fields_StrField.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_9drf_turbo_6fields_StrField.tp_dict, __pyx_vtabptr_9drf_turbo_6fields_StrField) < 0) __PYX_ERR(0, 291, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_StrField, (PyObject *)&__pyx_type_9drf_turbo_6fields_StrField) < 0) __PYX_ERR(0, 291, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_9drf_turbo_6fields_StrField) < 0) __PYX_ERR(0, 291, __pyx_L1_error)
  __pyx_ptype_9drf_turbo_6fields_StrField = &__pyx_type_9drf_turbo_6fields_StrField;
  __pyx_vtabptr_9drf_turbo_6fields_EmailField = &__pyx_vtable_9drf_turbo_6fields_EmailField;
  __pyx_vtable_9drf_turbo_6fields_EmailField.__pyx_base = *__pyx_vtabptr_9drf_turbo_6fields_StrField;
  __pyx_vtable_9drf_turbo_6fields_EmailField.__pyx_base.__pyx_base.serialize = (PyObject *(*)(struct __pyx_obj_9drf_turbo_6fields_Field *, PyObject *, PyObject *, int __pyx_skip_dispatch))__pyx_f_9drf_turbo_6fields_10EmailField_serialize;
  __pyx_vtable_9drf_turbo_6fields_EmailField.__pyx_base.__pyx_base.deserialize = (PyObject *(*)(struct __pyx_obj_9drf_turbo_6fields_Field *, PyObject *, PyObject *, int __pyx_skip_dispatch))__pyx_f_9drf_turbo_6fields_10EmailField_deserialize;
  __pyx_type_9drf_turbo_6fields_EmailField.tp_base = __pyx_ptype_9drf_turbo_6fields_StrField;
  if (PyType_Ready(&__pyx_type_9drf_turbo_6fields_EmailField) < 0) __PYX_ERR(0, 337, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_9drf_turbo_6fields_EmailField.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_9drf_turbo_6fields_EmailField.tp_dictoffset && __pyx_type_9drf_turbo_6fields_EmailField.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_9drf_turbo_6fields_EmailField.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  if (__Pyx_SetVtable(__pyx_type_9drf_turbo_6fields_EmailField.tp_dict, __pyx_vtabptr_9drf_turbo_6fields_EmailField) < 0) __PYX_ERR(0, 337, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_EmailField, (PyObject *)&__pyx_type_9drf_turbo_6fields_EmailField) < 0) __PYX_ERR(0, 337, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_9drf_turbo_6fields_EmailField) < 0) __PYX_ERR(0, 337, __pyx_L1_error)
  __pyx_ptype_9drf_turbo_6fields_EmailField = &__pyx_type_9drf_turbo_6fields_EmailField;
  __pyx_vtabptr_9drf_turbo_6fields_URLField = &__pyx_vtable_9drf_turbo_6fields_URLField;
  __pyx_vtable_9drf_turbo_6fields_URLField.__pyx_base = *__pyx_vtabptr_9drf_turbo_6fields_StrField;
  __pyx_vtable_9drf_turbo_6fields_URLField.__pyx_base.__pyx_base.deserialize = (PyObject *(*)(struct __pyx_obj_9drf_turbo_6fields_Field *, PyObject *, PyObject *, int __pyx_skip_dispatch))__pyx_f_9drf_turbo_6fields_8URLField_deserialize;
  __pyx_type_9drf_turbo_6fields_URLField.tp_base = __pyx_ptype_9drf_turbo_6fields_StrField;
  if (PyType_Ready(&__pyx_type_9drf_turbo_6fields_URLField) < 0) __PYX_ERR(0, 367, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_9drf_turbo_6fields_URLField.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_9drf_turbo_6fields_URLField.tp_dictoffset && __pyx_type_9drf_turbo_6fields_URLField.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_9drf_turbo_6fields_URLField.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  if (__Pyx_SetVtable(__pyx_type_9drf_turbo_6fields_URLField.tp_dict, __pyx_vtabptr_9drf_turbo_6fields_URLField) < 0) __PYX_ERR(0, 367, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_URLField, (PyObject *)&__pyx_type_9drf_turbo_6fields_URLField) < 0) __PYX_ERR(0, 367, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_9drf_turbo_6fields_URLField) < 0) __PYX_ERR(0, 367, __pyx_L1_error)
  __pyx_ptype_9drf_turbo_6fields_URLField = &__pyx_type_9drf_turbo_6fields_URLField;
  __pyx_vtabptr_9drf_turbo_6fields_RegexField = &__pyx_vtable_9drf_turbo_6fields_RegexField;
  __pyx_vtable_9drf_turbo_6fields_RegexField.__pyx_base = *__pyx_vtabptr_9drf_turbo_6fields_StrField;
  __pyx_vtable_9drf_turbo_6fields_RegexField.__pyx_base.__pyx_base.deserialize = (PyObject *(*)(struct __pyx_obj_9drf_turbo_6fields_Field *, PyObject *, PyObject *, int __pyx_skip_dispatch))__pyx_f_9drf_turbo_6fields_10RegexField_deserialize;
  __pyx_type_9drf_turbo_6fields_RegexField.tp_base = __pyx_ptype_9drf_turbo_6fields_StrField;
  if (PyType_Ready(&__pyx_type_9drf_turbo_6fields_RegexField) < 0) __PYX_ERR(0, 388, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_9drf_turbo_6fields_RegexField.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_9drf_turbo_6fields_RegexField.tp_dictoffset && __pyx_type_9drf_turbo_6fields_RegexField.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_9drf_turbo_6fields_RegexField.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  if (__Pyx_SetVtable(__pyx_type_9drf_turbo_6fields_RegexField.tp_dict, __pyx_vtabptr_9drf_turbo_6fields_RegexField) < 0) __PYX_ERR(0, 388, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_RegexField, (PyObject *)&__pyx_type_9drf_turbo_6fields_RegexField) < 0) __PYX_ERR(0, 388, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_9drf_turbo_6fields_RegexField) < 0) __PYX_ERR(0, 388, __pyx_L1_error)
  __pyx_ptype_9drf_turbo_6fields_RegexField = &__pyx_type_9drf_turbo_6fields_RegexField;
  __pyx_vtabptr_9drf_turbo_6fields_IPField = &__pyx_vtable_9drf_turbo_6fields_IPField;
  __pyx_vtable_9drf_turbo_6fields_IPField.__pyx_base = *__pyx_vtabptr_9drf_turbo_6fields_StrField;
  __pyx_vtable_9drf_turbo_6fields_IPField.__pyx_base.__pyx_base.deserialize = (PyObject *(*)(struct __pyx_obj_9drf_turbo_6fields_Field *, PyObject *, PyObject *, int __pyx_skip_dispatch))__pyx_f_9drf_turbo_6fields_7IPField_deserialize;
  __pyx_type_9drf_turbo_6fields_IPField.tp_base = __pyx_ptype_9drf_turbo_6fields_StrField;
  if (PyType_Ready(&__pyx_type_9drf_turbo_6fields_IPField) < 0) __PYX_ERR(0, 413, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_9drf_turbo_6fields_IPField.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_9drf_turbo_6fields_IPField.tp_dictoffset && __pyx_type_9drf_turbo_6fields_IPField.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_9drf_turbo_6fields_IPField.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  if (__Pyx_SetVtable(__pyx_type_9drf_turbo_6fields_IPField.tp_dict, __pyx_vtabptr_9drf_turbo_6fields_IPField) < 0) __PYX_ERR(0, 413, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_IPField, (PyObject *)&__pyx_type_9drf_turbo_6fields_IPField) < 0) __PYX_ERR(0, 413, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_9drf_turbo_6fields_IPField) < 0) __PYX_ERR(0, 413, __pyx_L1_error)
  __pyx_ptype_9drf_turbo_6fields_IPField = &__pyx_type_9drf_turbo_6fields_IPField;
  __pyx_vtabptr_9drf_turbo_6fields_PasswordField = &__pyx_vtable_9drf_turbo_6fields_PasswordField;
  __pyx_vtable_9drf_turbo_6fields_PasswordField.__pyx_base = *__pyx_vtabptr_9drf_turbo_6fields_StrField;
  __pyx_type_9drf_turbo_6fields_PasswordField.tp_base = __pyx_ptype_9drf_turbo_6fields_StrField;
  if (PyType_Ready(&__pyx_type_9drf_turbo_6fields_PasswordField) < 0) __PYX_ERR(0, 431, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_9drf_turbo_6fields_PasswordField.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_9drf_turbo_6fields_PasswordField.tp_dictoffset && __pyx_type_9drf_turbo_6fields_PasswordField.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_9drf_turbo_6fields_PasswordField.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  if (__Pyx_SetVtable(__pyx_type_9drf_turbo_6fields_PasswordField.tp_dict, __pyx_vtabptr_9drf_turbo_6fields_PasswordField) < 0) __PYX_ERR(0, 431, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_PasswordField, (PyObject *)&__pyx_type_9drf_turbo_6fields_PasswordField) < 0) __PYX_ERR(0, 431, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_9drf_turbo_6fields_PasswordField) < 0) __PYX_ERR(0, 431, __pyx_L1_error)
  __pyx_ptype_9drf_turbo_6fields_PasswordField = &__pyx_type_9drf_turbo_6fields_PasswordField;
  __pyx_vtabptr_9drf_turbo_6fields_UUIDField = &__pyx_vtable_9drf_turbo_6fields_UUIDField;
  __pyx_vtable_9drf_turbo_6fields_UUIDField.__pyx_base = *__pyx_vtabptr_9drf_turbo_6fields_StrField;
  __pyx_vtable_9drf_turbo_6fields_UUIDField.__pyx_base.__pyx_base.deserialize = (PyObject *(*)(struct __pyx_obj_9drf_turbo_6fields_Field *, PyObject *, PyObject *, int __pyx_skip_dispatch))__pyx_f_9drf_turbo_6fields_9UUIDField_deserialize;
  __pyx_type_9drf_turbo_6fields_UUIDField.tp_base = __pyx_ptype_9drf_turbo_6fields_StrField;
  if (PyType_Ready(&__pyx_type_9drf_turbo_6fields_UUIDField) < 0) __PYX_ERR(0, 446, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_9drf_turbo_6fields_UUIDField.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_9drf_turbo_6fields_UUIDField.tp_dictoffset && __pyx_type_9drf_turbo_6fields_UUIDField.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_9drf_turbo_6fields_UUIDField.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  if (__Pyx_SetVtable(__pyx_type_9drf_turbo_6fields_UUIDField.tp_dict, __pyx_vtabptr_9drf_turbo_6fields_UUIDField) < 0) __PYX_ERR(0, 446, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_UUIDField, (PyObject *)&__pyx_type_9drf_turbo_6fields_UUIDField) < 0) __PYX_ERR(0, 446, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_9drf_turbo_6fields_UUIDField) < 0) __PYX_ERR(0, 446, __pyx_L1_error)
  __pyx_ptype_9drf_turbo_6fields_UUIDField = &__pyx_type_9drf_turbo_6fields_UUIDField;
  __pyx_vtabptr_9drf_turbo_6fields_SlugField = &__pyx_vtable_9drf_turbo_6fields_SlugField;
  __pyx_vtable_9drf_turbo_6fields_SlugField.__pyx_base = *__pyx_vtabptr_9drf_turbo_6fields_Field;
  __pyx_vtable_9drf_turbo_6fields_SlugField.__pyx_base.deserialize = (PyObject *(*)(struct __pyx_obj_9drf_turbo_6fields_Field *, PyObject *, PyObject *, int __pyx_skip_dispatch))__pyx_f_9drf_turbo_6fields_9SlugField_deserialize;
  __pyx_type_9drf_turbo_6fields_SlugField.tp_base = __pyx_ptype_9drf_turbo_6fields_Field;
  if (PyType_Ready(&__pyx_type_9drf_turbo_6fields_SlugField) < 0) __PYX_ERR(0, 476, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_9drf_turbo_6fields_SlugField.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_9drf_turbo_6fields_SlugField.tp_dictoffset && __pyx_type_9drf_turbo_6fields_SlugField.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_9drf_turbo_6fields_SlugField.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  if (__Pyx_SetVtable(__pyx_type_9drf_turbo_6fields_SlugField.tp_dict, __pyx_vtabptr_9drf_turbo_6fields_SlugField) < 0) __PYX_ERR(0, 476, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_SlugField, (PyObject *)&__pyx_type_9drf_turbo_6fields_SlugField) < 0) __PYX_ERR(0, 476, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_9drf_turbo_6fields_SlugField) < 0) __PYX_ERR(0, 476, __pyx_L1_error)
  __pyx_ptype_9drf_turbo_6fields_SlugField = &__pyx_type_9drf_turbo_6fields_SlugField;
  __pyx_vtabptr_9drf_turbo_6fields_IntField = &__pyx_vtable_9drf_turbo_6fields_IntField;
  __pyx_vtable_9drf_turbo_6fields_IntField.__pyx_base = *__pyx_vtabptr_9drf_turbo_6fields_Field;
  __pyx_vtable_9drf_turbo_6fields_IntField.__pyx_base.serialize = (PyObject *(*)(struct __pyx_obj_9drf_turbo_6fields_Field *, PyObject *, PyObject *, int __pyx_skip_dispatch))__pyx_f_9drf_turbo_6fields_8IntField_serialize;
  __pyx_vtable_9drf_turbo_6fields_IntField.__pyx_base.deserialize = (PyObject *(*)(struct __pyx_obj_9drf_turbo_6fields_Field *, PyObject *, PyObject *, int __pyx_skip_dispatch))__pyx_f_9drf_turbo_6fields_8IntField_deserialize;
  __pyx_type_9drf_turbo_6fields_IntField.tp_base = __pyx_ptype_9drf_turbo_6fields_Field;
  if (PyType_Ready(&__pyx_type_9drf_turbo_6fields_IntField) < 0) __PYX_ERR(0, 506, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_9drf_turbo_6fields_IntField.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_9drf_turbo_6fields_IntField.tp_dictoffset && __pyx_type_9drf_turbo_6fields_IntField.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_9drf_turbo_6fields_IntField.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  if (__Pyx_SetVtable(__pyx_type_9drf_turbo_6fields_IntField.tp_dict, __pyx_vtabptr_9drf_turbo_6fields_IntField) < 0) __PYX_ERR(0, 506, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_IntField, (PyObject *)&__pyx_type_9drf_turbo_6fields_IntField) < 0) __PYX_ERR(0, 506, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_9drf_turbo_6fields_IntField) < 0) __PYX_ERR(0, 506, __pyx_L1_error)
  __pyx_ptype_9drf_turbo_6fields_IntField = &__pyx_type_9drf_turbo_6fields_IntField;
  __pyx_vtabptr_9drf_turbo_6fields_FloatField = &__pyx_vtable_9drf_turbo_6fields_FloatField;
  __pyx_vtable_9drf_turbo_6fields_FloatField.__pyx_base = *__pyx_vtabptr_9drf_turbo_6fields_Field;
  __pyx_vtable_9drf_turbo_6fields_FloatField.__pyx_base.serialize = (PyObject *(*)(struct __pyx_obj_9drf_turbo_6fields_Field *, PyObject *, PyObject *, int __pyx_skip_dispatch))__pyx_f_9drf_turbo_6fields_10FloatField_serialize;
  __pyx_vtable_9drf_turbo_6fields_FloatField.__pyx_base.deserialize = (PyObject *(*)(struct __pyx_obj_9drf_turbo_6fields_Field *, PyObject *, PyObject *, int __pyx_skip_dispatch))__pyx_f_9drf_turbo_6fields_10FloatField_deserialize;
  __pyx_type_9drf_turbo_6fields_FloatField.tp_base = __pyx_ptype_9drf_turbo_6fields_Field;
  if (PyType_Ready(&__pyx_type_9drf_turbo_6fields_FloatField) < 0) __PYX_ERR(0, 541, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_9drf_turbo_6fields_FloatField.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_9drf_turbo_6fields_FloatField.tp_dictoffset && __pyx_type_9drf_turbo_6fields_FloatField.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_9drf_turbo_6fields_FloatField.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  if (__Pyx_SetVtable(__pyx_type_9drf_turbo_6fields_FloatField.tp_dict, __pyx_vtabptr_9drf_turbo_6fields_FloatField) < 0) __PYX_ERR(0, 541, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_FloatField, (PyObject *)&__pyx_type_9drf_turbo_6fields_FloatField) < 0) __PYX_ERR(0, 541, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_9drf_turbo_6fields_FloatField) < 0) __PYX_ERR(0, 541, __pyx_L1_error)
  __pyx_ptype_9drf_turbo_6fields_FloatField = &__pyx_type_9drf_turbo_6fields_FloatField;
  __pyx_vtabptr_9drf_turbo_6fields_DecimalField = &__pyx_vtable_9drf_turbo_6fields_DecimalField;
  __pyx_vtable_9drf_turbo_6fields_DecimalField.__pyx_base = *__pyx_vtabptr_9drf_turbo_6fields_Field;
  __pyx_vtable_9drf_turbo_6fields_DecimalField.__pyx_base.serialize = (PyObject *(*)(struct __pyx_obj_9drf_turbo_6fields_Field *, PyObject *, PyObject *, int __pyx_skip_dispatch))__pyx_f_9drf_turbo_6fields_12DecimalField_serialize;
  __pyx_vtable_9drf_turbo_6fields_DecimalField.__pyx_base.deserialize = (PyObject *(*)(struct __pyx_obj_9drf_turbo_6fields_Field *, PyObject *, PyObject *, int __pyx_skip_dispatch))__pyx_f_9drf_turbo_6fields_12DecimalField_deserialize;
  __pyx_vtable_9drf_turbo_6fields_DecimalField.validate_precision = (PyObject *(*)(struct __pyx_obj_9drf_turbo_6fields_DecimalField *, PyObject *))__pyx_f_9drf_turbo_6fields_12DecimalField_validate_precision;
  __pyx_vtable_9drf_turbo_6fields_DecimalField.quantize = (PyObject *(*)(struct __pyx_obj_9drf_turbo_6fields_DecimalField *, PyObject *))__pyx_f_9drf_turbo_6fields_12DecimalField_quantize;
  __pyx_type_9drf_turbo_6fields_DecimalField.tp_base = __pyx_ptype_9drf_turbo_6fields_Field;
  if (PyType_Ready(&__pyx_type_9drf_turbo_6fields_DecimalField) < 0) __PYX_ERR(0, 579, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_9drf_turbo_6fields_DecimalField.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_9drf_turbo_6fields_DecimalField.tp_dictoffset && __pyx_type_9drf_turbo_6fields_DecimalField.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_9drf_turbo_6fields_DecimalField.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  if (__Pyx_SetVtable(__pyx_type_9drf_turbo_6fields_DecimalField.tp_dict, __pyx_vtabptr_9drf_turbo_6fields_DecimalField) < 0) __PYX_ERR(0, 579, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_DecimalField, (PyObject *)&__pyx_type_9drf_turbo_6fields_DecimalField) < 0) __PYX_ERR(0, 579, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_9drf_turbo_6fields_DecimalField) < 0) __PYX_ERR(0, 579, __pyx_L1_error)
  __pyx_ptype_9drf_turbo_6fields_DecimalField = &__pyx_type_9drf_turbo_6fields_DecimalField;
  __pyx_vtabptr_9drf_turbo_6fields_BoolField = &__pyx_vtable_9drf_turbo_6fields_BoolField;
  __pyx_vtable_9drf_turbo_6fields_BoolField.__pyx_base = *__pyx_vtabptr_9drf_turbo_6fields_Field;
  __pyx_vtable_9drf_turbo_6fields_BoolField.__pyx_base.serialize = (PyObject *(*)(struct __pyx_obj_9drf_turbo_6fields_Field *, PyObject *, PyObject *, int __pyx_skip_dispatch))__pyx_f_9drf_turbo_6fields_9BoolField_serialize;
  __pyx_vtable_9drf_turbo_6fields_BoolField.__pyx_base.deserialize = (PyObject *(*)(struct __pyx_obj_9drf_turbo_6fields_Field *, PyObject *, PyObject *, int __pyx_skip_dispatch))__pyx_f_9drf_turbo_6fields_9BoolField_deserialize;
  __pyx_type_9drf_turbo_6fields_BoolField.tp_base = __pyx_ptype_9drf_turbo_6fields_Field;
  if (PyType_Ready(&__pyx_type_9drf_turbo_6fields_BoolField) < 0) __PYX_ERR(0, 730, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_9drf_turbo_6fields_BoolField.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_9drf_turbo_6fields_BoolField.tp_dictoffset && __pyx_type_9drf_turbo_6fields_BoolField.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_9drf_turbo_6fields_BoolField.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  if (__Pyx_SetVtable(__pyx_type_9drf_turbo_6fields_BoolField.tp_dict, __pyx_vtabptr_9drf_turbo_6fields_BoolField) < 0) __PYX_ERR(0, 730, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_BoolField, (PyObject *)&__pyx_type_9drf_turbo_6fields_BoolField) < 0) __PYX_ERR(0, 730, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_9drf_turbo_6fields_BoolField) < 0) __PYX_ERR(0, 730, __pyx_L1_error)
  __pyx_ptype_9drf_turbo_6fields_BoolField = &__pyx_type_9drf_turbo_6fields_BoolField;
  __pyx_vtabptr_9drf_turbo_6fields_ChoiceField = &__pyx_vtable_9drf_turbo_6fields_ChoiceField;
  __pyx_vtable_9drf_turbo_6fields_ChoiceField.__pyx_base = *__pyx_vtabptr_9drf_turbo_6fields_Field;
  __pyx_vtable_9drf_turbo_6fields_ChoiceField.__pyx_base.serialize = (PyObject *(*)(struct __pyx_obj_9drf_turbo_6fields_Field *, PyObject *, PyObject *, int __pyx_skip_dispatch))__pyx_f_9drf_turbo_6fields_11ChoiceField_serialize;
  __pyx_vtable_9drf_turbo_6fields_ChoiceField.__pyx_base.deserialize = (PyObject *(*)(struct __pyx_obj_9drf_turbo_6fields_Field *, PyObject *, PyObject *, int __pyx_skip_dispatch))__pyx_f_9drf_turbo_6fields_11ChoiceField_deserialize;
  __pyx_type_9drf_turbo_6fields_ChoiceField.tp_base = __pyx_ptype_9drf_turbo_6fields_Field;
  if (PyType_Ready(&__pyx_type_9drf_turbo_6fields_ChoiceField) < 0) __PYX_ERR(0, 781, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_9drf_turbo_6fields_ChoiceField.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_9drf_turbo_6fields_ChoiceField.tp_dictoffset && __pyx_type_9drf_turbo_6fields_ChoiceField.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_9drf_turbo_6fields_ChoiceField.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_9drf_turbo_6fields_ChoiceField.tp_dict, __pyx_vtabptr_9drf_turbo_6fields_ChoiceField) < 0) __PYX_ERR(0, 781, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_ChoiceField, (PyObject *)&__pyx_type_9drf_turbo_6fields_ChoiceField) < 0) __PYX_ERR(0, 781, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_9drf_turbo_6fields_ChoiceField) < 0) __PYX_ERR(0, 781, __pyx_L1_error)
  __pyx_ptype_9drf_turbo_6fields_ChoiceField = &__pyx_type_9drf_turbo_6fields_ChoiceField;
  __pyx_vtabptr_9drf_turbo_6fields_MultipleChoiceField = &__pyx_vtable_9drf_turbo_6fields_MultipleChoiceField;
  __pyx_vtable_9drf_turbo_6fields_MultipleChoiceField.__pyx_base = *__pyx_vtabptr_9drf_turbo_6fields_ChoiceField;
  __pyx_vtable_9drf_turbo_6fields_MultipleChoiceField.__pyx_base.__pyx_base.serialize = (PyObject *(*)(struct __pyx_obj_9drf_turbo_6fields_Field *, PyObject *, PyObject *, int __pyx_skip_dispatch))__pyx_f_9drf_turbo_6fields_19MultipleChoiceField_serialize;
  __pyx_vtable_9drf_turbo_6fields_MultipleChoiceField.__pyx_base.__pyx_base.deserialize = (PyObject *(*)(struct __pyx_obj_9drf_turbo_6fields_Field *, PyObject *, PyObject *, int __pyx_skip_dispatch))__pyx_f_9drf_turbo_6fields_19MultipleChoiceField_deserialize;
  __pyx_type_9drf_turbo_6fields_MultipleChoiceField.tp_base = __pyx_ptype_9drf_turbo_6fields_ChoiceField;
  if (PyType_Ready(&__pyx_type_9drf_turbo_6fields_MultipleChoiceField) < 0) __PYX_ERR(0, 837, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_9drf_turbo_6fields_MultipleChoiceField.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_9drf_turbo_6fields_MultipleChoiceField.tp_dictoffset && __pyx_type_9drf_turbo_6fields_MultipleChoiceField.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_9drf_turbo_6fields_MultipleChoiceField.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  if (__Pyx_SetVtable(__pyx_type_9drf_turbo_6fields_MultipleChoiceField.tp_dict, __pyx_vtabptr_9drf_turbo_6fields_MultipleChoiceField) < 0) __PYX_ERR(0, 837, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_MultipleChoiceField, (PyObject *)&__pyx_type_9drf_turbo_6fields_MultipleChoiceField) < 0) __PYX_ERR(0, 837, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_9drf_turbo_6fields_MultipleChoiceField) < 0) __PYX_ERR(0, 837, __pyx_L1_error)
  __pyx_ptype_9drf_turbo_6fields_MultipleChoiceField = &__pyx_type_9drf_turbo_6fields_MultipleChoiceField;
  __pyx_vtabptr_9drf_turbo_6fields_DateTimeField = &__pyx_vtable_9drf_turbo_6fields_DateTimeField;
  __pyx_vtable_9drf_turbo_6fields_DateTimeField.__pyx_base = *__pyx_vtabptr_9drf_turbo_6fields_Field;
  __pyx_vtable_9drf_turbo_6fields_DateTimeField.__pyx_base.serialize = (PyObject *(*)(struct __pyx_obj_9drf_turbo_6fields_Field *, PyObject *, PyObject *, int __pyx_skip_dispatch))__pyx_f_9drf_turbo_6fields_13DateTimeField_serialize;
  __pyx_vtable_9drf_turbo_6fields_DateTimeField.__pyx_base.deserialize = (PyObject *(*)(struct __pyx_obj_9drf_turbo_6fields_Field *, PyObject *, PyObject *, int __pyx_skip_dispatch))__pyx_f_9drf_turbo_6fields_13DateTimeField_deserialize;
  __pyx_vtable_9drf_turbo_6fields_DateTimeField.get_default_timezone = (PyObject *(*)(struct __pyx_obj_9drf_turbo_6fields_DateTimeField *, int __pyx_skip_dispatch))__pyx_f_9drf_turbo_6fields_13DateTimeField_get_default_timezone;
  __pyx_vtable_9drf_turbo_6fields_DateTimeField.enforce_timezone = (PyObject *(*)(struct __pyx_obj_9drf_turbo_6fields_DateTimeField *, PyObject *, int __pyx_skip_dispatch))__pyx_f_9drf_turbo_6fields_13DateTimeField_enforce_timezone;
  __pyx_type_9drf_turbo_6fields_DateTimeField.tp_base = __pyx_ptype_9drf_turbo_6fields_Field;
  if (PyType_Ready(&__pyx_type_9drf_turbo_6fields_DateTimeField) < 0) __PYX_ERR(0, 880, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_9drf_turbo_6fields_DateTimeField.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_9drf_turbo_6fields_DateTimeField.tp_dictoffset && __pyx_type_9drf_turbo_6fields_DateTimeField.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_9drf_turbo_6fields_DateTimeField.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  if (__Pyx_SetVtable(__pyx_type_9drf_turbo_6fields_DateTimeField.tp_dict, __pyx_vtabptr_9drf_turbo_6fields_DateTimeField) < 0) __PYX_ERR(0, 880, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_DateTimeField, (PyObject *)&__pyx_type_9drf_turbo_6fields_DateTimeField) < 0) __PYX_ERR(0, 880, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_9drf_turbo_6fields_DateTimeField) < 0) __PYX_ERR(0, 880, __pyx_L1_error)
  __pyx_ptype_9drf_turbo_6fields_DateTimeField = &__pyx_type_9drf_turbo_6fields_DateTimeField;
  __pyx_vtabptr_9drf_turbo_6fields_DateField = &__pyx_vtable_9drf_turbo_6fields_DateField;
  __pyx_vtable_9drf_turbo_6fields_DateField.__pyx_base = *__pyx_vtabptr_9drf_turbo_6fields_Field;
  __pyx_vtable_9drf_turbo_6fields_DateField.__pyx_base.serialize = (PyObject *(*)(struct __pyx_obj_9drf_turbo_6fields_Field *, PyObject *, PyObject *, int __pyx_skip_dispatch))__pyx_f_9drf_turbo_6fields_9DateField_serialize;
  __pyx_vtable_9drf_turbo_6fields_DateField.__pyx_base.deserialize = (PyObject *(*)(struct __pyx_obj_9drf_turbo_6fields_Field *, PyObject *, PyObject *, int __pyx_skip_dispatch))__pyx_f_9drf_turbo_6fields_9DateField_deserialize;
  __pyx_type_9drf_turbo_6fields_DateField.tp_base = __pyx_ptype_9drf_turbo_6fields_Field;
  if (PyType_Ready(&__pyx_type_9drf_turbo_6fields_DateField) < 0) __PYX_ERR(0, 982, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_9drf_turbo_6fields_DateField.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_9drf_turbo_6fields_DateField.tp_dictoffset && __pyx_type_9drf_turbo_6fields_DateField.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_9drf_turbo_6fields_DateField.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  if (__Pyx_SetVtable(__pyx_type_9drf_turbo_6fields_DateField.tp_dict, __pyx_vtabptr_9drf_turbo_6fields_DateField) < 0) __PYX_ERR(0, 982, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_DateField, (PyObject *)&__pyx_type_9drf_turbo_6fields_DateField) < 0) __PYX_ERR(0, 982, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_9drf_turbo_6fields_DateField) < 0) __PYX_ERR(0, 982, __pyx_L1_error)
  __pyx_ptype_9drf_turbo_6fields_DateField = &__pyx_type_9drf_turbo_6fields_DateField;
  __pyx_vtabptr_9drf_turbo_6fields_TimeField = &__pyx_vtable_9drf_turbo_6fields_TimeField;
  __pyx_vtable_9drf_turbo_6fields_TimeField.__pyx_base = *__pyx_vtabptr_9drf_turbo_6fields_Field;
  __pyx_vtable_9drf_turbo_6fields_TimeField.__pyx_base.serialize = (PyObject *(*)(struct __pyx_obj_9drf_turbo_6fields_Field *, PyObject *, PyObject *, int __pyx_skip_dispatch))__pyx_f_9drf_turbo_6fields_9TimeField_serialize;
  __pyx_vtable_9drf_turbo_6fields_TimeField.__pyx_base.deserialize = (PyObject *(*)(struct __pyx_obj_9drf_turbo_6fields_Field *, PyObject *, PyObject *, int __pyx_skip_dispatch))__pyx_f_9drf_turbo_6fields_9TimeField_deserialize;
  __pyx_type_9drf_turbo_6fields_TimeField.tp_base = __pyx_ptype_9drf_turbo_6fields_Field;
  if (PyType_Ready(&__pyx_type_9drf_turbo_6fields_TimeField) < 0) __PYX_ERR(0, 1058, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_9drf_turbo_6fields_TimeField.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_9drf_turbo_6fields_TimeField.tp_dictoffset && __pyx_type_9drf_turbo_6fields_TimeField.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_9drf_turbo_6fields_TimeField.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  if (__Pyx_SetVtable(__pyx_type_9drf_turbo_6fields_TimeField.tp_dict, __pyx_vtabptr_9drf_turbo_6fields_TimeField) < 0) __PYX_ERR(0, 1058, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_TimeField, (PyObject *)&__pyx_type_9drf_turbo_6fields_TimeField) < 0) __PYX_ERR(0, 1058, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_9drf_turbo_6fields_TimeField) < 0) __PYX_ERR(0, 1058, __pyx_L1_error)
  __pyx_ptype_9drf_turbo_6fields_TimeField = &__pyx_type_9drf_turbo_6fields_TimeField;
  __pyx_vtabptr_9drf_turbo_6fields_FileField = &__pyx_vtable_9drf_turbo_6fields_FileField;
  __pyx_vtable_9drf_turbo_6fields_FileField.__pyx_base = *__pyx_vtabptr_9drf_turbo_6fields_Field;
  __pyx_vtable_9drf_turbo_6fields_FileField.__pyx_base.serialize = (PyObject *(*)(struct __pyx_obj_9drf_turbo_6fields_Field *, PyObject *, PyObject *, int __pyx_skip_dispatch))__pyx_f_9drf_turbo_6fields_9FileField_serialize;
  __pyx_vtable_9drf_turbo_6fields_FileField.__pyx_base.deserialize = (PyObject *(*)(struct __pyx_obj_9drf_turbo_6fields_Field *, PyObject *, PyObject *, int __pyx_skip_dispatch))__pyx_f_9drf_turbo_6fields_9FileField_deserialize;
  __pyx_type_9drf_turbo_6fields_FileField.tp_base = __pyx_ptype_9drf_turbo_6fields_Field;
  if (PyType_Ready(&__pyx_type_9drf_turbo_6fields_FileField) < 0) __PYX_ERR(0, 1132, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_9drf_turbo_6fields_FileField.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_9drf_turbo_6fields_FileField.tp_dictoffset && __pyx_type_9drf_turbo_6fields_FileField.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_9drf_turbo_6fields_FileField.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  if (__Pyx_SetVtable(__pyx_type_9drf_turbo_6fields_FileField.tp_dict, __pyx_vtabptr_9drf_turbo_6fields_FileField) < 0) __PYX_ERR(0, 1132, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_FileField, (PyObject *)&__pyx_type_9drf_turbo_6fields_FileField) < 0) __PYX_ERR(0, 1132, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_9drf_turbo_6fields_FileField) < 0) __PYX_ERR(0, 1132, __pyx_L1_error)
  __pyx_ptype_9drf_turbo_6fields_FileField = &__pyx_type_9drf_turbo_6fields_FileField;
  __pyx_vtabptr_9drf_turbo_6fields_ArrayField = &__pyx_vtable_9drf_turbo_6fields_ArrayField;
  __pyx_vtable_9drf_turbo_6fields_ArrayField.__pyx_base = *__pyx_vtabptr_9drf_turbo_6fields_Field;
  __pyx_vtable_9drf_turbo_6fields_ArrayField.__pyx_base.serialize = (PyObject *(*)(struct __pyx_obj_9drf_turbo_6fields_Field *, PyObject *, PyObject *, int __pyx_skip_dispatch))__pyx_f_9drf_turbo_6fields_10ArrayField_serialize;
  __pyx_vtable_9drf_turbo_6fields_ArrayField.__pyx_base.deserialize = (PyObject *(*)(struct __pyx_obj_9drf_turbo_6fields_Field *, PyObject *, PyObject *, int __pyx_skip_dispatch))__pyx_f_9drf_turbo_6fields_10ArrayField_deserialize;
  __pyx_vtable_9drf_turbo_6fields_ArrayField.run_child_validation = (PyObject *(*)(struct __pyx_obj_9drf_turbo_6fields_ArrayField *, PyObject *, PyObject *, int __pyx_skip_dispatch))__pyx_f_9drf_turbo_6fields_10ArrayField_run_child_validation;
  __pyx_type_9drf_turbo_6fields_ArrayField.tp_base = __pyx_ptype_9drf_turbo_6fields_Field;
  if (PyType_Ready(&__pyx_type_9drf_turbo_6fields_ArrayField) < 0) __PYX_ERR(0, 1180, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_9drf_turbo_6fields_ArrayField.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_9drf_turbo_6fields_ArrayField.tp_dictoffset && __pyx_type_9drf_turbo_6fields_ArrayField.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_9drf_turbo_6fields_ArrayField.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  if (__Pyx_SetVtable(__pyx_type_9drf_turbo_6fields_ArrayField.tp_dict, __pyx_vtabptr_9drf_turbo_6fields_ArrayField) < 0) __PYX_ERR(0, 1180, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_ArrayField, (PyObject *)&__pyx_type_9drf_turbo_6fields_ArrayField) < 0) __PYX_ERR(0, 1180, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_9drf_turbo_6fields_ArrayField) < 0) __PYX_ERR(0, 1180, __pyx_L1_error)
  __pyx_ptype_9drf_turbo_6fields_ArrayField = &__pyx_type_9drf_turbo_6fields_ArrayField;
  __pyx_vtabptr_9drf_turbo_6fields_DictField = &__pyx_vtable_9drf_turbo_6fields_DictField;
  __pyx_vtable_9drf_turbo_6fields_DictField.__pyx_base = *__pyx_vtabptr_9drf_turbo_6fields_Field;
  __pyx_vtable_9drf_turbo_6fields_DictField.__pyx_base.serialize = (PyObject *(*)(struct __pyx_obj_9drf_turbo_6fields_Field *, PyObject *, PyObject *, int __pyx_skip_dispatch))__pyx_f_9drf_turbo_6fields_9DictField_serialize;
  __pyx_vtable_9drf_turbo_6fields_DictField.__pyx_base.deserialize = (PyObject *(*)(struct __pyx_obj_9drf_turbo_6fields_Field *, PyObject *, PyObject *, int __pyx_skip_dispatch))__pyx_f_9drf_turbo_6fields_9DictField_deserialize;
  __pyx_vtable_9drf_turbo_6fields_DictField.run_child_validation = (PyObject *(*)(struct __pyx_obj_9drf_turbo_6fields_DictField *, PyObject *, PyObject *, int __pyx_skip_dispatch))__pyx_f_9drf_turbo_6fields_9DictField_run_child_validation;
  __pyx_type_9drf_turbo_6fields_DictField.tp_base = __pyx_ptype_9drf_turbo_6fields_Field;
  if (PyType_Ready(&__pyx_type_9drf_turbo_6fields_DictField) < 0) __PYX_ERR(0, 1259, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_9drf_turbo_6fields_DictField.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_9drf_turbo_6fields_DictField.tp_dictoffset && __pyx_type_9drf_turbo_6fields_DictField.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_9drf_turbo_6fields_DictField.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  if (__Pyx_SetVtable(__pyx_type_9drf_turbo_6fields_DictField.tp_dict, __pyx_vtabptr_9drf_turbo_6fields_DictField) < 0) __PYX_ERR(0, 1259, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_DictField, (PyObject *)&__pyx_type_9drf_turbo_6fields_DictField) < 0) __PYX_ERR(0, 1259, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_9drf_turbo_6fields_DictField) < 0) __PYX_ERR(0, 1259, __pyx_L1_error)
  __pyx_ptype_9drf_turbo_6fields_DictField = &__pyx_type_9drf_turbo_6fields_DictField;
  __pyx_vtabptr_9drf_turbo_6fields_JSONField = &__pyx_vtable_9drf_turbo_6fields_JSONField;
  __pyx_vtable_9drf_turbo_6fields_JSONField.__pyx_base = *__pyx_vtabptr_9drf_turbo_6fields_Field;
  __pyx_vtable_9drf_turbo_6fields_JSONField.__pyx_base.serialize = (PyObject *(*)(struct __pyx_obj_9drf_turbo_6fields_Field *, PyObject *, PyObject *, int __pyx_skip_dispatch))__pyx_f_9drf_turbo_6fields_9JSONField_serialize;
  __pyx_vtable_9drf_turbo_6fields_JSONField.__pyx_base.deserialize = (PyObject *(*)(struct __pyx_obj_9drf_turbo_6fields_Field *, PyObject *, PyObject *, int __pyx_skip_dispatch))__pyx_f_9drf_turbo_6fields_9JSONField_deserialize;
  __pyx_type_9drf_turbo_6fields_JSONField.tp_base = __pyx_ptype_9drf_turbo_6fields_Field;
  if (PyType_Ready(&__pyx_type_9drf_turbo_6fields_JSONField) < 0) __PYX_ERR(0, 1312, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_9drf_turbo_6fields_JSONField.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_9drf_turbo_6fields_JSONField.tp_dictoffset && __pyx_type_9drf_turbo_6fields_JSONField.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_9drf_turbo_6fields_JSONField.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  if (__Pyx_SetVtable(__pyx_type_9drf_turbo_6fields_JSONField.tp_dict, __pyx_vtabptr_9drf_turbo_6fields_JSONField) < 0) __PYX_ERR(0, 1312, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_JSONField, (PyObject *)&__pyx_type_9drf_turbo_6fields_JSONField) < 0) __PYX_ERR(0, 1312, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_9drf_turbo_6fields_JSONField) < 0) __PYX_ERR(0, 1312, __pyx_L1_error)
  __pyx_ptype_9drf_turbo_6fields_JSONField = &__pyx_type_9drf_turbo_6fields_JSONField;
  __pyx_vtabptr_9drf_turbo_6fields_RelatedField = &__pyx_vtable_9drf_turbo_6fields_RelatedField;
  __pyx_vtable_9drf_turbo_6fields_RelatedField.__pyx_base = *__pyx_vtabptr_9drf_turbo_6fields_Field;
  __pyx_vtable_9drf_turbo_6fields_RelatedField.__pyx_base.deserialize = (PyObject *(*)(struct __pyx_obj_9drf_turbo_6fields_Field *, PyObject *, PyObject *, int __pyx_skip_dispatch))__pyx_f_9drf_turbo_6fields_12RelatedField_deserialize;
  __pyx_type_9drf_turbo_6fields_RelatedField.tp_base = __pyx_ptype_9drf_turbo_6fields_Field;
  if (PyType_Ready(&__pyx_type_9drf_turbo_6fields_RelatedField) < 0) __PYX_ERR(0, 1350, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_9drf_turbo_6fields_RelatedField.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_9drf_turbo_6fields_RelatedField.tp_dictoffset && __pyx_type_9drf_turbo_6fields_RelatedField.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_9drf_turbo_6fields_RelatedField.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  if (__Pyx_SetVtable(__pyx_type_9drf_turbo_6fields_RelatedField.tp_dict, __pyx_vtabptr_9drf_turbo_6fields_RelatedField) < 0) __PYX_ERR(0, 1350, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_RelatedField, (PyObject *)&__pyx_type_9drf_turbo_6fields_RelatedField) < 0) __PYX_ERR(0, 1350, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_9drf_turbo_6fields_RelatedField) < 0) __PYX_ERR(0, 1350, __pyx_L1_error)
  __pyx_ptype_9drf_turbo_6fields_RelatedField = &__pyx_type_9drf_turbo_6fields_RelatedField;
  __pyx_vtabptr_9drf_turbo_6fields_ManyRelatedField = &__pyx_vtable_9drf_turbo_6fields_ManyRelatedField;
  __pyx_vtable_9drf_turbo_6fields_ManyRelatedField.__pyx_base = *__pyx_vtabptr_9drf_turbo_6fields_Field;
  __pyx_vtable_9drf_turbo_6fields_ManyRelatedField.__pyx_base.serialize = (PyObject *(*)(struct __pyx_obj_9drf_turbo_6fields_Field *, PyObject *, PyObject *, int __pyx_skip_dispatch))__pyx_f_9drf_turbo_6fields_16ManyRelatedField_serialize;
  __pyx_vtable_9drf_turbo_6fields_ManyRelatedField.__pyx_base.deserialize = (PyObject *(*)(struct __pyx_obj_9drf_turbo_6fields_Field *, PyObject *, PyObject *, int __pyx_skip_dispatch))__pyx_f_9drf_turbo_6fields_16ManyRelatedField_deserialize;
  __pyx_type_9drf_turbo_6fields_ManyRelatedField.tp_base = __pyx_ptype_9drf_turbo_6fields_Field;
  if (PyType_Ready(&__pyx_type_9drf_turbo_6fields_ManyRelatedField) < 0) __PYX_ERR(0, 1377, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_9drf_turbo_6fields_ManyRelatedField.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_9drf_turbo_6fields_ManyRelatedField.tp_dictoffset && __pyx_type_9drf_turbo_6fields_ManyRelatedField.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_9drf_turbo_6fields_ManyRelatedField.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  if (__Pyx_SetVtable(__pyx_type_9drf_turbo_6fields_ManyRelatedField.tp_dict, __pyx_vtabptr_9drf_turbo_6fields_ManyRelatedField) < 0) __PYX_ERR(0, 1377, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_ManyRelatedField, (PyObject *)&__pyx_type_9drf_turbo_6fields_ManyRelatedField) < 0) __PYX_ERR(0, 1377, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_9drf_turbo_6fields_ManyRelatedField) < 0) __PYX_ERR(0, 1377, __pyx_L1_error)
  __pyx_ptype_9drf_turbo_6fields_ManyRelatedField = &__pyx_type_9drf_turbo_6fields_ManyRelatedField;
  __pyx_vtabptr_9drf_turbo_6fields_ConstantField = &__pyx_vtable_9drf_turbo_6fields_ConstantField;
  __pyx_vtable_9drf_turbo_6fields_ConstantField.__pyx_base = *__pyx_vtabptr_9drf_turbo_6fields_Field;
  __pyx_vtable_9drf_turbo_6fields_ConstantField.__pyx_base.deserialize = (PyObject *(*)(struct __pyx_obj_9drf_turbo_6fields_Field *, PyObject *, PyObject *, int __pyx_skip_dispatch))__pyx_f_9drf_turbo_6fields_13ConstantField_deserialize;
  __pyx_type_9drf_turbo_6fields_ConstantField.tp_base = __pyx_ptype_9drf_turbo_6fields_Field;
  if (PyType_Ready(&__pyx_type_9drf_turbo_6fields_ConstantField) < 0) __PYX_ERR(0, 1413, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_9drf_turbo_6fields_ConstantField.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_9drf_turbo_6fields_ConstantField.tp_dictoffset && __pyx_type_9drf_turbo_6fields_ConstantField.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_9drf_turbo_6fields_ConstantField.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  if (__Pyx_SetVtable(__pyx_type_9drf_turbo_6fields_ConstantField.tp_dict, __pyx_vtabptr_9drf_turbo_6fields_ConstantField) < 0) __PYX_ERR(0, 1413, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_ConstantField, (PyObject *)&__pyx_type_9drf_turbo_6fields_ConstantField) < 0) __PYX_ERR(0, 1413, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_9drf_turbo_6fields_ConstantField) < 0) __PYX_ERR(0, 1413, __pyx_L1_error)
  __pyx_ptype_9drf_turbo_6fields_ConstantField = &__pyx_type_9drf_turbo_6fields_ConstantField;
  __pyx_vtabptr_9drf_turbo_6fields_RecursiveField = &__pyx_vtable_9drf_turbo_6fields_RecursiveField;
  __pyx_vtable_9drf_turbo_6fields_RecursiveField.__pyx_base = *__pyx_vtabptr_9drf_turbo_6fields_Field;
  __pyx_vtable_9drf_turbo_6fields_RecursiveField.__pyx_base.serialize = (PyObject *(*)(struct __pyx_obj_9drf_turbo_6fields_Field *, PyObject *, PyObject *, int __pyx_skip_dispatch))__pyx_f_9drf_turbo_6fields_14RecursiveField_serialize;
  __pyx_type_9drf_turbo_6fields_RecursiveField.tp_base = __pyx_ptype_9drf_turbo_6fields_Field;
  if (PyType_Ready(&__pyx_type_9drf_turbo_6fields_RecursiveField) < 0) __PYX_ERR(0, 1438, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_9drf_turbo_6fields_RecursiveField.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_9drf_turbo_6fields_RecursiveField.tp_dictoffset && __pyx_type_9drf_turbo_6fields_RecursiveField.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_9drf_turbo_6fields_RecursiveField.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  if (__Pyx_SetVtable(__pyx_type_9drf_turbo_6fields_RecursiveField.tp_dict, __pyx_vtabptr_9drf_turbo_6fields_RecursiveField) < 0) __PYX_ERR(0, 1438, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_RecursiveField, (PyObject *)&__pyx_type_9drf_turbo_6fields_RecursiveField) < 0) __PYX_ERR(0, 1438, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_9drf_turbo_6fields_RecursiveField) < 0) __PYX_ERR(0, 1438, __pyx_L1_error)
  __pyx_ptype_9drf_turbo_6fields_RecursiveField = &__pyx_type_9drf_turbo_6fields_RecursiveField;
  __pyx_vtabptr_9drf_turbo_6fields_MethodField = &__pyx_vtable_9drf_turbo_6fields_MethodField;
  __pyx_vtable_9drf_turbo_6fields_MethodField.__pyx_base = *__pyx_vtabptr_9drf_turbo_6fields_Field;
  __pyx_vtable_9drf_turbo_6fields_MethodField.__pyx_base.method_getter = (PyObject *(*)(struct __pyx_obj_9drf_turbo_6fields_Field *, PyObject *, PyObject *, int __pyx_skip_dispatch))__pyx_f_9drf_turbo_6fields_11MethodField_method_getter;
  __pyx_type_9drf_turbo_6fields_MethodField.tp_base = __pyx_ptype_9drf_turbo_6fields_Field;
  if (PyType_Ready(&__pyx_type_9drf_turbo_6fields_MethodField) < 0) __PYX_ERR(0, 1472, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_9drf_turbo_6fields_MethodField.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_9drf_turbo_6fields_MethodField.tp_dictoffset && __pyx_type_9drf_turbo_6fields_MethodField.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_9drf_turbo_6fields_MethodField.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  if (__Pyx_SetVtable(__pyx_type_9drf_turbo_6fields_MethodField.tp_dict, __pyx_vtabptr_9drf_turbo_6fields_MethodField) < 0) __PYX_ERR(0, 1472, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_MethodField, (PyObject *)&__pyx_type_9drf_turbo_6fields_MethodField) < 0) __PYX_ERR(0, 1472, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_9drf_turbo_6fields_MethodField) < 0) __PYX_ERR(0, 1472, __pyx_L1_error)
  __pyx_ptype_9drf_turbo_6fields_MethodField = &__pyx_type_9drf_turbo_6fields_MethodField;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_type_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_import_code", 0);
  /*--- Type import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_variable_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_import_code", 0);
  /*--- Variable import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_import_code", 0);
  /*--- Function import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}


#ifndef CYTHON_NO_PYINIT_EXPORT
#define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#elif PY_MAJOR_VERSION < 3
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" void
#else
#define __Pyx_PyMODINIT_FUNC void
#endif
#else
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" PyObject *
#else
#define __Pyx_PyMODINIT_FUNC PyObject *
#endif
#endif


#if PY_MAJOR_VERSION < 3
__Pyx_PyMODINIT_FUNC initfields(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC initfields(void)
#else
__Pyx_PyMODINIT_FUNC PyInit_fields(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC PyInit_fields(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
static CYTHON_SMALL_CODE int __Pyx_check_single_interpreter(void) {
    #if PY_VERSION_HEX >= 0x030700A1
    static PY_INT64_T main_interpreter_id = -1;
    PY_INT64_T current_id = PyInterpreterState_GetID(PyThreadState_Get()->interp);
    if (main_interpreter_id == -1) {
        main_interpreter_id = current_id;
        return (unlikely(current_id == -1)) ? -1 : 0;
    } else if (unlikely(main_interpreter_id != current_id))
    #else
    static PyInterpreterState *main_interpreter = NULL;
    PyInterpreterState *current_interpreter = PyThreadState_Get()->interp;
    if (!main_interpreter) {
        main_interpreter = current_interpreter;
    } else if (unlikely(main_interpreter != current_interpreter))
    #endif
    {
        PyErr_SetString(
            PyExc_ImportError,
            "Interpreter change detected - this module can only be loaded into one interpreter per process.");
        return -1;
    }
    return 0;
}
static CYTHON_SMALL_CODE int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name, int allow_none) {
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        if (allow_none || value != Py_None) {
            result = PyDict_SetItemString(moddict, to_name, value);
        }
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static CYTHON_SMALL_CODE PyObject* __pyx_pymod_create(PyObject *spec, CYTHON_UNUSED PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    if (__Pyx_check_single_interpreter())
        return NULL;
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__", 0) < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static CYTHON_SMALL_CODE int __pyx_pymod_exec_fields(PyObject *__pyx_pyinit_module)
#endif
#endif
{
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m) {
    if (__pyx_m == __pyx_pyinit_module) return 0;
    PyErr_SetString(PyExc_RuntimeError, "Module 'fields' has already been imported. Re-initialisation is not supported.");
    return -1;
  }
  #elif PY_MAJOR_VERSION >= 3
  if (__pyx_m) return __Pyx_NewRef(__pyx_m);
  #endif
  #if CYTHON_REFNANNY
__Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
if (!__Pyx_RefNanny) {
  PyErr_Clear();
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
  if (!__Pyx_RefNanny)
      Py_FatalError("failed to import 'refnanny' module");
}
#endif
  __Pyx_RefNannySetupContext("__Pyx_PyMODINIT_FUNC PyInit_fields(void)", 0);
  if (__Pyx_check_binary_version() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pxy_PyFrame_Initialize_Offsets
  __Pxy_PyFrame_Initialize_Offsets();
  #endif
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_AsyncGen_USED
  if (__pyx_AsyncGen_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(WITH_THREAD) && PY_VERSION_HEX < 0x030700F0 && defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  PyEval_InitThreads();
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_m = __pyx_pyinit_module;
  Py_INCREF(__pyx_m);
  #else
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("fields", __pyx_methods, 0, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_b);
  __pyx_cython_runtime = PyImport_AddModule((char *) "cython_runtime"); if (unlikely(!__pyx_cython_runtime)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_cython_runtime);
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main_drf_turbo__fields) {
    if (PyObject_SetAttr(__pyx_m, __pyx_n_s_name, __pyx_n_s_main) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "drf_turbo.fields")) {
      if (unlikely(PyDict_SetItemString(modules, "drf_turbo.fields", __pyx_m) < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Global type/function init code ---*/
  (void)__Pyx_modinit_global_init_code();
  if (unlikely(__Pyx_modinit_variable_export_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  (void)__Pyx_modinit_function_export_code();
  if (unlikely(__Pyx_modinit_type_init_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  (void)__Pyx_modinit_type_import_code();
  (void)__Pyx_modinit_variable_import_code();
  (void)__Pyx_modinit_function_import_code();
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif

  /* "drf_turbo/fields.pyx":3
 * cimport cython
 * 
 * from drf_turbo.utils import is_iterable_and_not_string,get_error_detail,is_collection,get_attribute,force_str             # <<<<<<<<<<<<<<
 * from drf_turbo.exceptions import *
 * from django.core.exceptions import ValidationError as DjangoValidationError
 */
  __pyx_t_1 = PyList_New(5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_is_iterable_and_not_string);
  __Pyx_GIVEREF(__pyx_n_s_is_iterable_and_not_string);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_is_iterable_and_not_string);
  __Pyx_INCREF(__pyx_n_s_get_error_detail);
  __Pyx_GIVEREF(__pyx_n_s_get_error_detail);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_s_get_error_detail);
  __Pyx_INCREF(__pyx_n_s_is_collection);
  __Pyx_GIVEREF(__pyx_n_s_is_collection);
  PyList_SET_ITEM(__pyx_t_1, 2, __pyx_n_s_is_collection);
  __Pyx_INCREF(__pyx_n_s_get_attribute);
  __Pyx_GIVEREF(__pyx_n_s_get_attribute);
  PyList_SET_ITEM(__pyx_t_1, 3, __pyx_n_s_get_attribute);
  __Pyx_INCREF(__pyx_n_s_force_str);
  __Pyx_GIVEREF(__pyx_n_s_force_str);
  PyList_SET_ITEM(__pyx_t_1, 4, __pyx_n_s_force_str);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_drf_turbo_utils, __pyx_t_1, -1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_is_iterable_and_not_string); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_is_iterable_and_not_string, __pyx_t_1) < 0) __PYX_ERR(0, 3, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_get_error_detail); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_get_error_detail, __pyx_t_1) < 0) __PYX_ERR(0, 3, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_is_collection); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_is_collection, __pyx_t_1) < 0) __PYX_ERR(0, 3, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_get_attribute); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_get_attribute, __pyx_t_1) < 0) __PYX_ERR(0, 3, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_force_str); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_force_str, __pyx_t_1) < 0) __PYX_ERR(0, 3, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "drf_turbo/fields.pyx":4
 * 
 * from drf_turbo.utils import is_iterable_and_not_string,get_error_detail,is_collection,get_attribute,force_str
 * from drf_turbo.exceptions import *             # <<<<<<<<<<<<<<
 * from django.core.exceptions import ValidationError as DjangoValidationError
 * import ipaddress,copy,json,uuid,decimal,re,datetime
 */
  __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_s__31);
  __Pyx_GIVEREF(__pyx_n_s__31);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s__31);
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_drf_turbo_exceptions, __pyx_t_2, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_import_star(__pyx_t_1) < 0) __PYX_ERR(0, 4, __pyx_L1_error);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "drf_turbo/fields.pyx":5
 * from drf_turbo.utils import is_iterable_and_not_string,get_error_detail,is_collection,get_attribute,force_str
 * from drf_turbo.exceptions import *
 * from django.core.exceptions import ValidationError as DjangoValidationError             # <<<<<<<<<<<<<<
 * import ipaddress,copy,json,uuid,decimal,re,datetime
 * from django.core.exceptions import ObjectDoesNotExist
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_ValidationError);
  __Pyx_GIVEREF(__pyx_n_s_ValidationError);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_ValidationError);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_django_core_exceptions, __pyx_t_1, -1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_ValidationError); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_DjangoValidationError, __pyx_t_1) < 0) __PYX_ERR(0, 5, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "drf_turbo/fields.pyx":6
 * from drf_turbo.exceptions import *
 * from django.core.exceptions import ValidationError as DjangoValidationError
 * import ipaddress,copy,json,uuid,decimal,re,datetime             # <<<<<<<<<<<<<<
 * from django.core.exceptions import ObjectDoesNotExist
 * from django.utils.dateparse import (
 */
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_ipaddress, 0, -1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_ipaddress, __pyx_t_2) < 0) __PYX_ERR(0, 6, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_copy, 0, -1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_copy, __pyx_t_2) < 0) __PYX_ERR(0, 6, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_json, 0, -1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_json, __pyx_t_2) < 0) __PYX_ERR(0, 6, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_uuid, 0, -1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_uuid, __pyx_t_2) < 0) __PYX_ERR(0, 6, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_decimal, 0, -1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_decimal, __pyx_t_2) < 0) __PYX_ERR(0, 6, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_re, 0, -1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_re, __pyx_t_2) < 0) __PYX_ERR(0, 6, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_datetime, 0, -1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_datetime, __pyx_t_2) < 0) __PYX_ERR(0, 6, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "drf_turbo/fields.pyx":7
 * from django.core.exceptions import ValidationError as DjangoValidationError
 * import ipaddress,copy,json,uuid,decimal,re,datetime
 * from django.core.exceptions import ObjectDoesNotExist             # <<<<<<<<<<<<<<
 * from django.utils.dateparse import (
 *     parse_date, parse_datetime, parse_time
 */
  __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_s_ObjectDoesNotExist);
  __Pyx_GIVEREF(__pyx_n_s_ObjectDoesNotExist);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_ObjectDoesNotExist);
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_django_core_exceptions, __pyx_t_2, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_1, __pyx_n_s_ObjectDoesNotExist); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_ObjectDoesNotExist, __pyx_t_2) < 0) __PYX_ERR(0, 7, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "drf_turbo/fields.pyx":9
 * from django.core.exceptions import ObjectDoesNotExist
 * from django.utils.dateparse import (
 *     parse_date, parse_datetime, parse_time             # <<<<<<<<<<<<<<
 * )
 * from rest_framework.settings import api_settings
 */
  __pyx_t_1 = PyList_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_parse_date);
  __Pyx_GIVEREF(__pyx_n_s_parse_date);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_parse_date);
  __Pyx_INCREF(__pyx_n_s_parse_datetime);
  __Pyx_GIVEREF(__pyx_n_s_parse_datetime);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_s_parse_datetime);
  __Pyx_INCREF(__pyx_n_s_parse_time);
  __Pyx_GIVEREF(__pyx_n_s_parse_time);
  PyList_SET_ITEM(__pyx_t_1, 2, __pyx_n_s_parse_time);

  /* "drf_turbo/fields.pyx":8
 * import ipaddress,copy,json,uuid,decimal,re,datetime
 * from django.core.exceptions import ObjectDoesNotExist
 * from django.utils.dateparse import (             # <<<<<<<<<<<<<<
 *     parse_date, parse_datetime, parse_time
 * )
 */
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_django_utils_dateparse, __pyx_t_1, -1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 8, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_parse_date); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 8, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_parse_date, __pyx_t_1) < 0) __PYX_ERR(0, 9, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_parse_datetime); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 8, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_parse_datetime, __pyx_t_1) < 0) __PYX_ERR(0, 9, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_parse_time); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 8, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_parse_time, __pyx_t_1) < 0) __PYX_ERR(0, 9, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "drf_turbo/fields.pyx":11
 *     parse_date, parse_datetime, parse_time
 * )
 * from rest_framework.settings import api_settings             # <<<<<<<<<<<<<<
 * from rest_framework import (
 *     ISO_8601
 */
  __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 11, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_s_api_settings);
  __Pyx_GIVEREF(__pyx_n_s_api_settings);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_api_settings);
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_rest_framework_settings, __pyx_t_2, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 11, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_1, __pyx_n_s_api_settings); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 11, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_api_settings, __pyx_t_2) < 0) __PYX_ERR(0, 11, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "drf_turbo/fields.pyx":13
 * from rest_framework.settings import api_settings
 * from rest_framework import (
 *     ISO_8601             # <<<<<<<<<<<<<<
 * )
 * from django.conf import settings
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 13, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_ISO_8601);
  __Pyx_GIVEREF(__pyx_n_s_ISO_8601);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_ISO_8601);

  /* "drf_turbo/fields.pyx":12
 * )
 * from rest_framework.settings import api_settings
 * from rest_framework import (             # <<<<<<<<<<<<<<
 *     ISO_8601
 * )
 */
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_rest_framework, __pyx_t_1, -1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_ISO_8601); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_ISO_8601, __pyx_t_1) < 0) __PYX_ERR(0, 13, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "drf_turbo/fields.pyx":15
 *     ISO_8601
 * )
 * from django.conf import settings             # <<<<<<<<<<<<<<
 * from django.utils import timezone
 * from pytz.exceptions import InvalidTimeError
 */
  __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_s_settings);
  __Pyx_GIVEREF(__pyx_n_s_settings);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_settings);
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_django_conf, __pyx_t_2, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_1, __pyx_n_s_settings); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_settings, __pyx_t_2) < 0) __PYX_ERR(0, 15, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "drf_turbo/fields.pyx":16
 * )
 * from django.conf import settings
 * from django.utils import timezone             # <<<<<<<<<<<<<<
 * from pytz.exceptions import InvalidTimeError
 * 
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_timezone);
  __Pyx_GIVEREF(__pyx_n_s_timezone);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_timezone);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_django_utils, __pyx_t_1, -1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_timezone); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_timezone, __pyx_t_1) < 0) __PYX_ERR(0, 16, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "drf_turbo/fields.pyx":17
 * from django.conf import settings
 * from django.utils import timezone
 * from pytz.exceptions import InvalidTimeError             # <<<<<<<<<<<<<<
 * 
 * EMAIL_REGEX = re.compile(
 */
  __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_s_InvalidTimeError);
  __Pyx_GIVEREF(__pyx_n_s_InvalidTimeError);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_InvalidTimeError);
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_pytz_exceptions, __pyx_t_2, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_1, __pyx_n_s_InvalidTimeError); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_InvalidTimeError, __pyx_t_2) < 0) __PYX_ERR(0, 17, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "drf_turbo/fields.pyx":19
 * from pytz.exceptions import InvalidTimeError
 * 
 * EMAIL_REGEX = re.compile(             # <<<<<<<<<<<<<<
 *     r"(^[-!#$%&'*+/=?^_`{}|~0-9A-Z]+(\.[-!#$%&'*+/=?^_`{}|~0-9A-Z]+)*"
 *     r'|^"([\001-\010\013\014\016-\037!#-\[\]-\177]|\\[\001-\011\013\014\016-\177])*'
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_re); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 19, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_compile); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 19, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "drf_turbo/fields.pyx":23
 *     r'|^"([\001-\010\013\014\016-\037!#-\[\]-\177]|\\[\001-\011\013\014\016-\177])*'
 *     r")@(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+[A-Z]{2,63}(?<!-)\.?$",
 *     re.IGNORECASE,             # <<<<<<<<<<<<<<
 * )
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_re); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 23, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_IGNORECASE); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 23, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "drf_turbo/fields.pyx":19
 * from pytz.exceptions import InvalidTimeError
 * 
 * EMAIL_REGEX = re.compile(             # <<<<<<<<<<<<<<
 *     r"(^[-!#$%&'*+/=?^_`{}|~0-9A-Z]+(\.[-!#$%&'*+/=?^_`{}|~0-9A-Z]+)*"
 *     r'|^"([\001-\010\013\014\016-\037!#-\[\]-\177]|\\[\001-\011\013\014\016-\177])*'
 */
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 19, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_kp_s_0_9A_Z___0_9A_Z_001_010_013_0);
  __Pyx_GIVEREF(__pyx_kp_s_0_9A_Z___0_9A_Z_001_010_013_0);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_kp_s_0_9A_Z___0_9A_Z_001_010_013_0);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_3);
  __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_1, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 19, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_EMAIL_REGEX, __pyx_t_3) < 0) __PYX_ERR(0, 19, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "drf_turbo/fields.pyx":26
 * )
 * 
 * URL_REGEX = re.compile(             # <<<<<<<<<<<<<<
 *     r'^(?:http|ftp)s?://'  # http:// or https://
 *     r'(?:(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+(?:[A-Z]{2,6}\.?|[A-Z0-9-]{2,}\.?)|'  # domain..
 */
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_re); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 26, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_compile); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 26, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "drf_turbo/fields.pyx":32
 *     r'\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})'  # ...or ip
 *     r'(?::\d+)?'  # optional port
 *     r'(?:/?|[/?]\S+)$', re.IGNORECASE)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_re); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 32, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_IGNORECASE); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 32, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "drf_turbo/fields.pyx":26
 * )
 * 
 * URL_REGEX = re.compile(             # <<<<<<<<<<<<<<
 *     r'^(?:http|ftp)s?://'  # http:// or https://
 *     r'(?:(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+(?:[A-Z]{2,6}\.?|[A-Z0-9-]{2,}\.?)|'  # domain..
 */
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 26, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_kp_s_http_ftp_s_A_Z0_9_A_Z0_9_0_61_A);
  __Pyx_GIVEREF(__pyx_kp_s_http_ftp_s_A_Z0_9_A_Z0_9_0_61_A);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_kp_s_http_ftp_s_A_Z0_9_A_Z0_9_0_61_A);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 26, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_URL_REGEX, __pyx_t_2) < 0) __PYX_ERR(0, 26, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "drf_turbo/fields.pyx":35
 * 
 * 
 * cdef object NO_DEFAULT = object()             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_builtin_object); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 35, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_XGOTREF(__pyx_v_9drf_turbo_6fields_NO_DEFAULT);
  __Pyx_DECREF_SET(__pyx_v_9drf_turbo_6fields_NO_DEFAULT, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_2 = 0;

  /* "drf_turbo/fields.pyx":67
 *     """
 * 
 *     is_method_field = False             # <<<<<<<<<<<<<<
 *     default_error_messages = {
 *         'required': 'This field is required.',
 */
  if (PyDict_SetItem((PyObject *)__pyx_ptype_9drf_turbo_6fields_Field->tp_dict, __pyx_n_s_is_method_field, Py_False) < 0) __PYX_ERR(0, 67, __pyx_L1_error)
  PyType_Modified(__pyx_ptype_9drf_turbo_6fields_Field);

  /* "drf_turbo/fields.pyx":69
 *     is_method_field = False
 *     default_error_messages = {
 *         'required': 'This field is required.',             # <<<<<<<<<<<<<<
 *         'null': 'This field may not be null.',
 *         }
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 69, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_required, __pyx_kp_s_This_field_is_required) < 0) __PYX_ERR(0, 69, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_null, __pyx_kp_s_This_field_may_not_be_null) < 0) __PYX_ERR(0, 69, __pyx_L1_error)
  if (PyDict_SetItem((PyObject *)__pyx_ptype_9drf_turbo_6fields_Field->tp_dict, __pyx_n_s_default_error_messages, __pyx_t_2) < 0) __PYX_ERR(0, 68, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_9drf_turbo_6fields_Field);

  /* "drf_turbo/fields.pyx":72
 *         'null': 'This field may not be null.',
 *         }
 *     _initial = None             # <<<<<<<<<<<<<<
 * 
 *     def __init__(
 */
  if (PyDict_SetItem((PyObject *)__pyx_ptype_9drf_turbo_6fields_Field->tp_dict, __pyx_n_s_initial_2, Py_None) < 0) __PYX_ERR(0, 72, __pyx_L1_error)
  PyType_Modified(__pyx_ptype_9drf_turbo_6fields_Field);

  /* "drf_turbo/fields.pyx":86
 *         dict style = None ,
 *         object validators = None,
 *         object default_value  = NO_DEFAULT,             # <<<<<<<<<<<<<<
 *         object initial = NO_DEFAULT,
 *         basestring field_name = None,
 */
  __Pyx_INCREF(__pyx_v_9drf_turbo_6fields_NO_DEFAULT);
  __pyx_k_ = __pyx_v_9drf_turbo_6fields_NO_DEFAULT;
  __Pyx_GIVEREF(__pyx_v_9drf_turbo_6fields_NO_DEFAULT);

  /* "drf_turbo/fields.pyx":87
 *         object validators = None,
 *         object default_value  = NO_DEFAULT,
 *         object initial = NO_DEFAULT,             # <<<<<<<<<<<<<<
 *         basestring field_name = None,
 *         object root = None ,
 */
  __Pyx_INCREF(__pyx_v_9drf_turbo_6fields_NO_DEFAULT);
  __pyx_k__2 = __pyx_v_9drf_turbo_6fields_NO_DEFAULT;
  __Pyx_GIVEREF(__pyx_v_9drf_turbo_6fields_NO_DEFAULT);

  /* "drf_turbo/fields.pyx":299
 * 
 *     default_error_messages = {
 *         'blank': 'May not be blank.',             # <<<<<<<<<<<<<<
 *         'invalid': 'Not a valid string.',
 *         'null' : 'Null characters are not allowed.',
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 299, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_blank, __pyx_kp_s_May_not_be_blank) < 0) __PYX_ERR(0, 299, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_invalid, __pyx_kp_s_Not_a_valid_string) < 0) __PYX_ERR(0, 299, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_null, __pyx_kp_s_Null_characters_are_not_allowed) < 0) __PYX_ERR(0, 299, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_min_length, __pyx_kp_s_Must_have_at_least_min_length_ch) < 0) __PYX_ERR(0, 299, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_max_length, __pyx_kp_s_Must_have_no_more_than_max_lengt) < 0) __PYX_ERR(0, 299, __pyx_L1_error)
  if (PyDict_SetItem((PyObject *)__pyx_ptype_9drf_turbo_6fields_StrField->tp_dict, __pyx_n_s_default_error_messages, __pyx_t_2) < 0) __PYX_ERR(0, 298, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_9drf_turbo_6fields_StrField);

  /* "drf_turbo/fields.pyx":306
 * 
 *     }
 *     _initial = ''             # <<<<<<<<<<<<<<
 *     def __init__(self,**kwargs) :
 *         self.allow_blank = kwargs.pop('allow_blank', False)
 */
  if (PyDict_SetItem((PyObject *)__pyx_ptype_9drf_turbo_6fields_StrField->tp_dict, __pyx_n_s_initial_2, __pyx_kp_s__9) < 0) __PYX_ERR(0, 306, __pyx_L1_error)
  PyType_Modified(__pyx_ptype_9drf_turbo_6fields_StrField);

  /* "drf_turbo/fields.pyx":346
 * 
 *     default_error_messages = {
 *         'invalid': 'Enter a valid email address.'             # <<<<<<<<<<<<<<
 *     }
 *     def __init__(self, **kwargs):
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 346, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_invalid, __pyx_kp_s_Enter_a_valid_email_address) < 0) __PYX_ERR(0, 346, __pyx_L1_error)
  if (PyDict_SetItem((PyObject *)__pyx_ptype_9drf_turbo_6fields_EmailField->tp_dict, __pyx_n_s_default_error_messages, __pyx_t_2) < 0) __PYX_ERR(0, 345, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_9drf_turbo_6fields_EmailField);

  /* "drf_turbo/fields.pyx":375
 * 
 *     default_error_messages = {
 *         'invalid': 'Enter a valid URL.'             # <<<<<<<<<<<<<<
 *     }
 * 
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 375, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_invalid, __pyx_kp_s_Enter_a_valid_URL) < 0) __PYX_ERR(0, 375, __pyx_L1_error)
  if (PyDict_SetItem((PyObject *)__pyx_ptype_9drf_turbo_6fields_URLField->tp_dict, __pyx_n_s_default_error_messages, __pyx_t_2) < 0) __PYX_ERR(0, 374, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_9drf_turbo_6fields_URLField);

  /* "drf_turbo/fields.pyx":397
 * 
 *     default_error_messages = {
 *         'invalid': 'This value does not match the required pattern.'             # <<<<<<<<<<<<<<
 *     }
 * 
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 397, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_invalid, __pyx_kp_s_This_value_does_not_match_the_re) < 0) __PYX_ERR(0, 397, __pyx_L1_error)
  if (PyDict_SetItem((PyObject *)__pyx_ptype_9drf_turbo_6fields_RegexField->tp_dict, __pyx_n_s_default_error_messages, __pyx_t_2) < 0) __PYX_ERR(0, 396, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_9drf_turbo_6fields_RegexField);

  /* "drf_turbo/fields.pyx":421
 * 
 *     default_error_messages = {
 *         'invalid': 'Enter a valid IPv4 or IPv6 address.',             # <<<<<<<<<<<<<<
 *     }
 * 
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 421, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_invalid, __pyx_kp_s_Enter_a_valid_IPv4_or_IPv6_addre) < 0) __PYX_ERR(0, 421, __pyx_L1_error)
  if (PyDict_SetItem((PyObject *)__pyx_ptype_9drf_turbo_6fields_IPField->tp_dict, __pyx_n_s_default_error_messages, __pyx_t_2) < 0) __PYX_ERR(0, 420, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_9drf_turbo_6fields_IPField);

  /* "drf_turbo/fields.pyx":455
 *     default_error_messages = {
 * 
 *         "invalid": "Not a valid UUID."             # <<<<<<<<<<<<<<
 *     }
 * 
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 455, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_invalid, __pyx_kp_s_Not_a_valid_UUID) < 0) __PYX_ERR(0, 455, __pyx_L1_error)
  if (PyDict_SetItem((PyObject *)__pyx_ptype_9drf_turbo_6fields_UUIDField->tp_dict, __pyx_n_s_default_error_messages, __pyx_t_2) < 0) __PYX_ERR(0, 453, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_9drf_turbo_6fields_UUIDField);

  /* "drf_turbo/fields.pyx":484
 *     """
 *     default_error_messages = {
 *         'invalid': 'Not a valid slug.',             # <<<<<<<<<<<<<<
 *         'invalid_unicode' : 'Nnot a valid unicode slug.'
 *     }
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 484, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_invalid, __pyx_kp_s_Not_a_valid_slug) < 0) __PYX_ERR(0, 484, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_invalid_unicode, __pyx_kp_s_Nnot_a_valid_unicode_slug) < 0) __PYX_ERR(0, 484, __pyx_L1_error)
  if (PyDict_SetItem((PyObject *)__pyx_ptype_9drf_turbo_6fields_SlugField->tp_dict, __pyx_n_s_default_error_messages, __pyx_t_2) < 0) __PYX_ERR(0, 483, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_9drf_turbo_6fields_SlugField);

  /* "drf_turbo/fields.pyx":515
 *     """
 *     default_error_messages = {
 *         'invalid': 'A valid integer is required.',             # <<<<<<<<<<<<<<
 *         'min_value': 'Must be greater than or equal to {min_value}.',
 *         'max_value': 'Must be less than or equal to {max_value}.',
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 515, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_invalid, __pyx_kp_s_A_valid_integer_is_required) < 0) __PYX_ERR(0, 515, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_min_value, __pyx_kp_s_Must_be_greater_than_or_equal_to) < 0) __PYX_ERR(0, 515, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_max_value, __pyx_kp_s_Must_be_less_than_or_equal_to_ma) < 0) __PYX_ERR(0, 515, __pyx_L1_error)
  if (PyDict_SetItem((PyObject *)__pyx_ptype_9drf_turbo_6fields_IntField->tp_dict, __pyx_n_s_default_error_messages, __pyx_t_2) < 0) __PYX_ERR(0, 514, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_9drf_turbo_6fields_IntField);

  /* "drf_turbo/fields.pyx":519
 *         'max_value': 'Must be less than or equal to {max_value}.',
 *     }
 *     re_decimal = re.compile(r'\.0*\s*$')  # allow e.g. '1.0' as an int, but not '1.2'             # <<<<<<<<<<<<<<
 *     def __init__(self,**kwargs):
 *         self.max_value = kwargs.pop('max_value', None)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_re); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 519, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_compile); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 519, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
    }
  }
  __pyx_t_2 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_3, __pyx_kp_s_0_s) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_kp_s_0_s);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 519, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_9drf_turbo_6fields_IntField->tp_dict, __pyx_n_s_re_decimal, __pyx_t_2) < 0) __PYX_ERR(0, 519, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_9drf_turbo_6fields_IntField);

  /* "drf_turbo/fields.pyx":551
 * 
 *     default_error_messages = {
 *         'invalid': 'A valid number is required.',             # <<<<<<<<<<<<<<
 *         'min_value': 'Must be greater than or equal to {min_value}.',
 *         'max_value': 'Must be less than or equal to {max_value}.',
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 551, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_invalid, __pyx_kp_s_A_valid_number_is_required) < 0) __PYX_ERR(0, 551, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_min_value, __pyx_kp_s_Must_be_greater_than_or_equal_to) < 0) __PYX_ERR(0, 551, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_max_value, __pyx_kp_s_Must_be_less_than_or_equal_to_ma) < 0) __PYX_ERR(0, 551, __pyx_L1_error)
  if (PyDict_SetItem((PyObject *)__pyx_ptype_9drf_turbo_6fields_FloatField->tp_dict, __pyx_n_s_default_error_messages, __pyx_t_2) < 0) __PYX_ERR(0, 550, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_9drf_turbo_6fields_FloatField);

  /* "drf_turbo/fields.pyx":593
 * 
 *     default_error_messages = {
 *         'invalid': 'A valid number is required.',             # <<<<<<<<<<<<<<
 *         'max_value': 'Must be less than or equal to {max_value}.',
 *         'min_value': 'Must be greater than or equal to {min_value}.',
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 593, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_invalid, __pyx_kp_s_A_valid_number_is_required) < 0) __PYX_ERR(0, 593, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_max_value, __pyx_kp_s_Must_be_less_than_or_equal_to_ma) < 0) __PYX_ERR(0, 593, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_min_value, __pyx_kp_s_Must_be_greater_than_or_equal_to) < 0) __PYX_ERR(0, 593, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_max_digits, __pyx_kp_s_Ensure_that_there_are_no_more_th) < 0) __PYX_ERR(0, 593, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_max_decimal_places, __pyx_kp_s_Ensure_that_there_are_no_more_th_2) < 0) __PYX_ERR(0, 593, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_max_whole_digits, __pyx_kp_s_Ensure_that_there_are_no_more_th_3) < 0) __PYX_ERR(0, 593, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_max_string_length, __pyx_kp_s_String_value_too_large) < 0) __PYX_ERR(0, 593, __pyx_L1_error)
  if (PyDict_SetItem((PyObject *)__pyx_ptype_9drf_turbo_6fields_DecimalField->tp_dict, __pyx_n_s_default_error_messages, __pyx_t_2) < 0) __PYX_ERR(0, 592, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_9drf_turbo_6fields_DecimalField);

  /* "drf_turbo/fields.pyx":601
 *         'max_string_length': 'String value too large.'
 *     }
 *     MAX_STRING_LENGTH = 1000  # Guard against malicious string inputs.             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, max_digits, decimal_places, max_value=None, min_value=None,coerce_to_string=None,rounding=None,**kwargs):
 */
  if (PyDict_SetItem((PyObject *)__pyx_ptype_9drf_turbo_6fields_DecimalField->tp_dict, __pyx_n_s_MAX_STRING_LENGTH, __pyx_int_1000) < 0) __PYX_ERR(0, 601, __pyx_L1_error)
  PyType_Modified(__pyx_ptype_9drf_turbo_6fields_DecimalField);

  /* "drf_turbo/fields.pyx":737
 *     """
 *     default_error_messages = {
 *         'invalid': 'Not a valid boolean.'             # <<<<<<<<<<<<<<
 *     }
 *     _initial = False
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 737, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_invalid, __pyx_kp_s_Not_a_valid_boolean) < 0) __PYX_ERR(0, 737, __pyx_L1_error)
  if (PyDict_SetItem((PyObject *)__pyx_ptype_9drf_turbo_6fields_BoolField->tp_dict, __pyx_n_s_default_error_messages, __pyx_t_2) < 0) __PYX_ERR(0, 736, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_9drf_turbo_6fields_BoolField);

  /* "drf_turbo/fields.pyx":739
 *         'invalid': 'Not a valid boolean.'
 *     }
 *     _initial = False             # <<<<<<<<<<<<<<
 *     coerce_values = {
 *         "true": True,
 */
  if (PyDict_SetItem((PyObject *)__pyx_ptype_9drf_turbo_6fields_BoolField->tp_dict, __pyx_n_s_initial_2, Py_False) < 0) __PYX_ERR(0, 739, __pyx_L1_error)
  PyType_Modified(__pyx_ptype_9drf_turbo_6fields_BoolField);

  /* "drf_turbo/fields.pyx":741
 *     _initial = False
 *     coerce_values = {
 *         "true": True,             # <<<<<<<<<<<<<<
 *         "True": True,
 *         "TRUE": True,
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(19); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 741, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_true, Py_True) < 0) __PYX_ERR(0, 741, __pyx_L1_error)

  /* "drf_turbo/fields.pyx":742
 *     coerce_values = {
 *         "true": True,
 *         "True": True,             # <<<<<<<<<<<<<<
 *         "TRUE": True,
 *         't' : True,
 */
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_True, Py_True) < 0) __PYX_ERR(0, 741, __pyx_L1_error)

  /* "drf_turbo/fields.pyx":743
 *         "true": True,
 *         "True": True,
 *         "TRUE": True,             # <<<<<<<<<<<<<<
 *         't' : True,
 *         "T" : True,
 */
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_TRUE, Py_True) < 0) __PYX_ERR(0, 741, __pyx_L1_error)

  /* "drf_turbo/fields.pyx":744
 *         "True": True,
 *         "TRUE": True,
 *         't' : True,             # <<<<<<<<<<<<<<
 *         "T" : True,
 *         "on": True,
 */
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_t, Py_True) < 0) __PYX_ERR(0, 741, __pyx_L1_error)

  /* "drf_turbo/fields.pyx":745
 *         "TRUE": True,
 *         't' : True,
 *         "T" : True,             # <<<<<<<<<<<<<<
 *         "on": True,
 *         "1": True,
 */
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_T, Py_True) < 0) __PYX_ERR(0, 741, __pyx_L1_error)

  /* "drf_turbo/fields.pyx":746
 *         't' : True,
 *         "T" : True,
 *         "on": True,             # <<<<<<<<<<<<<<
 *         "1": True,
 *         True : True,
 */
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_on, Py_True) < 0) __PYX_ERR(0, 741, __pyx_L1_error)

  /* "drf_turbo/fields.pyx":747
 *         "T" : True,
 *         "on": True,
 *         "1": True,             # <<<<<<<<<<<<<<
 *         True : True,
 *         1: True,
 */
  if (PyDict_SetItem(__pyx_t_2, __pyx_kp_s_1_2, Py_True) < 0) __PYX_ERR(0, 741, __pyx_L1_error)

  /* "drf_turbo/fields.pyx":748
 *         "on": True,
 *         "1": True,
 *         True : True,             # <<<<<<<<<<<<<<
 *         1: True,
 *         "off": False,
 */
  if (PyDict_SetItem(__pyx_t_2, Py_True, Py_True) < 0) __PYX_ERR(0, 741, __pyx_L1_error)

  /* "drf_turbo/fields.pyx":749
 *         "1": True,
 *         True : True,
 *         1: True,             # <<<<<<<<<<<<<<
 *         "off": False,
 *         "f" : False,
 */
  if (PyDict_SetItem(__pyx_t_2, __pyx_int_1, Py_True) < 0) __PYX_ERR(0, 741, __pyx_L1_error)

  /* "drf_turbo/fields.pyx":750
 *         True : True,
 *         1: True,
 *         "off": False,             # <<<<<<<<<<<<<<
 *         "f" : False,
 *         "F" : False,
 */
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_off, Py_False) < 0) __PYX_ERR(0, 741, __pyx_L1_error)

  /* "drf_turbo/fields.pyx":751
 *         1: True,
 *         "off": False,
 *         "f" : False,             # <<<<<<<<<<<<<<
 *         "F" : False,
 *         "false": False,
 */
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_f_2, Py_False) < 0) __PYX_ERR(0, 741, __pyx_L1_error)

  /* "drf_turbo/fields.pyx":752
 *         "off": False,
 *         "f" : False,
 *         "F" : False,             # <<<<<<<<<<<<<<
 *         "false": False,
 *         "False":False,
 */
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_F, Py_False) < 0) __PYX_ERR(0, 741, __pyx_L1_error)

  /* "drf_turbo/fields.pyx":753
 *         "f" : False,
 *         "F" : False,
 *         "false": False,             # <<<<<<<<<<<<<<
 *         "False":False,
 *         "FALSE" : False,
 */
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_false, Py_False) < 0) __PYX_ERR(0, 741, __pyx_L1_error)

  /* "drf_turbo/fields.pyx":754
 *         "F" : False,
 *         "false": False,
 *         "False":False,             # <<<<<<<<<<<<<<
 *         "FALSE" : False,
 *         "0": False,
 */
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_False, Py_False) < 0) __PYX_ERR(0, 741, __pyx_L1_error)

  /* "drf_turbo/fields.pyx":755
 *         "false": False,
 *         "False":False,
 *         "FALSE" : False,             # <<<<<<<<<<<<<<
 *         "0": False,
 *         "": False,
 */
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_FALSE, Py_False) < 0) __PYX_ERR(0, 741, __pyx_L1_error)

  /* "drf_turbo/fields.pyx":756
 *         "False":False,
 *         "FALSE" : False,
 *         "0": False,             # <<<<<<<<<<<<<<
 *         "": False,
 *         False : False,
 */
  if (PyDict_SetItem(__pyx_t_2, __pyx_kp_s_0, Py_False) < 0) __PYX_ERR(0, 741, __pyx_L1_error)

  /* "drf_turbo/fields.pyx":757
 *         "FALSE" : False,
 *         "0": False,
 *         "": False,             # <<<<<<<<<<<<<<
 *         False : False,
 *         0: False,
 */
  if (PyDict_SetItem(__pyx_t_2, __pyx_kp_s__9, Py_False) < 0) __PYX_ERR(0, 741, __pyx_L1_error)

  /* "drf_turbo/fields.pyx":758
 *         "0": False,
 *         "": False,
 *         False : False,             # <<<<<<<<<<<<<<
 *         0: False,
 *     }
 */
  if (PyDict_SetItem(__pyx_t_2, Py_False, Py_False) < 0) __PYX_ERR(0, 741, __pyx_L1_error)

  /* "drf_turbo/fields.pyx":759
 *         "": False,
 *         False : False,
 *         0: False,             # <<<<<<<<<<<<<<
 *     }
 *     coerce_null_values = {"", "null","Null","NULL","none","None","NONE"}
 */
  if (PyDict_SetItem(__pyx_t_2, __pyx_int_0, Py_False) < 0) __PYX_ERR(0, 741, __pyx_L1_error)
  if (PyDict_SetItem((PyObject *)__pyx_ptype_9drf_turbo_6fields_BoolField->tp_dict, __pyx_n_s_coerce_values, __pyx_t_2) < 0) __PYX_ERR(0, 740, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_9drf_turbo_6fields_BoolField);

  /* "drf_turbo/fields.pyx":761
 *         0: False,
 *     }
 *     coerce_null_values = {"", "null","Null","NULL","none","None","NONE"}             # <<<<<<<<<<<<<<
 * 
 *     cpdef inline serialize(self,value,dict context):
 */
  __pyx_t_2 = PySet_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 761, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PySet_Add(__pyx_t_2, __pyx_kp_s__9) < 0) __PYX_ERR(0, 761, __pyx_L1_error)
  if (PySet_Add(__pyx_t_2, __pyx_n_s_null) < 0) __PYX_ERR(0, 761, __pyx_L1_error)
  if (PySet_Add(__pyx_t_2, __pyx_n_s_Null) < 0) __PYX_ERR(0, 761, __pyx_L1_error)
  if (PySet_Add(__pyx_t_2, __pyx_n_s_NULL) < 0) __PYX_ERR(0, 761, __pyx_L1_error)
  if (PySet_Add(__pyx_t_2, __pyx_n_s_none) < 0) __PYX_ERR(0, 761, __pyx_L1_error)
  if (PySet_Add(__pyx_t_2, __pyx_n_s_None) < 0) __PYX_ERR(0, 761, __pyx_L1_error)
  if (PySet_Add(__pyx_t_2, __pyx_n_s_NONE) < 0) __PYX_ERR(0, 761, __pyx_L1_error)
  if (PyDict_SetItem((PyObject *)__pyx_ptype_9drf_turbo_6fields_BoolField->tp_dict, __pyx_n_s_coerce_null_values, __pyx_t_2) < 0) __PYX_ERR(0, 761, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_9drf_turbo_6fields_BoolField);

  /* "drf_turbo/fields.pyx":789
 *     """
 *     default_error_messages = {
 *         'invalid_choice': '"{input}" is not a valid choice.'             # <<<<<<<<<<<<<<
 *     }
 * 
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 789, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_invalid_choice, __pyx_kp_s_input_is_not_a_valid_choice) < 0) __PYX_ERR(0, 789, __pyx_L1_error)
  if (PyDict_SetItem((PyObject *)__pyx_ptype_9drf_turbo_6fields_ChoiceField->tp_dict, __pyx_n_s_default_error_messages, __pyx_t_2) < 0) __PYX_ERR(0, 788, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_9drf_turbo_6fields_ChoiceField);

  /* "drf_turbo/fields.pyx":845
 *     """
 *     default_error_messages = {
 *         'not_a_list': 'Expected a list of items but got type "{input_type}".',             # <<<<<<<<<<<<<<
 *         'empty': 'This selection may not be empty.',
 *         'invalid_choice': '"{input}" is not a valid choice.'
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 845, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_not_a_list, __pyx_kp_s_Expected_a_list_of_items_but_got) < 0) __PYX_ERR(0, 845, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_empty, __pyx_kp_s_This_selection_may_not_be_empty) < 0) __PYX_ERR(0, 845, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_invalid_choice, __pyx_kp_s_input_is_not_a_valid_choice) < 0) __PYX_ERR(0, 845, __pyx_L1_error)
  if (PyDict_SetItem((PyObject *)__pyx_ptype_9drf_turbo_6fields_MultipleChoiceField->tp_dict, __pyx_n_s_default_error_messages, __pyx_t_2) < 0) __PYX_ERR(0, 844, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_9drf_turbo_6fields_MultipleChoiceField);

  /* "drf_turbo/fields.pyx":890
 *     """
 *     default_error_messages = {
 *         'invalid': 'Not a valid datetime.',             # <<<<<<<<<<<<<<
 *         'date': 'Expected a datetime but got a date.',
 *         'make_aware':'Invalid datetime for the timezone "{timezone}".',
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 890, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_invalid, __pyx_kp_s_Not_a_valid_datetime) < 0) __PYX_ERR(0, 890, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_date, __pyx_kp_s_Expected_a_datetime_but_got_a_da) < 0) __PYX_ERR(0, 890, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_make_aware, __pyx_kp_s_Invalid_datetime_for_the_timezon) < 0) __PYX_ERR(0, 890, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_overflow, __pyx_kp_s_Datetime_value_out_of_range) < 0) __PYX_ERR(0, 890, __pyx_L1_error)
  if (PyDict_SetItem((PyObject *)__pyx_ptype_9drf_turbo_6fields_DateTimeField->tp_dict, __pyx_n_s_default_error_messages, __pyx_t_2) < 0) __PYX_ERR(0, 889, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_9drf_turbo_6fields_DateTimeField);

  /* "drf_turbo/fields.pyx":895
 *         'overflow': 'Datetime value out of range.'
 *     }
 *     datetime_parser = datetime.datetime.strptime             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, format=NO_DEFAULT,input_formats=None,default_timezone=None,**kwargs):
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_datetime); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 895, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_datetime); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 895, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_strptime); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 895, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_9drf_turbo_6fields_DateTimeField->tp_dict, __pyx_n_s_datetime_parser, __pyx_t_2) < 0) __PYX_ERR(0, 895, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_9drf_turbo_6fields_DateTimeField);

  /* "drf_turbo/fields.pyx":897
 *     datetime_parser = datetime.datetime.strptime
 * 
 *     def __init__(self, format=NO_DEFAULT,input_formats=None,default_timezone=None,**kwargs):             # <<<<<<<<<<<<<<
 *         if format is NO_DEFAULT:
 *             self.format = api_settings.DATETIME_FORMAT
 */
  __Pyx_INCREF(__pyx_v_9drf_turbo_6fields_NO_DEFAULT);
  __pyx_k__20 = __pyx_v_9drf_turbo_6fields_NO_DEFAULT;
  __Pyx_GIVEREF(__pyx_v_9drf_turbo_6fields_NO_DEFAULT);

  /* "drf_turbo/fields.pyx":994
 * 
 *     default_error_messages = {
 *         'invalid': 'Not a valid date.',             # <<<<<<<<<<<<<<
 *         'datetime': 'Expected a date but got a datetime.',
 *     }
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 994, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_invalid, __pyx_kp_s_Not_a_valid_date) < 0) __PYX_ERR(0, 994, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_datetime, __pyx_kp_s_Expected_a_date_but_got_a_dateti_2) < 0) __PYX_ERR(0, 994, __pyx_L1_error)
  if (PyDict_SetItem((PyObject *)__pyx_ptype_9drf_turbo_6fields_DateField->tp_dict, __pyx_n_s_default_error_messages, __pyx_t_2) < 0) __PYX_ERR(0, 993, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_9drf_turbo_6fields_DateField);

  /* "drf_turbo/fields.pyx":1068
 *     """
 *     default_error_messages = {
 *         'invalid': 'Not a valid time.',             # <<<<<<<<<<<<<<
 *     }
 *     datetime_parser = datetime.datetime.strptime
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1068, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_invalid, __pyx_kp_s_Not_a_valid_time) < 0) __PYX_ERR(0, 1068, __pyx_L1_error)
  if (PyDict_SetItem((PyObject *)__pyx_ptype_9drf_turbo_6fields_TimeField->tp_dict, __pyx_n_s_default_error_messages, __pyx_t_2) < 0) __PYX_ERR(0, 1067, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_9drf_turbo_6fields_TimeField);

  /* "drf_turbo/fields.pyx":1070
 *         'invalid': 'Not a valid time.',
 *     }
 *     datetime_parser = datetime.datetime.strptime             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, format=NO_DEFAULT,input_formats=None, **kwargs):
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_datetime); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1070, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_datetime); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1070, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_strptime); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1070, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_9drf_turbo_6fields_TimeField->tp_dict, __pyx_n_s_datetime_parser, __pyx_t_2) < 0) __PYX_ERR(0, 1070, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_9drf_turbo_6fields_TimeField);

  /* "drf_turbo/fields.pyx":1072
 *     datetime_parser = datetime.datetime.strptime
 * 
 *     def __init__(self, format=NO_DEFAULT,input_formats=None, **kwargs):             # <<<<<<<<<<<<<<
 *         if format is NO_DEFAULT:
 *             self.format = api_settings.DATETIME_FORMAT
 */
  __Pyx_INCREF(__pyx_v_9drf_turbo_6fields_NO_DEFAULT);
  __pyx_k__23 = __pyx_v_9drf_turbo_6fields_NO_DEFAULT;
  __Pyx_GIVEREF(__pyx_v_9drf_turbo_6fields_NO_DEFAULT);

  /* "drf_turbo/fields.pyx":1141
 *     """
 *     default_error_messages = {
 *         'required': 'No file was submitted.',             # <<<<<<<<<<<<<<
 *         'invalid': 'The submitted data was not a file. Check the encoding type on the form.',
 *         'no_name': 'No filename could be determined.',
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1141, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_required, __pyx_kp_s_No_file_was_submitted) < 0) __PYX_ERR(0, 1141, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_invalid, __pyx_kp_s_The_submitted_data_was_not_a_fil) < 0) __PYX_ERR(0, 1141, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_no_name, __pyx_kp_s_No_filename_could_be_determined) < 0) __PYX_ERR(0, 1141, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_empty, __pyx_kp_s_The_submitted_file_is_empty) < 0) __PYX_ERR(0, 1141, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_max_length, __pyx_kp_s_Ensure_this_filename_has_at_most) < 0) __PYX_ERR(0, 1141, __pyx_L1_error)
  if (PyDict_SetItem((PyObject *)__pyx_ptype_9drf_turbo_6fields_FileField->tp_dict, __pyx_n_s_default_error_messages, __pyx_t_2) < 0) __PYX_ERR(0, 1140, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_9drf_turbo_6fields_FileField);

  /* "drf_turbo/fields.pyx":1193
 * 
 *     default_error_messages = {
 *         'not_a_list': 'Expected a list of items but got type "{input_type}".',             # <<<<<<<<<<<<<<
 *         'empty': 'This list may not be empty.',
 *         'exact_items': 'Must have {exact_items} items.',
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1193, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_not_a_list, __pyx_kp_s_Expected_a_list_of_items_but_got) < 0) __PYX_ERR(0, 1193, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_empty, __pyx_kp_s_This_list_may_not_be_empty) < 0) __PYX_ERR(0, 1193, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_exact_items, __pyx_kp_s_Must_have_exact_items_items) < 0) __PYX_ERR(0, 1193, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_min_items, __pyx_kp_s_Must_have_at_least_min_items_ite) < 0) __PYX_ERR(0, 1193, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_max_items, __pyx_kp_s_Must_have_no_more_than_max_items) < 0) __PYX_ERR(0, 1193, __pyx_L1_error)
  if (PyDict_SetItem((PyObject *)__pyx_ptype_9drf_turbo_6fields_ArrayField->tp_dict, __pyx_n_s_default_error_messages, __pyx_t_2) < 0) __PYX_ERR(0, 1192, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_9drf_turbo_6fields_ArrayField);

  /* "drf_turbo/fields.pyx":1200
 * 
 *     }
 *     _initial = []             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self,child=None,**kwargs):
 */
  __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1200, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_9drf_turbo_6fields_ArrayField->tp_dict, __pyx_n_s_initial_2, __pyx_t_2) < 0) __PYX_ERR(0, 1200, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_9drf_turbo_6fields_ArrayField);

  /* "drf_turbo/fields.pyx":1267
 *     """
 *     default_error_messages = {
 *         'not_a_dict': 'Expected a dict of items but got type "{input_type}".',             # <<<<<<<<<<<<<<
 *         'empty': 'This dict may not be empty.',
 *     }
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1267, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_not_a_dict, __pyx_kp_s_Expected_a_dict_of_items_but_got) < 0) __PYX_ERR(0, 1267, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_empty, __pyx_kp_s_This_dict_may_not_be_empty) < 0) __PYX_ERR(0, 1267, __pyx_L1_error)
  if (PyDict_SetItem((PyObject *)__pyx_ptype_9drf_turbo_6fields_DictField->tp_dict, __pyx_n_s_default_error_messages, __pyx_t_2) < 0) __PYX_ERR(0, 1266, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_9drf_turbo_6fields_DictField);

  /* "drf_turbo/fields.pyx":1270
 *         'empty': 'This dict may not be empty.',
 *     }
 *     _initial = {}             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self,child=None,**kwargs):
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1270, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_9drf_turbo_6fields_DictField->tp_dict, __pyx_n_s_initial_2, __pyx_t_2) < 0) __PYX_ERR(0, 1270, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_9drf_turbo_6fields_DictField);

  /* "drf_turbo/fields.pyx":1321
 *     """
 *     default_error_messages = {
 *         'invalid': 'Not a valid JSON.'             # <<<<<<<<<<<<<<
 *     }
 * 
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1321, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_invalid, __pyx_kp_s_Not_a_valid_JSON) < 0) __PYX_ERR(0, 1321, __pyx_L1_error)
  if (PyDict_SetItem((PyObject *)__pyx_ptype_9drf_turbo_6fields_JSONField->tp_dict, __pyx_n_s_default_error_messages, __pyx_t_2) < 0) __PYX_ERR(0, 1320, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_9drf_turbo_6fields_JSONField);

  /* "drf_turbo/fields.pyx":1356
 *     """
 *     default_error_messages = {
 *         'does_not_exist': 'Invalid pk "{pk_value}" - object does not exist.',             # <<<<<<<<<<<<<<
 *         'incorrect_type': 'Incorrect type. Expected pk value, received {data_type}.',
 * 
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1356, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_does_not_exist, __pyx_kp_s_Invalid_pk_pk_value_object_does) < 0) __PYX_ERR(0, 1356, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_incorrect_type, __pyx_kp_s_Incorrect_type_Expected_pk_value) < 0) __PYX_ERR(0, 1356, __pyx_L1_error)
  if (PyDict_SetItem((PyObject *)__pyx_ptype_9drf_turbo_6fields_RelatedField->tp_dict, __pyx_n_s_default_error_messages, __pyx_t_2) < 0) __PYX_ERR(0, 1355, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_9drf_turbo_6fields_RelatedField);

  /* "drf_turbo/fields.pyx":1386
 *     """
 *     default_error_messages = {
 *         'not_a_list': 'Expected a list of items but got type "{input_type}".',             # <<<<<<<<<<<<<<
 *         'empty': 'This list may not be empty.'
 *     }
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1386, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_not_a_list, __pyx_kp_s_Expected_a_list_of_items_but_got) < 0) __PYX_ERR(0, 1386, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_empty, __pyx_kp_s_This_list_may_not_be_empty) < 0) __PYX_ERR(0, 1386, __pyx_L1_error)
  if (PyDict_SetItem((PyObject *)__pyx_ptype_9drf_turbo_6fields_ManyRelatedField->tp_dict, __pyx_n_s_default_error_messages, __pyx_t_2) < 0) __PYX_ERR(0, 1385, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_9drf_turbo_6fields_ManyRelatedField);

  /* "drf_turbo/fields.pyx":1421
 *     """
 *     default_error_messages = {
 *         'constant': 'Must be "{constant}".',             # <<<<<<<<<<<<<<
 *         'None': 'Must be None.'
 *     }
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1421, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_constant, __pyx_kp_s_Must_be_constant) < 0) __PYX_ERR(0, 1421, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_None, __pyx_kp_s_Must_be_None) < 0) __PYX_ERR(0, 1421, __pyx_L1_error)
  if (PyDict_SetItem((PyObject *)__pyx_ptype_9drf_turbo_6fields_ConstantField->tp_dict, __pyx_n_s_default_error_messages, __pyx_t_2) < 0) __PYX_ERR(0, 1420, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_9drf_turbo_6fields_ConstantField);

  /* "drf_turbo/fields.pyx":1479
 *     :param kwargs: The same keyword arguments that :class:`Field` receives.
 *     """
 *     is_method_field = True             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self,method_name=None,**kwargs):
 */
  if (PyDict_SetItem((PyObject *)__pyx_ptype_9drf_turbo_6fields_MethodField->tp_dict, __pyx_n_s_is_method_field, Py_True) < 0) __PYX_ERR(0, 1479, __pyx_L1_error)
  PyType_Modified(__pyx_ptype_9drf_turbo_6fields_MethodField);

  /* "(tree fragment)":1
 * def __pyx_unpickle_SkipField(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_9drf_turbo_6fields_1__pyx_unpickle_SkipField, NULL, __pyx_n_s_drf_turbo_fields); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_SkipField, __pyx_t_2) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":11
 *         __pyx_unpickle_SkipField__set_state(<SkipField> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_SkipField__set_state(SkipField __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 0 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[0])
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_9drf_turbo_6fields_3__pyx_unpickle_Field, NULL, __pyx_n_s_drf_turbo_fields); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_Field, __pyx_t_2) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_StrField(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_9drf_turbo_6fields_5__pyx_unpickle_StrField, NULL, __pyx_n_s_drf_turbo_fields); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_StrField, __pyx_t_2) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":11
 *         __pyx_unpickle_StrField__set_state(<StrField> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_StrField__set_state(StrField __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.allow_blank = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.max_length = __pyx_state[11]; __pyx_result.min_length = __pyx_state[12]; __pyx_result.read_only = __pyx_state[13]; __pyx_result.required = __pyx_state[14]; __pyx_result.root = __pyx_state[15]; __pyx_result.style = __pyx_state[16]; __pyx_result.trim_whitespace = __pyx_state[17]; __pyx_result.validators = __pyx_state[18]; __pyx_result.write_only = __pyx_state[19]
 *     if len(__pyx_state) > 20 and hasattr(__pyx_result, '__dict__'):
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_9drf_turbo_6fields_7__pyx_unpickle_EmailField, NULL, __pyx_n_s_drf_turbo_fields); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_EmailField, __pyx_t_2) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_URLField(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_9drf_turbo_6fields_9__pyx_unpickle_URLField, NULL, __pyx_n_s_drf_turbo_fields); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_URLField, __pyx_t_2) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":11
 *         __pyx_unpickle_URLField__set_state(<URLField> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_URLField__set_state(URLField __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.allow_blank = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.max_length = __pyx_state[11]; __pyx_result.min_length = __pyx_state[12]; __pyx_result.read_only = __pyx_state[13]; __pyx_result.required = __pyx_state[14]; __pyx_result.root = __pyx_state[15]; __pyx_result.style = __pyx_state[16]; __pyx_result.trim_whitespace = __pyx_state[17]; __pyx_result.validators = __pyx_state[18]; __pyx_result.write_only = __pyx_state[19]
 *     if len(__pyx_state) > 20 and hasattr(__pyx_result, '__dict__'):
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_9drf_turbo_6fields_11__pyx_unpickle_RegexField, NULL, __pyx_n_s_drf_turbo_fields); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_RegexField, __pyx_t_2) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_IPField(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_9drf_turbo_6fields_13__pyx_unpickle_IPField, NULL, __pyx_n_s_drf_turbo_fields); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_IPField, __pyx_t_2) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":11
 *         __pyx_unpickle_IPField__set_state(<IPField> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_IPField__set_state(IPField __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.allow_blank = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.max_length = __pyx_state[11]; __pyx_result.min_length = __pyx_state[12]; __pyx_result.read_only = __pyx_state[13]; __pyx_result.required = __pyx_state[14]; __pyx_result.root = __pyx_state[15]; __pyx_result.style = __pyx_state[16]; __pyx_result.trim_whitespace = __pyx_state[17]; __pyx_result.validators = __pyx_state[18]; __pyx_result.write_only = __pyx_state[19]
 *     if len(__pyx_state) > 20 and hasattr(__pyx_result, '__dict__'):
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_9drf_turbo_6fields_15__pyx_unpickle_PasswordField, NULL, __pyx_n_s_drf_turbo_fields); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_PasswordField, __pyx_t_2) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_UUIDField(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_9drf_turbo_6fields_17__pyx_unpickle_UUIDField, NULL, __pyx_n_s_drf_turbo_fields); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_UUIDField, __pyx_t_2) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":11
 *         __pyx_unpickle_UUIDField__set_state(<UUIDField> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_UUIDField__set_state(UUIDField __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.allow_blank = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.label = __pyx_state[10]; __pyx_result.max_length = __pyx_state[11]; __pyx_result.min_length = __pyx_state[12]; __pyx_result.read_only = __pyx_state[13]; __pyx_result.required = __pyx_state[14]; __pyx_result.root = __pyx_state[15]; __pyx_result.style = __pyx_state[16]; __pyx_result.trim_whitespace = __pyx_state[17]; __pyx_result.validators = __pyx_state[18]; __pyx_result.write_only = __pyx_state[19]
 *     if len(__pyx_state) > 20 and hasattr(__pyx_result, '__dict__'):
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_9drf_turbo_6fields_19__pyx_unpickle_SlugField, NULL, __pyx_n_s_drf_turbo_fields); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_SlugField, __pyx_t_2) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_IntField(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_9drf_turbo_6fields_21__pyx_unpickle_IntField, NULL, __pyx_n_s_drf_turbo_fields); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_IntField, __pyx_t_2) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":11
 *         __pyx_unpickle_IntField__set_state(<IntField> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_IntField__set_state(IntField __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.default_value = __pyx_state[4]; __pyx_result.error_messages = __pyx_state[5]; __pyx_result.field_name = __pyx_state[6]; __pyx_result.help_text = __pyx_state[7]; __pyx_result.initial = __pyx_state[8]; __pyx_result.label = __pyx_state[9]; __pyx_result.max_value = __pyx_state[10]; __pyx_result.min_value = __pyx_state[11]; __pyx_result.read_only = __pyx_state[12]; __pyx_result.required = __pyx_state[13]; __pyx_result.root = __pyx_state[14]; __pyx_result.style = __pyx_state[15]; __pyx_result.validators = __pyx_state[16]; __pyx_result.write_only = __pyx_state[17]
 *     if len(__pyx_state) > 18 and hasattr(__pyx_result, '__dict__'):
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_9drf_turbo_6fields_23__pyx_unpickle_FloatField, NULL, __pyx_n_s_drf_turbo_fields); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_FloatField, __pyx_t_2) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_DecimalField(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_9drf_turbo_6fields_25__pyx_unpickle_DecimalField, NULL, __pyx_n_s_drf_turbo_fields); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_DecimalField, __pyx_t_2) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":11
 *         __pyx_unpickle_DecimalField__set_state(<DecimalField> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_DecimalField__set_state(DecimalField __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.coerce_to_string = __pyx_state[4]; __pyx_result.decimal_places = __pyx_state[5]; __pyx_result.default_value = __pyx_state[6]; __pyx_result.error_messages = __pyx_state[7]; __pyx_result.field_name = __pyx_state[8]; __pyx_result.help_text = __pyx_state[9]; __pyx_result.initial = __pyx_state[10]; __pyx_result.label = __pyx_state[11]; __pyx_result.max_digits = __pyx_state[12]; __pyx_result.max_value = __pyx_state[13]; __pyx_result.max_whole_digits = __pyx_state[14]; __pyx_result.min_value = __pyx_state[15]; __pyx_result.read_only = __pyx_state[16]; __pyx_result.required = __pyx_state[17]; __pyx_result.root = __pyx_state[18]; __pyx_result.rounding = __pyx_state[19]; __pyx_result.style = __pyx_state[20]; __pyx_result.validators = __pyx_state[21]; __pyx_result.write_only = __pyx_state[22]
 *     if len(__pyx_state) > 23 and hasattr(__pyx_result, '__dict__'):
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_9drf_turbo_6fields_27__pyx_unpickle_BoolField, NULL, __pyx_n_s_drf_turbo_fields); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_BoolField, __pyx_t_2) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_ChoiceField(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_9drf_turbo_6fields_29__pyx_unpickle_ChoiceField, NULL, __pyx_n_s_drf_turbo_fields); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_ChoiceField, __pyx_t_2) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":11
 *         __pyx_unpickle_ChoiceField__set_state(<ChoiceField> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_ChoiceField__set_state(ChoiceField __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.allow_blank = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.choice_strings_to_display = __pyx_state[5]; __pyx_result.choice_strings_to_values = __pyx_state[6]; __pyx_result.choices = __pyx_state[7]; __pyx_result.default_value = __pyx_state[8]; __pyx_result.error_messages = __pyx_state[9]; __pyx_result.field_name = __pyx_state[10]; __pyx_result.help_text = __pyx_state[11]; __pyx_result.initial = __pyx_state[12]; __pyx_result.label = __pyx_state[13]; __pyx_result.read_only = __pyx_state[14]; __pyx_result.required = __pyx_state[15]; __pyx_result.root = __pyx_state[16]; __pyx_result.style = __pyx_state[17]; __pyx_result.validators = __pyx_state[18]; __pyx_result.write_only = __pyx_state[19]
 *     if len(__pyx_state) > 20 and hasattr(__pyx_result, '__dict__'):
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_9drf_turbo_6fields_31__pyx_unpickle_MultipleChoiceField, NULL, __pyx_n_s_drf_turbo_fields); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_MultipleChoiceFie, __pyx_t_2) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_DateTimeField(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_9drf_turbo_6fields_33__pyx_unpickle_DateTimeField, NULL, __pyx_n_s_drf_turbo_fields); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_DateTimeField, __pyx_t_2) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":11
 *         __pyx_unpickle_DateTimeField__set_state(<DateTimeField> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_DateTimeField__set_state(DateTimeField __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.default_timezone = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.field_name = __pyx_state[7]; __pyx_result.format = __pyx_state[8]; __pyx_result.help_text = __pyx_state[9]; __pyx_result.initial = __pyx_state[10]; __pyx_result.input_formats = __pyx_state[11]; __pyx_result.label = __pyx_state[12]; __pyx_result.read_only = __pyx_state[13]; __pyx_result.required = __pyx_state[14]; __pyx_result.root = __pyx_state[15]; __pyx_result.style = __pyx_state[16]; __pyx_result.timezone = __pyx_state[17]; __pyx_result.validators = __pyx_state[18]; __pyx_result.write_only = __pyx_state[19]
 *     if len(__pyx_state) > 20 and hasattr(__pyx_result, '__dict__'):
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_9drf_turbo_6fields_35__pyx_unpickle_DateField, NULL, __pyx_n_s_drf_turbo_fields); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_DateField, __pyx_t_2) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_TimeField(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_9drf_turbo_6fields_37__pyx_unpickle_TimeField, NULL, __pyx_n_s_drf_turbo_fields); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_TimeField, __pyx_t_2) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":11
 *         __pyx_unpickle_TimeField__set_state(<TimeField> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_TimeField__set_state(TimeField __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.default_value = __pyx_state[4]; __pyx_result.error_messages = __pyx_state[5]; __pyx_result.field_name = __pyx_state[6]; __pyx_result.format = __pyx_state[7]; __pyx_result.help_text = __pyx_state[8]; __pyx_result.initial = __pyx_state[9]; __pyx_result.input_formats = __pyx_state[10]; __pyx_result.label = __pyx_state[11]; __pyx_result.read_only = __pyx_state[12]; __pyx_result.required = __pyx_state[13]; __pyx_result.root = __pyx_state[14]; __pyx_result.style = __pyx_state[15]; __pyx_result.validators = __pyx_state[16]; __pyx_result.write_only = __pyx_state[17]
 *     if len(__pyx_state) > 18 and hasattr(__pyx_result, '__dict__'):
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_9drf_turbo_6fields_39__pyx_unpickle_FileField, NULL, __pyx_n_s_drf_turbo_fields); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_FileField, __pyx_t_2) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_ArrayField(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_9drf_turbo_6fields_41__pyx_unpickle_ArrayField, NULL, __pyx_n_s_drf_turbo_fields); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_ArrayField, __pyx_t_2) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":11
 *         __pyx_unpickle_ArrayField__set_state(<ArrayField> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_ArrayField__set_state(ArrayField __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.allow_empty = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.child = __pyx_state[5]; __pyx_result.default_value = __pyx_state[6]; __pyx_result.error_messages = __pyx_state[7]; __pyx_result.exact_items = __pyx_state[8]; __pyx_result.field_name = __pyx_state[9]; __pyx_result.help_text = __pyx_state[10]; __pyx_result.initial = __pyx_state[11]; __pyx_result.label = __pyx_state[12]; __pyx_result.max_items = __pyx_state[13]; __pyx_result.min_items = __pyx_state[14]; __pyx_result.read_only = __pyx_state[15]; __pyx_result.required = __pyx_state[16]; __pyx_result.root = __pyx_state[17]; __pyx_result.style = __pyx_state[18]; __pyx_result.validators = __pyx_state[19]; __pyx_result.write_only = __pyx_state[20]
 *     if len(__pyx_state) > 21 and hasattr(__pyx_result, '__dict__'):
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_9drf_turbo_6fields_43__pyx_unpickle_DictField, NULL, __pyx_n_s_drf_turbo_fields); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_DictField, __pyx_t_2) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_JSONField(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_9drf_turbo_6fields_45__pyx_unpickle_JSONField, NULL, __pyx_n_s_drf_turbo_fields); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_JSONField, __pyx_t_2) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":11
 *         __pyx_unpickle_JSONField__set_state(<JSONField> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_JSONField__set_state(JSONField __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.binary = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.decoder = __pyx_state[5]; __pyx_result.default_value = __pyx_state[6]; __pyx_result.encoder = __pyx_state[7]; __pyx_result.error_messages = __pyx_state[8]; __pyx_result.field_name = __pyx_state[9]; __pyx_result.help_text = __pyx_state[10]; __pyx_result.initial = __pyx_state[11]; __pyx_result.label = __pyx_state[12]; __pyx_result.read_only = __pyx_state[13]; __pyx_result.required = __pyx_state[14]; __pyx_result.root = __pyx_state[15]; __pyx_result.style = __pyx_state[16]; __pyx_result.validators = __pyx_state[17]; __pyx_result.write_only = __pyx_state[18]
 *     if len(__pyx_state) > 19 and hasattr(__pyx_result, '__dict__'):
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_9drf_turbo_6fields_47__pyx_unpickle_RelatedField, NULL, __pyx_n_s_drf_turbo_fields); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_RelatedField, __pyx_t_2) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_ManyRelatedField(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_9drf_turbo_6fields_49__pyx_unpickle_ManyRelatedField, NULL, __pyx_n_s_drf_turbo_fields); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_ManyRelatedField, __pyx_t_2) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":11
 *         __pyx_unpickle_ManyRelatedField__set_state(<ManyRelatedField> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_ManyRelatedField__set_state(ManyRelatedField __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.allow_empty = __pyx_state[0]; __pyx_result.allow_null = __pyx_state[1]; __pyx_result.attr = __pyx_state[2]; __pyx_result.attrs = __pyx_state[3]; __pyx_result.call = __pyx_state[4]; __pyx_result.child_relation = __pyx_state[5]; __pyx_result.default_value = __pyx_state[6]; __pyx_result.error_messages = __pyx_state[7]; __pyx_result.field_name = __pyx_state[8]; __pyx_result.help_text = __pyx_state[9]; __pyx_result.initial = __pyx_state[10]; __pyx_result.label = __pyx_state[11]; __pyx_result.read_only = __pyx_state[12]; __pyx_result.required = __pyx_state[13]; __pyx_result.root = __pyx_state[14]; __pyx_result.style = __pyx_state[15]; __pyx_result.validators = __pyx_state[16]; __pyx_result.write_only = __pyx_state[17]
 *     if len(__pyx_state) > 18 and hasattr(__pyx_result, '__dict__'):
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_9drf_turbo_6fields_51__pyx_unpickle_ConstantField, NULL, __pyx_n_s_drf_turbo_fields); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_ConstantField, __pyx_t_2) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_RecursiveField(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_9drf_turbo_6fields_53__pyx_unpickle_RecursiveField, NULL, __pyx_n_s_drf_turbo_fields); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_RecursiveField, __pyx_t_2) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":11
 *         __pyx_unpickle_RecursiveField__set_state(<RecursiveField> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_RecursiveField__set_state(RecursiveField __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.allow_null = __pyx_state[0]; __pyx_result.attr = __pyx_state[1]; __pyx_result.attrs = __pyx_state[2]; __pyx_result.call = __pyx_state[3]; __pyx_result.context = __pyx_state[4]; __pyx_result.default_value = __pyx_state[5]; __pyx_result.error_messages = __pyx_state[6]; __pyx_result.exclude = __pyx_state[7]; __pyx_result.field_name = __pyx_state[8]; __pyx_result.help_text = __pyx_state[9]; __pyx_result.initial = __pyx_state[10]; __pyx_result.label = __pyx_state[11]; __pyx_result.many = __pyx_state[12]; __pyx_result.only = __pyx_state[13]; __pyx_result.read_only = __pyx_state[14]; __pyx_result.required = __pyx_state[15]; __pyx_result.root = __pyx_state[16]; __pyx_result.style = __pyx_state[17]; __pyx_result.validators = __pyx_state[18]; __pyx_result.write_only = __pyx_state[19]
 *     if len(__pyx_state) > 20 and hasattr(__pyx_result, '__dict__'):
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_9drf_turbo_6fields_55__pyx_unpickle_MethodField, NULL, __pyx_n_s_drf_turbo_fields); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_MethodField, __pyx_t_2) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "drf_turbo/fields.pyx":1
 * cimport cython             # <<<<<<<<<<<<<<
 * 
 * from drf_turbo.utils import is_iterable_and_not_string,get_error_detail,is_collection,get_attribute,force_str
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_2) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init drf_turbo.fields", __pyx_clineno, __pyx_lineno, __pyx_filename);
    }
    Py_CLEAR(__pyx_m);
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init drf_turbo.fields");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #elif PY_MAJOR_VERSION >= 3
  return __pyx_m;
  #else
  return;
  #endif
}

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule(modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, "RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* PyObjectGetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#endif

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

/* PyErrExceptionMatches */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx_PyErr_ExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        if (__Pyx_PyErr_GivenExceptionMatches(exc_type, PyTuple_GET_ITEM(tuple, i))) return 1;
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err) {
    PyObject *exc_type = tstate->curexc_type;
    if (exc_type == err) return 1;
    if (unlikely(!exc_type)) return 0;
    if (unlikely(PyTuple_Check(err)))
        return __Pyx_PyErr_ExceptionMatchesTuple(exc_type, err);
    return __Pyx_PyErr_GivenExceptionMatches(exc_type, err);
}
#endif

/* PyErrFetchRestore */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
}
#endif

/* GetAttr */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *o, PyObject *n) {
#if CYTHON_USE_TYPE_SLOTS
#if PY_MAJOR_VERSION >= 3
    if (likely(PyUnicode_Check(n)))
#else
    if (likely(PyString_Check(n)))
#endif
        return __Pyx_PyObject_GetAttrStr(o, n);
#endif
    return PyObject_GetAttr(o, n);
}

/* GetAttr3 */
static PyObject *__Pyx_GetAttr3Default(PyObject *d) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (unlikely(!__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        return NULL;
    __Pyx_PyErr_Clear();
    Py_INCREF(d);
    return d;
}
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *o, PyObject *n, PyObject *d) {
    PyObject *r = __Pyx_GetAttr(o, n);
    return (likely(r)) ? r : __Pyx_GetAttr3Default(d);
}

/* PyDictVersioning */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    return likely(dict) ? __PYX_GET_DICT_VERSION(dict) : 0;
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj) {
    PyObject **dictptr = NULL;
    Py_ssize_t offset = Py_TYPE(obj)->tp_dictoffset;
    if (offset) {
#if CYTHON_COMPILING_IN_CPYTHON
        dictptr = (likely(offset > 0)) ? (PyObject **) ((char *)obj + offset) : _PyObject_GetDictPtr(obj);
#else
        dictptr = _PyObject_GetDictPtr(obj);
#endif
    }
    return (dictptr && *dictptr) ? __PYX_GET_DICT_VERSION(*dictptr) : 0;
}
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    if (unlikely(!dict) || unlikely(tp_dict_version != __PYX_GET_DICT_VERSION(dict)))
        return 0;
    return obj_dict_version == __Pyx_get_object_dict_version(obj);
}
#endif

/* GetModuleGlobalName */
#if CYTHON_USE_DICT_VERSIONS
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value)
#else
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name)
#endif
{
    PyObject *result;
#if !CYTHON_AVOID_BORROWED_REFS
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1
    result = _PyDict_GetItem_KnownHash(__pyx_d, name, ((PyASCIIObject *) name)->hash);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    } else if (unlikely(PyErr_Occurred())) {
        return NULL;
    }
#else
    result = PyDict_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
#endif
#else
    result = PyObject_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
    PyErr_Clear();
#endif
    return __Pyx_GetBuiltinName(name);
}

/* RaiseDoubleKeywords */
static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

/* ParseKeywords */
static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (__Pyx_PyUnicode_GET_LENGTH(**name) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (__Pyx_PyUnicode_GET_LENGTH(**argname) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

/* RaiseArgTupleInvalid */
static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* ArgTypeTest */
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact)
{
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    else if (exact) {
        #if PY_MAJOR_VERSION == 2
        if ((type == &PyBaseString_Type) && likely(__Pyx_PyBaseString_CheckExact(obj))) return 1;
        #endif
    }
    else {
        if (likely(__Pyx_TypeCheck(obj, type))) return 1;
    }
    PyErr_Format(PyExc_TypeError,
        "Argument '%.200s' has incorrect type (expected %.200s, got %.200s)",
        name, type->tp_name, Py_TYPE(obj)->tp_name);
    return 0;
}

/* PyCFunctionFastCall */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject * __Pyx_PyCFunction_FastCall(PyObject *func_obj, PyObject **args, Py_ssize_t nargs) {
    PyCFunctionObject *func = (PyCFunctionObject*)func_obj;
    PyCFunction meth = PyCFunction_GET_FUNCTION(func);
    PyObject *self = PyCFunction_GET_SELF(func);
    int flags = PyCFunction_GET_FLAGS(func);
    assert(PyCFunction_Check(func));
    assert(METH_FASTCALL == (flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)));
    assert(nargs >= 0);
    assert(nargs == 0 || args != NULL);
    /* _PyCFunction_FastCallDict() must not be called with an exception set,
       because it may clear it (directly or indirectly) and so the
       caller loses its exception */
    assert(!PyErr_Occurred());
    if ((PY_VERSION_HEX < 0x030700A0) || unlikely(flags & METH_KEYWORDS)) {
        return (*((__Pyx_PyCFunctionFastWithKeywords)(void*)meth)) (self, args, nargs, NULL);
    } else {
        return (*((__Pyx_PyCFunctionFast)(void*)meth)) (self, args, nargs);
    }
}
#endif

/* PyFunctionFastCall */
#if CYTHON_FAST_PYCALL
static PyObject* __Pyx_PyFunction_FastCallNoKw(PyCodeObject *co, PyObject **args, Py_ssize_t na,
                                               PyObject *globals) {
    PyFrameObject *f;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject **fastlocals;
    Py_ssize_t i;
    PyObject *result;
    assert(globals != NULL);
    /* XXX Perhaps we should create a specialized
       PyFrame_New() that doesn't take locals, but does
       take builtins without sanity checking them.
       */
    assert(tstate != NULL);
    f = PyFrame_New(tstate, co, globals, NULL);
    if (f == NULL) {
        return NULL;
    }
    fastlocals = __Pyx_PyFrame_GetLocalsplus(f);
    for (i = 0; i < na; i++) {
        Py_INCREF(*args);
        fastlocals[i] = *args++;
    }
    result = PyEval_EvalFrameEx(f,0);
    ++tstate->recursion_depth;
    Py_DECREF(f);
    --tstate->recursion_depth;
    return result;
}
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs) {
    PyCodeObject *co = (PyCodeObject *)PyFunction_GET_CODE(func);
    PyObject *globals = PyFunction_GET_GLOBALS(func);
    PyObject *argdefs = PyFunction_GET_DEFAULTS(func);
    PyObject *closure;
#if PY_MAJOR_VERSION >= 3
    PyObject *kwdefs;
#endif
    PyObject *kwtuple, **k;
    PyObject **d;
    Py_ssize_t nd;
    Py_ssize_t nk;
    PyObject *result;
    assert(kwargs == NULL || PyDict_Check(kwargs));
    nk = kwargs ? PyDict_Size(kwargs) : 0;
    if (Py_EnterRecursiveCall((char*)" while calling a Python object")) {
        return NULL;
    }
    if (
#if PY_MAJOR_VERSION >= 3
            co->co_kwonlyargcount == 0 &&
#endif
            likely(kwargs == NULL || nk == 0) &&
            co->co_flags == (CO_OPTIMIZED | CO_NEWLOCALS | CO_NOFREE)) {
        if (argdefs == NULL && co->co_argcount == nargs) {
            result = __Pyx_PyFunction_FastCallNoKw(co, args, nargs, globals);
            goto done;
        }
        else if (nargs == 0 && argdefs != NULL
                 && co->co_argcount == Py_SIZE(argdefs)) {
            /* function called with no arguments, but all parameters have
               a default value: use default values as arguments .*/
            args = &PyTuple_GET_ITEM(argdefs, 0);
            result =__Pyx_PyFunction_FastCallNoKw(co, args, Py_SIZE(argdefs), globals);
            goto done;
        }
    }
    if (kwargs != NULL) {
        Py_ssize_t pos, i;
        kwtuple = PyTuple_New(2 * nk);
        if (kwtuple == NULL) {
            result = NULL;
            goto done;
        }
        k = &PyTuple_GET_ITEM(kwtuple, 0);
        pos = i = 0;
        while (PyDict_Next(kwargs, &pos, &k[i], &k[i+1])) {
            Py_INCREF(k[i]);
            Py_INCREF(k[i+1]);
            i += 2;
        }
        nk = i / 2;
    }
    else {
        kwtuple = NULL;
        k = NULL;
    }
    closure = PyFunction_GET_CLOSURE(func);
#if PY_MAJOR_VERSION >= 3
    kwdefs = PyFunction_GET_KW_DEFAULTS(func);
#endif
    if (argdefs != NULL) {
        d = &PyTuple_GET_ITEM(argdefs, 0);
        nd = Py_SIZE(argdefs);
    }
    else {
        d = NULL;
        nd = 0;
    }
#if PY_MAJOR_VERSION >= 3
    result = PyEval_EvalCodeEx((PyObject*)co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, kwdefs, closure);
#else
    result = PyEval_EvalCodeEx(co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, closure);
#endif
    Py_XDECREF(kwtuple);
done:
    Py_LeaveRecursiveCall();
    return result;
}
#endif
#endif

/* PyObjectCall */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = Py_TYPE(func)->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCall2Args */
static CYTHON_UNUSED PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2) {
    PyObject *args, *result = NULL;
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(function)) {
        PyObject *args[2] = {arg1, arg2};
        return __Pyx_PyFunction_FastCall(function, args, 2);
    }
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(function)) {
        PyObject *args[2] = {arg1, arg2};
        return __Pyx_PyCFunction_FastCall(function, args, 2);
    }
    #endif
    args = PyTuple_New(2);
    if (unlikely(!args)) goto done;
    Py_INCREF(arg1);
    PyTuple_SET_ITEM(args, 0, arg1);
    Py_INCREF(arg2);
    PyTuple_SET_ITEM(args, 1, arg2);
    Py_INCREF(function);
    result = __Pyx_PyObject_Call(function, args, NULL);
    Py_DECREF(args);
    Py_DECREF(function);
done:
    return result;
}

/* PyObjectCallMethO */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = PyCFunction_GET_FUNCTION(func);
    self = PyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallOneArg */
#if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx__PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_New(1);
    if (unlikely(!args)) return NULL;
    Py_INCREF(arg);
    PyTuple_SET_ITEM(args, 0, arg);
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, &arg, 1);
    }
#endif
    if (likely(PyCFunction_Check(func))) {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_O)) {
            return __Pyx_PyObject_CallMethO(func, arg);
#if CYTHON_FAST_PYCCALL
        } else if (__Pyx_PyFastCFunction_Check(func)) {
            return __Pyx_PyCFunction_FastCall(func, &arg, 1);
#endif
        }
    }
    return __Pyx__PyObject_CallOneArg(func, arg);
}
#else
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_Pack(1, arg);
    if (unlikely(!args)) return NULL;
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
#endif

/* RaiseException */
#if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    __Pyx_PyThreadState_declare
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_PyThreadState_assign
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause) {
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if CYTHON_COMPILING_IN_PYPY
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#else
        PyThreadState *tstate = __Pyx_PyThreadState_Current;
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

/* UnpackUnboundCMethod */
static int __Pyx_TryUnpackUnboundCMethod(__Pyx_CachedCFunction* target) {
    PyObject *method;
    method = __Pyx_PyObject_GetAttrStr(target->type, *target->method_name);
    if (unlikely(!method))
        return -1;
    target->method = method;
#if CYTHON_COMPILING_IN_CPYTHON
    #if PY_MAJOR_VERSION >= 3
    if (likely(__Pyx_TypeCheck(method, &PyMethodDescr_Type)))
    #endif
    {
        PyMethodDescrObject *descr = (PyMethodDescrObject*) method;
        target->func = descr->d_method->ml_meth;
        target->flag = descr->d_method->ml_flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_STACKLESS);
    }
#endif
    return 0;
}

/* CallUnboundCMethod1 */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg) {
    if (likely(cfunc->func)) {
        int flag = cfunc->flag;
        if (flag == METH_O) {
            return (*(cfunc->func))(self, arg);
        } else if (PY_VERSION_HEX >= 0x030600B1 && flag == METH_FASTCALL) {
            if (PY_VERSION_HEX >= 0x030700A0) {
                return (*(__Pyx_PyCFunctionFast)(void*)(PyCFunction)cfunc->func)(self, &arg, 1);
            } else {
                return (*(__Pyx_PyCFunctionFastWithKeywords)(void*)(PyCFunction)cfunc->func)(self, &arg, 1, NULL);
            }
        } else if (PY_VERSION_HEX >= 0x030700A0 && flag == (METH_FASTCALL | METH_KEYWORDS)) {
            return (*(__Pyx_PyCFunctionFastWithKeywords)(void*)(PyCFunction)cfunc->func)(self, &arg, 1, NULL);
        }
    }
    return __Pyx__CallUnboundCMethod1(cfunc, self, arg);
}
#endif
static PyObject* __Pyx__CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg){
    PyObject *args, *result = NULL;
    if (unlikely(!cfunc->func && !cfunc->method) && unlikely(__Pyx_TryUnpackUnboundCMethod(cfunc) < 0)) return NULL;
#if CYTHON_COMPILING_IN_CPYTHON
    if (cfunc->func && (cfunc->flag & METH_VARARGS)) {
        args = PyTuple_New(1);
        if (unlikely(!args)) goto bad;
        Py_INCREF(arg);
        PyTuple_SET_ITEM(args, 0, arg);
        if (cfunc->flag & METH_KEYWORDS)
            result = (*(PyCFunctionWithKeywords)(void*)(PyCFunction)cfunc->func)(self, args, NULL);
        else
            result = (*cfunc->func)(self, args);
    } else {
        args = PyTuple_New(2);
        if (unlikely(!args)) goto bad;
        Py_INCREF(self);
        PyTuple_SET_ITEM(args, 0, self);
        Py_INCREF(arg);
        PyTuple_SET_ITEM(args, 1, arg);
        result = __Pyx_PyObject_Call(cfunc->method, args, NULL);
    }
#else
    args = PyTuple_Pack(2, self, arg);
    if (unlikely(!args)) goto bad;
    result = __Pyx_PyObject_Call(cfunc->method, args, NULL);
#endif
bad:
    Py_XDECREF(args);
    return result;
}

/* DictGetItem */
#if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key) {
    PyObject *value;
    value = PyDict_GetItemWithError(d, key);
    if (unlikely(!value)) {
        if (!PyErr_Occurred()) {
            if (unlikely(PyTuple_Check(key))) {
                PyObject* args = PyTuple_Pack(1, key);
                if (likely(args)) {
                    PyErr_SetObject(PyExc_KeyError, args);
                    Py_DECREF(args);
                }
            } else {
                PyErr_SetObject(PyExc_KeyError, key);
            }
        }
        return NULL;
    }
    Py_INCREF(value);
    return value;
}
#endif

/* GetTopmostException */
#if CYTHON_USE_EXC_INFO_STACK
static _PyErr_StackItem *
__Pyx_PyErr_GetTopmostException(PyThreadState *tstate)
{
    _PyErr_StackItem *exc_info = tstate->exc_info;
    while ((exc_info->exc_type == NULL || exc_info->exc_type == Py_None) &&
           exc_info->previous_item != NULL)
    {
        exc_info = exc_info->previous_item;
    }
    return exc_info;
}
#endif

/* SaveResetException */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = __Pyx_PyErr_GetTopmostException(tstate);
    *type = exc_info->exc_type;
    *value = exc_info->exc_value;
    *tb = exc_info->exc_traceback;
    #else
    *type = tstate->exc_type;
    *value = tstate->exc_value;
    *tb = tstate->exc_traceback;
    #endif
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
}
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = type;
    exc_info->exc_value = value;
    exc_info->exc_traceback = tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = type;
    tstate->exc_value = value;
    tstate->exc_traceback = tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
#endif

/* GetException */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb)
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb)
#endif
{
    PyObject *local_type, *local_value, *local_tb;
#if CYTHON_FAST_THREAD_STATE
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    local_type = tstate->curexc_type;
    local_value = tstate->curexc_value;
    local_tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#else
    PyErr_Fetch(&local_type, &local_value, &local_tb);
#endif
    PyErr_NormalizeException(&local_type, &local_value, &local_tb);
#if CYTHON_FAST_THREAD_STATE
    if (unlikely(tstate->curexc_type))
#else
    if (unlikely(PyErr_Occurred()))
#endif
        goto bad;
    #if PY_MAJOR_VERSION >= 3
    if (local_tb) {
        if (unlikely(PyException_SetTraceback(local_value, local_tb) < 0))
            goto bad;
    }
    #endif
    Py_XINCREF(local_tb);
    Py_XINCREF(local_type);
    Py_XINCREF(local_value);
    *type = local_type;
    *value = local_value;
    *tb = local_tb;
#if CYTHON_FAST_THREAD_STATE
    #if CYTHON_USE_EXC_INFO_STACK
    {
        _PyErr_StackItem *exc_info = tstate->exc_info;
        tmp_type = exc_info->exc_type;
        tmp_value = exc_info->exc_value;
        tmp_tb = exc_info->exc_traceback;
        exc_info->exc_type = local_type;
        exc_info->exc_value = local_value;
        exc_info->exc_traceback = local_tb;
    }
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = local_type;
    tstate->exc_value = local_value;
    tstate->exc_traceback = local_tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_SetExcInfo(local_type, local_value, local_tb);
#endif
    return 0;
bad:
    *type = 0;
    *value = 0;
    *tb = 0;
    Py_XDECREF(local_type);
    Py_XDECREF(local_value);
    Py_XDECREF(local_tb);
    return -1;
}

/* PyObjectCallNoArg */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, NULL, 0);
    }
#endif
#ifdef __Pyx_CyFunction_USED
    if (likely(PyCFunction_Check(func) || __Pyx_CyFunction_Check(func)))
#else
    if (likely(PyCFunction_Check(func)))
#endif
    {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_NOARGS)) {
            return __Pyx_PyObject_CallMethO(func, NULL);
        }
    }
    return __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL);
}
#endif

/* WriteUnraisableException */
static void __Pyx_WriteUnraisable(const char *name, CYTHON_UNUSED int clineno,
                                  CYTHON_UNUSED int lineno, CYTHON_UNUSED const char *filename,
                                  int full_traceback, CYTHON_UNUSED int nogil) {
    PyObject *old_exc, *old_val, *old_tb;
    PyObject *ctx;
    __Pyx_PyThreadState_declare
#ifdef WITH_THREAD
    PyGILState_STATE state;
    if (nogil)
        state = PyGILState_Ensure();
#ifdef _MSC_VER
    else state = (PyGILState_STATE)-1;
#endif
#endif
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&old_exc, &old_val, &old_tb);
    if (full_traceback) {
        Py_XINCREF(old_exc);
        Py_XINCREF(old_val);
        Py_XINCREF(old_tb);
        __Pyx_ErrRestore(old_exc, old_val, old_tb);
        PyErr_PrintEx(1);
    }
    #if PY_MAJOR_VERSION < 3
    ctx = PyString_FromString(name);
    #else
    ctx = PyUnicode_FromString(name);
    #endif
    __Pyx_ErrRestore(old_exc, old_val, old_tb);
    if (!ctx) {
        PyErr_WriteUnraisable(Py_None);
    } else {
        PyErr_WriteUnraisable(ctx);
        Py_DECREF(ctx);
    }
#ifdef WITH_THREAD
    if (nogil)
        PyGILState_Release(state);
#endif
}

/* RaiseTooManyValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected) {
    PyErr_Format(PyExc_ValueError,
                 "too many values to unpack (expected %" CYTHON_FORMAT_SSIZE_T "d)", expected);
}

/* RaiseNeedMoreValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index) {
    PyErr_Format(PyExc_ValueError,
                 "need more than %" CYTHON_FORMAT_SSIZE_T "d value%.1s to unpack",
                 index, (index == 1) ? "" : "s");
}

/* RaiseNoneIterError */
static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
}

/* FastTypeChecks */
#if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = a->tp_base;
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
#if PY_MAJOR_VERSION == 2
static int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject* exc_type2) {
    PyObject *exception, *value, *tb;
    int res;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&exception, &value, &tb);
    res = exc_type1 ? PyObject_IsSubclass(err, exc_type1) : 0;
    if (unlikely(res == -1)) {
        PyErr_WriteUnraisable(err);
        res = 0;
    }
    if (!res) {
        res = PyObject_IsSubclass(err, exc_type2);
        if (unlikely(res == -1)) {
            PyErr_WriteUnraisable(err);
            res = 0;
        }
    }
    __Pyx_ErrRestore(exception, value, tb);
    return res;
}
#else
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    int res = exc_type1 ? __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type1) : 0;
    if (!res) {
        res = __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
    return res;
}
#endif
static int __Pyx_PyErr_GivenExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    assert(PyExceptionClass_Check(exc_type));
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        PyObject *t = PyTuple_GET_ITEM(tuple, i);
        #if PY_MAJOR_VERSION < 3
        if (likely(exc_type == t)) return 1;
        #endif
        if (likely(PyExceptionClass_Check(t))) {
            if (__Pyx_inner_PyErr_GivenExceptionMatches2(exc_type, NULL, t)) return 1;
        } else {
        }
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        if (likely(PyExceptionClass_Check(exc_type))) {
            return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
        } else if (likely(PyTuple_Check(exc_type))) {
            return __Pyx_PyErr_GivenExceptionMatchesTuple(err, exc_type);
        } else {
        }
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    assert(PyExceptionClass_Check(exc_type1));
    assert(PyExceptionClass_Check(exc_type2));
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* KeywordStringCheck */
static int __Pyx_CheckKeywordStrings(
    PyObject *kwdict,
    const char* function_name,
    int kw_allowed)
{
    PyObject* key = 0;
    Py_ssize_t pos = 0;
#if CYTHON_COMPILING_IN_PYPY
    if (!kw_allowed && PyDict_Next(kwdict, &pos, &key, 0))
        goto invalid_keyword;
    return 1;
#else
    while (PyDict_Next(kwdict, &pos, &key, 0)) {
        #if PY_MAJOR_VERSION < 3
        if (unlikely(!PyString_Check(key)))
        #endif
            if (unlikely(!PyUnicode_Check(key)))
                goto invalid_keyword_type;
    }
    if ((!kw_allowed) && unlikely(key))
        goto invalid_keyword;
    return 1;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    return 0;
#endif
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
    return 0;
}

/* CallUnboundCMethod2 */
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030600B1
static CYTHON_INLINE PyObject *__Pyx_CallUnboundCMethod2(__Pyx_CachedCFunction *cfunc, PyObject *self, PyObject *arg1, PyObject *arg2) {
    if (likely(cfunc->func)) {
        PyObject *args[2] = {arg1, arg2};
        if (cfunc->flag == METH_FASTCALL) {
            #if PY_VERSION_HEX >= 0x030700A0
            return (*(__Pyx_PyCFunctionFast)(void*)(PyCFunction)cfunc->func)(self, args, 2);
            #else
            return (*(__Pyx_PyCFunctionFastWithKeywords)(void*)(PyCFunction)cfunc->func)(self, args, 2, NULL);
            #endif
        }
        #if PY_VERSION_HEX >= 0x030700A0
        if (cfunc->flag == (METH_FASTCALL | METH_KEYWORDS))
            return (*(__Pyx_PyCFunctionFastWithKeywords)(void*)(PyCFunction)cfunc->func)(self, args, 2, NULL);
        #endif
    }
    return __Pyx__CallUnboundCMethod2(cfunc, self, arg1, arg2);
}
#endif
static PyObject* __Pyx__CallUnboundCMethod2(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg1, PyObject* arg2){
    PyObject *args, *result = NULL;
    if (unlikely(!cfunc->func && !cfunc->method) && unlikely(__Pyx_TryUnpackUnboundCMethod(cfunc) < 0)) return NULL;
#if CYTHON_COMPILING_IN_CPYTHON
    if (cfunc->func && (cfunc->flag & METH_VARARGS)) {
        args = PyTuple_New(2);
        if (unlikely(!args)) goto bad;
        Py_INCREF(arg1);
        PyTuple_SET_ITEM(args, 0, arg1);
        Py_INCREF(arg2);
        PyTuple_SET_ITEM(args, 1, arg2);
        if (cfunc->flag & METH_KEYWORDS)
            result = (*(PyCFunctionWithKeywords)(void*)(PyCFunction)cfunc->func)(self, args, NULL);
        else
            result = (*cfunc->func)(self, args);
    } else {
        args = PyTuple_New(3);
        if (unlikely(!args)) goto bad;
        Py_INCREF(self);
        PyTuple_SET_ITEM(args, 0, self);
        Py_INCREF(arg1);
        PyTuple_SET_ITEM(args, 1, arg1);
        Py_INCREF(arg2);
        PyTuple_SET_ITEM(args, 2, arg2);
        result = __Pyx_PyObject_Call(cfunc->method, args, NULL);
    }
#else
    args = PyTuple_Pack(3, self, arg1, arg2);
    if (unlikely(!args)) goto bad;
    result = __Pyx_PyObject_Call(cfunc->method, args, NULL);
#endif
bad:
    Py_XDECREF(args);
    return result;
}

/* py_dict_pop */
static CYTHON_INLINE PyObject *__Pyx_PyDict_Pop(PyObject *d, PyObject *key, PyObject *default_value) {
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX > 0x030600B3
    if ((1)) {
        return _PyDict_Pop(d, key, default_value);
    } else
#endif
    if (default_value) {
        return __Pyx_CallUnboundCMethod2(&__pyx_umethod_PyDict_Type_pop, d, key, default_value);
    } else {
        return __Pyx_CallUnboundCMethod1(&__pyx_umethod_PyDict_Type_pop, d, key);
    }
}

/* BytesEquals */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
    if (s1 == s2) {
        return (equals == Py_EQ);
    } else if (PyBytes_CheckExact(s1) & PyBytes_CheckExact(s2)) {
        const char *ps1, *ps2;
        Py_ssize_t length = PyBytes_GET_SIZE(s1);
        if (length != PyBytes_GET_SIZE(s2))
            return (equals == Py_NE);
        ps1 = PyBytes_AS_STRING(s1);
        ps2 = PyBytes_AS_STRING(s2);
        if (ps1[0] != ps2[0]) {
            return (equals == Py_NE);
        } else if (length == 1) {
            return (equals == Py_EQ);
        } else {
            int result;
#if CYTHON_USE_UNICODE_INTERNALS
            Py_hash_t hash1, hash2;
            hash1 = ((PyBytesObject*)s1)->ob_shash;
            hash2 = ((PyBytesObject*)s2)->ob_shash;
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                return (equals == Py_NE);
            }
#endif
            result = memcmp(ps1, ps2, (size_t)length);
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & PyBytes_CheckExact(s2)) {
        return (equals == Py_NE);
    } else if ((s2 == Py_None) & PyBytes_CheckExact(s1)) {
        return (equals == Py_NE);
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
#endif
}

/* UnicodeEquals */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
#if PY_MAJOR_VERSION < 3
    PyObject* owned_ref = NULL;
#endif
    int s1_is_unicode, s2_is_unicode;
    if (s1 == s2) {
        goto return_eq;
    }
    s1_is_unicode = PyUnicode_CheckExact(s1);
    s2_is_unicode = PyUnicode_CheckExact(s2);
#if PY_MAJOR_VERSION < 3
    if ((s1_is_unicode & (!s2_is_unicode)) && PyString_CheckExact(s2)) {
        owned_ref = PyUnicode_FromObject(s2);
        if (unlikely(!owned_ref))
            return -1;
        s2 = owned_ref;
        s2_is_unicode = 1;
    } else if ((s2_is_unicode & (!s1_is_unicode)) && PyString_CheckExact(s1)) {
        owned_ref = PyUnicode_FromObject(s1);
        if (unlikely(!owned_ref))
            return -1;
        s1 = owned_ref;
        s1_is_unicode = 1;
    } else if (((!s2_is_unicode) & (!s1_is_unicode))) {
        return __Pyx_PyBytes_Equals(s1, s2, equals);
    }
#endif
    if (s1_is_unicode & s2_is_unicode) {
        Py_ssize_t length;
        int kind;
        void *data1, *data2;
        if (unlikely(__Pyx_PyUnicode_READY(s1) < 0) || unlikely(__Pyx_PyUnicode_READY(s2) < 0))
            return -1;
        length = __Pyx_PyUnicode_GET_LENGTH(s1);
        if (length != __Pyx_PyUnicode_GET_LENGTH(s2)) {
            goto return_ne;
        }
#if CYTHON_USE_UNICODE_INTERNALS
        {
            Py_hash_t hash1, hash2;
        #if CYTHON_PEP393_ENABLED
            hash1 = ((PyASCIIObject*)s1)->hash;
            hash2 = ((PyASCIIObject*)s2)->hash;
        #else
            hash1 = ((PyUnicodeObject*)s1)->hash;
            hash2 = ((PyUnicodeObject*)s2)->hash;
        #endif
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                goto return_ne;
            }
        }
#endif
        kind = __Pyx_PyUnicode_KIND(s1);
        if (kind != __Pyx_PyUnicode_KIND(s2)) {
            goto return_ne;
        }
        data1 = __Pyx_PyUnicode_DATA(s1);
        data2 = __Pyx_PyUnicode_DATA(s2);
        if (__Pyx_PyUnicode_READ(kind, data1, 0) != __Pyx_PyUnicode_READ(kind, data2, 0)) {
            goto return_ne;
        } else if (length == 1) {
            goto return_eq;
        } else {
            int result = memcmp(data1, data2, (size_t)(length * kind));
            #if PY_MAJOR_VERSION < 3
            Py_XDECREF(owned_ref);
            #endif
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & s2_is_unicode) {
        goto return_ne;
    } else if ((s2 == Py_None) & s1_is_unicode) {
        goto return_ne;
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        #if PY_MAJOR_VERSION < 3
        Py_XDECREF(owned_ref);
        #endif
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
return_eq:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_EQ);
return_ne:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_NE);
#endif
}

/* IterFinish */
static CYTHON_INLINE int __Pyx_IterFinish(void) {
#if CYTHON_FAST_THREAD_STATE
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject* exc_type = tstate->curexc_type;
    if (unlikely(exc_type)) {
        if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) {
            PyObject *exc_value, *exc_tb;
            exc_value = tstate->curexc_value;
            exc_tb = tstate->curexc_traceback;
            tstate->curexc_type = 0;
            tstate->curexc_value = 0;
            tstate->curexc_traceback = 0;
            Py_DECREF(exc_type);
            Py_XDECREF(exc_value);
            Py_XDECREF(exc_tb);
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#else
    if (unlikely(PyErr_Occurred())) {
        if (likely(PyErr_ExceptionMatches(PyExc_StopIteration))) {
            PyErr_Clear();
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#endif
}

/* UnpackItemEndCheck */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected) {
    if (unlikely(retval)) {
        Py_DECREF(retval);
        __Pyx_RaiseTooManyValuesError(expected);
        return -1;
    } else {
        return __Pyx_IterFinish();
    }
    return 0;
}

/* py_abs */
#if CYTHON_USE_PYLONG_INTERNALS
static PyObject *__Pyx_PyLong_AbsNeg(PyObject *n) {
    if (likely(Py_SIZE(n) == -1)) {
        return PyLong_FromLong(((PyLongObject*)n)->ob_digit[0]);
    }
#if CYTHON_COMPILING_IN_CPYTHON
    {
        PyObject *copy = _PyLong_Copy((PyLongObject*)n);
        if (likely(copy)) {
            __Pyx_SET_SIZE(copy, -Py_SIZE(copy));
        }
        return copy;
    }
#else
    return PyNumber_Negative(n);
#endif
}
#endif

/* PyObjectSetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_setattro))
        return tp->tp_setattro(obj, attr_name, value);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_setattr))
        return tp->tp_setattr(obj, PyString_AS_STRING(attr_name), value);
#endif
    return PyObject_SetAttr(obj, attr_name, value);
}
#endif

/* GetItemInt */
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (!j) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyList_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyTuple_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     CYTHON_NCP_UNUSED int wraparound,
                                                     CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((!boundscheck) || (likely(__Pyx_is_valid_index(n, PyList_GET_SIZE(o))))) {
            PyObject *r = PyList_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely(__Pyx_is_valid_index(n, PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return m->sq_item(o, i);
        }
    }
#else
    if (is_list || PySequence_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
}

/* ObjectGetItem */
#if CYTHON_USE_TYPE_SLOTS
static PyObject *__Pyx_PyObject_GetIndex(PyObject *obj, PyObject* index) {
    PyObject *runerr;
    Py_ssize_t key_value;
    PySequenceMethods *m = Py_TYPE(obj)->tp_as_sequence;
    if (unlikely(!(m && m->sq_item))) {
        PyErr_Format(PyExc_TypeError, "'%.200s' object is not subscriptable", Py_TYPE(obj)->tp_name);
        return NULL;
    }
    key_value = __Pyx_PyIndex_AsSsize_t(index);
    if (likely(key_value != -1 || !(runerr = PyErr_Occurred()))) {
        return __Pyx_GetItemInt_Fast(obj, key_value, 0, 1, 1);
    }
    if (PyErr_GivenExceptionMatches(runerr, PyExc_OverflowError)) {
        PyErr_Clear();
        PyErr_Format(PyExc_IndexError, "cannot fit '%.200s' into an index-sized integer", Py_TYPE(index)->tp_name);
    }
    return NULL;
}
static PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject* key) {
    PyMappingMethods *m = Py_TYPE(obj)->tp_as_mapping;
    if (likely(m && m->mp_subscript)) {
        return m->mp_subscript(obj, key);
    }
    return __Pyx_PyObject_GetIndex(obj, key);
}
#endif

/* HasAttr */
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *o, PyObject *n) {
    PyObject *r;
    if (unlikely(!__Pyx_PyBaseString_Check(n))) {
        PyErr_SetString(PyExc_TypeError,
                        "hasattr(): attribute name must be string");
        return -1;
    }
    r = __Pyx_GetAttr(o, n);
    if (unlikely(!r)) {
        PyErr_Clear();
        return 0;
    } else {
        Py_DECREF(r);
        return 1;
    }
}

/* SliceObject */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetSlice(PyObject* obj,
        Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** _py_start, PyObject** _py_stop, PyObject** _py_slice,
        int has_cstart, int has_cstop, CYTHON_UNUSED int wraparound) {
#if CYTHON_USE_TYPE_SLOTS
    PyMappingMethods* mp;
#if PY_MAJOR_VERSION < 3
    PySequenceMethods* ms = Py_TYPE(obj)->tp_as_sequence;
    if (likely(ms && ms->sq_slice)) {
        if (!has_cstart) {
            if (_py_start && (*_py_start != Py_None)) {
                cstart = __Pyx_PyIndex_AsSsize_t(*_py_start);
                if ((cstart == (Py_ssize_t)-1) && PyErr_Occurred()) goto bad;
            } else
                cstart = 0;
        }
        if (!has_cstop) {
            if (_py_stop && (*_py_stop != Py_None)) {
                cstop = __Pyx_PyIndex_AsSsize_t(*_py_stop);
                if ((cstop == (Py_ssize_t)-1) && PyErr_Occurred()) goto bad;
            } else
                cstop = PY_SSIZE_T_MAX;
        }
        if (wraparound && unlikely((cstart < 0) | (cstop < 0)) && likely(ms->sq_length)) {
            Py_ssize_t l = ms->sq_length(obj);
            if (likely(l >= 0)) {
                if (cstop < 0) {
                    cstop += l;
                    if (cstop < 0) cstop = 0;
                }
                if (cstart < 0) {
                    cstart += l;
                    if (cstart < 0) cstart = 0;
                }
            } else {
                if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                    goto bad;
                PyErr_Clear();
            }
        }
        return ms->sq_slice(obj, cstart, cstop);
    }
#endif
    mp = Py_TYPE(obj)->tp_as_mapping;
    if (likely(mp && mp->mp_subscript))
#endif
    {
        PyObject* result;
        PyObject *py_slice, *py_start, *py_stop;
        if (_py_slice) {
            py_slice = *_py_slice;
        } else {
            PyObject* owned_start = NULL;
            PyObject* owned_stop = NULL;
            if (_py_start) {
                py_start = *_py_start;
            } else {
                if (has_cstart) {
                    owned_start = py_start = PyInt_FromSsize_t(cstart);
                    if (unlikely(!py_start)) goto bad;
                } else
                    py_start = Py_None;
            }
            if (_py_stop) {
                py_stop = *_py_stop;
            } else {
                if (has_cstop) {
                    owned_stop = py_stop = PyInt_FromSsize_t(cstop);
                    if (unlikely(!py_stop)) {
                        Py_XDECREF(owned_start);
                        goto bad;
                    }
                } else
                    py_stop = Py_None;
            }
            py_slice = PySlice_New(py_start, py_stop, Py_None);
            Py_XDECREF(owned_start);
            Py_XDECREF(owned_stop);
            if (unlikely(!py_slice)) goto bad;
        }
#if CYTHON_USE_TYPE_SLOTS
        result = mp->mp_subscript(obj, py_slice);
#else
        result = PyObject_GetItem(obj, py_slice);
#endif
        if (!_py_slice) {
            Py_DECREF(py_slice);
        }
        return result;
    }
    PyErr_Format(PyExc_TypeError,
        "'%.200s' object is unsliceable", Py_TYPE(obj)->tp_name);
bad:
    return NULL;
}

/* dict_getitem_default */
static PyObject* __Pyx_PyDict_GetItemDefault(PyObject* d, PyObject* key, PyObject* default_value) {
    PyObject* value;
#if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY
    value = PyDict_GetItemWithError(d, key);
    if (unlikely(!value)) {
        if (unlikely(PyErr_Occurred()))
            return NULL;
        value = default_value;
    }
    Py_INCREF(value);
    if ((1));
#else
    if (PyString_CheckExact(key) || PyUnicode_CheckExact(key) || PyInt_CheckExact(key)) {
        value = PyDict_GetItem(d, key);
        if (unlikely(!value)) {
            value = default_value;
        }
        Py_INCREF(value);
    }
#endif
    else {
        if (default_value == Py_None)
            value = __Pyx_CallUnboundCMethod1(&__pyx_umethod_PyDict_Type_get, d, key);
        else
            value = __Pyx_CallUnboundCMethod2(&__pyx_umethod_PyDict_Type_get, d, key, default_value);
    }
    return value;
}

/* CallUnboundCMethod0 */
static PyObject* __Pyx__CallUnboundCMethod0(__Pyx_CachedCFunction* cfunc, PyObject* self) {
    PyObject *args, *result = NULL;
    if (unlikely(!cfunc->method) && unlikely(__Pyx_TryUnpackUnboundCMethod(cfunc) < 0)) return NULL;
#if CYTHON_ASSUME_SAFE_MACROS
    args = PyTuple_New(1);
    if (unlikely(!args)) goto bad;
    Py_INCREF(self);
    PyTuple_SET_ITEM(args, 0, self);
#else
    args = PyTuple_Pack(1, self);
    if (unlikely(!args)) goto bad;
#endif
    result = __Pyx_PyObject_Call(cfunc->method, args, NULL);
    Py_DECREF(args);
bad:
    return result;
}

/* py_dict_items */
static CYTHON_INLINE PyObject* __Pyx_PyDict_Items(PyObject* d) {
    if (PY_MAJOR_VERSION >= 3)
        return __Pyx_CallUnboundCMethod0(&__pyx_umethod_PyDict_Type_items, d);
    else
        return PyDict_Items(d);
}

/* Import */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *empty_list = 0;
    PyObject *module = 0;
    PyObject *global_dict = 0;
    PyObject *empty_dict = 0;
    PyObject *list;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (!py_import)
        goto bad;
    #endif
    if (from_list)
        list = from_list;
    else {
        empty_list = PyList_New(0);
        if (!empty_list)
            goto bad;
        list = empty_list;
    }
    global_dict = PyModule_GetDict(__pyx_m);
    if (!global_dict)
        goto bad;
    empty_dict = PyDict_New();
    if (!empty_dict)
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if ((1) && (strchr(__Pyx_MODULE_NAME, '.'))) {
                module = PyImport_ImportModuleLevelObject(
                    name, global_dict, empty_dict, list, 1);
                if (!module) {
                    if (!PyErr_ExceptionMatches(PyExc_ImportError))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_MAJOR_VERSION < 3
            PyObject *py_level = PyInt_FromLong(level);
            if (!py_level)
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, global_dict, empty_dict, list, py_level, (PyObject *)NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, global_dict, empty_dict, list, level);
            #endif
        }
    }
bad:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_import);
    #endif
    Py_XDECREF(empty_list);
    Py_XDECREF(empty_dict);
    return module;
}

/* ImportFrom */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name) {
    PyObject* value = __Pyx_PyObject_GetAttrStr(module, name);
    if (unlikely(!value) && PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Format(PyExc_ImportError,
        #if PY_MAJOR_VERSION < 3
            "cannot import name %.230s", PyString_AS_STRING(name));
        #else
            "cannot import name %S", name);
        #endif
    }
    return value;
}

/* PyObject_GenericGetAttrNoDict */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject *__Pyx_RaiseGenericGetAttributeError(PyTypeObject *tp, PyObject *attr_name) {
    PyErr_Format(PyExc_AttributeError,
#if PY_MAJOR_VERSION >= 3
                 "'%.50s' object has no attribute '%U'",
                 tp->tp_name, attr_name);
#else
                 "'%.50s' object has no attribute '%.400s'",
                 tp->tp_name, PyString_AS_STRING(attr_name));
#endif
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name) {
    PyObject *descr;
    PyTypeObject *tp = Py_TYPE(obj);
    if (unlikely(!PyString_Check(attr_name))) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    assert(!tp->tp_dictoffset);
    descr = _PyType_Lookup(tp, attr_name);
    if (unlikely(!descr)) {
        return __Pyx_RaiseGenericGetAttributeError(tp, attr_name);
    }
    Py_INCREF(descr);
    #if PY_MAJOR_VERSION < 3
    if (likely(PyType_HasFeature(Py_TYPE(descr), Py_TPFLAGS_HAVE_CLASS)))
    #endif
    {
        descrgetfunc f = Py_TYPE(descr)->tp_descr_get;
        if (unlikely(f)) {
            PyObject *res = f(descr, obj, (PyObject *)tp);
            Py_DECREF(descr);
            return res;
        }
    }
    return descr;
}
#endif

/* PyObject_GenericGetAttr */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name) {
    if (unlikely(Py_TYPE(obj)->tp_dictoffset)) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    return __Pyx_PyObject_GenericGetAttrNoDict(obj, attr_name);
}
#endif

/* PyObjectGetAttrStrNoError */
static void __Pyx_PyObject_GetAttrStr_ClearAttributeError(void) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (likely(__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        __Pyx_PyErr_Clear();
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name) {
    PyObject *result;
#if CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_TYPE_SLOTS && PY_VERSION_HEX >= 0x030700B1
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro == PyObject_GenericGetAttr)) {
        return _PyObject_GenericGetAttrWithDict(obj, attr_name, NULL, 1);
    }
#endif
    result = __Pyx_PyObject_GetAttrStr(obj, attr_name);
    if (unlikely(!result)) {
        __Pyx_PyObject_GetAttrStr_ClearAttributeError();
    }
    return result;
}

/* SetupReduce */
static int __Pyx_setup_reduce_is_named(PyObject* meth, PyObject* name) {
  int ret;
  PyObject *name_attr;
  name_attr = __Pyx_PyObject_GetAttrStr(meth, __pyx_n_s_name);
  if (likely(name_attr)) {
      ret = PyObject_RichCompareBool(name_attr, name, Py_EQ);
  } else {
      ret = -1;
  }
  if (unlikely(ret < 0)) {
      PyErr_Clear();
      ret = 0;
  }
  Py_XDECREF(name_attr);
  return ret;
}
static int __Pyx_setup_reduce(PyObject* type_obj) {
    int ret = 0;
    PyObject *object_reduce = NULL;
    PyObject *object_reduce_ex = NULL;
    PyObject *reduce = NULL;
    PyObject *reduce_ex = NULL;
    PyObject *reduce_cython = NULL;
    PyObject *setstate = NULL;
    PyObject *setstate_cython = NULL;
#if CYTHON_USE_PYTYPE_LOOKUP
    if (_PyType_Lookup((PyTypeObject*)type_obj, __pyx_n_s_getstate)) goto __PYX_GOOD;
#else
    if (PyObject_HasAttr(type_obj, __pyx_n_s_getstate)) goto __PYX_GOOD;
#endif
#if CYTHON_USE_PYTYPE_LOOKUP
    object_reduce_ex = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#else
    object_reduce_ex = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#endif
    reduce_ex = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce_ex); if (unlikely(!reduce_ex)) goto __PYX_BAD;
    if (reduce_ex == object_reduce_ex) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_reduce = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto __PYX_BAD;
#else
        object_reduce = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto __PYX_BAD;
#endif
        reduce = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce); if (unlikely(!reduce)) goto __PYX_BAD;
        if (reduce == object_reduce || __Pyx_setup_reduce_is_named(reduce, __pyx_n_s_reduce_cython)) {
            reduce_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_reduce_cython);
            if (likely(reduce_cython)) {
                ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce, reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
            } else if (reduce == object_reduce || PyErr_Occurred()) {
                goto __PYX_BAD;
            }
            setstate = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_setstate);
            if (!setstate) PyErr_Clear();
            if (!setstate || __Pyx_setup_reduce_is_named(setstate, __pyx_n_s_setstate_cython)) {
                setstate_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_setstate_cython);
                if (likely(setstate_cython)) {
                    ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate, setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                    ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                } else if (!setstate || PyErr_Occurred()) {
                    goto __PYX_BAD;
                }
            }
            PyType_Modified((PyTypeObject*)type_obj);
        }
    }
    goto __PYX_GOOD;
__PYX_BAD:
    if (!PyErr_Occurred())
        PyErr_Format(PyExc_RuntimeError, "Unable to initialize pickling for %s", ((PyTypeObject*)type_obj)->tp_name);
    ret = -1;
__PYX_GOOD:
#if !CYTHON_USE_PYTYPE_LOOKUP
    Py_XDECREF(object_reduce);
    Py_XDECREF(object_reduce_ex);
#endif
    Py_XDECREF(reduce);
    Py_XDECREF(reduce_ex);
    Py_XDECREF(reduce_cython);
    Py_XDECREF(setstate);
    Py_XDECREF(setstate_cython);
    return ret;
}

/* SetVTable */
static int __Pyx_SetVtable(PyObject *dict, void *vtable) {
#if PY_VERSION_HEX >= 0x02070000
    PyObject *ob = PyCapsule_New(vtable, 0, 0);
#else
    PyObject *ob = PyCObject_FromVoidPtr(vtable, 0);
#endif
    if (!ob)
        goto bad;
    if (PyDict_SetItem(dict, __pyx_n_s_pyx_vtable, ob) < 0)
        goto bad;
    Py_DECREF(ob);
    return 0;
bad:
    Py_XDECREF(ob);
    return -1;
}

/* CLineInTraceback */
#ifndef CYTHON_CLINE_IN_TRACEBACK
static int __Pyx_CLineForTraceback(CYTHON_NCP_UNUSED PyThreadState *tstate, int c_line) {
    PyObject *use_cline;
    PyObject *ptype, *pvalue, *ptraceback;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject **cython_runtime_dict;
#endif
    if (unlikely(!__pyx_cython_runtime)) {
        return c_line;
    }
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
#if CYTHON_COMPILING_IN_CPYTHON
    cython_runtime_dict = _PyObject_GetDictPtr(__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
        __PYX_PY_DICT_LOOKUP_IF_MODIFIED(
            use_cline, *cython_runtime_dict,
            __Pyx_PyDict_GetItemStr(*cython_runtime_dict, __pyx_n_s_cline_in_traceback))
    } else
#endif
    {
      PyObject *use_cline_obj = __Pyx_PyObject_GetAttrStr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback);
      if (use_cline_obj) {
        use_cline = PyObject_Not(use_cline_obj) ? Py_False : Py_True;
        Py_DECREF(use_cline_obj);
      } else {
        PyErr_Clear();
        use_cline = NULL;
      }
    }
    if (!use_cline) {
        c_line = 0;
        PyObject_SetAttr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback, Py_False);
    }
    else if (use_cline == Py_False || (use_cline != Py_True && PyObject_Not(use_cline) != 0)) {
        c_line = 0;
    }
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache */
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, ((size_t)new_max) * sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

/* AddTraceback */
#include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyObject *py_srcfile = 0;
    PyObject *py_funcname = 0;
    #if PY_MAJOR_VERSION < 3
    py_srcfile = PyString_FromString(filename);
    #else
    py_srcfile = PyUnicode_FromString(filename);
    #endif
    if (!py_srcfile) goto bad;
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        #else
        py_funcname = PyUnicode_FromString(funcname);
        #endif
    }
    if (!py_funcname) goto bad;
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    Py_DECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_srcfile);
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

/* CIntFromPyVerify */
#define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* CIntFromPy */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
    }
}

/* CIntFromPy */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* CStringEquals */
static CYTHON_INLINE int __Pyx_StrEq(const char *s1, const char *s2) {
    while (*s1 != '\0' && *s1 == *s2) { s1++; s2++; }
    return *s1 == *s2;
}

/* CheckBinaryVersion */
static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

/* VoidPtrExport */
static int __Pyx_ExportVoidPtr(PyObject *name, void *p, const char *sig) {
    PyObject *d;
    PyObject *cobj = 0;
    d = PyDict_GetItem(__pyx_d, __pyx_n_s_pyx_capi);
    Py_XINCREF(d);
    if (!d) {
        d = PyDict_New();
        if (!d)
            goto bad;
        if (__Pyx_PyObject_SetAttrStr(__pyx_m, __pyx_n_s_pyx_capi, d) < 0)
            goto bad;
    }
#if PY_VERSION_HEX >= 0x02070000
    cobj = PyCapsule_New(p, sig, 0);
#else
    cobj = PyCObject_FromVoidPtrAndDesc(p, (void *)sig, 0);
#endif
    if (!cobj)
        goto bad;
    if (PyDict_SetItem(d, name, cobj) < 0)
        goto bad;
    Py_DECREF(cobj);
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(cobj);
    Py_XDECREF(d);
    return -1;
}

/* InitStrings */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        if (PyObject_Hash(*t->p) == -1)
            return -1;
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#if !CYTHON_PEP393_ENABLED
static const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    char* defenc_c;
    PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
    if (!defenc) return NULL;
    defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    {
        char* end = defenc_c + PyBytes_GET_SIZE(defenc);
        char* c;
        for (c = defenc_c; c < end; c++) {
            if ((unsigned char) (*c) >= 128) {
                PyUnicode_AsASCIIString(o);
                return NULL;
            }
        }
    }
#endif
    *length = PyBytes_GET_SIZE(defenc);
    return defenc_c;
}
#else
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
}
#endif
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject* x) {
    int retval;
    if (unlikely(!x)) return -1;
    retval = __Pyx_PyObject_IsTrue(x);
    Py_DECREF(x);
    return retval;
}
static PyObject* __Pyx_PyNumber_IntOrLongWrongResultType(PyObject* result, const char* type_name) {
#if PY_MAJOR_VERSION >= 3
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type %.200s).  "
                "The ability to return an instance of a strict subclass of int "
                "is deprecated, and may be removed in a future version of Python.",
                Py_TYPE(result)->tp_name)) {
            Py_DECREF(result);
            return NULL;
        }
        return result;
    }
#endif
    PyErr_Format(PyExc_TypeError,
                 "__%.4s__ returned non-%.4s (type %.200s)",
                 type_name, type_name, Py_TYPE(result)->tp_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_Check(x) || PyLong_Check(x)))
#else
  if (likely(PyLong_Check(x)))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = m->nb_int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = m->nb_long(x);
  }
  #else
  if (likely(m && m->nb_int)) {
    name = "int";
    res = m->nb_int(x);
  }
  #endif
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
    res = PyNumber_Int(x);
  }
#endif
  if (likely(res)) {
#if PY_MAJOR_VERSION < 3
    if (unlikely(!PyInt_Check(res) && !PyLong_Check(res))) {
#else
    if (unlikely(!PyLong_CheckExact(res))) {
#endif
        return __Pyx_PyNumber_IntOrLongWrongResultType(res, name);
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(b);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b) {
  return b ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False);
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
